package com.ym.mec.cms.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.cms.dal.dao.SysConfigDao;
import com.ym.mec.cms.dal.dao.SysNewsInformationDao;
import com.ym.mec.cms.dal.dao.SysNewsTypeDao;
import com.ym.mec.cms.dal.dao.SysUserDao;
import com.ym.mec.cms.dal.entity.SysNewsInformation;
import com.ym.mec.cms.dal.entity.SysNewsType;
import com.ym.mec.cms.dal.entity.SysNewsTypeTree;
import com.ym.mec.cms.dal.entity.Teacher;
import com.ym.mec.cms.service.SysNewsTypeService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;

import com.ym.mec.util.date.DateUtil;
import com.ym.mec.web.WebFeignService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class SysNewsTypeServiceImpl extends BaseServiceImpl<Integer, SysNewsType> implements SysNewsTypeService {
    @Resource
    private SysNewsTypeDao sysNewsTypeDao;
    @Resource
    private SysNewsInformationDao informationDao;
    @Resource
    private WebFeignService webFeignService;
    @Resource
    private SysConfigDao sysConfigDao;
    @Resource
    private SysUserDao sysUserDao;

    @Override
    public BaseDAO<Integer, SysNewsType> getDAO() {
        return sysNewsTypeDao;
    }

    @Override
    public List<SysNewsTypeTree> queryTree(Integer parentId) {
        if (parentId == null) {
            parentId = 0;
        }
        List<SysNewsTypeTree> sysNewsTypeList = queryByParentId(parentId);
        if (sysNewsTypeList != null && sysNewsTypeList.size() > 0) {
            for (SysNewsTypeTree type : sysNewsTypeList) {
                type.setSubTypeList(queryTree(type.getId()));
            }
        }

        return sysNewsTypeList;
    }

    @Override
    public List<SysNewsTypeTree> queryByParentId(Integer parentId) {
        if (parentId == null) {
            parentId = 0;
        }
        return sysNewsTypeDao.queryByParentId(parentId);
    }

    public List<SysNewsType> listWithTree(Integer id, String memo,Integer userId,String organIds) {
        SysNewsType newsType = sysNewsTypeDao.get(id);
        //老师端，按钮配置
        String excludeFunctionIds = null;
        if(id == 23){
            Teacher teacher = sysUserDao.getTeacherById(userId);
            if(teacher != null){
                //是否禁用云课堂
                if(teacher.getCoursewareFlag()){
                    excludeFunctionIds = "1925";
                }
                //是否禁用云教练
                if(teacher.getCoursewareLimit()){
                    if(StringUtils.isNotEmpty(excludeFunctionIds)){
                        excludeFunctionIds += ",1374";
                    }else {
                        excludeFunctionIds = "1374";
                    }
                }
            }
        }
        /*//获取需要排除的用户编号
        String excludeUserIds = sysConfigDao.findConfigValue("exclude_user_ids");
        if(StringUtils.isNotEmpty(excludeUserIds) && excludeUserIds.contains(userId.toString())){
            //获取需要排除的功能点编号
            excludeFunctionIds = sysConfigDao.findConfigValue("exclude_function_ids");
        }*/
        List<SysNewsType> all = sysNewsTypeDao.findAll(null);
        String startTime = DateUtil.format(DateUtil.getFirstDayOfMonth(DateUtil.addMonths(new Date(), -1)),DateUtil.ISO_EXPANDED_DATE_FORMAT);
        for (SysNewsType e : all) {
            List<SysNewsInformation> list = informationDao.queryBySubType(e.getId(), memo,organIds,excludeFunctionIds);
            if (CollectionUtils.isEmpty(list)) {
            	list = informationDao.queryBySubType(e.getId(), null,organIds,excludeFunctionIds);
            }
            Iterator<SysNewsInformation> iterator = list.iterator();
			SysNewsInformation sni = null;
    		while(iterator.hasNext()){
    			sni = iterator.next();
    			if(TenantContextHolder.getTenantId() != 1 && StringUtils.indexOf(sni.getTitle(), "商城") >= 0){
    				iterator.remove();
    			}else if(sni.getTitle().equals("进阶课堂")){
                    //当前学员是否有可购买vip课
                    sni.setRedDot(webFeignService.queryVipPracticeGroups());
                }else if(sni.getTitle().equals("学员考勤异常")){
                    //当前老师是否有学员考勤异常
                    sni.setRedDot(informationDao.countStudentAttendanceError(startTime,userId));
                } else if (sni.getTitle().equals("训练列表")) {
                    //训练列表
                    sni.setRedDot(queryHomework(userId));
                }
    		}
            e.setInformationList(list);
        }
        List<SysNewsType> treeMenus = all.stream()
                .filter((e) -> e.getParentId().equals(newsType.getParentId()))
                .map((menu) -> menu.setChildren(getChildren(menu, all)))
                .filter(e -> e.getId().equals(id))
                .collect(Collectors.toList());
        return treeMenus;
    }

    private Boolean queryHomework(Integer userId) {
        int courseIdCount = informationDao.queryHomework(userId);
        if (courseIdCount > 0) {
            return true;
        }

        LocalDate now = LocalDate.now();
        // 获取周一
        LocalDate monday = now.minusDays(now.getDayOfWeek().getValue() - 1);
        int practiceIdCount = informationDao.queryPractice(userId,monday.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));

        if (practiceIdCount > 0) {
            return true;
        }
        return false;
    }

    private List<SysNewsType> getChildren(SysNewsType rootMenu, List<SysNewsType> allMenus) {
        List<SysNewsType> childrenList = allMenus.stream().filter(e -> e.getParentId().equals(rootMenu.getId()))
                .map(menu -> menu.setChildren(getChildren(menu, allMenus)))
                .collect(Collectors.toList());
        return childrenList;
    }

}