package com.ym.mec;

import com.spring4all.swagger.EnableSwagger2Doc;
import com.ym.mec.config.AppBeanNameGenerator;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.boot.logging.LogLevel;
import org.springframework.boot.logging.LoggingSystem;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;

@SpringBootApplication
@EnableDiscoveryClient
@EnableFeignClients("com.ym.mec")
@ComponentScan(basePackages = {"com.ym.mec", "org.snaker.engine", "com.yonge.log", "com.mec.redisson", "com.dayaedu.cbs.openfeign"})
@MapperScan(basePackages = {"com.ym.mec.biz.**.dao", "com.yonge.datasource.dao", "com.ym.mec.biz.dal.mapper"})
@Configuration
@EnableSwagger2Doc
public class AppServerApplication {
    public static void main(String[] args) {
        new SpringApplicationBuilder(AppServerApplication.class)
                .beanNameGenerator(new AppBeanNameGenerator())
                .run(args);
        configureGlobalLogger();
    }

    private static void configureGlobalLogger() {
        LoggingSystem loggingSystem = LoggingSystem.get(ClassLoader.getSystemClassLoader());
        // 设置业务日志记录器的日志级别为INFO
        loggingSystem.setLogLevel("businessLogger", LogLevel.INFO);
    }
}