package com.ym.mec.config;

import com.ym.mec.common.security.BaseAccessDeniedHandler;
import com.ym.mec.common.security.BaseAuthenticationEntryPoint;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;

@Configuration
@EnableResourceServer
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class ResourceServerConfig extends ResourceServerConfigurerAdapter {

    @Autowired
    private BaseAccessDeniedHandler baseAccessDeniedHandler;

    @Autowired
    private BaseAuthenticationEntryPoint baseAuthenticationEntryPoint;

    @Override
    public void configure(HttpSecurity http) throws Exception {
        http.csrf()
                .disable()
                .exceptionHandling()
                .accessDeniedHandler(baseAccessDeniedHandler)
                .authenticationEntryPoint(baseAuthenticationEntryPoint)
                .and()
                .authorizeRequests()
                .antMatchers("/api-web/task/**")
                .hasIpAddress("0.0.0.0/0")
                .antMatchers("/v2/api-docs",
                //common
                        "/*/v2/api-docs", "/*/code/*",

                //student
                        "/api-student/wechat/*","/api-student/tempDirectorTrainingCamp/*", "/api-student/student/registering", "/api-student/register/*", "/api-student/studentOrder/**", "/api-student/musicGroup/getSubjectGoodsAndInfo",
                        "/api-student/musicGroup/getMusicGroupStatus", "/api-student/musicGroup/getOrderStatus","/api-student/sysCouponCode/queryPage",
                        "/api-student/musicGroup/pay", "/api-student/musicGroup/rePay", "/api-student/studentManage/queryStudentPer",
                        "/api-student/sporadicChargeInfo/get", "/api-student/sporadicChargeInfo/sporadicPay", "/api-student/repair/getRepairInfo", "/api-student/repair/payRepair",
                        "/api-student/degree/*", "/api-student/getUploadSign", "/api-student/practiceGroup/queryOrderInfo", "/api-student/systemDate/query", "/api-student/organization/getOrgans",
                        "/api-student/repair/addGoodsSellOrder", "/api-student/repair/getStudentGoodsOrder", "/api-student/subjectChange/getChangeInfo", "/api-student/subjectChange/payChange", "/api-student/contracts/queryProduceContract",
                        "/api-student/repair/studentPaymentGoodsOrder", "/api-student/studentCompetition/getWinnerList", "/api-student/musicGroup/getGradeList",
                        "/api-student/studentCompetition/get","/api-student/musicGroup/preRegister", "/api-student/tenantInfo/queryTenantInfoByOrgan/**",
                        "/api-student/subject/list","/api-student/tenantApply/add","/api-student/questionnaireUserResult/add","/api-student/questionnaireTopic/getDetail",
                        "/api-student/open/**", "/api-student/liveGoodsMapper/page", "/api-student/liveGoodsMapper/getGoodsDetail",
                        "/api-student/musicEnlightenmentQuestionnaire/addEnlightenmentQuestionnaire", "/api-student/musicEnlightenmentQuestionnaire/getUserMusicEnlightenmentQuestionnaire",


                //teacher
                        "/api-teacher/teacher/queryStudentApply", "/api-teacher/teacher/querySubByMusicGroupId", "/api-teacher/studentRegistration/updateSubject",
                        "/api-teacher/studyReport/createEvaluate", "/api-teacher/teacherOrder/*", "/api-teacher/teacher/getRegisterOrPreList",
                        "/api-teacher/teacherContract/callback", "/api-teacher/eduPracticeGroup/queryOrderInfo", "/api-teacher/open/**",

                //web
                        "/api-web/classGroup/highClassGroups", "/api-web/api/*", "/api-web/appVersionInfo/queryByPlatform", "/api-web/eduDegree/*",
                        "/api-web/uploadFile", "/api-web/eduContracts/queryProduceContract", "/api-web/activity/doubleEleven2020Statis", "/api-web/replacementInstrument/queryPage",
                        "/api-web/replacementInstrumentActivity/queryReplacementsStat", "/api-web/eduStudentRegistration/queryPreApplyList",
                        "/api-web/eduSubject/findSubSubjects", "/api-web/eduFinancialExpenditure/batchAdd", "/api-web/eduSendNotice/*",
                        "/api-web/oaContracts/*", "/api-web/eduStudent/organStudentOverView", "/api-web/activity/countCloudTeacherActive",
                        "/api-web/activity/organDoubleEleven2021Statis", "/api-web/activity/doubleEleven2021Statis",
                        "/api-web/activity/organDoubleEleven2022Statis", "/api-web/activity/doubleEleven2022Statis","/api-web/activity/organDoubleEleven2022Statis1", "/api-web/activity/doubleEleven2022Statis1",
                        "/api-web/questionnaireTopic/getDetail", "/api-web/questionnaireUserResult/add",
                        "/api-web/tenantInfo/info/*", "/api-web/tenantInfo/checkInfo/*", "/api-web/tenantInfo/pay/*","/api-web/tenantInfo/testCheck", "/api-web/tenantInfo/getContract/*",
                        "/api-web/tenantPreJoin/add","/api-web/imLiveBroadcastRoom/queryRoomInfo","/api-web/imLiveBroadcastRoom/test",
                        "/api-web/eduOrganization/queryAllOrgan","/api-web/eduOrganization/queryAll","/api-web/eduOrganization/getOrganCooperation","/api-web/import/oaUploadFile",
                        "/api-web/tenantInfo/callback/**","/api-web/adapay/callback","/api-web/temp/*", "/api-web/open/**","/api-web/schoolActivity/detail/*",
                        "/api-web/yeepay/callback/**","/api-web/hfMerchantConfig/queryByTenantId"

                )
                .permitAll().anyRequest().authenticated().and().httpBasic();
    }


    @Override
    public void configure(ResourceServerSecurityConfigurer resources) throws Exception {
        resources.authenticationEntryPoint(baseAuthenticationEntryPoint).accessDeniedHandler(baseAccessDeniedHandler);
    }

}
