package com.ym.mec.student.controller;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static java.math.BigDecimal.ROUND_UP;

@RequestMapping("${app-config.url.student:}/activity")
@Api(tags = "活动")
@RestController
public class ActivityController extends BaseController {
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private SporadicChargeInfoService sporadicChargeInfoService;
    @Autowired
    private MarketActivityService marketActivityService;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private VipGroupActivityService vipGroupActivityService;
    @Autowired
    private StudentDao studentDao;
    @Autowired
    private VipGroupDao vipGroupDao;
    @Autowired
    private VipGroupCategoryDao vipGroupCategoryDao;
    @Autowired
    private MusicGroupPlanMakingService musicGroupPlanMakingService;
    @Autowired
    private SubjectService subjectService;

    @ApiOperation("活动购买")
    @PostMapping(value = "/buy")
    @AuditLogAnnotation(operateName = "学生端活动购买")
    public HttpResponseResult buy(@RequestBody ActivityPayParamDto activityPayParamDto) throws Exception {
        SysUser user = sysUserService.getUser();
        activityPayParamDto.setUserId(user.getId());
        activityPayParamDto.setOrganId(user.getOrganId());
        return vipGroupActivityService.buy(activityPayParamDto);
    }

    @ApiOperation("2022双十一活动购买")
    @PostMapping(value = "/buyDoubleEleven")
    @AuditLogAnnotation(operateName = "2022双十一活动购买")
    public HttpResponseResult buyDoubleEleven(@RequestBody BuyDoubleEleven2022Dto buyDoubleEleven2022Dto) throws Exception {
        if(StringUtils.isEmpty(buyDoubleEleven2022Dto.getActivityIdList())){
            throw new BizException("请选择活动");
        }
        SysUser sysUser = sysUserService.getUser();
        buyDoubleEleven2022Dto.setOrganId(sysUser.getOrganId());
        buyDoubleEleven2022Dto.setUserId(sysUser.getId());
        return vipGroupActivityService.buyDoubleEleven(buyDoubleEleven2022Dto);
    }

    @ApiOperation(value = "分部双11活动信息")
    @GetMapping("/doubleEleven2020")
    public HttpResponseResult<List<SporadicChargeInfo>> doubleEleven2020(){
        SysUser user = sysUserService.getUser();
        Integer organId = user.getOrganId();

        List<SporadicChargeInfo> activeInfos = sporadicChargeInfoService.getOrganActiveInfo(organId.toString(), 12);
        activeInfos.sort(Comparator.comparing(SporadicChargeInfo::getAmount).reversed());
        if (activeInfos.size() <= 0) {
            return failed("分部活动不存在");
        }
        return succeed(activeInfos);
    }

    @ApiOperation(value = "分部双11活动信息")
    @RequestMapping("/doubleEleven2022")
    public HttpResponseResult doubleEleven2022(){
        SysUser sysUser = sysUserService.getUser();
        Map<String,Object> resultMap = new HashMap<>(6);
        List<ActivityDetailDto> activityDetailDtos = marketActivityService.queryActivity(4, sysUser.getOrganId());
        resultMap.put("activityDetailList",activityDetailDtos);
        resultMap.put("startTime",null);
        resultMap.put("endTime",null);
        //是否是杭州分部
        resultMap.put("HZFlag",sysUser.getOrganId().equals(4));
        if(activityDetailDtos != null && activityDetailDtos.size() > 0){
            ActivityDetailDto activityDetailDto = activityDetailDtos.get(0);
            resultMap.put("startTime",activityDetailDto.getStartTime());
            resultMap.put("endTime",activityDetailDto.getEndTime());
        }
        return succeed(resultMap);
    }

/*    @ApiOperation("获取2022云教练活动列表")
    @PostMapping(value = "/queryActivity")
    @AuditLogAnnotation(operateName = "获取2022云教练活动列表")
    public HttpResponseResult<List<VipGroupActivity>> queryActivity(){
        String configValue = sysConfigDao.findConfigValue("2022_cloud_activity_config");
        List<Cloud2022ActivityDto> cloud2022ActivityDtos = JSONArray.parseArray(configValue, Cloud2022ActivityDto.class);
        String collect = cloud2022ActivityDtos.stream().map(e -> e.getActivityId()).collect(Collectors.joining(","));
        return succeed(vipGroupActivityService.getDao().queryByIds(collect));
    }*/

    @ApiOperation("获取2023乐理直播活动列表")
    @PostMapping(value = "/queryTheoryLiveActivity")
    @AuditLogAnnotation(operateName = "获取2023乐理直播活动列表")
    public HttpResponseResult<List<VipGroup>> queryTheoryLiveActivity(){
        String configValue = sysConfigDao.findConfigValue("2023_double_eleven_active_config");
        JSONObject jsonObject = JSONObject.parseObject(configValue);
        String theoryLive = jsonObject.getString("theoryLive");
        //转换成long数组
        List<Long> longs = Arrays.stream(theoryLive.split(",")).map(Long::parseLong).collect(Collectors.toList());
        return succeed(vipGroupDao.getVipGroupByIds(longs));
    }
    @ApiOperation("获取2023商品列表")
    @PostMapping(value = "/queryGoodsActivity")
    @AuditLogAnnotation(operateName = "获取2023商品列表")
    public HttpResponseResult<JSONObject> queryGoodsActivity(){
        String configValue = sysConfigDao.findConfigValue("2023_double_eleven_active_config");
        JSONObject jsonObject = JSONObject.parseObject(configValue);
        return succeed(jsonObject.getJSONObject("musical"));
    }
    @ApiOperation("校验用户分部是否参与活动")
    @PostMapping(value = "/checkActivityOrgan")
    @AuditLogAnnotation(operateName = "校验用户分部是否参与活动")
    public HttpResponseResult<Boolean> checkActivityOrgan(){
        String configValue = sysConfigDao.findConfigValue("2023_activity_organ");
        if (StringUtils.isEmpty(configValue)) {
            return succeed(true);
        }
        SysUser user = sysUserService.getUser();
        if(configValue.contains(user.getOrganId().toString())){
            return succeed(true);
        }
        return succeed(false);
    }


    @ApiOperation(value = "学员已购买的双十一活动信息")
    @GetMapping("/queryStudentDoubleEleven2022Order")
    public Object queryStudentDoubleEleven2022Order() {
        SysUser user = sysUserService.getUser();
        List<ActivityDetailDto> activityDetailDtos = marketActivityService.queryActivity(4, user.getOrganId());
        String activitys = studentPaymentOrderDao.queryStudentDoubleElevenOrder(user.getId());
        List<Integer> activityList = new ArrayList<>();
        if(StringUtils.isNotEmpty(activitys)){
            for (ActivityDetailDto activityDetailDto : activityDetailDtos) {
                if(activitys.contains(activityDetailDto.getActivityId().toString())){
                    if(activityDetailDto.getCategoryName().equals("1v1")){
                        if(activityDetailDto.getCourseNum().intValue() == 20){
                            activityList.add(1);
                        }else {
                            activityList.add(2);
                        }
                    }
                    if(activityDetailDto.getCategoryName().equals("1v2")){
                        if(activityDetailDto.getCourseNum().intValue() == 20){
                            activityList.add(3);
                        }else {
                            activityList.add(4);
                        }
                    }
                }
            }
            return succeed(StringUtils.join(activityList,","));
        }
        return succeed("");
    }

    @ApiOperation(value = "获取活动方案")
    @GetMapping("/get")
    public HttpResponseResult<VipGroupActivity> get(Integer id){
        VipGroupActivity vipGroupActivity = vipGroupActivityService.get(id);
        SysUser user = sysUserService.getUser();
        vipGroupActivity.setSinglePrice(vipGroupActivityService.getActivityPrice(vipGroupActivity,user.getOrganId(),new BigDecimal(1)));
        Map<String, String> categoryMap = vipGroupCategoryDao.findConditionCategorys().stream().collect(Collectors.toMap(e-> e.getId().toString(), ConditionDto::getName));
        vipGroupActivity.setVipGroupCategoryNames(categoryMap.get(vipGroupActivity.getVipGroupCategoryIdList()));
        vipGroupActivity.setGiveCategoryName(categoryMap.get(vipGroupActivity.getGiveCategoryId()));
        //如果是固定金额
        if(vipGroupActivity.getFixedCourseNumFlag()){
            vipGroupActivity.setDisplayPrice(vipGroupActivity.getMarketPrice());
        }else {
            vipGroupActivity.setDisplayPrice(vipGroupActivity.getSinglePrice().multiply(vipGroupActivity.getDiscount()).divide(new BigDecimal(100), 0, ROUND_UP));
        }
        vipGroupActivityService.setActivityType(vipGroupActivity);
        return succeed(vipGroupActivity);
    }

    @ApiOperation(value = "查询乐团编制")
    @GetMapping(value = "/querySubject")
    public HttpResponseResult<MusicGroupPlanMakingDto> query(Integer id) throws Exception {
        MusicGroupPlanMaking pm = musicGroupPlanMakingService.get(id);

        if (pm != null) {
            MusicGroupPlanMakingDto dto = new MusicGroupPlanMakingDto();

            BeanUtils.copyProperties(pm, dto);

            List<Subject> subjectList = subjectService.findBySubjectByIdList(pm.getSubjectIdList());
            if (subjectList != null) {
                dto.setSubjectMap(subjectList.stream().collect(Collectors.toMap(Subject::getId, Subject::getName)));
            }
            return succeed(dto);
        }
        return succeed();
    }

    @ApiOperation(value = "根据课程类型获取对应课程活动方案")
    @GetMapping("/findByVipGroupCategory")
    public HttpResponseResult<List<VipGroupActivity>> findByVipGroupCategory(){
        SysUser user = sysUserService.getUser();
        ActivityQueryDto activityQueryDto = new ActivityQueryDto();
        activityQueryDto.setStudentIds(user.getId().toString());
        activityQueryDto.setCourseType(null);
        activityQueryDto.setActivityChannel(3);
        Student student = studentDao.get(user.getId());
        Boolean isNewUser = student.getIsNewUser();
        if(isNewUser){
            activityQueryDto.setNewStudentNum(0);
        }else {
            activityQueryDto.setNewStudentNum(1);
        }
        if(student.getMemberRankSettingId() == null){
            activityQueryDto.setMemberNum(1);
        }else {
            activityQueryDto.setMemberNum(0);
        }
        activityQueryDto.setOrganId(user.getOrganId().toString());
        activityQueryDto.setEnable(true);
        List<VipGroupActivity> vipGroupActivities = vipGroupActivityService.findByVipGroupCategory(activityQueryDto);
        if(!CollectionUtils.isEmpty(vipGroupActivities)){
            Map<String, String> categoryMap = vipGroupCategoryDao.findConditionCategorys().stream().collect(Collectors.toMap(e-> e.getId().toString(), ConditionDto::getName));
            for (VipGroupActivity activity : vipGroupActivities) {
                activity.setVipGroupCategoryNames(categoryMap.get(activity.getVipGroupCategoryIdList()));
                activity.setGiveCategoryName(categoryMap.get(activity.getGiveCategoryId()));
                //如果是固定金额
                if(activity.getFixedCourseNumFlag()){
                    activity.setDisplayPrice(activity.getMarketPrice());
                }else {
                    activity.setDisplayPrice(activity.getSinglePrice().multiply(activity.getDiscount()).divide(new BigDecimal(100), 0, ROUND_UP));
                }
                vipGroupActivityService.setActivityType(activity);
            }
        }
        return succeed(vipGroupActivities);
    }
}
