package com.ym.mec.student.controller;

import com.google.common.collect.Lists;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.ImUserFriendDto;
import com.ym.mec.biz.dal.entity.ImGroup;
import com.ym.mec.biz.dal.entity.ImGroupMemberPlus;
import com.ym.mec.biz.dal.enums.im.ClientEnum;
import com.ym.mec.biz.dal.page.ImGroupNoticeQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.vo.ImUserWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.student:}/imGroup")
@Api(tags = "IM群服务")
@RestController
public class ImGroupController extends BaseController {

	@Autowired
	private ImGroupService imGroupService;

	@Autowired
	private ImUserFriendService imUserFriendService;

	@Autowired
	private ImGroupNoticeService imGroupNoticeService;

	@Autowired
	private ImGroupMemberService imGroupMemberService;

	@Autowired
	private SysUserService sysUserService;

    @Autowired
    private ImGroupCoreService imGroupCoreService;

    @Autowired
    private ImGroupMemberPlusService imGroupMemberPlusService;

	@ApiOperation("查询群列表")
	@GetMapping(value = "/queryGroupList")
	public Object queryGroupList(String search,String groupType) {
		return succeed(imGroupService.queryByUserId(sysUserService.getUserId(), search,groupType));
	}

	@ApiOperation("查询群详情")
	@GetMapping(value = "/queryGroupDetail")
	public Object queryGroupDetail(String imGroupId) {
		if (imGroupId.contains("S") || imGroupId.contains("I")){
			imGroupId = imGroupId.substring(1);
		}
        ImGroup imGroup = imGroupService.get(imGroupId);

        if (imGroup == null) {
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
        ImGroupMemberPlus admin = imGroupMemberPlusService.lambdaQuery()
            .eq(ImGroupMemberPlus::getImGroupId, imGroup.getId())
            .eq(ImGroupMemberPlus::getUserId, sysUserService.getUser().getId())
            .last("limit 1").one();
        if (admin == null) {
            // 该用户不在该群，无权限查看，处理用户退出群后，查看历史消息校验
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
		// 设置默认头像
		if (Objects.nonNull(imGroup) && Objects.nonNull(imGroup.getGroupType()) && StringUtils.isBlank(imGroup.getImg())) {
			imGroup.setImg(imGroup.getGroupType().getAvatar());
		}
		return succeed(imGroup);
	}

	@ApiOperation("查询群详情(重构版)")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "groupId", value = "群编号", required = true, dataType = "String")
	})
	@GetMapping(value = "/getDetail/{groupId}")
	public HttpResponseResult<ImGroup> groupDetail(@PathVariable("groupId") String imGroupId) {
		if (imGroupId.contains("S") || imGroupId.contains("I")){
			imGroupId = imGroupId.substring(1);
		}
        ImGroup imGroup = imGroupService.get(imGroupId);

        if (imGroup == null) {
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
        ImGroupMemberPlus admin = imGroupMemberPlusService.lambdaQuery()
            .eq(ImGroupMemberPlus::getImGroupId, imGroup.getId())
            .eq(ImGroupMemberPlus::getUserId, sysUserService.getUser().getId())
            .last("limit 1").one();
        if (admin == null) {
            // 该用户不在该群，无权限查看，处理用户退出群后，查看历史消息校验
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
		return succeed(imGroupService.get(imGroupId));
	}

	@ApiOperation("查询群成员列表")
	@GetMapping(value = "/queryGroupMemberList")
	public Object queryGroupMemberList(String imGroupId) {

		return succeed(imGroupService.queryMemberById(imGroupId));
	}

	@ApiOperation("查询群成员详情")
	@GetMapping(value = "/queryGroupMemberDetail")
	public Object queryGroupMemberDetail(String imGroupId, Integer userId) {

		return succeed(imGroupService.queryMember(imGroupId, userId));
	}

	@ApiOperation("查询好友详情")
	@GetMapping(value = "/queryFriendDetail")
	public Object queryFriendDetail(Integer userId) {
		ImUserFriendDto dto = imUserFriendService.queryFriendDetail(sysUserService.getUserId(), userId);
		if (dto == null) {
			dto = new ImUserFriendDto();
			SysUser user = sysUserService.queryUserById(userId);
			dto.setFriend(user);

			List<String> userTypes = Lists.newArrayList(user.getUserType().split(","));
			if (userTypes.size() > 1
				|| userTypes.contains(ClientEnum.TEACHER.getCode())
				|| userTypes.contains(ClientEnum.SYSTEM.getCode())) {
				// 增加用户身份数量 > 1 或者包含老师身份，统一使用实名
				dto.setFriendNickname(Optional.ofNullable(user.getRealName()).filter(StringUtils::isNotBlank).orElse(user.getUsername()));
			} else if (user.getUserType().contains("STUDENT") || user.getUserType().contains("SCHOOL")) {
				dto.setFriendNickname(user.getUsername());
			} else {
				dto.setFriendNickname(user.getRealName());
			}
			dto.setFriendId(userId);
		}
		return succeed(dto);
	}

	@ApiOperation("查询好友详情(重构版)")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer")
	})
	@GetMapping(value = "/imUserFriend/getDetail/{userId}")
	public HttpResponseResult<ImUserWrapper.ImUserFriend> getImUserFriendDetail(@PathVariable("userId") Integer userId) {
		ImUserFriendDto dto = imUserFriendService.queryFriendDetail(sysUserService.getUserId(), userId);
		if (dto == null) {
			dto = new ImUserFriendDto();
			SysUser user = sysUserService.queryUserById(userId);
			dto.setFriend(user);
			if (user.getUserType().contains("STUDENT") || user.getUserType().contains("SCHOOL")) {
				dto.setFriendNickname(user.getUsername());
			} else {
				dto.setFriendNickname(user.getRealName());
			}
			dto.setFriendId(userId);
		}

		// 返回数据结构封装
		ImUserWrapper.ImUserFriend userFriend = ImUserWrapper.ImUserFriend.builder()
			.id(dto.getId())
			.userId(dto.getUserId().longValue())
			.friendId(dto.getFriendId().longValue())
			.friendAvatar(dto.getFriend().getAvatar())
			.friendNickname(dto.getFriendNickname())
			.memo(dto.getMemo())
			.updateTime(dto.getUpdateTime())
			.createTime(dto.getCreateTime())
			.roleType(dto.getRoleType())
			.friendType(dto.getRoleType())
			.build();
		return succeed(userFriend);
	}

	@ApiOperation("查询好友列表")
	@GetMapping(value = "/queryFriendList")
	public Object queryFriendList(String search) {
		return succeed(imUserFriendService.queryFriendListByUserId(sysUserService.getUserId(), search));
	}

	@ApiOperation("查询好友列表(重构版)")
	@GetMapping(value = "/imUserFriend/queryAll")
	public HttpResponseResult<List<ImUserWrapper.ImUserFriend>> queryImUserFriendList(String search) {

		// 好友关系列表
		List<ImUserFriendDto> friendDtos = imUserFriendService.queryFriendListByUserId(sysUserService.getUserId(), search);

		List<ImUserWrapper.ImUserFriend> wrappers = Lists.newArrayList();
		if (CollectionUtils.isNotEmpty(friendDtos)) {

			// 封装返回数据结构
			for (ImUserFriendDto item : friendDtos) {

				wrappers.add(ImUserWrapper.ImUserFriend.builder()
					.id(item.getId())
					.userId(item.getUserId().longValue())
					.friendId(item.getFriendId().longValue())
					.friendAvatar(item.getFriend().getAvatar())
					.friendNickname(item.getFriendNickname())
					.memo(item.getMemo())
					.roleType(item.getRoleType())
					.friendType(item.getRoleType())
					.updateTime(item.getUpdateTime())
					.createTime(item.getCreateTime())
					.build());
			}
		}

		return succeed(wrappers);
	}

	@ApiOperation("查询群公告列表")
	@GetMapping(value = "/queryNoticeList")
	public Object queryNoticeList(ImGroupNoticeQueryInfo queryInfo) {
		return succeed(imGroupNoticeService.queryPage(queryInfo));
	}

	@ApiOperation("入群")
	@GetMapping(value = "/addGroupMember")
	@ApiImplicitParam(name = "imGroupId", value = "群编号", required = true, dataType = "String")
	public Object addGroupMember(String imGroupId) {
		return succeed(imGroupMemberService.join(imGroupId,sysUserService.getUserId(),"",false));
	}


    @ApiOperation("注册im")
    @GetMapping(value = "/imRegister")
    public Object imRegister(Integer userId) throws Exception {
        return succeed(imGroupCoreService.register(userId.toString(),"","测试",null));
    }

	@ApiOperation("主动退群")
	@PostMapping(value = "/quitGroup/v2")
	public Object quitGroupV2(String imGroupId, String userId) {
		List<Integer> userIds = Arrays.stream(userId.split(",")).mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
		return succeed(imGroupMemberService.quit(imGroupId,userIds,false));
	}
}
