package com.ym.mec.student.controller;


import com.ym.mec.biz.dal.dto.LessonExaminationResultDto1;
import com.ym.mec.biz.dal.dto.LessonExaminationSubmitDto;
import com.ym.mec.biz.dal.dto.StudentLessonExaminationDetailDto;
import com.ym.mec.biz.dal.entity.StudentLessonExaminationDetail;
import com.ym.mec.biz.dal.page.LessonExaminationQueryInfo1;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryDto;
import com.ym.mec.biz.service.LessonExaminationService;
import com.ym.mec.biz.service.StudentLessonExaminationDetailService;
import com.ym.mec.biz.service.StudentLessonExaminationService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 进度评测表(LessonExamination)表控制层
 *
 * @author zx
 * @since 2023-04-03 18:20:19
 */
@Api(tags = "进度评测表")
@RestController
@RequestMapping("${app-config.url.student:}/lessonExamination")
public class LessonExaminationController extends BaseController {

    @Resource
    private LessonExaminationService lessonExaminationService;
    @Resource
    private StudentLessonExaminationService studentLessonExaminationService;
    @Resource
    private StudentLessonExaminationDetailService studentLessonExaminationDetailService;
    @Resource
    private SysUserService sysUserService;

    @ApiOperation(value = "获取作业列表")
    @PostMapping("queryPage")
    public HttpResponseResult<PageInfo<LessonExaminationResultDto1>> queryPage(@RequestBody LessonExaminationQueryInfo1 queryInfo) {
        queryInfo.setStudentId(sysUserService.getUserId());
        return succeed(lessonExaminationService.queryPageByStudent(queryInfo));
    }

    @ApiOperation(value = "获取学员单个作业")
    @GetMapping("getOne")
    public HttpResponseResult<LessonExaminationResultDto1> getOne(Long lessonExaminationId) {
        return succeed(lessonExaminationService.getOne(sysUserService.getUserId(),lessonExaminationId));
    }

    @ApiOperation(value = "提交作业")
    @PostMapping("submit")
    public HttpResponseResult submit(@RequestBody LessonExaminationSubmitDto submitDto) {
        studentLessonExaminationService.submit(submitDto);
        return succeed();
    }

    @ApiOperation(value = "获取学员单个作业详情列表")
    @PostMapping("queryDetailAll")
    public HttpResponseResult<List<StudentLessonExaminationDetailDto>> queryAll(@RequestBody StudentLessonExaminationQueryDto query) {
        query.setUserId(sysUserService.getUserId());
        return succeed(studentLessonExaminationDetailService.queryAll(query));
    }

    @ApiOperation(value = "获取学员单个作业详情")
    @GetMapping("getDetail")
    public HttpResponseResult<StudentLessonExaminationDetail> getDetail(Long studentLessonExaminationDetailId) {
        return succeed(studentLessonExaminationDetailService.getById(studentLessonExaminationDetailId));
    }
}

