package com.ym.mec.student.controller;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.SysConfigDao;
import com.ym.mec.biz.dal.entity.LuckDrawCount;
import com.ym.mec.biz.dal.entity.LuckDrawLog;
import com.ym.mec.biz.dal.page.LuckDrawQueryInfo;
import com.ym.mec.biz.service.LuckDrawCountService;
import com.ym.mec.biz.service.LuckDrawLogService;
import com.ym.mec.biz.service.LuckDrawPrizeService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.string.ValueUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Objects;

@Api(tags = "抽奖")
@RestController
@RequestMapping(value = "${app-config.url.student:}/luckDraw")
public class LuckDrawController extends BaseController {

	@Autowired
	private LuckDrawPrizeService luckDrawPrizeService;

	@Autowired
	private LuckDrawLogService luckDrawLogService;

	@Autowired
	private LuckDrawCountService luckDrawCountService;

	@Autowired
	private SysUserService sysUserService;

	@Autowired
	private SysConfigDao sysConfigDao;

	@ApiOperation(value = "查询抽奖记录")
	@GetMapping(value = "/list")
	public Object list(LuckDrawQueryInfo queryInfo) {
		SysUser user = sysUserService.getUser();
		if(!queryInfo.getAllFlag()){
			queryInfo.setUserId(user.getId());
		}
		if(queryInfo.getGroupId().equals(3) && user.getOrganId().equals(4)){
			queryInfo.setGroupId(4);
		}
		PageInfo<LuckDrawLog> pageInfo = luckDrawLogService.queryPage(queryInfo);
		for (LuckDrawLog log : pageInfo.getRows()) {
			log.getUser().setPhone(ValueUtil.fuzzyMobile(log.getUser().getPhone()));
		}
		if(queryInfo.getAllFlag() && pageInfo.getTotal() < 30){
			String startTime = sysConfigDao.findConfigValue("start_time_of_1111");
			String endTime = sysConfigDao.findConfigValue("end_time_of_1111");
			if(DateUtil.contents(startTime,endTime,new Date())){
				List<LuckDrawLog> log;
				if(user.getOrganId().equals(4)){
					log = JSONObject.parseArray(str, LuckDrawLog.class);
				}else {
					log = JSONObject.parseArray(str2, LuckDrawLog.class);
				}
				pageInfo.getRows().addAll(log);
				pageInfo.setTotal(pageInfo.getTotal() + log.size());
			}
		}
		return succeed(pageInfo);
	}

	@ApiOperation(value = "查询我的抽奖记录")
	@GetMapping(value = "/myList")
	public Object myList(LuckDrawQueryInfo queryInfo) {
		SysUser user = sysUserService.getUser();
		if(queryInfo.getGroupId().equals(3) && user.getOrganId().equals(4)){
			queryInfo.setGroupId(4);
		}
		queryInfo.setUserId(user.getId());
		return succeed(luckDrawLogService.queryPage(queryInfo));
	}

	@ApiOperation(value = "抽奖")
	@PostMapping(value = "/draw")
	public Object draw(Integer groupId) {
		SysUser user = sysUserService.getUser();
		if(groupId.equals(3) && user.getOrganId().equals(4)){
			groupId = 4;
		}
		return succeed(luckDrawPrizeService.draw((long) user.getId(), groupId));
	}

	@ApiOperation(value = "查询个人抽奖机会")
	@GetMapping(value = "/queryCount")
	public Object queryCount(Integer groupId) {
		SysUser user = sysUserService.getUser();
		if(groupId.equals(3) && user.getOrganId().equals(4)){
			groupId = 4;
		}
		LuckDrawCount luckDrawCount = luckDrawCountService.findByIdAndGroupId(user.getId(), groupId);
		if(Objects.isNull(luckDrawCount)){
			luckDrawCount = new LuckDrawCount();
			luckDrawCount.setUserId(user.getId().longValue());
			luckDrawCount.setAvailableCount(0);
		}
		luckDrawCount.setHZFlag(user.getOrganId().equals(4));
		return succeed(luckDrawCount);
	}

	private static String str = "[\n" +
			"    {\"user\": {\"username\": \"吴文瑞\"},\"luckDrawPrize\":{\"name\": \"三等奖\"}},\n" +
			"    {\"user\": {\"username\": \"高勇\"},\"luckDrawPrize\":{\"name\": \"三等奖\"}},\n" +
			"    {\"user\": {\"username\": \"李艺彤\"},\"luckDrawPrize\":{\"name\": \"二等奖\"}},\n" +
			"    {\"user\": {\"username\": \"邹辰悦\"},\"luckDrawPrize\":{\"name\": \"三等奖\"}},\n" +
			"    {\"user\": {\"username\": \"王笑笑\"},\"luckDrawPrize\":{\"name\": \"三等奖\"}},\n" +
			"]";
	private static String str2 = "[\n" +
			"    {\"user\": {\"username\": \"陈昊然\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"王木槿\"},\"luckDrawPrize\":{\"name\": \"宇航员蓝牙音箱\"}},\n" +
			"    {\"user\": {\"username\": \"刘天钧\"},\"luckDrawPrize\":{\"name\": \"喜马拉雅会员\"}},\n" +
			"    {\"user\": {\"username\": \"刘天宇\"},\"luckDrawPrize\":{\"name\": \"喜马拉雅会员\"}},\n" +
			"    {\"user\": {\"username\": \"刘润玉\"},\"luckDrawPrize\":{\"name\": \"有道词典笔\"}},\n" +
			"    {\"user\": {\"username\": \"李暮词\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"陈叶开\"},\"luckDrawPrize\":{\"name\": \"百科全书\"}},\n" +
			"    {\"user\": {\"username\": \"赵崇光\"},\"luckDrawPrize\":{\"name\": \"百科全书\"}},\n" +
			"    {\"user\": {\"username\": \"赵绮年\"},\"luckDrawPrize\":{\"name\": \"小度G1\"}},\n" +
			"    {\"user\": {\"username\": \"吴玉堂\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"吴勇师\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"陈康文\"},\"luckDrawPrize\":{\"name\": \"喜马拉雅会员\"}},\n" +
			"    {\"user\": {\"username\": \"赵康健\"},\"luckDrawPrize\":{\"name\": \"百科全书\"}},\n" +
			"    {\"user\": {\"username\": \"赵高轩\"},\"luckDrawPrize\":{\"name\": \"小度G1\"}},\n" +
			"    {\"user\": {\"username\": \"赵玉书\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"赵叶欢\"},\"luckDrawPrize\":{\"name\": \"弯得林优选笛头\"}},\n" +
			"    {\"user\": {\"username\": \"李高兴\"},\"luckDrawPrize\":{\"name\": \"喜马拉雅会员\"}},\n" +
			"    {\"user\": {\"username\": \"吴文瑞\"},\"luckDrawPrize\":{\"name\": \"喜马拉雅会员\"}},\n" +
			"    {\"user\": {\"username\": \"高勇\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"李艺彤\"},\"luckDrawPrize\":{\"name\": \"宇航员蓝牙音箱\"}},\n" +
			"    {\"user\": {\"username\": \"邹辰悦\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"王笑笑\"},\"luckDrawPrize\":{\"name\": \"20节vip小课\"}},\n" +
			"    {\"user\": {\"username\": \"肖国庆\"},\"luckDrawPrize\":{\"name\": \"有道词典笔\"}},\n" +
			"    {\"user\": {\"username\": \"李红旗\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"王乐乐\"},\"luckDrawPrize\":{\"name\": \"乐器清洁套装\"}},\n" +
			"    {\"user\": {\"username\": \"孙朵朵\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"孙雨轩\"},\"luckDrawPrize\":{\"name\": \"百科全书\"}},\n" +
			"    {\"user\": {\"username\": \"孙文睿\"},\"luckDrawPrize\":{\"name\": \"宇航员蓝牙音箱\"}},\n" +
			"    {\"user\": {\"username\": \"高可心\"},\"luckDrawPrize\":{\"name\": \"SoonyAngle盲盒\"}},\n" +
			"    {\"user\": {\"username\": \"高可云\"},\"luckDrawPrize\":{\"name\": \"宇航员蓝牙音箱\"}},\n" +
			"    {\"user\": {\"username\": \"王胜利\"},\"luckDrawPrize\":{\"name\": \"有道词典笔\"}},\n" +
			"    {\"user\": {\"username\": \"李木子\"},\"luckDrawPrize\":{\"name\": \"百科全书\"}}\n" +
			"]";

}
