package com.ym.mec.student.controller;

import com.wf.captcha.SpecCaptcha;
import com.wf.captcha.utils.CaptchaUtil;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.service.SmsCodeService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.security.SecurityConstants;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("${app-config.url.student:}/code")
@Api(tags = "验证码服务")
public class SmsCodeController extends BaseController {

    @Autowired
    private SmsCodeService smsCodeService;
    @Autowired
    private RedissonClient redissonClient;

    @ApiOperation(value = "发送登录短信验证码")
    @ApiImplicitParam(name = "mobile", value = "手机号", required = true, dataType = "String")
    @PostMapping(value = "/sendSms")
    public Object sendLoginVerifyCode(String mobile) throws Exception {
        throw new BizException("你所使用的不是最新版本，请更新后使用");
    }

    @ApiOperation(value = "发送登录短信验证码")
    @ApiImplicitParams({ @ApiImplicitParam(name = "mobile", value = "手机号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "code", value = "验证码", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "验证码类型(register  login  director)", required = true, dataType = "String")})
    @PostMapping(value = "/sendSmsCode")
    public HttpResponseResult<Object> sendLoginVerifyCode1(String mobile, String code,String type) throws Exception {
        if(StringUtils.isEmpty(mobile) || StringUtils.isEmpty(code)){
            return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
        }
        String redisKey = MessageTypeEnum.KAPTCHA_SESSION_KEY + mobile;
        RBucket<String> bucket = redissonClient.getBucket(redisKey);
        if(bucket.isExists()){
            if(StringUtils.equalsIgnoreCase(bucket.get(),code)){
                smsCodeService.sendValidCode(mobile,"STUDENT",StringUtils.isEmpty(type) ? MessageTypeEnum.SMS_VERIFY_CODE_LOGIN.name() : type);
                return succeed();
            }
        }
        return failed(SecurityConstants.VERIFY_FAILURE);
    }

    @ApiOperation(value = "校验短信验证码")
    @ApiImplicitParams({ @ApiImplicitParam(name = "phone", value = "手机号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "code", value = "短信验证码", required = true, dataType = "String") })
    @PostMapping(value = "/verifySmsCode")
    public Object verifySmsCode(String phone,String code) {
        if(StringUtils.isEmpty(phone) || StringUtils.isEmpty(code)){
            return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
        }
        if(smsCodeService.verifyValidCode(phone,code)){
            return succeed();
        }
        return failed("验证码校验失败");
    }

    @PostMapping(value = "/verifyLoginImage")
    @ApiOperation("校验登录图形验证码")
    @ApiImplicitParams({ @ApiImplicitParam(name = "phone", value = "手机号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "code", value = "验证码", required = true, dataType = "String") })
    public Object verifyImageCode(String phone,String code){
        if(StringUtils.isEmpty(phone) || StringUtils.isEmpty(code)){
            return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
        }
        String redisKey = MessageTypeEnum.KAPTCHA_SESSION_KEY + phone;
        RBucket<String> bucket = redissonClient.getBucket(redisKey);
        if(bucket.isExists()){
            if(StringUtils.equalsIgnoreCase(bucket.get(),code)){
                return succeed();
            }
        }
        return failed(SecurityConstants.VERIFY_FAILURE);
    }

    @RequestMapping("/getLoginImage")
    @ApiOperation("获取登录图片验证码")
    @ApiImplicitParam(name = "phone", value = "手机号", required = true, dataType = "String")
    public void getLoginImage(HttpServletRequest request, HttpServletResponse response,String phone) throws Exception {
        if(StringUtils.isEmpty(phone)){
            throw new BizException("请输入手机号");
        }
        SpecCaptcha specCaptcha = new SpecCaptcha(125, 45, 4);
        redissonClient.getBucket(MessageTypeEnum.KAPTCHA_SESSION_KEY + phone).set(specCaptcha.text(),3,TimeUnit.MINUTES);
        CaptchaUtil.out(specCaptcha, request, response);
    }
}
