package com.ym.mec.student.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.entity.StudentCoursewarePlayRecord;
import com.ym.mec.biz.dal.wrapper.StudentCoursewarePlayRecordWrapper;
import com.ym.mec.biz.service.StudentCoursewarePlayRecordService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.student:}/studentCoursewarePlayRecord")
@Api(tags = "学生课件播放统计记录")
public class StudentCoursewarePlayRecordController extends BaseController {

    @Resource
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private StudentCoursewarePlayRecordService studentCoursewarePlayRecordService;

    @ApiOperation(value = "详情", notes = "学生课件播放统计记录-根据详情ID查询单条, 传入id")
    @PreAuthorize("@pcs.hasPermissions('studentCoursewarePlayRecord/detail')")
//    @GetMapping("/detail/{id}")
    public HttpResponseResult<StudentCoursewarePlayRecord> detail(@PathVariable("id") Long id) {

        StudentCoursewarePlayRecord wrapper = studentCoursewarePlayRecordService.detail(id);

        return succeed(wrapper);
    }

    @ApiOperation(value = "查询分页", notes = "学生课件播放统计记录- 传入 StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecordQuery")
    @PreAuthorize("@pcs.hasPermissions('studentCoursewarePlayRecord/page')")
//    @PostMapping("/page")
    public HttpResponseResult<PageInfo<StudentCoursewarePlayRecord>> page(@RequestBody StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecordQuery query) {

        IPage<StudentCoursewarePlayRecord> pages = studentCoursewarePlayRecordService.selectPage(QueryInfo.getPage(query), query);

        return succeed(PageUtil.pageInfo(pages));
    }

    @ApiOperation(value = "新增", notes = "学生课件播放统计记录- 传入 StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecord")
    @PostMapping("/save")
    public HttpResponseResult<JSONObject> add(@Validated @RequestBody StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecordSave studentCoursewarePlayRecord) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        studentCoursewarePlayRecord.setUserId(sysUser.getId());
        studentCoursewarePlayRecord.setOrganizationId(sysUser.getOrganId());
        // 新增数据
        studentCoursewarePlayRecordService.save(studentCoursewarePlayRecord);

        return succeed();
    }

    @ApiOperation(value = "修改", notes = "学生课件播放统计记录- 传入 StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecord")
    @PreAuthorize("@pcs.hasPermissions('studentCoursewarePlayRecord/update')")
//    @PostMapping("/update")
    public HttpResponseResult<JSONObject> update(@Validated @RequestBody StudentCoursewarePlayRecord studentCoursewarePlayRecord) {

        // 更新数据
        studentCoursewarePlayRecordService.updateById(studentCoursewarePlayRecord);

        return succeed();
    }

    @ApiOperation(value = "删除", notes = "学生课件播放统计记录- 传入id")
    @PreAuthorize("@pcs.hasPermissions('studentCoursewarePlayRecord/remove')")
//    @PostMapping("/remove")
    public HttpResponseResult<Boolean> remove(@RequestParam Long id) {

        return succeed(studentCoursewarePlayRecordService.removeById(id));
    }
}