package com.ym.mec.student.controller;

import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.entity.TempDirectorTrainingCamp;
import com.ym.mec.biz.dal.entity.TempDirectorTrainingCampDetail;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.dal.wrapper.TempDirectorTrainingCampDetailWrapper;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.biz.service.TempDirectorTrainingCampDetailService;
import com.ym.mec.biz.service.TempDirectorTrainingCampService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

@Api(tags = "乐队指导训练营活动")
@RestController
@RequestMapping("${app-config.url.student:}/tempDirectorTrainingCamp")
public class TempDirectorTrainingCampController extends BaseController {

    @Resource
    private TempDirectorTrainingCampService tempDirectorTrainingCampService;
    @Resource
    private TempDirectorTrainingCampDetailService tempDirectorTrainingCampDetailService;
    @Resource
    private StudentPaymentOrderService studentPaymentOrderService;

    @ApiOperation(value="获取训练营详情")
    @GetMapping(value = "/getDetail")
    public HttpResponseResult<TempDirectorTrainingCamp> getDetail(Integer campId){
        return succeed(tempDirectorTrainingCampService.lambdaQuery()
                .eq(TempDirectorTrainingCamp::getId,campId)
                .eq(TempDirectorTrainingCamp::getDelFlag,false)
                .last("LIMIT 1").one());
    }

    @ApiOperation(value="根据身份证号、手机号获取用户基本信息")
    @PostMapping(value = "/getUserDetail")
    public HttpResponseResult<TempDirectorTrainingCampDetail> getUserDetail(@RequestBody TempDirectorTrainingCampDetailWrapper.UserDetail userDetail){
        if(userDetail.getTempDirectorTrainingCampId() == null || (StringUtils.isEmpty(userDetail.getCardNo()) && StringUtils.isEmpty(userDetail.getMobile()))){
            throw new BizException("参数校验失败");
        }
        TempDirectorTrainingCampDetail one = tempDirectorTrainingCampDetailService.lambdaQuery()
                .eq(TempDirectorTrainingCampDetail::getTempDirectorTrainingCampId,userDetail.getTempDirectorTrainingCampId())
                .eq(StringUtils.isNotEmpty(userDetail.getCardNo()), TempDirectorTrainingCampDetail::getCardNo, userDetail.getCardNo())
                .eq(StringUtils.isNotEmpty(userDetail.getMobile()), TempDirectorTrainingCampDetail::getMobile, userDetail.getMobile()).last("LIMIT 1").one();
        return succeed(one);
    }

    @ApiOperation(value="训练营报名")
    @PostMapping(value = "/applyCamp")
    public HttpResponseResult<TempDirectorTrainingCampDetailWrapper.ApplyCampResult> applyCamp(@Valid @RequestBody TempDirectorTrainingCampDetailWrapper.ApplyCamp applyCamp){
        return succeed(tempDirectorTrainingCampDetailService.applyCamp(applyCamp));
    }

    @ApiOperation(value = "训练营报名支付")
    @PostMapping("/pay")
    public HttpResponseResult pay(@RequestBody TempDirectorTrainingCampDetailWrapper.PayDto payDto) throws Exception {
        return tempDirectorTrainingCampDetailService.pay(payDto);
    }

    @ApiOperation(value="获取支付详情")
    @GetMapping(value = "/getPayDetail")
    public HttpResponseResult<TempDirectorTrainingCampDetailWrapper.UserPayDetail> getPayDetail(Integer campId){
        return succeed(tempDirectorTrainingCampDetailService.detail(campId));
    }

    @ApiOperation(value="是否有待支付的订单")
    @GetMapping(value = "/hasUnpaidOrder")
    public HttpResponseResult<Boolean> hasUnpaidOrder(Integer campId){
        TempDirectorTrainingCampDetail campDetail = tempDirectorTrainingCampDetailService.getById(campId);
        List<StudentPaymentOrder> studentPaymentOrders = studentPaymentOrderService.queryByCondition(GroupType.SPORADIC,
                campId.toString(),
                campDetail.getUserId(),
                DealStatusEnum.ING,
                OrderTypeEnum.DIRECTOR_TRAINING_CAMP);
        return succeed(CollectionUtils.isNotEmpty(studentPaymentOrders));
    }
}

