package com.ym.mec.student.controller.open;

import com.ym.mec.biz.dal.vo.ImLiveBroadcastRoomVo;
import com.ym.mec.biz.dal.vo.RoomUserInfoVo;
import com.ym.mec.biz.service.ImLiveBroadcastRoomService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.entity.ImUserState;
import com.ym.mec.common.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * 直播房间开放接口
 *
 */
@Api(tags = "直播房间开放接口")
@RestController
@RequestMapping("${app-config.url.student:}/open/liveBroadcastRoom")
public class OpenLiveBroadcastRoomController extends BaseController {

    /**
     * 服务对象
     */
    @Resource
    private ImLiveBroadcastRoomService imLiveBroadcastRoomService;

    @ApiOperation("直播间游客微信身份")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "微信身份code", required = true, dataType = "String"),
    })
    @GetMapping("/visitorFingerprint")
    public HttpResponseResult<String> visitorFingerprint(@RequestParam String code) {

        // 游客访问直播间凭据信息
        return succeed(imLiveBroadcastRoomService.visitorFingerprint(code));
    }

    @ApiOperation("直播间游客访问凭据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roomUid", value = "直播间编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "fingerprint", value = "游客凭据", required = true, dataType = "String")
    })
    @GetMapping("/visitorCredentials")
    public HttpResponseResult<RoomUserInfoVo> visitorCredentials(@RequestParam String roomUid,
                                                                 @RequestParam String fingerprint) {

        // 游客访问直播间凭据信息
        return succeed(imLiveBroadcastRoomService.getVisitorCredentials(roomUid, fingerprint));
    }


    @ApiOperation("游客查询房间信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roomUid", value = "房间编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "游客编号", required = true, dataType = "Integer"),
    })
    @GetMapping("/visitorRoomInfo")
    public HttpResponseResult<ImLiveBroadcastRoomVo> visitorRoomInfo(@RequestParam String roomUid, @RequestParam Integer userId) {

        return succeed(imLiveBroadcastRoomService.visitorRoomInfo(roomUid, userId));
    }

    @ApiOperation("游客-进入房间")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roomUid", value = "房间编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "游客编号", required = true, dataType = "Integer"),
    })
    @GetMapping("/visitorJoinRoom")
    public HttpResponseResult<Object> visitorJoinRoom(@RequestParam String roomUid, @RequestParam Integer userId) {

        // 校验请求参数
        if (StringUtils.isAnyBlank(roomUid) || Objects.isNull(userId)) {
            throw new BizException("请求参数错误");
        }

        // 游客加入直播间
        imLiveBroadcastRoomService.visitorJoinRoom(roomUid, userId);

        return succeed();
    }

    @ApiOperation("游客退出直播间")
    @PostMapping("/visitorQuitRoom")
    public HttpResponseResult<Object> visitorQuitRoom(@RequestBody List<ImUserState> userState) {
        imLiveBroadcastRoomService.opsRoom(userState);
        return succeed();
    }

}

