package com.ym.mec.teacher.controller;

import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dao.VipGroupDefaultClassesUnitPriceDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.entity.VipGroupDefaultClassesUnitPrice;
import com.ym.mec.biz.dal.page.ActivityUserQueryInfo;
import com.ym.mec.biz.service.ActivityUserMapperService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Objects;

@Api(tags = "活动排课")
@RequestMapping("${app-config.url.teacher:}/activityUserMapper")
@RestController
public class ActivityUserMapperController extends BaseController {

    @Autowired
    private ActivityUserMapperService activityUserMapperService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private VipGroupDefaultClassesUnitPriceDao vipGroupDefaultClassesUnitPriceDao;

    @ApiOperation(value = "获取老师待排课课程类型列表")
    @RequestMapping("/queryWaitCourseCategory")
    public HttpResponseResult<List<CourseFormDto>> queryWaitCourse(){
        return succeed(activityUserMapperService.queryWaitCourseCategory(sysUserService.getUserId(),null));
    }

    @ApiOperation(value = "获取活动排课学员列表")
    @PostMapping("/queryActivityStudentList")
    public HttpResponseResult<List<ActivityUserDto>> queryActivityStudentList(@RequestBody ActivityUserQueryInfo activityUserQueryInfo){
        if(activityUserQueryInfo.getCategoryId() == null){
            throw new BizException("请选择课程类型");
        }
        if(activityUserQueryInfo.getCategoryId().equals(0)){
            activityUserQueryInfo.setCategoryId(null);
        }
        activityUserQueryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(activityUserMapperService.queryActivityStudentList(activityUserQueryInfo));
    }

    @ApiOperation(value = "获取活动排课可排课时长列表")
    @PostMapping("/queryActivityCourseTimeList")
    public HttpResponseResult<List<Integer>> queryActivityCourseTimeList(@RequestBody ActivityUserQueryInfo activityUserQueryInfo){
        if(activityUserQueryInfo.getCategoryId() == null){
            throw new BizException("请选择课程类型");
        }
        if(activityUserQueryInfo.getCategoryId().equals(0)){
            activityUserQueryInfo.setCategoryId(null);
        }
        activityUserQueryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(activityUserMapperService.queryActivityCourseTimeList(activityUserQueryInfo));
    }


    @ApiOperation(value = "教师端活动排课")
    @PostMapping("/createVipGroup")
    public HttpResponseResult createVipGroup(@RequestBody VipGroupApplyDto vipGroupApplyDto){
        VipGroupApplyBaseInfoDto applyBaseInfo = vipGroupApplyDto.getVipGroupApplyBaseInfo();
        if(Objects.isNull(applyBaseInfo.getEducationalTeacherId())){
            throw new BizException("请选择乐团主管");
        }
        applyBaseInfo.setUserId(sysUserService.getUserId());
        Teacher teacher = teacherDao.get(applyBaseInfo.getUserId());
        applyBaseInfo.setOrganId(teacher.getTeacherOrganId());
        vipGroupApplyDto.getCourseSchedules().stream().forEach(e->e.setEndClassTime(DateUtil.addMinutes(e.getStartClassTime(), applyBaseInfo.getSingleClassMinutes())));

        // 默认VIP课
        if (StringUtils.isBlank(applyBaseInfo.getGroupType())) {
            applyBaseInfo.setGroupType("VIP");
        }

        VipGroupDefaultClassesUnitPrice vipGroupDefaultClassesUnitPrice = vipGroupDefaultClassesUnitPriceDao.getByVipGroupCategory(applyBaseInfo.getVipGroupCategoryId(),
                applyBaseInfo.getOrganId(), applyBaseInfo.getGroupType(),null);
        if(Objects.nonNull(vipGroupDefaultClassesUnitPrice)){
            applyBaseInfo.setOnlineClassesUnitPrice(vipGroupDefaultClassesUnitPrice.getOnlineClassesUnitPrice());
            applyBaseInfo.setOfflineClassesUnitPrice(vipGroupDefaultClassesUnitPrice.getOfflineClassesUnitPrice());
        }
        return activityUserMapperService.createActivityVipGroup(vipGroupApplyDto);
    }

    @ApiOperation(value = "教师端网管课活动排课")
    @PostMapping("/createPracticeGroup")
    public HttpResponseResult createPracticeGroup(@RequestBody PracticeGroupApplyDto practiceGroupApplyDto){
        PracticeGroupApplyBaseInfoDto applyBaseInfo = practiceGroupApplyDto.getPracticeGroupApplyBaseInfoDto();
        if(Objects.isNull(applyBaseInfo.getEducationalTeacherId())){
            throw new BizException("请选择乐团主管");
        }
        applyBaseInfo.setUserId(sysUserService.getUserId());
        Teacher teacher = teacherDao.get(applyBaseInfo.getUserId());
        applyBaseInfo.setOrganId(teacher.getTeacherOrganId());
        for (CourseSchedule courseSchedule : practiceGroupApplyDto.getCourseSchedules()) {
            courseSchedule.setEndClassTime(DateUtil.addMinutes(courseSchedule.getStartClassTime(), applyBaseInfo.getSingleClassMinutes()));
        }
        return activityUserMapperService.createActivityPracticeGroup(practiceGroupApplyDto);
    }
}
