package com.ym.mec.teacher.controller;

import com.ym.mec.biz.dal.dao.CourseScheduleStudentPaymentDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.CourseScheduleEvaluate;
import com.ym.mec.biz.dal.page.CourseScheduleQueryInfo;
import com.ym.mec.biz.dal.page.ImGroupNoticeQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.HttpStatus;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Author Joburgess
 * @Date 2019/10/17
 */
@Api(tags = "班级服务")
@RequestMapping("${app-config.url.teacher:}/classGroup")
@RestController
public class ClassGroupController extends BaseController {
    @Resource
    private ClassGroupService classGroupService;
    @Resource
    private CourseScheduleEvaluateService courseScheduleEvaluateService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private ImGroupNoticeService imGroupNoticeService;
    @Resource
    private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
    @Resource
    private SubjectService subjectService;

    @ApiOperation(value = "教师关联班级获取")
    @GetMapping("/findTeacherClassGroups")
    public HttpResponseResult<List<TeacherClassGroupDto>> findTeacherClassGroups(String type, String status, String groupName){
        return succeed(classGroupService.findTeacherClassGroups(type,status,groupName));
    }

    @ApiOperation(value = "获取教师班级详情界面头部信息")
    @GetMapping("/findTeacherClassGroupInfo")
    public HttpResponseResult<TeacherClassHeadInfo> findTeacherClassGroupInfo(@ApiParam(value = "班级编号", required = true)Integer classGroupId){
        return succeed(classGroupService.findTeacherClassGroupInfo(classGroupId));
    }

    @ApiOperation(value = "获取班级对应学生列表")
    @GetMapping("/findTeacherClassStudents")
    public HttpResponseResult<PageInfo<TeacherClassStudentDto>> findTeacherClassStudents(CourseScheduleQueryInfo queryInfo){
        return succeed(classGroupService.findTeacherClassStudents(queryInfo));
    }

    @ApiOperation(value = "获取课程学员列表")
    @GetMapping("/getCourseStudents")
    public HttpResponseResult<List<BasicUserDto>> getCourseStudents(Long courseScheduleId){
        List<BasicUserDto> courseStudents = courseScheduleStudentPaymentDao.getCourseStudents(courseScheduleId);

        // 乐团声部为空，取student的声部
        if (!CollectionUtils.isEmpty(courseStudents)) {
            Set<Long> collect = courseStudents.stream().map(o -> o.getUserId()).map(o -> o.longValue()).collect(Collectors.toSet());

            Map<Integer, StudentSubjectDto> subjectDtoMap = subjectService.getSubjectByStudentId(collect);
            courseStudents.forEach(studentAttendanceViewDto -> {
                if (studentAttendanceViewDto.getSubjectIdList() == null) {
                    StudentSubjectDto subjectDto = subjectDtoMap.getOrDefault(studentAttendanceViewDto.getUserId(), null);
                    if (subjectDto.getSubjectId() != null) {
                        studentAttendanceViewDto.setSubjectName(subjectDto.getSubjectName());
                        studentAttendanceViewDto.setSubjectId(subjectDto.getSubjectId());
                        studentAttendanceViewDto.setSubjectIdList(subjectDto.getSubjectId().toString());
                    }
                }
            });
        }
        return succeed(courseStudents);
    }

    @ApiOperation(value = "提交陪练报告")
    @PostMapping(value = "/addStudyReport")
    public HttpResponseResult<Boolean> addStudyReport(@RequestBody CourseScheduleEvaluate courseScheduleEvaluate) {
        if(courseScheduleEvaluate.getClassGroupId()==null || courseScheduleEvaluate.getClassGroupId()<0){
            return failed(HttpStatus.BAD_REQUEST,"班级id必须大于0");
        }
        if(courseScheduleEvaluate.getItem()==null || courseScheduleEvaluate.getItem().isEmpty()){
            return failed(HttpStatus.BAD_REQUEST,"课程评价选项不能为空");
        }
        courseScheduleEvaluate.setTeacherId(sysUserService.getUserId());
        return succeed(courseScheduleEvaluateService.addStudyReport(courseScheduleEvaluate));
    }

    @ApiOperation(value = "根据群编号，获取用户群名片")
    @GetMapping("/getUserInfoWithUserId")
    public HttpResponseResult<Map<String,Object>> getUserInfoWithUserId(Integer groupId,Integer userId) {
        if (groupId == null || userId == null) {
            return failed("参数校验错误");
        }
        return succeed(classGroupService.getUserInfoWithUserId(groupId,userId));
    }

    @ApiOperation(value = "获取最新一条群公告")
    @RequestMapping("/getNewNotice")
    public HttpResponseResult<ImGroupNoticeDto> getNewNotice(String groupId) {
        return succeed(imGroupNoticeService.queryLatestNotice(groupId));
    }

    @ApiOperation(value = "获取群公告列表")
    @RequestMapping("/queryNoticePage")
    public HttpResponseResult<PageInfo<ImGroupNoticeDto>> queryPage(ImGroupNoticeQueryInfo queryInfo) {
        return succeed(imGroupNoticeService.queryPage(queryInfo));
    }

}
