package com.ym.mec.teacher.controller;

import com.ym.mec.biz.dal.dao.ClassGroupStudentMapperDao;
import com.ym.mec.biz.dal.dao.StudentRegistrationDao;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.entity.ExtracurricularExercises;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.page.ExtraExercilseQueryInfo;
import com.ym.mec.biz.dal.page.TeacherServeHomeworkQueryInfo;
import com.ym.mec.biz.service.ExtracurricularExercisesReplyService;
import com.ym.mec.biz.service.ExtracurricularExercisesService;
import com.ym.mec.biz.service.StudentExtracurricularExercisesSituationService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author Joburgess
 * @Date 2020/3/26
 */
@Api(tags = "课外训练服务")
@RequestMapping("${app-config.url.teacher:}/extracurricularExercises")
@RestController
public class ExtracurricularExercisesController extends BaseController {

    @Resource
    private ExtracurricularExercisesService extracurricularExercisesService;
    @Resource
    private ExtracurricularExercisesReplyService extracurricularExercisesReplyService;
    @Resource
    private StudentExtracurricularExercisesSituationService studentExtracurricularExercisesSituationService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private StudentRegistrationDao studentRegistrationDao;
    @Resource
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;
    @Resource
    private SubjectDao subjectDao;

    @ApiOperation(value = "获取教师有服务指标的课程列表")
    @GetMapping("/queryTeacherServeHomeworkDetail")
    public HttpResponseResult<PageInfo<TeacherServeHomeworkPojo>> queryTeacherServeHomeworkDetail(TeacherServeHomeworkQueryInfo queryInfo) {
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherServeHomeworkDetail1(queryInfo));
    }


    @ApiOperation(value = "获取教师有服务指标的课程列表")
    @GetMapping("/queryTeacherServeHomeworkDetail/v2")
    public HttpResponseResult<PageInfo<CourseHomeworkWrapper.CourseHomeworkList>> queryTeacherServeHomeworkDetailV2(CourseHomeworkWrapper.CourseHomeworkQuery queryInfo) {
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherServeHomeworkDetailV2(queryInfo));
    }

    @ApiOperation(value = "获取教师课外作业列表")
    @GetMapping("/queryTeacherExtraHomeworkDetail/v2")
    public HttpResponseResult<PageInfo<CourseHomeworkWrapper.CourseHomeworkList>> queryTeacherExtraHomeworkDetailV2(CourseHomeworkWrapper.CourseHomeworkQuery queryInfo) {
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherExtraHomeworkDetailV2(queryInfo));
    }


    @ApiOperation(value = "获取老师训练统计")
    @GetMapping("/getStatistics")
    public HttpResponseResult<TeacherExercisesServiceDto> getStatistics(ExtraExercilseQueryInfo queryInfo) {
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(studentExtracurricularExercisesSituationService.getStatistics(queryInfo));
    }

    @ApiOperation(value = "统计老师当前时间能布置作业的课程数量")
    @GetMapping("/countWaitCreateHomeworkNum")
    public HttpResponseResult<Integer> countWaitCreateHomeworkNum() {
        return succeed(studentExtracurricularExercisesSituationService.countWaitCreateHomeworkNum());
    }

    @ApiOperation(value = "创建课外训练")
    @PostMapping("/createExtraExercises")
    @AuditLogAnnotation(operateName = "创建课外训练")
    private HttpResponseResult<Object> createExtraExercises(ExtracurricularExercises exercises){
        exercises.setTeacherId(sysUserService.getUserId());
        exercises.setGroupType(ELessonTrainingType.EXTRACURRICULAR.getCode());
        extracurricularExercisesService.createExtraExercises(exercises);
        return succeed();
    }

    @ApiOperation(value = "创建课外训练")
    @PostMapping("/createExtraExercisesForJson")
    @AuditLogAnnotation(operateName = "创建课外训练")
    private HttpResponseResult<Object> createExtraExercisesForJson(@RequestBody ExtracurricularExercises exercises){
        if(null == exercises.getExpireDate()){
            Calendar instance = Calendar.getInstance();
            instance.add(Calendar.WEEK_OF_MONTH,1);
            exercises.setExpireDate(instance.getTime());
        }

        exercises.setTeacherId(sysUserService.getUserId());
        extracurricularExercisesService.createExtraExercises(exercises);
        return succeed();
    }

    @ApiOperation(value = "获取课外训练列表")
    @GetMapping("/findExtraExercilses")
    private HttpResponseResult<List<Map<String, Object>>> findExtraExercilses(ExtraExercilseQueryInfo queryInfo){
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(extracurricularExercisesService.findExtraExercilses(queryInfo));
    }

    @ApiOperation(value = "获取已布置训练列表（包含课后，课外）")
    @GetMapping("/findExtraExercilsesHomeworks")
    private HttpResponseResult<PageInfo<TeacherHomeworkListDto>> findExtraExercilsesHomeworks(ExtraExercilseQueryInfo queryInfo){
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(extracurricularExercisesService.findExtraExercilsesHomeworks(queryInfo));
    }

    @ApiOperation(value = "获取课外训练详情")
    @GetMapping("/findStudentExtraExerciseDetail")
    private HttpResponseResult findStudentExtraExerciseDetail(Long studentExerciseId){
        if(Objects.isNull(studentExerciseId)){
            return failed("请选择课外训练");
        }
        return succeed(extracurricularExercisesReplyService.findStudentExtraExerciseDetail(studentExerciseId));
    }

    @ApiOperation(value = "获取待布置学员列表")
    @GetMapping("/getDontServeStudents")
    public HttpResponseResult getDontServeStudents(String search, String musicGroupId, Long classGroupId, Integer subjectId,Integer hasMember,String studentIds){
        return succeed(extracurricularExercisesService.getDontServeStudents(sysUserService.getUserId(), search, musicGroupId, classGroupId, subjectId,hasMember,studentIds));
    }

    @ApiOperation(value = "获取待布置学员乐团列表")
    @GetMapping("/getDontServeStudentMusicGroups")
    public HttpResponseResult getDontServeStudentMusicGroups(String search){
        List<BasicUserDto> dontServeStudents = extracurricularExercisesService.getDontServeStudents(sysUserService.getUserId(), search, null, null,null,null,null);
        if(CollectionUtils.isEmpty(dontServeStudents)){
            return succeed(Collections.emptyList());
        }
        Set<Integer> studentIds = dontServeStudents.stream().map(BasicUserDto::getUserId).collect(Collectors.toSet());
        List<MusicGroup> studentNormalMusicGroups = studentRegistrationDao.getStudentNormalMusicGroups(studentIds);
        return succeed(studentNormalMusicGroups);
    }

    @ApiOperation(value = "获取服务学员查询条件")
    @GetMapping("/getDontServeStudentQueryCondition")
    public HttpResponseResult getDontServeStudentQueryCondition(String musicGroupId){
        Map<String, Object> result = new HashMap<>();
        Integer userId = sysUserService.getUserId();
        List<BasicUserDto> dontServeStudents = extracurricularExercisesService.getDontServeStudents(userId, null, musicGroupId, null, null,null,null);
        if(CollectionUtils.isEmpty(dontServeStudents)){
            result.put("musicGroups", Collections.emptyList());
            result.put("classGroups", Collections.emptyList());
            result.put("subjects", Collections.emptyList());
            return succeed(result);
        }
        Set<Integer> studentIds = dontServeStudents.stream().map(BasicUserDto::getUserId).collect(Collectors.toSet());
        List<MusicGroup> studentNormalMusicGroups = studentRegistrationDao.getStudentNormalMusicGroups(studentIds);
        result.put("musicGroups", studentNormalMusicGroups);

        List<ClassGroup> classGroups = classGroupStudentMapperDao.getStudentClassGroupWithTeacher(studentIds, userId, musicGroupId);
        result.put("classGroups", classGroups);

//        List<Subject> studentSubjects = studentDao.getStudentSubjects(studentIds);
//        result.put("subjects", studentSubjects);
        List<Integer> subjectIds = dontServeStudents.stream().map(BasicUserDto::getSubjectId).collect(Collectors.toList());
        List<Subject> studentSubjects = subjectDao.findBySubjectIds(subjectIds);
        result.put("subjects", studentSubjects);
        return succeed(result);
    }
}
