package com.ym.mec.teacher.controller;

import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.ym.mec.biz.dal.dto.ImGroupMemberDto;
import com.ym.mec.biz.dal.page.GroupMemberQueryInfo;
import com.ym.mec.biz.dal.wrapper.ImGroupMemberWrapper;
import com.ym.mec.biz.service.ImGroupMemberService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Optional;

@RequestMapping("${app-config.url.teacher:}/imGroupMember")
@Api(tags = "IM群成员服务")
@RestController
public class ImGroupMemberController extends BaseController {

	@Resource
	private ImGroupMemberService imGroupMemberService;

	@ApiOperation(value = "群成员禁言", notes = "群成员禁言- 传入 ImGroupMemberVo.GroupMemberMute")
	@PostMapping("/groupMute")
	public HttpResponseResult<Object> groupMemberMute(@RequestBody ImGroupMemberWrapper.GroupMemberMute member) {
		// 群成员不能为空
		if (CollectionUtils.isEmpty(member.getUserIds())) {
			throw BizException.from("群成员不能为空");
		}
		// 更新群成员禁言
		imGroupMemberService.groupMemberMute(member);
		return succeed();
	}

	@ApiOperation(value = "群成员身份设置", notes = "群成员身份设置- 传入 ImGroupMemberVo.GroupMemberRole")
	@PostMapping("/groupRole")
	public HttpResponseResult<Object> groupMemberRole(@RequestBody ImGroupMemberWrapper.GroupMemberRole member) {
		// 群成员不能为空
		if (CollectionUtils.isEmpty(member.getUserIds())) {
			throw BizException.from("群成员不能为空");
		}
		// 更新群成员禁言
		imGroupMemberService.groupMemberRole(member);
		return succeed();
	}

	@ApiOperation("分页查询群成员列表")
	@PostMapping(value = "/queryGroupMemberPage")
	public HttpResponseResult<PageInfo<ImGroupMemberDto>> queryGroupMemberPage(GroupMemberQueryInfo queryInfo) {
		return succeed(imGroupMemberService.queryGroupMemberPage(queryInfo));
	}

	@ApiOperation("添加群成员")
	@PostMapping(value = "/addGroupMember")
	@ApiImplicitParams({ @ApiImplicitParam(name = "imGroupId", value = "群编号", required = true, dataType = "String"),
			@ApiImplicitParam(name = "roleType", value = "指导老师,乐队指导,乐团主管,运营主管,学员不传", required = true, dataType = "String"),
			@ApiImplicitParam(name = "userId", value = "用户编号（多个逗号分割）", required = true, dataType = "String")})
	public Object addGroupMember(String imGroupId, String userId, String roleType) {
		imGroupMemberService.batchJoin(imGroupId,userId,roleType);
		return succeed();
	}
}
