package com.ym.mec.teacher.controller;

import com.ym.mec.biz.dal.dto.BaseMapDto;
import com.ym.mec.biz.dal.entity.StudentAttendance;
import com.ym.mec.biz.dal.enums.StudentAttendanceStatusEnum;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.CloudTeacherActiveWapper;
import com.ym.mec.biz.dal.page.EndCourseScheduleQueryInfo;
import com.ym.mec.biz.dal.page.StudentAttendanceQueryInfo;
import com.ym.mec.biz.dal.page.StudentManageQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Api(tags = "学生管理")
@RequestMapping("${app-config.url.teacher:}/student")
@RestController
public class StudentController extends BaseController {

    @Resource
    private SysUserCashAccountService sysUserCashAccountService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private StudentManageService studentManageService;
    @Resource
    private CourseScheduleService courseScheduleService;
    @Resource
    private StudentAttendanceService studentAttendanceService;

    @ApiOperation(value = "获取学员异常考勤列表")
    @PostMapping("/studentAttendanceError")
    public Object superFindCourseSchedules(@RequestBody EndCourseScheduleQueryInfo queryInfo){
        queryInfo.setTeacherIdList(sysUserService.getUserId().toString());
        queryInfo.setSearchType("STUDENT_ERR_ATTENDANCE");
        Date date = new Date();
        queryInfo.setStartTime(DateUtil.format(DateUtil.getFirstDayOfMonth(DateUtil.addMonths(date, -1)),DateUtil.DEFAULT_PATTERN));
        queryInfo.setEndTime(DateUtil.format(date,DateUtil.DEFAULT_PATTERN));
        return succeed(courseScheduleService.endFindCourseSchedules(queryInfo));
    }

    @ApiOperation(value = "获取某节课学生签到列表")
    @RequestMapping("/findStudentAttendance")
    public HttpResponseResult<List<StudentAttendance>> findStudentAttendance(StudentAttendanceQueryInfo queryInfo){
        //按考勤状态和回访状态排序
        Date date = new Date();
        queryInfo.setStartDateOfCourse(DateUtil.getFirstDayOfMonth(DateUtil.addMonths(date, -1)));
        queryInfo.setEndDateOfCourse(date);
        queryInfo.setOrderFlag(true);
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<StudentAttendance> rows = studentAttendanceService.getDao().findStudentAttendance(params);
        if(CollectionUtils.isNotEmpty(rows)){
            rows = rows.stream().filter(e->e.getStatus() == StudentAttendanceStatusEnum.TRUANT && e.getVisitFlag() == YesOrNoEnum.NO).collect(Collectors.toList());
        }
        return succeed(rows);
    }

    @ApiOperation(value = "获取学员账户信息")
    @GetMapping("/userCashAccount/get")
    public Object getStudentCashAccount(Integer id) {
        Optional.ofNullable(id).orElseThrow(() -> new BizException("学生id不能为空！"));
        return succeed(sysUserCashAccountService.get(id));
    }

    @ApiOperation(value = "云教练统计")
    @PostMapping("/cloudTeacherActive")
    public HttpResponseResult<CloudTeacherActiveWapper.ResultMap> cloudTeacherActive(@RequestBody CloudTeacherActiveWapper.Query queryInfo){
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(studentManageService.cloudTeacherActiveStatistics(queryInfo));
    }


    @ApiOperation(value = "获取学生列表")
    @PostMapping("/queryStudentList")
    public Object queryStudentList(@RequestBody StudentManageQueryInfo queryInfo){
        queryInfo.setTeacherId(sysUserService.getUserId());
        queryInfo.setIsExport(false);
        return succeed(studentManageService.findStudentsByOrganId(queryInfo));
    }

    @ApiOperation(value = "根据学生ID获取其所在的乐团")
    @GetMapping("/findStudentMusicGroups")
    public Object findStudentMusicGroups(@ApiParam(value = "学生编号", required = true) Integer userId){
        return succeed(studentManageService.findStudentMusicGroupsByUserId(userId));
    }

    @ApiOperation(value = "获取老师关联的学员声部列表")
    @GetMapping("/queryStuSubjectId")
    public HttpResponseResult<List<BaseMapDto>> queryStuSubjectId(){
        return succeed(studentManageService.queryStuSubjectId(sysUserService.getUserId()));
    }

}
