package com.ym.mec.teacher.controller;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.SysMessageVo;
import com.ym.mec.biz.dal.enums.MessageSendMode;
import com.ym.mec.biz.dal.page.SysMessageQueryInfo;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.validator.CommonValidator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@RestController
@Api(tags = "消息服务")
@RequestMapping("${app-config.url.teacher:}/sysMessage")
public class SysMessageController extends BaseController {

	@Autowired
	private SysMessageService sysMessageService;

	@Autowired
	private SysUserService sysUserService;

	@ApiOperation("获取所有消息列表")
	@GetMapping(value = "list")
	public Object list(SysMessageQueryInfo queryInfo) throws IOException {
		queryInfo.setUserId(sysUserService.getUserId());
		queryInfo.setType(3);
		queryInfo.setJpushType("TEACHER");
		return succeed(sysMessageService.queryPage(queryInfo));
	}

	@ApiOperation("获取消息分类类型")
	@GetMapping(value = "typeList")
	public Object typeList(){
		return succeed(JSON.parseArray("[{\"value\":\"全部\",\"key\":\"ALL\"}," +
				"{\"value\":\"考勤信息\",\"key\":\"ATTENDANCE\"}," +
				"{\"value\":\"课程信息\",\"key\":\"COURSE\"}," +
				"{\"value\":\"训练信息\",\"key\":\"WORK\"}," +
				"{\"value\":\"其他\",\"key\":\"DEFAULT\"}]"));
	}

	@ApiOperation("一键已读")
	@PostMapping("batchSetRead")
	public Object batchSetRead() {
		return sysMessageService.updateStatus(sysUserService.getUserId(), 1,"TEACHER")? succeed() : failed();
	}

	@ApiOperation("设置已读")
	@PostMapping("setRead")
	public Object setRead(Long id) {
		return sysMessageService.updateOneStatus(id) ? succeed() : failed();
	}

	@ApiOperation(value = "查询用户未读消息条数")
	@GetMapping("/queryCountOfUnread")
	public Object queryCountOfUnread() {
		SysUser sysUser = sysUserService.getUser();
		Map<String, Integer> map = sysMessageService.
				queryCountOfUnread(MessageSendMode.PUSH, sysUser.getId(),"TEACHER", sysUser.getTenantId());
		if (map == null || map.size() == 0) {
			return succeed();
		}
		return succeed(map);
	}

	@ApiOperation(value = "发送消息")
	@PostMapping("/sendMessage")
	public Object sendMessage(MessageSender messageSender, String content, String receiver, int readStatus, String url, String group) {
		SysUser sysUser = sysUserService.getUser();
		String mobileNo = sysUser.getPhone();
		if (StringUtils.isBlank(mobileNo) || !CommonValidator.isMobileNo(mobileNo)) {
			throw new BizException("请输入正确的手机号");
		}
//		sysMessageService.sendMessage(messageSender, sysUser.getId(), "", content, receiver, null, readStatus, url, group,"TEACHER");
		return succeed();
	}
}
