package com.ym.mec.teacher.controller;

import com.ym.mec.auth.api.dto.MusicScoreQueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.page.SysExamSongQueryInfo;
import com.ym.mec.biz.service.CbsMusicScoreCategoriesService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.SysMusicScoreCategoriesService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@RequestMapping("${app-config.url.teacher:}/sysMusicScoreCategories")
@Api(tags = "曲库分类服务")
@RestController
public class SysMusicScoreCategoriesController extends BaseController {

    @Resource
    private SysMusicScoreCategoriesService sysMusicScoreCategoriesService;
    @Resource
    private CbsMusicScoreCategoriesService cbsMusicScoreCategoriesService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private OrganizationService organizationService;
    @Resource
    private TeacherDao teacherDao;


    @ApiOperation(value = "分页查询")
    @GetMapping("/queryPage")
    public Object queryPage(SysExamSongQueryInfo queryInfo) {
        return succeed(sysMusicScoreCategoriesService.queryPage(queryInfo));
    }

    @ApiOperation(value = "树状列表")
    @GetMapping("/queryTree")
    public Object queryTree(MusicScoreQueryInfo queryInfo) {
        SysUser sysUser = sysUserService.getUser();
        //获取老师分部
        Teacher teacher = teacherDao.get(sysUser.getId());
        if (teacher == null) {
            return failed(HttpStatus.BAD_REQUEST, "老师信息不存在");
        }
        String organId = teacher.getTeacherOrganId() + "";
        if (StringUtils.isNotEmpty(teacher.getFlowOrganRange())){
            organId = organId + "," + teacher.getFlowOrganRange();
        }
        String teachingMaterialIds = organizationService.getTeachingMaterialIds(organId,sysUser.getTenantId());
        return succeed(cbsMusicScoreCategoriesService.queryCategoriesTreeList(teachingMaterialIds, queryInfo.getEnable()));
    }

    @ApiOperation(value = "获取分类详情")
    @GetMapping("/get")
    public Object get(Integer id) {
        return succeed(sysMusicScoreCategoriesService.get(id));
    }
}
