package com.ym.mec.teacher.controller;

import com.ym.mec.biz.dal.dao.StudentAttendanceDao;
import com.ym.mec.biz.dal.dto.StudentAttendanceDto;
import com.ym.mec.biz.dal.dto.StudentAttendanceViewDto;
import com.ym.mec.biz.dal.dto.TeacherSignOutDto;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.StudentAttendance;
import com.ym.mec.biz.dal.entity.TeacherAttendance;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.StudentAttendanceStatusEnum;
import com.ym.mec.biz.dal.enums.UpdateAttendanceEnum;
import com.ym.mec.biz.dal.page.TeacherAttendanceQueryInfo;
import com.ym.mec.biz.dal.wrapper.LiveGroupWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.date.DateUtil;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.teacher:}/teacherAttendance")
@Api(tags = "教师签到服务")
@RestController
public class TeacherAttendanceController extends BaseController {

    @Resource
    private TeacherAttendanceService teacherAttendanceService;
    @Resource
    private StudentAttendanceService studentAttendanceService;
    @Resource
    private StudentAttendanceDao studentAttendanceDao;
    @Resource
    private ClassGroupService classGroupService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysMessageService sysMessageService;
    @Resource
    private CourseScheduleService courseScheduleService;
    @Resource
    private MusicGroupService musicGroupService;
    @Resource
    private OrganizationService organizationService;

    @ApiOperation(value = "新增教师签到")
    @PostMapping("/add")
    @AuditLogAnnotation(operateName = "新增教师签到")
    public HttpResponseResult<Map<String, Object>> add(@RequestBody TeacherSignOutDto teacherSignOutDto) {
        return succeed(teacherAttendanceService.addTeacherAttendanceRecord(teacherSignOutDto));
    }

    @ApiOperation(value = "分页查询教师签到列表")
    @GetMapping("/queryPage")
    public HttpResponseResult<PageInfo<TeacherAttendance>> queryPage(TeacherAttendanceQueryInfo queryInfo) {
        return succeed(teacherAttendanceService.queryPage(queryInfo));
    }


    @ApiOperation(value = "根据班级ID获取当前课程的学生")
    @GetMapping("/getCurrentCourseStudents")
    public HttpResponseResult<List<StudentAttendanceViewDto>> getCurrentCourseStudents(Long courseScheduleId){
        return succeed(classGroupService.findAttendanceStudentByCourse(courseScheduleId));
    }



    @ApiOperation(value = "根据班级ID获取当前课程的学生")
    @PostMapping("/getLiveCurrentCourseStudents")
    public HttpResponseResult<List<StudentAttendanceViewDto>> getLiveCurrentCourseStudents(@RequestBody LiveGroupWrapper.LiveCourseStudentQuery query){
        return succeed(classGroupService.getLiveCurrentCourseStudents(query));
    }

    @ApiOperation(value = "获取用户所在分部的年级列表）")
    @GetMapping("/getGradeList")
    public HttpResponseResult<Map<Integer,String>> getGradeList(Long courseScheduleId) {
        CourseSchedule courseSchedule = courseScheduleService.get(courseScheduleId);
        return succeed(organizationService.getGradeList(courseSchedule.getOrganId()));
    }

    @ApiOperation(value = "点名")
    @PostMapping("/addStudentAttendances")
    @AuditLogAnnotation(operateName = "点名")
    public HttpResponseResult<Map<String, Object>> addStudentAttendances(@RequestBody StudentAttendanceDto studentAttendanceInfo){
        return succeed(studentAttendanceService.addStudentAttendances(studentAttendanceInfo));
    }

    @ApiOperation(value = "学员到课提醒")
    @PostMapping("/studentAttendanceNormalRemind")
    public HttpResponseResult<Object> studentAttendanceNormalRemind(Long courseId){
        int remindNum = studentAttendanceDao.countNormalRemindNum(courseId);
        if(remindNum > 0){
            return failed("您已发送过到课提醒");
        }
        CourseSchedule courseSchedule = courseScheduleService.get(courseId);
        //获取正常签到的学生信息
        List<StudentAttendance> studentAttendances = studentAttendanceDao.findByCourseId(courseId);
        if(CollectionUtils.isEmpty(studentAttendances)){
            return succeed();
        }
        List<StudentAttendance> normal = studentAttendances.stream().filter(studentAttendance -> studentAttendance.getStatus() == StudentAttendanceStatusEnum.NORMAL).collect(Collectors.toList());
        //教务推送
        if(Objects.nonNull(courseSchedule)&& GroupType.MUSIC.equals(courseSchedule.getGroupType())){
            MusicGroup musicGroup = musicGroupService.get(courseSchedule.getMusicGroupId());
            if(!Objects.isNull(musicGroup) && !Objects.isNull(musicGroup.getTransactionTeacherId())){
                int normalStudentNum = normal.size();
                long leaveStudentNum = studentAttendances.stream().filter(studentAttendance -> studentAttendance.getStatus() == StudentAttendanceStatusEnum.LEAVE).count();
                long truantStudentNum = studentAttendances.stream().filter(studentAttendance -> studentAttendance.getStatus() == StudentAttendanceStatusEnum.TRUANT).count();
                Map<Integer, String> receivers = new HashMap<>(1);
                receivers.put(musicGroup.getTransactionTeacherId(), musicGroup.getTransactionTeacherId().toString());
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SYSTEM_PUSH_NAMES_ACHIEVE,
                        receivers, null, 0, null, "SYSTEM", courseSchedule.getName(), normalStudentNum, leaveStudentNum, truantStudentNum);
            }
        }

        if (CollectionUtils.isNotEmpty(normal)) {
            Map<Integer, String> receivers = new HashMap<>(normal.size());
            normal.forEach(e -> receivers.put(e.getUserId(), e.getUserId().toString()));
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_NAMES_ACHIEVE,
                    receivers, null, 0, "2", "STUDENT", DateUtil.format(new Date(), DateUtil.DATE_FORMAT_MIN));
        }else{
            return succeed();
        }
        studentAttendanceDao.updateNormalRemind(courseId);
        return succeed();
    }

    @ApiOperation(value = "获取签退详情")
    @GetMapping("/getTeacherSignDetail")
    public HttpResponseResult<TeacherSignOutDto> getTeacherSignDetail(Integer courseScheduleId){
        return succeed(teacherAttendanceService.getTeacherSignDetail(courseScheduleId, sysUserService.getUserId()));
    }

    @ApiOperation(value = "提交考勤申述")
    @PostMapping("/addComplaints")
    public HttpResponseResult<Object> addComplaints(Long courseScheduleId, String content, String url, UpdateAttendanceEnum complaintsType){
        teacherAttendanceService.addComplaints(courseScheduleId,content,url,null,complaintsType);
        return succeed();
    }

    @ApiOperation(value = "撤销考勤申述")
    @PostMapping("/repealComplaints")
    public HttpResponseResult<Object> repealComplaints(Long courseScheduleId){
        teacherAttendanceService.repealComplaints(courseScheduleId,sysUserService.getUserId());
        return succeed();
    }
}
