package com.ym.mec.teacher.controller;

import com.ym.mec.biz.dal.dao.CourseScheduleDao;
import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.im.ClientEnum;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.entity.ImGroupModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.teacher:}/teacher")
@Api(tags = "教师服务")
@Slf4j
@RestController
public class TeacherController extends BaseController {

    @Autowired
    private TeacherService teacherService;
    @Autowired
    private ImGroupNoticeService imGroupNoticeService;
    @Autowired
    private SubjectService subjectService;
    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private SysUserService sysUserService;
	@Autowired
	private SubjectDao subjectDao;
	@Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
	@Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    private ImGroupCoreService imGroupCoreService;
    @Autowired
    private CourseScheduleDao courseScheduleDao;

    @Autowired
    private StudentCourseHomeworkService studentCourseHomeworkService;

    @ApiOperation(value = "修改教师个人中心信息")
    @PostMapping("/update")
    public Object update(Teacher teacher) {
        teacher.setUpdateTime(new Date());
        teacher.setId(sysUserService.getUserId());
        teacherService.updateTea1(teacher);
        return succeed();
    }

    @ApiOperation(value = "获取老师分部列表（包含流动分部）")
    @GetMapping("/queryOrganList")
    public HttpResponseResult<List<BaseMapDto<Integer,String>>> queryOrganList() {
        return succeed(teacherService.queryOrganList(sysUserService.getUserId()));
    }

    @ApiOperation(value = "课程配置课件")
    @GetMapping("/configCourseware")
    public HttpResponseResult configCourseware(Long courseId,String coursewareDetailId) {
        courseScheduleDao.configCourseware(courseId,coursewareDetailId);
        return succeed();
    }

    @ApiOperation(value = "根据教师编号查询教师基本信息")
    @GetMapping("/queryUserInfo")
    public HttpResponseResult<Teacher> queryUserInfo() {
		Teacher teacher = Optional.ofNullable(teacherService.get(sysUserService.getUserId())).
                orElseThrow(()->new BizException("老师信息不存在"));
		//专业技能
		if(StringUtils.isNotEmpty(teacher.getSubjectId())){
			teacher.setSubjectName(subjectDao.findBySubIds(teacher.getSubjectId()));
		}

        // 注册IM用户Token
        try {
            ImGroupWrapper.ImUserInfo register = imGroupCoreService.getUserSig(String.valueOf(teacher.getId()),
                ClientEnum.TEACHER.getCode());

            // 设置用户Token
            teacher.setImToken(register.getImToken());
            // 设置用户IM用户ID
            teacher.setImUserId(register.getImUserId());
        } catch (Exception e) {
            log.error("注册IM用户Token失败", e);
        }

        return succeed(teacher);
    }

    @ApiOperation(value = "根据教师编号查询教师基本信息")
    @GetMapping("/get")
    @ApiParam(value = "教师编号", required = true)
    public HttpResponseResult<Teacher> get(Integer id) {
        return succeed(teacherService.getDetail(id));
    }

    @ApiOperation(value = "根据教师编号查询教师课程班列表")
    @GetMapping("/getClass")
    @ApiParam(value = "教师编号", required = true)
    public HttpResponseResult<List<ClassGroup>> getClassGroup(Integer id) {
        return succeed(classGroupService.findClassGroup4Teacher(id));
    }

    @ApiOperation(value = "获取老师乐团课程信息列表")
    @GetMapping("/getTeacherMusicClass")
    @ApiParam(value = "教师编号", required = true)
    public HttpResponseResult<PageInfo<TeacherMusicClassInfoDto>> getTeacherMusicClass(TeacherMusicClassQueryInfo queryInfo){
        return succeed(classGroupService.getTeacherMusicClass(queryInfo));
    }

    @ApiOperation(value = "获取老师关联的乐团列表")
    @GetMapping("/queryTeacherMusicGroup")
    public HttpResponseResult<List<MusicGroup>> queryTeacherMusicGroup(){
        return succeed(classGroupService.getDao().queryTeacherMusicGroup(sysUserService.getUserId()));
    }

    @ApiOperation(value = "获取老师所在乐团的班级列表")
    @GetMapping("/queryTeacherMusicClass")
    public HttpResponseResult<List<ClassGroup>> queryTeacherMusicClass(String musicGroupId){
        return succeed(classGroupService.getDao().queryTeacherMusicClass(musicGroupId,sysUserService.getUserId()));
    }

    @ApiOperation(value = "获取老师VIP课程信息列表")
    @GetMapping("/getTeacherVipClass")
    public HttpResponseResult<PageInfo<TeacherVipClassInfoDto>> getTeacherVipClass(VipClassQueryInfo queryInfo){
        return succeed(classGroupService.getTeacherVipClass(queryInfo));
    }

    @ApiOperation(value = "获取教师所有聊天群组")
    @GetMapping("/queryTeacherGroups")
    public HttpResponseResult<List<ImGroupModel>> queryTeacherGroups(String search){
        return succeed(teacherService.queryTeacherGroups(sysUserService.getUserId(),search));
    }

    @ApiOperation(value = "获取当前教师通讯录列表")
    @GetMapping("/queryGroupStudents")
    public HttpResponseResult<List<BasicUserDto>> queryGroupStudents(String search){
        return succeed(teacherService.queryGroupStudents(sysUserService.getUserId(),search));
    }

    @ApiOperation(value = "获取当前教师通讯录学员列表")
    @GetMapping("/queryGroupStudentList")
    public HttpResponseResult<List<BasicUserDto>> queryGroupStudentList(String search){
        return succeed(teacherService.queryGroupStudentList(sysUserService.getUserId(),search));
    }

    @ApiOperation(value = "分页查询群公告列表")
    @GetMapping(value = "/queryNoticePage")
    public HttpResponseResult<PageInfo<ImGroupNotice>> queryNoticePage(QueryInfo queryInfo){
        return succeed(imGroupNoticeService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取教师上课结算列表")
    @GetMapping("/queryCloses")
    public HttpResponseResult<PageInfo<TeacherCloseDto>> queryCloses(TeacherCloseQueryInfo queryInfo){
        return succeed(teacherService.queryCloses(queryInfo));
    }

    @ApiOperation(value = "获取教师上课课酬未结算总额")
    @GetMapping("/sumTeacherNoPay")
    public HttpResponseResult<BigDecimal> findTeacherNoPay(Integer teacherId){
        return succeed(teacherService.findTeacherNoPay(teacherId));
    }

    @ApiOperation(value = "获取乐团学院学员报名列表")
    @GetMapping("/queryStudentApply")
    public HttpResponseResult<PageInfo<MusicGroupStudentApplyDto>> queryMusicGroupStudentApply(queryMusicGroupStudentQueryInfo queryInfo) {
        return succeed(teacherService.queryMusicGroupStudentApply(queryInfo));
    }

    @ApiOperation(value = "通过乐团编号查询乐团科目规划")
    @GetMapping("/querySubByMusicGroupId")
    @ApiImplicitParams({ @ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String")})
    public HttpResponseResult<List<Subject>> findSubByMusicGroupId(String musicGroupId){
        return succeed(subjectService.findSubByMusicGroupId(musicGroupId));
    }

	@ApiOperation(value = "实名认证")
    @PostMapping("/realNameAuthentication")
	@ApiImplicitParams({ @ApiImplicitParam(name = "realName", value = "姓名", required = true, dataType = "String"),
			@ApiImplicitParam(name = "idcardNo", value = "身份证号码", required = true, dataType = "String"),
			@ApiImplicitParam(name = "idcardFrontImg", value = "身份证正面", required = true, dataType = "String"),
			@ApiImplicitParam(name = "idcardBackImg", value = "身份证背面", required = true, dataType = "String"),
			@ApiImplicitParam(name = "idcardHandImg", value = "手持身份证照", required = true, dataType = "String") })
	public HttpResponseResult<Object> realNameAuthentication(String realName, String idcardNo, String idcardFrontImg, String idcardBackImg, String idcardHandImg) {
		teacherService.realNameAuthentication(sysUserService.getUserId(), realName, idcardNo, idcardFrontImg, idcardBackImg, idcardHandImg);
		return succeed();
	}


    @ApiOperation(value = "获取报名/预报名列表")
    @GetMapping("/getRegisterOrPreList")
    public HttpResponseResult<PageInfoReg<RegisterDto>> queryStudentApplyDetail(RegistrationOrPreQueryInfo queryInfo) {
        if (StringUtils.isNotBlank(queryInfo.getSearch())) {
            List<BasicUserDto> users = studentPaymentOrderDao.getUsers(queryInfo.getSearch());
            List<Integer> userIds = users.stream().map(BasicUserDto::getUserId).collect(Collectors.toList());
            if (userIds.size() > 0) {
                queryInfo.setUserIds(userIds);
            }
        }
        return succeed(studentRegistrationService.getRegisterOrPreList(queryInfo));
    }

    @ApiOperation(value = "查询教师关联的乐团预览信息")
    @GetMapping("/queryTeacherMusicStudentOverView")
    public HttpResponseResult<PageInfo<TeacherMusicStudentOverViewDto>> queryTeacherMusicStudentOverView(TeacherServeQueryInfo queryInfo){
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(teacherService.queryTeacherMusicStudentOverView(queryInfo));
    }


    @ApiOperation(value = "查询教师关联的乐团预览信息")
    @GetMapping("/queryTeacherMusicStudentOverView/v2")
    public HttpResponseResult<PageInfo<TeacherMusicStudentOverViewDto>> queryTeacherMusicStudentOverViewV2(TeacherServeQueryInfo queryInfo){
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(teacherService.queryTeacherMusicStudentOverViewV2(queryInfo));
    }



    @ApiOperation(value = "根据课程计划获取需要交作业的学生声部-公用")
    @PostMapping("/findCourseStudentsSubjectPublic/v2")
    public HttpResponseResult<List<Subject>> findCourseStudentsSubjectPublicV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        List<StudentCourseHomework> studentCourseHomeworkByCourseV2;
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            studentCourseHomeworkByCourseV2 = studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(
                    query);
        }else{
            studentCourseHomeworkByCourseV2 = studentCourseHomeworkService.findExtraExerciseStudentsV2(query);
        }

        if (CollectionUtils.isEmpty(studentCourseHomeworkByCourseV2)) {
            return succeed(new ArrayList<>());
        }
        // 获取声部
        List<Integer> collect = studentCourseHomeworkByCourseV2.stream().map(StudentCourseHomework::getSubjectId).collect(
                Collectors.toList());

        return succeed(subjectService.findBySubjectByIdList(collect));

    }
}
