package com.ym.mec.teacher.controller;

import com.dayaedu.cbs.openfeign.client.CoursewareFeignService;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareDetailWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.ClassGroupStudentMapperDao;
import com.ym.mec.biz.dal.dao.StudentAttendanceDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.LessonCourseware;
import com.ym.mec.biz.dal.entity.LessonCoursewareUserMapper;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.dal.vo.CourseScheduleWrapper;
import com.ym.mec.biz.dal.wrapper.LiveGroupWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author Joburgess
 * @Date 2019/9/27
 */
@Api(tags = "课程计划服务")
@RequestMapping("${app-config.url.teacher:}/teacherCourseSchedule")
@RestController
public class TeacherCourseScheduleController extends BaseController {

    @Autowired
    private CourseScheduleService scheduleService;
    @Autowired
    private StudentAttendanceService studentAttendanceService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private StudentAttendanceDao studentAttendanceDao;
    @Autowired
    private SysTenantConfigService sysTenantConfigService;
    @Autowired
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;
    @Autowired
    private SubjectService subjectService;
    @Resource
    private CoursewareFeignService coursewareFeignService;
    @Resource
    private LessonCoursewareUserMapperService lessonCoursewareUserMapperService;
    @Resource
    private LessonCoursewareService lessonCoursewareService;

    @ApiOperation(value = "根据月份获取该月有课的日期")
    @GetMapping("/getCourseScheduleDateByMonth")
    public Object getCourseScheduleDateByMonth(@ApiParam(value = "月份", required = true) @RequestParam Date month,
                                               @ApiParam(value = "课程类型") String type) {
        if(Objects.isNull(month)){
            return failed(HttpStatus.FORBIDDEN, "请指定日期!");
        }
        return succeed(scheduleService.getCourseScheduleDateByTeacher(month,type));
    }

    @ApiOperation(value = "根据日期获取当日排课")
    @GetMapping("/getCourseSchedulesWithDate")
    public Object getCourseSchedulesWithDate(@ApiParam(value = "日期", required = true) Date date,
                                             @ApiParam(value = "课程类型") String type){
        if(Objects.isNull(date)){
            return failed("请指定日期!");
        }
        SysUser user = sysUserService.getUser();
        if(null == user.getTenantId()){
            return failed("用户机构信息异常，请联系管理员");
        }
        String offlineSignInEarlyForwardTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.OFFLINE_SIGN_IN_EARLY_FORWARD_TIME, user.getTenantId());
        String configValue1 = sysTenantConfigService.getTenantConfigValue(SysConfigService.ENABLE_STUDENT_ATTENDANCE_TIME_RANGE_VIP, user.getTenantId());
        String courseBeforeBufferTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_BEFORE_BUFFER_TIME, user.getTenantId());
        String courseAfterBufferTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_AFTER_BUFFER_TIME, user.getTenantId());
        List<CourseScheduleDto> teacherCourseSchedulesWithDate = scheduleService.getTeacherCourseSchedulesWithDate(user.getId(),date,type,user.getTenantId());
        Map<String,Object> result = new HashMap<>(7);
        if (CollectionUtils.isNotEmpty(teacherCourseSchedulesWithDate)){
            for (CourseScheduleDto courseScheduleDto : teacherCourseSchedulesWithDate) {
                if(courseScheduleDto.getTeachMode() == TeachModeEnum.OFFLINE && courseScheduleDto.getStatus() != CourseStatusEnum.UNDERWAY
                        && StringUtils.isNotEmpty(courseScheduleDto.getCoursewareDetailId())){
                    //获取课件详情列表
                    R<CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail> voR = coursewareFeignService.lessonCoursewareDetailDetail(Long.parseLong(courseScheduleDto.getCoursewareDetailId()));
                    if (voR.getCode().equals(200)){
                        LessonCourseware lessonCourseware = lessonCoursewareService.lambdaQuery()
                                .eq(LessonCourseware::getLessonCourseId, voR.getData().getLessonCoursewareId()).last("LIMIT 1").one();
                        if (Objects.isNull(lessonCourseware) || StringUtils.equals(lessonCourseware.getTeacherRangeType(),"DISABLE")){
                            courseScheduleDto.setCoursewareEnable(false);
                            continue;
                        }
                        if(StringUtils.equals(lessonCourseware.getTeacherRangeType(),"ALL")){
                            continue;
                        }
                        LessonCoursewareUserMapper lessonCoursewareUserMapper = lessonCoursewareUserMapperService.lambdaQuery()
                                .eq(LessonCoursewareUserMapper::getLessonCoursewareId, lessonCourseware.getId())
                                .eq(LessonCoursewareUserMapper::getUserType, "TEACHER")
                                .eq(LessonCoursewareUserMapper::getUserId, user.getId()).last("LIMIT 1").one();
                        if(Objects.isNull(lessonCoursewareUserMapper)){
                            courseScheduleDto.setCoursewareEnable(false);
                        }
                    }
                }
            }
        }
        result.put("rows",teacherCourseSchedulesWithDate);
        result.put("appealHoursRange",4);
        result.put("offlineSignInEarlyForwardTime", StringUtils.isEmpty(offlineSignInEarlyForwardTime)?60:Integer.parseInt(offlineSignInEarlyForwardTime));
//        result.put("enableStudentAttendanceTimeRange",sysConfig.getParanValue(Integer.class));
        result.put("enableStudentAttendanceTimeRangeVip",StringUtils.isEmpty(configValue1)?20:Integer.parseInt(configValue1));
        //网络教室可以提前进入时间：对内和对外开课时间 (对内：joinRoomBeforeTime 对外：courseBeforeBufferTime)
        result.put("joinRoomBeforeTime",StringUtils.isEmpty(courseBeforeBufferTime)?10:Integer.parseInt(courseBeforeBufferTime));
        result.put("courseBeforeBufferTime", StringUtils.isEmpty(courseBeforeBufferTime)?10:Integer.parseInt(courseBeforeBufferTime));
        result.put("courseAfterBufferTime", StringUtils.isEmpty(courseAfterBufferTime)?10:Integer.parseInt(courseAfterBufferTime));
        return succeed(result);
    }

    @ApiOperation(value = "获取最近的一节课")
    @PostMapping("/getRecentCourseSchedule")
    public HttpResponseResult<CourseScheduleDto> getRecentCourseSchedule(Long courseId){
        CourseScheduleDto courseScheduleDto = scheduleService.getRecentCourseSchedule(sysUserService.getUserId(), courseId);
        if(courseScheduleDto != null && courseScheduleDto.getTeachMode() == TeachModeEnum.OFFLINE && courseScheduleDto.getStatus() != CourseStatusEnum.UNDERWAY
                && StringUtils.isNotEmpty(courseScheduleDto.getCoursewareDetailId())){
            //获取课件详情列表
            R<CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail> voR = coursewareFeignService.lessonCoursewareDetailDetail(Long.parseLong(courseScheduleDto.getCoursewareDetailId()));
            if (voR.getCode().equals(200)){
                LessonCourseware lessonCourseware = lessonCoursewareService.lambdaQuery()
                        .eq(LessonCourseware::getLessonCourseId, voR.getData().getLessonCoursewareId()).last("LIMIT 1").one();
                if (Objects.isNull(lessonCourseware) || StringUtils.equals(lessonCourseware.getTeacherRangeType(),"DISABLE")){
                    courseScheduleDto.setCoursewareEnable(false);
                }else if(StringUtils.equals(lessonCourseware.getTeacherRangeType(),"ALL")){
                    courseScheduleDto.setCoursewareEnable(true);
                }else {
                    LessonCoursewareUserMapper lessonCoursewareUserMapper = lessonCoursewareUserMapperService.lambdaQuery()
                            .eq(LessonCoursewareUserMapper::getLessonCoursewareId, lessonCourseware.getId())
                            .eq(LessonCoursewareUserMapper::getUserType, "TEACHER")
                            .eq(LessonCoursewareUserMapper::getUserId, sysUserService.getUserId()).last("LIMIT 1").one();
                    if(Objects.isNull(lessonCoursewareUserMapper)){
                        courseScheduleDto.setCoursewareEnable(false);
                    }
                }
            }
        }
        return succeed(courseScheduleDto);
    }

    @ApiOperation(value = "根据月份获取该月历史有课的日期")
    @GetMapping("/getHistoryCourseScheduleDateByMonth")
    public Object getHistoryCourseScheduleDateByTeacher(@ApiParam(value = "月份", required = true) @RequestParam Date month) {
        if(Objects.isNull(month)){
            return failed(HttpStatus.FORBIDDEN, "请指定日期!");
        }
        return succeed(scheduleService.getHistoryCourseScheduleDateByTeacher(sysUserService.getUserId(),month));
    }

    @ApiOperation(value = "根据日期获取当日历史课程")
    @GetMapping("/getHistoryCourseSchedulesWithDate")
    public Object getHistoryCourseSchedulesWithDate(@ApiParam(value = "日期", required = true) Date date){
        List<CourseScheduleDto> teacherCourseSchedulesWithDate = scheduleService.getTeacherHistoryCourseSchedulesWithDate(sysUserService.getUserId(),date);
        Map<String,Object> result = new HashMap<>();
        result.put("rows",teacherCourseSchedulesWithDate);
        result.put("appealDaysRange",1);
        result.put("complaintsDaysRange",1);
        return succeed(result);
    }

    @ApiOperation(value = "根据课程ID查询打卡界面课程详情")
    @GetMapping("/getCurrentCourseDetail")
    public Object getCurrentCourseDetail(Long courseID){
        if(Objects.isNull(courseID)){
            return failed("请指定课程!");
        }
        return succeed(scheduleService.getCurrentCourseDetail(courseID));
    }

    @ApiOperation(value = "获取老师指定班级的排课")
    @GetMapping("/queryTeacherClassCourseSchedule")
    public Object queryTeacherClassCourseSchedule(CourseScheduleQueryInfo queryInfo){
        queryInfo.setUserId(sysUserService.getUserId().longValue());
        return succeed(scheduleService.queryTeacherClassCourseSchedule(queryInfo));
    }

    @ApiOperation(value = "获取老师指定班级的排课")
    @GetMapping("/queryTeacherVipClassCourseSchedule")
    public Object queryTeacherVipClassCourseSchedule(CourseScheduleQueryInfo queryInfo){
        Map<String,Object> result = new HashMap<>(2);
        queryInfo.setUserId(sysUserService.getUserId().longValue());
        result.put("rows",scheduleService.queryTeacherVipClassCourseSchedule(queryInfo));
        result.put("appealHoursRange",4);
        return succeed(result);
    }

    @ApiOperation(value = "获取点名详情头部信息")
    @GetMapping("/findCourseAttendanceDetailHeadInfo")
    public Object findCourseAttendanceDetailHeadInfo(Long courseScheduleId){
        return succeed(scheduleService.findCourseAttendanceDetailHeadInfo(courseScheduleId));
    }

    @ApiOperation(value = "根据课程获取当前课程的学生列表")
    @GetMapping("/getCurrentCourseStudents")
    public Object getCurrentCourseStudents(StudentAttendanceQueryInfo queryInfo){
        if(Objects.isNull(queryInfo.getClassGroupId())){
            return failed("请指定班级!");
        }
        queryInfo.setFilterStatus(StudentAttendanceStatusEnum.DROP_OUT);
        return succeed(studentAttendanceService.getCurrentCourseStudents(queryInfo));
    }

    @ApiOperation(value = "课时调整")
    @PostMapping(value = "/classStartDateAdjust",consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @AuditLogAnnotation(operateName = "课时调整")
    public Object classStartDateAdjust(ClassDateAdjustDto classDateAdjustDto){
        if(Objects.isNull(classDateAdjustDto.getId())){
            return failed("请指定课程!");
        }
        if(classDateAdjustDto.getStartClassTime().before(new Date())){
            return failed("课程开始时间不能小于当前时间");
        }
        CourseSchedule oldCourseSchedule = scheduleService.get(classDateAdjustDto.getId());
        if(Objects.isNull(oldCourseSchedule)){
            return failed("未找到指定课程");
        }
        if(Objects.isNull(oldCourseSchedule.getTenantId())){
            return failed("课程机构信息异常");
        }
        Date now = new Date();
        if(now.compareTo(oldCourseSchedule.getStartClassTime())>0){
            throw new BizException("进行中或已结束课程不可调整");
        }
        
        //默认开课前4小时内，可调整，如果全员请假，可在当天24小时前调整
        String advanceLeaveHoursConfig = sysTenantConfigService.getTenantConfigValue(SysConfigService.ENABLE_TEACHER_COURSE_ADJUST_DEFAULT_HOURS,oldCourseSchedule.getTenantId());
        Integer advanceLeaveHours = 0;
        if(StringUtils.isNotEmpty(advanceLeaveHoursConfig)){
            advanceLeaveHours = Integer.parseInt(advanceLeaveHoursConfig);
        }
        
        List<Long> newCourseScheduleIds = new ArrayList<Long>();
        newCourseScheduleIds.add(oldCourseSchedule.getId());
        List<Map<Long, Integer>> courseLeaveStudentNumMaps = studentAttendanceDao.countCourseLeaveStudentNumWithFourHoursAgo(newCourseScheduleIds, advanceLeaveHours);
        Map<Long,Long> courseLeaveStudentNumMap = MapUtil.convertIntegerMap((courseLeaveStudentNumMaps));

        List<Integer> classGroupIds = new ArrayList<Integer>();
        classGroupIds.add(oldCourseSchedule.getClassGroupId());
        List<Map<Integer, Integer>> classGroupStudentNumMaps = classGroupStudentMapperDao.countClassGroupsStudentNum(classGroupIds, ClassGroupStudentStatusEnum.NORMAL);
        Map<Integer, Long> classGroupStudentNumMap = MapUtil.convertIntegerMap(classGroupStudentNumMaps);

        Long normalStudentNum = classGroupStudentNumMap.get(oldCourseSchedule.getClassGroupId());
        Long courseLeaveStudentNum = courseLeaveStudentNumMap.get(oldCourseSchedule.getId());
        if(oldCourseSchedule.getGroupType().equals(GroupType.VIP)
                &&Objects.nonNull(courseLeaveStudentNum)
                &&Objects.nonNull(normalStudentNum)
                &&courseLeaveStudentNum.intValue()==normalStudentNum.intValue()){
            if(!DateUtil.isSameDay(now,oldCourseSchedule.getStartClassTime())&&now.after(oldCourseSchedule.getStartClassTime())){
                throw new BizException("当前时间不可对相关课程进行调整");
            }
        }else{
            if(DateUtil.addHours(now,advanceLeaveHours).after(oldCourseSchedule.getStartClassTime())){
                throw new BizException("请在课程开始前{}小时进行调整",advanceLeaveHours);
            }
        }
        
        /*if (oldCourseSchedule.getGroupType().equals(GroupType.VIP)) {
            vipGroupService.checkVipCourseIsInScore(Long.valueOf(oldCourseSchedule.getMusicGroupId()));
        }*/
        
        if(Objects.isNull(classDateAdjustDto.getClassGroupId())){
            classDateAdjustDto.setClassGroupId(oldCourseSchedule.getClassGroupId());
        }
        classDateAdjustDto.setMusicGroupId(oldCourseSchedule.getMusicGroupId());
        List<CourseSchedule> courseSchedules=new ArrayList<>();
        courseSchedules.add(classDateAdjustDto);
        return scheduleService.courseAdjust(courseSchedules, true);
    }

    @ApiOperation(value = "对外课时调整")
    @PostMapping(value = "/updateCourseScheduleOfComm",consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Object updateCourseScheduleOfComm(CourseSchedule courseSchedule){
    	CourseSchedule oldCourseSchedule = scheduleService.get(courseSchedule.getId());
        if(Objects.isNull(oldCourseSchedule)){
            return failed("未找到指定课程");
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date date = new Date();
		String endDateStr = DateUtil.format(oldCourseSchedule.getClassDate(), "yyyy-MM-dd") + " " + DateUtil.format(oldCourseSchedule.getStartClassTime(), "HH:mm:ss");
		if (DateUtil.stringToDate(endDateStr, sdf).before(date)) {
			throw new BizException("调整失败，未开始的课程才能调整");
		}
		
		endDateStr = DateUtil.format(courseSchedule.getClassDate(), "yyyy-MM-dd") + " " + DateUtil.format(courseSchedule.getStartClassTime(), "HH:mm:ss");
		if (DateUtil.stringToDate(endDateStr, sdf).before(date)) {
			throw new BizException("调整失败，只能调整到未来的时间");
		}
		
        if(Objects.isNull(courseSchedule.getClassGroupId())){
            courseSchedule.setClassGroupId(oldCourseSchedule.getClassGroupId());
        }
        courseSchedule.setMusicGroupId(oldCourseSchedule.getMusicGroupId());
        List<CourseSchedule> courseSchedules=new ArrayList<>();
        courseSchedules.add(courseSchedule);
        scheduleService.courseAdjust(courseSchedules, true);
        return succeed();
    }

    @ApiOperation(value = "课时交换")
    @PostMapping(value = "/courseSwap",consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Object courseSwap(Long courseScheduleId1,Long courseScheduleId2){
        if(Objects.isNull(courseScheduleId1)||Objects.isNull(courseScheduleId2)){
            return failed("请指定课程!");
        }
        scheduleService.courseSwap(courseScheduleId1,courseScheduleId2);
        return succeed();
    }

    @GetMapping("/statisticsInfo")
    @ApiOperation("历史考勤统计-头信息")
    public Object getStatisticsInfo(Integer classGroupId) {
        return succeed(studentAttendanceService.getStatisticsInfo(classGroupId));
    }

    @GetMapping("/statisticsList")
    @ApiOperation("历史考勤统计-考勤列表")
    public Object statisticsList(CourseHomeworkQueryInfo queryInfo) {
        return succeed(studentAttendanceService.statisticsList(queryInfo));
    }

    @ApiOperation(value = "终极课表获取")
    @GetMapping("/superFindCourseSchedules")
    public Object superFindCourseSchedules(EndCourseScheduleQueryInfo queryInfo){
        return succeed(scheduleService.endFindCourseSchedules(queryInfo));
    }

    @ApiOperation(value = "教师端中间页课程评价数据获取")
    @GetMapping("/teacherCourseRemarkInfo")
    public Object teacherCourseRemarkInfo(Long courseScheduleId){
        return succeed(scheduleService.teacherCourseRemarkInfo(courseScheduleId));
    }

    @ApiOperation(value = "教师端中间页课程评价数据提交")
    @PostMapping("/teacherCourseRemarkCommit")
    public Object teacherCourseRemarkCommit(@RequestBody TeacherRemarkCommitDto teacherRemarkCommitDto){
        scheduleService.teacherCourseRemarkCommit(teacherRemarkCommitDto);
        return succeed();
    }

    @ApiOperation(value = "教师端中间页网络教室头部详情")
    @GetMapping("/teacherCourseHeadInfo")
    public Object teacherCourseHeadInfo(Long courseScheduleId){
        return succeed(scheduleService.teacherCourseHeadInfo(courseScheduleId));
    }

    @ApiOperation(value = "教师端中间页网络教室课程进度列表")
    @GetMapping("/teacherCourseRates")
    public Object teacherCourseRates(Long courseScheduleId){
        return succeed(scheduleService.teacherCourseRates(courseScheduleId));
    }

    @ApiOperation(value = "课时使用记录列表")
    @GetMapping("/teacherCourseMinutes")
    public Object teacherCourseMinutes(QueryInfo queryInfo){
        return succeed(scheduleService.teacherCourseMinutes(queryInfo));
    }

    @ApiOperation(value = "课程批量调整检测-仅时间")
    @PostMapping("/courseAdjustOnlyWithClassDateCheck")
    public HttpResponseResult courseAdjustOnlyWithClassDateCheck(@RequestBody VipGroupCourseAdjustInfoDto vipGroupCourseAdjustInfo){
        return succeed(scheduleService.courseAdjustOnlyWithClassDateCheck(vipGroupCourseAdjustInfo));
    }

    @ApiOperation(value = "课程批量调整")
    @PostMapping("/courseAdjustOnlyWithClassDate")
    @AuditLogAnnotation(operateName = "课程批量调整")
    public HttpResponseResult courseAdjustOnlyWithClassDate(@RequestBody List<CourseSchedule> courseSchedules){
        scheduleService.courseAdjustOnlyWithClassDate(courseSchedules,sysUserService.getUserId());
        return succeed();
    }

    @ApiOperation(value = "课程评价列表")
    @PostMapping("/courseEvaluateList")
    @AuditLogAnnotation(operateName = "课程评价列表")
    public HttpResponseResult courseEvaluateList(@RequestBody EvaluateCourseQueryInfo queryInfo){
        SysUser user = sysUserService.getUser();
        queryInfo.setTeacherId(user.getId());
        String offlineSignInEarlyForwardTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.OFFLINE_SIGN_IN_EARLY_FORWARD_TIME, user.getTenantId());
        String configValue1 = sysTenantConfigService.getTenantConfigValue(SysConfigService.ENABLE_STUDENT_ATTENDANCE_TIME_RANGE_VIP, user.getTenantId());
        Map<String,Object> result = new HashMap<>(3);
        result.put("pageInfo",scheduleService.courseEvaluateList(queryInfo));
        result.put("offlineSignInEarlyForwardTime", StringUtils.isEmpty(offlineSignInEarlyForwardTime)?60:Integer.parseInt(offlineSignInEarlyForwardTime));
        result.put("enableStudentAttendanceTimeRangeVip",StringUtils.isEmpty(configValue1)?20:Integer.parseInt(configValue1));
        return succeed(result);
    }

    @ApiOperation(value = "学员学习标准列表")
    @PostMapping("/studyStandardList")
    @AuditLogAnnotation(operateName = "学员学习标准列表")
    public HttpResponseResult studyStandardList(EvaluateCourseQueryInfo queryInfo){
        PageInfo<StudentStandardDto> studentStandardDtoPageInfo = scheduleService.studyStandardList(queryInfo);
        List<StudentStandardDto> courseStudents = studentStandardDtoPageInfo.getRows();
        // 乐团声部为空，取student的声部
        if (!CollectionUtils.isEmpty(courseStudents)) {
            Set<Long> collect = courseStudents.stream().map(o -> o.getUserId()).map(o -> o.longValue()).collect(Collectors.toSet());

            Map<Integer, StudentSubjectDto> subjectDtoMap = subjectService.getSubjectByStudentId(collect);
            courseStudents.forEach(studentAttendanceViewDto -> {
                if (studentAttendanceViewDto.getSubjectId() == null) {
                    StudentSubjectDto subjectDto = subjectDtoMap.getOrDefault(studentAttendanceViewDto.getUserId(), new StudentSubjectDto());
                    if (subjectDto.getSubjectId() != null) {
                        studentAttendanceViewDto.setSubjectName(subjectDto.getSubjectName());
                        studentAttendanceViewDto.setSubjectId(subjectDto.getSubjectId());
                    }
                }
            });
        }
        return succeed(studentStandardDtoPageInfo);
    }


    @ApiOperation(value = "学习标准待回访列表")
    @PostMapping("/queryStudyStandardWaitVisit")
    @AuditLogAnnotation(operateName = "学习标准待回访列表")
    public HttpResponseResult queryStudyStandardWaitVisit(EvaluateCourseQueryInfo queryInfo){
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(scheduleService.queryStudyStandardWaitVisit(queryInfo));
    }

    @ApiOperation(value = "获取老师直播课程时间")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "courseScheduleId", value = "课程id", required = true, dataType = "Long")
    })
    @GetMapping("/liveCourseScheduleTime")
    public HttpResponseResult<CourseScheduleWrapper.LiveCourseSchedule> liveCourseScheduleTime(@RequestParam Long courseScheduleId){

        if (Objects.isNull(courseScheduleId)) {
            throw new BizException("课程编号不能为空");
        }

        return succeed(scheduleService.getLiveCourseScheduleTime(courseScheduleId));
    }

    @ApiOperation(value = "老师乐直播直播课课程列表")
    @PostMapping("/liveCoursePage")
    public HttpResponseResult<PageInfo<CourseScheduleDto>> liveCoursePage(@RequestBody LiveGroupWrapper.LiveCoursePageQuery queryInfo){


        SysUser user = sysUserService.getUser();
        if(null == user.getTenantId()){
            return failed("用户机构信息异常，请联系管理员");
        }
        queryInfo.setTeacherId(user.getId());
        queryInfo.setTenantId(user.getTenantId());
        return succeed(PageUtil.pageInfo(scheduleService.liveCoursePage(queryInfo)));
    }

}
