package com.ym.mec.teacher.controller;

import com.microsvc.toolkit.middleware.live.message.TencentWrapper;
import com.ym.mec.biz.dal.dto.LiveRoomStatus;
import com.ym.mec.biz.dal.dto.RoomReservationUserSearch;
import com.ym.mec.biz.dal.page.LiveRoomGoodsOrderQueryInfo;
import com.ym.mec.biz.dal.vo.*;
import com.ym.mec.biz.service.ImLiveBroadcastRoomMemberService;
import com.ym.mec.biz.service.ImLiveBroadcastRoomService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 直播房间管理表(ImLiveBroadcastRoom)表控制层
 *
 * @author hgw
 * @since 2022-02-17 20:52:04
 */
@Api(tags = "直播房间管理表")
@RestController
@RequestMapping("${app-config.url.teacher:}/imLiveBroadcastRoom")
public class TeacherImLiveBroadcastRoomController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private ImLiveBroadcastRoomService imLiveBroadcastRoomService;
    @Autowired
    private ImLiveBroadcastRoomMemberService imLiveBroadcastRoomMemberService;

    @ApiImplicitParams({
            @ApiImplicitParam(name = "search", dataType = "String", value = "模糊查询关键字"),
            @ApiImplicitParam(name = "liveState", dataType = "Integer", value = "直播状态 0未开始 1开始 2结束"),
            @ApiImplicitParam(name = "startTime", dataType = "String", value = "开始时间"),
            @ApiImplicitParam(name = "endTime", dataType = "String", value = "结束时间"),
            @ApiImplicitParam(name = "popularize", dataType = "Integer", value = "是否在首页推广 0否 1是"),
            @ApiImplicitParam(name = "speakerId", dataType = "Integer", value = "老师ID"),
            @ApiImplicitParam(name = "clientType", dataType = "String", value = "用户类型 TEACHER 老师 EDUCATION 教务端"),
            @ApiImplicitParam(name = "sort", dataType = "String", value = "不传是默认web端排序  1：直播中--未开始--已结束（状态相同时，根据直播时间倒序）"),
            @ApiImplicitParam(name = "page", dataType = "Integer", value = "页数"),
            @ApiImplicitParam(name = "rows", dataType = "Integer", value = "每页数量"),
    })
    @ApiOperation("分页查询直播间列表")
    @PostMapping("/queryPage")
    public HttpResponseResult<PageInfo<ImLiveBroadcastRoomVo>> queryPage(@RequestBody Map<String, Object> param) {
        param.put("os", "teacher");
        return succeed(imLiveBroadcastRoomService.queryPage(param));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "search", dataType = "String", value = "模糊查询关键字"),
            @ApiImplicitParam(name = "subjectId", dataType = "Integer", value = "声部id"),
            @ApiImplicitParam(name = "organId", dataType = "Integer", value = "分部id"),
            @ApiImplicitParam(name = "page", dataType = "Integer", value = "页数"),
            @ApiImplicitParam(name = "rows", dataType = "Integer", value = "每页数量"),
    })
    @ApiOperation("直播间预约人员信息")
    @PostMapping("/query/roomUser")
    public HttpResponseResult<PageInfo<RoomReservationUserVo>> queryRoomUser(@RequestBody @Valid RoomReservationUserSearch query) {
        return succeed(imLiveBroadcastRoomService.queryRoomUser(query));
    }

    @ApiOperation("查询房间信息并校验房间是否合规")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roomUid", dataType = "String", value = "房间uid", required = true),
            @ApiImplicitParam(name = "userId", dataType = "Integer", value = "用户id", required = true),
            @ApiImplicitParam(name = "liveAssistant", dataType = "Boolean", value = "直播助手"),
    })
    @GetMapping("/queryRoom")
    public HttpResponseResult<ImLiveBroadcastRoomVo> queryRoomAndCheck(@ApiParam(value = "房间uid", required = true) String roomUid,
                                                                       @ApiParam(value = "用户id", required = true) Integer userId,
                                                                       @RequestParam(defaultValue = "false") Boolean liveAssistant) {

        // 请求来源业源
        int osType = 2; // 老师端
        if (Optional.ofNullable(liveAssistant).orElse(false)) {
            osType = 3; // 直播助手
        }

        return succeed(imLiveBroadcastRoomService.queryRoomAndCheck(roomUid, userId, osType));
    }

    @ApiOperation("关闭直播间")
    @GetMapping(value = "/roomDestroy")
    public HttpResponseResult<Object> roomDestroy(@ApiParam(value = "房间Uid", required = true) String roomUid) {
        imLiveBroadcastRoomService.roomDestroy(roomUid);
        return succeed();
    }

    @ApiOperation("同步点赞数量")
    @GetMapping("/syncLike")
    public HttpResponseResult<Object> syncLike(@ApiParam(value = "房间uid", required = true) String roomUid,
                                               @ApiParam(value = "点赞数", required = true) Integer likeNum) {
        imLiveBroadcastRoomService.syncLike(roomUid, likeNum);
        return succeed();
    }

    @ApiOperation("开启/关闭直播的录像-融云使用")
    @GetMapping("/opsLiveVideo")
    public HttpResponseResult<Object> opsLiveVideo(@ApiParam(value = "房间uid", required = true) String roomUid,
                                                   @ApiParam(value = "用户id", required = true) Integer userId,
                                                   @ApiParam(value = "type 1:开始直播-开始录像     2:关闭直播关闭录像", required = true) Integer type,
                                                   @ApiParam(value = "录制视频的尺寸-默认值是720x1280") String videoResolution) {
        if (type == 1) {
            videoResolution = Optional.ofNullable(videoResolution).orElse("720x1280");
            imLiveBroadcastRoomService.startLive(roomUid, userId, videoResolution);
        } else if (type == 2) {
            imLiveBroadcastRoomService.closeLive(roomUid, userId);
        } else {
            failed("type参数错误");
        }
        return succeed();
    }

    @ApiOperation("主讲人进入房间")
    @GetMapping("/speakerJoinRoom")
    public HttpResponseResult<ImLiveBroadcastRoomVo> speakerJoinRoom(String roomUid) {
        return succeed(imLiveBroadcastRoomService.speakerJoinRoom(roomUid,"mobile"));
    }

    @GetMapping("/shareGroup")
    public HttpResponseResult<Object> shareGroup(@ApiParam(value = "房间uid", required = true) String roomUid,
                                                 @ApiParam(value = "群编号", required = true) String groupIds) {
        imLiveBroadcastRoomService.shareGroup(roomUid, groupIds);
        return succeed();
    }

    @ApiOperation("获取房间人员")
    @GetMapping("/queryRoomUserInfo")
    public HttpResponseResult<List<BaseRoomUserVo>> queryRoomUserInfo(@ApiParam(value = "房间uid", required = true) String roomUid) {
        return succeed(imLiveBroadcastRoomService.queryRoomLimitOnlineUserInfo(roomUid));
    }

    @ApiOperation("设置是否允许连麦")
    @GetMapping("/whetherMic")
    public HttpResponseResult<Object> whetherMic(@ApiParam(value = "房间uid", required = true) String roomUid,
                                                 @ApiParam(value = "是否连麦 0：是 1否", required = true) Integer whetherMic) {
        imLiveBroadcastRoomService.whetherMic(roomUid,whetherMic);
        return succeed();
    }


    @ApiOperation("设置连麦状态")
    @PutMapping("/userWhetherMic")
    public HttpResponseResult<Object> userWhetherMic(@ApiParam(value = "房间uid", required = true) String roomUid,
                                                 @ApiParam(value = "用户id", required = true) Long userId,
                                                 @ApiParam(value = "连麦状态 0:未申请1:申请连麦中2:连麦中", required = true) Integer whetherMicStatus) {
        imLiveBroadcastRoomService.userWhetherMic(roomUid,userId,whetherMicStatus);
        return succeed();
    }

    @ApiOperation("查询直播间商品订单列表")
    @GetMapping("/queryLiveRoomGoodsOrderList")
    public HttpResponseResult<PageInfo<LiveRoomGoodsOrderVo>> queryLiveRoomGoodsOrderList(LiveRoomGoodsOrderQueryInfo queryInfo) {
        return succeed(imLiveBroadcastRoomService.queryLiveRoomGoodsOrderList(queryInfo));
    }

    @ApiOperation("分页查询直播间人员列表")
    @ApiImplicitParams({
      @ApiImplicitParam(name = "search", dataType = "String", value = "模糊搜索 学员编号姓名"),
      @ApiImplicitParam(name = "roomUid", dataType = "String", value = "房间uid"),
      @ApiImplicitParam(name = "onlineStatus", dataType = "String ", value = "0:离线 1：在线  不传是全部"),
      @ApiImplicitParam(name = "whetherMicStatus", dataType = "String", value = "连麦状态 0:未申请1:申请连麦中2:连麦中"),
      @ApiImplicitParam(name = "page", dataType = "Integer", value = "页数"),
      @ApiImplicitParam(name = "rows", dataType = "Integer", value = "每页数量"),
    })
    @PostMapping("/queryRoomMember")
    public HttpResponseResult<PageInfo<ImLiveBroadcastRoomMemberVo>> queryRoomMember(@RequestBody Map<String, Object> param) {
        return succeed(imLiveBroadcastRoomMemberService.queryRoomMember(param));
    }


    @ApiOperation("更新直播间状态")
    @PostMapping("/updateRoomStatus")
    public HttpResponseResult<Boolean> updateRoomStatus(@RequestBody @Valid LiveRoomStatus status ) {
        return succeed(imLiveBroadcastRoomService.updateRoomStatus(status));
    }

    @ApiOperation("直播间推流状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roomUid", dataType = "String", value = "房间Uid", required = true),
    })
    @GetMapping(value = "/liveStreamStatus")
    public HttpResponseResult<TencentWrapper.LiveStreamState> roomLiveStream(String roomUid) {

        // 直播间推流状态查询
        return succeed(imLiveBroadcastRoomService.roomLiveStreamStatus(roomUid));
    }
}

