package com.ym.mec.teacher.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.entity.VipGroupCategory;
import com.ym.mec.biz.dal.entity.VipGroupDefaultClassesUnitPrice;
import com.ym.mec.biz.service.VipGroupCategoryService;
import com.ym.mec.biz.service.VipGroupDefaultClassesUnitPriceService;
import com.ym.mec.common.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Api(tags = "vip课类别")
@RequestMapping("${app-config.url.teacher:}/vipGroupCategory")
@RestController
public class VipGroupCategoryController extends BaseController {

	@Autowired
	private VipGroupCategoryService vipGroupCategoryService;
	@Autowired
	private VipGroupDefaultClassesUnitPriceService vipGroupDefaultClassesUnitPriceService;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	
	@Autowired
	private TeacherDao teacherDao;

	@ApiOperation("单查询")
	@GetMapping(value = "/query")
	public Object query(int id) {
		return succeed(vipGroupCategoryService.get(id));
	}

	@ApiOperation("全查询")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "organId", value = "分部id", dataType = "int", paramType = "query"),
			@ApiImplicitParam(name = "includeMusicTheory", value = "是否包含小组", dataType = "int", paramType = "query", defaultValue = "0"),
			@ApiImplicitParam(name = "groupType", value = "课类别", dataType = "String", paramType = "query", defaultValue = "VIP")
	})
	@GetMapping(value = "/queryAll")
	public Object queryAll(Integer organId, Integer includeMusicTheory, @RequestParam(defaultValue = "VIP") String groupType) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if (Objects.isNull(user)) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
		if(Objects.nonNull(organId)){
			List<VipGroupCategory> result = vipGroupCategoryService.findAllByOrgan(organId.toString(), groupType);
			if(!CollectionUtils.isEmpty(result)){
				result = result.stream().filter(c->!c.getMusicTheory()).collect(Collectors.toList());
			}
			return succeed(result);
		}else{
			Teacher teacher = teacherDao.get(user.getId());
			if(Objects.isNull(teacher)){
				return failed("教师信息不存在");
			}
			List<VipGroupCategory> result = vipGroupCategoryService.findAllByOrgan(teacher.getTeacherOrganId().toString(), groupType);
			if(!CollectionUtils.isEmpty(result)&&(Objects.isNull(includeMusicTheory)||includeMusicTheory<=0)){
				result = result.stream().filter(c->!c.getMusicTheory()).collect(Collectors.toList());
			}
			return succeed(result);
		}
	}

	@ApiOperation("新增")
	@PostMapping(value = "/add")
	public Object add(VipGroupCategory vipGroupCategory) {

		// 设置默认值
		if (StringUtils.isBlank(vipGroupCategory.getGroupType())) {
			vipGroupCategory.setGroupType("VIP");
		}

		Date date = new Date();
		vipGroupCategory.setCreateTime(date);
		vipGroupCategory.setUpdateTime(date);
		vipGroupCategory.setDelFlag("0");

		// 强制更新分类名称
		if (CourseSchedule.CourseScheduleType.LIVE.getCode().equals(vipGroupCategory.getGroupType())) {
			vipGroupCategory.setName(vipGroupCategory.getSingleClassMinutes());
			vipGroupCategory.setStudentNum(0);
		}

		// 课程分类编号
		vipGroupCategoryService.insert(vipGroupCategory);

		// 直播流程，主动添加购买价格
		if (CourseSchedule.CourseScheduleType.LIVE.getCode().equals(vipGroupCategory.getGroupType())) {

			VipGroupDefaultClassesUnitPrice vipGroupDefaultClassesUnitPrice = new VipGroupDefaultClassesUnitPrice();
			vipGroupDefaultClassesUnitPrice.setOrganId(vipGroupCategory.getTenantId());
			vipGroupDefaultClassesUnitPrice.setVipGroupCategoryId(vipGroupCategory.getId());
			vipGroupDefaultClassesUnitPrice.setGroupType(vipGroupCategory.getGroupType());
			vipGroupDefaultClassesUnitPrice.setCreateTime(DateTime.now().toDate());
			vipGroupDefaultClassesUnitPrice.setUpdateTime(DateTime.now().toDate());
			// 售价
			vipGroupDefaultClassesUnitPrice.setOnlineClassesUnitPrice(Optional.ofNullable(vipGroupCategory.getOnlineClassesUnitPrice()).orElse(BigDecimal.ZERO));
			// 原价
			vipGroupDefaultClassesUnitPrice.setOfflineClassesUnitPrice(Optional.ofNullable(vipGroupCategory.getOfflineClassesUnitPrice()).orElse(BigDecimal.ZERO));

			vipGroupDefaultClassesUnitPriceService.insert(vipGroupDefaultClassesUnitPrice);
		}

		return succeed();
	}

	@ApiOperation("修改")
	@PostMapping(value = "/update")
	public Object update(VipGroupCategory vipGroupCategory) {
		Date date = new Date();
		vipGroupCategory.setUpdateTime(date);
		vipGroupCategoryService.update(vipGroupCategory);
		return succeed();
	}

	@ApiOperation("删除")
	@PostMapping(value = "/delete")
	public Object delete(int id) {
		VipGroupCategory vipGroupCategory = vipGroupCategoryService.get(id);
		Date date = new Date();
		vipGroupCategory.setUpdateTime(date);
		vipGroupCategory.setDelFlag("1");
		vipGroupCategoryService.update(vipGroupCategory);
		return succeed();
	}

}
