package com.ym.mec.web.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dao.SysUserCashAccountDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.StudentVipGroupShowListDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.page.StudentVipGroupQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.dto.SchoolDto;
import com.ym.mec.common.entity.*;
import com.ym.mec.thirdparty.adapay.entity.BaseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RequestMapping("${app-config.url.web:}/api")
@Api(tags = "对外接口")
@RestController
public class APIController extends BaseController {
	@Resource
	private SysUserCashAccountDao sysUserCashAccountDao;
	@Resource
	private TeacherDao teacherDao;
	@Resource
	private EmployeeDao employeeDao;
	@Resource
	private PracticeLessonApplyService practiceLessonApplyService;
	@Resource
	private PracticeGroupService practiceGroupService;
	@Resource
	private ImGroupService imGroupService;
	@Resource
	private SysUserFeignService sysUserFeignService;
	@Resource
	private SysTenantConfigService sysTenantConfigService;
	@Resource
	private SysUserService sysUserService;
	@Resource
	private VipGroupService vipGroupService;
	@Resource
	private SysConfigService sysConfigService;
	@Resource
	private StudentPaymentOrderService studentPaymentOrderService;
	@Resource
	private GoodsService goodsService;
	@Resource
	private CooperationOrganService cooperationOrganService;

	@GetMapping("/createCashAccount")
	public Boolean createCashAccount(Integer userId,Integer tenantId) {
		// 添加用户现金账户
		sysUserCashAccountDao.insert(new SysUserCashAccount(userId,tenantId, "CNY"));

		return true;
	}

	@GetMapping("/queryTeacherOrganId")
	public Integer queryTeacherOrganId(Integer userId) {
		Teacher teacher = teacherDao.get(userId);
		if (teacher != null) {
			return teacher.getTeacherOrganId();
		}
		return null;
	}

	@GetMapping("/practiceSum")
	public Object practiceSum() {
		return succeed(practiceLessonApplyService.practiceSum());
	}

	@GetMapping("/isPurchasedPracticeCourse")
	public Boolean isPurchasedPracticeCourse(Integer userId) {
		return practiceGroupService.isPurchasedPracticeCourse(userId);
	}

	@GetMapping("/updateNickName")
	public int updateNickName(Integer userId,String nickName) {
		return imGroupService.updateNickName(userId,nickName);
	}

	@GetMapping("/getTenantConfigValue")
	public String getTenantConfigValue(Integer tenantId,String paramName) {
		return sysTenantConfigService.getTenantConfigValue(paramName,tenantId);
	}

	@GetMapping("/queryOrganIdByUserId")
	public String queryOrganIdByUserId(Integer userId, String clientId) {
		if (StringUtils.equalsIgnoreCase("system", clientId) || StringUtils.equalsIgnoreCase("com/ym/app/web/controller/education", clientId)) {
			Employee employee = employeeDao.get(userId);
			if (employee != null) {
				return employee.getOrganIdList();
			}

		} else if (StringUtils.equalsIgnoreCase("teacher", clientId)) {
			Teacher teacher = teacherDao.get(userId);
			if (teacher != null) {
				return teacher.getOrganId() + "";
			}
		} else {
			SysUser user = sysUserFeignService.queryUserInfo();
			if (user != null) {
				return user.getOrganId() + "";
			}
		}
		return null;
	}

	@ApiOperation(value = "获取可购买vip、网管课列表")
	@RequestMapping(value = "/queryVipPracticeGroups")
	public Boolean queryVipPracticeGroups(){
		SysUser sysUser = sysUserService.getUser();
		StudentVipGroupQueryInfo queryInfo = new StudentVipGroupQueryInfo();
		queryInfo.setOrganId(sysUser.getOrganId());
		queryInfo.setUserId(sysUser.getId());
		List<StudentVipGroupShowListDto> listDtos = vipGroupService.queryVipPracticeGroups(queryInfo);
		return CollectionUtils.isEmpty(listDtos)?false:true;
	}



	// 商城创建订单
	@PostMapping("/mallCreateOrder")
	public HttpResponseResult<Map> mallCreateOrder(@RequestBody MallCreateOrderModel model) throws Exception {
		return succeed(studentPaymentOrderService.createOrder(model));
	}

	// 历史商城订单数据修复
	@GetMapping("/initMallOrder")
	public HttpResponseResult<Map> initMallOrder(Long orderId) throws Exception {
		studentPaymentOrderService.mallBuyOrderCallback1(orderId);
		return succeed();
	}

	// 商城订单优惠券金额
	@PostMapping("/couponAmount")
	public HttpResponseResult<Map> couponAmount(@RequestBody MallCreateOrderModel model) {
		return succeed(studentPaymentOrderService.couponAmount(model));
	}

	// 商城保存订单详情数据
	@PostMapping("/mallSaveOrderInfo")
	public HttpResponseResult<Map> mallSaveOrderInfo(@RequestBody MallCreateOrderModel model) {
		studentPaymentOrderService.mallSaveOrderInfo(model);
		return succeed();
	}

	// 商城支付成功/失败修改订单状态
	@PostMapping("/updateOrder")
	public HttpResponseResult updateOrder(@RequestBody Map map) throws Exception {
		studentPaymentOrderService.updateOrder(map);
		return succeed();
	}

	// 同步商城订单发货状态
	@PostMapping("/updateShippedStatus")
	public HttpResponseResult updateShippedStatus(@RequestBody List<OrderSkuSync> orderSkuSyncs) throws Exception {
		studentPaymentOrderService.updateShippedStatus(orderSkuSyncs);
		return succeed();
	}


	/**
	 * 商城取消订单接口
	 *
	 */
	@PostMapping("/cancelOrder")
	public HttpResponseResult<OrderCancelModel> cancelOrder(@RequestParam String orderNo,@RequestParam String reason) {

		OrderCancelModel cancelModel = new OrderCancelModel();
		StudentPaymentOrder orderByOrderNo = studentPaymentOrderService.findOrderByOrderNo(orderNo);
		if (orderByOrderNo == null) {
			cancelModel.setSuccess(true);
			cancelModel.setStatus(false);
			return succeed(cancelModel);
		}
		cancelModel = studentPaymentOrderService.cancelOrder(orderByOrderNo,reason);
		return succeed(cancelModel);
	}


	// 商城获取 退款数据
	@GetMapping("/getRefundInfo")
	public HttpResponseResult<Map<String,String>> getBaseUrl(@RequestParam String orderNo) {

		SysConfig baseApiUrl = sysConfigService.findByParamName(SysConfigService.BASE_API_URL);
		StudentPaymentOrder orderByOrderNo = studentPaymentOrderService.findOrderByOrderNo(orderNo);
		Map<String,String> result = new HashMap<>();
		result.put("transNo",orderByOrderNo.getTransNo());
		result.put("baseUrl",baseApiUrl.getParanValue());
		;
		return succeed(result);
	}



	// 商城获取 退款数据  只能全额退款
	@PostMapping("/refund")
	public HttpResponseResult<BaseResult<Map<String, Object>>> refund(@RequestBody RefundModel refundModel) {

		BaseResult<Map<String, Object>> mapBaseResult = studentPaymentOrderService.refund(refundModel.getOrderNo(),refundModel.getNotifyUrl(),refundModel.getRefundNo());

		return succeed(mapBaseResult);
	}



	// 商城获取 收货状态
	@PostMapping("/receive")
	public HttpResponseResult receive(@RequestParam String orderNo,@RequestParam boolean autoReceive) {

		studentPaymentOrderService.receive(orderNo,autoReceive);

		return succeed();
	}



	// 商城获取 收货状态
	@PostMapping("/balance")
	public HttpResponseResult<BigDecimal> balance(@RequestParam Long userId) {

		BigDecimal balance = studentPaymentOrderService.balance(userId);

		return succeed(balance);
	}

	// 组合商品状态同步
	@PostMapping("/goodsStatusSynchronize")
	public HttpResponseResult<Boolean> goodsStatusSynchronize(@RequestBody List<GoodsSubModel> goodsSubModelList) {
		return succeed(goodsService.goodsStatusSynchronize(goodsSubModelList));
	}

	// 组合商品库存同步
	@PostMapping("/goodsStockCountSynchronize")
	public HttpResponseResult<Boolean> goodsStockCountSynchronize(@RequestBody List<GoodsSubStockModel> goodsSubStockModels) {
		return succeed(goodsService.goodsStockCountSynchronize(goodsSubStockModels));
	}

	@PostMapping("/schoolListByIds")
	public HttpResponseResult<List<SchoolDto>> coopListByIds(@RequestBody List<Integer> coopIds) {
		return succeed(cooperationOrganService.coopListByIds(coopIds));
	}
}
