package com.ym.mec.web.controller;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.biz.dal.entity.AppVersionInfo;
import com.ym.mec.biz.dal.entity.AppVersionInfoBak;
import com.ym.mec.biz.dal.wrapper.AppVersionInfoWrapper;
import com.ym.mec.biz.service.AppVersionInfoService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.QueryInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RequestMapping("${app-config.url.web:}/appVersionInfo")
@Api(tags = "APP版本信息服务")
@RestController
public class AppVersionInfoController extends BaseController {

	@Autowired
	private AppVersionInfoService appVersionInfoService;

	@ApiOperation("分页查询")
	@GetMapping(value = "/list")
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/list')")
	public Object getList(AppVersionInfoWrapper.AppVersionInfoQuery queryInfo) {
		return succeed(appVersionInfoService.queryPage(queryInfo));
	}

	@ApiOperation("根据app客户端查询对象")
	@ApiImplicitParam(name = "platform", value = "平台名称", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/queryByPlatform")
	public Object queryByPlatform(String platform,String localVersion) {
		List<AppVersionInfo> list = appVersionInfoService.queryNewestByPlatform(platform);
		if (CollectionUtils.isNotEmpty(list)) {
			AppVersionInfo appVersionInfo = list.get(0);
			if("2.0.9".equals(appVersionInfo.getVersion()) && "android-education".equals(appVersionInfo.getPlatform())){
				AppVersionInfoBak appVersionInfoBak = JSONObject.parseObject(JSONObject.toJSONString(appVersionInfo),AppVersionInfoBak.class);
				appVersionInfoBak.setDownloadUrl(new String(appVersionInfo.getDownloadUrl()));
				/*if (StringUtils.isEmpty(localVersion)) {
					appVersionInfoBak.setIsForceUpdate(true);
				}*/
				return succeed(appVersionInfoBak);
			}
			//如果应用版本跨多个版本时若其中任何一个版本有强制更新，返回字段标记为强制更新
			if(!appVersionInfo.getIsForceUpdate() && StringUtils.isNotEmpty(localVersion)){
				//是否需要强制更新
				AppVersionInfo forceUpdate = appVersionInfoService.checkForceUpdate(platform,localVersion);
				if(forceUpdate != null){
					appVersionInfo.setIsForceUpdate(true);
				}
			}
			/*if (StringUtils.isEmpty(localVersion)) {
				appVersionInfo.setIsForceUpdate(true);
			}*/
			return succeed(appVersionInfo);
		}
		return failed();
	}

	@ApiOperation("单查询")
	@ApiImplicitParam(name = "id", value = "ID编号", required = true, dataType = "Integer", paramType = "path")
	@GetMapping(value = "/query")
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/query')")
	public Object query(Integer id) {
		return succeed(appVersionInfoService.get(id));
	}

	@ApiOperation("新增")
	@PostMapping(value = "/add", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/add')")
	public Object add(AppVersionInfo appVersionInfo) {
		appVersionInfoService.add(appVersionInfo);
		return succeed();
	}

	@ApiOperation("更新")
	@PostMapping(value = "/update", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@PreAuthorize("@pcs.hasPermissions('appVersionInfo/update')")
	public Object update(AppVersionInfo appVersionInfo) {
		appVersionInfoService.updateVersion(appVersionInfo);
		return succeed();
	}

}
