package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.page.CooperationOrganQueryInfo;
import com.ym.mec.biz.dal.wrapper.SchoolIndexStatWrapper;
import com.ym.mec.biz.service.CooperationOrganService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RequestMapping("${app-config.url.web:}/cooperationOrgan")
@Api(tags = "合作单位(教学点)服务")
@RestController
public class CooperationOrganController extends BaseController {

    @Autowired
    private CooperationOrganService cooperationOrganService;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "新增合作单位(教学点)")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('cooperationOrgan/add')")
    public Object add(@RequestBody CooperationOrgan cooperationOrgan) {
        cooperationOrganService.insert(cooperationOrgan);
        return succeed();
    }

    @ApiOperation(value = "删除合作单位(教学点)")
    @PostMapping("/del/{id}")
    @PreAuthorize("@pcs.hasPermissions('cooperationOrgan/del')")
    public Object del(@ApiParam(value = "合作单位(教学点)编号", required = true) @PathVariable("id") Integer id) {
        cooperationOrganService.delete(id);
        return succeed();
    }

    @ApiOperation(value = "修改合作单位(教学点)")
    @PutMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('cooperationOrgan/update')")
    public Object update(@RequestBody CooperationOrgan cooperationOrgan) {
        cooperationOrgan.setUpdateTime(new Date());
        cooperationOrganService.updateCooperation(cooperationOrgan);
        return succeed();
    }

    @ApiOperation(value = "更新合作单位乐团主管")
    @PostMapping("/cooperationEdu")
    public Object cooperationEdu(@RequestBody List<Integer> organIds) {
        cooperationOrganService.updateCooperationEdu(organIds);
        return succeed();
    }

    @ApiOperation(value = "分页查询合作单位(教学点)列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('cooperationOrgan/queryPage')")
    public Object queryPage(CooperationOrganQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(cooperationOrganService.queryPage(queryInfo));
    }

    @ApiOperation(value = "根据机构编号获取合作单位(教学点)列表")
    @GetMapping("/queryByOrganId")
    @PreAuthorize("@pcs.hasPermissions('cooperationOrgan/queryByOrganId')")
    public Object queryByOrganId(String organId,Boolean enable) {
        return succeed(cooperationOrganService.queryByOrganId(organId,enable));
    }

    @ApiOperation(value = "根据合作单位id获取学校缴费")
    @GetMapping("/getCalenders")
    @PreAuthorize("@pcs.hasPermissions('cooperationOrgan/getCalenders')")
    public HttpResponseResult<List<MusicGroupPaymentCalender>> getCalenders(Integer id) {
        return succeed(cooperationOrganService.getCooperationOrganCalender4School(id));
    }


    @ApiOperation(value = "合作单位的乐团")
    @GetMapping("/musicGroupPage")
    public HttpResponseResult<List<MusicGroup>> musicGroupPage(@RequestHeader Integer coopId) {
        if (coopId == null) {
            return succeed();
        }
        return succeed(cooperationOrganService.musicGroupPage(coopId));
    }


    @ApiOperation(value = "修改学校logo")
    @PostMapping("/updateLogo")
    public HttpResponseResult<Boolean> updateLogo(@RequestBody SchoolIndexStatWrapper.SchoolData schoolData,@RequestHeader Integer coopId) {
        if (coopId == null) {
            return succeed();
        }
        CooperationOrgan cooperationOrgan = new CooperationOrgan();
        cooperationOrgan.setId(coopId);
        cooperationOrgan.setLogo(schoolData.getLogo());
        return succeed(cooperationOrganService.update(cooperationOrgan) > 0);
    }

}
