package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dto.CourseConvertDetailDto;
import com.ym.mec.biz.dal.dto.CourseConvertLogDto;
import com.ym.mec.biz.dal.entity.ClassGroupDto;
import com.ym.mec.biz.dal.page.ConvertLogQueryInfo;
import com.ym.mec.biz.service.CourseConvertLogService;
import com.ym.mec.biz.service.CourseScheduleConvertService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;

/**
* @author zx
* @date 2022/9/21 09:55
*/
@RequestMapping("${app-config.url.web:}/courseScheduleConvert")
@Api(tags = "课程转换服务")
@RestController
public class CourseScheduleConvertController extends BaseController {

    @Autowired
    private CourseScheduleConvertService courseScheduleConvertService;
    @Autowired
    private CourseConvertLogService courseConvertLogService;

    @ApiOperation(value = "获取需要转换的学员列表")
    @PostMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleConvert/queryPage')")
    public HttpResponseResult<PageInfo<CourseConvertLogDto>> queryPage(@RequestBody ConvertLogQueryInfo queryInfo){
        return succeed(courseConvertLogService.queryConvertPage(queryInfo));
    }

    @ApiOperation(value = "获取转换后的课程列表")
    @GetMapping("/queryCourse")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleConvert/queryCourse')")
    public HttpResponseResult<List<CourseConvertDetailDto>> queryCourse(Integer courseConvertId){
        return succeed(courseConvertLogService.queryCourse(courseConvertId));
    }

    @ApiOperation(value = "获取需要转换的学员列表")
    @GetMapping("/queryStudent")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleConvert/queryStudent')")
    public HttpResponseResult<HashMap<String, Object>> queryStudent(String courseIds){
        if (StringUtils.isEmpty(courseIds)){
            throw new BizException("参数校验失败");
        }
        return succeed(courseScheduleConvertService.queryConvertStudent(courseIds));
    }

    @ApiOperation(value = "课程转换")
    @PostMapping("/action")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleConvert/action')")
    public HttpResponseResult<Boolean> action(@RequestBody List<ClassGroupDto> classGroups){
        return courseScheduleConvertService.action(classGroups);
    }

    @ApiOperation(value = "课程转换还原")
    @GetMapping("/revoke")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleConvert/revoke')")
    public HttpResponseResult revoke(Integer courseConvertId){
        courseScheduleConvertService.revoke(courseConvertId);
        return succeed();
    }
}

