package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dao.CourseScheduleTeacherSalaryDao;
import com.ym.mec.biz.dal.dto.TeacherAttendancesDto;
import com.ym.mec.biz.dal.enums.TeacherSalaryConfirmStatus;
import com.ym.mec.biz.dal.page.CourseSalaryQueryInfo4Web;
import com.ym.mec.biz.dal.page.CourseScheduleTeacherSalaryQueryInfo;
import com.ym.mec.biz.service.CourseScheduleTeacherSalaryService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@RequestMapping("${app-config.url.web:}/courseScheduleTeacherSalary")
@Api(tags = "教师薪酬服务")
@RestController
public class CourseScheduleTeacherSalaryController extends BaseController {

    @Autowired
    private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
    @Autowired
    private CourseScheduleTeacherSalaryDao courseScheduleTeacherSalaryDao;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "分页查询教师薪酬列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleTeacherSalary/queryPage')")
    public Object queryPage(CourseScheduleTeacherSalaryQueryInfo queryInfo) {
        return succeed(courseScheduleTeacherSalaryService.querySalaries(queryInfo));
    }

    @ApiOperation(value = "获取已结算的课酬记录")
    @GetMapping("/findIsSettlementCourseSalarys")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleTeacherSalary/findIsSettlementCourseSalarys')")
    public HttpResponseResult findIsSettlementCourseSalarys(CourseSalaryQueryInfo4Web queryInfo){
        queryInfo.setOrganIdList(organizationService.getEmployeeOrgan(queryInfo.getOrganIdList()));

        Map<String, Object> result=new HashMap<>();
        result.put("pageInfo", courseScheduleTeacherSalaryService.findIsSettlementCourseSalarys(queryInfo));
        int i = courseScheduleTeacherSalaryDao.countOpenConfirmSalarysWithMonth(queryInfo.getMonth(), TeacherSalaryConfirmStatus.UNCONFIRMED, queryInfo.getTeacherId());
        if(i>0){
            result.put("isOpenConfirm", 1);
        }else{
            int j = courseScheduleTeacherSalaryDao.countOpenConfirmSalarysWithMonth(queryInfo.getMonth(), null, queryInfo.getTeacherId());
            if(j<=0){
                result.put("isOpenConfirm", 2);
            }else{
                result.put("isOpenConfirm", 0);
            }
        }
        return succeed(result);
    }

    @ApiOperation(value = "开启课酬确认")
    @GetMapping("/openSalaryConfirm")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleTeacherSalary/openSalaryConfirm')")
    public HttpResponseResult openSalaryConfirm(String month){
        courseScheduleTeacherSalaryService.openSalaryConfirm(month);
        return succeed();
    }

    @ApiOperation(value = "关闭课酬确认")
    @GetMapping("/closeSalaryConfirm")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleTeacherSalary/closeSalaryConfirm')")
    public HttpResponseResult closeSalaryConfirm(String month){
        courseScheduleTeacherSalaryService.closeSalaryConfirm(month);
        return succeed();
    }

    @ApiOperation(value = "获取指定课程上的教师信息")
    @GetMapping("/getCourseTeachers")
    @PreAuthorize("@pcs.hasPermissions('courseScheduleTeacherSalary/getCourseTeachers')")
    public HttpResponseResult<List<TeacherAttendancesDto>> getCourseTeachers(Long courseScheduleId){
        if(Objects.isNull(courseScheduleId)){
            throw new BizException("请指定课程");
        }
        return succeed(courseScheduleTeacherSalaryService.getCourseTeachers(courseScheduleId));
    }
}
