package com.ym.mec.web.controller;


import com.ym.mec.biz.dal.dto.CourseShareDto;
import com.ym.mec.biz.dal.dto.StudentManageListDto;
import com.ym.mec.biz.dal.entity.CourseShare;
import com.ym.mec.biz.dal.page.CourseShareQueryInfo;
import com.ym.mec.biz.service.CourseShareService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author 袁亮
 * @version v1.0
 * @apiNote 课程分享接口
 */
@Api(tags = "课程分享接口控制器")
@RequestMapping("${app-config.url.web:}/courseShare")
@RestController
public class CourseShareController extends BaseController {

    @Autowired
    private CourseShareService courseShareService;

    @ApiOperation(value = "保存分享列表")
    @PostMapping("/saveCourseShare")
    @PreAuthorize("@pcs.hasPermissions('courseShare/saveCourseShare')")
    public HttpResponseResult<List<CourseShare>> saveCourseShare(@Validated @RequestBody CourseShareDto courseShareDto) {
        courseShareService.saveCourseShare(courseShareDto);
        return succeed();
    }

    @ApiOperation(value = "一键添加保存分享列表")
    @PostMapping("/saveCourseShareByOne")
    @PreAuthorize("@pcs.hasPermissions('courseShare/saveCourseShareByOne')")
    public HttpResponseResult<List<CourseShare>> saveCourseShareByOne(@Validated @RequestBody CourseShareDto courseShareDto) {
        courseShareService.saveCourseShareByOne(courseShareDto);
        return succeed();
    }

    @ApiOperation(value = "查询已经分享过的学生")
    @GetMapping("/queryCourseSharedStudent")
    @PreAuthorize("@pcs.hasPermissions('courseShare/queryCourseSharedStudent')")
    public HttpResponseResult<PageInfo<StudentManageListDto>> queryCourseSharedStudent(
            @Validated CourseShareQueryInfo shareQueryInfo) {
        return succeed(courseShareService.queryCourseSharedStudent(shareQueryInfo));
    }

    @ApiOperation(value = "查询可以分享的学生列表")
    @GetMapping("/queryAllCourseShareStudent")
    @PreAuthorize("@pcs.hasPermissions('courseShare/queryAllCourseShareStudent')")
    public HttpResponseResult<PageInfo<StudentManageListDto>> queryAllCourseShareStudent(
            @Validated CourseShareQueryInfo shareQueryInfo) {
        return succeed(courseShareService.queryAllCourseShareStudent(shareQueryInfo));
    }

    @ApiOperation(value = "导入学员")
    @PostMapping(value = "/importStudent")
    @PreAuthorize("@pcs.hasPermissions('courseShare/importStudent')")
    public HttpResponseResult<String> importStudent(@RequestParam("file") MultipartFile file,
                                                    @RequestParam("courseId") Long courseId) {
        String msg = courseShareService.importStudent(file, courseId);
        HttpResponseResult<String> succeed = succeed();
        succeed.setMsg(msg);
        return succeed;
    }

    @ApiOperation("删除共享的学员")
    @PostMapping("/deleteShareStudent")
    @PreAuthorize("@pcs.hasPermissions('courseShare/deleteShareStudent')")
    public Object deleteShareStudent(@Validated @RequestBody CourseShareDto courseShareDto) {
        courseShareService.deleteStudent(courseShareDto);
        return succeed();
    }
}
