package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dto.CourseHomeworkWrapper;
import com.ym.mec.biz.dal.page.ExtraExercilseQueryInfo;
import com.ym.mec.biz.service.ExtracurricularExercisesReplyService;
import com.ym.mec.biz.service.ExtracurricularExercisesService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Objects;

@Api(tags = "课外训练服务")
@RequestMapping("${app-config.url.web:}/extracurricularExercises")
@RestController
public class ExtracurricularExercisesController extends BaseController {

    @Autowired
    private ExtracurricularExercisesService extracurricularExercisesService;
    @Autowired
    private ExtracurricularExercisesReplyService extracurricularExercisesReplyService;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "获取课外训练列表")
    @GetMapping("/queryPageList")
    private HttpResponseResult findExtraExercilses(ExtraExercilseQueryInfo queryInfo){
        queryInfo.setOrganIdList(organizationService.getEmployeeOrgan(queryInfo.getOrganIdList()));
        return succeed(extracurricularExercisesService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取课外训练详情")
    @GetMapping("/findStudentExtraExerciseDetail")
    private HttpResponseResult findStudentExtraExerciseDetail(Long studentExerciseId){
        if(Objects.isNull(studentExerciseId)){
            return failed("请选择课外训练");
        }
        return succeed(extracurricularExercisesReplyService.findStudentExtraExerciseDetail(studentExerciseId));
    }


    @ApiOperation(value = "获取课外训练列表")
    @GetMapping("/queryPageList/v2")
    private HttpResponseResult<PageInfo<CourseHomeworkWrapper.WebExtraCourseHomework>> findExtraExercilsesV2(ExtraExercilseQueryInfo queryInfo){
        queryInfo.setOrganIdList(organizationService.getEmployeeOrgan(queryInfo.getOrganIdList()));
        return succeed(extracurricularExercisesService.findExtraExercilsesV2(queryInfo));
    }

    @ApiOperation(value = "获取课外训练详情")
    @GetMapping("/findStudentExtraExerciseDetail/v2")
    private HttpResponseResult<CourseHomeworkWrapper.CourseHomeworkList> findStudentExtraExerciseDetailV2(Long studentExerciseId){
        if(Objects.isNull(studentExerciseId)){
            return failed("请选择课外训练");
        }
        return succeed(extracurricularExercisesReplyService.findStudentExtraExerciseDetailV2(studentExerciseId));
    }
}
