package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.GoodsCategory;
import com.ym.mec.biz.dal.entity.GoodsProcurement;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.GoodsQuery;
import com.ym.mec.biz.dal.page.GoodsQueryInfo;
import com.ym.mec.biz.dal.wrapper.GoodsWrapper;
import com.ym.mec.biz.service.GoodsCategoryService;
import com.ym.mec.biz.service.GoodsService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/goods")
@Api(tags = "商品(教材、辅件)服务")
@RestController
public class GoodsController extends BaseController {

    @Autowired
    private GoodsService goodsService;
    @Autowired
    private GoodsCategoryService goodsCategoryService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private EmployeeDao employeeDao;

    @ApiOperation(value = "新增商品(教材、辅件)")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('goods/add')")
    public Object add(@RequestBody GoodsWrapper.Goods goods) {
        goodsService.addGoods(Collections.singletonList(goods), sysUserService.getUserId());
        return succeed();
    }

    @ApiOperation(value = "新增商品(教材、辅件)")
    @PostMapping("/addBatch")
    @PreAuthorize("@pcs.hasPermissions('goods/addBatch')")
    public Object addBatch(@RequestBody List<GoodsWrapper.Goods> goodsList) {
        goodsService.addGoods(goodsList, sysUserService.getUserId());
        return succeed();
    }

/*    @ApiOperation(value = "新增商品清单")
    @PostMapping("/addGoodsProcurement")
    @PreAuthorize("@pcs.hasPermissions('goods/addGoodsProcurement')")
    public Object addGoodsProcurement(GoodsProcurement goodsProcurement){
        goodsProcurement.setOperatorId(sysUserService.getUserId());
        goodsService.addGoodsProcurement(goodsProcurement);
        return succeed();
    }*/

    @ApiOperation(value = "删除商品(教材、辅件)")
    @PostMapping("/del/{id}")
    @PreAuthorize("@pcs.hasPermissions('goods/del')")
    public Object del(@ApiParam(value = "商品(教材、辅件)编号", required = true) @PathVariable("id") Integer id){
    	Goods goods = goodsService.get(id);
    	if(goods == null){
    		return failed("参数错误");
    	}
    	goods.setStatus(YesOrNoEnum.NO);
    	goods.setUpdateTime(new Date());
        goodsService.update(goods);
        return succeed();
    }

    @ApiOperation(value = "修改商品件状态")
    @PostMapping("/updateGoodsStatus")
    @PreAuthorize("@pcs.hasPermissions('goods/updateGoodsStatus')")
    public Object updateGoodsStatus(Integer goodsId, Integer status) {
        goodsService.updateGoodsStatus(goodsId, status);
        return succeed();
    }

    @ApiOperation(value = "修改商品(教材、辅件)")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('goods/update')")
    public Object update(@RequestBody GoodsWrapper.Goods goods){
        goods.setUpdateTime(new Date());
        goodsService.updateGoods(goods);
        return succeed();
    }

    @ApiOperation(value = "根据商品(教材、辅件)编号查询商品(教材、辅件)")
    @GetMapping("/get/{id}")
    public Object get(@ApiParam(value = "商品(教材、辅件)编号", required = true) @PathVariable("id") Integer id){
        GoodsWrapper.Goods goods = goodsService.getDetail(id);
        GoodsCategory goodsCategory = goodsCategoryService.get(goods.getGoodsCategoryId());
        if(Objects.nonNull(goodsCategory)){
            goods.setGoodsCategoryName(goodsCategory.getName());
        }
        return succeed(goods);
    }

    @ApiOperation(value = "分页查询商品(教材、辅件)列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('goods/queryPage')")
    public Object queryPage(GoodsQueryInfo queryInfo){
        queryInfo.setOrganId(null);
        Employee employee = employeeDao.get(sysUserService.getUserId());
        if (queryInfo.getStudentShowOrganId() == null &&
                queryInfo.getEducationShowOrganId() == null &&
                queryInfo.getCourseFeeShowOrganId() == null &&
                queryInfo.getMemberFeeShowOrganId() == null &&
                queryInfo.getReplacementShowOrganId() == null) {
            queryInfo.setOrganId(employee.getOrganIdList());
        }else if(StringUtils.isEmpty(employee.getOrganIdList())){
            return failed("用户所在分部异常");
        }
        return succeed(goodsService.queryPage(queryInfo));
    }

    @ApiOperation(value = "通过科目编号、商品分类 查询商品(教材、辅件)列表")
    @GetMapping("/queryGoodsBySubId")
    @PreAuthorize("@pcs.hasPermissions('goods/queryGoodsBySubId')")
    public Object findGoodsBySubId(GoodsQuery goodsQuery){
        return succeed(goodsService.findGoodsBySubId(goodsQuery));
    }


    @ApiOperation(value = "查询商城商品列表分页")
    @GetMapping("/queryGoodsSubByPage")
//    @PreAuthorize("@pcs.hasPermissions('goods/queryGoodsSubByPage')")
    public Object queryGoodsSubByPage(GoodsWrapper.GoodsSubQuery query) {
        return succeed(goodsService.queryGoodsSubByPage(query));

    }

    @ApiOperation(value = "查询商品类型")
    @GetMapping("/queryGoodsTypeList")
    public Object queryGoodsTypeList() {
        return succeed(goodsService.queryGoodsTypeList());
    }

    @ApiOperation(value = "查询商品分类")
    @GetMapping("/queryGoodsCategoryList")
    public Object queryGoodsCategoryList() {
        return succeed(goodsService.queryGoodsCategoryList());
    }

    @ApiOperation(value = "查询商品品牌")
    @GetMapping("/queryGoodsBrandList")
    public Object queryGoodsBrandList() {
        return succeed(goodsService.queryGoodsBrandList());
    }

    @ApiOperation(value = "复制商品")
    @PostMapping("/copyGoods")
    public HttpResponseResult<Object> copyGoods(Integer goodsId){
        goodsService.copyGoods(goodsId);
        return succeed();
    }
}
