package com.ym.mec.web.controller;

import com.google.common.collect.Lists;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.ImGroupDto;
import com.ym.mec.biz.dal.dto.ImGroupMemberDto;
import com.ym.mec.biz.dal.dto.ImUserFriendDto;
import com.ym.mec.biz.dal.dto.NameDto;
import com.ym.mec.biz.dal.entity.ImGroup;
import com.ym.mec.biz.dal.entity.ImGroupMemberPlus;
import com.ym.mec.biz.dal.enums.im.ClientEnum;
import com.ym.mec.biz.dal.page.ImGroupNoticeQueryInfo;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.vo.ImUserWrapper;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/imGroup")
@Api(tags = "IM群服务")
@RestController
public class ImGroupController extends BaseController {

	@Resource
	private ImGroupService imGroupService;

	@Resource
	private ImUserFriendService imUserFriendService;

	@Resource
	private ImGroupMemberService imGroupMemberService;

    @Resource
    private ImGroupMemberPlusService imGroupMemberPlusService;

	@Resource
	private ImGroupNoticeService imGroupNoticeService;

	@Resource
	private SysUserService sysUserService;

	@ApiOperation(value = "全体禁言/解除禁言")
	@PostMapping("/muteAll")
	public HttpResponseResult<Object> muteAll(@RequestBody ImGroupWrapper.GroupMuteAll groupMuteAll) {
		if (StringUtils.isEmpty(groupMuteAll.getGroupId()) || groupMuteAll.getMuteAll() == null){
			throw new com.microsvc.toolkit.common.webportal.exception.BizException("参数错误");
		}
		imGroupService.muteAll(groupMuteAll);
		return succeed();
	}

	@ApiOperation("查询群列表")
	@GetMapping(value = "/queryGroupList")
	public HttpResponseResult<List<ImGroup>> queryGroupList(String search,String groupType) {
		return succeed(imGroupService.queryByUserId(sysUserService.getUserId(), search,groupType));
	}

	@ApiOperation("创建群聊")
	@PostMapping(value = "/createGroup")
	public HttpResponseResult<String> queryGroupList(@RequestBody ImGroupDto imGroupDto) {
		return succeed(imGroupService.createGroup(imGroupDto));
	}

	@ApiOperation("查询群详情")
	@GetMapping(value = "/queryGroupDetail")
	public HttpResponseResult<ImGroup> queryGroupDetail(String imGroupId) {
		if (imGroupId.contains("S") || imGroupId.contains("I")){
			imGroupId = imGroupId.substring(1);
		}
		ImGroup imGroup = imGroupService.get(imGroupId);

        if (imGroup == null) {
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
        ImGroupMemberPlus admin = imGroupMemberPlusService.lambdaQuery()
            .eq(ImGroupMemberPlus::getImGroupId, imGroup.getId())
            .eq(ImGroupMemberPlus::getUserId, sysUserService.getUser().getId())
            .last("limit 1").one();
        if (admin == null) {
            // 该用户不在该群，无权限查看，处理用户退出群后，查看历史消息校验
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
		// 设置默认头像
		if (Objects.nonNull(imGroup) && Objects.nonNull(imGroup.getGroupType()) && StringUtils.isBlank(imGroup.getImg())) {
			imGroup.setImg(imGroup.getGroupType().getAvatar());
		}
		return succeed(imGroup);
	}

	@ApiOperation("查询群详情(重构版)")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "groupId", value = "群编号", required = true, dataType = "String")
	})
	@GetMapping(value = "/getDetail/{groupId}")
	public HttpResponseResult<ImGroup> groupDetail(@PathVariable("groupId") String imGroupId) {
		if (imGroupId.contains("S") || imGroupId.contains("I")){
			imGroupId = imGroupId.substring(1);
		}
        ImGroup imGroup = imGroupService.get(imGroupId);

        if (imGroup == null) {
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
        ImGroupMemberPlus admin = imGroupMemberPlusService.lambdaQuery()
            .eq(ImGroupMemberPlus::getImGroupId, imGroup.getId())
            .eq(ImGroupMemberPlus::getUserId, sysUserService.getUser().getId())
            .last("limit 1").one();
        if (admin == null) {
            // 该用户不在该群，无权限查看，处理用户退出群后，查看历史消息校验
            return failed(HttpStatus.NO_CONTENT, "群组不存在");
        }
		return succeed(imGroup);
	}

	@ApiOperation("修改群信息")
	@PostMapping(value = "/updateImGroup")
	public HttpResponseResult updateImGroup(@RequestBody ImGroup imGroup) {
		imGroup.setUpdateTime(new Date());
		imGroupService.updateImGroup(imGroup);
		return succeed();
	}

	@ApiOperation("修改群成员信息")
	@PostMapping(value = "/updateNickName")
	public HttpResponseResult updateNickName(Integer userId,String nickName) {
		imGroupService.updateNickName1(userId,nickName);
		return succeed();
	}

	@ApiOperation("查询群成员列表")
	@GetMapping(value = "/queryGroupMemberList")
	public HttpResponseResult<List<ImGroupMemberDto>> queryGroupMemberList(String imGroupId,String search) {
		return succeed(imGroupService.queryMemberById(imGroupId,search));
	}

	@ApiOperation("查询群学生列表")
	@GetMapping(value = "/queryGroupStudentList")
	public HttpResponseResult<List<ImGroupMemberDto>> queryGroupStudentList(String imGroupId) {
		return succeed(imGroupService.queryMemberById(imGroupId).stream().filter(e -> StringUtils.isBlank(e.getRoleType())).collect(Collectors.toList()));
	}

	@ApiOperation("查询群成员详情")
	@GetMapping(value = "/queryGroupMemberDetail")
	public HttpResponseResult<ImGroupMemberDto> queryGroupMemberDetail(String imGroupId, Integer userId) {
		return succeed(imGroupService.queryMember(imGroupId, userId));
	}

	@ApiOperation("查询好友详情")
	@GetMapping(value = "/queryFriendDetail")
	public HttpResponseResult<ImUserFriendDto> queryFriendDetail(Integer userId) {
		Integer currentUserId = sysUserService.getUserId();
		ImUserFriendDto dto = imUserFriendService.queryFriendDetail(currentUserId, userId);
		if (dto == null) {
			dto = new ImUserFriendDto();
			SysUser user = sysUserService.queryUserById(userId);
			dto.setFriend(user);
			//这里由原来的优先取老师的realName改成了优先取学员的username，
			// 因为管理员在修改通讯录备注的时候，如果是多角色，优先改的是学员的username
			List<String> userTypes = Lists.newArrayList(user.getUserType().split(","));
			if (userTypes.size() > 1
				|| userTypes.contains(ClientEnum.TEACHER.getCode())
				|| userTypes.contains(ClientEnum.SYSTEM.getCode())) {
				// 增加用户身份数量 > 1 或者包含老师身份，统一使用实名
				dto.setFriendNickname(Optional.ofNullable(user.getRealName()).filter(StringUtils::isNotBlank).orElse(user.getUsername()));
			} else if (user.getUserType().contains("STUDENT") || user.getUserType().contains("SCHOOL")) {
				dto.setFriendNickname(user.getUsername());
			} else {
				dto.setFriendNickname(user.getRealName());
			}
			dto.setFriendId(userId);
		}
		return succeed(dto);
	}

	@ApiOperation("查询好友详情(重构版)")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer")
	})
	@GetMapping(value = "/imUserFriend/getDetail/{userId}")
	public HttpResponseResult<ImUserWrapper.ImUserFriend> getImUserFriendDetail(@PathVariable("userId") Integer userId) {
		Integer currentUserId = sysUserService.getUserId();
		ImUserFriendDto dto = imUserFriendService.queryFriendDetail(currentUserId, userId);
		if (dto == null) {
			dto = new ImUserFriendDto();
			SysUser user = sysUserService.queryUserById(userId);
			dto.setFriend(user);
			//这里由原来的优先取老师的realName改成了优先取学员的username，
			// 因为管理员在修改通讯录备注的时候，如果是多角色，优先改的是学员的username
			if (user.getUserType().contains("STUDENT") || user.getUserType().contains("SCHOOL")) {
				dto.setFriendNickname(user.getUsername());
			} else {
				dto.setFriendNickname(user.getRealName());
			}
			dto.setFriendId(userId);
		}

		// 返回数据结构封装
		ImUserWrapper.ImUserFriend userFriend = ImUserWrapper.ImUserFriend.builder()
			.id(dto.getId())
			.userId(dto.getUserId().longValue())
			.friendId(dto.getFriendId().longValue())
			.friendAvatar(dto.getFriend().getAvatar())
			.friendNickname(dto.getFriendNickname())
			.memo(dto.getMemo())
			.updateTime(dto.getUpdateTime())
			.createTime(dto.getCreateTime())
			.roleType(dto.getRoleType())
			.friendType(dto.getRoleType())
			.build();
		return succeed(userFriend);
	}

	@ApiOperation("查询好友列表")
	@GetMapping(value = "/queryFriendList")
	public HttpResponseResult<List<ImUserFriendDto>> queryFriendList(String search) {
		return succeed(imUserFriendService.queryFriendListByUserId(sysUserService.getUserId(), search));
	}

	@ApiOperation("查询好友列表(重构版)")
	@GetMapping(value = "/imUserFriend/queryAll")
	public HttpResponseResult<List<ImUserWrapper.ImUserFriend>> queryImUserFriendList(String search) {

		// 好友关系列表
		List<ImUserFriendDto> friendDtos = imUserFriendService.queryFriendListByUserId(sysUserService.getUserId(), search);

		List<ImUserWrapper.ImUserFriend> wrappers = Lists.newArrayList();
		if (CollectionUtils.isNotEmpty(friendDtos)) {

			// 封装返回数据结构
			for (ImUserFriendDto item : friendDtos) {

				wrappers.add(ImUserWrapper.ImUserFriend.builder()
						.id(item.getId())
						.userId(item.getUserId().longValue())
						.friendId(item.getFriendId().longValue())
						.friendAvatar(item.getFriend().getAvatar())
						.friendNickname(item.getFriendNickname())
						.memo(item.getMemo())
						.roleType(item.getRoleType())
						.friendType(item.getRoleType())
						.updateTime(item.getUpdateTime())
						.createTime(item.getCreateTime())
					.build());
			}
		}

		return succeed(wrappers);
	}

	@ApiOperation("查询好友中的学生列表")
	@GetMapping(value = "/queryFriendStudentList")
	public HttpResponseResult<List<ImUserFriendDto>> queryFriendStudentList(String search) {
		return succeed(imUserFriendService.queryFriendListByUserId(sysUserService.getUserId(), search).stream().filter(e -> StringUtils.isBlank(e.getTags()))
				.collect(Collectors.toList()));
	}

	@ApiOperation("查询群公告列表")
	@GetMapping(value = "/queryNoticeList")
	public Object queryNoticeList(ImGroupNoticeQueryInfo queryInfo) {
		return succeed(imGroupNoticeService.queryPage(queryInfo));
	}

	@ApiOperation("添加群成员(单个用户)")
	@PostMapping(value = "/addGroupMember")
	@ApiImplicitParams({ @ApiImplicitParam(name = "imGroupId", value = "群编号", required = true, dataType = "String"),
			@ApiImplicitParam(name = "roleType", value = "指导老师,乐队指导,乐团主管,运营主管,学员不传", required = true, dataType = "String"),
			@ApiImplicitParam(name = "isAdmin", value = "是否管理员", required = true, dataType = "Boolean"),
			@ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer")})
	public Object addGroupMember(String imGroupId, Integer userId, String roleType, boolean isAdmin) {
		return succeed(imGroupMemberService.join(imGroupId,userId,roleType,isAdmin));
	}

	@ApiOperation("退群")
	@PostMapping(value = "/quitGroup")
	@AuditLogAnnotation(operateName = "退群",interfaceURL = "imGroup/quitGroup")
	@ApiImplicitParams({ @ApiImplicitParam(name = "imGroupId", value = "群编号", required = true, dataType = "String"),
			@ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "String")})
	public Object quitGroup(String imGroupId, String userId) {
		List<Integer> userIds = Arrays.stream(userId.split(",")).mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
		return succeed(imGroupMemberService.quit(imGroupId,userIds));
	}

    @ApiOperation(" 移交群主")
    @AuditLogAnnotation(operateName = "移交群主",interfaceURL = "imGroup/groupChangeOwner")
    @PostMapping(value = "/groupChangeOwner")
    public Object groupChangeOwner(@RequestBody ImGroupWrapper.GroupTransferOwner groupTransferOwner) {
		if (StringUtils.isEmpty(groupTransferOwner.getImGroupId()) || StringUtils.isEmpty(groupTransferOwner.getUserId())){
			throw new com.microsvc.toolkit.common.webportal.exception.BizException("参数错误");
		}
		groupTransferOwner.setCurrentOwner(sysUserService.getUserId().toString());
		if(StringUtils.equals(groupTransferOwner.getUserId(),groupTransferOwner.getCurrentOwner())){
			throw new com.microsvc.toolkit.common.webportal.exception.BizException("不能转交给自己");
		}
		imGroupService.transferOwner(groupTransferOwner);
		return succeed();
    }

	@ApiOperation("解散群")
	@PostMapping(value = "/dismissGroup")
	@AuditLogAnnotation(operateName = "解散群",interfaceURL = "imGroup/dismissGroup")
	@ApiImplicitParam(name = "imGroupId", value = "群编号", required = true, dataType = "String")
	public HttpResponseResult<Object> dismissGroup(String imGroupId) {
		imGroupService.dismissGroup(imGroupId,sysUserService.getUserId().toString());
		return succeed();
	}

	@ApiOperation("修改角色")
	@PostMapping(value = "/updateRoleType")
	@ApiImplicitParams({ @ApiImplicitParam(name = "imGroupId", value = "群编号", required = true, dataType = "String"),
			@ApiImplicitParam(name = "roleType", value = "指导老师,乐队指导,乐团主管,运营主管,学员不传", required = true, dataType = "String"),
			@ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer")})
	public Object updateRoleType(String imGroupId, Integer userId, String roleType) {
		return succeed(imGroupMemberService.updateRoleType(imGroupId,userId,roleType));
	}

	@ApiOperation("获取群聊、用户详情")
	@PostMapping(value = "/queryDetail")
	public HttpResponseResult<List<NameDto>> queryDetail(@RequestBody List<NameDto> nameDto) {
		return succeed(imGroupService.queryDetail(nameDto));
	}


    @ApiOperation("查询好友列表")
    @PostMapping(value = "/schoolQueryFriendList")
    public HttpResponseResult<List<ImUserFriendDto>> schoolQueryFriendList(@RequestBody ImGroupWrapper.ImQuery query) {
        query.setUserId(sysUserService.getUserId());
        return succeed(imUserFriendService.queryFriendListByUserId(query));
    }


    @ApiOperation("查询群列表")
    @PostMapping(value = "/schoolQueryGroupList")
    public HttpResponseResult<List<ImGroup>> schoolQueryGroupList(@RequestBody ImGroupWrapper.ImQuery query) {
        query.setUserId(sysUserService.getUserId());
        return succeed(imGroupService.queryByUserId(query));
    }



	@GetMapping("/groupTransfer")
	public Object groupTransfer(@RequestParam(required = false) String startTime,
								@RequestParam(required = false) String endTime,
								@RequestParam(required = false) Integer type
                                ) {
		ExecutorService executor = Executors.newSingleThreadExecutor();
		executor.execute(() -> imGroupService.groupTransfer(startTime, endTime,type));
		return succeed();
	}

	@GetMapping("/importUser")
	public Object importUser() {
		ExecutorService executor = Executors.newSingleThreadExecutor();
		executor.execute(() -> imGroupService.importUser());
		return succeed();
	}


    @GetMapping("/deleteGroup")
    public Object deleteGroup() {
        ExecutorService executor = Executors.newSingleThreadExecutor();
        executor.execute(() -> imGroupService.deleteGroup());
        return succeed();
    }

	@ApiOperation("主动退群")
	@PostMapping(value = "/quitGroup/v2")
	public Object quitGroupV2(String imGroupId, String userId) {
		List<Integer> userIds = Arrays.stream(userId.split(",")).mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
		return succeed(imGroupMemberService.quit(imGroupId,userIds,false));
	}

}
