package com.ym.mec.web.controller;

import com.google.common.collect.Lists;
import com.yonge.log.wrapper.HistoryMessageTencentWrapper;
import com.ym.mec.biz.service.ImGroupService;
import com.ym.mec.biz.service.UploadFileService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.entity.UploadReturnBean;
import com.yonge.log.dal.model.HistoryMessage;
import com.yonge.log.dal.model.HistoryMessageTencent;
import com.yonge.log.service.HistoryMessageService;
import com.yonge.log.service.HistoryMessageTenantService;
import com.yonge.mongodb.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/imHistoryMessage")
@Api(tags = "系统操作日志")
@Slf4j
@RestController
public class ImHistoryMessageController extends BaseController {

    @Autowired
    private HistoryMessageService historyMessageService;
    @Autowired
    private UploadFileService uploadFileService;

    @Autowired
    private ImGroupService imGroupService;

    @Autowired
    private HistoryMessageTenantService historyMessageTenantService;

    @GetMapping("/save")
    public Object saveImHistoryMessage(String fileDir) throws Exception {
        File file = new File(fileDir);
        UploadReturnBean uploadReturnBean = uploadFileService.uploadImHistoryMsgFile(file);
        historyMessageService.saveImHistoryMessage(file);
        return succeed();
    }


    @ApiOperation("IM导入消息")
    @PostMapping(value = "/ImportIM")
    public void ImportIM() {

        ExecutorService executor = Executors.newSingleThreadExecutor();
        executor.execute(() -> {

            // 100个线程的无界线程池
            com.yonge.mongodb.PageInfo<HistoryMessage> info;
//        historyMessageService.updateAllStatus(0);
            //计算总数据量
            int count = imGroupService.queryInfoCount(3);
            log.info("总数据量："+count);
            //计算调用次数
            int num = (int) Math.ceil(count / 100);
            for (int i = 0; i <= num; i++) {
                int size = 100;
                //获取融云消息
                //List<ImGroup> list = imGroupService.lambdaQuery().last("limit "+(page-1)*size+","+size).list();
                info = imGroupService.getRongYunInfo(1, size,3);
                List<HistoryMessage> footer = info.getRows();
                if (CollectionUtils.isEmpty(footer)) {
                    break;
                }
                //IM导入
                try {
                    // 过滤 groupId为空的
                    footer = footer.stream().filter(o -> StringUtils.isNotBlank(o.getGroupId())).collect(Collectors.toList());
                    imGroupService.importInfo(footer);
                } catch (Exception e) {
                    log.error("IM导入失败",e);
                }
                log.info("已导入：" + (i + 1) * size);
            }
        });
    }


    @ApiOperation("IM导入消息")
    @PostMapping(value = "/ImportIMPrivate")
    public void ImportIMPrivate() {

        ExecutorService executor = Executors.newSingleThreadExecutor();
        executor.execute(() -> {
            com.yonge.mongodb.PageInfo<HistoryMessage> info;
//        historyMessageService.updateAllStatus(0);
            //计算总数据量
            int count = imGroupService.queryInfoCount(1);
            log.info("总数据量："+count);
            //计算调用次数
            int num = (int) Math.ceil(count / 100);
            for (int i = 0; i <= num; i++) {
                int size = 100;
                //获取融云消息
                //List<ImGroup> list = imGroupService.lambdaQuery().last("limit "+(page-1)*size+","+size).list();
                info = imGroupService.getRongYunInfo(1, size,1);
                List<HistoryMessage> footer = info.getRows();
                if (CollectionUtils.isEmpty(footer)) {
                    break;
                }
                //IM导入
                try {
                    imGroupService.importInfo(footer);
                } catch (Exception e) {
                    log.error("IM导入失败",e);
                }
                log.info("已导入：" + (i + 1) * size);
            }
        });
    }

    @ApiOperation("融云im To 腾讯")
    @PostMapping(value = "/imToTencent")
    public void imToTencent() throws Exception {

        ExecutorService executor = Executors.newSingleThreadExecutor();
        executor.execute(() -> {
            // 删除旧数据
//        historyMessageTenantService.deleteOld();


            // 100个线程的无界线程池
            ExecutorService executorService = Executors.newFixedThreadPool(100);
            com.yonge.mongodb.PageInfo<HistoryMessage> info;
//        historyMessageService.updateAllStatus(0);
            //计算总数据量
            long count = historyMessageService.querySyncCount();
            log.info("总数据量："+count);
            //计算调用次数
            long num = (int) Math.ceil(count / 200);
            for (long i = 0; i <=num ; i++) {
                int size = 200;
                //获取融云消息
                //List<ImGroup> list = imGroupService.lambdaQuery().last("limit "+(page-1)*size+","+size).list();
                info = historyMessageService.getImToTencent(1, size);
                List<HistoryMessage> footer = info.getRows();
                if (CollectionUtils.isEmpty(footer)) {
                    break;
                }
                //IM转换
                List<Future<?>> list = new ArrayList<>();
                for (List<HistoryMessage> historyMessages : Lists.partition(footer, 10)) {
                    list.add(executorService.submit(() -> {
                        try {
                            imGroupService.imToTencent(historyMessages);
                        } catch (Exception e) {
                            log.error("IM转换失败",e);
                        }
                    }));
                }
                for (Future<?> future : list) {
                    try {
                        future.get();
                    } catch (Exception e){
                        log.error("IM转换失败",e);
                    }
                }
                log.info("已转换：" + (i + 1) * size);
            }
            executorService.shutdown();
        });
    }

    @ApiOperation("初始化消息体")
    @PostMapping(value = "/initMsgBodyJson")
    public void initMsgBodyJson() throws Exception {

        ExecutorService executor = Executors.newSingleThreadExecutor();
        executor.execute(() -> {
            //计算总数据量
            int count = historyMessageTenantService.initMsgBodyJsonCount();
            log.info("总数据量：" + count);
            //计算调用次数
            int num = (int) Math.ceil(count / 100);
            for (int i = 0; i <= num; i++) {
                int size = 100;
                //获取融云消息
                //List<ImGroup> list = imGroupService.lambdaQuery().last("limit "+(page-1)*size+","+size).list();
                historyMessageTenantService.initMsgBodyJson(1, size);
                log.info("已转换：" + (i + 1) * size);
            }
        });
    }

    @ApiOperation("历史聊天记录查询")
    @PostMapping(value = "/historyMessage")
    public HttpResponseResult<PageInfo<HistoryMessageTencentWrapper.HistoryMessageTencent>> historyMessage(@RequestBody  HistoryMessageTencentWrapper.HistoryMessageTencentQuery query) throws Exception {

        return succeedData(imGroupService.historyMessage(query));
    }


}
