package com.ym.mec.web.controller;


import com.ym.mec.biz.dal.dto.LessonExaminationResultDto;
import com.ym.mec.biz.dal.dto.StudentLessonExaminationDto1;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.page.LessonExaminationQueryInfo;
import com.ym.mec.biz.service.LessonExaminationService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.StudentLessonExaminationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 进度评测表(LessonExamination)表控制层
 *
 * @author zx
 * @since 2023-04-03 18:20:19
 */
@Api(tags = "进度评测表")
@RestController
@RequestMapping("${app-config.url.web:}/lessonExamination")
public class LessonExaminationController extends BaseController {

    @Resource
    private LessonExaminationService lessonExaminationService;
    @Resource
    private OrganizationService organizationService;
    @Resource
    private StudentLessonExaminationService studentLessonExaminationService;

    @ApiOperation(value = "获取作业学员列表")
    @PostMapping("queryStudentPage")
    @PreAuthorize("@pcs.hasPermissions('lessonExamination/queryStudentPage')")
    public HttpResponseResult<List<StudentLessonExaminationDto1>> queryStudentPage(Long lessonExaminationId,Integer subjectId) {
        return succeed(studentLessonExaminationService.queryStudentPage(lessonExaminationId,subjectId));
    }

    @ApiOperation(value = "获取作业列表")
    @PostMapping("queryPage")
    @PreAuthorize("@pcs.hasPermissions('lessonExamination/queryPage')")
    public HttpResponseResult<PageInfo<LessonExaminationResultDto>> queryPage(@RequestBody LessonExaminationQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(lessonExaminationService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取作业关联的所有声部")
    @GetMapping("querySubject")
    @PreAuthorize("@pcs.hasPermissions('lessonExamination/querySubject')")
    public HttpResponseResult<List<Subject>> querySubject(Long lessonExaminationId) {
        return succeed(lessonExaminationService.querySubject(lessonExaminationId));
    }
}

