package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.MemberRankOrganizationFeeMapper;
import com.ym.mec.biz.dal.page.MemberRankFeeQueryInfo;
import com.ym.mec.biz.service.MemberRankOrganizationFeeMapperService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.util.collection.MapUtil;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@RequestMapping("${app-config.url.web:}/memberRankOrganizationFeeMapper")
@Api(tags = "会员收费设置服务")
@RestController
public class MemberRankOrganizationFeeMapperController extends BaseController {

    @Autowired
    private MemberRankOrganizationFeeMapperService memberRankOrganizationFeeMapperService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "新增")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('memberRankOrganizationFeeMapper/add')")
    @AuditLogAnnotation(operateName = "新增系统功能类型")
    public Object add(@RequestBody MemberRankOrganizationFeeMapper memberRankOrganizationFeeMapper) {
        memberRankOrganizationFeeMapper.setOperatorId(sysUserService.getUserId());
        memberRankOrganizationFeeMapperService.add(memberRankOrganizationFeeMapper);
        return succeed();
    }

    @ApiOperation(value = "修改")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('memberRankOrganizationFeeMapper/update')")
    public Object update(@RequestBody MemberRankOrganizationFeeMapper memberRankOrganizationFeeMapper) {
        memberRankOrganizationFeeMapper.setOperatorId(sysUserService.getUserId());
        memberRankOrganizationFeeMapperService.updateFee(memberRankOrganizationFeeMapper);
        return succeed();
    }

    @ApiOperation(value = "查询")
    @PostMapping("/get")
    @PreAuthorize("@pcs.hasPermissions('memberRankOrganizationFeeMapper/get')")
    public Object get(Integer id) {
        return succeed(memberRankOrganizationFeeMapperService.get(id));
    }

    @ApiOperation(value = "获取所有的收费标准")
    @GetMapping("/findAll")
    @PreAuthorize("@pcs.hasPermissions('memberRankOrganizationFeeMapper/findAll')")
    public Object getMemberDefaultFee(MemberRankFeeQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        return succeed(memberRankOrganizationFeeMapperService.findAll(params));
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('memberRankOrganizationFeeMapper/queryPage')")
    public Object queryPage(MemberRankFeeQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(memberRankOrganizationFeeMapperService.queryPage(queryInfo));
    }
}
