package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dto.CooperationCalenderRefundDto;
import com.ym.mec.biz.dal.dto.CooperationCalenderRefundUpdateDto;
import com.ym.mec.biz.dal.entity.MusicGroupCalenderRefundPeriod;
import com.ym.mec.biz.dal.page.CooperationRefundQueryInfo;
import com.ym.mec.biz.service.MusicGroupCalenderRefundPeriodService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;

@RequestMapping("${app-config.url.web:}/musicGroupCalenderRefundPeriod")
@Api(tags = "学校缴费项目缴费周期")
@RestController
public class MusicGroupCalenderRefundPeriodController extends BaseController {

    @Autowired
    private MusicGroupCalenderRefundPeriodService musicGroupCalenderRefundPeriodService;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "获取合作单位费用汇总列表")
    @PostMapping("/queryCoopRefund")
    @PreAuthorize("@pcs.hasPermissions('musicGroupCalenderRefundPeriod/queryCoopRefund')")
    @AuditLogAnnotation(operateName = "获取合作单位费用汇总列表")
    public HttpResponseResult<HashMap<String, Object>> queryCoopRefund(@RequestBody CooperationRefundQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        HashMap<String, Object> resultMap = new HashMap<>(2);
        resultMap.put("pageInfo",musicGroupCalenderRefundPeriodService.queryCoopRefund(queryInfo));
        resultMap.put("amount",musicGroupCalenderRefundPeriodService.sumCoopRefund(queryInfo));
        return succeed(resultMap);
    }

    @ApiOperation(value = "获取合作单位学校缴费列表")
    @PostMapping("/queryCoopCalender")
    @PreAuthorize("@pcs.hasPermissions('musicGroupCalenderRefundPeriod/queryCoopCalender')")
    @AuditLogAnnotation(operateName = "获取合作单位学校缴费列表")
    public HttpResponseResult<PageInfo<CooperationCalenderRefundDto>> queryCoopCalender(@RequestBody CooperationRefundQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(musicGroupCalenderRefundPeriodService.queryCoopCalender(queryInfo));
    }

    @ApiOperation(value = "批量修改")
    @PostMapping("/batchUpdate")
    @PreAuthorize("@pcs.hasPermissions('musicGroupCalenderRefundPeriod/batchUpdate')")
    @AuditLogAnnotation(operateName = "批量修改")
    public HttpResponseResult batchUpdate(@RequestBody List<CooperationCalenderRefundUpdateDto> refundPeriods) {
        musicGroupCalenderRefundPeriodService.batchUpdate(refundPeriods);
        return succeed();
    }
}
