package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.MusicGroupOrganizationCourseSettings;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.page.CourseSettingQueryInfo;
import com.ym.mec.biz.service.MusicGroupOrganizationCourseSettingsService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/musicGroupOrganizationCourseSettings")
@Api(tags = "收费方式设置")
@RestController
public class MusicGroupOrganizationCourseSettingsController extends BaseController {
    @Autowired
    private MusicGroupOrganizationCourseSettingsService musicGroupOrganizationCourseSettingsService;
    
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "获取收费方式设置列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('musicGroupOrganizationCourseSettings/queryPage')")
    public HttpResponseResult<PageInfo<MusicGroupOrganizationCourseSettings>> queryPage(CourseSettingQueryInfo queryInfo) {
    	
    	PageInfo<MusicGroupOrganizationCourseSettings> pageInfo = musicGroupOrganizationCourseSettingsService.queryPage(queryInfo);
    	
    	List<Organization> organizationList = organizationService.findAllOrgans(queryInfo.getTenantId());
    	if(organizationList != null){
    		Map<Integer, String> organMap = organizationList.stream().collect(Collectors.toMap(Organization :: getId, Organization :: getName));
    		
    		for(MusicGroupOrganizationCourseSettings cs : pageInfo.getRows()){
    			cs.setOrganName(organMap.get(cs.getOrganId()));
    		}
    	}
    	
        return succeed(pageInfo);
    }

    @ApiOperation(value = "收费方式添加")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('musicGroupOrganizationCourseSettings/add')")
    public HttpResponseResult<MusicGroupOrganizationCourseSettings> add(@RequestBody MusicGroupOrganizationCourseSettings musicGroupOrganizationCourseSettings) {
        return succeed(musicGroupOrganizationCourseSettingsService.addSetting(musicGroupOrganizationCourseSettings));
    }

    @ApiOperation(value = "删除收费方式设置")
    @PostMapping("/delSetting")
    @PreAuthorize("@pcs.hasPermissions('musicGroupOrganizationCourseSettings/delSetting')")
    public HttpResponseResult<MusicGroupOrganizationCourseSettings> delSetting(Integer id) {
        return succeed(musicGroupOrganizationCourseSettingsService.delSetting(id));
    }

    @ApiOperation(value = "修改收费方式")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('musicGroupOrganizationCourseSettings/update')")
    @AuditLogAnnotation(operateName = "修改收费方式")
    public HttpResponseResult<MusicGroupOrganizationCourseSettings> update(@RequestBody MusicGroupOrganizationCourseSettings musicGroupOrganizationCourseSettings) {
        return succeed(musicGroupOrganizationCourseSettingsService.updateSetting(musicGroupOrganizationCourseSettings));
    }

}
