package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dao.ClassGroupStudentMapperDao;
import com.ym.mec.biz.dal.dto.HasFreeCourseTimesDto;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentBaseCalender;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentCalenderDto;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.dal.enums.ClassGroupStudentStatusEnum;
import com.ym.mec.biz.dal.page.BaseOrganQueryInfo;
import com.ym.mec.biz.dal.page.MusicGroupPaymentCalenderQueryInfo;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderCourseSettingsService;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.ui.ModelMap;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/musicGroupPaymentCalender")
@Api(tags = "乐团缴费日历服务")
@RestController
public class MusicGroupPaymentCalenderController extends BaseController {

    @Autowired
    private MusicGroupPaymentCalenderService musicGroupPaymentCalenderService;
    @Autowired
    private MusicGroupPaymentCalenderCourseSettingsService musicGroupPaymentCalenderCourseSettingsService;
    @Autowired
    private OrganizationService organizationService;
    @Autowired
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;

    @ApiOperation(value = "分页查询乐团缴费日历列表")
    @GetMapping(value = "/queryPage")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/queryPage')")
    public Object queryPage(MusicGroupPaymentCalenderQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(musicGroupPaymentCalenderService.queryCalenderPage(queryInfo));
    }


    @ApiOperation(value = "查询审核人")
    @GetMapping(value = "/queryUser")
    public Object queryUser(MusicGroupPaymentCalenderQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(musicGroupPaymentCalenderService.queryUser(queryInfo));
    }

    @ApiOperation(value = "获取缴费信息")
    @GetMapping("/getDetail")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/getDetail')")
    public Object getDetail(Long id) {
        return succeed(musicGroupPaymentCalenderService.getDetail(id));
    }

    @ApiOperation(value = "新增乐团缴费日历")
    @PostMapping(value = "/add", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/add')")
    public HttpResponseResult add(@RequestBody MusicGroupPaymentCalenderDto musicGroupPaymentCalenderDto) {
    	String batchNo = musicGroupPaymentCalenderService.create(musicGroupPaymentCalenderDto);
        ModelMap map = new ModelMap(1);
        map.put("musicGroupPaymentCalenderBatchNo", batchNo);
        return succeed(map);
    }

    @ApiOperation(value = "新增乐团缴费日历")
    @PostMapping(value = "/createCalender", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/createCalender')")
    public HttpResponseResult<ModelMap> createCalender(@RequestBody MusicGroupPaymentBaseCalender musicGroupPaymentBaseCalender) {
    	return musicGroupPaymentCalenderService.create1(musicGroupPaymentBaseCalender);
    }

    @ApiOperation(value = "修改乐团缴费日历")
    @PostMapping(value = "/updateCalender", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/updateCalender')")
    public HttpResponseResult<ModelMap> updateCalender(@RequestBody MusicGroupPaymentBaseCalender musicGroupPaymentBaseCalender) {
    	return musicGroupPaymentCalenderService.merge1(musicGroupPaymentBaseCalender);
    }

    @ApiOperation(value = "待审核的缴费项目撤回")
    @PostMapping(value = "/revoke")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/revoke')")
    public Object revoke(String batchNo) {
    	musicGroupPaymentCalenderService.revoke(batchNo);
        return succeed();
    }

    @ApiOperation(value = "删除乐团缴费日历")
    @PostMapping("/del")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/del')")
    public Object del(Long id) {
        musicGroupPaymentCalenderService.del(id);
        return succeed();
    }

    @ApiOperation(value = "删除乐团缴费日历")
    @PostMapping("/delByBatchNo")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/delByBatchNo')")
    public Object delByBatchNo(String batchNo) {
        musicGroupPaymentCalenderService.deleteByBatchNo(batchNo);
        return succeed();
    }

    @ApiOperation(value = "修改乐团缴费时间")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/update')")
    public Object update(@RequestBody MusicGroupPaymentCalenderDto musicGroupPaymentCalenderDto) {
    	String batchNo = musicGroupPaymentCalenderService.merge(musicGroupPaymentCalenderDto);
        ModelMap map = new ModelMap();
        map.put("musicGroupPaymentCalenderBatchNo", batchNo);
        return succeed();
    }

    @ApiOperation(value = "确认学校已缴费")
    @PostMapping("/makesureSchoolePaid")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/makesureSchoolePaid')")
	public Object makesureSchoolePaid(Long id, String memo) {
		musicGroupPaymentCalenderService.makesureSchoolePaid(id, memo);
		return succeed();
	}

    @ApiOperation(value = "乐团缴费日历审核列表")
    @GetMapping("/auditList")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/auditList')")
    public Object auditList(MusicGroupPaymentCalenderQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(musicGroupPaymentCalenderService.auditList(queryInfo));
    }

    @ApiOperation(value = "乐团缴费日历审核列表详情")
    @GetMapping("/auditListDetail")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/auditListDetail')")
    public Object auditListDetail(String batchNo) {
        return succeed(musicGroupPaymentCalenderService.auditListDetail(batchNo));
    }

    @ApiOperation(value = "乐团缴费日历审核通过")
    @PostMapping("/auditPass")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/auditPass')")
    public Object auditPass(String batchNo,String auditMemo) {
        musicGroupPaymentCalenderService.auditPass(batchNo,auditMemo);
        return succeed();
    }

    @ApiOperation(value = "乐团缴费日历审核拒绝")
    @PostMapping("/auditRefuse")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/auditRefuse')")
    public Object auditRefuse(String batchNo,String auditMemo) {
        musicGroupPaymentCalenderService.auditRefuse(batchNo,auditMemo);
        return succeed();
    }

    @ApiOperation(value = "获取指定学员在指定乐团下本次课排课时长")
    @GetMapping("/getMusicCourseSettingsWithStudents")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/getMusicCourseSettingsWithStudents')")
    public HttpResponseResult<Map<String, Integer>> getMusicCourseSettingsWithStudents(String musicGroupId, String studentIds, Long classGroupId){
        List<Integer> studentIdList = new ArrayList<>();
        if(StringUtils.isNotBlank(studentIds)){
            studentIdList = Arrays.stream(studentIds.split(",")).map(id -> Integer.valueOf(id)).collect(Collectors.toList());
        }
        if(Objects.nonNull(classGroupId)){
            List<StudentRegistration> studentList = classGroupStudentMapperDao.findClassStudentList(classGroupId.intValue(), ClassGroupStudentStatusEnum.NORMAL);
            studentIdList = studentList.stream().map(StudentRegistration::getUserId).collect(Collectors.toList());
        }
        if(CollectionUtils.isEmpty(studentIdList)){
            throw new BizException("未指定学员");
        }
        return succeed(musicGroupPaymentCalenderCourseSettingsService.getMusicCourseSettingsWithStudents(musicGroupId,studentIdList));
    }

    @ApiOperation(value = "学员已排课时长未消耗完")
    @GetMapping("/queryHasFreeCourseTimesStudent")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/queryHasFreeCourseTimesStudent')")
    public HttpResponseResult<PageInfo<HasFreeCourseTimesDto>> queryHasFreeCourseTimesStudent(BaseOrganQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(musicGroupPaymentCalenderService.queryHasFreeCourseTimesStudent(queryInfo));
    }
}
