package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.MusicGroupShippingAddress;
import com.ym.mec.biz.dal.page.MusicGroupShippingAddressQueryInfo;
import com.ym.mec.biz.service.MusicGroupShippingAddressService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

@Api(tags = "乐团收货地址")
@RestController
@RequestMapping("${app-config.url.web:}/musicGroupShippingAddress")
public class MusicGroupShippingAddressController extends BaseController {

	@Autowired
	private MusicGroupShippingAddressService musicGroupShippingAddressService;

	@ApiOperation(value = "查询所有收货地址列表")
	@GetMapping(value = "/queryPage", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@PreAuthorize("@pcs.hasPermissions('musicGroupShippingAddress/queryPage')")
	public HttpResponseResult<PageInfo<MusicGroupShippingAddress>> findAll(MusicGroupShippingAddressQueryInfo queryInfo) {

		return succeed(musicGroupShippingAddressService.queryPage(queryInfo));
	}

	@ApiOperation(value = "查询所有收货地址列表")
	@GetMapping(value = "/query", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@PreAuthorize("@pcs.hasPermissions('musicGroupShippingAddress/query')")
	public HttpResponseResult<MusicGroupShippingAddress> query(Integer id) {
		return succeed(musicGroupShippingAddressService.get(id));
	}

	@ApiOperation("新增")
	@PostMapping(value = "/add", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@AuditLogAnnotation(operateName = "乐团收货地址新增", interfaceURL = "musicGroupShippingAddress/add")
	@PreAuthorize("@pcs.hasPermissions('musicGroupShippingAddress/add')")
	public Object add(@RequestBody MusicGroupShippingAddress shippingAddress) {
		Date date = new Date();
		shippingAddress.setUpdateTime(date);
		shippingAddress.setCreateTime(date);
		shippingAddress.setDelFlag(0);
		return succeed(musicGroupShippingAddressService.insert(shippingAddress));
	}

	@ApiOperation("修改")
	@PostMapping(value = "/update", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@AuditLogAnnotation(operateName = "乐团收货地址修改", interfaceURL = "musicGroupShippingAddress/update")
	@PreAuthorize("@pcs.hasPermissions('musicGroupShippingAddress/update')")
	public Object update(@RequestBody MusicGroupShippingAddress shippingAddress) {
		Date date = new Date();
		shippingAddress.setUpdateTime(date);
		MusicGroupShippingAddress originNewsInfo = musicGroupShippingAddressService.get(shippingAddress.getId());
		if (originNewsInfo == null) {
			return failed("请求参数错误");
		}

		return succeed(musicGroupShippingAddressService.update(shippingAddress));
	}

	@ApiOperation("删除")
	@PostMapping(value = "/delete", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@AuditLogAnnotation(operateName = "删除收货地址", interfaceURL = "musicGroupShippingAddress/delete")
	@PreAuthorize("@pcs.hasPermissions('musicGroupShippingAddress/delete')")
	public Object delete(Integer id) {

		return succeed(musicGroupShippingAddressService.delete(id));
	}
}
