package com.ym.mec.web.controller;

import com.alibaba.fastjson.JSON;
import com.dayaedu.cbs.common.enums.message.EMessageSendMode;
import com.dayaedu.cbs.common.enums.message.ESendStatus;
import com.dayaedu.cbs.openfeign.wrapper.message.CbsMessageWrapper;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.ym.mec.biz.dal.dto.SysMessageDto;
import com.ym.mec.biz.dal.entity.SysMessage;
import com.ym.mec.biz.dal.enums.MessageSendMode;
import com.ym.mec.biz.dal.enums.SendStatusEnum;
import com.ym.mec.biz.dal.page.SysMessageQueryInfo;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.vo.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RestController
@Api(tags = "消息服务")
@RequestMapping("${app-config.url.web:}/schoolMessage")
public class SchoolMessageController extends BaseController {

    @Resource
    private SysMessageService sysMessageService;
    @Resource
    private SysUserService sysUserService;

    @ApiOperation("获取所有消息列表")
    @PostMapping(value = "list")
    public HttpResponseResult<com.ym.mec.common.page.PageInfo<SysMessage>> list(@RequestBody SysMessageQueryInfo queryInfo){
        queryInfo.setUserId(sysUserService.getUserId());
        queryInfo.setType(3);
        queryInfo.setJpushType("SCHOOL");
        PageInfo<CbsMessageWrapper.SysMessage> pageInfo = sysMessageService.queryListPage(queryInfo).feignData();

        List<SysMessage> sysMessages = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(pageInfo.getRows())){
            pageInfo.getRows().forEach(sysMessage -> {
                SysMessage message = new SysMessage();
                message.setId(sysMessage.getId().toString());
                message.setTitle(sysMessage.getTitle());
                message.setContent(sysMessage.getContent());
                if(sysMessage.getSendMode() != null){
                    message.setType(sysMessage.getSendMode() == EMessageSendMode.SMS?MessageSendMode.SMS:
                            sysMessage.getSendMode() == EMessageSendMode.EMAIL?MessageSendMode.EMAIL:
                                    sysMessage.getSendMode() == EMessageSendMode.PUSH?MessageSendMode.PUSH:MessageSendMode.SEO);
                }
                if(sysMessage.getStatus() != null){
                    message.setStatus(sysMessage.getStatus() == ESendStatus.FAILED? SendStatusEnum.FAILED:
                            sysMessage.getStatus() == ESendStatus.SENDING?SendStatusEnum.SENDING:
                                    sysMessage.getStatus() == ESendStatus.SUCCESSED?SendStatusEnum.SUCCESSED:SendStatusEnum.WAIT);
                }
                message.setReceiver(sysMessage.getReceiver());
                message.setSendTime(sysMessage.getSendTime());
                message.setErrorMsg(sysMessage.getErrorMsg());
                message.setCreateOn(sysMessage.getCreateTime());
                message.setModifyOn(sysMessage.getUpdateTime());
                if(sysMessage.getUserId() != null){
                    message.setUserId(sysMessage.getUserId().intValue());
                }
                if(sysMessage.getMessageConfigId() != null){
                    message.setMessageConfigId(sysMessage.getMessageConfigId().intValue());
                }
                message.setMemo(sysMessage.getMemo());
                if (sysMessage.getReadStatus() != null){
                    message.setReadStatus(sysMessage.getReadStatus()?1:0);
                }
                message.setGroup(sysMessage.getGroup());
                sysMessages.add(message);
            });
        }
        return succeed(PageUtil.pageInfo(pageInfo,sysMessages));
    }

    @ApiOperation("获取消息分类类型")
    @GetMapping(value = "typeList")
    public Object typeList(String memo){

        return succeed(JSON.parseArray("[{\"value\":\"全部消息\",\"key\":\"ALL\"}," +
                "{\"value\":\"老师信息\",\"key\":\"TEACHER\"}," +
                "{\"value\":\"学员信息\",\"key\":\"STUDENT\"}," +
                "{\"value\":\"系统消息\",\"key\":\"SYSTEM\"},"));
    }

    @ApiOperation("一键已读")
    @PostMapping("batchSetRead")
    public Object batchSetRead() {
        return sysMessageService.updateStatus(sysUserService.getUserId(), 1,"SCHOOL")? succeed() : failed();
    }
    @ApiOperation("设置已读")
    @PostMapping("setRead")
    public Object setRead(@ApiParam("id") Long id) {
        return sysMessageService.updateOneStatus(id)? succeed() : failed();
    }

    @ApiOperation(value = "查询用户未读消息条数")
    @GetMapping("/queryCountOfUnread")
    public Object queryCountOfUnread() {
        return succeed(sysMessageService.queryCountOfUnread(MessageSendMode.PUSH, sysUserService.getUserId(),"SCHOOL", null));
    }
}
