package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dto.StudentCourseConsumerDetailDto;
import com.ym.mec.biz.dal.dto.StudentCourseConsumerDto;
import com.ym.mec.biz.dal.dto.StudentCourseConsumerSumDto;
import com.ym.mec.biz.dal.page.StudentCourseConsumerQueryInfo;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.StudentCourseConsumerService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Api(tags = "学员课耗统计")
@RequestMapping("${app-config.url.web:}/studentCourseConsumer")
@RestController
public class StudentCourseConsumerController extends BaseController {

    @Autowired
    private StudentCourseConsumerService studentCourseConsumerService;
	@Autowired
	private OrganizationService organizationService;

    @ApiOperation(value = "实际课耗列表")
    @PostMapping("/queryCourseConsumer")
    @PreAuthorize("@pcs.hasPermissions('studentCourseConsumer/queryCourseConsumer')")
    public HttpResponseResult<List<StudentCourseConsumerDto>> queryCourseConsumer(@RequestBody StudentCourseConsumerQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentCourseConsumerService.queryCourseConsumer(queryInfo));
    }

    @ApiOperation(value = "实际课耗学员详情列表")
    @PostMapping("/queryCourseConsumerDetail")
    @PreAuthorize("@pcs.hasPermissions('studentCourseConsumer/queryCourseConsumerDetail')")
    public HttpResponseResult<PageInfo<StudentCourseConsumerDetailDto>> queryCourseConsumerDetail(@RequestBody StudentCourseConsumerQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentCourseConsumerService.queryCourseConsumerDetail(queryInfo));
    }

    @ApiOperation(value = "预计课耗列表")
    @PostMapping("/queryPreCourseConsumer")
    @PreAuthorize("@pcs.hasPermissions('studentCourseConsumer/queryPreCourseConsumer')")
    public HttpResponseResult<List<StudentCourseConsumerDto>> queryPreCourseConsumer(@RequestBody StudentCourseConsumerQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentCourseConsumerService.queryPreCourseConsumer(queryInfo));
    }

    @ApiOperation(value = "预计课耗学员详情列表")
    @PostMapping("/queryPreCourseConsumerDetail")
    @PreAuthorize("@pcs.hasPermissions('studentCourseConsumer/queryPreCourseConsumerDetail')")
    public HttpResponseResult<PageInfo<StudentCourseConsumerDetailDto>> queryPreCourseConsumerDetail(@RequestBody StudentCourseConsumerQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentCourseConsumerService.queryPreCourseConsumerDetail(queryInfo));
    }


    @ApiOperation(value = "课耗统计列表")
    @PostMapping("/queryCourseConsumerSum")
    @PreAuthorize("@pcs.hasPermissions('studentCourseConsumer/queryCourseConsumerSum')")
    public HttpResponseResult<List<StudentCourseConsumerSumDto>> queryCourseConsumerSum(@RequestBody StudentCourseConsumerQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentCourseConsumerService.queryCourseConsumerSum(queryInfo));
    }
}
