package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.dto.OutOrderInfoDto;
import com.ym.mec.biz.dal.dto.StudentPaymentRouteOrderDto;
import com.ym.mec.biz.dal.entity.StudentPaymentRouteOrder;
import com.ym.mec.biz.dal.enums.AuditStatusEnum;
import com.ym.mec.biz.dal.page.StudentPaymentOrderQueryInfo;
import com.ym.mec.biz.dal.page.StudentPaymentRouteOrderQueryInfo;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.StudentPaymentRouteOrderService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/routeOrder")
@Api(tags = "分润订单服务")
@RestController
public class StudentPaymentRouteOrderController extends BaseController {

    @Autowired
    private StudentPaymentRouteOrderService studentPaymentRouteOrderService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private OrganizationService organizationService;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;

    @ApiOperation(value = "财务订单列表")
    @GetMapping("/finance")
    @PreAuthorize("@pcs.hasPermissions('routeOrder/finance')")
    public HttpResponseResult<PageInfo<StudentPaymentRouteOrderDto>> queryOrderPage(StudentPaymentOrderQueryInfo queryInfo) {
    	if(StringUtils.isBlank(queryInfo.getRoutingOrganId())){
    		queryInfo.setRoutingOrganId(organizationService.getEmployeeOrgan(queryInfo.getRoutingOrganId()));
    	}
        if (StringUtils.isNotBlank(queryInfo.getSearch())) {
            List<BasicUserDto> users = studentPaymentOrderDao.getUsers(queryInfo.getSearch());
            List<Integer> userIds = users.stream().map(BasicUserDto::getUserId).collect(Collectors.toList());
            if (userIds.size() <= 0) {
                userIds.add(0);
            }
            queryInfo.setUserIds(userIds);
        }
        PageInfo<StudentPaymentRouteOrderDto> studentPaymentRouteOrderPageInfo = studentPaymentRouteOrderService.OrderQueryPage(queryInfo);
        return succeed(studentPaymentRouteOrderPageInfo);
    }

    @ApiOperation(value = "财务订单列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('routeOrder/queryPage')")
    public HttpResponseResult<PageInfo<StudentPaymentRouteOrder>> queryPage(StudentPaymentRouteOrderQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentPaymentRouteOrderService.queryAuditList(queryInfo));
    }

    @ApiOperation(value = "财务订单列表")
    @GetMapping("/query")
    @PreAuthorize("@pcs.hasPermissions('routeOrder/query')")
    public HttpResponseResult<StudentPaymentRouteOrder> query(Long id) {
    	StudentPaymentRouteOrder studentPaymentRouteOrder = studentPaymentRouteOrderService.get(id);
        return succeed(studentPaymentRouteOrder);
    }


    @ApiOperation(value = "添加订单")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('routeOrder/add')")
    public HttpResponseResult<StudentPaymentRouteOrder> add(@RequestBody StudentPaymentRouteOrder studentPaymentRouteOrder) {
        Integer userId = sysUserService.getUserId();
        if (studentPaymentRouteOrder.getServiceAmount() == null) {
        	studentPaymentRouteOrder.setServiceAmount(BigDecimal.ZERO);
        }
        if (studentPaymentRouteOrder.getSaleAmount() == null) {
        	studentPaymentRouteOrder.setSaleAmount(BigDecimal.ZERO);
        }
        studentPaymentRouteOrder.setCreateBy(userId);
        studentPaymentRouteOrder.setUpdateBy(userId);
        if(studentPaymentRouteOrder.getPayTime().after(new Date())){
            failed("交易日期不能大于当前时间");
        }

        return succeed(studentPaymentRouteOrderService.addOutOrder(studentPaymentRouteOrder));
    }


	@ApiOperation(value = "订单审核")
	@PostMapping("/auditOutOrder")
	@PreAuthorize("@pcs.hasPermissions('routeOrder/auditOutOrder')")
	public HttpResponseResult<String> auditOutOrder(Long studentPaymentRouteOrderId, AuditStatusEnum auditStatus, String memo) {
		studentPaymentRouteOrderService.auditOutOrder(studentPaymentRouteOrderId, auditStatus, memo,sysUserService.getUserId());
		return succeed();
	}


    @ApiOperation(value = "修改订单")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('routeOrder/update')")
    public HttpResponseResult<StudentPaymentRouteOrder> update(@RequestBody StudentPaymentRouteOrder studentPaymentRouteOrder) {
    	StudentPaymentRouteOrder originOrder = studentPaymentRouteOrderService.get(studentPaymentRouteOrder.getId());
    	if(originOrder == null){
    		return failed("参数不正确");
    	}
    	if(originOrder.getAuditStatus() != AuditStatusEnum.REJECT ){
    		return failed("只有“拒绝”状态才能修改订单");
    	}
        if (studentPaymentRouteOrder.getServiceAmount() == null) {
        	studentPaymentRouteOrder.setServiceAmount(BigDecimal.ZERO);
        }
        if (studentPaymentRouteOrder.getSaleAmount() == null) {
        	studentPaymentRouteOrder.setSaleAmount(BigDecimal.ZERO);
        }
        studentPaymentRouteOrder.setUpdateBy(sysUserService.getUserId());
        if(studentPaymentRouteOrder.getSchoolId() == null){
            studentPaymentRouteOrder.setSchoolId(999999999);
        }

        return succeed(studentPaymentRouteOrderService.updateOutOrder(studentPaymentRouteOrder));
    }

    @ApiOperation(value = "获取订单信息")
    @GetMapping("/getOrderInfo")
    @PreAuthorize("@pcs.hasPermissions('routeOrder/getOrderInfo')")
    public HttpResponseResult<OutOrderInfoDto> getOrderInfo(Long orderId) {
        return succeed(studentPaymentRouteOrderService.getRouteOrder(orderId));
    }

    @ApiOperation(value = "删除订单")
    @PostMapping("/delete")
    @PreAuthorize("@pcs.hasPermissions('routeOrder/delete')")
    public HttpResponseResult<Boolean> delete(Long orderId) {
        return succeed(studentPaymentRouteOrderService.deleteRouteOrder(orderId));
    }
}
