package com.ym.mec.web.controller;

import com.alibaba.fastjson.JSON;
import com.dayaedu.cbs.openfeign.client.MusicFeignClientService;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsSubjectWrapper;
import com.dayaedu.cbs.openfeign.wrapper.musicInstrument.CbsMusicalInstrumentWrapper;
import com.microsvc.toolkit.common.webportal.exception.BizException;
import com.ym.mec.biz.dal.dto.SubFeeSettingDto;
import com.ym.mec.biz.dal.dto.SubjectApplyDetailDto;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.entity.SubjectGoodsMapper;
import com.ym.mec.biz.dal.page.SubjectQueryInfo;
import com.ym.mec.biz.dal.wrapper.InstrumentWrapper;
import com.ym.mec.biz.dal.wrapper.SubjectWrapper;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/subject")
@Api(tags = "科目服务")
@RestController
@Slf4j
public class SubjectController extends BaseController {

    @Resource
    private SubjectService subjectService;
    @Resource
    private MusicFeignClientService musicFeignClientService;

    @ApiOperation(value = "修改、新增科目")
    @PostMapping("/upset")
    @PreAuthorize("@pcs.hasPermissions('subject/upset')")
    public HttpResponseResult<Object> update(@RequestBody Subject subject){
        subjectService.upSetSubject(subject);
        return succeed();
    }

    @ApiOperation(value = "根据科目编号查询科目")
    @GetMapping("/get/{id}")
    @PreAuthorize("@pcs.hasPermissions('subject/get')")
    public HttpResponseResult<Subject> get(@ApiParam(value = "科目编号", required = true) @PathVariable("id") Integer id){
        return succeed(subjectService.get(id));
    }

    @ApiOperation(value = "分页查询科目列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('subject/queryPage')")
    public HttpResponseResult<PageInfo<Subject>> queryPage(SubjectQueryInfo queryInfo){
        return succeed(subjectService.queryPage(queryInfo));
    }

    @ApiOperation(value = "分页查询科目树状列表")
    @GetMapping("/queryPageTree")
    @PreAuthorize("@pcs.hasPermissions('subject/queryPageTree')")
    public HttpResponseResult<PageInfo<Subject>> queryPageTree(SubjectQueryInfo queryInfo){
        return succeed(subjectService.queryPageTree(queryInfo));
    }

    @ApiOperation(value = "通过乐团编号查询乐团科目规划")
    @GetMapping("/querySubByMusicGroupId")
    @PreAuthorize("@pcs.hasPermissions('subject/querySubByMusicGroupId')")
    @ApiImplicitParams({ @ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String")})
    public HttpResponseResult<List<Subject>> findSubByMusicGroupId(String musicGroupId){
        return succeed(subjectService.findSubByMusicGroupId(musicGroupId));
    }

    @ApiOperation(value = "新建乐团时，获取设置声部费用默认数据")
    @GetMapping("/setSubjectInfo")
    @PreAuthorize("@pcs.hasPermissions('subject/setSubjectInfo')")
    @ApiImplicitParams({ @ApiImplicitParam(name = "chargeTypeId", value = "收费类型编号", required = true, dataType = "Integer")})
    public HttpResponseResult<SubFeeSettingDto> setSubjectInfo(Integer chargeTypeId){
        return succeed(subjectService.setSubjectInfo(chargeTypeId));
    }

    @ApiOperation(value = "修改、新增声部关联的商品列表")
    @PostMapping("/markGoods")
    @PreAuthorize("@pcs.hasPermissions('subject/markGoods')")
    public HttpResponseResult<Object> markGoods(@RequestBody List<SubjectGoodsMapper> subjectGoodsMappers){
        subjectService.markGoods(subjectGoodsMappers);
        return succeed();
    }

    @ApiOperation(value = "通过乐团编号获取声部列表以及声部报名、缴费、计划人数")
    @GetMapping("/findSubApplyDetail")
    @PreAuthorize("@pcs.hasPermissions('subject/findSubApplyDetail')")
    @ApiImplicitParams({ @ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String")})
    public HttpResponseResult<List<SubjectApplyDetailDto>> findSubApplyDetail(String musicGroupId){
        return succeed(subjectService.findSubApplyDetail(musicGroupId));
    }

    @ApiOperation(value = "获取子集科目列表按父Id")
    @GetMapping("/findSubSubjects")
    @PreAuthorize("@pcs.hasPermissions('subject/findSubSubjects')")
    public HttpResponseResult<List<Subject>> findSubSubjects(Integer parentSubjectId,Boolean cbsFlag){
        return succeed(subjectService.findSubSubjects(parentSubjectId,cbsFlag));
    }

    @ApiOperation(value = "搜索按Id和名称")
    @PostMapping("/queryPageByIdOrName")
    @PreAuthorize("@pcs.hasPermissions('subject/queryPageByIdOrName')")
    public HttpResponseResult<PageInfo<Subject>> queryPageByIdOrName(SubjectQueryInfo queryInfo){
        return succeed(subjectService.queryPageByIdOrName(queryInfo));
    }

    @ApiOperation(value = "列出所有分部")
    @PostMapping("/list")
    // @PreAuthorize("@pcs.hasPermissions('subject/list')")
    public HttpResponseResult<List<Subject>> list(){
        return succeed(subjectService.list());
    }


    /**
     * 声部分页
     */
    @ApiOperation(value = "内容平台对应的声部数据")
    @PostMapping("/cbsSubject/page")
    public HttpResponseResult<List<SubjectWrapper.CbsSubject>> subjectPage(@RequestBody SubjectWrapper.CbsSubjectQuery query){
        // 移除已被使用的声部ID
        if (Boolean.TRUE.equals(query.getRemoveUsed())) {
            List<Subject> list = subjectService.getDao().notInSubjectIds(query.getSubjectId());
            if (CollectionUtils.isNotEmpty(list)) {
                List<Long> cbsSubjectIds = list.stream().map(Subject::getCbsSubjectId).filter(Objects::nonNull).map(Integer::longValue).distinct().collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(cbsSubjectIds)) {
                    query.setRemoveIds(cbsSubjectIds);
                }
            }
        }

        try {
            query.setPage(1);
            query.setRows(-1);
            List<CbsSubjectWrapper.Subject> subjectPageInfo = musicFeignClientService.subjectPage(query).feignData().getRows();
            if (CollectionUtils.isEmpty(subjectPageInfo)) {
                return succeed(new ArrayList<>());
            }
            List<Integer> subjectIds = subjectPageInfo.stream().map(CbsSubjectWrapper.Subject::getId).map(Long::intValue).collect(Collectors.toList());

            // 查询乐器信息
            CbsMusicalInstrumentWrapper.MusicalInstrumentQuery musicalInstrumentQuery = new CbsMusicalInstrumentWrapper.MusicalInstrumentQuery();
            musicalInstrumentQuery.setSubjectIds(subjectIds);
            musicalInstrumentQuery.setPage(1);
            musicalInstrumentQuery.setRows(9999);
            List<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> rows = musicFeignClientService
                    .musicalInstrumentPage(musicalInstrumentQuery).feignData().getRows();
            Map<Long, List<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto>> subjectIdMap = rows.stream()
                    .collect(Collectors.groupingBy(o -> o.getSubjectId().longValue(), LinkedHashMap::new, Collectors.toList()));

            // 查询本地已经添加过的曲目数据
            SubjectWrapper.SubjectQuery query1 = new SubjectWrapper.SubjectQuery();
            query1.setHasInstrument(true);
            List<SubjectWrapper.Subject> list = subjectService.queryList(query1);
            // 转map
            Map<Long, List<SubjectWrapper.Subject>> subjectMap = list.stream()
                    .collect(Collectors.groupingBy(SubjectWrapper.Subject::getCbsSubjectId));

            List<SubjectWrapper.CbsSubject> cbsSubjects = new ArrayList<>();
            for (CbsSubjectWrapper.Subject subject : subjectPageInfo) {
                SubjectWrapper.CbsSubject cbsSubject = new SubjectWrapper.CbsSubject();
                BeanUtils.copyProperties(subject, cbsSubject);

                List<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> musicalInstrumentQueryDtos = subjectIdMap.get(cbsSubject.getId());
                if (CollectionUtils.isEmpty(musicalInstrumentQueryDtos)) {
                    continue;
                }
                cbsSubject.setInstruments(JSON.parseArray(JSON.toJSONString(musicalInstrumentQueryDtos), SubjectWrapper.Instrument.class));

                cbsSubjects.add(cbsSubject);
                // 本地已经添加过的曲目数据
                List<SubjectWrapper.Subject> subjects = subjectMap.get(subject.getId());
                if (CollectionUtils.isEmpty(subjects)) {
                    continue;
                }
                List<InstrumentWrapper.Instrument> instruments = new ArrayList<>();
                for (SubjectWrapper.Subject subject1 : subjects) {
                    if (CollectionUtils.isNotEmpty(subject1.getInstruments())) {
                        instruments.addAll(subject1.getInstruments());
                    }
                }
                if (CollectionUtils.isEmpty(instruments)) {
                    continue;
                }
                // instruments 转map
                Map<Integer, InstrumentWrapper.Instrument> instrumentMap = instruments.stream().collect(Collectors.toMap(InstrumentWrapper.Instrument::getId, o -> o));
                for (SubjectWrapper.Instrument instrument : cbsSubject.getInstruments()) {
                    instrument.setAdded(instrumentMap.containsKey(instrument.getId()));
                }

            }

            return succeed(cbsSubjects);
        } catch (Exception e) {
            log.error("内容平台对应的声部数据查询失败", e);
            throw BizException.from("内容平台服务异常");
        }
    }
}
