package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.ExtensionOrder;
import com.ym.mec.biz.dal.entity.StudentPaymentOrder;
import com.ym.mec.biz.dal.page.CourseScheduleSWQueryInfo;
import com.ym.mec.biz.dal.page.ExtensionSWQueryInfo;
import com.ym.mec.biz.service.CourseScheduleSwService;
import com.ym.mec.biz.service.ExtensionOrderService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/swTempJc")
@RestController
public class SwTempJcController extends BaseController {

	@Autowired
	private CourseScheduleSwService courseScheduleSwService;
	
	@Autowired
	private ExtensionOrderService extensionOrderService;
	
	@Autowired
	private StudentPaymentOrderService studentPaymentOrderService;
	
	private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @ApiOperation(value = "课表查询")
    @GetMapping("/queryPageForCourseSchedule")
    @PreAuthorize("@pcs.hasPermissions('swTempJc/queryPageForCourseSchedule')")
    public Object queryPageForCourseSchedule(CourseScheduleSWQueryInfo queryInfo){
        return succeed(courseScheduleSwService.queryPage(queryInfo));
    }

    @ApiOperation(value = "推广订单查询")
    @GetMapping("/queryPageForExtensionOrder")
    @PreAuthorize("@pcs.hasPermissions('swTempJc/queryPageForExtensionOrder')")
    public Object queryPageForExtensionOrder(ExtensionSWQueryInfo queryInfo){
    	
    	PageInfo<ExtensionOrder> pageInfo = extensionOrderService.queryPage(queryInfo);
    	
    	List<ExtensionOrder> extensionOrderList = pageInfo.getRows();
    	
    	List<String> orderNoList = extensionOrderList.stream().map(t -> t.getOrderNo()).collect(Collectors.toList());
    	
    	if(orderNoList != null && orderNoList.size() > 0){
    		
    		Map<String,StudentPaymentOrder> map = studentPaymentOrderService.findOrderByOrderNos(orderNoList).stream().collect(Collectors.toMap(StudentPaymentOrder :: getOrderNo, Function.identity()));

    		StudentPaymentOrder spo = null;
        	for(ExtensionOrder eo : extensionOrderList){
        		spo = map.get(eo.getOrderNo());
        		if(spo == null){
        			continue;
        		}
        		eo.setOrderAmount(spo.getExpectAmount());
        		eo.setPayTime(sdf.format(spo.getPayTime()));
        	}
    	}
    	
        return succeed(pageInfo);
    }
}
