package com.ym.mec.web.controller;

import com.alibaba.fastjson.JSON;
import com.dayaedu.cbs.common.enums.message.EMessageSendMode;
import com.dayaedu.cbs.common.enums.message.ESendStatus;
import com.dayaedu.cbs.openfeign.wrapper.message.CbsMessageWrapper;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.AppVersionInfoDao;
import com.ym.mec.biz.dal.dto.SysMessageVo;
import com.ym.mec.biz.dal.entity.AppVersionInfo;
import com.ym.mec.biz.dal.entity.SysMessage;
import com.ym.mec.biz.dal.enums.MessageSendMode;
import com.ym.mec.biz.dal.enums.SendStatusEnum;
import com.ym.mec.biz.dal.page.SysMessageQueryInfo;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.vo.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@RestController
@Api(tags = "消息服务")
@RequestMapping("${app-config.url.web:}/sysMessage")
public class SysMessageController extends BaseController {

	@Resource
	private SysMessageService sysMessageService;

	@Resource
	private SysUserService sysUserService;

	@Resource
	private AppVersionInfoDao appVersionInfoDao;

	@ApiOperation("获取所有消息列表")
	@GetMapping(value = "list")
	public Object list(SysMessageQueryInfo queryInfo){
		SysUser sysUser = sysUserService.getUser();
		queryInfo.setUserId(sysUser.getId());
		queryInfo.setType(MessageSendMode.SEO.getCode());
		com.microsvc.toolkit.common.response.paging.PageInfo<CbsMessageWrapper.SysMessage> pageInfo = sysMessageService.queryListPage(queryInfo).feignData();

		List<SysMessage> sysMessages = new ArrayList<>();
		if(org.apache.commons.collections.CollectionUtils.isNotEmpty(pageInfo.getRows())){
			pageInfo.getRows().forEach(sysMessage -> {
				SysMessage message = new SysMessage();
				message.setId(sysMessage.getId().toString());
				message.setTitle(sysMessage.getTitle());
				message.setContent(sysMessage.getContent());
				if(sysMessage.getSendMode() != null){
					message.setType(sysMessage.getSendMode() == EMessageSendMode.SMS?MessageSendMode.SMS:
							sysMessage.getSendMode() == EMessageSendMode.EMAIL?MessageSendMode.EMAIL:
									sysMessage.getSendMode() == EMessageSendMode.PUSH?MessageSendMode.PUSH:MessageSendMode.SEO);
				}
				if(sysMessage.getStatus() != null){
					message.setStatus(sysMessage.getStatus() == ESendStatus.FAILED? SendStatusEnum.FAILED:
							sysMessage.getStatus() == ESendStatus.SENDING?SendStatusEnum.SENDING:
									sysMessage.getStatus() == ESendStatus.SUCCESSED?SendStatusEnum.SUCCESSED:SendStatusEnum.WAIT);
				}
				message.setReceiver(sysMessage.getReceiver());
				message.setSendTime(sysMessage.getSendTime());
				message.setErrorMsg(sysMessage.getErrorMsg());
				message.setCreateOn(sysMessage.getCreateTime());
				message.setModifyOn(sysMessage.getUpdateTime());
				if(sysMessage.getUserId() != null){
					message.setUserId(sysMessage.getUserId().intValue());
				}
				if(sysMessage.getMessageConfigId() != null){
					message.setMessageConfigId(sysMessage.getMessageConfigId().intValue());
				}
				message.setMemo(sysMessage.getMemo());
				if (sysMessage.getReadStatus() != null){
					message.setReadStatus(sysMessage.getReadStatus()?1:0);
				}
				message.setGroup(sysMessage.getGroup());
				sysMessages.add(message);
			});
		}
		return succeed(PageUtil.pageInfo(pageInfo,sysMessages));
	}

	@ApiOperation("获取消息分类类型")
	@GetMapping(value = "typeList")
	public Object typeList(String memo){
		//如果用户使用的版本比当前版本号大，那么屏蔽缴费信息
		if(StringUtils.isNotEmpty(memo)){
			//获取当前使用的版本
			AppVersionInfo appVersionInfo = appVersionInfoDao.queryNewestByPlatform("ios-education").get(0);
			int defaultVersion = Integer.parseInt(appVersionInfo.getVersion().replaceAll("\\.",""));
			int currentVersion = Integer.parseInt(memo.replaceAll("\\.",""));
			if(currentVersion > defaultVersion){
				return succeed(JSON.parseArray("[{\"value\":\"全部\",\"key\":\"ALL\"}," +
						"{\"value\":\"学员信息\",\"key\":\"STUDENT\"}," +
						"{\"value\":\"乐团信息\",\"key\":\"MUSIC\"}," +
						"{\"value\":\"审批信息\",\"key\":\"OA\"}," +
						"{\"value\":\"其他\",\"key\":\"DEFAULT\"}]"));
			}
		}
		return succeed(JSON.parseArray("[{\"value\":\"全部\",\"key\":\"ALL\"}," +
				"{\"value\":\"缴费信息\",\"key\":\"PAY\"}," +
				"{\"value\":\"学员信息\",\"key\":\"STUDENT\"}," +
				"{\"value\":\"乐团信息\",\"key\":\"MUSIC\"}," +
				"{\"value\":\"审批信息\",\"key\":\"OA\"}," +
				"{\"value\":\"其他\",\"key\":\"DEFAULT\"}]"));
	}

	@ApiOperation("设置已读")
	@PostMapping("setRead")
	public Object setRead(Long id) {
		return sysMessageService.updateOneStatus(id)? succeed() : failed();
	}

	@ApiOperation(value = "查询用户未读消息条数")
	@GetMapping("/queryCountOfUnread")
	public Object queryCountOfUnread() {
		SysUser sysUser = sysUserService.getUser();
		return succeed(sysMessageService.queryCountOfUnread(MessageSendMode.SEO, sysUser.getId(),null, sysUser.getTenantId()));
	}


	@ApiOperation(value = "查询分页", notes = "传入sysMessageSearch")
	@PreAuthorize("@pcs.hasPermissions('sysMessage/page')")
	@PostMapping("/page")
	public HttpResponseResult<PageInfo<SysMessageVo.SysMessageList>> page(@RequestBody SysMessageVo.SysMessageQuery query) {
		com.microsvc.toolkit.common.response.paging.PageInfo<CbsMessageWrapper.SysMessage> pageInfo = sysMessageService.queryMessageCode(query).feignData();
		if(CollectionUtils.isEmpty(pageInfo.getRows())){
			return succeed(new PageInfo<>());
		}
		List<SysMessageVo.SysMessageList> sysMessageList = new ArrayList<>();
		Pattern pattern = Pattern.compile("\\b验证码\\s*(\\d{6})\\b");
		pageInfo.getRows().forEach(next -> {
			SysMessageVo.SysMessageList sysMessage = new SysMessageVo.SysMessageList();
			String content = next.getContent();
			sysMessage.setContent(content);
			sysMessage.setTitle(next.getTitle());
			sysMessage.setSendTime(next.getSendTime());
			sysMessage.setReceiver(next.getReceiver());
			if (StringUtils.isNotEmpty(content)) {
				Matcher matcher = pattern.matcher(content);
				if (matcher.find()) {
					sysMessage.setVerityCode(matcher.group().replace("验证码",""));
				}
			}
			sysMessageList.add(sysMessage);
		});
		return succeed(PageUtil.pageInfo(pageInfo, sysMessageList));
	}
}
