package com.ym.mec.web.controller;

import com.dayaedu.cbs.domain.wrapper.MusicSheetWrapper;
import com.ym.mec.biz.dal.dto.ReasonDto;
import com.ym.mec.biz.dal.entity.SysMusicScore;
import com.ym.mec.biz.dal.enums.ClientTypeEnum;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.SysExamSongQueryInfo;
import com.ym.mec.biz.service.SysMusicScoreService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.concurrent.CompletableFuture;

@RequestMapping("${app-config.url.web:}/sysMusicScore")
@Api(tags = "曲库")
@RestController
public class SysMusicScoreController extends BaseController {

    @Resource
    private SysMusicScoreService sysMusicScoreService;

    @ApiOperation(value = "修改应用扩展信息")
    @PreAuthorize("@pcs.hasPermissions('sysMusicScore/update', {'BACKEND'})")
    @PostMapping("/update")
    public HttpResponseResult<Object> update(@RequestBody MusicSheetWrapper.MusicSheetApplicationExtendWrapper addMusicSheetApplicationExtend) {
        sysMusicScoreService.updateCbs(addMusicSheetApplicationExtend);
        return succeed();
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('sysMusicScore/queryPage')")
    public HttpResponseResult<PageInfo<SysMusicScore>> queryPage(SysExamSongQueryInfo queryInfo) {
        return succeed(sysMusicScoreService.queryMusicScorePage(queryInfo));
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/queryPage2")
    public Object queryPage2(SysExamSongQueryInfo queryInfo) {
        queryInfo.setType("COMMON");
        queryInfo.setClientFlag(true);
        queryInfo.setClientType(ClientTypeEnum.SMART_PRACTICE);
        return succeed(sysMusicScoreService.queryMusicScorePage(queryInfo));
    }

    @PostMapping("/state/{id}")
    @ApiOperation(value = "启用/停用", notes = "传入id")
    @PreAuthorize("@pcs.hasPermissions('sysMusicScore/state')")
    public HttpResponseResult<Boolean> state(@ApiParam(value = "曲目编号", required = true)  @PathVariable Integer id) {
        if (id == null) {
            return failed("参数不能为空");
        }
        return succeed(sysMusicScoreService.state(id));
    }

    @PostMapping("/stateList")
    @ApiOperation(value = "批量启用/停用")
    @PreAuthorize("@pcs.hasPermissions('sysMusicScore/stateList')")
    public HttpResponseResult<Boolean> stateList(@RequestBody ReasonDto reasonDto) {
        if (StringUtils.isEmpty(reasonDto.getMusicSheetIds())) {
            return failed("参数不能为空");
        }
        if (reasonDto.getState() == null) {
            return failed("启用/禁用状态不能为空");
        }
        return succeed(sysMusicScoreService.stateList(reasonDto));
    }

    @ApiOperation(value = "删除")
    @PostMapping("/del")
    @PreAuthorize("@pcs.hasPermissions('sysMusicScore/del')")
    public HttpResponseResult<Object> del(Long id) {
        if (id == null) {
            return failed("参数不能为空");
        }
        sysMusicScoreService.del(id);
        return succeed();
    }

    @ApiOperation(value = "数据同步", notes = "曲谱表- 修改曲目缓存 MusicSheetVo.MusicSheet")
    @PostMapping("/sync")
    public HttpResponseResult<Object> sync() {
        CompletableFuture.runAsync(() -> {
            sysMusicScoreService.sync();
        });
        return succeed();
    }
}
