package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.SysSuggestion;
import com.ym.mec.biz.dal.page.SysSuggestionQueryInfo;
import com.ym.mec.biz.service.SysSuggestionService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

@Api(tags = "意见反馈")
@RestController
@RequestMapping("${app-config.url.web:}/sysSuggestion")
public class SysSuggestionController extends BaseController {

    @Autowired
    private SysSuggestionService sysSuggestionService;

    @ApiOperation(value = "获取详情")
    @GetMapping("/detail")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestion/detail')")
    public HttpResponseResult<SysSuggestion> detail(Long id) {
        return succeed(sysSuggestionService.getDetail(id));
    }

    @ApiOperation(value = "处理")
    @PostMapping("/dispose")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestion/dispose')")
    public HttpResponseResult<Object> dispose(@RequestBody SysSuggestion sysSuggestion) {
        sysSuggestionService.dispose(sysSuggestion);
        return succeed();
    }

    @ApiOperation(value = "删除")
    @GetMapping("/del")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestion/del')")
    public HttpResponseResult<Object> del(Long id) {
        sysSuggestionService.delete(id);
        return succeed();
    }

    @ApiOperation(value = "分页查询")
    @PostMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestion/queryPage')")
    public HttpResponseResult<PageInfo<SysSuggestion>> queryPage(@RequestBody SysSuggestionQueryInfo queryInfo) {
        return succeed(sysSuggestionService.queryPage(queryInfo));
    }

}
