package com.ym.mec.web.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.SysSuggestionDao;
import com.ym.mec.biz.dal.entity.SysSuggestionType;
import com.ym.mec.biz.service.SysSuggestionTypeService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Api(tags = "意见反馈类型")
@RestController
@RequestMapping("${app-config.url.web:}/sysSuggestionType/")
public class SysSuggestionTypeController extends BaseController {

    @Autowired
    private SysSuggestionTypeService sysSuggestionTypeService;
    @Autowired
    private SysSuggestionDao sysSuggestionDao;
    @Autowired
    private SysUserService sysUserService;

    @ApiOperation(value = "新增")
    @RequestMapping("add")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestionType/add')")
    public Object add(@RequestBody SysSuggestionType suggestionType) {
        if(StringUtils.isEmpty(suggestionType.getType())){
            return failed("反馈类型不能为空");
        }
        SysSuggestionType one = sysSuggestionTypeService.lambdaQuery().eq(SysSuggestionType::getType, suggestionType.getType()).one();
        if(one != null){
            return failed("反馈类型已存在");
        }
        SysUser sysUser = sysUserService.getUser();
        suggestionType.setOperatorId(sysUser.getId());
        suggestionType.setOperatorName(sysUser.getRealName());
        sysSuggestionTypeService.save(suggestionType);
        return succeed();
    }

    @ApiOperation(value = "修改")
    @RequestMapping("update")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestionType/update')")
    public Object update(@RequestBody SysSuggestionType suggestionType) {
        if(StringUtils.isEmpty(suggestionType.getType())){
            return failed("反馈类型不能为空");
        }
        SysSuggestionType one = sysSuggestionTypeService.lambdaQuery()
                .eq(SysSuggestionType::getType, suggestionType.getType())
                .ne(SysSuggestionType::getId, suggestionType.getId())
                .one();
        if(one != null){
            return failed("反馈类型已存在");
        }
        SysUser sysUser = sysUserService.getUser();
        suggestionType.setOperatorId(sysUser.getId());
        suggestionType.setOperatorName(sysUser.getRealName());
        sysSuggestionTypeService.updateById(suggestionType);
        return succeed();
    }

    @ApiOperation(value = "删除")
    @GetMapping("del")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestionType/del')")
    public Object del(Long id) {
        //是否被使用
        Integer num = sysSuggestionDao.countByTypeId(id);
        if(num != null && num > 0){
            throw new BizException("反馈类型已被使用");
        }
        sysSuggestionTypeService.removeById(id);
        return succeed();
    }

    @ApiOperation(value = "分页查询")
    @PostMapping("queryPage")
    @PreAuthorize("@pcs.hasPermissions('sysSuggestionType/queryPage')")
    public HttpResponseResult<PageInfo<SysSuggestionType>> queryPage(@RequestBody QueryInfo queryInfo) {
        Page<SysSuggestionType> page = PageUtil.getPage(queryInfo.getPage(), queryInfo.getRows());
        IPage<SysSuggestionType> iPage = sysSuggestionTypeService.page(page, Wrappers.<SysSuggestionType>lambdaQuery()
                .eq(StringUtils.isNotBlank(queryInfo.getSearch()), SysSuggestionType::getType, queryInfo.getSearch())
                .orderByDesc(SysSuggestionType::getId));
        return succeed(PageUtil.pageInfo(iPage));
    }

    @ApiOperation(value = "查询")
    @PostMapping("queryAll")
    public HttpResponseResult<List<SysSuggestionType>> queryAll() {
        return succeed(sysSuggestionTypeService.lambdaQuery()
                .eq(SysSuggestionType::getTenantId, TenantContextHolder.getTenantId())
                .orderByDesc(SysSuggestionType::getId).list());
    }

}
