package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.SysUserContracts;
import com.ym.mec.biz.service.SysUserContractsService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/sysUserContracts")
@Api(tags = "用户协议服务")
@RestController
public class SysUserContractsController extends BaseController {

	@Autowired
	private SysUserContractsService sysUserContractsService;
	
    @ApiOperation(value = "查询学生最新协议")
    @GetMapping("/getLatest")
    @PreAuthorize("@pcs.hasPermissions('sysUserContracts/getLatest')")
	public HttpResponseResult<List<SysUserContracts>> getLatest(Integer userId) {
		
//		SysUserContracts sysUserContracts = sysUserContractsService.getLatestUserContract(userId);
		List<SysUserContracts> sysUserContracts = sysUserContractsService.getUserAllContracts(userId);

		if(CollectionUtils.isEmpty(sysUserContracts)){
			return failed("该学员尚未签署协议");
		}

		Map<Integer, List<SysUserContracts>> versionContractMap = sysUserContracts.stream().collect(Collectors.groupingBy(SysUserContracts::getVersion));

		List<SysUserContracts> result = new ArrayList<>();
		for (Map.Entry<Integer, List<SysUserContracts>> versionContractMapEntry : versionContractMap.entrySet()) {
			versionContractMapEntry.getValue().sort(Comparator.comparing(SysUserContracts::getCreateTime).reversed());
			result.add(versionContractMapEntry.getValue().get(0));
		}

		return succeed(result);
	}

}
