package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.StandardEnum;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/teacher")
@Api(tags = "教师服务")
@RestController
public class TeacherController extends BaseController {

    @Autowired
    private TeacherService teacherService;
    @Autowired
    private OrganizationService organizationService;
    @Autowired
    private ClassGroupService classGroupService;

    @Autowired
    private CourseHomeworkService courseHomeworkService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private StudentCourseHomeworkService studentCourseHomeworkService;

    @ApiOperation(value = "获取老师列表（基本信息）")
    @GetMapping("/queryTeacherBasicInfo")
    @PreAuthorize("@pcs.hasPermissions('teacher/queryTeacherBasicInfo')")
    public HttpResponseResult<PageInfo<SysUserDto>> queryTeacherBasicInfo(UserBasicQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(teacherService.queryTeacherBasicInfo(queryInfo));
    }

    @ApiOperation(value = "新增教师")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('teacher/add')")
    public HttpResponseResult<Integer> add(Teacher teacher) throws Exception {
        return succeed(teacherService.add(teacher));
    }

    @ApiOperation(value = "分页查询教师列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('teacher/queryPage')")
    public HttpResponseResult<PageInfo<Teacher>> queryPage(TeacherQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        if(queryInfo.getTenantId() == 28){
            queryInfo.setTenantId(1);
            queryInfo.setOrganId(null);
        }
        return succeed(teacherService.queryPageDetail(queryInfo));
    }

    @ApiOperation(value = "修改教师信息")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('teacher/update')")
    @AuditLogAnnotation(operateName = "修改教师信息")
    public HttpResponseResult<Object> update(Teacher teacher) {
        teacherService.updateTea(teacher);
        return succeed();
    }

    @ApiOperation(value = "修改教师信息(不包含离职日期等信息)")
    @PostMapping("/update1")
    @PreAuthorize("@pcs.hasPermissions('teacher/update1')")
    @AuditLogAnnotation(operateName = "修改教师信息(不包含离职日期等信息)")
    public HttpResponseResult<Object> update1(Teacher teacher) {
        teacherService.updateTea1(teacher);
        return succeed();
    }

    @ApiOperation(value = "根据教师编号查询教师基本信息")
    @GetMapping("/get")
    @ApiParam(value = "教师编号", required = true)
    @PreAuthorize("@pcs.hasPermissions('teacher/get')")
    @ApiImplicitParams({@ApiImplicitParam(name = "teacherId", value = "教师编号", required = true, dataType = "Integer")})
    public HttpResponseResult<Teacher> get(Integer teacherId) {
        return succeed(teacherService.getDetail(teacherId));
    }

    @ApiOperation(value = "获取分部所有老师")
    @GetMapping("/findTeachers")
    @PreAuthorize("@pcs.hasPermissions('teacher/findTeachers','system')")
    public HttpResponseResult<List<TeacherBasicDto>> findTeachers(String organId, Boolean demissionFlag, Boolean isForzenWithQueryCondition) {
        organId = organizationService.getEmployeeOrgan(organId);
        demissionFlag = false;
        if(isForzenWithQueryCondition == null){
            isForzenWithQueryCondition = false;
        }
        return succeed(teacherService.findTeachers(organId,demissionFlag,isForzenWithQueryCondition));
    }

    @ApiOperation(value = "根据科目获取部门下的教师")
    @GetMapping("/findOrganTeacherBySubject")
    @PreAuthorize("@pcs.hasPermissions('teacher/findOrganTeacherBySubject')")
    public HttpResponseResult<List<Teacher>> findOrganTeacherBySubject(Integer subjecId) {
        return succeed(teacherService.findOrganTeacherBySubject(subjecId));
    }

    @ApiOperation(value = "获取分部教师")
    @GetMapping("/findTeacherByOrganId")
    @PreAuthorize("@pcs.hasPermissions('teacher/findTeacherByOrganId')")
    public HttpResponseResult<List<Teacher>> findTeacherByOrganId(String organId, String subjectIds, String jobNature) {
        organId = organizationService.getEmployeeOrgan(organId);
        return succeed(teacherService.findTeacherByOrganId(organId,subjectIds,jobNature));
    }


    @ApiOperation(value = "获取机构教师")
    @GetMapping("/findTeacherByTenantId")
    @PreAuthorize("@pcs.hasPermissions('teacher/findTeacherByTenantId')")
    public HttpResponseResult<List<Teacher>> findTeacherByTenantId(String subjectIds, String jobNature) {
        List<Organization> allOrgans = organizationService.findAllOrgans(TenantContextHolder.getTenantId());
        String organIds = allOrgans.stream().map(o -> o.getId().toString()).collect(Collectors.joining(","));
        return succeed(teacherService.findTeacherByOrganId(organIds,subjectIds,jobNature));
    }

    @ApiOperation(value = "分页获取教师名称编号列表")
    @GetMapping("/queryNameList")
    @PreAuthorize("@pcs.hasPermissions('teacher/queryNameList', 'system', 'education')")
    public HttpResponseResult<PageInfo<Teacher>> queryPageName(TeacherNameQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(teacherService.queryPageName(queryInfo));
    }

    @ApiOperation(value = "获取乐团所有老师")
    @GetMapping("/findMusicGroupTeachers")
    @PreAuthorize("@pcs.hasPermissions('teacher/findMusicGroupTeachers')")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团编号", required = true, dataType = "String")})
    public HttpResponseResult<List<BasicUserDto>> findMusicGroupTeachers(String musicGroupId){
        return succeed(teacherService.queryMusicGroupTeacher(musicGroupId));
    }

    @ApiOperation(value = "分页查询乐团管理--详情--老师列表")
    @GetMapping("/queryMusicGroupTeachers")
    @PreAuthorize("@pcs.hasPermissions('teacher/queryMusicGroupTeachers')")
    public HttpResponseResult<PageInfo<MusicGroupTeachersDto>> queryMusicGroupTeachers(MusicGroupTeachersQueryInfo queryInfo){
        return succeed(teacherService.queryMusicGroupTeachers(queryInfo));
    }

    @ApiOperation(value = "分页查询乐团管理--详情--老师上课记录")
    @GetMapping("/queryTeacherAttendance")
    @PreAuthorize("@pcs.hasPermissions('teacher/queryTeacherAttendance')")
    public HttpResponseResult<PageInfo<MusicGroupTeacherAttendanceDto>> queryTeacherAttendance(MusicGroupTeacherAttendanceQueryInfo queryInfo){
        return succeed(teacherService.queryTeacherAttendance(queryInfo));
    }

    @ApiOperation(value = "获取老师乐团课程信息列表")
    @GetMapping("/getTeacherMusicClass")
    @ApiParam(value = "教师编号", required = true)
    @PreAuthorize("@pcs.hasPermissions('teacher/getTeacherMusicClass')")
    public HttpResponseResult<PageInfo<TeacherMusicClassInfoDto>> getTeacherMusicClass(TeacherMusicClassQueryInfo queryInfo){
        return succeed(classGroupService.getTeacherMusicClass(queryInfo));
    }

    @ApiOperation(value = "获取老师VIP课程信息列表")
    @GetMapping("/getTeacherVipClass")
    @PreAuthorize("@pcs.hasPermissions('teacher/getTeacherVipClass')")
    public HttpResponseResult<PageInfo<TeacherVipClassInfoDto>> getTeacherVipClass(VipClassQueryInfo queryInfo){
        return succeed(classGroupService.getTeacherVipClass(queryInfo));
    }

    @ApiOperation(value = "获取教师上课结算列表")
    @GetMapping("/queryCloses")
    @PreAuthorize("@pcs.hasPermissions('teacher/queryCloses')")
    public HttpResponseResult<PageInfo<TeacherCloseDto>> queryCloses(TeacherCloseQueryInfo queryInfo){
        return succeed(teacherService.queryCloses(queryInfo));
    }

    @ApiOperation(value = "获取教师上课课酬未结算总额")
    @GetMapping("/sumTeacherNoPay")
    @PreAuthorize("@pcs.hasPermissions('teacher/sumTeacherNoPay')")
    public HttpResponseResult<BigDecimal> findTeacherNoPay(Integer teacherId){
        return succeed(teacherService.findTeacherNoPay(teacherId));
    }


    @ApiOperation(value = "获取教师布置的作业")
    @GetMapping("/getCourseScheduleHomework")
    @PreAuthorize("@pcs.hasPermissions('teacher/getCourseScheduleHomework')")
    public HttpResponseResult<CourseHomework> getCourseScheduleHomework(Integer courseScheduleId){
        return succeed(teacherService.getCourseScheduleHomework(courseScheduleId));
    }

    @ApiOperation(value = "获取老师指导的学员列表")
    @GetMapping("/queryStudent")
    @PreAuthorize("@pcs.hasPermissions('teacher/queryStudent')")
    public HttpResponseResult<PageInfo<BasicUserDto>> queryStudent(QueryInfo queryInfo){
        return succeed(teacherService.queryStudent(queryInfo));
    }




    @ApiOperation(value = "获取作业详情")
    @GetMapping(value = "/findCourseHomeworkDetail")
    public HttpResponseResult<CourseHomeworkWrapper.CourseHomeworkList> findCourseHomeworkDetail(Integer courseScheduleId){

        return succeed(courseHomeworkService.findCourseHomeworkDetail(courseScheduleId));
    }




    @ApiOperation(value = "根据课程计划获取需要交作业的学生-公用")
    @PostMapping("/findCourseStudentsPublic/v2")
    public HttpResponseResult<List<StudentCourseHomework>> findCourseStudentsPublicV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        List<StudentCourseHomework>  result = new ArrayList<>();
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            result = studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(query);
        }else{
            result = studentCourseHomeworkService.findExtraExerciseStudentsV2(query);
        }
        // 完成才返回提交时间
        for (StudentCourseHomework studentCourseHomework : result) {
            if (!StandardEnum.STANDARD.equals(studentCourseHomework.getStandardFlag())) {
                studentCourseHomework.setSubmitTime(null);
            }
        }
        return succeed(result);
    }



    @ApiOperation(value = "根据课程计划获取需要交作业的学生声部-公用")
    @PostMapping("/findCourseStudentsSubjectPublic/v2")
    public HttpResponseResult<List<Subject>> findCourseStudentsSubjectPublicV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        List<StudentCourseHomework> studentCourseHomeworkByCourseV2;
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            studentCourseHomeworkByCourseV2 = studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(
                query);
        }else{
            studentCourseHomeworkByCourseV2 = studentCourseHomeworkService.findExtraExerciseStudentsV2(query);
        }

        if (CollectionUtils.isEmpty(studentCourseHomeworkByCourseV2)) {
            return succeed(new ArrayList<>());
        }
        // 获取声部
        List<Integer> collect = studentCourseHomeworkByCourseV2.stream().map(StudentCourseHomework::getSubjectId).collect(
            Collectors.toList());

        return succeed(subjectService.findBySubjectByIdList(collect));

    }
}
