package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.entity.TeacherDefaultVipGroupSalary;
import com.ym.mec.biz.service.TeacherDefaultVipGroupSalaryService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author Joburgess
 * @Date 2019/10/9
 */
@Api(tags = "老师默认VIP课薪酬")
@RequestMapping("${app-config.url.web:}/teacherDefaultVipGroupSalary")
@RestController
public class TeacherDefaultVipGroupSalaryController extends BaseController {

    @Autowired
    private TeacherDefaultVipGroupSalaryService teacherDefaultVipGroupSalaryService;

    @ApiOperation(value = "根据老师编号及课程类型编号获取默认课酬")
    @GetMapping("/findByTeacherAndCategory")
    @PreAuthorize("@pcs.hasPermissions('teacherDefaultVipGroupSalary/findByTeacherAndCategory')")
    public Object findByTeacherAndCategory(Integer userId, Integer categoryId){
        return succeed(teacherDefaultVipGroupSalaryService.findByTeacherAndCategory(userId,categoryId));
    }

    @ApiOperation(value = "获取教师的vip课酬列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "search", value = "关键字匹配", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "groupType", value = "课程类型", dataType = "String", paramType = "query"),
    })
    @GetMapping("/queryPageByTeacherId")
    @PreAuthorize("@pcs.hasPermissions('teacherDefaultVipGroupSalary/queryPageByTeacherId')")
    public Object queryPageByTeacherId(QueryInfo queryInfo, @RequestParam(defaultValue = "VIP") String groupType){

        PageInfo<TeacherDefaultVipGroupSalary> salaryPageInfo = teacherDefaultVipGroupSalaryService.queryPage(queryInfo);

        List<String> groupTypes = Arrays.stream(groupType.split(",")).collect(Collectors.toList());

        List<TeacherDefaultVipGroupSalary> collect = salaryPageInfo.getRows().stream()
                .filter(x -> groupTypes.contains(x.getGroupType())).collect(Collectors.toList());
        // 重置过滤后的数据
        salaryPageInfo.setRows(collect);

        return succeed(salaryPageInfo);
    }

    @ApiOperation(value = "批量新增、修改教师vip课酬")
    @PostMapping("/upSet")
    @PreAuthorize("@pcs.hasPermissions('teacherDefaultVipGroupSalary/upSet')")
    @AuditLogAnnotation(operateName = "批量新增、修改教师vip课酬")
    public Object setVipGroupSalary(@RequestBody TeacherDefaultVipGroupSalaryTemp teacherDefaultVipGroupSalaryTemp){
        teacherDefaultVipGroupSalaryService.upSet(teacherDefaultVipGroupSalaryTemp.teacherDefaultVipGroupSalaries, teacherDefaultVipGroupSalaryTemp.startDate);
        return succeed();
    }

    static class TeacherDefaultVipGroupSalaryTemp{
    	
    	private List<TeacherDefaultVipGroupSalary> teacherDefaultVipGroupSalaries;
    	
    	private Date startDate;

		public List<TeacherDefaultVipGroupSalary> getTeacherDefaultVipGroupSalaries() {
			return teacherDefaultVipGroupSalaries;
		}

		public void setTeacherDefaultVipGroupSalaries(List<TeacherDefaultVipGroupSalary> teacherDefaultVipGroupSalaries) {
			this.teacherDefaultVipGroupSalaries = teacherDefaultVipGroupSalaries;
		}

		public Date getStartDate() {
			return startDate;
		}

		public void setStartDate(Date startDate) {
			this.startDate = startDate;
		}
    }
}
