package com.ym.mec.web.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.wrapper.TempDirectorTrainingCampWrapper;
import com.ym.mec.biz.service.TempDirectorTrainingCampService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;

@Api(tags = "乐队指导训练营活动")
@RestController
@RequestMapping("${app-config.url.web:}/tempDirectorTrainingCamp")
public class TempDirectorTrainingCampController extends BaseController {

    @Resource
    private TempDirectorTrainingCampService tempDirectorTrainingCampService;

    @Resource
    private SysUserFeignService sysUserFeignService;

    @ApiOperation("创建训练营")
    @PostMapping(value = "/add")
    public HttpResponseResult<Object> add(@Valid @RequestBody TempDirectorTrainingCampWrapper.AddCamp dto) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        dto.setUpdateBy(sysUser.getId());
        tempDirectorTrainingCampService.add(dto);
        return succeed();
    }

    @ApiOperation("修改训练营")
    @PostMapping(value = "/modify")
    public HttpResponseResult<Object> modify(@Valid @RequestBody TempDirectorTrainingCampWrapper.AddCamp dto) {
        if (dto.getId() == null) {
            return failed("训练营id不能为空");
        }
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        dto.setUpdateBy(sysUser.getId());
        tempDirectorTrainingCampService.modify(dto);
        return succeed();
    }


    @ApiOperation(value = "分页获取")
    @PostMapping("/queryPage")
    public HttpResponseResult<PageInfo<TempDirectorTrainingCampWrapper.TempDirectorTrainingCamp>> queryPage(@RequestBody TempDirectorTrainingCampWrapper.TempDirectorTrainingCampQueryInfo queryInfo){
        IPage<TempDirectorTrainingCampWrapper.TempDirectorTrainingCamp> tempDirectorTrainingCampIPage = tempDirectorTrainingCampService.queryPage(QueryInfo.getPage(queryInfo), queryInfo);

        return succeed(QueryInfo.pageInfo(tempDirectorTrainingCampIPage));
    }
}

