package com.ym.mec.web.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.TempDirectorTrainingCamp;
import com.ym.mec.biz.dal.wrapper.TempDirectorTrainingCampDetailWrapper;
import com.ym.mec.biz.service.TempDirectorTrainingCampDetailService;
import com.ym.mec.biz.service.TempDirectorTrainingCampService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;

@Api(tags = "乐队指导训练营活动")
@RestController
@Slf4j
@RequestMapping("${app-config.url.web:}/tempDirectorTrainingDetailCamp")
public class TempDirectorTrainingCampDetailController extends BaseController {
    @Resource
    private TempDirectorTrainingCampDetailService tempDirectorTrainingCampDetailService;
    @Resource
    private TempDirectorTrainingCampService tempDirectorTrainingCampService;


    @ApiOperation(value = "分页获取")
    @PostMapping("/queryPage")
    public HttpResponseResult<PageInfo<TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetail>> queryPage(@RequestBody TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetailQuery queryInfo){
        if (queryInfo.getTempDirectorTrainingCampId()==null) {
            return failed("训练营编号不能为空");
        }
        IPage<TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetail> tempDirectorTrainingCampIPage = tempDirectorTrainingCampDetailService.queryPage(QueryInfo.getPage(queryInfo), queryInfo);

        return succeed(QueryInfo.pageInfo(tempDirectorTrainingCampIPage));
    }



    @ApiOperation(value = "导出")
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetailQuery queryInfo) throws IOException {
        if (queryInfo.getTempDirectorTrainingCampId()==null) {
            throw new BizException("训练营编号不能为空");
        }
        TempDirectorTrainingCamp trainingCamp = tempDirectorTrainingCampService.lambdaQuery()
                .eq(TempDirectorTrainingCamp::getId, queryInfo.getTempDirectorTrainingCampId())
                .eq(TempDirectorTrainingCamp::getDelFlag, false)
                .last("LIMIT 1").one();
        if (trainingCamp==null) {
            throw new BizException("训练营不存在");
        }
        IPage<TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetail> tempDirectorTrainingCampIPage = tempDirectorTrainingCampDetailService.queryPage(new Page<>(1,-1), queryInfo);
        List<TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetail> records = tempDirectorTrainingCampIPage.getRecords();
        if (records.size()==0) {
            throw new BizException("没有可导出数据");
        }
        OutputStream outputStream = response.getOutputStream();
        try {
            HSSFWorkbook workbook = POIUtil.exportExcel(new String[]{"姓名", "联系电话", "身份证号", "所在城市", "缴费状态", "缴费金额", "缴费时间", "目前从事职业"
                    , "第一主专业", "之前乐队指导学习基础", "目标效果", "需求或建议"}, new String[]{
                    "username", "mobile", "cardNo", "cityInfo", "paymentStatusName", "paymentAmount", "paymentDateStr",
                    "job", "subjectName", "study"
                    , "studyTarget", "suggestion"}, records);
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attac:wq" +
                    "hment;filename=乐队指导训练营-" + DateUtil.getDate(new Date()) + ".xls");

            outputStream = response.getOutputStream();
            workbook.write(outputStream);
            outputStream.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @ApiOperation(value = "统计")
    @PostMapping("/queryPageStat")
    public HttpResponseResult<TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetailStat> queryPageStat(@RequestBody TempDirectorTrainingCampDetailWrapper.TempDirectorTrainingCampDetailQuery queryInfo){
        if (queryInfo.getTempDirectorTrainingCampId()==null) {
            return failed("训练营编号不能为空");
        }
        return succeed(tempDirectorTrainingCampDetailService.queryPageStat(queryInfo));
    }

}

