package com.ym.mec.web.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ClassDateAdjustDto;
import com.ym.mec.biz.dal.dto.ReturnFeeDto;
import com.ym.mec.biz.dal.dto.VipGroupApplyBaseInfoDto;
import com.ym.mec.biz.dal.dto.VipGroupApplyDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.StudentApplyRefunds.StudentApplyRefundsStatus;
import com.ym.mec.biz.dal.enums.AuditStatusEnum;
import com.ym.mec.biz.dal.enums.VipGroupStatusEnum;
import com.ym.mec.biz.dal.page.VipGroupAttendanceQueryInfo;
import com.ym.mec.biz.dal.page.VipGroupQueryInfo;
import com.ym.mec.biz.dal.page.VipGroupSalaryQueryInfo;
import com.ym.mec.biz.dal.page.VipGroupTeachingRecordQueryInfo;
import com.ym.mec.biz.dal.wrapper.LiveGroupWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.ui.ModelMap;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author Joburgess
 * @Date 2019/9/21
 */
@Api(tags = "vip课")
@RequestMapping("${app-config.url.web:}/vipGroupManage")
@RestController
public class VipGroupManageController extends BaseController {

    @Autowired
    private VipGroupService vipGroupService;
    @Autowired
    private CourseScheduleService scheduleService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private ImGroupMemberService imGroupMemberService;
    @Autowired
    private ImUserFriendService imUserFriendService;
    @Autowired
    private ImGroupService imGroupService;
    @Autowired
    private StudentApplyRefundsService studentApplyRefundsService;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private VipGroupCategoryDao vipGroupCategoryDao;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "vip修改")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/update')")
    public HttpResponseResult update(VipGroup vipGroup){
        VipGroup oldVipGroup = vipGroupService.get(vipGroup.getId());
        if(Objects.nonNull(vipGroup.getCoursesExpireDate())){
            if(Objects.isNull(oldVipGroup)){
                throw new BizException("此课程组不存在");
            }
            if(!VipGroupStatusEnum.PROGRESS.equals(oldVipGroup.getStatus())&&!VipGroupStatusEnum.FINISHED.equals(oldVipGroup.getStatus())&&!VipGroupStatusEnum.PAUSE.equals(oldVipGroup.getStatus())){
                throw new BizException("此课程组状态暂不支持修改");
            }
        }
        ClassGroup classGroup = classGroupDao.findByMusicGroupAndType(oldVipGroup.getId().toString(),oldVipGroup.getGroupType());
        if(Objects.isNull(classGroup)){
            throw new BizException("课程信息错误");
        }
        VipGroupCategory vipGroupCategory = vipGroupCategoryDao.get(oldVipGroup.getVipGroupCategoryId());
        if(vipGroupCategory != null && vipGroupCategory.getMusicTheory()){
            if(VipGroupStatusEnum.PROGRESS.equals(vipGroup.getStatus())&&!VipGroupStatusEnum.FINISHED.equals(oldVipGroup.getStatus())&&oldVipGroup.getStatus().getCode()>VipGroupStatusEnum.PROGRESS.getCode()){
                throw new BizException("此课程组状态暂不支持修改");
            }
            if(classGroup.getStudentNum()<1){
                throw new BizException("班级人数未达到1人");
            }
        }
        Date now = new Date();
        if(VipGroupStatusEnum.FINISHED.equals(oldVipGroup.getStatus())&&(vipGroup.getCoursesExpireDate().compareTo(now)>=0|| DateUtil.isSameDay(vipGroup.getCoursesExpireDate(), now))){
            vipGroup.setStatus(VipGroupStatusEnum.PROGRESS);

            Map<Integer,String> userRoleMap = new HashMap<>();
            if(Objects.nonNull(vipGroup.getEducationalTeacherId())){
                userRoleMap.put(vipGroup.getEducationalTeacherId(),"乐团主管");
            }

            CourseSchedule lastCourse = courseScheduleDao.findLastCourseWithGroup(vipGroup.getId().toString(),oldVipGroup.getGroupType());
            if(Objects.isNull(lastCourse)){
                userRoleMap.put(vipGroup.getUserId(),"指导老师");
            }else{
                userRoleMap.put(lastCourse.getActualTeacherId(),"指导老师");
            }

            List<ClassGroupStudentMapper> classGroupStudents = classGroupStudentMapperDao.findByClassGroup(classGroup.getId());
            if(!CollectionUtils.isEmpty(classGroupStudents)){
                //生成学生单课缴费信息
                for (ClassGroupStudentMapper classGroupStudent : classGroupStudents) {
                    userRoleMap.put(classGroupStudent.getUserId(),null);
                }
            }

            imGroupService.create(classGroup.getId().toString(), null, classGroup.getName(),
                    classGroup.getName(), vipGroup.getName(), null, null,oldVipGroup.getGroupType(),
                    "VIP".equals(oldVipGroup.getGroupType())?ImGroup.GroupTypeEnum.VIP:ImGroup.GroupTypeEnum.LIVE);
            imGroupMemberService.join(classGroup.getId().toString(), userRoleMap);
            imUserFriendService.refreshGroupImUserFriend(classGroup.getMusicGroupId(),classGroup.getGroupType());
        }
        vipGroupService.update(vipGroup);
        return succeed();
    }

    @ApiOperation(value = "管理端vip课申请")
    @PostMapping("/vipGroupApply")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/vipGroupApply')")
    public Object vipGroupApply(@RequestBody VipGroupApplyDto vipGroupApplyDto){
        VipGroupApplyBaseInfoDto applyBaseInfo = vipGroupApplyDto.getVipGroupApplyBaseInfo();
        Integer userId = applyBaseInfo.getUserId();
        Teacher teacher = teacherDao.get(userId);
        if(Objects.isNull(teacher)){
            return failed("请指定指导老师!");
        }
        // 设置默认参数VIP
        if (StringUtils.isBlank(applyBaseInfo.getGroupType())) {
            applyBaseInfo.setGroupType(CourseSchedule.CourseScheduleType.VIP.getCode());
        }
        // VIP课参数校验
        if (CourseSchedule.CourseScheduleType.VIP.getCode().equals(applyBaseInfo.getGroupType())) {
            if(Objects.isNull(applyBaseInfo.getOrganId())){
                Integer firstStudentId = applyBaseInfo.getFirstStudentId();
                SysUser student = teacherDao.getUser(firstStudentId);
                if(Objects.isNull(student)||Objects.isNull(student.getOrganId())){
                    return failed("学员信息异常");
                }
                applyBaseInfo.setOrganId(student.getOrganId());
            }
        }else {
            //直播课分部跟随创建时的授课老师
            applyBaseInfo.setOrganId(teacher.getTeacherOrganId());
        }
        return vipGroupService.createVipGroup(vipGroupApplyDto);
    }


    @ApiOperation(value = "直播课详情")
    @GetMapping("/liveGroupDetail/{id}")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/liveGroupDetail')")
    public HttpResponseResult<VipGroupApplyDto> liveGroupDetail(@PathVariable Long id){
        return succeed(vipGroupService.liveGroupDetail(id));
    }

    @ApiOperation(value = "获取教务与指导老师公共分部")
    @GetMapping("/getPublicOrgans")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getPublicOrgans')")
    public HttpResponseResult getPublicOrgans(Integer eduTeacherId, Integer teacherId){
        if(Objects.isNull(eduTeacherId)){
            eduTeacherId=sysUserService.getUserId();
        }
        return succeed(vipGroupService.getPublicOrgans(eduTeacherId, teacherId));
    }

    @ApiOperation(value = "全查询")
    @PostMapping("/queryAll")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/queryAll')")
    public Object queryAll(@RequestBody VipGroupQueryInfo queryInfo) {
        if("VIP".equals(queryInfo.getGroupType())){
            queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        }else {
            queryInfo.setLiveOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        }
        return succeed(vipGroupService.findVipGroups(queryInfo));
    }

    @ApiOperation(value = "删除课程组")
    @PostMapping("/deleteVipGroup")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/deleteVipGroup')")
    public HttpResponseResult deleteVipGroup(Long vipGroupId){
        vipGroupService.deleteVipGroup(vipGroupId);
        return succeed();
    }

    @ApiOperation(value = "获取小课学员")
    @PostMapping(value = "/findVipGroupStudents")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupStudents')")
    public Object findVipGroupStudents(@RequestBody VipGroupQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupStudents(queryInfo));
    }

    @ApiOperation(value = "获取还有课程余额的学员")
    @PostMapping(value = "/findHaveCourseBalanceStudents")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findHaveCourseBalanceStudents')")
    public Object findHaveCourseBalanceStudents(@RequestBody VipGroupQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(vipGroupService.findHaveCourseBalanceStudents(queryInfo));
    }

    @ApiOperation(value = "添加vip课学员")
    @PostMapping(value = "/addVipGroupStudents")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/addVipGroupStudents')")
    public Object addVipGroupStudents(@RequestBody List<VipGroupStudentCoursePrice> studentCoursePrices){
        if(CollectionUtils.isEmpty(studentCoursePrices)){
            throw new BizException("请指定学员");
        }
        List<Integer> ids=studentCoursePrices.stream().map(VipGroupStudentCoursePrice::getStudentId).collect(Collectors.toList());
        Map<Integer, VipGroupStudentCoursePrice> studentCoursePriceMap = studentCoursePrices.stream().collect(Collectors.toMap(VipGroupStudentCoursePrice::getStudentId, v->v, (v1, v2)->v1));
        vipGroupService.addVipGroupStudents(studentCoursePrices.get(0).getVipGroupId(), ids, studentCoursePriceMap);
        return succeed();
    }

    @ApiOperation(value = "vip课详情")
    @GetMapping("/getVipGroupDetail")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getVipGroupDetail')")
    public Object getVipGroupDetail(Long vipGroupId){
        return succeed(vipGroupService.getVipGroupDetail(vipGroupId));
    }

    @ApiOperation(value = "vip课详情")
    @GetMapping("/getVipGroupDetailForAudit")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getVipGroupDetailForAudit')")
    public Object getVipGroupDetailForAudit(Long vipGroupId){
        return succeed(vipGroupService.getVipGroupDetailForAudit(vipGroupId));
    }

    @ApiOperation(value = "获取vip课考勤记录")
    @PostMapping("/getVipGroupAttendances")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getVipGroupAttendances')")
    public Object getVipGroupAttendances(@RequestBody VipGroupAttendanceQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupAttendances(queryInfo));
    }

    @ApiOperation(value = "休学")
    @PostMapping("/pauseForStudent")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/pauseForStudent')")
    public Object pauseForStudent(Long vipGroupId,Integer studentId){
        vipGroupService.pauseForStudent(vipGroupId,studentId);
        return succeed();
    }

    @ApiOperation(value = "恢复休学")
    @PostMapping("/recoverForStudent")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/recoverForStudent')")
    public Object recoverForStudent(@RequestBody StudentRecoverDto studentRecoverInfo){
        vipGroupService.recoverForStudent(studentRecoverInfo);
        return succeed();
    }

    @ApiOperation(value = "获取学生休学信息")
    @PostMapping("/getStudentPauseInfo")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getStudentPauseInfo')")
    public Object getStudentPauseInfo(Long vipGroupId,Integer studentId){
        return succeed(vipGroupService.getStudentPauseInfo(vipGroupId,studentId));
    }

    @ApiOperation(value = "根据vip课所在部门与声部获取教师")
    @GetMapping("/findTeacherWithVipGroupOrganAndSubject")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findTeacherWithVipGroupOrganAndSubject')")
    public Object findTeacherWithVipGroupOrganAndSubject(Long vipGroupId){
        return succeed(vipGroupService.findTeacherWithVipGroupOrganAndSubject(vipGroupId));
    }

    @ApiOperation(value = "退课申请")
    @PostMapping("/applyRefundForStudent")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/applyRefundForStudent')")
    public Object applyRefundForStudent(ReturnFeeDto returnFeeDto){
        return vipGroupService.applyRefundForStudent(returnFeeDto);
    }

    @ApiOperation(value = "退课申请审核")
    @PostMapping("/applyRefundAudit")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/applyRefundAudit')")
	public Object applyRefundAudit(Long id, StudentApplyRefundsStatus status, String remark, BigDecimal amount,boolean confirmReturnActivityGive) {
		return vipGroupService.applyRefundAudit(id, status, remark, amount,confirmReturnActivityGive);
	}

    @ApiOperation(value = "查询退费详情")
    @GetMapping("/queryApplyRefundDetail")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/queryApplyRefundDetail')")
	public Object queryApplyRefundDetail(Long id) {
		StudentApplyRefunds studentApplyRefunds = studentApplyRefundsService.get(id);
		if (studentApplyRefunds == null) {
			return failed("参数输入有误");
		}

		Long orderId = studentApplyRefunds.getOrigPaymentOrderId();

		StudentPaymentOrder paymentOrder = studentPaymentOrderService.get(orderId);
		if (paymentOrder == null) {
			return failed("支付订单信息找不到");
		}
		String vipGroupId = paymentOrder.getMusicGroupId();
		VipGroup vipGroup = vipGroupService.get(Long.parseLong(vipGroupId));
		if (vipGroup == null) {
			return failed("vip课信息找不到");
		}

		SysUser student = sysUserService.queryUserById(studentApplyRefunds.getUserId());

		SysUser teacher = sysUserService.queryUserById(vipGroup.getUserId());

		ModelMap model = new ModelMap();
		model.put("student", student.getUsername());
		model.put("teacher", teacher.getUsername());
		model.put("className", vipGroup.getName());
		model.put("studentApplyRefunds", studentApplyRefunds);

		return succeed(model);
	}

    @ApiOperation(value = "获取VIP课教学记录")
    @PostMapping("/findVipGroupTeachingRecord")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupTeachingRecord')")
    public Object findVipGroupTeachingRecord(@RequestBody VipGroupTeachingRecordQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupTeachingRecord(queryInfo));
    }

    @ApiOperation(value = "获取vip课基本信息")
    @GetMapping("/findTeachingRecordBaseInfo")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findTeachingRecordBaseInfo')")
    public Object findTeachingRecordBaseInfo(Long vipGroupId){
        return succeed(vipGroupService.findTeachingRecordBaseInfo(vipGroupId));
    }

    @ApiOperation("课酬总费用")
    @PostMapping("/getVipGroupCostCount")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/getVipGroupCostCount')")
    public Object getVipGroupCostCount(@RequestBody VipGroupApplyBaseInfoDto vipGroup){
        Map results = vipGroupService.countVipGroupPredictFee(vipGroup,vipGroup.getUserId());
        return succeed(results);
    }

    @ApiOperation(value = "停止vip课")
    @PostMapping("/stopVipGroup")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/stopVipGroup')")
    public Object stopVipGroup(Long vipGroupId,String stopReason){
        if(Objects.isNull(vipGroupId)|| StringUtils.isBlank(stopReason)){
            return failed("请指定课程组并指定取消原因");
        }
        vipGroupService.stopGroup(vipGroupId,stopReason);
        return succeed();
    }

    @ApiOperation(value = "获取vip课财务信息")
    @PostMapping("/findVipGroupSalarys")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupSalarys')")
    public Object findVipGroupSalarys(@RequestBody VipGroupSalaryQueryInfo queryInfo){
        return succeed(vipGroupService.findVipGroupSalarys(queryInfo));
    }

    @ApiOperation(value = "获取当前课程上课学员")
    @GetMapping("/findVipGroupAttendanceStudents")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/findVipGroupAttendanceStudents')")
    public Object findVipGroupAttendanceStudents(Long courseScheduleId){
        return succeed(vipGroupService.findVipGroupAttendanceStudents(courseScheduleId));
    }

    @ApiOperation(value = "课时调整")
    @PostMapping(value = "/classStartDateAdjust")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/classStartDateAdjust')")
    public Object classStartDateAdjust(ClassDateAdjustDto classDateAdjustDto){
        if(Objects.isNull(classDateAdjustDto.getId())){
            return failed( "请指定课程!");
        }
        CourseSchedule courseSchedule = scheduleService.get(classDateAdjustDto.getId());
        if(Objects.isNull(classDateAdjustDto.getClassGroupId())){
            classDateAdjustDto.setClassGroupId(courseSchedule.getClassGroupId());
        }
        List<CourseSchedule> courseSchedules=new ArrayList<>();
        courseSchedules.add(classDateAdjustDto);
        return scheduleService.courseAdjust(courseSchedules, classDateAdjustDto.getAllowZeroSalary());
    }

    @ApiOperation(value = "vip课审核")
    @PostMapping(value = "/vipApplyAudit")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/vipApplyAudit')")
    public Object vipApplyAudit(Long vipGroupId, AuditStatusEnum auditStatus, String reason){
        vipGroupService.vipApplyAudit(vipGroupId,auditStatus,reason);
        return succeed();
    }

    @ApiOperation(value = "更新vip课基本信息")
    @PostMapping(value = "/updateVipBaseInfo")
    public Object updateVipBaseInfo(@RequestBody VipGroupApplyBaseInfoDto vipGroupApplyBaseInfo){
        vipGroupService.updateVipGroupBaseInfo(vipGroupApplyBaseInfo);
        return succeed();
    }

    @ApiOperation(value = "取消达到报名时间未达到报名人数的vip课")
    @PostMapping(value = "/stopVipGroupWithNoCreate")
    public Object stopVipGroupWithNoCreate(){
        vipGroupService.stopVipGroupWithNoCreate(TenantContextHolder.getTenantId());
        return succeed();
    }

    @ApiOperation(value = "获取学生指定vip课的剩余课时费用")
    @GetMapping(value = "/getStudentSurplusCourseFee")
    public Object getStudentSurplusCourseFee(Long vipGroupId, Integer studentId){
        return succeed(vipGroupService.getStudentSurplusCourseFee(vipGroupId,studentId));
    }

    @ApiOperation(value = "更新直播课元素")
    @PostMapping(value = "/updateLiveGroup")
    @PreAuthorize("@pcs.hasPermissions('vipGroupManage/updateLiveGroup')")
    public Object updateLiveGroup(@RequestBody VipGroupApplyDto vipGroupApplyDto){
        return succeed(vipGroupService.updateLiveGroup(vipGroupApplyDto));
    }

    @ApiOperation(value = "直播课学生列表")
    @PostMapping(value = "/liveStudentList")
    public HttpResponseResult<PageInfo<LiveGroupWrapper.LiveGroupStudentList>> liveStudentList(@Validated @RequestBody LiveGroupWrapper.LiveGroupStudentQuery query){

        IPage<LiveGroupWrapper.LiveGroupStudentList> iPage = vipGroupService.liveStudentList(QueryInfo.getPage(query), query);
        return succeed(PageUtil.pageInfo(iPage));
    }

    @ApiOperation(value = "课程列表")
    @PostMapping(value = "/liveCourseList")
    public HttpResponseResult<Object> liveCourseList(@Validated @RequestBody LiveGroupWrapper.LiveGroupStudentQuery query){

        return succeed(vipGroupService.liveCourseList(query));
    }

    @ApiOperation(value = "删除直播课")
    @GetMapping(value = "/delLiveGroup/{id}")
    public HttpResponseResult<Object> delLiveGroup(@PathVariable Integer id){
        return succeed(vipGroupService.del(id));
    }
}
