package com.ym.mec.web.controller.degree;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.Degree;
import com.ym.mec.biz.dal.wrapper.DegreeWrapper;
import com.ym.mec.biz.service.DegreeService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/degree")
@Api(tags = "考级信息")
public class DegreeInfoController extends BaseController {

    @Autowired
    private DegreeService degreeService;

	@ApiOperation(value = "详情", notes = "考级信息-根据详情ID查询单条, 传入id")
    //@PreAuthorize("@pcs.hasPermissions('degree/detail')")
    @GetMapping("/detail/{id}")
    public HttpResponseResult<Degree> detail(@PathVariable("id") Long id) {
    
    	Degree wrapper = degreeService.detail(id);
        
        return succeed(wrapper);
	}
    
    @ApiOperation(value = "查询分页", notes = "考级信息- 传入 DegreeWrapper.DegreeQuery") 
    //@PreAuthorize("@pcs.hasPermissions('degree/page')")
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<DegreeWrapper.Degree>> page(@RequestBody DegreeWrapper.DegreeQuery query) {
        
        IPage<DegreeWrapper.Degree> pages = degreeService.selectPage(QueryInfo.getPage(query), query);
        
        return succeed(QueryInfo.pageInfo(pages));
	}
    
    @ApiOperation(value = "新增", notes = "考级信息- 传入 DegreeWrapper.Degree")
//    @PreAuthorize("@pcs.hasPermissions('degree/save')")
	@PostMapping("/save")
	public HttpResponseResult<JSONObject> add(@Validated @RequestBody DegreeWrapper.Degree degree) {
        
        // 新增数据
        degreeService.add(degree);
        
        return succeed();
	}
    
    @ApiOperation(value = "修改", notes = "考级信息- 传入 DegreeWrapper.Degree")
//    @PreAuthorize("@pcs.hasPermissions('degree/update')")
	@PostMapping("/update")
	public HttpResponseResult<JSONObject> update(@Validated @RequestBody DegreeWrapper.Degree degree) {

        if (degree.getId() == null) {
            return failed("id不能为空");
        }
        // 更新数据
        degreeService.update(degree);
        
        return succeed();
	}

	@ApiOperation(value = "删除", notes = "考级信息- 传入id")
    @PreAuthorize("@pcs.hasPermissions('degree/remove')")
//	@PostMapping("/remove")
	public HttpResponseResult<Boolean> remove(@RequestParam Long id) {
    
		return succeed(degreeService.removeById(id));
	}


    @ApiOperation(value = "详情统计", notes = "考级信息- 传入id")
//    @PreAuthorize("@pcs.hasPermissions('degree/statistic')")
	@GetMapping("/statistic")
    public HttpResponseResult<DegreeWrapper.DegreeStatistical> statistic(@RequestParam Long id) {

        return succeed(degreeService.statistic(id));
    }
}