package com.ym.mec.web.controller.degree;

import com.alibaba.fastjson.JSONObject;
import com.microsvc.toolkit.common.response.paging.PageInfo;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.DegreeNews;
import com.ym.mec.biz.dal.enums.DegreeNewsTypeEnum;
import com.ym.mec.biz.dal.vo.DegreeNewsVo;
import com.ym.mec.biz.dal.wrapper.DegreeNewsWrapper;
import com.ym.mec.biz.service.DegreeNewsService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Objects;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/degreeNews")
@Api(tags = "考级资讯表")
public class DegreeNewsController extends BaseController {

    @Autowired
    private DegreeNewsService degreeNewsService;
    @Autowired
    private SysUserService sysUserService;

	@ApiOperation(value = "详情", notes = "考级资讯表-根据详情ID查询单条, 传入id")
    @PreAuthorize("@pcs.hasPermissions('degreeNews/detail')")
    @GetMapping("/detail/{id}")
    public HttpResponseResult<DegreeNews> detail(@PathVariable("id") Long id) {
        return succeed(degreeNewsService.detail(id));
	}
    
    @ApiOperation(value = "查询分页", notes = "考级资讯表- 传入 DegreeNewsWrapper.DegreeNewsQuery") 
    @PreAuthorize("@pcs.hasPermissions('degreeNews/page')")
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<DegreeNewsVo>> page(@RequestBody DegreeNewsWrapper.DegreeNewsQuery query) {
        return succeed(QueryInfo.pageInfo(degreeNewsService.selectPage(QueryInfo.getPage(query), query)));
	}
    
    @ApiOperation(value = "新增", notes = "考级资讯表- 传入 DegreeNewsWrapper.DegreeNews")
    @PreAuthorize("@pcs.hasPermissions('degreeNews/save')")
	@PostMapping("/save")
	public HttpResponseResult<JSONObject> add(@Validated @RequestBody DegreeNews degreeNews) {
        degreeNews.setOperatorId(sysUserService.getUserId());
        degreeNewsService.save(degreeNews);
        return succeed();
	}
    
    @ApiOperation(value = "修改", notes = "考级资讯表- 传入 DegreeNewsWrapper.DegreeNews")
    @PreAuthorize("@pcs.hasPermissions('degreeNews/update')")
	@PostMapping("/update")
	public HttpResponseResult<JSONObject> update(@Validated @RequestBody DegreeNews degreeNews) {
        degreeNews.setOperatorId(sysUserService.getUserId());
        degreeNewsService.updateById(degreeNews);
        return succeed();
	}

    @ApiOperation(value = "启用、停用")
    @PreAuthorize("@pcs.hasPermissions('degreeNews/enable')")
	@PostMapping("/enable")
	public HttpResponseResult<JSONObject> enable(Long newsId) {
        DegreeNews degreeNews = degreeNewsService.getById(newsId);

        if (Objects.isNull(degreeNews)) {
            throw new BizException("考级资讯编号无效");
        }

        // 考级须知，若是启用，则停用其他的
        if (degreeNews.getType() == DegreeNewsTypeEnum.GRADING_NOTICE
                && !degreeNews.getStatus()) {
            degreeNewsService.lambdaUpdate()
                    .eq(DegreeNews::getTenantId, degreeNews.getTenantId())
                    .eq(Objects.nonNull(degreeNews.getOrganIds()), DegreeNews::getOrganIds, degreeNews.getOrganIds())
                    .eq(DegreeNews::getType, DegreeNewsTypeEnum.GRADING_NOTICE)
                    .set(DegreeNews::getStatus, false)
                    .update();
        }

        degreeNews.setStatus(!degreeNews.getStatus());
        degreeNewsService.updateById(degreeNews);
        return succeed();
	}

	@ApiOperation(value = "删除", notes = "考级资讯表- 传入id")
    @PreAuthorize("@pcs.hasPermissions('degreeNews/remove')")
	@PostMapping("/remove")    
	public HttpResponseResult<Boolean> remove(@RequestParam Long id) {
		return succeed(degreeNewsService.removeById(id));
	}
}