package com.ym.mec.web.controller.education;

import com.ym.mec.biz.dal.dao.ClassGroupDao;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.page.CooperationOrganQueryInfo;
import com.ym.mec.biz.service.CooperationOrganService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RequestMapping("${app-config.url.web:}/eduCooperationOrgan")
@Api(tags = "合作单位(教学点)服务")
@RestController
public class EduCooperationOrganController extends BaseController {

    @Resource
    private CooperationOrganService cooperationOrganService;
    @Resource
    private OrganizationService organizationService;
    @Resource
    private ClassGroupDao classGroupDao;

    @ApiOperation(value = "分页查询合作单位(教学点)列表")
    @GetMapping("/queryPage")
    public HttpResponseResult<PageInfo<CooperationOrgan>> queryPage(CooperationOrganQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(cooperationOrganService.queryPage(queryInfo));
    }

    @ApiOperation(value = "根据分部获取合作单位(教学点)列表")
    @GetMapping("/queryByOrganId")
    public HttpResponseResult<List<CooperationOrgan>> queryByOrganId(String organId, Boolean enable) {
        return succeed(cooperationOrganService.queryByOrganId(organizationService.getEmployeeOrgan(organId),enable));
    }

    @ApiOperation(value = "合作单位的乐团")
    @GetMapping("/musicGroupPage")
    public HttpResponseResult<List<MusicGroup>> musicGroupPage(String organId,String coopId) {
        return succeed(cooperationOrganService.musicGroupPage(organizationService.getEmployeeOrgan(organId),coopId));
    }

    @ApiOperation(value = "获取乐团班级列表")
    @GetMapping("/musicGroupClassPage")
    public HttpResponseResult<List<ClassGroup>> musicGroupClassPage(String musicGroupId) {
        return succeed(classGroupDao.findClassGroups(musicGroupId));
    }
}
