package com.ym.mec.web.controller.education;

import com.ym.mec.auth.api.dto.MusicScoreQueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.SysMusicScoreAccompaniment;
import com.ym.mec.biz.dal.enums.ClientTypeEnum;
import com.ym.mec.biz.dal.page.SysExamSongQueryInfo;
import com.ym.mec.biz.service.CbsMusicScoreCategoriesService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.SysMusicScoreAccompanimentService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

@RequestMapping("${app-config.url.web:}/eduMusicScore")
@Api(tags = "智能陪练")
@RestController
public class EduMusicScoreController extends BaseController {
    @Resource
    private CbsMusicScoreCategoriesService cbsMusicScoreCategoriesService;
    @Resource
    private SysMusicScoreAccompanimentService sysMusicScoreAccompanimentService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private EmployeeDao employeeDao;
    @Resource
    private OrganizationService organizationService;

    @ApiOperation(value = "树状列表")
    @GetMapping("/queryTree")
    public Object queryTree(MusicScoreQueryInfo queryInfo) {
        SysUser sysUser = sysUserService.getUser();
        String organId = organizationService.getEmployeeOrgan(sysUser.getId(), null, sysUser.getIsSuperAdmin() && sysUser.getTenantId() == -1);
        String teachingMaterialIds = organizationService.getTeachingMaterialIds(organId, sysUser.getTenantId());
        return succeed(cbsMusicScoreCategoriesService.queryCategoriesTreeList(teachingMaterialIds, queryInfo.getEnable()));
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/queryPage")
    public HttpResponseResult<List<SysMusicScoreAccompaniment>> queryPage(SysExamSongQueryInfo queryInfo) {
        queryInfo.setShowFlag(1);
        return succeed(sysMusicScoreAccompanimentService.queryAll(queryInfo));
    }

    @ApiOperation(value = "分页查询")
    @GetMapping("/queryPageLimit")
    public Object queryPageLimit(SysExamSongQueryInfo queryInfo) {
        SysUser sysUser = sysUserService.getUser();
        String type = queryInfo.getType();
        if(StringUtils.isEmpty(type)){
            queryInfo.setType("ALL");
        }
        if(queryInfo.getClientType() == null){
            queryInfo.setClientType(ClientTypeEnum.SMART_PRACTICE);
        }
        queryInfo.setShowFlag(1);
        Employee employee = employeeDao.get(sysUser.getId());
        if (StringUtils.isEmpty(queryInfo.getOrganId())) {
            queryInfo.setOrganId(employee.getOrganIdList());
        }else if(StringUtils.isEmpty(employee.getOrganIdList())){
            return failed("用户所在分部异常");
        }else {
            List<String> list = Arrays.asList(employee.getOrganIdList().split(","));
            if(!list.containsAll(Arrays.asList(queryInfo.getOrganId().split(",")))){
                return failed("非法请求");
            }
        }
        queryInfo.setEnable(true);
        return succeed(sysMusicScoreAccompanimentService.queryScorePage(queryInfo));
    }

    @ApiOperation(value = "获取伴奏声部列表")
    @GetMapping("/querySubjectIds")
    public Object querySubjectIds(Integer categoriesId) {
        return succeed(sysMusicScoreAccompanimentService.querySubjectIds(categoriesId));
    }
}
