package com.ym.mec.web.controller.education;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dao.StudentInstrumentDao;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.dto.BrandDto;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.mall.MallFeignService;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@RequestMapping("${app-config.url.web:}/eduRepair")
@Api(tags = "教务维修服务")
@RestController
public class EduRepairController extends BaseController {

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private EmployeeDao employeeDao;
    @Autowired
    private StudentRepairService studentRepairService;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private SubjectService subjectService;
    @Autowired
    private GoodsService goodsService;
    @Autowired
    private GoodsCategoryService goodsCategoryService;
    @Autowired
    private StudentGoodsSellService studentGoodsSellService;
    @Autowired
    private StudentInstrumentService studentInstrumentService;
    @Autowired
    private StudentInstrumentDao studentInstrumentDao;
    @Resource
    private MallFeignService mallFeignService;

    @ApiOperation("获取学生列表")
    @GetMapping(value = "/getStudents")
    public HttpResponseResult<PageInfo<BasicUserDto>> getStudents(RepairStudentQueryInfo queryInfo) {
        SysUser user = sysUserService.getUser();
        Employee employee = employeeDao.get(user.getId());
        if (Objects.isNull(employee)) {
            return failed(HttpStatus.NOT_ACCEPTABLE, "员工信息不存在");
        }
        queryInfo.setEmployeeId(user.getId());
        queryInfo.setOrganIdList(employee.getOrganIdList());
        queryInfo.setTenantId(user.getTenantId());
        return succeed(studentRepairService.getStudents(queryInfo));
    }

    @ApiOperation("获取商品购买教务老师关联的学员列表")
    @GetMapping(value = "/queryEduStudents")
    public HttpResponseResult<PageInfo<BasicUserDto>> queryEduStudents(QueryInfo queryInfo) {
        return succeed(studentRepairService.queryEduStudents(queryInfo));
    }

    @ApiOperation("添加商品销售订单")
    @PostMapping(value = "/addGoodsSellOrder")
    public HttpResponseResult addGoodsSellOrder(@RequestBody StudentGoodsSell studentGoodsSell) throws Exception {
        if (studentGoodsSell.getIsRepeatPay() == false) {
            List<StudentPaymentOrder> list = studentPaymentOrderService.queryByCondition(GroupType.GOODS_SELL, null,studentGoodsSell.getUserId(), DealStatusEnum.ING,
                    OrderTypeEnum.RENEW);
            if (list.size() > 0) {
                return failed(HttpStatus.CONTINUE, "该学员有待支付的订单");
            }
        }
        Integer userId = sysUserService.getUserId();
        studentGoodsSell.setTeacherId(userId);
        studentGoodsSell.setAuthorUser(userId);
        Map map = studentRepairService.addGoodsSellOrder(studentGoodsSell);
        if(map.containsKey("tradeState")){
            return failed(HttpStatus.CREATED, map,"恭喜您,购买成功!");
        }
        return succeed(map);
    }

    @ApiOperation("添加维修单")
    @PostMapping(value = "/addRepair")
    public HttpResponseResult addRepair(@RequestBody StudentRepair repairInfo) throws Exception {
        SysUser sysUser = sysUserService.getUser();
        repairInfo.setEmployeeId(sysUser.getId());
        repairInfo.setEmployeeName(sysUser.getRealName());
        return succeed(studentRepairService.addRepair(repairInfo));
    }

    @ApiOperation("获取学生的维修记录")
    @GetMapping(value = "/getStudentRepairList")
    public HttpResponseResult getStudentRepairList(RepairStudentQueryInfo queryInfo) {
        queryInfo.setEmployeeId(sysUserService.getUserId());
        queryInfo.setPayStatus(2);
        PageInfo<StudentRepair> repairPageInfo = studentRepairService.queryPage(queryInfo);
        List<StudentRepair> rows = repairPageInfo.getRows();
        if (!rows.isEmpty()) {
            Map<Long, String> brandIdNameMap = mallFeignService.getList().stream().collect(Collectors.toMap(BrandDto::getId, BrandDto::getName));
            rows.forEach(row -> {
                String goodsJson = row.getGoodsJson();
                if (StringUtils.isNotEmpty(goodsJson)) {
                    JSONArray goods = JSON.parseArray(goodsJson);
                    for (Object good : goods) {
                        JSONObject goodObject = (JSONObject) good;
                        String brand = goodObject.getString("brand");
                        if (Objects.nonNull(brand) && NumberUtils.isNumber(brand)) {
                            goodObject.put("brandName", brandIdNameMap.getOrDefault(Long.valueOf(brand), ""));
                        }
                    }
                    row.setGoodsJson(JSON.toJSONString(goods));
                }
            });
        }
        return succeed(repairPageInfo);
    }

    @ApiOperation("获取乐器种类")
    @GetMapping(value = "/findSubSubjects")
    public HttpResponseResult findSubSubjects() {
        return succeed(subjectService.findSubSubjects( null,null));
    }

    @ApiOperation("获取学生信息")
    @GetMapping(value = "/getStudentInfo")
    public HttpResponseResult getStudentInfo(Integer studentId) {
        return succeed(studentRepairService.getStudentInfo(studentId));
    }

    @ApiOperation("获取维修记录详情")
    @GetMapping(value = "/getRepairInfo")
    public HttpResponseResult getRepairInfo(Integer id) {
        return succeed(studentRepairService.getRepairInfo(id));
    }


    @ApiOperation("获取维修列表")
    @GetMapping(value = "/queryPage")
    public HttpResponseResult queryPage(RepairStudentQueryInfo queryInfo) {
        queryInfo.setEmployeeId(sysUserService.getUserId());
        Date endTime = queryInfo.getEndTime();
        if(endTime != null){
            queryInfo.setEndTime(DateUtil.addDays(endTime, 1));
        }
        queryInfo.setPayStatus(2);
        return succeed(studentRepairService.queryPage(queryInfo));
    }


    @ApiOperation("维修完成")
    @PostMapping(value = "/repairSuccess")
    public HttpResponseResult repairSuccess(Integer id,String description,Integer repairStatus) {
        studentRepairService.repairSuccess(id,description,repairStatus);
        return succeed();
    }

    @ApiOperation(value = "分页查询商品(教材、辅件)列表")
    @GetMapping("/queryGoodsPage")
    public Object queryPage(GoodsQueryInfo queryInfo){
        queryInfo.setOrganId(null);
        Employee employee = employeeDao.get(sysUserService.getUserId());
        if (queryInfo.getStudentShowOrganId() == null &&
                queryInfo.getEducationShowOrganId() == null &&
                queryInfo.getCourseFeeShowOrganId() == null &&
                queryInfo.getMemberFeeShowOrganId() == null &&
                queryInfo.getReplacementShowOrganId() == null) {
            queryInfo.setEducationShowOrganId(employee.getOrganIdList());
        }else if(StringUtils.isEmpty(employee.getOrganIdList())){
            return failed("用户所在分部异常");
        }
        return succeed(goodsService.queryPage(queryInfo));
    }

    @ApiOperation(value = "分页查询商品分类列表")
    @GetMapping("/queryGoodsCategoryPage")
    public Object queryGoodsCategoryPage(GoodsCategoryQueryInfo queryInfo) {
        return succeed(goodsCategoryService.queryPage(queryInfo));
    }

    @ApiOperation(value = "分页查询教务老师关联的学员商品订单")
    @GetMapping("/queryStudentGoodsOrders")
    public Object queryStudentGoodsOrders(GoodsSellQueryInfo queryInfo) {
        queryInfo.setTeacherId(sysUserService.getUserId());
        return succeed(studentGoodsSellService.queryStudentGoodsOrders(queryInfo));
    }

    @ApiOperation(value = "获取乐器与乐保列表")
    @GetMapping("/getInstrumentList")
    public HttpResponseResult<PageInfo<StudentInstrument>> getInstrumentList(StudentInstrumentQueryInfo queryInfo) {
        studentInstrumentDao.batchUpdate();
        return succeed(studentInstrumentService.queryPage(queryInfo));
    }
}
