package com.ym.mec.web.controller.education;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.TeacherServeDto;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author Joburgess
 * @Date 2020/4/10
 */
@RequestMapping("${app-config.url.web:}/eduExercisesSituation")
@Api(tags = "课外训练统计服务")
@RestController
public class EduStudentExtracurricularExercisesSituationController extends BaseController {

    @Autowired
    private StudentExtracurricularExercisesSituationService studentExtracurricularExercisesSituationService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private ExtracurricularExercisesMessageService extracurricularExercisesMessageService;
    @Autowired
    private StudentCourseHomeworkReplyService studentCourseHomeworkReplyService;
    @Autowired
    private StudentServeService studentServeService;
    @Autowired
    private OrganizationService organizationService;

    @ApiOperation(value = "课外训练-教学")
    @GetMapping("/findStudentExtracurricularExercisesSituations")
    public HttpResponseResult findStudentExtracurricularExercisesSituations(StudentExercisesSituationQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentExtracurricularExercisesSituationService.findStudentExtracurricularExercisesSituations(queryInfo));
    }

    @ApiOperation(value = "服务指标详情")
    @GetMapping("/findServiceStudentDetail")
    public HttpResponseResult findServiceStudentDetail(StudentServiceDetailQueryInfo queryInfo) {
        return succeed(studentExtracurricularExercisesSituationService.findServiceStudentDetail(queryInfo));
    }

    @ApiOperation(value = "教师服务指标明细")
    @GetMapping("/queryTeacherServeInfo")
    public HttpResponseResult queryTeacherServeInfo(TeacherServeQueryInfo queryInfo) {
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherServeInfo(queryInfo));
    }

    @ApiOperation(value = "回复查询")
    @GetMapping("/findStudentHomeworkComments")
    public Object findStudentHomeworkComments(StudentCourseHomeworkReplyQueryInfo queryInfo){
        if(!queryInfo.isExtra()){
            return succeed(studentCourseHomeworkReplyService.queryPage(queryInfo));
        }else{
            return succeed(extracurricularExercisesMessageService.findStudentExtraExerciseComments(queryInfo));
        }
    }

    @ApiOperation(value = "教师作业布置提醒")
    @PostMapping("/teacherServeRemindPush")
    public HttpResponseResult teacherServeRemindPush(@RequestBody List<TeacherServeDto> teacherServes) {
        SysUser sysUser = sysUserService.getUser();
        studentExtracurricularExercisesSituationService.teacherServeRemindPush(teacherServes, sysUser.getId(), sysUser.getRealName());
        return succeed();
    }

    @ApiOperation(value = "教师服务课后作业")
    @GetMapping("/queryTeacherServeHomeworkDetail")
    public HttpResponseResult queryTeacherServeHomeworkDetail(TeacherServeHomeworkQueryInfo queryInfo) {
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherServeHomeworkDetail(queryInfo));
    }

    @ApiOperation(value = "教师服务课外作业")
    @GetMapping("/queryTeacherServeExtraDetail")
    public HttpResponseResult queryTeacherServeExtraDetail(TeacherServeHomeworkQueryInfo queryInfo) {
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherServeExtraDetail(queryInfo));
    }

    @ApiOperation(value = "乐团作业情况")
    @GetMapping("/musicGroupHomeworkStat")
    public HttpResponseResult musicGroupHomeworkStat(String musicGroupId, String startDay, String endDay) {
        return succeed(studentServeService.musicGroupHomeworkStat(musicGroupId, null, startDay, endDay));
    }

}
