package com.ym.mec.web.controller.education;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.AppVersionInfoDao;
import com.ym.mec.biz.dal.entity.AppVersionInfo;
import com.ym.mec.biz.dal.enums.MessageSendMode;
import com.ym.mec.biz.dal.page.SysMessageQueryInfo;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.validator.CommonValidator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Map;

@RestController
@Api(tags = "消息服务")
@RequestMapping("${app-config.url.web:}/sysEduMessage")
public class SysEduMessageController extends BaseController {

	@Resource
	private SysMessageService sysMessageService;

	@Resource
	private SysUserService sysUserService;

	@Resource
	private AppVersionInfoDao appVersionInfoDao;

	@ApiOperation("获取所有消息列表")
	@GetMapping(value = "list")
	public Object list(SysMessageQueryInfo queryInfo){
		queryInfo.setUserId(sysUserService.getUserId());
		queryInfo.setType(3);
		queryInfo.setJpushType("SYSTEM");
		return succeed(sysMessageService.queryPage(queryInfo));
	}

	@ApiOperation("获取消息分类类型")
	@GetMapping(value = "typeList")
	public Object typeList(String memo){
		//如果用户使用的版本比当前版本号大，那么屏蔽缴费信息
		if(StringUtils.isNotEmpty(memo)){
			//获取当前使用的版本
			AppVersionInfo appVersionInfo = appVersionInfoDao.queryNewestByPlatform("ios-education").get(0);
			int defaultVersion = Integer.parseInt(appVersionInfo.getVersion().replaceAll("\\.",""));
			int currentVersion = Integer.parseInt(memo.replaceAll("\\.",""));
			if(currentVersion > defaultVersion){
				return succeed(JSON.parseArray("[{\"value\":\"全部\",\"key\":\"ALL\"}," +
						"{\"value\":\"学员信息\",\"key\":\"STUDENT\"}," +
						"{\"value\":\"乐团信息\",\"key\":\"MUSIC\"}," +
						"{\"value\":\"其他\",\"key\":\"DEFAULT\"}]"));
			}
		}
		return succeed(JSON.parseArray("[{\"value\":\"全部\",\"key\":\"ALL\"}," +
				"{\"value\":\"缴费信息\",\"key\":\"PAY\"}," +
				"{\"value\":\"学员信息\",\"key\":\"STUDENT\"}," +
				"{\"value\":\"乐团信息\",\"key\":\"MUSIC\"}," +
				"{\"value\":\"其他\",\"key\":\"DEFAULT\"}]"));
	}

	@ApiOperation("一键已读")
	@PostMapping("batchSetRead")
	public Object batchSetRead() {
		return sysMessageService.updateStatus(sysUserService.getUserId(), 1,"SYSTEM") ? succeed() : failed();
	}

	@ApiOperation("设置已读")
	@PostMapping("setRead")
	public Object setRead(Long id) {
		return sysMessageService.updateOneStatus(id)? succeed() : failed();
	}

	@ApiOperation(value = "查询用户未读消息条数")
	@GetMapping("/queryCountOfUnread")
	public Object queryCountOfUnread() {
		SysUser sysUser = sysUserService.getUser();
		Map<String, Integer> map = sysMessageService.queryCountOfUnread(MessageSendMode.PUSH, sysUser.getId(),"SYSTEM", sysUser.getTenantId());

		if (map == null || map.size() == 0) {
			return succeed();
		}
		return succeed(map);
	}

	@ApiOperation(value = "发送消息")
	@PostMapping("/sendMessage")
	public Object sendMessage(MessageSender messageSender, String content, String receiver, int readStatus, String url, String group) {
		SysUser sysUser = sysUserService.getUser();
		Integer userId = sysUser.getId();

		String mobileNo = sysUser.getPhone();
		if (StringUtils.isBlank(mobileNo) || !CommonValidator.isMobileNo(mobileNo)) {
			throw new BizException("请输入正确的手机号");
		}
//		sysMessageService.sendMessage(messageSender, userId, "", content, receiver, null, readStatus, url, group,"SYSTEM");
		return succeed();
	}
}
