package com.ym.mec.web.controller.open;

import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.entity.SysConfig;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.dal.wrapper.Double11StaticWrapper;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.collection.MapUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.rmi.MarshalledObject;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author 袁亮
 * @apiNote 2023双11销售统计
 */
@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/open/double11Statics")
@Api(tags = "2023双11统计")
public class OpenDouble11StaticsController extends BaseController {

    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;

    @Autowired
    private OrganizationService organizationService;

    @Autowired
    private SysConfigService sysConfigService;


    @ApiOperation(value = "2023双11销售统计", notes = "2023双11销售统计")
    @GetMapping("/saleStatics")
    public R<Double11StaticWrapper.SaleStatics> saleStatics() {
        Double11StaticWrapper.SaleStaticsQuery query = new Double11StaticWrapper.SaleStaticsQuery();

        SysConfig config = sysConfigService.findByParamName("2023_double_eleven_active_config");
        SysConfig orgs = sysConfigService.findByParamName("2023_activity_organ");
        if (orgs != null) {
            query.setOrganId(orgs.getParanValue());
        }
        Double11StaticWrapper.SaleStaticConfigParam configParam =
                JSON.parseObject(config.getParanValue(), Double11StaticWrapper.SaleStaticConfigParam.class);

        Double11StaticWrapper.SaleStatics result = new Double11StaticWrapper.SaleStatics();
        // 1V1
        query.setSaleStaticsType(1);
        setQueryParam(query, configParam);
        Double11StaticWrapper.SaleAmountAndUser s1v1 = studentPaymentOrderDao.saleStatics(query);
        result.setTotalSale1V1(s1v1.getSaleAmount());
        result.setTotalUser1V1PurchaseNumber(s1v1.getUserPurchaseNumber());

        // 1v12
        query.setSaleStaticsType(2);
        setQueryParam(query, configParam);
        Double11StaticWrapper.SaleAmountAndUser s1v2 = studentPaymentOrderDao.saleStatics(query);
        result.setTotalSale1V2(s1v2.getSaleAmount());
        result.setTotalUser1V2PurchaseNumber(s1v2.getUserPurchaseNumber());

        // 乐理直播
        query.setSaleStaticsType(3);
        setQueryParam(query, configParam);
        Double11StaticWrapper.SaleAmountAndUser theory = studentPaymentOrderDao.saleStatics(query);
        result.setTotalSaleTheory(theory.getSaleAmount());
        result.setTotalUserTheoryPurchaseNumber(theory.getUserPurchaseNumber());

        // 云教练
        query.setSaleStaticsType(4);
        setQueryParam(query, configParam);
        Double11StaticWrapper.SaleAmountAndUser cloudTeacher = studentPaymentOrderDao.saleStatics(query);
        result.setTotalSaleCloudTeacher(cloudTeacher.getSaleAmount());
        result.setTotalUserCloudTeacherPurchaseNumber(cloudTeacher.getUserPurchaseNumber());

        // 乐器
        query.setSaleStaticsType(5);
        setQueryParam(query, configParam);
        Double11StaticWrapper.SaleAmountAndUser good = studentPaymentOrderDao.musicalSaleStatics(query);
        result.setTotalSaleMusical(good.getSaleAmount());
        result.setTotalUserMusicalPurchaseNumber(good.getUserPurchaseNumber());

        // 保价卡
        query.setSaleStaticsType(6);
        setQueryParam(query, configParam);
        Double11StaticWrapper.SaleAmountAndUser sporadic = studentPaymentOrderDao.saleStatics(query);
        result.setTotalSaleMaintenance(sporadic.getSaleAmount());
        result.setTotalUserMaintenancePurchaseNumber(sporadic.getUserPurchaseNumber());

        // 小组课
        query.setSaleStaticsType(7);
        setQueryParam(query, configParam);
        Double11StaticWrapper.SaleAmountAndUser smallGroups = studentPaymentOrderDao.saleStatics(query);
        result.setSmallGroupsMaintenance(smallGroups.getSaleAmount());
        result.setSmallGroupsPurchaseNumber(smallGroups.getUserPurchaseNumber());

        BigDecimal totalAmount = result.getTotalSale1V1()
                .add(result.getTotalSale1V2())
                .add(result.getTotalSaleMaintenance())
                .add(result.getTotalSaleMusical())
                .add(result.getTotalSaleTheory())
                .add(result.getTotalSaleCloudTeacher())
                .add(result.getSmallGroupsMaintenance());
        result.setTotalSaleAmount(totalAmount);
        return R.from(result);
    }

    @ApiOperation(value = "2023双11销售统计详情", notes = "2023双11销售统计详情")
    @GetMapping("/saleStaticsDetail")
    public R<List<Double11StaticWrapper.SaleAmountAndUser>> saleStaticsDetail(Double11StaticWrapper.SaleStaticsQuery query) {


        SysConfig config = sysConfigService.findByParamName("2023_double_eleven_active_config");
        Double11StaticWrapper.SaleStaticConfigParam configParam =
                JSON.parseObject(config.getParanValue(), Double11StaticWrapper.SaleStaticConfigParam.class);
        Map<Integer, Organization> orgIdMap = new HashMap<>();
        SysConfig orgs = sysConfigService.findByParamName("2023_activity_organ");
        if (orgs != null) {
            query.setOrganId(orgs.getParanValue());
            Map<Integer, Organization> mapById = organizationService.getMapById(Arrays.stream(query.getOrganId()
                    .split(",")).map(Integer::valueOf).collect(Collectors.toList()));
            orgIdMap.putAll(mapById);
            configParam.setOrganId(orgs.getParanValue());
        } else {
            List<Organization> allOrgans = organizationService.findAllOrgans(query.getTenantId());
            Map<Integer, Organization> mapById = allOrgans.stream()
                    .collect(Collectors.toMap(Organization::getId, Function.identity()));
            orgIdMap.putAll(mapById);
            List<String> organId =
                    allOrgans.stream().map(next -> String.valueOf(next.getId())).collect(Collectors.toList());
            configParam.setOrganId(String.join(",", organId));
        }
        List<Double11StaticWrapper.SaleAmountAndUser> result;
        Integer saleStaticsType = query.getSaleStaticsType();
        if (0 == saleStaticsType) { //将不同类型数据汇总
            List<Double11StaticWrapper.SaleAmountAndUser> allResult = new ArrayList<>();
            for (int i = 1; i <= 7; i++) {
                query.setSaleStaticsType(i);
                setQueryParam(query, configParam);
                List<Double11StaticWrapper.SaleAmountAndUser> oneResult = queryOrgSaleAmount(query, orgIdMap);
                allResult.addAll(oneResult);
            }

            List<Double11StaticWrapper.StudentCount> studentNumByOrgan =
                    studentPaymentOrderDao.count2023Double11StudentsByOrgan(configParam);
            Map<Integer, Integer> countByOgranId = studentNumByOrgan.stream()
                    .collect(Collectors.toMap(Double11StaticWrapper.StudentCount::getOrganId, Double11StaticWrapper.StudentCount::getCount));


            Map<Integer, List<Double11StaticWrapper.SaleAmountAndUser>> groupByOrg = allResult.stream()
                    .collect(Collectors.groupingBy(Double11StaticWrapper.SaleAmountAndUser::getOrganId));
            result = groupByOrg.entrySet().stream()
                    .map(entry -> {
                        Double11StaticWrapper.SaleAmountAndUser amount = new Double11StaticWrapper.SaleAmountAndUser();
                        amount.setOrganId(entry.getKey());
                        amount.setOrganName(orgIdMap.getOrDefault(entry.getKey(), new Organization()).getName());
                        for (Double11StaticWrapper.SaleAmountAndUser v : entry.getValue()) {
                            amount.setActualAmount(amount.getActualAmount().add(v.getActualAmount()));
                            amount.setBalancePaymentAmount(amount.getBalancePaymentAmount().add(v.getBalancePaymentAmount()));
                        }
                        amount.setUserPurchaseNumber(countByOgranId.getOrDefault(entry.getKey(),0));
                        return amount;
                    }).collect(Collectors.toList());
        } else {
            setQueryParam(query, configParam);
            result = queryOrgSaleAmount(query, orgIdMap);
        }

        // 排序
        String orderBy = Optional.ofNullable(query.getOrderBy()).orElse("saleAmount");
        int asc = Optional.ofNullable(query.getAsc()).orElse(0);
        if ("saleAmount".equals(orderBy)) {
            result.sort((o1, o2) -> {
                if (1 == asc) {
                    return o1.getSaleAmount().compareTo(o2.getSaleAmount());
                } else {
                    return o2.getSaleAmount().compareTo(o1.getSaleAmount());
                }
            });
        } else if ("userCount".equals(orderBy)) {
            result.sort((o1, o2) -> {
                if (1 == asc) {
                    return o1.getUserPurchaseNumber().compareTo(o2.getUserPurchaseNumber());
                } else {
                    return o2.getUserPurchaseNumber().compareTo(o1.getUserPurchaseNumber());
                }
            });
        }
        return R.from(result);
    }


    @ApiOperation(value = "2023双11销售统计详情", notes = "2023双11销售统计详情")
    @GetMapping("/queryStudentPaymentOrder")
    public R<PageInfo<Double11StaticWrapper.SaleStaticsStudent>> queryStudentPaymentOrder(Double11StaticWrapper.SaleStaticsStudentQuery query) {
        Double11StaticWrapper.SaleStaticsQuery temp = new Double11StaticWrapper.SaleStaticsQuery();
        SysConfig config = sysConfigService.findByParamName("2023_double_eleven_active_config");
        Double11StaticWrapper.SaleStaticConfigParam configParam =
                JSON.parseObject(config.getParanValue(), Double11StaticWrapper.SaleStaticConfigParam.class);
        temp.setSaleStaticsType(query.getSaleStaticsType());
        setQueryParam(temp, configParam);

        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, temp);

        query.setOrderBy("t.pay_time_");
        MapUtil.populateMap(params, query);
        PageInfo<Double11StaticWrapper.SaleStaticsStudent> pageInfo = new PageInfo<>(query.getPage(), query.getRows());
        List<Double11StaticWrapper.SaleStaticsStudent> dataList = new ArrayList<>();
        Integer count = studentPaymentOrderDao.queryPaymentStudentCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentPaymentOrderDao.queryPaymentStudentByPage(params);
            dataList.forEach(next -> {
                String phone = next.getPhone();
                if (StringUtils.isNotEmpty(phone)) {
                    next.setPhone(phone.replaceAll("(\\d{3})\\d*(\\d{4})", "$1****$2"));
                }
            });
        }
        pageInfo.setRows(dataList);
        return R.from(pageInfo);
    }

    @ApiOperation(value = "获取分部列表")
    @GetMapping("/getOrgans")
    public HttpResponseResult<List<Organization>> getOrgans() {
        SysConfig config = sysConfigService.findByParamName("2023_double_eleven_active_config");
        Double11StaticWrapper.SaleStaticConfigParam configParam =
                JSON.parseObject(config.getParanValue(), Double11StaticWrapper.SaleStaticConfigParam.class);
        Integer tenantId = Optional.ofNullable(configParam.getTenantId()).orElse(1);
        List<Organization> allOrgans = organizationService.findAllOrgans(tenantId);
        allOrgans.sort(Comparator.comparing(Organization::getName));
        return succeed(allOrgans);
    }

    private void setQueryParam(Double11StaticWrapper.SaleStaticsQuery query,
                               Double11StaticWrapper.SaleStaticConfigParam configParam) {

        Integer saleStaticsType = query.getSaleStaticsType();
        query.clearValue();
        if (StringUtils.isEmpty(query.getOrganId())) {
            query.setTenantId(Optional.ofNullable(configParam.getTenantId()).orElse(1));
        }

        if (saleStaticsType == 1) {
            // 1V1
            query.setGroupType(GroupType.ACTIVITY);
            query.setType(OrderTypeEnum.SMALL_CLASS_TO_BUY);
            query.setActivityId(configParam.getV1());
        } else if (saleStaticsType == 2) {
            // 1v2
            query.setGroupType(GroupType.ACTIVITY);
            query.setType(OrderTypeEnum.SMALL_CLASS_TO_BUY);
            query.setActivityId(configParam.getV2());
        } else if (saleStaticsType == 3) {
            // 乐理
            query.setGroupType(GroupType.LIVE);
            query.setType(OrderTypeEnum.LIVE_GROUP_BUY);
            query.setMusicGroupId(configParam.getTheoryLive());
        } else if (saleStaticsType == 4) {
            // 云教练
            query.setGroupType(GroupType.ACTIVITY);
            query.setType(OrderTypeEnum.MEMBER);
            query.setActivityId(configParam.getCloudTeacher());
        } else if (saleStaticsType == 5) {
            // 乐器销售
            SysConfig good = sysConfigService.findByParamName("2023_activity_good");
            query.setGoodId(good.getParanValue());

            SysConfig startTime = sysConfigService.findByParamName("2023_activity_start_time");
            query.setStartPayTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(
                    Optional.ofNullable(startTime.getParanValue()).orElse("2023-11-11 00:00:00"),
                    new ParsePosition(0)));

            SysConfig endTime = sysConfigService.findByParamName("2023_activity_end_time");
            query.setEndPayTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(
                    Optional.ofNullable(endTime.getParanValue()).orElse("2023-11-11 23:59:59"),
                    new ParsePosition(0)));
            configParam.setStartPayTime(query.getStartPayTime());
            configParam.setEndPayTime(query.getEndPayTime());
            configParam.setGoodId(good.getParanValue());
        } else if (saleStaticsType == 6) {
            // 保价卡销售
            query.setGroupType(GroupType.SPORADIC);
            query.setType(OrderTypeEnum.SPORADIC);
            query.setMusicGroupId(configParam.getSporadic());
        }else if (saleStaticsType == 7) {
            // 小组课
            query.setGroupType(GroupType.ACTIVITY);
            query.setType(OrderTypeEnum.SMALL_CLASS_TO_BUY);
            query.setActivityId(configParam.getTheory());
        }
    }

    private List<Double11StaticWrapper.SaleAmountAndUser> queryOrgSaleAmount(Double11StaticWrapper.SaleStaticsQuery query,
                                                                             Map<Integer, Organization> orgIdMap) {
        List<Double11StaticWrapper.SaleAmountAndUser> saleAmount = studentPaymentOrderDao.saleStaticsByOrgId(query);
        Map<Integer, Double11StaticWrapper.SaleAmountAndUser> idDataMap = saleAmount.stream()
                .collect(Collectors.toMap(Double11StaticWrapper.SaleAmountAndUser::getOrganId, Function.identity()));

        return orgIdMap.entrySet().stream()
                .map(next -> {
                    if (idDataMap.containsKey(next.getKey())) {
                        Double11StaticWrapper.SaleAmountAndUser saleAmountAndUser = idDataMap.get(next.getKey());
                        saleAmountAndUser.setOrganName(next.getValue().getName());
                        return saleAmountAndUser;
                    }

                    Double11StaticWrapper.SaleAmountAndUser statics = new Double11StaticWrapper.SaleAmountAndUser();
                    statics.setOrganId(next.getKey());
                    statics.setOrganName(next.getValue().getName());
                    statics.setActualAmount(new BigDecimal(0));
                    statics.setBalancePaymentAmount(new BigDecimal(0));
                    statics.setUserPurchaseNumber(0);
                    return statics;
                }).collect(Collectors.toList());
    }

}
