package com.ym.mec.web.controller.open;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.vo.SchoolStaffVo;
import com.ym.mec.biz.dal.wrapper.SchoolStaffWrapper;
import com.ym.mec.biz.service.CooperationOrganService;
import com.ym.mec.biz.service.SchoolStaffService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Objects;
import java.util.Optional;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/open/school")
@Api(tags = "学校开放接口")
public class OpenSchoolController extends BaseController {

    @Autowired
    private SchoolStaffService schoolStaffService;
    @Autowired
    private CooperationOrganService cooperationOrganService;
    @Autowired
    private SysConfigService sysConfigService;


    @ApiOperation(value = "学校员工新增", notes = "学校员工- 传入 SchoolStaffWrapper.SchoolStaff")
	@PostMapping("/staffSave")
	public HttpResponseResult<JSONObject> add(@Validated @RequestBody SchoolStaffVo.SchoolStaff info) {

        // 参数有效性校验
        if (Objects.isNull(info.getUserType())) {
            throw new BizException("领队类型无效");
        }

        // 数据对象转换
        SchoolStaffWrapper.SchoolStaff wrapper = SchoolStaffWrapper.SchoolStaff.from(info.jsonString());
        // 新增数据
        schoolStaffService.add(wrapper);
        
        return succeed();
	}

    @ApiOperation(value = "合作单位查询", notes = "合作单位学校查询- 主键Id")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "合作单位ID", dataType = "long")
    })
    @GetMapping("/cooperation")
    public HttpResponseResult<SchoolStaffWrapper.SchoolCooperation> queryCooperationById(@RequestParam Long id) {

    	CooperationOrgan cooperationOrgan = cooperationOrganService.get(Optional.ofNullable(id).orElse(-1L).intValue());
        if (Objects.isNull(cooperationOrgan)) {
            throw new BizException("合作单位编号无效");
        }

        // 学校员工身份统计
        SchoolStaffWrapper.SchoolStaffStat staffStat = schoolStaffService.getUserTypeStatByCooperationId(cooperationOrgan.getId());

        // 数据对象转换
        SchoolStaffWrapper.SchoolCooperation schoolCooperation = SchoolStaffWrapper.SchoolCooperation
                .builder()
                .cooperation(cooperationOrgan)
                .staffStat(staffStat)
                .build();

        return succeed(schoolCooperation);
    }

    @ApiOperation(value = "查询参数")
    @GetMapping(value = "queryByParamName")
    public Object queryByParamName(String paramName) {
        if(StringUtils.isBlank(paramName)){
            return failed("参数不能为空");
        }
        return succeed(sysConfigService.findByParamName(paramName));
    }

}