package com.ym.mec.web.controller.payment;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yeepay.g3.sdk.yop.encrypt.DigitalEnvelopeDTO;
import com.yeepay.g3.sdk.yop.utils.DigitalEnvelopeUtils;
import com.yeepay.g3.sdk.yop.utils.RSAKeyUtils;
import com.ym.mec.biz.dal.dto.YeepayMember;
import com.ym.mec.biz.dal.entity.HfMember;
import com.ym.mec.biz.dal.enums.PaymentChannelEnum;
import com.ym.mec.biz.dal.page.HfMemberQueryInfo;
import com.ym.mec.biz.service.HfMemberService;
import com.ym.mec.biz.service.HfMerchantConfigService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import com.ym.mec.thirdparty.yeepay.YeepayMerchantService;
import com.ym.mec.thirdparty.yeepay.YeepayUploadService;
import com.ym.mec.thirdparty.yeepay.entity.MerchantInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.Date;

@RequestMapping("${app-config.url.web:}/yeepay")
@Api(tags = "易宝支付服务")
@RestController
public class YeepayController extends BaseController {
    private static final Logger logger = LoggerFactory.getLogger(YeepayController.class);

    @Autowired
    private HfMemberService hfMemberService;
    
    @Autowired
    private HfMerchantConfigService hfMerchantConfigService;
    
    @Autowired
    private YeepayUploadService yeepayUploadService;
    
    @Autowired
    private YeepayMerchantService yeepayMerchantService;

    @Value("${spring.profiles.active:dev}")
    private String profiles;

    @ApiOperation(value = "企业用户列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('yeepay/queryPage')")
    public HttpResponseResult<PageInfo<HfMember>> queryPage(HfMemberQueryInfo queryInfo) {
    	
    	queryInfo.setPayerName(PaymentChannelEnum.YEEPAY.getCode());
        return succeed(hfMemberService.queryPage(queryInfo));
    }

    @ApiOperation(value = "企业用户注册查询")
    @GetMapping("/registerQuery")
    @PreAuthorize("@pcs.hasPermissions('yeepay/registerQuery')")
    public HttpResponseResult<JSONObject> registerQuery(String requestNo) throws IOException {
		
		HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(TenantContextHolder.getTenantId(), PaymentChannelEnum.YEEPAY.getCode());
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}][{}]商户信息找不到", TenantContextHolder.getTenantId(), PaymentChannelEnum.YEEPAY.getCode());
        }
        
        return succeed(yeepayMerchantService.registerQueryV2(hfMerchantConfig, requestNo));
    }

    @ApiOperation("新建企业用户")
    @PostMapping(value = "addSubMerchant")
    @PreAuthorize("@pcs.hasPermissions('yeepay/addSubMerchant')")
    public HttpResponseResult<Boolean> addSubMerchant(YeepayMember member) throws Exception {
		/*
		 * if (!profiles.equals("prod")) { return failed("仅生产环境可用"); }
		 */
        
        member.setPayerName(PaymentChannelEnum.YEEPAY.getCode());
        
        return succeed(hfMemberService.createMember(member, null));
    }

    @ApiOperation("修改企业用户(状态是failed才能修改)")
    @PostMapping(value = "updateSubMerchant")
    @PreAuthorize("@pcs.hasPermissions('yeepay/updateSubMerchant')")
    public HttpResponseResult<Boolean> updateSubMerchant(MerchantInfo merchantInfo) throws Exception {

    	return null;
    }

    @PostMapping("/callback/{requestNo}")
	public String callback(@PathVariable("requestNo") String requestNo, HttpServletRequest request) {
    	
    	HfMember hfMember = hfMemberService.queryByRequestNo(requestNo);
		if(hfMember == null){
			return "请求编号["+ requestNo +"]在系统中不存在";
		}
		hfMember.setUpdateTime(new Date());

		HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(hfMember.getTenantId(), PaymentChannelEnum.YEEPAY.getCode());
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}][{}]商户信息找不到", hfMember.getTenantId(), PaymentChannelEnum.YEEPAY.getCode());
        }
        
		try {

			String content = request.getParameter("response");
			
			// 构造结果通知请求对象
			DigitalEnvelopeDTO dto = new DigitalEnvelopeDTO();
			dto.setCipherText(content);
			PrivateKey privateKey =RSAKeyUtils.string2PrivateKey(hfMerchantConfig.getRsaPrivateKey());
			PublicKey publicKey = RSAKeyUtils.string2PublicKey(hfMerchantConfig.getRsaPublicKey());
			
			dto = DigitalEnvelopeUtils.decrypt(dto, privateKey, publicKey);
			
			logger.info("易宝支付回调信息：response:{} plaintText:{}", content, dto.getPlainText());
			
			JSONObject dataObj = JSON.parseObject(dto.getPlainText());
			
			if("COMPLETED".equals(dataObj.getString("applicationStatus"))) {
				//审核成功
				hfMember.setStatus("succeeded");
				hfMemberService.update(hfMember);
			}else if("REVIEW_BACK".equals(dataObj.getString("applicationStatus"))) {
				//审核驳回
				hfMember.setStatus("failed");
				hfMember.setMemo(dataObj.getString("auditOpinion"));
				hfMemberService.update(hfMember);
			}
			
			return "SUCCESS";
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
    }

    @ApiOperation("修改企业用户(状态是failed才能修改)")
    @PostMapping(value = "upload")
    @PreAuthorize("@pcs.hasPermissions('yeepay/upload')")
    public HttpResponseResult<String> upload(MultipartFile multipartFile) throws Exception {
    	

        File file = new File("/var/tmp/" + multipartFile.getOriginalFilename());
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        FileOutputStream fos = new FileOutputStream(file);
        InputStream inputStream = multipartFile.getInputStream();

        try {
            IOUtils.copy(inputStream, fos);
            inputStream.close();
            fos.close();
            
            HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(TenantContextHolder.getTenantId(), PaymentChannelEnum.YEEPAY.getCode());
            if(hfMerchantConfig == null){
            	throw new BizException("请配置机构的易宝商户信息");
            }
        	
            return succeed(yeepayUploadService.upload(hfMerchantConfig, file));
        } catch (Exception e) {
            return failed(e.getMessage());
        } finally {
        	IOUtils.closeQuietly(fos);
            IOUtils.closeQuietly(inputStream);
            FileUtils.deleteQuietly(file);
        }

    }

}
