package com.ym.mec.web.controller.school;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.config.validator.group.ValidGroups;
import com.ym.mec.biz.dal.wrapper.CoursePatrolEvaluationWrapper;
import com.ym.mec.biz.service.CoursePatrolEvaluationService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/coursePatrolEvaluation")
@Api(tags = "巡堂详情")
public class CoursePatrolEvaluationController extends BaseController {

    @Autowired
    private CoursePatrolEvaluationService coursePatrolEvaluationService;

    @Autowired
    private OrganizationService organizationService;
	@ApiOperation(value = "web-详情", notes = "巡堂详情-根据详情ID查询单条, 传入id")
    @GetMapping("/detail/{id}")
    public HttpResponseResult<CoursePatrolEvaluationWrapper.CoursePatrolEvaluationInfo> detail(@PathVariable("id") Long id) {

        CoursePatrolEvaluationWrapper.CoursePatrolEvaluationInfo wrapper = coursePatrolEvaluationService.detail(id);
        
        return succeed(wrapper);
	}

    @ApiOperation(value = "学校端-查询巡堂统计分页", notes = "巡堂详情- 传入 CoursePatrolEvaluationWrapper.CoursePatrolEvaluationQuery")
    @PostMapping("/pageStat")
    public HttpResponseResult<PageInfo<CoursePatrolEvaluationWrapper.CoursePatrolEvaluationStat>> pageStat(@RequestBody CoursePatrolEvaluationWrapper.CoursePatrolEvaluationQuery query, @RequestHeader Integer coopId) {
        if (coopId == null) {
            return succeed();
        }
        query.setCooperationOrganId(coopId);

        IPage<CoursePatrolEvaluationWrapper.CoursePatrolEvaluationStat> page = coursePatrolEvaluationService.pageStat(QueryInfo.getPage(query), query);
        return succeed(PageUtil.pageInfo(page));
    }


    @ApiOperation(value = "web-查询巡堂分页", notes = "巡堂详情- 传入 CoursePatrolEvaluationWrapper.CoursePatrolEvaluationQuery")
    @PostMapping("/webPage")
    public HttpResponseResult<PageInfo<CoursePatrolEvaluationWrapper.CoursePatrolEvaluationInfo>> webPage(@RequestBody CoursePatrolEvaluationWrapper.CoursePatrolEvaluationQuery query) {

        query.setOrganId(organizationService.getEmployeeOrgan(query.getOrganId()));
        query.setPatrolFlag(true);
        IPage<CoursePatrolEvaluationWrapper.CoursePatrolEvaluationInfo> pages = coursePatrolEvaluationService.selectPage(QueryInfo.getPage(query), query);

        return succeed(PageUtil.pageInfo(pages));
    }

    @ApiOperation(value = "学校端-查询巡堂分页", notes = "巡堂详情- 传入 CoursePatrolEvaluationWrapper.CoursePatrolEvaluationQuery")
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<CoursePatrolEvaluationWrapper.CoursePatrolEvaluationInfo>> page(@RequestBody CoursePatrolEvaluationWrapper.CoursePatrolEvaluationQuery query,@RequestHeader Integer coopId) {
        if (coopId == null) {
            return succeed();
        }
        query.setCooperationOrganId(coopId);
        IPage<CoursePatrolEvaluationWrapper.CoursePatrolEvaluationInfo> pages = coursePatrolEvaluationService.selectPage(QueryInfo.getPage(query), query);
        
        return succeed(PageUtil.pageInfo(pages));
	}
    
    @ApiOperation(value = "学校端-添加巡堂信息", notes = "巡堂详情- 传入 CoursePatrolEvaluationWrapper.CoursePatrolEvaluation")
	@PostMapping("/save")
	public HttpResponseResult<JSONObject> add(@Validated(value = ValidGroups.Add.class) @RequestBody CoursePatrolEvaluationWrapper.CoursePatrolEvaluation coursePatrolEvaluation,@RequestHeader Integer coopId) {
        if (coopId == null) {
            return succeed();
        }
        coursePatrolEvaluation.setCooperationOrganId(coopId);
        // 新增数据
        coursePatrolEvaluationService.add(coursePatrolEvaluation);
        
        return succeed();
	}
    
    @ApiOperation(value = "web-处理意见，学校端-修改", notes = "巡堂详情- 传入 CoursePatrolEvaluationWrapper.CoursePatrolEvaluation")
	@PostMapping("/update")
	public HttpResponseResult<JSONObject> update(@Validated(value = ValidGroups.Update.class) @RequestBody CoursePatrolEvaluationWrapper.CoursePatrolEvaluation coursePatrolEvaluation) {
        
        // 更新数据
        coursePatrolEvaluationService.update(coursePatrolEvaluation);
        
        return succeed();
	}
}