package com.ym.mec.web.controller.school;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.config.validator.group.ValidGroups;
import com.ym.mec.biz.dal.wrapper.SchoolActivityWrapper;
import com.ym.mec.biz.service.SchoolActivityDetailUserService;
import com.ym.mec.biz.service.SchoolActivityService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/schoolActivity")
@Api(tags = "学校活动")
public class SchoolActivityController extends BaseController {

    @Autowired
    private SchoolActivityService schoolActivityService;

    @Autowired
    private SchoolActivityDetailUserService schoolActivityDetailUserService;

	@ApiOperation(value = "详情", notes = "学校活动-根据详情ID查询单条, 传入id")
    @GetMapping("/detail/{id}")
    public HttpResponseResult<SchoolActivityWrapper.SchoolActivityInfo> detail(@PathVariable("id") Long id) {

        SchoolActivityWrapper.SchoolActivityInfo wrapper = schoolActivityService.detail(id);
        
        return succeed(wrapper);
	}
    
    @ApiOperation(value = "查询分页", notes = "学校活动- 传入 SchoolActivityWrapper.SchoolActivityQuery") 
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<SchoolActivityWrapper.SchoolActivity>> page(@RequestBody SchoolActivityWrapper.SchoolActivityQuery query, @RequestHeader Integer coopId) {

        if (coopId == null) {
            return succeed();
        }
        query.setCooperationOrganId(coopId);
        IPage<SchoolActivityWrapper.SchoolActivity> pages = schoolActivityService.selectPage(QueryInfo.getPage(query), query);
        
        return succeed(PageUtil.pageInfo(pages));
	}


    @ApiOperation(value = "查询乐团学生信息")
    @PostMapping("/userList")
    public HttpResponseResult<List<SchoolActivityWrapper.SchoolActivitySubject>> userList(@RequestBody SchoolActivityWrapper.SchoolActivityStudentQuery query) {

        return succeed(schoolActivityDetailUserService.getUserList(query));
    }
    
    @ApiOperation(value = "新增", notes = "学校活动- 传入 SchoolActivityWrapper.SchoolActivity")
	@PostMapping("/save")
	public HttpResponseResult<Boolean> add(@Validated @RequestBody SchoolActivityWrapper.SchoolActivityInfo schoolActivity, @RequestHeader Integer coopId) {
        if (coopId == null) {
            return succeed();
        }
        schoolActivity.setCooperationOrganId(coopId);
        
        return succeed(schoolActivityService.add(schoolActivity));
	}
    
    @ApiOperation(value = "修改", notes = "学校活动- 传入 SchoolActivityWrapper.SchoolActivity")
	@PostMapping("/update")
	public HttpResponseResult<Boolean> update(@Validated(value = ValidGroups.Update.class) @RequestBody SchoolActivityWrapper.SchoolActivityInfo schoolActivity, @RequestHeader Integer coopId) {
        if (coopId == null) {
            return succeed();
        }
        schoolActivity.setCooperationOrganId(coopId);

        return succeed(schoolActivityService.update(schoolActivity));
	}
}