package com.ym.mec.web.controller.school;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.SchoolActivityDetail;
import com.ym.mec.biz.dal.wrapper.SchoolActivityDetailWrapper;
import com.ym.mec.biz.service.SchoolActivityDetailService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/schoolActivityDetail")
@Api(tags = "活动详情")
public class SchoolActivityDetailController extends BaseController {

    @Autowired
    private SchoolActivityDetailService schoolActivityDetailService;

	@ApiOperation(value = "详情", notes = "活动详情-根据详情ID查询单条, 传入id")
    @GetMapping("/detail/{id}")
    public HttpResponseResult<SchoolActivityDetail> detail(@PathVariable("id") Long id) {
    
    	SchoolActivityDetail wrapper = schoolActivityDetailService.detail(id);
        
        return succeed(wrapper);
	}
    
    @ApiOperation(value = "查询分页", notes = "活动详情- 传入 SchoolActivityDetailWrapper.SchoolActivityDetailQuery") 
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<SchoolActivityDetail>> page(@RequestBody SchoolActivityDetailWrapper.SchoolActivityDetailQuery query) {
        
        IPage<SchoolActivityDetail> pages = schoolActivityDetailService.selectPage(QueryInfo.getPage(query), query);
        
        return succeed(PageUtil.pageInfo(pages));
	}
    
    @ApiOperation(value = "新增", notes = "活动详情- 传入 SchoolActivityDetailWrapper.SchoolActivityDetail")
	@PostMapping("/save")
	public HttpResponseResult<JSONObject> add(@Validated @RequestBody SchoolActivityDetail schoolActivityDetail) {
        
        // 新增数据
        schoolActivityDetailService.save(schoolActivityDetail);
        
        return succeed();
	}
    
    @ApiOperation(value = "修改", notes = "活动详情- 传入 SchoolActivityDetailWrapper.SchoolActivityDetail")
	@PostMapping("/update")
	public HttpResponseResult<JSONObject> update(@Validated @RequestBody SchoolActivityDetail schoolActivityDetail) {
        
        // 更新数据
        schoolActivityDetailService.updateById(schoolActivityDetail);
        
        return succeed();
	}

	@ApiOperation(value = "删除", notes = "活动详情- 传入id")
	@PostMapping("/remove")
	public HttpResponseResult<Boolean> remove(@RequestParam Long id) {
    
		return succeed(schoolActivityDetailService.removeById(id));
	}
}