package com.ym.mec.web.controller.school;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.SchoolActivityDetailUser;
import com.ym.mec.biz.dal.wrapper.SchoolActivityDetailUserWrapper;
import com.ym.mec.biz.service.SchoolActivityDetailUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/schoolActivityDetailUser")
@Api(tags = "活动详情人员")
public class SchoolActivityDetailUserController extends BaseController {

    @Autowired
    private SchoolActivityDetailUserService schoolActivityDetailUserService;

	@ApiOperation(value = "详情", notes = "活动详情人员-根据详情ID查询单条, 传入id")
    @GetMapping("/detail/{id}")
    public HttpResponseResult<SchoolActivityDetailUser> detail(@PathVariable("id") Long id) {
    
    	SchoolActivityDetailUser wrapper = schoolActivityDetailUserService.detail(id);
        
        return succeed(wrapper);
	}
    
    @ApiOperation(value = "查询分页", notes = "活动详情人员- 传入 SchoolActivityDetailUserWrapper.SchoolActivityDetailUserQuery") 
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<SchoolActivityDetailUser>> page(@RequestBody SchoolActivityDetailUserWrapper.SchoolActivityDetailUserQuery query) {
        
        IPage<SchoolActivityDetailUser> pages = schoolActivityDetailUserService.selectPage(QueryInfo.getPage(query), query);
        
        return succeed(PageUtil.pageInfo(pages));
	}
    
    @ApiOperation(value = "新增", notes = "活动详情人员- 传入 SchoolActivityDetailUserWrapper.SchoolActivityDetailUser")
	@PostMapping("/save")
	public HttpResponseResult<JSONObject> add(@Validated @RequestBody SchoolActivityDetailUser schoolActivityDetailUser) {
        
        // 新增数据
        schoolActivityDetailUserService.save(schoolActivityDetailUser);
        
        return succeed();
	}
    
    @ApiOperation(value = "修改", notes = "活动详情人员- 传入 SchoolActivityDetailUserWrapper.SchoolActivityDetailUser")
	@PostMapping("/update")
	public HttpResponseResult<JSONObject> update(@Validated @RequestBody SchoolActivityDetailUser schoolActivityDetailUser) {
        
        // 更新数据
        schoolActivityDetailUserService.updateById(schoolActivityDetailUser);
        
        return succeed();
	}

	@ApiOperation(value = "删除", notes = "活动详情人员- 传入id")
	@PostMapping("/remove")
	public HttpResponseResult<Boolean> remove(@RequestParam Long id) {
    
		return succeed(schoolActivityDetailUserService.removeById(id));
	}
}