package com.ym.mec.web.controller.school;

import com.ym.mec.biz.dal.school.dto.ClassesForDayDto;
import com.ym.mec.biz.dal.school.dto.CourseStudentDto;
import com.ym.mec.biz.dal.wrapper.DailySummaryOfClassesForTheCurrentSemesterWrapper;
import com.ym.mec.biz.service.CourseScheduleService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("${app-config.url.web:}/schoolCourseSchedule")
@Api(tags = "学校端课程计划服务")
public class SchoolCourseScheduleController extends BaseController {

    @Autowired
    private CourseScheduleService courseScheduleService;

    @ApiOperation(value = "课时顶部统计接口---获取当前学期课程汇总")
    @GetMapping("/getSummaryOfCurrentSemesterCourses")
    public HttpResponseResult<List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto>> getSummaryOfCurrentSemesterCourses(@RequestHeader Integer coopId) {
        return succeed(courseScheduleService.getSummaryOfCurrentSemesterCourses(coopId));
    }

    @ApiOperation(value = "课时每周分组数据接口----获取当前学期每天课程汇总")
    @GetMapping("/getDailySummaryOfClassesForTheCurrentSemester")
    public HttpResponseResult<List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterMapDto>> getDailySummaryOfClassesForTheCurrentSemester(@RequestHeader Integer coopId) {
        return succeed(courseScheduleService.getDailySummaryOfClassesForTheCurrentSemester(coopId));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "classDate", dataType = "String", required = true, value = "上课时间（年月日）"),
    })
    @ApiOperation(value = "课时每天课程数据详情----获取当天课程列表")
    @GetMapping("/queryListClassesForDay")
    public HttpResponseResult<List<ClassesForDayDto>> queryListClassesForDay(String classDate,@RequestHeader Integer coopId) {
        return succeed(courseScheduleService.getListClassesForDay(coopId,classDate));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "courseId", dataType = "Long", required = true, value = "课程编号"),
    })
    @ApiOperation(value = "课时获取单个的课程数据详情----获取课程详情")
    @GetMapping("/getCourseDetail")
    public HttpResponseResult<ClassesForDayDto> getCourseDetail(Long courseId) {
        return succeed(courseScheduleService.getCourseDetail(courseId));
    }

    @ApiImplicitParams({
            @ApiImplicitParam(name = "courseId", dataType = "Long", required = true, value = "课程编号"),
    })
    @ApiOperation(value = "课时获取单节课学员名单--获取课程学员列表")
    @GetMapping("/queryCourseStudentList")
    public HttpResponseResult<List<CourseStudentDto>> queryCourseStudentList(Long courseId) {
        return succeed(courseScheduleService.queryCourseStudentList(courseId));
    }
}
