package com.ym.mec.web.controller.school;

import com.ym.mec.biz.dal.entity.ApprovalStatus;
import com.ym.mec.biz.dal.entity.MusicGroupQuit;
import com.ym.mec.biz.dal.school.dto.MusicGroupQuitDetailDto;
import com.ym.mec.biz.dal.school.dto.MusicGroupQuitPageDto;
import com.ym.mec.biz.dal.school.page.MusicGroupQuitQueryInfo;
import com.ym.mec.biz.service.MusicGroupQuitService;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/schoolMusicGroupQuit")
@Api(tags = "学校学员退团服务")
public class SchoolMusicGroupQuitController extends BaseController {

    @Autowired
    private MusicGroupQuitService musicGroupQuitService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private MusicGroupService musicGroupService;

	@ApiOperation(value = "分页查询")
    @PostMapping("/queryPage")
    public HttpResponseResult<PageInfo<MusicGroupQuitPageDto>> queryPage(@RequestBody MusicGroupQuitQueryInfo queryInfo,@RequestHeader Integer coopId) {
        queryInfo.setCoopId(coopId);
        return succeed(musicGroupQuitService.queryPageBySchool(queryInfo));
	}

    @ApiOperation(value = "申请退团")
    @PostMapping("/quitMusicGroup")
    @ApiImplicitParams({@ApiImplicitParam(name = "musicGroupId", value = "乐团id", required = true, dataType = "String"),
            @ApiImplicitParam(name = "reason", value = "原因", required = true, dataType = "String"),
            @ApiImplicitParam(name = "reasonEnum", value = "原因", required = true, dataType = "String")})
    public HttpResponseResult quitMusicGroup(String musicGroupId,Integer userId, String reason, MusicGroupQuit.ReasonEnum reasonEnum) throws Exception {
        Integer operatorId = sysUserService.getUserId();
        List<MusicGroupQuit> musicGroupQuits = new ArrayList<>();
        for (String groupId : musicGroupId.split(",")) {
            MusicGroupQuit musicGroupQuit = new MusicGroupQuit();
            musicGroupQuit.setUserId(userId);
            musicGroupQuit.setMusicGroupId(groupId);
            musicGroupQuit.setUserComment(reason);
            musicGroupQuit.setReasonEnum(reasonEnum);
            musicGroupQuit.setStatus(ApprovalStatus.PROCESSING);
            musicGroupQuit.setOperatorId(operatorId);
            musicGroupQuits.add(musicGroupQuit);
        }
        musicGroupService.batchQuitMusicGroup(musicGroupQuits);
        return succeed();
    }

	@ApiOperation(value = "获取退团详情")
    @GetMapping("/getDetail")
    public HttpResponseResult<MusicGroupQuitDetailDto> getDetail(Long musicGroupQuitId) {
        return succeed(musicGroupQuitService.getDetailBySchool(musicGroupQuitId));
	}

	@ApiOperation(value = "退团审批通过")
    @GetMapping("/auditPass")
    public HttpResponseResult auditPass(Long musicGroupQuitId,String reason) {
        musicGroupQuitService.auditPassBySchool(musicGroupQuitId,reason);
        return succeed();
	}

	@ApiOperation(value = "退团审批拒绝")
    @GetMapping("/auditReject")
    public HttpResponseResult auditReject(Long musicGroupQuitId,String reason) {
        musicGroupQuitService.auditRejectBySchool(musicGroupQuitId,reason);
        return succeed();
	}
}