package com.ym.mec.web.controller.school;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.entity.SchoolStaff;
import com.ym.mec.biz.dal.enums.ParamEnum;
import com.ym.mec.biz.dal.enums.im.ClientEnum;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;
import com.ym.mec.biz.dal.wrapper.SchoolStaffWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Objects;

@Slf4j
@Validated
@RestController
@RequestMapping("${app-config.url.web:}/schoolStaff")
@Api(tags = "学校员工")
public class SchoolStaffController extends BaseController {

    @Autowired
    private SchoolStaffService schoolStaffService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private CooperationOrganService cooperationOrganService;

    @Autowired
    private OrganizationService organizationService;

    @Autowired
    private EmployeeService employeeService;

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private StudentManageService studentManageService;
    @Autowired
    private ImGroupCoreService imGroupCoreService;

	@ApiOperation(value = "详情", notes = "学校员工-根据详情ID查询单条, 传入id")
    @PreAuthorize("@pcs.hasPermissions('schoolStaff/detail')")
    @GetMapping("/detail/{id}")
    public HttpResponseResult<SchoolStaffWrapper.SchoolStaff> detail(@PathVariable("id") Long id) {
    

        return succeed(schoolStaffService.detail(id));
	}



    @ApiOperation(value = "修改用户")
    @PostMapping("/updateSchoolUser")
    public HttpResponseResult updateSchoolUser(SysUser sysUser) {
        SysUser user = sysUserService.getUser();
        Date date = new Date();
        sysUser.setId(user.getId());
        sysUser.setUpdateTime(date);
        studentManageService.updateSchoolUser(sysUser);
        return succeed();
    }
    
    @ApiOperation(value = "查询分页", notes = "学校员工- 传入 SchoolStaffWrapper.SchoolStaffQuery") 
    @PreAuthorize("@pcs.hasPermissions('schoolStaff/page')")
    @PostMapping("/page")
    public HttpResponseResult<PageInfo<SchoolStaffWrapper.SchoolStaff>> page(@RequestBody SchoolStaffWrapper.SchoolStaffQuery query) {
        query.setOrganId(organizationService.getEmployeeOrgan(query.getOrganId()));
        query.setTenantId(TenantContextHolder.getTenantId());
//        query.setCoopStatus(true);

        IPage<SchoolStaffWrapper.SchoolStaff> pages = schoolStaffService.selectPage(QueryInfo.getPage(query), query);
        
        return succeed(PageUtil.pageInfo(pages));
	}

    @ApiOperation(value = "员工排序信息", notes = "按身份分组，返回所有员工信息")
    @GetMapping("/getAllStaffs")
    public HttpResponseResult<List<SchoolStaffWrapper.SchoolStaff>> getAllStaffs() {

        // 查询用户信息
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (Objects.isNull(sysUser)) {
            throw new BizException("请先登录");
        }

        List<SchoolStaffWrapper.SchoolStaff> list = schoolStaffService.getAllStaffsBySchoolId(Long.valueOf(sysUser.getId()));

        return succeed(list);
    }
    
    @ApiOperation(value = "新增", notes = "学校员工- 传入 SchoolStaffWrapper.SchoolStaff")
    @PreAuthorize("@pcs.hasPermissions('schoolStaff/save')")
	@PostMapping("/save")
	public HttpResponseResult<JSONObject> add(@Validated @RequestBody SchoolStaff schoolStaff) {
        
        // 新增数据
        schoolStaffService.save(schoolStaff);
        
        return succeed();
	}
    
    @ApiOperation(value = "修改", notes = "学校员工- 传入 SchoolStaffWrapper.SchoolStaff")
    @PreAuthorize("@pcs.hasPermissions('schoolStaff/update')")
	@PostMapping("/update")
	public HttpResponseResult<JSONObject> update(@Validated @RequestBody SchoolStaff schoolStaff) {
        
        // 更新数据
        schoolStaffService.updateById(schoolStaff);
        
        return succeed();
	}

	@ApiOperation(value = "删除", notes = "学校员工- 传入id")
    @PreAuthorize("@pcs.hasPermissions('schoolStaff/remove')")
	@PostMapping("/remove")    
	public HttpResponseResult<Boolean> remove(@RequestParam Long id) {
    
		return succeed(schoolStaffService.del(id));
	}

    @ApiOperation(value = "查询学校员工信息")
    @GetMapping("/queryUserInfo")
    public HttpResponseResult<SchoolStaffWrapper.StaffSysUser> queryStaffSysUser() {

        // 查询用户信息
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (Objects.isNull(sysUser)) {
            throw new BizException("请先登录");
        }

        SchoolStaffWrapper.StaffSysUser user = SchoolStaffWrapper.StaffSysUser.from(JSON.toJSONString(sysUser));
        // 查询学校员工信息
        SchoolStaff schoolStaff = schoolStaffService.queryByUserId(Long.valueOf(sysUser.getId()));
        if (Objects.isNull(schoolStaff)) {
            throw new BizException("员工账号不存在");
        }

        // 设置数据
        user.schoolId(schoolStaff.getSchoolId())
                .userId(schoolStaff.getUserId())
                .schoolUserType(schoolStaff.getUserType())
                .organIdList(schoolStaff.getOrganIdList())
                .tenantId(schoolStaff.getTenantId())
                .setResetFlag(false);

        // 默认初始密钥
        int phoneStrLen = sysUser.getPhone().length();
        String defaultPassword = "gym" + sysUser.getPhone().substring(phoneStrLen - 4, phoneStrLen);
        // 密码重置标识
        user.setResetFlag(false);

        String password = sysUser.getPassword();
        if (StringUtils.isNotBlank(password)
                && (new BCryptPasswordEncoder().matches(ParamEnum.INIT_PASSWORD.getCode().toString(), password)
                || new BCryptPasswordEncoder().matches(defaultPassword, password))) {
            user.setResetFlag(true);
        }

        // 学校信息
        CooperationOrgan cooperation = cooperationOrganService.get(schoolStaff.getSchoolId().intValue());
        if (Objects.isNull(cooperation)) {
            throw new BizException("合作单位无效");
        }
        user.schoolName(cooperation.getName()).logo(cooperation.getLogo());

        // 注册IM用户Token
        try {
            ImGroupWrapper.ImUserInfo register = imGroupCoreService.getUserSig(String.valueOf(sysUser.getId()),
                ClientEnum.SCHOOL.getCode());

            // 设置用户Token
            user.setImToken(register.getImToken());
            // 设置用户IM用户ID
            user.setImUserId(register.getImUserId());
        } catch (Exception e) {
            log.error("注册IM用户Token失败", e);
        }

        return succeed(user);
    }

    @ApiOperation(value = "合作单位查询", notes = "学校合作单位查询")
    @GetMapping("/cooperation")
    public HttpResponseResult<SchoolStaffWrapper.SchoolCooperation> queryCooperation() {

        // 查询用户信息
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (Objects.isNull(sysUser)) {
            throw new BizException("请先登录");
        }

        // 查询学校员工信息
        SchoolStaff schoolStaff = schoolStaffService.queryByUserId(Long.valueOf(sysUser.getId()));
        if (Objects.isNull(schoolStaff)) {
            throw new BizException("员工账号不存在");
        }

        CooperationOrgan cooperationOrgan = cooperationOrganService.get(schoolStaff.getSchoolId().intValue());
        if (Objects.isNull(cooperationOrgan)) {
            throw new BizException("合作单位编号无效");
        }

        // 学校员工身份统计
        SchoolStaffWrapper.SchoolStaffStat staffStat = schoolStaffService.getUserTypeStatByCooperationId(cooperationOrgan.getId());

        // 数据对象转换
        SchoolStaffWrapper.SchoolCooperation schoolCooperation = SchoolStaffWrapper.SchoolCooperation
                .builder()
                .cooperation(cooperationOrgan)
                .staffStat(staffStat)
                .build();

        return succeed(schoolCooperation);
    }

    @ApiOperation(value = "更新合作单位", notes = "学校合作单位查询")
    @GetMapping("/updateCooperation")
    public HttpResponseResult<Boolean> updateCooperation(@RequestBody SchoolStaffWrapper.Cooperation info) {

        // 查询用户信息
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (Objects.isNull(sysUser)) {
            throw new BizException("请先登录");
        }

        // 查询学校员工信息
        SchoolStaff schoolStaff = schoolStaffService.queryByUserId(Long.valueOf(sysUser.getId()));
        if (Objects.isNull(schoolStaff)) {
            throw new BizException("员工账号不存在");
        }

        // 设置默认参数
        info.tenantId(schoolStaff.getTenantId())
                .setId(schoolStaff.getSchoolId().intValue());

        // 更新合作单位信息
        schoolStaffService.updateCooperation(info);

        return succeed();
    }


    @ApiOperation(value = "实名认证")
    @PostMapping("/realNameAuthentication")
    @ApiImplicitParams({ @ApiImplicitParam(name = "realName", value = "姓名", required = true, dataType = "String"),
            @ApiImplicitParam(name = "idcardNo", value = "身份证号码", required = true, dataType = "String")})
    public Object realNameAuthentication(String realName, String idcardNo) {

        if (StringUtils.isBlank(realName)) {
            throw new BizException("姓名不能为空");
        }
        if (StringUtils.isBlank(idcardNo)) {
            throw new BizException("身份证号不能为空");
        }
        schoolStaffService.realNameAuthentication(sysUserFeignService.queryUserInfo(),realName, idcardNo);
        return succeed();
    }

    @ApiOperation(value = "密码重置")
    @PostMapping("/resetPassword")
    public HttpResponseResult resetPassword(Integer userId){
        employeeService.resetPassword(userId);
        return succeed();
    }



    @ApiOperation(value = "账户修改")
    @PostMapping("/updateAccount")
    public HttpResponseResult updateAccount(@Validated @RequestBody SchoolStaffWrapper.SchoolStaffUpdate schoolStaffUpdate){
        schoolStaffService.updateAccount(schoolStaffUpdate);
        return succeed();
    }
}