package com.ym.mec.web.controller.school;


import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.StudentAttendance;
import com.ym.mec.biz.dal.entity.StudentCourseHomework;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.enums.EHomeWorkStatus;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.page.LessonExaminationQueryInfo;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryDto;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Api(tags = "学校学员课后作业")
@RestController
@RequestMapping("${app-config.url.web:}/schoolStudentHomework")
public class SchoolStudentHomeworkController extends BaseController {

    @Resource
    private StudentLessonExaminationService studentLessonExaminationService;
    @Resource
    private StudentLessonExaminationDetailService studentLessonExaminationDetailService;
    @Resource
    private StudentCourseHomeworkService studentCourseHomeworkService;
    @Resource
    private StudentExtracurricularExercisesSituationService studentExtracurricularExercisesSituationService;
    @Resource
    private LessonExaminationService lessonExaminationService;
    @Resource
    private StudentAttendanceService studentAttendanceService;
    @Resource
    private MusicGroupDao musicGroupDao;
    @Resource
    private SubjectService subjectService;
    @Resource
    private CourseHomeworkService courseHomeworkService;
    @ApiOperation(value = "获取教师有服务指标的课程列表")
    @PostMapping("/queryHomework")
    public HttpResponseResult<PageInfo<CourseHomeworkWrapper.CourseHomeworkList>>
    queryHomework(@RequestBody CourseHomeworkWrapper.CourseHomeworkQuery queryInfo,@RequestHeader Integer coopId) {
        //获取合作单位关联的所有乐团列表
        List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
        if (CollectionUtils.isEmpty(musicGroupIds)) {
            return succeed(new PageInfo<>());
        }
        queryInfo.setMusicGroupIds(musicGroupIds);
        if(queryInfo.getHomeWorkStatus() == null){
            queryInfo.setHomeWorkStatus(EHomeWorkStatus.ASSIGNED);
        }
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherServeHomeworkDetailV2(queryInfo));
    }

    @ApiOperation(value = "获取课外作业列表")
    @PostMapping("/queryExtraHomework")
    public HttpResponseResult<PageInfo<CourseHomeworkWrapper.CourseHomeworkList>>
    queryExtraHomework(@RequestBody CourseHomeworkWrapper.CourseHomeworkQuery queryInfo,@RequestHeader Integer coopId) {
        //获取合作单位关联的所有乐团列表
        List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
        if (CollectionUtils.isEmpty(musicGroupIds)) {
            return succeed(new PageInfo<>());
        }
        queryInfo.setMusicGroupIds(musicGroupIds);
        return succeed(studentExtracurricularExercisesSituationService.queryTeacherExtraHomeworkDetailV2(queryInfo));
    }

    @ApiOperation(value = "获取进度评测列表")
    @PostMapping("queryLessonExamination")
    public HttpResponseResult<PageInfo<LessonExaminationResultDto>>
    queryLessonExamination(@RequestBody LessonExaminationQueryInfo queryInfo, @RequestHeader Integer coopId) {
        //获取合作单位关联的所有乐团列表
        List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
        if (CollectionUtils.isEmpty(musicGroupIds)) {
            return succeed(new PageInfo<>());
        }
        queryInfo.setMusicGroupIds(musicGroupIds);
        return succeed(lessonExaminationService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取单个进度评测")
    @GetMapping("getOneLessonExamination")
    public HttpResponseResult<LessonExaminationResultDto> queryPage(Long lessonExaminationId) {
        LessonExaminationQueryInfo queryInfo = new LessonExaminationQueryInfo();
        queryInfo.setLessonExaminationId(lessonExaminationId);
        LessonExaminationResultDto resultDto = lessonExaminationService.queryPage(queryInfo).getRows().get(0);
        return succeed(resultDto);
    }

    @ApiOperation(value = "获取进度评测学员列表")
    @PostMapping("queryLessonExaminationPage")
    public HttpResponseResult<PageInfo<StudentLessonExaminationDto>> queryLessonExaminationPage(@RequestBody StudentLessonExaminationQueryInfo queryInfo) {
        return succeed(studentLessonExaminationService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取学员评测详情列表")
    @PostMapping("queryStudentLessonExaminationDetail")
    public HttpResponseResult<List<StudentLessonExaminationDetailDto>> queryStudentLessonExaminationDetail(@RequestBody StudentLessonExaminationQueryDto query) {
        return succeed(studentLessonExaminationDetailService.queryAll(query));
    }

    @ApiOperation(value = "学员请假列表")
    @PostMapping("queryStudentLeave")
    public HttpResponseResult<PageInfo<CourseHomeworkWrapper.StudentLeaveResult>>
    queryStudentLeave(@RequestBody CourseHomeworkWrapper.StudentLeaveQuery queryInfo, @RequestHeader Integer coopId) {
        if(StringUtils.isEmpty(queryInfo.getMusicGroupIds())){
            List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
            if (CollectionUtils.isEmpty(musicGroupIds)) {
                return succeed(new PageInfo<>());
            }
            queryInfo.setMusicGroupIds(musicGroupIds.stream().collect(Collectors.joining(",")));
        }
        return succeed(studentAttendanceService.queryStudentLeave(queryInfo));
    }

    @ApiOperation(value = "获取课后评测学员列表")
    @PostMapping("queryLessonStudent")
    public HttpResponseResult<PageInfo<StudentLessonExaminationDto>> queryLessonStudent(@RequestBody StudentLessonExaminationQueryInfo queryInfo) {
        return succeed(studentLessonExaminationService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取单个学员评测")
    @PostMapping("getLessonStudent")
    public HttpResponseResult<StudentLessonExaminationDto> getLessonStudent(@RequestBody StudentLessonExaminationQueryInfo queryInfo) {
        return succeed(studentLessonExaminationService.queryPage(queryInfo).getRows().get(0));
    }

    @ApiOperation(value = "获取学员评测详情列表")
    @PostMapping("queryStudentLessonDetail")
    public HttpResponseResult<List<StudentLessonExaminationDetailDto>> queryStudentLessonDetail(@RequestBody StudentLessonExaminationQueryDto query) {
        return succeed(studentLessonExaminationDetailService.queryAll(query));
    }


    @ApiOperation(value = "根据课程计划获取需要交作业的学生-公用")
    @PostMapping("/findCourseStudentsPublic/v2")
    public HttpResponseResult<List<StudentCourseHomework>> findCourseStudentsPublicV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            return succeed(studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(query));
        }else{
            return succeed(studentCourseHomeworkService.findExtraExerciseStudentsV2(query));
        }
    }

    @ApiOperation(value = "查询单个学生的训练")
    @GetMapping("/findCourseHomeworkStudentDetailPublic/v2")
    public HttpResponseResult<StudentLessonTrainingDetailWrapper.StudentLessonTraining> findCourseHomeworkStudentDetailPublicV2(@RequestParam Long courseScheduleId, @RequestParam Long userId, @RequestParam ELessonTrainingType type){
        if(ELessonTrainingType.HOMEWORK.equals(type)){

            // 获取学生作业界面详细信息
            return succeed(studentCourseHomeworkService.findCourseHomeworkStudentDetailPublicV2(courseScheduleId, userId,type));
        }else{

            return succeed(studentCourseHomeworkService.findExtracurricularExercisesDetailPublicV2(courseScheduleId, userId, type));
        }
    }

    @ApiOperation(value = "根据课程计划获取需要交作业的学生声部-公用")
    @PostMapping("/findCourseStudentsSubjectPublic/v2")
    public HttpResponseResult<List<Subject>> findCourseStudentsSubjectPublicV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        List<StudentCourseHomework> studentCourseHomeworkByCourseV2;
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            studentCourseHomeworkByCourseV2 = studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(
                    query);
        }else{
            studentCourseHomeworkByCourseV2 = studentCourseHomeworkService.findExtraExerciseStudentsV2(query);
        }

        if (org.springframework.util.CollectionUtils.isEmpty(studentCourseHomeworkByCourseV2)) {
            return succeed(new ArrayList<>());
        }
        // 获取声部
        List<Integer> collect = studentCourseHomeworkByCourseV2.stream().map(StudentCourseHomework::getSubjectId).collect(
                Collectors.toList());

        return succeed(subjectService.findBySubjectByIdList(collect));

    }

    @ApiOperation(value = "获取作业详情")
    @GetMapping(value = "/findCourseHomeworkDetail")
    public HttpResponseResult<CourseHomeworkWrapper.CourseHomeworkList> findCourseHomeworkDetail(Integer courseScheduleId,ELessonTrainingType type){
        if (ELessonTrainingType.HOMEWORK.equals(type)) {
            return succeed(courseHomeworkService.findCourseHomeworkDetail(courseScheduleId));
        } else {
            return succeed(courseHomeworkService.findCourseExtraHomeworkDetail(courseScheduleId));
        }
    }

    @ApiOperation(value = "根据课程计划获取需要交作业的学生统计")
    @PostMapping("/findCourseStudentsPublicSubject/v2")
    public HttpResponseResult<CourseHomeworkWrapper.StudentHomeworkRecordStat> findCourseStudentsPublicSubjectV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        List<StudentCourseHomework> list;
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            list =  (studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(query));
        }else{
            list =  (studentCourseHomeworkService.findExtraExerciseStudentsV2(query));
        }

        CourseHomeworkWrapper.StudentHomeworkRecordStat stat = new CourseHomeworkWrapper.StudentHomeworkRecordStat();
        stat.setAllNum(list.size());
        stat.setFinishNum((int) list.stream().filter(StudentCourseHomework::getFinishFlag).count());
        stat.setUnFinishNum(stat.getAllNum()-stat.getFinishNum());
        stat.setStudentCourseHomeworkList(list);
        return succeed(stat);
    }
}

