package com.ym.mec.web.controller.school;

import com.ym.mec.biz.dal.school.dto.TeacherAttendanceDto;
import com.ym.mec.biz.dal.school.dto.TeacherAttendanceListDto;
import com.ym.mec.biz.dal.school.dto.TeacherAttendanceSumDto;
import com.ym.mec.biz.dal.school.page.TeacherAttendanceQueryDto;
import com.ym.mec.biz.dal.school.page.TeacherAttendanceQueryInfo;
import com.ym.mec.biz.dal.wrapper.TeacherAttendanceWrapper;
import com.ym.mec.biz.service.TeacherAttendanceService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RequestMapping("${app-config.url.web:}/schoolTeacherAttendance")
@Api(tags = "学校端教师考勤服务")
@RestController
public class SchoolTeacherAttendanceController extends BaseController {

    @Autowired
    private TeacherAttendanceService teacherAttendanceService;

    @ApiOperation(value = "首页教师考勤列表查询----获取本学期课程关联的老师列表，以及老师考勤状况汇总")
    @PostMapping("/queryTeacherAttendance")
    public HttpResponseResult<List<TeacherAttendanceSumDto>> queryTeacherAttendance(@RequestBody TeacherAttendanceQueryInfo queryInfo,@RequestHeader Integer coopId){
        queryInfo.setCoopId(coopId);
        return succeed(teacherAttendanceService.queryTeacherAttendance(queryInfo));
    }

    @ApiOperation(value = "获取单个老师本学期考勤汇总")
    @PostMapping("/getTeacherAttendance")
    public HttpResponseResult<TeacherAttendanceSumDto> getTeacherAttendance(@RequestBody TeacherAttendanceQueryDto queryDto,@RequestHeader Integer coopId){
        TeacherAttendanceQueryInfo queryInfo = new TeacherAttendanceQueryInfo();
        queryInfo.setCoopId(coopId);
        queryInfo.setTeacherId(queryDto.getTeacherId());
        queryInfo.setClassGroupId(queryDto.getClassGroupId());
        queryInfo.setCourseStatus(queryDto.getCourseStatus());
        List<TeacherAttendanceSumDto> sumDtos = teacherAttendanceService.queryTeacherAttendance(queryInfo);
        return succeed(CollectionUtils.isEmpty(sumDtos)?new TeacherAttendanceSumDto():sumDtos.get(0));
    }


    @ApiImplicitParams({
            @ApiImplicitParam(name = "courseId", dataType = "Long", required = true, value = "课程编号"),
            @ApiImplicitParam(name = "teacherId", dataType = "Integer", required = true, value = "老师编号"),
    })
    @ApiOperation(value = "课时获取单节课老师考勤详情----获取课程老师考勤")
    @GetMapping("/getCourseTeacherAttendance")
    public HttpResponseResult<TeacherAttendanceDto> getCourseTeacherAttendance(Long courseId, Integer teacherId) {
        return succeed(teacherAttendanceService.getCourseTeacherAttendance(courseId,teacherId));
    }


    @ApiOperation(value = "获取单个老师所在班级考勤列表----获取本学期当前班级老师所有课程考勤")
    @PostMapping("/getClassTeacherAttendance")
    public HttpResponseResult<List<TeacherAttendanceListDto>> getClassTeacherAttendance(@RequestBody TeacherAttendanceWrapper.ClassTeacherAttendanceQuery query) {
        return succeed(teacherAttendanceService.getClassTeacherAttendance(query));
    }
}
