package com.ym.mec.web.controller.student;

import com.ym.mec.biz.dal.dto.StudentAttendanceDto;
import com.ym.mec.biz.dal.page.CourseScheduleQueryInfo;
import com.ym.mec.biz.dal.page.StudentAttendanceQueryInfo;
import com.ym.mec.biz.service.ClassGroupService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.StudentAttendanceService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.exception.BizException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

@RequestMapping("${app-config.url.web:}/studentAttendance")
@Api(tags = "学生考勤服务")
@RestController
public class StudentAttendanceController extends BaseController {

    @Autowired
    private StudentAttendanceService studentAttendanceService;
    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private OrganizationService organizationService;

    //修复逻辑:是连堂课，有签退记录，某个月,遍历有学员签到记录的每节课
    @GetMapping("/repairStudentAttendance")
    public void repairStudentAttendance(Integer month) {
        if(month == null){
            throw new BizException("参数校验失败");
        }
        studentAttendanceService.repairStudentAttendance(month);
    }

    @ApiOperation(value = "点名完成")
    @PostMapping("/addStudentAttendances")
    public Object addStudentAttendances(@RequestBody StudentAttendanceDto studentAttendanceDto){
        studentAttendanceService.addStudentAttendances(studentAttendanceDto);
        return succeed();
    }

    @ApiOperation(value = "获取学生个人签到信息")
    @GetMapping("/queryStudentPersonalAttendances")
    public Object queryStudentPersonalAttendances(@RequestBody StudentAttendanceQueryInfo queryInfo){
        return succeed(studentAttendanceService.getStudentPersonalAttendances(queryInfo));
    }

    @ApiOperation(value = "获取某节课学生签到列表")
    @RequestMapping("/findStudentAttendance")
    @PreAuthorize("@pcs.hasPermissions('studentAttendance/findStudentAttendance')")
    public Object findStudentAttendance(StudentAttendanceQueryInfo queryInfo){
        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        //按考勤状态和回访状态排序
        queryInfo.setOrderFlag(true);
        return succeed(studentAttendanceService.findStudentAttendance(queryInfo));
    }

    @ApiOperation(value = "查询课程学生列表")
    @GetMapping("/findAttendanceStudentByCourseWithPage")
    @PreAuthorize("@pcs.hasPermissions('studentAttendance/findAttendanceStudentByCourseWithPage')")
    public Object findAttendanceStudentByCourseWithPage(CourseScheduleQueryInfo queryInfo){
        return succeed(classGroupService.findAttendanceStudentByCourseWithPage(queryInfo));
    }

    @ApiOperation(value = "更新学生签到记录")
    @PostMapping("/updateStudentAttendances")
    @PreAuthorize("@pcs.hasPermissions('studentAttendance/updateStudentAttendances')")
    public Object updateStudentAttendances(@RequestBody StudentAttendanceDto studentAttendanceInfo){
        studentAttendanceService.updateStudentAttendances(studentAttendanceInfo);
        return succeed();
    }
}
