package com.ym.mec.auth.api.client;

import com.dayaedu.cbs.openfeign.wrapper.qrcode.CbsQrCodeScanWrapper;
import com.ym.mec.auth.api.client.fallback.SysUserFeignServiceFallback;
import com.ym.mec.auth.api.dto.SysUserQueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.common.config.FeignConfiguration;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.ApiOperation;
import org.json.JSONObject;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@FeignClient(contextId = "sysUserFeignService", name = "auth-server", configuration = { FeignConfiguration.class }, fallback = SysUserFeignServiceFallback.class)
public interface SysUserFeignService {

	@GetMapping(value = "user/queryUserById/{userId}")
	public SysUser queryUserById(@PathVariable("userId") Integer userId);

	@GetMapping(value = "user/queryUserByPhone")
	public SysUser queryUserByMobile(@RequestParam("mobile") String mobile);

	@PostMapping(value = "user/add", consumes = MediaType.APPLICATION_JSON_VALUE)
	public HttpResponseResult<Integer> addUser(SysUser user);

	@GetMapping(value = "queryUserInfo")
	public SysUser queryUserInfo();

	@GetMapping(value = "role/queryRoleCodeListByUserId")
	public List<String> queryRoleCodeListByUserId(@RequestParam("userId") Integer userId);

	@PostMapping(value = "user/updateSysUser", consumes = MediaType.APPLICATION_JSON_VALUE)
	public Boolean updateSysUser(SysUser user);

	@PostMapping(value = "exit/{clientId}/{phone}", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	public HttpResponseResult<String> exitByPhone(@PathVariable("clientId") String clientId, @PathVariable("phone") String phone);



	@PostMapping(value = "/refreshToken")
	@ApiOperation(value = "刷新token")
	HttpResponseResult refreshToken(@RequestParam("refreshToken")String refreshToken,
									@RequestParam("clientId")String clientId,
									@RequestParam("clientSecret")String clientSecret);

	@PostMapping(value = "exit")
	@ApiOperation(value = "退出登录")
	HttpResponseResult<String> logout();

	@PostMapping(value = "user/list")
	HttpResponseResult<List<SysUser>> page(@RequestBody SysUserQueryInfo queryInfo);

	@PostMapping(value = "open/qrcode/userInfo")
	CbsQrCodeScanWrapper.UserInfo userInfo(@RequestBody CbsQrCodeScanWrapper.QrCodeScanUserInfoReq req);

	@PostMapping(value = "open/qrcode/login")
	CbsQrCodeScanWrapper.QrCodeScanToken login(@RequestBody CbsQrCodeScanWrapper.QrCodeScanReq req);

	@PostMapping(value = "smsLogin")
	HttpResponseResult<Map<String,Object>> smsLogin(@RequestParam("phone")String phone,
													@RequestParam("clientId")String clientId,
													@RequestParam("clientType")String clientType,
													@RequestParam("clientSecret")String clientSecret);
}
