package com.ym.mec.auth.core.handler;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.entity.SysUserLogin;
import com.ym.mec.auth.api.entity.SysUserLoginLog;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.service.SysUserLoginLogService;
import com.ym.mec.auth.service.SysUserLoginService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.security.SecurityConstants;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.web.WebFeignService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.exceptions.UnapprovedClientAuthenticationException;
import org.springframework.security.oauth2.provider.ClientDetails;
import org.springframework.security.oauth2.provider.ClientDetailsService;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.TokenRequest;
import org.springframework.security.oauth2.provider.token.AuthorizationServerTokenServices;
import org.springframework.security.web.authentication.SavedRequestAwareAuthenticationSuccessHandler;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.Base64;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Component
public class BaseAuthenticationSuccessEventHandler extends SavedRequestAwareAuthenticationSuccessHandler {

	@Autowired
	private SysUserLoginService sysUserLoginService;
	@Autowired
	private SysUserLoginLogService sysUserLoginLogService;
	@Autowired
	private SysUserService sysUserService;
	@Autowired
	private ObjectMapper objectMapper;
	@Autowired
	private ImFeignService imFeignService;
	@Autowired
	private WebFeignService userFeignService;
	
	@Autowired
	private ClientDetailsService clientDetailsService;

	@Value("${auth.sysconfig.tenantId}")
	private Integer tenantClientId;

	@Autowired
	private AuthorizationServerTokenServices defaultAuthorizationServerTokenServices;

	private final static Logger logger = LoggerFactory.getLogger(BaseAuthenticationSuccessEventHandler.class);

	@Override
	public void onAuthenticationSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication){
		logger.info("用户：{} 登录成功", authentication.getPrincipal());
		HashMap<String,String> hashMap = objectMapper.convertValue(authentication.getPrincipal(), HashMap.class);
		String username = hashMap.get("username");
		SysUser sysUser;
		if(username.startsWith(SecurityConstants.USERNAME_PRINCIPAL_PREFIX)){
			sysUser = sysUserService.queryByUsername(username.split(":")[1]);
		}else {
			sysUser = sysUserService.queryLockByPhone(username.split(":")[1]);
		}

		// 如果用户没有imToken则注册im
		String name = sysUser.getUsername();
		if(StringUtils.isBlank(name) && StringUtils.isNotBlank(sysUser.getRealName())){
			name = sysUser.getRealName();
		}
		// IM用户信息
		ImUserModel userModel = new ImUserModel(sysUser.getId().toString(), name, null);
		// 用户身份
		userModel.setClientType("");
		// 用户头象
		userModel.setAvatar(sysUser.getAvatar());
		ImUserModel.ImUserResult register = imFeignService.userImTokenInfo(userModel);
		logger.info("用户：{} 注册IM成功", JSON.toJSONString(register));
		if (Objects.nonNull(register)) {
			sysUser.setImToken(register.getImToken());
			sysUser.setUpdateTime(new Date());
			sysUserService.update(sysUser);
		}

		Date date = new Date();
		//修改添加登录信息
		SysUserLogin userLogin = sysUserLoginService.findLoginByUserId(sysUser.getId());
		if(userLogin == null){
			userLogin = new SysUserLogin();
			userLogin.setLastLoginIp(request.getRemoteAddr());
			userLogin.setUserId(sysUser.getId());
			userLogin.setLoginCount(1);
			userLogin.setTenantId(sysUser.getTenantId());
			sysUserLoginService.insert(setUserLogin(userLogin,date));
		}else {
			userLogin.setLastLoginIp(request.getRemoteAddr());
			userLogin.setLoginCount(userLogin.getLoginCount() + 1);
			sysUserLoginService.update(setUserLogin(userLogin,date));
		}
		//添加登录日志
		SysUserLoginLog sysUserLoginLog = new SysUserLoginLog();
		sysUserLoginLog.setLoginTime(date);
		sysUserLoginLog.setLoginIp(request.getRemoteAddr());
		sysUserLoginLog.setUserId(sysUser.getId());
		sysUserLoginLog.setTenantId(sysUser.getTenantId());
		sysUserLoginLogService.insert(sysUserLoginLog);
		
		try {
			String clientId = request.getParameter("clientId");
			String clientSecret = request.getParameter("clientSecret");
			if (clientId == null || clientSecret == null) {
				throw new UnapprovedClientAuthenticationException("请求头中client信息为空");
			}
			String base64ClientCredentials = Base64.getEncoder().encodeToString((clientId + ":" + clientSecret).getBytes());
			HttpHeaders headers = new HttpHeaders();
			headers.add("Authorization", "Basic " + base64ClientCredentials);
			headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

			ClientDetails clientDetails = clientDetailsService.loadClientByClientId(clientId);
			Map<String, String> requestParameters = new HashMap<>();
			requestParameters.put("client_type", request.getParameter("clientType"));
			TokenRequest tokenRequest = new TokenRequest(requestParameters, clientId, clientDetails.getScope(), "password");
			OAuth2Request oAuth2Request = tokenRequest.createOAuth2Request(clientDetails);

			OAuth2Authentication oAuth2Authentication = new OAuth2Authentication(oAuth2Request, authentication);
			OAuth2AccessToken oAuth2AccessToken = defaultAuthorizationServerTokenServices.createAccessToken(oAuth2Authentication);
			logger.info("获取token 成功：{}", oAuth2AccessToken.getValue());
			Map<String,Object> map = new HashMap<>(3);
			map.put("password", StringUtils.isEmpty(sysUser.getPassword())?false:true);
			map.put("authentication",oAuth2AccessToken);
			
			String tenantId = "1";
			if (StringUtils.equalsIgnoreCase(clientId, "TEACHER")) {
				// 查询老师所在分部
				Integer organId = userFeignService.queryTeacherOrganId(sysUser.getId());
				if (organId == tenantClientId) {
					tenantId = "2";
				}
			} else if (StringUtils.equalsIgnoreCase(clientId, "STUDENT")) {
				if (sysUser.getOrganId() == tenantClientId) {
					tenantId = "2";
				}
			}
			map.put("tenantId", tenantId);
			map.put("companyId", sysUser.getTenantId());

			// 设置登录管户端类型
			if (clientId.toUpperCase().equals(SysUserType.EDUCATION.getCode())) {

				String collect = Arrays.stream(sysUser.getUserType().split(","))
						.filter(x -> x.equals(SysUserType.SYSTEM.getCode()) || x.equals(SysUserType.SCHOOL.getCode()))
						.collect(Collectors.joining(","));
				// 登录客户端类型
				map.put("clientType", collect);
			}

			response.setContentType("application/json; charset=utf-8");
			HttpResponseResult result = new HttpResponseResult(true, HttpStatus.OK.value(), map, "");
			response.getWriter().write(objectMapper.writeValueAsString(result));
		} catch (IOException e) {
			throw new BadCredentialsException("Failed to decode basic authentication token");
		}
	}

	private SysUserLogin setUserLogin(SysUserLogin userLogin,Date date){
		userLogin.setFailCount(0);
		userLogin.setLastLoginTime(date);
		return userLogin;
	}

}
