package com.ym.mec.auth.core.provider;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.web.controller.queryInfo.QRLoginDto;
import com.ym.mec.common.redis.service.RedisCache;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.transaction.annotation.Transactional;

import com.ym.mec.auth.api.dto.SysUserInfo;
import com.ym.mec.auth.api.entity.LoginEntity;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.config.token.PhoneAuthenticationToken;
import com.ym.mec.auth.service.SysUserDeviceService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.common.service.IdGeneratorService;

public class PhoneAuthenticationProvider extends AbstractAuthenticationProvider {

    private UserDetailsService userDetailsService;

    private IdGeneratorService smsCodeService;

    private SysUserService sysUserService;

    private SysUserDeviceService sysUserDeviceService;

    private RedisCache<String,Object> redisCache;

    @Override
    protected void additionalAuthenticationChecks(UserDetails userDetails, Authentication authentication) throws AuthenticationException {

        if (authentication.getCredentials() == null) {
            throw new BadCredentialsException(this.messages.getMessage("PhoneAuthenticationProvider.badCredentials", "Bad credentials"));
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public UserDetails retrieveUser(String username, Authentication authentication) throws AuthenticationException {
        LoginEntity loginEntity = (LoginEntity) authentication.getCredentials();
        if (loginEntity == null) {
            throw new BadCredentialsException("Bad credentials");
        }
        String smsCode = loginEntity.getSmsCode();
        String phone = loginEntity.getPhone();
        String clientId = loginEntity.getClientId();

        // 验证码验证
        if (!loginEntity.getClientId().toLowerCase().startsWith("qr_") && !smsCodeService.verifyValidCode(phone, smsCode)) {
            throw new BadCredentialsException("验证码校验失败");
        }


        Boolean isRegister = loginEntity.getIsRegister();

        String deviceNum = loginEntity.getDeviceNum();


        SysUserInfo userInfo;

        /*if (clientId.startsWith("QR_")) {
            Object data = redisCache.get(loginEntity.getPhone());
            if (data == null) {
                throw new LockedException("用户不存在");
            } else {
                redisCache.delete(loginEntity.getPhone());
                QRLoginDto loginDto = (QRLoginDto) data;
                if (loginDto.getPrivateKey().equals(loginEntity.getSmsCode())) {
                    userInfo = loginDto.getUserInfo();
                    username = username.replaceAll(loginDto.getCode(),userInfo.getSysUser().getPhone());
                } else {
                    throw new LockedException("用户不存在");
                }
            }
        } else {
        }*/
        userInfo = sysUserService.queryUserInfoByPhone(phone);

		if (userInfo == null) {

			// 强制默认管理登录类型
			if ("EDUCATION".equals(clientId)) {
				clientId = "SYSTEM";
			}

			if (isRegister == false || StringUtils.equals("SYSTEM", clientId)) {
				throw new LockedException("用户不存在");
			}

			userInfo = sysUserService.initUser(loginEntity.getTenantId(), loginEntity.getOrganId(), loginEntity.getPhone(), clientId);

			if (StringUtils.isNotBlank(deviceNum)) {
				sysUserDeviceService.bindDevice(clientId, userInfo.getSysUser().getId(), deviceNum, userInfo.getSysUser().getTenantId());
			}
		} else {
			SysUser user = userInfo.getSysUser();
			if (user == null) {
				throw new LockedException("用户不存在");
			}

			// 管理端登录或学校端登录
			if(SysUserType.EDUCATION.getCode().equals(clientId.toUpperCase())){

				List<String> collect = Arrays.stream(userInfo.getSysUser().getUserType().split(","))
						.map(String::toUpperCase)
						.collect(Collectors.toList());

				// 管理端登录身份
				if (collect.contains(SysUserType.SYSTEM.getCode())) {
					clientId = SysUserType.SYSTEM.getCode();
				}

				// 学校员工身份
				if (collect.contains(SysUserType.SCHOOL.getCode())) {
					clientId = SysUserType.SCHOOL.getCode();
				}
			}

			//如果机构不一致，不允许跨机构注册
//			if(loginEntity.getTenantId() != null && loginEntity.getTenantId() != 0 &&
//					!loginEntity.getTenantId().equals(user.getTenantId())){
//				throw new LockedException("用户已注册");
//			}
            if (StringUtils.isNotBlank(deviceNum)) {
                sysUserDeviceService.bindDevice(clientId, user.getId(), deviceNum, userInfo.getSysUser().getTenantId());
            }

            if (clientId.startsWith("QR_")) {
            } else  if (!userInfo.getSysUser().getUserType().contains(clientId)) {
                if (isRegister == false || StringUtils.equals("SYSTEM", clientId)) {
                    throw new LockedException("用户不存在");
                } else {
                    user.setUserType(user.getUserType() + "," + clientId);
                    user.setUpdateTime(new Date());

                    if (userInfo.getSysUser().getUserType().contains("SCHOOL")) {
                        throw new LockedException("用户已存在学校账户");
                    }
                    // 添加userType以及附加信息
                    if (StringUtils.equals("STUDENT", clientId)) {
                        if (StringUtils.isBlank(loginEntity.getOrganId())) {
                            user.setOrganId(sysUserService.getLesseeOrganId());
                            loginEntity.setTenantId(1);
                        } else {
                            user.setOrganId(Integer.parseInt(loginEntity.getOrganId()));
                        }
                        sysUserService.saveStudent(user.getId(),loginEntity.getTenantId());
                    } else if (StringUtils.equals("TEACHER", clientId)) {
                        sysUserService.saveTeacher(user.getId(),loginEntity.getTenantId());
                    }
                    sysUserService.update(user);
                }
            }
        }

        UserDetails loadedUser;
        try {
            loadedUser = userDetailsService.loadUserByUsername(username);
        } catch (UsernameNotFoundException e) {
            throw e;
        } catch (Exception e) {
            throw new InternalAuthenticationServiceException(e.getMessage(), e);
        }

        if (loadedUser == null) {
            throw new InternalAuthenticationServiceException("账户不存在");
        } else {
            return loadedUser;
        }
    }

    @Override
    protected Authentication createSuccessAuthentication(Object principal, Authentication authentication, UserDetails user) {
        PhoneAuthenticationToken result = new PhoneAuthenticationToken(principal, authentication.getCredentials(), user.getAuthorities());
        result.setDetails(authentication.getDetails());
        return result;
    }

    @Override
    public boolean supports(Class<?> aClass) {
        return PhoneAuthenticationToken.class.isAssignableFrom(aClass);
    }

    public void setUserDetailsService(UserDetailsService userDetailsService) {
        this.userDetailsService = userDetailsService;
    }

    public void setRedisCache(RedisCache<String, Object> redisCache) {
        this.redisCache = redisCache;
    }

    public void setSysUserService(SysUserService sysUserService) {
        this.sysUserService = sysUserService;
    }

    public void setSmsCodeService(IdGeneratorService smsCodeService) {
        this.smsCodeService = smsCodeService;
    }

    public void setSysUserDeviceService(SysUserDeviceService sysUserDeviceService) {
        this.sysUserDeviceService = sysUserDeviceService;
    }
}
