package com.ym.mec.auth.core.service;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.oauth2.common.util.OAuth2Utils;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.token.DefaultAuthenticationKeyGenerator;

public class CustomAuthenticationKeyGenerator extends DefaultAuthenticationKeyGenerator {
	
	private static final String CLIENT_ID = "client_id";
	private static final String CLIENT_TYPE = "client_type";

	private static final String SCOPE = "scope";

	private static final String USERNAME = "username";

	@Override
	public String extractKey(OAuth2Authentication authentication) {
		Map<String, String> values = new LinkedHashMap<String, String>();
		OAuth2Request authorizationRequest = authentication.getOAuth2Request();
		if (!authentication.isClientOnly()) {
			values.put(USERNAME, StringUtils.substringAfter(authentication.getName(), ":"));
		}
		String clientId = authorizationRequest.getClientId();
//		clientId = clientId.replace("QR_", "");
		values.put(CLIENT_ID, clientId);
		if (authorizationRequest.getScope() != null) {
			values.put(SCOPE, OAuth2Utils.formatParameterList(new TreeSet<String>(authorizationRequest.getScope())));
		}
		String deviceId = authorizationRequest.getRequestParameters().get(CLIENT_TYPE);
		if (StringUtils.isNotBlank(deviceId)) {
			values.put(CLIENT_TYPE, deviceId);
		}
		return generateKey(values);
	}

}
