package com.ym.mec.auth.service;

import com.ym.mec.auth.api.dto.SysUserInfo;
import com.ym.mec.auth.api.dto.SysUserQueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.service.BaseService;

import java.util.List;

public interface SysUserService extends BaseService<Integer, SysUser> {

    /**
     * 根据用户名查询对象
     *
     * @param username
     * @return
     */
    SysUser queryByUsername(String username);

    /**
     * 根据手机号查询对象
     *
     * @param phone
     * @return
     */
    SysUser queryByPhone(String phone);

    /**
     * 根据手机号查询对象
     *
     * @param phone
     * @return
     */
    SysUser queryLockByPhone(String phone);

    /**
     * 根据手机号查询对象
     *
     * @param phone
     * @return
     */
    SysUser queryByPhoneAndClient(String phone, String client);

    /**
     * 根据用户名获取系统用户详细信息
     *
     * @param username
     * @return
     */
    SysUserInfo queryUserInfoByUsername(String username);

    /**
     * 根据手机号获取系统用户详细信息
     *
     * @param phone
     * @return
     */
    SysUserInfo queryUserInfoByPhone(String phone);

    /**
     * 新增用户
     *
     * @param sysUser
     * @return
     */
    Integer add(SysUser sysUser);

    /**
     * 获取用户基本信息
     *
     * @param userId
     * @return
     */
    ImUserModel getBasic(Integer userId);

    /**
     * 修改用户基本信息
     *
     * @param sysUser
     */
    void updateBaseInfo(SysUser sysUser);

    /**
     * 获取用户基本信息
     *
     * @param userId
     * @return
     */
    SysUser queryUserInfo(Integer userId);

    /**
     * 上线时初始化用户数据
     *
     * @param phone
     * @param clientId
     * @return
     */
    SysUserInfo initUser(Integer tenantId, String organId, String phone, String clientId);

    /**
     * 刷新token
     *
     * @param sysUser
     */
    void refreshImToken(SysUser sysUser);

    /**
     * 保存student数据
     *
     * @param userId
     */
    void saveStudent(Integer userId, Integer tenantId);

    /**
     * 保存teacher数据
     *
     * @param userId
     */
    void saveTeacher(Integer userId, Integer tenantId);

    Integer getLesseeOrganId();

    List<SysUser> queryLikeByPhoneOrName(String search);

    String queryClientByPhone(String phone);
    /**
     * 商城同步管理用户
     *
     * @param queryInfo
     * @return
     */
    List<SysUser> queryEmployeeList(SysUserQueryInfo queryInfo);
}