package com.ym.mec.auth.web.controller;

import com.alibaba.fastjson.JSON;
import com.dayaedu.cbs.openfeign.service.CbsQrCodeScanService;
import com.dayaedu.cbs.openfeign.wrapper.qrcode.CbsQrCodeScanWrapper;
import com.huifu.adapay.core.util.StringUtil;
import com.ym.mec.auth.api.dto.SysUserInfo;
import com.ym.mec.auth.api.dto.SysUserQueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.service.SysRoleService;
import com.ym.mec.auth.service.SysUserRoleService;
import com.ym.mec.auth.service.SysUserService;
import com.ym.mec.auth.service.TenantInfoService;
import com.ym.mec.auth.web.controller.queryInfo.QRLoginDto;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.entity.ImResult;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.redis.service.RedisCache;
import com.ym.mec.common.security.AuthUser;
import com.ym.mec.common.security.SecurityConstants;
import com.ym.mec.common.security.SecurityUtils;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.thirdparty.user.realname.RealnameAuthenticationPluginContext;
import com.ym.mec.thirdparty.user.realname.provider.LinkfaceRealnameAuthenticationPlugin;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.*;

@RestController()
@RequestMapping("user")
@Api(tags = "会员服务")
public class UserController extends BaseController {

	@Autowired
	private SysUserService sysUserService;
	@Autowired
	private SysRoleService sysRoleService;
	@Autowired
	private ImFeignService imFeignService;
	@Autowired
	private SysUserRoleService sysUserRoleService;
	@Autowired
	private IdGeneratorService smsCodeService;
	@Autowired
	private TenantInfoService tenantInfoService;
	@Autowired
	private RedisCache<String,Object> redisCache;

	@Value("${message.debugMode}")
	private boolean debugMode;
	@Autowired
	private RealnameAuthenticationPluginContext realnameAuthenticationPluginContext;

	@ApiOperation(value = "分页查询用户信息")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userType", value = "用户类型", required = false, dataType = "String"),
			@ApiImplicitParam(name = "createStartDate", value = "开始注册时间", required = true, dataType = "String"),
			@ApiImplicitParam(name = "createEndDate", value = "结束注册时间", required = true, dataType = "String") })
	@GetMapping(value = "/queryPage")
	@PreAuthorize("@pcs.hasPermissions('user/queryPage')")
	public Object queryPage(SysUserQueryInfo queryInfo) {
		return succeed(sysUserService.queryPage(queryInfo));
	}

	@ApiOperation(value = "查询用户信息接口")
	@GetMapping("/query")
	@PreAuthorize("@pcs.hasPermissions('user/query')")
	public Object query() {
		AuthUser user = SecurityUtils.getUser();
		if (user != null) {
			return succeed(sysUserService.get(user.getUserId()));
		}
		return failed();
	}

	@ApiOperation(value = "根据用户编号获取用户基本信息")
	@GetMapping("/getBasic")
	public Object getBasic(Integer userId) {
		if (userId == null) {
			return failed("参数校验异常");
		}
		return succeed(sysUserService.getBasic(userId));
	}

	@GetMapping(value = "/findByPhone")
	public Object findByPhone(String mobile) {
		return succeed(sysUserService.queryByPhone(mobile));
	}

	@GetMapping(value = "/queryUserByPhone")
	public SysUser queryUserByPhone(String mobile) {
		return sysUserService.queryByPhone(mobile);
	}

	@GetMapping(value = "/noAuth/queryUserByPhone")
	public Object noAuthQueryUserByPhone(String mobile) {
		SysUser sysUser = sysUserService.queryByPhone(mobile);
		if(sysUser == null){
			return succeed(0);
		}
		return succeed(1);
	}

	@GetMapping(value = "/queryUserById/{userId}")
	public SysUser queryUserById(@PathVariable("userId") Integer userId) {
		return sysUserService.get(userId);
	}

	@PostMapping(value = "/updateSysUser", consumes = MediaType.APPLICATION_JSON_VALUE)
	public boolean updateSysUser(@RequestBody SysUser user) {
		user.setUpdateTime(new Date());
		sysUserService.update(user);
		return true;
	}

	@ApiOperation(value = "新增用户")
	@PostMapping(value = "/add", consumes = MediaType.APPLICATION_JSON_VALUE)
	public HttpResponseResult<Integer> add(@RequestBody SysUser sysUser) {
		return succeed(sysUserService.add(sysUser));
	}

	@ApiOperation(value = "设置密码")
	@PostMapping(value = "/setPassword", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
	@ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String")
	public Object setPassword(String password, HttpServletRequest request) {
		if (StringUtils.isEmpty(password)) {
			Enumeration<String> parameterNames = request.getParameterNames();
			String element = parameterNames.nextElement();
			if(StringUtils.isNotEmpty(element)){
				password = JSON.parseObject(element).getString("password");
			}
		}
		if (StringUtils.isEmpty(password)) {
			return failed("参数校验失败");
		}
		AuthUser authUser = SecurityUtils.getUser();
		if (authUser == null) {
			return failed("获取用户信息失败");
		}
		SysUser sysUser = sysUserService.get(authUser.getUserId());
		if (sysUser == null) {
			return failed("用户不存在");
		}
		if(StringUtils.isEmpty(sysUser.getImToken())){
			String username = sysUser.getUsername();
			if(sysUser.getUserType().contains("TEACHER") || sysUser.getUserType().contains("SYSTEM")){
				username = sysUser.getRealName();
			}
			ImResult register = imFeignService.register(new ImUserModel(sysUser.getId().toString(), username, sysUser.getAvatar()));
			sysUser.setImToken(register.getToken());
		}
		password = new BCryptPasswordEncoder().encode(password);
		sysUser.setPassword(password);
		sysUser.setUpdateTime(new Date());
		sysUserService.update(sysUser);
		return succeed();
	}

	@ApiOperation(value = "修改密码")
	@PostMapping(value = "/updatePassword", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
	@ApiImplicitParams({ @ApiImplicitParam(name = "mobile", value = "手机号", required = true, dataType = "String"),
			@ApiImplicitParam(name = "authCode", value = "验证码", required = true, dataType = "String"),
			@ApiImplicitParam(name = "newPassword", value = "新密码", required = true, dataType = "String") })
	public Object updatePassword(String mobile, String authCode, String newPassword) {
		if (StringUtils.isEmpty(mobile) || StringUtils.isEmpty(authCode) || StringUtils.isEmpty(newPassword)) {
			return failed("参数校验异常");
		}
		SysUser sysUser = sysUserService.queryByPhone(mobile);
		if (sysUser == null) {
			return failed("用户不存在");
		}
		if (smsCodeService.verifyValidCode(mobile, authCode)) {
			String password = new BCryptPasswordEncoder().encode(newPassword);
			sysUser.setPassword(password);
			sysUser.setUpdateTime(new Date());
			sysUserService.update(sysUser);
			return succeed();
		} else {
			return failed("验证码错误");
		}
	}


	@ApiOperation(value = "修改手机号")
	@PostMapping(value = "/updatePhone")
	public Object updatePhone(String oldPhone, String authCode, String newPhone) {
		if (StringUtils.isEmpty(oldPhone) || StringUtils.isEmpty(authCode) || StringUtils.isEmpty(newPhone)) {
			return failed("参数校验异常");
		}
		if(oldPhone.equals(newPhone)){
			return failed("新旧手机号一致");
		}
		SysUser sysUser = sysUserService.queryByPhone(oldPhone);
		if (sysUser == null) {
			return failed("用户不存在");
		}
		SysUser newSysUser = sysUserService.queryByPhone(newPhone);
		if (newSysUser != null) {
			return failed("手机号被占用");
		}
		if (smsCodeService.verifyValidCode(newPhone, authCode)) {
			tenantInfoService.updatePhone(newPhone, oldPhone);
			sysUser.setPhone(newPhone);
			sysUser.setUpdateTime(new Date());
			sysUserService.update(sysUser);
			return succeed();
		} else {
			return failed("验证码错误");
		}
	}

	@ApiOperation(value = "修改密码")
	@PostMapping(value = "/updatePassword2", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
	@ApiImplicitParams({ @ApiImplicitParam(name = "mobile", value = "手机号", required = true, dataType = "String"),
			@ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer"),
			@ApiImplicitParam(name = "newPassword", value = "新密码", required = true, dataType = "String") })
	public Object updatePassword2(Integer userId, String mobile, String newPassword) {
		if (StringUtils.isEmpty(mobile) || userId == null || StringUtils.isEmpty(newPassword)) {
			return failed("参数校验异常");
		}
		SysUser sysUser = sysUserService.queryByPhone(mobile);
		if (sysUser == null) {
			return failed("用户不存在");
		}
		if (sysUser.getId().intValue() != userId) {
			return failed("非法操作");
		}
		String password = new BCryptPasswordEncoder().encode(newPassword);
		sysUser.setPassword(password);
		sysUser.setUpdateTime(new Date());
		sysUserService.update(sysUser);
		return succeed();
	}

	@ApiOperation(value = "修改用户")
	@PostMapping(value = "/update", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	public Object update(SysUser sysUser) {
		AuthUser user = SecurityUtils.getUser();
		if (user != null) {
			Date date = new Date();
			sysUser.setId(user.getUserId());
			sysUser.setUpdateTime(date);
			if(sysUser.getBirthdate() != null && DateUtil.daysBetween(sysUser.getBirthdate(),date) <= 0){
				throw new BizException("出生日期不可超过当前时间");
			}

			if(!debugMode&&StringUtils.isNotBlank(sysUser.getCertificateType())&&"IDENTITY".equals(sysUser.getCertificateType())){
				realnameAuthenticationPluginContext.getRealnameAuthenticationPlugin(LinkfaceRealnameAuthenticationPlugin.getName()).verify(sysUser.getRealName(), sysUser.getIdCardNo());
			}

			sysUserService.updateBaseInfo(sysUser);
			return succeed();
		}
		return failed();
	}

	@ApiOperation(value = "刷新token")
	@PostMapping(value = "/refreshImToken")
	public Object refreshImToken() {
		AuthUser user = SecurityUtils.getUser();
		if (user != null) {
			SysUser sysUser = sysUserService.get(user.getUserId());
			String username;
			if(sysUser.getUserType().contains("TEACHER") || sysUser.getUserType().contains("SYSTEM")){
				username = sysUser.getRealName();
			}else {
				username = sysUser.getUsername();
			}
			if(StringUtils.isEmpty(username)){
				return failed("获取Token失败: 用户名为空");
			}else {
				ImResult register = imFeignService.register(new ImUserModel(sysUser.getId().toString(), username, sysUser.getAvatar()));
				if(StringUtils.isEmpty(register.getToken())){
					return failed("获取Token失败: 请联系教务老师");
				}
				sysUser.setImToken(register.getToken());
				sysUserService.refreshImToken(sysUser);
				return succeed(register.getToken());
			}
		}
		return failed();
	}

	@ApiOperation(value = "校验用户手机号和密码")
	@GetMapping(value = "/checkUser")
	public Object checkUser(String phone,String password) {
		if(StringUtils.isEmpty(phone) || StringUtils.isEmpty(password)){
			throw new BizException("请录入用户名和密码");
		}
		AuthUser user = SecurityUtils.getUser();
		if (user != null) {
			SysUser sysUser = sysUserService.get(user.getUserId());
			if(!sysUser.getPhone().equals(phone)){
				return failed("手机号校验失败");
			}
			if(new BCryptPasswordEncoder().matches(password,sysUser.getPassword())){
				return succeed();
			}else {
				return failed("密码校验失败");
			}
		}
		return failed();
	}

	@ApiOperation(value = "获取用户角色列表")
	@GetMapping("/getRole")
	@PreAuthorize("@pcs.hasPermissions('user/getRole')")
	public Object getRole() {
		AuthUser user = SecurityUtils.getUser();
		if (user != null) {
			SysUser sysUser = sysUserService.get(user.getUserId());
			if(sysUser.getIsSuperAdmin()){
				QueryInfo queryInfo = new QueryInfo();
				queryInfo.setRows(9999);
				return succeed(sysRoleService.queryPage(queryInfo).getRows());
			}
			return succeed(sysRoleService.findRoleByUserId(user.getUserId()));
		}
		return failed();
	}

	@ApiOperation(value = "用户角色新增")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer"),
			@ApiImplicitParam(name = "roleIds", value = "角色id,逗号分隔", required = true, dataType = "String") })
	@PostMapping(value = "/addRole", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	@PreAuthorize("@pcs.hasPermissions('user/addRole')")
	public Object getRole(String userId, String roleIds) {
		if (StringUtils.isEmpty(userId) || StringUtils.isEmpty(roleIds)) {
			return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
		}
		sysUserRoleService.batchInsert(Integer.parseInt(userId), roleIds);
		return succeed();
	}

	@ApiOperation(value = "用户角色删除")
	@ApiImplicitParams({ @ApiImplicitParam(name = "userId", value = "用户编号", required = true, dataType = "Integer"),
			@ApiImplicitParam(name = "roleIds", value = "角色id,逗号分隔", required = true, dataType = "String") })
	@PostMapping(value = "/delRole", consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
	@PreAuthorize("@pcs.hasPermissions('user/delRole')")
	public Object delRole(String userId, String roleIds) {
		if (StringUtils.isEmpty(userId) || StringUtils.isEmpty(roleIds)) {
			return failed(SecurityConstants.PARAM_VERIFY_EXCEPTION);
		}
		sysUserRoleService.batchDel(Integer.parseInt(userId), roleIds);
		return succeed();
	}

    @ApiOperation(value = "通过手机号及姓名进行模糊查询")
    @GetMapping(value = "/queryLike")
    public Object queryLikeByPhoneOrName(String search) {
        return succeed(sysUserService.queryLikeByPhoneOrName(search));
    }

    @ApiOperation(value = "通过手机号查询client")
    @GetMapping(value = "/queryClient")
    public Object queryClientByPhone(String phone) {
        return succeed(sysUserService.queryClientByPhone(phone));
    }

	@ApiOperation(value = "用户注销")
	@PostMapping(value = "/destroy")
	public Object destroy(String authCode) {
		AuthUser user = SecurityUtils.getUser();
		if (user != null) {
			SysUser sysUser = sysUserService.get(user.getUserId());
			if(smsCodeService.verifyValidCode(sysUser.getPhone(), authCode)){
				sysUser.setPhone("Z" + sysUser.getPhone());
				sysUser.setUpdateTime(new Date());
				sysUserService.update(sysUser);
				return succeed();
			}
			return failed("验证码错误");
		}
		return failed("请登录");
	}

	@ApiOperation(value = "查询平台用户信息")
	@PostMapping(value = "/list")
	public HttpResponseResult<List<SysUser>> page(@RequestBody SysUserQueryInfo queryInfo) {
		return succeed(sysUserService.queryEmployeeList(queryInfo));
	}



	@GetMapping("/open/getQRLoginCode")
	@ApiOperation(value = "获取二维码登录code(不需要鉴权)")
	public HttpResponseResult<String> getQRLoginCode(String clientId) {

		String uuid = UUID.randomUUID().toString();

		QRLoginDto qrLoginDto = new QRLoginDto();
		qrLoginDto.setCode(uuid);
		qrLoginDto.setClientId(clientId);
		redisCache.put(uuid,qrLoginDto,5*60);
		return succeed(uuid);
	}


	@GetMapping("/open/pollingQRLoginCode")
	@ApiOperation(value = "前端轮询登录(不需要鉴权)")
	public HttpResponseResult<QRLoginDto> pollingQRLoginCode(@ApiParam(value = "二维码登录code", required = true) @RequestParam("code") String code) {
		if (StringUtil.isEmpty(code)) {
			throw new BizException("登录失败");
		}

		Object obj = redisCache.get(code);
		if (null != obj) {
			QRLoginDto dto = (QRLoginDto) obj;
			dto.setUserInfo(null);
			return succeed(dto);
		} else {
			QRLoginDto qrLoginDto = new QRLoginDto();
			qrLoginDto.setExpireFlag(true);
			return succeed(qrLoginDto);
		}
	}

	@GetMapping(value = "/qrLogin")
	@ApiOperation(value = "二维码登录-扫码")
	public HttpResponseResult<QRLoginDto> qrLogin(
		@ApiParam(value = "二维码登录code", required = true) @RequestParam("code") String code
	) throws IOException {
		AuthUser authUser = SecurityUtils.getUser();
		if (authUser == null) {
			throw new BizException("请先登录");
		}
		SysUser sysUser = sysUserService.get(authUser.getUserId());


		Object data = redisCache.get(code);
		if (null == data) {
			QRLoginDto qrLoginDto = new QRLoginDto();
			qrLoginDto.setExpireFlag(true);
			return succeed(qrLoginDto);
		}

		QRLoginDto dto = (QRLoginDto) data;
		if (!sysUser.getUserType().contains((dto).getClientId().replace("QR_", "").toUpperCase(Locale.ROOT))) {
			throw new BizException("登录失败");
		}
		redisCache.put(code,data,5*60);
		dto.setUserInfo(null);
		return succeed(dto);
	}

	@GetMapping(value = "/doQrLogin")
	@ApiOperation(value = "二维码登录-确认登录")
	public HttpResponseResult<QRLoginDto> doQrLogin(
		@ApiParam(value = "二维码登录code", required = true) @RequestParam("code") String code
	) {
		AuthUser authUser = SecurityUtils.getUser();
		if (authUser == null) {
			throw new BizException("请先登录");
		}
		SysUser sysUser = sysUserService.get(authUser.getUserId());

		Object data = redisCache.get(code);
		if (null == data) {
			QRLoginDto qrLoginDto = new QRLoginDto();
			qrLoginDto.setExpireFlag(true);
			return succeed(qrLoginDto);
		}


		SysUserInfo userInfo = sysUserService.queryUserInfoByPhone(sysUser.getPhone());
		QRLoginDto dto = (QRLoginDto) data;


		if (!sysUser.getUserType().contains((dto).getClientId().replace("QR_", "").toUpperCase(Locale.ROOT))) {
			throw new BizException("登录失败");
		}

		dto.setUserInfo(userInfo);


		String uuid = UUID.randomUUID().toString();
		dto.setPrivateKey(uuid);
		redisCache.put(code,dto,5*60);

		dto.setUserInfo(null);

		return succeed(dto);
	}

}
