package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.Practice4OrganDto;
import com.ym.mec.biz.dal.dto.PracticeGroupsDto;
import com.ym.mec.biz.dal.entity.CourseScheduleEvaluate;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.common.dal.BaseDAO;

import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

public interface CourseScheduleEvaluateDao extends BaseDAO<Long, CourseScheduleEvaluate> {

    CourseScheduleEvaluate findByClassGroupId(@Param("classGroupId") Integer classGroupId);

    List<CourseScheduleEvaluate> findByClassGroupIds(@Param("classGroupIds") List<Integer> classGroupIds);

    List<PracticeGroupsDto> getPracticeGroupsFree(Integer tenantId);

    List<PracticeGroupsDto> getPracticeGroupsBuy(Integer tenantId);

    List<PracticeGroupsDto> getPracticeGroupsRenew(Integer tenantId);

    List<PracticeGroupsDto> getHasReportNums(Integer tenantId);

    List<PracticeGroupsDto> getOrganPracticeGroups(Integer tenantId);

    List<PracticeGroupsDto> getOrganMoney(Integer tenantId);

    /**
     * @param expiredDate:
     * @return java.util.List<com.ym.mec.biz.dal.entity.CourseScheduleEvaluate>
     * @describe 推送指定时间及之前的未推送的报告
     * @author Joburgess
     * @date 2020/2/28
     */
    List<CourseScheduleEvaluate> findExpiredDateBeforeReport(@Param("expiredDate") Date expiredDate);

    /**
     * 根据课程组id获取报告列表
     *
     * @param groupId
     * @return
     */
    List<CourseScheduleEvaluate> findByGroupId(@Param("groupId") Integer groupId, @Param("status") Integer status);

    /**
     * 根据id获取报告
     *
     * @return
     */
    CourseScheduleEvaluate findById(@Param("id") Integer id);

    /**
     * 获取当前课程教师的评价列表
     *
     * @param teacherId
     * @param courseScheduleId
     * @return
     */
    List<CourseScheduleEvaluate> findByCourseAndTeacher(@Param("teacherId") Integer teacherId, @Param("courseScheduleId") Long courseScheduleId);

    /**
     * 获取要提交的报告
     *
     * @param teacherId
     * @return
     */
    List<CourseScheduleEvaluate> getNeedPostReports(@Param("teacherId") Integer teacherId, @Param("tenantId") Integer tenantId);

    /**
     * 批量插入报告
     *
     * @param list
     * @return
     */
    int batchAdd(@Param("list") List<CourseScheduleEvaluate> list);

    /**
     * 获取班级课程数
     *
     * @param classGroupId
     * @return
     */
    Integer getClassGroupCourseTimes(@Param("classGroupId") Integer classGroupId);

    /**
     * 查询当天是否已有报告
     *
     * @param classGroupIds
     * @param nowDate
     * @return
     */
    List<CourseScheduleEvaluate> hasReportList(@Param("classGroupIds") List<Integer> classGroupIds, @Param("nowDate") Date nowDate);

    /**
     * 获取当月成交金额，人数
     *
     * @param type
     * @return
     */
    List<PracticeGroupsDto> getMonthOrganMoney(@Param("type") OrderTypeEnum type, @Param("startTime") Date startTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取当月新成交金额，人数
     *
     * @return
     */
    List<PracticeGroupsDto> getOrganNewPracticeGroupsOfMonth(@Param("type") OrderTypeEnum type, @Param("startTime") Date startTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取分部体验人数(去除禁止)
     *
     * @return
     */
    List<Practice4OrganDto> getTryNums(Integer tenantId);

    /**
     * 网管课体验人数
     *
     * @return
     */
    List<Practice4OrganDto> getPracticeTryNums(Integer tenantId);

    /**
     * vip课体验人数
     *
     * @return
     */
    List<Practice4OrganDto> getVipTryNums(Integer tenantId);

    /**
     * 网管课转化人数
     *
     * @param classDate
     * @return
     */
    List<Practice4OrganDto> getPracticeBuyNums(@Param("classDate") Date classDate, @Param("endDate") Date endDate, @Param("tenantId") Integer tenantId);

    /**
     * VIP课转化人数
     *
     * @param classDate
     * @return
     */
    List<Practice4OrganDto> getVipBuyNums(@Param("classDate") Date classDate, @Param("endDate") Date endDate, @Param("tenantId") Integer tenantId);

    /**
     * 网管课和vip课同事存在的人数
     *
     * @param classDate
     * @param endDate
     * @return
     */
    List<Practice4OrganDto> getPracticeAndVipNums(@Param("classDate") Date classDate, @Param("endDate") Date endDate, @Param("tenantId") Integer tenantId);

    /**
     * 禁止中vip和网管同时存在人数
     *
     * @return
     */
    List<Practice4OrganDto> getPracticeAndVipTryNums(Integer tenantId);

    /**
     * 获取所有的体验人数
     * @return
     */
    List<Practice4OrganDto> getAllTryNums(Integer tenantId);
}
