package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.ImGroupMemberDto;
import com.ym.mec.biz.dal.dto.NameDto;
import com.ym.mec.biz.dal.entity.ImGroup;
import com.ym.mec.biz.dal.entity.ImHistoryMessage;
import com.ym.mec.biz.dal.enums.EFriendRoleType;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupModel;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

public interface ImGroupDao extends BaseDAO<String, ImGroup> {

    ImGroup getLocked(String imGroupId);

    /**
     * 查询用户所在的群列表
     *
     * @param userId
     * @param search
     * @return
     */
    List<ImGroup> queryByUserId(@Param("userId") Integer userId,
                                @Param("search") String search,
                                @Param("groupType") String groupType);

    /**
     * 查询群成员列表
     *
     * @param imGroupId
     * @param search
     * @return
     */
    List<ImGroupMemberDto> queryMemberById(@Param("imGroupId") String imGroupId, @Param("search") String search);

    ImGroupMemberDto queryMember(@Param("imGroupId") String imGroupId, @Param("userId") Integer userId);

    int updateNickname(@Param("userId") Integer userId, @Param("nickName") String nickName);

    int updateUserFriendNickname(@Param("userId") Integer userId, @Param("nickName") String nickName);

    void updateMemberNum();

    List<NameDto> queryNameByIds(@Param("ids") String ids);

    List<ImGroup> queryByUserIdV2(@Param("userId") Integer userId, @Param("search") String search, @Param("groupType") String groupType, @Param("musicGroupId") String musicGroupId, @Param("classType") String classType);

    List<ImGroupMemberDto> queryMemberByIdV2(@Param("imGroupId") String imGroupId, @Param("search") String search, @Param("subjectId") Integer subjectId, @Param("vipFlag") Boolean vipFlag);

    ImGroup queryBySchoolId(@Param("schoolId") Long schoolId);

    List<ImGroup> getByIds(@Param("musicGroupIds") List<String> musicGroupIds);

    /**
     * 批量添加群成员数量
     *
     * @param imGroupIds
     * @param i
     */
    void addMemberNum(@Param("imGroupIds") List<String> imGroupIds, @Param("num") int i);

    List<ImGroup> getByUserId(@Param("param") ImGroupWrapper.ImQuery query);

    int updateMember(@Param("imGroupId") String imGroupId, @Param("num") int num);

    /**
     * 修改学校信息
     *
     * @param userId
     * @param userType
     */
    void updateSchoolRoleType(@Param("userId") Long userId, @Param("userType") EFriendRoleType userType);

    /**
     * 更新群组导入成功标志
     * @param id
     */
    void updateImportStatusSuccess(@Param("id") String id);

    /**
     * 查询数据总数
     * @return
     */
    int queryInfoCount();

    /**
     * 查总数据
     * @param result
     * @param size
     * @return
     */
    List<ImHistoryMessage> selectAll(@Param("result") int result, @Param("size") int size);

    /**
     * 更新状态
     * @param info
     */
    void updateStatus(@Param("info") List<ImHistoryMessage> info);

    //获取相应角色的衔接群
    List<ImGroupModel> queryUserGroupList(@Param("transferUserId") Integer transferUserId,
                                          @Param("userId") Integer userId,
                                          @Param("roleType") String roleType,
                                          @Param("organId") Integer organId);
}