package com.ym.mec.biz.dal.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ym.mec.biz.dal.dto.RoomMemberNumDto;
import com.ym.mec.biz.dal.entity.ImLiveBroadcastRoomMember;
import com.ym.mec.biz.dal.vo.ImLiveBroadcastRoomMemberVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * 直播间人员关系表(ImLiveBroadcastRoomMember)表数据库访问层
 *
 * @author hgw
 * @since 2022-02-21 14:26:58
 */
public interface ImLiveBroadcastRoomMemberDao extends BaseMapper<ImLiveBroadcastRoomMember> {

    int insertBatch(@Param("entities") List<ImLiveBroadcastRoomMember> entities);

    IPage<ImLiveBroadcastRoomMemberVo> queryMemberPage(Page<ImLiveBroadcastRoomMemberVo> page, @Param("param") Map<String, Object> param);

    /**
     * 设置在线状态
     *
     * @param userIds      用户id
     * @param groupId      房间编号
     * @param onlineStatus 状态
     */
    void updateOnlineStatus(@Param("userIds") List<Long> userIds, @Param("groupId") String groupId, @Param(
        "onlineStatus") Integer onlineStatus);
    /**
     * 设置直播状态
     *
     * @param userIds      用户id
     * @param groupId      房间编号
     * @param liveRoomStatus 状态
     */
    void updateLiveRoomStatus(@Param("userIds") List<Long> userIds, @Param("groupId") String groupId, @Param(
        "liveRoomStatus") Integer liveRoomStatus);

    List<ImLiveBroadcastRoomMember> queryMember(@Param("userIds") List<Long> userIds, @Param("groupId") String groupId);

    /**
     * 修改用户连麦状态
     *
     * @param roomUid          直播房间id
     * @param userId           用户id
     * @param whetherMicStatus 连麦状态
     */
    void userWhetherMic(@Param("roomUid") String roomUid, @Param("userId") Long userId, @Param(
        "whetherMicStatus") Integer whetherMicStatus);

    /**
     * 查询房间人数
     *
     * @param roomUid 房间id
     */
    RoomMemberNumDto queryMemberNum(@Param("roomUid") String roomUid);

    /**
     * 学后报名乐团
     * @param studentIds 学生编号
     * @return List<ImLiveBroadcastRoomMemberVo>
     */
    List<ImLiveBroadcastRoomMemberVo> selectStudentMusicGroup(@Param("studentIds") List<Integer> studentIds);

    /**
     * 学生声部
     * @param studentIds 学生编号
     * @return List<ImLiveBroadcastRoomMemberVo>
     */
    List<ImLiveBroadcastRoomMemberVo> selectStudentSubject(@Param("studentIds") List<Integer> studentIds);
}

