package com.ym.mec.biz.dal.dao;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicMemberDto;
import com.ym.mec.biz.dal.entity.School;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.biz.dal.enums.MusicGroupStatusEnum;
import com.ym.mec.biz.dal.enums.StudentMusicGroupStatusEnum;
import com.ym.mec.biz.dal.page.ExportUserAccountQueryInfo;
import com.ym.mec.biz.dal.page.IndexDataQueryInfo;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImGroupModel;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Set;

public interface MusicGroupDao extends BaseDAO<String, MusicGroup> {

    MusicGroup getLocked(String id);

    /**
     * 查询乐团基本信息
     * @param id
     * @return
     */
    MusicGroupBasicDto queryBasicInfo(String id);

    /**
     * 获取学员所在乐团详情列表
     *
     * @param userId
     * @return
     */
    List<MusicCardDto> queryUserMusicGroups(Integer userId);

    /**
     * 获取个人中心乐团数据
     *
     * @param userId
     * @return
     */
    MusicCardDto queryPersonalMusicGroups(@Param("userId") Integer userId);

    /**
     * 获取用户有效的乐团数
     *
     * @param userId
     * @return
     */
    Integer countUserMusicGroups(Integer userId);

    /**
     * 根据课程类型获取剩余课时
     *
     * @param userId
     * @param type
     * @return
     */
    Integer countCourseByType(@Param("userId") Integer userId, @Param("type") ClassGroupTypeEnum type);

    /**
     * 获取学员所在乐团
     *
     * @param userId
     * @return
     */
    List<ImGroupModel> queryUserGroups(@Param("userId") Integer userId, @Param("search") String search);

    /**
     * 获取乐团总人数
     *
     * @param musicGroupIds
     * @return
     */
    List<Map<String, BigDecimal>> countMusicGroupUserNum(@Param("musicGroupIds") String musicGroupIds);

    /**
     * 获取乐团所有学员基本信息
     * @param musicGroupId
     * @return
     */
//    List<ImUserModel> findGroupUsers(String musicGroupId);

    /**
     * 获取教师所在乐团列表
     *
     * @param userId
     * @param search
     * @return
     */
    List<ImGroupModel> queryTeacherGroups(@Param("userId") Integer userId, @Param("search") String search);

    /**
     * 获取当前乐团所有教师列表
     * @param musicGroupId
     * @return
     */
//    Set<Integer> queryTeacherIds(String musicGroupId);

    /**
     * 根据乐团编号列表获取乐团在读人数map
     *
     * @param musicGroupIds
     * @return
     */
    List<Map<String, Long>> countPayNum(@Param("musicGroupIds") String musicGroupIds);

    /**
     * 获取乐团列表
     *
     * @param musicGroupIds
     * @return
     */
    List<MusicGroup> queryListByIds(@Param("musicGroupIds") String musicGroupIds);

    /**
     * 查询首页的乐团数量
     *
     * @param ogranId
     * @return
     */
    Map<String, Integer> queryOrganMusicGroupNum(@Param("organId") String ogranId, @Param("tenantId") Integer tenantId);

    /**
     * 查询本月乐团课程数
     *
     * @param organId
     * @return
     */
    Integer queryCurrentMonthCoursesNum(@Param("organId") String organId, @Param("tenantId") Integer tenantId);

    /**
     * 获取乐团下的教师map
     *
     * @param musicGroupId
     * @return
     */
    List<Map<Integer, String>> queryTeacherMap(String musicGroupId);

    /**
     * 获取乐团下的教师map
     *
     * @param musicGroupId
     * @return
     */
    List<Map<Integer, String>> queryTeacherIdMap(String musicGroupId);

    /**
     * 根据角色编号获取有权限的用户列表
     *
     * @param roleIds
     * @return
     */
    Set<Integer> queryUserIdByRoleId(@Param("roleIds") Set<Integer> roles, @Param("organId") Integer organId);

    /**
     * 获取教务老师时当前用户的乐团
     *
     * @param userIds
     * @return
     */
    String queryEducationalMusicGroupId(Integer userIds);


    /**
     * 查询用户的乐团信息
     *
     * @param userId
     * @return
     */
    MusicGroup findUserMusicGroup(@Param("userId") Integer userId);

    /**
     * @param userId: 用户编号
     * @return com.ym.mec.biz.dal.entity.MusicGroup
     * @describe 查询用户的乐团信息
     * @author Joburgess
     * @date 2020/2/4
     */
    List<MusicGroup> findUserMusicGroups(@Param("userId") Integer userId);

    /**
     * 查询乐团列表
     *
     * @param params
     * @return
     */
    List<MusicGroup> findMusicGroups(Map<String, Object> params);

    int countMusicGroups(Map<String, Object> params);

    /**
     * 获取乐团信息
     *
     * @param list
     * @return
     */
    MusicGroup findMusicGroup(List<Long> list);

    /**
     * 获取乐团名称
     *
     * @param musicGroupIds
     * @return
     */
    List<Map<String, String>> queryMusicGroupNameMap(@Param("musicGroupIds") Set<String> musicGroupIds);

    /**
     * 查询用户的学校
     *
     * @param userId
     * @return
     */
    School findUserSchool(@Param("userId") Integer userId);

    /**
     * 根据班级群，获取乐团信息
     *
     * @param groupId
     * @return
     */
    MusicGroup findByClassId(Integer groupId);

    /**
     * 获取用户所在乐团名
     *
     * @param userIds
     * @return
     */
    List<Map<Integer, String>> queryUserMusicNames(@Param("userIds") Set<Integer> userIds);

    /**
     * 获取用户再乐团的状态
     *
     * @param userIds
     * @return
     */
    List<Map<Integer, String>> queryUserMusicStatus(@Param("userIds") Set<Integer> userIds);

    List<MusicGroupRegisterDto> getMusicGroupRegister(@Param("musicGroupStatusList") List<MusicGroupStatusEnum> musicGroupStatusList, @Param("organIds") String organIds, @Param("tenantId") Integer tenantId);

    List<RegisterSubjectDto> getRegisterSubjectNum(@Param("musicGroupIds") List<String> musicGroupIds,
                                                   @Param("studentMusicGroupStatus") StudentMusicGroupStatusEnum studentMusicGroupStatus,
                                                   @Param("payStatus") Integer payStatus);

    List<MusicGroupRegisterDto> getMusicGroupRegisterOrder(@Param("musicGroupIds") List<String> musicGroupIds);

    List<MusicGroupRegisterDto> getMusicGroupRegisterTotalMoney(@Param("musicGroupIds") List<String> musicGroupIds);

    /**
     * 获取老师的乐团列表
     *
     * @param teacherId
     * @return
     */
    List<MusicGroup> getTeacherMusicGroup(@Param("teacherId") Integer teacherId);

    /**
     * @param userId:
     * @return java.util.List<com.ym.mec.biz.dal.dto.MusicCardDto>
     * @describe 获取学员所在乐团名称
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/9/7
     * @time 14:16
     */
    List<MusicCardDto> queryUserMusicInfos(Integer userId);

    /**
     * @param organId:
     * @return java.util.List<com.ym.mec.biz.dal.dto.MusicCardDto>
     * @describe 获取分部下乐团名称列表
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/9/7
     * @time 14:31
     */
    List<MusicCardDto> queryOrganMusicInfos(@Param("organId") String organId, @Param("tenantId") Integer tenantId);

    /**
     * @param userId:
     * @return java.lang.Integer
     * @describe 获取学员第一个教务老师
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/9/24
     * @time 10:27
     */
    Integer getFirstEduTeacherId(@Param("userId") Integer userId);

    /**
     * @param studentId:
     * @return com.ym.mec.biz.dal.entity.MusicGroup
     * @describe 获取学员关联的乐团
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/9/28
     * @time 13:47
     */
    MusicGroup getStuMusic(Integer studentId);

    /**
     * @param studentId:
     * @param educationalTeacherId:
     * @return com.ym.mec.biz.dal.entity.MusicGroup
     * @describe 获取学员和乐团关联的乐团
     * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
     * @author zouxuan
     * @date 2020/9/28
     * @time 13:54
     */
    MusicGroup getStuEduMusic(@Param("studentId") Integer studentId, @Param("educationalTeacherId") Integer educationalTeacherId);

    /**
     * 根据班级编号获取乐团信息
     *
     * @param classGroupId
     * @return
     * @author zouxuan
     */
    MusicGroup findByClassGroupId(Integer classGroupId);

    /**
     * 根据分部和教务老师查询乐团列表
     *
     * @param educationId
     * @param organIds
     * @return
     */
    List<String> queryIdsByEduIdAndOrganIds(@Param("educationId") Integer educationId, @Param("organIds") String organIds, @Param("tenantId") Integer tenantId);

    /**
     * 获取乐团的教学点
     *
     * @param id
     * @return
     */
    School getMusicGroupSchool(@Param("id") String id);

    /**
     * 获取乐团主管进行中的乐团
     *
     * @param eduTeacherId
     * @return
     */
    List<MusicGroup> getByEduTeacher(@Param("organId") Integer organId, @Param("eduTeacherId") Integer eduTeacherId, @Param("search") String search);

    /**
     * 获取进行中的乐团的教务老师
     *
     * @param organId
     * @return
     */
    List<SysUser> getMusicGroupEduTeacher(@Param("organId") Integer organId);

    /**
     * 获取用户的合作单位
     *
     * @param userId
     * @return
     */
    CooperationOrgan findUserMusicGroupCooperationOrgan(@Param("userId") Integer userId);

    /**
     * 根据ids获取乐团信息
     *
     * @param ids
     * @return
     */
    List<MusicGroup> getMusicGroupByIds(@Param("ids") List<String> ids);

    List<String> getNoClassStudentMusicGroupIds(@Param("organIds") List<Integer> organIds, @Param("tenantId") Integer tenantId);

    CooperationOrgan findCooperationOrganEduTeacher(@Param("id") Integer id);

    /**
     * 获取乐团合作单位进行中 、暂停、关闭 的乐团
     *
     * @param cooperationOrganId
     * @param musicGroupId
     * @return
     */
    Integer getCooperationMusicGroupNum(@Param("cooperationOrganId") Integer cooperationOrganId, @Param("musicGroupId") String musicGroupId);

    /**
     * 获取课程关联教务老师名称
     *
     * @param courseIds
     * @return
     */
    List<Map<Long, String>> queryEduNameMapByCourseId(@Param("courseIds") List<Long> courseIds);

    /**
     * 根据合作单位获取乐团列表
     *
     * @param cooperationOrganId
     * @return
     */
    List<MusicGroup> findByCooperationIdAndStatus(Integer cooperationOrganId);

    /**
     * 根据合作单位获取乐团列表
     *
     * @param cooperationOrganId
     * @return
     */
    List<MusicGroup> findByCooperationId(@Param("cooperationId") Integer cooperationOrganId);

    List<MusicGroup> findByCoopIdAndOrganId(@Param("organIds") String organIds, @Param("coopId") String coopId);

    /**
     * 批量修改乐团主管
     *
     * @param cooperationOrganId
     * @param educationUserId
     */
    void batchUpdateEdu(@Param("cooperationOrganId") Integer cooperationOrganId, @Param("educationUserId") Integer educationUserId);

    /**
     * 获取学生最新的乐团
     *
     * @param userId
     * @return
     */
    MusicGroup getStudentLastMusicGroup(@Param("userId") Integer userId);


    /**
     * 获取回款统计的乐团
     *
     * @param organId
     * @param cooperationOrganIds
     * @return
     */
    List<MusicGroupRegAndMoneyDto> getMusicGroupByStatus(@Param("organId") String organId, @Param("cooperationOrganIds") List<Integer> cooperationOrganIds, @Param("musicGroupName") String musicGroupName, @Param("tenantId") Integer tenantId);

    /**
     * 获取乐团已经过期或者即将过期的学员列表
     * @param params
     * @return
     */
    int countMusicMemberList(Map<String, Object> params);

    /**
     * 获取乐团已经过期或者即将过期的学员列表
     * @param params
     * @return
     */
    List<MusicMemberDto> queryMusicMemberList(Map<String, Object> params);

    int updateIsShowRecordOfStudent(@Param("musicGroupId") String musicGroupId, @Param("isShowRecordOfStudent") Boolean isShowRecordOfStudent);

    /**
     * @describe 查询乐团相关学员数量
     * @author Joburgess
     * @date 2021/8/12 0012
     * @param params:
     * @return java.util.List<java.lang.String>
     */
    List<TeacherMusicStudentOverViewDto> queryTeacherMusicStudentOverView(@Param("musicGroupIds") List<String> musicGroupIds);

    /**
     * 获取在读，进行中的乐团名称
     * @param studentIds
     * @return
     */
    List<Map<Integer, String>> queryNormalGroupName(@Param("studentIds") List<Integer> studentIds);

    /**
     * @description: 乐团剩余时长未排课
     * @param organIds
     * @param tenantId
     * @return java.util.List<java.lang.String>
     * @author zx
     * @date 2022/3/3 15:15
     */
    List<String> queryHasSubCourseTimes(@Param("organIds") String organIds, @Param("tenantId") Integer tenantId);

    /**
     * @description: 获取课程时间消耗异常的学员数
     * @param params
     * @return int
     * @author zx
     * @date 2022/3/4 11:22
     */
    Integer countHasFreeCourseTimes(Map<String, Object> params);

    /**
     * @description: 获取课程时间消耗异常的学员
     * @param params
     * @return java.util.List<com.ym.mec.biz.dal.dto.HasFreeCourseTimesDto>
     * @author zx
     * @date 2022/3/4 11:40
     */
    List<HasFreeCourseTimesDto> queryHasFreeCourseTimes(Map<String, Object> params);

    /**
     * @description: 获取首页退学学员数量
     * @param queryInfo
     * @return java.lang.Integer
     * @author zx
     * @date 2022/3/17 16:20
     */
    Integer countQuitNum(@Param("queryInfo") IndexDataQueryInfo queryInfo);

    //乐团统计数据导出
    Integer countMusicGroupCourseList(@Param("queryInfo") ExportUserAccountQueryInfo queryInfo);

    //乐团统计数据导出
    List<MusicGroupExportDto> exportMusicGroupCourseList(@Param("queryInfo") ExportUserAccountQueryInfo queryInfo);
    //查询
    List<MusicGroupBasicDto> findMusicByCourseIds(@Param("courseIds") List<Long> courseIds);

    /**
    * @description: 获取乐团教务老师
     * @param musicGroupIds
    * @return java.util.Map<java.lang.String,java.lang.String>
    * @author zx
    * @date 2022/12/7 14:00
    */
    List<Map<String, String>> queryMusicEduMap(@Param("musicGroupIds") List<String> musicGroupIds);

    List<Map<Integer, String>> queryUserEduMap(@Param("userIdList") List<Integer> userIdList);

    List<Map<Integer, String>> queryUserCoopMap(@Param("userIdList") List<Integer> userIdList);

    /**
    * @description: 根据合作单位获取进行中的乐团列表
     * @param coopId
    * @return java.util.List<java.lang.String>
    * @author zx
    * @date 2023/5/5 17:08
    */
    List<String> findNormalByCooperationId(@Param("coopId") Integer coopId);

    List<MusicGroup> userMusicGroup(@Param("studentId") Integer studentId, @Param("coopId") Integer coopId, @Param("statusList") List<StudentMusicGroupStatusEnum> statusList);

    List<ExportMusicGroupEchelonDataDto> queryCoopEchelonData(@Param("organIds") List<Integer> organIds);

    List<ExportMusicGroupEchelonDataDto> queryMusicGroupEchelonData(@Param("musicGroupIds") List<String> musicGroupIds);

    List<String> getDelImGroupId();
}
