package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.StudentSubTotalCourseTimesDto;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentStudentCourseDetail;
import com.ym.mec.biz.dal.wrapper.StudentWrapper;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface MusicGroupPaymentStudentCourseDetailDao extends BaseDAO<Long, MusicGroupPaymentStudentCourseDetail> {

    /**
     * @param musicGroupPaymentStudentCourseDetails:
     * @return int
     * @describe 批量更新
     * @author qnc99
     * @date 2020/12/1 0001
     */
    int batchUpdate(@Param("paymentCourseDetails") List<MusicGroupPaymentStudentCourseDetail> musicGroupPaymentStudentCourseDetails);

    /**
     * @param studentId:
     * @param courseType:
     * @param courseMinutes:
     * @return com.ym.mec.biz.dal.entity.MusicGroupPaymentStudentCourseDetail
     * @describe 获取指定学员，指定课程，指定课程时长下的可用缴费记录
     * @author Joburgess
     * @date 2020.11.04
     */
    List<MusicGroupPaymentStudentCourseDetail> getUnUseWithStudentAndCourseTypeAndCourseMinutes(@Param("studentId") Integer studentId,
                                                                                                @Param("courseType") CourseSchedule.CourseScheduleType courseType,
                                                                                                @Param("musicGroupId") String musicGroupId);

    /**
     * @param studentId:
     * @param courseType:
     * @return com.ym.mec.biz.dal.entity.MusicGroupPaymentStudentCourseDetail
     * @describe 获取指定学员，指定课程类型下的可用缴费批次号
     * @author Joburgess
     * @date 2020.11.04
     */
    String getUnUseBatchNoWithStudentAndCourseTypeAndCourseMinutes(@Param("musicGroupId") String musicGroupId,
                                                                   @Param("studentId") Integer studentId,
                                                                   @Param("courseType") CourseSchedule.CourseScheduleType courseType,
                                                                   @Param("batchNos") List<String> batchNos);

    /**
     * @param studentIds:
     * @return java.util.List<com.ym.mec.biz.dal.entity.MusicGroupPaymentStudentCourseDetail>
     * @describe
     * @author Joburgess
     * @date 2020.11.04
     */
    List<MusicGroupPaymentStudentCourseDetail> getUnUseWithStudents(@Param("musicGroupId") String musicGroupId, @Param("studentIds") List<Integer> studentIds);

    int batchInsert(List<MusicGroupPaymentStudentCourseDetail> musicGroupPaymentStudentCourseDetailList);

    /**
     * 查询对象
     *
     * @param musicGroupPaymentCalenderDetailIdList
     * @return
     */
    List<MusicGroupPaymentStudentCourseDetail> queryByMusicGroupPaymentStudentCourseDetailId(List<Long> musicGroupPaymentCalenderDetailIdList);

    /**
     * 获取学员关联的每种课程类型的剩余可排课时长
     *
     * @param musicGroupId
     * @return
     */
    List<StudentSubTotalCourseTimesDto> queryStudentSubTotalCourseTimes(@Param("musicGroupId") String musicGroupId);

    Integer countStudentSubTotalCourseTimes(Map<String, Object> params);

    List<StudentSubTotalCourseTimesDto> pageStudentSubTotalCourseTimes(Map<String, Object> params);

    List<MusicGroupPaymentStudentCourseDetail> findByCalenderAndUserId(@Param("calenderId") String calenderId, @Param("userId") Integer userId);

    /**
     * @param musicGroupId
     * @param courseTypes
     * @param studentIds
     * @return void
     * @description: 校验学员排课时长是否一致
     * @author zx
     * @date 2022/3/3 19:04
     */
    Integer checkCourseTimes(@Param("musicGroupId") String musicGroupId,
                             @Param("courseTypes") List<CourseSchedule.CourseScheduleType> courseTypes,
                             @Param("studentIds") List<Integer> studentIds);

    /**
     * 清除指定乐团学生的剩余未排课时长
     * @param musicGroupId
     * @return
     */
    int clearRemainCourseMinutesByMusicGroupId(String musicGroupId);

    void delByIds(@Param("courseDetailIds") List<Long> courseDetailIds);

    List<MusicGroupPaymentStudentCourseDetail> findByUserIdAndMusicGroupId(@Param("userId") Integer userId, @Param("musicGroupId") String musicGroupId);

    List<StudentWrapper.ClassStudentDto> findStudentCourseDetail(@Param("musicGroupId") String musicGroupId,
                                                                 @Param("studentIds") List<Integer> studentIds,
                                                                 @Param("query") StudentWrapper.ClassStudentQuery query);
}
