package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.entity.MusicGroupQuit;
import com.ym.mec.biz.dal.school.dto.MusicGroupQuitPageDto;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;
import java.util.Set;

public interface MusicGroupQuitDao extends BaseDAO<Long, MusicGroupQuit> {

    /**
     * @param musicGroupId: 乐团编号
     * @return int
     * @describe 统计指定乐团退团人数
     * @author Joburgess
     * @date 2019/10/28
     */
    int countMusicGroupQuitNum(String musicGroupId);

    /**
     * 获取用户退出的乐团
     *
     * @param userId
     * @return
     */
    List<MusicGroupQuit> getUserMusicQuit(@Param("userId") Integer userId);

    /**
     * 获取学生乐团最后一次退团记录
     * @param musicGroupId
     * @param userIds
     * @return
     */
    List<MusicGroupQuit> getQuits(@Param("musicGroupId") String musicGroupId, @Param("userIds") List<Integer> userIds);
    
    /**
     * 获取学生乐团最后一次退团记录
     * @param userId
     * @param musicGroupId
     * @return
     */
    MusicGroupQuit queryByUserIdAndMusicGroupId(@Param("userId") Integer userId, @Param("musicGroupId") String musicGroupId);

    /**
     * 学员是否有待审核的退团记录
     * @param studentIds
     * @param musicGroupId
     * @return
     */
    List<Map<Integer, Long>> queryQuitMapByStudentId(@Param("studentIds") Set<Long> studentIds, @Param("musicGroupId") String musicGroupId);

    /**
    * @description: 获取学校端退团审批列表
     * @param params
    * @return int
    * @author zx
    * @date 2023/5/11 09:38
    */
    int countBySchool(Map<String, Object> params);

    /**
    * @description: 获取学校端退团审批列表
     * @param params
    * @return java.util.List<com.ym.mec.biz.dal.school.dto.MusicGroupQuitPageDto>
    * @author zx
    * @date 2023/5/11 09:39
    */
    List<MusicGroupQuitPageDto> queryPageBySchool(Map<String, Object> params);

    Boolean countRedPointBySchool(@Param("musicGroupIds") List<String> musicGroupIds);

    void finish(@Param("musicGroupId") String musicGroupId);
}
