package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.StudentPaymentOrderExportDto;
import com.ym.mec.biz.dal.entity.OperatingReport;
import com.ym.mec.biz.dal.entity.SellOrder;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.OrderSkuSync;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

public interface SellOrderDao extends BaseDAO<Integer, SellOrder> {

    int batchInsert(@Param("sellOrders") List<SellOrder> sellOrders);

    int batchUpdate(@Param("sellOrders") List<SellOrder> sellOrders);

    List<SellOrder> lockSellOrders(@Param("sellOrderIds") List<Integer> sellOrderIds);

    /**
     * 获取订单的销售列表
     *
     * @param orderId
     * @return
     */
    List<SellOrder> getOrderSellOrder(@Param("orderId") Long orderId);


    /**
     * 获取分部学校的收入支出(乐团订单)
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getMusicGroupMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取分部学校的收入支出(VIP 网管课订单)
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getMusicVipPracticeMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取分部学校的收入支出(外部订单)
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getOutOrderMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取分部学校的收入支出(充值)
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getRechargeMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取分部学校的收入支出(零星支付)
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getSporadicMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 维修单
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getRepairGoodsSellGroupMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 维修单,总收入
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getRepairGoodsSellGroupIncome(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);


    /**
     * 商品销售
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getGoodsSellGroupMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);


    /**
     * 根据订单号删除
     *
     * @param orderId
     * @return
     */
    int deleteByOrderId(@Param("orderId") Long orderId);

    /**
     * 获取乐团订单总收入
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getMusicGroupIncome(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 外部订单总收入
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getOutOrderIncome(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取订单的销售成本
     *
     * @param orderIdList
     * @return
     */
    BigDecimal getOrderSellCost(@Param("orderIdList") List<Long> orderIdList);

    /**
     * 获取声部更换的销售、成本
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<OperatingReport> getSubjectChangeMonthReport(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 获取声部更换的总收入
     *
     * @return
     */
    List<OperatingReport> getSubjectChangeGroupIncome(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);

    /**
     * 统计退货的商品费用
     *
     * @return
     */
    List<OperatingReport> getRefundIncome(@Param("startTime") Date startTime, @Param("endTime") Date endTime, @Param("tenantId") Integer tenantId);


    /**
     * @param sellOrderIds:
     * @return java.util.List<com.ym.mec.biz.dal.entity.SellOrder>
     * @describe 获取指定的销售记录
     * @author Joburgess
     * @date 2020.10.13
     */
    List<SellOrder> getSellOrders(@Param("sellOrderIds") List<Integer> sellOrderIds);

    /**
     * @return java.util.List<com.ym.mec.biz.dal.entity.SellOrder>
     * @describe 获取无批次号商品销售记录编号列表
     * @author Joburgess
     * @date 2020.10.13
     */
    List<Integer> getNoneBatchNoSellOrderIds(Integer tenantId);


    /**
     * 获取订单组合商品的子商品
     *
     * @param orderId
     * @param parentGoodsId
     * @return
     */
    List<SellOrder> getSellOrderByParentGoodsId(@Param("orderId") Long orderId, @Param("parentGoodsId") Integer parentGoodsId);

    /**
     * 获取退货的sellOrder
     *
     * @param orderIdList
     * @return
     */
    List<SellOrder> getRefundSellOrder(@Param("orderIdList") List<Long> orderIdList);

    /**
     * 获取订单的各类型费用
     *
     * @param orderId
     * @return
     */
    StudentPaymentOrderExportDto getFeeByType(@Param("orderId") Long orderId);

    /**
     * 获取订单的各类型费用
     *
     * @param orderIdList
     * @return
     */
    List<StudentPaymentOrderExportDto> queryFeeByType(@Param("orderIdList") List<Long> orderIdList);

    /**
     * 获取打包辅件中乐保
     * @param orderId
     * @return
     */
    BigDecimal getChildRepair(@Param("orderId") Long orderId);

    /**
     * 获取打包辅件中乐保
     * @param orderIdList
     * @return
     */
    List<Map<Long,BigDecimal>> queryChildRepair(@Param("orderIdList") List<Long> orderIdList);

    //统计外部销售收入总金额
    List<Map<Long, String>> sumOutOrderAmount(@Param("outOrderList") List<Long> outOrderList);

    List<SellOrder> querySellOrderByOrderIds(@Param("orderIds") Set<Long> paymentOrderIds);

    /**
    * @description: 获取商城商品分类
     * @param productIdList
    * @return java.util.Map<java.lang.Integer,java.lang.String>
    * @author zx
    * @date 2022/11/7 11:15
    */
    List<Map<Long, String>> getMallProductMap(@Param("productIdList") Set<String> productIdList, @Param("database") String database);

    List<String> getMallProductCategory(@Param("database") String database);

    List<SellOrder> queryByOrganNos(@Param("orderNos") List<String> orderNos);

    void deleteByOrderNo(@Param("orderNo") String orderNo);
}
