package com.ym.mec.biz.dal.dto;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.StudentCourseHomework;
import com.ym.mec.biz.dal.enums.EHomeWorkStatus;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.ImSendTypeEnum;
import com.ym.mec.common.page.QueryInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author liujunchi
 * @date 2023-04-03
 */
@Data
@ApiModel(value = "课程作业包装类")
public class CourseHomeworkWrapper {

    @Data
    @ApiModel(value = "课程作业查询条件")
    public static class CourseHomeworkQuery extends QueryInfo {


        @ApiModelProperty(value = "课程id")
        private Integer courseScheduleId;

        @ApiModelProperty("当月")
        private Date month;

        @ApiModelProperty(value = "开始时间",required = true)
        private Date startTime;

        @ApiModelProperty(value = "结束时间",required = true)
        private Date endTime;

        @ApiModelProperty(value = "老师id")
        private Integer teacherId;

        @ApiModelProperty(value = "课程类型 课后练习 ：MUSIC  小课练习： 包含VIP,PRACTICE")
        private String courseType;

        @ApiModelProperty(value = "乐团id")
        private String musicGroupId;

        @ApiModelProperty(value = "乐团id")
        private List<String> musicGroupIds;

        @ApiModelProperty(value = "班级类型  SINGLE:声部班，MIX:合奏班")
        private String classType;

        @ApiModelProperty(value = "作业练习状态 UNASSIGNED:未布置，ASSIGNED:已布置，DEADLINE:已截止")
        private EHomeWorkStatus homeWorkStatus;


        @ApiModelProperty(value = "作业类型 来源（HOMEWORK,EXTRACURRICULAR,EXTRA）",required = true)
        private ELessonTrainingType type;
    }

    @Data
    @ApiModel(value = "学校端学员请假列表")
    public static class StudentLeaveQuery extends QueryInfo {

        @ApiModelProperty(value = "课程id")
        private Integer courseScheduleId;

        @ApiModelProperty(value = "班级id")
        private Integer classGroupId;

        @ApiModelProperty(value = "课程类型")
        private String courseScheduleType;

        @ApiModelProperty(value = "班级类型")
        private String classGroupType;

        @ApiModelProperty(value = "开始时间(年月日)",required = true)
        private String startTime;

        @ApiModelProperty(value = "结束时间(年月日)",required = true)
        private String endTime;

        @ApiModelProperty(value = "乐团id")
        private String musicGroupIds;
    }

    @Data
    @ApiModel(value = "学校端学员请假列表")
    public static class StudentLeaveResult {
        @ApiModelProperty(value = "上课时间")
        private String classDate;

        @ApiModelProperty(value = "上课开始时间")
        private String startClassTime;

        @ApiModelProperty(value = "上课结束时间")
        private String endClassTime;

        @ApiModelProperty(value = "请假时间")
        private String leaveTime;

        @ApiModelProperty(value = "班级名称")
        private String classGroupName;

        @ApiModelProperty(value = "课程类型")
        private CourseSchedule.CourseScheduleType courseScheduleType;

        @ApiModelProperty(value = "请假原因")
        private String remark;

        @ApiModelProperty(value = "学生姓名")
        private String username;

        @ApiModelProperty(value = "学生头像")
        private String avatar;

        @ApiModelProperty(value = "手机号")
        private String phone;
    }



    @Data
    @ApiModel(value = "课程作业列表")
    public static class CourseHomeworkList {

        @ApiModelProperty(value = "课程id")
        private Integer courseScheduleId;

        @ApiModelProperty(value = "作业id")
        private Long courseHomeworkId;

        @ApiModelProperty(value = "课程日期")
        private Date classDate;

        @ApiModelProperty(value = "开始时间")
        private Date startTime;

        @ApiModelProperty(value = "结束时间")
        private Date endTime;

        @ApiModelProperty(value = "作业练习状态 UNASSIGNED:未布置，ASSIGNED:已布置，DEADLINE:已截止")
        private EHomeWorkStatus homeWorkStatus;

        @ApiModelProperty(value = "老师id")
        private Integer teacherId;

        @ApiModelProperty("老师名")
        private String teacherName;

        @ApiModelProperty("课程名称")
        private String courseScheduleName;

        @ApiModelProperty("老师头像")
        private String teacherImg;

        @ApiModelProperty(value = "乐团id")
        private String musicGroupId;

        @ApiModelProperty(value = "乐团名称")
        private String musicGroupName;

        @ApiModelProperty(value = "乐团图片")
        private String musicGroupImg;

        @ApiModelProperty(value = "班级类型  NORMAL:声部班，MIX:合奏班")
        private String classType;

        @ApiModelProperty(value = "声部名称")
        private String subjectName;

        @ApiModelProperty("班级id")
        private Integer classGroupId;

        @ApiModelProperty("班级名称")
        private String classGroupName;

        @ApiModelProperty("教学形式（线上、线下）")
        private String teachMode;

        @ApiModelProperty("学生人数")
        private Integer studentNum;

        @ApiModelProperty("已完成人数")
        private Integer finishNum;

        @ApiModelProperty("布置时间")
        private Date assignTime;

        @ApiModelProperty("截止时间")
        private Date expiryDate;

        @ApiModelProperty("完成状态 false 未完成 true 已完成")
        private Boolean finishStatus;

        @ApiModelProperty("提交时间")
        private Date submitTime;

        @ApiModelProperty(value = "作业类型 来源（HOMEWORK,EXTRACURRICULAR,EXTRA）",required = true)
        private ELessonTrainingType type;

        @ApiModelProperty("是否可布置")
        private Boolean assignFlag;

        private String studentIdList;


        @ApiModelProperty("练习内容")
        private List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> trainingDetailList;

        @ApiModelProperty("练习组内容")
        private List<StudentLessonTrainingStat> trainingGroupList;
    }


    @Data
    @ApiModel(value = "StudentLessonTrainingStat-练习组统计")
    public static class StudentLessonTrainingStat {

        @ApiModelProperty("练习组")
        private Integer group;

        @ApiModelProperty("声部名 逗号隔开")
        private String subjectName;

        @ApiModelProperty("练习组人数")
        private Integer studentNum;

        @ApiModelProperty("应提交人数")
        private Integer shouldSubmitNum;

        @ApiModelProperty("提交人数")
        private Integer submitNum;

        @ApiModelProperty("达标人数")
        private Integer passNum;


        @ApiModelProperty("练习内容")
        private List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> trainingDetailList;
    }


    @Data
    @ApiModel(value = "StudentCourseHomeworkQuery-学生课程作业详情")
    public static class StudentCourseHomeworkQuery {

        @ApiModelProperty(value = "用户ID")
        private Long userId;

        @ApiModelProperty(value = "课程编号",required = true)
        @NotNull(message = "课程编号不能为空")
        private Integer courseScheduleId;

        @ApiModelProperty(value = "作业类型 来源（HOMEWORK,EXTRACURRICULAR,EXTRA）",required = true)
        @NotNull(message = "作业类型不能为空")
        private ELessonTrainingType type;

    }


    @Data
    @ApiModel(value = "AddStudentHomeworkRecord-添加学生作业记录")
    public static class AddStudentHomeworkRecord {

        @ApiModelProperty(value = "用户ID")
        private Long userId;

        @ApiModelProperty(value = "作业详情id",required = true)
        @NotNull(message = "作业详情id不能为空")
        private Long id;

    }


    @Data
    @ApiModel(value = "StudentCourseHomeworkDetail-学生课程作业详情")
    public static class StudentCourseHomeworkDetail {

        @ApiModelProperty(value = "学生作业ID")
        private Integer studentCourseHomeworkId;

        @ApiModelProperty(value = "课程ID")
        private Long courseScheduleId;

        @ApiModelProperty(value = "课程名称")
        private String courseName;

        @ApiModelProperty(value = "上课开始时间")
        private Date startClassTime;

        @ApiModelProperty(value = "作业过期时间")
        private String expiryDate;

        @ApiModelProperty(value = "结束时间")
        private Date endClassTime;

        @ApiModelProperty(value = "乐团id")
        private String musicGroupId;

        @ApiModelProperty(value = "乐团名称")
        private String musicGroupName;

        @ApiModelProperty(value = "乐团图片")
        private String musicGroupImg;

        @ApiModelProperty("班级id")
        private Integer classGroupId;

        @ApiModelProperty("班级名称")
        private String classGroupName;

        @ApiModelProperty("教学形式（线上、线下）")
        private String teachMode;

        @ApiModelProperty("完成状态 false 未完成 true 已完成")
        private Boolean finishStatus;


        @ApiModelProperty(value = "作业类型 来源（HOMEWORK,EXTRACURRICULAR,EXTRA）")
        private ELessonTrainingType type;

        @ApiModelProperty(value = "练习内容")
        private List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetail;
    }


    @Data
    @ApiModel(value = "WebExtraCourseHomework-课外作业列表")
    public static class WebExtraCourseHomework {

        /**  */
        private Long id;

        private String musicGroupId;

        /** 老师编号 */
        @ApiModelProperty(value="等级")
        private Integer teacherId;

        private SysUser teacher = new SysUser();

        /** 学生列表 */
        @ApiModelProperty(value="学生列表")
        private String studentIdList;

        @ApiModelProperty("班级id")
        private Integer classGroupId;

        @ApiModelProperty("作业来源分类")
        private String groupType;


        private String type;

        /** 批次号 */
        @ApiModelProperty(value="批次号")
        private String batchNo;

        /** 标题 */
        @ApiModelProperty(value="标题")
        private String title;

        /** 附件地址（多个用逗号分隔） */
        @ApiModelProperty(value="附件地址（多个用逗号分隔）")
        private String attachments;

        /** 作业内容 */
        @ApiModelProperty(value="作业内容")
        private String content;

        /** 截止日期 */
        @ApiModelProperty(value="截止日期")
        private java.util.Date expireDate;

        /** 完成人数 */
        @ApiModelProperty(value="完成人数")
        private Integer completedNum;

        /** 预计人数 */
        @ApiModelProperty(value="预计人数")
        private Integer expectNum;

        /**  */
        private java.util.Date createTime;

        /**  */
        private java.util.Date updateTime;

        @ApiModelProperty(value="是否及时回复")
        private int isReplied;

        @ApiModelProperty(value="是否提交")
        private int isSubmit;

        private int status;

        private String organName;

        private ImSendTypeEnum msgType = ImSendTypeEnum.TXT;

        /** 伴奏列表（多个用逗号分隔） */
        @ApiModelProperty(value="伴奏列表（多个用逗号分隔）")
        private String musicScoreId;

        private String musicScoreName;

        private List<MusicScoreSubjectDto> musicScoreSubjectDtos;


        @ApiModelProperty("乐团名")
        private String musicGroupName;

        @ApiModelProperty("班级名")
        private String classGroupName;

        private String versionTag;


        @ApiModelProperty("布置时间")
        private Date assignTime;

        @ApiModelProperty("练习内容")
        private List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> trainingDetailList;
    }




    @Data
    @ApiModel(value = "StudentHomeworkRecordStat-学生作业记录统计")
    public static class StudentHomeworkRecordStat {

        @ApiModelProperty(value = "全部")
        private Integer allNum;

        @ApiModelProperty(value = "已完成")
        private Integer finishNum;

        @ApiModelProperty(value = "未完成")
        private Integer unFinishNum;

        @ApiModelProperty(value = "列表")
        private  List<StudentCourseHomework> studentCourseHomeworkList;

    }
}
