package com.ym.mec.biz.dal.dto;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.StandardEnum;
import com.ym.mec.common.page.QueryInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 练习内容
 * 2023-03-31 14:18:47
 */
@ApiModel(value = "StudentLessonTrainingDetailWrapper对象", description = "练习内容查询对象")
public class StudentLessonTrainingDetailWrapper {


    @Data
    @ApiModel(" AddStudentLessonTrainingDetail-练习内容")
    public static class AddStudentLessonTrainingDetail {

        @ApiModelProperty("曲目id")
        private Long musicScoreId;

        @ApiModelProperty("曲目名称")
        private String musicScoreName;

        @ApiModelProperty("练习速度")
        private Integer trainingSpeed;

        @ApiModelProperty("标准练习次数")
        private Integer times;

        @ApiModelProperty("分谱")
        private Integer partIndex;

        @ApiModelProperty("声部")
        private Integer subjectId;

        @ApiModelProperty("开始小节")
        private Integer start;

        @ApiModelProperty("结束小节")
        private Integer end;

        @ApiModelProperty("作业类型,VIDEO,MUSIC_SCORE")
        private String homeworkType = "MUSIC_SCORE";
    }

    @Data
	@ApiModel(" StudentLessonTrainingDetail-练习内容")
    public static class StudentLessonTrainingDetail {


        @ApiModelProperty("主键ID")
        private Long id;

        @ApiModelProperty("用户ID")
        private Long userId;

        @ApiModelProperty("练习编号")
        private Long courseHomeworkId;

        @ApiModelProperty("曲目id")
        private Long musicScoreId;

        @ApiModelProperty("曲目名称")
        private String  musicScoreName;

        @ApiModelProperty("作业类型,VIDEO,MUSIC_SCORE")
        private String homeworkType = "MUSIC_SCORE";

        @ApiModelProperty("分谱")
        private Integer partIndex;

        @ApiModelProperty("开始小节")
        private Integer start;

        @ApiModelProperty("结束小节")
        private Integer end;

        @ApiModelProperty("练习速度")
        private Integer trainingSpeed;

        @ApiModelProperty("已练习次数")
        private Integer trainingTimes;

        @ApiModelProperty("练习状态")
        private StandardEnum trainingStatus;

        @ApiModelProperty("标准练习次数")
        private Integer times;

        @ApiModelProperty("声部id")
        private Long subjectId;

        @ApiModelProperty("声部名称")
        private String subjectName;

        @ApiModelProperty("作业分组")
        private Integer group;

        @ApiModelProperty("播放地址")
        private String content;

        @ApiModelProperty("资源是否有效")
        private Boolean validFlag = false;
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }
	}



    @Data
    @ApiModel(" StudentLessonTrainingQuery-学生列表")
    public static class StudentLessonTrainingQuery {


        @ApiModelProperty(value = "用户ID")
        private Long userId;

        @ApiModelProperty(value = "课程编号",required = true)
        @NotNull(message = "课程编号不能为空")
        private Integer courseScheduleId;

        @ApiModelProperty(value = "作业类型 来源（HOMEWORK,EXTRACURRICULAR,EXTRA）",required = true)
        @NotNull(message = "作业类型不能为空")
        private ELessonTrainingType type;

        @ApiModelProperty(value = "声部id")
        private Integer subjectId;

        @ApiModelProperty(value = "true:完成 false未完成",hidden = true)
        private Boolean standardFlag;

        @ApiModelProperty(value = "提交开始时间",required = true)
        private String submitStartTime;

        @ApiModelProperty(value = "提交结束时间",required = true)
        private String submitEndTime;

        @ApiModelProperty(value = "提交状态",required = true)
        private Integer submitStatus;

        @ApiModelProperty(value = "达标状态,'NOT_STANDARD(\"未达标\"), STANDARD(\"已达标\"), EXCELLENT(\"优秀\")")
        private String standardStatus;

        @ApiModelProperty(value = "模糊查询")
        private String search;

	}


    @Data
    @ApiModel(" HomeworkGroupInfoQuery-作业内容查询")
    public static class HomeworkGroupInfoQuery {


        @ApiModelProperty(value = "课程编号",required = true)
        @NotNull(message = "课程编号不能为空")
        private Integer courseScheduleId;

        @ApiModelProperty(value = "作业类型 来源（HOMEWORK,EXTRACURRICULAR,EXTRA）",required = true)
        @NotNull(message = "作业类型不能为空")
        private ELessonTrainingType type;

        @ApiModelProperty(value = "组号",required = true)
        @NotNull(message = "组号不能为空")
        private Integer group;

    }


    @Data
    @ApiModel(" StudentLessonTraining-学生练习内容查询对象")
    public static class StudentLessonTraining {


        @ApiModelProperty(value = "用户ID")
        private Long userId;

        @ApiModelProperty(value = "用户名")
        private String userName;

        @ApiModelProperty(value = "头像")
        private String avatar;

        @ApiModelProperty("是否是vip")
        private Boolean vipFlag;

        @ApiModelProperty("声部id")
        private Integer subjectId;

        @ApiModelProperty("声部名称")
        private String subjectName;

        @ApiModelProperty(value = "0未达标、1达标、2优秀",required = true)
        private StandardEnum standardFlag;

        @ApiModelProperty(value = "是否完成")
        private Boolean finishFlag;

        @ApiModelProperty(value = "练习内容")
        private List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetail;
    }

}
