package com.ym.mec.biz.dal.entity;

import com.ym.mec.biz.dal.dto.CourseTimeDto;
import com.ym.mec.biz.dal.enums.ClassGroupTypeEnum;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.common.entity.BaseEntity;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ClassGroupDto extends BaseEntity {

	@ApiModelProperty(value = "是否确认生成课程", required = true)
	private Boolean confirmGenerate = false;

	@ApiModelProperty(value = "是否转换课", required = true)
	private Boolean convertFlag = true;

	@ApiModelProperty(value = "排课校验方式：0按总时长，1按课次", required = true)
	private int checkType = 0;

	@ApiModelProperty(value = "选择的需要转换的课程编号", required = true)
	private String courseIds;

	@ApiModelProperty(value = "班级编号", required = true)
	private Integer classGroupId;

	@NotNull(message = "课程组类型不能为空")
	@ApiModelProperty(value = "课程组类型", required = true)
	private GroupType groupType;

	@ApiModelProperty(value = "确认0元课酬", required = true)
	private Boolean allowZeroSalary = false;

	/**  */
	@ApiModelProperty(value = "乐团编号", required = true)
	private String musicGroupId;

	/**  */
	@ApiModelProperty(value = "科目编号(多个,号分割)", required = true)
	private String subjectIdList;

	@NotBlank(message = "学生编号不能为空")
	@ApiModelProperty(value = "学生编号(多个,号分割)", required = true)
	private String userIds;

	/** 班级名称 */
	@ApiModelProperty(value = "班级名称", required = true)
	private String name;

	/** 预计招生人数 */
	@ApiModelProperty(value = "预计招生人数", required = true)
	private Integer expectStudentNum;

	@ApiModelProperty(value = "学生数量", required = false)
	private Integer studentNum;

	/** 班级类型(普通班级、合奏班级) */
	@ApiModelProperty(value = "课程类型", required = true)
	@NotNull(message = "课程类型不能为空")
	private CourseSchedule.CourseScheduleType courseScheduleType = CourseSchedule.CourseScheduleType.HIGH_ONLINE;

	/** 班级类型(普通班级、合奏班级) */
	@ApiModelProperty(value = "课程类型", required = true)
	@NotNull(message = "课程类型不能为空")
	private ClassGroupTypeEnum type;

	@ApiModelProperty(value = "班级老师设置", required = true)
	private List<ClassGroupTeacherMapper> classGroupTeacherMapperList;

	@ApiModelProperty(value = "排课周期", required = true)
	private List<CourseTimeDto> courseTimeDtoList;

	@ApiModelProperty(value = "获取可转换的课程时长", required = true)
	Map<String, Integer> courseTypeMinutesMap;

	@ApiModelProperty(value = "老师课酬", required = true)
	List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();

	@ApiModelProperty(value = "学员课程关联", required = true)
	List<CourseScheduleStudentPayment> courseScheduleStudentPayments = new ArrayList<>();

	public ClassGroupTypeEnum getType() {
		return type;
	}

	public CourseSchedule.CourseScheduleType getCourseScheduleType() {
		return courseScheduleType;
	}

	public void setCourseScheduleType(CourseSchedule.CourseScheduleType courseScheduleType) {
		this.courseScheduleType = courseScheduleType;
	}

	public void setType(ClassGroupTypeEnum type) {
		this.type = type;
	}

	public Boolean getConfirmGenerate() {
		return confirmGenerate;
	}

	public void setConfirmGenerate(Boolean confirmGenerate) {
		this.confirmGenerate = confirmGenerate;
	}

	public List<CourseScheduleStudentPayment> getCourseScheduleStudentPayments() {
		return courseScheduleStudentPayments;
	}

	public void setCourseScheduleStudentPayments(List<CourseScheduleStudentPayment> courseScheduleStudentPayments) {
		this.courseScheduleStudentPayments = courseScheduleStudentPayments;
	}

	public List<CourseScheduleTeacherSalary> getCourseScheduleTeacherSalaryList() {
		return courseScheduleTeacherSalaryList;
	}

	public void setCourseScheduleTeacherSalaryList(List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList) {
		this.courseScheduleTeacherSalaryList = courseScheduleTeacherSalaryList;
	}

	public Boolean getConvertFlag() {
		return convertFlag;
	}

	public void setConvertFlag(Boolean convertFlag) {
		this.convertFlag = convertFlag;
	}

	public int getCheckType() {
		return checkType;
	}

	public void setCheckType(int checkType) {
		this.checkType = checkType;
	}

	public String getCourseIds() {
		return courseIds;
	}

	public void setCourseIds(String courseIds) {
		this.courseIds = courseIds;
	}

	public Map<String, Integer> getCourseTypeMinutesMap() {
		return courseTypeMinutesMap;
	}

	public void setCourseTypeMinutesMap(Map<String, Integer> courseTypeMinutesMap) {
		this.courseTypeMinutesMap = courseTypeMinutesMap;
	}

	public Boolean getAllowZeroSalary() {
		return allowZeroSalary;
	}

	public void setAllowZeroSalary(Boolean allowZeroSalary) {
		this.allowZeroSalary = allowZeroSalary;
	}

	public Integer getStudentNum() {
		return studentNum;
	}

	public void setStudentNum(Integer studentNum) {
		this.studentNum = studentNum;
	}

	public List<ClassGroupTeacherMapper> getClassGroupTeacherMapperList() {
		return classGroupTeacherMapperList;
	}

	public void setClassGroupTeacherMapperList(List<ClassGroupTeacherMapper> classGroupTeacherMapperList) {
		this.classGroupTeacherMapperList = classGroupTeacherMapperList;
	}

	public Integer getClassGroupId() {
		return classGroupId;
	}

	public void setClassGroupId(Integer classGroupId) {
		this.classGroupId = classGroupId;
	}

	public GroupType getGroupType() {
		return groupType;
	}

	public void setGroupType(GroupType groupType) {
		this.groupType = groupType;
	}

	public String getMusicGroupId() {
		return musicGroupId;
	}

	public void setMusicGroupId(String musicGroupId) {
		this.musicGroupId = musicGroupId;
	}

	public String getSubjectIdList() {
		return subjectIdList;
	}

	public void setSubjectIdList(String subjectIdList) {
		this.subjectIdList = subjectIdList;
	}

	public String getUserIds() {
		return userIds;
	}

	public void setUserIds(String userIds) {
		this.userIds = userIds;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Integer getExpectStudentNum() {
		return expectStudentNum;
	}

	public void setExpectStudentNum(Integer expectStudentNum) {
		this.expectStudentNum = expectStudentNum;
	}

	public List<CourseTimeDto> getCourseTimeDtoList() {
		return courseTimeDtoList;
	}

	public void setCourseTimeDtoList(List<CourseTimeDto> courseTimeDtoList) {
		this.courseTimeDtoList = courseTimeDtoList;
	}
}
