package com.ym.mec.biz.dal.entity;

import com.ym.mec.biz.dal.enums.ExpenditureTypeEnum;
import com.ym.mec.biz.dal.enums.FeeProjectEnum;
import com.ym.mec.common.entity.BaseEntity;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Objects;

import static com.ym.mec.biz.dal.enums.ExpenditureTypeEnum.REFUND;

/**
 * 对应数据库表(financial_expenditure):
 */
@Data
public class FinancialExpenditure {

	private Integer id;
	/**  */
	@ApiModelProperty(value = "批次号",required = false)
	private String batchNo;
	/**  */
	@ApiModelProperty(value = "oa审批类型",required = false)
	private Integer processId;
	/**  */
	@ApiModelProperty(value = "财务流程编号",required = false)
	private String financialProcessNo;
	/**  */
	@ApiModelProperty(value = "钉钉流程编号",required = false)
	private String dingtalkProcessNo;
	/**  */
	@ApiModelProperty(value = "分部id",required = false)
	private Integer organId;
	/**  */
	@ApiModelProperty(value = "学校/合作单位id",required = false)
	private Integer cooperationOrganId;
	/**  */
	@ApiModelProperty(value = "申请人",required = false)
	private String applyUser;
	/**  */
	@ApiModelProperty(value = "申请人",required = false)
	private Integer applyUserId;

	@ApiModelProperty(value = "支出类型（1：固定费用、2：变动费用:3：业务退费）",required = false)
	private ExpenditureTypeEnum type = REFUND;

	@ApiModelProperty(value = "费用项目",required = false)
	private FeeProjectEnum feeProject = FeeProjectEnum.REFUND;
	/**  */
	@ApiModelProperty(value = "费用",required = false)
	private BigDecimal amount = BigDecimal.ZERO;
	/**  */
	@ApiModelProperty(value = "费用明细",required = false)
	private String itemDetail;
	/**  */
	@ApiModelProperty(value = "费用分摊日期",required = false)
	private String effectiveTime;
	/**  */
	@ApiModelProperty(value = "审批通过时间",required = false)
	private Date aprovalTime;

	@ApiModelProperty(value = "当月实际金额",required = false)
	private BigDecimal currentAmount = BigDecimal.ZERO;;
	/**  */
	@ApiModelProperty(value = "支付时间",required = false)
	private Date paymentTime;
	/**  */
	@ApiModelProperty(value = "事由",required = false)
	private String cause;

	@ApiModelProperty(value = "分部",required = false)
	private String organName;

	@ApiModelProperty(value = "学校/合作单位",required = false)
	private String cooperationOrganName;

	@ApiModelProperty(value = "费用类型Str",required = false)
	private String feeType;

	@ApiModelProperty(value = "费用项目str",required = false)
	private String feeProjectItem;

	@ApiModelProperty(value = "收款帐号",required = false)
	private String merNo;

	@ApiModelProperty(value = "开户行",required = false)
	private String bankAddress;

	@ApiModelProperty(value = "收款账户名",required = false)
	private String merNoName;
	/**  */
	private Date createTime;
	/**  */
	private Date updateTime;
	/**  */
	private Integer delFlag;

	private Long vipGroupId;

	private String groupType;

	private String musicGroupId;

	private String returnFeeType;

	private Integer studentId;

	private boolean refundInstrumentFeeFlag;

	private boolean refundTeachingAssistantsFeeFlag;

	private boolean maintenanceFeeFlag;

	private Integer tenantId = TenantContextHolder.getTenantId();

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		FinancialExpenditure that = (FinancialExpenditure) o;
		return musicGroupId.equals(that.musicGroupId) && studentId.equals(that.studentId);
	}

	@Override
	public int hashCode() {
		return Objects.hash(musicGroupId, studentId);
	}
}
