package com.ym.mec.biz.dal.entity;

import com.ym.mec.biz.dal.enums.GoodsType;
import com.ym.mec.biz.dal.enums.StockType;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.math.BigDecimal;
import java.util.List;

/**
 * 对应数据库表(goods):
 */
@Data
public class Goods {

	/** 商品编号 */
	@ApiModelProperty(value = "商品编号",required = false)
	private Integer id;

	/** 产品分类 */
	@ApiModelProperty(value = "产品分类",required = false)
	private Integer goodsCategoryId;

	/** 产品分类 */
	@ApiModelProperty(value = "产品分类名称",required = false)
	private String goodsCategoryName;

	/** 商品货号 */
	@ApiModelProperty(value = "商品货号",required = false)
	private String sn;

	/** 商品名称 */
	@ApiModelProperty(value = "商品名称",required = false)
	private String name;

	/** 品牌 */
	@ApiModelProperty(value = "品牌",required = false)
	private String brand;

	/** 规格 */
	@ApiModelProperty(value = "规格",required = false)
	private String specification;

	/** 展示图片的地址 */
	@ApiModelProperty(value = "展示图片的地址",required = false)
	private String image;

	/** 库存数量 */
	@ApiModelProperty(value = "库存数量",required = false)
	private Integer stockCount;

//	@ApiModelProperty(value = "税务库存")
//	private Integer taxStockCount;

	/** 总销量数 */
	@ApiModelProperty(value = "总销量数",required = false)
	private Integer sellCount;

	/** 市场销售价 */
	@ApiModelProperty(value = "市场销售价",required = false)
	private BigDecimal marketPrice;

	/** 折扣价 */
	@ApiModelProperty(value = "商品零售价",required = false)
	private BigDecimal discountPrice;

	/** 团购价 */
	@ApiModelProperty(value = "团购价",required = false)
	private BigDecimal groupPurchasePrice;

	@ApiModelProperty(value = "商品价格1")
	private BigDecimal costPrice;

	/** 协议成本价 */
	@ApiModelProperty(value = "协议成本价",required = false)
	private BigDecimal agreeCostPrice;

	@ApiModelProperty(value = "分部成本价",required = false)
	private BigDecimal organCostPrice;

	/** 页面简介 */
	@ApiModelProperty(value = "页面简介",required = false)
	private String brief;

	/** 商品的详细描述 */
	@ApiModelProperty(value = "商品的详细描述",required = false)
	private String desc;

	/** 是否是新品（1，是  0，否） */
	@ApiModelProperty(value = "是否是新品（1，是  0，否）",required = false)
	private YesOrNoEnum isNew;

	/** 是否置顶（1，置顶   0，不置顶） */
	@ApiModelProperty(value = "是否置顶（1，置顶   0，不置顶）",required = false)
	private YesOrNoEnum isTop;

	/** 状态（1，上架  0，下架） */
	@ApiModelProperty(value = "状态（1，上架  0，下架）",required = false)
	private YesOrNoEnum status;

	/** 备注 */
	@ApiModelProperty(value = "备注",required = false)
	private String memo;

	/** 发布时间 */
	@ApiModelProperty(value = "发布时间",required = false)
	private java.util.Date publishTime;

	/** 发布时间 */
	@ApiModelProperty(value = "附件商品列表编号（用逗号分开）",required = false)
	private String complementGoodsIdList;

	/** 发布时间 */
//	@ApiModelProperty(value = "附件商品详情",required = false)
//	private String complementGoodsJson;

	/** 辅件列表 */
	@ApiModelProperty(value = "辅件列表(子商品)",required = false)
	private List<Goods> goodsList;

	/** 声部列表编号 */
	private String subjectIds;

	/** 创建时间 */
	private java.util.Date createTime;

	/** 修改时间 */
	private java.util.Date updateTime;

	@ApiModelProperty(value = "商品类型", required = false)
	private GoodsType type;

	@ApiModelProperty(value = "备查货号，进货渠道")
	private String supplyChannel;

	@ApiModelProperty(value = "客户端展示分部")
	private String studentShowOrganId;

	@ApiModelProperty(value = "教务端展示分部")
	private String educationShowOrganId;

	@ApiModelProperty(value = "课程收费团展示分部")
	private String courseFeeShowOrganId;

	@ApiModelProperty(value = "会员收费团展示分部")
	private String memberFeeShowOrganId;
	
	@ApiModelProperty(value = "免费团展示分部")
	private String freeFeeShowOrganId;

	@ApiModelProperty(value = "乐器置换展示分部")
	private String replacementShowOrganId;

	@ApiModelProperty(value = "客户端展示分部")
	private String studentShowOrganName;

	@ApiModelProperty(value = "教务端展示分部")
	private String educationShowOrganName;

	@ApiModelProperty(value = "课程收费团展示分部")
	private String courseFeeShowOrganName;

	@ApiModelProperty(value = "会员收费团展示分部")
	private String memberFeeShowOrganName;
	
	@ApiModelProperty(value = "免费团展示分部")
	private String freeFeeShowOrganName;

	@ApiModelProperty(value = "乐器置换展示分部")
	private String replacementShowOrganName;

	@ApiModelProperty(value = "库存类型")
	private StockType stockType;

	@ApiModelProperty(value = "库存预警")
	private YesOrNoEnum stockWarning;

	@ApiModelProperty(value = "是否是组合商品，1：是，0：不是")
	private Boolean groupGoods;

	@ApiModelProperty(value = "是否删除")
	private Boolean delFlag;

	private Integer tenantId = TenantContextHolder.getTenantId();

	private String childId;

	private String childName;

	private String childSn;

	private String childOrganCostPrice;

	private String brandName;

	@ApiModelProperty(value = "被复制的次数")
	private Integer copyNum;

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this);
	}
}
