package com.ym.mec.biz.dal.entity;

import io.swagger.annotations.ApiModelProperty;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import lombok.Data;
import org.apache.commons.lang3.builder.ToStringBuilder;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.dal.enums.PaymentChannelTypeEnum;
import com.ym.mec.common.entity.BaseEntity;

import org.springframework.data.annotation.Version;

/**
 * 对应数据库表(student_payment_order):
 */
@Data
public class StudentPaymentOrder extends BaseEntity {

	/**  */
	private Long id;
	
	private GroupType groupType;
	
	@Version
	private Integer version;
	/**  */
	@ApiModelProperty(value = "用户编号",required = true)
	private Integer userId;

	@ApiModelProperty(value = "分部编号",required = true)
	private Integer organId;

	@ApiModelProperty(value = "分配分部编号",required = true)
	private Integer routingOrganId;

	/** 订单类型(报名、续费、小课购买、其他) */
	@ApiModelProperty(value = "订单类型",required = true)
	private OrderTypeEnum type;
	
	/** 预计金额 */
	@ApiModelProperty(value = "预计金额",required = true)
	private BigDecimal expectAmount = BigDecimal.ZERO;
	
	/** 实际金额 */
	@ApiModelProperty(value = "实际金额",required = true)
	private BigDecimal actualAmount = BigDecimal.ZERO;

	/** 公户金额 */
	@ApiModelProperty(value = "公户金额",required = true)
	private BigDecimal comAmount = BigDecimal.ZERO;

	/** 私户金额 */
	@ApiModelProperty(value = "私户金额",required = true)
	private BigDecimal perAmount = BigDecimal.ZERO;
	
	@ApiModelProperty(value = "余额付款金额",required = true)
	private BigDecimal balancePaymentAmount = BigDecimal.ZERO;
	
	/** 交易流水号 */
	@ApiModelProperty(value = "交易流水号",required = true)
	private String transNo;
	
	/** 交易状态 */
	@ApiModelProperty(value = "交易状态",required = true)
	private DealStatusEnum status;
	
	/** 备注 */
	@ApiModelProperty(value = "备注",required = true)
	private String memo;
	
	/**  */
	private java.util.Date createTime;
	
	/**  */
	private java.util.Date updateTime;
	
	/** 付款渠道 */
	@ApiModelProperty(value = "付款渠道",required = true)
	private String paymentChannel;
	
	/** 业务渠道 */
	@ApiModelProperty(value = "业务渠道",required = true)
	private String paymentBusinessChannel;
	
	/** 付款账户 */
	@ApiModelProperty(value = "付款账户",required = true)
	private String paymentAccountNo;
	
	/** 订单号 */
	@ApiModelProperty(value = "订单号",required = true)
	private String orderNo;
	
	/**  */
	@ApiModelProperty(value = "乐团编号",required = true)
	private String musicGroupId;
	
	/**  */
	@ApiModelProperty(value = "课程组编号",required = true)
	private Integer classGroupId;

	private Date payTime;

	//收款账户
	private String merNos;

	@ApiModelProperty(value = "收货状态，NO_RECEIVE 未确认,MANUAL_RECEIVE 手动确认,AUTO_RECEIVE 自动确认",required = true)
	private String receiveStatus;

	private PaymentChannelTypeEnum paymentChannelType;
	
	private SysUser user = new SysUser();

	//乐器优惠金额
	private BigDecimal remitFee;

	//课程优惠金额
	private BigDecimal courseRemitFee;
	
	private String batchNo;

	@ApiModelProperty(value = "优惠券编号列表",required = true)
	private String couponCodeId;

	@ApiModelProperty(value = "优惠券减免总额",required = true)
	private BigDecimal couponRemitFee = BigDecimal.ZERO;

	private String activityId;

	private Integer activityBuyNum = 0;

	@ApiModelProperty(value = "缴费项目编号",required = true)
	private Long calenderId;

	private Integer cooperationId;

	// 需要关闭的订单号
	private List<String> closeOrderNoList;
}
