package com.ym.mec.biz.dal.entity;

import com.ym.mec.biz.dal.enums.TeachModeEnum;
import com.ym.mec.common.entity.BaseEntity;
import com.ym.mec.common.enums.BaseEnum;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.PastOrPresent;
import javax.validation.constraints.Positive;
import java.util.Date;

@Data
public class StudentVisit extends BaseEntity {

    public enum VisiterTypeEnum implements BaseEnum<String, TeachModeEnum> {
        TEACHER("TEACHER", "指导老师"), EDU_TEACHER("EDU_TEACHER", "乐团主管");

        private String code;

        private String msg;

        VisiterTypeEnum(String code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getMsg() {
            return msg;
        }

        public void setMsg(String msg) {
            this.msg = msg;
        }

        @Override
        public String getCode() {
            return this.code;
        }
    }

    public enum FeedbackTypeEnum implements BaseEnum<String,FeedbackTypeEnum> {
        SLEEPY("SLEEPY", "沉睡"),
        LOST("LOST", "流失"), PAUSE("PAUSE", "暂停"),NORMAL("NORMAL", "在读"),
        OTHER("OTHER", "其他"),HOLIDAY_COURSE("HOLIDAY_COURSE", "假期课程");

        private String code;

        private String msg;

        FeedbackTypeEnum(String code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getMsg() {
            return msg;
        }

        public void setMsg(String msg) {
            this.msg = msg;
        }

        @Override
        public String getCode() {
            return this.code;
        }
    }
    public enum FeedbackTypeDescEnum implements BaseEnum<String,FeedbackTypeDescEnum> {
        PRICE("PRICE","价格原因"), TIME("TIME","时间原因"),
        TEACHING("TEACHING","教学不满意"),
        SICK("SICK","生病"),
        SOMETHING("SOMETHING","有事"),
        NO_PARTICIPATE("NO_PARTICIPATE","不想参加"),
        OTHER("OTHER","其他")
        ;

        private String code;

        private String msg;

        FeedbackTypeDescEnum(String code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getMsg() {
            return msg;
        }

        public void setMsg(String msg) {
            this.msg = msg;
        }

        @Override
        public String getCode() {
            return this.code;
        }
    }

    private Integer id;

    /**
     * 乐团id
     */
    @ApiModelProperty(value = "乐团id", required = true)
    private String musicGroupId;

    /**
     * 分部id
     */
    @ApiModelProperty(value = "分部id", required = false)
    private Integer organId;

    /**
     * 学生id
     */
    @ApiModelProperty(value = "学生id", required = true)
    @Positive(message = "学生id必须大于0")
    private Integer studentId;

    private String phone;

    /**
     * 老师id
     */
    @ApiModelProperty(value = "老师id", required = true)
    @Positive(message = "老师id必须大于0")
    private Integer teacherId;

    /**
     * 老师类型
     */
    @ApiModelProperty(value = "老师类型", required = false)
    private VisiterTypeEnum visiterType;

    /**
     * 回访类型
     */
    @ApiModelProperty(value = "回访类型", required = true)
    @NotBlank(message = "回访类型不能为空")
    private String type;

    /**
     * 回访目的
     */
    @ApiModelProperty(value = "回访目的", required = true)
    private String purpose;

    /**
     * 学生情况
     */
    @ApiModelProperty(value = "学生近况", required = true)
    private String overview;

    /**
     * 家长反馈类型
     */
    @ApiModelProperty(value = "家长反馈", required = true)
    private FeedbackTypeEnum feedbackType;

    /**
     * 家长反馈类型
     */
    @ApiModelProperty(value = "家长反馈类型明细", required = true)
    private FeedbackTypeDescEnum feedbackTypeDesc;

    /**
     * 家长反馈
     */
    @ApiModelProperty(value = "家长反馈", required = true)
    private String feedback;

    /**
     * 回访日期
     */
    @ApiModelProperty(value = "回访日期", required = true)
    @PastOrPresent(message = "回访日期不能大于当天")
    private Date visitTime;

    /**
     * 添加时间
     */
    private Date createTime;

    private Long objectId;

    private Integer probStatus = 0;

    private String visitCyCle;


    @ApiModelProperty(value = "回访图片", required = true)
    private String attachments;
}
