package com.ym.mec.biz.dal.entity;

import com.ym.mec.biz.dal.enums.CouponDetailTypeEnum;
import com.ym.mec.biz.dal.enums.CouponTypeEnum;
import com.ym.mec.biz.dal.enums.EffectiveTypeEnum;
import com.ym.mec.common.entity.BaseEntity;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.builder.ToStringBuilder;

import javax.validation.constraints.Max;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.PositiveOrZero;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 对应数据库表(sys_coupon):
 */
@Data
public class SysCoupon extends BaseEntity implements Serializable{

	private Integer id;

	@ApiModelProperty("优惠券名称")
	private String name;

	@ApiModelProperty("描述")
	private String description;

	@ApiModelProperty("状态：0停用，1启用")
	private Integer status;

    @NotNull(message = "请选择一个优惠券发放的类型！")
    @PositiveOrZero(message = "优惠券发放类型不正确！")
    @Max(value = 1,message = "优惠券发放类型不正确！")
    @ApiModelProperty(value = "0手动领取  1手动发放")
    private Integer issuanceType;

	@ApiModelProperty("券类型:FULL_REDUCTION(满减券)，DISCOUNT(折扣券)")
	private CouponTypeEnum type;

    //@NotNull(message = "优惠券详细分类不能为空!")
    @ApiModelProperty(value = "详情-CouponDetailTypeEnum-枚举类")
    private CouponDetailTypeEnum typeDetail;

	@ApiModelProperty("面值")
	private java.math.BigDecimal faceValue;

	@ApiModelProperty("满多少")
	private java.math.BigDecimal fullAmount;

	@ApiModelProperty("限制兑换的最大次数")
	private Integer limitExchangeNum;

	@ApiModelProperty("期限（单位：天）")
	private Integer deadline;

	@ApiModelProperty("有效期类型")
	private EffectiveTypeEnum effectiveType;

	@ApiModelProperty("有效起始时间")
	private java.util.Date effectiveStartTime;

	@ApiModelProperty("有效截止时间")
	private java.util.Date effectiveExpireTime;

	@ApiModelProperty("领取结束时间")
	private java.util.Date endDate;

	@ApiModelProperty("领取开始时间")
	private java.util.Date startDate;

	@ApiModelProperty("库存量")
	private Integer stockCount;

	@ApiModelProperty("消耗数量")
	private Integer consumeNum;

	@ApiModelProperty("预警值")
	private Integer warningStockNum;

	private Integer warningStatus;

	private java.util.Date createTime;

	private java.util.Date updateTime;

	@ApiModelProperty("非数据库字段-可以领取的优惠券数量")
	private Integer canBeGetNum;

	@ApiModelProperty("领取优惠券条件JSON: {organId:分部id(逗号隔开)}")
	private String useCondition;

	@ApiModelProperty("使用系统 MALL:商城  MEC:管乐迷")
	private String useSystem;

    @Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this);
	}

}
