package com.ym.mec.biz.dal.entity;

import com.ym.mec.biz.dal.enums.DeviceTypeEnum;
import com.ym.mec.biz.dal.enums.FeatureType;
import com.ym.mec.biz.dal.enums.HeardLevelEnum;
import com.ym.mec.common.entity.BaseEntity;
import com.ym.mec.util.date.DateUtil;

import lombok.AllArgsConstructor;
import lombok.Data;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.util.Date;

/**
 * 对应数据库表(sys_music_compare_record):
 */
@Data
public class SysMusicCompareRecord extends BaseEntity {

	/**  */
	private Long id;

	/** 行为编号 */
	private String behaviorId;
	
	/** 用户编号 */
	private Integer userId;
	
	/** 教程编号 */
	private Integer sysMusicScoreId;

	private HeardLevelEnum heardLevel;

	private String sysMusicScoreName;
	
	/** 评分数据 */
	private String scoreData;

	private int notesDataIndex;

	/** 总分 */
	private BigDecimal score;

	/** 音准 */
	private BigDecimal intonation;

	/** 节奏 */
	private BigDecimal cadence;

	/** 完成度 */
	private BigDecimal integrity;

	/** 录音文件地址 */
	private String recordFilePath;

	private String videoFilePath;

	/** 周一日期 */
	private String monday = LocalDate.now().with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue()).toString();

	private FeatureType feature;
	
	/** 创建时间 */
	private java.util.Date createTime;

	/** 创建时间无时分秒*/
	private String createDate;

	private DeviceTypeEnum deviceType;

	private float sourceTime = 0;

	private float playTime = 0;

	private int speed = 90;

	private String clientId;
	
	private String partIndex;

	//文件是否过期
	private boolean expire;

	private Integer subjectId;

	public SysMusicCompareRecord(FeatureType feature) {
		this.feature = feature;
	}

	public SysMusicCompareRecord() {
	}

	public void setCreateTime(Date createTime) {
		this.createTime = createTime;
		this.createDate = DateUtil.format(createTime, DateUtil.DEFAULT_PATTERN);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this);
	}

}
