package com.ym.mec.biz.dal.entity;

import com.ym.mec.biz.dal.enums.PeriodEnum;
import com.ym.mec.common.entity.BaseEntity;

import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.util.Date;

import static com.ym.mec.biz.dal.enums.PeriodEnum.DAY;
import static com.ym.mec.biz.dal.enums.PeriodEnum.MONTH;

/**
 * 对应数据库表(vip_group_activity):
 */
@Data
public class VipGroupActivity{

	@ApiModelProperty(value = "活动编号")
	private Integer id;

	@ApiModelProperty(value = "活动名称")
	private String name;

	@ApiModelProperty(value = "活动类型，课程或者会员,0课程，1会员")
	private Integer activityType = 0;

	@ApiModelProperty(value = "活动类型，VIP、PRACTICE、MEMBER、THEORY")
	private String activityTypeEnum;

	@ApiModelProperty(value = "单节课时长")
	private Integer singleCourseTime = 45;

	@ApiModelProperty(value = "单节课时长")
	private Integer giveSingleCourseTime = 45;

	@ApiModelProperty(value = "活动描述")
	private String description;

	@ApiModelProperty(value = "课程形式")
	private String vipGroupCategoryIdList;

	@ApiModelProperty(value = "活动开始时间")
	private java.util.Date startTime;

	@ApiModelProperty(value = "活动结束时间")
	private java.util.Date endTime;

	@ApiModelProperty(value = "适用分部")
	private String organId;

	@ApiModelProperty(value = "适用分部")
	private String organName;

	@ApiModelProperty(value = "可排课开始时间")
	private java.util.Date coursesStartTime;

	@ApiModelProperty(value = "可排课截至时间")
	private java.util.Date coursesEndTime;

	@ApiModelProperty(value = "折扣）")
	private BigDecimal discount = new BigDecimal(100);

	@ApiModelProperty(value = "满赠达标课时:-1：没有满赠）")
	private Integer fullMinusCourseTimes = -1;

	@ApiModelProperty(value = "赠课类型")
	private String giveCourseType;

	@ApiModelProperty(value = "课类型")
	private String courseType;

	@ApiModelProperty(value = "赠课形式")
	private String giveCategoryId;

	@ApiModelProperty(value = "赠课形式")
	private String giveCategoryName;

	@ApiModelProperty(value = "人数")
	private Integer giveCategoryNum;

	@ApiModelProperty(value = "赠课数量")
	private Integer giveCourseNum = 0;

	@ApiModelProperty(value = "会员编号")
	private Integer memberRankId = 1;

	@ApiModelProperty(value = "会员时长")
	private Integer memberTime = 0;

	@ApiModelProperty(value = "赠送会员编号")
	private Integer giveMemberRankId = 1;

	@ApiModelProperty(value = "赠送会员时长")
	private Integer giveMemberTime = 0;

	@ApiModelProperty(value = "赠送抽奖次数")
	private Integer givePrizeNum = 0;

	@ApiModelProperty(value = "是否充值到余额")
	private boolean isPayToBalance = false;

	@ApiModelProperty(value = "创建时间")
	private java.util.Date createTime;

	@ApiModelProperty(value = "更新时间")
	private java.util.Date updateTime;

	@ApiModelProperty(value = "结算标准")
	private String salarySettlementJson;

	private String vipGroupCategoryNames;

	@ApiModelProperty(value = "学员数量")
	private Integer vipGroupCategoryNum;

	@ApiModelProperty(value = "是否删除")
	private Boolean delFlag = false;

	@ApiModelProperty(value = "是否可用")
	private Boolean enable = false;

	@ApiModelProperty(value = "线上课是否参与梯度奖励")
	private Integer onlineClassJoinGradientRewards = 0;

	@ApiModelProperty(value = "线下课是否参与梯度奖励")
	private Integer offlineClassJoinGradientRewards = 0;

	@ApiModelProperty(value = "是否固定课程数量")
	private Boolean fixedCourseNumFlag = false;

	@ApiModelProperty(value = "最少课程数量")
	private Integer minCourseNum = -1;

	@ApiModelProperty(value = "最多可排课数量")
	private Integer maxCourseNum = -1;

	@ApiModelProperty(value = "每位学员最大购买次数")
	private Integer studentMaxUsedTimes = -1;

	@ApiModelProperty(value = "适用学生类型:-1：所有；0：老用户；1：新用户，2会员，3非会员")
	private String applyToStudentType;

	@ApiModelProperty(value = "课程可调整方向：1不限制，0线上不可到线下，2禁止线下到线上，3双向不可调")
	private Integer allowOnlineToOffline = 1;

	@ApiModelProperty(value = "赠送课程可调整方向：1不限制，0线上不可到线下，2禁止线下到线上，3双向不可调")
	private Integer giveAllowOnlineToOffline = 1;

	@ApiModelProperty(value = "-1：所有；0：线上；1：线下")
	private Integer teachMode = -1;

	@ApiModelProperty(value = "-1：所有；0：线上；1：线下")
	private Integer giveTeachMode = -1;

	private PeriodEnum periodEnum = MONTH;

	private PeriodEnum givePeriodEnum = DAY;

	private String status = "DRAFT";

	@ApiModelProperty(value = "活动价格")
	private BigDecimal marketPrice = BigDecimal.ZERO;

	@ApiModelProperty(value = "活动原价")
	private BigDecimal originalPrice = BigDecimal.ZERO;

	//用于标记活动类型，购买的活动类型1、1v1 20节，2、1v1 40节，3、1v2 20节，4、1v2 40节，5、乐理课
	private Integer code;

	@ApiModelProperty(value = "活动渠道，1乐团渠道，2常规活动,3学员购买")
	private Integer activityChannel;

	@ApiModelProperty(value = "赠送的优惠券的编号")
	private Integer giveCouponId;

	@ApiModelProperty(value = "赠送的优惠券的数量")
	private Integer giveCouponNum;

	@ApiModelProperty(value = "单价")
	private BigDecimal singlePrice;

	@ApiModelProperty(value = "活动价用于前端展示")
	private BigDecimal displayPrice;

	private Integer tenantId = TenantContextHolder.getTenantId();
}