package com.ym.mec.biz.dal.enums;

import com.ym.mec.common.enums.BaseEnum;
import com.ym.mec.common.exception.BizException;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public enum FivePlusGradeEnum implements BaseEnum<Integer, FivePlusGradeEnum> {
	GRADE1(1, "一年级"),
	GRADE2(2, "二年级"),
	GRADE3(3, "三年级"),
	GRADE4(4, "四年级"),
	GRADE5(5, "五年级"),
	GRADE6(6, "六年级/初一"),
	GRADE7(7, "七年级/初二"),
	GRADE8(8, "八年级/初三"),
	GRADE9(9, "九年级/初四"),
	GRADE10(10, "高一"),
	GRADE11(11, "高二"),
	GRADE12(12, "高三");

	private Integer code;

	private String desc;

	private FivePlusGradeEnum(Integer code, String desc) {
		this.code = code;
		this.desc = desc;
	}

	@Override
	public Integer getCode() {
		return code;
	}

	public String getDesc() {
		return desc;
	}
	
	public static FivePlusGradeEnum get(String desc) {
		for (FivePlusGradeEnum vpg : FivePlusGradeEnum.values()) {
			if (vpg.getDesc().equals(desc)) {
				return vpg;
			}
		}
		return null;
	}


    public static FivePlusGradeEnum[] getArray(String gradeType) {
        if (StringUtils.isBlank(gradeType)) {
            return FivePlusGradeEnum.values();
        }
        Set<FivePlusGradeEnum> list = new HashSet<>();
        for (String s : gradeType.split(",")) {
            EGroupGradeType o = EGroupGradeType.get(s);
            if (o == null) {
                throw new BizException("gradeType参数错误");
            }
            switch (o) {
                case PRIMARY:
                    list.addAll(Arrays.asList(GRADE1, GRADE2, GRADE3, GRADE4, GRADE5));
                    break;
                case MIDDLE:
                    list.addAll(Arrays.asList(GRADE6,GRADE7, GRADE8, GRADE9));
                    break;
                case HIGH:
                    list.addAll(Arrays.asList(GRADE10, GRADE11, GRADE12));
                    break;
                case MIXED:
                    list.addAll(Arrays.asList(GRADE1, GRADE2, GRADE3, GRADE4, GRADE5, GRADE6, GRADE7, GRADE8, GRADE9));

                    break;
                default:
                    list.addAll(Arrays.asList(FivePlusGradeEnum.values()));
            }

        }
        return list.toArray(new FivePlusGradeEnum[0]);
    }

    }
