package com.ym.mec.biz.dal.enums.im;

import com.baomidou.mybatisplus.annotation.EnumValue;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.enums.ConverterEnum;
import lombok.Getter;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

/**
 * 客户端类型
 * Created by Eric.Shang on 2022/11/4.
 */
@Getter
public enum EClientType implements ConverterEnum<String> {

    BACKEND(1, "后端", "cooleshow-backend", "", "backend_login_locked"),
    SCHOOL(2, "教务端", "cooleshow-school", "school_device_num", "school_login_locked"),
    TEACHER(3, "老师端", "cooleshow-teacher", "teacher_device_num", "teacher_login_locked"),
    STUDENT(4, "学生端", "cooleshow-student", "student_device_num", "student_login_locked"),
    ;

    private final int value;
    private final String name;
    private final String clientId;
    private final String deviceParam;
    private final String lockedParam;

    @EnumValue
    private final String code;

    EClientType(int value, String name, String clientId, String param, String lockParam) {
        this.value = value;
        this.name = name;
        this.clientId = clientId;
        this.deviceParam = param;
        this.lockedParam = lockParam;

        this.code = this.name();
    }

    /**
     * 客户端类型
     * @param clientId 授权client_id
     * @return EClientType
     */
    public static EClientType clientType(String clientId) {

        if (StringUtils.isNoneBlank(clientId)) {

            for (EClientType item : EClientType.values()) {

                if (Lists.newArrayList(item.getClientId().split(",")).contains(clientId)) {

                    return item;
                }
            }
        }

        return BACKEND;
    }

    public static EClientType clientName(String name) {

        if (StringUtils.isNoneBlank(name)) {

            for (EClientType item : EClientType.values()) {

                if (Lists.newArrayList(item.name()).contains(name)) {

                    return item;
                }
            }
        }

        return BACKEND;
    }


    /**
     * 客户端对象比较
     * @param code 客户端类型
     * @return boolean
     */
    public boolean match(String code) {
        return getCode().equals(code.toUpperCase());
    }

    /**
     * jwtToken失效时间
     * @param clientType 客户端类型
     * @return boolean
     */
    public static boolean jwtTokenExpire(String clientType) {

        List<EClientType> clientTypes = Lists.newArrayList(BACKEND, TEACHER);
        for (EClientType item : clientTypes) {

            // 需要设置token失效时间
            if (item.getCode().equals(clientType)) {
                return true;
            }
        }
        return false;
    }
}
