package com.ym.mec.biz.dal.enums.school;

import com.baomidou.mybatisplus.annotation.EnumValue;
import com.google.common.collect.Lists;
import com.ym.mec.common.enums.BaseEnum;
import lombok.Getter;
import org.apache.catalina.LifecycleState;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

/***
 * 学校员工类型
 * 1人：校长、乐团领队
 * 多人：分管领导、负责老师
 */
@Getter
public enum ESchoolStaffType implements BaseEnum<String, ESchoolStaffType> {

	SCHOOLMASTER("校长"),
	ORCHESTRA_LEADER("乐团领队"),
	SCHOOL_LEADER("分管领导"),
	TEACHER("负责老师"),
	;

	private final String describe;

	@EnumValue
	private final String code;

	ESchoolStaffType(String describe) {
		this.describe = describe;

		this.code = this.name();
	}

	@Override
	public String getCode() {
		return this.name();
	}

	/**
	 * 参数合法性校验
	 * @param code 枚举code
	 * @return boolean
	 */
	public static boolean invalid(String code) {

		if (StringUtils.isNotEmpty(code)) {
			for (ESchoolStaffType e : ESchoolStaffType.values()) {
				if (e.getCode().equals(code)) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * 排序返回员工身份
	 * @return List<ESchoolStaffType>
	 */
	public static List<ESchoolStaffType> sorted() {
		return Lists.newArrayList(SCHOOLMASTER, ORCHESTRA_LEADER, SCHOOL_LEADER, TEACHER);
	}
}
