package com.ym.mec.biz.dal.vo;

import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.enums.school.ESchoolStaffType;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

/**
 * 学校员工
 * 2023-05-06 11:08:00
 */
@ApiModel(value = "SchoolStaffVo对象", description = "学校员工查询视图对象")
public class SchoolStaffVo {

    @Data
    @ApiModel(" SchoolStaffQuery-学校员工")
    public static class SchoolStaffQuery implements QueryInfo {
    
        @ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolStaffQuery from(String json) {
            return JSON.parseObject(json, SchoolStaffQuery.class);
        }
    }
    
    @Data
    @ApiModel(" SchoolStaff-学校员工")
    public static class SchoolStaff {
    	
		@ApiModelProperty("主键")
        private Long id;
                
        @NotNull(message = "合作单位编号不能为空")
		@ApiModelProperty("合作单位编号")
        private Long schoolId;
                
		@ApiModelProperty("用户身份(校长，乐团领队，分管领导，负责老师)")
        private ESchoolStaffType userType;
                
        @NotBlank(message = "用户名称不能为空")
        @Length(min = 1, max = 8, message = "用户名称长度必须在1-8个字符之间")
        @ApiModelProperty("用户名称")
        private String username;

        @NotBlank(message = "手机号码不能为空")
        @Pattern(regexp = "1[3456789][0-9]{9}", message = "手机号码格式不正确")
        @ApiModelProperty("手机号码")
        private String mobile;

        @NotBlank(message = "验证码不能为空")
        @ApiModelProperty("验证码")
        private String smsCode;
                
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolStaff from(String json) {
            return JSON.parseObject(json, SchoolStaff.class);
        }

    }
    
}