package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.update.Update;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.config.validator.group.ValidGroups;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.enums.CourseStatusEnum;
import com.ym.mec.biz.dal.enums.EEvaluateProblemType;
import com.ym.mec.biz.dal.enums.EEvaluateStatus;
import com.ym.mec.biz.dal.enums.ESchoolActivityProgramType;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.util.Date;
import java.util.Optional;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotNull;

/**
 * 巡堂详情
 * 2023-05-04 11:32:00
 */
@ApiModel(value = "CoursePatrolEvaluationWrapper对象", description = "巡堂详情查询对象")
public class CoursePatrolEvaluationWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" CoursePatrolEvaluationQuery-巡堂详情")
    public static class CoursePatrolEvaluationQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("老师姓名/编号/手机号")
		private String keyword;

        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;

        @ApiModelProperty("分部ID")
        private String organId;


        @ApiModelProperty("合作单位ID ")
        private Integer cooperationOrganId;

        @ApiModelProperty("课程类型")
        private CourseSchedule.CourseScheduleType courseType;


        @ApiModelProperty("优秀EXCELLENT、良好GOOD、合格QUALIFIED、不合格UNQUALIFIED")
        private EEvaluateStatus evaluateStatus;


        @ApiModelProperty("问题类型 课堂纪律 CLASSROOM_DISCIPLINE, 放学组织 AFTER_SCHOOL_ORGANIZATION, 环境卫生 ENVIRONMENTAL_SANITATION, 教态仪表 TEACHING_DEMEANOR, 沟通表达 COMMUNICATION_EXPRESSION, 专业能力 PROFESSIONAL_ABILITY, 其他 OTHER")
        private EEvaluateProblemType problemType;

        @ApiModelProperty("处理状态")
        private Boolean handleStatus;

        @ApiModelProperty("评价状态")
        private Boolean evaluateFlag;

        @ApiModelProperty("巡堂ID")
        private Integer id;

        @JsonIgnore
        // 必须存在巡堂
        private Boolean patrolFlag;

        @ApiModelProperty(value = "课程状态", required = false)
        private CourseStatusEnum status;

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static CoursePatrolEvaluationQuery from(String json) {
            return JSON.parseObject(json, CoursePatrolEvaluationQuery.class);
        }
    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" CoursePatrolEvaluationStatQuery-巡堂查询")
    public static class CoursePatrolEvaluationStatQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static CoursePatrolEvaluationQuery from(String json) {
            return JSON.parseObject(json, CoursePatrolEvaluationQuery.class);
        }
    }

    @Data
    @ApiModel("CoursePatrolEvaluationStat-巡堂详情统计")
    public static class CoursePatrolEvaluationStat {

        @ApiModelProperty("日期")
        private String date;

        @ApiModelProperty("声部课 数量")
        private Integer singleCourseCount;

        @ApiModelProperty("合奏课数量")
        private Integer minCourseCount;


        @ApiModelProperty("巡堂数量")
        private Integer patrolCount;

        @ApiModelProperty("总数量")
        private Integer totalCount;


    }


    @Data
	@ApiModel(" CoursePatrolEvaluation-巡堂详情")
    public static class CoursePatrolEvaluationInfo {

        @ApiModelProperty("巡堂编号")
        private Integer id;

        @ApiModelProperty("分部ID")
        private Integer organId;

        @ApiModelProperty("分部名称")
        private String organName;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        @ApiModelProperty("合作单位名称")
        private String cooperationOrganName;

        @ApiModelProperty("老师ID")
        private Integer teacherId;

        @ApiModelProperty("老师姓名")
        private String teacherName;

        @ApiModelProperty("老师头像")
        private String teacherAvatar;

        @ApiModelProperty("课程ID")
        private Integer courseScheduleId;

        @ApiModelProperty("课程类型")
        private CourseSchedule.CourseScheduleType courseType;

        @ApiModelProperty("课程名称")
        private String courseName;

        @ApiModelProperty("课程日期")
        private Date courseDate;

        /** 上课时间 */
        @ApiModelProperty(value = "上课时间")
        private Date startClassTime;

        /** 结束时间 */
        @ApiModelProperty(value = "结束时间")
        private Date endClassTime;

        @ApiModelProperty(value = "课程状态")
        private CourseStatusEnum courseStatus;

        @ApiModelProperty("操作用户ID")
        private Integer userId;

        @ApiModelProperty("优秀EXCELLENT、良好GOOD、合格QUALIFIED、不合格UNQUALIFIED")
        private EEvaluateStatus evaluateStatus;

        @ApiModelProperty("问题类型 课堂纪律 CLASSROOM_DISCIPLINE, 放学组织 AFTER_SCHOOL_ORGANIZATION, 环境卫生 ENVIRONMENTAL_SANITATION, 教态仪表 TEACHING_DEMEANOR, 沟通表达 COMMUNICATION_EXPRESSION, 专业能力 PROFESSIONAL_ABILITY, 其他 OTHER")
        private String problemType;

        @ApiModelProperty("问题描述")
        private String problemDesc;


        @ApiModelProperty("处理状态")
        private Boolean handleStatus;

        @ApiModelProperty("建议")
        private String suggestion;

        @ApiModelProperty("附件")
        private String attachmentUrl;

        @ApiModelProperty("评价状态")
        private Boolean evaluateFlag;
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static CoursePatrolEvaluationInfo from(String json) {
            return JSON.parseObject(json, CoursePatrolEvaluationInfo.class);
        }
	}

    @Data
    @ApiModel(" CoursePatrolEvaluation-保存巡堂信息")
    public static class CoursePatrolEvaluation {

        @ApiModelProperty("巡堂编号")
        @NotNull(message = "巡堂编号不能为空",groups = {ValidGroups.Update.class})
        private Integer id;

        @ApiModelProperty(value = "课程ID")
        @NotNull(message = "课程ID不能为空",groups = {ValidGroups.Add.class})
        private Integer courseScheduleId;

        @ApiModelProperty(value = "优秀EXCELLENT、良好GOOD、合格QUALIFIED、不合格UNQUALIFIED")
        private EEvaluateStatus evaluateStatus;

        @ApiModelProperty("问题类型 课堂纪律 CLASSROOM_DISCIPLINE, 放学组织 AFTER_SCHOOL_ORGANIZATION, 环境卫生 ENVIRONMENTAL_SANITATION, 教态仪表 TEACHING_DEMEANOR, 沟通表达 COMMUNICATION_EXPRESSION, 专业能力 PROFESSIONAL_ABILITY, 其他 OTHER")
        private String problemType;

        @ApiModelProperty("问题描述")
        private String problemDesc;

        @ApiModelProperty("建议")
        private String suggestion;

        @ApiModelProperty("附件")
        private String attachmentUrl;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static CoursePatrolEvaluation from(String json) {
            return JSON.parseObject(json, CoursePatrolEvaluation.class);
        }
    }
}