package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.TableField;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.enums.degree.EDegreeFeeType;
import com.ym.mec.biz.dal.enums.DegreeTypeEnum;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

/**
 * 考级等级费用配置
 * 2023-04-10 10:24:25
 */
@ApiModel(value = "DegreeLevelFeeWrapper对象", description = "考级等级费用配置查询对象")
public class DegreeLevelFeeWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" DegreeLevelFeeQuery-考级等级费用配置")
    public static class DegreeLevelFeeQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("关键字匹配")
		private String keyword;

        @ApiModelProperty("声部")
        private Integer subjectId;

        @ApiModelProperty("级别")
        private Integer level;

        @ApiModelProperty("考级分类, GRADE,THEORY")
        private DegreeTypeEnum degreeType;

        @ApiModelProperty("是否上传证件")
        private Boolean credentials;

        private Integer tenantId = TenantContextHolder.getTenantId();

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static DegreeLevelFeeQuery from(String json) {
            return JSON.parseObject(json, DegreeLevelFeeQuery.class);
        }
    }  

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
	@ApiModel(" DegreeLevelFee-考级等级费用配置")
    public static class DegreeLevelFee {

        @ApiModelProperty("考级费用配置")
        private List<LevelFeeConfig> gradeFeeConfigs;

        @ApiModelProperty("乐理费用配置")
        private List<LevelFeeConfig> theoryFeeConfigs;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static DegreeLevelFee from(String json) {
            return JSON.parseObject(json, DegreeLevelFee.class);
        }
	}

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" LevelFeeConfig-考级等级费用配置")
    public static class LevelFeeConfig implements Serializable {

        @ApiModelProperty("主键ID")
        private Integer id;

        @ApiModelProperty("声部ID")
        private Integer subjectId;

        @ApiModelProperty("声部名称")
        private String subjectName;

        @ApiModelProperty("等级")
        private Integer level;

        @ApiModelProperty("费用")
        private BigDecimal fee;

        @ApiModelProperty("考级分类, GRADE,THEORY")
        private EDegreeFeeType degreeType;

        @ApiModelProperty("上传证书")
        private Boolean credentials;

        @ApiModelProperty("声部价格配置")
        private List<LevelFeeConfig> subjectFeeConfigs;
    }

}