package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.TableField;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.util.Optional;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

/**
 * 考级资讯表
 * 2023-04-10 10:24:25
 */
@ApiModel(value = "DegreeNewsWrapper对象", description = "考级资讯表查询对象")
public class DegreeNewsWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" DegreeNewsQuery-考级资讯表")
    public static class DegreeNewsQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("关键字匹配")
		private String keyword;

        @ApiModelProperty("用户所在机构")
        private Integer organId;

        @ApiModelProperty("用户所在分部")
        private String defaultOrganId;

        @ApiModelProperty("是否启用")
        private Boolean status;

        @ApiModelProperty("咨询类型")
        private String type;

        @ApiModelProperty("用户机构ID")
        private Integer tenantId = TenantContextHolder.getTenantId();

        @ApiModelProperty(value = "排序方式", hidden = true)
        private Boolean sortedTime;

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static DegreeNewsQuery from(String json) {
            return JSON.parseObject(json, DegreeNewsQuery.class);
        }
    }  

	@ApiModel(" DegreeNews-考级资讯表")
    public static class DegreeNews {
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static DegreeNews from(String json) {
            return JSON.parseObject(json, DegreeNews.class);
        }
	}

}
