package com.ym.mec.biz.dal.wrapper;

import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.Map;
import java.util.Optional;

@Data
public class Double11StaticWrapper {

    @ApiModel("2023销售统计查询参数")
    @Data
    public static class SaleStaticsQuery {

        @ApiModelProperty("分部")
        private String organId;

        @ApiModelProperty("统计类型，0：汇总，1:1V1，2:1v2,3:乐理，4:云教练，5:乐器销售，6:报价卡")
        @NotNull
        private Integer saleStaticsType = 0;

        @ApiModelProperty("排序字段，saleAmount：销售金额，userCount:人数")
        @Pattern(regexp = "saleAmount|userCount", message = "不支持的类型")
        private String orderBy;

        @ApiModelProperty("1:升序 0:降序")
        private Integer asc = 0;

        @ApiModelProperty(hidden = true)
        private Integer tenantId;

        @ApiModelProperty(hidden = true)
        private GroupType groupType;

        @ApiModelProperty(hidden = true)
        private OrderTypeEnum type;

        @ApiModelProperty(hidden = true)
        private String activityId;

        @ApiModelProperty(hidden = true)
        private String musicGroupId;

        @ApiModelProperty(hidden = true)
        private String goodId;

        @ApiModelProperty(hidden = true)
        private Date startPayTime;

        @ApiModelProperty(hidden = true)
        private Date endPayTime;

        public void clearValue() {
            this.setGroupType(null);
            this.setType(null);
            this.setActivityId(null);
            this.setGoodId(null);
            this.setMusicGroupId(null);
            this.setStartPayTime(null);
            this.setEndPayTime(null);
        }

    }

    @ApiModel("分部学生购买")
    @Data
    public static class SaleStaticsStudentQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("分部")
        private String organId;

        @ApiModelProperty("学生姓名")
        private String userName;

        @ApiModelProperty("统计类型，1:1V1，2:1v2,3:乐理，4:云教练，5:乐器销售，6:报价卡")
        private Integer saleStaticsType;

        @ApiModelProperty("默认为时间")
        private String orderBy;

        @ApiModelProperty("1:升序，0:降序")
        private Integer asc = 0;

        @ApiModelProperty(hidden = true)
        private Long tenantId;

        @ApiModelProperty(hidden = true)
        private GroupType groupType;

        @ApiModelProperty(hidden = true)
        private OrderTypeEnum type;

        @ApiModelProperty(hidden = true)
        private String activityId;

        @ApiModelProperty(hidden = true)
        private String musicGroupId;

        @ApiModelProperty(hidden = true)
        private String goodId;
    }

    @Data
    public static class SaleStaticsStudent {

        @ApiModelProperty("分部")
        private Long organId;

        private String organName;

        private String userName;

        private String phone;

        private Date payTime;

        private BigDecimal payAmount;

    }


    @ApiModel(value = "销售额")
    @Data
    public static class SaleStatics {

        @ApiModelProperty("总销售额")
        private BigDecimal totalSaleAmount;

        @ApiModelProperty("1V1销售额")
        private BigDecimal totalSale1V1;

        @ApiModelProperty("1V1销售购买人数")
        private Integer totalUser1V1PurchaseNumber;

        @ApiModelProperty("1V2销售额")
        private BigDecimal totalSale1V2;

        @ApiModelProperty("1V2销售购买人数")
        private Integer totalUser1V2PurchaseNumber;

        @ApiModelProperty("乐理销售额")
        private BigDecimal totalSaleTheory;

        @ApiModelProperty("乐理销售购买人数")
        private Integer totalUserTheoryPurchaseNumber;

        @ApiModelProperty("云教练销售额")
        private BigDecimal totalSaleCloudTeacher;

        @ApiModelProperty("云教练销售购买人数")
        private Integer totalUserCloudTeacherPurchaseNumber;

        @ApiModelProperty("乐器销售额")
        private BigDecimal totalSaleMusical;

        @ApiModelProperty("乐器销售购买人数")
        private Integer totalUserMusicalPurchaseNumber;

        @ApiModelProperty("保价卡销售额")
        private BigDecimal totalSaleMaintenance;

        @ApiModelProperty("保价卡销售购买人数")
        private Integer totalUserMaintenancePurchaseNumber;

        @ApiModelProperty("小组课销售额")
        private BigDecimal smallGroupsMaintenance;

        @ApiModelProperty("小组课购买人数")
        private Integer smallGroupsPurchaseNumber;

        @ApiModelProperty("分部ID")
        private Long orgId;

        @ApiModelProperty("分部名称")
        private Long orgName;

    }

    @ApiModel("分部统计详情")
    @Data
    public static class SaleAmountAndUser {

        @ApiModelProperty("分部")
        private Integer organId;

        @ApiModelProperty("分部")
        private String organName;

        @ApiModelProperty("总销售额")
        private BigDecimal saleAmount;

        @ApiModelProperty("购买人数")
        private Integer userPurchaseNumber;

        @ApiModelProperty(hidden = true)
        private BigDecimal actualAmount = new BigDecimal(0);

        @ApiModelProperty(hidden = true)
        private BigDecimal balancePaymentAmount  = new BigDecimal(0);

        public BigDecimal getSaleAmount() {
            BigDecimal a = Optional.ofNullable(actualAmount).orElse(new BigDecimal(0));
            BigDecimal b = Optional.ofNullable(balancePaymentAmount).orElse(new BigDecimal(0));
            BigDecimal saleAmount = a.add(b);
            return saleAmount.setScale(2, RoundingMode.HALF_UP);
        }
    }

    @Data
    public static class SaleStaticConfigParam {
        // {"1v1":680,"1v2":"930","theoryLive":"778,776,775","cloudTeacher":"11160","musical":{"flute":10,
        // "clarinet":"9","altoSaxophone":"8","trumpet":"7","trombone":"6","upperTuba":"5"},"sporadic":680}
        private String v1;
        private String v2;
        private String theoryLive;
        private String cloudTeacher;
        // 保价卡
        private String sporadic;
        // 小组课
        private String theory;

        private Map<String, Long> musical;

        private Integer tenantId;

        private Date startPayTime;

        private Date endPayTime;

        private String organId;

        private String goodId;
    }
    @Data
    public static class StudentCount {
        private Integer organId;

        private Integer count;
    }
}
