package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.ym.mec.biz.dal.enums.GoodsType;
import com.ym.mec.biz.dal.enums.StockType;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class GoodsWrapper {

    @ApiModel("商品")
    @Data
    public static class Goods {

        @ApiModelProperty(value = "商品编号")
        private Integer id;

        @ApiModelProperty(value = "产品分类")
        private Integer goodsCategoryId;

        @ApiModelProperty(value = "产品分类名称")
        private String goodsCategoryName;

        @ApiModelProperty(value = "商品货号")
        private String sn;

        @ApiModelProperty(value = "商品名称")
        private String name;

        @ApiModelProperty(value = "品牌")
        private String brand;

        @ApiModelProperty(value = "规格")
        private String specification;

        @ApiModelProperty(value = "展示图片的地址")
        private String image;

        @ApiModelProperty(value = "库存数量")
        private Integer stockCount;

        @ApiModelProperty(value = "税务库存")
        private Integer taxStockCount;

        @ApiModelProperty(value = "总销量数")
        private Integer sellCount;

        @ApiModelProperty(value = "市场销售价")
        private BigDecimal marketPrice;

        @ApiModelProperty(value = "商品零售价")
        private BigDecimal discountPrice;

        @ApiModelProperty(value = "团购价")
        private BigDecimal groupPurchasePrice;

        @ApiModelProperty(value = "商品价格1")
        private BigDecimal costPrice;

        @ApiModelProperty(value = "协议成本价")
        private BigDecimal agreeCostPrice;

        @ApiModelProperty(value = "分部成本价")
        private BigDecimal organCostPrice;

        @ApiModelProperty(value = "页面简介")
        private String brief;

        @ApiModelProperty(value = "商品的详细描述")
        private String desc;

        @ApiModelProperty(value = "是否是新品（1，是  0，否）")
        private YesOrNoEnum isNew;

        @ApiModelProperty(value = "是否置顶（1，置顶   0，不置顶）")
        private YesOrNoEnum isTop;

        @ApiModelProperty(value = "状态（1，上架  0，下架）")
        private YesOrNoEnum status;

        @ApiModelProperty(value = "备注")
        private String memo;

        @ApiModelProperty(value = "发布时间")
        private Date publishTime;

        @ApiModelProperty(value = "附件商品列表编号（用逗号分开）")
        private String complementGoodsIdList;

        @ApiModelProperty(value = "辅件列表(子商品)")
        private List<com.ym.mec.biz.dal.entity.Goods> goodsList;

        private String subjectIds;

        private java.util.Date createTime;

        private java.util.Date updateTime;

        @ApiModelProperty(value = "商品类型")
        private GoodsType type;

        @ApiModelProperty(value = "备查货号，进货渠道")
        private String supplyChannel;

        @ApiModelProperty(value = "客户端展示分部")
        private String studentShowOrganId;

        @ApiModelProperty(value = "教务端展示分部")
        private String educationShowOrganId;

        @ApiModelProperty(value = "课程收费团展示分部")
        private String courseFeeShowOrganId;

        @ApiModelProperty(value = "会员收费团展示分部")
        private String memberFeeShowOrganId;

        @ApiModelProperty(value = "免费团展示分部")
        private String freeFeeShowOrganId;

        @ApiModelProperty(value = "乐器置换展示分部")
        private String replacementShowOrganId;

        @ApiModelProperty(value = "客户端展示分部")
        private String studentShowOrganName;

        @ApiModelProperty(value = "教务端展示分部")
        private String educationShowOrganName;

        @ApiModelProperty(value = "课程收费团展示分部")
        private String courseFeeShowOrganName;

        @ApiModelProperty(value = "会员收费团展示分部")
        private String memberFeeShowOrganName;

        @ApiModelProperty(value = "免费团展示分部")
        private String freeFeeShowOrganName;

        @ApiModelProperty(value = "乐器置换展示分部")
        private String replacementShowOrganName;

        @ApiModelProperty(value = "库存类型")
        private StockType stockType;

        @ApiModelProperty(value = "库存预警")
        private YesOrNoEnum stockWarning;

        @ApiModelProperty(value = "是否是组合商品，1：是，0：不是")
        private Boolean groupGoods;

        private Integer tenantId = TenantContextHolder.getTenantId();

        private String childId;

        private String childName;

        private String childSn;

        private String childOrganCostPrice;

        @ApiModelProperty(value = "品牌名称")
        private String brandName;

        private List<GoodsSub> goodsSubList = new ArrayList<>();

        @Override
        public String toString() {
            return ToStringBuilder.reflectionToString(this);
        }
    }


    @Data
    @ApiModel(" GoodsSub_-组合商品，子商品信息")
    public static class GoodsSub {

        @ApiModelProperty("id")
        private Integer id;

        @ApiModelProperty("商品编号")
        private Integer goodsId;

        @ApiModelProperty("商城商品编号")
        private Integer mallGoodsId;

        @ApiModelProperty("商城商品名称")
        private String mallGoodsName;

        @ApiModelProperty("skuID")
        private Integer sku;

        @ApiModelProperty("sku编号")
        private String skuCode;

        @ApiModelProperty("销售价格")
        private BigDecimal price;

        @ApiModelProperty("商品状态，1:上架、0:下架")
        private Boolean goodsStatus;

        @ApiModelProperty("成本")
        private BigDecimal goodsPrice;

        @ApiModelProperty("货号")
        private String productSn;

        @ApiModelProperty("品牌ID")
        private Long brandId;

        @ApiModelProperty("品牌名称")
        private String brandName;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static GoodsSub from(String json) {
            return JSON.parseObject(json, GoodsSub.class);
        }
    }


    @Data
    @ApiModel("商城商品分页查询参数")
    public static class GoodsSubQuery extends QueryInfo {

        @ApiModelProperty("上架状态，1：上架，0：下架")
        private Integer publishStatus;

        @ApiModelProperty("商品名称模糊关键字")
        private String keyword;

        @ApiModelProperty("商品货号")
        private String productSn;

        @ApiModelProperty("商品编号,多个逗号隔开")
        private String productIds;

        @ApiModelProperty("商品分类")
        private Long productCategoryId;

        @ApiModelProperty("商品品牌")
        private Long brandId;

    }

}
