package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

public class ImGroupWrapper {

    @Data
    public static class ImQuery {

        @ApiModelProperty("群名称/用户名")
        private String keyword;

        @ApiModelProperty("群类型  ")
        private String groupType;

        @ApiModelProperty("乐团ID")
        private String musicGroupId;

        @ApiModelProperty("角色 ")
        private String roleType;

        @ApiModelProperty("当前用户ID")
        private Integer userId;

        @ApiModelProperty("用户状态 false ：查询未锁定")
        private Boolean lockFlag;

        private List<Integer> userIds;
    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("IM用户信息")
    public static class ImUserInfo implements Serializable {

        @ApiModelProperty("用户ID")
        private String imUserId;

        @ApiModelProperty("授权Token")
        private String imToken;

        public ImUserInfo imUserId(String imUserId) {
            this.imUserId = imUserId;
            return this;
        }
    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" ImGroup-即时通讯群组")
    public static class ImGroup {


        @ApiModelProperty("主键ID")
        private String id;

        @ApiModelProperty("群组ID")
        private String groupId;

        @ApiModelProperty("群名称")
        private String name;

        @ApiModelProperty("群简介")
        private String introduce;

        @ApiModelProperty("群成员数")
        private Integer memberNum;

        @ApiModelProperty("群备注")
        private String memo;

        @ApiModelProperty("群头像")
        private String img;

        @ApiModelProperty("群类型 ")
        private String type;

        @ApiModelProperty("是否自动通过审核1是0否")
        private Boolean autoPassFlag;

        @ApiModelProperty("课程组id")
        private Long courseGroupId;

        @ApiModelProperty("修改时间;")
        private Date updateTime;

        @ApiModelProperty("创建人")
        private Long createBy;

        @ApiModelProperty("创建时间")
        private Date createTime;

        @ApiModelProperty("机构ID")
        private Integer tenantId;

        @ApiModelProperty("学校ID")
        private Long schoolId;

        /** 群标签 */
        private String tags;

        // 群类型
        private com.ym.mec.biz.dal.entity.ImGroup.GroupTypeEnum groupType;


        public static ImGroup from(String json) {
            return JSON.parseObject(json, ImGroup.class);
        }

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public ImGroup id(String id) {
            this.id = id;
            return this;
        }

        public ImGroup groupId(String groupId) {
            this.groupId = groupId;
            return this;
        }

        public ImGroup type(String type) {
            this.type = type;
            return this;
        }

    }

    @Data
    @ApiModel(" GroupMuteAll-群组禁言")
    public static class GroupMuteAll implements Serializable {

        @ApiModelProperty(value = "群ID",required = true)
        private String groupId;

        @ApiModelProperty(value = "是否禁言",required = true)
        private Boolean muteAll;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("配置信息")
    public static class ConfigJson implements Serializable {

        @ApiModelProperty("群聊禁言（1:禁言，管理员可以发言，0:不禁言）")
        private Boolean mute = false;

        public static ConfigJson from(String json) {
            if (StringUtils.isBlank(json)) {
                return new ConfigJson();
            }
            return JSON.parseObject(json, ConfigJson.class);
        }
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("转交群主")
    public static class GroupTransferOwner {
        @ApiModelProperty(value = "群ID",required = true)
        private String imGroupId;

        @ApiModelProperty("新群主ID")
        private String userId;

        @ApiModelProperty("当前群主")
        private String currentOwner;
    }
}
