package com.ym.mec.biz.dal.wrapper;

import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.enums.CourseStatusEnum;
import com.ym.mec.biz.dal.enums.JoinCourseType;
import com.ym.mec.biz.dal.enums.ShareModeEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotNull;
import java.util.Date;

public class LiveGroupWrapper {


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" LiveGroupStudentQuery-直播课学生列表查询")
    public static class LiveGroupStudentQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @NotNull(message = "直播课ID不能为空")
        @ApiModelProperty("直播课ID")
        private Long liveGroupId;

        private Long classGroupId;

        @ApiModelProperty("共享方式，NO_SHARE:未共享，OPEN:公开，PRIVATE:私密")
        private ShareModeEnum shareMode;

        @ApiModelProperty("模糊查询")
        private String search;
    }



    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" LiveGroupStudentList-直播课学生列表")
    public static class LiveGroupStudentList {

        @ApiModelProperty("订单号")
        private String orderNo;

        @ApiModelProperty("学生编号")
        private Integer studentId;

        @ApiModelProperty("分部名称")
        private String organName;

        @ApiModelProperty("学生姓名")
        private String studentName;


        @ApiModelProperty("学校名称")
        private String schoolName;

        @ApiModelProperty("声部")
        private String subjectName;

        @ApiModelProperty("手机号")
        private String phone;

        @ApiModelProperty("报名时间")
        private Date createTime;

        @ApiModelProperty("学生状态")
        private int studentStatus = 0;

        @ApiModelProperty("学生班级状态")
        private String classStatus;


    }



    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" LiveCoursePageQuery-乐直播课程列表查询")
    public static class LiveCoursePageQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;


        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;

        @ApiModelProperty("模糊查询")
        private String search;

        @ApiModelProperty("课程状态")
        private String status;

        private Integer teacherId;

        private Integer tenantId;

        //未开始直播课，提前开始时间
        private Integer notStartCourseTime;

    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" LiveCourseStudentQuery-乐直播课程学生列表查询")
    public static class LiveCourseStudentQuery {



        @ApiModelProperty("课程ID")
        private Long courseScheduleId;

        @ApiModelProperty(value = "学生在学状态")
        private JoinCourseType status;

        @ApiModelProperty(value = "学生姓名")
        private String search;

    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" LiveCourseInfo-学生通过分享进入直播课程")
    public static class LiveCourseInfo {



        @ApiModelProperty("课程ID")
        private Integer courseScheduleId;

        @ApiModelProperty(value = "学生ID")
        private Integer studentId;

        @ApiModelProperty(value = "直播房间号")
        private String roomUid;


        @ApiModelProperty(value = "主讲人ID")
        private Integer speakerId;

        @ApiModelProperty(value = "能否进入分享房间")
        private boolean canJoinShareRoom;

    }
}
