package com.dayaedu.cbs.domain.wrapper;

import com.alibaba.fastjson.JSON;
import com.dayaedu.cbs.common.enums.music.*;
import com.dayaedu.cbs.common.enums.school.EMusicSheetType;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * 曲谱表
 * 2023-12-29 00:05:53
 */
@ApiModel(value = "MusicSheetWrapper对象", description = "曲谱表查询对象")
public class MusicSheetWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" MusicSheetQuery-曲谱表")
    public static class MusicSheetQuery implements QueryInfo {

    	@ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("关键字匹配")
		private String keyword;

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static MusicSheetQuery from(String json) {
            return JSON.parseObject(json, MusicSheetQuery.class);
        }
    }

	@ApiModel(" MusicSheet-曲谱表")
    public static class MusicSheet {

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static MusicSheet from(String json) {
            return JSON.parseObject(json, MusicSheet.class);
        }
	}

    @Data
    @ApiModel(" AddMusicSheet-曲谱新增/修改表")
    public class AddMusicSheet {


        @ApiModelProperty("主键")
        private Long id;

        @ApiModelProperty("曲谱分类ID")
        private Long musicCategoryId;

        @ApiModelProperty("曲谱封面图")
        @NotBlank(message = "曲谱封面图不能为空")
        private String musicCover;

        @ApiModelProperty("曲谱名称")
        @NotBlank(message = "曲谱名称不能为空")
        private String name;

        @ApiModelProperty("描述")
        private String remark;

        @ApiModelProperty("曲谱乐器(多个,用逗号分隔)")
        private String musicalInstrumentIds;

        @ApiModelProperty("作曲人")
        @NotBlank(message = "音乐人不能为空")
        private String composer;

        @ApiModelProperty("曲谱类型(SINGLE:单曲 CONCERT:合奏)")
        @NotNull(message = "曲谱类型不能为空")
        private EMusicSheetType musicSheetType;

        @ApiModelProperty("是否使用系统节拍器(0:否;1:是)")
        private Boolean isUseSystemBeat;

        @ApiModelProperty("是否可评测(0:否;1:是)")
        private Boolean isEvaluated;

        @ApiModelProperty("是否可转谱(0:否; 1:是)")
        private Boolean isConvertibleScore;

        @ApiModelProperty("是否展示指法(0:否;1:是)")
        private Boolean isShowFingering;

        @ApiModelProperty("是否播入节拍器(0: 否 1:是)")
        private Boolean isPlayBeat;

        @ApiModelProperty("多声轨选择")
        private String multiTracksSelection;

        @ApiModelProperty("播放速度")
        private Integer playSpeed;

        @ApiModelProperty("播放模式(MIDI;MP3)")
        private String playMode;

        @ApiModelProperty("XML文件")
        private String xmlFileUrl;

        @ApiModelProperty("MIDI文件(保留字段)")
        private String midiFileUrl;

        @ApiModelProperty("曲谱JSON")
        private String musicJson;

        @ApiModelProperty("曲谱缓存JSON(后续使用)")
        private String musicSvgJson;

        @ApiModelProperty("曲谱缓存")
        private String musicSvg;

        @ApiModelProperty("简谱缓存")
        private String musicJianSvg;

        @ApiModelProperty("首调缓存")
        private String musicFirstSvg;

        @ApiModelProperty("谱面图片JSON(后续使用)")
        private String musicImgJson;

        @ApiModelProperty("五线谱图片")
        private String musicImg;

        @ApiModelProperty("首调图片")
        private String musicFirstImg;

        @ApiModelProperty("固定调图片")
        private String musicJianImg;

        @ApiModelProperty("谱面参数设置")
        private String extConfigJson;

        @ApiModelProperty("谱面样式参数设置")
        private String extStyleConfigJson;

        @ApiModelProperty("可用类型(PLATFORM: 平台; ORG: 机构; PERSON: 个人)")
        private String availableType;

        @ApiModelProperty("来源类型(PLATFORM: 平台; ORG: 机构; PERSON: 个人)")
        private String sourceType;

        @ApiModelProperty("音频类型(HOMEMODE: 自制 COMMON: 普通)")
        private String audioType;

        @ApiModelProperty("曲目状态（0：停用，1：启用）")
        private Boolean status;

        @ApiModelProperty("停用原因")
        private String reason;


        @ApiModelProperty("更新人")
        private Long updateBy;

        @ApiModelProperty("更新时间")
        private Date updateTime;

        @ApiModelProperty("创建人")
        private Long createBy;

        @ApiModelProperty("创建时间")
        private Date createTime;

    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" MusicSheet-管乐迷曲谱表")
    public static class MusicSheetApplicationExtendWrapper {
        @ApiModelProperty("曲目ID")
        private @NotNull(
                message = "曲目ID不能为空"
        ) Long musicSheetId;

        @ApiModelProperty("分类ID")
        private Long musicSheetCategoryId;

        @ApiModelProperty("启用状态")
        private Boolean status;

        @ApiModelProperty("排序")
        private Integer sortNo;

        @ApiModelProperty("收费类型(FREE:免费;VIP:会员;CHARGE:单曲收费)")
        private String paymentType;

        @ApiModelProperty("是否可转谱(0:否; 1:是)")
        private Boolean isConvertibleScore;

        @ApiModelProperty("谱面类型")
        private EDefaultScoreType scoreType;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" MusicSheet-管乐迷曲谱表")
    public static class MecMusicSheet extends MusicSheet {

        @ApiModelProperty("曲目分类")
        private String categoriesName;

        @ApiModelProperty("可见会员等级，用于判断是否收费")
        private String rankIds;

        @ApiModelProperty("排序值")
        private Integer order;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static MusicSheet from(String json) {
            return JSON.parseObject(json, MusicSheet.class);
        }
    }

    @Data
    public static class MusicSheetApplication{

        @ApiModelProperty("主键")
        private String id;

        @ApiModelProperty("业务平台曲目ID")
        private String bizId;

        @ApiModelProperty("曲目分类ID")
        private Long musicCategoryId;

        @ApiModelProperty("曲目分类名称")
        private String musicCategoryName;

        @ApiModelProperty("曲目标签")
        private String musicTag;

        @ApiModelProperty("曲目标签名称")
        private String musicTagNames;

        @ApiModelProperty("曲目封面图")
        private String musicCover;

        @ApiModelProperty("曲目名称")
        private String name;

        @ApiModelProperty("是否审核版本")
        private Boolean appAuditFlag;

        @ApiModelProperty("曲目声部(多个,用逗号分隔)")
        private String subjectIds;

        @ApiModelProperty("曲目声部code")
        private String subjectCodes;

        @ApiModelProperty("曲目声部名称")
        private String subjectNames;

        @ApiModelProperty("曲目乐器(多个,用逗号分隔)")
        private String musicalInstrumentIds;

        @ApiModelProperty("曲目乐器code")
        private String musicalInstrumentIdCodes;

        @ApiModelProperty("曲目乐器名称")
        private String musicalInstrumentNames;

        @ApiModelProperty("作曲人/音乐人")
        private String composer;

        @ApiModelProperty("曲目类型(SINGLE:单曲 CONCERT:合奏)")
        private EMusicSheetType musicSheetType;

        @ApiModelProperty("演奏是否使用系统节拍器(0:否;1:是)")
        private Boolean isUseSystemBeat;

        @ApiModelProperty("演唱是否使用系统节拍器(0:否;1:是)")
        private Boolean isUseSingSystemBeat;

        @ApiModelProperty("是否可评测(0:否;1:是)") //  midi默认1, mp3 有伴奏1
        private Boolean isEvaluated;

        @ApiModelProperty("是否可转谱(0:否; 1:是)") // 固定调，首调 有值 1
        private Boolean isConvertibleScore;

        @ApiModelProperty("是否展示指法(0:否;1:是)") // moren 1
        private Boolean isShowFingering;

        @ApiModelProperty("是否播入演奏节拍器(0: 否 1:是)")
        private Boolean isPlayBeat;

        @ApiModelProperty("是否播入演唱节拍器(0: 否 1:是)")
        private Boolean isPlaySingBeat;

        @ApiModelProperty("是否全部声部可使用(0: 否 1:是)")
        private Boolean isAllSubject;

        @ApiModelProperty("多声轨选择")
        private String multiTracksSelection;

        @ApiModelProperty("播放速度")
        private String playSpeed;

        @ApiModelProperty("音频类型，SING：演唱，PLAY：演奏")
        private String audioPlayTypes;

        @ApiModelProperty("播放模式(MIDI;MP3)")
        private EMusicPlayMode playMode;

        @ApiModelProperty("XML文件")
        private String xmlFileUrl;

        @ApiModelProperty("MIDI文件(保留字段)")
        private String midiFileUrl;

        @ApiModelProperty("曲目JSON")
        private String musicJson;

        @ApiModelProperty("曲目缓存JSON(后续使用)")
        private String musicSvgJson;

        @ApiModelProperty("曲目缓存")
        private String musicSvg;

        @ApiModelProperty("评测标准 节奏 AMPLITUDE 音准 FREQUENCY 分贝 DECIBELS")
        private EEvaluationStandard evaluationStandard;

        @ApiModelProperty("简谱缓存")
        private String musicJianSvg;

        @ApiModelProperty("首调缓存")
        private String musicFirstSvg;

        @ApiModelProperty("谱面图片JSON(后续使用)")
        private String musicImgJson;

        @ApiModelProperty("五线谱图片")
        private String musicImg;

        @ApiModelProperty("首调图片")
        private String musicFirstImg;

        @ApiModelProperty("固定调图片")
        private String musicJianImg;

        @ApiModelProperty("谱面参数设置")
        private String extConfigJson;

        @ApiModelProperty("谱面样式参数设置")
        private String extStyleConfigJson;

        @ApiModelProperty("可用类型(PLATFORM: 平台; ORG: 机构; PERSON: 个人)") // 暂时没有
        private EMusicAvailableType availableType;

        @ApiModelProperty("来源类型/作者属性(PLATFORM: 平台; ORG: 机构; PERSON: 个人)") // 作者属性
        private EMusicSourceType sourceType;

        @ApiModelProperty("音频类型(HOMEMODE: 自制 COMMON: 普通)")
        private EAudioType audioType;

        @ApiModelProperty("曲目状态（0：停用，1：启用）")
        private Boolean status;

        @ApiModelProperty("停用原因")
        private String reason;

        @ApiModelProperty("备注")
        private String remark;

        @ApiModelProperty("逻辑删除标识(0: 未删 1:已删)")
        private Boolean delFlag;

        @ApiModelProperty("更新人")
        private Long updateBy;

        @ApiModelProperty("更新时间")
        private Date updateTime;

        @ApiModelProperty("创建人")
        private Long createBy;

        @ApiModelProperty("上传人名称")
        private String createByName;

        @ApiModelProperty("创建时间")
        private Date createTime;

        @ApiModelProperty("评测频率,详情返回，指定应用端")
        private String evaluationFrequency;

        @ApiModelProperty("是否支持混音节拍器")
        private Boolean isMixBeat;

        @ApiModelProperty("总谱五线谱PDF文件")
        private String musicPdfUrl;

        @ApiModelProperty("总谱首调PDF文件")
        private String firstPdfUrl;

        @ApiModelProperty("总谱固定调PDF文件")
        private String jianPdfUrl;

        @ApiModelProperty("曲目伴奏")
        private List<CbsMusicSheetWrapper.MusicSheetAccompaniment> musicSheetAccompanimentList;


        @ApiModelProperty("曲目原音")
        private List<CbsMusicSheetWrapper.MusicSheetSound> musicSheetSoundList;


        @ApiModelProperty("所属人信息")
        private CbsMusicSheetWrapper.MusicSheetExtend musicSheetExtend;

        @ApiModelProperty("乐器信息")
        private List<CbsMusicSheetWrapper.MusicalInstrument> musicalInstruments;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static CbsMusicSheetWrapper.MusicSheet from(String json) {
            return JSON.parseObject(json, CbsMusicSheetWrapper.MusicSheet.class);
        }

    }

}
