package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.conditions.update.Update;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.config.validator.group.ValidGroups;
import com.ym.mec.biz.dal.enums.ESchoolActivityProgramType;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.util.Date;
import java.util.List;
import java.util.Optional;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

/**
 * 活动详情
 * 2023-05-04 11:32:00
 */
@ApiModel(value = "SchoolActivityDetailWrapper对象", description = "活动详情查询对象")
public class SchoolActivityDetailWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" SchoolActivityDetailQuery-活动详情")
    public static class SchoolActivityDetailQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("关键字匹配")
		private String keyword;
        
        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolActivityDetailQuery from(String json) {
            return JSON.parseObject(json, SchoolActivityDetailQuery.class);
        }
    }  

    @Data
	@ApiModel(" SchoolActivityDetail-活动详情列表")
    public static class SchoolActivityDetail {

        @ApiModelProperty("活动详情编号")
        private Integer id;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        @ApiModelProperty("活动id")
        private Integer schoolActivityId;

        @ApiModelProperty(value = "节目名称",required = true)
        @NotNull(message = "节目名称不能为空")
        private String name;

        @ApiModelProperty(value = "节目类型 合奏 ENSEMBLE 独奏 SOLO 重奏 REPRISE 齐奏 UNISON",required = true)
        @NotNull(message = "节目类型不能为空")
        private ESchoolActivityProgramType type;

        @ApiModelProperty(value = "乐团ID",required = true)
        @NotNull(message = "乐团ID不能为空")
        private String musicGroupId;

        @ApiModelProperty("乐团名称")
        private String musicGroupName;

        @ApiModelProperty("声部ID集合 逗号隔开")
        private String subjectIdList;

        @ApiModelProperty("声部名称集合 逗号隔开")
        private String subjectNameList;

        @ApiModelProperty(value = "学生人数",required = true)
        @NotNull(message = "学生人数不能为空")
        private Integer studentNum;

        @ApiModelProperty(value = "节目时长-秒",required = true)
        @NotNull(message = "节目时长不能为空")
        private Integer time;

        @ApiModelProperty(value = "节目排序",required = true)
        @NotNull(message = "节目排序不能为空")
        private Integer sort;

        @ApiModelProperty("附件")
        private String attachmentUrl;

        @ApiModelProperty("学生数据集合")
        @NotEmpty(message = "学生数据集合不能为空")
        private List<SchoolActivityWrapper.SchoolActivitySubject> studentList;

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolActivityDetail from(String json) {
            return JSON.parseObject(json, SchoolActivityDetail.class);
        }
	}

}