package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.microsvc.toolkit.config.validator.group.ValidGroups;
import com.ym.mec.biz.dal.enums.ESchoolActivityStatus;
import com.ym.mec.biz.dal.enums.ESchoolActivityType;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * 学校活动
 * 2023-05-04 11:32:00
 */
@ApiModel(value = "SchoolActivityWrapper对象", description = "学校活动查询对象")
public class SchoolActivityWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" SchoolActivityQuery-学校活动")
    public static class SchoolActivityQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("关键字匹配")
		private String keyword;

        @ApiModelProperty("合作单位")
        private Integer cooperationOrganId;

        @ApiModelProperty("学生ID")
        private Integer studentId;

        @ApiModelProperty("乐团ID")
        private String musicGroupId;

        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;


        @ApiModelProperty("活动类型 开放日 OPEN_DAY 汇演 PERFORMANCE 音乐会 CONCERT 比赛 COMPETITION 讲座 LECTURE 校庆 SCHOOL_DAY 迎宾 WELCOME 其他 OTHER")
        private ESchoolActivityType type;
        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolActivityQuery from(String json) {
            return JSON.parseObject(json, SchoolActivityQuery.class);
        }
    }  

    @Data
	@ApiModel(" SchoolActivity-学校活动")
    public static class SchoolActivity {

        @ApiModelProperty("活动编号")
        private Integer id;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        @ApiModelProperty("活动开始日期")
        private Date startTime;

        @ApiModelProperty("活动结束日期")
        private Date endTime;

        @ApiModelProperty("活动状态 未开始 NOT_START 进行中 PROCESSING 已结束 END,")
        private ESchoolActivityStatus status;

        @ApiModelProperty("活动名称")
        private String name;

        @ApiModelProperty("活动类型 开放日 OPEN_DAY 汇演 PERFORMANCE 音乐会 CONCERT 比赛 COMPETITION 讲座 LECTURE 校庆 SCHOOL_DAY 迎宾 WELCOME 其他 OTHER")
        private ESchoolActivityType type;
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolActivity from(String json) {
            return JSON.parseObject(json, SchoolActivity.class);
        }
	}


    @Data
    @ApiModel(" SchoolActivityInfo-学校活动详情")
    public static class SchoolActivityInfo {

        @ApiModelProperty("活动编号")
        @NotNull(message = "活动编号不能为空",groups = {ValidGroups.Update.class})
        private Integer id;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        @ApiModelProperty("合作单位名称")
        private String cooperationOrganName;

        @ApiModelProperty("合作单位头像")
        private String cooperationOrganLogo;

        @ApiModelProperty(value = "活动开始日期",required = true)
        @NotNull(message = "活动开始日期不能为空")
        private Date startTime;

        @ApiModelProperty("活动结束日期")
        private Date endTime;

        @ApiModelProperty("活动状态 未开始 NOT_START 进行中 PROCESSING 已结束 END,")
        private ESchoolActivityStatus status;

        @ApiModelProperty(value = "活动名称",required = true)
        private String name;

        @ApiModelProperty(value = "活动类型 开放日 OPEN_DAY 汇演 PERFORMANCE 音乐会 CONCERT 比赛 COMPETITION 讲座 LECTURE 校庆 SCHOOL_DAY 迎宾 WELCOME 其他 OTHER",required = true)
        @NotNull(message = "活动类型不能为空")
        private ESchoolActivityType type;

        @ApiModelProperty("活动详情")
        @Valid
        @NotEmpty(message = "活动节目不能为空")
        private List<SchoolActivityDetailWrapper.SchoolActivityDetail> detail;
    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" SchoolActivityStudentQuery-学校活动学生查询对象")
    public static class SchoolActivityStudentQuery {


        @ApiModelProperty("活动节目编号")
        private Integer activityDetailId;

        @ApiModelProperty("乐团ID")
        private String musicGroupId;

        @ApiModelProperty("声部ID集合")
        private List<Integer> subjectIds;
    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" SchoolActivitySubject-学校活动声部")
    public static class SchoolActivitySubject {

        @ApiModelProperty("声部ID")
        @NotNull(message = "声部ID不能为空")
        private Integer subjectId;

        @ApiModelProperty("声部名称")
        private String subjectName;

        @ApiModelProperty("声部学生数量")
        private Integer studentCount;

        @ApiModelProperty("学生列表")
        @NotEmpty(message = "学生列表不能为空")
        private List<SchoolActivityStudent> studentList;

    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" SchoolActivityStudent-学校活动声部学生")
    public static class SchoolActivityStudent {

        @ApiModelProperty("学生ID")
        @NotNull(message = "学生ID不能为空")
        private Integer studentId;

        @ApiModelProperty("学生名")
        private String studentName;

        @ApiModelProperty("是否已经选择")
        private Boolean selected;


        @ApiModelProperty("学生头像")
        private String studentAvatar;

        @ApiModelProperty("学生声部id")
        private Integer subjectId;

        @ApiModelProperty("学生声部")
        private String subjectName;

    }

}