package com.ym.mec.biz.dal.wrapper;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.TableField;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.enums.school.ESchoolStaffType;
import com.ym.mec.common.tenant.TenantContextHolder;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * 学校员工
 * 2023-05-06 10:49:23
 */
@ApiModel(value = "SchoolStaffWrapper对象", description = "学校员工查询对象")
public class SchoolStaffWrapper {

    @Data
	@Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" SchoolStaffQuery-学校员工")
    public static class SchoolStaffQuery implements QueryInfo {
    
    	@ApiModelProperty("当前页")
        private Integer page;
        
        @ApiModelProperty("分页行数")
        private Integer rows;
        
        @ApiModelProperty("老师姓名/编号/手机号")
		private String keyword;

        @ApiModelProperty("合作单位编号")
        private Long schoolId;


        @ApiModelProperty("合作单位编号")
        private List<Long> schoolIds;


        @ApiModelProperty("机构编号")
        private Integer tenantId;

        @ApiModelProperty("分布ID")
        private String organId;


        @ApiModelProperty("合作单位禁用")
        private Boolean coopStatus;

        @ApiModelProperty("学校员工ID")
        private Long staffId;

        public String getKeyword() {
            return Optional.ofNullable(keyword).filter(StringUtils::isNotBlank).orElse(null);
        }
        
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolStaffQuery from(String json) {
            return JSON.parseObject(json, SchoolStaffQuery.class);
        }

        public Integer getTenantId() {
            return Optional.ofNullable(tenantId).orElse(TenantContextHolder.getTenantId());
        }
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
	@ApiModel(" SchoolStaff-学校员工")
    public static class SchoolStaff {

        @ApiModelProperty("主键")
        private Long id;

        @ApiModelProperty("分部ID")
        private String organId;

        @ApiModelProperty("分部名称")
        private String organName;

        @ApiModelProperty("学校ID")
        private Long schoolId;

        @ApiModelProperty("学校名称")
        private String schoolName;

        @ApiModelProperty("用户帐号ID")
        private Long userId;

        @ApiModelProperty("用户身份(校长，乐团领队，分管领导，负责老师)")
        private ESchoolStaffType userType;

        @ApiModelProperty("所在分部编号")
        private String organIdList;

        @ApiModelProperty("机构编号")
        private Integer tenantId;

        @ApiModelProperty("用户名称")
        private String username;

        @ApiModelProperty("联系方式")
        private String mobile;

        @ApiModelProperty("用户头像")
        private String avatar;

        @ApiModelProperty("帐号状态(注销 -1，冻结 9，激活 1)")
        private String status;

        @ApiModelProperty("注册时间")
        private Date createTime;

        @ApiModelProperty("验证码")
        private String smsCode;


        @ApiModelProperty("是否验证验证码")
        private Boolean checkSmsCode;
        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public static SchoolStaff from(String json) {
            return JSON.parseObject(json, SchoolStaff.class);
        }


        public SchoolStaff organIdList(String organIdList) {
            this.organIdList = organIdList;
            return this;
        }

        public SchoolStaff tenantId(Integer tenantId) {
            this.tenantId = tenantId;
            return this;
        }
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel(" StaffSysUser-学校员工")
    public static class StaffSysUser implements Serializable {

        @ApiModelProperty(value = "用户名")
        private String username;

        @ApiModelProperty(value = "手机号")
        private String phone;

        @ApiModelProperty(value = "头像")
        private String avatar;

        @ApiModelProperty(value = "部门ID")
        private Integer organId;

        @ApiModelProperty(value = "0-正常，9-锁定")
        private Integer lockFlag;

        @ApiModelProperty(value = "0-正常，1-删除")
        private YesOrNoEnum delFlag;

        @ApiModelProperty(value = "微信openid")
        private String wxOpenid;

        @ApiModelProperty(value = "QQ openid")
        private String qqOpenid;

        @ApiModelProperty("用户类型")
        private String userType;

        @ApiModelProperty(value = "性别0女1男")
        private Integer gender;

        @ApiModelProperty(value = "民族")
        private String nation;

        @ApiModelProperty(value = "生日")
        private Date birthdate;

        @ApiModelProperty(value = "邮箱")
        private String email;

        @ApiModelProperty(value = "融云token")
        private String imToken;

        @ApiModelProperty(value = "身份证号")
        private String idCardNo;

        @ApiModelProperty(value = "真实姓名")
        private String realName;

        @ApiModelProperty(value = "微信号")
        private String wechatId;

        @ApiModelProperty("服务标签")
        private Integer serviceTag;

        @ApiModelProperty("运营标签")
        private Integer operatingTag;

        @ApiModelProperty(value = "联系地址")
        private String contactAddress;

        @ApiModelProperty(value = "邮编")
        private String postalCode;

        @ApiModelProperty("证书类型")
        private String certificateType;

        @ApiModelProperty(value = "银行卡号")
        private String bankCard;

        @ApiModelProperty(value = "开户行")
        private String openBankAddress;

        @ApiModelProperty("职位名称")
        private String positionName;

        @ApiModelProperty(value = "用户职位")
        private String positions;

        @ApiModelProperty("分部编号列表")
        private String organIdList;

        @ApiModelProperty("密码重置标识")
        private Boolean resetFlag;

        @ApiModelProperty("学校ID")
        private Long schoolId;

        @ApiModelProperty("用户帐号ID")
        private Long userId;

        @ApiModelProperty("用户身份(校长，乐团领队，分管领导，负责老师)")
        private ESchoolStaffType schoolUserType;

        @ApiModelProperty("机构编号")
        private Integer tenantId;

        @ApiModelProperty("合作单位名称")
        private String schoolName;

        @ApiModelProperty("合作单位Logo")
        private String logo;

        @ApiModelProperty("IM用户ID")
        private String imUserId;

        public static StaffSysUser from(String json) {
            return JSON.parseObject(json, StaffSysUser.class);
        }


        public StaffSysUser schoolId(Long schoolId) {
            this.schoolId = schoolId;
            return this;
        }

        public StaffSysUser userId(Long userId) {
            this.userId = userId;
            return this;
        }

        public StaffSysUser schoolUserType(ESchoolStaffType schoolUserType) {
            this.schoolUserType = schoolUserType;
            return this;
        }

        public StaffSysUser tenantId(Integer tenantId) {
            this.tenantId = tenantId;
            return this;
        }

        public StaffSysUser organIdList(String organIdList) {
            this.organIdList = organIdList;
            return this;
        }

        public StaffSysUser schoolName(String schoolName) {
            this.schoolName = schoolName;
            return this;
        }

        public StaffSysUser logo(String logo) {
            this.logo = logo;
            return this;
        }
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("学校统计信息")
    public static class  SchoolStaffStat implements Serializable {

        @ApiModelProperty("校长数")
        private Integer schoolmasterNums;

        @ApiModelProperty("分管领导数")
        private Integer schoolLeaderNums;

        @ApiModelProperty("乐团领队")
        private Integer orchestraLeaderNums;

        @ApiModelProperty("负责老师数")
        private Integer teacherNums;

        @ApiModelProperty("乐团数")
        private Integer orchestraNums;

        @ApiModelProperty("在读学员数")
        private Integer studentNums;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("学校合作单位")
    public static class SchoolCooperation implements Serializable {

        @ApiModelProperty("合作单位")
        private CooperationOrgan cooperation;

        @ApiModelProperty("员工身份统计")
        private SchoolStaffStat staffStat;
    }

    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    @ApiModel("合作单位")
    public static class Cooperation implements Serializable {

        @ApiModelProperty("合作单位编号")
        private Integer id;

        @ApiModelProperty("合作单位logo")
        private String logo;

        @ApiModelProperty("合作单位名称")
        private String name;

        @ApiModelProperty("更新时间")
        private Date updateTime;

        @ApiModelProperty("机构编号")
        private Integer tenantId;

        @ApiModelProperty("帐号状态(注销 -1，冻结 9，激活 0)")
        private String status;
        public Date getUpdateTime() {
            return Optional.ofNullable(updateTime).orElse(DateTime.now().toDate());
        }

        public String jsonString() {
            return JSON.toJSONString(this);
        }

        public Integer getTenantId() {
            return Optional.ofNullable(tenantId).orElse(TenantContextHolder.getTenantId());
        }

        public Cooperation tenantId(Integer tenantId) {
            this.tenantId = tenantId;
            return this;
        }
    }


    @Data
    @ApiModel("SchoolStaffUpdate-后台修改账户")
    public static class SchoolStaffUpdate {

        // 修改用户手机号，合作单位，账号类型
        @ApiModelProperty("用户ID")
        @NotNull(message = "用户ID不能为空")
        private Long staffId;

        @ApiModelProperty("手机号")
//        @NotNull(message = "手机号不能为空")
        private String phone;

        @ApiModelProperty("合作单位ID")
//        @NotNull(message = "合作单位ID不能为空")
        private Long coopId;

        @ApiModelProperty("用户类型")
//        @NotNull(message = "用户类型不能为空")
        private ESchoolStaffType userType;

    }
}