package com.ym.mec.biz.dal.wrapper;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.enums.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

public class StudentWrapper {

    @Data
    @ApiModel("StudentQuery-学生查询")
    public static class StudentQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("关键字匹配")
        private String keyword;

        @ApiModelProperty("乐团ID")
        private String musicGroupId;

        @ApiModelProperty("声部ID")
        private Integer subjectId;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        @ApiModelProperty("是否vip")
        private Boolean vipFlag;


        @ApiModelProperty("学生ID")
        private Integer studentId;

        @ApiModelProperty("学生乐团状态")
        private List<StudentMusicGroupStatusEnum> statusList;

    }


    @Data
    @ApiModel("StudentAttendanceQuery-学生考勤查询")
    public static class StudentAttendanceQuery {


        @ApiModelProperty("乐团ID")
        private String musicGroupId;

        @ApiModelProperty("乐团ID集合")
        private List<String> musicGroupIds;

        @ApiModelProperty("学生ID集合")
        private List<Integer> studentIds;

        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date  endTime;

    }


    @Data
    @ApiModel("StudentStat-学生统计")
    public static class StudentStat{

        @ApiModelProperty("在读学生人数")
        private Integer studentCount;

        @ApiModelProperty("退团学生人数")
        private Integer quitCount;

    }



    @Data
    @ApiModel("StudentHomework-学生作业")
    public static class StudentHomework{


        @ApiModelProperty("学生ID")
        private Integer studentId;

        @ApiModelProperty("达标状态")
        private StandardEnum standardFlag;

        @ApiModelProperty("作业类型")
        private ELessonTrainingType type;

    }

    @Data
    @ApiModel("StudentList-学生列表")
    public static class StudentList{

        @ApiModelProperty("学生ID")
        private Integer studentId;

        @ApiModelProperty("学生姓名")
        private String studentName;

        @ApiModelProperty("乐团ID")
        @JsonIgnore
        private String musicGroupIds;

        @ApiModelProperty("学生头像")
        private String studentAvatar;

        @ApiModelProperty("声部ID")
        private Integer subjectId;

        @ApiModelProperty("声部名称")
        private String subjectName;

        @ApiModelProperty("是否vip")
        private Boolean vipFlag;

        @ApiModelProperty("实际出勤数")
        private Integer actualAttendanceCount;

        @ApiModelProperty("应出勤数")
        private Integer shouldAttendanceCount;

        @ApiModelProperty("正常出勤数")
        private Integer normalAttendanceCount;

        @ApiModelProperty("迟到")
        private Integer lateCount;

        @ApiModelProperty("请假")
        private Integer leaveCount;

        @ApiModelProperty("旷课")
        private Integer truancyCount;

        @ApiModelProperty("实际提交作业数")
        private Integer actualSubmitCount;

        @ApiModelProperty("应提交作业数")
        private Integer shouldSubmitCount;

        @ApiModelProperty("合格")
        private Integer qualifiedCount;

        @ApiModelProperty("不合格")
        private Integer unqualifiedCount;

        @ApiModelProperty("未提交")
        private Integer unsubmitCount;

    }



    @EqualsAndHashCode(callSuper = true)
    @Data
    @ApiModel("StudentDetail-学生详情")
    public static class StudentDetail extends StudentList{

        @ApiModelProperty(value = "性别0女1男")
        private Integer gender;

        @ApiModelProperty(value = "手机号")
        private String phone;

        @ApiModelProperty(value = "分部id")
        private Integer organId;


        @ApiModelProperty(value = "年级")
        private String currentGrade;

        @ApiModelProperty(value = "年级（数字表示）")
        private Integer currentGradeNum;

        @ApiModelProperty(value = "班级")
        private String currentClass;

        @ApiModelProperty(value = "艺术实践次数")
        private Integer artPracticeCount;

        @ApiModelProperty(value = "在团状态 在团 IN 退团 OUT 申请退团 APPLY_OUT 报名：APPLY 休学：QUIT_SCHOOL")
        private EInGroupStatus inGroupStatus;

        @ApiModelProperty(value = "退团时间")
        private Date quitTime;
    }



    @Data
    @ApiModel("UpdateStudent-学生修改")
    public static class UpdateStudent {

        @ApiModelProperty(value = "年级")
        private String currentGrade;

        @ApiModelProperty(value = "年级（数字表示）")
        private Integer currentGradeNum;

        @ApiModelProperty(value = "班级")
        private String currentClass;

        @NotNull(message = "学生ID不能为空")
        @ApiModelProperty(value = "学生ID")
        private Integer studentId;

        @NotNull(message = "乐团ID不能为空")
        @ApiModelProperty(value = "乐团ID")
        private String musicGroupId;

        @ApiModelProperty(value = "合作机构ID")
        private Integer cooperationOrganId;
    }

    @Data
    @ApiModel("班级在读学员列表")
    public static class ClassStudentQuery {

        @ApiModelProperty(value = "班级编号", required = false)
        private Integer classGroupId;

        @ApiModelProperty(value = "模糊匹配", required = false)
        private String search;

        @ApiModelProperty(value = "声部", required = false)
        private Integer subjectId;

        @ApiModelProperty(value = "有剩余时长的课程类型,(classroom,comprehensive,high,high_online,mix,single,training_mix,training_single,no_course)", required = false)
        private String courseType;

        @ApiModelProperty(value = "排序字段", required = false)
        private String sortField;

        @ApiModelProperty(value = "排序方式，ASC,DESC", required = false)
        private String sortType;

    }

    @Data
    @ApiModel("班级在读学员列表")
    public static class ClassStudentDto {

        @ApiModelProperty(value = "学生编号", required = false)
        private Integer userId;

        @ApiModelProperty(value = "学生姓名", required = false)
        private String name;

        @ApiModelProperty(value = "性别（0，女  1，男", required = false)
        private Integer gender;

        @ApiModelProperty(value = "实际科目名称", required = false)
        private String subjectName;

        @ApiModelProperty(value = "家长电话", required = false)
        private String parentsPhone;

        @ApiModelProperty(value = "乐团编号", required = false)
        private String musicGroupId;

        @ApiModelProperty(value = "声部课剩余时长", required = false)
        private Integer single;

        @ApiModelProperty(value = "合奏课剩余时长", required = false)
        private Integer mix;

        @ApiModelProperty(value = "课堂课剩余时长", required = false)
        private Integer classroom;

        @ApiModelProperty(value = "综合课剩余时长", required = false)
        private Integer comprehensive;

        @ApiModelProperty(value = "基础技能课剩余时长", required = false)
        private Integer high;

        @ApiModelProperty(value = "线上基础技能课剩余时长", required = false)
        private Integer highOnline;

        @ApiModelProperty(value = "集训声部课剩余时长", required = false)
        private Integer trainingSingle;

        @ApiModelProperty(value = "集训合奏课剩余时长", required = false)
        private Integer trainingMix;
    }
}
