package com.ym.mec.biz.dal.wrapper;

import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.enums.CourseStatusEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

public class TeachingPointWrapper {

    @Data
    @ApiModel("TeachingPointQuery-教学点查询")
    public static class TeachingPointQuery implements QueryInfo {

        @ApiModelProperty("当前页")
        private Integer page;

        @ApiModelProperty("分页行数")
        private Integer rows;

        @ApiModelProperty("分部")
        private Integer organId;

        @ApiModelProperty("合作单位ID")
        private Integer cooperationOrganId;

        @ApiModelProperty("乐团ID")
        private String musicGroupId;


        @ApiModelProperty("开始时间")
        private Date startTime;

        @ApiModelProperty("结束时间")
        private Date endTime;


        @ApiModelProperty("课程状态")
        private CourseStatusEnum courseStatus;
    }


    @Data
    @ApiModel("TeachingPoint-教学点")
    public static class TeachingPoint{


        @NotBlank(message = "教学点不能为空")
        @ApiModelProperty("教学点")
        private String teachingPoint;

//        @NotNull(message = "课程ID不能为空")
        @ApiModelProperty("班级ID")
        private Integer classGroupId;


//        @NotNull(message = "课程ID不能为空")
        @ApiModelProperty("课程ID")
        private Integer courseScheduleId;

    }


    @Data
    @ApiModel("CourseTeachingPoint-课程教学信息")
    public static class CourseTeachingPoint {

        @ApiModelProperty("课程ID")
        private Long courseId;

        @ApiModelProperty("合作单位")
        private String coopName;

        @ApiModelProperty("课程名")
        private String courseName;

        @ApiModelProperty("课程类型")
        private CourseSchedule.CourseScheduleType courseType;

        @ApiModelProperty("分部")
        private Integer organId;

        @ApiModelProperty("分部")
        private String organName;

        @ApiModelProperty(value = "课程状态")
        private CourseStatusEnum courseStatus;


        @ApiModelProperty("课程日期")
        private Date courseDate;

        /** 上课时间 */
        @ApiModelProperty(value = "上课时间")
        private Date startClassTime;

        @ApiModelProperty(value = "结束时间")
        private Date endClassTime;

        @ApiModelProperty("乐团ID")
        private String musicGroupId;

        @ApiModelProperty("乐团名")
        private String musicGroupName;

        @ApiModelProperty("老师ID")
        private Integer teacherId;

        @ApiModelProperty("老师名")
        private String teacherName;

        @ApiModelProperty("老师头像")
        private String teacherAvatar;

        @ApiModelProperty("签到照片")
        private String signPhoto;

        @ApiModelProperty("签退照片")
        private String signOutPhoto;

        @ApiModelProperty("教学点")
        private String teachingPoint;
    }



    @Data
    @ApiModel("ClassTeachingPoint-班级教学信息")
    public static class ClassTeachingPoint {

        @ApiModelProperty("班级ID")
        private Integer classGroupId;

        @ApiModelProperty("班级名")
        private String classGroupName;

        @ApiModelProperty("乐团ID")
        private String musicGroupId;

        @ApiModelProperty("乐团名")
        private String musicGroupName;

        @ApiModelProperty("老师ID")
        private Integer teacherId;

        @ApiModelProperty("老师名")
        private String teacherName;

        @ApiModelProperty("老师头像")
        private String teacherAvatar;

        @ApiModelProperty("教学点")
        private String teachingPoint;
    }

}
