package com.ym.mec.biz.dal.wrapper;

import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.dto.PayParamBasicDto;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.math.BigDecimal;
import java.util.Date;

@ApiModel(value = "TempDirectorTrainingCampDetailWrapper对象")
public class TempDirectorTrainingCampDetailWrapper {


    @Data
    @ApiModel("PayDto")
    public static class PayDto extends PayParamBasicDto {

        @ApiModelProperty(value = "训练营详情编号",required = false)
        private Integer tempDirectorTrainingCampDetailId;
    }

    @Data
    @ApiModel("UserDetail")
    public static class UserDetail {

        @ApiModelProperty(value = "身份证号")
        private String cardNo;

        @ApiModelProperty(value = "手机号")
        private String mobile;

        @ApiModelProperty(value = "训练营编号")
        private Integer tempDirectorTrainingCampId;
    }

    @Data
    @ApiModel("UserPayDetail")
    public static class UserPayDetail {

        @ApiModelProperty(value = "用户名")
        private String username;

        @ApiModelProperty(value = "训练营名称")
        private String campName;

        @ApiModelProperty(value = "训练营详情编号")
        private Integer campDetailId;
    }

    @Data
    @ApiModel("ApplyCampResult")
    public static class ApplyCampResult {
        @ApiModelProperty(value = "支付状态")
        private String paymentStatus;

        @ApiModelProperty(value = "训练营详情编号")
        private Integer campDetailId;
    }

    @Data
    @ApiModel(" applyCamp-新增/修改")
    public static class ApplyCamp {

        @ApiModelProperty(value = "训练营编号")
        @NotNull(message = "训练营编号不能为空")
        private Integer tempDirectorTrainingCampId;

        @ApiModelProperty(value = "姓名")
        @NotEmpty(message = "请输入您的真实姓名")
        @Pattern(regexp = "^[\\u4e00-\\u9fa5]{2,10}$", message = "姓名必须是2到10个中文字符")
        private String username;

        @ApiModelProperty(value = "手机号")
        @NotEmpty(message = "手机号不能为空")
        @Pattern(regexp = "^1[3-9]\\d{9}$", message = "手机号格式不正确")
        private String mobile;

        @ApiModelProperty(value = "身份证号")
        @NotEmpty(message = "身份证号不能为空")
        @Pattern(regexp = "(^\\d{6}(18|19|20)\\d{2}(0[1-9]|1[0-2])(0[1-9]|[12]\\d|3[01])\\d{3}(\\d|X|x)$)|(^\\d{6}\\d{2}(0[1-9]|1[0-2])(0[1-9]|[12]\\d|3[01])\\d{3}$)", message = "身份证号格式不正确")
        private String cardNo;

        @ApiModelProperty(value = "省份")
        @NotEmpty(message = "请选择您所在的城市")
        @Pattern(regexp = "^[\\u4e00-\\u9fa5]{2,20}$", message = "省份名称必须是2到20个中文字符")
        private String province;

        @ApiModelProperty(value = "省份")
        private String provinceCode;

        @ApiModelProperty(value = "城市")
        private String cityCode;

        @ApiModelProperty(value = "城市")
        private String city;

        @ApiModelProperty(value = "区县")
        private String regionCode;

        @ApiModelProperty(value = "区县")
        private String region;

        @ApiModelProperty(value = "职业")
        @NotEmpty(message = "请选择目前从事的职业")
        private String job;

        @ApiModelProperty(value = "声部")
        @NotEmpty(message = "请选择您的专业")
        private String subjectName;

        @ApiModelProperty(value = "之前乐队指导学习基础")
        @NotEmpty(message = "请选择学习基础")
        private String study;

        @ApiModelProperty(value = "目标效果")
        private String studyTarget;

        @ApiModelProperty(value = "需求或建议")
        private String suggestion;

        @ApiModelProperty(value = "验证码")
        private String authCode;
    }

    @Data
    public static class TempDirectorTrainingCampDetailQuery implements QueryInfo {
        /**
         * 默认当前页码
         */
        private Integer page = 1;
        /**
         * 默认页码大小
         */
        private Integer rows = 20;

        @ApiModelProperty(value = "用户姓名/手机号")
        private String search;

        @ApiModelProperty(value = "训练营编号")
        private Integer tempDirectorTrainingCampId;

        @ApiModelProperty(value = "省份")
        private String provinceCode;

        @ApiModelProperty(value = "城市")
        private String cityCode;


        @ApiModelProperty(value = "区县")
        private String regionCode;


        @ApiModelProperty(value = "已缴费 PAID,未缴费 NO,已退费 REFUND")
        private String paymentStatus;
    }

    @Data
    public static class TempDirectorTrainingCampDetail {


        @ApiModelProperty(value = "主键")
        private Integer id;

        @ApiModelProperty(value = "训练营编号")
        private Integer tempDirectorTrainingCampId;

        @ApiModelProperty(value = "姓名")
        private String username;

        @ApiModelProperty(value = "手机号")
        private String mobile;

        @ApiModelProperty(value = "身份证号")
        private String cardNo;

        @ApiModelProperty(value = "省份")
        private String province;

        @ApiModelProperty(value = "省份")
        private String provinceCode;

        @ApiModelProperty(value = "城市")
        private String cityCode;

        @ApiModelProperty(value = "城市")
        private String city;

        @ApiModelProperty(value = "区县")
        private String regionCode;

        @ApiModelProperty(value = "区县")
        private String region;

        @ApiModelProperty(value = "所在城市")
        private String cityInfo;

        @ApiModelProperty(value = "已缴费 PAID,未缴费 NO,已退费 REFUND")
        private String paymentStatus = "NO";

        @ApiModelProperty(value = "缴费状态")
        private String paymentStatusName;

        @ApiModelProperty(value = "缴费金额")
        private String paymentAmount;

        @ApiModelProperty(value = "是否早鸟报名")
        private Boolean birdFlag = false;

        @ApiModelProperty(value = "缴费时间")
        private Date paymentDate;

        @ApiModelProperty(value = "缴费时间")
        private String paymentDateStr;

        @ApiModelProperty(value = "职业")
        private String job;

        @ApiModelProperty(value = "声部")
        private String subjectName;

        @ApiModelProperty(value = "之前乐队指导学习基础")
        private String study;

        @ApiModelProperty(value = "目标效果")
        private String studyTarget;

        @ApiModelProperty(value = "需求或建议")
        private String suggestion;

        @ApiModelProperty(value = "创建时间")
        private Date createTime;

        @ApiModelProperty(value = "修改时间")
        private Date updateTime;
    }


    @Data
    public static class TempDirectorTrainingCampDetailStat {

        @ApiModelProperty("报名人数")
        private Integer applyCount;

        @ApiModelProperty("缴费人数")
        private Integer payCount;

        @ApiModelProperty("早鸟人数")
        private Integer birdCount;

        @ApiModelProperty("收款金额")
        private BigDecimal payAmount;

    }


    @Data
    @Builder
    @NoArgsConstructor
    @AllArgsConstructor
    public static class DownloadInfo {

        @ApiModelProperty("文件名称")
        private String fileName;

        @ApiModelProperty("下载地址")
        private String downloadPath;

    }
}
