package com.ym.mec.biz.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.microsvc.toolkit.middleware.live.message.TencentWrapper;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dto.ImLiveBroadcastRoomDto;
import com.ym.mec.biz.dal.dto.LiveRoomStatus;
import com.ym.mec.biz.dal.dto.RoomReservationUserSearch;
import com.ym.mec.biz.dal.dto.TencentData;
import com.ym.mec.biz.dal.entity.ImLiveBroadcastRoom;
import com.ym.mec.biz.dal.page.LiveRoomGoodsOrderQueryInfo;
import com.ym.mec.biz.dal.vo.BaseRoomUserVo;
import com.ym.mec.biz.dal.vo.ImLiveBroadcastRoomVo;
import com.ym.mec.biz.dal.vo.RoomReservationUserVo;
import com.ym.mec.biz.dal.vo.LiveRoomGoodsOrderVo;
import com.ym.mec.biz.dal.vo.RoomUserInfoVo;
import com.ym.mec.common.entity.ImUserState;
import com.ym.mec.common.page.PageInfo;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * 直播房间管理表(ImLiveBroadcastRoom)表服务接口
 *
 * @author hgw
 * @since 2022-02-17 20:52:05
 */
public interface ImLiveBroadcastRoomService extends IService<ImLiveBroadcastRoom> {

    PageInfo<ImLiveBroadcastRoomVo> queryPage(Map<String, Object> param);

    ImLiveBroadcastRoomVo queryRoomAndCheck(String roomUid, Integer userId, Integer osType);

    /**
     * 游客直播间信息
     * @param roomUid 直播间编号
     * @param userId 游客编号
     * @return ImLiveBroadcastRoomVo
     */
    ImLiveBroadcastRoomVo visitorRoomInfo(String roomUid, Integer userId);

    ImLiveBroadcastRoomVo queryRoomInfo(String roomUid);

    void add(ImLiveBroadcastRoomDto dto);

    void update(ImLiveBroadcastRoomDto dto);

    void whetherChat(Integer id, Integer whetherChat);

    /**
     * 关闭房间-融云
     * 获取所有直播间缓存数据并写入数据库后并清理缓存
     *
     * @param roomUid 直播间Uid
     */
    boolean roomDestroy(String roomUid);

    boolean roomDestroy(Integer id);

    void delete(Integer id);

    /**
     * 推广直播间-每个机构只能有一个直播间在首页推广
     *
     * @param id         直播间id
     * @param popularize 是否在首页推广 0否 1是
     */
    void opsPopularize(Integer id, Integer popularize);

    /**
     * 查询该机构目前推广的直播间
     */
    ImLiveBroadcastRoomVo queryPopularizeRoom();

    void destroyExpiredLiveRoom();

    void sendForcedOffline(ImLiveBroadcastRoom room);

    void syncLike(String roomUid, Integer likeNum);

    void opsRoom(List<ImUserState> userState);

    ImLiveBroadcastRoomVo speakerJoinRoom(String roomUid, String os);

    void setGroupMemberDefinedData(ImLiveBroadcastRoomVo roomVo, Integer onlineUser, Integer totalUser);

    void joinRoom(String roomUid, Integer userId);

    void joinRoom(String roomUid, Integer userId, Boolean microphoneFlag);

    /**
     * 游客加入直播间
     * @param roomUid 直播间编号
     * @param userId 游客编号
     */
    void visitorJoinRoom(String roomUid, Integer userId);

    /**
     * 游客加入直播间凭据
     * @param roomUid 直播间编号
     * @param fingerprint 游客凭据
     * @return RoomUserInfoVo
     */
    RoomUserInfoVo getVisitorCredentials(String roomUid, String fingerprint);

    /**
     * 游客微信身份
     * @param code 授权码
     * @return 用户openId
     */
    String visitorFingerprint(String code);

    /**
     * 直播间访问用户信息
     * @param userId 用户编号
     * @param roomUid 直播间编号
     * @return SysUser
     */
    SysUser getRoomSysUser(Integer userId, String roomUid);

    void startLive(String roomUid, Integer userId,String videoResolution);

    void startLive(String roomUid, Integer userId, String videoResolution, String sequence);

    void closeLive(String roomUid, Integer userId);

    void closeLive(String roomUid, Integer userId, String sequence);

    void createLiveRoom();

    void createLiveRoom(ImLiveBroadcastRoom room);

    void createLiveRoomNoTransactional(ImLiveBroadcastRoom room);

    Map<String, Object> test(String roomUid, String userFlag);

    /**
     * @param roomUid 直播间uid
     * @author zx
     * @since 2022/2/23 16:17
     * 分享直播链接
     */
    void shareGroup(String roomUid, String groupIds);

    List<BaseRoomUserVo> queryRoomOnlineUserInfo(String roomUid);

    List<BaseRoomUserVo> queryRoomLimitOnlineUserInfo(String roomUid);

    /**
     * 根据传入的用户id查询用户信息
     *
     * @param userIdList 用户id集合
     */
    List<BaseRoomUserVo> queryBaseUserInfo(List<Integer> userIdList);

    /**
     * 获取直播间预约用户信息
     *
     * @param query
     * @return
     */
    PageInfo<RoomReservationUserVo> queryRoomUser(RoomReservationUserSearch query);

    /**
     * 导出直播间预约用户信息
     *
     * @param query
     * @param response
     */
    void exportReservationRoomUser(RoomReservationUserSearch query, HttpServletResponse response) throws IOException;

    //查询直播间商品订单列表
    PageInfo<LiveRoomGoodsOrderVo> queryLiveRoomGoodsOrderList(LiveRoomGoodsOrderQueryInfo queryInfo);


    /**
     * 设置是否连麦
     *
     * @param roomUid 房间号
     * @param whetherMic 是否连麦 0： 是 1：否
     */
    void whetherMic(String roomUid, Integer whetherMic);

    /**
     * 更新主播直播间状态
     *
     * @param status 直播间状态
     * @return
     */
    Boolean updateRoomStatus(LiveRoomStatus status);

    /**
     * 直播群成员在线状态回调处理
     *
     */
    void callbackOnMemberStateChange(TencentData.CallbackOnMemberStateChange callbackOnMemberStateChange);
    /**
     * 群成员离开之后回调
     */
    void callbackAfterMemberExit(TencentData.CallbackAfterMemberExit callbackAfterMemberExit);

    /**
     * 新成员入群之后回调
     */
    void callbackAfterNewMemberJoin(TencentData.CallbackAfterNewMemberJoin callbackAfterNewMemberJoin);

    /**
     * 直播间销毁定时任务
     */
    void destroyLiveRoom();

    boolean tryDestroyLiveRoom(ImLiveBroadcastRoom imLiveBroadcastRoom);

    /**
     * 更新直播推流时间
     * @param event TencentData.CallbackStreamStateEvent
     */
    void updateLiveRoomPushStreamTime(TencentData.CallbackStreamStateEvent event);

    /**
     * 修改用户连麦状态
     *
     * @param roomUid 直播房间id
     * @param userId 用户id
     * @param whetherMicStatus 连麦状态
     */
    void userWhetherMic(String roomUid, Long userId, Integer whetherMicStatus);

    /**
     * 同步直播间点赞数
     *
     * @param roomUid 直播间uid
     */
    int syncLikeCount(String roomUid);

    /**
     * 直播间推流状态查询
     * @param roomUid 直播间ID
     * @return TencentWrapper.LiveStreamState
     */
    TencentWrapper.LiveStreamState roomLiveStreamStatus(String roomUid);

    void batchUpdateLiveRoomTitle(List<String> list, String roomTitle, String liveRemark);

    /**
     * 获取直播间信息
     * @param liveRoomId 直播间id
     * @return
     */
    ImLiveBroadcastRoom getByRoomUid(String liveRoomId);
}

