package com.ym.mec.biz.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.mapper.SchoolStaffMapper;
import com.ym.mec.biz.dal.wrapper.SchoolStaffWrapper;
import com.ym.mec.biz.dal.entity.SchoolStaff;

import java.util.List;

/**
 * 学校员工
 * 2023-05-06 10:53:06
 */
public interface SchoolStaffService extends IService<SchoolStaff>  {

    SchoolStaffMapper getMapper();

	/**
     * 查询详情
     * @param id 详情ID
     * @return SchoolStaff
     */
    SchoolStaffWrapper.SchoolStaff detail(Long id);

    /**
     * 分页查询
     * @param page IPage<SchoolStaff>
     * @param query SchoolStaffWrapper.SchoolStaffQuery
     * @return IPage<SchoolStaffWrapper.SchoolStaff>
     */
    IPage<SchoolStaffWrapper.SchoolStaff> selectPage(IPage<SchoolStaffWrapper.SchoolStaff> page, SchoolStaffWrapper.SchoolStaffQuery query);
	
    /**
     * 添加
     * @param schoolStaff SchoolStaffWrapper.SchoolStaff
     * @return Boolean
     */
     Boolean add(SchoolStaffWrapper.SchoolStaff schoolStaff);   

    /**
     * 更新
     * @param schoolStaff SchoolStaffWrapper.SchoolStaff
     * @return Boolean
     */
     Boolean update(SchoolStaffWrapper.SchoolStaff schoolStaff);

    /**
     * 学校员工信息
     * @param userId 用户ID
     * @return SchoolStaff
     */
    SchoolStaff queryByUserId(Long userId);

    /**
     * 根据用户ID获取学校所有员工
     * @param userId 用户ID
     * @return List<SchoolStaffWrapper.SchoolStaff>
     */
    List<SchoolStaffWrapper.SchoolStaff> getAllStaffsBySchoolId(Long userId);

    /**
     * 根据用户ID获取学校所有员工身份
     * @param schoolId 学校ID
     * @return SchoolStaffWrapper.SchoolStaffStat
     */
    SchoolStaffWrapper.SchoolStaffStat getUserTypeStatByCooperationId(Integer schoolId);

    /***
     * 更新合作单位信息
     * @param info SchoolStaffWrapper.Cooperation
     */
    void updateCooperation(SchoolStaffWrapper.Cooperation info);

    /**
     * 删除用户
     * @param id 用户ID
     */
    Boolean del(Long id);

    /**
     * 实名认证
     *
     * @param realName 姓名
     * @param idcardNo 身份证号
     */
    void realNameAuthentication(SysUser user, String realName, String idcardNo);

    /**
     * 查询学校员工信息
     *
     * @param userId 用户ID
     * @return SchoolStaff
     */
    SchoolStaff getByUserId(Integer userId);

    /**
     * 更新账号信息
     *
     * @param schoolStaffUpdate SchoolStaffWrapper.SchoolStaffUpdate
     */
    void updateAccount(SchoolStaffWrapper.SchoolStaffUpdate schoolStaffUpdate);

    IPage<SchoolStaffWrapper.SchoolStaff> selectBaseInfoByPage(IPage<SchoolStaffWrapper.SchoolStaff> page,
                                                               SchoolStaffWrapper.SchoolStaffQuery query);
}