package com.ym.mec.biz.service;


import com.ym.mec.biz.dal.dao.SellOrderDao;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.KitGroupPurchaseTypeEnum;
import com.ym.mec.common.dto.OrderCreate;
import com.ym.mec.common.service.BaseService;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.function.Function;


public interface SellOrderService extends BaseService<Integer, SellOrder> {

    SellOrderDao getDao();

    /**
     * 添加销售订单
     *
     * @param goodsIds 商品列表
     * @param totalAmount 订单总额
     * @param balance 余额支付
     * @param couponRemitFee 优惠券减免
     * @return
     */
//    List<SellOrder> addSellOrder(Long orderId, String musicGroupId, List<Integer> goodsIds, BigDecimal totalAmount, BigDecimal balance, BigDecimal couponRemitFee);

    /**
     * 将订单详情加入销售列表
     *
     * @param orderDetails
     * @param studentPaymentOrder
     * @return
     */
    void addOrderDetail2SellOrder(List<StudentPaymentOrderDetail> orderDetails, StudentPaymentOrder studentPaymentOrder, String musicGroup);

    void batchInsert(List<SellOrder> sellOrders);


    /**
     * 获取组合商品的相关商品
     *
     * @param orderId
     * @param parentGoodsId
     * @return
     */
    List<SellOrder> getSellOrderByParentGoodsId(Long orderId, Integer parentGoodsId);

    /**
     * 退货
     *
     * @param sellOrders
     * @param reBackFee  是否退费
     * @return
     */
    List<SellOrder> refund(List<SellOrder> sellOrders, Boolean reBackFee);

    /**
     * 根据订单id退货
     *
     * @param orderId
     * @param reBackFee 是否退费
     * @return
     */
    List<SellOrder> refundByOrderId(Long orderId, Boolean reBackFee);

    /**
     * 获取订单退货的sellOrder
     * @param orderIdList
     * @return
     */
    List<SellOrder> getRefundSellOrder(List<Long> orderIdList);


    /**
     * 获取订单的实际销售金额（除去余额部分）
     * @param orderNo
     * @return
     */
    Map<String,BigDecimal> getSellAmount(String orderNo);

    List<OrderCreate.OrderItem> convertMallOrder(List<SellOrder> sellOrderList);

    void syncSellOrder2Mall(List<SellOrder> sellOrders, StudentPaymentOrder studentPaymentOrder,String musicGroupId);

    void mallRefundByOrderId(List<SellOrder> sellOrders);

    void updateSellOrder(List<SellOrder> sellOrderList, StudentPaymentOrder studentPaymentOrder);

    List<SellOrder> initSellOrder(StudentPaymentOrder order, String musicGroupId, List<Integer> goodsIds,
                                  KitGroupPurchaseTypeEnum kitGroupPurchaseType, Boolean saveFlag);

    List<SellOrder> initSellOrder(StudentPaymentOrder order, String musicGroupId, List<Integer> goodsIds, KitGroupPurchaseTypeEnum kitGroupPurchaseType,
                                  BigDecimal totalBalance, BigDecimal expectTotalAmount, BigDecimal couponRemitAmount,
                                  BigDecimal actualTotalAmount,Boolean saveFlag);

    void deleteByOrderNo(String orderNo);

    void batchUpdate(List<SellOrder> sellOrderList);
}
