package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.LessonExamination;
import com.ym.mec.biz.dal.entity.StudentExtracurricularExercisesSituation;
import com.ym.mec.biz.dal.entity.StudentLessonExamination;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.BaseService;

import java.util.List;
import java.util.Map;

public interface StudentExtracurricularExercisesSituationService extends BaseService<Long, StudentExtracurricularExercisesSituation> {
	
	PageInfo<MusicGroupExtracurricularSummaryDto> queryStudentExerciseMusicGroupList(StudentExerciseMusicGroupQueryInfo queryInfo);

    PageInfo<StudentExercisesSituationDto> findStudentExtracurricularExercisesSituations(StudentExercisesSituationQueryInfo queryInfo);

    /**
     * @describe 获取学生服务详情
     * @author Joburgess
     * @date 2020.04.29
     * @param queryInfo:
     * @return com.ym.mec.common.page.PageInfo<com.ym.mec.biz.dal.dto.StudentServiceDetailDto>
     */
    List<Map<String, Object>> findServiceStudentDetail(StudentServiceDetailQueryInfo queryInfo);

    /**
     * @describe 获取教师服务指标明细
     * @author Joburgess
     * @date 2021/3/9 0009
     * @param queryInfo:
     * @return com.ym.mec.common.page.PageInfo<com.ym.mec.biz.dal.dto.TeacherServeDto>
     */
    PageInfo<TeacherServeDto> queryTeacherServeInfo(TeacherServeQueryInfo queryInfo);

    /**
     * @describe 教师作业布置提醒
     * @author Joburgess
     * @date 2021/3/9 0009
     * @param teacherServes:
     * @return void
     */
    void teacherServeRemindPush(List<TeacherServeDto> teacherServes, Integer operatorId, String operatorName);

    PageInfo<TeacherServeHomeworkDto> queryTeacherServeHomeworkDetail(TeacherServeHomeworkQueryInfo queryInfo);

    PageInfo<TeacherServeHomeworkPojo> queryTeacherServeHomeworkDetail1(TeacherServeHomeworkQueryInfo queryInfo);

    PageInfo<TeacherServeExtraDto> queryTeacherServeExtraDetail(TeacherServeHomeworkQueryInfo queryInfo);

    /**
     * 获取能布置作业的课程数量
     * @return
     */
    int countWaitCreateHomeworkNum();
    
    /**
     * 根据条件查询老师的服务指标
     * @param queryInfo
     * @return
     */
    List<TeacherServeDto> queryTeacherPerformanceIndicator(TeacherServeQueryInfo queryInfo);

    //批量新增
    void batchInsert(List<StudentExtracurricularExercisesSituation> situations);

    //删除指定学员服务指标
    void deleteByMonday(String monday,List<Integer> studentIds);

    //删除指定学员服务指标
    void deleteByStudent(Integer studentId,String monday);

    TeacherExercisesServiceDto getStatistics(ExtraExercilseQueryInfo queryInfo);

    /**
     * 老师端课后作业列表
     *
     */
    PageInfo<CourseHomeworkWrapper.CourseHomeworkList> queryTeacherServeHomeworkDetailV2(CourseHomeworkWrapper.CourseHomeworkQuery queryInfo);

    /**
     * 老师课外作业列表
     *
     */
    PageInfo<CourseHomeworkWrapper.CourseHomeworkList> queryTeacherExtraHomeworkDetailV2(CourseHomeworkWrapper.CourseHomeworkQuery queryInfo);

    /**
    * @description: 老师布置测验后更新服务指标相关数据
     * @param lessonExamination
     * @param studentIds
    * @return void
    * @author zx
    * @date 2023/4/10 16:18
    */
    void updateNum(LessonExamination lessonExamination, List<Integer> studentIds);

    /**
    * @description: 学员提交作业后更新服务指标
     * @param lessonExamination
     * @param studentLessonExamination
    * @return void
    * @author zx
    * @date 2023/4/10 16:38
    */
    void updateLastSubmitTime(LessonExamination lessonExamination, StudentLessonExamination studentLessonExamination);
}
