package com.ym.mec.biz.service;

import com.dayaedu.cbs.common.enums.EClientType;
import com.dayaedu.cbs.openfeign.wrapper.message.CbsMessageWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dto.SysMessageVo;
import com.ym.mec.biz.dal.entity.ImSendGroupMessage;
import com.ym.mec.biz.dal.entity.SysMessage;
import com.ym.mec.biz.dal.enums.MessageSendMode;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.page.SysMessageQueryInfo;
import com.ym.mec.biz.dal.wrapper.SysMessageWrapper;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.BaseService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;

import java.net.MalformedURLException;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

public interface SysMessageService extends BaseService<Long, SysMessage> {

	R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMessageWrapper.SysMessage>> queryListPage(SysMessageQueryInfo queryInfo);

	/**
	 * 推送站内消息
	 * @param userIds
	 * @param messageType
	 * @return
	 */
	boolean batchSeoMessage(Set<Integer> userIds, MessageTypeEnum messageType,String memo, Object... args);

	/**
	 * 推送im文本消息,包含跳转链接
	 * @param senderId
	 * @param messageType
	 * @return
	 */
	void batchSendImMessage(MessageTypeEnum messageType,String senderId,String extra, String[] targetIds,String url, Object... args);

	/**
	 * 推送im 模板文本消息,包含跳转链接
	 * @param senderId
	 * @param messageType
	 * @return
	 */
	void batchSendImGroupMessage(MessageTypeEnum messageType,String senderId,String extra, String[] targetIds,String url, Object... args);

	/**
	 * 发送文本消息
	 * @return
	 */
	void batchSendImGroupTxtMessage(ImSendGroupMessage messageDto) throws MalformedURLException;

	/**
	 * 发送自定义文本消息
	 * @param senderId
	 * @param messageContent
	 * @return
	 */
	void batchSendImGroupMessage(String messageContent,String senderId,String extra, String[] targetIds,String objectName);


	/**
	* @description: 发送图片消息
	* @return void
	* @author zx
	* @date 2022/5/17 10:05
	*/
	void batchSendImGroupImgMessage(ImSendGroupMessage messageDto) throws Exception;

	/**
	* @description: 发送文件消息
	 * @param senderId 发送者编号
	 * @param extra 扩展字段
	 * @param name	文件名称
	 * @param targetIds	接收者编号
	 * @param fileUrl	文件地址
	 * @param size	文件大小
	* @return void
	* @author zx
	* @date 2022/5/17 10:05
	*/
	void batchSendImGroupFileMessage(ImSendGroupMessage messageDto);

	/**
	 * 发送消息
	 * @param senderTenantId 消息发送者所在的机构编号
	 * @param messageSender 消息发送者
	 * @param messageType 消息类型
	 * @param receivers 消息接收者（Key:用户编号  value:消息接收对象）
	 * @param triggerTime 触发时间
	 * @param readStatus 阅读状态（0-未读  1-已读）
	 * @param url 超链接地址
	 * @param args 参数
	 * @return
	 */
	public void batchSendMessage(Integer senderTenantId, MessageSender messageSender, MessageTypeEnum messageType, Map<Integer,String> receivers, Date triggerTime,
			Integer readStatus, String url,String jpushType, Object... args);

	/**
	 * 发送消息
	 * @param messageSender 消息发送者
	 * @param messageType 消息类型
	 * @param receivers 消息接收者（Key:用户编号  value:消息接收对象）
	 * @param triggerTime 触发时间
	 * @param readStatus 阅读状态（0-未读  1-已读）
	 * @param url 超链接地址
	 * @param args 参数
	 * @return
	 */
	public void batchSendMessage(MessageSender messageSender, MessageTypeEnum messageType, Map<Integer,String> receivers, Date triggerTime,
			Integer readStatus, String url,String jpushType, Object... args);


	/**
	 * 推送极光自定义消息
	 * @param messageType 消息类型
	 * @param receivers 消息接收者（Key:用户编号  value:消息接收对象）
	 * @param triggerTime 触发时间
	 * @param readStatus 阅读状态（0-未读  1-已读）
	 * @param url 超链接地址
	 * @param args 参数
	 * @return
	 */
	public void batchPushMessage(MessageTypeEnum messageType, Map<Integer,String> receivers, Date triggerTime,
			Integer readStatus, String url,String jpushType,String sound,String channelId, Object... args);


	/**
	 * 发送验证码
	 * @param messageSender 消息发送者
	 * @param userId 用户编号
	 * @param mode 消息模式
	 * @param messageType 消息类型
	 * @param receiver 消息接受者
	 * @return
	 */
	public boolean sendSecurityCode(MessageSender messageSender, Integer userId, MessageTypeEnum messageType, String receiver,String jpushType);

	/**
	 * 获取已经发送的验证码
	 * @param type
	 * @param mobileNOOrEmailAddr
	 * @return
	 */
	public String getSendedVerificationCode(MessageTypeEnum type, String mobileNOOrEmailAddr);

	/**
	 * 查询消息未读条数
	 * @param type
	 * @param userId
	 * @return
	 */
	public Map<String, Integer> queryCountOfUnread(MessageSendMode type, Integer userId,String jpushType, Integer tenantId);

	/**
	 *
	 * @param userId 用户id
	 * @param status 状态/1 已读,0未读
	 * @return
	 */
	public Boolean updateStatus(Integer userId, int status,String jpushType);

	/**
	 * 修改单条消息状态
	 */
	public Boolean updateOneStatus(Long id);
	
	public List<CbsMessageWrapper.SysMessage> queryUserInRecentMinList(String mobile, int recentMin, MessageSendMode type);

	/**
	 * 发送私聊消息
	 * @param studentId 接收人id
	 * @param content 消息内容
	 * @param extra 扩展字段
	 */
    void sendPrivateMessage(String studentId, String content, String extra);

	/**
	 * 发送私聊消息
	 * @param sender
	 * @param receiver
	 * @param content
	 */
	void sendNoAuthPrivateMessage(String sender,String receiver, String content);

	/**
	 * 每天9点推送前一天22点之后的推送消息
	 */
	void pushWaitSendMessageTask();

	//获取短信验证码
	R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMessageWrapper.SysMessage>> queryMessageCode(SysMessageVo.SysMessageQuery query);


    PageInfo<SysMessageWrapper.Message> queryPage(SysMessageQueryInfo queryInfo);
}