package com.ym.mec.biz.service;

import com.ym.mec.biz.dal.dao.TeacherAttendanceDao;
import com.ym.mec.biz.dal.dto.StudentStandardDto;
import com.ym.mec.biz.dal.dto.TeacherAttendanceComplaintsDto;
import com.ym.mec.biz.dal.dto.TeacherAttendanceDetailDto;
import com.ym.mec.biz.dal.dto.TeacherSignOutDto;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.TeacherAttendance;
import com.ym.mec.biz.dal.enums.UpdateAttendanceEnum;
import com.ym.mec.biz.dal.page.TeacherAttendanceComplaintsQueryInfo;
import com.ym.mec.biz.dal.page.TeacherCloseQueryInfo;
import com.ym.mec.biz.dal.school.dto.TeacherAttendanceDto;
import com.ym.mec.biz.dal.school.dto.TeacherAttendanceListDto;
import com.ym.mec.biz.dal.school.dto.TeacherAttendanceSumDto;
import com.ym.mec.biz.dal.school.page.TeacherAttendanceQueryInfo;
import com.ym.mec.biz.dal.wrapper.TeacherAttendanceWrapper;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.BaseService;

import java.util.List;
import java.util.Map;

public interface TeacherAttendanceService extends BaseService<Long, TeacherAttendance> {

	TeacherAttendanceDao getDao();

	/**
	* @description: 处理学员学习标准是否达标
	 * @param studentStandardDtos
	 * @param courseScheduleId
	* @return void
	* @author zx
	* @date 2022/3/1 17:17
	*/
	void saveStudentStandard(List<StudentStandardDto> studentStandardDtos, Long courseScheduleId);

	/**
	 * @Author: Joburgess
	 * @Date: 2019/9/10
	 * 添加教师签到记录
	 */
	Map<String, Object> addTeacherAttendanceRecord(TeacherSignOutDto teacherSignOutDto);

	/**
	 * @describe 线上课老师签退
	 * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
	 * @author zouxuan
	 * @date 2020/9/27
	 * @time 9:52
	 * @param courseScheduleId:
	 * @param userId:
	 * @return void
	 */
	void addTeacherAttendanceSignOut(Long courseScheduleId,
									Integer userId,
									 String deviceNum);

	/**
	 * @describe 教师线上课签到
	 * @apiNote 时光荏苒，认真工作的时间总是过得很快，而我、享受这一刻！
	 * @author zouxuan
	 * @date 2020/9/26
	 * @time 19:59
	 * @param userId:
	 * @param courseScheduleId:
	 * @return void
	 */
	void addTeacherAttendanceSignIn(Long firstCourseId,Integer userId, Long courseScheduleId,String deviceNum);

	/**
	 * 获取教师个人的签到记录
	 * @return
	 */
	PageInfo getTeacherPersonalAttendances(TeacherCloseQueryInfo queryInfo);

	/**
	 * 获取教师个人的签到记录
	 * @return
	 */
	PageInfo queryTeacherAttendances(TeacherCloseQueryInfo queryInfo);

	/**
	 * 推送未签退消息提醒
	 * @return
	 */
	boolean pushNoSignOutMessage(Integer tenantId);

	/**
	 * @describe 获取签到详情
	 * @author Joburgess
	 * @date 2019/12/25
	 * @param courseScheduleId: 课程编号列表
	 * @param userId: 用户编号
	 * @return com.ym.mec.biz.dal.dto.TeacherSignOutDto
	 */
	TeacherSignOutDto getTeacherSignDetail(Integer courseScheduleId, Integer userId);

	/**
	 * 获取单节课教师签到记录
	 * @param courseScheduleId
	 * @return
	 */
	TeacherAttendance getTeacherPersonalAttendance(Integer courseScheduleId);

	/**
	 * @describe 更新课程签到信息
	 * @author Joburgess
	 * @date 2020/1/17
	 * @param teacherAttendance: 签到信息
	 * @return void
	 */
	void updateTeacherAttendance(TeacherAttendance teacherAttendance);

	/**
	 * 获取教师签到详情
	 * @param courseScheduleId
	 * @return
	 */
	TeacherAttendanceDetailDto getTeacherPersonalAttendanceDetail(Integer courseScheduleId);

	/**
	 * 推送教师异常考勤
	 */
    void pushTeacherExceptionAttendanceTask();

	/**
	 * 提交考勤申述
	 * @param courseScheduleId
	 * @param content
	 */
	void addComplaints(Long courseScheduleId, String content,String url,Integer userId, UpdateAttendanceEnum complaintsType);

	/**
	 * 撤销考勤申述
	 * @param courseScheduleId
	 */
	void repealComplaints(Long courseScheduleId,Integer userId);

	/**
	 * 获取教师申述列表
	 * @param queryInfo
	 */
	PageInfo<TeacherAttendanceComplaintsDto> queryTeacherAttendanceComplaints(TeacherAttendanceComplaintsQueryInfo queryInfo);

	/**
	 * 同意异常考勤申述
	 * @param teacherAttendanceId
	 */
	void agreeTeacherAttendanceComplaints(long teacherAttendanceId,String content);

	/**
	 * 拒绝教师课酬申述
	 * @param teacherAttendanceId
	 */
	void rejectTeacherAttendanceComplaints(long teacherAttendanceId,String content);

	/**
	 * 更新签退设备号
	 * @param courseId
	 * @param userId
	 * @param signInDeviceNum
	 * @param signOutDeviceNum
	 */
	void updateDeviceNum(Integer courseId, String userId,String signInDeviceNum, String signOutDeviceNum);

	void addTeacherAttendance(List<CourseSchedule> courseScheduleList);

	/**
	* @description: 学校端获取本学期课程关联的老师列表，以及老师考勤状况汇总
	 * @param queryInfo
	* @return com.ym.mec.biz.dal.school.dto.TeacherAttendanceSumDto
	* @author zx
	* @date 2023/5/4 14:39
	*/
	List<TeacherAttendanceSumDto> queryTeacherAttendance(TeacherAttendanceQueryInfo queryInfo);

	/**
	* @description: 获取本学期当前班级老师所有课程考勤
	* @return java.util.List<com.ym.mec.biz.dal.school.dto.TeacherAttendanceListDto>
	* @author zx
	* @date 2023/5/5 09:54
	*/
	List<TeacherAttendanceListDto> getClassTeacherAttendance(TeacherAttendanceWrapper.ClassTeacherAttendanceQuery query);

	/**
	* @description: 获取课程老师考勤
	 * @param courseId
	* @return com.ym.mec.biz.dal.school.dto.TeacherAttendanceDto
	* @author zx
	* @date 2023/5/5 09:55
	*/
	TeacherAttendanceDto getCourseTeacherAttendance(Long courseId, Integer teacherId);

    Map<Long,TeacherAttendance> getMapByCourseIds(List<Long> courseIds);
}
