package com.ym.mec.biz.service.im;

import com.microsvc.toolkit.middleware.im.message.GroupMemberWrapper;
import com.ym.mec.biz.dal.wrapper.ImGroupMemberWrapper;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;

import java.util.List;

/**
 * IM群组服务
 * Created by Eric.Shang on 2022/11/26.
 */
public interface ImGroupCoreService {

    /**
     * IM 用户注册
     * @param userId 用户ID
     * @param username 用户名
     * @param avatar 用户头象
     * @return IM聊天Token
     */
    ImGroupWrapper.ImUserInfo register(String userId, String clientType, String username, String avatar) throws Exception;

    /**
     * IM 用户签名
     * @param userId 用户ID
     * @param clientType 用户身份
     * @return ImGroupWrapper.ImUserInfo
     */
    ImGroupWrapper.ImUserInfo getUserSig(String userId, String clientType);

    /**
     * IM 用户注册
     * @param userId 用户Id
     * @return String
     */
    String getImUserId(String userId, String clientType);

    /**
     * 解析IM用户规则
     * @param imUserId IM用户Id
     * @return String
     */
    String analysisImUserId(String imUserId);

    /**
     * 添加群组信息
     *
     * @param groupMember 用户信息
     * @param imGroup     ImGroupWrapper.ImGroup
     * @return Boolean
     * @throws Exception Exception
     */
    String groupCreate(ImGroupMemberWrapper.ImGroupMember groupMember, ImGroupWrapper.ImGroup imGroup) throws Exception;

    /**
     * 群组解散
     * @param groupId 群组ID
     * @throws Exception Exception
     */
    void groupDismiss( String groupId) throws Exception;

    /**
     * 用户主动退出群聊
     * @param groupMember 用户信息
     * @param groupId 群组ID
     * @throws Exception Exception
     */
    void groupQuit(GroupMemberWrapper.ImGroupMember groupMember, String groupId) throws Exception;


    /**
     * 批量退群
     *
     */
    void groupQuit(List<GroupMemberWrapper.ImGroupMember> imGroupMembers, String groupId) throws Exception;


    /**
     * 批量退群
     *
     */
    void groupQuit(List<GroupMemberWrapper.ImGroupMember> imGroupMembers, String groupId,Boolean quit) throws Exception;

    /**
     * 用户主动退出群聊
     * @param userId 用户信息
     * @param clientType EClientType
     * @param imGroupId 群组ID
     * @throws Exception Exception
     */
    void groupQuit(Long userId, String clientType, String imGroupId) throws Exception;

    /**
     * 添加群组成员
     * @param groupId 群组ID
     * @param groupMembers List<ImGroupMemberWrapper.ImGroupMember>
     * @throws Exception Exception
     */
    void saveImGroupMemberList(String groupId, List<ImGroupMemberWrapper.ImGroupMember> groupMembers) throws Exception;

    /**
     * 群成员加入
     * @param imGroupId 群主ID
     * @param groupMembers 群成员
     * @throws Exception Exception
     */
    void groupMemberJoin(String imGroupId, List<ImGroupMemberWrapper.ImGroupMember> groupMembers) throws Exception;

    /**
     * 转让群主
     *
     * @param groupId 群组ID
     * @param newOwner 新群主
     * @param oldOwner 旧群主
     */
    void changeGroupOwner(String groupId, String newOwner, String oldOwner) throws Exception;

    /**
     * 更新群信息
     * @param groupId 群ID
     * @param groupName 群名称
     * @param faceUrl 群头像
     * @throws Exception Exception
     */
    void modifyGroupInfo(String groupId, String groupName, String faceUrl) throws Exception;

    /**
     * 群成员加入三方IM状态
     * @param groupMemberIds 群成员ID
     * @param groupJoin 加群状态
     */
    void updateGroupMemberJoinStatus(List<Long> groupMemberIds, Boolean groupJoin);

    /**
     * 检查IM用户ID是否符合规则
     *
     * @param imUserId IM用户ID
     * @return
     */
    boolean checkImUserId(String imUserId);
}
