package com.ym.mec.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ym.mec.biz.dal.wrapper.AppVersionInfoWrapper;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ym.mec.biz.dal.dao.AppVersionInfoDao;
import com.ym.mec.biz.dal.entity.AppVersionInfo;
import com.ym.mec.biz.service.AppVersionInfoService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import org.springframework.transaction.annotation.Transactional;

@Service
public class AppVersionInfoServiceImpl extends BaseServiceImpl<Integer, AppVersionInfo>  implements AppVersionInfoService {
	
	@Autowired
	private AppVersionInfoDao appVersionInfoDao;

	@Override
	public BaseDAO<Integer, AppVersionInfo> getDAO() {
		return appVersionInfoDao;
	}

	@Override
	public List<AppVersionInfo> queryNewestByPlatform(String platform) {
		return appVersionInfoDao.queryNewestByPlatform(platform);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void add(AppVersionInfo appVersionInfo) {
		if(StringUtils.isEmpty(appVersionInfo.getPlatform())){
			throw new BizException("参数校验异常");
		}
		if (StringUtils.equals(appVersionInfo.getStatus(), "newest")) {
			//修改其他的为历史版本
			appVersionInfoDao.batchUpdateStatus(appVersionInfo.getPlatform());
		}
		appVersionInfoDao.insert(appVersionInfo);
	}

	@Override
	public void updateVersion(AppVersionInfo appVersionInfo) {
		if(StringUtils.isEmpty(appVersionInfo.getPlatform())){
			throw new BizException("参数校验异常");
		}
		if (StringUtils.equals(appVersionInfo.getStatus(), "newest")) {
			appVersionInfoDao.batchUpdateStatus(appVersionInfo.getPlatform());
		}
		appVersionInfo.setUpdateTime(new Date());
		appVersionInfoDao.update(appVersionInfo);
	}

    @Override
    public List<AppVersionInfo> queryLatestByPlatform(String platform, String appStatus) {
		return appVersionInfoDao.queryLatestByPlatform(platform, appStatus);

	}

    @Override
	public PageInfo<AppVersionInfo> queryPage(AppVersionInfoWrapper.AppVersionInfoQuery queryInfo) {
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		PageInfo<AppVersionInfo> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		List<AppVersionInfo> dataList = new ArrayList<>();
		int count = this.findCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = appVersionInfoDao.queryPage(params);
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public AppVersionInfo checkForceUpdate(String platform, String localVersion) {
		return appVersionInfoDao.checkForceUpdate(platform, localVersion);
	}

}