package com.ym.mec.biz.service.impl;


import com.dayaedu.cbs.openfeign.client.MusicFeignClientService;
import com.dayaedu.cbs.openfeign.wrapper.music.MusicSheetCategoriesWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.auth.api.dto.MusicScoreQueryInfo;
import com.ym.mec.biz.dal.entity.SysMusicScoreCategories;
import com.ym.mec.biz.service.CbsMusicScoreCategoriesService;
import com.ym.mec.common.exception.BizException;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class CbsMusicScoreCategoriesServiceImpl implements CbsMusicScoreCategoriesService {

	@Resource
	private MusicFeignClientService musicFeignClientService;
	@Value("${openfeign-client.app-id:1745637981387108354}")
	public Long applicationId;

	@Override
	public List<SysMusicScoreCategories> queryTree(MusicScoreQueryInfo menuQueryInfo) {
		MusicSheetCategoriesWrapper.MusicSheetCategoriesQuery query = new MusicSheetCategoriesWrapper.MusicSheetCategoriesQuery();
		query.setPage(menuQueryInfo.getPage());
		query.setRows(menuQueryInfo.getRows());
		query.setKeyword(menuQueryInfo.getSearch());
		query.setParentId(menuQueryInfo.getParentId().longValue());
		query.setEnable(menuQueryInfo.getEnable());
		R<List<MusicSheetCategoriesWrapper.MusicSheetCategories>> listR = musicFeignClientService.queryTree(query);
		if (listR.getCode() != 200) {
			throw new BizException("查询曲目分类失败,{}", listR.getMessage());
		}
		List<MusicSheetCategoriesWrapper.MusicSheetCategories> musicSheetCategories = listR.feignData();
		return musicSheetCategories.stream().map(this::convertMusicSheetCategoryToSysMusicScoreCategory).collect(Collectors.toList());
	}

	@Override
	public SysMusicScoreCategories getChildTree(SysMusicScoreCategories sysMusicScoreCategories) {
		R<MusicSheetCategoriesWrapper.MusicSheetCategories> childTree = musicFeignClientService.getChildTree(convertSysMusicScoreCategoryToMusicSheetCategory(sysMusicScoreCategories));
		if (childTree.getCode() != 200) {
			throw new BizException("查询曲目分类失败,{}", childTree.getMessage());
		}
		MusicSheetCategoriesWrapper.MusicSheetCategories musicSheetCategories = childTree.feignData();
		if (musicSheetCategories != null) {
			return convertMusicSheetCategoryToSysMusicScoreCategory(musicSheetCategories);
		}
		return sysMusicScoreCategories;
	}

	@Override
	public List<Integer> getAllCategoryIdList(String categoriesIds) {
		R<List<Integer>> allCategoryIdList = musicFeignClientService.getAllCategoryIdList(categoriesIds);
		if (allCategoryIdList.getCode() != 200) {
			throw new BizException("查询曲目分类失败,{}", allCategoryIdList.getMessage());
		}
		return allCategoryIdList.feignData();
	}

	@Override
	public List<SysMusicScoreCategories> queryByIds(String ids) {
		if (StringUtils.isEmpty(ids)) {
			return new ArrayList<>();
		}
		R<List<MusicSheetCategoriesWrapper.MusicSheetCategories>> listR = musicFeignClientService.findByIds(ids);
		if (listR.getCode() != 200) {
			throw new BizException("查询曲目分类失败,{}", listR.getMessage());
		}
		List<MusicSheetCategoriesWrapper.MusicSheetCategories> musicSheetCategories = listR.feignData();
		return musicSheetCategories.stream().map(this::convertMusicSheetCategoryToSysMusicScoreCategory).collect(Collectors.toList());
	}

	@Override
	public List<SysMusicScoreCategories> queryCategoriesTreeList(String categoryIds, Boolean isQueryEnable) {
		categoryIds = categoryIds.endsWith(",") ? categoryIds.substring(0, categoryIds.length() - 1) : categoryIds;
		List<SysMusicScoreCategories> categoryList = this.queryByIds(categoryIds);
		if(CollectionUtils.isNotEmpty(categoryList)){
			String collect = categoryList.stream().map(SysMusicScoreCategories::getParentId).distinct().map(Object::toString).collect(Collectors.joining(","));
			List<Integer> categoryListIds = categoryList.stream().map(SysMusicScoreCategories::getId).collect(Collectors.toList());
			List<SysMusicScoreCategories> sysMusicScoreCategories = this.queryByIds(collect);
			List<SysMusicScoreCategories> rows = new ArrayList<SysMusicScoreCategories>();
			for (SysMusicScoreCategories musicScoreCategories : sysMusicScoreCategories) {
				rows.add(this.getChildTree(musicScoreCategories));
			}
			for (SysMusicScoreCategories row : rows) {
				//剔除子集不包含在categoryListIds中的数据
				List<SysMusicScoreCategories> sysMusicScoreCategoriesList = row.getSysMusicScoreCategoriesList();
				if(CollectionUtils.isNotEmpty(sysMusicScoreCategoriesList)){
					sysMusicScoreCategoriesList.removeIf(e -> !categoryListIds.contains(e.getId()));
				}
			}
			return rows;
		}
		return new ArrayList<>();
	}

	@Override
	public Map<Integer, String> getCategoriesNameMap(List<Integer> categoriesIds) {
		R<Map<Integer, String>> mapR = musicFeignClientService.getCategoryNames(categoriesIds);
		if (mapR.getCode() != 200) {
			throw new BizException("查询曲目分类失败,{}", mapR.getMessage());
		}
		return mapR.feignData();
	}

	@Override
	public List<SysMusicScoreCategories> queryTreeByAppId() {
		R<List<MusicSheetCategoriesWrapper.MusicSheetCategories>> listR = musicFeignClientService.queryTreeByAppId(applicationId);
		if (listR.getCode() != 200) {
			throw new BizException("查询曲目分类失败,{}", listR.getMessage());
		}
		List<MusicSheetCategoriesWrapper.MusicSheetCategories> musicSheetCategories = listR.feignData();
		return musicSheetCategories.stream().map(this::convertMusicSheetCategoryToSysMusicScoreCategory).collect(Collectors.toList());
	}

	@Override
	public SysMusicScoreCategories getChildTreeById(Long var1) {
		R<MusicSheetCategoriesWrapper.MusicSheetCategories> childTreeById = musicFeignClientService.getChildTreeById(var1);
		if (childTreeById.getCode() != 200) {
			throw new BizException("查询曲目分类失败,{}", childTreeById.getMessage());
		}
		MusicSheetCategoriesWrapper.MusicSheetCategories musicSheetCategories = childTreeById.feignData();
		if (musicSheetCategories != null) {
			return convertMusicSheetCategoryToSysMusicScoreCategory(musicSheetCategories);
		}
		return null;
	}

	public SysMusicScoreCategories getParentTree(SysMusicScoreCategories sysMusicScoreCategories) {
		if (sysMusicScoreCategories != null && sysMusicScoreCategories.getParentId() > 0) {
			List<SysMusicScoreCategories> categories = this.queryByIds(sysMusicScoreCategories.getParentId().toString());
			if (CollectionUtils.isNotEmpty(categories)) {
				SysMusicScoreCategories parentCategories = categories.get(0);
				if (parentCategories != null) {
					if(parentCategories.getParentId() > 0){
						return getParentTree(parentCategories);
					}
					return parentCategories;
				}
			}
		}
		return null;
	}


	private void insertSubToParent(List<SysMusicScoreCategories> parentTree, SysMusicScoreCategories targetCategory) {
		for (SysMusicScoreCategories target : parentTree) {
			if (CollectionUtils.isEmpty(targetCategory.getSysMusicScoreCategoriesList())) {
				return;
			}
			if (targetCategory.getId().intValue() == target.getId()) {
				insertSubToParent(target.getSysMusicScoreCategoriesList(),targetCategory.getSysMusicScoreCategoriesList().get(0));
				return;
			}
		}
		parentTree.add(targetCategory);
	}

	private MusicSheetCategoriesWrapper.MusicSheetCategories convertSysMusicScoreCategoryToMusicSheetCategory(SysMusicScoreCategories sysMusicScoreCategory) {
		MusicSheetCategoriesWrapper.MusicSheetCategories musicSheetCategory = new MusicSheetCategoriesWrapper.MusicSheetCategories();
		musicSheetCategory.setId(Long.valueOf(sysMusicScoreCategory.getId()));
		musicSheetCategory.setName(sysMusicScoreCategory.getName());
		musicSheetCategory.setParentId(Long.valueOf(sysMusicScoreCategory.getParentId()));
		musicSheetCategory.setEnable(sysMusicScoreCategory.getEnable());
		musicSheetCategory.setCoverImg(sysMusicScoreCategory.getCoverImg());
		musicSheetCategory.setMusicSheetNum(sysMusicScoreCategory.getMusicScoreNum());
		List<SysMusicScoreCategories> sysMusicScoreCategoriesList = sysMusicScoreCategory.getSysMusicScoreCategoriesList();
		if(CollectionUtils.isNotEmpty(sysMusicScoreCategoriesList)){
			List<MusicSheetCategoriesWrapper.MusicSheetCategories> categoriesList = sysMusicScoreCategoriesList.stream()
					.map(this::convertSysMusicScoreCategoryToMusicSheetCategory).collect(Collectors.toList());
			musicSheetCategory.setMusicSheetCategoriesList(categoriesList);
		}
		return musicSheetCategory;
	}

	private SysMusicScoreCategories convertMusicSheetCategoryToSysMusicScoreCategory(MusicSheetCategoriesWrapper.MusicSheetCategories musicSheetCategory) {
		SysMusicScoreCategories sysMusicScoreCategory = new SysMusicScoreCategories();
		sysMusicScoreCategory.setId(musicSheetCategory.getId().intValue());
		sysMusicScoreCategory.setName(musicSheetCategory.getName());
		sysMusicScoreCategory.setParentId(musicSheetCategory.getParentId().intValue());
		sysMusicScoreCategory.setEnable(musicSheetCategory.getEnable());
		sysMusicScoreCategory.setCoverImg(musicSheetCategory.getCoverImg());
		sysMusicScoreCategory.setMusicScoreNum(musicSheetCategory.getMusicSheetNum());
		List<MusicSheetCategoriesWrapper.MusicSheetCategories> categoriesList = musicSheetCategory.getMusicSheetCategoriesList();
		if(CollectionUtils.isNotEmpty(categoriesList)){
			List<SysMusicScoreCategories> sysMusicScoreCategoriesList = categoriesList.stream()
					.map(this::convertMusicSheetCategoryToSysMusicScoreCategory).collect(Collectors.toList());
			sysMusicScoreCategory.setSysMusicScoreCategoriesList(sysMusicScoreCategoriesList);
		}
		return sysMusicScoreCategory;
	}
}