package com.ym.mec.biz.service.impl;

import com.dayaedu.cbs.openfeign.service.CbsSubjectService;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsSubjectApiWrapper;
import com.google.common.collect.Lists;
import com.ym.mec.biz.dal.entity.Instrument;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.wrapper.InstrumentWrapper;
import com.ym.mec.biz.dal.wrapper.SubjectWrapper;
import com.ym.mec.biz.service.InstrumentService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.common.page.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class CbsSubjectServiceImpl implements CbsSubjectService {

    private static final Logger log = LoggerFactory.getLogger(CbsSubjectServiceImpl.class);

    @Resource
    private SubjectService subjectService;
    @Resource
    private InstrumentService instrumentService;

    public Boolean checkSubjectEnable(CbsSubjectApiWrapper.CheckSubjectDto checkSubjectDto) {
        List<Subject> subjects = subjectService.getDao().queryCbsList(Lists.newArrayList(checkSubjectDto.getSubjectId().longValue()));
        if (CollectionUtils.isNotEmpty(subjects)){
            return true;
        }
        return false;
    }

    public List<CbsSubjectApiWrapper.Subject> list(CbsSubjectApiWrapper.SubjectQuery query) {
        SubjectWrapper.SubjectQuery subjectQuery = new SubjectWrapper.SubjectQuery();
        subjectQuery.setPage(1);
        subjectQuery.setRows(999);
        subjectQuery.setEnableFlag(query.getEnable());
        List<Long> subjectIds = query.getSubjectIds();
        if (CollectionUtils.isNotEmpty(subjectIds)) {
            subjectQuery.setSubjectIds(subjectIds.stream().map(String::valueOf).collect(Collectors.joining(",")));
        }
        PageInfo<SubjectWrapper.Subject> subjectPageInfo = subjectService.selectPage(subjectQuery);
        Map<Integer, SubjectWrapper.Subject> subjectIdNameMap = subjectPageInfo.getRows().stream()
                .collect(Collectors.toMap(SubjectWrapper.Subject::getId,o->o,(v1,v2)->v2));

        List<CbsSubjectApiWrapper.Subject> subjects = subjectService.getDao().queryCbsSubjects(query);
        for (CbsSubjectApiWrapper.Subject subject : subjects) {
            if(subject.getCbsSubjectId() != null){
                SubjectWrapper.Subject cbsSubject = subjectIdNameMap.get(subject.getSubjectId().intValue());
                if (cbsSubject != null) {
                    List<InstrumentWrapper.Instrument> instruments = cbsSubject.getInstruments();
                    if (CollectionUtils.isNotEmpty(instruments)) {
                        subject.setSubjectName(instruments.stream().map(InstrumentWrapper.Instrument::getName).collect(Collectors.joining(",")));
                        subject.setInstrumentId(instruments.get(0).getId().longValue());
                    }
                }
            }
        }
        return subjects;
    }

    public List<Long> instrumentList(CbsSubjectApiWrapper.InstrumentQuery query){
        List<Instrument> list = instrumentService.lambdaQuery().in(Instrument::getId, query.getMusicalInstrumentIds()).list();
        return list.stream().map(e->e.getId().longValue()).collect(Collectors.toList());
    }
}