package com.ym.mec.biz.service.impl;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import com.microsvc.toolkit.middleware.oss.OssPluginContext;
import com.microsvc.toolkit.middleware.oss.impl.TencentOssPlugin;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.enums.CertificateTypeEnum;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderDao;
import com.ym.mec.biz.dal.dao.MusicGroupStudentFeeDao;
import com.ym.mec.biz.dal.dao.StudentDao;
import com.ym.mec.biz.dal.dao.StudentRegistrationDao;
import com.ym.mec.biz.dal.dao.SysUserContractsDao;
import com.ym.mec.biz.dal.dto.CirculationUser;
import com.ym.mec.biz.dal.dto.OAFinancialDto;
import com.ym.mec.biz.dal.dto.StudentInfo;
import com.ym.mec.biz.dal.entity.CooperationOrgan.OwnershipType;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee;
import com.ym.mec.biz.dal.entity.MusicGroupSubjectGoodsGroup;
import com.ym.mec.biz.dal.entity.MusicGroupSubjectPlan;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.entity.SysUserContracts;
import com.ym.mec.biz.dal.entity.SysUserContracts.ContractType;
import com.ym.mec.biz.dal.entity.SysUserTsign;
import com.ym.mec.biz.dal.entity.TenantConfig;
import com.ym.mec.biz.dal.entity.TenantContractTemplate;
import com.ym.mec.biz.dal.entity.TenantInfo;
import com.ym.mec.biz.dal.entity.VipGroup;
import com.ym.mec.biz.dal.enums.CourseViewTypeEnum;
import com.ym.mec.biz.dal.enums.KitGroupPurchaseTypeEnum;
import com.ym.mec.biz.dal.enums.OrderTypeEnum;
import com.ym.mec.biz.service.ContractService;
import com.ym.mec.biz.service.GoodsService;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.biz.service.MusicGroupSubjectGoodsGroupService;
import com.ym.mec.biz.service.MusicGroupSubjectPlanService;
import com.ym.mec.biz.service.OrganizationService;
import com.ym.mec.biz.service.StudentRegistrationService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.biz.service.SysUserContractsService;
import com.ym.mec.biz.service.SysUserTsignService;
import com.ym.mec.biz.service.TenantConfigService;
import com.ym.mec.biz.service.TenantContractTemplateService;
import com.ym.mec.biz.service.TenantInfoService;
import com.ym.mec.biz.service.VipGroupService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.thirdparty.eseal.ESealPlugin;
import com.ym.mec.thirdparty.user.realname.RealnameAuthenticationPluginContext;
import com.ym.mec.thirdparty.user.realname.provider.LinkfaceRealnameAuthenticationPlugin;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.freemarker.FreemarkerTemplateEngine;
import com.ym.mec.util.money.MoneyUtil;
import com.ym.mec.util.pdf.PDFUtil;

import javax.annotation.PostConstruct;

@Service
public class ContractServiceImpl implements ContractService {

    @Autowired
    private SysUserTsignService sysUserTsignService;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private SysUserContractsService sysUserContractsService;

    @Autowired
    private SysUserContractsDao sysUserContractsDao;

    @Autowired
    private StudentRegistrationService studentRegistrationService;

    @Autowired
    private StudentRegistrationDao studentRegistrationDao;

    @Autowired
    private MusicGroupSubjectPlanService musicGroupSubjectPlanService;

    @Autowired
    private MusicGroupSubjectGoodsGroupService musicGroupSubjectGoodsGroupService;

    @Autowired
    private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;

    @Autowired
    private MusicGroupStudentFeeDao musicGroupStudentFeeDao;

    @Autowired
    @Lazy
    private VipGroupService vipGroupService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private ESealPlugin eSealPlugin;

    @Autowired
    private GoodsService goodsService;

    @Autowired
    @Lazy
    private MusicGroupService musicGroupService;

    @Autowired
    private StudentDao studentDao;

    @Autowired
    private EmployeeDao employeeDao;
    
    @Autowired
    private TenantInfoService tenantInfoService;
    
    @Autowired
    private TenantConfigService tenantConfigService;
    
    @Autowired
    @Lazy
    private OrganizationService organizationService;
    
    @Autowired
    private TenantContractTemplateService tenantContractTemplateService;

    @Value("${contract.baseDir:/var/pdf}")
    private String contractBaseDir;

    @Value("${message.debugMode}")
    private boolean debugMode;

    @Autowired
    private RealnameAuthenticationPluginContext realnameAuthenticationPluginContext;
    @Autowired
    private OssPluginContext ossPluginContext;

    private DateFormat dateFormatOss = new SimpleDateFormat("yyyy/MM/dd");

    private DateFormat dateFormat1 = new SimpleDateFormat("yyMMddHHmmSS");

    private final Logger logger = LoggerFactory.getLogger(ContractService.class);

    @PostConstruct
    public void afterPropertiesSet() {
        // 注册企业账户
        Map<String, String> organList = new HashMap<String, String>();
        organList.put("91420106333619290A", "2,武汉长乐长风乐器销售有限公司");
        organList.put("91440300326364429H", "1,深圳大雅乐盟网络教育股份有限公司");

//        organList.forEach((code, name) -> {
//            SysUserTsign sysUserTsign = sysUserTsignService.queryByCardNo(code);
//            if (Objects.isNull(sysUserTsign)) {
//                String[] nameSplit = name.split(",");
//                addTsign(Integer.parseInt(nameSplit[0]), code, nameSplit[1], 1);
//            }
//        });
    }

    /**
     * 校验企业营业信息，不存在则添加一个
     */
    @Override
	public void addTsign(Integer userId, String organCode, String orgName, Integer tenantId) {
		Optional.ofNullable(organCode).orElseThrow(() -> new BizException("营业执照编号不能为空!"));
		Optional.ofNullable(orgName).orElseThrow(() -> new BizException("营业名称不能为空"));

		SysUserTsign sysUserTsign = null;

		SysUserTsign orignSysUserTsign = sysUserTsignService.queryByCardNo(organCode);

		if (orignSysUserTsign == null) {

			String accountId = eSealPlugin.createOrganAccount(orgName, organCode);
			if (StringUtils.isBlank(accountId)) {
				throw new BizException("创建企业电子存证账户失败");
			}

			String sealData = eSealPlugin.createOrganSeal(accountId, "", "");
			if (StringUtils.isBlank(sealData)) {
				throw new BizException("创建电子存证印章失败");
			}

			sysUserTsign = new SysUserTsign(-userId, accountId, sealData, orgName, organCode, tenantId);
			sysUserTsignService.insert(sysUserTsign);

		} else if (!StringUtils.equals(orignSysUserTsign.getName(), orgName)) {
			if (eSealPlugin.updateOrganAccount(orignSysUserTsign.getAccountId(), orgName, organCode)) {
				orignSysUserTsign.setName(orgName);
				sysUserTsignService.update(orignSysUserTsign);
			}
		}

	}

	@Override
    public SysUserTsign register(Integer userId, String realName, String idCardNo, String mobileNo) {
        if (StringUtils.isBlank(realName) || StringUtils.isBlank(idCardNo) || StringUtils.isBlank(mobileNo)) {
            logger.error("用户电子签章参数校验失败 userId:{}", userId);
            return null;
        }

        SysUserTsign sysUserTsign = sysUserTsignService.get(userId);
        if (sysUserTsign == null) {

            String accountId = "";
            String sealData = "";

            SysUser user = studentDao.getUser(userId);
            if (user == null) {
                throw new BizException("用户信息查询失败");
            }

            if (CertificateTypeEnum.IDENTITY.getCode().equals(user.getCertificateType())) {
                if (!debugMode) {
                    realnameAuthenticationPluginContext.getRealnameAuthenticationPlugin(LinkfaceRealnameAuthenticationPlugin.getName()).verify(realName, idCardNo);
                }

                accountId = eSealPlugin.createUserAccount(realName, idCardNo, mobileNo);

                if (StringUtils.isBlank(accountId)) {
                    throw new BizException("创建电子存证账户失败");
                }

                sealData = eSealPlugin.createUserSeal(accountId);

                if (StringUtils.isBlank(sealData)) {
                    throw new BizException("创建电子存证印章失败");
                }
            }

            sysUserTsign = new SysUserTsign(userId, accountId, sealData, realName, idCardNo, user.getTenantId());

            sysUserTsignService.insert(sysUserTsign);

            return sysUserTsign;
        }

        return null;
    }

    @Override
    public boolean transferRegisterContract(Integer userId) {

        SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

        if (sysUserTsign == null) {
            SysUser user = sysUserFeignService.queryUserById(userId);
            if (user == null) {
                throw new BizException("用户信息查询失败");
            }
            sysUserTsign = register(userId, user.getRealName(), user.getIdCardNo(), user.getPhone());
        }

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/register/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        templateEngine.render(params, "register.ftl", srcFile);

        // 生成借款协议PDF
        try {
            PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(srcFile), srcPdfPath, "simsun.ttc");
        } catch (IOException e) {
            throw new BizException("生成pdf协议失败", e);
        }

        if (sysUserTsign != null) {
            eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);
        }

        Date date = new Date();
        // 上传到oss
        String dateStrOss = dateFormatOss.format(date);
        dateStrOss = "daya/users/" + dateStrOss + "/" + DateUtil.getHour(date);
        //String pdfFilePath = storagePluginContext.uploadFile(KS3StoragePlugin.PLUGIN_NAME, dateStrOss, srcFile);
        String pdfFilePath = ossPluginContext.getPluginService(TencentOssPlugin.PLUGIN_NAME).uploadFile(dateStrOss, srcFile);

        SysUserContracts sysUserContracts = new SysUserContracts();
        sysUserContracts.setCreateTime(date);
        sysUserContracts.setType(ContractType.REGISTER);
        sysUserContracts.setUrl(pdfFilePath);
        sysUserContracts.setUserId(userId);

        sysUserContractsService.insert(sysUserContracts);

        FileUtils.deleteQuietly(srcFile);

        return true;
    }

    @Override
    public String queryRegisterContract(Integer userId) {

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/register/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        templateEngine.render(params, "register.ftl", srcFile);

        String result = "";
        try {
            result = FileUtils.readFileToString(srcFile);
        } catch (IOException e) {
            throw new BizException("读取注册协议出错", e);
        } finally {
            FileUtils.deleteQuietly(srcFile);
        }

        return result;
    }

    @Override
    public boolean transferMusicGroupCoursesContract(Integer userId, String musicGroupId, OrderTypeEnum orderType) {
        if (true) {
            transferProduceContract(userId, musicGroupId, orderType);
            return true;
        }
        SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

        if (sysUserTsign == null) {
            SysUser user = sysUserFeignService.queryUserById(userId);
            if (user == null) {
                throw new BizException("用户信息查询失败");
            }
            sysUserTsign = register(userId, user.getRealName(), user.getIdCardNo(), user.getPhone());
        }
        Date date = new Date();

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/courses/" + dateFormat1.format(date) + "/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        // 查询参数信息
        StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
        params.put("studentInfo", studentInfo);

        // 课程类型
        params.put("classesType", "乐团团课");

        // 课程费用
        StudentRegistration studentRegistration = studentRegistrationService.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if (studentRegistration == null) {
            throw new BizException("用户 报名信息不存在");
        }
        MusicGroupStudentFee musicGroupStudentFee = musicGroupStudentFeeDao.findByUser(userId, musicGroupId);
        if (musicGroupStudentFee == null) {
            MusicGroupSubjectPlan musicGroupSubjectPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(musicGroupId,
                    studentRegistration.getActualSubjectId());
            if (musicGroupSubjectPlan == null) {
                throw new BizException("声部课程费用设置找不到");
            }
            params.put("courseFee", musicGroupSubjectPlan.getFee().doubleValue());
        } else {
            params.put("courseFee", musicGroupStudentFee.getTemporaryCourseFee().doubleValue() > 0 ? musicGroupStudentFee.getTemporaryCourseFee().doubleValue()
                    : musicGroupStudentFee.getCourseFee().doubleValue());
        }

        // 收费形式
        List<MusicGroupPaymentCalender> calenders = musicGroupPaymentCalenderDao.findByMusicGroupId(musicGroupId);

		/*if (calenders == null || calenders.size() == 0) {
			
		} else {
			params.put("paymentcalender", calenders.stream().map(cal -> cal.getPaymentMonth().toString()).collect(Collectors.joining("月,")) + "月");
		}*/
        params.put("paymentcalender", "一次性");

        params.put("isShowVisualSeal", false);

        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        if (musicGroup == null) {
            throw new BizException("乐团信息没找到");
        }

        params.put("ownershipType", musicGroup.getOwnershipType().name());

        templateEngine.render(params, "courses.ftl", srcFile);

        // 生成借款协议PDF
        try {
            PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(srcFile), srcPdfPath, "simsun.ttc");
        } catch (IOException e) {
            throw new BizException("生成pdf协议失败", e);
        }
        String organCode = "";
        if (musicGroup.getOwnershipType() == OwnershipType.OWN) {
            organCode = "91440300326364429H";
        } else {
            organCode = "91420106333619290A";
        }
        SysUserTsign organTsign = sysUserTsignService.queryByCardNo(organCode);
        if (organTsign == null) {
            throw new BizException("甲方未创建签章");
        }

        eSealPlugin.organSign(organTsign.getSealData(), srcPdfPath, srcPdfPath);
        if (sysUserTsign != null) {
            eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);
        }

        // 上传到oss
        String dateStrOss = dateFormatOss.format(date);
        dateStrOss = "daya/musicCourses/" + dateStrOss + "/" + DateUtil.getHour(date);
        //String pdfFilePath = storagePluginContext.uploadFile(KS3StoragePlugin.PLUGIN_NAME, dateStrOss, srcFile);
        String pdfFilePath = ossPluginContext.getPluginService(TencentOssPlugin.PLUGIN_NAME).uploadFile(dateStrOss, srcFile);

        SysUserContracts sysUserContracts = new SysUserContracts();
        sysUserContracts.setCreateTime(date);
        sysUserContracts.setType(ContractType.COURSES);
        sysUserContracts.setUrl(pdfFilePath);
        sysUserContracts.setUserId(userId);

        sysUserContractsService.insert(sysUserContracts);

        FileUtils.deleteQuietly(srcFile);

        return true;
    }

    @Override
    public String queryMusicGroupCoursesContract(Integer userId, String musicGroupId) {

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/courses/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
        params.put("studentInfo", studentInfo);

        // 课程类型
        params.put("classesType", "乐团团课");

        // 课程费用
        StudentRegistration studentRegistration = studentRegistrationService.queryByUserIdAndMusicGroupId(userId, musicGroupId);
        if (studentRegistration == null) {
            throw new BizException("用户报名信息不存在");
        }
        MusicGroupStudentFee musicGroupStudentFee = musicGroupStudentFeeDao.findByUser(userId, musicGroupId);
        if (musicGroupStudentFee == null) {
            MusicGroupSubjectPlan musicGroupSubjectPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(musicGroupId,
                    studentRegistration.getActualSubjectId());
            if (musicGroupSubjectPlan == null) {
                throw new BizException("声部课程费用设置找不到");
            }
            params.put("courseFee", musicGroupSubjectPlan.getFee().doubleValue());
        } else {
            params.put(
                    "courseFee",
                    (musicGroupStudentFee.getTemporaryCourseFee() != null && musicGroupStudentFee.getTemporaryCourseFee().doubleValue() > 0) ? musicGroupStudentFee
                            .getTemporaryCourseFee().doubleValue() : musicGroupStudentFee.getCourseFee().doubleValue());
        }

        // 收费形式
        List<MusicGroupPaymentCalender> calenders = musicGroupPaymentCalenderDao.findByMusicGroupId(musicGroupId);

		/*if (calenders == null || calenders.size() == 0) {
			params.put("paymentcalender", "一次性");
		} else {
			params.put("paymentcalender", calenders.stream().map(cal -> cal.getPaymentMonth().toString()).collect(Collectors.joining("月,")) + "月");
		}*/

        params.put("paymentcalender", "一次性");

        params.put("isShowVisualSeal", true);

        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        if (musicGroup == null) {
            throw new BizException("乐团信息没找到");
        }

        params.put("ownershipType", musicGroup.getOwnershipType().name());

        templateEngine.render(params, "courses.ftl", srcFile);

        String result = "";
        try {
            result = FileUtils.readFileToString(srcFile);
        } catch (IOException e) {
            throw new BizException("读取课程协议出错", e);
        } finally {
            FileUtils.deleteQuietly(srcFile);
        }

        return result;
    }

    @Override
    public boolean transferVipGroupCoursesContract(Integer userId, Long vipGroupId, OrderTypeEnum orderType) {
        if (true) {
            transferProduceContract(userId, null, orderType);
            return true;
        }

        SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

        if (sysUserTsign == null) {
            SysUser user = sysUserFeignService.queryUserById(userId);
            if (user == null) {
                throw new BizException("用户信息查询失败");
            }
            sysUserTsign = register(userId, user.getRealName(), user.getIdCardNo(), user.getPhone());
        }
        Date date = new Date();

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/vipCourses/" + dateFormat1.format(date) + "/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        // 查询参数信息
        StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);

        // 课程费用
        VipGroup vipGroup = vipGroupService.get(vipGroupId);
        if (vipGroup == null) {
            throw new BizException("VIP课找不到");
        }
        String classesType = "VIP课";
        String vipCourses = "vipCourses/";
        ContractType contractType = ContractType.VIP_COURSES;
        if("LIVE".equals(vipGroup.getGroupType())){
            classesType = "直播课";
            vipCourses = "liveCourses/";
            contractType = ContractType.LIVE_COURSES;
        }
        params.put("studentInfo", studentInfo);
        params.put("classesType", classesType);
        params.put("courseFee", vipGroup.getTotalPrice().doubleValue());

        // 收费形式
        params.put("paymentcalender", "一次性");

        params.put("isShowVisualSeal", false);

        params.put("ownershipType", "OWN");

        templateEngine.render(params, "courses.ftl", srcFile);

        // 生成借款协议PDF
        try {
            PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(srcFile), srcPdfPath, "simsun.ttc");
        } catch (IOException e) {
            throw new BizException("生成pdf协议失败", e);
        }
        String organCode = "91440300326364429H";
        SysUserTsign organTsign = sysUserTsignService.queryByCardNo(organCode);
        if (organTsign == null) {
            throw new BizException("甲方未创建签章");
        }

        eSealPlugin.organSign(organTsign.getSealData(), srcPdfPath, srcPdfPath);

        if (sysUserTsign != null) {
            eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);
        }

        // 上传到oss
        String dateStrOss = dateFormatOss.format(date);
        dateStrOss = "daya/" + vipCourses + dateStrOss + "/" + DateUtil.getHour(date);
        //String pdfFilePath = storagePluginContext.uploadFile(KS3StoragePlugin.PLUGIN_NAME, dateStrOss, srcFile);
        String pdfFilePath = ossPluginContext.getPluginService(TencentOssPlugin.PLUGIN_NAME).uploadFile(dateStrOss, srcFile);

        SysUserContracts sysUserContracts = new SysUserContracts();
        sysUserContracts.setCreateTime(date);
        sysUserContracts.setType(contractType);
        sysUserContracts.setUrl(pdfFilePath);
        sysUserContracts.setUserId(userId);

        sysUserContractsService.insert(sysUserContracts);

        FileUtils.deleteQuietly(srcFile);

        return true;
    }

    @Override
    public String queryVipGroupCoursesContract(Integer userId, Long vipGroupId) {

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/courses/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
        params.put("studentInfo", studentInfo);

        // 课程类型
        params.put("classesType", "VIP课");

        // 课程费用
        VipGroup vipGroup = vipGroupService.get(vipGroupId);
        if (vipGroup == null) {
            throw new BizException("VIP课找不到");
        }
        params.put("courseFee", vipGroup.getTotalPrice().doubleValue());

        // 收费形式
        params.put("paymentcalender", "一次性");

        params.put("isShowVisualSeal", true);

        params.put("ownershipType", "OWN");

        templateEngine.render(params, "courses.ftl", srcFile);

        String result = "";
        try {
            result = FileUtils.readFileToString(srcFile);
        } catch (IOException e) {
            throw new BizException("读取课程协议出错", e);
        } finally {
            FileUtils.deleteQuietly(srcFile);
        }

        return result;
    }

    @Override
    public boolean transferGoodsContract(Integer userId, String musicGroupId, String goodsIds, OrderTypeEnum orderType) {

        transferProduceContract(userId, musicGroupId, orderType);
        return true;
    }

    @Override
    public String queryGoodsContract(Integer userId, String musicGroupId, String goodsIds, KitGroupPurchaseTypeEnum kitGroupPurchaseTypeEnum) {

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/goods/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        // 查询参数信息
        StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);

        int subjectId = studentInfo.getSubject().getId();

        Subject subject = subjectService.get(subjectId);
        studentInfo.setSubject(subject);

        params.put("studentInfo", studentInfo);

        List<Goods> goodsList = goodsService.findGoodsByIds(goodsIds);
        params.put("goodsList", goodsList);

        double depositFee = 0d;
        if (kitGroupPurchaseTypeEnum == KitGroupPurchaseTypeEnum.LEASE) {
            MusicGroupSubjectGoodsGroup musicGroupSubjectPlan = musicGroupSubjectGoodsGroupService.query(musicGroupId, subjectId, goodsIds);
            if (musicGroupSubjectPlan != null) {
                depositFee = musicGroupSubjectPlan.getDepositFee().doubleValue();
            }
        }
        params.put("depositFee", depositFee);
        params.put("depositFeeFmt", MoneyUtil.toChinese(depositFee + ""));

        params.put("isShowVisualSeal", true);

        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
        if (musicGroup == null) {
            throw new BizException("乐团信息没找到");
        }

        params.put("ownershipType", musicGroup.getOwnershipType().name());

        templateEngine.render(params, "goods.ftl", srcFile);

        String result = "";
        try {
            result = FileUtils.readFileToString(srcFile);
        } catch (IOException e) {
            throw new BizException("读取商品协议出错", e);
        } finally {
            FileUtils.deleteQuietly(srcFile);
        }

        return result;
    }

    @Override
    public String queryPracticeCoursesContract(Integer userId, int courseSectionNum, Date startDate, Date endDate, BigDecimal fee) {
        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/practice/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("courseSectionNum", courseSectionNum);
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        params.put("fee", fee);

        params.put("isShowVisualSeal", false);

        params.put("ownershipType", "OWN");

        templateEngine.render(params, "practice.ftl", srcFile);

        String result = "";
        try {
            result = FileUtils.readFileToString(srcFile);
        } catch (IOException e) {
            throw new BizException("读取网管课购买协议出错", e);
        } finally {
            FileUtils.deleteQuietly(srcFile);
        }

        return result;
    }

    @Override
    @Async
    public boolean transferPracticeCoursesContract(Integer userId, int courseSectionNum, Date startDate, Date endDate, BigDecimal fee, OrderTypeEnum orderType) {
        if (true) {
            transferProduceContract(userId, null, orderType);
            return true;
        }

        SysUserTsign sysUserTsign = sysUserTsignService.get(userId);

        if (sysUserTsign == null) {
            SysUser user = sysUserFeignService.queryUserById(userId);
            if (user == null) {
                throw new BizException("用户信息查询失败");
            }
            sysUserTsign = register(userId, user.getRealName(), user.getIdCardNo(), user.getPhone());
        }

        Date date = new Date();

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/practice/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("courseSectionNum", courseSectionNum);
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        params.put("fee", fee);

        params.put("isShowVisualSeal", false);

        params.put("ownershipType", "OWN");

        templateEngine.render(params, "practice.ftl", srcFile);

        // 生成借款协议PDF
        try {
            PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(srcFile), srcPdfPath, "simsun.ttc");
        } catch (IOException e) {
            throw new BizException("生成pdf协议失败", e);
        }
        String organCode = "91440300326364429H";
        SysUserTsign organTsign = sysUserTsignService.queryByCardNo(organCode);
        if (organTsign == null) {
            throw new BizException("甲方未创建签章");
        }

        eSealPlugin.organSign(organTsign.getSealData(), srcPdfPath, srcPdfPath);

        if (sysUserTsign != null) {
            eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);
        }

        // 上传到oss
        String dateStrOss = dateFormatOss.format(date);
        dateStrOss = "daya/practice/" + dateStrOss + "/" + DateUtil.getHour(date);
        //String pdfFilePath = storagePluginContext.uploadFile(KS3StoragePlugin.PLUGIN_NAME, dateStrOss, srcFile);
        String pdfFilePath = ossPluginContext.getPluginService(TencentOssPlugin.PLUGIN_NAME).uploadFile(dateStrOss, srcFile);

        SysUserContracts sysUserContracts = new SysUserContracts();
        sysUserContracts.setCreateTime(date);
        sysUserContracts.setType(ContractType.PRACTICE);
        sysUserContracts.setUrl(pdfFilePath);
        sysUserContracts.setUserId(userId);

        sysUserContractsService.insert(sysUserContracts);

        FileUtils.deleteQuietly(srcFile);

        return true;
    }

    @Override
    public void transferProduceContract(Integer userId, String musicGroupId, OrderTypeEnum orderType) {
    	transferProduceContract(userId, musicGroupId, orderType, new Date());
    }

    @Override
    @Async
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED, propagation = Propagation.REQUIRES_NEW)
	public void transferProduceContract(Integer userId, String musicGroupId, OrderTypeEnum orderType, Date currentDate) {
        SysUser user = studentDao.lockUserReturnInfo(userId);

        if (user == null) {
            logger.error("用户不存在({})", userId);
            return;
        }

        if (StringUtils.isBlank(user.getRealName()) || StringUtils.isBlank(user.getIdCardNo())) {
            logger.error("身份信息缺失({})", userId);
            return;
        }

        SysUserTsign sysUserTsign = sysUserTsignService.get(userId);
        if (sysUserTsign == null) {
            try {
                sysUserTsign = register(userId, user.getRealName(), user.getIdCardNo(), user.getPhone());
            } catch (Exception e) {
                logger.error("用户电子签章注册失败", e);
                return;
            }
        }
        Date date = new Date();

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/product/" + dateFormat1.format(date) + "/" + userId + "_" + LocalDateTime.now().getSecond() + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        // 查询参数信息
        StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
        params.put("studentInfo", studentInfo);

        params.put("isShowVisualSeal", false);
        
        if(currentDate != null){
        	params.put("currentDate", currentDate);
        }else{
        	params.put("currentDate", date);
        }

        if (StringUtils.isBlank(musicGroupId)) {
            params.put("ownershipType", "OWN");
            //MusicGroup userLastNormalMusicGroup = studentRegistrationDao.getUserLastNormalMusicGroup(userId);
        } else {
            MusicGroup musicGroup = musicGroupService.get(musicGroupId);
            if (musicGroup == null) {
                logger.error("乐团信息没找到({})", musicGroupId);
                return;
            }
            params.put("ownershipType", musicGroup.getOwnershipType().name());
        }

        // 查询最新协议模板
		TenantContractTemplate tenantContractTemplate = tenantContractTemplateService.queryLatestContractTemplate(user.getTenantId(), null, null);
		if(tenantContractTemplate == null){
			logger.error("最新协议模板查询失败");
			return;
		}
		String mainContent = tenantContractTemplate.getContractTemplateContent();
		params.put("contractMainContent", mainContent);

        // 查询是否已生成协议
        List<SysUserContracts> userContracts = sysUserContractsService.getUserContractWithType(userId, ContractType.PRODUCT, tenantContractTemplate.getVersion());
        if (!CollectionUtils.isEmpty(userContracts)) {
            return;
        }
        
        //如果分部有企业信息，就用分部上的企业信息，否则用机构上的企业信息
        String companyName = null;
        TenantInfo tenantInfo = tenantInfoService.get(studentInfo.getTenantId());;
        Organization organizationDto = null;
        if(orderType != OrderTypeEnum.PRACTICE_GROUP_BUY && orderType != OrderTypeEnum.PRACTICE_GROUP_RENEW && orderType != OrderTypeEnum.SMALL_CLASS_TO_BUY){
        	organizationDto = organizationService.get(studentInfo.getOrganId());
        }
		if (organizationDto == null || StringUtils.isBlank(organizationDto.getCorporateName()) || StringUtils.isBlank(organizationDto.getCorporateCode())) {
			companyName = tenantInfo.getTsignName();
		} else {
			companyName = organizationDto.getCorporateName();
		}
		params.put("companyName", companyName);
        
		if (studentInfo.getTenantId() == 1) {
			if (OwnershipType.OWN.name().equals(params.get("ownershipType"))) {
				//params.put("companyName", "深圳大雅乐盟网络教育股份有限公司");
				//params.put("sealPicture", "https://oss.dayaedu.com/website/cachet.png");
			} else {
				params.put("companyName", "武汉长乐长风乐器销售有限公司");
				params.put("sealPicture", "https://oss.dayaedu.com/website/clcf.png");
			}
		}

        templateEngine.render(params, "latest_contract_template.ftl", srcFile);

        // 生成借款协议PDF
        try {
            PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(srcFile), srcPdfPath, "simsun.ttc");
        } catch (IOException e) {
            logger.error("生成产品协议失败", e);
            return;
        }

        String organCode = StringUtils.equals(companyName, tenantInfo.getTsignName()) ? tenantInfo.getTsignCode() : organizationDto.getCorporateCode();
        
		if (user.getTenantId() == 1) {
			if (!OwnershipType.OWN.name().equals(params.get("ownershipType"))) {
				organCode = "91420106333619290A";
			}
		}
        SysUserTsign organTsign = sysUserTsignService.queryByCardNo(organCode);
        if (organTsign == null) {
            //创建企业签章
            addTsign(tenantInfo.getUserId(), organCode, companyName, user.getTenantId());
        }

        eSealPlugin.organSign(organTsign.getSealData(), srcPdfPath, srcPdfPath);
        if (sysUserTsign != null && CertificateTypeEnum.IDENTITY.getCode().equals(user.getCertificateType())) {
            eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), srcPdfPath, srcPdfPath);
        }

        // 上传到oss
        String dateStrOss = dateFormatOss.format(date);
        dateStrOss = "daya/product/" + dateStrOss + "/" + DateUtil.getHour(date);
        //String pdfFilePath = storagePluginContext.uploadFile(KS3StoragePlugin.PLUGIN_NAME, dateStrOss, srcFile);
        String pdfFilePath = ossPluginContext.getPluginService(TencentOssPlugin.PLUGIN_NAME).uploadFile(dateStrOss, srcFile);

        SysUserContracts sysUserContracts = new SysUserContracts();
        sysUserContracts.setCreateTime(date);
        sysUserContracts.setType(ContractType.PRODUCT);
        sysUserContracts.setUrl(pdfFilePath);
        sysUserContracts.setUserId(userId);
        sysUserContracts.setVersion(tenantContractTemplate.getVersion());
        sysUserContracts.setTenantId(user.getTenantId());

        sysUserContractsService.insert(sysUserContracts);

        FileUtils.deleteQuietly(srcFile);
    }

	@Override
    @Transactional(isolation = Isolation.READ_COMMITTED)
    public Map<String, Object> queryProductContract(Integer userId, String musicGroupId, OrderTypeEnum orderType) {
        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/product/" + userId + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        StudentInfo studentInfo = studentRegistrationService.queryStudentInfo(userId);
        if (Objects.isNull(studentInfo)) {
            studentInfo = new StudentInfo();
        }
        if (StringUtils.isEmpty(studentInfo.getCertificateType())) {
            studentInfo.setCertificateType(CertificateTypeEnum.IDENTITY.getCode());
        }
        params.put("studentInfo", studentInfo);
        params.put("isShowVisualSeal", true);

        Map<String, Object> result = new HashMap<>();
        result.put("courseViewType", CourseViewTypeEnum.COURSE_FEE);
		
        if (StringUtils.isBlank(musicGroupId)) {
            params.put("ownershipType", "OWN");
            MusicGroup userLastNormalMusicGroup = studentRegistrationDao.getUserLastNormalMusicGroup(userId);
            if (Objects.nonNull(userLastNormalMusicGroup)) {
                result.put("courseViewType", userLastNormalMusicGroup.getCourseViewType());
            }
        } else {
            MusicGroup musicGroup = musicGroupService.get(musicGroupId);
            if (musicGroup == null) {
                throw new BizException("乐团信息没找到");
            }
            params.put("ownershipType", musicGroup.getOwnershipType().name());
            result.put("courseViewType", musicGroup.getCourseViewType());
        }

        //如果分部有企业信息，就用分部上的企业信息，否则用机构上的企业信息
        String companyName = null, sealPicture = null;
        TenantInfo tenantInfo = null;
        Organization organizationDto = null;
        if(orderType != OrderTypeEnum.PRACTICE_GROUP_BUY && orderType != OrderTypeEnum.PRACTICE_GROUP_RENEW && orderType != OrderTypeEnum.SMALL_CLASS_TO_BUY){
        	organizationDto = organizationService.get(studentInfo.getOrganId());
        }
		if (organizationDto == null || StringUtils.isBlank(organizationDto.getCorporateName()) || StringUtils.isBlank(organizationDto.getCorporateCode())) {

			tenantInfo = tenantInfoService.get(studentInfo.getTenantId());
			if (tenantInfo == null) {
				throw new BizException("机构信息不存在");
			}
	        
			TenantConfig tenantConfig = tenantConfigService.queryByTenantId(studentInfo.getTenantId());
			if(tenantConfig == null){
				throw new BizException("机构配置信息查询失败");
			}
			
			companyName = tenantInfo.getTsignName();
			sealPicture = tenantConfig.getCorporateChops();
		} else {
			companyName = organizationDto.getCorporateName();
			sealPicture = organizationDto.getCorporateSeal();
		}
		
		params.put("companyName", companyName);
		params.put("sealPicture", sealPicture);
        
		if (studentInfo.getTenantId() == 1) {
			if (OwnershipType.OWN.name().equals(params.get("ownershipType"))) {
				//params.put("companyName", "深圳大雅乐盟网络教育股份有限公司");
				//params.put("sealPicture", "https://oss.dayaedu.com/website/cachet.png");
			} else {
				params.put("companyName", "武汉长乐长风乐器销售有限公司");
				params.put("sealPicture", "https://oss.dayaedu.com/website/clcf.png");
			}
		}
		
		TenantContractTemplate tenantContractTemplate = tenantContractTemplateService.queryLatestContractTemplate(studentInfo.getTenantId(), null, null);
		if(tenantContractTemplate == null){
			throw new BizException("最新协议模板查询失败");
		}
		String mainContent = tenantContractTemplate.getContractTemplateContent();
		params.put("contractMainContent", mainContent);

        templateEngine.render(params, "latest_contract_template.ftl", srcFile);

        String html = "";
        try {
            html = FileUtils.readFileToString(srcFile);
        } catch (IOException e) {
            throw new BizException("读取产品协议出错", e);
        } finally {
            FileUtils.deleteQuietly(srcFile);
        }

        List<SysUserContracts> userContractWithType = sysUserContractsService.getUserContractWithType(userId, ContractType.PRODUCT,tenantContractTemplate.getVersion());

        result.put("exists", !CollectionUtils.isEmpty(userContractWithType));
        result.put("productContract", html);
        return result;
    }

    @Override
    public String transferOaFinancial(OAFinancialDto financialDto) {
        List<Integer> circulationUserIds = financialDto.getCirculationUsers().stream().map(CirculationUser::getUserId).collect(Collectors.toList());
        List<Integer> executorIds = financialDto.getExecutors().stream().map(CirculationUser::getUserId).collect(Collectors.toList());
        circulationUserIds.addAll(executorIds);
        List<SysUser> users = employeeDao.getUsers(circulationUserIds);

        Integer workOrderId = null;
        List<SysUserTsign> sysUserTsigns = new ArrayList<>();
        for (SysUser user : users) {
            for (CirculationUser circulationUser : financialDto.getCirculationUsers()) {
                if (workOrderId == null) {
                    workOrderId = circulationUser.getWorkOrder();
                }
                if (!circulationUser.getUserId().equals(user.getId())) continue;
                circulationUser.setRealName(user.getRealName());
            }
            for (CirculationUser executor : financialDto.getExecutors()) {
                if (!executor.getUserId().equals(user.getId())) continue;
                executor.setRealName(user.getRealName());
            }
            SysUserTsign sysUserTsign = sysUserTsignService.get(user.getId());
            if (sysUserTsign == null) {
                try {
                    sysUserTsign = register(user.getId(), user.getRealName(), user.getIdCardNo(), user.getPhone());
                } catch (Exception e) {
                    logger.error("用户电子签章注册失败", e);
                    continue;
                }
            }
            sysUserTsigns.add(sysUserTsign);
        }

        Date date = new Date();

        // 合成freemarker
        String srcPdfPath = contractBaseDir + "/oa/" + dateFormat1.format(date) + "/" + workOrderId + "_" + LocalDateTime.now().getSecond() + ".pdf";

        File srcFile = new File(srcPdfPath);

        File debtFile = new File(srcFile.getParent());
        if (!debtFile.exists()) {
            debtFile.mkdirs();
        }

        FreemarkerTemplateEngine templateEngine = FreemarkerTemplateEngine.getInstance();
        templateEngine.setClassForTemplateLoading(ContractServiceImpl.class, "/config/contracts/");

        Map<String, Object> params = new HashMap<String, Object>();

        // 查询参数信息
        params.put("organName", financialDto.getOrganName());
        params.put("realName", financialDto.getRealName());
        params.put("feeType", financialDto.getFeeType());
        params.put("totalMoney", financialDto.getTotalMoney().setScale(2, RoundingMode.HALF_UP));
        params.put("hasLoan", financialDto.getHasLoan());
        params.put("memo", financialDto.getMemo());
        params.put("circulationUsers", financialDto.getCirculationUsers());
        params.put("executors", financialDto.getExecutors());

        templateEngine.render(params, "financial.ftl", srcFile);

        // 生成借款协议PDF
        try {
            PDFUtil.renderToPDFByData(ContractServiceImpl.class.getResource("/").getFile(), FileUtils.readFileToString(srcFile), srcPdfPath, "simsun.ttc");
        } catch (IOException e) {
            logger.error("生成签章协议失败", e);
        }

        for (SysUserTsign sysUserTsign : sysUserTsigns) {
            eSealPlugin.userSign(sysUserTsign.getAccountId(), sysUserTsign.getSealData(), sysUserTsign.getUserId().toString(), srcPdfPath, srcPdfPath);
        }

        //上传到oss
        String dateStrOss = dateFormatOss.format(date);
        dateStrOss = "daya/oa/" + dateStrOss + "/" + DateUtil.getHour(date);
        //String pdfFilePath = storagePluginContext.uploadFile(KS3StoragePlugin.PLUGIN_NAME, dateStrOss, srcFile);
        String pdfFilePath = ossPluginContext.getPluginService(TencentOssPlugin.PLUGIN_NAME).uploadFile(dateStrOss, srcFile);

        FileUtils.deleteQuietly(srcFile);

        return pdfFilePath;
    }
}
