package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.MusicGroupStatusEnum;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.dto.SchoolDto;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class CooperationOrganServiceImpl extends BaseServiceImpl<Integer, CooperationOrgan>  implements CooperationOrganService {
	
	@Resource
	private CooperationOrganDao cooperationOrganDao;
	@Resource
	private CooperationOrganLinkmanDao cooperationOrganLinkmanDao;
	@Resource
	private MusicGroupDao musicGroupDao;
	@Resource
	private ClassGroupDao classGroupDao;
	@Resource
	private ImGroupMemberService imGroupMemberService;
	@Resource
	private ImGroupService imGroupService;
	@Resource
	private EmployeeDao employeeDao;
	@Resource
	private SysUserFeignService sysUserFeignService;
    @Resource
    private SchoolStaffService schoolStaffService;
    @Resource
    private SysUserService sysUserService;

	@Override
	public BaseDAO<Integer, CooperationOrgan> getDAO() {
		return cooperationOrganDao;
	}
	@Override
	public CooperationOrganDao getDao() {
		return cooperationOrganDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public long insert(CooperationOrgan cooperationOrgan) {
		Date date = new Date();
		cooperationOrgan.setCreateTime(date);
		cooperationOrgan.setUpdateTime(date);
		super.insert(cooperationOrgan);

		List<CooperationOrganLinkman> list = cooperationOrgan.getCooperationOrganLinkmanList();

		if (list != null && list.size() > 0) {
			for (CooperationOrganLinkman co : list) {
				co.setCreateTime(date);
				co.setCooperationOrganId(cooperationOrgan.getId());
			}
			cooperationOrganLinkmanDao.batchInsert(list);
		}

		return 1;
	}

	@Override
	public List<CooperationOrgan> queryByOrganId(String organId) {
		return cooperationOrganDao.queryByOrganId(organId);
	}


    @Override
    public List<CooperationOrgan> queryByOrganId(String organId,Boolean enable) {
        if (enable == null || enable) {
            return cooperationOrganDao.queryByOrganId(organId);
        }
        return cooperationOrganDao.queryByOrganIdAndEnable(organId,enable);
    }

    @Override
    public List<MusicGroupPaymentCalender> getCooperationOrganCalender4School(Integer id) {
		return cooperationOrganDao.getCooperationOrganCalender4School(id);
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateCooperation(CooperationOrgan cooperationOrgan) {
		CooperationOrgan cooperationOrgan1 = cooperationOrganDao.get(cooperationOrgan.getId());
		//修改合作单位下的学校账号
		if(!cooperationOrgan.getOrganId().equals(cooperationOrgan1.getOrganId())){
			schoolStaffService.lambdaUpdate().eq(SchoolStaff::getSchoolId, cooperationOrgan.getId())
					.set(SchoolStaff::getOrganIdList, cooperationOrgan.getOrganId()).update();
		}
		//修改了乐团主管后，修改关联乐团的乐团主管
		if(!cooperationOrgan.getEducationUserId().equals(cooperationOrgan1.getEducationUserId())){
			SysUser sysUser = sysUserService.queryUserById(cooperationOrgan.getEducationUserId());
			//获取进行中、暂停、筹备中的乐团列表
			List<MusicGroup> musicGroups = musicGroupDao.findByCooperationIdAndStatus(cooperationOrgan.getId());
			if(CollectionUtils.isNotEmpty(musicGroups)){
				for (MusicGroup musicGroup : musicGroups) {
					//获取当前乐团所有班级
					List<ClassGroup> classGroups = classGroupDao.findClassGroups(musicGroup.getId());
					if (classGroups == null || classGroups.size() == 0) {
						continue;
					}
					Integer educationalTeacherId = musicGroup.getEducationalTeacherId();
					classGroups.forEach(e -> {
						if(educationalTeacherId != null){
							imGroupMemberService.quit(e.getId().toString(), educationalTeacherId);
						}
						imGroupMemberService.join(e.getId().toString(),"乐团主管",false,null,sysUser);
					});
				}
			}
			//更新预报名群信息
			List<MusicGroup> musicGroupList = musicGroupDao.findByCooperationId(cooperationOrgan.getId());
			if(musicGroupList.size() > 0){
				for (MusicGroup musicGroup : musicGroupList) {
					//不是导入的乐团
					if(musicGroup.getId().length() > 10){
						//是否有预报名的群聊
						ImGroup imGroup = imGroupService.get(musicGroup.getId());
						if(imGroup != null){
							imGroupMemberService.quit(musicGroup.getId(),musicGroup.getEducationalTeacherId());
							imGroupMemberService.join(musicGroup.getId(),"乐团主管",false,imGroup,sysUser);
						}
					}
				}
			}
			//修改乐团主管
			musicGroupDao.batchUpdateEdu(cooperationOrgan.getId(),cooperationOrgan.getEducationUserId());
		}
		
		//修改了合作单位联系人
		cooperationOrganLinkmanDao.deleteByCooperationOrganId(cooperationOrgan.getId());
		List<CooperationOrganLinkman> cooperationOrganLinkmanList = cooperationOrgan.getCooperationOrganLinkmanList();
		if(cooperationOrganLinkmanList != null && cooperationOrganLinkmanList.size() > 0){
			cooperationOrganLinkmanDao.batchInsert(cooperationOrganLinkmanList);
		}
		
		cooperationOrganDao.update(cooperationOrgan);

		// 修改合作单位下的用户，改为锁定
		schoolStaffService.getMapper().updateStatusAndRecoveryStatus(cooperationOrgan.getIsEnable(), cooperationOrgan.getId());

		if (!cooperationOrgan.getIsEnable()) {
			//用户退出登陆
			CompletableFuture.runAsync(() -> {
				List<SchoolStaff> schoolStaffs = schoolStaffService.getMapper().getByCoopIdAndType(cooperationOrgan.getId(), null);
				for (SchoolStaff schoolStaff : schoolStaffs) {
					sysUserFeignService.exitByPhone("education", sysUserService.queryUserById(schoolStaff.getUserId().intValue()).getPhone());
				}
			});
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateCooperationEdu(List<Integer> organIds) {
		List<CooperationOrgan> organList = cooperationOrganDao.getCooperationOrganByIds(organIds);
		for (CooperationOrgan cooperationOrgan : organList) {
			if(cooperationOrgan.getEducationUserId() != null){
				Employee employee = employeeDao.get(cooperationOrgan.getEducationUserId());
				if(employee != null){
					//获取进行中、暂停、筹备中的乐团列表
					List<MusicGroup> musicGroups = musicGroupDao.findByCooperationIdAndStatus(cooperationOrgan.getId());
					if(musicGroups.size() > 0){
						for (MusicGroup musicGroup : musicGroups) {
							//获取当前乐团所有班级
							List<ClassGroup> classGroups = classGroupDao.findClassGroups(musicGroup.getId());
							if (classGroups == null || classGroups.size() == 0) {
								continue;
							}
							Integer educationalTeacherId = musicGroup.getEducationalTeacherId();
							List<Integer> quitUserIdList = new ArrayList<Integer>();
							if(educationalTeacherId != null){
								quitUserIdList.add(educationalTeacherId);
							}
							Map<Integer, String> userRoleMap = new HashMap<Integer, String>(1);
							userRoleMap.put(cooperationOrgan.getEducationUserId(), "乐团主管");
							classGroups.forEach(e -> {
								if(educationalTeacherId != null){
									imGroupMemberService.quit(e.getId().toString(), quitUserIdList);
								}
								imGroupMemberService.join(e.getId().toString(), userRoleMap);
							});
						}
					}
					//更新预报名群信息
					List<MusicGroup> musicGroupList = musicGroupDao.findByCooperationId(cooperationOrgan.getId());
					if(musicGroupList.size() > 0){
						for (MusicGroup musicGroup : musicGroupList) {
							//不是导入的乐团
							if(musicGroup.getId().length() > 10){
								//是否有预报名的群聊
								ImGroup imGroup = imGroupService.get(musicGroup.getId());
								if(imGroup != null){
									imGroupMemberService.quit(musicGroup.getId(),musicGroup.getEducationalTeacherId());
									imGroupMemberService.join(musicGroup.getId(),cooperationOrgan.getEducationUserId(),"乐团主管",false);
								}
							}
						}
					}
					//修改乐团主管
					musicGroupDao.batchUpdateEdu(cooperationOrgan.getId(),cooperationOrgan.getEducationUserId());
					System.out.println(cooperationOrgan.getName() + "  OK");
				}
			}
		}
	}

    /**
     * 根据合作单位id获取合作单位信息
     *
     * @param cooperationIds 合作单位id
     */
    @Override
    public Map<Integer, CooperationOrgan> getMapByIds(List<Integer> cooperationIds) {
        if (CollectionUtils.isEmpty(cooperationIds)) {
            return Collections.emptyMap();
        }
        List<CooperationOrgan> cooperationOrgans = cooperationOrganDao.getCooperationOrganByIds(cooperationIds);
        if (CollectionUtils.isEmpty(cooperationOrgans)) {
            return Collections.emptyMap();
        }
        return cooperationOrgans.stream().collect(Collectors.toMap(CooperationOrgan::getId, Function.identity()));
    }

    @Override
    public List<MusicGroup> musicGroupPage(Integer coopId) {

        List<MusicGroup> musicGroupList = musicGroupDao.findByCooperationId(coopId);

        if (CollectionUtils.isEmpty(musicGroupList)) {
            return Collections.emptyList();
        }
        return musicGroupList.stream().filter(o -> o.getStatus().equals(MusicGroupStatusEnum.PROGRESS)).collect(Collectors.toList());
    }

    @Override
    public List<MusicGroup> musicGroupPage(String organIds,String coopId) {

        List<MusicGroup> musicGroupList = musicGroupDao.findByCoopIdAndOrganId(organIds,coopId);

        if (CollectionUtils.isEmpty(musicGroupList)) {
            return Collections.emptyList();
        }
        return musicGroupList.stream().filter(o -> o.getStatus().equals(MusicGroupStatusEnum.PROGRESS)).collect(Collectors.toList());
    }

    @Override
    public List<SchoolDto> coopListByIds(List<Integer> coopIds) {
		List<CooperationOrgan> schools = cooperationOrganDao.getCooperationOrganByIds(coopIds);
		return schools.stream().map(e -> {
			SchoolDto schoolDto = new SchoolDto();
			schoolDto.setId(e.getId());
			schoolDto.setName(e.getName());
			return schoolDto;
		}).collect(Collectors.toList());
    }

}