package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.CourseScheduleDao;
import com.ym.mec.biz.dal.dao.SubjectChangeDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.CooperationShareProfitDto;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.util.collection.MapUtil;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.biz.dal.entity.CooperationShareProfit;
import com.ym.mec.biz.service.CooperationShareProfitService;
import com.ym.mec.biz.dal.dao.CooperationShareProfitDao;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class CooperationShareProfitServiceImpl extends BaseServiceImpl<Integer, CooperationShareProfit>  implements CooperationShareProfitService {
	
	@Autowired
	private CooperationShareProfitDao cooperationShareProfitDao;
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private TeacherDao teacherDao;
	
	@Autowired
	private SubjectChangeDao subjectChangeDao;

	@Override
	public BaseDAO<Integer, CooperationShareProfit> getDAO() {
		return cooperationShareProfitDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public PageInfo<CooperationShareProfitDto> queryDtoPage(QueryInfo queryInfo){
		PageInfo<CooperationShareProfitDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<CooperationShareProfitDto> dataList = null;
		int count = cooperationShareProfitDao.countDto(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = cooperationShareProfitDao.queryDtoPage(params);
			List<Integer> eduUserIds = dataList.stream().map(e -> e.getEducationUserId()).distinct().collect(Collectors.toList());
			List<String> groupIds = dataList.stream().map(e -> e.getMusicGroupId()).distinct().collect(Collectors.toList());
			List<Integer> organIds = dataList.stream().map(e -> e.getOrganId()).distinct().collect(Collectors.toList());
			Map<Integer, String> eduNameMap = MapUtil.convertMybatisMap(teacherDao.queryNameByIdList(eduUserIds));
			Map<Integer, String> organNameMap = getMap("organization", "id_", "name_", organIds,queryInfo.getTenantId(), Integer.class, String.class);
			Map<String, String> groupNameMap = getMap("music_group", "id_", "name_", groupIds,queryInfo.getTenantId(), String.class, String.class);
			for (CooperationShareProfitDto dto : dataList) {
				dto.setEducationUserName(eduNameMap.get(dto.getEducationUserId()));
				dto.setOrganName(organNameMap.get(dto.getOrganId()));
				dto.setMusicGroupName(groupNameMap.get(dto.getMusicGroupId()));
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void init(List<Long> courseIds) {
		if(CollectionUtils.isEmpty(courseIds)){
			return;
		}
		List<CourseSchedule> courseSchedules = courseScheduleDao.findByCourseScheduleIds(courseIds);
		List<String> musicGroupIds = courseSchedules.stream().map(e -> e.getMusicGroupId()).distinct().collect(Collectors.toList());
		if(CollectionUtils.isEmpty(musicGroupIds)){
			return;
		}
		musicGroupIds.removeAll(cooperationShareProfitDao.queryNotShare(musicGroupIds));
		musicGroupIds.removeAll(Collections.singleton(null));
		if(CollectionUtils.isEmpty(musicGroupIds)){
			return;
		}
		
		List<CooperationShareProfit> profits = cooperationShareProfitDao.initShareProfit(musicGroupIds);
		if(CollectionUtils.isEmpty(profits)){
			return;
		}
		
		//存在声部更换的学生
		BigDecimal courseMargin = BigDecimal.ZERO;
		for(CooperationShareProfit sp : profits){
			courseMargin = subjectChangeDao.getTotalCourseMargin(sp.getMusicGroupId());
			if(courseMargin != null){
				sp.setTotalAmount(courseMargin.add(sp.getTotalAmount()));
			}
		}
		
		cooperationShareProfitDao.batchInsert(profits);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void addCredentials(CooperationShareProfit profit) {
		CooperationShareProfit shareProfit = Optional.
				ofNullable(cooperationShareProfitDao.get(profit.getId())).orElseThrow(()->new BizException("分润信息不存在"));
		if(StringUtils.equals(shareProfit.getPaymentStatus(),"PAID_COMPLETED")){
			throw new BizException("操作失败：请勿重复录入");
		}
		shareProfit.setCredentials(profit.getCredentials());
		shareProfit.setPaymentNo(profit.getPaymentNo());
		shareProfit.setMemo(profit.getMemo());
		shareProfit.setPaymentStatus("PAID_COMPLETED");
		cooperationShareProfitDao.update(shareProfit);
	}
}