package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.CourseConvertDetailDto;
import com.ym.mec.biz.dal.dto.CourseConvertLogDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.page.ConvertLogQueryInfo;
import com.ym.mec.biz.service.CourseConvertLogService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class CourseConvertLogServiceImpl extends BaseServiceImpl<Integer, CourseConvertLog> implements CourseConvertLogService {

	@Autowired
	private CourseConvertLogDao courseConvertLogDao;
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private CourseScheduleTeacherSalaryDao courseScheduleTeacherSalaryDao;
	@Autowired
	private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
	@Autowired
	private TeacherAttendanceDao teacherAttendanceDao;
	@Autowired
	private SysUserService sysUserService;
	@Autowired
	private ClassGroupDao classGroupDao;

	@Override
	public BaseDAO<Integer, CourseConvertLog> getDAO() {
		return courseConvertLogDao;
	}

	@Override
	public CourseConvertLogDao getDao() {
		return courseConvertLogDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
    public CourseConvertLog save(List<Long> oldCourseIdList, List<Integer> newClassGroupIds, List<Long> courseIdList,
					 List<CourseScheduleTeacherSalary> teacherSalaries,
					 List<CourseScheduleStudentPayment> studentPayments) {
		SysUser user = sysUserService.getUser();
		CourseConvertLog courseConvertLog = new CourseConvertLog();
		courseConvertLog.setOperatorName(user.getRealName());
		courseConvertLog.setOperator(user.getId());
		courseConvertLog.setOldCourseIds(StringUtils.join(oldCourseIdList,","));
		String userIds = studentPayments.stream().map(e -> e.getUserId().toString()).distinct().collect(Collectors.joining(","));
		courseConvertLog.setStudentIds(userIds);

		//获取老师课酬json
		courseConvertLog.setOldCourseTeacherSalaryJson(JSONObject.toJSONString(teacherSalaries));
		//删除老师课酬
		courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(oldCourseIdList);

		//获取学员课程价值
		courseConvertLog.setOldCourseStudentMapperJson(JSONObject.toJSONString(studentPayments));
		//删除学员课程价值
		courseScheduleStudentPaymentDao.deleteByCourseSchedule(oldCourseIdList);

		//获取课程列表
		List<CourseSchedule> courseSchedules = courseScheduleDao.findByCourseScheduleIds(oldCourseIdList);
		Integer oldClassId = courseSchedules.stream().map(e -> e.getClassGroupId()).findFirst().get();
		ClassGroup classGroup = classGroupDao.get(oldClassId);
		courseConvertLog.setMusicGroupId(classGroup.getMusicGroupId());
		courseConvertLog.setOldClassId(oldClassId);
		courseConvertLog.setOldClassName(classGroup.getName());
		courseConvertLog.setOldCourseJson(JSONObject.toJSONString(courseSchedules));
		//删除课程
		courseScheduleDao.batchDelete(oldCourseIdList);

		//记录老师考勤
		List<TeacherAttendance> teacherAttendances = teacherAttendanceDao.findByCourseSchedules(oldCourseIdList);
		courseConvertLog.setOldTeacherAttendanceJson(JSONObject.toJSONString(teacherAttendances));
		//删除考勤
		teacherAttendanceDao.batchDeleteByCourseSchedules(oldCourseIdList);

		courseConvertLog.setNewCourseIds(StringUtils.join(courseIdList,","));
		courseConvertLog.setNewClassIds(StringUtils.join(newClassGroupIds,","));
		courseConvertLogDao.insert(courseConvertLog);
		return courseConvertLog;
	}

	@Override
	public PageInfo<CourseConvertLogDto> queryConvertPage(ConvertLogQueryInfo queryInfo) {
		PageInfo<CourseConvertLogDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<CourseConvertLogDto> dataList = null;
		int count = courseConvertLogDao.countConvertPage(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = courseConvertLogDao.queryConvertPage(params);
			for (CourseConvertLogDto dto : dataList) {
				dto.setConvertCourseNum(dto.getOldCourseIds().split(",").length);
				dto.setConvertAfterClassNum(dto.getNewClassIds().split(",").length);
				dto.setConvertAfterCourseNum(dto.getNewCourseIds().split(",").length);
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

    @Override
    public List<CourseConvertDetailDto> queryCourse(Integer courseConvertId) {
        return courseConvertLogDao.queryCourse(courseConvertId);
    }
}