package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.ym.mec.biz.dal.dao.CourseScheduleConvertDao;
import com.ym.mec.biz.dal.dao.StudentDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.dto.CourseConvertSumDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.CourseStatusEnum;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.collection.MapUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.GroupType.MUSIC;

@Service
public class CourseScheduleConvertServiceImpl implements CourseScheduleConvertService {

    @Autowired
    private CourseScheduleConvertDao courseScheduleConvertDao;
    @Autowired
    private StudentDao studentDao;
    @Autowired
    private ClassGroupTeacherMapperService classGroupTeacherMapperService;
    @Autowired
    private CourseScheduleService courseScheduleService;
    @Autowired
    private ClassGroupStudentMapperService classGroupStudentMapperService;
    @Autowired
    private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
    @Autowired
    private CourseScheduleStudentPaymentService courseScheduleStudentPaymentService;
    @Autowired
    private CourseConvertLogService courseConvertLogService;
    @Autowired
    private ImUserFriendService imUserFriendService;
    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private TeacherAttendanceService teacherAttendanceService;
    @Autowired
    private CloudTeacherFreeCourseService cloudTeacherFreeCourseService;
    @Autowired
    private TeacherDao teacherDao;

    @Override
    public HashMap<String, Object> queryConvertStudent(String courseIds) {
        List<Long> oldCourseIdList = Arrays.stream(courseIds.split(",")).map(id -> Long.parseLong(id)).collect(Collectors.toList());
        //校验课程是否可以进行转换
        this.checkConvertCourseIds(oldCourseIdList);
        //获取可转换课程时长，学员数量，课程数量
        CourseConvertSumDto courseConvertSumDto = courseScheduleConvertDao.sumCourseConvert(oldCourseIdList);
        //获取学员列表
        List<BasicUserDto> basicUserDtos = Optional.ofNullable(courseScheduleConvertDao.queryStudents(oldCourseIdList)).
                orElseThrow(()->new BizException("操作失败：所选课程没有学员"));
        List<Integer> userIds = basicUserDtos.stream().map(e -> e.getUserId()).collect(Collectors.toList());
        Map<Integer,String> subjectMap = MapUtil.convertIntegerMap(studentDao.getStudentSubjectMapList(userIds));
        basicUserDtos.stream().forEach(e->e.setSubjectName(subjectMap.get(e.getUserId())));
        HashMap<String, Object> resultMap = new HashMap<>(2);
        resultMap.put("courseConvertSum",courseConvertSumDto);
        resultMap.put("students",basicUserDtos);
        return resultMap;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult action(List<ClassGroupDto> classGroups) {
        ClassGroupDto classGroupDto = classGroups.get(0);
        List<Long> oldCourseIdList = Arrays.stream(classGroupDto.getCourseIds().split(",")).map(id -> Long.parseLong(id)).collect(Collectors.toList());
        //校验课程是否可以进行转换
        this.checkConvertCourseIds(oldCourseIdList);
        if(!classGroupDto.getConfirmGenerate()){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        //校验主教助教是否冲突
        classGroups.stream().forEach(e->classGroupTeacherMapperService.checkTeachingBishop(e.getClassGroupTeacherMapperList()));
        int classNum = classGroups.size();
        //课程列表
        List<CourseSchedule> courseScheduleList = new ArrayList<>();
        //老师课酬
        List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();
        //学员课程价值
        List<CourseScheduleStudentPayment> courseScheduleStudentPayments = new ArrayList<>();
        List<Integer> studentIdList = new ArrayList<>();

        for (ClassGroupDto groupDto : classGroups) {
            List<Integer> userIdStrSet = Arrays.stream(groupDto.getUserIds().split(",")).map(id -> Integer.valueOf(id)).collect(Collectors.toList());
            studentIdList.addAll(userIdStrSet);
            groupDto.setStudentNum(userIdStrSet.size());
            //创建班级
            ClassGroup classGroup = new ClassGroup();
            BeanUtils.copyProperties(groupDto,classGroup);
            classGroupService.insert(classGroup);
            groupDto.setClassGroupId(classGroup.getId());
            //设置班级上的老师
            classGroupTeacherMapperService.addMapper(groupDto);
            //2、插入班级学生关联关系
            classGroupStudentMapperService.addMapper(classGroup,userIdStrSet);
            //排课
            groupDto.setCheckType(classNum>1?1:0);
            HttpResponseResult<List<CourseSchedule>> result = courseScheduleService.createCourse(groupDto);
            if(result.getCode() != 200){
                StringBuffer sb = new StringBuffer("班级");
                result.setMsg(sb.append(classGroup.getName()).append(":").append(result.getMsg()).toString());
                if(groupDto.getConfirmGenerate()){
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                }
                return result;
            }else {
                if(groupDto.getConfirmGenerate()){
                    courseScheduleList.addAll(result.getData());
                    courseScheduleTeacherSalaryList.addAll(groupDto.getCourseScheduleTeacherSalaryList());
                    courseScheduleStudentPayments.addAll(groupDto.getCourseScheduleStudentPayments());
                }else {
                    List<CourseSchedule> schedules = result.getData();
                    List<Integer> teacherIdList = schedules.stream().map(e -> e.getActualTeacherId()).collect(Collectors.toList());
                    Map<Integer, String> maps = MapUtil.convertIntegerMap(teacherDao.queryNameByIdList(teacherIdList));
                    for (CourseSchedule e : schedules) {e.setActualTeacherName(maps.get(e.getActualTeacherId()));}
                    return result;
                }
            }
        }
        //获取总课酬
        List<CourseScheduleTeacherSalary> teacherSalaries = courseScheduleTeacherSalaryService.getDao().findByCourseSchedules(oldCourseIdList);
        //获取学员课程价值
        List<CourseScheduleStudentPayment> studentPayments = courseScheduleStudentPaymentService.getDao().findByCourseScheduleIds(oldCourseIdList);
        //保存老师课酬
        courseScheduleTeacherSalaryService.splitTeacherSalary(courseScheduleTeacherSalaryList,teacherSalaries);
        //生成老师考勤
        teacherAttendanceService.addTeacherAttendance(courseScheduleList);
        //保存学员课程关联
        courseScheduleStudentPaymentService.splitStudentPayment(courseScheduleStudentPayments,studentPayments);
        List<Integer> newClassGroupIds = classGroups.stream().map(e -> e.getClassGroupId()).collect(Collectors.toList());
        //记录云教练课程
        List<Long> courseIdList = courseScheduleList.stream().map(e -> e.getId()).collect(Collectors.toList());
        cloudTeacherFreeCourseService.courseConvert(oldCourseIdList,courseIdList);
        //记录课程转换相关信息
        courseConvertLogService.save(oldCourseIdList,newClassGroupIds,courseIdList,teacherSalaries,studentPayments);
        // 刷新群组
        imUserFriendService.refreshGroupImUserFriend(classGroupDto.getMusicGroupId(), MUSIC);
        return BaseController.succeed();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void revoke(Integer courseConvertId) {
        CourseConvertLog courseConvertLog = courseConvertLogService.get(courseConvertId);
        //校验是否可以还原
        this.checkConvertRevoke(courseConvertLog);
        List<Long> newCourseIdList = Arrays.stream(courseConvertLog.getNewCourseIds().split(",")).map(e -> Long.parseLong(e)).collect(Collectors.toList());
        List<Integer> userIdList = Arrays.stream(courseConvertLog.getStudentIds().split(",")).map(e -> Integer.parseInt(e)).collect(Collectors.toList());
        List<CourseSchedule> courseSchedules = JSONArray.parseArray(courseConvertLog.getOldCourseJson(), CourseSchedule.class);
//        courseScheduleService.getDao().batchUpdateDelFlag(courseConvertLog.getOldCourseIds());
        courseScheduleService.getDao().batchInsert(courseSchedules);

        String studentMapperJson = courseConvertLog.getOldCourseStudentMapperJson();
        List<CourseScheduleStudentPayment> studentPayments = JSONArray.parseArray(studentMapperJson, CourseScheduleStudentPayment.class);
        //获取学员课程价值
        List<CourseScheduleStudentPayment> paymentList = courseScheduleStudentPaymentService.getDao().findByCourseScheduleIds(newCourseIdList);
        //获取现有所有学员列表
        List<Integer> currentUserIdList = paymentList.stream().map(e -> e.getUserId()).distinct().collect(Collectors.toList());
        userIdList.removeAll(currentUserIdList);
        if(!CollectionUtils.isEmpty(userIdList)){
            studentPayments = studentPayments.stream().filter(e->!userIdList.contains(e.getUserId())).collect(Collectors.toList());
        }
        courseScheduleStudentPaymentService.getDao().batchInsert(studentPayments);

        //保存老师课酬
        String teacherSalaryJson = courseConvertLog.getOldCourseTeacherSalaryJson();
        courseScheduleTeacherSalaryService.getDao().batchInsert(JSONArray.parseArray(teacherSalaryJson,CourseScheduleTeacherSalary.class));
        //保存考勤
        String teacherAttendanceJson = courseConvertLog.getOldTeacherAttendanceJson();
        teacherAttendanceService.getDao().batchInsert(JSONArray.parseArray(teacherAttendanceJson,TeacherAttendance.class));

        //删除转换后的课程、课酬、学员课程价值
        this.delByCourse(newCourseIdList);
        courseConvertLogService.getDao().updateRevoke(courseConvertId);
    }

    private void delByCourse(List<Long> newCourseIdList){
        //删除转换后的课程、课酬、学员课程价值
        courseScheduleService.getDao().batchDelete(newCourseIdList);
        //删除老师考勤
        teacherAttendanceService.getDao().batchDeleteByCourseSchedules(newCourseIdList);
        //删除老师课酬
        courseScheduleTeacherSalaryService.getDao().batchDeleteByCourseScheduleIds(newCourseIdList);
        //删除学员课程价值
        courseScheduleStudentPaymentService.getDao().deleteByCourseSchedule(newCourseIdList);
        //删除课程成本
        cloudTeacherFreeCourseService.getDao().deleteByCourseIds(newCourseIdList);
    }

    private void checkConvertRevoke(CourseConvertLog courseConvertLog) {
        Optional.ofNullable(courseConvertLog).orElseThrow(()->new BizException("课程转换记录不存在"));
        if(courseConvertLog.getRevokeFlag()){
            throw new BizException("课程转换记录已还原");
        }
        //原班级是否改动
        Integer oldClassId = courseConvertLog.getOldClassId();
        ClassGroup classGroup = classGroupService.getDao().get(oldClassId);
        if(Objects.isNull(classGroup) || classGroup.getDelFlag() == 1){
            throw new BizException("操作失败：原班级信息不存在");
        }
        String studentIds = courseConvertLog.getStudentIds();
        //学员是否还在班级
        Integer num = classGroupStudentMapperService.getDao().countByUserIdsAndClassIds(studentIds,oldClassId);
        if(num != null && num < studentIds.split(",").length){
            throw new BizException("操作失败：部分学员已退班");
        }
        //课程是否有消耗
        String newCourseIds = courseConvertLog.getNewCourseIds();
        List<Long> courseIdList = Arrays.stream(newCourseIds.split(",")).map(e -> Long.parseLong(e)).collect(Collectors.toList());
        List<CourseSchedule> courseSchedules = courseScheduleService.getDao().findByCourseScheduleIds(courseIdList);
        if(CollectionUtils.isEmpty(courseSchedules)){
            throw new BizException("操作失败：没有可转换的课程");
        }
        if(courseSchedules.stream().anyMatch(e->e.getStatus() != CourseStatusEnum.NOT_START)){
            throw new BizException("操作失败：存在已结束课程");
        }
        if(courseSchedules.size() < courseIdList.size()){
            throw new BizException("操作失败：存在被删除的课程");
        }
        //学员课表是否变动
        List<CourseScheduleStudentPayment> studentPayments = courseScheduleStudentPaymentService.getDao().findByCourseScheduleIds(courseIdList);
        if(CollectionUtils.isEmpty(studentPayments)){
            throw new BizException("操作失败：没有可操作的学员");
        }
        //课程里面增加学员不允许还原、减少可以
        long count = studentPayments.stream().map(e -> e.getCreateTime()).distinct().count();
        if(count > 1){
            throw new BizException("操作失败：课程学员有变更");
        }
    }

    private void checkConvertCourseIds(List<Long> oldCourseIdList){
        //是否同一个班级
        int classNum = courseScheduleConvertDao.countClassNum(oldCourseIdList);
        if(classNum > 1){
            throw new BizException("操作失败：请选择同一个班级的课程");
        }
        //获取合法的课程列表
        int num = courseScheduleConvertDao.countLegalCourse(oldCourseIdList);
        if(num < oldCourseIdList.size()){
            throw new BizException("操作失败：所选部分课程不支持课程转换");
        }
        //校验学员时长是否一致
        Integer errorNum = courseScheduleConvertDao.checkStudentCourseNum(oldCourseIdList,oldCourseIdList.size());
        if (errorNum != null && errorNum > 0){
            throw new BizException("操作失败: 课程关联学员不一致");
        }
    }
}
