package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.vo.DegreeNewsVo;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import lombok.extern.slf4j.Slf4j;
import com.ym.mec.biz.dal.entity.DegreeNews;
import com.ym.mec.biz.dal.wrapper.DegreeNewsWrapper;
import com.ym.mec.biz.dal.mapper.DegreeNewsMapper;
import com.ym.mec.biz.service.DegreeNewsService;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 考级资讯表
 * 2023-04-10 10:24:25
 */
@Slf4j
@Service
public class DegreeNewsServiceImpl extends ServiceImpl<DegreeNewsMapper, DegreeNews> implements DegreeNewsService {

    @Autowired
    private TeacherDao teacherDao;

	/**
     * 查询详情
     * @param id 详情ID
     * @return DegreeNews
     */
	@Override
    public DegreeNews detail(Long id) {
        
        return baseMapper.selectById(id);
    }
    
    /**
     * 分页查询
     * @param page IPage<DegreeNews>
     * @param query DegreeNewsWrapper.DegreeNewsQuery
     * @return IPage<DegreeNews>
     */
    @Override
    public IPage<DegreeNewsVo> selectPage(IPage<DegreeNewsVo> page, DegreeNewsWrapper.DegreeNewsQuery query) {
        List<DegreeNewsVo> degreeNewsVos = baseMapper.selectPage(page, query);
        if(CollectionUtils.isNotEmpty(degreeNewsVos)){
            List<Integer> operatorIds = degreeNewsVos.stream().map(e -> e.getOperatorId()).distinct().collect(Collectors.toList());
            Map<Integer, String> nameIdMap = MapUtil.convertIntegerMap(teacherDao.queryNameByIdList(operatorIds));
            for (DegreeNewsVo vo : degreeNewsVos) {
                vo.setOperatorName(nameIdMap.get(vo.getOperatorId()));
            }
        }
        return page.setRecords(degreeNewsVos);
    }
	
    /**
     * 添加
     * @param degreeNews DegreeNewsWrapper.DegreeNews
     * @return Boolean
     */
    @Override
    public Boolean add(DegreeNewsWrapper.DegreeNews degreeNews) {    	
        
        return this.save(JSON.parseObject(degreeNews.jsonString(), DegreeNews.class));
    }

    /**
     * 更新
     * @param degreeNews DegreeNewsWrapper.DegreeNews
     * @return Boolean
     */
    @Override
    public Boolean update(DegreeNewsWrapper.DegreeNews degreeNews){

        return this.updateById(JSON.parseObject(degreeNews.jsonString(), DegreeNews.class));       
    }
}