package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.DegreeRegistrationActivityDto;
import com.ym.mec.biz.dal.dto.DegreeRegistrationDto;
import com.ym.mec.biz.dal.dto.PageInfoDegree;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.DegreeQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class DegreeRegistrationServiceImpl extends BaseServiceImpl<Integer, DegreeRegistration> implements DegreeRegistrationService {

    @Autowired
    private DegreeRegistrationDao degreeRegistrationDao;
    @Autowired
    private IdGeneratorService idGeneratorService;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private PayService payService;
    @Autowired
    private StudentPaymentOrderDetailDao studentPaymentOrderDetailDao;
    @Autowired
    private SysUserCashAccountService sysUserCashAccountService;
    @Autowired
    private StudentTeacherMapperDao studentTeacherMapperDao;
    @Autowired
    private DegreeLevelFeeService degreeLevelFeeService;
    @Autowired
    private DegreeService degreeService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SubjectDao subjectDao;

    @Autowired
    private OrganizationService organizationService;

    @Override
    public BaseDAO<Integer, DegreeRegistration> getDAO() {
        return degreeRegistrationDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult pay(DegreeRegistrationActivityDto degreeRegistration) throws Exception {
        //判断用户是否已存在订单
        List<StudentPaymentOrder> paymentOrders = studentPaymentOrderService.getUserOrderByType(degreeRegistration.getUserId(), OrderTypeEnum.DEGREE_REGISTRATION, DealStatusEnum.ING);
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(paymentOrders)) {
            HttpResponseResult result = studentPaymentOrderService.checkRepeatPay(paymentOrders.get(0), degreeRegistration.getIsRepay());
            if (result.getCode() != 200) {
                return result;
            }else {
                //关闭指定订单后，处理考级明细为未支付
                StudentPaymentOrder order = paymentOrders.get(0);
                List<DegreeRegistration> degreeRegistrations = degreeRegistrationDao.findByByOrderId(order.getOrderNo());
                if(!CollectionUtils.isEmpty(degreeRegistrations)){
                    degreeRegistrations.forEach(e->e.setStatus(0));
                    degreeRegistrationDao.batchUpdate(degreeRegistrations);
                }
            }
        }
        Date nowDate = new Date();
        String orderNo = idGeneratorService.generatorId("payment") + "";
        //获取收费项价格
        BigDecimal money = BigDecimal.ZERO;
        List<DegreeRegistrationDto> registrationDtos = degreeRegistration.getDegreeRegistrationDtos();
        List<DegreeRegistration> degreeRegistrationList = new ArrayList<>();
        for (DegreeRegistrationDto registrationDto : registrationDtos) {
            DegreeRegistration degree = new DegreeRegistration();
            BeanUtils.copyProperties(degreeRegistration,degree);
            String level = registrationDto.getLevel();
            degree.setOrderNo(orderNo);
            degree.setLevel(level);
            degree.setDegreeId(degreeRegistration.getDegreeId());
            degree.setTheoryCert(registrationDto.getTheoryCert());
            degree.setType(registrationDto.getType());
            degree.setStatus(1);
            DegreeLevelFeeNew one;
            if(registrationDto.getType() == DegreeTypeEnum.THEORY){
                one = degreeLevelFeeService.lambdaQuery()
                        .eq(DegreeLevelFeeNew::getDegreeType, registrationDto.getType())
                        .eq(DegreeLevelFeeNew::getLevel, registrationDto.getLevel())
                        .eq(DegreeLevelFeeNew::getTenantId,degreeRegistration.getTenantId())
                        .one();
            }else {
                one = degreeLevelFeeService.lambdaQuery()
                        .eq(DegreeLevelFeeNew::getSubjectId, degreeRegistration.getSubjectId())
                        .eq(DegreeLevelFeeNew::getDegreeType, registrationDto.getType())
                        .eq(DegreeLevelFeeNew::getLevel, registrationDto.getLevel())
                        .eq(DegreeLevelFeeNew::getTenantId,degreeRegistration.getTenantId())
                        .one();
            }
            if(Objects.isNull(one)){
                throw new BizException("该级别已不可报名");
            }
            degree.setMoney(one.getFee());
            money = money.add(one.getFee());
            degreeRegistrationList.add(degree);
        }
        if(money.compareTo(degreeRegistration.getPrice())!=0){
            throw new BizException("商品价格不符");
        }
        degreeRegistrationDao.batchInsert(degreeRegistrationList);

        Integer userId = degreeRegistration.getUserId();
        String channelType = "";
        StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
        studentPaymentOrder.setUserId(userId);
        studentPaymentOrder.setGroupType(GroupType.DEGREE);
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.DEGREE_REGISTRATION);
        studentPaymentOrder.setExpectAmount(money);
        
        if(degreeRegistration.isUseBalance()){
        	SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(userId);
            if (userCashAccount == null) {
                throw new BizException("用户账户找不到");
            }
            BigDecimal deductBalance = BigDecimal.ZERO;
            if (userCashAccount.getBalance().subtract(money).doubleValue() >= 0) {
                // 更新订单信息
                studentPaymentOrder.setActualAmount(BigDecimal.ZERO);
                studentPaymentOrder.setBalancePaymentAmount(money);
                deductBalance = money;
                studentPaymentOrder.setPaymentChannel("BALANCE");
            } else {
                studentPaymentOrder.setActualAmount(money.subtract(userCashAccount.getBalance()));
                studentPaymentOrder.setBalancePaymentAmount(userCashAccount.getBalance());
                deductBalance = userCashAccount.getBalance();
            }
            money = money.subtract(deductBalance);
            sysUserCashAccountService.updateBalance(userId, deductBalance.negate(), PlatformCashAccountDetailTypeEnum.PAY_FEE, "考级报名");
        }else{
            studentPaymentOrder.setActualAmount(money);
            studentPaymentOrder.setBalancePaymentAmount(BigDecimal.ZERO);
        }
        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        studentPaymentOrder.setMusicGroupId(degreeRegistration.getDegreeId().toString());
        studentPaymentOrder.setOrganId(degreeRegistration.getOrganId());
        studentPaymentOrder.setRoutingOrganId(degreeRegistration.getOrganId());
        studentPaymentOrderService.insert(studentPaymentOrder);
        studentPaymentOrder.setVersion(0);

        StudentPaymentOrderDetail studentPaymentOrderDetail = new StudentPaymentOrderDetail();
        studentPaymentOrderDetail.setType(OrderDetailTypeEnum.DEGREE_REGISTRATION);
        studentPaymentOrderDetail.setPrice(studentPaymentOrder.getExpectAmount());
        studentPaymentOrderDetail.setRemitFee(BigDecimal.ZERO);
        studentPaymentOrderDetail.setPaymentOrderId(studentPaymentOrder.getId());
        studentPaymentOrderDetail.setCreateTime(nowDate);
        studentPaymentOrderDetail.setUpdateTime(nowDate);
        studentPaymentOrderDetailDao.insert(studentPaymentOrderDetail);

        if (money.compareTo(BigDecimal.ZERO) == 0) {
            Map<String, String> notifyMap = new HashMap<>();
            notifyMap.put("tradeState", "1");
            notifyMap.put("merOrderNo", studentPaymentOrder.getOrderNo());
            notifyMap.put("channelType", channelType);
            notifyMap.put("orderNo", "");
            studentPaymentOrderService.updateOrder(notifyMap);
            return BaseController.failed(HttpStatus.CREATED, notifyMap,"恭喜您,支付成功!");
        }

        String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);

        String receiver = "kjRegister";
        Map payMap = payService.getPayMap(
                money,
                BigDecimal.ZERO,
                orderNo,
                baseApiUrl + "/api-student/studentOrder/paymentResult?orderNo=" + orderNo,
                "考级报名活动",
                "考级报名活动",
                degreeRegistration.getOrganId(),
                receiver
        );

        studentPaymentOrder.setOrganId(degreeRegistration.getOrganId());
        studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
        studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
        studentPaymentOrder.setUpdateTime(nowDate);
        studentPaymentOrderService.update(studentPaymentOrder);
        return BaseController.succeed(payMap);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateStatus(StudentPaymentOrder studentPaymentOrder) {
        int updateCount = studentPaymentOrderService.update(studentPaymentOrder);
        if (updateCount <= 0) {
            throw new BizException("订单更新失败");
        }
        if(DealStatusEnum.SUCCESS.equals(studentPaymentOrder.getStatus())){
            List<StudentPaymentOrderDetail> orderDetails = studentPaymentOrderDetailDao.getOrderDetail(studentPaymentOrder.getId());
            BigDecimal additionCoursePrice = BigDecimal.ZERO;
            if(!CollectionUtils.isEmpty(orderDetails)){
                additionCoursePrice = orderDetails.stream().map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            }
            if(additionCoursePrice.compareTo(BigDecimal.ZERO)>0){
                sysUserCashAccountService.appendCourseBalance(studentPaymentOrder.getUserId(), additionCoursePrice, PlatformCashAccountDetailTypeEnum.RECHARGE, "2020考级报名");
            }
        }

        if (studentPaymentOrder.getStatus() == DealStatusEnum.CLOSE || studentPaymentOrder.getStatus() == DealStatusEnum.FAILED) {
            if (studentPaymentOrder.getBalancePaymentAmount() != null && studentPaymentOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
                sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(), PlatformCashAccountDetailTypeEnum.REFUNDS, "支付失败-退回");
            }
        }
        List<DegreeRegistration> degreeList = degreeRegistrationDao.getLockByOrderId(studentPaymentOrder.getOrderNo());
        Date date = new Date();
        for (DegreeRegistration degree : degreeList) {
            degree.setUpdateTime(date);
            degree.setOrderNo(studentPaymentOrder.getOrderNo());
            degree.setTransNo(studentPaymentOrder.getTransNo());
            switch (studentPaymentOrder.getStatus()){
                case SUCCESS:
                    degree.setStatus(2);
                    break;
                case FAILED:
                case CLOSE:
                    degree.setStatus(0);
                    break;
            }
        }
        if (degreeRegistrationDao.batchUpdate(degreeList) <= 0) {
            throw new BizException("更新支付状态失败");
        }
        return true;
    }

    @Override
    public PageInfoDegree<DegreeRegistration> getPageList(DegreeQueryInfo queryInfo) {
        PageInfoDegree<DegreeRegistration> pageInfo = new PageInfoDegree<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);


        List<DegreeRegistration> dataList = new ArrayList<>();
        int count = degreeRegistrationDao.queryCount(params);

        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = degreeRegistrationDao.queryPage(params);
            List<Integer> userIds = dataList.stream().map(e -> e.getUserId()).collect(Collectors.toList());
            Map<Integer,String> vipTeacherMapper = MapUtil.convertMybatisMap(studentTeacherMapperDao.queryTeacherMapper(userIds,"VIP"));
            Map<Integer,String> teacherMapper = MapUtil.convertMybatisMap(studentTeacherMapperDao.queryTeacherMapper(userIds,"NORMAL"));
            for (DegreeRegistration degreeRegistration : dataList) {
                degreeRegistration.setVipTeacherName(vipTeacherMapper.get(degreeRegistration.getUserId()));
                degreeRegistration.setNormalTeacherName(teacherMapper.get(degreeRegistration.getUserId()));
            }
            // 设置声部
            Set<Integer> subjectIdSet = dataList.stream().map(o -> o.getSubjectId()).filter(Objects::nonNull).collect(Collectors.toSet());
            if(!CollectionUtils.isEmpty(subjectIdSet)){
                List<Subject> subjectList = subjectDao.findBySubjectIds(new ArrayList<>(subjectIdSet));
                Map<Integer, Subject> subjectMap = subjectList.stream().collect(Collectors.toMap(Subject::getId, Function.identity()));
                for (DegreeRegistration degreeRegistration : dataList) {
                    if (degreeRegistration.getSubjectId() == null) {
                        continue;
                    }
                    Subject subject = subjectMap.get(degreeRegistration.getSubjectId());
                    if (subject != null) {
                        degreeRegistration.setSubjectName(subject.getName());
                    }
                }
            }
            // 设置分部
            List<Integer> organIdSet = dataList.stream().map(o -> o.getOrganId()).filter(Objects::nonNull).collect(Collectors.toList());
            Integer tenantId = TenantContextHolder.getTenantId();

            List<Organization> organs = organizationService.findOrgans(organIdSet, tenantId);

            if (!CollectionUtils.isEmpty(organs)) {
                // id 分组
                Map<Integer, String> organMap = organs.stream().collect(Collectors.toMap(Organization::getId, o -> o.getName(), (k1, k2) -> k1));
                dataList.forEach(o -> o.setOrganName(organMap.get(o.getOrganId())));
            }

            DegreeRegistration degree = degreeRegistrationDao.getTotalAmount(params);
            pageInfo.setApplyNum(degreeRegistrationDao.countApplyNum(params));
            pageInfo.setTotalAmount(degree.getMoney());
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult check(DegreeRegistrationActivityDto degreeRegistration, SysUser sysUser) {
        Date nowDate = new Date();
        Degree degree = degreeService.getById(degreeRegistration.getDegreeId());
        if(!degree.getTenantId().equals(sysUser.getTenantId())){
            return BaseController.failed(HttpStatus.CHECKPOINT,"暂未开放此活动");
        }
        degreeRegistration.setUserId(sysUser.getId());
        //是否对分部开放
        if(StringUtils.isNotEmpty(degree.getOrganIds())){
            List<Integer> studentIdList = Arrays.stream(degree.getOrganIds().split(",")).map(id -> Integer.valueOf(id)).collect(Collectors.toList());
            if(!studentIdList.contains(degreeRegistration.getOrganId())){
                return BaseController.failed(HttpStatus.CHECKPOINT,"暂未开放此活动");
            }
        }
        if (degree.getStartTime().compareTo(nowDate) > 0) {
            return BaseController.failed(HttpStatus.CHECKPOINT,"活动还未开始，谢谢关注");
        }
        if (degree.getEndTime().compareTo(nowDate) <= 0) {
            return BaseController.failed(HttpStatus.CHECKPOINT,"活动已结束，谢谢关注");
        }
        List<DegreeRegistrationDto> registrationDtos = degreeRegistration.getDegreeRegistrationDtos();
        if(CollectionUtils.isEmpty(registrationDtos)){
            return BaseController.failed(HttpStatus.CHECKPOINT,"请选择考级项目");
        }
        List<DegreeRegistrationDto> collect = registrationDtos.stream().filter(e -> e.getType() == DegreeTypeEnum.GRADE).collect(Collectors.toList());
        if(!CollectionUtils.isEmpty(collect)){
            String levels = collect.stream().map(e -> e.getLevel()).collect(Collectors.joining(","));
            DegreeRegistration userLevelDegree = degreeRegistrationDao.getUserLevelDegrees(degreeRegistration.getUserId(),
                    degree.getId(),DegreeTypeEnum.GRADE,levels,degreeRegistration.getSubjectId());
            if(Objects.nonNull(userLevelDegree)){
                StringBuffer sb = new StringBuffer("您选择的");
                sb.append("\"").append("专业考级-级别").append(userLevelDegree.getLevel()).append("\"").append("已报名，请勿重复选择");
                return BaseController.failed(HttpStatus.CHECKPOINT, sb.toString());
            }
        }
        collect = registrationDtos.stream().filter(e -> e.getType() == DegreeTypeEnum.THEORY).collect(Collectors.toList());
        if(!CollectionUtils.isEmpty(collect)){
            String levels = collect.stream().map(e -> e.getLevel()).collect(Collectors.joining(","));
            DegreeRegistration userLevelDegree = degreeRegistrationDao.getUserLevelDegrees(degreeRegistration.getUserId(),
                    degree.getId(),DegreeTypeEnum.THEORY,levels,null);
            if(Objects.nonNull(userLevelDegree)){
                StringBuffer sb = new StringBuffer("您选择的");
                sb.append("\"").append("乐理考级-级别").append(userLevelDegree.getLevel()).append("\"").append("已报名，请勿重复选择");
                return BaseController.failed(HttpStatus.CHECKPOINT, sb.toString());
            }
        }
        return BaseController.succeed();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int refund(Integer degreeRegistrationId) {
        DegreeRegistration degreeRegistration = degreeRegistrationDao.getLock(degreeRegistrationId);
        if(Objects.isNull(degreeRegistration)){
            throw new BizException("报名信息不存在");
        }
        if(degreeRegistration.getStatus() != 2){
            throw new BizException("当前状态不支持退费");
        }
        StudentPaymentOrder order = studentPaymentOrderService.findOrderByOrderNo(degreeRegistration.getOrderNo());
        if(Objects.isNull(order)){
            throw new BizException("订单信息不存在");
        }
        if(order.getStatus() != DealStatusEnum.SUCCESS){
            throw new BizException("当前订单状态不支持退费");
        }
        //退到账户余额
        if(degreeRegistration.getMoney().compareTo(BigDecimal.ZERO) <= 0){
            throw new BizException("订单金额异常");
        }
        sysUserCashAccountService.updateBalance(order.getUserId(), degreeRegistration.getMoney(), PlatformCashAccountDetailTypeEnum.REFUNDS, "考级退费");
        degreeRegistration.setStatus(3);
        degreeRegistration.setUpdateTime(new Date());
        return degreeRegistrationDao.update(degreeRegistration);
    }
}
