package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.ExtraExercilseQueryInfo;
import com.ym.mec.biz.dal.page.ExtraExercilseReplyQueryInfo;
import com.ym.mec.biz.dal.page.StudentCourseHomeWorkQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ExtracurricularExercisesReplyServiceImpl extends BaseServiceImpl<Long, ExtracurricularExercisesReply> implements ExtracurricularExercisesReplyService {
	
	@Resource
	private ExtracurricularExercisesReplyDao extracurricularExercisesReplyDao;
	@Resource
	private ExtracurricularExercisesDao extracurricularExercisesDao;
	@Resource
	private SysMessageService sysMessageService;
	@Resource
	private TeacherDao teacherDao;
	@Resource
	private StudentDao studentDao;
	@Resource
	private SysConfigDao sysConfigDao;
	@Resource
	private CourseScheduleDao courseScheduleDao;
	@Resource
	private SysMusicScoreDao sysMusicScoreDao;
	@Resource
	private StudentServeService studentServeService;
    @Resource
    private MusicGroupService musicGroupService;
    @Resource
    private ClassGroupDao classGroupDao;
    @Resource
    private CourseHomeworkService courseHomeworkService;
    @Resource
    private StudentLessonTrainingDetailService studentLessonTrainingDetailService;

	@Override
	public BaseDAO<Long, ExtracurricularExercisesReply> getDAO() {
		return extracurricularExercisesReplyDao;
	}

	@Override
	public List<StudentCourseHomework> findExtraExerciseStudents(StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query) {
		Long extraExerciseId = query.getCourseScheduleId().longValue();
		ExtracurricularExercises extracurricularExercises = extracurricularExercisesDao.get(extraExerciseId);
		if(extracurricularExercises == null){
			throw new BizException("课外训练不存在");
		}
		List<ExtraExerciseStudentsDto> extraExerciseStudents = extracurricularExercisesReplyDao.findExtraExerciseStudent(query);
        List<StudentCourseHomework> studentCourseHomeworks=new ArrayList<>();
        for (ExtraExerciseStudentsDto extraExerciseStudent : extraExerciseStudents) {
            StudentCourseHomework studentCourseHomework= JSON.parseObject(JSON.toJSONString(extraExerciseStudent),StudentCourseHomework.class);
            studentCourseHomework.setId(extraExerciseStudent.getId());
            studentCourseHomework.setCourseHomeworkId(extraExerciseId);
            studentCourseHomework.setCourseScheduleId(extraExerciseId);
			studentCourseHomework.setUserId(extraExerciseStudent.getUserId());
			studentCourseHomework.setUserName(extraExerciseStudent.getStudentName());
			studentCourseHomework.setStatus(extraExerciseStudent.getStatus()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
			studentCourseHomework.setIsReplied(extraExerciseStudent.getIsReplied()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
			studentCourseHomework.setIsView(extraExerciseStudent.getIsView()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
			studentCourseHomework.setAvatar(extraExerciseStudent.getHeadUrl());
            studentCourseHomework.setType(ELessonTrainingType.valueOf(extracurricularExercises.getGroupType()));
            studentCourseHomeworks.add(studentCourseHomework);
		}
		return studentCourseHomeworks;
	}

	@Override
	public List<StudentCourseHomework> findExtraExerciseStudents(Long extraExerciseId, String studentName) {
		if(Objects.isNull(extraExerciseId)){
			throw new BizException("请选择课外训练");
		}
		List<ExtraExerciseStudentsDto> extraExerciseStudents = extracurricularExercisesReplyDao.findExtraExerciseStudents(extraExerciseId, studentName);

        ExtracurricularExercises extracurricularExercises = extracurricularExercisesDao.get(extraExerciseId);
        if(extracurricularExercises == null){
            throw new BizException("课外训练不存在");
        }


        // 设置乐团声部
        Map<Integer, StudentSubjectDto> subjectDtoMap= new HashMap<>();
        if (StringUtils.isNotBlank(extracurricularExercises.getMusicGroupId())) {

            // 学生ID集合
            Set<Integer> userIdList = extraExerciseStudents.stream()
                    .map(ExtracurricularExercisesReply::getUserId)
                    .filter(Objects::nonNull)
                    .collect(Collectors.toSet());
            // 设置声部
            subjectDtoMap = studentLessonTrainingDetailService
                    .getStudentMusicGroupSubject(new ArrayList<>(userIdList), extracurricularExercises.getMusicGroupId());
        }

        List<StudentCourseHomework> studentCourseHomeworks=new ArrayList<>();
        for (ExtraExerciseStudentsDto extraExerciseStudent : extraExerciseStudents) {
            StudentCourseHomework studentCourseHomework= JSON.parseObject(JSON.toJSONString(extraExerciseStudent),StudentCourseHomework.class);
            studentCourseHomework.setId(extraExerciseStudent.getId());
            studentCourseHomework.setCourseHomeworkId(extraExerciseId);
            studentCourseHomework.setCourseScheduleId(extraExerciseId);
			studentCourseHomework.setUserId(extraExerciseStudent.getUserId());
			studentCourseHomework.setUserName(extraExerciseStudent.getStudentName());
			studentCourseHomework.setStatus(extraExerciseStudent.getStatus()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
			studentCourseHomework.setIsReplied(extraExerciseStudent.getIsReplied()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
			studentCourseHomework.setIsView(extraExerciseStudent.getIsView()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
			studentCourseHomework.setAvatar(extraExerciseStudent.getHeadUrl());
            studentCourseHomework.setType(ELessonTrainingType.valueOf(extracurricularExercises.getGroupType()));

            StudentSubjectDto studentSubjectDto = subjectDtoMap.get(extraExerciseStudent.getUserId());
            if (studentSubjectDto != null) {
                studentCourseHomework.setSubjectId(studentSubjectDto.getSubjectId());
                studentCourseHomework.setSubjectName(studentSubjectDto.getSubjectName());
            }

            studentCourseHomeworks.add(studentCourseHomework);
		}


		return studentCourseHomeworks;
	}

	@Override
	public CourseHomeworkStudentDetailDto findStudentExtraExerciseDetail(Long studentExerciseId) {
		ExtraExerciseStudentsDto extraExerciseReply = extracurricularExercisesReplyDao.findExtraExerciseReply(studentExerciseId);
		if(extraExerciseReply == null){
			throw new BizException("找不到学生作业记录");
		}
		CourseHomeworkStudentDetailDto detail=new CourseHomeworkStudentDetailDto();
		detail.setStandardFlag(extraExerciseReply.getStandardFlag());
		detail.setCourseHomeworkId(extraExerciseReply.getId());
		detail.setClassGroupName(extraExerciseReply.getTitle() + "•" + extraExerciseReply.getStudentName());
		detail.setContent(extraExerciseReply.getContent());
		detail.setAttachments(extraExerciseReply.getAttachments());
		detail.setStatus(extraExerciseReply.getStatus()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
		detail.setIsReplied(extraExerciseReply.getIsReplied()==0?YesOrNoEnum.NO:YesOrNoEnum.YES);
		detail.setTeacherId(extraExerciseReply.getTeacherId());
		Teacher teacher = teacherDao.get(extraExerciseReply.getTeacherId());
		detail.setTeacherName(teacher.getRealName());
		detail.setStudentId(extraExerciseReply.getUserId());
		detail.setStudentName(extraExerciseReply.getStudentName());
		detail.setExpiryDate(extraExerciseReply.getExpireDate());
		detail.setType("EXTRA");
		detail.setExpire(DateUtil.isOverdue(extraExerciseReply.getCreateTime(),
				sysConfigDao.findConfigValue("file_expiration_time")));
		Date now=new Date();
		if (DateUtil.daysBetween(extraExerciseReply.getHomeworkCreateTime(), now) > 7){
			detail.setSubmitFlag(false);
		}

		String configValue = sysConfigDao.findConfigValue(SysConfigService.HOMEWORK_OPEN_FLAG);
		if(StringUtils.isEmpty(configValue)){
			configValue = "0";
		}
		detail.setHomeworkOpenFlag(Integer.parseInt(configValue));
		if(StringUtils.isNotEmpty(extraExerciseReply.getMusicScoreId())){
			List<SysMusicScore> sysMusicScores = sysMusicScoreDao.findByIds(extraExerciseReply.getMusicScoreId());
			detail.setSysMusicScoreList(sysMusicScores);
			SysMusicScore sysMusicScore = sysMusicScores.get(0);
			detail.setSysMusicScore(sysMusicScore);
			detail.setMusicScoreId(sysMusicScore.getId().toString());
			detail.setMusicScoreName(sysMusicScore.getName());
			detail.setMusicScoreContent(extraExerciseReply.getMusicScoreContent());
		}
		Student student = studentDao.get(extraExerciseReply.getUserId());
		detail.setHasMember(student.getMemberRankSettingId()==null?0:1);
//		}
		return detail;
	}

	@Override
	public PageInfo<ExtraExerciseStudentsDto> findStudentExtraExercises(ExtraExercilseQueryInfo queryInfo) {
		PageInfo<ExtraExerciseStudentsDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<ExtraExerciseStudentsDto> dataList = null;
		int count = extracurricularExercisesReplyDao.countStudentExtraExercises(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = extracurricularExercisesReplyDao.findStudentExtraExercises(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void submitExtraExercise(ExtracurricularExercisesReply extracurricularExercisesReply) {
		if(Objects.isNull(extracurricularExercisesReply.getId())){
			throw new BizException("请选择课外训练");
		}
		if(StringUtils.isBlank(extracurricularExercisesReply.getAttachments())){
			throw new BizException("请先提交作业文件");
		}
		ExtracurricularExercisesReply existExtra = extracurricularExercisesReplyDao.get(extracurricularExercisesReply.getId());
		if(Objects.isNull(existExtra)){
			throw new BizException("作业不存在");
		}
//		boolean push=Objects.isNull(existExtra.getAttachments());
		boolean push=true;
		ExtracurricularExercises extracurricularExercises = extracurricularExercisesDao.get(existExtra.getExtracurricularExercisesId());
		if(Objects.isNull(extracurricularExercises)){
			throw new BizException("作业不存在");
		}
		Date now=new Date();
		if (DateUtil.daysBetween(extracurricularExercises.getCreateTime(), now) > 7){
			throw new BizException("该作业已超出可提交时间范围，无法提交作业。");
		}
		existExtra.setAttachments(extracurricularExercisesReply.getAttachments());
		existExtra.setStatus(1);
		existExtra.setSubmitTime(new Date());
		extracurricularExercisesReplyDao.update(existExtra);

//		LocalDate nowDate = LocalDateTime.now(DateUtil.zoneId).toLocalDate();
//		LocalDate monDayDate = nowDate.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue());
//		LocalDate createDateTime = LocalDateTime.ofInstant(existExtra.getCreateTime().toInstant(), DateUtil.zoneId).toLocalDate();
//		LocalDate createMonday = createDateTime.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue());

		int submitStudentNum=extracurricularExercisesReplyDao.countIsSubmitStudents(existExtra.getExtracurricularExercisesId());
		extracurricularExercises.setCompletedNum(submitStudentNum);
		extracurricularExercisesDao.update(extracurricularExercises);

		studentServeService.updateExercisesSituation(extracurricularExercises.getMusicGroupId(), extracurricularExercises.getCreateTime(), Arrays.asList(existExtra.getUserId()), extracurricularExercises.getTeacherId());

		if(push){
			Map<Integer, String> userMap = new HashMap<>();
			userMap.put(extracurricularExercises.getTeacherId(), extracurricularExercises.getTeacherId().toString());
			SysUser user = teacherDao.getUser(existExtra.getUserId());
			String notifyUrl = "?courseScheduleID=" + existExtra.getId() + "&userId=" + existExtra.getUserId() + "&studentCourseHomeworkId=" + existExtra.getId()+ "&extra=1";
			sysMessageService.batchSendImMessage(MessageTypeEnum.EXTRA_SUBMIT_IM,existExtra.getUserId().toString(),"dayaedu"+notifyUrl,
					new String[]{extracurricularExercises.getTeacherId().toString()},
					null,extracurricularExercises.getTitle());
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_EXTRA_SUBMIT,
					userMap, null, 0, 9+notifyUrl, "TEACHER",
					extracurricularExercises.getTitle(), user.getUsername());
		}
	}

	@Override
	public PageInfo<ExtraExerciseStudentsDto> findExtraExercises(ExtraExercilseReplyQueryInfo queryInfo) {
		PageInfo<ExtraExerciseStudentsDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<ExtraExerciseStudentsDto> dataList = null;
		int count = extracurricularExercisesReplyDao.countExtraExercises(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = extracurricularExercisesReplyDao.findExtraExercises(params);
//			List<Long> replyIds = dataList.stream().map(ExtraExerciseStudentsDto::getId).collect(Collectors.toList());
			List<Integer> teacherIds = dataList.stream().map(ExtraExerciseStudentsDto::getTeacherId).collect(Collectors.toList());
			List<SimpleUserDto> teachers = teacherDao.findSimpleTeachers(teacherIds, queryInfo.getTenantId());
			Map<Integer, String> idNameMap = teachers.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, SimpleUserDto::getUserName));
			for (ExtraExerciseStudentsDto extraExerciseStudentsDto : dataList) {
				extraExerciseStudentsDto.setTeacherName(idNameMap.get(extraExerciseStudentsDto.getTeacherId()));
				if(Objects.nonNull(queryInfo.getExistVipCourse())){
					extraExerciseStudentsDto.setExistVipCourse(queryInfo.getExistVipCourse());
				}
			}
			if(Objects.isNull(queryInfo.getExistVipCourse())){
				List<Integer> studentIds = dataList.stream().map(ExtraExerciseStudentsDto::getUserId).collect(Collectors.toList());
				List<Map<Integer, Long>> studentVipCoursesMaps = courseScheduleDao.countStudentVipCoursesInFuture(studentIds);
				Map<Integer, Long> studentVipCourseMap = MapUtil.convertIntegerMap(studentVipCoursesMaps);
				for (ExtraExerciseStudentsDto exercisesSituationDto : dataList) {
					Long vipCourses=studentVipCourseMap.get(exercisesSituationDto.getUserId());
					if(Objects.isNull(vipCourses)||vipCourses<=0){
						exercisesSituationDto.setExistVipCourse(0);
					}else{
						exercisesSituationDto.setExistVipCourse(1);
					}
				}
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<EduHomeworkCardDto> getEduHomeworkList(StudentCourseHomeWorkQueryInfo queryInfo) {
		PageInfo<EduHomeworkCardDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		int count = extracurricularExercisesReplyDao.countEduHomeworkList(params);
		pageInfo.setTotal(count);
		params.put("offset", pageInfo.getOffset());
		List<EduHomeworkCardDto> dataList = extracurricularExercisesReplyDao.getEduHomeworkList(params);

		pageInfo.setRows(dataList);
		return pageInfo;
	}

    /**
     * 查询学生课外训练情况
     *
     * @param queryInfo
     */
    @Override
    public PageInfo<StudentHomeworkRecordDto> findStudentExtraExercisesV2(ExtraExercilseQueryInfo queryInfo) {
        queryInfo.setVersionTag("v2");
        PageInfo<ExtraExerciseStudentsDto> studentExtraExercises = findStudentExtraExercises(queryInfo);

        // 构建返回数据
        List<ExtraExerciseStudentsDto> rows = studentExtraExercises.getRows();
        if (CollectionUtils.isEmpty(rows)) {
            return new PageInfo<>();
        }
        List<StudentHomeworkRecordDto> recordDtoList = new ArrayList<>();
        for (ExtraExerciseStudentsDto row : rows) {
            StudentHomeworkRecordDto studentHomeworkRecordDto = new StudentHomeworkRecordDto();
            studentHomeworkRecordDto.setStudentCourseHomeworkId(row.getExtracurricularExercisesId().intValue());
            studentHomeworkRecordDto.setTeacherId(row.getTeacherId());
            studentHomeworkRecordDto.setTeacherName(row.getTeacherName());
            studentHomeworkRecordDto.setHeadUrl(row.getHeadUrl());
            studentHomeworkRecordDto.setHeadUrl(row.getHeadUrl());
            studentHomeworkRecordDto.setStatus(row.getStatus());
            studentHomeworkRecordDto.setExpiryDate(row.getExpireDate());
            studentHomeworkRecordDto.setMusicGroupId(row.getMusicGroupId());
            studentHomeworkRecordDto.setClassGroupId(row.getClassGroupId());
            studentHomeworkRecordDto.setType(ELessonTrainingType.valueOf(row.getType()));
            studentHomeworkRecordDto.setSubmitTime(row.getSubmitTime());
            studentHomeworkRecordDto.setStandardFlag(row.getStandardFlag());
            recordDtoList.add(studentHomeworkRecordDto);
        }


        // 构建返回数据

        // 乐团信息

        // 乐团信息
        List<String> musicGroupIds = recordDtoList.stream().map(e -> e.getMusicGroupId())
            .filter(Objects::nonNull).collect(Collectors.toList());
        Map<String, MusicGroup> musicGroupMap = new HashMap<>();

        if (!CollectionUtils.isEmpty(musicGroupIds)) {
            List<MusicGroup> musicGroups = musicGroupService.queryListByIds(new HashSet<>(musicGroupIds));
            musicGroupMap = musicGroups.stream().collect(Collectors.toMap(MusicGroup::getId, Function.identity()));
        }

        // 班级名称

        List<Integer> classIds = recordDtoList.stream().map(e -> e.getClassGroupId())
                                              .filter(Objects::nonNull).collect(Collectors.toList());
        Map<Integer, ClassGroup> classGroupMap = new HashMap<>();

        if (!CollectionUtils.isEmpty(classIds)) {
            List<ClassGroup> classGroups = classGroupDao.queryByIds(classIds);
            classGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, Function.identity()));
        }

        for (StudentHomeworkRecordDto studentHomeworkRecordDto : recordDtoList) {
            // 乐团信息
            MusicGroup musicGroup = musicGroupMap.get(studentHomeworkRecordDto.getMusicGroupId());
            if (Objects.nonNull(musicGroup)) {
                studentHomeworkRecordDto.setMusicGroupName(musicGroup.getName());
                studentHomeworkRecordDto.setMusicGroupImg(musicGroup.getImg());
            }

            // 班级信息
            ClassGroup classGroup = classGroupMap.get(studentHomeworkRecordDto.getClassGroupId());
            if (Objects.nonNull(classGroup)) {
                studentHomeworkRecordDto.setClassGroupName(classGroup.getName());
            }

            // 设置完成状态
            if (studentHomeworkRecordDto.getStandardFlag() == null) {
                studentHomeworkRecordDto.setFinishStatus(false);
            } else
            if (studentHomeworkRecordDto.getStandardFlag().equals(StandardEnum.STANDARD)
                    || studentHomeworkRecordDto.getStandardFlag().equals(StandardEnum.EXCELLENT)){
                studentHomeworkRecordDto.setFinishStatus(true);
            } else {
                studentHomeworkRecordDto.setFinishStatus(false);
            }

            // 设置提交时间
            studentHomeworkRecordDto.setCourseScheduleId(studentHomeworkRecordDto.getStudentCourseHomeworkId().longValue());

        }

        PageInfo<StudentHomeworkRecordDto> pageInfo = new PageInfo<>(studentExtraExercises.getPageNo(),studentExtraExercises.getLimit());
        pageInfo.setRows(recordDtoList);
        pageInfo.setTotal(studentExtraExercises.getTotal());
        return pageInfo;
    }

    @Override
    public CourseHomeworkWrapper.CourseHomeworkList findStudentExtraExerciseDetailV2(Long studentExerciseId) {

        return courseHomeworkService.findCourseExtraHomeworkDetail( studentExerciseId.intValue());


    }
}
