package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.ImSendTypeEnum;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.ExtraExercilseQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ExtracurricularExercisesServiceImpl extends BaseServiceImpl<Long, ExtracurricularExercises> implements ExtracurricularExercisesService {

    @Autowired
    private ExtracurricularExercisesDao extracurricularExercisesDao;
    @Autowired
    private ExtracurricularExercisesReplyDao extracurricularExercisesReplyDao;
    @Autowired
    private StudentExtracurricularExercisesSituationDao studentExtracurricularExercisesSituationDao;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private CourseHomeworkDao courseHomeworkDao;
    @Autowired
    private StudentServeService studentServeService;
    @Autowired
    private StudentCourseHomeworkDao studentCourseHomeworkDao;
    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private StudentLessonTrainingDetailService studentLessonTrainingDetailService;

    @Override
    public BaseDAO<Long, ExtracurricularExercises> getDAO() {
        return extracurricularExercisesDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public void createExtraExercises(ExtracurricularExercises exercises) {
        if (exercises.getMusicGroupId() != null) {
            exercises.setGroupType(ELessonTrainingType.EXTRACURRICULAR.getCode());
        } else {
            exercises.setGroupType(ELessonTrainingType.EXTRA.getCode());
        }
        List<MusicScoreSubjectDto> scoreSubjectDtoList = exercises.getMusicScoreSubjectDtos();
        if(StringUtils.isBlank(exercises.getStudentIdList())){
            if(CollectionUtils.isEmpty(scoreSubjectDtoList)){
                throw new BizException("请指定学生");
            }
            List<Integer> studentIdList = new ArrayList<>();
            for (MusicScoreSubjectDto musicScoreSubjectDto : scoreSubjectDtoList) {
                studentIdList.addAll(musicScoreSubjectDto.getUserIdList());
            }
            exercises.setStudentIdList(StringUtils.join(studentIdList,","));
        }

        teacherDao.getLocked(exercises.getTeacherId());
        Teacher teacher = teacherDao.get(exercises.getTeacherId());

        ExtracurricularExercises repeatLastExercises = extracurricularExercisesDao.findRepeatLastExercises(exercises.getTeacherId(), exercises.getStudentIdList(), exercises.getContent());
        if(Objects.nonNull(repeatLastExercises)&&DateUtil.secondsBetween(repeatLastExercises.getCreateTime(),new Date())<60){
            return;
        }

        if(Objects.isNull(exercises.getExpireDate())){
            LocalDateTime localDateTime=LocalDateTime.now();
            localDateTime=localDateTime.plusDays(Integer.parseInt(sysConfigService.findByParamName("homework_expire_time").getParanValue()))
                    .withHour(21).withMinute(0).withSecond(0).withNano(0);
            exercises.setExpireDate(Date.from(localDateTime.atZone(DateUtil.zoneId).toInstant()));
        }
        List<Integer> studentIds = Arrays.asList(exercises.getStudentIdList().split(",")).stream().mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
        exercises.setBatchNo(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")));
        exercises.setExpectNum(studentIds.size());
        extracurricularExercisesDao.insert(exercises);
        String dateStr = DateUtil.dateToString(exercises.getExpireDate(), "MM月dd日HH点");

        ExtracurricularExercisesReply studentExtraExercise=new ExtracurricularExercisesReply();
        studentExtraExercise.setExtracurricularExercisesId(exercises.getId());
        studentExtraExercise.setStatus(0);
        studentExtraExercise.setIsReplied(0);
        studentExtraExercise.setIsView(0);
        studentExtraExercise.setIsRepliedTimely(0);
        studentExtraExercise.setMusicScoreId(exercises.getMusicScoreId());

        List<ExtracurricularExercisesReply> extracurricularExercisesReplies = new ArrayList<>();

        List<StudentLessonTrainingDetail> studentLessonTrainingDetailList = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(scoreSubjectDtoList)){
            for (MusicScoreSubjectDto musicScoreSubjectDto : scoreSubjectDtoList) {
                // 原本作业逻辑
                for (Integer userId : musicScoreSubjectDto.getUserIdList()) {
                    studentExtraExercise.setUserId(userId);
                    studentExtraExercise.setMusicScoreId(StringUtils.join(musicScoreSubjectDto.getMusicScoreIdList(),","));
                    studentExtraExercise.setMusicScoreContent(JSONObject.toJSONString(musicScoreSubjectDto.getMusicScoreHomeworkDtoList()));
                    extracurricularExercisesReplies.add(JSON.parseObject(JSON.toJSONString(studentExtraExercise),ExtracurricularExercisesReply.class));
                }
                if (CollectionUtils.isEmpty(musicScoreSubjectDto.getStudentLessonTrainingDetails())) {
                    throw new BizException("有部分声部未选择曲目");
                }
                for (StudentLessonTrainingDetailWrapper.AddStudentLessonTrainingDetail studentLessonTrainingDetail : musicScoreSubjectDto.getStudentLessonTrainingDetails()) {
                    studentLessonTrainingDetail.setSubjectId(0);
                }
                // 作业详情
                studentLessonTrainingDetailList.addAll(studentLessonTrainingDetailService
                                   .homeWorkDetail(musicScoreSubjectDto.getStudentLessonTrainingDetails(), musicScoreSubjectDto.getUserIdList(), exercises.getId(),
                                                   ELessonTrainingType.valueOf(exercises.getGroupType()),musicScoreSubjectDto.getGroup()));

            }
        } else {
            for (Integer studentId : studentIds) {
                studentExtraExercise.setUserId(studentId);
                extracurricularExercisesReplies.add(JSON.parseObject(JSON.toJSONString(studentExtraExercise),ExtracurricularExercisesReply.class));

            }
            // 作业详情
            studentLessonTrainingDetailList.addAll(studentLessonTrainingDetailService
                                                       .homeWorkDetail(exercises.getStudentLessonTrainingDetails(), studentIds,exercises.getId(),
                                                       ELessonTrainingType.valueOf(exercises.getGroupType()), null));
        }
        for (ExtracurricularExercisesReply extracurricularExercisesReply : extracurricularExercisesReplies) {
            extracurricularExercisesReplyDao.insert(extracurricularExercisesReply);
        }
        if (CollectionUtils.isNotEmpty(studentLessonTrainingDetailList)) {
            // 学生ID集合
            Set<Integer> userIdList = studentLessonTrainingDetailList.stream()
                    .map(StudentLessonTrainingDetail::getUserId)
                    .filter(Objects::nonNull)
                    .map(Long::intValue)
                    .collect(Collectors.toSet());
            // 设置声部
            Map<Integer, StudentSubjectDto> subjectDtoMap = studentLessonTrainingDetailService
                    .getStudentMusicGroupSubject(new ArrayList<>(userIdList), exercises.getMusicGroupId());

            studentLessonTrainingDetailList.forEach(studentLessonTrainingDetail -> {
                if (studentLessonTrainingDetail.getUserId() == null) {
                    return;
                }
                StudentSubjectDto studentSubjectDto = subjectDtoMap.get(studentLessonTrainingDetail.getUserId().intValue());
                if (Objects.nonNull(studentSubjectDto)) {
                    studentLessonTrainingDetail.setSubjectId(studentSubjectDto.getSubjectId().longValue());
                }
            });
            studentLessonTrainingDetailService.saveBatch(studentLessonTrainingDetailList);
            exercises.setVersionTag("v2");
        } else {
            exercises.setVersionTag("v1");
        }
        extracurricularExercisesDao.update(exercises);

        for (ExtracurricularExercisesReply extracurricularExercisesReply : extracurricularExercisesReplies) {
            Integer studentId = extracurricularExercisesReply.getUserId();
            if (exercises.getVersionTag().equals("v2")) {

                String notifyUrl = "homeworkId=" + exercises.getId() + "&homeworkType=" +exercises.getGroupType();
                String extra = "dayaedu?type=homework&" + notifyUrl ;
                sysMessageService.batchSendImMessage(MessageTypeEnum.EXTRA_REMIND_IM_DETAIL_V2, exercises.getTeacherId().toString(), extra,
                        new String[]{studentId.toString()},
                        null, dateStr);

                Map<Integer, String> userMap = new HashMap<>();
                userMap.put(studentId, studentId.toString());
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_EXTRA_REMIND_V2,
                        userMap, null, 0, "homework?" + notifyUrl, "STUDENT",
                        teacher.getRealName());
            } else {

                String notifyUrl = "?courseScheduleID=" + extracurricularExercisesReply.getId() + "&studentCourseHomeworkId=" + extracurricularExercisesReply.getId() + "&extra=1";
                String extra = "dayaedu" + notifyUrl + "&userId=" + studentId;
                if (exercises.getMsgType() != ImSendTypeEnum.TXT) {
                    sysMessageService.batchSendImMessage(MessageTypeEnum.EXTRA_REMIND_IM_DETAIL, exercises.getTeacherId().toString(), extra,
                            new String[]{studentId.toString()},
                            null, exercises.getTitle(), dateStr);
                } else {
                    sysMessageService.batchSendImMessage(MessageTypeEnum.EXTRA_REMIND_IM, exercises.getTeacherId().toString(), extra,
                            new String[]{studentId.toString()},
                            null, exercises.getTitle(), dateStr, exercises.getContent());
                }
                Map<Integer, String> userMap = new HashMap<>();
                userMap.put(studentId, studentId.toString());
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_EXTRA_REMIND,
                        userMap, null, 0, 3 + notifyUrl, "STUDENT",
                        teacher.getRealName(), exercises.getTitle(), dateStr);
            }
        }
        studentServeService.updateExercisesSituation(exercises.getMusicGroupId(), new Date(), studentIds, exercises.getTeacherId());
    }

    @Override
    public List<Map<String, Object>> findExtraExercilses(ExtraExercilseQueryInfo queryInfo) {
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<Map<String, Object>> result = new ArrayList<>();

        List<ExtracurricularExercises> dataList = extracurricularExercisesDao.findExtraExercises(params);
        if(!CollectionUtils.isEmpty(dataList)){
            List<Long> extraExerciseIds = dataList.stream().map(ExtracurricularExercises::getId).collect(Collectors.toList());
            List<ExtracurricularExercisesReply> extraExerciseStudentsByExtraExercises = extracurricularExercisesReplyDao.findExtraExerciseStudentsByExtraExercises(extraExerciseIds);
            Map<Long, List<ExtracurricularExercisesReply>> idStudentsMap = extraExerciseStudentsByExtraExercises.stream().collect(Collectors.groupingBy(ExtracurricularExercisesReply::getExtracurricularExercisesId));
            for (ExtracurricularExercises extracurricularExercises : dataList) {
                List<ExtracurricularExercisesReply> extracurricularExercisesReplies = idStudentsMap.get(extracurricularExercises.getId());
                if(CollectionUtils.isEmpty(extracurricularExercisesReplies)){
                    continue;
                }
                long noRepliedNum = extracurricularExercisesReplies.stream().filter(e -> e.getStatus()==1&&e.getIsReplied() == 0).count();
                if(noRepliedNum<=0){
                    extracurricularExercises.setIsReplied(1);
                }
                long notSubmitNum = extracurricularExercisesReplies.stream().filter(e -> e.getStatus() == 0).count();
                if(notSubmitNum<=0){
                    extracurricularExercises.setIsSubmit(1);
                }
                if(noRepliedNum>0){
                    extracurricularExercises.setStatus(1);
                }else if(noRepliedNum<=0&&notSubmitNum>0){
                    extracurricularExercises.setStatus(2);
                }else if(noRepliedNum<=0&&notSubmitNum<=0){
                    extracurricularExercises.setStatus(3);
                }
            }

            Map<String, List<ExtracurricularExercises>> collect = dataList.stream().collect(Collectors.groupingBy(e -> DateUtil.dateToString(e.getCreateTime(), "yyyy-MM-dd")));
            List<Date> dates=new ArrayList<>();
            collect.keySet().forEach(ds-> dates.add(DateUtil.stringToDate(ds,"yyyy-MM-dd")));
            dates.sort(Comparator.comparing(Date::getTime));
            dates.sort(Comparator.reverseOrder());
            for (Date date : dates) {
                String key=DateUtil.dateToString(date, "yyyy-MM-dd");
                Map<String, Object> r = new HashMap<>();
                r.put("day", key);
                collect.get(key).sort(Comparator.comparing(ExtracurricularExercises::getCreateTime).reversed());
                r.put("list", collect.get(key));
                result.add(r);
            }
        }
        return result;
    }

    @Override
    public List<BasicUserDto> getDontServeStudents(Integer teacherId,
                                                   String search,
                                                   String musicGroupId,
                                                   Long classGroupId,
                                                   Integer subjectId,
                                                   Integer hasMember,
                                                   String studentIds) {
        LocalDate nowDate = LocalDateTime.now(DateUtil.zoneId).toLocalDate();
        LocalDate monDayDate = nowDate.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue());
        return extracurricularExercisesDao.findNoExercisesStudentsInThisWeekWithTeacher(teacherId, monDayDate.toString(), musicGroupId, classGroupId, subjectId, search,hasMember,studentIds);
    }

    @Override
    public void studentServeRemind() {
        LocalDate nowDate = LocalDateTime.now(DateUtil.zoneId).toLocalDate();
        LocalDate monDayDate = nowDate.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue());
        List<Map<Integer, Long>> noFinishedServiceTeacher = studentExtracurricularExercisesSituationDao.getNoFinishedServiceTeacher(monDayDate.toString());
        Map<Integer, Long> noFinishedServiceTeacherMap= MapUtil.convertIntegerMap(noFinishedServiceTeacher);
        for (Map.Entry<Integer, Long> integerLongEntry : noFinishedServiceTeacherMap.entrySet()) {
            Map<Integer, String> userMap = new HashMap<>();
            userMap.put(integerLongEntry.getKey(), integerLongEntry.getKey().toString());
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_PUBLIC_EXTRA_REMIND,
                    userMap, null, 0, "11" , "TEACHER", integerLongEntry.getValue());
        }
    }

    @Override
    public PageInfo<TeacherHomeworkListDto> findExtraExercilsesHomeworks(ExtraExercilseQueryInfo queryInfo) {
        PageInfo<TeacherHomeworkListDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        int count = extracurricularExercisesDao.countExtraExercilsesHomeworks(params);
        List<TeacherHomeworkListDto> dataList = new ArrayList<>();
        if(count > 0){
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            List<ExtracurricularExercisesHomeworkListDto> homeworksExercises = extracurricularExercisesDao.findExtraExercilsesHomeworks(params);
            if(homeworksExercises == null || homeworksExercises.size() == 0){
                return pageInfo;
            }
            //课外训练
            List<ExtracurricularExercisesHomeworkListDto> exercises = homeworksExercises.stream().filter(e -> e.getType().equals("EXERCISES")).collect(Collectors.toList());
            if(exercises.size() > 0){
                List<Integer> exercisesIdList = exercises.stream().map(e -> e.getHomeworkId()).collect(Collectors.toList());
                List<TeacherHomeworkListDto> byIdList = extracurricularExercisesDao.findByIdList(exercisesIdList);
                if(byIdList != null && byIdList.size() > 0){
                    List<Integer> extraExerciseIds = byIdList.stream().map(TeacherHomeworkListDto::getHomeworkId).collect(Collectors.toList());
                    List<ExtracurricularExercisesReply> extraExerciseStudentsByExtraExercises = extracurricularExercisesReplyDao.findExtraExerciseStudentsByExtraExercises(extraExerciseIds);
                    Map<Long, List<ExtracurricularExercisesReply>> idStudentsMap = extraExerciseStudentsByExtraExercises.stream().collect(Collectors.groupingBy(ExtracurricularExercisesReply::getExtracurricularExercisesId));
                    for (TeacherHomeworkListDto homeworkListDto : byIdList) {
                        List<ExtracurricularExercisesReply> extracurricularExercisesReplies = idStudentsMap.get(homeworkListDto.getHomeworkId().longValue());
                        if(CollectionUtils.isEmpty(extracurricularExercisesReplies)){
                            continue;
                        }
                        long noRepliedNum = extracurricularExercisesReplies.stream().filter(e -> e.getStatus()==1&&e.getIsReplied() == 0).count();
                        if(noRepliedNum<=0){
                            homeworkListDto.setIsReplied(1);
                        }
                        long notSubmitNum = extracurricularExercisesReplies.stream().filter(e -> e.getStatus() == 0).count();
                        if(notSubmitNum<=0){
                            homeworkListDto.setIsSubmit(1);
                        }
                        if(noRepliedNum>0){
                            homeworkListDto.setStatus(1);
                        }else if(noRepliedNum<=0&&notSubmitNum>0){
                            homeworkListDto.setStatus(2);
                        }else if(noRepliedNum<=0&&notSubmitNum<=0){
                            homeworkListDto.setStatus(3);
                        }
                    }
                    dataList.addAll(byIdList);
                }
            }
            //课后作业
            List<ExtracurricularExercisesHomeworkListDto> homeworkList = homeworksExercises.stream().filter(e -> e.getType().equals("HOMEWORK")).collect(Collectors.toList());
            if(homeworkList.size() > 0){
                List<Integer> homeworkIdList = homeworkList.stream().map(e -> e.getHomeworkId()).collect(Collectors.toList());
                List<TeacherHomeworkListDto> byIdList = courseHomeworkDao.findByIdList(homeworkIdList);
                if(byIdList != null && byIdList.size() > 0){
                    List<Long> courseIds = byIdList.stream().mapToLong(TeacherHomeworkListDto::getCourseScheduleId).boxed().collect(Collectors.toList());
                    List<StudentCourseHomework> allStudentCourseHomeworks = studentCourseHomeworkDao.findByCourses(courseIds);
                    Map<Long, List<StudentCourseHomework>> homeworkStudentMap = allStudentCourseHomeworks.stream().collect(Collectors.groupingBy(StudentCourseHomework::getCourseScheduleId));
                    for (TeacherHomeworkListDto teacherHomeworkListDto : byIdList) {
                        List<StudentCourseHomework> studentCourseHomeworks = homeworkStudentMap.get(teacherHomeworkListDto.getCourseScheduleId().longValue());
                        if(CollectionUtils.isEmpty(studentCourseHomeworks)){
                            continue;
                        }
                        long noRepliedNum=studentCourseHomeworks.stream().filter(e -> YesOrNoEnum.YES.equals(e.getStatus())&&YesOrNoEnum.NO.equals(e.getIsReplied())).count();
                        if(noRepliedNum<=0){
                            teacherHomeworkListDto.setIsReplied(YesOrNoEnum.YES.getCode());
                        }
                        long notSubmitNum = studentCourseHomeworks.stream().filter(e -> e.getStatus().equals(YesOrNoEnum.NO)).count();
                        if(notSubmitNum<=0){
                            teacherHomeworkListDto.setIsSubmit(1);
                        }
                        if(noRepliedNum>0){
                            teacherHomeworkListDto.setStatus(1);
                        }else if(noRepliedNum<=0&&notSubmitNum>0){
                            teacherHomeworkListDto.setStatus(2);
                        }else if(noRepliedNum<=0&&notSubmitNum<=0){
                            teacherHomeworkListDto.setStatus(3);
                        }
                    }
                    dataList.addAll(byIdList);
                }
            }
            dataList.removeAll(Collections.singleton(null));
            int startRow = queryInfo.getPage() * queryInfo.getRows() - queryInfo.getRows();
            dataList = dataList.stream().sorted(Comparator.comparing(TeacherHomeworkListDto::getDay).reversed()).
                    sorted(Comparator.comparing(TeacherHomeworkListDto::getStatus))
                    .skip(startRow).limit(queryInfo.getRows()).
                    collect(Collectors.toList());
            pageInfo.setRows(dataList);
        }
        return pageInfo;
    }

    @Override
    public PageInfo<CourseHomeworkWrapper.WebExtraCourseHomework> findExtraExercilsesV2(ExtraExercilseQueryInfo queryInfo) {
        PageInfo<ExtracurricularExercises> extracurricularExercisesPageInfo = this.queryPage(queryInfo);
        List<ExtracurricularExercises> rows = extracurricularExercisesPageInfo.getRows();
        if (CollectionUtils.isEmpty(rows)) {
            return new PageInfo<>();
        }

        PageInfo<CourseHomeworkWrapper.WebExtraCourseHomework> convert = extracurricularExercisesPageInfo.convert(
            o -> JSON.parseObject(JSON.toJSONString(o), CourseHomeworkWrapper.WebExtraCourseHomework.class));

        List<CourseHomeworkWrapper.WebExtraCourseHomework> rows1 = convert.getRows();

        // 设置班级名称
        List<Integer> classIds = rows1.stream().map(CourseHomeworkWrapper.WebExtraCourseHomework::getClassGroupId).filter(Objects::nonNull).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(classIds)) {
            return convert;
        }

        List<ClassGroup> classGroups = classGroupDao.queryByIds(classIds);

        // 分组
        Map<Integer, ClassGroup> classGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, Function.identity()));

        rows1.forEach(o -> {
            if (o.getClassGroupId() == null) {
                return;
            }
            ClassGroup classGroup = classGroupMap.get(o.getClassGroupId());
            if (classGroup != null) {
                o.setClassGroupName(classGroup.getName());
            }
        });

        return convert;
    }
}
