package com.ym.mec.biz.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.ym.mec.biz.dal.dao.FinancialExpenditureDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.FinancialExpenditure;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.enums.ExpenditureTypeEnum;
import com.ym.mec.biz.dal.enums.FeeProjectEnum;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.TemplateTypeEnum;
import com.ym.mec.biz.dal.page.FinancialExpenditureQueryInfo;
import com.ym.mec.biz.service.FinancialExpenditureService;
import com.ym.mec.biz.service.GroupClassService;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.biz.service.VipGroupService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import com.ym.mec.util.ini.IniFileUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
public class FinancialExpenditureServiceImpl extends BaseServiceImpl<Long, FinancialExpenditure> implements FinancialExpenditureService {

    @Autowired
    private FinancialExpenditureDao financialExpenditureDao;
    @Autowired
    private OrganizationDao organizationDao;
    @Autowired
    private VipGroupService vipGroupService;
    @Autowired
    private GroupClassService groupClassService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private FinancialExpenditureService financialExpenditureService;
    @Autowired
    private RedissonClient redissonClient;

    @Override
    public BaseDAO<Long, FinancialExpenditure> getDAO() {
        return financialExpenditureDao;
    }

    private static final Logger businessLogger = LoggerFactory.getLogger(FinancialExpenditureServiceImpl.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<FinancialExpenditure> importFinancialExpenditure(MultipartFile file) throws Exception {
        Map<String, List<Map<String, Object>>> sheetsListMap = POIUtil.importExcel(new ByteArrayInputStream(file.getBytes()), 2, file.getOriginalFilename());
        InputStream inputStream = new ClassPathResource("columnMapper.ini").getInputStream();
        Map<String, String> columns = IniFileUtil.readIniFile(inputStream, TemplateTypeEnum.FINANCIAL_EXPENDITURE.getMsg());
        List<FinancialExpenditure> financialExpenditures = new ArrayList<>();
        Map<String, Integer> organMap = getMap("organization", "name_", "id_", true, TenantContextHolder.getTenantId(), String.class, Integer.class);
        Map<String, Integer> cooperationOrganMap = getMap("cooperation_organ", "name_", "id_", true, TenantContextHolder.getTenantId(), String.class, Integer.class);
        StringBuffer sb = new StringBuffer();
        for (String e : sheetsListMap.keySet()) {
            List<Map<String, Object>> sheet = sheetsListMap.get(e);
            List<Object> collect = sheet.stream().map(m -> m.get("钉钉流程编号")).collect(Collectors.toList());
            if (collect == null || collect.size() == 0) {
                continue;
            }
            valueIsNull:
            for (int j = 0; j < sheet.size(); j++) {
                int rowNum = j + 2;
                Map<String, Object> row = sheet.get(j);
                if (row.size() == 0) {
                    continue;
                }
                JSONObject objectMap = new JSONObject();
                for (String s : row.keySet()) {
                    if (!columns.containsKey(s)) {
                        continue;
                    }
                    String columnValue = columns.get(s);
                    if (columnValue.equals("type")) {
                        for (ExpenditureTypeEnum expenditureType : ExpenditureTypeEnum.values()) {
                            if (expenditureType.getDesc().equals(row.get(s).toString())) {
                                objectMap.put(columnValue, expenditureType);
                                break;
                            }
                        }
                        continue;
                    }
                    if (columnValue.equals("dingtalkProcessNo")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：钉钉流程编号不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("incomeType", row.get(s));
                        }
                    }
                    if (columnValue.equals("feeProject")) {
                        for (FeeProjectEnum feeProject : FeeProjectEnum.values()) {
                            if (feeProject.getDesc().equals(row.get(s).toString())) {
                                objectMap.put(columnValue, feeProject);
                                break;
                            }
                        }
                        continue;
                    }
                    if (columnValue.equals("amount")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：费用不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("amount", row.get(s));
                        }
                        continue;
                    }
                    if (columnValue.equals("organName") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        Integer integer = organMap.get(row.get(s));
                        if (integer == null) {
                            sb.append("第" + rowNum).append("行数据导入失败：分部校验失败;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("organId", integer);
                        }
                        continue;
                    }
                    if (columnValue.equals("cooperationOrganName") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        Integer integer = cooperationOrganMap.get(row.get(s));
                        if (integer == null) {
                            sb.append("第" + rowNum).append("行数据导入失败：合作单位校验失败;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("cooperationOrganId", integer);
                        }
                        continue;
                    }
                    if (columnValue.equals("paymentTime") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        String toString = row.get(s).toString();
                        if (StringUtils.isNotEmpty(toString)) {
                            Date date = DateUtil.stringToDate(toString, DateUtil.DEFAULT_PATTERN);
                            if (date == null) {
                                throw new BizException("导入数据错误  付款时间格式错误");
                            } else {
                                objectMap.put(columnValue, DateUtil.format(date, DateUtil.DEFAULT_PATTERN));
                            }
                        }
                        continue;
                    }
                    objectMap.put(columnValue, row.get(s));
                }
                FinancialExpenditure financialExpenditure = null;
                try {
                    financialExpenditure = JSONObject.parseObject(objectMap.toJSONString(), FinancialExpenditure.class);
                    financialExpenditures.add(financialExpenditure);
                } catch (Exception ex) {
                    throw new BizException("导入数据出错,请检查Excel表格");
                }
            }
        }
        if (sb.length() > 0) {
            throw new BizException(sb.toString());
        }
        if (financialExpenditures.size() != 0) {
            financialExpenditureDao.batchInsert(financialExpenditures);
        }
        return financialExpenditures;
    }

    @Override
    public PageInfo<FinancialExpenditureDto> queryFinancialExpenditurePage(FinancialExpenditureQueryInfo queryInfo) {
        PageInfo<FinancialExpenditureDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<FinancialExpenditureDto> dataList = null;
        int count = financialExpenditureDao.findFinancialExpenditureCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = financialExpenditureDao.queryFinancialExpenditurePage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public void batchDel(String ids) {
        if (StringUtils.isEmpty(ids)) {
            throw new BizException("参数校验失败");
        }
        financialExpenditureDao.batchDel(ids);
    }

    @Override
    public List<FinancialExpenditure> batchAdd(List<FinancialExpenditure> financialExpenditures) {
        financialExpenditureDao.batchInsert(financialExpenditures);
        return financialExpenditures;
    }

    //需要处理的oa模板id
    Set<Integer> processValuesToCheck = new HashSet<>(Arrays.asList(33, 22, 19, 51, 28));

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void syncOaPayLog(Integer workOrderId,String fileUrl) throws Exception {
        businessLogger.info("syncOaPayLog--- workOrderId:{},fileUrl:{}",workOrderId,fileUrl);
        //防止幂等
        String key = "syncOaPayLog:" + workOrderId;
        RBucket<Object> bucket = redissonClient.getBucket(key);
        if (!bucket.trySet(workOrderId, 2L, TimeUnit.SECONDS)) {
            throw new BizException("正在同步oa审批记录，请勿重复操作");
        }
        PWorkOrderInfo pWorkOrderInfo = financialExpenditureDao.getWorkOrderInfo(workOrderId);
        if (pWorkOrderInfo != null){
            Integer hasFinancial = financialExpenditureDao.findByBatchNoAndProcessNo(workOrderId);
            if(hasFinancial != null){
                return;
            }
            Date endTime = pWorkOrderInfo.getUpdateTime();
            String realName = "";
            if(pWorkOrderInfo.getCreator() != null){
                SimpleUserDto simpleUser = teacherDao.getSimpleUser(pWorkOrderInfo.getCreator());
                if(Objects.nonNull(simpleUser)){
                    realName = simpleUser.getRealName();
                }
            }
            //获取模板
            String formStructure = financialExpenditureDao.getFormStructure(workOrderId);
            //获取用户填写的表单
            JSONObject formData = JSONObject.parseObject(financialExpenditureDao.getFormData(workOrderId));
            OaFormStructureDto oaFormStructureDto = JSONObject.parseObject(formStructure, OaFormStructureDto.class);
            List<OaInputDto> oaInputDtos = oaFormStructureDto.getList();
            //课程退费和乐团退费需要单独处理
            if(processValuesToCheck.contains(pWorkOrderInfo.getProcess())){
                List<FinancialExpenditure> financialExpenditureList = new ArrayList<>();
                Integer organId = null;
                String note = "";
                FeeProjectEnum feeProjectEnum = FeeProjectEnum.REFUND;
                BigDecimal totalAmount = BigDecimal.ZERO;

                for (OaInputDto oaInputDto : oaInputDtos) {
                    String submitForm = oaInputDto.getModel();
                    Object submitValue = formData.get(submitForm);
                    if (Objects.isNull(submitValue) || StringUtils.isEmpty(submitValue.toString())) {
                        continue;
                    }
                    if (StringUtils.equals("分部", oaInputDto.getName())) {
                        organId = Integer.parseInt(submitValue.toString());
                    }
                    if (StringUtils.equals("情况说明", oaInputDto.getName())) {
                        note = submitValue.toString();
                    }
                    if (oaInputDto.getName().contains("金额")) {
                        totalAmount = new BigDecimal(submitValue.toString());
                    }
                    if (StringUtils.equals("费用类别", oaInputDto.getName()) || StringUtils.equals("费用类型", oaInputDto.getName())) {
                        feeProjectEnum = FeeProjectEnum.valueOfDesc(submitValue.toString());
                    }
                }
                Boolean continueFlag = false;
                for (OaInputDto oaInputDto : oaInputDtos) {
                    String submitForm = oaInputDto.getModel();
                    Object submitValue = formData.get(submitForm);
                    if(Objects.isNull(submitValue)){
                        continue;
                    }
                    if(!StringUtils.equals("子表单",oaInputDto.getName()) && !StringUtils.equals("分摊明细",oaInputDto.getName())){
                        continue;
                    }
                    List<HashMap> hashMaps = JSONObject.parseArray(formData.get(submitForm).toString(), HashMap.class);
                    if(StringUtils.equals("分摊明细",oaInputDto.getName())){
                        //处理房租的分摊明细
                        nullAmount:
                        for (HashMap hashMap : hashMaps) {
                            List<OaColumnDto> columns = oaInputDto.getColumns();
                            if (columns != null && columns.size() > 0) {
                                FinancialExpenditure financialExpenditure = new FinancialExpenditure();
                                for (OaColumnDto column : columns) {
                                    List<OaInputDto> columnList = column.getList();
                                    if (columnList != null && columnList.size() > 0) {
                                        for (OaInputDto inputDto : columnList) {
                                            String name = inputDto.getName();
                                            Object o = hashMap.get(inputDto.getModel());
                                            if (Objects.isNull(o) || StringUtils.isEmpty(o.toString())) {
                                                continueFlag = false;
                                                continue nullAmount;
                                            }
                                            continueFlag = true;
                                            if (StringUtils.equals(name,"金额")) {
                                                financialExpenditure.setCurrentAmount(new BigDecimal(Double.parseDouble(o.toString())));
                                            } else if (StringUtils.equals(name,"日期")) {
                                                financialExpenditure.setEffectiveTime(o.toString());
                                            }
                                        }
                                    }
                                }
                                financialExpenditure.setFeeProject(feeProjectEnum);
                                financialExpenditure.setType(getExpenditureType(feeProjectEnum));
                                financialExpenditure.setAprovalTime(endTime);
                                financialExpenditure.setPaymentTime(endTime);
                                financialExpenditure.setAmount(totalAmount);
                                financialExpenditure.setCause(note);
                                financialExpenditure.setBatchNo(pWorkOrderInfo.getId().toString());
                                financialExpenditure.setFinancialProcessNo(pWorkOrderInfo.getId().toString());
                                financialExpenditure.setApplyUser(realName);
                                financialExpenditure.setApplyUserId(pWorkOrderInfo.getCreator());
                                financialExpenditure.setProcessId(pWorkOrderInfo.getProcess());
                                financialExpenditure.setOrganId(organId);
                                financialExpenditureList.add(financialExpenditure);
                            }
                        }
                    }else {
                        if(continueFlag){
                            continue;
                        }
                        nullUser:
                        for (HashMap hashMap : hashMaps) {
                            FinancialExpenditure financialExpenditure = new FinancialExpenditure();
                            financialExpenditure.setAprovalTime(endTime);
                            financialExpenditure.setBatchNo(pWorkOrderInfo.getId().toString());
                            financialExpenditure.setFinancialProcessNo(pWorkOrderInfo.getId().toString());
                            financialExpenditure.setApplyUser(realName);
                            financialExpenditure.setApplyUserId(pWorkOrderInfo.getCreator());
                            financialExpenditure.setProcessId(pWorkOrderInfo.getProcess());
                            financialExpenditure.setEffectiveTime(DateUtil.format(endTime,DateUtil.ISO_YEAR_MONTH_FORMAT));
                            financialExpenditure.setFeeProject(feeProjectEnum);
                            financialExpenditure.setType(getExpenditureType(feeProjectEnum));
                            //解析用户录入
                            this.setFinancialExpenditure(oaInputDto,hashMap,financialExpenditure);
                            if(StringUtils.isEmpty(financialExpenditure.getMusicGroupId()) && financialExpenditure.getVipGroupId() == null){
                                continue ;
                            }
                            if(financialExpenditure.getStudentId() == null){
                                continue ;
                            }
                            if(financialExpenditure.getPaymentTime() == null){
                                financialExpenditure.setPaymentTime(endTime);
                            }
                            if(StringUtils.isEmpty(financialExpenditure.getEffectiveTime())){
                                financialExpenditure.setEffectiveTime(DateUtil.format(financialExpenditure.getPaymentTime(),DateUtil.ISO_YEAR_MONTH_FORMAT));
                            }
                            financialExpenditureList.add(financialExpenditure);
                        }
                    }
                }
                //处理业务退费（vip退学、乐团退费.....）
                this.processBusinessReturns(pWorkOrderInfo,financialExpenditureList,realName,fileUrl);
                if(CollectionUtils.isEmpty(financialExpenditureList)){
                    businessLogger.error("oa业务流程处理失败，请检查");
                    return;
                }
                financialExpenditureDao.batchInsert(financialExpenditureList);
            }else {
                FinancialExpenditure financialExpenditure = new FinancialExpenditure();
                financialExpenditure.setAprovalTime(endTime);
                financialExpenditure.setPaymentTime(endTime);
                financialExpenditure.setBatchNo(workOrderId.toString());
                financialExpenditure.setFinancialProcessNo(workOrderId.toString());
                financialExpenditure.setApplyUser(realName);
                financialExpenditure.setApplyUserId(pWorkOrderInfo.getCreator());
                financialExpenditure.setProcessId(pWorkOrderInfo.getProcess());
                financialExpenditure.setEffectiveTime(DateUtil.format(endTime,DateUtil.ISO_YEAR_MONTH_FORMAT));
                Integer organId = financialExpenditureDao.getDeptId(pWorkOrderInfo.getDeptId());
                Organization organization = organizationDao.get(organId);
                if(organization != null){
                    financialExpenditure.setOrganId(organization.getId());
                    financialExpenditure.setOrganName(organization.getName());
                }
                for (OaInputDto oaInputDto : oaInputDtos) {
                    String name = oaInputDto.getName();
                    Object o = formData.get(oaInputDto.getModel());
                    if(Objects.isNull(o) || StringUtils.isEmpty(o.toString())){
                        continue;
                    }
                    if(StringUtils.contains(name,"金额")){
                        financialExpenditure.setAmount(new BigDecimal(Double.parseDouble(o.toString())));
                        financialExpenditure.setCurrentAmount(financialExpenditure.getAmount());
                    }else if(StringUtils.equals(name,"情况说明")){
                        financialExpenditure.setCause(o.toString());
                    }else if(StringUtils.equals(name,"费用类型") || StringUtils.equals(name,"费用类别")){
                        FeeProjectEnum feeProjectEnum = FeeProjectEnum.valueOfDesc(o.toString());
                        if(feeProjectEnum != null){
                            financialExpenditure.setFeeProject(feeProjectEnum);
                            financialExpenditure.setType(getExpenditureType(financialExpenditure.getFeeProject()));
                        }
                    }
                }
                financialExpenditureDao.insert(financialExpenditure);
            }
        }
    }

    private void setFinancialExpenditure(OaInputDto oaInputDto,HashMap hashMap,FinancialExpenditure financialExpenditure){
        List<OaColumnDto> columns = oaInputDto.getColumns();
        if (columns != null && columns.size() > 0) {
            for (OaColumnDto column : columns) {
                List<OaInputDto> columnList = column.getList();
                if (columnList != null && columnList.size() > 0) {
                    //比对模板、获取用户录入
                    for (OaInputDto inputDto : columnList) {
                        String name = inputDto.getName();
                        Object o = hashMap.get(inputDto.getModel());
                        if (Objects.isNull(o) || StringUtils.isEmpty(o.toString())) {
                            continue;
                        }
                        if (StringUtils.equals(name,"课程组编号")) {
                            financialExpenditure.setVipGroupId(Long.parseLong(o.toString()));
                        } else if (StringUtils.equals(name,"课程类型")) {
                            financialExpenditure.setGroupType(o.toString());
                        } else if (StringUtils.equals(name,"乐团编号")) {
                            financialExpenditure.setMusicGroupId(o.toString());
                        } else if (StringUtils.equals(name,"退费项目")) {
                            financialExpenditure.setReturnFeeType(o.toString());
                        } else if (StringUtils.equals(name,"学员编号")) {
                            financialExpenditure.setStudentId(Integer.parseInt(o.toString()));
                        } else if (StringUtils.equals(name,"分部")) {
                            Organization organization = organizationDao.get(Integer.parseInt(o.toString()));
                            if (organization != null) {
                                financialExpenditure.setOrganId(organization.getId());
                                financialExpenditure.setOrganName(organization.getName());
                            }
                        } else if (StringUtils.equals(name,"学校")) {
                            financialExpenditure.setCooperationOrganId(Integer.parseInt(o.toString()));
                        } else if (StringUtils.contains(name,"金额")) {
                            financialExpenditure.setAmount(financialExpenditure.getAmount().add(new BigDecimal(Double.parseDouble(o.toString()))));
                            financialExpenditure.setCurrentAmount(financialExpenditure.getAmount());
                        } else if (StringUtils.equals(name,"情况说明") || StringUtils.equals(name,"事由")) {
                            financialExpenditure.setCause(o.toString());
                        }else if (StringUtils.equals(name,"费用类型") || StringUtils.equals(name,"费用类别")) {
                            FeeProjectEnum projectEnum = FeeProjectEnum.valueOfDesc(o.toString());
                            if(projectEnum != null){
                                financialExpenditure.setFeeProject(projectEnum);
                                financialExpenditure.setType(getExpenditureType(projectEnum));
                            }
                        }else if (StringUtils.equals(name,"日期") || StringUtils.equals(name,"费用发生日期")) {
                            financialExpenditure.setPaymentTime(DateUtil.stringToDate(o.toString(),DateUtil.DEFAULT_PATTERN));
                        } else if (StringUtils.equals(name,"费用分摊月份")) {
                            financialExpenditure.setEffectiveTime(o.toString());
                        }
                    }
                }
            }
        }
    }

    //处理业务退费
    private void processBusinessReturns(PWorkOrderInfo pWorkOrderInfo,List<FinancialExpenditure> financialExpenditureList,
                                        String realName,String fileUrl) throws Exception {
        //课程退费
        if(pWorkOrderInfo.getProcess().equals(22)){
            for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                ReturnFeeDto returnFeeDto = new ReturnFeeDto();
                returnFeeDto.setGroupType(GroupType.valueOfDesc(financialExpenditure.getGroupType()));
                returnFeeDto.setAmount(financialExpenditure.getAmount());
                returnFeeDto.setStudentId(financialExpenditure.getStudentId());
                returnFeeDto.setConfirmReturnActivityGive(true);
                returnFeeDto.setGroupId(financialExpenditure.getVipGroupId());
                returnFeeDto.setVipGroupId(financialExpenditure.getVipGroupId());
                if(returnFeeDto.getGroupType() == GroupType.VIP || returnFeeDto.getGroupType() == GroupType.LIVE){
                    vipGroupService.applyRefundForStudentOa(returnFeeDto);
                }else if(returnFeeDto.getGroupType() == GroupType.COMM || returnFeeDto.getGroupType() == GroupType.PRACTICE){
                    groupClassService.cancelGroupOa(returnFeeDto);
                }
            }
        }else if(pWorkOrderInfo.getProcess().equals(33) || pWorkOrderInfo.getProcess().equals(19)){
            //处理学员乐团退费项目
            this.processRefundItem(financialExpenditureList);
            //处理excel文件
            List<FinancialExpenditure> financialExpenditures = this.processExcelFile(fileUrl, financialExpenditureList);
            if(CollectionUtils.isNotEmpty(financialExpenditures)){
                for (FinancialExpenditure financialExpenditure : financialExpenditures) {
                    financialExpenditure.setAprovalTime(pWorkOrderInfo.getUpdateTime());
                    financialExpenditure.setPaymentTime(pWorkOrderInfo.getUpdateTime());
                    financialExpenditure.setEffectiveTime(DateUtil.format(pWorkOrderInfo.getUpdateTime(),DateUtil.ISO_YEAR_MONTH_FORMAT));
                    financialExpenditure.setApplyUser(realName);
                    financialExpenditure.setFeeProject(FeeProjectEnum.REFUND);
                    financialExpenditure.setType(getExpenditureType(financialExpenditure.getFeeProject()));
                    financialExpenditure.setBatchNo(pWorkOrderInfo.getId().toString());
                    financialExpenditure.setFinancialProcessNo(pWorkOrderInfo.getId().toString());
                }
                financialExpenditureList = financialExpenditures;
            }
            //批量处理oa审批退团逻辑
            financialExpenditureList.stream().forEach(e->musicGroupService.directQuitMusicGroupOa(e));
        }
    }

    //处理学员乐团退费项目
    private void processRefundItem(List<FinancialExpenditure> financialExpenditureList){
        for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
            String returnFeeType = financialExpenditure.getReturnFeeType();
            if(StringUtils.isNotEmpty(returnFeeType)){
                if(returnFeeType.contains("乐器")){
                    financialExpenditure.setRefundInstrumentFeeFlag(true);
                }
                if(returnFeeType.contains("乐保")){
                    financialExpenditure.setMaintenanceFeeFlag(true);
                }
                if(returnFeeType.contains("教辅")){
                    financialExpenditure.setRefundTeachingAssistantsFeeFlag(true);
                }
            }
        }
    }

    //处理excel文件
    private List<FinancialExpenditure> processExcelFile(String fileUrl, List<FinancialExpenditure> financialExpenditureList) throws Exception {
        List<FinancialExpenditure> financialExpenditures = new ArrayList<>();
        if(StringUtils.isNotEmpty(fileUrl)){
            String[] split = fileUrl.split(",");
            for (String s : split) {
                URL url = new URL(s);
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setConnectTimeout(3*1000);
                File file = File.createTempFile(UUID.randomUUID().toString(), ".xls");
                FileUtils.copyInputStreamToFile(conn.getInputStream(),file);
                byte[] bytes = FileUtils.readFileToByteArray(file);
                String fileName = file.getName();
                //删除临时文件
                if (file.exists()) {
                    file.delete();
                }
                financialExpenditures.addAll(financialExpenditureService.checkOaQuitMusicGroupExcel(bytes, fileName));
            }
            //去除重复数据
            financialExpenditures.removeAll(Collections.singleton(null));
            if(CollectionUtils.isNotEmpty(financialExpenditureList)){
                for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                    if(financialExpenditures.contains(financialExpenditure)) {
                        continue;
                    }
                    financialExpenditures.add(financialExpenditure);
                }
            }
        }
        return financialExpenditures;
    }


    public static ExpenditureTypeEnum getExpenditureType(FeeProjectEnum feeProjectEnum) {
        switch (feeProjectEnum) {
            case REFUND:
                return ExpenditureTypeEnum.REFUND;
            case HOUSE:
            case OFFICE:
            case MAINS_PROPERTY:
                return ExpenditureTypeEnum.FIXED;
            default:
                return ExpenditureTypeEnum.VARIABLE;
        }
    }

    @Override
    public void checkCourseReturnFee(Map<String,String> paramMap) {
        String tplInfoId = paramMap.get("tplInfoId");
        if(StringUtils.isEmpty(tplInfoId)){
            throw new BizException("参数校验失败");
        }
        if(tplInfoId.equals("11") || tplInfoId.equals("8") || tplInfoId.equals("20")){
            String data = paramMap.get("formData");
            if(StringUtils.isEmpty(data)){
                return;
            }
            JSONObject formData = JSONObject.parseObject(data);
            String formStructure = financialExpenditureDao.getTplInfo(tplInfoId);
            Date now = new Date();
            OaFormStructureDto oaFormStructureDto = JSONObject.parseObject(formStructure, OaFormStructureDto.class);
            List<OaInputDto> oaInputDtos = oaFormStructureDto.getList();
            List<FinancialExpenditure> financialExpenditureList = new ArrayList<>();
            for (OaInputDto oaInputDto : oaInputDtos) {
                if(!"子表单".equals(oaInputDto.getName())){
                    continue;
                }
                String submitForm = oaInputDto.getModel();
                if(Objects.isNull(formData.get(submitForm))){
                    continue;
                }
                List<HashMap> hashMaps = JSONObject.parseArray(formData.get(submitForm).toString(), HashMap.class);
                nullUser: for (HashMap hashMap : hashMaps) {
                    FinancialExpenditure financialExpenditure = new FinancialExpenditure();
                    financialExpenditure.setAprovalTime(now);
                    financialExpenditure.setPaymentTime(now);
                    financialExpenditure.setEffectiveTime(DateUtil.format(now,DateUtil.ISO_YEAR_MONTH_FORMAT));
                    List<OaColumnDto> columns = oaInputDto.getColumns();
                    if(columns != null && columns.size() > 0){
                        for (OaColumnDto column : columns) {
                            List<OaInputDto> columnList = column.getList();
                            if(columnList != null && columnList.size() > 0){
                                for (OaInputDto inputDto : columnList) {
                                    String name = inputDto.getName();
                                    if(StringUtils.equals(name,"课程组编号")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(Objects.nonNull(o) && StringUtils.isNotEmpty(o.toString())){
                                            financialExpenditure.setVipGroupId(Long.parseLong(o.toString()));
                                            continue;
                                        }
                                    }else if(StringUtils.equals(name,"课程类型")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(Objects.nonNull(o) && StringUtils.isNotEmpty(o.toString())){
                                            financialExpenditure.setGroupType(o.toString());
                                            continue;
                                        }
                                    }else if(StringUtils.equals(name,"乐团编号")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(Objects.nonNull(o) && StringUtils.isNotEmpty(o.toString())){
                                            financialExpenditure.setMusicGroupId(o.toString());
                                            continue;
                                        }
                                    }else if(StringUtils.equals(name,"学员编号")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(Objects.nonNull(o) && StringUtils.isNotEmpty(o.toString())){
                                            financialExpenditure.setStudentId(Integer.parseInt(o.toString()));
                                            continue;
                                        }else {
                                            continue nullUser;
                                        }
                                    }else if(StringUtils.equals(name,"金额")){
                                        Object o = hashMap.get(inputDto.getModel());
                                        if(Objects.nonNull(o) && StringUtils.isNotEmpty(o.toString())){
                                            financialExpenditure.setAmount(new BigDecimal(Double.parseDouble(o.toString())));
                                            financialExpenditure.setCurrentAmount(financialExpenditure.getAmount());
                                            continue;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    financialExpenditureList.add(financialExpenditure);
                }
            }
            if(CollectionUtils.isEmpty(financialExpenditureList)){
                return;
            }
            //课程退费
            if(tplInfoId.equals("11")){
                long count = financialExpenditureList.stream().filter(e -> StringUtils.isEmpty(e.getGroupType()) ||
                        Objects.isNull(e.getAmount()) ||
                        Objects.isNull(e.getStudentId()) ||
                        Objects.isNull(e.getVipGroupId())).count();
                if(count > 0L){
                    throw new BizException("表单信息有误,请核实");
                }
                for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                    ReturnFeeDto returnFeeDto = new ReturnFeeDto();
                    returnFeeDto.setGroupType(GroupType.valueOfDesc(financialExpenditure.getGroupType()));
                    returnFeeDto.setAmount(financialExpenditure.getAmount());
                    returnFeeDto.setStudentId(financialExpenditure.getStudentId());
                    returnFeeDto.setConfirmReturnActivityGive(true);
                    returnFeeDto.setGroupId(financialExpenditure.getVipGroupId());
                    returnFeeDto.setVipGroupId(financialExpenditure.getVipGroupId());
                    if(returnFeeDto.getGroupType() == GroupType.VIP || returnFeeDto.getGroupType() == GroupType.LIVE){
                        vipGroupService.checkApplyRefundForStudentOa(returnFeeDto);
                    }else if(returnFeeDto.getGroupType() == GroupType.COMM || returnFeeDto.getGroupType() == GroupType.PRACTICE){
                        groupClassService.checkCancelGroupOa(returnFeeDto);
                    }
                }
            }else {
                long count = financialExpenditureList.stream().
                        filter(e -> StringUtils.isEmpty(e.getMusicGroupId()) ||
                        Objects.isNull(e.getAmount()) ||
                        Objects.isNull(e.getStudentId())).count();
                if(count > 0l){
                    throw new BizException("表单信息有误,请核实");
                }
                for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                    musicGroupService.checkDirectQuitMusicGroupOa(financialExpenditure);
                }
            }
        }
    }

    @Override
    public List<FinancialExpenditure> checkOaQuitMusicGroupExcel(byte[] buf,String filename) throws Exception {
        Map<String, List<Map<String, Object>>> sheetsListMap = POIUtil.importExcel(new ByteArrayInputStream(buf), 2, filename);
        InputStream inputStream = new ClassPathResource("columnMapper.ini").getInputStream();
        Map<String, String> columns = IniFileUtil.readIniFile(inputStream, TemplateTypeEnum.OA_QUIT_MUSIC_GROUP.getMsg());
        List<FinancialExpenditure> financialExpenditureList = new ArrayList<>();
        List<Organization> organs = organizationDao.findAllOrgans(1);
        Map<String, List<Organization>> collect = organs.stream().collect(Collectors.groupingBy(Organization::getName));
        for (String e : sheetsListMap.keySet()) {
            List<Map<String, Object>> sheet = sheetsListMap.get(e);
            for (Map<String, Object> row : sheet) {
                if (row.size() == 0) {
                    continue;
                }
                JSONObject objectMap = new JSONObject();
                for (String s : row.keySet()) {
                    if (!columns.containsKey(s)) {
                        continue;
                    }
                    String columnValue = columns.get(s);
                    if(Objects.equals(columnValue,"refundInstrumentFeeFlag")
                            || Objects.equals(columnValue,"refundTeachingAssistantsFeeFlag")
                            || Objects.equals(columnValue,"maintenanceFeeFlag")
                            || Objects.equals(columnValue,"refundCourseFee")){
                        if(Objects.nonNull(row.get(s))){
                            objectMap.put(columnValue, Objects.equals("是",row.get(s))?true:false);
                        }
                    }else if(Objects.equals(columnValue,"organName")){
                        if(Objects.isNull(row.get(s)) || StringUtils.isEmpty(row.get(s).toString())){
                            throw new BizException("乐团退费模板错误: 请录入分部信息");
                        }
                        if(!collect.containsKey(row.get(s).toString())){
                            throw new BizException("乐团退费模板错误: 分部 {} 不存在",row.get(s).toString());
                        }
                        objectMap.put("organId",collect.get(row.get(s).toString()).get(0).getId());
                    }else {
                        objectMap.put(columnValue, row.get(s));
                    }
                }
                FinancialExpenditure financialExpenditure = JSONObject.parseObject(objectMap.toJSONString(),FinancialExpenditure.class);
                if(StringUtils.isEmpty(financialExpenditure.getMusicGroupId())
                        || Objects.isNull(financialExpenditure.getStudentId())){
                    throw new BizException("乐团退费模板错误: 乐团编号或学员编号不可为空");
                }
                if(Objects.isNull(financialExpenditure.getOrganId())){
                    throw new BizException("乐团退费模板错误: 请填写分部信息");
                }
                if(Objects.isNull(financialExpenditure.getAmount())){
                    throw new BizException("乐团退费模板错误: 退费金额不可为空");
                }
                if(StringUtils.isEmpty(financialExpenditure.getMerNo()) ||
                        StringUtils.isEmpty(financialExpenditure.getBankAddress()) ||
                        StringUtils.isEmpty(financialExpenditure.getMerNoName())){
                    throw new BizException("乐团退费模板错误: 请完整填写收款账户信息");
                }
                financialExpenditureList.add(financialExpenditure);
            }
        }
        if(CollectionUtils.isNotEmpty(financialExpenditureList)){
            for (FinancialExpenditure financialExpenditure : financialExpenditureList) {
                musicGroupService.checkDirectQuitMusicGroupOa(financialExpenditure);
            }
        }
        return financialExpenditureList;
    }

}
