package com.ym.mec.biz.service.impl;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ym.mec.biz.dal.config.AdapayNotifyConstants;
import com.ym.mec.biz.dal.dao.HfMemberDao;
import com.ym.mec.biz.dal.dto.YeepayMember;
import com.ym.mec.biz.dal.entity.HfMember;
import com.ym.mec.biz.dal.enums.PaymentChannelEnum;
import com.ym.mec.biz.service.HfMemberService;
import com.ym.mec.biz.service.HfMerchantConfigService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.adapay.CorpMemberService;
import com.ym.mec.thirdparty.adapay.SettleAccountService;
import com.ym.mec.thirdparty.adapay.entity.AccountInfo;
import com.ym.mec.thirdparty.adapay.entity.EnterpriseUser;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import com.ym.mec.thirdparty.yeepay.YeepayMerchantService;
import com.ym.mec.thirdparty.yeepay.entity.MerchantInfo;
import com.ym.mec.util.string.MessageFormatter;

@Service
public class HfMemberServiceImpl extends BaseServiceImpl<Integer, HfMember> implements HfMemberService {
	
    @Autowired
    private HfMemberDao hfMemberDao;
    
    @Autowired
    private SettleAccountService settleAccountService;
    
    @Autowired
    private CorpMemberService corpMemberService;
    
    @Autowired
    private HfMerchantConfigService hfMerchantConfigService;
    
    @Autowired
    private YeepayMerchantService yeepayMerchantService;

    @Value("${spring.profiles.active:dev}")
    private String env;

	private String cardName;

    @Override
    public BaseDAO<Integer, HfMember> getDAO() {
        return hfMemberDao;
    }

    @Override
	public HfMember getByMemberId(String memberId, String payerName) {
		return hfMemberDao.getByMemberId(memberId, payerName);
	}

	@Override
	public HfMember getByOrganId(Integer organId, String payerName) {
		return hfMemberDao.getByOrganId(organId, payerName);
	}

	@Override
	public HfMember queryByRequestNo(String requestNo) {
		return hfMemberDao.queryByRequestNo(requestNo);
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createMember(HfMember member, File file) throws Exception {
        HfMember hasMember = hfMemberDao.getByOrganId(member.getOrganId(), member.getPayerName());
        if (hasMember != null) {
            throw new BizException("该分部已创建成功，请核查");
        }
        hasMember = hfMemberDao.getByName(member.getName(), member.getPayerName());
        if (hasMember != null) {
            throw new BizException("公司名与已有公司名相同，请核查");
        }
        
        switch (PaymentChannelEnum.codeOf(member.getPayerName())) {
		case ADAPAY:
			createMemberWithAdapay(member, file);
			break;
			
		case YEEPAY:
			Map<String, Object> response = createMemberWithYeepay((YeepayMember) member);
			member.setRequestNo(response.get("requestNo").toString());
			member.setMemberId(response.get("merchantNo").toString());
			break;

		default:
			
			return false;
		}

        Date nowDate = new Date();
        member.setCreateTime(nowDate);
        member.setUpdateTime(nowDate);
        member.setStatus("pending");
        insert(member);

        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateMember(HfMember member, File file) throws Exception {
        HfMember hfMember = hfMemberDao.get(member.getId());
        if (hfMember == null) {
            throw new BizException("商户不存在请核对");
        }

        Integer tenantId = member.getTenantId();
        
        HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(tenantId, member.getPayerName());
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}]汇付商户信息找不到", tenantId);
        }
        
        EnterpriseUser enterpriseUser = new EnterpriseUser();
        enterpriseUser.setAddress(member.getAddress());
        enterpriseUser.setAreaCode(member.getAreaCode());
        enterpriseUser.setAttachFile(file);
        enterpriseUser.setBankAcctType(member.getBankAcctType());
        enterpriseUser.setBankCode(member.getBankCode());
        enterpriseUser.setBusinessScope(member.getBusinessScope());
        enterpriseUser.setCardName(member.getCardName());
        enterpriseUser.setCardNo(member.getCardNo());
        enterpriseUser.setLegalCertId(member.getLegalCertId());
        enterpriseUser.setLegalCertIdExpires(member.getLegalCertIdExpires());
        enterpriseUser.setLegalMp(member.getLegalMp());
        enterpriseUser.setLegalPerson(member.getLegalPerson());
        enterpriseUser.setMemberId(member.getMemberId());
        enterpriseUser.setName(member.getName());
        
        if(StringUtils.equals(env, "prod")){
            enterpriseUser.setNotifyUrl(AdapayNotifyConstants.corp_member_notify_url_prod);
        }else {
            enterpriseUser.setNotifyUrl(AdapayNotifyConstants.corp_member_notify_url_test);
        }
        
        enterpriseUser.setProvCode(member.getProvCode());
        enterpriseUser.setSocialCreditCode(member.getSocialCreditCode());
        enterpriseUser.setSocialCreditCodeExpires(member.getSocialCreditCodeExpires());
        
        corpMemberService.update(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), enterpriseUser, enterpriseUser.getNotifyUrl());

        Date nowDate = new Date();
        member.setUpdateTime(nowDate);
        member.setStatus("pending");
        if (update(member) <= 0) {
            throw new BizException("更新商户信息失败,请重试");
        }
        
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean mergeSettleAccount(String memberId, AccountInfo accountInfo) throws Exception {
        HfMember hfMember = hfMemberDao.getByMemberId(memberId, PaymentChannelEnum.ADAPAY.getCode());
        if (!hfMember.getStatus().equals("succeeded")) {
            throw new BizException("商户审核通过，才能绑定结算卡");
        }
        /*hfMember.setCardNo(accountInfo.getCardId());
        hfMember.setBankCode(accountInfo.getBankCode());
        hfMember.setBankAcctType(accountInfo.getBankAcctType());
        hfMember.setCardName(accountInfo.getCardName());*/
        
        accountInfo.setAreaCode(hfMember.getAreaCode());
        accountInfo.setBankAcctType(hfMember.getBankAcctType());
        accountInfo.setBankCode(hfMember.getBankCode());
        accountInfo.setBankName(hfMember.getBankCode());
        accountInfo.setCardId(hfMember.getCardNo());
        accountInfo.setCardName(hfMember.getCardName());
        accountInfo.setMemberId(memberId);
        accountInfo.setProvCode(hfMember.getProvCode());
        accountInfo.setTelNo(hfMember.getLegalMp());
        
        Integer tenantId = hfMember.getTenantId();
        
        HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(tenantId, PaymentChannelEnum.ADAPAY.getCode());
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}]汇付商户信息找不到", tenantId);
        }

        //删除结算账户
        if (StringUtils.isNotBlank(hfMember.getSettleAccountId())) {
            settleAccountService.delete(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), memberId, hfMember.getSettleAccountId());
        }

        //创建结算账户
        String settleAccountId = settleAccountService.create(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), accountInfo);

        hfMember.setSettleAccountId(settleAccountId);
        if (hfMemberDao.update(hfMember) <= 0) {
            throw new BizException("绑卡失败，请重试");
        }
        return true;
    }
    
	private boolean createMemberWithAdapay(HfMember member, File file) {
		Integer tenantId = member.getTenantId();

		HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(tenantId, member.getPayerName());
		if (hfMerchantConfig == null) {
			throw new BizException("机构[{}]汇付商户信息找不到", tenantId);
		}

		EnterpriseUser enterpriseUser = new EnterpriseUser();
		enterpriseUser.setAddress(member.getAddress());
		enterpriseUser.setAreaCode(member.getAreaCode());
		enterpriseUser.setAttachFile(file);
		enterpriseUser.setBankAcctType(member.getBankAcctType());
		enterpriseUser.setBankCode(member.getBankCode());
		enterpriseUser.setBusinessScope(member.getBusinessScope());
		enterpriseUser.setCardName(member.getCardName());
		enterpriseUser.setCardNo(member.getCardNo());
		enterpriseUser.setLegalCertId(member.getLegalCertId());
		enterpriseUser.setLegalCertIdExpires(member.getLegalCertIdExpires());
		enterpriseUser.setLegalMp(member.getLegalMp());
		enterpriseUser.setLegalPerson(member.getLegalPerson());
		enterpriseUser.setMemberId(member.getMemberId());
		enterpriseUser.setName(member.getName());

		if (StringUtils.equals(env, "prod")) {
			enterpriseUser.setNotifyUrl(MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_prod, "adapay"));
		} else {
			enterpriseUser.setNotifyUrl(MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_test, "adapay"));
		}
		enterpriseUser.setProvCode(member.getProvCode());
		enterpriseUser.setSocialCreditCode(member.getSocialCreditCode());
		enterpriseUser.setSocialCreditCodeExpires(member.getSocialCreditCodeExpires());

		return corpMemberService.create(hfMerchantConfig.getMerKey(), hfMerchantConfig.getAppId(), enterpriseUser, enterpriseUser.getNotifyUrl());
	}
	
	private Map<String, Object> createMemberWithYeepay(YeepayMember member) throws IOException {

		HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(member.getTenantId(), member.getPayerName());
		if (hfMerchantConfig == null) {
			throw new BizException("机构[{}]易宝商户信息找不到", member.getTenantId());
		}
		
		MerchantInfo merchantInfo = new MerchantInfo();
		merchantInfo.setAddress(member.getAddress());
		merchantInfo.setBankAccountType(member.getBankAcctType());
		merchantInfo.setBankCardNo(member.getCardNo());
		merchantInfo.setBankCode(member.getBankCode());
		merchantInfo.setCityCode(member.getAreaCode());
		merchantInfo.setContactEmail(member.getContactEmail());
		merchantInfo.setContactLicenceNo(member.getContactLicenceNo());
		merchantInfo.setContactMobile(member.getContactMobile());
		merchantInfo.setContactName(member.getContactName());
		merchantInfo.setDistrictCode(member.getDistrictCode());
		merchantInfo.setLegalLicenceBackUrl(member.getLegalLicenceBackUrl());
		merchantInfo.setLegalLicenceFrontUrl(member.getLegalLicenceFrontUrl());
		merchantInfo.setLegalLicenceNo(member.getLegalCertId());
		merchantInfo.setLegalLicenceType(member.getLegalLicenceType());
		merchantInfo.setLegalName(member.getLegalPerson());
		merchantInfo.setLicenceNo(member.getSocialCreditCode());
		merchantInfo.setLicenceUrl(member.getLicenceUrl());
		merchantInfo.setMerchantType(member.getMerchantType());
		merchantInfo.setMerchantName(member.getName());
		merchantInfo.setMobile(member.getLegalMp());
		merchantInfo.setOpenAccountLicenceUrl(member.getOpenAccountLicenceUrl());
		merchantInfo.setProvinceCode(member.getProvCode());
		merchantInfo.setSettlementDirection("BANKCARD");

		String notifyURL = MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_test, "yeepay");
		if (StringUtils.equals(env, "prod")) {
			notifyURL = MessageFormatter.arrayFormat(AdapayNotifyConstants.corp_member_notify_url_prod, "yeepay");
		}
		
		return yeepayMerchantService.registerSaasMerchantV2(hfMerchantConfig, merchantInfo, notifyURL);
	}
}
