package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.middleware.im.ImPluginContext;
import com.microsvc.toolkit.middleware.im.ImPluginService;
import com.microsvc.toolkit.middleware.im.impl.RongCloudImPlugin;
import com.microsvc.toolkit.middleware.im.message.GroupMemberWrapper;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ImGroupMemberDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.EFriendRoleType;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.page.GroupMemberQueryInfo;
import com.ym.mec.biz.dal.wrapper.ImGroupMemberWrapper;
import com.ym.mec.biz.service.ImGroupMemberService;
import com.ym.mec.biz.service.ImGroupService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.im.entity.GroupMember;
import com.ym.mec.im.entity.GroupModel;
import com.ym.mec.util.collection.MapUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.Map.Entry;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ImGroupMemberServiceImpl extends BaseServiceImpl<Long, ImGroupMember> implements ImGroupMemberService {
	@Resource
	private ImGroupMemberDao imGroupMemberDao;
	@Resource
	private ImGroupDao imGroupDao;
	@Resource
	private ImGroupService imGroupService;
	@Resource
	private TeacherDao teacherDao;
	@Resource
	private ClassGroupDao classGroupDao;
	@Resource
	private MusicGroupDao musicGroupDao;
	@Resource
	private PracticeGroupDao practiceGroupDao;
	@Resource
	private VipGroupDao vipGroupDao;
	@Resource
	private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;
	@Resource
	private ClassGroupStudentMapperDao classGroupStudentMapperDao;
	@Resource
	private ImFeignService imFeignService;
	@Resource
	private ImGroupCoreService imGroupCoreService;
	@Resource
	private ImPluginContext imPluginContext;
    @Resource
    private SysUserService sysUserService;
	@Override
	public BaseDAO<Long, ImGroupMember> getDAO() {
		return imGroupMemberDao;
	}

	public void joinGroup(String imGroupId, Integer userId, String roleType, boolean isAdmin, Map<Integer,String> userRoleMap){
		int groupId = Integer.parseInt(imGroupId);
		ClassGroup classGroup = classGroupDao.get(groupId);
		String tags = classGroup.getName();
		//相关用户加入群组
		//获取班级关联的老师列表
		Set<Integer> teacherIds = classGroupTeacherMapperDao.queryTeacherIdsByClassGroupId(groupId,null,null);
		//获取班级关联的学员列表
		Set<Integer> studentIds = classGroupStudentMapperDao.queryStudentIdsByClassGroupId(groupId,null,null);
		if(studentIds != null && studentIds.size() > 0){
			for(Integer studentId : studentIds){
				userRoleMap.put(studentId, null);
			}
		}
		if(teacherIds != null && teacherIds.size() > 0){
			for(Integer teacherId : teacherIds){
				userRoleMap.put(teacherId, "指导老师");
			}
		}
		if(classGroup.getGroupType() == GroupType.MUSIC){
			MusicGroup musicGroup = musicGroupDao.get(classGroup.getMusicGroupId());
			tags = musicGroup.getName();
			if(musicGroup.getEducationalTeacherId() != null){
				userRoleMap.put(musicGroup.getEducationalTeacherId(), "乐团主管");
			}
			if(musicGroup.getDirectorUserId() != null){
				userRoleMap.put(musicGroup.getDirectorUserId(), "乐队指导");
			}
			if(musicGroup.getTransactionTeacherId() != null){
				userRoleMap.put(musicGroup.getTransactionTeacherId(), "衔接老师");
			}
		}else if(classGroup.getGroupType() == GroupType.PRACTICE){
			PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(classGroup.getMusicGroupId()));
			tags = practiceGroup.getName();
			userRoleMap.put(practiceGroup.getUserId(), "指导老师");
			if(practiceGroup.getEducationalTeacherId() != null){
				userRoleMap.put(practiceGroup.getEducationalTeacherId(), "乐团主管");
			}
		}else if(classGroup.getGroupType() == GroupType.VIP){
			VipGroup vipGroup = vipGroupDao.get(Long.parseLong(classGroup.getMusicGroupId()));
			userRoleMap.put(vipGroup.getUserId(), "指导老师");
			if(vipGroup.getEducationalTeacherId() != null){
				userRoleMap.put(vipGroup.getEducationalTeacherId(), "乐团主管");
			}
			tags = vipGroup.getName();
		}
		imGroupService.create(imGroupId, null, classGroup.getName(), tags,
				tags, tags, null, classGroup.getGroupType().getCode(), ImGroup.GroupTypeEnum.valueOf(classGroup.getType().getCode()));
		if(userId != null){
			join(imGroupId,userId,roleType,isAdmin);
		}
		join(imGroupId,userRoleMap);
	}

	@Override
	public ImGroupMemberDao getDao() {
		return imGroupMemberDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean join(String imGroupId, Integer userId, String roleType, boolean isAdmin) {
		ImGroup imGroup = imGroupDao.getLocked(imGroupId);
		if (imGroup == null) {
			joinGroup(imGroupId,userId,roleType,isAdmin,new HashMap<>());
			return true;
		}
		//检查用户是否已存在
		List<ImGroupMember> imGroupMemberList = imGroupMemberDao.queryByImGroupIdAndUserId(imGroupId, userId.toString());
		if(CollectionUtils.isNotEmpty(imGroupMemberList)){
			return true;
		}
		ImGroupMember imGroupMember = new ImGroupMember();
		Date date = new Date();
		imGroupMember.setGroupType(imGroup.getGroupType());
		imGroupMember.setCreateTime(date);
		imGroupMember.setImGroupId(imGroupId);
		imGroupMember.setIsAdmin(isAdmin);
		SysUser user = teacherDao.getUser(userId);
		imGroupMember.setRoleType(roleType);
		if(StringUtils.isNotEmpty(roleType)){
			imGroupMember.setType(EFriendRoleType.getByName(roleType).name());
			imGroupMember.setNickname(StringUtils.isEmpty(user.getRealName())?user.getUsername():user.getRealName());
		}else {
			imGroupMember.setNickname(StringUtils.isEmpty(user.getUsername())?user.getRealName():user.getUsername());
		}
		imGroupMember.setUpdateTime(date);
		imGroupMember.setUserId(userId);
		//初始化群主数据
		this.initImGroupMember(imGroupMember,imGroup.getGroupType());

		imGroupMemberDao.insert(imGroupMember);

		imGroup.setMemberNum(1);
		imGroup.setUpdateTime(date);
		imGroupDao.update(imGroup);

		String groupId = imGroup.getId();

		if (RongCloudImPlugin.PLUGIN_NAME.equals(imPluginContext.defaultService())) {
			// 融云IM用户加群
			GroupMember groupMember = new GroupMember(userId.toString(), groupId);
			imFeignService.groupJoin(new GroupModel(groupId, new GroupMember[] { groupMember }, imGroup.getName()));
		} else {
			try {
				List<ImGroupMemberWrapper.ImGroupMember> groupMembers = Lists.newArrayList();

				groupMembers.add(ImGroupMemberWrapper.ImGroupMember.builder()
						.groupId(groupId)
						.isAdmin(imGroupMember.getIsAdmin())
						.roleType(imGroupMember.getRoleType())
						.type(imGroupMember.getType())
						.nickname(imGroupMember.getNickname())
						.userId(userId.longValue())
						.imUserId(userId.toString())
						.groupRoleType(imGroupMember.getGroupRoleType())
						.avatar(sysUserService.getImAvatar(user))
						.updateTime(DateTime.now().toDate())
						.createTime(DateTime.now().toDate())
					.build());
				// 腾讯云IM用户加群
				imGroupCoreService.groupMemberJoin(groupId, groupMembers);
			} catch (Exception e) {
				log.error("腾讯云IM用户加群失败 groupId={}, userId={}", imGroupId, userId, e);
			}
		}
		return true;
	}

	@Override
	public void initImGroupMember(ImGroupMember imGroupMember, ImGroup.GroupTypeEnum groupType){
		try {
			List<GroupMemberWrapper.Member> members = imPluginContext.getPluginService().groupUserGagList(imGroupMember.getImGroupId());
			if(CollectionUtils.isNotEmpty(members)){
				List<String> imGroupUserIds = members.stream().map(e -> e.getId()).collect(Collectors.toList());
				if(CollectionUtils.isNotEmpty(imGroupUserIds)){
					if(imGroupUserIds.contains(imGroupMember.getUserId().toString())){
						imGroupMember.setMuteFlag(true);
					}
				}
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		//是否有群主
		if(!imGroupMember.getIsAdmin()){
			ImGroupMember admin = imGroupMemberDao.getAdmin(imGroupMember.getImGroupId());
			if(Objects.nonNull(admin)){
				return;
			}
			switch (groupType) {
				case NORMAL:
				case MIX:
				case HIGH:
				case SNAP:
				case HIGH_ONLINE:
				case MUSIC_NETWORK:
				case PARENT_MEETING:
					if("乐团主管".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
					}
					break;
				case VIP:
				case DEMO:
				case PRACTICE:
				case COMM:
				case REPERTOIRE_PLAY:
				case LIVE:
					if("指导老师".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
					}
					break;
				case TRAINING:
					if("分部经理".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
					}
					break;
				case SCHOOL:
					if("乐团领队".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
					}
					break;
			}
		}
	}

	@Override
	public void initImGroupMembers(List<ImGroupMember> imGroupMembers, ImGroup.GroupTypeEnum groupType){
		if (CollectionUtils.isEmpty(imGroupMembers)) {
			return;
		}
		String imGroupId = imGroupMembers.get(0).getImGroupId();
		try {
			List<GroupMemberWrapper.Member> members = imPluginContext.getPluginService().groupUserGagList(imGroupId);
			if(CollectionUtils.isNotEmpty(members)){
				List<String> imGroupUserIds = members.stream().map(e -> e.getId()).collect(Collectors.toList());
				if(CollectionUtils.isNotEmpty(imGroupUserIds)){
					for (ImGroupMember imGroupMember : imGroupMembers) {
						if(imGroupUserIds.contains(imGroupMember.getUserId().toString())){
							imGroupMember.setMuteFlag(true);
						}
					}
				}
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		//是否有群主
		long count = imGroupMembers.stream().filter(e -> e.getIsAdmin()).count();
		if(count > 0){
			return;
		}
		ImGroupMember admin = imGroupMemberDao.getAdmin(imGroupId);
		if(Objects.nonNull(admin)){
			return;
		}
		for (ImGroupMember imGroupMember : imGroupMembers) {
			switch (groupType) {
				case NORMAL:
				case MIX:
				case HIGH:
				case SNAP:
				case HIGH_ONLINE:
				case MUSIC_NETWORK:
				case PARENT_MEETING:
					if("乐团主管".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
						return;
					}
					break;
				case VIP:
				case DEMO:
				case PRACTICE:
				case COMM:
				case REPERTOIRE_PLAY:
				case LIVE:
					if("指导老师".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
						return;
					}
					break;
				case TRAINING:
					if("分部经理".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
						return;
					}
					break;
				case SCHOOL:
					if("乐团领队".equals(imGroupMember.getRoleType())){
						imGroupMember.setIsAdmin(true);
						return;
					}
					break;
			}
		}
	}

	@Transactional(rollbackFor = Exception.class)
	public boolean join(String imGroupId,String roleType, boolean isAdmin,ImGroup imGroup,SysUser user) {
		if (imGroup == null) {
			imGroup = imGroupDao.getLocked(imGroupId);
		}
		if (imGroup == null) {
			joinGroup(imGroupId,user.getId(),roleType,isAdmin,new HashMap<>());
			return true;
		}
		//检查用户是否已存在
		List<ImGroupMember> imGroupMemberList = imGroupMemberDao.queryByImGroupIdAndUserId(imGroupId, user.getId().toString());
		if(CollectionUtils.isNotEmpty(imGroupMemberList)){
			return true;
		}

		ImGroupMember imGroupMember = new ImGroupMember();
		Date date = new Date();

		imGroupMember.setCreateTime(date);
		imGroupMember.setImGroupId(imGroupId);
		imGroupMember.setIsAdmin(isAdmin);
		imGroupMember.setRoleType(roleType);
		if(StringUtils.isNotEmpty(roleType)){
			imGroupMember.setType(EFriendRoleType.getByName(roleType).name());
			imGroupMember.setNickname(StringUtils.isEmpty(user.getRealName())?user.getUsername():user.getRealName());
		}else {
			imGroupMember.setNickname(StringUtils.isEmpty(user.getUsername())?user.getRealName():user.getUsername());
		}
		imGroupMember.setUpdateTime(date);
		imGroupMember.setUserId(user.getId());
		this.initImGroupMember(imGroupMember,imGroup.getGroupType());
		imGroupMemberDao.insert(imGroupMember);

		imGroup.setMemberNum(1);
		imGroup.setUpdateTime(date);
		imGroupDao.update(imGroup);

		String groupId = imGroup.getId();

		if (RongCloudImPlugin.PLUGIN_NAME.equals(imPluginContext.defaultService())) {
			// 融云IM用户加群
			GroupMember groupMember = new GroupMember(user.getId().toString(), groupId);
			imFeignService.groupJoin(new GroupModel(groupId, new GroupMember[] { groupMember }, imGroup.getName()));
		} else {
			// 腾讯云IM
			List<ImGroupMemberWrapper.ImGroupMember> groupMembers = Lists.newArrayList();
			try {

				groupMembers.add(ImGroupMemberWrapper.ImGroupMember.builder()
						.groupId(groupId)
						.userId(imGroupMember.getUserId().longValue())
						.imUserId(imGroupMember.getUserId().toString())
						.nickname(imGroupMember.getNickname())
						.isAdmin(imGroupMember.getIsAdmin())
						.groupRoleType(imGroupMember.getGroupRoleType())
						.roleType(imGroupMember.getRoleType())
						.type(imGroupMember.getType())
						.avatar(sysUserService.getImAvatar(user))
					.build());
				// 腾讯云IM用户加群
				imGroupCoreService.groupMemberJoin(groupId, groupMembers);
			} catch (Exception e) {
				log.error("腾讯云IM用户加群失败 groupId={}, groupMembers={}", imGroupId, JSON.toJSONString(groupMembers), e);
			}
		}

		return true;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean join(String imGroupId, Map<Integer, String> userRoleMap) {
		ImGroup imGroup = imGroupDao.getLocked(imGroupId);
		if (imGroup == null) {
			joinGroup(imGroupId,null,null,false,userRoleMap);
			return true;
		}
		//检查用户是否已存在
		List<Integer> existUserIdList = imGroupMemberDao.queryByImGroupIdAndUserId(imGroupId.toString(), userRoleMap.keySet().stream().filter(Objects::nonNull).map(Objects::toString)
		.collect(Collectors.joining(","))).stream().map(e -> e.getUserId()).collect(Collectors.toList());

		Date date = new Date();
		List<ImGroupMember> imGroupMemberList = new ArrayList<ImGroupMember>();
		List<GroupMember> groupMemberList = new ArrayList<GroupMember>();

		String groupId = imGroup.getId().toString();
		List<Integer> usernameIds = new ArrayList<>();
		List<Integer> realNameIds = new ArrayList<>();

//		Map<Integer, String> nameIdMap = MapUtil.convertIntegerMap(teacherDao.queryNickNameByRole(userRoleMap.keySet(),userRoleMap));

		List<ImGroupMemberWrapper.ImGroupMember> groupMembers = Lists.newArrayList();
		for (Entry<Integer, String> entry : userRoleMap.entrySet()) {
			if(existUserIdList.contains(entry.getKey())||Objects.isNull(entry.getKey())){
				continue;
			}
			ImGroupMember imGroupMember = new ImGroupMember();
			imGroupMember.setCreateTime(date);
			imGroupMember.setImGroupId(imGroupId);
			imGroupMember.setIsAdmin(false);
			imGroupMember.setRoleType(entry.getValue());
			if(StringUtils.isEmpty(entry.getValue())){
				usernameIds.add(entry.getKey());
			}else {
				imGroupMember.setType(EFriendRoleType.getByName(entry.getValue()).name());
				realNameIds.add(entry.getKey());
			}
			imGroupMember.setUpdateTime(date);
			imGroupMember.setUserId(entry.getKey());
			this.initImGroupMember(imGroupMember,imGroup.getGroupType());
			imGroupMemberList.add(imGroupMember);
			groupMemberList.add(new GroupMember(imGroupMember.getUserId().toString(), groupId));

			// 腾讯云IM群用户信息
			groupMembers.add(ImGroupMemberWrapper.ImGroupMember.builder()
					.groupId(groupId)
					.userId(imGroupMember.getUserId().longValue())
					.imUserId(imGroupMember.getUserId().toString())
					.isAdmin(imGroupMember.getIsAdmin())
					.nickname(imGroupMember.getNickname())
					.groupRoleType(imGroupMember.getGroupRoleType())
					.roleType(imGroupMember.getRoleType())
					.type(imGroupMember.getType())
					.updateTime(imGroupMember.getUpdateTime())
					.createTime(imGroupMember.getCreateTime())
				.build());
		}


		if (CollectionUtils.isNotEmpty(imGroupMemberList)) {
			Map<Integer, String> usernameMap = new HashMap<>();
			Map<Integer, String> realNameMap = new HashMap<>();
			if(CollectionUtils.isNotEmpty(usernameIds)){
				usernameMap = MapUtil.convertIntegerMap(teacherDao.queryUserNameByIdList(usernameIds));
			}
			if(CollectionUtils.isNotEmpty(realNameIds)){
				realNameMap = MapUtil.convertIntegerMap(teacherDao.queryNameByIdList(realNameIds));
			}
			for (ImGroupMember imGroupMember : imGroupMemberList) {
				if(StringUtils.isEmpty(imGroupMember.getRoleType())){
					imGroupMember.setNickname(usernameMap.get(imGroupMember.getUserId()));
				}else {
					imGroupMember.setNickname(realNameMap.get(imGroupMember.getUserId()));
				}
			}
			this.initImGroupMembers(imGroupMemberList,imGroup.getGroupType());
			imGroupMemberDao.batchInsert(imGroupMemberList);
			imGroup.setMemberNum(imGroupMemberList.size());
			imGroup.setUpdateTime(date);
			imGroupDao.update(imGroup);

			if (RongCloudImPlugin.PLUGIN_NAME.equals(imPluginContext.defaultService())) {
				// 融云IM用户加群
				imFeignService.groupJoin(new GroupModel(groupId, groupMemberList.stream().toArray(GroupMember[]::new), imGroup.getName()));
			} else {
				try {
					List<SysUser> userList = teacherDao.getUserList(groupMembers.stream().map(e -> e.getUserId()).collect(Collectors.toList()));
					Map<Integer,SysUser> userMap = userList.stream().collect(Collectors.toMap(SysUser::getId, e -> e));
					for (ImGroupMemberWrapper.ImGroupMember item : groupMembers) {
                        SysUser user = userMap.get(item.getUserId().intValue());
						if (user != null) {
							// 用户头像
							item.setAvatar(sysUserService.getImAvatar(user));
							if(StringUtils.isNotEmpty(item.getRoleType())){
								item.setNickname(StringUtils.isEmpty(user.getRealName())?user.getUsername():user.getRealName());
							}else {
								item.setNickname(StringUtils.isEmpty(user.getUsername())?user.getRealName():user.getUsername());
							}
						}
					}
					// 腾讯云IM用户加群
					imGroupCoreService.groupMemberJoin(groupId, groupMembers);
				} catch (Exception e) {
					log.error("腾讯云IM用户加群失败 groupId={}, members={}", imGroupId, JSON.toJSONString(groupMembers), e);
				}
			}
		}

		return true;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean quit(String imGroupId, Integer userId) {
		List<Integer> userIdList = new ArrayList<Integer>();
		userIdList.add(userId);

		return quit(imGroupId, userIdList);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean quit(String imGroupId, List<Integer> userIdList) {

        return quit(imGroupId, userIdList, true);
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean quit(String imGroupId, List<Integer> userIdList,Boolean quit) {
		ImGroup imGroup = imGroupDao.getLocked(imGroupId);
		if (imGroup == null) {
			return true;
		}

		List<ImGroupMember> imGroupMemberList = imGroupMemberDao.queryByImGroupIdAndUserId(imGroupId.toString(), userIdList.stream().map(Objects::toString)
				.collect(Collectors.joining(",")));

		if (imGroupMemberList == null || imGroupMemberList.size() == 0) {
			return true;
		}

		imGroupMemberDao.batchDelete(imGroupMemberList);

		imGroup.setMemberNum(-userIdList.size());
		imGroup.setUpdateTime(new Date());
		imGroupDao.update(imGroup);

		String groupId = imGroup.getId().toString();

		List<GroupMember> groupMemberList = new ArrayList<GroupMember>();

		List<GroupMemberWrapper.ImGroupMember> imGroupMembers = Lists.newArrayList();
		for (Integer userId : userIdList) {
			// 融云IM用户
			groupMemberList.add(new GroupMember(userId.toString(), groupId));

			// 腾讯云IM用户
			imGroupMembers.add(GroupMemberWrapper.ImGroupMember.builder()
					.groupId(groupId)
					.userId(userId.longValue())
					.imUserId(userId.toString())
				.build());
		}

		if (RongCloudImPlugin.PLUGIN_NAME.equals(imPluginContext.defaultService())) {
			// 融云IM用户退群
			imFeignService.groupQuit(new GroupModel(groupId, groupMemberList.stream().toArray(GroupMember[]::new), imGroup.getName()));
		} else {
			try {
				// 腾讯云IM用户退群
				imGroupCoreService.groupQuit(imGroupMembers, groupId,quit);
			} catch (Exception e) {
				log.error("腾讯云IM用户退群失败", e);
			}
		}
		return true;
	}

	@Override
	public boolean updateRoleType(String imGroupId, Integer userId, String roleType) {
		List<Integer> userIdList = new ArrayList<Integer>();
		userIdList.add(userId);
		List<ImGroupMember> imGroupMemberList = imGroupMemberDao.queryByImGroupIdAndUserId(imGroupId, userIdList.stream().map(Objects::toString)
				.collect(Collectors.joining(",")));

		if (CollectionUtils.isEmpty(imGroupMemberList)) {
			throw new BizException("更新失败：找不到用户信息");
		}
		Date date = new Date();
		ImGroupMember imGroupMember = imGroupMemberList.get(0);
		imGroupMember.setGroupRoleType(null);
		imGroupMember.setRoleType(roleType);
		imGroupMember.setType(EFriendRoleType.getByName(roleType).name());
		imGroupMember.setUpdateTime(date);
		ImGroup group = imGroupDao.get(imGroupId);
		if(!imGroupMember.getIsAdmin()){
			this.initImGroupMembers(imGroupMemberList,group.getGroupType());
			if(imGroupMember.getIsAdmin()){
				try {
					imPluginContext.getPluginService().changeGroupOwner(imGroupId,imGroupMember.getUserId().toString(),null);
				} catch (Exception e) {
					log.error(e.getMessage());
					throw new BizException("更新失败：设置群主异常");
				}
			}
		}
		imGroupMemberDao.update(imGroupMember);
		return true;
	}

	@Override
	public void batchInsert(List<ImGroupMember> imGroupMemberList) {
		imGroupMemberDao.batchInsert(imGroupMemberList);
	}


	@Override
	public void batchDeleteByGroupIdAndUserId(List<ImGroupMember> imGroupMemberList) {
		imGroupMemberDao.batchDeleteByGroupIdAndUserId(imGroupMemberList);
	}


	@Override
	public void delRepeat() {
		imGroupMemberDao.delRepeat();
	}

    @Override
    public PageInfo<ImGroupMemberDto> queryGroupMemberPage(GroupMemberQueryInfo queryInfo) {
		if(StringUtils.isEmpty(queryInfo.getImGroupId())){
			throw new BizException("参数校验失败");
		}
		PageInfo<ImGroupMemberDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<ImGroupMemberDto> dataList = null;
		int count = imGroupMemberDao.countGroupMember(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = imGroupMemberDao.queryGroupMemberPage(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
    }

	@Override
	public void batchJoin(String imGroupId, String userId, String roleType) {
		List<Integer> userIds = Arrays.stream(userId.split(",")).mapToInt(Integer::parseInt).boxed().collect(Collectors.toList());
		Map<Integer, String> userRoleMap = new HashMap<>(userIds.size());
		userIds.stream().forEach(e->userRoleMap.put(e,roleType));
		this.join(imGroupId,userRoleMap);
	}

    @Override
	@Transactional(rollbackFor = Exception.class)
    public void groupMemberMute(ImGroupMemberWrapper.GroupMemberMute member) {
		try {
			// 更新群成员禁言状态
			imGroupMemberDao.updateGroupMemberMute(member.getGroupId(), member.getUserIds(), member.getGroupMute());
			ImPluginService pluginService = imPluginContext.getPluginService();
			if (member.getGroupMute()) {
				// 群成员禁言
				pluginService.groupUserGagCreate(member.getUserIds(), member.getGroupId(), -1L);
			} else {
				// 群成员解除禁言
				pluginService.groupUserGagRemove(member.getUserIds(), member.getGroupId());
			}

		} catch (Exception e) {
			log.error("群成员禁言失败", e);
			throw com.microsvc.toolkit.common.webportal.exception.BizException.from("群成员禁言失败，请联系管理员!");
		}
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void groupMemberRole(ImGroupMemberWrapper.GroupMemberRole member) {
		try {
			// 更新群成员角色
			imGroupMemberDao.batchUpdateGroupMemberRole(member.getGroupId(), member.getUserIds(), member.getGroupRoleType().getCode());

			for (List<String> items : Lists.partition(member.getUserIds(), 50)) {

				items.parallelStream().forEach(item -> {

					try {
						ImPluginService pluginService = imPluginContext.getPluginService();
						switch (member.getGroupRoleType()) {
							case Owner:
								// 设置群主
								pluginService.changeGroupOwner(member.getGroupId(), item,null);
								break;
							case Admin:
								// 设置管理员
								pluginService.groupAdminCreate(member.getGroupId(), item);
								break;
							case Member:
								// 取消管理员
								pluginService.groupAdminRemove(member.getGroupId(), item);
								break;
							default:
								break;
						}
					} catch (Exception e) {
						log.error("群成员身份更新异常", e);
						throw com.microsvc.toolkit.common.webportal.exception.BizException.from("群成员身份更新异常，请联系管理员!");
					}
				});
			}

		} catch (Exception e) {
			log.error("群成员身份更新异常", e);
			throw com.microsvc.toolkit.common.webportal.exception.BizException.from("群成员身份更新异常，请联系管理员!");
		}
	}

	@Override
	public void groupMemberRole1(ImGroupMemberWrapper.GroupMemberRole member) {
		ImPluginService pluginService = imPluginContext.getPluginService();
		try {
			for (String userId : member.getUserIds()) {
				switch (member.getGroupRoleType()) {
					case Owner:
						// 设置群主
						pluginService.changeGroupOwner(member.getGroupId(), userId,null);
						break;
					case Admin:
						// 设置管理员
						pluginService.groupAdminCreate(member.getGroupId(), userId);
						break;
					case Member:
						// 取消管理员
						pluginService.groupAdminRemove(member.getGroupId(), userId);
						break;
					default:
						break;
				}
			}
			imGroupMemberDao.batchUpdateGroupMemberRole(member.getGroupId(), member.getUserIds(), member.getGroupRoleType().getCode());
		}catch (Exception e) {
			log.error("群成员身份更新异常", e);
		}
	}
}
