package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.middleware.common.http.DateUtil;
import com.microsvc.toolkit.middleware.common.http.ImageUtil;
import com.microsvc.toolkit.middleware.im.ImPluginContext;
import com.microsvc.toolkit.middleware.im.message.GroupMemberWrapper;
import com.microsvc.toolkit.middleware.im.message.MessageWrapper;
import com.microsvc.toolkit.middleware.im.message.TencentRequest;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ImGroupDto;
import com.ym.mec.biz.dal.dto.ImGroupMemberDto;
import com.ym.mec.biz.dal.dto.NameDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.EFriendRoleType;
import com.ym.mec.biz.dal.enums.MusicGroupStatusEnum;
import com.ym.mec.biz.dal.enums.im.ClientEnum;
import com.ym.mec.biz.dal.enums.im.EImGroupRoleType;
import com.ym.mec.biz.dal.enums.school.ESchoolStaffType;
import com.ym.mec.biz.dal.mapper.SchoolStaffMapper;
import com.ym.mec.biz.dal.page.GroupMemberQueryInfo;
import com.ym.mec.biz.dal.wrapper.ImGroupMemberWrapper;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;
import com.ym.mec.biz.dal.wrapper.ImHistoryMessageWrapper;
import com.ym.mec.biz.dal.wrapper.SchoolStaffWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.UploadReturnBean;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.yonge.log.dal.model.HistoryMessage;
import com.yonge.log.dal.model.HistoryMessageTencent;
import com.yonge.log.service.HistoryMessageService;
import com.yonge.log.service.HistoryMessageTenantService;
import com.yonge.log.wrapper.HistoryMessageTencentWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ImGroupServiceImpl extends BaseServiceImpl<String, ImGroup> implements ImGroupService {

    @Resource
    private ImGroupDao imGroupDao;

    @Resource
    private ImGroupMemberDao imGroupMemberDao;

    @Resource
    private ImGroupNoticeDao imGroupNoticeDao;

    @Resource
    private ImGroupMemberService imGroupMemberService;

    @Resource
    private SysUserService sysUserService;

    @Resource
    private ImUserFriendDao imUserFriendDao;

    @Resource
    private SysConfigService sysConfigService;

    @Resource
    private CooperationOrganService cooperationOrganService;

    @Resource
    private SchoolStaffMapper schoolStaffMapper;

    @Resource
    private MusicGroupDao musicGroupDao;

    @Resource
    private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;

    @Autowired
    private TeacherService teacherService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private SchoolStaffService schoolStaffService;

    @Autowired
    private EmployeeService employeeService;

    @Autowired
    private UploadFileService uploadFileService;

    @Autowired
    private HistoryMessageService historyMessageService;

    @Resource
    private ImGroupCoreService imGroupCoreService;

    @Autowired
    private ImPluginContext imPluginContext;

    @Autowired
    private HistoryMessageTenantService historyMessageTencentService;

    @Autowired
    private ImGroupService imGroupService;

    @Override
    public BaseDAO<String, ImGroup> getDAO() {
        return imGroupDao;
    }

    @Resource
    private TeacherDao teacherDao;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ImGroup create(String id, Integer userId, String name, String introduce, String memo, String tags, String img, String type, ImGroup.GroupTypeEnum groupType) {
        ImGroup imGroup = imGroupDao.get(id);
        if (imGroup != null) {
            return imGroup;
        }

        imGroup = new ImGroup();

        Date date = new Date();

        imGroup.setId(id);
        imGroup.setCreateTime(date);
        imGroup.setIntroduce(introduce);

        if(userId != null){
            imGroup.setMemberNum(1);
        }else{
            imGroup.setMemberNum(0);
        }
        imGroup.setMemo(memo);
        imGroup.setName(name);
        imGroup.setTags(tags);
        imGroup.setImg(img);
        imGroup.setType(type);
        imGroup.setGroupType(groupType);
        imGroup.setUpdateTime(date);

        try {
            String imGroupId = createImGroup(imGroup, userId, null);
            imGroup.setId(imGroupId);
        } catch (Exception e) {
            log.error("群组创建失败",e);
            throw new BizException("群组创建失败");
        }

        return imGroup;
    }


    private String createImGroup(ImGroup imGroup, Integer userId, String roleType) throws Exception {
        ImGroupWrapper.ImGroup group = ImGroupWrapper.ImGroup.from(JSON.toJSONString(imGroup));
        //记录群成员
        com.ym.mec.biz.dal.wrapper.ImGroupMemberWrapper.ImGroupMember groupMember = null;
        if (userId != null) {
            SysUser sysUser = teacherDao.getUser(userId);
            groupMember = new ImGroupMemberWrapper.ImGroupMember();
            groupMember.setUserId(userId.longValue());
            groupMember.setClientType("");
            groupMember.setAvatar(sysUserService.getImAvatar(sysUser));
            groupMember.setRoleType(roleType);
            groupMember.setTenantId(Optional.ofNullable(imGroup.getTenantId()).orElse(TenantContextHolder.getTenantId()));
            groupMember.setGroupRoleType(StringUtils.isEmpty(roleType)?EImGroupRoleType.Member.getCode():EImGroupRoleType.Admin.getCode());
            if(StringUtils.isNotEmpty(roleType)){
                groupMember.setType(EFriendRoleType.getByName(roleType).name());
                groupMember.setNickname(StringUtils.isEmpty(sysUser.getRealName())?sysUser.getUsername():sysUser.getRealName());
            }else {
                groupMember.setNickname(StringUtils.isEmpty(sysUser.getUsername())?sysUser.getRealName():sysUser.getUsername());
            }
            if(StringUtils.equals(roleType,"乐团领队")){
                groupMember.setIsAdmin(true);
                groupMember.setGroupRoleType(EImGroupRoleType.Owner.getCode());
            }
        }
        return imGroupCoreService.groupCreate(groupMember, group);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createGroup(ImGroupDto imGroupDto){
        List<String> userIdList = imGroupDto.getUserIdList();
        Integer currentUserId = null;
        if (CollectionUtils.isEmpty(userIdList)){
            userIdList.add(sysUserService.getUserId().toString());
        }
        String groupName = imGroupDto.getGroupName();
        String type = imGroupDto.getType();
        Optional.ofNullable(groupName).filter(StringUtils::isNotBlank).orElseThrow(() -> new BizException("群名称不能为空"));
        Optional.ofNullable(type).filter(StringUtils::isNotBlank).orElseThrow(() -> new BizException("群类型不能为空"));
        Date date = new Date();
        ImGroup imGroup = new ImGroup();
        String groupId = UUID.randomUUID().toString();
        imGroup.setId(groupId);
        imGroup.setCreateTime(date);
        imGroup.setIntroduce(groupName);
        imGroup.setMemo(groupName);
        imGroup.setMemberNum(userIdList.size());
        imGroup.setName(groupName);
        imGroup.setType(type);
        imGroup.setGroupType(imGroupDto.getGroupType());
        imGroup.setUpdateTime(date);
        try {
            String imGroupId = createImGroup(imGroup, null, null);
            imGroup.setId(imGroupId);
        } catch (Exception e) {
            log.error("群组创建失败",e);
            throw new BizException("群组创建失败");
        }
        //当前用户不知道具体角色，由于是通过后台添加，所以默认给了乐团主管
        List<ImGroupMember> groupMemberList = imGroupMemberDao.queryMembers(groupId, userIdList, TenantContextHolder.getTenantId());
        imGroupMemberService.initImGroupMembers(groupMemberList,imGroup.getGroupType());
        addImGroupMember(groupId, groupMemberList);
        return groupId;
    }

    private void addImGroupMember(String groupId, List<ImGroupMember> groupMemberList) {
        ImGroup imGroup = imGroupDao.get(groupId);
        List<com.ym.mec.biz.dal.wrapper.ImGroupMemberWrapper.ImGroupMember> imGroupMembers = Lists.newArrayList();
        for (ImGroupMember groupMember : groupMemberList) {
            SysUser user = teacherDao.getUser(groupMember.getUserId());
            imGroupMembers.add(ImGroupMemberWrapper.ImGroupMember.builder()
                    .groupId(groupMember.getImGroupId())
                    .userId(groupMember.getUserId().longValue())
                    .avatar(sysUserService.getImAvatar(user))
                    .nickname(sysUserService.getImName(user))
                    .isAdmin(groupMember.getIsAdmin())
                    .groupRoleType(groupMember.getGroupRoleType())
                    .tenantId(Optional.ofNullable(imGroup.getTenantId()).orElse(TenantContextHolder.getTenantId()))
                    .imUserId(getImUserId(groupMember.getUserId().toString(), ""))
                    .roleType(groupMember.getRoleType())
                    .type(groupMember.getType())
                    .build());
        }
        try {
            imGroupCoreService.saveImGroupMemberList(groupId, imGroupMembers);
        } catch (Exception e) {
            log.error("添加群成员用户失败",e);
            throw new BizException("添加群成员用户失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean cancel(String imGroupId) {
        try {
            imGroupCoreService.groupDismiss(imGroupId);
        } catch (Exception e) {
            log.error("解散群组失败",e);
            throw new BizException("解散群组失败");
        }

        return true;
    }

    @Override
    public List<ImGroup> queryByUserId(Integer userId, String search,String groupType) {
        List<ImGroup> imGroups = imGroupDao.queryByUserId(userId, search, groupType).stream()
            .filter(Objects::nonNull).collect(Collectors.toList());
        for (ImGroup item : imGroups) {
            if (Objects.nonNull(item.getGroupType()) && StringUtils.isBlank(item.getImg())) {
                item.setImg(item.getGroupType().getAvatar());
            }
        }
        return imGroups;
    }

    @Override
    public List<ImGroupMemberDto> queryMemberById(String imGroupId) {
        return getGroupNickname(queryMemberById(imGroupId, null));
    }


    @Override
    public List<ImGroupMemberDto> queryMemberById(String imGroupId,String search) {
        return getGroupNickname(imGroupDao.queryMemberById(imGroupId,search));
    }

    /**
     * 群用户昵称
     * @param imGroupMemberDtos List<ImGroupMemberDto>
     * @return List<ImGroupMemberDto>
     */
    private List<ImGroupMemberDto> getGroupNickname(List<ImGroupMemberDto> imGroupMemberDtos) {

        for (ImGroupMemberDto imGroupMemberDto : imGroupMemberDtos) {
            imGroupMemberDto.setType(EFriendRoleType.getByName(imGroupMemberDto.getRoleType()).name());
            imGroupMemberDto.getUser().setUsername(imGroupMemberDto.getGroupNickname());
            imGroupMemberDto.setNickname(imGroupMemberDto.getGroupNickname());

            if (Objects.nonNull(imGroupMemberDto.getUser()) && StringUtils.isNotBlank(imGroupMemberDto.getUser().getUserType())) {

                List<String> userTypes = Lists.newArrayList(imGroupMemberDto.getUser().getUserType().split(","));
                if (userTypes.size() > 1
                    || userTypes.contains(ClientEnum.TEACHER.getCode())
                    || userTypes.contains(ClientEnum.SYSTEM.getCode())) {
                    // 增加用户身份数量 > 1 或者包含老师身份，统一使用实名
                    imGroupMemberDto.setGroupNickname(Optional.ofNullable(imGroupMemberDto.getUser().getRealName()).filter(StringUtils::isNotBlank).orElse(imGroupMemberDto.getUser().getUsername()));
                }
            }

            // 群成员头像
            if (Objects.nonNull(imGroupMemberDto.getUser())) {
                imGroupMemberDto.setAvatar(imGroupMemberDto.getUser().getAvatar());
            }
        }
        return imGroupMemberDtos;
    }

    @Override
    public ImGroupMemberDto queryMember(String imGroupId, Integer userId) {
        ImGroupMemberDto imGroupMemberDto = imGroupDao.queryMember(imGroupId, userId);
        if(Objects.isNull(imGroupMemberDto)){
            imGroupMemberDto = imGroupDao.queryMember(null, userId);
            if (Objects.nonNull(imGroupMemberDto)){
                imGroupMemberDto.setRoleType(null);
            }
        }
        return imGroupMemberDto;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateNickName(Integer userId, String nickName) {
        //修改群成员备注
        int i = imGroupDao.updateNickname(userId, nickName);
        imGroupDao.updateUserFriendNickname(userId, nickName);
        return i;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateNickName1(Integer userId, String nickName) {
        //修改群成员备注
        imGroupDao.updateNickname(userId, nickName);
        imGroupDao.updateUserFriendNickname(userId, nickName);
        //修改sysUser名称，如果包含学员的角色，那么修改userName。否则修改realName
        SysUser sysUser = sysUserService.queryUserById(userId);
        if(sysUser.getUserType().contains("STUDENT")){
            sysUser.setUsername(nickName);
        }else {
            sysUser.setRealName(nickName);
        }
        sysUserService.updateSysUser(sysUser);

        // IM用户信息更新
        try {
            imGroupCoreService.register(userId.toString(),"",nickName,sysUserService.getImAvatar(sysUser));
        } catch (Exception e) {
            log.error("更新用户信息失败",e);
            throw new BizException("更新用户信息失败");
        }

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateImGroup(ImGroup imGroup) {
        imGroup.setMemberNum(null);
        imGroupDao.update(imGroup);
        try {
            imGroupCoreService.modifyGroupInfo(imGroup.getId(),imGroup.getName(),imGroup.getImg());
        } catch (Exception e) {
            log.error("群信息修改失败",e);
            throw new BizException("群信息修改失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void dismissGroup(String groupId,String userId) {
        ImGroup imGroup = imGroupService.get(groupId);
        if (imGroup == null) {
            throw new BizException("群组不存在");
        }
        if(StringUtils.isNotEmpty(userId)){
            ImGroupMember admin = imGroupMemberService.getDao().getAdmin(groupId);
            if (admin == null) {
                throw new BizException("群主信息不存在");
            }
            if (!StringUtils.equals(admin.getUserId().toString(), userId)) {
                throw new BizException("当前操作人不是群主");
            }
        }
        // 删除群成员
        imGroupMemberDao.deleteByImGroupId(groupId);
        // 删除群公告
        imGroupNoticeDao.deleteByImGroupId(groupId);
        // 删除群信息
        imGroupDao.delete(groupId);
        try {
            imPluginContext.getPluginService().groupDismiss(groupId,new ArrayList<>());
        } catch (Exception e) {
            log.error("解散群组失败",e);
            throw new BizException("解散群组失败");
        }
    }

    @Override
    public List<NameDto> queryDetail(List<NameDto> nameDto) {
        List<NameDto> dtos = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(nameDto)){
            Map<Integer, List<NameDto>> collect = nameDto.stream().collect(Collectors.groupingBy(NameDto::getType));
            for (Integer integer : collect.keySet()) {
                List<NameDto> nameDtos = collect.get(integer);
                if(CollectionUtils.isNotEmpty(nameDtos)){
                    String idList = nameDtos.stream().map(NameDto::getId).collect(Collectors.joining(","));
                    if(integer.equals(3)){
                        dtos.addAll(imGroupDao.queryNameByIds(idList));
                    }else if (integer.equals(1)){
                        Integer userId = sysUserService.getUserId();
                        dtos.addAll(imUserFriendDao.queryNameByIds(idList,userId));
                    }
                }
            }
        }
        return dtos;
    }

    @Override
    public List<ImGroup> queryByUserIdV2(Integer userId, String search, String groupType, String musicGroupId, String classType) {
        return imGroupDao.queryByUserIdV2(userId, search, groupType, musicGroupId, classType);
    }

    @Override
    public List<ImGroupMemberDto> queryMemberByIdV2(String imGroupId, String search, Integer subjectId, Boolean vipFlag) {
        return imGroupDao.queryMemberByIdV2(imGroupId,search,subjectId,vipFlag);
    }

    /**
     * 用户退出所有群
     * <p>
     * 删除时判断是否群主，
     * 如果是，则随机分配一个群主，
     * 如果删除的是最后一个群成员，那么解散群
     * @param userId 用户id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void quitAllGroup(Long userId) {

        List<ImGroupMember> imGroupMembers = imGroupMemberDao.queryMembersByUserId(userId);

        if (CollectionUtils.isEmpty(imGroupMembers)) {
            return;
        }
        for (ImGroupMember imGroupMember : imGroupMembers) {
            // 如果是群主，随机分配一个群主
            if (imGroupMember.getIsAdmin()) {
                try {
                    // 获取一个群成员
                    ImGroupMember imGroupMember1 = imGroupMemberDao.queryOneByGroupIdAndNotUser(imGroupMember.getImGroupId(), userId);
                    if (Objects.nonNull(imGroupMember1)) {
                        groupChangeOwner(imGroupMember1.getUserId(),imGroupMember.getImGroupId());
                    }
                } catch (Exception e) {
                    log.error("更换群主失败{},{}", userId, imGroupMember.getImGroupId(), e);
                }
            }
            try {
                imGroupCoreService.groupQuit(userId,"",imGroupMember.getImGroupId());
            } catch (Exception e) {
                log.error("用户退出群失败{},{}", userId, imGroupMember.getImGroupId(), e);
            }
        }
        List<String> imGroupIds = imGroupMembers.stream().map(ImGroupMember::getImGroupId).collect(Collectors.toList());
        // 群组没人，删除群组
        List<ImGroup> imGroups = imGroupDao.queryByUserId(userId.intValue(),null, ImGroup.GroupTypeEnum.SCHOOL.getCode()).stream()
            .filter(Objects::nonNull).collect(Collectors.toList());


        imGroupMemberDao.deleteByUserId(userId);
        // 更新群组人员数
        imGroupDao.addMemberNum(imGroupIds,-1);

        if (CollectionUtils.isEmpty(imGroups)) {
            return;
        }


        List<ImGroup> groups = imGroups.stream().filter(imGroup -> imGroup.getMemberNum().equals(1)).collect(Collectors.toList());
        for (ImGroup group : groups) {
            try {
                dismissGroup(group.getId(),userId.toString());
            } catch (Exception e) {
                log.error("解散群组失败{},{}", userId, group.getId(), e);
            }
            imGroupDao.delete(group.getId());
        }
    }

    /**
     * 用户加入群
     *
     * @param schoolId 学校id
     * @param userId   用户id
     * @param schoolStaffType   是否是群主
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addSchoolStaffGroup(Long schoolId, Integer userId,String name,String avatar, ESchoolStaffType schoolStaffType) throws Exception {

        // 查询群是否存在
        ImGroup imGroup = imGroupDao.queryBySchoolId(schoolId);


        // 注册主播用户信息到三方平台
        imGroupCoreService.register(userId.toString(),"", name, sysUserService.getImAvatar(teacherDao.getUser(userId)));

        boolean createFlag = false;
        // 群组不存在，先创建群组
        if (Objects.isNull(imGroup)) {
            createFlag = true;
            CooperationOrgan cooperationOrgan = cooperationOrganService.get(schoolId.intValue());
            imGroup = new ImGroup();
            imGroup.setSchoolId(schoolId);
            imGroup.setId(IdWorker.getIdStr());
            imGroup.setName("教学团队衔接群");
            imGroup.setIntroduce(imGroup.getName());
            imGroup.setMemo(imGroup.getName());
            imGroup.setMemberNum(0);
            //imGroup.setImg(sysConfigService.findByParamName(SysConfigService.SCHOOL_IM_GROUP_IMG).getParanValue());
            imGroup.setType(ImGroup.GroupTypeEnum.SCHOOL.getCode());
            imGroup.setGroupType(ImGroup.GroupTypeEnum.SCHOOL);
            imGroup.setCreateTime(new Date());
            imGroup.setUpdateTime(new Date());
            imGroup.setTenantId(cooperationOrgan.getTenantId());

            createImGroup(imGroup,userId,schoolStaffType.getDescribe());

            initSchoolGroup(imGroup.getSchoolId());
        } else {
            SysUser user = teacherDao.getUser(userId);
            ImGroupMember imGroupMember = new ImGroupMember();
            imGroupMember.setImGroupId(imGroup.getId());
            imGroupMember.setUserId(userId);
            imGroupMember.setRoleType(schoolStaffType.getDescribe());
            imGroupMember.setNickname(name);
            if (StringUtils.isNotEmpty(schoolStaffType.getDescribe())) {
                imGroupMember.setType(EFriendRoleType.getByName(schoolStaffType.getDescribe()).name());
            }
            imGroupMember.setTenantId(imGroup.getTenantId());
            imGroupMemberService.initImGroupMember(imGroupMember,imGroup.getGroupType());

            ImGroupMemberWrapper.ImGroupMember groupMember = new ImGroupMemberWrapper.ImGroupMember();
            groupMember.setUserId(userId.longValue());
            groupMember.setImUserId(userId.toString());
            groupMember.setNickname(name);
            groupMember.setAvatar(sysUserService.getImAvatar(user));
            groupMember.setGroupId(imGroupMember.getImGroupId());
            groupMember.setType(imGroupMember.getType());
            groupMember.setRoleType(imGroupMember.getRoleType());
            groupMember.setGroupRoleType(imGroupMember.getGroupRoleType());
            groupMember.setIsAdmin(imGroupMember.getIsAdmin());
            imGroupCoreService.groupMemberJoin(imGroup.getId(), Lists.newArrayList(groupMember));
            // 更新群主流程
            /*if (schoolStaffType.equals(ESchoolStaffType.ORCHESTRA_LEADER)) {
                groupChangeOwner(userId, imGroup.getId());
            }*/
            imGroupMemberDao.insert(imGroupMember);

        }
        updateImGroupUserNumber(imGroup.getId());

    }

    private void initSchoolGroup(Long schoolId) {


        // 查询乐团主管
        List<MusicGroup> musicGroupList = musicGroupDao.findByCooperationId(schoolId.intValue());
        if (CollectionUtils.isEmpty(musicGroupList)) {
            return;
        }
        musicGroupList = musicGroupList.stream()
                .filter(o -> o.getStatus().equals(MusicGroupStatusEnum.PROGRESS))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(musicGroupList)) {
            return;
        }

        // 乐团主管
        List<Integer> groupLeaderIds = musicGroupList.stream().map(MusicGroup::getEducationalTeacherId).collect(Collectors.toList());

        // 乐团ID集合
        List<String> musicGroupIds = musicGroupList.stream().map(MusicGroup::getId).collect(Collectors.toList());

        // 班级列表主教/助教老师
        List<ClassGroupTeacherMapper> teacherMapperList = classGroupTeacherMapperDao.findAllByMusicGroupIds(musicGroupIds);

        List<Integer> teacherIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(teacherMapperList)) {
            teacherIds.addAll(teacherMapperList.stream().map(ClassGroupTeacherMapper::getUserId).collect(Collectors.toSet()));
        }

        List<GroupMemberWrapper.ImGroupMember> imGroupMembers = new ArrayList<>();
        for (Integer groupLeaderId : groupLeaderIds) {
            imGroupMembers.add(GroupMemberWrapper.ImGroupMember.builder()
                    .userId(groupLeaderId.longValue()).roleType(EFriendRoleType.ORCHESTRA_MANAGER.getMsg()).imUserIdFormat(false).build());
        }

        for (Integer teacherId : teacherIds) {
            imGroupMembers.add(GroupMemberWrapper.ImGroupMember.builder()
                    .userId(teacherId.longValue()).roleType(EFriendRoleType.ORCHESTRA_TEACHER.getMsg()).imUserIdFormat(false).build());
        }

        addSchoolGroup(schoolId, imGroupMembers);
    }

    /**
     * 加入学校群聊
     *
     * @param schoolId 学校ID
     * @param imGroupMembers List<GroupMemberWrapper.ImGroupMember>
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addSchoolGroup(Long schoolId, List<GroupMemberWrapper.ImGroupMember> imGroupMembers) {
        // 添加乐团领队，班级老师
        List<ImGroupMember> imGroupMemberList =new ArrayList<>();


        // 查询群是否存在
        ImGroup imGroup = imGroupDao.queryBySchoolId(schoolId);
        if (imGroup == null) {
            return;
        }
        if (CollectionUtils.isEmpty(imGroupMembers)) {
            return;
        }

        for (GroupMemberWrapper.ImGroupMember groupMember : imGroupMembers) {
            ImGroupMember imGroupMember = new ImGroupMember();
            imGroupMember.setImGroupId(imGroup.getId());
            imGroupMember.setUserId(groupMember.getUserId().intValue());
            imGroupMember.setIsAdmin(false);
            imGroupMember.setRoleType(groupMember.getRoleType());
            imGroupMember.setNickname(groupMember.getNickname());
            if (StringUtils.isNotEmpty(groupMember.getRoleType())) {
                imGroupMember.setType(EFriendRoleType.getByName(groupMember.getRoleType()).name());
            }
            imGroupMember.setTenantId(imGroup.getTenantId());
            imGroupMemberList.add(imGroupMember);
            groupMember.setImUserIdFormat(false);
        }

        imGroupMemberService.initImGroupMembers(imGroupMemberList,imGroup.getGroupType());
        addImGroupMember(imGroup.getId(), imGroupMemberList);
        updateImGroupUserNumber(imGroup.getId());
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void quitSchoolGroup(Long schoolId, List<GroupMemberWrapper.ImGroupMember> imGroupMembers) {
        // 添加乐团领队，班级老师


        // 查询群是否存在
        ImGroup imGroup = imGroupDao.queryBySchoolId(schoolId);
        if (imGroup == null) {
            return;
        }

        if (CollectionUtils.isEmpty(imGroupMembers)) {
            return;
        }

        //List<Integer> userIds = new ArrayList<>();
        for (GroupMemberWrapper.ImGroupMember imGroupMember : imGroupMembers) {
            imGroupMember.setImUserId(String.valueOf(imGroupMember.getUserId()));
            //userIds.add(imGroupMember.getUserId().intValue());
        }
        try {
            imGroupCoreService.groupQuit(imGroupMembers,imGroup.getId());
        } catch (Exception e) {
            log.error("退出群成员失败{}", imGroup.getId(), e);
        }
        updateImGroupUserNumber(imGroup.getId());
    }

    private void updateImGroupUserNumber(String imGroupId) {
        ImGroup imGroup = new ImGroup();
        imGroup.setId(imGroupId);
        imGroup.setMemberNum(imGroupMemberDao.getByGroupId(Lists.newArrayList(imGroupId)).size());
        imGroupDao.updateMember(imGroup.getId(),imGroup.getMemberNum());
    }

    /**
     * 群主移交
     * @param userId 用户id
     * @param imGroupId     群组id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean groupChangeOwner(Integer userId, String imGroupId) throws Exception {


        ImGroupMember imGroupMember = imGroupMemberDao.getAdmin(imGroupId);
        Integer adminId = userId;
        if (Objects.nonNull(imGroupMember)) {
            adminId = imGroupMember.getUserId();
            imGroupMember.setIsAdmin(false);
            imGroupMemberDao.update(imGroupMember);
        }
        imGroupCoreService.changeGroupOwner(imGroupId, userId.toString(), adminId.toString());

        imGroupMemberDao.updateAdmin(imGroupId, userId);
        return true;
    }

    /**
     * 加群
     *
     * @param imGroupIds    群组ID
     * @param userId          用户信息
     * @param roleType      用户类型
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addImGroup(List<String> imGroupIds, Long  userId, String roleType) {
        if (CollectionUtils.isEmpty(imGroupIds)) {
            return;
        }
        List<ImGroup> groupList = imGroupDao.getByIds(imGroupIds);
        // id 分组
        Map<String, ImGroup> map = groupList.stream().collect(Collectors.toMap(ImGroup::getId, Function.identity()));
        for (String musicGroupId : imGroupIds) {
            ImGroup imGroup = map.get(musicGroupId);
            if (imGroup == null) {
                continue;
            }
            try {

                SysUser user = teacherDao.getUser(userId.intValue());
                ImGroupMember imGroupMember = new ImGroupMember();
                imGroupMember.setImGroupId(imGroup.getId());
                imGroupMember.setUserId(userId.intValue());
                imGroupMember.setIsAdmin(false);
                imGroupMember.setRoleType(roleType);
                if(StringUtils.isNotEmpty(roleType)){
                    imGroupMember.setType(EFriendRoleType.getByName(roleType).name());
                    imGroupMember.setNickname(StringUtils.isEmpty(user.getRealName())?user.getUsername():user.getRealName());
                }else {
                    imGroupMember.setNickname(StringUtils.isEmpty(user.getUsername())?user.getRealName():user.getUsername());
                }
                imGroupMember.setType(EFriendRoleType.getByName(roleType).name());
                imGroupMember.setTenantId(imGroup.getTenantId());
                imGroupMemberService.initImGroupMember(imGroupMember,imGroup.getGroupType());

                ImGroupMemberWrapper.ImGroupMember groupMember = new ImGroupMemberWrapper.ImGroupMember();
                groupMember.setUserId(userId);
                groupMember.setNickname(imGroupMember.getNickname());
                groupMember.setAvatar(sysUserService.getImAvatar(user));
                groupMember.setRoleType(roleType);
                groupMember.setType(imGroupMember.getType());
                groupMember.setGroupId(imGroupMember.getImGroupId());
                groupMember.setGroupRoleType(imGroupMember.getGroupRoleType());
                groupMember.setIsAdmin(imGroupMember.getIsAdmin());
                imGroupCoreService.groupMemberJoin(imGroup.getId(), Lists.newArrayList(groupMember));

                imGroupMemberDao.insert(imGroupMember);
                updateImGroupUserNumber(imGroup.getId());
            } catch (Exception e) {
                log.error("加入群聊失败",e);
            }

        }
    }

    /**
     * 查询群组
     *
     * @param query 查询条件
     * @return 群组列表
     */
    @Override
    public List<ImGroup> queryByUserId(ImGroupWrapper.ImQuery query) {
        List<ImGroup> imGroups = imGroupDao.getByUserId(query).stream()
            .filter(Objects::nonNull).collect(Collectors.toList());

        // 群默认头像
        for (ImGroup item : imGroups) {
            if (Objects.nonNull(item.getGroupType()) && StringUtils.isBlank(item.getImg())) {
                item.setImg(item.getGroupType().getAvatar());
            }
        }
        return imGroups;
    }

    /**
     * 退出群组
     *
     * @param imGroupIds 群组ID
     * @param userId     用户ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void quitGroup(List<String> imGroupIds, Integer userId) {
        if (CollectionUtils.isEmpty(imGroupIds)) {
            return;
        }

        List<ImGroup> groupList = imGroupDao.getByIds(imGroupIds);

        List<com.ym.mec.biz.dal.entity.ImGroupMember> imGroupMemberList = new ArrayList<>();
        for (ImGroup imGroup : groupList) {
            try {
                imGroupCoreService.groupQuit(userId.longValue(), "",imGroup.getId());
            } catch (Exception e) {
                log.error("退出群聊失败",e);
            }
            ImGroupMember imGroupMember = new ImGroupMember();
            imGroupMember.setImGroupId(imGroup.getId());
            imGroupMember.setUserId(userId);
            imGroupMemberList.add(imGroupMember);
        }

        imGroupMemberService.batchDeleteByGroupIdAndUserId(imGroupMemberList);
        for (String imGroupId : imGroupIds) {
            updateImGroupUserNumber(imGroupId);
        }
    }

    /**
     * 学校人员加入群组
     *
     * @param cooperationOrganId 合作机构id
     * @param imGroupId 群组id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void schoolJoinGroup(Integer cooperationOrganId, String imGroupId) {
        CooperationOrgan cooperationOrgan = cooperationOrganService.get(cooperationOrganId);
        if (cooperationOrgan == null) {
            return;
        }
        List<SchoolStaff> schoolStaffs = schoolStaffMapper.getByCoopIdAndType(cooperationOrganId, ESchoolStaffType.ORCHESTRA_LEADER);
        if (CollectionUtils.isEmpty(schoolStaffs)) {
            return;
        }
        // 查询群是否存在
        ImGroup imGroup = imGroupDao.get(imGroupId);
        if (imGroup == null) {
            return;
        }
        SchoolStaff schoolStaff = schoolStaffs.get(0);
        addImGroup(Lists.newArrayList(imGroup.getId()),schoolStaff.getUserId(),ESchoolStaffType.ORCHESTRA_LEADER.getDescribe());
    }


    /**
     * IM 用户注册
     *
     * @param userId 用户Id
     * @return String
     */
    @Override
    public String getImUserId(String userId, String clientType) {
        /*if (StringUtils.isNotBlank(imConfig.getAppPrefix()) && !userId.startsWith(imConfig.getAppPrefix())) {
            imUserId = MessageFormat.format("{0}_{1}", imConfig.getAppPrefix(), userId);
        }*/
        return userId;
    }


    /**
     * 群迁移
     */
    @Override
    public void groupTransfer(String startTime, String endTime,Integer type) {

        // 100个线程的无界线程池

        // 100个线程的无界线程池
        ThreadPoolExecutor executorService =  new ThreadPoolExecutor(100, 100,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<Runnable>());
        int page = 1;
        int size = 100;
        QueryInfo queryInfo = new QueryInfo();
        queryInfo.setPage(page);
        queryInfo.setRows(size);
//        queryInfo.setImportFlag(false);
        PageInfo<ImGroup> imGroupPageInfo = this.queryPage(queryInfo);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        int total = imGroupPageInfo.getTotal();
        int importImGroupCount = 0;
        int num = (int) Math.ceil(total / 100);
        for (int i = 0; i <=num ; i++) {
            queryInfo.setPage(page);
            imGroupPageInfo = this.queryPage(queryInfo);
            List<ImGroup> rows = imGroupPageInfo.getRows();
            try {
                if (StringUtils.isNotEmpty(startTime)) {
                    Date startDate = sdf.parse(startTime);
                    rows.removeIf(next -> next.getCreateTime().before(startDate));
                }

                if (StringUtils.isNotEmpty(endTime)) {
                    Date endDate = sdf.parse(endTime);
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(endDate);
                    calendar.set(Calendar.HOUR_OF_DAY, 23);
                    calendar.set(Calendar.MINUTE, 59);
                    calendar.set(Calendar.SECOND, 59);
                    rows.removeIf(next -> next.getCreateTime().after(calendar.getTime()));
                }
            } catch (ParseException e) {
                throw new BizException("时间区间参数错误，格式为：yyyy-MM-dd");
            }

            if (type == 1) {
                rows.forEach(imGroup -> {

                    executorService.submit(() -> {
                        log.info("群销毁开始：{}", imGroup.getId());
                        // 先删除群组
                        try {
                            // 解散群
                            imPluginContext.getPluginService().groupDismiss(imGroup.getId(), new ArrayList<>());
                        } catch (Exception e) {
                            log.error(String.format("群迁移删除群聊失败：%s", e.getMessage()), e);
                        }
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                            log.error("线程等待异常", e);
                        }
                    });
                });
            } else {

                for (ImGroup row : rows) {
                    executorService.execute(() -> {
                        groupTransfer(Lists.newArrayList(row));
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                            log.error("线程等待异常", e);
                        }
                    });
                }

            }
            page ++;
            importImGroupCount += rows.size();
            log.info("------------------------------- import im group --------------------------------------------");
            log.info("import im group success count:{}/{}", importImGroupCount, total);
        }

        while (executorService.getQueue().size() > 0) {
            try {
                log.info("线程池中正在执行的任务数量：{},等待数量：{}", executorService.getActiveCount(),executorService.getQueue().size());
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                log.error("线程等待异常", e);
            }
        }
        log.info("-------------------- import im group finished and success! -------------------------------");
        executorService.shutdown();
    }

    private void groupTransfer(List<ImGroup> records) {

        for (ImGroup imGroup : records) {

            log.info("群迁移开始：{}", imGroup.getId());

            GroupMemberQueryInfo groupMemberQueryInfo = new GroupMemberQueryInfo();
            groupMemberQueryInfo.setPage(1);
            groupMemberQueryInfo.setRows(9999);
            groupMemberQueryInfo.setImGroupId(imGroup.getId());
            PageInfo<ImGroupMemberDto> pageInfo = imGroupMemberService.queryGroupMemberPage(groupMemberQueryInfo);
            ImGroupMember admin = pageInfo.getRows().stream().filter(ImGroupMember::getIsAdmin).findFirst().orElse(null);
            if (admin == null) {
                log.info("群主不存在，groupId:{}",imGroup.getId());
//                continue;
            }
            try {
                MessageWrapper.ImportGroup importGroup = new MessageWrapper.ImportGroup();
                if (admin != null) {
                    importGroup.setOwnerAccount(getImUserId(admin.getUserId().toString(), getClientType(admin.getRoleType())));
                }
                importGroup.setType("Public");
                importGroup.setGroupId(imGroup.getId());
                importGroup.setName(StringUtils.isBlank(imGroup.getName())?imGroup.getId():imGroup.getName());
                importGroup.setIntroduction(imGroup.getIntroduce());
                importGroup.setNotification(imGroup.getMemo());
                importGroup.setFaceUrl(imGroup.getImg());
                if (StringUtils.isEmpty(imGroup.getImg())) {
                    importGroup.setFaceUrl(Optional.ofNullable(imGroup.getGroupType()).orElse(ImGroup.GroupTypeEnum.NORMAL).getAvatar());
                }
                importGroup.setApplyJoinOption("FreeAccess");
                if (imGroup.getCreateTime() == null) {
                    imGroup.setCreateTime(DateTime.now().plusYears(-1).toDate());
                }
                importGroup.setCreateTime(imGroup.getCreateTime().getTime() / 1000);
//                if (admin != null) {
//                    register(admin.getUserId().toString(), getClientType(admin.getRoleType()),"");
//                }
                imPluginContext.getPluginService().importGroup(importGroup);

                List<ImGroupMember> groupMembers = pageInfo.getRows().stream()
                        .filter(next -> !next.getIsAdmin()).collect(Collectors.toList());


                groupMembers = groupMembers
                    .stream()
                    .filter(next -> next.getUserId() != null && next.getUserId() > 0)
                    .collect(Collectors.toList());

                if (groupMembers.isEmpty()) {
                    imGroupDao.updateImportStatusSuccess(imGroup.getId());
                    continue;
                }

//                for (ImGroupMember member : groupMembers) {
//                    try {
//                        register(member.getUserId().toString(), getClientType(member.getRoleType()),"");
//                    }catch (Exception e){
//                        log.error("用户注册失败",e);
//                    }
//                }
                MessageWrapper.ImportGroupMember importGroupMember = new MessageWrapper.ImportGroupMember();
                importGroupMember.setGroupId(imGroup.getId());
                Date date = new DateTime(imGroup.getCreateTime()).plusSeconds(3).toDate();
                List<MessageWrapper.ImportGroupMemberData> members =
                        groupMembers.stream().map(next -> {
                            MessageWrapper.ImportGroupMemberData data = new MessageWrapper.ImportGroupMemberData();
                            data.setMemberAccount(getImUserId(next.getUserId().toString(),
                                    getClientType(next.getRoleType())));
                            if (next.getCreateTime() == null){
                                next.setCreateTime(DateTime.now().plusMonths(-6).toDate());
                            }
                            if (next.getCreateTime().getTime() / 1000 <= importGroup.getCreateTime()) {
                                data.setJoinTime(date.getTime() / 1000);
                            } else {
                                data.setJoinTime(next.getCreateTime().getTime() / 1000);
                            }
                            data.setUnreadMsgNum(0);
                            return data;
                        }).collect(Collectors.toList());
                importGroupMember.setMemberList(members);
                List<List<MessageWrapper.ImportGroupMemberData>> partition = Lists.partition(members, 88);
                for (List<MessageWrapper.ImportGroupMemberData> importGroupMemberData : partition) {
                    importGroupMember.setMemberList(importGroupMemberData);
                    imPluginContext.getPluginService().importGroupMember(importGroupMember);
                }
                imGroupDao.updateImportStatusSuccess(imGroup.getId());
            } catch (Exception e) {
                log.error(String.format("群迁移失败,失败群组：%s", JSON.toJSONString(imGroup)));
                log.error("群迁移失败：", e);
            }
            log.info("群迁移成功：{}", imGroup.getId());
        }
    }

    private void register(String userId, String clientType,String avatar) {
        SysUser user = teacherDao.getUser(Integer.valueOf(userId));
        if (user == null) {
            return;
        }
        if (StringUtils.isEmpty(avatar)) {
            avatar = user.getAvatar();
        }
        try {
            String username = user.getUsername();

            // 老师默认使用真实姓名
            List<String> userTypes = Lists.newArrayList(user.getUserType().split(","));
            if (userTypes.size() > 1
                || userTypes.contains(ClientEnum.TEACHER.getCode())
                || userTypes.contains(ClientEnum.SYSTEM.getCode())) {
                username = Optional.ofNullable(user.getRealName()).filter(StringUtils::isNotBlank).orElse(username);
            }
            // 自动激活学生IM帐号
//            imGroupCoreService.register(userId,clientType, username, sysUserService.getImAvatar(user));
            imGroupCoreService.register(userId,clientType, username, avatar);
        } catch (Exception e) {
            log.error("register user error,userId:{}", userId);
            log.error("register user error", e);
        }
    }


    public String getClientType(String roleType) {
        EFriendRoleType byName = EFriendRoleType.getByName(roleType);
        if (EFriendRoleType.STUDENT.equals(byName)) {
            return "STUDENT";
        } else if (EFriendRoleType.ORCHESTRA_TEACHER.equals(byName)) {
            return "TEACHER";
        } else if (EFriendRoleType.ORCHESTRA_MANAGER.equals(byName) ||
                EFriendRoleType.MAINTENANCE_TECHNICIAN.equals(byName)) {
            return "EMPLOYEE";
        } else {
            return "SCHOOL";
        }
    }

    /**
     * 导入用户
     */
    @Override
    public void importUser() {
        int page = 1;
        int rows = 100;

        int count = 0;

        // 100个线程的无界线程池
        ThreadPoolExecutor executorService =  new ThreadPoolExecutor(100, 100,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<Runnable>());

        log.info("------------------------------- import user student ---------------------------");
        // 学生花费时间计算
        long start = System.currentTimeMillis();
        List<String[]> student = getStudent(page, rows);
        long end = System.currentTimeMillis();
        log.info("查询学生花费时间：{}毫秒", end - start);
        SysConfig studentAvatar = sysConfigService.findByParamName(SysConfigService.STUDENT_DEFAULT_AVATAR);
        while (!student.isEmpty()) {
            for (String[] next : student) {
                String avatar = next[2];
                if (StringUtils.isEmpty(avatar)) {
                    avatar = studentAvatar.getParanValue();
                }
                String finalAvatar = avatar;
                executorService.execute(() -> {
                    register(next[0], next[1], finalAvatar);
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        log.error("线程等待异常", e);
                    }
                });
            }

            count += student.size();
            log.info("import im student success count:{}", count);
            page++;
            student = getStudent(page, rows);
        }

        log.info("------------------------------- import user staff ---------------------------");
        // USER_DEFAULT_AVATAR
        count = 0;
        page = 1;
        SysConfig staffAvatar = sysConfigService.findByParamName(SysConfigService.USER_DEFAULT_AVATAR);
        start = System.currentTimeMillis();
        List<String[]> staff = getStaff(page, rows);
        end = System.currentTimeMillis();
        log.info("查询老师花费时间：{}毫秒", end - start);
        while (!staff.isEmpty()) {

            for (String[] next : staff) {
                String avatar = next[2];
                if (StringUtils.isEmpty(avatar)) {
                    avatar = staffAvatar.getParanValue();
                }
                String finalAvatar = avatar;
                executorService.execute(() -> {
                    register(next[0], next[1], finalAvatar);
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        log.error("线程等待异常", e);
                    }
                });
            }

            count += staff.size();
            log.info("import im staff success count:{}", count);
            page++;
            staff = getStaff(page, rows);
        }

        log.info("------------------------------- import user emps ---------------------------");
        // TEACHER_DEFAULT_AVATAR
        count = 0;
        page = 1;
        SysConfig empAvatar = sysConfigService.findByParamName(SysConfigService.TEACHER_DEFAULT_AVATAR);
        List<String[]> emps = getEmp(page, rows);
        while (!emps.isEmpty()) {
            for (String[] next : emps) {
                String avatar = next[2];
                if (StringUtils.isEmpty(avatar)) {
                    avatar = empAvatar.getParanValue();
                }
                String finalAvatar = avatar;
                executorService.execute(() -> {
                    register(next[0], next[1], finalAvatar);
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        log.error("线程等待异常", e);
                    }
                });
            }
            count += emps.size();
            log.info("import im emps success count:{}", count);
            page++;
            emps = getEmp(page, rows);
        }

        log.info("------------------------------- import user teacher ---------------------------");
        count = 0;
        page = 1;
        SysConfig teacherAvatar = sysConfigService.findByParamName(SysConfigService.TEACHER_DEFAULT_AVATAR);
        List<String[]> teachers = getTeachers(page, rows);
        while (!teachers.isEmpty()) {
            for (String[] next : teachers) {
                String avatar = next[2];
                if (StringUtils.isEmpty(avatar)) {
                    avatar = teacherAvatar.getParanValue();
                }
                String finalAvatar = avatar;
                executorService.execute(() -> {
                    register(next[0], next[1], finalAvatar);
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        log.error("线程等待异常", e);
                    }
                });
            }
            count += teachers.size();
            log.info("import im teacher success count:{}", count);
            page++;
            teachers = getTeachers(page, rows);
        }

        while (executorService.getQueue().size() > 0) {
            try {
                log.info("线程池中正在执行的任务数量：{},等待数量：{}", executorService.getActiveCount(),executorService.getQueue().size());
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                log.error("线程等待异常", e);
            }
        }
        executorService.shutdown();
        log.info("------------------------------- import user success ---------------------------");
    }

    @Override
    public int queryInfoCount(int targetType) {
        long count = historyMessageService.selectCount(0,new DateTime().plusMonths(-2).toDate(),targetType);
        return (int) count;
    }

    private List<String[]> getEmp(int page, int rows) {
        QueryInfo queryInfo = new QueryInfo();
        queryInfo.setPage(page);
        queryInfo.setRows(rows);
        PageInfo<Employee> employeePageInfo = employeeService.queryBaseInfoByPage(queryInfo);
        return employeePageInfo.getRows().stream().map(next -> {
            String[] strList = new String[3];
            strList[0] = next.getUserId().toString();
            strList[1] = "SYSTEM";
            strList[2] = next.getAvatar();
            return strList;
        }).collect(Collectors.toList());
    }

    private List<String[]> getStaff(int page, int rows) {
        IPage<SchoolStaffWrapper.SchoolStaff> staffPage = PageUtil.getPage(page, rows);
        IPage<SchoolStaffWrapper.SchoolStaff> iPage = schoolStaffService.selectBaseInfoByPage(staffPage,
                new SchoolStaffWrapper.SchoolStaffQuery());
        return iPage.getRecords().stream().map(next -> {
            String[] strList = new String[3];
            strList[0] = next.getUserId().toString();
            strList[1] = "SCHOOL";
            strList[2] = next.getAvatar();
            return strList;
        }).collect(Collectors.toList());
    }

    private List<String[]> getStudent(int page, int rows) {
        QueryInfo queryInfo = new QueryInfo();
        queryInfo.setPage(page);
        queryInfo.setRows(rows);
        PageInfo<Student> studentPageInfo = studentService.queryBaseInfoByPage(queryInfo);
        return studentPageInfo.getRows().stream().map(next -> {
            String[] strList = new String[3];
            strList[0] = next.getUserId().toString();
            strList[1] = "STUDENT";
            strList[2] = next.getAvatar();
            return strList;
        }).collect(Collectors.toList());
    }

    private List<String[]> getTeachers(int page, int rows) {
        QueryInfo queryInfo = new QueryInfo();
        queryInfo.setPage(page);
        queryInfo.setRows(rows);
        PageInfo<Teacher> teacherPageInfo = teacherService.queryBaseInfoByPage(queryInfo);
        return teacherPageInfo.getRows().stream().map(next -> {
            String[] strList = new String[3];
            strList[0] = next.getId().toString();
            strList[1] = "TEACHER";
            strList[2] = next.getAvatar();
            return strList;
        }).collect(Collectors.toList());
    }



    @Override
    public com.yonge.mongodb.PageInfo<HistoryMessage> getRongYunInfo(int page, int size, int targetType) {
        com.yonge.mongodb.PageInfo<HistoryMessage> pageInfo = historyMessageService.selectPage(page, size, 0,new DateTime().plusMonths(-2).toDate(),targetType);
        //List<HistoryMessage> imHistoryMessageslist = imGroupDao.selectAll(result,size);
        return pageInfo;
    }



    /**
     * IM导入消息
     */
    @Override
    public void importInfo(List<HistoryMessage> info) throws Exception {

        for (HistoryMessage i : info) {//判断消息类型
//            log.info("消息导入 HistoryMessage：{}", JSON.toJSONString(i));
            try {
                Integer type = i.getTargetType();
                if (type == 1) {
                    //单聊会话
                    MessageWrapper.PrivateImportMessage privateImportMessage = new MessageWrapper.PrivateImportMessage();
                    //2：表示历史消息导入，消息不计入未读计数，且消息不会推送到终端
                    privateImportMessage.setSyncFromOldSystem(2);
                    //设置发送人
                    //根据发送者分场景讨论
                    String id = i.getFromUserId();
                    String imUserId;
                    if (id.contains(":STUDENT")) {
                        String[] split = id.split(":");
                        String userId = split[0];
                        String clientType = split[1];
                        imUserId = getImUserId(userId, clientType);
                    } else {
                        imUserId = getImUserId(id, "TEACHER");
                    }

                    String targetId = i.getTargetId();
                    String targetIdUserId;
                    if (targetId.contains(":STUDENT")) {
                        String[] split = targetId.split(":");
                        String userId = split[0];
                        String clientType = split[1];
                        targetIdUserId = getImUserId(userId, clientType);
                    } else {
                        targetIdUserId = getImUserId(targetId, "TEACHER");
                    }

                    privateImportMessage.setFromAccount(imUserId);
                    //设置接收人
                    privateImportMessage.setToAccount(targetIdUserId);
                    //设置随机数
                    if (i.getMsgRandom() == null) {
                        int nextInt = Math.abs(new Random().nextInt());
                        privateImportMessage.setMsgRandom(nextInt);
                        i.setMsgRandom(nextInt);
                    } else {
                        privateImportMessage.setMsgRandom(i.getMsgRandom());
                    }

                    if (i.getMsgSeq() == null) {
                        int nextInt = Math.abs(new Random().nextInt());
                        i.setMsgSeq(nextInt);
                        privateImportMessage.setMsgSeq(nextInt);
                    } else {
                        privateImportMessage.setMsgSeq(i.getMsgSeq());
                    }
                    //设置发送时间
                    String time = i.getDateTime();
                    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    try {
                        Date date = df.parse(time);
                        long dateTime = date.getTime();
                        dateTime = dateTime / (1000);
                        if (i.getMsgTimeStamp() == null) {
                            privateImportMessage.setMsgTimeStamp(dateTime);
                            i.setMsgTimeStamp(dateTime);
                        } else {
                            privateImportMessage.setMsgTimeStamp(i.getMsgTimeStamp());

                        }
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }

                    //设置body
                    //TencentRequest.MessageBody body = new TencentRequest.MessageBody();

                    JSONObject jsonObject = JSONObject.parseObject(i.getContent());

                    List<TencentRequest.MessageBody> list = getPrivateMessge(i, jsonObject);
                    if (list.isEmpty()) {
                        updateStatus(i,1);
                        continue;
                    }

               /* body.setMsgType(i.getClassname());
                body.setMsgContent(i.getContent());*/

                    /*list.add(body);*/
               /* list.stream().forEach(item->{
                    item.setMsgContent(i.getContent());
                    item.setMsgType(i.getClassname());
                });*/

                /*if (list.stream().map(TencentRequest.MessageBody::getMsgType).collect(Collectors.toList()).get(0).equals("RC:TxtMsg")) {
                    //文本对象
                    list.forEach(item -> item.setMsgType("TIMTextElem"));
                } else if (list.stream().map(TencentRequest.MessageBody::getMsgType).collect(Collectors.toList()).get(0).equals("RC:ImgMsg")) {
                    //图文对象
                    list.forEach(item -> item.setMsgType("TIMImageElem"));
                } else if (list.stream().map(TencentRequest.MessageBody::getMsgType).collect(Collectors.toList()).get(0).equals("RC:HQVCMsg")){
                    //高清语音消息对象
                    list.forEach(item -> item.setMsgType("TIMSoundElem"));
                } else if (list.stream().map(TencentRequest.MessageBody::getMsgType).collect(Collectors.toList()).get(0).equals("RC:HQVCMsg")){
                    list.forEach(item -> item.setMsgType("TIMSoundElem"));
                }*/
                    privateImportMessage.setTencentMessageBody(list);
                    privateImportMessage.setCloudCustomData(jsonObject.getString("extra"));

//                    log.info("导入私聊消息：{}", JSON.toJSONString(privateImportMessage));
                    try {
                        imPluginContext.getPluginService().importPrivateMessage(privateImportMessage);
                        //为已导入数据更改标识
                        updateStatus(i,1);
                        historyMessageService.updateMsg(i.getMsgUID(),i.getMsgSeq(),i.getMsgRandom(),i.getMsgTimeStamp());
                        log.info("导入私聊消息成功");
                    } catch (Exception e) {
                        updateStatus(i,2);
                        log.error("导入私聊IM消息失败 msg:{},entity:{}", JSON.toJSONString(privateImportMessage), i, e);
                    }
                } else if (type == 3) {
                    //群组会话
                    MessageWrapper.GroupImportMessage groupImportMessage = new MessageWrapper.GroupImportMessage();
                    List<MessageWrapper.GroupImportMessageData> list = new ArrayList<>();
                    MessageWrapper.GroupImportMessageData data1 = new MessageWrapper.GroupImportMessageData();
                    //TencentRequest.MessageBody body = new TencentRequest.MessageBody();
                    //设置群组Id
                    groupImportMessage.setGroupId(i.getGroupId());
                    //设置发送人
                    String imUserId;
                    String id = i.getFromUserId();
                    //根据发送者分场景讨论
                    if (id.contains(":STUDENT")) {
                        String[] split = id.split(":");
                        String userId = split[0];
                        String clientType = split[1];
                        imUserId = getImUserId(userId, clientType);
                    } else {
                        imUserId = getImUserId(id, "TEACHER");
                    }


                    data1.setFromAccount(imUserId);
                    //设置随机数
                    data1.setRandom(Math.abs(new Random().nextInt()));
                    //设置发送时间

                    String time = i.getDateTime();
                    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    try {
                        Date date = df.parse(time);
                        long dateTime = date.getTime();
                        dateTime = dateTime / (1000);
                        data1.setSendTime(dateTime);
                    } catch (ParseException e) {
                        log.error("时间转换失败", e);
                    }
                /*bodyList.stream().forEach(item->{
                            item.setMsgContent(i.getContent());
                            item.setMsgType(i.getClassname());
                        });*/
                    JSONObject jsonObject = JSONObject.parseObject(i.getContent());


                    List<TencentRequest.MessageBody> bodyList = getGroupMessage(i, jsonObject);

                    if (CollectionUtils.isEmpty(bodyList)){
                        updateStatus(i,1);
                        continue;
                    }
                /*body.setMsgContent(i.getContent());
                body.setMsgType(i.getClassname());*/

                    //文本对象
                /*if (bodyList.stream().map(TencentRequest.MessageBody::getMsgType).collect(Collectors.toList()).get(0).equals("RC:TxtMsg")) {
                    bodyList.forEach(item -> item.setMsgType("TIMTextElem"));
                } else if (bodyList.stream().map(TencentRequest.MessageBody::getMsgType).collect(Collectors.toList()).get(0).equals("RC:ImgMsg")) {
                    //暂未支持图文对象
                    bodyList.forEach(item -> item.setMsgType("TIMCustomElem"));
                }*/

                    data1.setTencentMessageBody(bodyList);
                    data1.setCloudCustomData(jsonObject.getString("extra"));

                    list.add(data1);
                    //导入消息列表
                    groupImportMessage.setMsgList(list);
//                    log.info("导入群聊消息：{}", JSON.toJSONString(groupImportMessage));
                    try {
                        imPluginContext.getPluginService().importGroupMessage(groupImportMessage);
                        //为已导入数据更改标识
                        updateStatus(i,1);
                        log.info("导入消息成功");
                    } catch (Exception e) {
                        updateStatus(i,2);
                        log.error("导入群组IM消息失败 msg:{},entity:{}", JSON.toJSONString(groupImportMessage), i, e);
                    }
                }
            }catch (Exception e) {
                log.error("导入消息失败:", e);
                updateStatus(i,2);
            }
        }
    }


    @Override
    @Transactional
    public void imToTencent(List<HistoryMessage> list) {

        for (HistoryMessage historyMessage : list) {
            try {
                HistoryMessageTencent tencent = new HistoryMessageTencent();
                tencent.setId(historyMessage.getMsgUID());
                // 时间
                String dateTime = historyMessage.getDateTime();
                LocalDateTime ldt = LocalDateTime.parse(dateTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"));
                // yyyyMMddHH
                String date = ldt.format(DateTimeFormatter.ofPattern("yyyyMMddHH"));
                tencent.setMsgTime(Long.parseLong(date));

                List<TencentRequest.MessageBody> bodyList = new ArrayList<>();
                JSONObject jsonObject = JSONObject.parseObject(historyMessage.getContent());
                // 类型
                if (historyMessage.getTargetType() == 1) {
                    tencent.setChatType("C2C");
                    tencent.setToAccount(historyMessage.getTargetId());
                    bodyList = getPrivateMessge(historyMessage,jsonObject);
                } else if (historyMessage.getTargetType() == 3) {
                    tencent.setChatType("GROUP");
                    tencent.setGroupId(historyMessage.getGroupId());
                    bodyList = getGroupMessage(historyMessage,jsonObject);
                } else {

                    historyMessage.setSyncFlag(1);
                    historyMessageService.updateSyncFlag(historyMessage.getMsgUID(),1);
                    continue;
                }

                if (CollectionUtils.isEmpty(bodyList)) {

                    historyMessage.setSyncFlag(1);
                    historyMessageService.updateSyncFlag(historyMessage.getMsgUID(),1);
                    continue;
                }
                bodyList = bodyList.stream().filter(Objects::nonNull).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(bodyList)) {
                    historyMessage.setSyncFlag(1);
                    historyMessageService.updateSyncFlag(historyMessage.getMsgUID(),1);
                    continue;
                }
                // 发送人
                tencent.setFromAccount(historyMessage.getFromUserId());
                //类型
                tencent.setAppService("MEC");
                tencent.setMsgFromPlatform(historyMessage.getSource());
                tencent.setMsgTimestamp(ldt.toInstant(ZoneId.systemDefault().getRules().getOffset(ldt)).toEpochMilli()/1000);

                // 消息转换
                tencent.setCloudCustomData(jsonObject.getString("extra"));
                tencent.setMsgBody(JSON.toJSONString(bodyList));
                tencent.setMsgBodyJson(JSON.parseObject(tencent.getMsgBody(), JSONArray.class));

                historyMessageTencentService.insert(tencent);

                historyMessage.setSyncFlag(1);
                historyMessageService.updateSyncFlag(historyMessage.getMsgUID(),1);
            } catch (Exception e) {
                historyMessage.setSyncFlag(2);
                historyMessageService.updateSyncFlag(historyMessage.getMsgUID(),1);
            }

        }

    }

    @NotNull
    @Override
    public List<TencentRequest.MessageBody> getGroupMessage(HistoryMessage i, JSONObject jsonObject) {
        List<TencentRequest.MessageBody> bodyList = new ArrayList<>();
        if (i.getClassname().equals("RC:TxtMsg")) {
            //文本
            //文本
            String content = jsonObject.getString("content");

            // http开头的链接 扩展字段没有值 放在扩展字段里
            if (StringUtils.isNotBlank(content) && content.startsWith("http")) {
                String extra = jsonObject.getString("extra");
                if (StringUtils.isEmpty(extra)) {
                    jsonObject.put("extra", content);
                }
            }
            TencentRequest.MessageBody body1 = getTimTextElem(content);
            bodyList.add(body1);
        } else if (i.getClassname().equals("RC:ImgMsg")) {
            //图片
            TencentRequest.MessageBody body1 = getTimImageElem(jsonObject.getString("imageUri"));
            bodyList.add(body1);
        } else if (i.getClassname().equals("RC:GIFMsg")) {
            //GIf
            String remoteUrl = jsonObject.getString("remoteUrl");
            int gifDataSize = jsonObject.getInteger("gifDataSize");
            int width = jsonObject.getInteger("width");
            int height = jsonObject.getInteger("height");
            TencentRequest.MessageBody body1 = getTimGifElem(remoteUrl, gifDataSize, width, height);
            bodyList.add(body1);
        } else if (i.getClassname().equals("RC:HQVCMsg")) {
            //语音
            String remoteUrl = jsonObject.getString("remoteUrl");
            int duration = jsonObject.getInteger("duration");
            TencentRequest.MessageBody body1 = getTimSoundElem(remoteUrl, duration);
            bodyList.add(body1);
        } else if (i.getClassname().equals("RC:FileMsg")) {
            //文件
            String fileUrl = jsonObject.getString("fileUrl");
            String size = jsonObject.getString("size");
            String name = jsonObject.getString("name");

            TencentRequest.MessageBody body1 = getTimFileElem(fileUrl, size, name);
            bodyList.add(body1);
        } else if (i.getClassname().equals("RC:SightMsg")) {
            //视频
            String sightUrl = jsonObject.getString("sightUrl");
            String size = jsonObject.getString("size");
            int duration = jsonObject.getInteger("duration");
            String content = jsonObject.getString("content");
            TencentRequest.MessageBody body1 = null;
            try {
                body1 = getTimVideoFileElem(sightUrl, size, duration, content);
            } catch (IOException e) {
                log.error("视频转换失败", e);
            }
            bodyList.add(body1);
        } else if (i.getClassname().equals("RC:LBSMsg")) {
            //位置
            double latitude = jsonObject.getDouble("latitude");
            double longitude = jsonObject.getDouble("longitude");
            String poi = jsonObject.getString("poi");
            TencentRequest.MessageBody body1 = getTimLocationElem(latitude, longitude, poi);
            bodyList.add(body1);
        } else if (i.getClassname().equals("RC:ImgTextMsg")) {
            //图文
            TencentRequest.MessageBody body1 = getTimTextElem(jsonObject.getString("content"));
            TencentRequest.MessageBody body2 = getTimImageElem(jsonObject.getString("imageUri"));
            bodyList.add(body1);
            bodyList.add(body2);
        } else if (i.getClassname().equals("RC:CombineMsg")) {
            //合并转发
            String remoteUrl = jsonObject.getString("remoteUrl");
            int conversationType = jsonObject.getInteger("conversationType");
            getTimRelayElem();
        }
        return bodyList;
    }

    @NotNull
    @Override
    public List<TencentRequest.MessageBody> getPrivateMessge(HistoryMessage i, JSONObject jsonObject) {
        List<TencentRequest.MessageBody> list = new ArrayList<>();
        if (i.getClassname().equals("RC:TxtMsg")) {
            //文本
            String content = jsonObject.getString("content");

            // http开头的链接 扩展字段没有值 放在扩展字段里
            if (StringUtils.isNotBlank(content) && content.startsWith("http")) {
                String extra = jsonObject.getString("extra");
                if (StringUtils.isEmpty(extra)) {
                    jsonObject.put("extra", content);
                }
            }

            TencentRequest.MessageBody body1 = getTimTextElem(content);
            list.add(body1);
        } else if (i.getClassname().equals("RC:ImgMsg")) {
            //图片
            TencentRequest.MessageBody body1 = getTimImageElem(jsonObject.getString("imageUri"));
            list.add(body1);
        } else if (i.getClassname().equals("RC:GIFMsg")) {
            //GIf
            String remoteUrl = jsonObject.getString("remoteUrl");
            int gifDataSize = jsonObject.getInteger("gifDataSize");
            int width = jsonObject.getInteger("width");
            int height = jsonObject.getInteger("height");
            TencentRequest.MessageBody body1 = getTimGifElem(remoteUrl, gifDataSize, width, height);
            list.add(body1);
        } else if (i.getClassname().equals("RC:HQVCMsg")) {
            //语音
            String remoteUrl = jsonObject.getString("remoteUrl");
            int duration = jsonObject.getInteger("duration");
            TencentRequest.MessageBody body1 = getTimSoundElem(remoteUrl, duration);
            list.add(body1);
        } else if (i.getClassname().equals("RC:FileMsg")) {
            //文件
            String fileUrl = jsonObject.getString("fileUrl");
            String size = jsonObject.getString("size");
            String name = jsonObject.getString("name");

            TencentRequest.MessageBody body1 = getTimFileElem(fileUrl, size, name);
            list.add(body1);
        } else if (i.getClassname().equals("RC:SightMsg")) {
            //视频
            String sightUrl = jsonObject.getString("sightUrl");
            String size = jsonObject.getString("size");
            int duration = jsonObject.getInteger("duration");
            String content = jsonObject.getString("content");
            TencentRequest.MessageBody body1 = null;
            try {
                body1 = getTimVideoFileElem(sightUrl, size, duration, content);
            } catch (IOException e) {
                e.printStackTrace();
            }
            list.add(body1);
        } else if (i.getClassname().equals("RC:LBSMsg")) {
            //位置
            double latitude = jsonObject.getDouble("latitude");
            double longitude = jsonObject.getDouble("longitude");
            String poi = jsonObject.getString("poi");
            TencentRequest.MessageBody body1 = getTimLocationElem(latitude, longitude, poi);
            list.add(body1);
        } else if (i.getClassname().equals("RC:ImgTextMsg")) {
            //图文
            TencentRequest.MessageBody body1 = getTimTextElem(jsonObject.getString("content"));
            TencentRequest.MessageBody body2 = getTimImageElem(jsonObject.getString("imageUri"));
            list.add(body1);
            list.add(body2);
        }
        return list;
    }

    @Override
    public com.yonge.mongodb.PageInfo<HistoryMessageTencentWrapper.HistoryMessageTencent> historyMessage(HistoryMessageTencentWrapper.HistoryMessageTencentQuery query) {
        com.yonge.mongodb.PageInfo<HistoryMessageTencentWrapper.HistoryMessageTencent> pageInfo = historyMessageTencentService.historyMessage(query);

        List<HistoryMessageTencentWrapper.HistoryMessageTencent> rows = pageInfo.getRows();
        if (CollectionUtils.isEmpty(rows)) {
            return pageInfo;
        }

        for (HistoryMessageTencentWrapper.HistoryMessageTencent row : rows) {
            // 时间戳 转换
            Long msgTime = row.getMsgTimestamp();
            if (msgTime != null) {
                String dateTime = String.valueOf(msgTime);
                if (dateTime.length() == 10) {
                    dateTime = dateTime + "000";
                }
                row.setMsgTime(Long.parseLong(DateUtil.format(new Date(Long.parseLong(dateTime)), "yyyyMMddHH")));
            }
            // 更换访问路径
            row.setMsgBody(urlTypeChange(row.getMsgBody()));
            row.setCloudCustomData(urlTypeChange(row.getCloudCustomData()));

        }
        // 发送人信息
        List<Integer> fromAccountList = rows.stream()
            .map(HistoryMessageTencentWrapper.HistoryMessageTencent::getFromAccount)
            .filter(StringUtils::isNotEmpty)
            .map(Integer::parseInt)
            .collect(Collectors.toList());
        Map<Integer, SysUser> userMap = studentService.getUserMapByIds(fromAccountList);
        rows.forEach(item -> {
            if (StringUtils.isEmpty(item.getFromAccount())) {
                return;
            }
            SysUser sysUser = userMap.get(Integer.parseInt(item.getFromAccount()));
            if (sysUser != null) {
                item.setFromAccountName(sysUserService.getImName(sysUser));
                item.setFromAccountAvatar(sysUserService.getImAvatar(sysUser));
            }
        });
        // 接收人
        List<Integer> toAccountList = rows.stream()
            .map(HistoryMessageTencentWrapper.HistoryMessageTencent::getToAccount)
            .filter(StringUtils::isNotEmpty)
            .map(Integer::parseInt)
            .collect(Collectors.toList());
        Map<Integer, SysUser> toUserMap = studentService.getUserMapByIds(toAccountList);
        rows.forEach(item -> {
            if (StringUtils.isEmpty(item.getToAccount())) {
                return;
            }
            SysUser sysUser = toUserMap.get(Integer.parseInt(item.getToAccount()));
            if (sysUser != null) {
                item.setToAccountName(sysUserService.getImName(sysUser));
                item.setToAccountAvatar(sysUserService.getImAvatar(sysUser));
            }
        });
        // 群
        List<String> groupIdList = rows.stream()
            .map(HistoryMessageTencentWrapper.HistoryMessageTencent::getGroupId)
            .filter(StringUtils::isNotEmpty)
            .collect(Collectors.toList());
        Map<String, ImGroup> groupMap = this.getMapByIds(groupIdList);
        rows.forEach(item -> {
            if (StringUtils.isEmpty(item.getGroupId())) {
                return;
            }
            ImGroup imGroup = groupMap.get(item.getGroupId());
            if (imGroup != null) {
                item.setGroupName(imGroup.getName());
                item.setGroupAvatar(imGroup.getImg());
            }
        });

        return pageInfo;
    }

    @Override
    public Map<String, ImGroup> getMapByIds(List<String> groupIdList) {
        if (CollectionUtils.isEmpty(groupIdList)) {
            return new HashMap<>();
        }
        List<ImGroup> imGroups = imGroupDao.getByIds(groupIdList);
        if (CollectionUtils.isEmpty(imGroups)) {
            return new HashMap<>();
        }
        return imGroups.stream().collect(Collectors.toMap(ImGroup::getId, item -> item));
    }

    @Override
    public void deleteGroup() {

        // 100个线程的无界线程池
        ThreadPoolExecutor executorService =  new ThreadPoolExecutor(100, 100,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<Runnable>());

        log.info("-------------------------------deleteGroup start---------------------------");


        List<String> imGroupIds =
            musicGroupDao.getDelImGroupId();
        for (String imGroupId : imGroupIds) {
            executorService.execute(() -> {
                imGroupService.delGroup(imGroupId);
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    log.error("线程等待异常", e);
                }
            });
        }

        while (executorService.getQueue().size() > 0) {
            try {
                log.info("线程池中正在执行的任务数量：{},等待数量：{}", executorService.getActiveCount(),executorService.getQueue().size());
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                log.error("线程等待异常", e);
            }
        }
        executorService.shutdown();
        log.info("------------------------------- deleteGroup end---------------------------");
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delGroup(String imGroupId) {
        if (StringUtils.isEmpty(imGroupId)) {
            return;
        }
        try {
            imGroupCoreService.groupDismiss(imGroupId);
        } catch (Exception e) {
            log.error("删除群组失败:{}",imGroupId, e);
            throw new BizException("删除群组失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void muteAll(ImGroupWrapper.GroupMuteAll groupMute) {
        // 更新群组禁言状态
        ImGroup imGroup = imGroupService.get(groupMute.getGroupId());
        if (imGroup == null) {
            throw new BizException("群组不存在");
        }
        ImGroupWrapper.ConfigJson from = ImGroupWrapper.ConfigJson.from(imGroup.getConfigJson());
        from.setMute(groupMute.getMuteAll());
        imGroup.setConfigJson(JSON.toJSONString(from));
        imGroup.setMemberNum(0);
        imGroupService.update(imGroup);

        // 调用IM接口
        try {
            if (Boolean.TRUE.equals(groupMute.getMuteAll())) {
                imPluginContext.getPluginService().groupBanCreate(groupMute.getGroupId());
            } else {
                imPluginContext.getPluginService().groupBanRemove(groupMute.getGroupId());
            }
        } catch (Exception e) {
            log.error("群组禁言状态修改失败", e);
            throw new BizException("群组禁言状态修改失败");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void transferOwner(ImGroupWrapper.GroupTransferOwner groupTransferOwner) {
        ImGroup imGroup = imGroupService.get(groupTransferOwner.getImGroupId());
        if (imGroup == null) {
            throw new BizException("群组不存在");
        }
        ImGroupMember admin = imGroupMemberService.getDao().getAdmin(groupTransferOwner.getImGroupId());
        if (admin == null) {
            throw new BizException("群主信息不存在");
        }
        if (!StringUtils.equals(admin.getUserId().toString(), groupTransferOwner.getCurrentOwner())) {
            throw new BizException("当前操作人不是群主");
        }
        List<ImGroupMember> imGroupMembers = imGroupMemberService.getDao().queryByImGroupIdAndUserId(groupTransferOwner.getImGroupId(), groupTransferOwner.getUserId());
        if (CollectionUtils.isEmpty(imGroupMembers)) {
            throw new BizException("新群主不在群组中");
        }
        ImGroupMember imGroupMember = imGroupMembers.get(0);
        if (!Objects.equals(imGroupMember.getGroupRoleType(), EImGroupRoleType.Admin.getCode())) {
            throw new BizException("新群主不是管理员");
        }
        // 更新群主
        imGroupMember.setIsAdmin(true);
        imGroupMemberService.update(imGroupMember);
        admin.setGroupRoleType(EImGroupRoleType.Admin.getCode());
        admin.setIsAdmin(false);
        imGroupMemberService.update(admin);
        // 调用IM接口
        try {
            //设置成群主
            imPluginContext.getPluginService().changeGroupOwner(groupTransferOwner.getImGroupId(), groupTransferOwner.getUserId(),groupTransferOwner.getCurrentOwner());
            //设置成管理员
            imPluginContext.getPluginService().groupAdminCreate(admin.getImGroupId(),admin.getUserId().toString());
        } catch (Exception e) {
            log.error("转让群主失败", e);
            throw new BizException("转让群主失败");
        }
    }


    private TencentRequest.MessageBody getTimGifElem(String remoteUr,int size,int width,int height) {
        if (StringUtils.isEmpty(remoteUr) && size == 0 && width == 0 && height == 0) {
            return null;
        }

        // 腾讯云消息
        // 腾讯云消息
        //转换url格式
        String newRemoteUr = urlTypeChange(remoteUr);
        ImageUtil.ImageInfo imageReq = ImageUtil.ImageInfo.builder()
                .imgUrl(newRemoteUr)
                .suffix("GIF")
                .build();

        // 获取图片基本信息
        ImageUtil.imageToBase64(imageReq);

        // 图片格式。 JPG = 1 GIF = 2 PNG = 3 BMP = 4 其他 = 255
        // 图片类型。原图 = 1，大图 = 2，缩略图 = 3
        int imageFormat = 1;
        if ("PNG".equalsIgnoreCase(imageReq.getSuffix())) {
            imageFormat = 3;
        }
        if ("GIF".equalsIgnoreCase(imageReq.getSuffix())) {
            imageFormat = 2;
        }
        return TencentRequest.MessageBody.builder()
                .msgType("TIMImageElem")
                .msgContent(TencentRequest.ImageMessageBody.builder()
                        .uuid(imageReq.getMd5())
                        .imageFormat(imageFormat)
                        .type(1)
                        .height(height)
                        .width(width)
                        .url(newRemoteUr)
                        .size(imageReq.getSize()).build())
                .build();

    }

    /**
     * 更新已修改数据的状态
     * @param info HistoryMessage
     */
    public void updateStatus(HistoryMessage info,int status) {
        historyMessageService.updateStatus(info.getMsgUID(), status);
    }


    //构造文本请求体
    public static TencentRequest.MessageBody getTimTextElem(String textMessage) {

        if (StringUtils.isEmpty(textMessage)) {
            return null;
        }

        return TencentRequest.MessageBody.builder()
                .msgType("TIMTextElem")
                .msgContent(TencentRequest.TextMessageBody.builder().text(textMessage).build())
                .build();
    }


    //构造图片请求体
    public static TencentRequest.MessageBody getTimImageElem(String imgMessage) {



        if (StringUtils.isEmpty(imgMessage)) {
            return null;
        }

        // 腾讯云消息
        // 腾讯云消息

        String newImgMessage = urlTypeChange(imgMessage);
        ImageUtil.ImageInfo imageReq = ImageUtil.ImageInfo.builder()
                .imgUrl(newImgMessage)
                .suffix(newImgMessage.substring(newImgMessage.lastIndexOf(".") + 1))
                .build();

        // 获取图片基本信息
        ImageUtil.imageToBase64(imageReq);

        // 图片格式。 JPG = 1 GIF = 2 PNG = 3 BMP = 4 其他 = 255
        // 图片类型。原图 = 1，大图 = 2，缩略图 = 3
        int imageFormat = 1;
        if ("PNG".equalsIgnoreCase(imageReq.getSuffix())) {
            imageFormat = 3;
        }
        if ("GIF".equalsIgnoreCase(imageReq.getSuffix())) {
            imageFormat = 2;
        }
        return TencentRequest.MessageBody.builder()
                .msgType("TIMImageElem")
                .msgContent(TencentRequest.ImageMessageBody.builder()
                        .uuid(imageReq.getMd5())
                        .imageFormat(imageFormat)
                        .type(1)
                        .height(imageReq.getHeight())
                        .width(imageReq.getWidth())
                        .url(newImgMessage)
                        .size(imageReq.getSize()).build())
                .build();
    }


    //构造语音请求体
    public static TencentRequest.MessageBody getTimSoundElem(String remoteUrl,int duration) {
        if (StringUtils.isEmpty(remoteUrl) && duration == 0) {
            return null;
        }

        ImHistoryMessageWrapper.TimSoundElem timSoundElem = new ImHistoryMessageWrapper.TimSoundElem();

        String newRemoteUrl = urlTypeChange(remoteUrl);
        timSoundElem.setUrl(newRemoteUrl);
        timSoundElem.setUUID(UUID.randomUUID().toString());
        timSoundElem.setSecond(duration);
        timSoundElem.setDownload_Flag(2);

        TencentRequest.MessageBody messageBody = new TencentRequest.MessageBody();
        messageBody.setMsgType("TIMSoundElem");
        messageBody.setMsgContent(timSoundElem);
        return  messageBody;
}

    //构建文件请求体
    public static TencentRequest.MessageBody getTimFileElem(String fileUrl, String size, String name) {
        if (StringUtils.isEmpty(fileUrl) && StringUtils.isEmpty(size) &&  StringUtils.isEmpty(name)) {
            return null;
        }
        ImHistoryMessageWrapper.TimFileElem timFileElem = new ImHistoryMessageWrapper.TimFileElem();
        String newFileUrl = urlTypeChange(fileUrl);
        timFileElem.setUrl(newFileUrl);
        timFileElem.setUUID(UUID.randomUUID().toString());
        timFileElem.setFileName(name);
        timFileElem.setFileSize(Integer.valueOf(size));
        timFileElem.setDownload_Flag(2);

        TencentRequest.MessageBody messageBody = new TencentRequest.MessageBody();
        messageBody.setMsgType("TIMFileElem");
        messageBody.setMsgContent(timFileElem);
        return  messageBody;
    }

    //构建视频请求体
    public  TencentRequest.MessageBody getTimVideoFileElem(String sightUrl, String size, int duration, String content) throws IOException {
        if (StringUtils.isEmpty(sightUrl) && StringUtils.isEmpty(sightUrl) && duration == 0) {
            return null;
        }
        ByteArrayInputStream bais = null;
        String  format = "";
        //判断是否有前缀
        if (content.contains(",")){
            String[] split = content.split(",");
            String s = split[0];
            format = s.substring(s.indexOf("/")+ 1,s.indexOf(";"));
            content = split[1];
        } else {
            format = "JPG";
        }

        byte[] bytes = Base64.getDecoder().decode(content);
        //构建字节数组输入流
        bais = new ByteArrayInputStream(bytes);



        UploadReturnBean uploadReturnBean = uploadFileService.uploadFile(bais, format);
        String url = uploadReturnBean.getUrl();

        String newUrl = urlTypeChange(url);
        URL url2 =  new URL(newUrl);
        BufferedImage read = ImageIO.read(url2);
        int width = read.getWidth();
        int height = read.getHeight();

        ImageUtil.ImageInfo imageReq = ImageUtil.ImageInfo.builder()
                .imgUrl(newUrl)
                .suffix(format)
                .build();

        // 获取图片基本信息
        ImageUtil.imageToBase64(imageReq);

        ImHistoryMessageWrapper.TimVideoFileElem timVideoFileElem = new ImHistoryMessageWrapper.TimVideoFileElem();
        String newSightUrl = urlTypeChange(sightUrl);
        timVideoFileElem.setVideoUrl(newSightUrl);
        timVideoFileElem.setVideoUUID(UUID.randomUUID().toString());
        timVideoFileElem.setVideoSize(Integer.valueOf(size));
        timVideoFileElem.setVideoSecond(duration);
        timVideoFileElem.setVideoFormat("mp4");
        timVideoFileElem.setVideoDownloadFlag(2);
        timVideoFileElem.setThumbUrl(newUrl);
        timVideoFileElem.setThumbUUID(imageReq.getMd5());
        timVideoFileElem.setThumbWidth( width);
        timVideoFileElem.setThumbHeight( height);
        //timVideoFileElem.setThumbSize((imageReq.getSize().intValue()));
        timVideoFileElem.setThumbDownloadFlag(2);
        timVideoFileElem.setThumbFormat(format);

        TencentRequest.MessageBody messageBody = new TencentRequest.MessageBody();
        messageBody.setMsgType("TIMVideoFileElem");
        messageBody.setMsgContent(timVideoFileElem);
        return  messageBody;
    }

    //构建位置请求体
    private TencentRequest.MessageBody getTimLocationElem(double latitude, double longitude, String poi) {
        if (latitude == 0 && longitude == 0 &&  StringUtils.isEmpty(poi)) {
            return null;
        }
        ImHistoryMessageWrapper.TimLocationElem timLocationElem = new ImHistoryMessageWrapper.TimLocationElem();
        timLocationElem.setDesc(poi);
        timLocationElem.setLatitude(latitude);
        timLocationElem.setLongitude(longitude);

        TencentRequest.MessageBody messageBody = new TencentRequest.MessageBody();
        messageBody.setMsgType("TIMLocationElem");
        messageBody.setMsgContent(timLocationElem);
        return  messageBody;
    }


    //构建合并转发请求体
    private TencentRequest.MessageBody getTimRelayElem() {
        ImHistoryMessageWrapper.TimRelayElem timRelayElem = new ImHistoryMessageWrapper.TimRelayElem();
        //todo
        //timRelayElem.setMsgNum();

        TencentRequest.MessageBody messageBody = new TencentRequest.MessageBody();
        messageBody.setMsgType("TIMRelayElem");
        messageBody.setMsgContent(timRelayElem);
        return  messageBody;
    }



    //url路径改造
    public static String urlTypeChange(String url){
        if (StringUtils.isBlank(url)){
            log.warn("url为空");
            return url;
        }
        String replace;
        if (url.contains("https://ks3-cn-beijing.ksyuncs.com")){
             replace = url.replace("https://ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com");
        } else if (url.contains("https://daya.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://daya.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/daya");
        } else if (url.contains("https://appstore.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://appstore.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/appstore");
        } else if (url.contains("https://cloud-coach.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://cloud-coach.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/cloud-coach");
        } else if (url.contains("https://contracts.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://contracts.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/contracts");
        } else if (url.contains("https://cooperation.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://cooperation.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/cooperation");
        } else if (url.contains("https://daya-docs.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://daya-docs.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/daya-docs");
        } else if (url.contains("https://gyt.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://gyt.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/gyt");
        } else if (url.contains("https://homework.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://homework.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/homework");
        } else if (url.contains("https://i-m.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://i-m.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/i-m");
        } else if (url.contains("https://live-rewind.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://live-rewind.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/live-rewind");
        } else if (url.contains("https://mall.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://mall.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/mall");
        } else if (url.contains("https://musescoreconverresult.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://musescoreconverresult.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/musescoreconverresult");
        } else if (url.contains("https://news-info.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://news-info.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/news-info");
        } else if (url.contains("https://question-bank.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://question-bank.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/question-bank");
        } else if (url.contains("https://video-course.ks3-cn-beijing.ksyuncs.com")){
            replace = url.replace("https://video-course.ks3-cn-beijing.ksyuncs.com", "https://oss.dayaedu.com/video-course");
        } else if (url.contains("https://ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com");
        } else if (url.contains("https://daya.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://daya.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/daya");
        } else if (url.contains("https://appstore.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://appstore.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/appstore");
        } else if (url.contains("https://cloud-coach.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://cloud-coach.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/cloud-coach");
        } else if (url.contains("https://contracts.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://contracts.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/contracts");
        } else if (url.contains("https://cooperation.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://cooperation.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/cooperation");
        } else if (url.contains("https://daya-docs.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://daya-docs.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/daya-docs");
        } else if (url.contains("https://gyt.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://gyt.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/gyt");
        } else if (url.contains("https://homework.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://homework.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/homework");
        } else if (url.contains("https://i-m.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://i-m.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/i-m");
        } else if (url.contains("https://live-rewind.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://live-rewind.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/live-rewind");
        } else if (url.contains("https://mall.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://mall.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/mall");
        } else if (url.contains("https://musescoreconverresult.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://musescoreconverresult.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/musescoreconverresult");
        } else if (url.contains("https://news-info.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://news-info.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/news-info");
        } else if (url.contains("https://question-bank.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://question-bank.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/question-bank");
        } else if (url.contains("https://video-course.ks3-cn-beijing.ksyun.com")){
            replace = url.replace("https://video-course.ks3-cn-beijing.ksyun.com", "https://oss.dayaedu.com/video-course");
        }else {
            return url;
        }
        return replace;
    }


}