package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.ImLiveBroadcastRoomMemberDao;
import com.ym.mec.biz.dal.dao.OrganizationDao;
import com.ym.mec.biz.dal.entity.ImLiveBroadcastRoom;
import com.ym.mec.biz.dal.entity.ImLiveBroadcastRoomMember;
import com.ym.mec.biz.dal.entity.Organization;
import com.ym.mec.biz.dal.vo.ImLiveBroadcastRoomDetailVo;
import com.ym.mec.biz.dal.vo.ImLiveBroadcastRoomMemberVo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.page.WrapperUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 直播间人员关系表(ImLiveBroadcastRoomMember)表服务实现类
 *
 * @author hgw
 * @since 2022-02-21 14:26:59
 */
@Slf4j
@Service("imLiveBroadcastRoomMemberService")
public class ImLiveBroadcastRoomMemberServiceImpl extends ServiceImpl<ImLiveBroadcastRoomMemberDao, ImLiveBroadcastRoomMember> implements ImLiveBroadcastRoomMemberService {

    @Autowired
    private ImLiveBroadcastRoomDataService imLiveBroadcastRoomDataService;
    @Autowired
    private ImLiveRoomVideoService imLiveRoomVideoService;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private ImLiveBroadcastRoomService imLiveBroadcastRoomService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private OrganizationDao organizationDao;

    @Override
    public ImLiveBroadcastRoomMemberDao getDao() {
        return this.baseMapper;
    }

    /**
     * 查询直播详情
     */
    @Override
    public ImLiveBroadcastRoomDetailVo queryRoomDetail(Map<String, Object> param) {
        String roomUid = WrapperUtil.toStr(param, "roomUid", "请传入房间编号");
        ImLiveBroadcastRoomDetailVo vo = imLiveBroadcastRoomDataService.getDao().queryByRoomUid(roomUid);
        vo.setVideoList(imLiveRoomVideoService.queryList(roomUid));
        return vo;
    }

    @Override
    public PageInfo<ImLiveBroadcastRoomMemberVo> queryRoomMember(Map<String, Object> param) {
        Page<ImLiveBroadcastRoomMemberVo> pageInfo = PageUtil.getPageInfo(param);

        IPage<ImLiveBroadcastRoomMemberVo> page = baseMapper.queryMemberPage(pageInfo, param);

        List<ImLiveBroadcastRoomMemberVo> records = page.getRecords();

        if (CollectionUtils.isNotEmpty(records)) {

            // 游客默认头像
            String defaultAvatar = Optional.ofNullable(sysConfigService.findByParamName(SysConfigService.VISITOR_DEFAULT_AVATAR)
                            .getParanValue()).orElse("");

            String roomUid = MapUtils.getString(param, "roomUid", RandomStringUtils.random(5));
            // 直播间信息
            ImLiveBroadcastRoom room = imLiveBroadcastRoomService.lambdaQuery()
                    .eq(ImLiveBroadcastRoom::getRoomUid, roomUid)
                    .last("LIMIT 1")
                    .one();
            if (Objects.isNull(room)) {
                throw new BizException("直播间不存在");
            }
            // 直播间主讲老师
            SysUser speaker = sysUserFeignService.queryUserById(room.getSpeakerId());

            // 机构名称
            List<Integer> organIds = records.stream()
                    .map(ImLiveBroadcastRoomMemberVo::getOrganId).distinct().collect(Collectors.toList());

            Map<Integer, String> organMap = organizationDao.findOrgans(organIds).stream()
                    .collect(Collectors.toMap(Organization::getId, Organization::getName, (o, n) -> n));

            List<Integer> studentIds = records.stream()
                    .filter(x -> StringUtils.isBlank(x.getFingerprint()))
                    .map(ImLiveBroadcastRoomMemberVo::getStudentId)
                    .filter(Objects::nonNull)
                    .distinct().collect(Collectors.toList());

            // 学生报名乐团
            Map<Integer, String> musicGroupMap = Maps.newConcurrentMap();
            // 学生声部名称
            Map<Integer, String> subjectMap = Maps.newConcurrentMap();

            // 直播客户端查询在线用户信息时，忽略学生相关信息查询
            if (MapUtils.getIntValue(param, "rows", 10) < 100) {

                for (List<Integer> item : Lists.partition(studentIds, 500)) {
                    // 并发查询学生信息
                    Lists.partition(item, 50).parallelStream().forEach(data -> {

                        // 学生报名乐团
                        Map<Integer, String> musicMap = baseMapper.selectStudentMusicGroup(data).stream()
                                .filter(x -> Objects.nonNull(x) && Objects.nonNull(x.getStudentId()))
                                .collect(Collectors.toMap(ImLiveBroadcastRoomMemberVo::getStudentId, x -> Optional.ofNullable(x.getMusicGroupName()).orElse(""), (o, n) -> n));

                        if (MapUtils.isNotEmpty(musicMap)) {
                            musicGroupMap.putAll(musicMap);
                        }

                        // 学生声部名称
                        Map<Integer, String> collect = baseMapper.selectStudentSubject(studentIds).stream()
                                .filter(x -> Objects.nonNull(x) && Objects.nonNull(x.getStudentId()))
                                .collect(Collectors.toMap(ImLiveBroadcastRoomMemberVo::getStudentId, x -> Optional.ofNullable(x.getSubName()).orElse(""), (o, n) -> n));
                        if (MapUtils.isNotEmpty(collect)) {
                            subjectMap.putAll(collect);
                        }
                    });
                }
            }

            for (ImLiveBroadcastRoomMemberVo item : records) {

                if (StringUtils.isNotEmpty(item.getFingerprint())) {
                    // 设置游客默认头像
                    item.setAvatar(defaultAvatar);
                } else {
                    // 设置默认数据
                    item.subName(subjectMap.getOrDefault(item.getStudentId(), ""))
                            .organName(organMap.getOrDefault(item.getOrganId(), ""))
                            .musicGroupName(musicGroupMap.getOrDefault(item.getStudentId(), ""))
                            .roomTitle(room.getRoomTitle())
                            .setSpeakerName(speaker.getRealName());
                }

            }
        }

        return PageUtil.pageInfo(page);
    }

    /**
     * 查询在房间人员
     *
     * @param roomUid 直播间编号
     */
    @Override
    public List<Integer> getLiveRoomMember(String roomUid) {
        List<ImLiveBroadcastRoomMember> list = this.lambdaQuery()
                                                   .eq(ImLiveBroadcastRoomMember::getRoomUid, roomUid)
                                                   .eq(ImLiveBroadcastRoomMember::getLiveRoomStatus, true)
                                                   .list();
        if (list == null || list.isEmpty())
            return new ArrayList<>();

        return list.stream().map(ImLiveBroadcastRoomMember::getUserId).collect(Collectors.toList());
    }

    @Override
    public List<ImLiveBroadcastRoomMember> getByRoomUidAndUserIds(String liveRoomId, Set<Long> userIds) {

        if (StringUtils.isBlank(liveRoomId) || CollectionUtils.isEmpty(userIds)) {
            return new ArrayList<>();
        }
        return this.lambdaQuery()
                .eq(ImLiveBroadcastRoomMember::getRoomUid, liveRoomId)
                .in(ImLiveBroadcastRoomMember::getUserId, userIds)
                .list();

    }

}

