package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.json.ali.AliJSONMap;
import com.microsvc.toolkit.middleware.live.LivePluginContext;
import com.microsvc.toolkit.middleware.live.LivePluginService;
import com.microsvc.toolkit.middleware.live.impl.RongCloudLivePlugin;
import com.microsvc.toolkit.middleware.live.impl.TencentCloudLivePlugin;
import com.microsvc.toolkit.middleware.live.message.LiveRoomMessage;
import com.microsvc.toolkit.middleware.live.message.LiveRoomUser;
import com.microsvc.toolkit.middleware.live.message.RTCRequest;
import com.microsvc.toolkit.middleware.live.message.RTCRoom;
import com.microsvc.toolkit.middleware.live.message.TencentRequest;
import com.microsvc.toolkit.middleware.live.message.TencentWrapper;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ImLiveBroadcastRoomDto;
import com.ym.mec.biz.dal.dto.LiveRoomStatus;
import com.ym.mec.biz.dal.dto.RoomMemberNumDto;
import com.ym.mec.biz.dal.dto.RoomReservationUserSearch;
import com.ym.mec.biz.dal.dto.TencentData;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.enums.live.ELiveViewMode;
import com.ym.mec.biz.dal.page.LiveRoomGoodsOrderQueryInfo;
import com.ym.mec.biz.dal.vo.*;
import com.ym.mec.biz.redisson.RedissonMessageService;
import com.ym.mec.biz.service.*;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.common.entity.ImRoomMessage;
import com.ym.mec.common.entity.ImUserState;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.page.WrapperUtil;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import com.ym.mec.util.http.HttpUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.jetbrains.annotations.NotNull;
import org.joda.time.DateTime;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RBucket;
import org.redisson.api.RLock;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.text.MessageFormat;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 直播房间管理表(ImLiveBroadcastRoom)表服务实现类
 *
 * @author hgw
 * @since 2022-02-17 20:52:05
 */
@Service("imLiveBroadcastRoomService")
public class ImLiveBroadcastRoomServiceImpl extends ServiceImpl<ImLiveBroadcastRoomDao, ImLiveBroadcastRoom> implements ImLiveBroadcastRoomService {

    private final static Logger log = LoggerFactory.getLogger(ImLiveBroadcastRoomServiceImpl.class);

    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private ImLiveBroadcastRoomMemberService liveBroadcastRoomMemberService;
    @Autowired
    private ImLiveBroadcastRoomMemberDao liveBroadcastRoomMemberDao;
    @Autowired
    private ImLiveBroadcastRoomDataService liveBroadcastRoomDataService;
    @Autowired
    private LiveGoodsMapperDao liveGoodsMapperDao;
    @Autowired
    private ImLiveRoomReservationService imLiveRoomReservationService;
    @Autowired
    private ImLiveRoomBlackService imLiveRoomBlackService;
    @Autowired
    private ImLiveRoomVideoService imLiveRoomVideoService;
    @Autowired
    private LivePluginContext livePluginContext;
    @Autowired
    private RedissonMessageService redissonMessageService;

    @Autowired
    private ImLiveBroadcastRoomService imLiveBroadcastRoomService;

    @Autowired
    private ImFeignService imFeignService;

    @Autowired
    private CourseShareService courseShareService;

    @Autowired
    private CourseScheduleDao courseScheduleDao;

    @Autowired
    private  ImGroupService imGroupService;

    @Autowired
    private ImGroupCoreService imGroupCoreService;


    //待替换的变量
    public static final String USER_ID = "${userId}";
    public static final String ROOM_UID = "${roomUid}";

    //直播间累计用户信息-指只要进入到该房间的用户都要记录
    public static final String LIVE_ROOM_TOTAL_USER_LIST = String.join(":", "IM:LIVE_ROOM_TOTAL_USER_LIST", ROOM_UID);
    //直播间在线用户信息
    public static final String LIVE_ROOM_ONLINE_USER_LIST = String.join(":", "IM:LIVE_ROOM_ONLINE_USER_LIST", ROOM_UID);
    //当前房间主讲人心跳
    public static final String LIVE_ROOM_SPEAKER_HEART_BEAT = String.join(":", "IM:LIVE_ROOM_SPEAKER_HEART_BEAT", ROOM_UID);
    //房间点赞数
    public static final String LIVE_ROOM_LIKE = String.join(":", "IM:LIVE_ROOM_LIKE", ROOM_UID);
    //计算人员观看时长锁
    public static final String LIVE_LOOK_LOCK = String.join(":", "IM:LIVE_LOOK_LOCK", ROOM_UID);
    //用户对应的直播间Uid
    public static final String LIVE_USER_ROOM = String.join(":", "IM:LIVE_ROOM_USER", USER_ID);
    //记录人员最后变更的状态消息时间
    public static final String LIVE_USER_STATE_TIME = String.join(":", "IM:LIVE_USER_STATE_TIME", USER_ID);
    //主讲人信息
    public static final String LIVE_SPEAKER_INFO = String.join(":", "IM:LIVE_SPEAKER_INFO", ROOM_UID, USER_ID);
    // 直播间游客信息
    public static final String LIVE_VISITOR_INFO = String.join(":", "IM:LIVE_VISITOR_INFO", ROOM_UID, USER_ID);
    //主讲人最近一次加入房间的clientIp
    public static final String LIVE_SPEAKER_LAST_CLIENT_IP = String.join(":", "IM:LIVE_SPEAKER_LAST_CLIENT_IP", ROOM_UID, USER_ID);
    //直播提前开始时间
    public static final int PRE_LIVE_TIME_MINUTE = 30;
    //主讲人信息
    public static final String SPEAKER_ROOM_ING_INFO_LOCK = String.join(":", "IM:SPEAKER_ROOM_ING_INFO_LOCK", USER_ID);

    /**
     * 进入直播间检查数据
     *
     * @param roomUid 房间uid
     * @param userId  用户id
     * @param osType  默认学生端  0:pc端 1:学生端 2:老师端
     */
    @Override
    public ImLiveBroadcastRoomVo queryRoomAndCheck(String roomUid, Integer userId, Integer osType) {

        SysUser sysUser = Optional.ofNullable(userId)
                .map(this::getSysUser)
                .orElseGet(this::getSysUser);

        // 默认学生端查询
        osType = Optional.ofNullable(osType).orElse(1);

        // 移动端-学生端
        if (osType == 1) {
            //学生端
            return studentQueryRoomAndCheck(roomUid, sysUser);
        }
        // 移动端-老师端
        if (osType == 2) {
            //老师端
            return teacherQueryRoomAndCheck(roomUid, sysUser);
        }
        // PC端-直播助手
        if (osType == 3) {
            // 校验直播间是否存在
            ImLiveBroadcastRoomVo vo = Optional.ofNullable(roomUid).map(this::queryRoomInfo)
                    .orElseThrow(() -> new BizException("直播间不存在"));

            if (vo.getOs().equals("mobile")) {
                throw new BizException("该直播间仅可在电脑端直播");
            }
        }

        return queryRoomAndCheck(roomUid, sysUser);
    }

    /**
     * 游客直播间信息
     *
     * @param roomUid 直播间编号
     * @param userId  游客编号
     * @return ImLiveBroadcastRoomVo
     */
    @Override
    public ImLiveBroadcastRoomVo visitorRoomInfo(String roomUid, Integer userId) {

        // 直播间游客信息
        RMap<Integer, String> roomTotalUser = getTotalUserCache(roomUid);
        if (!roomTotalUser.containsKey(userId)) {
            throw new BizException("您无法观看该直播,请刷新重试");
        }

        // 游客信息
        RoomUserInfoVo userInfo = JSONObject.toJavaObject(JSONObject.parseObject(roomTotalUser.get(userId)),
                RoomUserInfoVo.class);

        // 游客直播间信息校验
        ImLiveBroadcastRoomVo roomVo = getVisitorLiveBroadcastRoomVo(roomUid);

        // 用户直播间黑名单
        roomVo.setBlacklistFlag(0);
        //黑名单查询-查询当前用户是否在黑名单中
        int count = imLiveRoomBlackService.count(Wrappers.<ImLiveRoomBlack>lambdaQuery()
                .eq(ImLiveRoomBlack::getRoomUid, roomVo.getRoomUid())
                .eq(ImLiveRoomBlack::getUserId, userInfo.getUserId()));
        if (count > 0) {
            roomVo.setBlacklistFlag(1);
        }

        LivePluginService pluginService = livePluginContext.getPluginService(roomVo.getServiceProvider());
        // 直播房间配置信息
        String userSig = "";
        try {
            // 生成聊天签名
            userSig = pluginService.register(imGroupCoreService.getImUserId(userInfo.getUserId().toString(),SysUserType.VISITOR.getCode()), userInfo.getUserName(), userInfo.getAvatar());

        } catch (Exception e) {
            log.error("直播房间游客注册失败: userId={}", userInfo.getUserId(), e);
        }

        // 直播房间统计信息
        getRoomData(roomVo);

        // 直播间配置信息
        return roomVo.userSig(userSig).liveRoomConfig(pluginService.getLiveRoomConfig());
    }

    public ImLiveBroadcastRoomVo studentQueryRoomAndCheck(String roomUid, SysUser sysUser) {
        //如果是学生端，则需要检查是否有权限进入
        Map<String, Object> param = new HashMap<>();
        param.put("roomUid", roomUid);
        ImLiveBroadcastRoomVo vo = Optional.of(param).map(this::getImLiveBroadcastRoomVo)
                .orElseThrow(() -> new BizException("您无法观看该直播"));
        if (vo.getSpeakerId().equals(sysUser.getId())) {
            throw new BizException("您是该直播间的主讲人，请从直播端进入");
        }
        if (vo.getLiveState() == 0) {
            throw new BizException("直播尚未开始");
        }
        return queryRoomAndCheck(vo, sysUser,SysUserType.STUDENT);
    }

    public ImLiveBroadcastRoomVo teacherQueryRoomAndCheck(String roomUid, SysUser sysUser) {
        ImLiveBroadcastRoomVo vo = Optional.ofNullable(roomUid).map(this::queryRoomInfo)
                .orElseThrow(() -> new BizException("直播间不存在"));
        if (vo.getOs().equals("pc")) {
            throw new BizException("该直播间仅可在移动端直播");
        }
        return queryRoomAndCheck(vo, sysUser, SysUserType.TEACHER);
    }

    public ImLiveBroadcastRoomVo queryRoomAndCheck(String roomUid, SysUser sysUser) {
        ImLiveBroadcastRoomVo vo = Optional.ofNullable(roomUid).map(this::queryRoomInfo)
                .orElseThrow(() -> new BizException("直播间不存在"));
        return queryRoomAndCheck(vo, sysUser,vo.getClientType());
    }

    public ImLiveBroadcastRoomVo queryRoomAndCheck(ImLiveBroadcastRoomVo room, SysUser sysUser,SysUserType sysUserType) {

        //直播间信息校验
        if (!room.getTenantId().equals(sysUser.getTenantId())) {
            throw new BizException("您不是该直播机构人员,不可观看!");
        }
        if (room.getLiveState() == 2) {
            //如果直播结束了还是推广状态则将推广修改为取消
            if (room.getPopularize() == 1) {
                this.update(Wrappers.<ImLiveBroadcastRoom>lambdaUpdate()
                        .set(ImLiveBroadcastRoom::getPopularize, 0)
                        .eq(ImLiveBroadcastRoom::getId, room.getId()));
            }
            throw new BizException("直播已结束！");
        }

        if (room.getRoomState() == 1) {
            throw new BizException("直播间不存在");
        }
        if (room.getLiveState() == 0) {
            Date liveStartTime = DateUtil.addMinutes(room.getLiveStartTime(), -PRE_LIVE_TIME_MINUTE);
            throw new BizException(DateUtil.format(liveStartTime, "yyyy年MM月dd日 HH点mm分") + " 可进入直播间准备");
        }

        room.setBlacklistFlag(0);
        //黑名单查询-查询当前用户是否在黑名单中
        int count = imLiveRoomBlackService.count(Wrappers.<ImLiveRoomBlack>lambdaQuery()
                .eq(ImLiveRoomBlack::getRoomUid, room.getRoomUid())
                .eq(ImLiveRoomBlack::getUserId, sysUser.getId()));
        if (count > 0) {
            room.setBlacklistFlag(1);
        }
        LivePluginService pluginService = livePluginContext.getPluginService(room.getServiceProvider());
        // 直播房间配置信息
        String userSig = "";
        try {

            userSig = pluginService.register(imGroupCoreService.getImUserId(String.valueOf(sysUser.getId()),sysUserType.getCode()), sysUser.getUsername(),
                    sysUser.getAvatar());
        } catch (Exception e) {
            log.error("直播房间用户注册失败: userId={}", sysUser.getId(), e);
        }

        // 直播房间统计信息
        getRoomData(room);

        // 直播间配置信息
        return room.userSig(userSig).liveRoomConfig(pluginService.getLiveRoomConfig());
    }

    /**
     * 查询直播间信息
     *
     * @param roomUid 直播间uid
     */
    @Override
    public ImLiveBroadcastRoomVo queryRoomInfo(String roomUid) {
        ImLiveBroadcastRoomVo roomVo = getImLiveBroadcastRoomVo(roomUid);
        if (roomVo == null) {
            return null;
        }
        getRoomData(roomVo);
        return roomVo;
    }

    private ImLiveBroadcastRoomVo getImLiveBroadcastRoomVo(String roomUid) {
        List<ImLiveBroadcastRoomVo> list = baseMapper.queryPage(new HashMap<String, Object>() {{
            put("roomUid", roomUid);
            put("allRoom", 1);
        }});
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    /**
     * 分页查询直播间列表
     */
    @Override
    public PageInfo<ImLiveBroadcastRoomVo> queryPage(Map<String, Object> param) {
        param.put("groupType", "NORMAL");
        String os = WrapperUtil.toStr(param, "os");
        if (StringUtils.isNotBlank(os) && os.toUpperCase(Locale.ROOT).equals("TEACHER")) {
            param.put("speakerId", getSysUser().getId());
            param.put("clientType", SysUserType.TEACHER.getCode());
        } else if (StringUtils.isNotBlank(os) && os.toUpperCase(Locale.ROOT).equals("EDUCATION")) {
            param.put("clientType", SysUserType.EDUCATION.getCode());
            param.put("speakerId", getSysUser().getId());
        }
        Page<ImLiveBroadcastRoomVo> pageInfo = PageUtil.concatTimePage(param, "startTime", "endTime");
        // pageInfo.setDesc("a.created_time_");
        param.put("tenantId", TenantContextHolder.getTenantId());
        IPage<ImLiveBroadcastRoomVo> page = baseMapper.queryPage(pageInfo, param);

        // 查询预约人数
        List<ImLiveBroadcastRoomVo> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return PageUtil.pageInfo(page);
        }

        List<String> roomIdList = records.stream().map(ImLiveBroadcastRoomVo::getRoomUid).collect(Collectors.toList());

        List<ImLiveBroadcastRoomVo> countRoomReservationList = imLiveRoomReservationService.countRoomReservation(roomIdList);
        Map<String, List<ImLiveBroadcastRoomVo>> collect = countRoomReservationList.stream()
                .collect(Collectors.groupingBy(ImLiveBroadcastRoomVo::getRoomUid));
        for (ImLiveBroadcastRoomVo record : records) {
            List<ImLiveBroadcastRoomVo> imLiveBroadcastRoomVos = collect.get(record.getRoomUid());
            if (CollectionUtils.isEmpty(imLiveBroadcastRoomVos)) {
                continue;
            }
            record.setRoomReservationNum(imLiveBroadcastRoomVos.get(0).getRoomReservationNum());
        }

        // 查询直播间视频数
        Map<String, Integer> uidCountMap = imLiveRoomVideoService.queryRoomUidCountMap(roomIdList);
        for (ImLiveBroadcastRoomVo record : records) {
            Integer count = uidCountMap.get(record.getRoomUid());
            if (count == null) {
                record.setVideoNum(0);
            }
            record.setVideoNum(count);
        }

        return PageUtil.pageInfo(page);
    }

    /**
     * 预创建直播间
     *
     * @param dto 直播间信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(ImLiveBroadcastRoomDto dto) {
        SysUser sysUser = getSysUser(dto.getSpeakerId());
        Date now = new Date();
        if (now.getTime() > dto.getLiveStartTime().getTime()) {
            throw new BizException("设置的直播开始时间不能小于当前时间");
        }
        if (StringUtils.isBlank(dto.getServiceProvider())) {
            //查询房间过期时间
            String liveClient = sysConfigDao.findConfigValue("live_client");
            dto.setServiceProvider(liveClient);
        }
        ImLiveBroadcastRoom obj = new ImLiveBroadcastRoom();
        BeanUtils.copyProperties(dto, obj);
        String roomUid = "LIVE-" + sysUser.getId() + "-" + now.getTime();
        obj.setTenantId(TenantContextHolder.getTenantId());
        obj.setRoomUid(roomUid);
        obj.setRoomConfig(JSONObject.toJSONString(dto.getRoomConfig()));
        obj.setLiveState(0);
        obj.setRoomState(0);
        obj.setCreatedBy(getSysUser().getId());
        obj.setCreatedTime(now);
        this.save(obj);
        //推送老师端-直播已经创建
        sendRoomLiveState(sysUser, obj, MessageTypeEnum.JIGUANG_LIVE_CREATED);


        CompletableFuture.runAsync(() -> {

            Date endTime = DateUtil.addMinutes(now, PRE_LIVE_TIME_MINUTE);
            if (endTime.getTime() >= dto.getLiveStartTime().getTime()) {
                imLiveBroadcastRoomService.createLiveRoom(obj);
            }
        });
    }

    /**
     * 修改直播间信息
     *
     * @param dto ImLiveBroadcastRoomDto
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(ImLiveBroadcastRoomDto dto) {
        Date now = new Date();
        Integer userId = getSysUser().getId();
        ImLiveBroadcastRoom obj = this.getById(dto.getId());
        obj.setUpdatedBy(userId);
        obj.setUpdatedTime(now);
        //如果直播已经开始则只能修改是否全员禁麦 和聊天功能
        if (obj.getLiveState() == 1) {
            obj.setRoomConfig(JSONObject.toJSONString(dto.getRoomConfig()));
            this.updateById(obj);
            return;
        }
        if (now.getTime() > dto.getLiveStartTime().getTime()) {
            throw new BizException("设置的直播开始时间不能小于当前时间");
        }
        //本次修改后的类型
        BeanUtils.copyProperties(dto, obj);
        obj.setRoomConfig(JSONObject.toJSONString(dto.getRoomConfig()));
        log.info("update room  >>>  :{}", JSONObject.toJSONString(obj));
        this.updateById(obj);
    }

    /**
     * 是否禁言
     *
     * @param id          房间id
     * @param whetherChat 是否允许聊天互动  0允许 1不允许
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void whetherChat(Integer id, Integer whetherChat) {
        if (whetherChat == null) {
            return;
        }
        ImLiveBroadcastRoom obj = this.getById(id);
        ImLiveBroadcastRoomDto.RoomConfig roomConfig = getRoomConfig(obj.getRoomConfig()).orElse(null);
        if (Objects.isNull(roomConfig)) {
            return;
        }
        roomConfig.setWhether_chat(whetherChat);
        obj.setRoomConfig(JSONObject.toJSONString(roomConfig));
        this.updateById(obj);


        ImLiveBroadcastRoomVo roomVo = getImLiveBroadcastRoomVo(obj.getRoomUid());
        setWhetherChatDefinedData(roomVo, whetherChat);
    }

    private void setWhetherChatDefinedData(ImLiveBroadcastRoomVo roomVo, Integer whetherChat){
        if (whetherChat == null) {
            return;
        }
        setGroupDefinedData(roomVo,EGroupDefinedDataType.GLOBAL_BAN,whetherChat == 1?
            EOnOffStatus.ON.getCode() :EOnOffStatus.OFF.getCode());
    }

    /**
     * 删除直播间
     *
     * @param id 直播间id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Integer id) {
        ImLiveBroadcastRoom obj = this.getById(id);
        if (obj.getLiveState() == 1) {
            throw new BizException("直播已经开始，无法删除");
        }
        obj.setId(id);
        obj.setRoomState(1);
        obj.setUpdatedBy(getSysUser().getId());
        obj.setUpdatedTime(new Date());
        obj.setPopularize(0);
        this.updateById(obj);
    }

    /**
     * 推广直播间-每个机构只能有一个直播间在首页推广
     *
     * @param id         直播间id
     * @param popularize 是否在首页推广 0否 1是
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void opsPopularize(Integer id, Integer popularize) {
        if (!WrapperUtil.checkInObj(popularize, 0, 1)) {
            throw new BizException("参数错误");
        }
        Optional<ImLiveBroadcastRoom> roomOptional = Optional.ofNullable(id)
                .map(this::getById);
        roomOptional.orElseThrow(() -> new BizException("直播间不存在"));
        roomOptional.filter(room -> room.getRoomState() == 0)
                .orElseThrow(() -> new BizException("直播间已经删除，无法设置推广"));
        roomOptional.filter(room -> room.getLiveState() != 2)
                .orElseThrow(() -> new BizException("直播已结束，无法设置推广"));
        ImLiveBroadcastRoom obj = roomOptional.get();
        if (Objects.equals(obj.getPopularize(), popularize)) {
            return;
        }
        //推广该直播间，先清除其该机构他直播间的推广状态
        if (popularize == 1) {
            this.update(Wrappers.<ImLiveBroadcastRoom>lambdaUpdate()
                    .set(ImLiveBroadcastRoom::getPopularize, 0)
                    .eq(ImLiveBroadcastRoom::getRoomState, 0)
                    .eq(ImLiveBroadcastRoom::getTenantId, obj.getTenantId()));
        }
        //更新直播间推广状态
        obj.setPopularize(popularize);
        this.updateById(obj);
    }

    /**
     * 查询该机构目前推广的直播间
     */
    @Override
    public ImLiveBroadcastRoomVo queryPopularizeRoom() {
        Map<String, Object> param = new HashMap<>();
        param.put("tenantId", TenantContextHolder.getTenantId());
        param.put("popularize", 1);
        param.put("liveState", 1);
        return getImLiveBroadcastRoomVo(param);
    }

    private ImLiveBroadcastRoomVo getImLiveBroadcastRoomVo(Map<String, Object> param) {
        PageInfo<ImLiveBroadcastRoomVo> pageInfo = imLiveRoomReservationService.queryPageStudent(param);
        List<ImLiveBroadcastRoomVo> list = pageInfo.getRows();
        if (CollectionUtils.isNotEmpty(list)) {
            getRoomData(list.get(0));
            return list.get(0);
        }
        return null;
    }

    /**
     * 定时任务，每分钟执行
     * 销毁主讲人退出超过30分钟的直播间
     */
    public void destroyExpiredLiveRoom() {
        //查询状态是 未开始和已开始 的直播间
        List<ImLiveBroadcastRoom> list = this.list(Wrappers.<ImLiveBroadcastRoom>lambdaQuery()
                .in(ImLiveBroadcastRoom::getLiveState, 0, 1)
                .eq(ImLiveBroadcastRoom::getRoomState, 0));
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        //查询房间过期时间
        String expiredMinuteStr = sysConfigDao.findConfigValue("destroy_expired_live_room_minute");
        if (StringUtils.isEmpty(expiredMinuteStr)) {
            log.info("roomDestroy>>>> 未查询到配置：destroy_expired_live_room_minute");
            return;
        }
        int expiredMinute = Integer.parseInt(expiredMinuteStr);
        Date now = new Date();
        list.forEach(room -> {
            try {
                destroyExpiredLiveRoom(now, room, expiredMinute);
            } catch (Exception e) {
                log.error("roomDestroy>>>> failed roomId：{} msg：{}", room.getId(), e.getMessage());
            }
        });
    }

    private boolean destroyExpiredLiveRoom(Date now, ImLiveBroadcastRoom room, int expiredMinute) throws Exception {
        log.error("roomDestroy destroyExpiredLiveRoom >>>> now {} roomInfo : {} expiredMinute:{}", now, JSONObject.toJSONString(room), expiredMinute);
        //过期时间 = LiveStartTime + expiredMinute
        Date expiredTime = DateUtil.addMinutes(room.getLiveStartTime(), expiredMinute);
        //当前时间 小于 过期时间 则不销毁
        if (now.getTime() <= expiredTime.getTime()) {
            return false;
        }
        //获取直播间主讲人信息
        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(room.getRoomUid(), room.getSpeakerId().toString());
        if (speakerCache.isExists()) {
            RoomSpeakerInfo speakerInfo = speakerCache.get();
            //查询用户是否在线
            LiveRoomUser liveRoomUser = livePluginContext.getPluginService(room.getServiceProvider()).chatRoomUserExist(room.getId().toString(),speakerInfo.getSpeakerId().toString());
            if (Objects.nonNull(liveRoomUser) && liveRoomUser.getExist()  &&
                "1".equals(liveRoomUser.getStatus()) && StringUtils.equals(liveRoomUser.getStatus(), "1")) {
                log.info("roomDestroy destroyExpiredLiveRoom  is online >>>> roomId:{} speakerId:{}", room.getId(), speakerInfo.getSpeakerId());
                return false;
            }
            //校验房间心跳是否过期没续租
            RBucket<Date> lastRoomHeartbeatCache = redissonClient.getBucket(LIVE_ROOM_SPEAKER_HEART_BEAT.replace(ROOM_UID, room.getRoomUid()));
            if (lastRoomHeartbeatCache.isExists()) {
                //获取最后一次房间心跳时间
                Date lastDate = lastRoomHeartbeatCache.get();
                //房间心跳过期时间 = 将房间心跳时间+过期分钟
                Date lastRoomDateExpired = DateUtil.addMinutes(lastDate, expiredMinute);
                //当前时间 小于 房间心跳过期时间 则不销毁
                if (now.getTime() <= lastRoomDateExpired.getTime()) {
                    log.info("roomDestroy destroyExpiredLiveRoom  last room heartbeat  >>>> roomId:{} speakerId:{}", room.getId(), speakerInfo.getSpeakerId());
                    return false;
                }
            }
            //1.主播没有进入房间，则直接销毁房间
            if (Objects.isNull(speakerInfo.getJoinRoomTime())) {
                log.info("roomDestroy not joinRoom >>>> cache : {}", JSONObject.toJSONString(test(room.getRoomUid())));
                return roomDestroy(room);
            }

            //2.已知主播已进入了房间 就判断是否退出过房间
            if (Objects.nonNull(speakerInfo.getExitRoomTime())) {
                //如果退出时间 大于 进入时间，则证明退出后再也没有进入房间
                if (speakerInfo.getExitRoomTime().getTime() > speakerInfo.getJoinRoomTime().getTime()) {
                    //最终过期时间 =  退出时间+expiredMinute过期时间
                    Date exitExpiredTime = DateUtil.addMinutes(speakerInfo.getExitRoomTime(), expiredMinute);
                    //现在时间 大于等于 最终过期时间 则证明退出后至少30分钟没进入过房间
                    if (now.getTime() >= exitExpiredTime.getTime()) {
                        log.info("roomDestroy exitExpiredTime >>>> cache : {}", JSONObject.toJSONString(test(room.getRoomUid())));
                        return roomDestroy(room);
                    }
                }
            }
        }
        return false;
    }

    /**
     * 关闭房间-融云
     * 获取所有直播间缓存数据并写入数据库后并清理缓存
     *
     * @param roomUid 直播间Uid
     */
    @Override
    public boolean roomDestroy(String roomUid) {
        ImLiveBroadcastRoom room = this.getOne(Wrappers.<ImLiveBroadcastRoom>lambdaQuery()
                .eq(ImLiveBroadcastRoom::getRoomUid, roomUid));
        return opsRoomDestroy(room);
    }

    /**
     * 关闭房间-融云
     * 获取所有直播间缓存数据并写入数据库后并清理缓存
     *
     * @param id 直播房间表id
     */
    @Override
    public boolean roomDestroy(Integer id) {
        ImLiveBroadcastRoom room = this.getById(id);
        return opsRoomDestroy(room);
    }

    private boolean opsRoomDestroy(ImLiveBroadcastRoom room) {
        if (Objects.isNull(room)) {
            return false;
        }
        if (room.getLiveState() == 0) {
            throw new BizException("直播未开始");
        }
        if (room.getLiveState() == 2) {
            throw new BizException("直播已经结束,请刷新数据！");
        }
        //销毁房间
        return roomDestroy(room);
    }

    @Override
    public void sendForcedOffline(ImLiveBroadcastRoom room) {

        LivePluginService pluginService = livePluginContext.getPluginService(room.getServiceProvider());

        LiveRoomMessage message = new LiveRoomMessage();
        message.setIsIncludeSender(1);
        message.setFromUserId(room.getSpeakerId().toString());
        message.setToChatRoomId(room.getRoomUid());
        message.setObjectName(ImRoomMessage.FORCED_OFFLINE);

        SysUser sysUser = sysUserFeignService.queryUserById(room.getSpeakerId());
        if (Objects.nonNull(sysUser)) {

            // 发送用户信息
            LiveRoomMessage.MessageUser messageUser = LiveRoomMessage.MessageUser.builder()
                    .sendUserId(imGroupCoreService.getImUserId(String.valueOf(room.getSpeakerId()),room.getClientType().getCode()))
                    .sendUserName(sysUser.getUsername())
                    .avatarUrl(sysUser.getAvatar())
                    .build();

            message.setContent(LiveRoomMessage.MessageContent.builder().sendUserInfo(messageUser).build());
        }

        try {
            pluginService.sendChatRoomMessage(message);
        } catch (Exception e) {

            log.error("sendForcedOffline error", e);
        }
        log.info("roomDestroy>>>> FORCED_OFFLINE {}", JSONObject.toJSONString(message));
    }

    public boolean roomDestroy(ImLiveBroadcastRoom room) {
        //10秒内同一个房间不能重复销毁-防重复销毁
        RBucket<Object> bucket = redissonClient.getBucket("IM:ROOMDESTROY:" + room.getRoomUid());
        if (!bucket.trySet(1, 10, TimeUnit.SECONDS)) {
            return false;
        }
        log.error("roomDestroy>>>> room : {}", JSONObject.toJSONString(room));
        String roomUid = room.getRoomUid();
        Integer speakerId = room.getSpeakerId();

        //获取所有直播间缓存数据并写入数据库后并清理缓存
        CompletableFuture.runAsync(() -> insertAndCleanLiveData(roomUid, speakerId));
        log.info("roomDestroy>>>> insertAndCleanLiveData {}", JSONObject.toJSONString(room));

        //将房间状态改为已销毁
        ImLiveBroadcastRoom liveRoomUpdate = new ImLiveBroadcastRoom();
        liveRoomUpdate.setId(room.getId());

        Date date = new Date();
        liveRoomUpdate.setRoomState(2);
        liveRoomUpdate.setLiveState(2);
        Integer userId = -1;
        try {
            userId = getSysUser().getId();
        } catch (Exception ignored) {
        }
        liveRoomUpdate.setSpeakerStatus(0);
        liveRoomUpdate.setPushStatus(0);
        liveRoomUpdate.setUpdatedBy(userId);
        liveRoomUpdate.setUpdatedTime(date);
        liveRoomUpdate.setLiveEndTime(date);
        //销毁直播间后要关闭推广
        liveRoomUpdate.setPopularize(0);

        this.updateById(liveRoomUpdate);

        //下架商品
        liveGoodsMapperDao.updateStatus(room.getRoomUid(), null, false);

        //向聊天室发自定义消息踢出所有人
        try {
            LivePluginService pluginService = livePluginContext.getPluginService(room.getServiceProvider());

            sendForcedOffline(room);

            //销毁直播间
            pluginService.chatRoomDestroy(roomUid);


            TencentWrapper.LiveStreamState liveStreamState = pluginService.liveStreamState(
                getStreamId(room.getRoomUid(),Integer.parseInt(imGroupCoreService.getImUserId(String.valueOf(room.getSpeakerId()),room.getClientType().getCode()))));
            if (liveStreamState == null) {
                log.error("查询直播间流失败,返回结果为空");
            } else if ("active".equals(liveStreamState.getStreamState())) {
                pluginService.liveStreamStop(getStreamId(room.getRoomUid(), Integer.parseInt(imGroupCoreService.getImUserId(String.valueOf(room.getSpeakerId()),room.getClientType().getCode()))));
            }

            // 录制任务Id
            if (room.getServiceProvider().equals(TencentCloudLivePlugin.PLUGIN_NAME)) {

                List<String> collect = imLiveRoomVideoService.lambdaQuery()
                        .eq(ImLiveRoomVideo::getRoomUid, roomUid).list().stream()
                        .map(ImLiveRoomVideo::getRecordId)
                        .filter(StringUtils::isNotEmpty)
                        .distinct().collect(Collectors.toList());

                for (String taskId : collect) {
                    // 删除录制任务
                    pluginService.rtcRoomRecordStop(taskId);
                }
            }

            //            imFeignService.destroyLiveRoom(roomUid);
//            log.info("roomDestroy>>>> destroyLiveRoom {}", JSONObject.toJSONString(message));
        } catch (Exception e) {
            log.error("roomDestroy>>>> errorMsg{}", e.getMessage(), e.getCause());
            return false;
        }
        return true;
    }

    private String getStreamId(String roomUid, Integer speakerId) {
        return roomUid + "_" + speakerId;
    }

    //获取该直播间所有数据写入数据库-并清理缓存
    private void insertAndCleanLiveData(String roomUid, Integer speakerId) {
        log.info("insertAndCleanLiveData >>>> roomUid : {}", roomUid);
        Date now = new Date();
        //总观看人数
        List<ImLiveBroadcastRoomMember> memberList = new ArrayList<>();
        //获取直播间所有人数据写入 im_live_broadcast_room_member
        RMap<Integer, String> roomTotalUserCache = getTotalUserCache(roomUid);
        if (roomTotalUserCache.isExists()) {
            List<RoomUserInfoVo> roomTotalUser = roomTotalUserCache.values().parallelStream()
                    .map(a -> JSONObject.toJavaObject(JSONObject.parseObject(a), RoomUserInfoVo.class))
                    .collect(Collectors.toList());
            for (RoomUserInfoVo v : roomTotalUser) {
                ImLiveBroadcastRoomMember member = new ImLiveBroadcastRoomMember();
                member.setTenantId(v.getTenantId());
                member.setRoomUid(roomUid);
                member.setUserId(v.getUserId());
                member.setOnlineStatus(0);
                member.setLiveRoomStatus(0);
                member.setJoinTime(v.getFirstJoinTime());
                member.setTotalTime(getLookMinutes(v.getDynamicLookTime(), now, v.getTotalViewTime()));
                memberList.add(member);
            }
        }

        //获取直播点赞
        int like = 0;
        like = syncLikeCount(roomUid);
        RBucket<Object> likeCache = redissonClient.getBucket(LIVE_ROOM_LIKE.replace(ROOM_UID, roomUid));
        if (likeCache.isExists()) {
            //删除房间点赞数据
            likeCache.delete();
        }
        int speakerLiveTime = 0;
        //获取直播间主讲人信息 写入im_live_broadcast_room_data
        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(roomUid, speakerId.toString());
        if (speakerCache.isExists()) {
            ImLiveBroadcastRoomData liveData = new ImLiveBroadcastRoomData();
            RoomSpeakerInfo speakerInfo = speakerCache.get();
            log.info("insertAndCleanLiveData >>>> speakerInfo : {}", JSONObject.toJSONString(speakerInfo));
            liveData.setTenantId(speakerInfo.getTenantId());
            liveData.setRoomUid(roomUid);
            liveData.setLikeNum(like);
            liveData.setTotalUserNum(CollectionUtils.isNotEmpty(memberList) ? memberList.size() : 0);
            liveData.setUpdatedTime(now);
            liveData.setLiveTime(getLookMillisecond(speakerInfo.getStartLiveTime(), now, speakerInfo.getTotalLiveTime()));
            liveBroadcastRoomDataService.save(liveData);
            //删除房间主讲人数据
            speakerCache.delete();
            //获取主讲人直播时长
            speakerLiveTime = liveData.getLiveTime();
        }

        //写入im_live_broadcast_room_member表，校验用户观看时长，不能大于主讲人直播时长
        if (CollectionUtils.isNotEmpty(memberList)) {
            for (ImLiveBroadcastRoomMember member : memberList) {
                if (member.getTotalTime() > speakerLiveTime) {
                    member.setTotalTime(speakerLiveTime);
                }
            }
            //添加人员数据
            Lists.partition(memberList, 500)
                    .forEach(list -> liveBroadcastRoomMemberService.getDao().insertBatch(list));
        }
        //获取在线人员信息
        RMap<Integer, String> onlineUserCache = getOnlineUserCache(roomUid);
        //删除人员对应直播间编号信息
        onlineUserCache.forEach((id, s) -> redissonClient.getBucket(LIVE_USER_ROOM.replace(USER_ID, id.toString())).delete());
        //删除直播间所有用户数据
        roomTotalUserCache.delete();
        //删除在线用户数据
        onlineUserCache.delete();
    }

    /**
     * 同步点赞数量
     *
     * @param roomUid 房间uid
     * @param likeNum 点赞数
     */
    @Override
    public void syncLike(String roomUid, Integer likeNum) {

        // 同步直播间点赞数
        if (Optional.ofNullable(likeNum).orElse(0) > 0) {

            redissonClient.getBucket(LIVE_ROOM_LIKE.replace(ROOM_UID, roomUid)).set(likeNum);

            // 同步点赞数到直播间数据表
            ImLiveBroadcastRoomVo roomVo = getImLiveBroadcastRoomVo(roomUid);
            setGroupDefinedData(roomVo,EGroupDefinedDataType.LIKES,likeNum.toString());
            //增加房间心跳
            redissonClient.getBucket(LIVE_ROOM_SPEAKER_HEART_BEAT.replace(ROOM_UID, roomUid)).set(new Date());
        }

        // TODO: 后续同步直播间数据（在线用户等），暂时不做
    }

    /**
     * <p>主讲人处理进入和退出房间数据
     * <p>观看者只处理退出房间数据
     *
     * @param userState 用户状态数据
     */
    @Override
    public void opsRoom(List<ImUserState> userState) {
        if (CollectionUtils.isEmpty(userState)) {
            return;
        }
        userState.forEach(user -> {
            log.info("opsRoom>>>> {}", JSONObject.toJSONString(user));
            if (StringUtils.isBlank(user.getStatus())) {
                log.debug("opsRoom>>>> user status is null");
                return;
            }
            Date now = new Date();
            //获取当前用户状态变更的时间
            long userStateTime = Optional.ofNullable(user.getTime()).orElse(now.getTime());
            String userid = user.getUserid();
            RBucket<Long> userStateTimeCache = redissonClient.getBucket(LIVE_USER_STATE_TIME.replace(USER_ID, userid));
            if (userStateTimeCache.isExists()) {
                //缓存的时间比当前传入时间大则放弃这条数据
                long cacheTime = userStateTimeCache.get();
                if (cacheTime > userStateTime) {
                    log.debug("opsRoom>>>> cacheTime > userStateTime, cacheTime: {}, userStateTime: {}", cacheTime, userStateTime);
                    return;
                }
            }
            //获取当前用户所在房间的uid
            RBucket<String> userRoom = redissonClient.getBucket(LIVE_USER_ROOM.replace(USER_ID, userid));
            if (!userRoom.isExists()) {
                log.debug("opsRoom>>>> userRoom is not exists, userState: {}", JSON.toJSONString(userState));
                return;
            }
            String roomUid = Optional.ofNullable(user.getRoomUid()).orElse(userRoom.get());

            //向直播间发送当前在线人数消息
            ImLiveBroadcastRoomVo roomVo = this.getImLiveBroadcastRoomVo(roomUid);
            if (Objects.isNull(roomVo)) {
                log.info("opsRoom>>>> roomVo is null, userState: {}", JSON.toJSONString(userState));
                return;
            }

            // 腾讯云直播不需要处理融云im的消息
            if (roomVo.getServiceProvider().equals(TencentCloudLivePlugin.PLUGIN_NAME) && StringUtils.isNotBlank(user.getClientIp())) {
                return;
            }

            //将最新的时间写入缓存
            userStateTimeCache.set(userStateTime, 5L, TimeUnit.MINUTES);

            //查询userId是不是主讲人 ,如果是主讲人则返回
            if (isSpeaker(user, now, userid, roomUid)) {
                return;
            }


            Integer userId = Integer.valueOf(userid);
            //这里开始只处理观看者的数据，观看者只接受退出消息 status=0 是进入房间
            if (user.getStatus().equals("0")) {

                // 更新用户在线状态
                liveBroadcastRoomMemberService.lambdaUpdate()
                        .eq(ImLiveBroadcastRoomMember::getTenantId, roomVo.getTenantId())
                        .eq(ImLiveBroadcastRoomMember::getRoomUid, roomVo.getRoomUid())
                        .eq(ImLiveBroadcastRoomMember::getUserId, userId)
                        .set(ImLiveBroadcastRoomMember::getOnlineStatus, 1)
                        .update();


                // 用户进入直播间，发送统计数据
                // sendLiveRoomStatMessage(userid, roomVo);

                // 缓存JoinRoom用户信息到redis
                RBucket<Object> bucket = redissonClient.getBucket(RedissonMessageService.LIVE_ROOM_MEMBER + roomUid);
                if (!bucket.isExists()) {
                    bucket.set(0, 30, TimeUnit.MINUTES);
                    // 发布删除缓存消息
                    redissonMessageService.publish(RedissonMessageService.TOPIC_MESSAGE, roomUid);
                }

                log.debug("opsRoom>>>> user enter room, userState: {}", JSON.toJSONString(userState));
                return;
            }

            //从房间累计用户信息中查询该用户的信息
            RMap<Integer, String> roomTotalUser = this.getTotalUserCache(roomUid);
            //该房间未查询到用户数据则不处理
            if (!roomTotalUser.isExists()) {
                log.debug("opsRoom>>>> roomTotalUser is not exists, userState: {}", JSON.toJSONString(userState));
                return;
            }
            if (!roomTotalUser.containsKey(userId)) {
                log.debug("opsRoom>>>> roomTotalUser is not containsKey, userState: {}", JSON.toJSONString(userState));
                return;
            }
            String userJsonStr = roomTotalUser.get(userId);
            log.info("opsRoom>>>> userJsonStr  {}", userJsonStr);
            //查询用户数据
            RoomUserInfoVo userInfo = JSONObject.toJavaObject(JSONObject.parseObject(userJsonStr), RoomUserInfoVo.class);
            //如果有动态观看时间则证明主播开播过，需要计算当前用户观看时长
            if (Objects.nonNull(userInfo.getDynamicLookTime())) {
                userInfo.setTotalViewTime(this.getLookMinutes(userInfo.getDynamicLookTime(), userInfo.getTotalViewTime()));
                userInfo.setDynamicLookTime(null);
            }
            roomTotalUser.fastPut(userId, JSONObject.toJSONString(userInfo));
            //查询在线人员列表
            RMap<Integer, String> onlineUserInfo = this.getOnlineUserCache(roomUid);
            if (!onlineUserInfo.isExists()) {
                log.debug("opsRoom>>>> onlineUserInfo is not exists, userState: {}", JSON.toJSONString(userState));
                return;
            }
            //从在线人员列表删除该人员
            onlineUserInfo.fastRemove(userId);

            // 更新用户离线状态
            ImLiveBroadcastRoomMember roomMember = new ImLiveBroadcastRoomMember();
            roomMember.setOnlineStatus(0);

            // 用户离开直播间
            if (user.getStatus().equals("3")) {
                // 直播间用户离开状态
                roomMember.setLiveRoomStatus(0);

            }

            // 更新用户在线状态为离线
            liveBroadcastRoomMemberService.lambdaUpdate()
                    .eq(ImLiveBroadcastRoomMember::getTenantId, roomVo.getTenantId())
                    .eq(ImLiveBroadcastRoomMember::getRoomUid, roomVo.getRoomUid())
                    .eq(ImLiveBroadcastRoomMember::getUserId, userId)
                    .update(roomMember);

            log.info("opsRoom>>>> looker userInfo: {}", JSONObject.toJSONString(userInfo));

            // 直播间用户离开直播间消息
            sendLiveRoomLoginOutMessage(userid, roomVo);

            // 在线用户消息同步
            sendOnlineUserCount(roomVo, userId, onlineUserInfo.size());

            // 直播间统计数据
            sendLiveRoomStatMessage(userid, roomVo);
        });
    }

    /**
     * 直播间统计数据同步
     * @param userid 用户Id
     * @param roomVo ImLiveBroadcastRoomVo
     */
    private void sendLiveRoomStatMessage(String userid, ImLiveBroadcastRoomVo roomVo) {

        userid = setFromUserId(userid,roomVo);
        // 消息发送用户
        LiveRoomMessage.MessageUser messageUser = null;
        SysUser sysUser = getRoomSysUser(Integer.parseInt(userid), roomVo.getRoomUid());
        if (Objects.nonNull(sysUser)) {
            // 发送用户信息
            messageUser = LiveRoomMessage.MessageUser
                    .builder()
                    .sendUserId(sysUser.getId().toString())
                    .sendUserName(sysUser.getUsername())
                    .avatarUrl(sysUser.getAvatar())
                    .build();
        }

        LiveRoomMessage.MessageContent messageContent = LiveRoomMessage.MessageContent
                .builder()
                .onlineUsers(roomVo.getLookNum().longValue())
                .viewers(roomVo.getTotalLookNum().longValue())
                .likes(roomVo.getLikeNum().longValue())
                .sendUserInfo(messageUser)
                .build();

        LiveRoomMessage message = LiveRoomMessage.builder()
                .isIncludeSender(1)
                .objectName(LiveRoomMessage.STAT_SYNC)
                .fromUserId(userid)
                .toChatRoomId(roomVo.getRoomUid())
                .content(messageContent)
                .build();

        //发送消息
        try {
            // livePluginContext.getPluginService(roomVo.getServiceProvider()).sendChatRoomMessage(message);
            log.info("sendLiveRoomStatMessage>>>> message: {}", JSONObject.toJSONString(message));
        } catch (Exception e) {
            log.error("sendLiveRoomStatMessage>>>> error {}", e.getMessage());
            log.error("sendLiveRoomStatMessage>>>> sendMessage {} :", JSONObject.toJSONString(message));
        }

    }

    private String setFromUserId(String fromUserId, ImLiveBroadcastRoomVo roomVo) {
        if (roomVo.getServiceProvider().equals(RongCloudLivePlugin.PLUGIN_NAME)) {
            return fromUserId;
        }
        return roomVo.getSpeakerId().toString();
    }

    /**
     * 直播间用户离开消息
     * @param userid 用户id
     * @param roomVo ImLiveBroadcastRoomVo
     */
    private void sendLiveRoomLoginOutMessage(String userid, ImLiveBroadcastRoomVo roomVo) {

        String fromUserId = setFromUserId(userid, roomVo);
        // 消息发送用户
        LiveRoomMessage.MessageUser messageUser = null;
        SysUser sysUser = getRoomSysUser(Integer.parseInt(userid), roomVo.getRoomUid());
        if (Objects.nonNull(sysUser)) {
            // 发送用户信息
            messageUser = LiveRoomMessage.MessageUser
                    .builder()
                    .sendUserId(sysUser.getId().toString())
                    .sendUserName(sysUser.getUsername())
                    .avatarUrl(sysUser.getAvatar())
                    .build();
        }

        LiveRoomMessage.MessageContent messageContent = LiveRoomMessage.MessageContent
                .builder()
                .targetId(sysUser.getId().toString())
                .sendUserInfo(messageUser)
                .build();

        LiveRoomMessage message = LiveRoomMessage.builder()
                .isIncludeSender(1)
                .objectName(LiveRoomMessage.LOOKER_LOGIN_OUT)
                .fromUserId(imGroupCoreService.getImUserId(fromUserId,roomVo.getClientType().getCode()))
                .toChatRoomId(roomVo.getRoomUid())
                .content(messageContent)
                .build();
        try {
            // 黑名单才发送退出消息
            //用户离开直播间发送退出房间消息给主讲人
            // if (!imLiveRoomBlackService.checkBlackUser(Integer.parseInt(userid), roomVo.getRoomUid())) {
            //     return;
            // }
            LivePluginService pluginService = livePluginContext.getPluginService(roomVo.getServiceProvider());
            pluginService.sendChatRoomMessage(message);

            log.info("sendLiveRoomLoginOutMessage>>>> looker LOOKER_LOGIN_OUT : roomId={}, userId={}", roomVo.getRoomUid(), userid);
        } catch (Exception e) {
            log.error("sendLiveRoomLoginOutMessage>>>>  looker error LOOKER_LOGIN_OUT {}", e.getMessage());
            log.error("sendLiveRoomLoginOutMessage>>>>  looker error sendMessage {} : LOOKER_LOGIN_OUT : : roomId={}, userId={}", message, roomVo.getRoomUid(), userid);
        }
    }

    /**
     * 向直播间发送当前在线人数消息
     *
     * @param roomVo    直播间
     * @param fromUserId 发送人id
     * @param count      人数
     */
    private void sendOnlineUserCount(ImLiveBroadcastRoomVo roomVo, Integer fromUserId, Integer count) {

        String roomUid = roomVo.getRoomUid();
        //校验传入参数，房间uid和发送人id不能为空
        if (!WrapperUtil.checkObj(roomUid, fromUserId, count)) {
            log.info(" sendOnlineUserCount>>>> param is null   roomUid: {}  fromUserId:{}  count:{}", roomUid, fromUserId, count);
            return;
        }

        // 直播间统计数据
        getRoomData(roomVo);

        String userId = setFromUserId(fromUserId.toString(), roomVo);
        // 缓存JoinRoom用户信息到redis
        RBucket<Object> bucket = redissonClient.getBucket(RedissonMessageService.LIVE_ROOM_MEMBER + roomUid);
        if (!bucket.isExists()) {
            bucket.set(0, 30, TimeUnit.MINUTES);
            // 发布删除缓存消息
            redissonMessageService.publish(RedissonMessageService.TOPIC_MESSAGE, roomUid);
        }

        // 消息发送用户
        LiveRoomMessage.MessageUser messageUser = null;
        SysUser sysUser = getRoomSysUser(fromUserId, roomUid);
        if (Objects.nonNull(sysUser)) {
            // 发送用户信息
            messageUser = LiveRoomMessage.MessageUser
                    .builder()
                    .sendUserId(sysUser.getId().toString())
                    .sendUserName(sysUser.getUsername())
                    .avatarUrl(sysUser.getAvatar())
                    .build();
        }

        LiveRoomMessage.MessageContent messageContent = LiveRoomMessage.MessageContent
                .builder()
                .count(roomVo.getLookNum().longValue())
                .onlineUsers(roomVo.getLookNum().longValue())
                .viewers(roomVo.getTotalLookNum().longValue())
                .likes(roomVo.getLikeNum().longValue())
                .sendUserInfo(messageUser)
                .build();

        LiveRoomMessage message = LiveRoomMessage.builder()
                .isIncludeSender(1)
                .objectName(LiveRoomMessage.MEMBER_COUNT)
                .fromUserId(userId)
                .toChatRoomId(roomUid)
                .content(messageContent)
                .build();

        //发送消息
        try {
            livePluginContext.getPluginService(roomVo.getServiceProvider()).sendChatRoomMessage(message);
            log.info("sendOnlineUserCount>>>> message: {}", JSONObject.toJSONString(message));
        } catch (Exception e) {
            log.error("sendOnlineUserCount>>>> error {}", e.getMessage());
            log.error("sendOnlineUserCount>>>> sendMessage {} :", JSONObject.toJSONString(message));
        }
    }

    /**
     * 查询userId是不是主讲人
     *
     * @return true 是主讲人 false 不是主讲人
     */
    private boolean isSpeaker(ImUserState user, Date now, String userid, String roomUid) {

        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(roomUid, userid);
        if (!speakerCache.isExists()) {
            log.info("isSpeaker>>>> 主播用户缓存不存在, roomId: {}, userId={}", roomUid, userid);
            return false;
        }

        // 当前用户为主播
        RoomSpeakerInfo speakerInfo = speakerCache.get();
        //最后一次进入房间的clientIp
        RBucket<String> lastClientIp = redissonClient.getBucket(LIVE_SPEAKER_LAST_CLIENT_IP.replace(ROOM_UID, roomUid).replace(USER_ID, userid));
        //主讲人进入房间
        if (user.getStatus().equals("0")) {

            speakerInfo.setJoinRoomTime(now);
            log.info("isSpeaker>>>> join speakerCache {}", JSONObject.toJSONString(speakerInfo));
            speakerCache.set(speakerInfo);
            //将本次进入房间的clientIp添加到主讲人最后一次clientIp缓存中
            if (StringUtils.isNotBlank(user.getClientIp())) {
                lastClientIp.set(user.getClientIp());
            }

            // 更新主播在线状态
            lambdaUpdate()
                    .eq(ImLiveBroadcastRoom::getRoomUid, roomUid)
                    .eq(ImLiveBroadcastRoom::getSpeakerId, Integer.parseInt(userid))
                    .set(ImLiveBroadcastRoom::getSpeakerStatus, 1)
                    .update();

            return true;
        }

        //直播间信息
        ImLiveBroadcastRoomVo roomVo = this.getImLiveBroadcastRoomVo(roomUid);
        if (Objects.isNull(roomVo)) {
            log.info("isSpeaker>>>> roomVo is null, roomId: {}, userId={}", roomUid, userid);
            return true;
        }

        if (roomVo.getServiceProvider().equals(RongCloudLivePlugin.PLUGIN_NAME)) {

            //校验本次退出直播间的clientIp 是不是上次进入房间的clientIp
            if (StringUtils.isNotBlank(user.getClientIp())) {
                if (lastClientIp.isExists()) {
                    //如果是上次进入房间的clientIp和本次退出房间的clientIp不相同，则直接忽略
                    if (!user.getClientIp().equals(lastClientIp.get())) {
                        log.info("isSpeaker>>>> 上次进入房间的clientIp和本次退出房间的clientIp不相同, roomId: {}, userId={}", roomUid, userid);
                        return true;
                    }
                }
            }
        }

        //如果退出时间大于进入时间就无需再次退出-直接返回
        if (compareDate.apply(speakerInfo.getExitRoomTime(), speakerInfo.getJoinRoomTime())) {
            log.info("isSpeaker>>>> 退出时间大于进入时间, roomId: {}, userId={}", roomUid, userid);
            return true;
        }

        //主讲人退出房间关闭录像
        closeLive(speakerInfo);

        syncLikeCount(roomUid);

        //主讲人退出房间
        speakerInfo.setExitRoomTime(now);
        log.info("opsRoom>>>> exit speakerCache {}", JSONObject.toJSONString(speakerInfo));
        speakerCache.set(speakerInfo);

        ImLiveBroadcastRoom room = new ImLiveBroadcastRoom();
        room.setSpeakerStatus(0);

        // 主播离开直播间事件
        if (user.getStatus().equals("3")) {
            // 更新推流状态
            room.setPushStatus(0);

            // 腾讯云直播方案，推送主播离开消息
            if (roomVo.getServiceProvider().equals(TencentCloudLivePlugin.PLUGIN_NAME)) {
                // 主播离开消息通知
                sendLiveRoomLoginOutMessage(userid, roomVo);
            }
        }
        // 更新主播状态
        lambdaUpdate()
                .eq(ImLiveBroadcastRoom::getRoomUid, roomUid)
                .eq(ImLiveBroadcastRoom::getSpeakerId, Integer.parseInt(userid))
                .update(room);

        // 设置直播群组自定义数据
        setGroupDefinedData(roomVo,EGroupDefinedDataType.ANCHOR_STATUS,EAnchorStatus.OFFLINE.getCode());

        return true;
    }

    /**
     * 主讲人登录专用
     *
     * @param roomUid 直播间Id
     */
    public ImLiveBroadcastRoomVo speakerJoinRoom(String roomUid, String os) {
        SysUser sysUser = getSysUser();
        ImLiveBroadcastRoomVo roomVo;
        if (os.equals("mobile")) {
            roomVo = teacherQueryRoomAndCheck(roomUid, sysUser);
        } else {
            roomVo = queryRoomAndCheck(roomUid, sysUser);
        }
        Integer userId = sysUser.getId();
        if (!userId.equals(roomVo.getSpeakerId())) {
            throw new BizException("您不是该直播间的主讲人");
        }
        Date now = new Date();
        //获取直播间信息
        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(roomUid, userId.toString());
        if (!speakerCache.isExists()) {

            //没有主讲人信息则生成一个
            createSpeakerInfo(this.getById(roomVo.getId()), sysUser);

            // 查询主播缓存信息
            speakerCache = getRoomSpeakerInfoCache(roomUid, userId.toString());
        }
        RoomSpeakerInfo speakerInfo = speakerCache.get();
        speakerInfo.setJoinRoomTime(now);
        speakerCache.set(speakerInfo);
        //记录用户当前房间uid
        redissonClient.getBucket(LIVE_USER_ROOM.replace(USER_ID, userId.toString())).set(roomUid, 12L, TimeUnit.HOURS);

        // 更新直播间主播状态
        lambdaUpdate()
                .eq(ImLiveBroadcastRoom::getRoomUid, roomUid)
                .eq(ImLiveBroadcastRoom::getSpeakerId, userId)
                .set(ImLiveBroadcastRoom::getSpeakerStatus, 1)
                .update();

        // 设置直播群组自定义数据
        setGroupDefinedData(roomVo,EGroupDefinedDataType.ANCHOR_STATUS,EAnchorStatus.ONLINE.getCode());

        return roomVo;
    }


    @Override
    public void setGroupMemberDefinedData(ImLiveBroadcastRoomVo roomVo, Integer onlineUser, Integer totalUser) {
        List<TencentRequest.ChatRoomGroupDefinedData> appDefinedData = new ArrayList<>();
        appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                  .key(EGroupDefinedDataType.MEMBER_ONLINE.getCode())
                                                                  .value(onlineUser.toString())
                                                                  .build());
        appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                  .key(EGroupDefinedDataType.MEMBER_TOTAL.getCode())
                                                                  .value(totalUser.toString())
                                                                  .build());
        try {
            livePluginContext.getPluginService(roomVo.getServiceProvider())
                             .chatRoomGroupDefinedData(TencentRequest.ChatRoomGroup.builder()
                                                                                   .groupId(roomVo.getRoomUid())
                                                                                   .appDefinedData(appDefinedData)
                                                                                   .build());
            log.info("设置直播群观看人数 roomUid:{}, data {}",roomVo.getRoomUid(), JSONObject.toJSONString(appDefinedData));
        } catch (Exception e) {
            log.error("设置直播群观看人数失败", e);
            throw new BizException("设置直播群观看人数失败");
        }
    }

    private void setGroupDefinedData(ImLiveBroadcastRoomVo roomVo, EGroupDefinedDataType type, String value) {
        List<TencentRequest.ChatRoomGroupDefinedData> appDefinedData = new ArrayList<>();
        appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                  .key(type.getCode())
                                                                  .value(value)
                                                                  .build());
        try {
            livePluginContext.getPluginService(roomVo.getServiceProvider())
                             .chatRoomGroupDefinedData(TencentRequest.ChatRoomGroup.builder()
                                                                                   .groupId(roomVo.getRoomUid())
                                                                                   .appDefinedData(appDefinedData)
                                                                                   .build());
            log.info("设置直播群配置成功 roomUid:{}, data {}",roomVo.getRoomUid(), JSONObject.toJSONString(appDefinedData));
        } catch (Exception e) {
            log.error("设置直播群配置失败", e);
        }
    }

    /**
     * 进入房间-修改用户对应的房间号信息
     *
     * @param roomUid 房间uid
     * @param userId  用户id
     */
    public void joinRoom(String roomUid, Integer userId) {
        joinRoom(roomUid, userId, true);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void joinRoom(String roomUid, Integer userId,Boolean microphoneFlag) {
        //查询房间信息
        ImLiveBroadcastRoomVo imLiveBroadcastRoomVo = getImLiveBroadcastRoomVo(roomUid);
        if (Objects.isNull(imLiveBroadcastRoomVo)) {
            log.info("opsRoom>>>> joinRoom error roomUid: {}", roomUid);
            return;
        }

        // 判断用户机构是否有权限进入该直播间
        SysUser sysUser = sysUserFeignService.queryUserById(userId);
        if (Objects.isNull(sysUser)) {
            log.info("opsRoom>>>> joinRoom user null error userId: {}", userId);
            return;
        }
        if (!sysUser.getTenantId().equals(imLiveBroadcastRoomVo.getTenantId())) {
            log.info("opsRoom>>>> joinRoom error tenant diff userId: {}, tenantId: {}", userId, sysUser.getTenantId());
            throw new BizException("当前用户所在机构没有加入直播间的权限");
        }

        // 加入直播间用户信息
        joinRoomUserInfo(roomUid, userId, imLiveBroadcastRoomVo,microphoneFlag);
    }

    /**
     * 加入直播间用户信息
     * @param roomUid 直播间编号
     * @param userId 用户编号
     * @param imLiveBroadcastRoomVo ImLiveBroadcastRoomVo
     */
    private void joinRoomUserInfo(String roomUid, Integer userId, ImLiveBroadcastRoomVo imLiveBroadcastRoomVo) {
        joinRoomUserInfo(roomUid, userId, imLiveBroadcastRoomVo,true);
    }
    private void joinRoomUserInfo(String roomUid, Integer userId, ImLiveBroadcastRoomVo imLiveBroadcastRoomVo,Boolean microphoneFlag) {

        //记录用户当前房间uid
        redissonClient.getBucket(LIVE_USER_ROOM.replace(USER_ID, userId.toString())).set(roomUid, 12L, TimeUnit.HOURS);

        //判断是否第一次进房间
        RoomUserInfoVo userInfo;

        //房间累计用户信息-指只要进入到该房间的用户都要记录
        RMap<Integer, String> roomTotalUser = getTotalUserCache(roomUid);
        if (roomTotalUser.containsKey(userId)) {
            //多次进入更新动态进入时间
            userInfo = JSONObject.toJavaObject(JSONObject.parseObject(roomTotalUser.get(userId)), RoomUserInfoVo.class);

            ImLiveBroadcastRoomMember liveRoomMember = liveBroadcastRoomMemberService.lambdaQuery()
                    .eq(ImLiveBroadcastRoomMember::getTenantId, userInfo.getTenantId())
                    .eq(ImLiveBroadcastRoomMember::getRoomUid, imLiveBroadcastRoomVo.getRoomUid())
                    .eq(ImLiveBroadcastRoomMember::getUserId, userInfo.getUserId())
                    .list().stream().findFirst().orElse(null);

            if (Objects.isNull(liveRoomMember)) {

                // 初次进入房间
                getLiveRoomUserInfo(userId, imLiveBroadcastRoomVo, microphoneFlag);
            } else {
                // 更新直播间用户信息
                ImLiveBroadcastRoomMember roomMember = new ImLiveBroadcastRoomMember();
                roomMember.setId(liveRoomMember.getId());
                roomMember.setOnlineStatus(1);
                roomMember.setLiveRoomStatus(1);
                roomMember.setMicrophoneFlag(microphoneFlag);

                liveBroadcastRoomMemberService.updateById(roomMember);
            }

        } else {
            //第一次进该房间 写入用户首次进入时间
            userInfo = getLiveRoomUserInfo(userId, imLiveBroadcastRoomVo, microphoneFlag);
        }

        //查询主讲人信息
        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(roomUid, imLiveBroadcastRoomVo.getSpeakerId().toString());
        if (speakerCache.isExists()) {
            //如果用户进来时主讲人已经开启直播则修改学生观看时间
            Integer state = speakerCache.get().getState();
            if (Objects.nonNull(state) && state == 0 && imLiveBroadcastRoomVo.getPushStatus() == 1) {
                userInfo.setDynamicLookTime(DateTime.now().toDate());
            }
        }

        // 直播间用户、在线用户数据缓存
        roomTotalUser.fastPut(userId, JSONObject.toJSONString(userInfo));
        //在线人员列表
        RMap<Integer, String> onlineUserInfo = getOnlineUserCache(roomUid);
        //进入房间写如在线人员列表
        onlineUserInfo.fastPut(userId, JSONObject.toJSONString(userInfo));
        //向直播间发送当前在线人数消息
        this.sendOnlineUserCount(imLiveBroadcastRoomVo, userId, onlineUserInfo.size());

        // 黑名单发送进入消息
        if (imLiveRoomBlackService.checkBlackUser(userId, imLiveBroadcastRoomVo.getRoomUid())) {
            // 发送进入消息
            this.sendBlackJoinRoom(userId, imLiveBroadcastRoomVo);
        }

        log.info("joinRoom>>>> sendOnlineUserCount>>>>   roomUid: {}  fromUserId:{}  count:{}", roomUid, userId, onlineUserInfo.size());
        log.info("joinRoom>>>> userInfo: {}", JSONObject.toJSONString(userInfo));
    }

    /**
     * 游客加入直播间
     *
     * @param roomUid 直播间编号
     * @param userId  游客编号
     */
    @Override
    public void visitorJoinRoom(String roomUid, Integer userId) {

        // 游客直播间信息
        ImLiveBroadcastRoomVo roomVo = getVisitorLiveBroadcastRoomVo(roomUid);

        // 加入直播间用户信息
        joinRoomUserInfo(roomUid, userId, roomVo,false);
    }

    /**
     * 游客访问直播间信息
     *
     * @param roomUid 直播间编号
     */
    private ImLiveBroadcastRoomVo getVisitorLiveBroadcastRoomVo(String roomUid) {
        // 查询直播间信息
        ImLiveBroadcastRoomVo roomVo = getImLiveBroadcastRoomVo(roomUid);
        if (Objects.isNull(roomVo) || roomVo.getRoomState() == 1) {
            log.info("visitorJoinRoom>>>> joinRoom LIVE_ROOM_CANCEL roomUid: {}", roomUid);
            throw new BizException("直播间不存在");
        }

        // 判断当前直播间观看模式是否为游客模式且，直播状态
        if (ELiveViewMode.VISITOR != roomVo.getViewMode()) {
            log.info("visitorJoinRoom>>>> joinRoom LOGOUT roomUid: {} viewMode: {}", roomUid, roomVo.getViewMode());
            throw new BizException("请登陆");
        }

        // 判断当前直播间是否在直播中,直播状态 0未开始 1开始 2结束
        if (roomVo.getLiveState() == 0) {
            log.info("visitorJoinRoom>>>> joinRoom LIVE_NOT_START roomUid: {} liveState: {}", roomUid, roomVo.getLiveState());

            Date liveStartTime = DateUtil.addMinutes(roomVo.getLiveStartTime(), -PRE_LIVE_TIME_MINUTE);
            throw new BizException(DateUtil.format(liveStartTime, "yyyy年MM月dd日 HH点mm分") + " 可进入直播间准备");
        }
        if (roomVo.getLiveState() == 2) {
            log.info("visitorJoinRoom>>>> joinRoom LIVE_CLOSED roomUid: {} liveState: {}", roomUid, roomVo.getLiveState());

            //如果直播结束了还是推广状态则将推广修改为取消
            if (roomVo.getPopularize() == 1) {
                this.update(Wrappers.<ImLiveBroadcastRoom>lambdaUpdate()
                        .set(ImLiveBroadcastRoom::getPopularize, 0)
                        .eq(ImLiveBroadcastRoom::getId, roomVo.getId()));
            }

            throw new BizException("直播已结束！");
        }

        return roomVo;
    }

    /**
     * 游客加入直播间凭据
     *
     * @param roomUid     直播间编号
     * @param fingerprint 游客凭据
     * @return RoomUserInfoVo
     */
    @Override
    public RoomUserInfoVo getVisitorCredentials(String roomUid, String fingerprint) {

        // 游客直播间信息校验
        ImLiveBroadcastRoomVo roomVo = getVisitorLiveBroadcastRoomVo(roomUid);

        String today = DateTime.now().toString("yyyyMMdd");
        // 为当前游客生成惟一用户ID
        String cacheKey = MessageFormat.format("fingerprint:{0}:{1}", today, fingerprint);
        RBucket<Object> bucket = redissonClient.getBucket(cacheKey);

        Integer userId;
        if (bucket.isExists()) {

            // 查询游客用户ID
            userId = Optional.ofNullable(bucket.get()).map(x -> (int) x).orElse(-1);
        } else {

            // 用户Id缓存key
            String userIdKey = MessageFormat.format("fingerprint:userId:{0}", today);

            RAtomicLong atomicLong = redissonClient.getAtomicLong(userIdKey);
            if (!atomicLong.isExists()) {
                // 设置默认初始化值
                atomicLong.set(1000);
                // 设置缓存失效时间
                atomicLong.expire(1L, TimeUnit.DAYS);
            }

            // 生成游客用户ID
            userId = (int) atomicLong.getAndIncrement();
            // 设置用户ID缓存
            bucket.set(userId, 12L, TimeUnit.HOURS);
        }

        //房间累计用户信息-指只要进入到该房间的用户都要记录
        RMap<Integer, String> roomTotalUser = getTotalUserCache(roomUid);

        RoomUserInfoVo userInfo;
        if (roomTotalUser.containsKey(userId)) {
            //多次进入更新动态进入时间
            userInfo = JSONObject.toJavaObject(JSONObject.parseObject(roomTotalUser.get(userId)), RoomUserInfoVo.class);

            Integer exists = liveBroadcastRoomMemberService.lambdaQuery()
                    .eq(ImLiveBroadcastRoomMember::getTenantId, userInfo.getTenantId())
                    .eq(ImLiveBroadcastRoomMember::getRoomUid, roomUid)
                    .eq(ImLiveBroadcastRoomMember::getUserId, userInfo.getUserId())
                    .count();

            if (exists <= 0) {
                // 初次进入房间
                getLiveRoomVisitorInfo(userId, roomVo, fingerprint);
            }

        } else {
            //第一次进该房间 写入用户首次进入时间
            userInfo = getLiveRoomVisitorInfo(userId, roomVo, fingerprint);
        }

        // 缓存用户信息
        roomTotalUser.fastPut(userId, JSONObject.toJSONString(userInfo));

        // 游客缓存信息
        getRoomVisitorInfoCache(roomUid, userId.toString()).set(userInfo, 12L, TimeUnit.HOURS);

        // 返回直播间访客信息
        return userInfo;
    }

    /**
     * 游客微信身份
     *
     * @param code 授权码
     * @return 用户openId
     */
    @Override
    public String visitorFingerprint(String code) {

        // 微信公众号appId
        String appId = sysConfigDao.findConfigValue(SysConfigService.WECHAT_APPID);
        // 微信公众号secret
        String secret = sysConfigDao.findConfigValue(SysConfigService.WECHAT_SECRET);

        if (StringUtils.isAnyBlank(appId, secret, code)) {
            // 若微信未配置，产生64位随机数
            return RandomStringUtils.random(64);
        }

        try {
            // 获取用户OpenId接口
            String url = "https://api.weixin.qq.com/sns/oauth2/access_token?appid={0}&secret={1}&code={2}&grant_type=authorization_code";

            // 封装请求参数
            url = MessageFormat.format(url, appId, secret, code);

            String wxMpOAuth2AccessTokenUrl = HttpUtil.get(url, null);
            log.info("请求微信授权接口的参数:{}", wxMpOAuth2AccessTokenUrl);
            if (StringUtils.isEmpty(wxMpOAuth2AccessTokenUrl)) {
                throw new BizException(999,"授权失败,请重新授权");
            }
            Map<String, Object> weChatRes = AliJSONMap.tomap(wxMpOAuth2AccessTokenUrl);

            if (!weChatRes.containsKey("openid")) {
                log.error("获取微信openId失败：[{}]", JSON.toJSONString(weChatRes));

                throw new BizException(999,"授权失败,请重新授权");
            }

            return MapUtils.getString(weChatRes, "openid", "");

        } catch (Exception e) {
            log.error("获取微信openId失败：[{}]", e.getMessage(), e);

            throw new BizException(999,"授权失败,请重新授权");
        }

    }

    /**
     * 获取直播间游客信息
     * @param userId 游客编号
     * @param imLiveBroadcastRoomVo ImLiveBroadcastRoomVo
     * @param fingerprint 游客凭据
     * @return RoomUserInfoVo
     */
    private RoomUserInfoVo getLiveRoomVisitorInfo(Integer userId, ImLiveBroadcastRoomVo imLiveBroadcastRoomVo, String fingerprint) {

        RoomUserInfoVo userInfo = new RoomUserInfoVo();
        // 生成游客账号信息，缓存信息
        userInfo.setTenantId(-1);
        userInfo.setUserId(userId);
        userInfo.setUserName(MessageFormat.format("游客{0}", String.valueOf(userId)));
        userInfo.setTotalViewTime(0);
        userInfo.setFirstJoinTime(DateTime.now().toDate());

        // 游客默认头像
        String defaultAvatar = sysConfigDao.findByParamName(SysConfigService.VISITOR_DEFAULT_AVATAR).getParanValue();
        userInfo.setAvatar(Optional.ofNullable(defaultAvatar).orElse(""));

        // 记录直播间用户信息
        ImLiveBroadcastRoomMember roomMember = new ImLiveBroadcastRoomMember();
        roomMember.setTenantId(userInfo.getTenantId());
        roomMember.setRoomUid(imLiveBroadcastRoomVo.getRoomUid());
        roomMember.setUserId(userInfo.getUserId());
        roomMember.setJoinTime(DateTime.now().toDate());
        roomMember.setTotalTime(0);
        roomMember.setOnlineStatus(1);
        roomMember.setBanStatus(0);
        roomMember.setLiveRoomStatus(1);
        roomMember.setFingerprint(fingerprint);
        roomMember.setVisitorName(userInfo.getUserName());

        // 保存直播间用户信息
        liveBroadcastRoomMemberService.save(roomMember);

        return userInfo;
    }

    /**
     * 发送黑名单进入消息
     * @param userId 用户id
     */
    private void sendBlackJoinRoom(Integer userId,ImLiveBroadcastRoomVo roomVo) {

        String roomUid = roomVo.getRoomUid();
        //校验传入参数，房间uid和发送人id不能为空
        if (!WrapperUtil.checkObj(roomUid, userId)) {
            log.info(" sendOnlineUserCount>>>> param is null roomUid: {}  fromUserId:{} ", roomUid, userId);
            return;
        }

        String fromUserid = setFromUserId(userId.toString(),roomVo);
        // 消息发送用户
        LiveRoomMessage.MessageUser messageUser = null;

        // 直播间访问用户信息
        SysUser sysUser = getRoomSysUser(userId, roomUid);
        if (Objects.nonNull(sysUser)) {
            // 发送用户信息
            messageUser = LiveRoomMessage.MessageUser
                    .builder()
                    .sendUserId(sysUser.getId().toString())
                    .sendUserName(sysUser.getUsername())
                    .avatarUrl(sysUser.getAvatar())
                    .blackFlag(true)
                    .build();
        }


        LiveRoomMessage.MessageContent messageContent = LiveRoomMessage.MessageContent
            .builder()
            .sendUserInfo(messageUser)
            .build();

        LiveRoomMessage message = LiveRoomMessage.builder()
                                                 .isIncludeSender(1)
                                                 .objectName(LiveRoomMessage.WELCOME)
                                                 .fromUserId(fromUserid)
                                                 .toChatRoomId(roomUid)
                                                 .content(messageContent)
                                                 .build();

        //发送消息
        try {
            livePluginContext.getPluginService(roomVo.getServiceProvider()).sendChatRoomMessage(message);
            log.info("sendBlackJoinRoom>>>> message: {}", JSONObject.toJSONString(message));
        } catch (Exception e) {
            log.error("sendBlackJoinRoom>>>> sendMessage {} :", JSONObject.toJSONString(message));
            log.error("sendBlackJoinRoom>>>> error", e);
        }
    }

    /**
     * 直播间访问用户信息
     * @param userId 用户编号
     * @param roomUid 直播间编号
     * @return SysUser
     */
    @Override
    public SysUser getRoomSysUser(Integer userId, String roomUid) {

        SysUser sysUser = null;
        // 直播间游客或学生用户匹配
        RBucket<RoomUserInfoVo> visitorInfoCache = getRoomVisitorInfoCache(roomUid, userId.toString());
        if (getRoomVisitorInfoCache(roomUid, userId.toString()).isExists()) {

            // 游客身份信息
            RoomUserInfoVo userInfoVo = visitorInfoCache.get();
            if (Objects.nonNull(userInfoVo)) {
                sysUser = new SysUser();
                sysUser.setId(userInfoVo.getUserId());
                sysUser.setUsername(userInfoVo.getUserName());
                sysUser.setAvatar(userInfoVo.getAvatar());
            }

        } else {

            // 登录用户信息
            sysUser = sysUserFeignService.queryUserById(userId);
        }

        return sysUser;
    }

    /**
     * 获取直播间用户信息
     * @param userId 用户id
     * @param imLiveBroadcastRoomVo 直播间信息
     * @return RoomUserInfoVo
     */
    @NotNull
    private RoomUserInfoVo getLiveRoomUserInfo(Integer userId, ImLiveBroadcastRoomVo imLiveBroadcastRoomVo) {
        return getLiveRoomUserInfo(userId, imLiveBroadcastRoomVo,true);

    }
    private RoomUserInfoVo getLiveRoomUserInfo(Integer userId, ImLiveBroadcastRoomVo imLiveBroadcastRoomVo,Boolean microphoneFlag) {

        RoomUserInfoVo userInfo = getUserInfo(userId);
        userInfo.setFirstJoinTime(DateTime.now().toDate());
        userInfo.setTotalViewTime(0);

        // 记录直播间用户信息
        ImLiveBroadcastRoomMember roomMember = new ImLiveBroadcastRoomMember();
        roomMember.setTenantId(userInfo.getTenantId());
        roomMember.setRoomUid(imLiveBroadcastRoomVo.getRoomUid());
        roomMember.setUserId(userInfo.getUserId());
        roomMember.setJoinTime(DateTime.now().toDate());
        roomMember.setTotalTime(0);
        roomMember.setOnlineStatus(1);
        roomMember.setBanStatus(0);
        roomMember.setLiveRoomStatus(1);
        roomMember.setMicrophoneFlag(microphoneFlag);

        ImLiveBroadcastRoomMember liveRoomMember = liveBroadcastRoomMemberService.lambdaQuery()
                .eq(ImLiveBroadcastRoomMember::getTenantId, userInfo.getTenantId())
                .eq(ImLiveBroadcastRoomMember::getRoomUid, imLiveBroadcastRoomVo.getRoomUid())
                .eq(ImLiveBroadcastRoomMember::getUserId, userInfo.getUserId())
                .list().stream().findFirst().orElse(null);
        if (Objects.nonNull(liveRoomMember)) {
           return userInfo;
        }


        // 保存直播间用户信息
        liveBroadcastRoomMemberService.save(roomMember);

        return userInfo;
    }

    /**
     * 开始直播-录像
     *
     * @param roomUid 房间uid
     */


    public void startLive(String roomUid, Integer userId, String videoResolution) {
        startLive(roomUid, userId, videoResolution, null);
    }

    @Override
    public void startLive(String roomUid, Integer userId, String videoResolution, String sequence) {
        //查询房间信息
        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(roomUid, userId.toString());
        if (!speakerCache.isExists()) {
            return;
        }
        RoomSpeakerInfo roomSpeakerInfo = speakerCache.get();
        //已是直播状态则直接返回
        if (intEquals(roomSpeakerInfo.getState(), 0)) {
            return;
        }


        // 设置推流状态房间信息
        setPushStatus(roomSpeakerInfo.getRoomUid(), 1);

        //是否允许录像
        if (intEquals(roomSpeakerInfo.getWhetherVideo(), 0)) {
            //开始录制视频
            try {
                // 查询房间信息
                ImLiveBroadcastRoomVo imLiveBroadcastRoomVo = getImLiveBroadcastRoomVo(roomUid);
                if (Objects.isNull(imLiveBroadcastRoomVo)) {
                    log.info("opsRoom>>>> startRecord error roomUid: {}", roomUid);
                    return;
                }
                LivePluginService pluginService = livePluginContext.getPluginService(imLiveBroadcastRoomVo.getServiceProvider());
                RTCRoom rtcRoom = pluginService.rtcRoomInfo(roomSpeakerInfo.getRoomUid());
                if (rtcRoom == null) {
                    log.info("startRecord>>>> rtcRoom is null");
                    return;
                }

                if (imLiveBroadcastRoomVo.getServiceProvider().equals(RongCloudLivePlugin.PLUGIN_NAME)) {
                    imFeignService.startRecord(roomUid, videoResolution);
                } else if (imLiveBroadcastRoomVo.getServiceProvider().equals(TencentCloudLivePlugin.PLUGIN_NAME) && roomSpeakerInfo.getVideoTemplate() == 0) {
                    // 直播开始时间
                    DateTime dateTime = new DateTime(imLiveBroadcastRoomVo.getLiveStartTime());

                    RTCRoom.RecordResp recordResp = pluginService.rtcRoomRecordStart(RTCRequest.RecordStart.builder()
                           .startTime(dateTime.getMillis() )
                           .endTime(dateTime.plusDays(1).getMillis() )
                           .streamName(MessageFormat.format("{0}_{1}", imLiveBroadcastRoomVo.getRoomUid(), imGroupCoreService.getImUserId(String.valueOf(imLiveBroadcastRoomVo.getSpeakerId()),imLiveBroadcastRoomVo.getClientType().getCode())))
                           .build());
                    roomSpeakerInfo.setVideoTemplate(1);

                    log.info("startLive 直播录录开启: recordResp={}", JSON.toJSONString(recordResp));
                }


            } catch (Exception e) {
                log.error("startRecord error: {}", e.getMessage());
            }
        }
        Date now = new Date();
        //开始直播
        roomSpeakerInfo.setState(0);
        roomSpeakerInfo.setSequence(sequence);
        roomSpeakerInfo.setStartLiveTime(now);
        speakerCache.set(roomSpeakerInfo);
        log.info("startLive>>>> roomSpeakerInfo: {}", JSONObject.toJSONString(roomSpeakerInfo));
        //主播开启直播，查询所有在直播间的用户并写入观看时间
        CompletableFuture.runAsync(() -> this.asyncOpsLiveLookTime(roomUid, 1, now));
    }

    /**
     * 关闭直播-录像
     *
     * @param roomUid 房间uid
     * @param userId  老师id
     */

    public void closeLive(String roomUid, Integer userId) {
        closeLive(roomUid, userId, null);
    }
    @Override
    public void closeLive(String roomUid, Integer userId, String sequence) {
        //查询房间主播信息
        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(roomUid, userId.toString());
        if (!speakerCache.isExists()) {
            return;
        }
        RoomSpeakerInfo roomSpeakerInfo = speakerCache.get();

        //关闭直播
        closeLive(roomSpeakerInfo,sequence);
        speakerCache.set(roomSpeakerInfo);
    }

    /**
     * 关闭直播-录像
     *
     * @param roomSpeakerInfo 房间主播信息
     */
    private void closeLive(RoomSpeakerInfo roomSpeakerInfo) {
        closeLive(roomSpeakerInfo,null);
    }
    private void closeLive(RoomSpeakerInfo roomSpeakerInfo,String sequence) {
        //直播状态 true 直播中 false关闭直播
        boolean stateFlag = intEquals(roomSpeakerInfo.getState(), 0);
        //是否录像 true允许 false不允许
        boolean whetherVideoFlag = intEquals(roomSpeakerInfo.getWhetherVideo(), 0);


        // 设置推流状态房间信息
        // 查询房间信息
        ImLiveBroadcastRoomVo imLiveBroadcastRoomVo = getImLiveBroadcastRoomVo(roomSpeakerInfo.getRoomUid());

        if (imLiveBroadcastRoomVo == null) {
            log.warn("closeLive imLiveBroadcastRoomVo is null");
            return;
        }


        LivePluginService pluginService = livePluginContext.getPluginService(imLiveBroadcastRoomVo.getServiceProvider());
        if (StringUtils.isBlank(sequence)) {
            try {
                TencentWrapper.LiveStreamState liveStreamState = pluginService.liveStreamState(getStreamId(imLiveBroadcastRoomVo.getRoomUid(), Integer.parseInt(imGroupCoreService.getImUserId(String.valueOf(imLiveBroadcastRoomVo.getSpeakerId()),imLiveBroadcastRoomVo.getClientType().getCode()))));
                if (liveStreamState == null) {
                    log.error("查询直播间流失败,返回结果为空");
                } else {
                    log.info("查询直播间流状态:{},roomUid:{}", JSON.toJSONString(liveStreamState), imLiveBroadcastRoomVo.getRoomUid());
                    if (!"active".equals(liveStreamState.getStreamState())) {
                        setPushStatus(roomSpeakerInfo.getRoomUid(), 0);
                    }
                }
            } catch (Exception e) {
                log.error("查询直播间流失败,roomUid:{}", imLiveBroadcastRoomVo.getRoomUid(), e);
            }
        } else if (sequence.equals(roomSpeakerInfo.getSequence())) {
            setPushStatus(roomSpeakerInfo.getRoomUid(), 0);
        }


        //允许录像并在直播中
        if (whetherVideoFlag && stateFlag) {
            try {
                //停止录制视频
                // imFeignService.stopRecord(roomSpeakerInfo.getRoomUid());

                RTCRoom rtcRoom = pluginService.rtcRoomInfo(roomSpeakerInfo.getRoomUid());
                if (rtcRoom == null) {
                    log.warn("closeLive rtcRoom is null");
                    return;
                }

                // 融云停止录制 腾讯云录制在直播间销毁时自动停止
                if (imLiveBroadcastRoomVo.getServiceProvider().equals(RongCloudLivePlugin.PLUGIN_NAME)) {
                    pluginService.rtcRoomRecordStop(rtcRoom.getSessionId());
                }
            } catch (Exception e) {
                log.error("stopRecord error: {}", e.getMessage());
            }
        }
        //直播状态 true 直播中
        if (stateFlag) {
            Date now = new Date();
            roomSpeakerInfo.setEndLiveTime(now);
            roomSpeakerInfo.setState(1);
            //如果开播时间和本次操作结束播放时间小于1分钟则不计算观看时间
            int liveMinutes = getLookMillisecond(roomSpeakerInfo.getStartLiveTime(), null);
            if (liveMinutes > 1) {
                //写入本次直播时长
                roomSpeakerInfo.setTotalLiveTime(getLookMillisecond(roomSpeakerInfo.getStartLiveTime(), roomSpeakerInfo.getTotalLiveTime()));
                //关闭直播后异步执行计算房间人员观看时长
                CompletableFuture.runAsync(() -> this.asyncOpsLiveLookTime(roomSpeakerInfo.getRoomUid(), 2, now));
            }
            //计算完后将开始直播时间设置为空，待下次开启后再计算
            roomSpeakerInfo.setStartLiveTime(null);
        }
        log.info("closeLive>>>> roomSpeakerInfo: {}", JSONObject.toJSONString(roomSpeakerInfo));
    }

    /**
     * 设置推流状态房间信息
     * @param roomUid 房间uid
     * @param status 0:关闭直播 1:开启直播
     */
    private void setPushStatus(String roomUid, Integer status) {
        if (status == null) {
            return;
        }
        ImLiveBroadcastRoomVo imLiveBroadcastRoomVo = getImLiveBroadcastRoomVo(roomUid);
        if (imLiveBroadcastRoomVo == null) {
            log.warn("setPushStatus imLiveBroadcastRoomVo is null");
            return;
        }

        ImLiveBroadcastRoom imLiveBroadcastRoom = new ImLiveBroadcastRoom();
        imLiveBroadcastRoom.setId(imLiveBroadcastRoomVo.getId());
        imLiveBroadcastRoom.setPushStatus(status);
        this.updateById(imLiveBroadcastRoom);
        setGroupDefinedData(imLiveBroadcastRoomVo,EGroupDefinedDataType.LIVE_STATUS,status == 1?
            EOnOffStatus.ON.getCode():EOnOffStatus.OFF.getCode());

    }

    /**
     * 打开/关闭直播后-异步计算房间人员观看时长
     *
     * @param roomUid 房间uid
     * @param type    type 1:开始直播-开始录像     2:关闭直播关闭录像
     */
    private void asyncOpsLiveLookTime(String roomUid, Integer type, Date now) {
        //加锁-避免快速点击开启直播和关闭直后异步执行后直播数据错误
        boolean b = this.runIfLockCanGet(LIVE_LOOK_LOCK.replace(ROOM_UID, roomUid), () -> {
            //查询所有在直播间的用户并计算观看时长
            RMap<Integer, String> roomTotalUser = getTotalUserCache(roomUid);
            if (!roomTotalUser.isExists()) {
                return;
            }
            //查询在线人员列表
            RMap<Integer, String> onlineUserInfo = getOnlineUserCache(roomUid);
            boolean onlineUserInfoExists = onlineUserInfo.isExists();
            roomTotalUser.forEach((id, userInfoStr) -> {
                RoomUserInfoVo userInfo = JSONObject.toJavaObject(JSONObject.parseObject(userInfoStr), RoomUserInfoVo.class);
                //获取当前用户是否在房间状态 true在 false不在
                if (onlineUserInfoExists && onlineUserInfo.containsKey(id)) {
                    if (type.equals(1)) {
                        //开启直播后对当前在房间的用户写入观看时间
                        userInfo.setDynamicLookTime(now);
                    } else if (type.equals(2)) {
                        userInfo.setTotalViewTime(getLookMinutes(userInfo.getDynamicLookTime(), now, userInfo.getTotalViewTime()));
                        userInfo.setDynamicLookTime(null);
                    } else {
                        return;
                    }
                    roomTotalUser.fastPut(id, JSONObject.toJSONString(userInfo));
                }
            });
        }, 2, 1, TimeUnit.MINUTES);
        if (!b) {
            try {
                Thread.sleep(200L);
            } catch (InterruptedException e) {
                log.error("asyncOpsLiveLookTime error: {}", e.getMessage());
            }
            this.asyncOpsLiveLookTime(roomUid, type, now);
        }
    }

    /**
     * 定时任务，每分钟执行
     * 提前30分钟主动去融云注册并创建房间
     */
    @Transactional
    public void createLiveRoom() {
        log.info("createLiveRoom>>>>");
        RBucket<Object> createLock = redissonClient.getBucket("IM:LIVE_ROOM_CREATE_LOCK");
        if (!createLock.trySet(1, 1, TimeUnit.MINUTES)) {
            log.info("createLiveRoom>>>>>createLock is exists");
            return;
        }
        Date now = new Date();
        Date endTime = DateUtil.addMinutes(now, PRE_LIVE_TIME_MINUTE);
        List<ImLiveBroadcastRoom> list = this.list(new WrapperUtil<ImLiveBroadcastRoom>()
                .hasEq("live_state_", 0)
                .hasEq("room_state_", 0)
                // .hasGe("live_start_time_", now)
                .hasLe("live_start_time_", endTime)
                .queryWrapper().orderByAsc("live_start_time_"));
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        Collection<ImLiveBroadcastRoom> values = list.stream()
                                                     .collect(Collectors.toMap(o -> o.getSpeakerId(), o -> o, (k1, k2) -> k1))
                                                     .values();

        for (ImLiveBroadcastRoom value : values) {
            try {
                createLiveRoom(value);
            } catch (Exception e) {
                log.error("createLiveRoom error:{}", e.getMessage());
            }
        }
        createLock.delete();
    }

    /**
     * 去融云创建房间
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createLiveRoom(ImLiveBroadcastRoom room) {
        createLiveRoomNoTransactional(room);
    }

    @Override
    public void createLiveRoomNoTransactional(ImLiveBroadcastRoom room) {
        if (StringUtils.isBlank(room.getGroupType())) {
            room.setGroupType("NORMAL");
        }
        log.info("createLiveRoom>>>>>>roomUid:{}", room.getRoomUid());
        try {
            //生成主讲人信息到缓存
            SysUser sysUser = getSysUser(room.getSpeakerId());

            // 判断直播间时间
            Date endTime = DateUtil.addMinutes(new Date(), PRE_LIVE_TIME_MINUTE);

            if (room.getLiveStartTime().compareTo(endTime)> 0) {
                throw new BizException("直播间时间未到");
            }

            //记录用户当前房间uid
            RLock lock = redissonClient.getLock(SPEAKER_ROOM_ING_INFO_LOCK.replace(USER_ID, room.getSpeakerId().toString()));
            boolean b = lock.tryLock(10L, TimeUnit.SECONDS);
            if (!b) {
                log.info("createLiveRoom>>>>>>lock is exists");
                return;
            }
            try {

                // 判断是否有正在直播的房间
                /*List<ImLiveBroadcastRoom> list = this.lambdaQuery()
                                                     .eq(ImLiveBroadcastRoom::getSpeakerId, room.getSpeakerId())
                                                     .eq(ImLiveBroadcastRoom::getLiveState, 1)
                                                     .eq(ImLiveBroadcastRoom::getRoomState, 0)
                                                     .list();
                if (CollectionUtils.isNotEmpty(list)) {
                    throw new BizException("当前用户已经有正在直播的房间");
                }
                ImLiveBroadcastRoom one = this.lambdaQuery()
                                              .eq(ImLiveBroadcastRoom::getSpeakerId, room.getSpeakerId())
                                              .eq(ImLiveBroadcastRoom::getLiveState, 0)
                                              .eq(ImLiveBroadcastRoom::getRoomState, 0)
                                              .orderByAsc(ImLiveBroadcastRoom::getLiveStartTime)
                                              .last("limit 1")
                                              .one();
                if (one == null) {
                    throw new BizException("当前用户没有待直播的房间");
                } else if (!one.getRoomUid().equals(room.getRoomUid())) {
                    throw new BizException("当前用户存在更早的待直播的房间");
                }*/

                ImLiveBroadcastRoomVo roomVo = getImLiveBroadcastRoomVo(room.getRoomUid());
                //去融云创建房间
                LivePluginService pluginService = livePluginContext.getPluginService(room.getServiceProvider());

                // 注册主播用户信息到三方平台

                pluginService.register(imGroupCoreService.getImUserId(sysUser.getId().toString(),room.getClientType().getCode()), sysUser.getUsername(), sysUser.getAvatar());
                // 创建直播间IM群
                pluginService.chatRoomCreate(room.getRoomUid(), room.getRoomTitle(),imGroupCoreService.getImUserId(sysUser.getId().toString(),room.getClientType().getCode()));

                // 设置直播间房间属性默认值
                setDefaultRoomDefinedInfo(roomVo);
                createSpeakerInfo(room, sysUser);
                Boolean whetherVideoFlag = getRoomConfig(room.getRoomConfig()).map(o -> o.getWhether_video() == 0).orElse(true);

                // 腾讯云直播，提前生成录制规则
                if (room.getServiceProvider().equals(TencentCloudLivePlugin.PLUGIN_NAME) && whetherVideoFlag) {

                    DateTime now = DateTime.now();

                    RTCRequest.RecordStart recordStart = RTCRequest.RecordStart.builder()
                                                   .streamName(MessageFormat.format("{0}_{1}", room.getRoomUid(), imGroupCoreService.getImUserId(room.getSpeakerId().toString(),room.getClientType().getCode())))
                                                   .extra("")
                                                   .startTime(now.toDateTime().getMillis())
                                                   .endTime(now.plusDays(1).toDateTime().getMillis())
                                                   .build();

                    if ("NORMAL".equals(room.getGroupType()) || ("LIVE".equals(room.getGroupType()) && StringUtils.isBlank(room.getVideoRecord()))) {

                        // 生成录制任务
                        log.info("createLiveRoom>>>>>> recordStart:{}", recordStart.jsonString());
                        RTCRoom.RecordResp resp = pluginService.rtcRoomRecordStart(recordStart);

                        // 设置录制任务Id
                        ImLiveBroadcastRoom update = new ImLiveBroadcastRoom();
                        update.setId(room.getId());
                        if (StringUtils.isBlank(room.getVideoRecord())) {
                            update.setVideoRecord(resp.getRecordId());
                        } else {
                            update.setVideoRecord(room.getVideoRecord() + "," + resp.getRecordId());
                        }
                        updateById(update);
                    }
                }

                // 直播间类型: NORMAL, LIVE
                if (room.getGroupType().equals("NORMAL")) {
                    // imFeignService.createLiveRoom(room.getRoomUid(), room.getRoomTitle());
                    //推送预约直播间消息
                    imLiveRoomReservationService.push(room);

                    //推送直播开始消息
                    this.sendRoomLiveState(sysUser, room, MessageTypeEnum.JIGUANG_LIVE_STARTED);
                }

                // 查询黑名单人员
                List<ImLiveRoomBlack> blackList = imLiveRoomBlackService.lambdaQuery()
                                                                        .eq(ImLiveRoomBlack::getRoomUid, room.getRoomUid())
                                                                        .list();

                if (CollectionUtils.isNotEmpty(blackList)) {
                    // 将黑名单人员踢出房间
                    for (ImLiveRoomBlack black : blackList) {
                        imLiveRoomBlackService.setBlack(black.getUserId(),roomVo);
                    }
                }
            } finally {
                if (lock.getHoldCount() != 0 && lock.isHeldByCurrentThread()) {
                    lock.unlock();
                }
            }

        } catch (Exception e) {
            log.error(">>>>>>>>>> createLiveRoom error roomUid:{}", room.getRoomUid(), e);
            throw new BizException(e.getMessage());
        }
    }

    private void setDefaultRoomDefinedInfo(ImLiveBroadcastRoomVo roomVo) {
        try {
            List<TencentWrapper.ChatRoomGroupCounter> chatRoomGroupDefinedData = livePluginContext
                .getPluginService(roomVo.getServiceProvider()).getChatRoomGroupDefinedData(roomVo.getRoomUid());
            if (CollectionUtils.isNotEmpty(chatRoomGroupDefinedData)){
                return;
            }
        } catch (Exception e) {
            log.error("获取直播群配置失败", e);
        }
        List<TencentRequest.ChatRoomGroupDefinedData> appDefinedData = new ArrayList<>();
        Arrays.stream(EGroupDefinedDataType.values()).forEach(key -> {
            switch (key) {
                case ANCHOR_STATUS:
                    appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                              .key(key.getCode())
                                                                              .value("OFFLINE")
                                                                              .build());
                    break;
                case GLOBAL_BAN:
                    appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                              .key(key.getCode())
                                                                              .value(getRoomConfig(roomVo.getRoomConfig())
                                                                                         .map(o -> o.getWhether_chat() == 0 ? "OFF" : "ON")
                                                                                         .orElse("OFF"))
                                                                              .build());
                    break;
                case LIKES:
                    appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                              .key(key.getCode())
                                                                              .value("0")
                                                                              .build());
                    break;

                case MEMBER_ONLINE:
                    appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                              .key(key.getCode())
                                                                              .value("0")
                                                                              .build());
                    break;
                case MEMBER_TOTAL:
                    appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                              .key(key.getCode())
                                                                              .value("0")
                                                                              .build());
                    break;
                case LIVE_STATUS:
                    appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                              .key(key.getCode())
                                                                              .value("OFF")
                                                                              .build());
                    break;
                case ANCHOR_CAMERA:
                    appDefinedData.add(TencentRequest.ChatRoomGroupDefinedData.builder()
                                                                              .key(key.getCode())
                                                                              .value("ON")
                                                                              .build());
                    break;
            }
        });
        try {
            livePluginContext.getPluginService(roomVo.getServiceProvider())
                             .chatRoomGroupDefinedData(TencentRequest.ChatRoomGroup.builder()
                                                                                   .groupId(roomVo.getRoomUid())
                                                                                   .appDefinedData(appDefinedData)
                                                                                   .build());
            log.info("设置直播群配置成功 roomUid:{}", roomVo.getRoomUid());
        } catch (Exception e) {
            log.error("设置直播群配置失败", e);
        }
    }

    //生成主讲人信息
    private void createSpeakerInfo(ImLiveBroadcastRoom room, SysUser sysUser) {
        Date now = new Date();
        RoomSpeakerInfo speakerInfo = new RoomSpeakerInfo();
        speakerInfo.setSpeakerId(sysUser.getId());
        speakerInfo.setSpeakerName(sysUser.getRealName());
        speakerInfo.setCreateRoomTime(now);
        speakerInfo.setRoomUid(room.getRoomUid());
        speakerInfo.setTenantId(sysUser.getTenantId());
        speakerInfo.setTotalLiveTime(0);

        //获取是否允许录像
        boolean video = getRoomConfig(room.getRoomConfig())
                .filter(c -> Objects.nonNull(c.getWhether_video()))
                .map(c -> c.getWhether_video() == 0)
                .orElse(false);
        if (video) {
            //可以录制视频
            speakerInfo.setWhetherVideo(0);
            // 设置模板
            speakerInfo.setVideoTemplate(1);
        } else {
            //不可以录制视频
            speakerInfo.setWhetherVideo(1);
            speakerInfo.setVideoTemplate(0);
        }
        speakerInfo.setOs(room.getOs());
        //写入主讲人信息
        getRoomSpeakerInfoCache(room.getRoomUid(), room.getSpeakerId().toString()).set(speakerInfo);

        //生成0点赞
        redissonClient.getBucket(LIVE_ROOM_LIKE.replace(ROOM_UID, room.getRoomUid())).set(0);

        //修改房间状态为开始
        ImLiveBroadcastRoom liveRoomUpdate = new ImLiveBroadcastRoom();
        liveRoomUpdate.setId(room.getId());
        liveRoomUpdate.setLiveState(1);
        liveRoomUpdate.setUpdatedTime(now);
        liveRoomUpdate.setUpdatedBy(-2);
        liveRoomUpdate.setSpeakerStatus(1);
        baseMapper.updateById(liveRoomUpdate);
    }

    private Optional<ImLiveBroadcastRoomDto.RoomConfig> getRoomConfig(String roomConfig) {
        return Optional.ofNullable(roomConfig)
                .map(c -> JSON.parseObject(c, ImLiveBroadcastRoomDto.RoomConfig.class));
    }

    /**
     * 推送老师端-直播已经创建/开始的消息
     *
     * @param user 用户信息
     * @param room 房间信息
     * @param en   直播列表地址
     *             <P> /#/liveRoomList?t=0 未开始
     *             <P>/#/liveRoomList 直播中
     */
    private void sendRoomLiveState(SysUser user, ImLiveBroadcastRoom room, MessageTypeEnum en) {
        String baseUrl;
        if (SysUserType.EDUCATION.equals(room.getClientType())) {
            baseUrl=sysConfigDao.findConfigValue(SysConfigService.EDU_TEACHER_BASE_URL_MANAGER);
            return;
        } else {
            baseUrl=sysConfigDao.findConfigValue(SysConfigService.TEACHER_BASE_URL_MTEACHER);
        }
        if (StringUtils.isBlank(baseUrl)) {
            log.error("sendRoomState error: teacherBaseUrl is null");
            return;
        }
        baseUrl = "13?" + baseUrl;
        Map<Integer, String> pushMap = new HashMap<>();
        pushMap.put(user.getId(), user.getId().toString());
        String memo;
        Object[] objs;
        try {
            String liveDateStr = DateUtil.format(room.getLiveStartTime(), "yyyy年MM月dd日 HH点mm分");
            //创建直播间
            if (en.equals(MessageTypeEnum.JIGUANG_LIVE_CREATED)) {
                //创建直播间-发送消息-跳到未开始页面
                memo = baseUrl + "/#/liveRoomList?t=0";
                SysUser createUser = getSysUser(room.getCreatedBy());
                objs = new Object[]{createUser.getUsername(), liveDateStr, room.getLiveRemark()};
            } else if (en.equals(MessageTypeEnum.JIGUANG_LIVE_STARTED)) {
                //直播开始-发送消息-跳到直播中页面
                memo = baseUrl + "/#/liveRoomList";
                objs = new Object[]{liveDateStr, room.getRoomTitle()};
            } else {
                log.error("sendRoomState error: MessageTypeEnum error " + en);
                return;
            }
            //发送消息
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, en, pushMap,
                    null, 0, memo, "TEACHER", objs);
        } catch (Exception e) {
            log.error("sendRoomState error ", e.getCause());
        }
    }

    /**
     * 直播间统计数据
     * 观看数、点赞数，在线人数
     * @param roomVo ImLiveBroadcastRoomVo
     */
    private void getRoomData(ImLiveBroadcastRoomVo roomVo) {

        // 同步点赞数
        int likeNums = syncLikeCount(roomVo.getRoomUid());

        roomVo.setLikeNum(likeNums);
        roomVo.setTotalLookNum(0);
        roomVo.setLookNum(0);

        //累计总用户数量
        // roomVo.setTotalLookNum(getNum.apply(this::getTotalUserCache, roomVo.getRoomUid()));
        // //在房间观看用户数量
        // roomVo.setLookNum(getNum.apply(this::getOnlineUserCache, roomVo.getRoomUid()));

        // 不在从缓存中获取观看人数，改为从数据库中获取

        RoomMemberNumDto roomMemberNumDto = liveBroadcastRoomMemberDao.queryMemberNum(roomVo.getRoomUid());
        if (roomMemberNumDto == null) {
            roomMemberNumDto = new RoomMemberNumDto();
            roomMemberNumDto.setTotalNum(0);
            roomMemberNumDto.setOnlineNum(0);
        }
        roomVo.setTotalLookNum(roomMemberNumDto.getTotalNum() == null? 0 : roomMemberNumDto.getTotalNum());
        roomVo.setLookNum(roomMemberNumDto.getOnlineNum() == null? 0 : roomMemberNumDto.getOnlineNum());

        // 直播视频数
        List<ImLiveRoomVideoVo> imLiveRoomVideoVos = imLiveRoomVideoService.queryList(roomVo.getRoomUid());
        if (CollectionUtils.isNotEmpty(imLiveRoomVideoVos)) {
            roomVo.setVideoNum(imLiveRoomVideoVos.size());
        } else {
            roomVo.setVideoNum(0);
        }
    }

    /**
     * 获取房间缓存中的用户数量/观看人数
     *
     * <p> func：查询用户数量/观看人数的方法
     * <p> roomUid ：房间uid
     * <p> return ：用户数量/观看人数
     */
    private final BiFunction<Function<String, RMap<Integer, ?>>, String, Integer> getNum = (func, roomUid) -> Optional.of(roomUid)
            .map(func)
            .filter(RMap::isExists)
            .map(RMap::size)
            .orElse(0);

    private SysUser getSysUser(Integer userId) {
        return Optional.ofNullable(userId)
                .map(sysUserFeignService::queryUserById)
                .orElseThrow(() -> new BizException("用户不存在."));
    }

    private SysUser getSysUser() {
        //修改机构基础信息
        return Optional.ofNullable(sysUserFeignService.queryUserInfo())
                .orElseThrow(() -> new BizException("用户不存在."));
    }

    /**
     * 测试
     */
    public Map<String, Object> test(String roomUid) {
        return test(roomUid, null);
    }

    public Map<String, Object> test(String roomUid, String userFlag) {
        //test
        Map<String, Object> result = new HashMap<>();
        //校验房间心跳是否过期没续租
        RBucket<Date> lastRoomHeartbeatCache = redissonClient.getBucket(LIVE_ROOM_SPEAKER_HEART_BEAT.replace(ROOM_UID, roomUid));
        if (lastRoomHeartbeatCache.isExists()) {
            result.put("房间心跳", DateUtil.dateToString(lastRoomHeartbeatCache.get(), DateUtil.EXPANDED_DATE_TIME_FORMAT));
        } else {
            result.put("房间心跳", "房间心跳不存在");
        }
        String userId = "";
        try {
            String[] split = roomUid.split("-");
            userId = split[1];
        } catch (Exception ignored) {
        }
        //获取主讲人信息
        RBucket<RoomSpeakerInfo> speakerCache = getRoomSpeakerInfoCache(roomUid, userId);
        if (speakerCache.isExists()) {
            result.put("主讲人信息", speakerCache.get());
        } else {
            result.put("主讲人信息", "主讲人信息不存在");
        }
        //点赞数
        Object like = redissonClient.getBucket(LIVE_ROOM_LIKE.replace(ROOM_UID, roomUid)).get();
        if (Objects.isNull(like)) {
            like = 0;
        }
        result.put("点赞数", like);

        int totalLook = 0;
        int look = 0;
        //正在房间观看的用户数据
        List<BaseRoomUserVo> inRoomUserInfo = queryRoomOnlineUserInfo(roomUid);
        //累计总观看的用户数量
        List<RoomUserInfoVo> totalUserInfo = queryTotalRoomUserInfo(roomUid);
        if (CollectionUtils.isNotEmpty(totalUserInfo)) {
            if (CollectionUtils.isNotEmpty(inRoomUserInfo)) {
                look = inRoomUserInfo.size();
                Optional.ofNullable(userFlag)
                        .ifPresent(a -> result.put("正在观看的人员信息", inRoomUserInfo));
            } else {
                result.put("正在观看的人员信息", "没有正在观看的人员数据");
            }
            totalLook = totalUserInfo.size();
            Optional.ofNullable(userFlag)
                    .ifPresent(a -> result.put("总人员数据", totalUserInfo));
        } else {
            result.put("总人员数据", "没有人员数据");
        }
        result.put("总观看人数", totalLook);
        result.put("实时观看数", look);
        return result;
    }

    @Override
    public void shareGroup(String roomUid, String groupIds) {
        ImLiveBroadcastRoomVo imLiveBroadcastRoomVo = queryRoomInfo(roomUid);
        if (imLiveBroadcastRoomVo == null) {
            throw new BizException("直播间信息异常！");
        }
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
        sysMessageService.batchSendImGroupMessage(MessageTypeEnum.IM_SHARE_LIVE_URL, sysUser.getId().toString(), "liveRoom?" + roomUid, groupIds.split(","), null,
                imLiveBroadcastRoomVo.getTenantName(), imLiveBroadcastRoomVo.getRoomTitle(), imLiveBroadcastRoomVo.getSpeakerName(),
                DateUtil.format(imLiveBroadcastRoomVo.getLiveStartTime(), DateUtil.CHINESE_DATA_FORMAT_1),
                imLiveBroadcastRoomVo.getLiveRemark(), baseApiUrl + "/#/liveClassTransfer?roomUid=" + roomUid);
    }

    /**
     * 查询直播间在线的用户
     *
     * @param roomUid 直播间uid
     */
    @Override
    public List<BaseRoomUserVo> queryRoomLimitOnlineUserInfo(String roomUid) {
        RMap<Integer, String> onlineUserCache = getOnlineUserCache(roomUid);
        return onlineUserCache.values().parallelStream()
                .map(a -> JSONObject.toJavaObject(JSONObject.parseObject(a), BaseRoomUserVo.class))
                .limit(200)
                .collect(Collectors.toList());
    }

    /**
     * 查询直播间在线的用户
     *
     * @param roomUid 直播间uid
     */
    @Override
    public List<BaseRoomUserVo> queryRoomOnlineUserInfo(String roomUid) {
        RMap<Integer, String> onlineUserInfo = getOnlineUserCache(roomUid);
        return onlineUserInfo.values().parallelStream()
                .map(a -> JSONObject.toJavaObject(JSONObject.parseObject(a), BaseRoomUserVo.class))
                .collect(Collectors.toList());
    }

    /**
     * 根据传入的用户id查询用户信息
     *
     * @param userIdList 用户id集合
     */
    public List<BaseRoomUserVo> queryBaseUserInfo(List<Integer> userIdList) {
        return Optional.ofNullable(userIdList)
                .filter(CollectionUtils::isNotEmpty)
                .map(baseMapper::queryBaseUserInfo)
                .orElseThrow(() -> new BizException("用户id集合不能为空"));
    }

    @Override
    public PageInfo<RoomReservationUserVo> queryRoomUser(RoomReservationUserSearch query) {
        IPage<RoomReservationUserVo> page = PageUtil.getPage(query.getPage(), query.getRows());
        page = baseMapper.queryRoomUser(page, query);
        return PageUtil.pageInfo(page);
    }

    /**
     * 导出直播间预约用户信息
     *
     */
    public void exportReservationRoomUser(RoomReservationUserSearch query, HttpServletResponse response) throws IOException {
        //训练营详情-查询条件时间设置
        List<RoomReservationUserVo> list = baseMapper.queryRoomUser(query);
        OutputStream outputStream = response.getOutputStream();
        HSSFWorkbook workbook = null;
        try {
            String[] header = {"编号", "姓名", "手机号", "分部", "声部"};
            String[] body = {"userId", "username", "phone", "organName", "subjectName"};
            workbook = POIUtil.exportExcel(header, body, list);
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=replacement-" + DateUtil.getDate(new Date()) + ".xls");
            response.flushBuffer();
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
            outputStream.flush();
            workbook.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    workbook.close();
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @Override
    public PageInfo<LiveRoomGoodsOrderVo> queryLiveRoomGoodsOrderList(LiveRoomGoodsOrderQueryInfo queryInfo) {
        PageInfo<LiveRoomGoodsOrderVo> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<LiveRoomGoodsOrderVo> dataList = null;
        int count = baseMapper.countLiveRoomGoodsOrderList(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = baseMapper.queryLiveRoomGoodsOrderList(params);
            pageInfo.setStatInfo(baseMapper.sumLiveRoomGoodsOrderList(params));
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public void whetherMic(String roomUid, Integer whetherMic) {
        ImLiveBroadcastRoomVo obj = this.getImLiveBroadcastRoomVo(roomUid);
        if (obj == null) {
            throw new BizException("直播间不存在");
        }
        ImLiveBroadcastRoomDto.RoomConfig roomConfig = getRoomConfig(obj.getRoomConfig()).orElse(null);
        if (Objects.isNull(roomConfig)) {
            return;
        }
        ImLiveBroadcastRoom imLiveBroadcastRoom = new ImLiveBroadcastRoom();
        imLiveBroadcastRoom.setId(obj.getId());
        roomConfig.setWhether_mic(whetherMic);
        imLiveBroadcastRoom.setRoomConfig(JSONObject.toJSONString(roomConfig));
        this.updateById(imLiveBroadcastRoom);
    }

    /**
     * 更新主播直播间状态
     *
     * @param liveRoom 直播间状态
     */
    @Override
    public Boolean updateRoomStatus(LiveRoomStatus liveRoom) {
        // 获取直播间信息
        ImLiveBroadcastRoomVo room = this.getImLiveBroadcastRoomVo(liveRoom.getRoomUid());
        if (room == null) {
            throw new BizException("直播间不存在");
        }
        ImLiveBroadcastRoom imLiveBroadcastRoom = new ImLiveBroadcastRoom();
        imLiveBroadcastRoom.setId(room.getId());
        imLiveBroadcastRoom.setSpeakerStatus(liveRoom.getSpeakerStatus());
        // 设置直播群组自定义数据
        if (liveRoom.getSpeakerStatus() != null) {
            setGroupDefinedData(room, EGroupDefinedDataType.ANCHOR_STATUS,
                                liveRoom.getSpeakerStatus() == 0 ? EAnchorStatus.OFFLINE.getCode() : EAnchorStatus.ONLINE.getCode());
        }

        imLiveBroadcastRoom.setPushStatus(liveRoom.getPushStatus());
        // 设置推流状态
        setPushStatus(room.getRoomUid(), liveRoom.getPushStatus());

        imLiveBroadcastRoom.setBanStatus(liveRoom.getBanStatus());
        whetherChat(room.getId(), liveRoom.getBanStatus());

        // 不用前端传时间
        // imLiveBroadcastRoom.setLiveTotalTime(liveRoom.getLiveTotalTime());

        // 设置摄像头状态
        if (liveRoom.getCameraStatus() != null) {
            setGroupDefinedData(room, EGroupDefinedDataType.ANCHOR_CAMERA,
                    liveRoom.getCameraStatus() == 1 ? EOnOffStatus.ON.getCode():EOnOffStatus.OFF.getCode());
            return true;
        }

        // 设置全员闭麦状态
        if (Objects.nonNull(liveRoom.getMicStatus())) {
            setGroupDefinedData(room, EGroupDefinedDataType.ANCHOR_MIC,
                    liveRoom.getMicStatus() == 1 ? EOnOffStatus.ON.getCode() : EOnOffStatus.OFF.getCode());
            return true;
        }

        return this.updateById(imLiveBroadcastRoom);
    }


    /**
     * 直播群成员在线状态回调处理
     *
     */
    @Override
    @Transactional
    public void callbackOnMemberStateChange(TencentData.CallbackOnMemberStateChange callbackOnMemberStateChange) {

        if (callbackOnMemberStateChange == null) {
            return;
        }

        int onlineStatus = callbackOnMemberStateChange.getEventType().equals("Online")?1:0;

        // 用户id
        if (CollectionUtils.isEmpty(callbackOnMemberStateChange.getMemberList())) {
            return;
        }
        List<Long> userIds = callbackOnMemberStateChange.getMemberList()
                                                        .stream()
                                                        .map(o -> Long.parseLong(o.getMemberAccount()))
                                                        .collect(Collectors.toList());


        List<ImLiveBroadcastRoomMember> imLiveBroadcastRoomMembers = liveBroadcastRoomMemberDao.queryMember(userIds,
                                                                                                            callbackOnMemberStateChange.getGroupId());

        if (CollectionUtils.isEmpty(imLiveBroadcastRoomMembers)) {
            return;
        }

        // 根据用户id分组
        Map<Integer, ImLiveBroadcastRoomMember> userMap = imLiveBroadcastRoomMembers.stream()
                                                                                    .collect(Collectors.toMap(ImLiveBroadcastRoomMember::getUserId,o ->o, (k1, k2) -> k1));

        List<ImUserState> imUserStates = new ArrayList<>();
        for (Long userId : userIds) {
            Integer liveRoomStatus = userMap.getOrDefault(userId.intValue(), new ImLiveBroadcastRoomMember())
                                           .getLiveRoomStatus();
            ImUserState imUserState = new ImUserState();
            imUserState.setUserid(userId.toString());
            imUserState.setStatus(onlineStatus == 1 && liveRoomStatus == 1?"0":"1");
            imUserState.setOs(callbackOnMemberStateChange.getOptPlatform());
            imUserState.setTime(new Date().getTime());
            imUserState.setRoomUid(callbackOnMemberStateChange.getGroupId());
            imUserStates.add(imUserState);
        }
        liveBroadcastRoomMemberDao.updateOnlineStatus(userIds, callbackOnMemberStateChange.getGroupId(),onlineStatus);
        CompletableFuture.runAsync(() -> {
            opsRoom(imUserStates);
        });

    }

    @Override
    @Transactional
    public void callbackAfterMemberExit(TencentData.CallbackAfterMemberExit callbackAfterMemberExit) {
        if (callbackAfterMemberExit == null) {
            return;
        }

        Integer liveRoomStatus = 0;

        // 用户id
        if (CollectionUtils.isEmpty(callbackAfterMemberExit.getExitMemberList())) {
            return;
        }
        List<Long> userIds = callbackAfterMemberExit.getExitMemberList()
                                                        .stream()
                                                        .map(o -> Long.parseLong(o.getMemberAccount()))
                                                        .collect(Collectors.toList());


        List<ImUserState> imUserStates = new ArrayList<>();
        for (Long userId : userIds) {
            ImUserState imUserState = new ImUserState();
            imUserState.setUserid(userId.toString());
            imUserState.setStatus("3");
            imUserState.setOs(callbackAfterMemberExit.getOptPlatform());
            imUserState.setTime(callbackAfterMemberExit.getEventTime().atZone(ZoneId.systemDefault()).toEpochSecond());
            imUserState.setRoomUid(callbackAfterMemberExit.getGroupId());
            imUserStates.add(imUserState);
        }
        liveBroadcastRoomMemberDao.updateLiveRoomStatus(userIds, callbackAfterMemberExit.getGroupId(),liveRoomStatus);
        CompletableFuture.runAsync(() -> {
            opsRoom(imUserStates);
        });
    }

    @Override
    @Transactional
    public void callbackAfterNewMemberJoin(TencentData.CallbackAfterNewMemberJoin callbackAfterNewMemberJoin) {
        if (callbackAfterNewMemberJoin == null) {
            return;
        }

        Integer liveRoomStatus = 1;

        // 用户id
        if (CollectionUtils.isEmpty(callbackAfterNewMemberJoin.getNewMemberList())) {
            return;
        }
        List<Long> userIds = callbackAfterNewMemberJoin.getNewMemberList()
                                                    .stream()
                                                    .map(o -> Long.parseLong(o.getMemberAccount()))
                                                    .collect(Collectors.toList());


        List<ImLiveBroadcastRoomMember> imLiveBroadcastRoomMembers = liveBroadcastRoomMemberDao.queryMember(userIds,
                                                                                    callbackAfterNewMemberJoin.getGroupId());

        if (CollectionUtils.isEmpty(imLiveBroadcastRoomMembers)) {
            return;
        }

        // 根据用户id分组
        Map<Integer, ImLiveBroadcastRoomMember> userMap = imLiveBroadcastRoomMembers.stream()
                .collect(Collectors.toMap(ImLiveBroadcastRoomMember::getUserId,o ->o, (k1, k2) -> k1));

        List<ImUserState> imUserStates = new ArrayList<>();
        for (Long userId : userIds) {
            ImUserState imUserState = new ImUserState();
            imUserState.setUserid(userId.toString());
            imUserState.setStatus(userMap.getOrDefault(userId.intValue(), new ImLiveBroadcastRoomMember()).getOnlineStatus() == 1 ? "0" : "1");
            imUserState.setOs(callbackAfterNewMemberJoin.getOptPlatform());
            imUserState.setTime(callbackAfterNewMemberJoin.getEventTime().atZone(ZoneId.systemDefault()).toEpochSecond());
            imUserState.setRoomUid(callbackAfterNewMemberJoin.getGroupId());
            imUserStates.add(imUserState);
        }
        liveBroadcastRoomMemberDao.updateLiveRoomStatus(userIds, callbackAfterNewMemberJoin.getGroupId(),liveRoomStatus);
        CompletableFuture.runAsync(() -> {
            opsRoom(imUserStates);
        });
    }

    @Override
    public void destroyLiveRoom() {
        //查询状态是 未开始和已开始 的直播间
        List<ImLiveBroadcastRoom> list = this.list(Wrappers.<ImLiveBroadcastRoom>lambdaQuery()
                                                           .in(ImLiveBroadcastRoom::getLiveState,  1)
                                                           .eq(ImLiveBroadcastRoom::getRoomState, 0));
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (ImLiveBroadcastRoom imLiveBroadcastRoom : list) {

            tryDestroyLiveRoom(imLiveBroadcastRoom);

        }
    }

    @Override
    public boolean tryDestroyLiveRoom(ImLiveBroadcastRoom imLiveBroadcastRoom) {
        LivePluginService pluginService = livePluginContext.getPluginService(
            imLiveBroadcastRoom.getServiceProvider());
        try {
            if (pluginService == null) {
                log.error("查询直播间流失败,未找到对应的插件");
                return false;
            }
            if (pluginService.pluginName().equals(TencentCloudLivePlugin.PLUGIN_NAME)) {
                TencentWrapper.LiveStreamState liveStreamState = pluginService.liveStreamState(
                    getStreamId(imLiveBroadcastRoom.getRoomUid(), Integer.parseInt(imGroupCoreService.getImUserId(String.valueOf(imLiveBroadcastRoom.getSpeakerId()),imLiveBroadcastRoom.getClientType().getCode()))));
                if (liveStreamState == null) {
                    log.error("查询直播间流失败,返回结果为空");
                    return false;
                }
                log.info("查询直播间流状态:{},roomUid:{}", JSON.toJSONString(liveStreamState), imLiveBroadcastRoom.getRoomUid());
                if (!"active".equals(liveStreamState.getStreamState())) {
                    return roomDestroy(imLiveBroadcastRoom.getRoomUid());
                }
            } else if (pluginService.pluginName().equals(RongCloudLivePlugin.PLUGIN_NAME)) {
                // 融云走原有逻辑
                return destroyExpiredLiveRoom(new Date(), imLiveBroadcastRoom, 0);
            }
        } catch (Exception e) {

            log.error("查询直播间流失败", e);
            return false;
        }
        return true;
    }

    /**
     * 更新直播推流时间
     *
     * @param event TencentData.CallbackStreamStateEvent
     */
    @Override
    public void updateLiveRoomPushStreamTime(TencentData.CallbackStreamStateEvent event) {

        // 直播间ROOM_UID
        String roomId = event.getStreamId().split("_")[0];

        ImLiveBroadcastRoomVo room = getImLiveBroadcastRoomVo(roomId);
        if (Objects.isNull(room)) {
            log.warn("updateLiveRoomPushStreamTime 无效的直播间ID, roomId:{}", roomId);
            return;
        }

        // 更新直播时长
        ImLiveBroadcastRoom entity = new ImLiveBroadcastRoom();
        // 主键ID
        entity.setId(room.getId());
        // 直播累积播放时长
        int liveTotalTime = Optional.ofNullable(room.getLiveTotalTime()).orElse(0) + Integer.parseInt(event.getPushDuration());
        // 累积播放时长
        entity.setLiveTotalTime(liveTotalTime);

        updateById(entity);
    }

    /**
     * 修改用户连麦状态
     *
     * @param roomUid          直播房间id
     * @param userId           用户id
     * @param whetherMicStatus 连麦状态
     */
    @Override
    public void userWhetherMic(String roomUid, Long userId, Integer whetherMicStatus) {
        liveBroadcastRoomMemberDao.userWhetherMic(roomUid, userId, whetherMicStatus);
    }

    /**
     * 同步直播间点赞数
     *
     * @param roomUid 直播间uid
     * @return int
     */
    @Override
    public int syncLikeCount(String roomUid) {

        // 直播间信息
        ImLiveBroadcastRoomVo room = getImLiveBroadcastRoomVo(roomUid);
        if (room == null) {
            return 0;
        }

        // 融云直播间
        if (room.getServiceProvider().equals(RongCloudLivePlugin.PLUGIN_NAME)) {
            //点赞数
            return (int) Optional.ofNullable(redissonClient.getBucket(LIVE_ROOM_LIKE.replace(ROOM_UID, roomUid)).get()).orElse(0);
        }

        // 腾讯云直播
        LivePluginService pluginService = livePluginContext.getPluginService(room.getServiceProvider());
        List<TencentWrapper.ChatRoomGroupCounter> chatRoomGroupCounters = null;
        try {
            chatRoomGroupCounters = pluginService.chatRoomGroupCounterDataList(
                room.getRoomUid());
        } catch (Exception e) {
            log.error("同步直播间点赞数失败", e);
        }
        if (CollectionUtils.isEmpty(chatRoomGroupCounters)) {
            return 0;
        }
        Optional<TencentWrapper.ChatRoomGroupCounter> first = chatRoomGroupCounters.stream()
                                                                                   .filter(a -> a.getKey()
                                                                                                 .equals(
                                                                                                     EGroupDefinedDataType.LIKES.getCode()))
                                                                                   .findFirst();

        if (first.isPresent()) {
            int i = Integer.parseInt(first.get().getValue());

            ImLiveBroadcastRoomData one = liveBroadcastRoomDataService.lambdaQuery()
                                                                      .eq(ImLiveBroadcastRoomData::getRoomUid, roomUid)
                                                                      .last("limit 1")
                                                                      .one();
            // 同步点赞数
            if (one != null) {
                ImLiveBroadcastRoomData imLiveBroadcastRoomData = new ImLiveBroadcastRoomData();
                imLiveBroadcastRoomData.setId(one.getId());
                imLiveBroadcastRoomData.setLikeNum(i);
                liveBroadcastRoomDataService.updateById(imLiveBroadcastRoomData);
            }

            return i;
        }
        return 0;
    }

    /**
     * 直播间推流状态查询
     *
     * @param roomUid 直播间ID
     * @return TencentWrapper.LiveStreamState
     */
    @Override
    public TencentWrapper.LiveStreamState roomLiveStreamStatus(String roomUid) {

        // 直播间信息
        ImLiveBroadcastRoomVo roomVo = getImLiveBroadcastRoomVo(roomUid);
        if (Objects.isNull(roomVo)) {
            throw new BizException("无效的直播间ID");
        }

        TencentWrapper.LiveStreamState liveStreamState = TencentWrapper.LiveStreamState
                .builder()
                .streamState("inactive")
                .requestId("")
                .build();
        // 直播间推流状态
        try {
            LivePluginService pluginService = livePluginContext.getPluginService(roomVo.getServiceProvider());

            liveStreamState = pluginService.liveStreamState(getStreamId(roomUid, Integer.parseInt(imGroupCoreService.getImUserId(String.valueOf(roomVo.getSpeakerId()),roomVo.getClientType().getCode()))));
        } catch (Exception e) {
            log.error("roomLiveStreamStatus roomUid={}", roomUid, e);
        }

        return liveStreamState;
    }

    @Override
    public void batchUpdateLiveRoomTitle(List<String> list, String roomTitle, String liveRemark) {
        this.lambdaUpdate()
                .in(ImLiveBroadcastRoom::getRoomUid, list)
                .set(ImLiveBroadcastRoom::getRoomTitle, roomTitle)
                .set(ImLiveBroadcastRoom::getLiveRemark, liveRemark)
                .update();
    }

    @Override
    public ImLiveBroadcastRoom getByRoomUid(String liveRoomId) {
        List<ImLiveBroadcastRoom> list = this.lambdaQuery()
                .eq(ImLiveBroadcastRoom::getRoomUid, liveRoomId)
                .list();
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }


    /**
     * 查询直播间所有用户信息
     *
     * @param roomUid 直播间uid
     */
    private List<RoomUserInfoVo> queryTotalRoomUserInfo(String roomUid) {
        RMap<Integer, String> roomTotalUser = getTotalUserCache(roomUid);
        return roomTotalUser.values().parallelStream()
                .map(a -> JSONObject.toJavaObject(JSONObject.parseObject(a), RoomUserInfoVo.class))
                .collect(Collectors.toList());
    }

    /**
     * 获取直播间在线用户信息缓存
     *
     * @param roomUid 直播间uid
     * @return RMap key:userId value:BaseRoomUserVo -> jsonToString
     */
    private RMap<Integer, String> getOnlineUserCache(String roomUid) {
        return redissonClient.getMap(LIVE_ROOM_ONLINE_USER_LIST.replace(ROOM_UID, roomUid));
    }

    /**
     * 获取直播间所有用户信息缓存
     *
     * @param roomUid 直播间uid
     * @return RMap key:userId value:RoomUserInfoVo -> jsonToString
     */
    private RMap<Integer, String> getTotalUserCache(String roomUid) {
        return redissonClient.getMap(LIVE_ROOM_TOTAL_USER_LIST.replace(ROOM_UID, roomUid));
    }

    /**
     * 获取房间主讲人消息
     *
     * @param roomUid 房间uid
     * @param userId  用户id
     */
    private RBucket<RoomSpeakerInfo> getRoomSpeakerInfoCache(String roomUid, String userId) {
        return redissonClient.getBucket(LIVE_SPEAKER_INFO.replace(USER_ID, userId).replace(ROOM_UID, roomUid));
    }

    /**
     * 直播间游客信息
     * @param roomUid 直播间编号
     * @param userId 游客编号
     * @return RBucket<RoomUserInfoVo>
     */
    private RBucket<RoomUserInfoVo> getRoomVisitorInfoCache(String roomUid, String userId) {
        return redissonClient.getBucket(LIVE_VISITOR_INFO.replace(USER_ID, userId).replace(ROOM_UID, roomUid));
    }

    private RoomUserInfoVo getUserInfo(Integer userId) {
        RoomUserInfoVo userInfo = new RoomUserInfoVo();
        userInfo.setUserId(userId);
        SysUser sysUser = sysUserFeignService.queryUserById(userId);
        String name = userId.toString();
        if (Objects.nonNull(sysUser)) {
            name = sysUser.getPhone();
            if (sysUser.getUserType().contains("STUDENT")) {
                name = sysUser.getUsername();
            } else {
                if (StringUtils.isNotBlank(sysUser.getRealName())) {
                    name = sysUser.getRealName();
                }
            }
        }
        userInfo.setUserName(name);
        userInfo.setTenantId(sysUser.getTenantId());
        return userInfo;
    }

    /**
     * 计算观看时长差-分钟数不满一分钟为0
     *
     * @param startDT    开始时间
     * @param nowMinutes 现在观看时长
     */
    private int getLookMinutes(Date startDT, Integer nowMinutes) {
        return getLookMinutes(startDT, new Date(), nowMinutes);
    }

    /**
     * 计算观看时长差-分钟数不满一分钟为0
     *
     * @param startDT    开始时间
     * @param endDT      结束时间
     * @param nowMinutes 现在观看时长
     */
    private int getLookMinutes(Date startDT, Date endDT, Integer nowMinutes) {
        if (Objects.isNull(nowMinutes)) {
            nowMinutes = 0;
        }
        if (Objects.isNull(startDT)) {
            return nowMinutes;
        }
        if (startDT.getTime() > endDT.getTime()) {
            return nowMinutes;
        }
        //课程结束时间-课程开始时间
        long durationTime = endDT.getTime() - startDT.getTime();
        //相差多少分钟
        int minutesBetween = new Long(durationTime / 1000 / 60).intValue();
        minutesBetween += nowMinutes;
        return Math.max(minutesBetween, 0);
    }


    /**
     * 计算观看时长差-毫秒数
     *
     * @param startDT    开始时间
     * @param nowMinutes 现在观看时长
     */
    private int getLookMillisecond(Date startDT, Integer nowMinutes) {
        return getLookMillisecond(startDT, new Date(), nowMinutes);
    }
    /**
     * 计算观看时长差-毫秒数
     *
     * @param startDT    开始时间
     * @param endDT      结束时间
     * @param nowMinutes 现在观看时长
     */
    private int getLookMillisecond(Date startDT, Date endDT, Integer nowMinutes) {
        if (Objects.isNull(nowMinutes)) {
            nowMinutes = 0;
        }
        if (Objects.isNull(startDT)) {
            return nowMinutes;
        }
        if (startDT.getTime() > endDT.getTime()) {
            return nowMinutes;
        }
        //课程结束时间-课程开始时间
        long durationTime = endDT.getTime() - startDT.getTime();
        //相差多少分钟
        int minutesBetween = new Long(durationTime).intValue();
        minutesBetween += nowMinutes;
        return Math.max(minutesBetween, 0);
    }

    /**
     * 判断Integer是否相等-null值不相等
     *
     * @param key1 第一个Integer
     * @param key2 第二个Integer
     * @return 相等 true 不相等 false
     */
    private boolean intEquals(Integer key1, Integer key2) {
        return Objects.nonNull(key1) && Objects.nonNull(key2) && Objects.equals(key1, key2);
    }

    /**
     * 分布式锁
     *
     * @param lockName lockKey
     * @param runnable 任务
     * @param waitTime 等待抢锁的时间，若该key已被占用则等待抢锁。
     * @param timeout  超时时间
     * @param unit     时间单位
     * @return true 锁成功  false 锁失败
     */
    private boolean runIfLockCanGet(final String lockName, Runnable runnable, final long waitTime, final long timeout, TimeUnit unit) {
        RLock lock = redissonClient.getLock(lockName);
        if (Objects.isNull(lock)) {
            log.info("runIfLockCanGet lock is null lockName : {}", lockName);
            return false;
        }
        try {
            if (lock.tryLock(waitTime, timeout, unit)) {
                if (Objects.nonNull(runnable)) {
                    runnable.run();
                }
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            log.error("runIfLockCanGet error lockName : {}", lockName, e);
            throw new BizException("runIfLockCanGet error lockName :" + lockName, e);
        } finally {
            this.unlock(lock);
        }
    }

    /**
     * 解锁
     */
    private void unlock(RLock lock) {
        if (lock.getHoldCount() != 0) {
            lock.unlock();
        }
    }

    /**
     * 对比时间,2个时间一个为空返回false,否则对比时间大小
     * <p>- date1 时间1
     * <p>- date2 时间2
     */
    private final BiFunction<Date, Date, Boolean> compareDate = (date1, date2) -> {
        if (Objects.nonNull(date1) && Objects.nonNull(date2)) {
            return date1.getTime() > date2.getTime();
        } else {
            return false;
        }
    };

    /**
     * 主讲人信息
     */
    public static class RoomSpeakerInfo implements Serializable {
        //主讲人id
        private Integer speakerId;
        //主讲人名称
        private String speakerName;
        //直播状态 0 直播中 1关闭直播
        private Integer state;
        //房间uid
        private String roomUid;
        // 直播流sequence
        private String sequence;

        //房间创建时间
        private Date createRoomTime;
        //进入房间时间
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
        private Date joinRoomTime;
        //开始直播时间
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
        private Date startLiveTime;
        //开始直播时间
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
        private Date endLiveTime;
        //退出房间时间
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
        private Date exitRoomTime;
        //总直播时间
        private Integer totalLiveTime = 0;
        //是否录像 0允许 1不允许
        private Integer whetherVideo;
        //机构
        private Integer tenantId;
        //播出端-  pc网页端 移动端mobile
        private String os = "pc";

        // 腾讯视频模板 0: 没有 1：有
        private Integer videoTemplate;

        public String getSequence() {
            return sequence;
        }

        public void setSequence(String sequence) {
            this.sequence = sequence;
        }

        public Integer getVideoTemplate() {
            return videoTemplate;
        }

        public void setVideoTemplate(Integer videoTemplate) {
            this.videoTemplate = videoTemplate;
        }

        public Integer getSpeakerId() {
            return speakerId;
        }

        public void setSpeakerId(Integer speakerId) {
            this.speakerId = speakerId;
        }

        public String getSpeakerName() {
            return speakerName;
        }

        public void setSpeakerName(String speakerName) {
            this.speakerName = speakerName;
        }

        /**
         * 直播状态 0 直播中 1关闭直播
         */
        public Integer getState() {
            return state;
        }

        /**
         * 直播状态 0 直播中 1关闭直播
         */
        public void setState(Integer state) {
            this.state = state;
        }

        public String getRoomUid() {
            return roomUid;
        }

        public void setRoomUid(String roomUid) {
            this.roomUid = roomUid;
        }

        public Date getJoinRoomTime() {
            return joinRoomTime;
        }

        public void setJoinRoomTime(Date joinRoomTime) {
            this.joinRoomTime = joinRoomTime;
        }

        public Date getStartLiveTime() {
            return startLiveTime;
        }

        public void setStartLiveTime(Date startLiveTime) {
            this.startLiveTime = startLiveTime;
        }

        public Integer getTotalLiveTime() {
            return totalLiveTime;
        }

        public void setTotalLiveTime(Integer totalLiveTime) {
            this.totalLiveTime = totalLiveTime;
        }

        public Date getCreateRoomTime() {
            return createRoomTime;
        }

        public void setCreateRoomTime(Date createRoomTime) {
            this.createRoomTime = createRoomTime;
        }

        public Date getExitRoomTime() {
            return exitRoomTime;
        }

        public void setExitRoomTime(Date exitRoomTime) {
            this.exitRoomTime = exitRoomTime;
        }

        public Integer getWhetherVideo() {
            return whetherVideo;
        }

        public void setWhetherVideo(Integer whetherVideo) {
            this.whetherVideo = whetherVideo;
        }

        public Integer getTenantId() {
            return tenantId;
        }

        public void setTenantId(Integer tenantId) {
            this.tenantId = tenantId;
        }

        public Date getEndLiveTime() {
            return endLiveTime;
        }

        public void setEndLiveTime(Date endLiveTime) {
            this.endLiveTime = endLiveTime;
        }

        public String getOs() {
            return os;
        }

        public void setOs(String os) {
            this.os = os;
        }
    }

}

