package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.ym.mec.biz.dal.dao.ImLiveRoomVideoDao;
import com.ym.mec.biz.dal.entity.ImLiveRoomVideo;
import com.ym.mec.biz.dal.vo.ImLiveRoomVideoVo;
import com.ym.mec.biz.service.ImLiveRoomVideoService;
import com.ym.mec.common.page.WrapperUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 直播视频记录(ImLiveRoomVideo)表服务实现类
 *
 * @author hgw
 * @since 2022-03-07 19:06:55
 */
@Service("imLiveRoomVideoService")
public class ImLiveRoomVideoServiceImpl extends ServiceImpl<ImLiveRoomVideoDao, ImLiveRoomVideo> implements ImLiveRoomVideoService {

    private final static Logger log = LoggerFactory.getLogger(ImLiveRoomVideoServiceImpl.class);

    @Override
    public ImLiveRoomVideoDao getDao() {
        return this.baseMapper;
    }

    public List<ImLiveRoomVideoVo> queryList(String roomUid) {
        return baseMapper.queryVideoList(roomUid);
    }

    /**
     * 直播间的视频数
     *
     * @param roomIdList 直播间id
     */
    @Override
    public Map<String, Integer> queryRoomUidCountMap(List<String> roomIdList) {
        if (roomIdList == null || roomIdList.isEmpty()) {
            return Maps.newHashMap();
        }
        List<ImLiveRoomVideo> list = this.lambdaQuery().in(ImLiveRoomVideo::getRoomUid, roomIdList).list();
        Map<String, Integer> map = Maps.newHashMap();
        for (ImLiveRoomVideo imLiveRoomVideo : list) {
            String roomUid = imLiveRoomVideo.getRoomUid();
            Integer count = map.get(roomUid);
            if (count == null) {
                count = 0;
            }
            map.put(roomUid, count + 1);
        }
        return map;


    }

}

