package com.ym.mec.biz.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.EFriendRoleType;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.im.ClientEnum;
import com.ym.mec.biz.dal.enums.school.ESchoolStaffType;
import com.ym.mec.biz.dal.mapper.SchoolStaffMapper;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;
import com.ym.mec.biz.service.TeacherService;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import com.ym.mec.biz.dal.dto.ImUserFriendDto;
import com.ym.mec.biz.service.ImUserFriendService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import static com.ym.mec.biz.dal.enums.GroupType.*;

@Service
public class ImUserFriendServiceImpl extends BaseServiceImpl<Long, ImUserFriend> implements ImUserFriendService {

    @Autowired
    private ImUserFriendDao imUserFriendDao;
    @Autowired
    private TeacherService teacherService;
    @Autowired
    private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;
    @Autowired
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private VipGroupDao vipGroupDao;
    @Autowired
    private PracticeGroupDao practiceGroupDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Autowired
    private SchoolStaffMapper schoolStaffMapper;

    @Autowired
    private ImGroupMemberDao imGroupMemberDao;

    @Autowired
    private ImGroupCoreService imGroupCoreService;

    @Override
    public BaseDAO<Long, ImUserFriend> getDAO() {
        return imUserFriendDao;
    }

    @Override
    public boolean addFriend(Integer userId, Integer friendUserId, String nickname, String memo) {
        // 检查是否已经是好友
        ImUserFriend userFriend = imUserFriendDao.query(userId, friendUserId);
        if (userFriend != null) {
            throw new BizException("好友已存在");
        }

        Date date = new Date();

        userFriend = new ImUserFriend();
        userFriend.setFriendId(friendUserId);
        userFriend.setUserId(friendUserId);
        userFriend.setFriendNickname(nickname);
        userFriend.setMemo(memo);
        userFriend.setCreateTime(date);
        userFriend.setUpdateTime(date);
        imUserFriendDao.insert(userFriend);

        return true;
    }

    @Override
    public boolean deleteFriend(Integer userId, Integer friendUserId) {
        ImUserFriend userFriend = imUserFriendDao.query(userId, friendUserId);
        if (userFriend == null) {
            throw new BizException("好友不存在");
        }
        imUserFriendDao.delete(userFriend.getId());
        return true;
    }

    @Override
    public boolean updateFriendNickname(Integer userId, Integer friendUserId, String nickname, String memo) {
        ImUserFriend userFriend = imUserFriendDao.query(userId, friendUserId);
        if (userFriend == null) {
            throw new BizException("好友不存在");
        }
        Date date = new Date();
        userFriend.setFriendNickname(nickname);
        userFriend.setMemo(memo);
        userFriend.setUpdateTime(date);
        imUserFriendDao.update(userFriend);

        return true;
    }

    @Override
    public List<ImUserFriendDto> queryFriendListByUserId(Integer userId, String search) {
        String syncImUserFriendKey = "syncImUserFriend:" + userId;
        if(redisTemplate.hasKey(syncImUserFriendKey)){
            List<ImUserFriendDto> imUserFriendDtoList = imUserFriendDao.queryFriendListByUserId(userId, search);

            // 获取好友昵称
            return getImUserFriendNickname(imUserFriendDtoList);
        }else {
            List<ImUserFriendDto> imUserFriendDtoList = teacherService.queryGroupStudents1(userId, null);

            // 获取好友昵称
            return getImUserFriendNickname(imUserFriendDtoList);
        }
    }

    /**
     * 获取好友昵称
     * @param imUserFriendDtoList List<ImUserFriendDto>
     */
    private List<ImUserFriendDto> getImUserFriendNickname(List<ImUserFriendDto> imUserFriendDtoList) {

        for (ImUserFriendDto item : imUserFriendDtoList) {
            // 设置用户imUserId
            item.setImUserId(imGroupCoreService.getImUserId(String.valueOf(item.getUserId()),""));

            // 设置用户好友昵称
            if (Objects.nonNull(item.getFriend()) && StringUtils.isNotBlank(item.getFriend().getUserType())) {
                String username = item.getFriend().getUsername();

                List<String> userTypes = Lists.newArrayList(item.getFriend().getUserType().split(","));
                if (userTypes.size() > 1
                    || userTypes.contains(ClientEnum.TEACHER.getCode())
                    || userTypes.contains(ClientEnum.SYSTEM.getCode())) {
                    username = Optional.ofNullable(item.getFriend().getRealName()).filter(StringUtils::isNotBlank).orElse(username);
                }
                // 用户好友默认用户名
                if (StringUtils.isNotBlank(username)) {
                    item.setFriendNickname(username);
                }
            }
        }
        return imUserFriendDtoList;
    }

    @Override
    public ImUserFriendDto queryFriendDetail(Integer userId, Integer friendUserId) {
        ImUserFriendDto imUserFriendDto = imUserFriendDao.queryFriendDetail(userId, friendUserId);

        if (Objects.nonNull(imUserFriendDto) && Objects.nonNull(imUserFriendDto.getFriend())) {

            List<String> userTypes = Lists.newArrayList(imUserFriendDto.getFriend().getUserType().split(","));
            if (userTypes.size() > 1
                || userTypes.contains(ClientEnum.TEACHER.getCode())
                || userTypes.contains(ClientEnum.SYSTEM.getCode())) {
                // 增加用户身份数量 > 1 或者包含老师身份，统一使用实名
                imUserFriendDto.setFriendNickname(Optional.ofNullable(imUserFriendDto.getFriend().getRealName())
                    .filter(StringUtils::isNotBlank).orElse(imUserFriendDto.getFriend().getUsername()));
            }

            imUserFriendDto.setImUserId(imGroupCoreService.getImUserId(String.valueOf(imUserFriendDto.getUserId()),""));
        }
        return imUserFriendDto;
    }

    @Override
    public void batchInsert(List<ImUserFriend> imUserFriends) {
        for (ImUserFriend imUserFriend : imUserFriends) {
            if(Objects.equals(imUserFriend.getUserId(), imUserFriend.getFriendId())){
                continue;
            }
            // 检查是否已经是好友
            ImUserFriend userFriend = imUserFriendDao.query(imUserFriend.getUserId(), imUserFriend.getFriendId());
            if (userFriend != null) {
                continue;
            }
            imUserFriendDao.insert(imUserFriend);
        }
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refreshCourseImUserFriend(Long courseId) {
        CourseSchedule courseSchedule = courseScheduleDao.get(courseId);
        //获取班级关联的老师列表
        Set<Integer> teacherIds = classGroupTeacherMapperDao.queryTeacherIdsByClassGroupId(courseSchedule.getClassGroupId(),null,null);
        //获取班级关联的学员列表
        Set<Integer> studentIds = classGroupStudentMapperDao.queryStudentIdsByClassGroupId(courseSchedule.getClassGroupId(),null,null);
        ClassGroup classGroup = classGroupDao.get(courseSchedule.getClassGroupId());
        if(classGroup == null){
            return;
        }
        GroupType groupType = classGroup.getGroupType();
        List<ImUserFriend> imUserFriends = saveImUserFriends(studentIds, teacherIds, groupType, classGroup.getMusicGroupId());
        if(imUserFriends.size() > 0){
            batchInsert(imUserFriends);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refreshClassImUserFriend(Integer classGroupId) {
        //获取班级关联的老师列表
        Set<Integer> teacherIds = classGroupTeacherMapperDao.queryTeacherIdsByClassGroupId(classGroupId,null,null);
        //获取班级关联的学员列表
        Set<Integer> studentIds = classGroupStudentMapperDao.queryStudentIdsByClassGroupId(classGroupId,null,null);
        ClassGroup classGroup = classGroupDao.get(classGroupId);
        GroupType groupType = classGroup.getGroupType();
        List<ImUserFriend> imUserFriends = saveImUserFriends(studentIds, teacherIds, groupType, classGroup.getMusicGroupId());
        if(imUserFriends.size() > 0){
            batchInsert(imUserFriends);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void refreshGroupImUserFriend(String musicGroupId,GroupType groupType) {
        List<ClassGroup> classGroups = classGroupDao.queryClassGroups(musicGroupId, groupType==null?null:groupType.getCode());
        List<ImUserFriend> imUserFriends = new ArrayList<>();
        classGroups.forEach(e->{
            //获取班级关联的老师列表
            Set<Integer> teacherIds = classGroupTeacherMapperDao.queryTeacherIdsByClassGroupId(e.getId(),null,null);
            //获取班级关联的学员列表
            Set<Integer> studentIds = classGroupStudentMapperDao.queryStudentIdsByClassGroupId(e.getId(),null,null);
            imUserFriends.addAll(saveImUserFriends(studentIds,teacherIds,groupType,musicGroupId));
        });

        setSchoolStaffImFriend(musicGroupId, imUserFriends);

        if(imUserFriends.size() > 0){
            batchInsert(imUserFriends);
        }
        //更新用户朋友昵称是空的历史数据(线上没有发现这种问题)
        imUserFriendDao.updateNullNickName();
    }

    private void setSchoolStaffImFriend(String musicGroupId, List<ImUserFriend> imUserFriends) {
        // 更新
        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);

        if (musicGroup != null) {
            Map<Integer,String> studentSubjectName = new HashMap<>();
            //获取乐团关联的学员列表
            Set<Integer> studentIds = new HashSet<>(studentRegistrationDao.queryNormalUserList(musicGroupId));
            if(studentIds.size() > 0){
                studentSubjectName = MapUtil.convertMybatisMap(studentRegistrationDao.getStudentSubjectNameByStuIds(musicGroup.getId(),studentIds));
            }
            Map<Integer, String> usernameMap = new HashMap<>();
            if(studentIds.size() > 0){
                usernameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(studentIds)));
            }
            Set<Integer> educationIds = new HashSet<>();
            educationIds.add(musicGroup.getEducationalTeacherId());

            //学校乐团领队关联 乐团管理人员、教务、学员

            Set<Integer> schoolStaffIds = new HashSet<>();
            SchoolStaff schoolStaff = getSchoolOrchestraLeaderByMusicGroupId(musicGroupId);
            if (schoolStaff != null) {
                schoolStaffIds.add(schoolStaff.getUserId().intValue());
            }
            Map<Integer, String> schoolStaffRealNameMap = new HashMap<>();
            if (schoolStaffIds.size() > 0) {
                schoolStaffRealNameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(schoolStaffIds)));
            }
            Map<Integer, String> educationRealNameMap;
            educationRealNameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(educationIds)));
            //学校乐团领队关联教务
            for (Integer schoolStaffId : schoolStaffIds) {
                this.addImUserFriend(educationIds,null,educationRealNameMap,null
                        ,schoolStaffId, imUserFriends,EFriendRoleType.ORCHESTRA_MANAGER.name());
                //学校乐团领队关联学员
                this.addImUserFriend(studentIds,studentSubjectName,usernameMap,musicGroup.getName()
                        ,schoolStaffId, imUserFriends,EFriendRoleType.STUDENT.name());

            }

            //教务关联的老师、学员
            for (Integer educationId : educationIds) {
                //老师关联学校乐团领队
                this.addImUserFriend(schoolStaffIds,null,schoolStaffRealNameMap,null,educationId, imUserFriends);
            }
            //学员关联的老师、教务、维修技师
            for (Integer studentId : studentIds) {
                //老师关联学校乐团领队
                this.addImUserFriend(schoolStaffIds,null,schoolStaffRealNameMap,null,studentId, imUserFriends);
            }

        }
    }

    private SchoolStaff getSchoolOrchestraLeaderByMusicGroupId(String musicGroupId){

        MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
        // 更新学校端乐团主管的群好友列表
        if (musicGroup != null && musicGroup.getCooperationOrganId() != null) {
            // 获取乐团主管
            List<SchoolStaff> schoolStaffs = schoolStaffMapper.getByCoopIdAndType(musicGroup.getCooperationOrganId(), ESchoolStaffType.ORCHESTRA_LEADER);

            if (CollectionUtils.isNotEmpty(schoolStaffs)) {
                return schoolStaffs.get(0);
            }
        }
        return null;
    }

    @Override
    public List<ImUserFriendDto> queryFriendListByUserIdV2(Integer userId, String search, String musicGroupId, Integer subjectId, Boolean vipFlag) {
        // 逻辑过于复杂 拿出数据后筛选
        List<ImUserFriendDto> imUserFriendDtoList = imUserFriendDao.queryFriendListByUserIdV2(userId, search, musicGroupId, subjectId, vipFlag);
        imUserFriendDtoList.stream().forEach(i->{
            i.setImUserId(imGroupCoreService.getImUserId(String.valueOf(i.getUserId()),""));
        });
        return imUserFriendDtoList;

    }


    /**
     * 根据条件查询指定用户的好友列表
     */
    @Override
    public List<ImUserFriendDto> queryFriendListByUserId(ImGroupWrapper.ImQuery query) {

        List<Integer> userIds = new ArrayList<>();
        if (StringUtils.isNotBlank(query.getMusicGroupId()) ) {
            MusicGroup musicGroup = musicGroupDao.get(query.getMusicGroupId());

            List<Integer> teacherIds = new ArrayList<>();
            List<Integer> studentIds = new ArrayList<>();
            List<ClassGroupTeacherMapper> classGroupTeacherMappers = classGroupTeacherMapperDao.getByMusicGroupId(query.getMusicGroupId());
            if (CollectionUtils.isNotEmpty(classGroupTeacherMappers)) {
                for (ClassGroupTeacherMapper classGroupTeacherMapper : classGroupTeacherMappers) {
                    teacherIds.add(classGroupTeacherMapper.getUserId());
                }
            }

            List<StudentRegistration> musicGroupStu = studentRegistrationDao.getMusicGroupStu(query.getMusicGroupId());
            if (CollectionUtils.isNotEmpty(musicGroupStu)) {
                for (StudentRegistration studentRegistration : musicGroupStu) {
                    studentIds.add(studentRegistration.getUserId());
                }
            }
            userIds.add(musicGroup.getEducationalTeacherId());
            userIds.addAll(teacherIds);
            userIds.addAll(studentIds);

        }
        if (CollectionUtils.isNotEmpty(userIds)) {
            query.setUserIds(userIds);
            query.setMusicGroupId(null);
        }
        List<ImUserFriendDto> friendListByUserId = imUserFriendDao.getFriendListByUserId(query);
        friendListByUserId.stream().forEach(f->{
            f.setImUserId(imGroupCoreService.getImUserId(String.valueOf(f.getUserId()),""));
        });
        return friendListByUserId;

    }

    public List<ImUserFriend> saveImUserFriends(Set<Integer> studentIds,Set<Integer> teacherIds,GroupType groupType,String musicGroupId){
        List<ImUserFriend> imUserFriends = new ArrayList<>();
        Set<Integer> educationIds = new HashSet<>();
        Integer repairUserId = null;
        String repairUserName = null;
        MusicGroup musicGroup = new MusicGroup();
        //获取当前学员声部
        Map<Integer,String> studentSubjectName = new HashMap<>();
        Map<Integer, String> teacherSubjectNameMap = new HashMap<>();
        Map<Integer, String> schoolStaffRealNameMap = new HashMap<>();
        if(teacherIds.size() > 0){
            teacherSubjectNameMap = MapUtil.convertIntegerMap(teacherDao.querySubjectNameByIdList(new ArrayList<>(teacherIds)));
        }
        if(groupType == MUSIC){
            //学员关联乐团管理人员通讯录
            musicGroup = musicGroupDao.get(musicGroupId);
            educationIds.add(musicGroup.getDirectorUserId());
            educationIds.add(musicGroup.getEducationalTeacherId());
            if(musicGroup.getRepairUserId() != null){
                repairUserId = musicGroup.getRepairUserId();
                repairUserName = teacherDao.queryNameById(repairUserId);
            }
            if(studentIds.size() > 0){
                studentSubjectName = MapUtil.convertMybatisMap(studentRegistrationDao.getStudentSubjectNameByStuIds(musicGroup.getId(),studentIds));
            }

        }else if(groupType == VIP || groupType == LIVE){
            VipGroup vipGroup = vipGroupDao.get(Long.parseLong(musicGroupId));
            educationIds.add(vipGroup.getEducationalTeacherId());
        }else if(groupType == PRACTICE){
            PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(musicGroupId));
            educationIds.add(practiceGroup.getEducationalTeacherId());
        }
        studentIds.removeAll(Collections.singleton(null));
        teacherIds.removeAll(Collections.singleton(null));
        educationIds.removeAll(Collections.singleton(null));
        Map<Integer, String> usernameMap = new HashMap<>();
        if(studentIds.size() > 0){
            usernameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(studentIds)));
        }
        Map<Integer, String> teacherRealNameMap = new HashMap<>();
        if(teacherIds.size() > 0){
            teacherRealNameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(teacherIds)));
        }
        Map<Integer, String> educationRealNameMap = new HashMap<>();
        if(educationIds.size() > 0){
            educationRealNameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(educationIds)));
        }

        //学校乐团领队关联 乐团管理人员、教务、学员

        Set<Integer> schoolStaffIds = new HashSet<>();
        SchoolStaff schoolStaff = getSchoolOrchestraLeaderByMusicGroupId(musicGroupId);
        if (schoolStaff != null) {
            schoolStaffIds.add(schoolStaff.getUserId().intValue());
        }
        if (schoolStaffIds.size() > 0) {
             schoolStaffRealNameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(schoolStaffIds)));

        }
        //老师关联教务、学员
        for (Integer teacherId : teacherIds) {
            //老师关联学员
            this.addImUserFriend(studentIds,studentSubjectName,usernameMap,musicGroup.getName(),teacherId,imUserFriends);
            //老师关联教务
            this.addImUserFriend(educationIds,null,educationRealNameMap,null,teacherId,imUserFriends);
            //老师关联学校乐团领队
            this.addImUserFriend(schoolStaffIds,null,schoolStaffRealNameMap,null,teacherId,imUserFriends);
        }
        //教务关联的老师、学员
        for (Integer educationId : educationIds) {
            //教务关联学员
            this.addImUserFriend(studentIds,studentSubjectName,usernameMap,musicGroup.getName(),educationId,imUserFriends);
            //教务关联老师
            this.addImUserFriend(teacherIds,teacherSubjectNameMap,teacherRealNameMap,musicGroup.getName(),educationId,imUserFriends);
        }
        //学员关联的老师、教务、维修技师
        for (Integer studentId : studentIds) {
            //学员关联的老师
            this.addImUserFriend(teacherIds,teacherSubjectNameMap,teacherRealNameMap,musicGroup.getName(),studentId,imUserFriends);
            //学员关联的教务
            this.addImUserFriend(educationIds,null,educationRealNameMap,null,studentId,imUserFriends);
            //学员关联维修技师
            if(repairUserId != null){
                if(repairUserId.equals(studentId)){
                    continue;
                }
                ImUserFriend imUserFriend = new ImUserFriend();
                imUserFriend.setUserId(studentId);
                imUserFriend.setFriendId(repairUserId);
                imUserFriend.setFriendNickname(repairUserName);
                imUserFriend.setMemo(musicGroup.getName());
                imUserFriend.setTags("维修技师");
                imUserFriends.add(imUserFriend);
            }
        }

        if(schoolStaff != null){
            //学校乐团领队关联乐团管理人员
            this.addImUserFriend(teacherIds,teacherSubjectNameMap,teacherRealNameMap,musicGroup.getName()
                    ,schoolStaff.getUserId().intValue(),imUserFriends,EFriendRoleType.ORCHESTRA_TEACHER.name());
        }
        //维修技师关联学员
        if(repairUserId != null){
            this.addImUserFriend(studentIds,studentSubjectName,usernameMap,musicGroup.getName(),repairUserId,imUserFriends);
        }

        return imUserFriends;
    }

    private List<ImUserFriend> schoolStaffFriend(Integer coopId) {
        List<ImUserFriend> imUserFriends = new ArrayList<>();
        Map<Integer, String> usernameMap;
        // 获取乐团主管
        List<SchoolStaff> schoolStaffs = schoolStaffMapper.getByCoopIdAndType(coopId,null);

        if (CollectionUtils.isNotEmpty(schoolStaffs)) {

            Set<Integer> userIds = schoolStaffs.stream().map(o -> o.getUserId().intValue())
                    .collect(Collectors.toSet());
            usernameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(userIds)));

            // ID toMap userType
            Map<Integer, String> roleTypeMap = schoolStaffs.stream()
                    .collect(Collectors.toMap(o -> o.getUserId().intValue(), o -> EFriendRoleType.getByName(o.getUserType().getCode()).name(),(k1,k2) -> k2));
            // 设置好友关系
            for (Integer userId : userIds) {
                this.addImUserFriend(userIds, null, usernameMap, null, userId, imUserFriends);
            }
            for (ImUserFriend imUserFriend : imUserFriends) {
                imUserFriend.setRoleType(roleTypeMap.get(imUserFriend.getFriendId()));
            }
        }
        return imUserFriends;
    }

    /**
     * 刷新学校端好友列表
     *
     * @param coopId 合作商id
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void refreshSchoolStaffFriend(Integer coopId) {
        List<ImUserFriend> imUserFriends = schoolStaffFriend(coopId);

        if(imUserFriends.size() > 0){
            batchInsert(imUserFriends);
        }
    }


    private void userFriends(List<ImUserFriend> imUserFriends, MusicGroup musicGroup, Long userId, Set<Integer> userIds,Map<Integer, String> roleTypeMap) {
        Map<Integer, String> usernameMap;
        if (CollectionUtils.isNotEmpty(userIds)) {
            usernameMap = MapUtil.convertMybatisMap(teacherDao.queryUserNameByIdList(new ArrayList<>(userIds)));

            //获取当前学员声部
            Map<Integer,String> studentSubjectName = new HashMap<>();
            Map<Integer, String> teacherSubjectNameMap = new HashMap<>();
            List<Integer> studentIds = new ArrayList<>();
            List<Integer> teacherIds = new ArrayList<>();
            for (Integer id : userIds) {
                if( EFriendRoleType.STUDENT.name().equals(roleTypeMap.get(id))){
                    studentIds.add(id);
                }else if (EFriendRoleType.ORCHESTRA_TEACHER.name().equals(roleTypeMap.get(id))){
                    teacherIds.add(id);
                }
            }
            if(studentIds.size() > 0){
                studentSubjectName = MapUtil.convertMybatisMap(studentRegistrationDao.getStudentSubjectNameByStuIds(musicGroup.getId(),studentIds));
            }

            if(teacherIds.size() > 0){
                teacherSubjectNameMap = MapUtil.convertIntegerMap(teacherDao.querySubjectNameByIdList(new ArrayList<>(teacherIds)));
            }
            studentSubjectName.putAll(teacherSubjectNameMap);
            // 设置好友关系
            this.addImUserFriend(userIds,studentSubjectName,usernameMap, musicGroup.getName(), userId.intValue(), imUserFriends);

            for (Integer user : userIds) {
                // 设置好友关系
                this.addImUserFriend(new HashSet<>(Lists.newArrayList(
                        userId.intValue())),null,usernameMap, musicGroup.getName(),user, imUserFriends,roleTypeMap);
            }
        }
    }

    public void addImUserFriend(Set<Integer> friendUserIds,Map<Integer,String> tagsMap,
                                Map<Integer,String> nameMap,String memo,Integer userId,List<ImUserFriend> imUserFriends,Map<Integer,String> roleTypeMap) {
        addImUserFriend(friendUserIds,tagsMap,nameMap,memo,userId,imUserFriends);
        for (ImUserFriend imUserFriend : imUserFriends) {
            imUserFriend.setRoleType(roleTypeMap.get(imUserFriend.getFriendId()));
        }

    }


    public void addImUserFriend(Set<Integer> friendUserIds,Map<Integer,String> tagsMap,
                                Map<Integer,String> nameMap,String memo,Integer userId,List<ImUserFriend> imUserFriends){
        addImUserFriend(friendUserIds,tagsMap,nameMap,memo,userId,imUserFriends,"");
    }
    public void addImUserFriend(Set<Integer> friendUserIds,Map<Integer,String> tagsMap,
                                Map<Integer,String> nameMap,String memo,Integer userId,List<ImUserFriend> imUserFriends,String roleType){
        if(friendUserIds != null && friendUserIds.size() > 0){
            for (Integer friendId : friendUserIds) {
                if(userId == friendId){
                    continue;
                }
                ImUserFriend imUserFriend = new ImUserFriend();
                imUserFriend.setUserId(userId);
                imUserFriend.setFriendId(friendId);
                imUserFriend.setFriendNickname(nameMap.get(friendId));
                imUserFriend.setMemo(memo);
                imUserFriend.setRoleType(roleType);
                if(tagsMap != null){
                    imUserFriend.setTags(tagsMap.get(friendId));
                }
                imUserFriends.add(imUserFriend);
            }
        }
    }
}