package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayaedu.cbs.common.enums.school.EMaterialType;
import com.dayaedu.cbs.openfeign.client.CoursewareFeignService;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareDetailWrapper;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsLessonCoursewareWrapper;
import com.dayaedu.cbs.openfeign.wrapper.courseware.CbsMaterialRefWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.entity.LessonCourseware;
import com.ym.mec.biz.dal.entity.LessonCoursewareUserMapper;
import com.ym.mec.biz.dal.entity.SysMusicScore;
import com.ym.mec.biz.dal.mapper.LessonCoursewareMapper;
import com.ym.mec.biz.dal.wrapper.LessonCoursewareWrapper;
import com.ym.mec.biz.service.LessonCoursewareService;
import com.ym.mec.biz.service.LessonCoursewareUserMapperService;
import com.ym.mec.biz.service.SysMusicScoreService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class LessonCoursewareServiceImpl extends ServiceImpl<LessonCoursewareMapper, LessonCourseware> implements LessonCoursewareService {

    private final static Logger log = LoggerFactory.getLogger(LessonCoursewareServiceImpl.class);


    @Override
    public LessonCoursewareMapper getDao() {
        return this.baseMapper;
    }
    @Resource
    private CoursewareFeignService coursewareFeignService;
    @Resource
    private LessonCoursewareUserMapperService lessonCoursewareUserMapperService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SubjectDao subjectDao;
    @Resource
    private TeacherDao teacherDao;
    @Resource
    private SysMusicScoreService sysMusicScoreService;

    @Override
    public IPage<LessonCoursewareWrapper.LessonCoursewareDto> selectPage(IPage<LessonCoursewareWrapper.LessonCoursewareDto> page, LessonCoursewareWrapper.LessonCoursewareQuery query) {
        if(StringUtils.isNotEmpty(query.getSearch())){
            List<LessonCoursewareWrapper.LessonCoursewareDto> dtos = baseMapper.list(query);
            if(CollectionUtils.isNotEmpty(dtos)){
                List<Long> lessonCoursewareIds = dtos.stream().map(LessonCoursewareWrapper.LessonCoursewareDto::getLessonCoursewareId).collect(Collectors.toList());
                CbsLessonCoursewareWrapper.LambdaQuery lambdaQuery = new CbsLessonCoursewareWrapper.LambdaQuery();
                lambdaQuery.setIds(lessonCoursewareIds);
                lambdaQuery.setName(query.getSearch());
                List<CbsLessonCoursewareWrapper.Entity> entityList = coursewareFeignService.lessonCoursewareLambdaQuery(lambdaQuery).feignData();
                if(CollectionUtils.isNotEmpty(entityList)){
                    List<Long> collect = entityList.stream().map(e -> e.getId()).collect(Collectors.toList());
                    query.setLessonCoursewareIds(collect);
                }else {
                    return page;
                }
            }
        }
        List<LessonCoursewareWrapper.LessonCoursewareDto> dtos = baseMapper.selectPage(page, query);
        if(CollectionUtils.isNotEmpty(dtos)){
            List<Long> lessonCoursewareIds = dtos.stream().map(LessonCoursewareWrapper.LessonCoursewareDto::getLessonCoursewareId).collect(Collectors.toList());
            CbsLessonCoursewareWrapper.LambdaQuery lambdaQuery = new CbsLessonCoursewareWrapper.LambdaQuery();
            lambdaQuery.setIds(lessonCoursewareIds);
            List<CbsLessonCoursewareWrapper.Entity> entityList = coursewareFeignService.lessonCoursewareLambdaQuery(lambdaQuery).feignData();
            if(CollectionUtils.isNotEmpty(entityList)){
                Map<Long, CbsLessonCoursewareWrapper.Entity> entityMap = entityList.stream().collect(Collectors.groupingBy(CbsLessonCoursewareWrapper.Entity::getId,
                        Collectors.collectingAndThen(Collectors.toList(), e -> e.get(0))));
                List<Integer> collect = dtos.stream().map(e -> e.getSubjectId()).collect(Collectors.toList());
                String subjectIds = collect.stream().map(e -> String.valueOf(e)).collect(Collectors.joining(","));
                List<Map<Integer, String>> subjectNames = subjectDao.queryNameByIds(subjectIds);
                Map<Integer, String> subjectNameMap = MapUtil.convertMybatisMap(subjectNames);

                List<Integer> userIds = dtos.stream().map(e -> e.getCreateBy()).filter(Objects::nonNull).distinct().collect(Collectors.toList());
                userIds.addAll(dtos.stream().map(e -> e.getUpdateBy()).filter(Objects::nonNull).distinct().collect(Collectors.toList()));
                userIds.removeAll(Collections.singleton(null));
                Map<Integer, String> userNameMap = new HashMap<>();
                if(CollectionUtils.isNotEmpty(userIds)){
                    userNameMap = MapUtil.convertMybatisMap(teacherDao.queryNameByIdList(userIds));
                }
                for (LessonCoursewareWrapper.LessonCoursewareDto e : dtos) {
                    CbsLessonCoursewareWrapper.Entity entity = entityMap.get(e.getLessonCoursewareId());
                    if (Objects.isNull(entity)) {
                        continue;
                    }
                    e.setCreateName(userNameMap.get(e.getCreateBy()));
                    e.setUpdateName(userNameMap.get(e.getUpdateBy()));
                    e.setSubjectName(subjectNameMap.get(e.getSubjectId()));
                    e.setName(entity.getName());
                    e.setCover(entity.getCoverImg());
                    e.setCourseNum(entity.getCourseNum());
                }
            }
        }
        return page.setRecords(dtos);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateRangeType(LessonCoursewareWrapper.UpdateRange updateRange) {
        LessonCourseware lessonCourseware = this.getById(updateRange.getLessonCoursewareId());
        if (Objects.isNull(lessonCourseware)) {
            throw new RuntimeException("课件不存在");
        }
        lessonCourseware.setStudentRangeType(updateRange.getStudentRangeType());
        lessonCourseware.setTeacherRangeType(updateRange.getTeacherRangeType());
        lessonCourseware.setUpdateBy(sysUserService.getUserId());
        this.updateById(lessonCourseware);
        lessonCoursewareUserMapperService.lambdaUpdate()
                .eq(LessonCoursewareUserMapper::getLessonCoursewareId, updateRange.getLessonCoursewareId()).remove();
        if(StringUtils.equals(updateRange.getStudentRangeType(), "RANGE") && CollectionUtils.isNotEmpty(updateRange.getStudentIds())){
            List<LessonCoursewareUserMapper> list = updateRange.getStudentIds().stream().map(e -> {
                LessonCoursewareUserMapper mapper = new LessonCoursewareUserMapper();
                mapper.setLessonCoursewareId(updateRange.getLessonCoursewareId());
                mapper.setUserId(e);
                mapper.setUserType("STUDENT");
                return mapper;
            }).collect(Collectors.toList());
            lessonCoursewareUserMapperService.saveBatch(list);
        }
        if(StringUtils.equals(updateRange.getTeacherRangeType(), "RANGE") && CollectionUtils.isNotEmpty(updateRange.getTeacherIds())){
            List<LessonCoursewareUserMapper> list = updateRange.getTeacherIds().stream().map(e -> {
                LessonCoursewareUserMapper mapper = new LessonCoursewareUserMapper();
                mapper.setLessonCoursewareId(updateRange.getLessonCoursewareId());
                mapper.setUserId(e);
                mapper.setUserType("TEACHER");
                return mapper;
            }).collect(Collectors.toList());
            lessonCoursewareUserMapperService.saveBatch(list);
        }
    }

    @Override
    public LessonCoursewareWrapper.UpdateRangeDetail getRangeType(Integer lessonCoursewareId) {
        LessonCourseware lessonCourseware = this.getById(lessonCoursewareId);
        if (Objects.isNull(lessonCourseware)) {
            throw new RuntimeException("课件不存在");
        }
        R<CbsLessonCoursewareWrapper.LessonCourseware> lessonCoursewareR = coursewareFeignService.lessonCoursewareDetail(lessonCourseware.getLessonCourseId());
        if (!lessonCoursewareR.getCode().equals(200)) {
            throw new BizException("获取课件详情失败 {}", lessonCoursewareR.getMessage());
        }
        LessonCoursewareWrapper.UpdateRangeDetail detail = new LessonCoursewareWrapper.UpdateRangeDetail();
        detail.setName(lessonCoursewareR.getData().getName());
        detail.setLessonCoursewareId(lessonCoursewareId);
        detail.setStudentRangeType(lessonCourseware.getStudentRangeType());
        detail.setTeacherRangeType(lessonCourseware.getTeacherRangeType());
        if(lessonCourseware.getStudentRangeType().equals("RANGE")){
            List<LessonCoursewareUserMapper> studentList = lessonCoursewareUserMapperService.lambdaQuery()
                    .eq(LessonCoursewareUserMapper::getLessonCoursewareId, lessonCoursewareId)
                    .eq(LessonCoursewareUserMapper::getUserType, "STUDENT").list();
            if(CollectionUtils.isNotEmpty(studentList)){
                List<Integer> studentIds = studentList.stream().map(LessonCoursewareUserMapper::getUserId).collect(Collectors.toList());
                List<SimpleUserDto> userDtoList = teacherDao.getUsersSimpleInfo(studentIds);
                if (CollectionUtils.isNotEmpty(userDtoList)) {
                    for (SimpleUserDto userDto : userDtoList) {
                        userDto.setUserName(userDto.getNickName());
                    }
                }
                detail.setStudentList(userDtoList);
            }
        }
        if(lessonCourseware.getTeacherRangeType().equals("RANGE")){
            List<LessonCoursewareUserMapper> teacherList = lessonCoursewareUserMapperService.lambdaQuery()
                    .eq(LessonCoursewareUserMapper::getLessonCoursewareId, lessonCoursewareId)
                    .eq(LessonCoursewareUserMapper::getUserType, "TEACHER").list();
            if(CollectionUtils.isNotEmpty(teacherList)){
                List<Integer> teacherIds = teacherList.stream().map(LessonCoursewareUserMapper::getUserId).collect(Collectors.toList());
                List<SimpleUserDto> userDtoList = teacherDao.getUsersSimpleInfo(teacherIds);
                detail.setTeacherList(userDtoList);
            }
        }
        return detail;
    }

    @Override
    public CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail getLessonCourseDetail(Long id,List<Integer> categoryIdList) {
        CbsLessonCoursewareDetailWrapper.LessonCoursewareDetail detail = coursewareFeignService.lessonCoursewareDetailDetail(id).feignData();
        if (detail == null) {
            return null;
        }
        //获取知识点下的素材列表、下面关联的资源列表,如果applicationIdList是空的，那么把他剔除掉
        this.setTreeResourceId(detail.getKnowledgePointList(),categoryIdList);
        return detail;
    }

    //递归处理CbsLessonCoursewareDetailWrapper.KnowledgePointSmall
    private void setTreeResourceId(List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> knowledgePointList,List<Integer> categoryIdList){
        if(CollectionUtils.isNotEmpty(knowledgePointList)){
            knowledgePointList.forEach(knowledgePointSmall -> {
                this.setResourceId(knowledgePointSmall,categoryIdList);
                List<CbsLessonCoursewareDetailWrapper.KnowledgePointSmall> children = knowledgePointSmall.getChildren();
                if(CollectionUtils.isNotEmpty(children)){
                    this.setTreeResourceId(children,categoryIdList);
                }
            });
        }
    }


    private void setResourceId(CbsLessonCoursewareDetailWrapper.KnowledgePointSmall knowledgePointSmall,List<Integer> categoryIdList){
        List<CbsLessonCoursewareDetailWrapper.MaterialSmall> materialList = knowledgePointSmall.getMaterialList();
        if(CollectionUtils.isNotEmpty(materialList)){
            for (CbsLessonCoursewareDetailWrapper.MaterialSmall materialSmall : materialList) {
                if(materialSmall.getType() == EMaterialType.SONG){
                    SysMusicScore musicScore = sysMusicScoreService.getDao().getCbsId(Long.parseLong(materialSmall.getContent()));
                    //没有启用的曲目不显示
                    if(musicScore != null && musicScore.getShowFlag() == 1 && musicScore.getClientShowFlag() == 1) {
                        materialSmall.setContent(musicScore.getId().toString());
                    }else {
                        materialSmall.setContent(null);
                    }
                }
                //
                List<CbsMaterialRefWrapper.AddMaterialRef> materialRefs = materialSmall.getMaterialRefs();
                if (CollectionUtils.isNotEmpty(materialRefs)){
                    for (CbsMaterialRefWrapper.AddMaterialRef materialRef : materialRefs) {
                        CbsMaterialRefWrapper.RelateMaterialInfo relateMaterialInfo = materialRef.getRelateMaterialInfo();
                        if(relateMaterialInfo != null && StringUtils.equals(relateMaterialInfo.getType(),"SONG")
                                && CollectionUtils.isNotEmpty(relateMaterialInfo.getApplicationIdList())){
                            SysMusicScore musicScore = sysMusicScoreService.getDao().getCbsId(relateMaterialInfo.getId());
                            if(musicScore != null && musicScore.getShowFlag() == 1 && musicScore.getClientShowFlag() == 1
                                    && categoryIdList.contains(musicScore.getCbsMusicCategoriesId())) {
                                materialRef.setResourceIdStr(musicScore.getId().toString());
                            }
                        }
                    }
                }
            }
            //剔除掉视频素材，且没有content的数据,且不在可见分类内的数据
            materialList.removeIf(materialSmall -> materialSmall.getType() == EMaterialType.SONG
                    && StringUtils.isEmpty(materialSmall.getContent()));
        }
    }
}

