package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.LessonExaminationResultDto;
import com.ym.mec.biz.dal.dto.LessonExaminationResultDto1;
import com.ym.mec.biz.dal.dto.LessonExaminationSaveDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.page.LessonExaminationQueryInfo;
import com.ym.mec.biz.dal.page.LessonExaminationQueryInfo1;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 进度评测表(LessonExamination)表服务实现类
 *
 * @author zx
 * @since 2023-04-03 18:20:44
 */
@Service("lessonExaminationService")
public class LessonExaminationServiceImpl extends ServiceImpl<LessonExaminationDao, LessonExamination> implements LessonExaminationService {

    private final static Logger log = LoggerFactory.getLogger(LessonExaminationServiceImpl.class);

    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private StudentLessonExaminationService studentLessonExaminationService;
    @Autowired
    private StudentLessonExaminationDetailService studentLessonExaminationDetailService;
    @Autowired
    private StudentExtracurricularExercisesSituationService studentExtracurricularExercisesSituationService;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private OrganizationDao organizationDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private SysMessageService sysMessageService;

    @Override
    public LessonExaminationDao getDao() {
        return this.baseMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void publish(LessonExaminationSaveDto param) {
        log.info("publish param: {}",param);
        if(CollectionUtils.isEmpty(param.getStudentLessonExaminationSaveDtos())){
            throw new BizException("请选择学员");
        }
        String expireTime = sysConfigDao.findConfigValue("homework_expire_time");
        if(org.apache.commons.lang3.StringUtils.isEmpty(expireTime)){
            expireTime = "7";
        }
        Date expireDate = DateUtil.addDays(new Date(), Integer.parseInt(expireTime));
        LessonExamination lessonExamination = new LessonExamination();
        lessonExamination.setExpectNum(param.getExpectNum());
        if(param.getCourseScheduleId() != null){
            CourseSchedule schedule = courseScheduleDao.get(param.getCourseScheduleId());
            if(Objects.nonNull(schedule)){
                lessonExamination.setOrganId(schedule.getOrganId());
                lessonExamination.setMusicGroupId(schedule.getMusicGroupId());
                lessonExamination.setClassGroupId(schedule.getClassGroupId());
                lessonExamination.setCourseScheduleId(schedule.getId());
            }
        }else if(param.getClassGroupId() != null){
            ClassGroup classGroup = classGroupDao.get(param.getClassGroupId());
            if(Objects.nonNull(classGroup)){
                lessonExamination.setClassGroupId(classGroup.getId());
                lessonExamination.setMusicGroupId(classGroup.getMusicGroupId());
                MusicGroup musicGroup = musicGroupDao.get(classGroup.getMusicGroupId());
                if(Objects.nonNull(musicGroup)){
                    lessonExamination.setOrganId(musicGroup.getOrganId());
                }
            }
        }else if(StringUtils.isNotEmpty(param.getMusicGroupId())){
            MusicGroup musicGroup = musicGroupDao.get(param.getMusicGroupId());
            if(Objects.nonNull(musicGroup)){
                lessonExamination.setMusicGroupId(musicGroup.getId());
                lessonExamination.setOrganId(musicGroup.getOrganId());
            }
        }else {
            Teacher teacher = teacherDao.get(param.getTeacherId());
            if(Objects.nonNull(teacher)){
                lessonExamination.setOrganId(teacher.getTeacherOrganId());
            }
        }
        lessonExamination.setExpireDate(expireDate);
        lessonExamination.setTeacherId(param.getTeacherId());
        baseMapper.insert(lessonExamination);
        List<Integer> studentIds = studentLessonExaminationService.save(lessonExamination.getId(), param.getStudentLessonExaminationSaveDtos());
        studentLessonExaminationDetailService.save(lessonExamination.getId(),param.getStudentLessonExaminationSaveDtos());
        //更新服务指标相关数量信息
        studentExtracurricularExercisesSituationService.updateNum(lessonExamination,studentIds);
        //消息推送
        String teacherName = teacherDao.queryNameById(param.getTeacherId());
        String notifyUrl = "evaluating?evaluateId=" + lessonExamination.getId();
        String extra = "dayaedu" + notifyUrl + "&type=evaluating&evaluateId=" + lessonExamination.getId();
        String[] strings = studentIds.stream().map(e->e.toString()).toArray(String[]::new);
        Map<Integer, String> userMap = studentIds.stream().collect(Collectors.toMap(e -> e, v -> v.toString()));
        sysMessageService.batchSendImMessage(MessageTypeEnum.IM_PUBLISH_LESSON_EXAMINATION,param.getTeacherId().toString(), extra,
                strings,null, teacherName);
        sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_PUBLISH_LESSON_EXAMINATION,
                userMap, null, 0, notifyUrl, "STUDENT", teacherName);
    }

    @Override
    public PageInfo<LessonExaminationResultDto> queryPage(LessonExaminationQueryInfo queryInfo) {
        PageInfo<LessonExaminationResultDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        List<LessonExaminationResultDto> dataList = null;
        int count = baseMapper.findCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = baseMapper.queryPage(params);
            Set<String> musicGroupIds = dataList.stream().filter(e -> StringUtils.isNotEmpty(e.getMusicGroupId())).
                    map(e -> e.getMusicGroupId()).collect(Collectors.toSet());
            Map<String, String> musicGroupNameMap = Collections.EMPTY_MAP;
            if(CollectionUtils.isNotEmpty(musicGroupIds)){
                musicGroupNameMap = MapUtil.convertIntegerMap(musicGroupDao.queryMusicGroupNameMap(musicGroupIds));
            }
            List<Long> examinationIds = dataList.stream().map(e -> e.getId()).collect(Collectors.toList());
            Map<Long,String> nameMap = Collections.EMPTY_MAP;
            if(CollectionUtils.isNotEmpty(examinationIds)){
                nameMap = MapUtil.convertMybatisMap(studentLessonExaminationService.getDao().getStudentName(examinationIds));
            }
            List<Integer> teacherIds = dataList.stream().map(e -> e.getTeacherId()).distinct().collect(Collectors.toList());
            Map<Integer, String> teacherNameMap = MapUtil.convertIntegerMap(teacherDao.queryNameByIdList(teacherIds));
            Set<Integer> organIds = dataList.stream().map(e -> e.getOrganId()).collect(Collectors.toSet());
            Map<Integer, String> organNameMap = MapUtil.convertIntegerMap(organizationDao.findOrganNameMapList(organIds));
            for (LessonExaminationResultDto vo : dataList) {
                vo.setOrganName(organNameMap.get(vo.getOrganId()));
                vo.setMusicGroupName(musicGroupNameMap.get(vo.getMusicGroupId()));
                vo.setTeacherName(teacherNameMap.get(vo.getTeacherId()));
                String username = nameMap.get(vo.getId());
                if(StringUtils.isNotEmpty(username)){
                    if(vo.getExpectNum() > 1){
                        StringBuffer sb = new StringBuffer(username);
                        sb.append("等").append(vo.getExpectNum()).append("名学员");
                        vo.setStudentNames(sb.toString());
                    }else {
                        vo.setStudentNames(username);
                    }
                }
            }
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<LessonExaminationResultDto1> queryPageByStudent(LessonExaminationQueryInfo1 queryInfo) {
        PageInfo<LessonExaminationResultDto1> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<LessonExaminationResultDto1> dataList = null;
        int count = baseMapper.findCountByStudent(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = baseMapper.queryPageByStudent(params);
            Set<String> musicGroupIds = dataList.stream().filter(e -> StringUtils.isNotEmpty(e.getMusicGroupId())).
                    map(e -> e.getMusicGroupId()).collect(Collectors.toSet());
            Map<String, String> musicGroupNameMap = Collections.EMPTY_MAP;
            if(CollectionUtils.isNotEmpty(musicGroupIds)){
                musicGroupNameMap = MapUtil.convertIntegerMap(musicGroupDao.queryMusicGroupNameMap(musicGroupIds));
            }
            for (LessonExaminationResultDto1 vo : dataList) {
                vo.setMusicGroupName(musicGroupNameMap.get(vo.getMusicGroupId()));
            }
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public LessonExaminationResultDto1 getOne(Integer userId, Long lessonExaminationId) {
        return baseMapper.getOneByStudent(userId,lessonExaminationId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void pushEndMsg() {
        //获取截止的测验列表
        List<LessonExamination> lessonExaminations = baseMapper.queryEndList();
        if(CollectionUtils.isNotEmpty(lessonExaminations)){
            List<Long> examinations = new ArrayList<>();
            for (LessonExamination vo : lessonExaminations) {
                examinations.add(vo.getId());
                Map<Integer, String> userMap = new HashMap<>(1);
                userMap.put(vo.getTeacherId(), vo.getTeacherId().toString());
                String notifyUrl = "evaluating?evaluateId=" + vo.getId();
                String dateToString = DateUtil.dateToString(vo.getCreateTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_LESSON_EXAMINATION_END,
                        userMap, null, 0, notifyUrl, "TEACHER", dateToString);
            }
            this.lambdaUpdate().in(LessonExamination::getId,examinations)
                    .set(LessonExamination::getPushFlag,true).update();
        }
    }

    @Override
    public List<Subject> querySubject(Long lessonExaminationId) {
        return baseMapper.querySubject(lessonExaminationId);
    }

}

