package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.MusicGroupCalenderRefundPeriodDao;
import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dto.CooperationCalenderRefundDto;
import com.ym.mec.biz.dal.dto.CooperationCalenderRefundUpdateDto;
import com.ym.mec.biz.dal.dto.CooperationRefundDto;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentBaseCalender;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.page.CooperationRefundQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PayUserType.SCHOOL;

@Service
public class MusicGroupCalenderRefundPeriodServiceImpl extends BaseServiceImpl<Integer, MusicGroupCalenderRefundPeriod>  implements MusicGroupCalenderRefundPeriodService {

    @Autowired
    private MusicGroupCalenderRefundPeriodDao musicGroupCalenderRefundPeriodDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private SellOrderService sellOrderService;
    @Autowired
    private MusicGroupPaymentCalenderAddressService musicGroupPaymentCalenderAddressService;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;

    @Override
    public BaseDAO<Integer, MusicGroupCalenderRefundPeriod> getDAO() {
        return musicGroupCalenderRefundPeriodDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void savePeriod(MusicGroupPaymentBaseCalender baseCalender) {
        if(baseCalender.getPayUserType() != SCHOOL || baseCalender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) == 0){
            return;
        }
        List<MusicGroupCalenderRefundPeriod> refundPeriods = baseCalender.getMusicGroupCalenderRefundPeriods();
        if(CollectionUtils.isEmpty(refundPeriods)){
            throw new BizException("请录入回款周期");
        }
        MusicGroup musicGroup = musicGroupDao.get(baseCalender.getMusicGroupId());
        Date date = new Date();
        refundPeriods.forEach(e-> {
            e.setCalenderId(baseCalender.getCalenderId());
            e.setSubRefundAmount(e.getRefundAmount());
            e.setOrganId(musicGroup.getOrganId());
            e.setCooperationOrganId(musicGroup.getCooperationOrganId());
            e.setCreateTime(date);
            e.setUpdateTime(date);
        });
        musicGroupCalenderRefundPeriodDao.batchInsert(refundPeriods);
    }

    @Override
    public List<MusicGroupCalenderRefundPeriod> findByCalenderId(Long calenderId) {
        return musicGroupCalenderRefundPeriodDao.findByCalenderId(calenderId);
    }

    @Override
    public PageInfo<CooperationRefundDto> queryCoopRefund(CooperationRefundQueryInfo queryInfo) {
        PageInfo<CooperationRefundDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<CooperationRefundDto> dataList = null;
        int count = musicGroupCalenderRefundPeriodDao.countCoopRefund(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = musicGroupCalenderRefundPeriodDao.queryCoopRefund(params);
            List<Integer> organIds = dataList.stream().map(e -> e.getOrganId()).distinct().collect(Collectors.toList());
            List<Integer> eduUserIds = dataList.stream().map(e -> e.getEducationUserId()).distinct().collect(Collectors.toList());
            Map<Integer, String> maps = MapUtil.convertIntegerMap(teacherDao.queryNameByIdList(eduUserIds));
            Map<Integer, String> userOrganNameMap = this.getMap("organization","id_","name_",organIds,queryInfo.getTenantId(),Integer.class,String.class);
            dataList.forEach(e->{
                e.setEducationUserName(maps.get(e.getEducationUserId()));
                e.setOrganName(userOrganNameMap.get(e.getOrganId()));
                e.setIncome(e.getReceivable().subtract(e.getPrepaidFee()));
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public BigDecimal sumCoopRefund(CooperationRefundQueryInfo queryInfo) {
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        return musicGroupCalenderRefundPeriodDao.sumCoopRefund(params);
    }

    @Override
    public PageInfo<CooperationCalenderRefundDto> queryCoopCalender(CooperationRefundQueryInfo queryInfo) {
        PageInfo<CooperationCalenderRefundDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<CooperationCalenderRefundDto> dataList = null;
        int count = musicGroupCalenderRefundPeriodDao.countCoopCalender(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = musicGroupCalenderRefundPeriodDao.queryCoopCalender(params);
            dataList.forEach(e->{
                int compareTo = e.getPrepaidFee().compareTo(BigDecimal.ZERO);
                e.setRefundStatus(compareTo > 0 ? "ING":"END");
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteByCalenderId(Long id) {
        musicGroupCalenderRefundPeriodDao.deleteByCalenderId(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean calcRefundAmount(Long calenderId,String orderNo, BigDecimal amount) {
        if(calenderId != null){
            //获取回款周期
            List<MusicGroupCalenderRefundPeriod> refundPeriods = musicGroupCalenderRefundPeriodDao.findByCalenderId(calenderId);
            if(CollectionUtils.isNotEmpty(refundPeriods)){
                Date date = new Date();
                BigDecimal subAmount = amount;
                for (int i = 0; i < refundPeriods.size(); i++) {
                    MusicGroupCalenderRefundPeriod refundPeriod = refundPeriods.get(i);
                    if(i == refundPeriods.size() - 1 && refundPeriod.getRefundFlag()){
                        refundPeriod.setIncome(refundPeriod.getIncome().add(subAmount));
                        if(StringUtils.isEmpty(refundPeriod.getOrderNos())){
                            refundPeriod.setOrderNos(orderNo);
                        }else {
                            refundPeriod.setOrderNos(refundPeriod.getOrderNos() + "," + orderNo);
                        }
                        refundPeriod.setUpdateTime(date);
                        break;
                    }
                    if(refundPeriod.getRefundFlag()){
                        continue;
                    }
                    if(subAmount.compareTo(refundPeriod.getSubRefundAmount()) >= 0){
                        if(i == refundPeriods.size() - 1){
                            refundPeriod.setIncome(refundPeriod.getIncome().add(subAmount));
                            subAmount = BigDecimal.ZERO;
                        }else {
                            refundPeriod.setIncome(refundPeriod.getIncome().add(refundPeriod.getSubRefundAmount()));
                            subAmount = subAmount.subtract(refundPeriod.getSubRefundAmount());
                        }
                        refundPeriod.setSubRefundAmount(BigDecimal.ZERO);
                        refundPeriod.setRefundFlag(true);
                    }else {
                        refundPeriod.setIncome(refundPeriod.getIncome().add(subAmount));
                        refundPeriod.setSubRefundAmount(refundPeriod.getSubRefundAmount().subtract(subAmount));
                        subAmount = BigDecimal.ZERO;
                    }
                    if(StringUtils.isEmpty(refundPeriod.getOrderNos())){
                        refundPeriod.setOrderNos(orderNo);
                    }else {
                        refundPeriod.setOrderNos(refundPeriod.getOrderNos() + "," + orderNo);
                    }
                    refundPeriod.setUpdateTime(date);
                }
                musicGroupCalenderRefundPeriodDao.batchUpdate(refundPeriods);
                return refundPeriods.get(refundPeriods.size()-1).getRefundFlag();
            }
        }
        return false;
    }

    @Override
    public void batchUpdate(List<CooperationCalenderRefundUpdateDto> refundPeriods) {
        List<Integer> periodsIds = refundPeriods.stream().map(e -> e.getId()).collect(Collectors.toList());
        List<MusicGroupCalenderRefundPeriod> periods = musicGroupCalenderRefundPeriodDao.queryByIds(periodsIds);
        BigDecimal originalAmount = periods.stream().map(e -> e.getRefundAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);
        Map<Integer, CooperationCalenderRefundUpdateDto> periodMap = refundPeriods.stream().collect(Collectors.toMap(e -> e.getId(), e -> e));
        periods.forEach(e->{
            CooperationCalenderRefundUpdateDto dto = periodMap.get(e.getId());
            e.setRefundAmount(dto.getRefundAmount());
            e.setMemo(dto.getMemo());
            e.setRefundDate(dto.getRefundDate());
            e.setResponsiblePerson(dto.getResponsiblePerson());
            if (e.getRefundAmount().compareTo(e.getIncome()) <= 0) {
                e.setSubRefundAmount(BigDecimal.ZERO);
                e.setRefundFlag(true);
            } else {
                e.setSubRefundAmount(e.getRefundAmount().subtract(e.getIncome()));
                e.setRefundFlag(false);
            }
        });
        musicGroupCalenderRefundPeriodDao.batchUpdate(periods);
        BigDecimal currentAmount = periods.stream().map(e -> e.getRefundAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);
        if(originalAmount.compareTo(currentAmount) != 0){
            //变更订单信息
            Long calenderId = periods.get(0).getCalenderId();
            MusicGroupPaymentCalenderAddress address =
                    musicGroupPaymentCalenderAddressService.lambdaQuery().eq(MusicGroupPaymentCalenderAddress::getCalenderId, calenderId).one();
            if(Objects.isNull(address)){
                return;
            }
            StudentPaymentOrder order = studentPaymentOrderService.findOrderByOrderNo(address.getOrderNo());
            if (Objects.isNull(order)) {
                return;
            }
            order.setExpectAmount(currentAmount);
            order.setActualAmount(currentAmount);
            studentPaymentOrderService.update(order);
            List<SellOrder> orderSellOrder = sellOrderService.getDao().getOrderSellOrder(order.getId());
            if (CollectionUtils.isEmpty(orderSellOrder)) {
                return;
            }
            sellOrderService.updateSellOrder(orderSellOrder,order);
        }
    }
}