package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.FeeStudentDto;
import com.ym.mec.biz.dal.dto.MusicArrearageStudentDto;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentCalenderDetailDto;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus;
import com.ym.mec.biz.dal.enums.CourseViewTypeEnum;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.OrderDetailTypeEnum;
import com.ym.mec.biz.dal.page.ArrearageStudentsQueryInfo;
import com.ym.mec.biz.dal.page.MusicCalenderDetailQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.thirdparty.yqpay.DateUtils;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PayUserType.SCHOOL;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PayUserType.STUDENT;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum.AUDITING;
import static com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus.NON_PAYMENT;
import static com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus.PAID_COMPLETED;

@Service
public class MusicGroupPaymentCalenderDetailServiceImpl extends BaseServiceImpl<Long, MusicGroupPaymentCalenderDetail> implements MusicGroupPaymentCalenderDetailService {

    @Autowired
    private MusicGroupPaymentCalenderDetailDao musicGroupPaymentCalenderDetailDao;
    @Autowired
    private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
    @Autowired
    private MusicGroupPaymentCalenderService musicGroupPaymentCalenderService;
    @Autowired
    private MusicGroupPaymentStudentCourseDetailService musicGroupPaymentStudentCourseDetailService;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private MusicGroupStudentFeeDao musicGroupStudentFeeDao;
    @Autowired
    private ActivityUserMapperService activityUserMapperService;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private MusicGroupPaymentCalenderMemberService musicGroupPaymentCalenderMemberService;
    @Autowired
    private MusicGroupPaymentCalenderActivityService musicGroupPaymentCalenderActivityService;
    @Autowired
    private StudentPaymentOrderDetailDao studentPaymentOrderDetailDao;
    @Autowired
    private CloudTeacherOrderService cloudTeacherOrderService;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private VipGroupActivityDao vipGroupActivityDao;
    @Autowired
    private OperatingReportCloudService operatingReportCloudService;
    @Autowired
    private MemberFeeSettingService memberFeeSettingService;

    @Override
    public BaseDAO<Long, MusicGroupPaymentCalenderDetail> getDAO() {
        return musicGroupPaymentCalenderDetailDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String openPayment(String ids, Date startPaymentDate, Date deadlinePaymentDate) {
        if (StringUtils.isEmpty(ids)) {
            throw new BizException("参数校验失败");
        }
        //获取可开启缴费列表
        List<MusicGroupPaymentCalenderDetail> calenderDetails = musicGroupPaymentCalenderDetailDao.queryCanOpenList(ids);
        if (calenderDetails.size() == 0) {
            return "操作失败：所选学员已开启缴费，或已完成缴费";
        }
        int length = ids.split(",").length;
        Date date = new Date();

        Long calenderId = calenderDetails.get(0).getMusicGroupPaymentCalenderId();
        MusicGroupPaymentCalender calender = musicGroupPaymentCalenderService.get(calenderId);
        calenderDetails.forEach(e -> {
            e.setUpdateTime(date);
            e.setOpen(1);
            e.setStartPaymentDate(startPaymentDate);
            e.setDeadlinePaymentDate(deadlinePaymentDate);
        });
        musicGroupPaymentCalenderDetailDao.batchUpdate(calenderDetails);
        Set<Integer> studentIds = calenderDetails.stream().map(e -> e.getUserId()).collect(Collectors.toSet());
        //推送消息
        if (calender.getPayUserType() == STUDENT && studentIds.size() > 0) {
            Map<Integer, String> push = new HashMap<>();
            for (Integer userId : studentIds) {
                push.put(userId, userId + "");
            }
            MusicGroup musicGroup = musicGroupDao.get(calender.getMusicGroupId());
            String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL_MDAYA);
            String memo = "4?" + baseUrl + "/#/musicGroupRenew?calenderId=" + calender.getId() + "&id=" + calender.getMusicGroupId();
            // 发送续费通知
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
                    null, 0, memo, "STUDENT", musicGroup.getName());
            //获取未缴费且不是0元的学员列表
            Map<Integer, String> studentMaps = MapUtil.convertMybatisMap(musicGroupPaymentCalenderDetailDao.queryNoPaymentAndNotZeroStudent(calender.getId(), studentIds));
            if (studentMaps.size() > 0) {
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.STUDENT_SMS_WAIT_RENEW_MESSAGE, studentMaps,
                        null, 0, memo, null, musicGroup.getName());
            }
        }
        StringBuffer append = new StringBuffer("操作成功: 成功开启缴费").append(studentIds.size()).append("条,失败").append(length - studentIds.size()).append("条");
        return append.toString();
    }

    @Override
    public List<SimpleUserDto> findMusicGroupStudentWithSubject(String musicGroupId, Integer subjectId) {
        if (Objects.isNull(musicGroupId)) {
            throw new BizException("请指定乐团");
        }
        return musicGroupPaymentCalenderDetailDao.findMusicGroupStudentWithSubject(musicGroupId, subjectId);
    }

    @Override
    public void refreshUserMusicGroupPaymentStatusTask() {
        musicGroupPaymentCalenderDetailDao.refreshUserMusicGroupPaymentStatusTask();
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchAdd(MusicGroupPaymentCalender musicGroupPaymentCalender, List<MusicGroupPaymentCalenderStudentDetail> musicGroupPaymentCalenderStudentDetails) {
        Map<Integer, List<MusicGroupPaymentCalenderStudentDetail>> collect = musicGroupPaymentCalenderStudentDetails.stream().collect(Collectors.groupingBy(e -> e.getUserId()));
        MusicGroupPaymentCalenderDetail musicGroupPaymentCalenderDetail = null;
        Long musicGroupPaymentCalenderId = musicGroupPaymentCalender.getId();
        for (Integer studentId : collect.keySet()) {
            musicGroupPaymentCalenderDetail = new MusicGroupPaymentCalenderDetail();
            musicGroupPaymentCalenderDetail.setMusicGroupPaymentCalenderId(musicGroupPaymentCalenderId);
            musicGroupPaymentCalenderDetail.setPaymentStatus(PaymentStatus.NON_PAYMENT);
            List<MusicGroupPaymentCalenderStudentDetail> calenderStudentDetails = collect.get(studentId);
            BigDecimal totalPrice = calenderStudentDetails.stream().map(e -> e.getCourseCurrentPrice()).reduce(BigDecimal.ZERO, BigDecimal::add);
//			musicGroupPaymentCalenderDetail.setExpectAmount(totalPrice);
            if (musicGroupPaymentCalender.getStatus() != AUDITING) {
                if (totalPrice.doubleValue() == 0d) {
                    musicGroupPaymentCalenderDetail.setPaymentStatus(PaymentStatus.PAID_COMPLETED);
                    musicGroupPaymentCalender.setActualNum(musicGroupPaymentCalender.getActualNum() + 1);
//					musicGroupPaymentCalenderDetail.setActualAmount(totalPrice);
                }
            }
            musicGroupPaymentCalenderDetail.setMusicGroupId(musicGroupPaymentCalender.getMusicGroupId());
            musicGroupPaymentCalenderDetail.setUserId(studentId);
            musicGroupPaymentCalenderDetail.setStartPaymentDate(musicGroupPaymentCalender.getStartPaymentDate());
            musicGroupPaymentCalenderDetail.setDeadlinePaymentDate(musicGroupPaymentCalender.getDeadlinePaymentDate());
            musicGroupPaymentCalenderDetail.setResponsibleUserId(musicGroupPaymentCalender.getOperator());
            musicGroupPaymentCalenderDetailDao.insert(musicGroupPaymentCalenderDetail);
        }
        musicGroupPaymentCalender.setExpectNum(collect.keySet().size());
        musicGroupPaymentCalenderService.update(musicGroupPaymentCalender);
        //给学员推送缴费通知
        if (musicGroupPaymentCalender.getStatus() == PaymentCalenderStatusEnum.OPEN) {
            musicGroupPaymentCalenderService.pushWaitRenewMessage(musicGroupPaymentCalenderId, musicGroupDao.get(musicGroupPaymentCalender.getMusicGroupId()), collect.keySet());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchAdd(String batchNo, Set<Integer> userIdList) {
        List<MusicGroupPaymentCalender> musicGroupPaymentCalenderList = musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
        if (CollectionUtils.isEmpty(musicGroupPaymentCalenderList)) {
            throw new BizException("操作失败：缴费项目不存在");
        }
        MusicGroupPaymentCalender musicGroupPaymentCalender = musicGroupPaymentCalenderList.get(0);
        String musicGroupId = musicGroupPaymentCalender.getMusicGroupId();
        //获取欠费学员列表
        if(musicGroupPaymentCalender.getPayUserType() == STUDENT){
            List<Integer> noPaymentUserIds = musicGroupPaymentCalenderDetailDao.queryNoPaymentUserIds(musicGroupId, new ArrayList<>(userIdList), null);
            if (noPaymentUserIds.size() > 0) {
                throw new BizException("操作失败：有欠费的学员不允许创建缴费");
            }
        }
        //获取缴费状态在审核中或者已拒绝的缴费项目的学员
        String studentIds = musicGroupPaymentCalenderDao.queryCalenderStudentIds(musicGroupId, batchNo);
        if (StringUtils.isNotEmpty(studentIds)) {
            for (Integer integer : userIdList) {
                if (studentIds.contains(integer.toString())) {
                    throw new BizException("操作失败：所选学员有待审核或已拒绝的缴费项目");
                }
            }
        }
        Long musicGroupPaymentCalenderId = musicGroupPaymentCalender.getId();

        if (musicGroupPaymentCalender.getPaymentType() == MusicGroupPaymentCalender.PaymentType.MUSIC_APPLY) {
            throw new BizException("操作失败：报名缴费项不允许添加学员");
        }
        if (musicGroupPaymentCalender.getStatus() == PaymentCalenderStatusEnum.AUDITING
                || musicGroupPaymentCalender.getStatus() == PaymentCalenderStatusEnum.REJECT) {
            throw new BizException("当前缴费状态不能添加学生");
        }
        //判断学生是否已存在
        Set<Integer> userList = musicGroupPaymentCalenderDetailDao.queryStudentIds(musicGroupPaymentCalenderId);
        if (Collections.disjoint(userList, userIdList) == false) {
            throw new BizException("操作失败：包含已存在的学员");
        }

        Date date = new Date();
        MusicGroupPaymentCalenderDetail musicGroupPaymentCalenderDetail = null;
        Map<Integer, Long> userCalenderDetailMap = new HashMap<>();
        Map<Integer, Integer> activityuserMap = new HashMap<>();
        List<MusicGroupPaymentCalenderDetail> musicGroupPaymentCalenderDetailList = new ArrayList<>();

        List<MusicGroupPaymentCalenderActivity> calenderActivities = musicGroupPaymentCalenderActivityService.findByCalenderId(musicGroupPaymentCalenderId);
        Map<Integer, VipGroupActivity> collect = new HashMap<>();
        if(CollectionUtils.isNotEmpty(calenderActivities)){
            List<VipGroupActivity> vipGroupActivities = vipGroupActivityDao.queryByIds(calenderActivities.stream().
                    map(e -> e.getActivityId().toString()).distinct().collect(Collectors.joining(",")));
            collect = vipGroupActivities.stream().collect(Collectors.groupingBy(e -> e.getId(),
                    Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
        }
        MusicGroupPaymentCalenderMember calenderMember = musicGroupPaymentCalenderMemberService.findByCalenderId(musicGroupPaymentCalenderId);
        for (Integer studentId : userIdList) {
            musicGroupPaymentCalenderDetail = new MusicGroupPaymentCalenderDetail();
            musicGroupPaymentCalenderDetail.setMusicGroupPaymentCalenderId(musicGroupPaymentCalenderId);
            musicGroupPaymentCalenderDetail.setCreateTime(date);
            if (musicGroupPaymentCalender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) == 0) {
                musicGroupPaymentCalenderDetail.setPaymentStatus(PaymentStatus.PAID_COMPLETED);
            } else {
                musicGroupPaymentCalenderDetail.setPaymentStatus(PaymentStatus.NON_PAYMENT);
            }
            musicGroupPaymentCalenderDetail.setUpdateTime(date);
            musicGroupPaymentCalenderDetail.setUserId(studentId);
            musicGroupPaymentCalenderDetail.setStartPaymentDate(musicGroupPaymentCalender.getStartPaymentDate());
            musicGroupPaymentCalenderDetail.setDeadlinePaymentDate(musicGroupPaymentCalender.getDeadlinePaymentDate());
            musicGroupPaymentCalenderDetail.setResponsibleUserId(musicGroupPaymentCalender.getOperator());
            musicGroupPaymentCalenderDetail.setMusicGroupId(musicGroupPaymentCalender.getMusicGroupId());
            //设置云教练、课程收入
            this.setCourseIncome(calenderMember,musicGroupPaymentCalenderDetail);
            Integer organId = musicGroupPaymentCalender.getOrganId();
            musicGroupPaymentCalenderDetailList.add(musicGroupPaymentCalenderDetail);
            if (musicGroupPaymentCalender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) == 0 || musicGroupPaymentCalender.getPayUserType() == SCHOOL) {
                // 添加会员
                if (calenderMember != null) {
                    CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
                    cloudTeacherOrder.setOrganId(organId);
                    cloudTeacherOrder.setType(calenderMember.getPeriod());
                    cloudTeacherOrder.setAmount(BigDecimal.ZERO);
                    cloudTeacherOrder.setStudentId(studentId);
                    cloudTeacherOrder.setLevel(calenderMember.getMemberRankSettingId());
                    cloudTeacherOrder.setTime(calenderMember.getNum().intValue());
                    cloudTeacherOrder.setStatus(1);
                    if(musicGroupPaymentCalender.getPayUserType() == SCHOOL){
                        cloudTeacherOrder.setRemark("学校缴费新增学员");
                    }else {
                        cloudTeacherOrder.setRemark("缴费项目0元新增学员");
                    }
                    cloudTeacherOrder.setMusicGroupId(musicGroupId);
                    cloudTeacherOrderService.save(cloudTeacherOrder,calenderMember.getAutoActivationFlag());
                }
                //添加小课包
                if (calenderActivities != null && calenderActivities.size() > 0) {
                    List<ActivityUserMapper> activityUserMapperList = new ArrayList<>();
                    for (MusicGroupPaymentCalenderActivity calenderActivity : calenderActivities) {
                        ActivityUserMapper activityUserMapper = new ActivityUserMapper();
                        activityUserMapper.setActualPrice(BigDecimal.ZERO);
                        activityUserMapper.setSubNoCoursePrice(BigDecimal.ZERO);
                        activityUserMapper.setActivityId(calenderActivity.getActivityId());
                        activityUserMapper.setUserId(studentId);
                        activityUserMapper.setOrganId(organId);
                        if(musicGroupPaymentCalender.getPayUserType() == SCHOOL){
                            activityUserMapper.setAddMemo("学校缴费新增学员");
                        }else {
                            activityUserMapper.setAddMemo("缴费项目0元新增学员");
                        }
                        VipGroupActivity activity = collect.get(activityUserMapper.getActivityId());
                        activityUserMapperService.copyProperty(activityUserMapper,activity,false,true);
                        if ("网管课".equals(calenderActivity.getCategoryName())) {
                            activityUserMapper.setPracticeFlag(1);
                        } else {
                            activityUserMapper.setVipFlag(1);
                        }
                        activityUserMapperList.add(activityUserMapper);
                    }
                    activityUserMapperService.batchInsert(activityUserMapperList);
                    activityUserMapperList.stream().forEach(e -> activityuserMap.put(e.getUserId(), e.getId()));
                }
            }
        }

        if (musicGroupPaymentCalenderDetailList.size() > 0) {
            musicGroupPaymentCalenderDetailDao.batchInsert(musicGroupPaymentCalenderDetailList);
            musicGroupPaymentCalenderDetailList.stream().forEach(e -> userCalenderDetailMap.put(e.getUserId(), e.getId()));
        }
        //更新预计缴费人数
        musicGroupPaymentCalender.setExpectNum(musicGroupPaymentCalender.getExpectNum() + userIdList.size());
        musicGroupPaymentCalender.setUpdateTime(date);
        if (musicGroupPaymentCalender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) == 0) {
            musicGroupPaymentCalender.setActualNum(musicGroupPaymentCalender.getActualNum() + userIdList.size());
        }
        musicGroupPaymentCalenderService.update(musicGroupPaymentCalender);
        //创建学生课排课分钟数
        musicGroupPaymentStudentCourseDetailService.batchInsert(userIdList,musicGroupPaymentCalender,userCalenderDetailMap);

        //给学员推送缴费通知
        if (musicGroupPaymentCalender.getPayUserType() == STUDENT && musicGroupPaymentCalender.getStatus() == PaymentCalenderStatusEnum.OPEN) {
            musicGroupPaymentCalenderService.pushWaitRenewMessage(musicGroupPaymentCalender.getId(),
                    musicGroupDao.get(musicGroupPaymentCalender.getMusicGroupId()), userIdList);
        }
    }

    public void setCourseIncome(MusicGroupPaymentCalenderMember calenderMember, MusicGroupPaymentCalenderDetail calenderDetail){
        if(Objects.nonNull(calenderMember)){
            MusicGroup musicGroup = musicGroupDao.get(calenderDetail.getMusicGroupId());
            if(musicGroup.getCourseViewType() == CourseViewTypeEnum.MEMBER_FEE){
                //获取云教练溢出费用
                BigDecimal courseIncome = memberFeeSettingService.getAmount(musicGroup.getOrganId(), 1, calenderMember.getPeriod(),
                        calenderMember.getNum(), MemberFeeSettingServiceImpl::getIncome);
                if(calenderMember.getActualAmount().compareTo(courseIncome) > 0){
                    calenderDetail.setExpectMemberAmount(courseIncome);
                    calenderDetail.setActualAmount(calenderMember.getActualAmount().subtract(courseIncome));
                }
            }else {
                calenderDetail.setExpectMemberAmount(calenderMember.getActualAmount());
                calenderDetail.setActualAmount(BigDecimal.ZERO);
            }
        }
    }

    @Override
    public List<FeeStudentDto> queryFeeStudents(String musicGroupId, String search, Integer subjectId) {
        return musicGroupStudentFeeDao.queryFeeStudents(musicGroupId, search, subjectId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchDel(String musicGroupPaymentCalenderDetailIds) {
        List<MusicGroupPaymentCalenderDetail> details = musicGroupPaymentCalenderDetailDao.queryListByIds(musicGroupPaymentCalenderDetailIds);
        if (details == null || details.size() == 0) {
            throw new BizException("删除失败: 数据不可为空");
        }
        Long musicGroupPaymentCalenderId = details.get(0).getMusicGroupPaymentCalenderId();
        MusicGroupPaymentCalender musicGroupPaymentCalender = musicGroupPaymentCalenderDao.get(musicGroupPaymentCalenderId);
        if (musicGroupPaymentCalender == null) {
            throw new BizException("查询缴费项目失败[{}]", musicGroupPaymentCalender);
        }
        long count = 0;
        if (musicGroupPaymentCalender.getPayUserType() == STUDENT) {
            // 是否已缴费
            count = details.stream().filter(e -> e.getPaymentStatus() != NON_PAYMENT).count();
            if (count > 0) {
                throw new BizException("删除失败: 存在已缴费的学员");
            }
        }
        List<Long> musicGroupPaymentCalenderDetailIdList = new ArrayList<Long>();
        for (String s : musicGroupPaymentCalenderDetailIds.split(",")) {
            musicGroupPaymentCalenderDetailIdList.add(Long.parseLong(s));
        }
        //是否已排课
        List<MusicGroupPaymentStudentCourseDetail> musicGroupPaymentStudentCourseDetailList =
                musicGroupPaymentStudentCourseDetailService.queryByMusicGroupPaymentStudentCourseDetailId(musicGroupPaymentCalenderDetailIdList);
        count = musicGroupPaymentStudentCourseDetailList.stream().filter(e -> e.getUsedCourseMinutes() > 0).count();
        if (count > 0) {
            throw new BizException("删除失败：存在已排课的学员");
        }

        int currentNum = musicGroupPaymentCalender.getExpectNum() - details.size();
        musicGroupPaymentCalender.setExpectNum(currentNum < 0 ? 0 : currentNum);
        musicGroupPaymentCalenderDao.update(musicGroupPaymentCalender);

        musicGroupPaymentCalenderDetailDao.batchDel(musicGroupPaymentCalenderDetailIds);
        musicGroupPaymentStudentCourseDetailService.deleteByMusicGroupPaymentCalenderDetailId(musicGroupPaymentCalenderDetailIdList);
    }

    @Override
    public PageInfo<MusicArrearageStudentDto> queryArrearageStudents(ArrearageStudentsQueryInfo queryInfo) {
        PageInfo<MusicArrearageStudentDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<MusicArrearageStudentDto> dataList = null;
        int count = musicGroupPaymentCalenderDetailDao.countArrearageStudents(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = musicGroupPaymentCalenderDetailDao.queryArrearageStudents(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<MusicGroup> getNoPaymentStudentMusicGroups(String organIdsStr) {
        Set<Integer> organIds = null;
        if (StringUtils.isNotBlank(organIdsStr)) {
            organIds = Arrays.stream(organIdsStr.split(",")).map(Integer::new).collect(Collectors.toSet());
        }
        return musicGroupPaymentCalenderDetailDao.getNoPaymentStudentMusicGroups(organIds, TenantContextHolder.getTenantId());
    }

    @Override
    public void pushWaitRenewMessage(Long calenderId, String studentIds) {
        List<MusicGroupPaymentCalenderDetail> calenderDetails = musicGroupPaymentCalenderDetailDao.queryNoPaymentCanPushByCalenderId(calenderId, studentIds);
        if (calenderDetails == null || calenderDetails.size() == 0) {
            throw new BizException("发送失败：没有可以发送缴费提醒的学员");
        }
        Set<Integer> studentIdList = calenderDetails.stream().map(e -> e.getUserId()).collect(Collectors.toSet());
        MusicGroupPaymentCalender paymentCalender = musicGroupPaymentCalenderDao.get(calenderId);
        //未缴费
        Map<Integer, String> studentMaps = MapUtil.convertMybatisMap(musicGroupPaymentCalenderDetailDao.queryNoPaymentAndNotZeroStudent(calenderId, studentIdList));
        MusicGroup musicGroup = musicGroupDao.get(paymentCalender.getMusicGroupId());
        String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL_MDAYA);
        String memo = "4?" + baseUrl + "/#/musicGroupRenew?calenderId=" + calenderId + "&id=" + musicGroup.getId();
        Map<Integer, String> push = new HashMap<>();
        for (Integer userId : studentIdList) {
            push.put(userId, userId.toString());
        }
        // 发送续费通知
        sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
                null, 0, memo, "STUDENT", musicGroup.getName());
        sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.STUDENT_SMS_WAIT_RENEW_MESSAGE, studentMaps,
                null, 0, memo, null, musicGroup.getName());
    }

    @Override
    public PageInfo<MusicGroupPaymentCalenderDetailDto> queryDetailPage(MusicCalenderDetailQueryInfo queryInfo) {
        PageInfo<MusicGroupPaymentCalenderDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<MusicGroupPaymentCalenderDetailDto> dataList = null;
        int count = musicGroupPaymentCalenderDetailDao.countDetailPage(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = musicGroupPaymentCalenderDetailDao.queryDetailPage(params);
            List<StudentPaymentOrderDetail> orderDetail = studentPaymentOrderDetailDao.findByCalenderId(dataList.get(0).getMusicGroupPaymentCalenderId());
            Map<Integer, List<StudentPaymentOrderDetail>> orderDetailMap = orderDetail.stream().collect(Collectors.groupingBy(StudentPaymentOrderDetail::getUserId));
            for (MusicGroupPaymentCalenderDetailDto row : dataList) {
                if(row.getDeadlinePaymentDateMgpc() != null){
                    row.setDeadlinePaymentDateStr(DateUtils.formatDate(row.getDeadlinePaymentDateMgpc(), DateUtils.YYYY_MM_DD_DEF) + " 23:59:59");
                }
                List<StudentPaymentOrderDetail> orderDetailList = orderDetailMap.get(row.getUserId());
                if (CollectionUtils.isNotEmpty(orderDetailList)) {
                    BigDecimal totalAmount = orderDetailList.stream().map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                    //辅件
                    BigDecimal accessoriesAmount = orderDetailList.stream().filter(e -> e.getType() == OrderDetailTypeEnum.ACCESSORIES).map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                    row.setAccessoriesAmount(accessoriesAmount);
                    //会员金额
                    BigDecimal cloudAmount = orderDetailList.stream().filter(e -> e.getType() == OrderDetailTypeEnum.CLOUD_TEACHER || e.getType() == OrderDetailTypeEnum.CLOUD_TEACHER_PLUS).map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                    row.setCloudAmount(cloudAmount);
                    //乐器
                    BigDecimal musicalAmount = orderDetailList.stream().filter(e -> e.getType() == OrderDetailTypeEnum.MUSICAL).map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                    row.setMusicalAmount(musicalAmount);
                    //乐保
                    BigDecimal maintenanceAmount = orderDetailList.stream().filter(e -> e.getType() == OrderDetailTypeEnum.MAINTENANCE).map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                    row.setMaintenanceAmount(maintenanceAmount);
                    //活动
                    BigDecimal activityAmount = orderDetailList.stream().filter(e -> e.getType() == OrderDetailTypeEnum.PRACTICE || e.getType() == OrderDetailTypeEnum.VIP).map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
                    row.setActivityAmount(activityAmount);

                    totalAmount = totalAmount.subtract(accessoriesAmount).subtract(cloudAmount).subtract(musicalAmount).subtract(maintenanceAmount).subtract(activityAmount);
                    row.setCourseAmount(totalAmount);
                }
            }
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public MusicGroupPaymentCalenderDetail addCalenderDetail(StudentPaymentOrder studentPaymentOrder) {
        //缴费详情 calender detail
        Long calenderId = studentPaymentOrder.getCalenderId();
        MusicGroupPaymentCalender musicGroupPaymentCalender = musicGroupPaymentCalenderService.get(calenderId);
        //查询缴费项目关联的订单
        List<StudentPaymentOrder> orderList = studentPaymentOrderDao.findByCalenderId(calenderId, studentPaymentOrder.getUserId());
        if (orderList.size() == 1) {
            // 更新缴费人数
            Integer expectNum = 0;
            if (musicGroupPaymentCalender.getPaymentType() == MusicGroupPaymentCalender.PaymentType.MUSIC_APPLY) {
                expectNum = 1;
            }
            musicGroupPaymentCalenderDao.updatePaymentNum(studentPaymentOrder.getCalenderId(),1,expectNum);
        }
        Date date = new Date();
        MusicGroupPaymentCalenderDetail musicGroupPaymentCalenderDetail = musicGroupPaymentCalenderDetailDao.findByCalenderIdAndUserId(calenderId, studentPaymentOrder.getUserId());
        //只有报名订单，缴费详情才会是null
        if (musicGroupPaymentCalenderDetail == null) {
            musicGroupPaymentCalenderDetail = new MusicGroupPaymentCalenderDetail();
            musicGroupPaymentCalenderDetail.setTenantId(studentPaymentOrder.getTenantId());
            musicGroupPaymentCalenderDetail.setMusicGroupPaymentCalenderId(musicGroupPaymentCalender.getId());
            musicGroupPaymentCalenderDetail.setUserId(studentPaymentOrder.getUserId());
            musicGroupPaymentCalenderDetail.setResponsibleUserId(musicGroupPaymentCalender.getOperator());
            musicGroupPaymentCalenderDetail.setPaymentStatus(PAID_COMPLETED);
            musicGroupPaymentCalenderDetail.setPayTime(date);
            musicGroupPaymentCalenderDetail.setPaymentOrderId(studentPaymentOrder.getId());
            musicGroupPaymentCalenderDetail.setUseInCourse(0);
            musicGroupPaymentCalenderDetail.setOpen(1);
            musicGroupPaymentCalenderDetail.setUserStatus(null);
            musicGroupPaymentCalenderDetail.setDeadlinePaymentDate(musicGroupPaymentCalender.getDeadlinePaymentDate());
            musicGroupPaymentCalenderDetail.setStartPaymentDate(musicGroupPaymentCalender.getStartPaymentDate());
            musicGroupPaymentCalenderDetailDao.insert(musicGroupPaymentCalenderDetail);
        } else {
            musicGroupPaymentCalenderDetail.setPaymentStatus(PAID_COMPLETED);
            musicGroupPaymentCalenderDetail.setPayTime(date);
            musicGroupPaymentCalenderDetail.setPaymentOrderId(studentPaymentOrder.getId());
            musicGroupPaymentCalenderDetailDao.update(musicGroupPaymentCalenderDetail);
        }
        //保存、更新云教练缴费项目预收
        operatingReportCloudService.updateSet(musicGroupPaymentCalender,musicGroupPaymentCalenderDetail.getExpectMemberAmount());
        //续费项目加学员时就生成了课程时长，报名项目在缴费成功后才需要添加课程时长
        musicGroupPaymentStudentCourseDetailService.batchInsert(studentPaymentOrder.getUserId(),
                musicGroupPaymentCalender,
                musicGroupPaymentCalenderDetail.getId(),
                studentPaymentOrder.getId());
        return musicGroupPaymentCalenderDetail;
    }
}
