package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.dao.GoodsDao;
import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderGoodsDao;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentBaseCalender;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.service.GoodsService;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderBaseService;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderGoodsService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service("musicGroupPaymentCalenderGoodsService")
public class MusicGroupPaymentCalenderGoodsServiceImpl extends ServiceImpl<MusicGroupPaymentCalenderGoodsDao, MusicGroupPaymentCalenderGoods>
        implements MusicGroupPaymentCalenderGoodsService, MusicGroupPaymentCalenderBaseService<MusicGroupPaymentCalenderGoods> {

    private final static Logger log = LoggerFactory.getLogger(MusicGroupPaymentCalenderGoodsServiceImpl.class);

    @Override
    public MusicGroupPaymentCalenderGoodsDao getDao() {
        return this.baseMapper;
    }

    @Autowired
    private GoodsDao goodsDao;

    @Override
    public MusicGroupPaymentCalender.PaymentCalenderStatusEnum checkComponentAmount(MusicGroupPaymentBaseCalender baseCalender) {
        return null;
    }

    @Override
    public BigDecimal getActualAmount(MusicGroupPaymentBaseCalender baseCalender) {
        List<MusicGroupPaymentCalenderGoods> goodsList = baseCalender.getCalenderGoodsList();
        if(CollectionUtils.isNotEmpty(goodsList)){
            //计算现价
            return JSONObject.parseObject(baseCalender.getCalenderFeeJson()).getBigDecimal("allTotal");
        }
        return BigDecimal.ZERO;
    }

    @Override
    public BigDecimal getOriginalAmount(MusicGroupPaymentBaseCalender musicGroupPaymentBaseCalender) {
        List<MusicGroupPaymentCalenderGoods> goodsList = musicGroupPaymentBaseCalender.getCalenderGoodsList();
        if(CollectionUtils.isNotEmpty(goodsList)){
            List<Integer> goodsIds = goodsList.stream().map(e -> e.getGoodsId()).collect(Collectors.toList());
            List<Goods> goodies = goodsDao.getGoodies(goodsIds);
            Map<Integer,Goods> goodsMap = goodies.stream().collect(Collectors.toMap(Goods::getId, e -> e));
            //计算原价
            return goodsList.stream().map(e -> {
                Goods goods = goodsMap.get(e.getGoodsId());
                return goods.getDiscountPrice().multiply(new BigDecimal(e.getNum()));
            }).reduce(BigDecimal.ZERO,BigDecimal::add);
        }
        return BigDecimal.ZERO;
    }

    @Override
    public BigDecimal getCurrentAmount(MusicGroupPaymentBaseCalender baseCalender) {
        return getOriginalAmount(baseCalender);
    }

    @Override
    public <E> E initBean(E bean) {
        return null;
    }

}

