package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.MusicGroupPaymentCalenderRepairDao;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentBaseCalender;
import com.ym.mec.biz.dal.dto.WaitSendMessageDto;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalenderRepair;
import com.ym.mec.biz.dal.enums.CalenderBaseServiceEnum;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum.AUDITING;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum.NO;

@Service
public class MusicGroupPaymentCalenderRepairServiceImpl extends BaseServiceImpl<Long, MusicGroupPaymentCalenderRepair>
        implements MusicGroupPaymentCalenderBaseService<MusicGroupPaymentCalenderRepair>, MusicGroupPaymentCalenderRepairService {

    @Autowired
    private MusicGroupPaymentCalenderRepairDao musicGroupPaymentCalenderRepairDao;

    @Override
    public BaseDAO<Long, MusicGroupPaymentCalenderRepair> getDAO() {
        return musicGroupPaymentCalenderRepairDao;
    }

    @Autowired
    private SysTenantConfigService sysTenantConfigService;
    @Autowired
    private SysMessageService sysMessageService;

    public MusicGroupPaymentCalenderRepairServiceImpl() {
        MusicGroupPaymentCalenderBaseService.calenderBaseServiceMap.put(CalenderBaseServiceEnum.MUSIC_REPAIR,this);
    }

    @Override
    public MusicGroupPaymentCalender.PaymentCalenderStatusEnum checkComponentAmount(MusicGroupPaymentBaseCalender baseCalender) {
        MusicGroupPaymentCalenderRepair musicRepair = baseCalender.getMusicRepair();
        if(musicRepair != null){
            Integer tenantId = TenantContextHolder.getTenantId();
            String configValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.ONE_YEAR_MUSICAL_REPAIR_AMOUNT, tenantId);
            if(StringUtils.isEmpty(configValue)){
                throw new BizException("操作失败：请配置乐保零售价");
            }
            //获取会员总价
            BigDecimal repairPaymentAmount = new BigDecimal(configValue).multiply(musicRepair.getNum()).setScale(0, BigDecimal.ROUND_HALF_UP);
            if(repairPaymentAmount.compareTo(musicRepair.getActualAmount()) != 0){
                return AUDITING;
            }
        }
        return NO;
    }

    @Override
    public BigDecimal getActualAmount(MusicGroupPaymentBaseCalender baseCalender) {
        MusicGroupPaymentCalenderRepair musicRepair = baseCalender.getMusicRepair();
        if(musicRepair != null){
            return musicRepair.getActualAmount();
        }
        return BigDecimal.ZERO;
    }

    @Override
    public BigDecimal getOriginalAmount(MusicGroupPaymentBaseCalender baseCalender) {
        return getAmount(baseCalender,SysConfigService.ONE_YEAR_MUSICAL_REPAIR_ORIGINAL_AMOUNT);
    }

    @Override
    public BigDecimal getCurrentAmount(MusicGroupPaymentBaseCalender baseCalender) {
        return getAmount(baseCalender,SysConfigService.ONE_YEAR_MUSICAL_REPAIR_AMOUNT);
    }

    @Override
    public <E> E initBean(E bean) {
        return bean;
    }

    @Override
    public void deleteByCalenderId(Long calenderId) {
        musicGroupPaymentCalenderRepairDao.deleteByCalenderId(calenderId);
    }

    @Override
    public MusicGroupPaymentCalenderRepair findByCalenderId(Long calenderId) {
        return musicGroupPaymentCalenderRepairDao.findByCalenderId(calenderId);
    }

    @Override
    public void pushWaitSendMessageTask() {
        //获取截止回款日期还未回款的数据
        List<WaitSendMessageDto> messageDtos = musicGroupPaymentCalenderRepairDao.queryWaitSendMessage();
        if (CollectionUtils.isNotEmpty(messageDtos)) {
            for (WaitSendMessageDto messageDto : messageDtos) {
                //发送消息
                Map<Integer, String> receivers = new HashMap<>(1);
                receivers.put(messageDto.getUserId(), messageDto.getPhone());
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS,
                        MessageTypeEnum.SMS_WAIT_REMINDER_PAYMENT, receivers, null, 0, null, null,messageDto.getSchoolName(),messageDto.getBatchNo());
            }
//            List<Integer> ids = messageDtos.stream().map(e -> e.getId()).collect(Collectors.toList());
//            musicGroupPaymentCalenderRepairDao.updateSendRemind(ids);
        }
    }

    private BigDecimal getAmount(MusicGroupPaymentBaseCalender baseCalender,String configName){
        MusicGroupPaymentCalenderRepair musicRepair = baseCalender.getMusicRepair();
        if(musicRepair != null){
            Integer tenantId = TenantContextHolder.getTenantId();
            String configValue = sysTenantConfigService.getTenantConfigValue(configName, tenantId);
            if(StringUtils.isEmpty(configValue)){
                throw new BizException("操作失败：请配置乐保单价");
            }
            //获取会员总价
            return new BigDecimal(configValue).multiply(musicRepair.getNum()).setScale(0, BigDecimal.ROUND_HALF_UP);
        }
        return BigDecimal.ZERO;
    }
}
