package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.dto.MusicGroupPaymentCalenderDto.MusicGroupPaymentDateRange;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PayUserType;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentType;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.BaseOrganQueryInfo;
import com.ym.mec.biz.dal.page.MusicGroupPaymentCalenderQueryInfo;
import com.ym.mec.biz.dal.wrapper.MusicGroupCalenderRefundPeriodWrapper;
import com.ym.mec.biz.dal.wrapper.MusicGroupPaymentCalenderAddressWrapper;
import com.ym.mec.biz.event.source.GroupEventSource;
import com.ym.mec.biz.event.source.SendSeoMessageSource;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.api.CommonResult;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.dto.OrderCreate;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.WrapperUtil;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.mall.MallFeignService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.ModelMap;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PayUserType.SCHOOL;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PayUserType.STUDENT;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentCalenderStatusEnum.*;
import static com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender.PaymentType.*;
import static com.ym.mec.biz.dal.enums.PaymentStatusEnum.YES;
import static com.ym.mec.biz.dal.enums.SysUserRoleEnum.ORGAN_MANAGER;

@Service
public class MusicGroupPaymentCalenderServiceImpl extends BaseServiceImpl<Long, MusicGroupPaymentCalender> implements MusicGroupPaymentCalenderService {

    @Autowired
    private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
    @Autowired
    private MusicGroupPaymentCalenderDetailDao musicGroupPaymentCalenderDetailDao;
    @Autowired
    private MusicGroupPaymentCalenderCourseSettingsServiceImpl musicGroupPaymentCalenderCourseSettingsService;
    @Autowired
    private MusicGroupPaymentStudentCourseDetailService musicGroupPaymentStudentCourseDetailService;
    @Autowired
    private MusicGroupSchoolTermCourseDetailDao musicGroupSchoolTermCourseDetailDao;
    @Autowired
    private MusicGroupSchoolTermStudentCourseDetailDao musicGroupSchoolTermStudentCourseDetailDao;
    @Autowired
    private IdGeneratorService idGeneratorService;
    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private MusicGroupBuildLogDao musicGroupBuildLogDao;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private OrganizationDao organizationDao;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private SendSeoMessageSource sendSeoMessageSource;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private ImUserFriendService imUserFriendService;
    @Autowired
    private MusicGroupStudentClassAdjustDao musicGroupStudentClassAdjustDao;
    @Autowired
    private MusicGroupPaymentCalenderDetailService musicGroupPaymentCalenderDetailService;
    @Autowired
    private MusicGroupPaymentCalenderStudentDetailDao musicGroupPaymentCalenderStudentDetailDao;
    @Autowired
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private GroupEventSource groupEventSource;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private MemberRankPrivilegesService memberRankPrivilegesService;
    @Autowired
    private CloudTeacherOrderService cloudTeacherOrderService;
    @Autowired
    private MusicGroupPaymentCalenderAddressService musicGroupPaymentCalenderAddressService;
    @Autowired
    private MusicGroupPaymentCalenderGoodsServiceImpl musicGroupPaymentCalenderGoodsService;
    @Autowired
    private MusicGroupPaymentCalenderActivityServiceImpl musicGroupPaymentCalenderActivityService;
    @Autowired
    private MusicGroupPaymentCalenderMemberServiceImpl musicGroupPaymentCalenderMemberService;
    @Autowired
    private MusicGroupPaymentCalenderRepairServiceImpl musicGroupPaymentCalenderRepairService;
    @Autowired
    private StudentDao studentDao;
    @Autowired
    private ActivityUserMapperService activityUserMapperService;
    @Autowired
    private StudentRegistrationService studentRegistrationService;
    @Autowired
    private VipGroupActivityDao vipGroupActivityDao;
    @Autowired
    private OperatingReportCloudService operatingReportCloudService;
    @Autowired
    private MusicGroupCalenderRefundPeriodService musicGroupCalenderRefundPeriodService;
    @Autowired
    private ImGroupService imGroupService;
    @Autowired
    private ImGroupMemberService imGroupMemberService;
    @Autowired
    private MemberFeeSettingService memberFeeSettingService;
    @Autowired
    private CloudCoachPaymentProgramDao cloudCoachPaymentProgramDao;
    @Autowired
    private MallFeignService mallFeignService;
    @Autowired
    private StudentPaymentRouteOrderService studentPaymentRouteOrderService;
    @Autowired
    private GoodsSubService goodsSubService;
    @Autowired
    private GoodsDao goodsDao;
    @Autowired
    private SellOrderService sellOrderService;
    @Autowired
    private SysAreaService sysAreaService;

    @Override
    public BaseDAO<Long, MusicGroupPaymentCalender> getDAO() {
        return musicGroupPaymentCalenderDao;
    }

    public MusicGroupPaymentCalenderDao getDao() {
        return musicGroupPaymentCalenderDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String create(MusicGroupPaymentCalenderDto musicGroupPaymentCalenderDto) {
        PaymentType paymentType = musicGroupPaymentCalenderDto.getPaymentType();
        if (paymentType != SPAN_GROUP_CLASS_ADJUST) {
            throw new BizException("操作失败：该入口不支持{}", paymentType.getDesc());
        }
        List<MusicGroupPaymentDateRange> musicGroupPaymentDateRangeList = musicGroupPaymentCalenderDto.getMusicGroupPaymentDateRangeList();
        if (musicGroupPaymentDateRangeList.size() > 1) {
            throw new BizException("创建缴费失败：不支持多周期缴费");
        }
        String musicGroupId = musicGroupPaymentCalenderDto.getMusicGroupId();
        MusicGroup musicGroup = musicGroupDao.getLocked(musicGroupId);
        if (musicGroup == null) {
            throw new BizException("乐团查询失败，请检查参数");
        }
        SysUser sysUser = sysUserService.getUser();
        // 所有缴费项目已完成排课才能创建下一个缴费项目
        /*String organBatchNo = musicGroupPaymentStudentCourseDetailDao.getUnUseBatchNoWithStudentAndCourseTypeAndCourseMinutes(musicGroupId, null, null, musicGroupPaymentCalenderDto.getIgnoreBatchNoList());
        if (StringUtils.isNoneBlank(organBatchNo)) {
            if (StringUtils.isEmpty(musicGroupPaymentCalenderDto.getBatchNo()) || !musicGroupPaymentCalenderDto.getBatchNo().equals(organBatchNo)) {
                throw new BizException("当前乐团存在未排课的缴费项目，请先完成排课再操作");
            }
        }*/

        List<MusicGroupPaymentCalenderStudentDetail> calenderStudentDetails = musicGroupPaymentCalenderDto.getMusicGroupPaymentCalenderStudentDetails();

        PaymentCalenderStatusEnum status = musicGroupPaymentCalenderDto.getStatus();

        String batchNo = idGeneratorService.generatorId() + "";

        MusicGroupPaymentDateRange musicGroupPaymentDateRange = musicGroupPaymentDateRangeList.get(0);
        MusicGroupPaymentCalender musicGroupPaymentCalender = new MusicGroupPaymentCalender();
        musicGroupPaymentCalender.setAttribute1(musicGroupPaymentCalenderDto.getAttribute1());
        musicGroupPaymentCalender.setAttribute2(musicGroupPaymentCalenderDto.getAttribute2());
        musicGroupPaymentCalender.setDeadlinePaymentDate(musicGroupPaymentDateRange.getDeadlinePaymentDate());
        musicGroupPaymentCalender.setMemo(musicGroupPaymentCalenderDto.getMemo());
        musicGroupPaymentCalender.setMusicGroupId(musicGroupId);
        musicGroupPaymentCalender.setMusicGroupOrganizationCourseSettingId(musicGroupPaymentCalenderDto.getMusicGroupOrganizationCourseSettingId());

        musicGroupPaymentCalender.setOriginalTotalAmount(calenderStudentDetails.stream().map(e -> e.getCourseOriginalPrice()).reduce(BigDecimal.ZERO, BigDecimal::add));
        musicGroupPaymentCalender.setCurrentTotalAmount(calenderStudentDetails.stream().map(e -> e.getCourseCurrentPrice()).reduce(BigDecimal.ZERO, BigDecimal::add));
        musicGroupPaymentCalender.setPaymentType(paymentType);
        musicGroupPaymentCalender.setPayUserType(musicGroupPaymentCalenderDto.getPayUserType());
        musicGroupPaymentCalender.setStartPaymentDate(musicGroupPaymentDateRange.getStartPaymentDate());
        musicGroupPaymentCalender.setStudentIds(musicGroupPaymentCalenderDto.getStudentIds());
        musicGroupPaymentCalender.setOperator(sysUser.getId());
        musicGroupPaymentCalender.setStatus(status);
        musicGroupPaymentCalender.setOrganId(musicGroup.getOrganId());
        if (musicGroupPaymentCalender.getStatus() != AUDITING) {
            Date date = new Date();
            Date day = DateUtil.stringToDate(DateUtil.format(date, DateUtil.DEFAULT_PATTERN), DateUtil.DEFAULT_PATTERN);
            if (date.after(musicGroupPaymentCalender.getStartPaymentDate())) {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OPEN);
            } else if (day.after(musicGroupPaymentCalender.getDeadlinePaymentDate())) {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OVER);
            } else {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.NO);
            }
        }
        // 设置批次号
        musicGroupPaymentCalender.setBatchNo(batchNo);
        musicGroupPaymentCalenderDto.setBatchNo(batchNo);
        musicGroupPaymentCalenderDao.insert(musicGroupPaymentCalender);
        //如果是跨团班级合并,保存用户缴费详情
        //记录学员缴费详情
        musicGroupPaymentCalenderStudentDetailDao.batchInsert(musicGroupPaymentCalenderDto.getMusicGroupPaymentCalenderStudentDetails(), musicGroupPaymentCalender.getBatchNo());
        if (musicGroupPaymentCalender.getStatus() != AUDITING) {
            //如果是跨团班级合并,添加学员
            musicGroupPaymentCalenderDetailService.batchAdd(musicGroupPaymentCalender, musicGroupPaymentCalenderDto.getMusicGroupPaymentCalenderStudentDetails());
            //将0元未缴费学员缴费状态更新为已缴费
            this.updateNoPaymentAndZeroPaymentStatus(musicGroupPaymentCalender, true);
        }
        if (status == AUDITING) {
            // 如果是审核中
            Organization organization = organizationDao.get(musicGroup.getOrganId());
            sendSeoMessageSource.sendSeoMessage(musicGroup.getOrganId(), new SysUserRoleEnum[]{ORGAN_MANAGER},
                    null, null, MessageTypeEnum.BACKSTAGE_PAYMENT_CALENDER_AUDIT, organization.getName(), musicGroup.getName());
        } else {
            imUserFriendService.refreshGroupImUserFriend(musicGroupId, GroupType.MUSIC);
        }
        return batchNo;
    }

    public <T> void checkObjectNull(T t, Consumer consumer) {
        if (t == null) {
            consumer.accept(null);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<ModelMap> create1(MusicGroupPaymentBaseCalender musicGroupPaymentBaseCalender) {
        checkObjectNull(musicGroupPaymentBaseCalender, o -> {
            throw new BizException("参数校验异常");
        });
        String musicGroupId = musicGroupPaymentBaseCalender.getMusicGroupId();
        MusicGroup musicGroup = musicGroupDao.getLocked(musicGroupId);
        checkObjectNull(musicGroup, o -> {
            throw new BizException("乐团查询失败，请检查参数");
        });

        SysUser sysUser = sysUserService.getUser();

        //进行中加学员拓展信息
        CalenderAddStudent calenderAddStudent = musicGroupPaymentBaseCalender.getCalenderAddStudent();
        MusicGroupPaymentCalender musicGroupPaymentCalender = new MusicGroupPaymentCalender();
        BeanUtils.copyProperties(musicGroupPaymentBaseCalender,musicGroupPaymentCalender);
        if(musicGroupPaymentBaseCalender.getCalenderGoodsList() != null){
            musicGroupPaymentCalender.setPaymentType(GOODS_PURCHASE);
            musicGroupPaymentCalender.setPayUserType(SCHOOL);
        }else {
            //判断缴费项目类型
            if (musicGroup.getStatus() == MusicGroupStatusEnum.DRAFT || musicGroup.getStatus() == MusicGroupStatusEnum.AUDIT_FAILED || musicGroup.getStatus() == MusicGroupStatusEnum.FEE_AUDIT_FAILED) {
                musicGroupPaymentCalender.setPaymentType(MUSIC_APPLY);
            } else {
                if (calenderAddStudent != null && calenderAddStudent.getStudentIds() != null) {
                    musicGroupPaymentCalender.setPaymentType(ADD_STUDENT);
                    //预计缴费人数
                    musicGroupPaymentCalender.setAttribute1(calenderAddStudent.getClassGroupIds());
                    musicGroupPaymentCalender.setStudentIds(calenderAddStudent.getStudentIds().toString());
                    musicGroupPaymentCalender.setExpectNum(1);
                    //校验是否可以加学员
                    checkAddStudent(calenderAddStudent, musicGroup);
                } else {
                    if (musicGroupPaymentBaseCalender.getMusicGroupOrganizationCourseSettingId() != null) {
                        musicGroupPaymentCalender.setPaymentType(MUSIC_RENEW);
                    } else {
                        musicGroupPaymentCalender.setPaymentType(ADD_COURSE);
                    }
                }
            }
            // 不是进行中，只能创建一次缴费
            if (musicGroup.getStatus() != MusicGroupStatusEnum.PROGRESS && musicGroupPaymentCalender.getPaymentType() != GOODS_PURCHASE) {
                List<MusicGroupPaymentCalender> list = musicGroupPaymentCalenderDao.findByMusicGroupId(musicGroupId);
                //排除商品采购
                List<MusicGroupPaymentCalender> collect = list.stream().filter(e -> e.getPaymentType() != GOODS_PURCHASE).collect(Collectors.toList());
                if (collect.size() > 0) {
                    throw new BizException("创建失败，已经存在缴费信息");
                }
            }
        }
        musicGroupPaymentBaseCalender.setMusicGroup(musicGroup);

        //计算缴费项目原价
        BigDecimal courseOriginalAmount = musicGroupPaymentCalenderCourseSettingsService.getOriginalAmount(musicGroupPaymentBaseCalender);
        BigDecimal memberOriginalAmount = musicGroupPaymentCalenderMemberService.getOriginalAmount(musicGroupPaymentBaseCalender);
        BigDecimal repairOriginalAmount = musicGroupPaymentCalenderRepairService.getOriginalAmount(musicGroupPaymentBaseCalender);
        BigDecimal activityOriginalAmount = musicGroupPaymentCalenderActivityService.getOriginalAmount(musicGroupPaymentBaseCalender);
        BigDecimal goodsOriginalAmount = musicGroupPaymentCalenderGoodsService.getOriginalAmount(musicGroupPaymentBaseCalender);

        BigDecimal originalTotalAmount = courseOriginalAmount.add(memberOriginalAmount)
                .add(activityOriginalAmount)
                .add(goodsOriginalAmount)
                .add(repairOriginalAmount).setScale(0, BigDecimal.ROUND_HALF_UP);

        //计算缴费项目总金额(前端录入)
        BigDecimal courseActualAmount = musicGroupPaymentCalenderCourseSettingsService.getActualAmount(musicGroupPaymentBaseCalender);
        BigDecimal memberActualAmount = musicGroupPaymentCalenderMemberService.getActualAmount(musicGroupPaymentBaseCalender);
        BigDecimal repairActualAmount = musicGroupPaymentCalenderRepairService.getActualAmount(musicGroupPaymentBaseCalender);
        BigDecimal activityActualAmount = musicGroupPaymentCalenderActivityService.getActualAmount(musicGroupPaymentBaseCalender);
        BigDecimal goodsActualAmount = musicGroupPaymentCalenderGoodsService.getActualAmount(musicGroupPaymentBaseCalender);

        BigDecimal actualTotalAmount = courseActualAmount.add(memberActualAmount)
                .add(repairActualAmount)
                .add(goodsActualAmount)
                .add(activityActualAmount).setScale(0, BigDecimal.ROUND_HALF_UP);
        //标记是否云教练缴费
        if(musicGroupPaymentCalender.getPayUserType() == STUDENT && musicGroup.getCourseViewType() == CourseViewTypeEnum.MEMBER_FEE){
            if(musicGroupPaymentCalender.getPaymentType() == ADD_STUDENT){
                musicGroupPaymentCalender.setCloudTeacherPaymentFlag(true);
            }else if(memberActualAmount.compareTo(BigDecimal.ZERO) > 0 &&
                    ((courseActualAmount.compareTo(BigDecimal.ZERO) == 0 && CollectionUtils.isNotEmpty(musicGroupPaymentBaseCalender.getMusicGroupPaymentCalenderCourseSettingsList()))
                            || (activityActualAmount.compareTo(BigDecimal.ZERO) == 0 && CollectionUtils.isNotEmpty(musicGroupPaymentBaseCalender.getCalenderActivityList())))) {
                musicGroupPaymentCalender.setCloudTeacherPaymentFlag(true);
            }
        }

        BigDecimal studentNum = BigDecimal.ONE;
        if(musicGroupPaymentBaseCalender.getPayUserType() == SCHOOL){
            actualTotalAmount = musicGroupPaymentBaseCalender.getCurrentTotalAmount();
            if(StringUtils.equals(musicGroupPaymentBaseCalender.getCalenderFeeType(),"STUDENT")){
                String studentNumber = JSONObject.parseObject(musicGroupPaymentBaseCalender.getCalenderFeeJson()).getString("sutdentNumber");
                studentNum = new BigDecimal(studentNumber);
            }
        }

        PaymentCalenderStatusEnum status;
        if (musicGroupPaymentCalender.getPaymentType() == MUSIC_APPLY || musicGroupPaymentCalender.getPayUserType() == SCHOOL) {
            status = AUDITING;
            if(musicGroupPaymentCalender.getPaymentType() == MUSIC_APPLY){
            	if(musicGroup.getStatus() == MusicGroupStatusEnum.DRAFT || musicGroup.getStatus() == MusicGroupStatusEnum.AUDIT_FAILED){
            		status = DRAFT;
            	}
            }
        } else {
            //计算项目原现价
            BigDecimal courseCurrentAmount = musicGroupPaymentCalenderCourseSettingsService.getCurrentAmount(musicGroupPaymentBaseCalender);
            //如果课程价格是0元，那么需要审核
            if(courseCurrentAmount.compareTo(BigDecimal.ZERO) == 0 &&
                    musicGroupPaymentBaseCalender.getMusicGroupPaymentCalenderCourseSettingsList() != null &&
                    musicGroupPaymentBaseCalender.getMusicGroupPaymentCalenderCourseSettingsList().size() > 0){
                status = AUDITING;
            }else {
                BigDecimal memberCurrentAmount = musicGroupPaymentCalenderMemberService.getCurrentAmount(musicGroupPaymentBaseCalender);
                BigDecimal repairCurrentAmount = musicGroupPaymentCalenderRepairService.getCurrentAmount(musicGroupPaymentBaseCalender);
                BigDecimal activityCurrentAmount = musicGroupPaymentCalenderActivityService.getCurrentAmount(musicGroupPaymentBaseCalender);
                BigDecimal goodsCurrentAmount = musicGroupPaymentCalenderGoodsService.getCurrentAmount(musicGroupPaymentBaseCalender);
                BigDecimal currentTotalAmount = courseCurrentAmount.add(memberCurrentAmount)
                        .add(goodsCurrentAmount)
                        .add(repairCurrentAmount)
                        .add(activityCurrentAmount).setScale(0, BigDecimal.ROUND_HALF_UP);
                status = actualTotalAmount.compareTo(currentTotalAmount) == 0 ? NO : AUDITING;
            }
        }
        Date date = new Date();

        musicGroupPaymentCalender.setMusicGroupId(musicGroupId);
        musicGroupPaymentCalender.setOperator(sysUser.getId());
        musicGroupPaymentCalender.setCreateTime(date);
        musicGroupPaymentCalender.setUpdateTime(date);
        musicGroupPaymentCalender.setCurrentTotalAmount(actualTotalAmount);
        musicGroupPaymentCalender.setOriginalTotalAmount(originalTotalAmount.multiply(studentNum));
        musicGroupPaymentCalender.setStatus(status);
        musicGroupPaymentCalender.setOrganId(musicGroup.getOrganId());

        if (musicGroupPaymentCalender.getStatus() != AUDITING && musicGroupPaymentCalender.getPaymentType() != MUSIC_APPLY) {
            if (date.after(musicGroupPaymentCalender.getStartPaymentDate())) {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OPEN);
            } else if (date.after(musicGroupPaymentCalender.getDeadlinePaymentDate())) {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OVER);
            } else {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.NO);
            }
        }
        // 设置批次号
        String batchNo = idGeneratorService.generatorId() + "";
        musicGroupPaymentCalender.setBatchNo(batchNo);

        musicGroupPaymentCalenderDao.insert(musicGroupPaymentCalender);
        musicGroupPaymentBaseCalender.setCurrentTotalAmount(musicGroupPaymentCalender.getCurrentTotalAmount());
        musicGroupPaymentBaseCalender.setCalenderId(musicGroupPaymentCalender.getId());
        //如果是学校缴费，需要保存还款周期
        musicGroupCalenderRefundPeriodService.savePeriod(musicGroupPaymentBaseCalender);

        //云教练团经营报表相关
        operatingReportCloudService.save(musicGroupPaymentCalender);
        Long calenderId = musicGroupPaymentCalender.getId();
        //保存会员缴费信息
        MusicGroupPaymentCalenderMember calenderMember = musicGroupPaymentBaseCalender.getCalenderMember();
        if (calenderMember != null) {
            calenderMember = musicGroupPaymentCalenderMemberService.initBean(calenderMember);
            calenderMember.setActualAmount(memberActualAmount);
            calenderMember.setOriginalAmount(memberOriginalAmount);
            calenderMember.setCalenderId(calenderId);
            musicGroupPaymentCalenderMemberService.insert(calenderMember);
        }
        //保存乐保缴费信息
        MusicGroupPaymentCalenderRepair musicRepair = musicGroupPaymentBaseCalender.getMusicRepair();
        if (musicRepair != null) {
            musicRepair.setOriginalAmount(repairOriginalAmount);
            musicRepair.setActualAmount(repairActualAmount);
            musicRepair.setCalenderId(calenderId);
            musicGroupPaymentCalenderRepairService.insert(musicRepair);
        }
        //保存活动缴费信息
        List<MusicGroupPaymentCalenderActivity> calenderActivityList = musicGroupPaymentBaseCalender.getCalenderActivityList();
        if (CollectionUtils.isNotEmpty(calenderActivityList)) {
            musicGroupPaymentCalenderActivityService.initBean(calenderActivityList);
            musicGroupPaymentCalenderActivityService.batchInsert(musicGroupPaymentBaseCalender, calenderId);
        }
        //保存课程费用列表
        List<MusicGroupPaymentCalenderCourseSettings> courseSettingsList = musicGroupPaymentBaseCalender.getMusicGroupPaymentCalenderCourseSettingsList();
        if(CollectionUtils.isNotEmpty(courseSettingsList)){
            courseSettingsList.forEach(e -> {
                e.setOverflowCoursePrice(BigDecimal.ZERO);
            });
            //是否会员团
            if(musicGroup.getCourseViewType() == CourseViewTypeEnum.MEMBER_FEE && Objects.nonNull(calenderMember)){
                //获取云教练溢出费用
                BigDecimal courseIncome = memberFeeSettingService.getAmount(musicGroup.getOrganId(), 1, calenderMember.getPeriod(),
                        calenderMember.getNum(), MemberFeeSettingServiceImpl::getIncome);
                calenderMember.setIncome(courseIncome);
                musicGroupPaymentCalenderMemberService.update(calenderMember);
                //获取云教练溢出费用
                BigDecimal currentFee = memberFeeSettingService.getAmount(musicGroup.getOrganId(), 1, calenderMember.getPeriod(),
                        calenderMember.getNum(), MemberFeeSettingServiceImpl::getCurrentFee);
                //平摊金额
                distributeTotalAmount(courseSettingsList,currentFee.subtract(courseIncome));
            }
            musicGroupPaymentCalender.setMusicGroupPaymentCalenderCourseSettingsList(courseSettingsList);
            musicGroupPaymentCalenderCourseSettingsService.batchInsert(musicGroupPaymentCalender);
        }
        //保存商品缴费信息
        if (musicGroupPaymentCalender.getPaymentType() == GOODS_PURCHASE) {
            if(CollectionUtils.isNotEmpty(musicGroupPaymentBaseCalender.getCalenderGoodsList())){
                List<Integer> goodsIds = musicGroupPaymentBaseCalender.getCalenderGoodsList().stream().map(e -> e.getGoodsId()).collect(Collectors.toList());
                Map<Integer, Goods> goodsMap = goodsDao.getGoodies(goodsIds).stream().collect(Collectors.toMap(Goods::getId, e -> e));
                for (MusicGroupPaymentCalenderGoods e : musicGroupPaymentBaseCalender.getCalenderGoodsList()) {
                    Goods goods = goodsMap.get(e.getGoodsId());
                    e.setGoodsSn(goods.getSn());
                    e.setTotalPrice(goods.getDiscountPrice().multiply(new BigDecimal(e.getNum())));
                    e.setSinglePrice(goods.getDiscountPrice());
                    e.setCalenderId(calenderId);
                    e.setGoodsType(goods.getType().getCode());
                    e.setGoodsName(goods.getName());
                    List<ComplementGoodsDto> childGoods = goodsSubService.getBaseMapper().queryChildGoods(e.getGoodsId());
                    if(CollectionUtils.isEmpty(childGoods)){
                        throw new BizException("商品{}没有子商品",goods.getName());
                    }
                    e.setChildGoodsJson(JSON.toJSONString(childGoods));
                }
                musicGroupPaymentCalenderGoodsService.saveBatch(musicGroupPaymentBaseCalender.getCalenderGoodsList());
            }
            if(musicGroupPaymentBaseCalender.getCalenderAddress() != null){
                musicGroupPaymentBaseCalender.getCalenderAddress().setCalenderId(calenderId);
                musicGroupPaymentCalenderAddressService.save(musicGroupPaymentBaseCalender.getCalenderAddress());
            }
        }

        // 如果是报名，需要修改乐团状态
        if (musicGroupPaymentCalender.getPaymentType() == MUSIC_APPLY) {
        	// 记录操作日志
            musicGroupBuildLogDao.insert(new MusicGroupBuildLog(musicGroupId, "报名缴费项目创建成功", sysUser.getId(), ""));
            if(musicGroup.getStatus() != MusicGroupStatusEnum.DRAFT && musicGroup.getStatus() != MusicGroupStatusEnum.AUDIT_FAILED){
                musicGroup.setStatus(MusicGroupStatusEnum.FEE_AUDIT);
            }
            musicGroup.setApplyExpireDate(musicGroupPaymentCalender.getDeadlinePaymentDate());
            musicGroup.setUpdateTime(date);
            musicGroupDao.update(musicGroup);
        }
        
        Organization organization = organizationDao.get(musicGroup.getOrganId());
        sendSeoMessageSource.sendSeoMessage(musicGroup.getOrganId(), new SysUserRoleEnum[]{ORGAN_MANAGER},
                null, null, MessageTypeEnum.BACKSTAGE_PAYMENT_CALENDER_AUDIT, organization.getName(), musicGroup.getName());
        TenantContextHolder.setTenantId(organization.getTenantId());
        if (musicGroupPaymentCalender.getPaymentType() == PaymentType.ADD_STUDENT && musicGroupPaymentCalender.getStatus() != AUDITING ){
            addStudent(musicGroupPaymentCalender, musicGroup);
            //将0元未缴费学员缴费状态更新为已缴费
            this.updateNoPaymentAndZeroPaymentStatus(musicGroupPaymentCalender, true);
            List<MusicGroupPaymentCalenderCourseSettings> settingsList = musicGroupPaymentBaseCalender.getMusicGroupPaymentCalenderCourseSettingsList();
            // 学生加到班级
            classGroupService.addStudentToClassGroupAndCourseArranging(calenderAddStudent.getStudentIds(), calenderAddStudent.getClassGroupIds(),
                    batchNo, settingsList, musicGroup);
            //记录经营报表学员课程费用
//            if(CollectionUtils.isNotEmpty(settingsList)){
//                BigDecimal coursePrice = settingsList.stream().map(e -> e.getCourseCurrentPrice()).reduce(BigDecimal.ZERO, BigDecimal::add);
//                studentMusicCourseFeeService.addExpectPrice(calenderAddStudent.getStudentIds(),musicGroupId,coursePrice);
//            }
            ExecutorService executor = Executors.newFixedThreadPool(5);
            CompletableFuture.runAsync(() -> {
                imUserFriendService.refreshGroupImUserFriend(musicGroupId, GroupType.MUSIC);
            }, executor);
            executor.shutdown();
        }
        ModelMap map = new ModelMap(1);
        map.put("musicGroupPaymentCalenderBatchNo", batchNo);
        TenantContextHolder.clearTenantId();
        return BaseController.succeed(map);
    }

    public static void distributeTotalAmount(List<MusicGroupPaymentCalenderCourseSettings> settings, BigDecimal totalAmount) {
        if(CollectionUtils.isEmpty(settings) || BigDecimal.ZERO.compareTo(totalAmount) == 0){
            return;
        }
        BigDecimal remainingAmount = totalAmount;
        int lastIndex = settings.size() - 1;
        BigDecimal totalUnitPrice = settings.stream().map(e -> e.getUnitPrice()).reduce(BigDecimal.ZERO, BigDecimal::add);

        // 分摊总金额
        for (int i = 0; i < lastIndex; i++) {
            MusicGroupPaymentCalenderCourseSettings setting = settings.get(i);
            BigDecimal ratio = setting.getUnitPrice().divide(totalUnitPrice, 10, RoundingMode.DOWN);
            BigDecimal itemAmount = totalAmount.multiply(ratio).setScale(2, RoundingMode.DOWN);
            setting.setOverflowCoursePrice(itemAmount.add(setting.getOverflowCoursePrice()));
            remainingAmount = remainingAmount.subtract(itemAmount);
        }

        // 将剩余的金额放在最后一个Item对象中
        MusicGroupPaymentCalenderCourseSettings lastItem = settings.get(lastIndex);
        lastItem.setOverflowCoursePrice(lastItem.getOverflowCoursePrice().add(remainingAmount));
    }



    //校验缴费项目进行中加学员
    public void checkAddStudent(CalenderAddStudent calenderAddStudent, MusicGroup musicGroup) {
        //获取缴费状态在审核中或者已拒绝的缴费项目的学员
        Integer userId = calenderAddStudent.getStudentIds();
        if (musicGroup.getCourseViewType() == CourseViewTypeEnum.MEMBER_FEE) {
            //该学员是否在其他的会员团
            boolean hasMemberGroup = studentRegistrationDao.checkHasMemberGroup(musicGroup.getId(), userId);
            if (hasMemberGroup) {
                throw new BizException("操作失败：学员已在其他系统收费乐团中，不可报名该乐团请联系教务老师");
            }
        }

        String studentIds = musicGroupPaymentCalenderDao.queryCalenderStudentIds(musicGroup.getId(), null);
        if (StringUtils.isNotEmpty(studentIds)) {
            if (studentIds.contains(userId.toString())) {
                throw new BizException("创建缴费失败：所选学员有待审核或已拒绝的缴费项目");
            }
        }
        List<MusicGroupPaymentCalenderDetail> musicGroupPaymentCalenderDetailList = musicGroupPaymentCalenderDetailDao.queryNotPaymentStudentByUserIdAndMusicGroupId(userId, musicGroup.getId());
        if (musicGroupPaymentCalenderDetailList != null && musicGroupPaymentCalenderDetailList.size() > 0) {
            throw new BizException("创建缴费失败：已存在缴费项目");
        }
    }

    public Map<String, Object> getCalenderDetail(Map<String, Object> result, Long calenderId) {
        //课程
        List<MusicGroupPaymentCalenderCourseSettings> courseSettingsList = musicGroupPaymentCalenderCourseSettingsService.queryCalenderCourseSettings(calenderId);
        result.put("course", courseSettingsList==null?null:courseSettingsList.size() == 0?null:courseSettingsList);
        //乐保
        result.put("repair", musicGroupPaymentCalenderRepairService.findByCalenderId(calenderId));
        MusicGroupPaymentCalenderMember calenderMember = musicGroupPaymentCalenderMemberService.findByCalenderId(calenderId);
        //会员信息
        result.put("member", calenderMember);
        //活动相关信息
        List<MusicGroupPaymentCalenderActivity> byCalenderId = musicGroupPaymentCalenderActivityService.findByCalenderId(calenderId);
        result.put("activity", byCalenderId==null?null:byCalenderId.size() == 0?null:byCalenderId);
        if (calenderMember != null) {
            result.put("memberPrivilegesItemList", memberRankPrivilegesService.queryByMemberRankId(calenderMember.getMemberRankSettingId()));
        }
        //获取商品采购信息
        result.put("goods",musicGroupPaymentCalenderGoodsService.lambdaQuery().eq(MusicGroupPaymentCalenderGoods::getCalenderId,calenderId).list());
        MusicGroupPaymentCalenderAddress one = musicGroupPaymentCalenderAddressService.lambdaQuery().eq(MusicGroupPaymentCalenderAddress::getCalenderId, calenderId).one();
        if(Objects.nonNull(one)){
            MusicGroupPaymentCalenderAddressWrapper.MusicGroupPaymentCalenderAddress musicGroupPaymentCalenderAddress = JSON
                    .parseObject(JSON.toJSONString(one), MusicGroupPaymentCalenderAddressWrapper.MusicGroupPaymentCalenderAddress.class);
            // 查询地区信息
            List<String> areaIds = new ArrayList<>();
            areaIds.add(one.getProvince());
            areaIds.add(one.getCity());
            areaIds.add(one.getRegion());
            areaIds = areaIds.stream().filter(StringUtils::isNotBlank).distinct().collect(Collectors.toList());
            Map<String, String> nameMapByIds = sysAreaService.getNameMapByIds(areaIds);
            musicGroupPaymentCalenderAddress.setProvinceName(nameMapByIds.getOrDefault(one.getProvince(),""));
            musicGroupPaymentCalenderAddress.setCityName(nameMapByIds.getOrDefault(one.getCity(),""));
            musicGroupPaymentCalenderAddress.setRegionName(nameMapByIds.getOrDefault(one.getRegion(),""));
            result.put("address", musicGroupPaymentCalenderAddress);
        }
        return result;
    }

    @Override
    public Map<String, Object> getDetail(Long id) {
        Map<String, Object> result = new HashMap<>(11);
        //获取缴费项目详情
        MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(id);
        //统计预计缴费人数
        Map<Long, Long> expectNumMap = MapUtil.convertIntegerMap(musicGroupPaymentCalenderDao.countExpectNum(id));
        if (expectNumMap != null) {
            calender.setExpectNum(expectNumMap.get(id) == null ? 0 : expectNumMap.get(id).intValue());
        }
        //统计实际缴费人数
        Map<Long, Long> actualNumMap = MapUtil.convertIntegerMap(musicGroupPaymentCalenderDao.countActualNum(id));
        if (actualNumMap != null) {
            calender.setActualNum(actualNumMap.get(id) == null ? 0 : actualNumMap.get(id).intValue());
        }
        //如果是学校缴费并且金额不为0，返回费用信息
        if (calender.getPayUserType() == SCHOOL && calender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) > 0){
            List<MusicGroupCalenderRefundPeriod> list = musicGroupCalenderRefundPeriodService.findByCalenderId(id);
            if (CollectionUtils.isEmpty(list)) {

                result.put("calenderRefundPeriods", list);
            } else {
                List<MusicGroupCalenderRefundPeriodWrapper.MusicGroupCalenderRefundPeriod> parseArray = JSON.parseArray(JSON.toJSONString(list), MusicGroupCalenderRefundPeriodWrapper.MusicGroupCalenderRefundPeriod.class);

                List<Integer> userIds = list.stream().map(o -> o.getResponsiblePerson()).filter(Objects::nonNull).distinct().collect(Collectors.toList());
                if (CollectionUtils.isNotEmpty(userIds)) {
                    Map<Long, SysUser> mapByIds = sysUserService.getMapByIds(userIds);
                    parseArray.forEach(o -> {
                        if (o.getResponsiblePerson() != null) {
                            o.setResponsiblePersonName(mapByIds.getOrDefault(o.getResponsiblePerson().longValue(),new SysUser()).getRealName());
                            o.setResponsiblePersonPhone(mapByIds.getOrDefault(o.getResponsiblePerson().longValue(),new SysUser()).getPhone());
                        }
                    });
                }
                result.put("calenderRefundPeriods", parseArray);
            }
        }

        if (calender.getOrganId() != null) {
            Organization organization = organizationDao.get(calender.getOrganId());
            calender.setOrganName(organization.getName());
        }
        if (StringUtils.isNotEmpty(calender.getMusicGroupId())) {
            MusicGroup musicGroup = musicGroupDao.get(calender.getMusicGroupId());
            calender.setMusicGroupName(musicGroup.getName());
        }
        result.put("calender", calender);
        //汇总收款金额
        BigDecimal sumActualAmount = musicGroupPaymentCalenderDetailDao.sumActualAmount(id);
        result.put("sumActualAmount", sumActualAmount);
        //进行中加学员
        if (calender.getPaymentType() == ADD_STUDENT) {
            String studentIds = calender.getStudentIds();
            if (StringUtils.isNotEmpty(studentIds)) {
                SysUser sysUser = studentDao.getUser(Integer.parseInt(studentIds));
                result.put("phone", sysUser.getPhone());
                result.put("username", sysUser.getUsername());
            }
        }
        //获取跨团合班学员缴费详情
        result.put("musicMerger", musicGroupPaymentCalenderStudentDetailDao.findByBatchNo(calender.getBatchNo()));
        return getCalenderDetail(result, id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String merge(MusicGroupPaymentCalenderDto musicGroupPaymentCalenderDto) {

        String batchNo = musicGroupPaymentCalenderDto.getBatchNo();

        List<MusicGroupPaymentCalender> musicGroupPaymentCalenderList = musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
        if (musicGroupPaymentCalenderList == null || musicGroupPaymentCalenderList.size() == 0) {
            throw new BizException("[批次号]参数错误");
        }
        PaymentCalenderStatusEnum status = musicGroupPaymentCalenderList.get(0).getStatus();
        if (status != PaymentCalenderStatusEnum.REJECT && status != PaymentCalenderStatusEnum.DRAFT) {
            throw new BizException("{}的缴费状态不允许修改", status.getDesc());
        }
        PaymentType paymentType = musicGroupPaymentCalenderDto.getPaymentType();
        if (paymentType != SPAN_GROUP_CLASS_ADJUST) {
            throw new BizException("操作失败：该入口不支持{}", paymentType.getDesc());
        }
        List<MusicGroupPaymentDateRange> musicGroupPaymentDateRangeList = musicGroupPaymentCalenderDto.getMusicGroupPaymentDateRangeList();
        if (musicGroupPaymentDateRangeList.size() > 1) {
            throw new BizException("创建缴费失败：不支持多周期缴费");
        }
        String musicGroupId = musicGroupPaymentCalenderDto.getMusicGroupId();

        MusicGroup musicGroup = musicGroupDao.getLocked(musicGroupId);
        if (musicGroup == null) {
            throw new BizException("乐团查询失败，请检查参数");
        }
        SysUser sysUser = sysUserService.getUser();

        List<Long> calenderIds = musicGroupPaymentCalenderList.stream().map(MusicGroupPaymentCalender::getId).collect(Collectors.toList());
        //删除原来数据
        musicGroupPaymentCalenderDao.delByIds(calenderIds);
        musicGroupPaymentCalenderCourseSettingsService.deleteByMusicGroupPaymentCalenderId(calenderIds);

        status = PaymentCalenderStatusEnum.NO;

        Date date = new Date();

        MusicGroupStudentClassAdjust adjust = musicGroupStudentClassAdjustDao.findByBatchNo(batchNo);
        //获取默认的学员缴费详情
        List<Integer> studentIdList = JSON.parseArray(adjust.getStudentIds(), Integer.class);
        //获取欠费学员列表
        List<Integer> noPaymentUserIds = musicGroupPaymentCalenderDetailDao.queryNoPaymentUserIds(musicGroup.getId(), studentIdList,null);
        if (noPaymentUserIds.size() > 0) {
            throw new BizException("操作失败：有欠费的学员不允许创建缴费");
        }
        //获取缴费状态在审核中或者已拒绝的缴费项目的学员
        String studentIds = musicGroupPaymentCalenderDao.queryCalenderStudentIds(musicGroupId, batchNo);
        if (StringUtils.isNotEmpty(studentIds)) {
            for (Integer integer : studentIdList) {
                if (studentIds.contains(integer.toString())) {
                    throw new BizException("操作失败：所选学员有待审核或已拒绝的缴费项目");
                }
            }
        }

        List<MusicGroupPaymentCalenderStudentDetail> musicGroupPaymentCalenderStudentDetails = classGroupService.queryStudentPaymentCalenders(adjust.getMasterClassGroupId(),
                adjust.getClassGroupStudents(), studentIdList);
        List<MusicGroupPaymentCalenderStudentDetail> calenderStudentDetails = musicGroupPaymentCalenderDto.getMusicGroupPaymentCalenderStudentDetails();

        boolean containsAll = musicGroupPaymentCalenderStudentDetails.containsAll(calenderStudentDetails);
        if (!containsAll) {
            status = AUDITING;
        }
        //删除之前的记录
        musicGroupPaymentCalenderStudentDetailDao.delByBatchNo(batchNo);
        //保存现有记录
        musicGroupPaymentCalenderStudentDetailDao.batchInsert(calenderStudentDetails, batchNo);

        MusicGroupPaymentDateRange musicGroupPaymentDateRange = musicGroupPaymentDateRangeList.get(0);
        MusicGroupPaymentCalender musicGroupPaymentCalender = new MusicGroupPaymentCalender();
        musicGroupPaymentCalender.setAttribute1(musicGroupPaymentCalenderDto.getAttribute1());
        musicGroupPaymentCalender.setAttribute2(musicGroupPaymentCalenderDto.getAttribute2());
        musicGroupPaymentCalender.setDeadlinePaymentDate(musicGroupPaymentDateRange.getDeadlinePaymentDate());
        musicGroupPaymentCalender.setMemo(musicGroupPaymentCalenderDto.getMemo());
        musicGroupPaymentCalender.setMusicGroupId(musicGroupId);
        musicGroupPaymentCalender.setMusicGroupOrganizationCourseSettingId(musicGroupPaymentCalenderDto.getMusicGroupOrganizationCourseSettingId());
        musicGroupPaymentCalender.setOriginalTotalAmount(calenderStudentDetails.stream().map(e -> e.getCourseOriginalPrice()).reduce(BigDecimal.ZERO, BigDecimal::add));
        musicGroupPaymentCalender.setCurrentTotalAmount(calenderStudentDetails.stream().map(e -> e.getCourseCurrentPrice()).reduce(BigDecimal.ZERO, BigDecimal::add));
        musicGroupPaymentCalender.setPaymentType(paymentType);
        musicGroupPaymentCalender.setPayUserType(musicGroupPaymentCalenderDto.getPayUserType());
        musicGroupPaymentCalender.setStartPaymentDate(musicGroupPaymentDateRange.getStartPaymentDate());
        musicGroupPaymentCalender.setStudentIds(musicGroupPaymentCalenderDto.getStudentIds());
        musicGroupPaymentCalender.setOperator(sysUser.getId());
        musicGroupPaymentCalender.setCreateTime(date);
        musicGroupPaymentCalender.setUpdateTime(date);
        musicGroupPaymentCalender.setStatus(status);
        musicGroupPaymentCalender.setOrganId(musicGroup.getOrganId());
        if (musicGroupPaymentCalender.getStatus() != AUDITING) {
            Date day = DateUtil.stringToDate(DateUtil.format(date, DateUtil.DEFAULT_PATTERN), DateUtil.DEFAULT_PATTERN);
            if (date.after(musicGroupPaymentCalender.getStartPaymentDate())) {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OPEN);
            } else if (day.after(musicGroupPaymentCalender.getDeadlinePaymentDate())) {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OVER);
            } else {
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.NO);
            }
        }
        // 设置批次号
        musicGroupPaymentCalender.setBatchNo(batchNo);
        musicGroupPaymentCalenderDto.setBatchNo(batchNo);
        musicGroupPaymentCalenderDao.insert(musicGroupPaymentCalender);

        if (musicGroupPaymentCalender.getStatus() != AUDITING) {
            // 如果是进行中加学生
            if (musicGroupPaymentCalender.getPaymentType() == PaymentType.ADD_STUDENT) {
                addStudent(musicGroupPaymentCalender, musicGroup);
            } else if (musicGroupPaymentCalender.getPaymentType() == SPAN_GROUP_CLASS_ADJUST) {
                //如果是跨团班级合并,添加学员
                musicGroupPaymentCalenderDetailService.batchAdd(musicGroupPaymentCalender, musicGroupPaymentCalenderDto.getMusicGroupPaymentCalenderStudentDetails());
            }
            //将0元未缴费学员缴费状态更新为已缴费
            this.updateNoPaymentAndZeroPaymentStatus(musicGroupPaymentCalender, true);
        }

        if (status == AUDITING) {
            // 如果是审核中
            Organization organization = organizationDao.get(musicGroup.getOrganId());
            sendSeoMessageSource.sendSeoMessage(musicGroup.getOrganId(), new SysUserRoleEnum[]{ORGAN_MANAGER},
                    null, null, MessageTypeEnum.BACKSTAGE_PAYMENT_CALENDER_AUDIT, organization.getName(), musicGroup.getName());
        } else {
            //删除和解冻班级，课程信息
            List<Map> classGroupStudents = JSON.parseArray(adjust.getClassGroupStudents(), Map.class);
            List<Long> courseIds = JSON.parseArray(adjust.getSubLockCourseIds(), Long.class);
            List<Long> allLockCourseIds = JSON.parseArray(adjust.getAllLockCourseIds(), Long.class);
            classGroupService.spanGroupClassAdjustPass(adjust.getMasterClassGroupId()
                    , studentIdList, courseIds, classGroupStudents, allLockCourseIds, batchNo, adjust.getMasterTotalPrice());
        }
        return batchNo;
    }

    //将0元未缴费学员缴费状态更新为已缴费
    @Transactional(rollbackFor = Exception.class)
    public void updateNoPaymentAndZeroPaymentStatus(MusicGroupPaymentCalender musicGroupPaymentCalender, Boolean updateFlag) {
        if (musicGroupPaymentCalender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) == 0) {
            int j = musicGroupPaymentCalenderDetailDao.updateNoPaymentAndZeroPaymentStatus(musicGroupPaymentCalender.getId());
            if (j > 0) {
                musicGroupPaymentCalender.setActualNum((musicGroupPaymentCalender.getActualNum() == null ? 0 : musicGroupPaymentCalender.getActualNum()) + j);
                if (updateFlag) {
                    musicGroupPaymentCalenderDao.update(musicGroupPaymentCalender);
                }
            }
        }
    }

    @Override
    public boolean makesureSchoolePaid(Long id, String memo) {
        MusicGroupPaymentCalender musicGroupPaymentCalender = musicGroupPaymentCalenderDao.get(id);
        if (musicGroupPaymentCalender == null) {
            throw new BizException("查询缴费信息失败");
        }

        if (musicGroupPaymentCalender.getPayUserType() != PayUserType.SCHOOL) {
            throw new BizException("只有学校缴费才能确认缴费状态");
        }

        musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.PAID);
        musicGroupPaymentCalender.setMemo(memo);
        musicGroupPaymentCalender.setUpdateTime(new Date());
        musicGroupPaymentCalenderDao.update(musicGroupPaymentCalender);

        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addStudent(MusicGroupPaymentCalender musicGroupPaymentCalender,
                           MusicGroup musicGroup) {
        //如果是进行中加学生
        Date date = new Date();
        String studentIdStr = musicGroupPaymentCalender.getStudentIds();
        if (StringUtils.isNoneBlank(studentIdStr)) {
            //当前乐团是否有预排课计划
            MusicGroupSchoolTermCourseDetail termCourseDetail = musicGroupSchoolTermCourseDetailDao.findByCourseDateAndMusicGroupId(musicGroup.getId(), null, 0);

            List<MusicGroupPaymentCalenderActivity> calenderActivities = musicGroupPaymentCalenderActivityService.findByCalenderId(musicGroupPaymentCalender.getId());
            Map<Integer, VipGroupActivity> collect = new HashMap<>();
            if(CollectionUtils.isNotEmpty(calenderActivities)){
                List<VipGroupActivity> vipGroupActivities = vipGroupActivityDao.queryByIds(calenderActivities.stream().
                        map(e -> e.getActivityId().toString()).distinct().collect(Collectors.joining(",")));
                collect = vipGroupActivities.stream().collect(Collectors.groupingBy(e -> e.getId(),
                        Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
            }
            MusicGroupPaymentCalenderMember calenderMember = musicGroupPaymentCalenderMemberService.findByCalenderId(musicGroupPaymentCalender.getId());
            for (String studentId : studentIdStr.split(",")) {
                if (termCourseDetail != null) {
                    //是否已经预排部分课程
                    List<CourseSchedule> courseSchedules = courseScheduleDao.queryPreCourseListByMusicGroupId(musicGroup.getId());
                    if (courseSchedules.size() > 0) {
                        throw new BizException("操作失败：该乐团存在预排课课程，请优先完成预排课");
                    } else {
                        MusicGroupSchoolTermStudentCourseDetail studentCourseDetail = new MusicGroupSchoolTermStudentCourseDetail();
                        studentCourseDetail.setMusicGroupSchoolTermCourseDetailId(termCourseDetail.getId());
                        studentCourseDetail.setUserId(Integer.parseInt(studentId));
                        studentCourseDetail.setMemberCourseMinutes(termCourseDetail.getTotalCourseTime());
                        musicGroupSchoolTermStudentCourseDetailDao.insert(studentCourseDetail);
                    }
                }
                // 学生加到calenderDetail
                MusicGroupPaymentCalenderDetail musicGroupPaymentCalenderDetail = new MusicGroupPaymentCalenderDetail();
                musicGroupPaymentCalenderDetail.setMusicGroupPaymentCalenderId(musicGroupPaymentCalender.getId());
                musicGroupPaymentCalenderDetail.setCreateTime(date);
                musicGroupPaymentCalenderDetail.setPaymentStatus(PaymentStatus.NON_PAYMENT);
                musicGroupPaymentCalenderDetail.setUpdateTime(date);
                musicGroupPaymentCalenderDetail.setUserId(Integer.parseInt(studentId));
                musicGroupPaymentCalenderDetail.setStartPaymentDate(musicGroupPaymentCalender.getStartPaymentDate());
                musicGroupPaymentCalenderDetail.setDeadlinePaymentDate(musicGroupPaymentCalender.getDeadlinePaymentDate());

                musicGroupPaymentCalenderDetailDao.insert(musicGroupPaymentCalenderDetail);
                //如果是0元那么更新fee表和studentRegistration表
                if (musicGroupPaymentCalender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) == 0) {
                    // 添加会员有效时长
                    if (calenderMember != null) {
                        CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
                        cloudTeacherOrder.setLevel(calenderMember.getMemberRankSettingId());
                        cloudTeacherOrder.setOrganId(musicGroup.getOrganId());
                        cloudTeacherOrder.setType(calenderMember.getPeriod());
                        cloudTeacherOrder.setStudentId(Integer.parseInt(studentId));
                        cloudTeacherOrder.setTime(calenderMember.getNum().intValue());
                        cloudTeacherOrder.setAmount(BigDecimal.ZERO);
                        cloudTeacherOrder.setStatus(1);
                        cloudTeacherOrder.setRemark("进行中乐团0元加学生");
                        cloudTeacherOrder.setMusicGroupId(musicGroup.getId());
                        cloudTeacherOrderService.save(cloudTeacherOrder,calenderMember.getAutoActivationFlag());
                    }
                    //添加小课包
                    if (calenderActivities != null && calenderActivities.size() > 0) {
                        List<ActivityUserMapper> activityUserMapperList = new ArrayList<>();
                        for (MusicGroupPaymentCalenderActivity calenderActivity : calenderActivities) {
                            ActivityUserMapper activityUserMapper = new ActivityUserMapper();
                            activityUserMapper.setActualPrice(BigDecimal.ZERO);
                            activityUserMapper.setSubNoCoursePrice(BigDecimal.ZERO);
                            activityUserMapper.setActivityId(calenderActivity.getActivityId());
                            activityUserMapper.setUserId(Integer.parseInt(studentId));
                            activityUserMapper.setOrganId(musicGroup.getOrganId());
                            if ("网管课".equals(calenderActivity.getCategoryName())) {
                                activityUserMapper.setPracticeFlag(1);
                            } else {
                                activityUserMapper.setVipFlag(1);
                            }
                            activityUserMapperService.copyProperty(activityUserMapper,collect.get(activityUserMapper.getActivityId()),false,true);
                            activityUserMapperList.add(activityUserMapper);
                        }
                        activityUserMapperService.batchInsert(activityUserMapperList);
                    }
                    //如果是进行中加学员
                    StudentRegistration studentRegistration = studentRegistrationDao.queryByUserIdAndMusicGroupId(Integer.parseInt(studentId), musicGroupPaymentCalender.getMusicGroupId());
                    studentRegistration.setPaymentStatus(YES);
                    studentRegistration.setMusicGroupStatus(StudentMusicGroupStatusEnum.NORMAL);
                    studentRegistrationDao.update(studentRegistration);


                    // 刷新im
                    imUserFriendService.refreshGroupImUserFriend(studentRegistration.getMusicGroupId(), null);


                    //统计变更学员数
                    groupEventSource.musicGroupStudentChangeEvent(musicGroupPaymentCalender.getMusicGroupId(), StudentMusicGroupStatusEnum.NORMAL, new ArrayList<>(Arrays.asList(Integer.parseInt(studentId))));
                }
            }
        }
    }


    @Override
    public PageInfo<MusicGroupPaymentCalenderAuditDto> auditList(MusicGroupPaymentCalenderQueryInfo queryInfo) {
        PageInfo<MusicGroupPaymentCalenderAuditDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<MusicGroupPaymentCalenderAuditDto> dataList = null;
        int count = musicGroupPaymentCalenderDao.countAuditList(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = musicGroupPaymentCalenderDao.queryAuditList(params);
            List<Integer> collect = dataList.stream().map(e -> e.getOrganId()).collect(Collectors.toList());
            collect.removeAll(Collections.singleton(null));
            Map<Integer, String> organNameMap = getMap("organization", "id_", "name_", collect, queryInfo.getTenantId(), Integer.class, String.class);
            List<Integer> operatorIds = dataList.stream().map(e -> e.getOperator()).collect(Collectors.toList());
            operatorIds.removeAll(Collections.singleton(null));
            Map<Integer, String> realNameMap = getMap("sys_user", "id_", "real_name_", operatorIds, queryInfo.getTenantId(), Integer.class, String.class);
            dataList.forEach(e -> {
                e.setOrganName(organNameMap.get(e.getOrganId()));
                e.setOperatorName(realNameMap.get(e.getOperator()));
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public MusicGroupPaymentCalenderAuditDetailDto auditListDetail(String batchNo) {
        MusicGroupPaymentCalenderAuditDetailDto calenderAuditDetailDto = new MusicGroupPaymentCalenderAuditDetailDto();
        MusicGroupPaymentCalenderAuditDto auditDto = musicGroupPaymentCalenderDao.getAuditDetail(batchNo);
        calenderAuditDetailDto.setAuditDto(auditDto);
        //获取缴费周期
        List<MusicGroupPaymentCalender> groupPaymentCalenders = musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
        if (groupPaymentCalenders == null || groupPaymentCalenders.size() == 0) {
            throw new BizException("缴费项目异常，请联系管理员");
        }
        calenderAuditDetailDto.setMusicGroupPaymentCalenders(groupPaymentCalenders);
        MusicGroupPaymentCalender paymentCalender = groupPaymentCalenders.get(0);
//		if(paymentCalender.getMemberRankSettingId() != null){
//			MemberRankSetting memberRankSetting = memberRankSettingDao.get(paymentCalender.getMemberRankSettingId());
//			if(memberRankSetting != null){
//				paymentCalender.setMemberRankSettingName(memberRankSetting.getName());
//			}
//		}
        if (paymentCalender.getPaymentType() == ADD_STUDENT) {
            try {
                String studentIds = groupPaymentCalenders.get(0).getStudentIds();
                if (StringUtils.isNotEmpty(studentIds)) {
                    SysUser sysUser = sysUserService.queryUserById(Integer.parseInt(studentIds));
                    calenderAuditDetailDto.setPhone(sysUser.getPhone());
                    calenderAuditDetailDto.setUsername(sysUser.getUsername());
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        //获取收费标准
        calenderAuditDetailDto.setMusicGroupPaymentCalenderCourseSettings(musicGroupPaymentCalenderCourseSettingsService.queryCalenderCourseSettingsByBatchNo(batchNo));
        //获取跨团合班学员缴费详情
        calenderAuditDetailDto.setMusicGroupPaymentCalenderStudentDetails(musicGroupPaymentCalenderStudentDetailDao.findByBatchNo(batchNo));
        return calenderAuditDetailDto;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void auditPass(String batchNs, String auditMemo) {
        String[] split = batchNs.split(",");
        Date date = new Date();
        for (String batchNo : split) {
            List<MusicGroupPaymentCalender> musicGroupPaymentCalenders = musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
            if (CollectionUtils.isEmpty(musicGroupPaymentCalenders)) {
                throw new BizException("缴费项目不存在");
            }
            MusicGroupPaymentCalender calender = musicGroupPaymentCalenders.get(0);
            MusicGroup musicGroup = musicGroupDao.get(calender.getMusicGroupId());

            if (musicGroup.getStatus() != MusicGroupStatusEnum.PROGRESS && musicGroup.getStatus() != MusicGroupStatusEnum.FEE_AUDIT && calender.getPaymentType() != GOODS_PURCHASE) {
                throw new BizException("当前乐团状态不支持此操作");
            }
            
            if(calender.getStatus() != PaymentCalenderStatusEnum.AUDITING){
            	throw new BizException("缴费项目不是在审核中状态");
            }
            
            for (MusicGroupPaymentCalender musicGroupPaymentCalender : musicGroupPaymentCalenders) {
                //如果是报名项目，将乐团改为审核中，缴费项目修改状态，审核状态改为审核通过
                Date day = DateUtil.stringToDate(DateUtil.format(date, DateUtil.DEFAULT_PATTERN), DateUtil.DEFAULT_PATTERN);
                if (musicGroupPaymentCalender.getPayUserType() == SCHOOL) {
                    if (musicGroupPaymentCalender.getCurrentTotalAmount().compareTo(BigDecimal.ZERO) > 0) {
                        musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OPEN);
                    } else {
                        musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.PAID);
                    }
                } else if (date.after(musicGroupPaymentCalender.getStartPaymentDate())) {
                    musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OPEN);
                } else if (day.after(musicGroupPaymentCalender.getDeadlinePaymentDate())) {
                    musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.OVER);
                } else {
                    musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.NO);
                }
                musicGroupPaymentCalender.setAuditMemo(auditMemo);
                musicGroupPaymentCalender.setUpdateTime(date);
                musicGroupPaymentCalender.setAudit(sysUserService.getUserId());
                musicGroupPaymentCalender.setAuditTime(date);

                if (musicGroupPaymentCalender.getPaymentType() == PaymentType.ADD_STUDENT) {
                    musicGroupPaymentCalender.setExpectNum(1);
//                    List<MusicGroupPaymentCalenderCourseSettings> musicGroupPaymentCalenderCourseSettings = musicGroupPaymentCalenderCourseSettingsService.queryCalenderCourseSettings(musicGroupPaymentCalender.getId());
                    addStudent(musicGroupPaymentCalender, musicGroup);
                }
            }
            if (calender.getPaymentType() == ADD_STUDENT || calender.getPaymentType() == SPAN_GROUP_CLASS_ADJUST) {
                List<Integer> userIds = Arrays.stream(calender.getStudentIds().split(",")).mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
                //获取欠费学员列表
                List<Integer> noPaymentUserIds = musicGroupPaymentCalenderDetailDao.queryNoPaymentUserIds(calender.getMusicGroupId(), userIds,null);
                if (noPaymentUserIds.size() > 0) {
                    throw new BizException("{} 有欠费的学员不允许创建缴费", musicGroup.getName());
                }
                //获取缴费状态在审核中或者已拒绝的缴费项目的学员
                String studentIds = musicGroupPaymentCalenderDao.queryCalenderStudentIds(calender.getMusicGroupId(), batchNo);
                if (StringUtils.isNotEmpty(studentIds)) {
                    for (Integer integer : userIds) {
                        if (studentIds.contains(integer.toString())) {
                            throw new BizException("{} 的 {} 缴费项目，所选学员有待审核或已拒绝的缴费项目", musicGroup.getName(), calender.getPaymentType().getCode());
                        }
                    }
                }
            }
            //如果是报名，并且所有的报名都审核通过，需要修改乐团状态
            if (calender.getPaymentType() == MUSIC_APPLY) {
                //统计乐团还在审核中或者审核被拒的缴费
                int count = musicGroupPaymentCalenderDao.countAuditReject(calender.getMusicGroupId(), calender.getId());
                if (count == 0) {
                    musicGroup.setStatus(MusicGroupStatusEnum.PRE_APPLY);
                    musicGroup.setUpdateTime(date);
                    musicGroupDao.update(musicGroup);
                    //记录操作日志
                    musicGroupBuildLogDao.insert(new MusicGroupBuildLog(musicGroup.getId(), "报名缴费项目通过（费用审核中 -> 预报名）",sysUserService.getUserId(), ""));
                    this.createPreImGroup(musicGroup);
                }
            } else if (calender.getPaymentType() == ADD_STUDENT) {
                // 学生加到班级
                String classGroupIdStr = calender.getAttribute1();
                if (StringUtils.isNotBlank(classGroupIdStr)) {
                    classGroupService.addStudentToClassGroupAndCourseArranging(Integer.parseInt(calender.getStudentIds()), classGroupIdStr, batchNo,
                            musicGroupPaymentCalenderCourseSettingsService.queryCalenderCourseSettingsByBatchNo(batchNo), musicGroup);
                }
            } else if (calender.getPaymentType() == SPAN_GROUP_CLASS_ADJUST) {
                List<MusicGroupPaymentCalenderStudentDetail> calenderStudentDetails = musicGroupPaymentCalenderStudentDetailDao.findByBatchNo(batchNo);
                //将学员加入新乐团、扣除原乐团剩余课程余额、补充到现有乐团
                Map<Integer, List<MusicGroupPaymentCalenderStudentDetail>> collect = calenderStudentDetails.stream().collect(Collectors.groupingBy(e -> e.getUserId()));
                MusicGroupStudentClassAdjust adjust = musicGroupStudentClassAdjustDao.findByBatchNo(batchNo);
                List<Map> classGroupStudents = JSON.parseArray(adjust.getClassGroupStudents(), Map.class);
                for (Map<Integer, String> classGroupStudent : classGroupStudents) {
                    Integer classGroupId = classGroupStudent.keySet().iterator().next();
                    studentRegistrationService.insertStudent(classGroupStudent.get(classGroupId), musicGroupDao.findByClassGroupId(classGroupId).getId(), musicGroup.getId(), collect,null);
                }
                //删除和解冻班级，课程信息
                List<Integer> studentIds = JSON.parseArray(adjust.getStudentIds(), Integer.class);
                List<Long> courseIds = JSON.parseArray(adjust.getSubLockCourseIds(), Long.class);
                List<Long> allLockCourseIds = JSON.parseArray(adjust.getAllLockCourseIds(), Long.class);
                //如果是跨团班级合并,添加学员
                musicGroupPaymentCalenderDetailService.batchAdd(calender, calenderStudentDetails);
                classGroupService.spanGroupClassAdjustPass(adjust.getMasterClassGroupId()
                        , studentIds, courseIds, classGroupStudents, allLockCourseIds, batchNo, adjust.getMasterTotalPrice());
            } else if (calender.getPaymentType() == GOODS_PURCHASE) {
                MusicGroupPaymentCalenderAddress address = musicGroupPaymentCalenderAddressService.lambdaQuery()
                        .eq(MusicGroupPaymentCalenderAddress::getCalenderId, calender.getId()).one();
                if (address == null) {
                    throw new BizException("收货地址不存在");
                }
                //获取商品信息
                List<MusicGroupPaymentCalenderGoods> goods = musicGroupPaymentCalenderGoodsService.lambdaQuery()
                        .eq(MusicGroupPaymentCalenderGoods::getCalenderId, calender.getId()).list();
                if (CollectionUtils.isEmpty(goods)) {
                    throw new BizException("商品信息不存在");
                }
                String orderNo = "Y" + idGeneratorService.generatorId("payment");
                //获取账期金额
                List<MusicGroupCalenderRefundPeriod> refundPeriods = musicGroupCalenderRefundPeriodService.findByCalenderId(calender.getId());
                BigDecimal totalAmount = refundPeriods.stream().map(e -> e.getRefundAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);
                //管乐迷应收订单创建
                StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
                studentPaymentOrder.setActualAmount(totalAmount);
                studentPaymentOrder.setCooperationId(musicGroup.getCooperationOrganId());
                studentPaymentOrder.setMusicGroupId(calender.getMusicGroupId());
                studentPaymentOrder.setCalenderId(calender.getId());
                studentPaymentOrder.setType(OrderTypeEnum.SCHOOL_GOODS_PURCHASE);
                studentPaymentOrder.setGroupType(GroupType.OUTORDER);
                studentPaymentOrder.setOrganId(musicGroup.getOrganId());
                studentPaymentOrder.setRoutingOrganId(musicGroup.getOrganId());
                studentPaymentOrder.setExpectAmount(totalAmount);
                studentPaymentOrder.setStatus(DealStatusEnum.SUCCESS);
                studentPaymentOrder.setCreateTime(date);
                studentPaymentOrder.setUpdateTime(date);
                studentPaymentOrder.setPayTime(date);
                studentPaymentOrder.setOrderNo(orderNo);
                studentPaymentOrderDao.insert(studentPaymentOrder);
                //保存商品订单详情
                this.saveSellOrder(goods, studentPaymentOrder);
                address.setOrderNo(orderNo);
                musicGroupPaymentCalenderAddressService.updateById(address);
                // 同步订单到商城
                this.syncMallOrder(address, calender, musicGroup, goods);
            }
            for (MusicGroupPaymentCalender musicGroupPaymentCalender : musicGroupPaymentCalenders) {
                //将0元未缴费学员缴费状态更新为已缴费
                this.updateNoPaymentAndZeroPaymentStatus(musicGroupPaymentCalender, false);
            }
            musicGroupPaymentCalenderDao.batchUpdate(musicGroupPaymentCalenders);
            if (calender.getPaymentType() == ADD_STUDENT) {
                imUserFriendService.refreshGroupImUserFriend(calender.getMusicGroupId(), GroupType.MUSIC);
            }
        }
    }

    public void saveSellOrder(List<MusicGroupPaymentCalenderGoods> goods, StudentPaymentOrder studentPaymentOrder) {
        List<SellOrder> sellOrderList = new ArrayList<>();
        BigDecimal originalTotalAmount = goods.stream().map(e -> e.getTotalPrice()).reduce(BigDecimal.ZERO, BigDecimal::add);
        //获取优惠金额
        BigDecimal totalDiscountAmount = originalTotalAmount.subtract(studentPaymentOrder.getExpectAmount());
        //待分配优惠
        BigDecimal waitDiscountAmount = totalDiscountAmount;
        for (int k = 0; k < goods.size(); k++) {
            MusicGroupPaymentCalenderGoods calenderGood = goods.get(k);
            //获取比例
            BigDecimal totalProportion = BigDecimal.ZERO;
            if(studentPaymentOrder.getExpectAmount().compareTo(BigDecimal.ZERO) > 0){
                totalProportion = calenderGood.getTotalPrice().divide(originalTotalAmount, 16, RoundingMode.DOWN);
            }
            //获取现价
            BigDecimal currentTotalAmount;
            if (k == goods.size() - 1) {
                currentTotalAmount = calenderGood.getTotalPrice().subtract(waitDiscountAmount);
            } else {
                BigDecimal amount = totalDiscountAmount.multiply(totalProportion).setScale(2, RoundingMode.DOWN);
                waitDiscountAmount = waitDiscountAmount.subtract(amount);
                currentTotalAmount = calenderGood.getTotalPrice().subtract(amount);
            }
            BigDecimal subCurrentTotalAmount = currentTotalAmount;
            //待分配金额
            for (Integer i = 0; i < calenderGood.getNum(); i++) {
                List<ComplementGoodsDto> goodsDtoList = JSON.parseArray(calenderGood.getChildGoodsJson(), ComplementGoodsDto.class);
                //获取总成本
                BigDecimal totalCostPrice = goodsDtoList.stream().map(e->e.getOrganCostPrice()).reduce(BigDecimal.ZERO, BigDecimal::add);
                //待分配金额
                BigDecimal totalAmount;
                if(i == calenderGood.getNum() - 1){
                    totalAmount = subCurrentTotalAmount;
                }else {
                    BigDecimal divide = currentTotalAmount.divide(new BigDecimal(calenderGood.getNum()), 2, RoundingMode.DOWN);
                    totalAmount = divide;
                    subCurrentTotalAmount = subCurrentTotalAmount.subtract(divide);
                }
                BigDecimal waitAmount = totalAmount;
                for (int j = 0; j < goodsDtoList.size(); j++) {
                    ComplementGoodsDto goodsDto = goodsDtoList.get(j);
                    //获取比例
                    BigDecimal proportion = BigDecimal.ZERO;
                    if (totalCostPrice.compareTo(BigDecimal.ZERO) > 0) {
                        proportion = goodsDto.getOrganCostPrice().divide(totalCostPrice, 16, RoundingMode.DOWN);
                    }
                    SellOrder sellOrder = new SellOrder();
                    sellOrder.setGoodsSkuId(goodsDto.getSkuStockId());
                    sellOrder.setOrganSellCost(goodsDto.getOrganCostPrice());
                    sellOrder.setOrderId(studentPaymentOrder.getId());
                    sellOrder.setType(StringUtils.endsWithIgnoreCase("INSTRUMENT", calenderGood.getGoodsType()) ? SellTypeEnum.INSTRUMENT
                            : StringUtils.endsWithIgnoreCase("ACCESSORIES", calenderGood.getGoodsType()) ? SellTypeEnum.ACCESSORIES
                            : StringUtils.endsWithIgnoreCase("TEACHING", calenderGood.getGoodsType()) ? SellTypeEnum.TEACHING
                            : StringUtils.endsWithIgnoreCase("STAFF", calenderGood.getGoodsType()) ? SellTypeEnum.STAFF
                            : SellTypeEnum.OTHER);
                    sellOrder.setOrderNo(studentPaymentOrder.getOrderNo());
                    if (j == goodsDtoList.size() - 1) {
                        sellOrder.setOrderAmount(waitAmount);
                    } else {
                        BigDecimal amount = totalAmount.multiply(proportion).setScale(2, RoundingMode.DOWN);
                        waitAmount = waitAmount.subtract(amount);
                        sellOrder.setOrderAmount(amount);
                    }
                    sellOrder.setActualAmount(sellOrder.getOrderAmount());
                    sellOrder.setExpectAmount(sellOrder.getOrderAmount());
                    sellOrder.setCouponRemitAmount(BigDecimal.ZERO);
                    sellOrder.setBalanceAmount(BigDecimal.ZERO);
                    sellOrder.setParentGoodsId(calenderGood.getGoodsId());
                    sellOrder.setNum(1);
                    sellOrder.setGoodsName(calenderGood.getGoodsName());
                    sellOrder.setAccountType(AccountType.INTERNAL);
                    sellOrder.setCooperationOrganId(studentPaymentOrder.getCooperationId());
                    sellOrder.setOrganId(studentPaymentOrder.getOrganId());
                    sellOrder.setTransNo(studentPaymentOrder.getTransNo());
                    sellOrder.setPaymentChannel(studentPaymentOrder.getPaymentChannel());
                    sellOrder.setMerNo(studentPaymentOrder.getMerNos());
                    sellOrder.setSellTime(studentPaymentOrder.getCreateTime());
                    sellOrder.setTenantId(studentPaymentOrder.getTenantId());
                    sellOrderList.add(sellOrder);
                }
            }
        }
        sellOrderService.batchInsert(sellOrderList);
    }

    public void updateSellOrder(List<SellOrder> sellOrderList, StudentPaymentOrder studentPaymentOrder) {
        //获取总价
        BigDecimal totalCostPrice = sellOrderList.stream().map(e->e.getExpectAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);
        //待分配金额
        BigDecimal waitAmount = studentPaymentOrder.getExpectAmount();
        for (int i = 0; i < sellOrderList.size(); i++) {
            SellOrder sellOrder = sellOrderList.get(i);
            //获取比例
            BigDecimal proportion = BigDecimal.ZERO;
            if (totalCostPrice.compareTo(BigDecimal.ZERO) > 0) {
                proportion = sellOrder.getExpectAmount().divide(totalCostPrice, 16, RoundingMode.DOWN);
            }
            if (i == sellOrderList.size() - 1) {
                sellOrder.setOrderAmount(waitAmount);
            } else {
                BigDecimal amount = studentPaymentOrder.getExpectAmount().multiply(proportion).setScale(2, RoundingMode.DOWN);
                waitAmount = waitAmount.subtract(amount);
                sellOrder.setOrderAmount(amount);
            }
            sellOrder.setActualAmount(sellOrder.getOrderAmount());
            sellOrder.setExpectAmount(sellOrder.getOrderAmount());
        }
        sellOrderService.batchUpdate(sellOrderList);
    }

    public void syncMallOrder(MusicGroupPaymentCalenderAddress address, MusicGroupPaymentCalender calender,
                              MusicGroup musicGroup, List<MusicGroupPaymentCalenderGoods> goods){
        // 查询地区信息
        List<String> areaIds = new ArrayList<>();
        areaIds.add(address.getProvince());
        areaIds.add(address.getCity());
        areaIds.add(address.getRegion());
        areaIds = areaIds.stream().filter(StringUtils::isNotBlank).distinct().collect(Collectors.toList());
        Map<String, String> nameMapByIds = sysAreaService.getNameMapByIds(areaIds);
        OrderCreate orderCreate = new OrderCreate();
        orderCreate.setOrderNo(address.getOrderNo());
        orderCreate.setOrchestraId(musicGroup.getId());
        orderCreate.setMemberId(musicGroup.getCooperationOrganId().longValue());
        orderCreate.setTotalAmount(calender.getCurrentTotalAmount());
        orderCreate.setOrderFormType("MEC");
        orderCreate.setPlatformType("SCHOOL");
        orderCreate.setSourceType(2);
        orderCreate.setReceiverName(address.getName());
        orderCreate.setReceiverPostCode(address.getPostCode());
        orderCreate.setReceiverPhone(address.getPhone());
        orderCreate.setReceiverProvince(nameMapByIds.getOrDefault(address.getProvince(),""));
        orderCreate.setReceiverCity(nameMapByIds.getOrDefault(address.getCity(),""));
        orderCreate.setReceiverRegion(nameMapByIds.getOrDefault(address.getRegion(),""));
        orderCreate.setReceiverDetailAddress(address.getAddress());
        orderCreate.setStatus(1);
        orderCreate.setPayType(4);
        List<OrderCreate.OrderItem> orderItems = new ArrayList<>();
        for (MusicGroupPaymentCalenderGoods e : goods) {
            BigDecimal totalPrice = e.getTotalPrice();
            List<ComplementGoodsDto> goodsDtoList = JSON.parseArray(e.getChildGoodsJson(), ComplementGoodsDto.class);
            //总金额按比例分配
            //待分配
            BigDecimal waitRemitFee = totalPrice;
            BigDecimal totalAmount = WrapperUtil.sumList(goodsDtoList, ComplementGoodsDto::getOrganCostPrice);
            for (int i = 0; i < goodsDtoList.size(); i++) {
                ComplementGoodsDto goodsDto = goodsDtoList.get(i);
                OrderCreate.OrderItem orderItemCreate = new OrderCreate.OrderItem();
                orderItemCreate.setProductQuantity(e.getNum());
                orderItemCreate.setProductSkuId(goodsDto.getSkuStockId().longValue());
                //如果是最后一件商品
                if (i == goodsDtoList.size() - 1) {
                    orderItemCreate.setRealAmount(waitRemitFee);
                } else {
                    //获取比例
                    BigDecimal ratioAmount = goodsDto.getOrganCostPrice().divide(totalAmount, 16, RoundingMode.DOWN);
                    //获取分配的金额
                    BigDecimal multiply = ratioAmount.multiply(totalPrice).setScale(2, RoundingMode.DOWN);
                    orderItemCreate.setRealAmount(multiply);
                    waitRemitFee = waitRemitFee.subtract(multiply);
                }
                orderItems.add(orderItemCreate);
            }
        }
        orderCreate.setOrderItemList(orderItems);
        CommonResult<Boolean> result = mallFeignService.productOrderCreate(orderCreate);
        if (result.getCode() != 200) {
            throw new BizException(result.getMessage());
        }
    }

    private void createPreImGroup(MusicGroup musicGroup){
        // 创建群
        imGroupService.create(musicGroup.getId(), null, "乐团预报名", null,
                musicGroup.getName(), musicGroup.getName(), null, GroupType.MUSIC.getCode(), ImGroup.GroupTypeEnum.PARENT_MEETING);

        Map<Integer, String> userRoleMap = new HashMap<Integer, String>();

        if (musicGroup.getEducationalTeacherId() != null) {
            userRoleMap.put(musicGroup.getEducationalTeacherId(), "乐团主管");
        }
        if (musicGroup.getDirectorUserId() != null) {
            userRoleMap.put(musicGroup.getDirectorUserId(), "乐队指导");
        }
        if (musicGroup.getTransactionTeacherId() != null) {
            userRoleMap.put(musicGroup.getTransactionTeacherId(), "衔接老师");
        }
        // 添加成员
        imGroupMemberService.join(musicGroup.getId(), userRoleMap);
        //缴费老师推送消息
        sendSeoMessageSource.sendSeoMessage(musicGroup.getOrganId(), null,
                null, musicGroup.getEducationalTeacherId(), MessageTypeEnum.BACKSTAGE_CREATE_MUSIC_GROUP_ADJUST_SUCCESS, musicGroup.getName());
        Map<Integer, String> map = new HashMap<>(1);
        map.put(musicGroup.getEducationalTeacherId(), musicGroup.getEducationalTeacherId().toString());
        if (map != null && map.size() > 0) {
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.BACKSTAGE_CREATE_MUSIC_GROUP_ADJUST_SUCCESS,
                    map, null, 0, null, "SYSTEM", musicGroup.getName());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void auditRefuse(String batchNs, String auditMemo) {
        SysUser sysUser = sysUserService.getUser();
        String[] split = batchNs.split(",");
        Date date = new Date();
        
        List<String> musicGroupIdList = new ArrayList<String>();
        
        for (String batchNo : split) {
            List<MusicGroupPaymentCalender> musicGroupPaymentCalenders = musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
            if (musicGroupPaymentCalenders == null || musicGroupPaymentCalenders.size() == 0) {
                throw new BizException("缴费项目不存在");
            }
            for (MusicGroupPaymentCalender musicGroupPaymentCalender : musicGroupPaymentCalenders) {
                
                if(musicGroupPaymentCalender.getStatus() != PaymentCalenderStatusEnum.AUDITING){
                	throw new BizException("缴费项目不是在审核中状态");
                }
                
                musicGroupPaymentCalender.setStatus(PaymentCalenderStatusEnum.REJECT);
                musicGroupPaymentCalender.setUpdateTime(date);
                musicGroupPaymentCalender.setAuditMemo(auditMemo);
            }
            MusicGroupPaymentCalender calender = musicGroupPaymentCalenders.get(0);

			MusicGroup musicGroup = musicGroupDao.get(calender.getMusicGroupId());
            if (musicGroup.getStatus() != MusicGroupStatusEnum.PROGRESS && musicGroup.getStatus() != MusicGroupStatusEnum.FEE_AUDIT && calender.getPaymentType() != GOODS_PURCHASE){
                throw new BizException("当前乐团状态不支持此操作");
            }
            //如果是报名，并且所有的报名都审核通过，需要修改乐团状态
            if (calender.getPaymentType() == MUSIC_APPLY) {
				if (!musicGroupIdList.contains(calender.getMusicGroupId())) {
					musicGroup.setStatus(MusicGroupStatusEnum.FEE_AUDIT_FAILED);
					musicGroup.setUpdateTime(date);
					musicGroupDao.update(musicGroup);
					// 记录操作日志
					musicGroupBuildLogDao.insert(new MusicGroupBuildLog(musicGroup.getId(), "报名缴费项目失败（费用审核中 -> 审核拒绝）", sysUser.getId(), ""));
					
					musicGroupIdList.add(calender.getMusicGroupId());
				}
            }
            musicGroupPaymentCalenderDao.batchUpdate(musicGroupPaymentCalenders);
            
        }
    }

    //推送待续费通知
    public void pushWaitRenewMessage(Long calenderId, MusicGroup musicGroup, Set<Integer> studentIds) {
        if (studentIds == null) {
            studentIds = musicGroupPaymentCalenderDetailDao.queryNoPaymentStudentIds(calenderId);
        }
        if (studentIds.size() > 0) {
            String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL_MDAYA);
            StringBuffer memo = new StringBuffer("4?").append(baseUrl).append("/#/musicGroupRenew?calenderId=").append(calenderId).append("&id=").append(musicGroup.getId());
            Map<Integer, String> push = new HashMap<>(studentIds.size());
            studentIds.forEach(e -> push.put(e, e.toString()));
            // 发送续费通知
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
                    null, 0, memo.toString(), "STUDENT", musicGroup.getName());

            //获取未缴费且不是0元的学员列表
            Map<Integer, String> studentMaps = MapUtil.convertMybatisMap(musicGroupPaymentCalenderDetailDao.queryNoPaymentAndNotZeroStudent(calenderId, studentIds));
            if (studentMaps.size() > 0) {
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.STUDENT_SMS_WAIT_RENEW_MESSAGE, studentMaps,
                        null, 0, memo.toString(), null, musicGroup.getName());
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public void revoke(String batchNo) {
        List<MusicGroupPaymentCalender> calenders = musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
        if (calenders.size() == 0) {
            throw new BizException("缴费批次不存在");
        }
        SysUser sysUser = sysUserService.getUser();
        Date date = new Date();
        calenders.forEach(e -> {
            if (e.getPaymentType() == ADD_STUDENT) {
                throw new BizException("{} 的缴费项目不允许撤回", e.getStatus().getDesc());
            }
            if (e.getStatus() != AUDITING) {
                throw new BizException("{} 的缴费状态不允许撤回", e.getStatus().getDesc());
            }
            if (e.getPaymentType() == MUSIC_APPLY) {
                //如果是报名，并且所有的报名都审核通过，需要修改乐团状态
                MusicGroup musicGroup = musicGroupDao.get(e.getMusicGroupId());
                musicGroup.setStatus(MusicGroupStatusEnum.DRAFT);
                musicGroup.setUpdateTime(date);
                musicGroupDao.update(musicGroup);
                //记录操作日志
                musicGroupBuildLogDao.insert(new MusicGroupBuildLog(musicGroup.getId(), "报名缴费项目撤回（费用审核中 -> 草稿）", sysUser.getId(), ""));
            }
            e.setStatus(DRAFT);
        });
        musicGroupPaymentCalenderDao.batchUpdate(calenders);
    }

    @Override
    public List<MusicGroupPaymentCalender> findByBatchNo(String batchNo) {
        return musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
    }

    @Override
    public PageInfo<MusicGroupPaymentCalenderResultDto> queryCalenderPage(MusicGroupPaymentCalenderQueryInfo queryInfo) {
        PageInfo<MusicGroupPaymentCalenderResultDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<MusicGroupPaymentCalenderResultDto> dataList = null;
        int count = musicGroupPaymentCalenderDao.countCalender(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = musicGroupPaymentCalenderDao.queryCalenderPage(params);
            Set<Integer> organIdList = dataList.stream().map(e -> e.getOrganId()).collect(Collectors.toSet());
            Map<Integer, String> organNameMap = getMap("organization", "id_", "name_", organIdList, queryInfo.getTenantId(), Integer.class, String.class);
            List<Integer> operatorIds = dataList.stream().map(e -> e.getOperator()).collect(Collectors.toList());
            operatorIds.removeAll(Collections.singleton(null));
            Map<Integer, String> realNameMap = getMap("sys_user", "id_", "real_name_", operatorIds, queryInfo.getTenantId(), Integer.class, String.class);
            dataList.forEach(e -> {
                e.setOrganName(organNameMap.get(e.getOrganId()));
                e.setOperatorName(realNameMap.get(e.getOperator()));
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<ModelMap> merge1(MusicGroupPaymentBaseCalender musicGroupPaymentBaseCalender) {
    	
    	Long calenderId = musicGroupPaymentBaseCalender.getCalenderId();
    	
    	MusicGroupPaymentCalender musicGroupPaymentCalender = musicGroupPaymentCalenderDao.get(calenderId);
    	if(musicGroupPaymentCalender == null){
    		throw new BizException("缴费项目[{}]查询失败", calenderId);
    	}
    	
    	if(musicGroupPaymentCalender.getStatus() != PaymentCalenderStatusEnum.DRAFT
                && musicGroupPaymentCalender.getStatus() != PaymentCalenderStatusEnum.REJECT
        && musicGroupPaymentCalender.getPaymentType() != GOODS_PURCHASE){
    		throw new BizException("当前缴费项目状态不支持修改");
    	}
    	
        del(musicGroupPaymentBaseCalender.getCalenderId());
        return create1(musicGroupPaymentBaseCalender);
    }

    @Override
    public MusicGroupPaymentCalender getMusicGroupRegCalender(String musicGroupId) {
        return musicGroupPaymentCalenderDao.getMusicGroupRegCalender(musicGroupId);
    }

    private BigDecimal getActualAmount(MusicGroupPaymentCalenderBase baseCalender) {
        if (baseCalender != null) {
            return baseCalender.getActualAmount();
        }
        return BigDecimal.ZERO;
    }

    @Override
    public BigDecimal getRepairAmount(Long calenderId) {
        return getActualAmount(musicGroupPaymentCalenderRepairService.findByCalenderId(calenderId));
    }

    @Override
    public BigDecimal getMemberAmount(Long calenderId) {
        return getActualAmount(musicGroupPaymentCalenderMemberService.findByCalenderId(calenderId));
    }

    @Override
    public BigDecimal getActivityAmount(List<Long> buyCalenderActivityIdList) {
        List<MusicGroupPaymentCalenderActivity> activityList = musicGroupPaymentCalenderActivityService.findByIds(buyCalenderActivityIdList);
        return activityList.stream().map(e -> e.getActualAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);
    }

    @Override
    public MusicGroupPaymentCalender getApplyCalenderByMusicId(String musicGroupId) {
        return musicGroupPaymentCalenderDao.findByMusicGroupRegCalender(musicGroupId);
    }

    @Override
    public PageInfo<HasFreeCourseTimesDto> queryHasFreeCourseTimesStudent(BaseOrganQueryInfo queryInfo) {
        PageInfo<HasFreeCourseTimesDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<HasFreeCourseTimesDto> dataList = null;

        int count = musicGroupDao.countHasFreeCourseTimes(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = musicGroupDao.queryHasFreeCourseTimes(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public Map<Long, Integer> findCooperationByIds(List<Long> calenderIds) {
        return (Map<Long, Integer>)MapUtil.convertIntegerMap(musicGroupPaymentCalenderDao.findCooperationByIds(calenderIds));
    }

    @Override
    public Object queryUser(MusicGroupPaymentCalenderQueryInfo queryInfo) {
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        if ("MUSIC".equals(queryInfo.getType())) {
            return musicGroupPaymentCalenderDao.queryUser(params);
        } else if ("CLOUD".equals(queryInfo.getType())) {
            return cloudCoachPaymentProgramDao.queryUser(params);
        } else {
            return null;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public boolean autoUpdateMusicGroupPaymentCalenderStatus() {
        Date date = new Date();
        List<PaymentCalenderStatusEnum> statusList = new ArrayList<>();
        statusList.add(PaymentCalenderStatusEnum.NO);
        statusList.add(PaymentCalenderStatusEnum.OPEN);
        List<MusicGroupPaymentCalender> musicGroupPaymentCalenderList = musicGroupPaymentCalenderDao.queryByPaymentStatus(null, statusList, PayUserType.STUDENT.getCode());

        List<MusicGroupPaymentCalender> updateMusicGroupPaymentCalenderList = new ArrayList<MusicGroupPaymentCalender>();
        Date day = DateUtil.stringToDate(DateUtil.format(date, DateUtil.DEFAULT_PATTERN), DateUtil.DEFAULT_PATTERN);
        for (MusicGroupPaymentCalender mgpc : musicGroupPaymentCalenderList) {
            if (mgpc.getDeadlinePaymentDate() != null && day.after(mgpc.getDeadlinePaymentDate()) && mgpc.getStatus() == PaymentCalenderStatusEnum.OPEN) {
                // “进行中”更新至“已结束”
                mgpc.setUpdateTime(date);
                mgpc.setStatus(PaymentCalenderStatusEnum.OVER);
                //将0元未缴费学员缴费状态更新为已缴费
                this.updateNoPaymentAndZeroPaymentStatus(mgpc, false);
                updateMusicGroupPaymentCalenderList.add(mgpc);
                //如果是进行中加学员，将报名表状态改为NORMAL
                if (mgpc.getPaymentType() == MusicGroupPaymentCalender.PaymentType.ADD_STUDENT) {
                    studentRegistrationDao.updatePaymentStatusByClaenderId(mgpc.getId());
                }
            } else if (mgpc.getStartPaymentDate() != null && date.after(mgpc.getStartPaymentDate()) && mgpc.getStatus() == PaymentCalenderStatusEnum.NO) {
                MusicGroup musicGroup = musicGroupDao.get(mgpc.getMusicGroupId());
                if (musicGroup == null) {
                    continue;
                }
                //如果乐团还在审核中
                if (musicGroup.getStatus() == MusicGroupStatusEnum.DRAFT || musicGroup.getStatus() == MusicGroupStatusEnum.AUDIT || musicGroup.getStatus() == MusicGroupStatusEnum.AUDIT_FAILED || musicGroup.getStatus() == MusicGroupStatusEnum.CLOSE) {
                    continue;
                }
                // “未开始”更新至“进行中”
                mgpc.setUpdateTime(date);
                mgpc.setStatus(PaymentCalenderStatusEnum.OPEN);
                //将0元未缴费学员缴费状态更新为已缴费
                this.updateNoPaymentAndZeroPaymentStatus(mgpc, false);
                updateMusicGroupPaymentCalenderList.add(mgpc);
                if (mgpc.getPayUserType() == STUDENT) {
                    //推送待续费通知
                    pushWaitRenewMessage(mgpc.getId(), musicGroup, null);
                }
            }
        }
        if (updateMusicGroupPaymentCalenderList.size() > 0) {
            musicGroupPaymentCalenderDao.batchUpdate(updateMusicGroupPaymentCalenderList);
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchInsert(List<MusicGroupPaymentCalender> musicGroupPaymentCalenders) {
        if (musicGroupPaymentCalenders != null && musicGroupPaymentCalenders.size() > 0) {
            musicGroupPaymentCalenderDao.delByGroupId(musicGroupPaymentCalenders.get(0).getMusicGroupId());
            musicGroupPaymentCalenders.forEach(e -> {
                musicGroupPaymentCalenderDao.insert(e);
            });
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void del(Long id) {
        if (id == null) {
            throw new BizException("参数校验失败");
        }
        MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.getForLock(id);
        if (calender == null) {
            throw new BizException("缴费信息不存在");
        }

        //缴费项目中没有学生即可删除缴费项目
        List<MusicGroupPaymentCalenderDetail> musicGroupPaymentCalenderDetailList = musicGroupPaymentCalenderDetailDao.queryByCalenderId(id, null);
        if (musicGroupPaymentCalenderDetailList != null && musicGroupPaymentCalenderDetailList.size() > 0) {
            throw new BizException("操作失败：缴费项目中已存在学员");
        }
        //商品采购订单
        MusicGroupPaymentCalenderAddress address = null;
        if(calender.getStatus() == OPEN && calender.getPaymentType() == GOODS_PURCHASE){
            //TODO 商品采购订单是否已经发货
            address = musicGroupPaymentCalenderAddressService.lambdaQuery().eq(MusicGroupPaymentCalenderAddress::getCalenderId, id).one();
            if(address != null){
                if (address.getDeliveryFlag()) {
                    throw new BizException("操作失败：商品采购订单已发货");
                }
                musicGroupPaymentCalenderAddressService.removeById(address.getId());
                //商品采购订单是否已经录入
                StudentPaymentRouteOrder byOrderNo = studentPaymentRouteOrderService.getDao().getByOrderNo(address.getOrderNo());
                if (byOrderNo != null) {
                    throw new BizException("操作失败：商品采购订单已录入");
                }
                //删除关联的商品采购订单
                if(StringUtils.isNotEmpty(address.getOrderNo())){
                    studentPaymentOrderDao.deleteByOrderNo(address.getOrderNo());
                    sellOrderService.deleteByOrderNo(address.getOrderNo());
                }
            }
            musicGroupPaymentCalenderGoodsService.lambdaUpdate().eq(MusicGroupPaymentCalenderGoods::getCalenderId,id).remove();
        }

        //如果是学校缴费，删除缴费周期
        musicGroupCalenderRefundPeriodService.deleteByCalenderId(id);
        MusicGroupStudentClassAdjust adjust = musicGroupStudentClassAdjustDao.findByBatchNo(calender.getBatchNo());
        if (!Objects.isNull(adjust)) {
//            throw new BizException("操作失败：班级调整的缴费项目不支持");
            List<Map> classGroupStudents = JSON.parseArray(adjust.getClassGroupStudents(), Map.class);
            //删除班级关联的学员
            List<Integer> classGroupIds = new ArrayList<>();
            if (classGroupStudents != null && classGroupStudents.size() > 0) {
                for (Map<Integer, String> classGroupStudent : classGroupStudents) {
                    classGroupIds.addAll(classGroupStudent.keySet());
                }
            }
            List<Long> allLockCourseIds = JSON.parseArray(adjust.getAllLockCourseIds(), Long.class);
            //解冻课程
            courseScheduleDao.batchUpdateLockByCourseIds(allLockCourseIds, 0,null);
            //解冻班级
            classGroupIds.add(adjust.getMasterClassGroupId());
            classGroupService.batchUpdateLockByClassGroupIds(classGroupIds, 0);
            musicGroupStudentClassAdjustDao.delete(adjust.getId());
        }

        if (calender.getPaymentType() == PaymentType.MUSIC_APPLY) {
            // 是否存在缴费中的
            List<StudentPaymentOrder> studentPaymentOrderList = studentPaymentOrderDao.queryByDealStatus(calender.getMusicGroupId(), OrderTypeEnum.APPLY,
                    DealStatusEnum.ING);
            if (studentPaymentOrderList != null && studentPaymentOrderList.size() > 0) {
                throw new BizException("操作失败：存在缴费中的学生");
            }
        }

        musicGroupPaymentCalenderDao.delete(id);
        musicGroupPaymentCalenderDetailDao.deleteByCalenderId(id);
        musicGroupPaymentStudentCourseDetailService.deleteByMusicGroupPaymentCalenderId(id.toString(),null);
        musicGroupPaymentCalenderActivityService.delByCalenderId(id);
        musicGroupPaymentCalenderMemberService.deleteByCalenderId(id);
        musicGroupPaymentCalenderRepairService.deleteByCalenderId(id);
        List<Long> calenderIds = new ArrayList<Long>();
        calenderIds.add(id);
        musicGroupPaymentCalenderCourseSettingsService.deleteByMusicGroupPaymentCalenderId(calenderIds);

        //如果乐团是“费用审核”，则需要修改状态
        MusicGroup musicGroup = musicGroupDao.get(calender.getMusicGroupId());
        if (musicGroup != null) {
            if (musicGroup.getStatus() == MusicGroupStatusEnum.FEE_AUDIT) {
                musicGroup.setStatus(MusicGroupStatusEnum.PRE_BUILD_FEE);
                musicGroup.setUpdateTime(new Date());
                musicGroupDao.update(musicGroup);
            }
        }
        //商品采购订单
        if(calender.getPaymentType() == GOODS_PURCHASE && address != null){
            //关闭商城订单
            CommonResult<Boolean> result = mallFeignService.productUpdateOrderStatus(address.getOrderNo(), 4);
            if (result.getCode() != 200) {
                throw new BizException("操作失败：关闭商城订单失败",result.getMessage());
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteByBatchNo(String batchNo) {
        if (StringUtils.isBlank(batchNo)) {
            throw new BizException("参数错误");
        }
        List<MusicGroupPaymentCalender> musicGroupPaymentCalenderList = musicGroupPaymentCalenderDao.findByBatchNo(batchNo);
        for (MusicGroupPaymentCalender musicGroupPaymentCalender : musicGroupPaymentCalenderList) {
            del(musicGroupPaymentCalender.getId());
        }
        return true;
    }

    @Override
    public MusicGroupPaymentCalender findByMusicGroupRegCalender(String musicGroupId) {
        return musicGroupPaymentCalenderDao.findByMusicGroupRegCalender(musicGroupId);
    }
}
