package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.GoodsDao;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.page.ReplacementInstrumentQueryInfo;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.dto.BrandDto;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.mall.MallFeignService;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import com.ym.mec.biz.dal.entity.ReplacementInstrument;
import com.ym.mec.biz.service.ReplacementInstrumentService;
import com.ym.mec.biz.dal.dao.ReplacementInstrumentDao;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ReplacementInstrumentServiceImpl extends BaseServiceImpl<Integer, ReplacementInstrument> implements ReplacementInstrumentService {

    @Autowired
    private ReplacementInstrumentDao replacementInstrumentDao;
    @Autowired
    private GoodsDao goodsDao;
    @Resource
    private MallFeignService mallFeignService;

    @Override
    public BaseDAO<Integer, ReplacementInstrument> getDAO() {
        return replacementInstrumentDao;
    }

    @Override
    public PageInfo<ReplacementInstrument> getPageList(ReplacementInstrumentQueryInfo queryInfo) {
        PageInfo<ReplacementInstrument> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<ReplacementInstrument> dataList = new ArrayList<>();
        int count = goodsDao.getReplacementInstrumentCount(params);

        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = goodsDao.getReplacementGoodsPage(params);
            List<String> brandIdList = dataList.stream().map(ReplacementInstrument::getBrand).filter(StringUtils::isNotEmpty).distinct().collect(Collectors.toList());
            if (!brandIdList.isEmpty()) {
                Map<Long, String> idNameMap = mallFeignService.getList().stream().collect(Collectors.toMap(BrandDto::getId, BrandDto::getName));
                for (ReplacementInstrument instrument : dataList) {
                    String brand = instrument.getBrand();
                    if (StringUtils.isNotEmpty(brand) && NumberUtils.isNumber(brand)) {
                        instrument.setBrandName(idNameMap.getOrDefault(Long.valueOf(brand), ""));
                    }
                }
            }
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }
}