package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.wrapper.SchoolActivityWrapper;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.biz.service.SchoolActivityDetailUserService;
import com.ym.mec.biz.service.SubjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.beans.BeanUtils;
import lombok.extern.slf4j.Slf4j;
import com.ym.mec.biz.dal.entity.SchoolActivityDetail;
import com.ym.mec.biz.dal.wrapper.SchoolActivityDetailWrapper;
import com.ym.mec.biz.dal.mapper.SchoolActivityDetailMapper;
import com.ym.mec.biz.service.SchoolActivityDetailService;
import org.springframework.util.CollectionUtils;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 活动详情
 * 2023-05-04 11:32:00
 */
@Slf4j
@Service
public class SchoolActivityDetailServiceImpl extends ServiceImpl<SchoolActivityDetailMapper, SchoolActivityDetail> implements SchoolActivityDetailService {

    @Autowired
    private MusicGroupService musicGroupService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SchoolActivityDetailUserService schoolActivityDetailUserService;

    /**
     * 查询详情
     * @param id 详情ID
     * @return SchoolActivityDetail
     */
    @Override
    public SchoolActivityDetail detail(Long id) {
        
        return baseMapper.selectById(id);
    }
    
    /**
     * 分页查询
     * @param page IPage<SchoolActivityDetail>
     * @param query SchoolActivityDetailWrapper.SchoolActivityDetailQuery
     * @return IPage<SchoolActivityDetail>
     */
    @Override
    public IPage<SchoolActivityDetail> selectPage(IPage<SchoolActivityDetail> page, SchoolActivityDetailWrapper.SchoolActivityDetailQuery query) {
        
        return page.setRecords(baseMapper.selectPage(page, query));
    }

    /**
     * 添加
     * @param schoolActivityDetail SchoolActivityDetailWrapper.SchoolActivityDetail
     * @return Boolean
     */
    @Override
    public Boolean add(SchoolActivityDetailWrapper.SchoolActivityDetail schoolActivityDetail) {

        return this.save(JSON.parseObject(schoolActivityDetail.jsonString(), SchoolActivityDetail.class));
    }

    /**
     * 更新
     * @param schoolActivityDetail SchoolActivityDetailWrapper.SchoolActivityDetail
     * @return Boolean
     */
    @Override
    public Boolean update(SchoolActivityDetailWrapper.SchoolActivityDetail schoolActivityDetail){

        return this.updateById(JSON.parseObject(schoolActivityDetail.jsonString(), SchoolActivityDetail.class));       
    }

    /**
     * 获取活动的节目列表
     *
     * @param activityId 活动ID
     */
    @Override
    public List<SchoolActivityDetailWrapper.SchoolActivityDetail> selectListByActivityId(Long activityId) {
        if (activityId == null) {
            return null;
        }

        List<SchoolActivityDetail> list = this.lambdaQuery()
                .eq(SchoolActivityDetail::getSchoolActivityId, activityId)
                .orderByAsc(SchoolActivityDetail::getSort)
                .list();
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }

        List<SchoolActivityDetailWrapper.SchoolActivityDetail> schoolActivityDetails = JSON
                .parseArray(JSON.toJSONString(list), SchoolActivityDetailWrapper.SchoolActivityDetail.class);

        // 查询人员数据

        // id 集合
        List<Integer> detailIds = schoolActivityDetails.stream()
                .map(SchoolActivityDetailWrapper.SchoolActivityDetail::getId)
                .collect(Collectors.toList());

//        Map<Integer, List<SchoolActivityWrapper.SchoolActivitySubject>> studentListMap = schoolActivityDetailUserService.getSchoolActivitySubjectMapByDetailIds(detailIds);

        // 设置乐团名
        // 乐团ID集合
        List<String> musicGroupIds = schoolActivityDetails.stream()
                .map(SchoolActivityDetailWrapper.SchoolActivityDetail::getMusicGroupId)
                .collect(Collectors.toList());

        Map<String, MusicGroup> groupMap = musicGroupService.getMapByIds(musicGroupIds);

        // 设置声部名
        List<Integer> subjectIds = schoolActivityDetails.stream()
                .flatMap(o -> Arrays.stream(o.getSubjectIdList().split(",")))
                .filter(Objects::nonNull)
                .map(Integer::parseInt)
                .collect(Collectors.toList());

        Map<Integer, Subject> subjectMap = subjectService.getMapByIds(subjectIds);

        for (SchoolActivityDetailWrapper.SchoolActivityDetail schoolActivityDetail : schoolActivityDetails) {

            // 设置声部 人员数据
            SchoolActivityWrapper.SchoolActivityStudentQuery query = new SchoolActivityWrapper.SchoolActivityStudentQuery();
            query.setMusicGroupId(schoolActivityDetail.getMusicGroupId());
            query.setActivityDetailId(schoolActivityDetail.getId());
            List<SchoolActivityWrapper.SchoolActivitySubject> schoolActivitySubjects = schoolActivityDetailUserService.getUserList(query);

//            List<SchoolActivityWrapper.SchoolActivitySubject> schoolActivitySubjects = studentListMap.get(schoolActivityDetail.getId());
//            if (!CollectionUtils.isEmpty(schoolActivitySubjects)) {
                schoolActivityDetail.setStudentList(schoolActivitySubjects);
//            }

            // 设置乐团名
            MusicGroup musicGroup = groupMap.get(schoolActivityDetail.getMusicGroupId());
            if (musicGroup != null) {
                schoolActivityDetail.setMusicGroupName(musicGroup.getName());
            }

            // 设置声部名
            String subjectNameList = Arrays.stream(schoolActivityDetail.getSubjectIdList().split(","))
                    .filter(Objects::nonNull)
                    .map(Integer::parseInt)
                    .map(subjectMap::get)
                    .filter(Objects::nonNull)
                    .map(Subject::getName)
                    .collect(Collectors.joining(","));
            schoolActivityDetail.setSubjectNameList(subjectNameList);

        }

        return schoolActivityDetails;
    }
}