package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.api.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.EFriendRoleType;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.MusicGroupStatusEnum;
import com.ym.mec.biz.dal.enums.StudentMusicGroupStatusEnum;
import com.ym.mec.biz.dal.enums.school.ESchoolStaffType;
import com.ym.mec.biz.dal.mapper.SchoolStaffMapper;
import com.ym.mec.biz.dal.wrapper.SchoolStaffWrapper;
import com.ym.mec.biz.dal.wrapper.StatGroupWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.thirdparty.user.realname.RealnameAuthenticationPluginContext;
import com.ym.mec.thirdparty.user.realname.provider.LinkfaceRealnameAuthenticationPlugin;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.MessageFormat;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 学校员工
 * 2023-05-06 10:53:06
 */
@Slf4j
@Service
public class SchoolStaffServiceImpl extends ServiceImpl<SchoolStaffMapper, SchoolStaff> implements SchoolStaffService {

    @Autowired
    private CooperationOrganService cooperationOrganService;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private CooperationOrganDao cooperationOrganDao;
    @Autowired
    private SmsCodeService smsCodeService;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private StudentDao studentDao;

    @Autowired
    private ImUserFriendDao imUserFriendDao;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private ImGroupDao imGroupDao;

    @Autowired
    private RealnameAuthenticationPluginContext realnameAuthenticationPluginContext;

    @Value("${message.debugMode:false}")
    private boolean debugMode;

    @Autowired
    private SysMessageService sysMessageService;

    @Autowired
    private SchoolStaffService schoolStaffService;

    @Autowired
    private SysMessageConfigDao sysMessageConfigDao;

    @Autowired
    private OrganizationService organizationService;

    @Autowired
    private ImGroupService imGroupService;

    @Autowired
    private ImUserFriendService imUserFriendService;

    @Autowired
    private ClassGroupService classGroupService;

    @Autowired
    private SysUserService sysUserService;

    @Override
    public SchoolStaffMapper getMapper() {
        return baseMapper;
    }

    /**
     * 查询详情
     * @param id 详情ID
     * @return SchoolStaff
     */
    @Override
    public SchoolStaffWrapper.SchoolStaff detail(Long id) {

        IPage<SchoolStaffWrapper.SchoolStaff> page = selectPage(new Page<>(1, 1), SchoolStaffWrapper.SchoolStaffQuery.builder().staffId(id).build());

        if (CollectionUtils.isEmpty(page.getRecords())) {
            return null;
        }
        return page.getRecords().get(0);
    }
    
    /**
     * 分页查询
     * @param page IPage<SchoolStaff>
     * @param query SchoolStaffWrapper.SchoolStaffQuery
     * @return IPage<SchoolStaffWrapper.SchoolStaff>
     */
    @Override
    public IPage<SchoolStaffWrapper.SchoolStaff> selectPage(IPage<SchoolStaffWrapper.SchoolStaff> page, SchoolStaffWrapper.SchoolStaffQuery query) {
        List<SchoolStaffWrapper.SchoolStaff> schoolStaffs = baseMapper.selectPage(page, query);
        if (CollectionUtils.isEmpty(schoolStaffs)) {
            return page;
        }

        // 设置分部名称

        // 分部ID集合
        List<Integer> organIds = schoolStaffs.stream()
                .flatMap(o -> Arrays.stream(o.getOrganId().split(",")))
                .map(Integer::parseInt)
                .collect(Collectors.toList());


        Map<Integer, Organization> organizationMap = organizationService.getMapByIds(organIds);

        for (SchoolStaffWrapper.SchoolStaff schoolStaff : schoolStaffs) {
            String collect = Arrays.stream(schoolStaff.getOrganId().split(","))
                    .map(Integer::parseInt)
                    .map(organizationMap::get)
                    .filter(Objects::nonNull)
                    .map(Organization::getName)
                    .collect(Collectors.joining(","));
            schoolStaff.setOrganName(collect);
        }

        return page.setRecords(schoolStaffs);
    }

    /**
     * 添加
     * @param schoolStaff SchoolStaffWrapper.SchoolStaff
     * @return Boolean
     */
    @Transactional
    @Override
    public Boolean add(SchoolStaffWrapper.SchoolStaff schoolStaff) {

        String submitLockKey = MessageFormat.format("school_staff_lock:{0}", schoolStaff.getMobile());
        // 数据重复提交校验
        RBucket<Object> bucket = redissonClient.getBucket(submitLockKey);
        if (!bucket.trySet(schoolStaff.getMobile(), 5L, TimeUnit.SECONDS)) {
            throw new BizException("请勿重复提交");
        }

        // 验证码合法性校验
        if (Optional.ofNullable(schoolStaff.getCheckSmsCode()).orElse(true)) {

            if (Optional.of(debugMode).orElse(false)) {
                // 调试模式，验证码不匹配
                if (!Objects.equals(schoolStaff.getSmsCode(), SmsCodeService.DEFAULT_SMS_CODE)) {
                    throw new BizException("验证码错误");
                }
            } else {
                // 非调试模式，验证码合法性校验
                if (!smsCodeService.verifyValidCode(schoolStaff.getMobile(), schoolStaff.getSmsCode())) {
                    throw new BizException("验证码错误");
                }
            }
        }

        // 合作单位编号
        CooperationOrgan cooperation = cooperationOrganService.get(schoolStaff.getSchoolId().intValue());
        if (Objects.isNull(cooperation)) {
            // 删除缓存锁
            redissonClient.getBucket(submitLockKey).delete();

            throw new BizException("合作单位编号无效");
        }

        // 合作单位状态判断
        if (Boolean.FALSE.equals(cooperation.getIsEnable())) {
            // 删除缓存锁
            redissonClient.getBucket(submitLockKey).delete();

            throw new BizException("合作单位已禁用");
        }

        // 系统用户信息
        SysUser user = getSysUser(schoolStaff, cooperation);

        // 设置默认参数
        schoolStaff.tenantId(cooperation.getTenantId())
                .organIdList(String.valueOf(cooperation.getOrganId()))
                .setUserId(Long.valueOf(user.getId()));

        // 校长或乐团领队最多一人
        if (ESchoolStaffType.SCHOOLMASTER == schoolStaff.getUserType()
                || ESchoolStaffType.ORCHESTRA_LEADER == schoolStaff.getUserType()) {
            // 校长或乐团领队，需要校验人数是否葡，默认最多添加一人
            Integer exists = lambdaQuery()
                    .eq(SchoolStaff::getSchoolId, schoolStaff.getSchoolId())
                    .ne(SchoolStaff::getStatus, "-1")
                    .eq(SchoolStaff::getUserType, schoolStaff.getUserType().getCode()).count();
            if (exists > 0) {
                // 删除缓存锁
                redissonClient.getBucket(submitLockKey).delete();

                throw new BizException("校长或乐团领队最多添加一人");
            }
        }

        // 学校员工不允许同时存在多个学校ID
        {
            Integer exists = lambdaQuery()
                    //.ne(SchoolStaff::getSchoolId, schoolStaff.getSchoolId())
                    .eq(SchoolStaff::getUserId, user.getId())
                    .ne(SchoolStaff::getStatus, "-1")
                    .count();
            if (exists > 0) {
                // 删除缓存锁
                redissonClient.getBucket(submitLockKey).delete();

                throw new BizException("用户员工账号已存在");
            }
        }

        // 添加学校员工信息
        schoolStaff.setStatus("1");
        this.save(JSON.parseObject(schoolStaff.jsonString(), SchoolStaff.class));


        // 自动创建学校管理IM群 自动添加员工到学校乐团群
        try {
            // 重置当前合作单位机构ID
            TenantContextHolder.setTenantId(cooperation.getTenantId());

            // 创建学校im群
            imGroupService.addSchoolStaffGroup(schoolStaff.getSchoolId(), user.getId(),user.getUsername(),sysUserService.getImAvatar(user),schoolStaff.getUserType());
        } catch (Exception e) {
            log.error("加入群组失败 ",e);
            throw new BizException("加入群组失败");
        }

        // 创建学校员工IM好友
        imUserFriendService.refreshSchoolStaffFriend(schoolStaff.getSchoolId().intValue());

        // 异步添加乐团领队到其他乐团群
//         CompletableFuture.runAsync(() -> {

             if (schoolStaff.getUserType().equals(ESchoolStaffType.ORCHESTRA_LEADER)) {
                 addMusicGroup(schoolStaff, user);
             }
//         });

        // 删除缓存锁
        redissonClient.getBucket(submitLockKey).delete();

        // 消息推送
        Map<Integer, String> userMap = new HashMap<>();
        userMap.put(user.getId(), user.getId().toString());
        sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.JIGUANG_SCHOOL_ACCOUNT_REGISTER_NOTICE,
                userMap, null, 0, null,
                "SCHOOL",cooperation.getName(),schoolStaff.getUserType().getDescribe());


        // 推送给学校自己外的其他用户
        IPage<SchoolStaffWrapper.SchoolStaff> page = schoolStaffService.selectPage(new Page<>(1, -1),
                SchoolStaffWrapper.SchoolStaffQuery.builder().schoolId(schoolStaff.getSchoolId()).build());
        List<SchoolStaffWrapper.SchoolStaff> records = page.getRecords();
        if (CollectionUtils.isNotEmpty(records)) {

            Map<Integer, String> usersMap = new HashMap<>();
            records.stream()
                    .filter( o -> !o.getUserId().equals(schoolStaff.getUserId()))
                    .forEach(o -> usersMap.put(o.getUserId().intValue(), o.getUserId().toString()));

            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.JIGUANG_SCHOOL_ACCOUNT_REGISTER,
                    usersMap, null, 0, null,
                    "SCHOOL",schoolStaff.getUsername(),cooperation.getName(),schoolStaff.getUserType().getDescribe());
        }

        // 推送给自己的短信
        Map<Integer, String> selfMap = new HashMap<>();
        selfMap.put(user.getId(), user.getPhone());


        SysMessageConfig sysMessageConfig = sysMessageConfigDao.queryByType(MessageTypeEnum.SMS_SCHOOL_ACCOUNT_REGISTER.getCode());
        String url = sysMessageConfig.getUrl();

        sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.AWSMS, MessageTypeEnum.SMS_SCHOOL_ACCOUNT_REGISTER,
                selfMap, null, 0, null,
                "SCHOOL",cooperation.getName(),schoolStaff.getUserType().getDescribe(),url);

        return true;
    }

    private void addMusicGroup(SchoolStaffWrapper.SchoolStaff schoolStaff, SysUser user) {

//        CompletableFuture.runAsync( () -> {
            // 乐团领队添加到所有班级群
            List<MusicGroup> musicGroups = cooperationOrganService.musicGroupPage(schoolStaff.getSchoolId().intValue());
            if (CollectionUtils.isEmpty(musicGroups)) {
                return;
            }
            // 创建班级im好友
            for (MusicGroup musicGroup : musicGroups) {
                imUserFriendService.refreshGroupImUserFriend(musicGroup.getId(),null);
            }

            // id 集合
            List<String> musicGroupIds = musicGroups.stream().map(MusicGroup::getId).collect(Collectors.toList());

            // 班级群
            List<ClassGroup> classGroups = classGroupService.getClassGroupByMusicIds(musicGroupIds);

            if (CollectionUtils.isEmpty(classGroups)) {
                return;
            }
            // ID集合
            List<String> classGroupIds = classGroups.stream().map(o -> o.getId().toString()).collect(Collectors.toList());
            List<String> imGroupIds = new ArrayList<>();
            imGroupIds.addAll(classGroupIds);
    //        imGroupIds.addAll(musicGroupIds);

            imGroupService.addImGroup(imGroupIds, user.getId().longValue(), schoolStaff.getUserType().getDescribe());

//        });


    }

    private void delMusicGroup(SchoolStaffWrapper.SchoolStaff schoolStaff, SysUser user) {

        // 乐团领队添加到所有班级群
        List<MusicGroup> musicGroups = cooperationOrganService.musicGroupPage(schoolStaff.getSchoolId().intValue());

        if (CollectionUtils.isEmpty(musicGroups)) {
            return;
        }
        // id 集合
        List<String> musicGroupIds = musicGroups.stream().map(MusicGroup::getId).collect(Collectors.toList());

        // 班级群
        List<ClassGroup> classGroups = classGroupService.getClassGroupByMusicIds(musicGroupIds);

        if (CollectionUtils.isEmpty(classGroups)) {
            return;
        }


        // 解除im好友关联
        imUserFriendDao.deleteByUserIdAndFriendId(schoolStaff.getUserId());

        // 创建学校im好友
        imUserFriendService.refreshSchoolStaffFriend(schoolStaff.getSchoolId().intValue());
        // ID集合
        List<String> classGroupIds = classGroups.stream().map(o -> o.getId().toString()).collect(Collectors.toList());
        List<String> imGroupIds = new ArrayList<>();
        imGroupIds.addAll(classGroupIds);
        //        imGroupIds.addAll(musicGroupIds);

        imGroupService.quitGroup(imGroupIds, user.getId());

    }

    /**
     * 系统用户信息
     * @param schoolStaff SchoolStaffWrapper.SchoolStaff
     * @param cooperation CooperationOrgan
     * @return SysUser
     */
    private SysUser getSysUser(SchoolStaffWrapper.SchoolStaff schoolStaff, CooperationOrgan cooperation) {

        // 用户默认密码：gym + 联系人手机后4位
        int phoneStrLen = schoolStaff.getMobile().length();
        // 生成默认密码
        String defaultPassword = new BCryptPasswordEncoder()
                .encode("gym" + schoolStaff.getMobile().substring(phoneStrLen - 4, phoneStrLen));

        // 用户帐号信息
        SysUser user = teacherDao.getUserWithPhone(schoolStaff.getMobile());
        if (Objects.nonNull(user)) {
            // 学校员工帐号平台惟一，不能与其他帐号共存
            if (StringUtils.isNotBlank(user.getUserType())) {
                throw new BizException(999, "帐号已存在");
            }

            // 后续流程暂时忽略，用户允许存在多个身份时才走以下创建账号流程
            // 校验用户是否已存在学校员工帐号
            List<String> collect = Arrays.stream(user.getUserType().split(",")).collect(Collectors.toList());
            if (collect.contains(SysUserType.SCHOOL.getCode())) {

                // 匹配合作单位编号是否一致
                List<SchoolStaff> staffs = lambdaQuery().eq(SchoolStaff::getUserId, user.getId()).list();
                if (CollectionUtils.isNotEmpty(staffs)) {
                    SchoolStaff staff = staffs.get(0);

                    if (staff.getSchoolId().longValue() != schoolStaff.getSchoolId()) {
                        throw new BizException("帐号已存在");
                    }
                }
            }

            // 设置默认密码
            SysUser updateInfo = new SysUser();
            updateInfo.setId(user.getId());
            updateInfo.setTenantId(user.getTenantId());

            if (StringUtils.isEmpty(user.getPassword())) {
                // 修改 gym + 联系人手机后4位
                updateInfo.setPassword(defaultPassword);
            }

            // 用户身份，排除学校身份
            Set<String> userTypes = Arrays.stream(user.getUserType().split(",")).collect(Collectors.toSet());
            userTypes.add(SysUserType.SCHOOL.getCode());

            updateInfo.setUserType(String.join(",", userTypes));
            // 更新用户信息
            teacherDao.updateUser(updateInfo);

        } else {

            // 查询用户默认头象
            String defaultAvatar = Optional.ofNullable(sysConfigService.findByParamName(SysConfigService.USER_DEFAULT_AVATAR))
                    .map(SysConfig::getParanValue).orElse("");

            // 创建默认用户信息
            user = new SysUser();
            user.setUsername(schoolStaff.getUsername());
            user.setPassword(defaultPassword);
            user.setAvatar(defaultAvatar);
            user.setPhone(schoolStaff.getMobile());
            user.setTenantId(cooperation.getTenantId());
            user.setOrganId(cooperation.getOrganId());
            user.setDelFlag(YesOrNoEnum.NO);
            user.setLockFlag(YesOrNoEnum.NO.getCode());
            user.setUserType(SysUserType.SCHOOL.getCode());
            user.setUpdateTime(DateTime.now().toDate());
            user.setCreateTime(DateTime.now().toDate());

            // 添加用户信息
            teacherDao.addSysUser(user);
        }
        return user;
    }

    /**
     * 更新
     * @param schoolStaff SchoolStaffWrapper.SchoolStaff
     * @return Boolean
     */
    @Override
    public Boolean update(SchoolStaffWrapper.SchoolStaff schoolStaff){

        return this.updateById(JSON.parseObject(schoolStaff.jsonString(), SchoolStaff.class));       
    }

    /**
     * 学校员工信息
     *
     * @param userId 用户ID
     * @return SchoolStaff
     */
    @Override
    public SchoolStaff queryByUserId(Long userId) {

        // 员工信息
        List<SchoolStaff> staffs = lambdaQuery().eq(SchoolStaff::getUserId, userId).ne(SchoolStaff::getStatus,"-1").list();
        if (CollectionUtils.isEmpty(staffs)) {
            return null;
        }

        return staffs.get(0);
    }

    /**
     * 根据用户ID获取学校所有员工
     *
     * @param userId 用户ID
     * @return List<SchoolStaffWrapper.SchoolStaff>
     */
    @Override
    public List<SchoolStaffWrapper.SchoolStaff> getAllStaffsBySchoolId(Long userId) {

        // 员工信息
        List<SchoolStaff> staffs = lambdaQuery().eq(SchoolStaff::getUserId, userId).list();
        if (CollectionUtils.isEmpty(staffs)) {
            throw new BizException("员工信息不存在");
        }

        // 登陆员工信息
        SchoolStaff staff = staffs.get(0);

        // 员工所在学校人员信息
        SchoolStaffWrapper.SchoolStaffQuery query = SchoolStaffWrapper.SchoolStaffQuery
                .builder()
                .page(1)
                .rows(9999)
                .schoolId(staff.getSchoolId())
                .build();
        List<SchoolStaffWrapper.SchoolStaff> records = baseMapper.selectPage(QueryInfo.getPage(query), query);

        List<SchoolStaffWrapper.SchoolStaff> sorted = Lists.newArrayList();
        // 首先排除自己的信息，默认添加到第一位
        records.stream().filter(x -> x.getUserId().longValue() == staff.getUserId()).findFirst().ifPresent(sorted::add);

        if (CollectionUtils.isNotEmpty(sorted)) {
            // 删除自己的信息
            records.remove(sorted.get(0));
        }

        // 对其他员工信息，重新排序后返回
        if (CollectionUtils.isNotEmpty(records)) {

            // 对返回数据排序：默认自己第一位，后面按照校长、分管领导，乐团领队、负责老师
            Map<ESchoolStaffType, List<SchoolStaffWrapper.SchoolStaff>> collect = records.stream()
                    .collect(Collectors.groupingBy(SchoolStaffWrapper.SchoolStaff::getUserType));

            for (ESchoolStaffType item : ESchoolStaffType.sorted()) {

                if (collect.containsKey(item)) {
                    sorted.addAll(collect.get(item));
                }
            }
        }

        return sorted;
    }

    /**
     * 根据用户ID获取学校所有员工身份
     *
     * @param schoolId 学校ID
     * @return SchoolStaffWrapper.SchoolStaffStat
     */
    @Override
    public SchoolStaffWrapper.SchoolStaffStat getUserTypeStatByCooperationId(Integer schoolId) {

        Map<String, Integer> collect = baseMapper.selectUserTypeNumsStat(schoolId).stream()
                .collect(Collectors.toMap(StatGroupWrapper::getGid, StatGroupWrapper::getTotal, (o, n) -> n));

        // 学校统计条件
        HashMap<String, Object> paramMap = new HashMap<String, Object>() {{
            put("schoolId", schoolId);
            put("status", MusicGroupStatusEnum.PROGRESS);
        }};

        CooperationOrgan cooperationOrgan = cooperationOrganDao.get(schoolId);
        if (Objects.nonNull(cooperationOrgan)) {
            paramMap.put("tenantId", cooperationOrgan.getTenantId());
        }
        // 学校乐团统计
        int orchestraNums = musicGroupDao.countMusicGroups(paramMap);

        paramMap.put("studentStatus", StudentMusicGroupStatusEnum.NORMAL);
        // 学校在读学生
        int studentNums = studentDao.countMusicGroupStudents(paramMap);

        // 统计学校员工身份
        return SchoolStaffWrapper.SchoolStaffStat.builder()
                .schoolmasterNums(collect.getOrDefault(ESchoolStaffType.SCHOOLMASTER.getCode(), 0))
                .schoolLeaderNums(collect.getOrDefault(ESchoolStaffType.SCHOOL_LEADER.getCode(), 0))
                .orchestraLeaderNums(collect.getOrDefault(ESchoolStaffType.ORCHESTRA_LEADER.getCode(), 0))
                .teacherNums(collect.getOrDefault(ESchoolStaffType.TEACHER.getCode(), 0))
                .orchestraNums(orchestraNums)
                .studentNums(studentNums)
                .build();
    }

    /***
     * 更新合作单位信息
     * @param info SchoolStaffWrapper.Cooperation
     */
    @Override
    public void updateCooperation(SchoolStaffWrapper.Cooperation info) {

        info.setStatus("0");
        // 合作单位信息
        cooperationOrganDao.update(JSON.parseObject(info.jsonString(), CooperationOrgan.class));
    }

    /**
     * 删除用户
     *
     * @param id 用户ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean del(Long id) {
        // 删除用户信息
        SchoolStaff schoolStaff = schoolStaffService.getById(id);
        if (schoolStaff == null) {
            throw new BizException("用户信息不存在");
        }
        schoolStaff.setStatus("-1");
        this.updateById(schoolStaff);

        // sys_user 删除
        baseMapper.delUser(schoolStaff.getUserId());


        // 解除im好友关联
        imUserFriendDao.deleteByUserIdAndFriendId(schoolStaff.getUserId());

        // 删除时判断是否群主，
        //1）如果是，则随机分配一个群主，
        //2)如果删除的是最后一个群成员，那么解散群
        imGroupService.quitAllGroup(schoolStaff.getUserId());

        sysUserFeignService.exitByPhone("education", sysUserService.queryUserById(schoolStaff.getUserId().intValue()).getPhone());

        return true;

    }

    /**
     * 实名认证
     *
     * @param realName 姓名
     * @param idcardNo 身份证号
     */
    @Override
    public void realNameAuthentication(SysUser user,String realName, String idcardNo) {


        SchoolStaff schoolStaff = schoolStaffService.getByUserId(user.getId());
        if (schoolStaff == null) {
            throw new BizException("用户信息不存在");
        }

        // 验证数据合法
        if (!debugMode) {
            realnameAuthenticationPluginContext.getRealnameAuthenticationPlugin(LinkfaceRealnameAuthenticationPlugin.getName()).verify(realName, idcardNo);
        }

        // 保存到sys_user表中
        user.setRealName(realName);
        user.setIdCardNo(idcardNo);
        user.setUpdateTime(new Date());
        teacherDao.updateUser(user);
    }

    /**
     * 查询学校员工信息
     *
     * @param userId 用户ID
     * @return SchoolStaff
     */
    @Override
    public SchoolStaff getByUserId(Integer userId) {
        return this.lambdaQuery()
                .eq(SchoolStaff::getUserId, userId)
                .ne(SchoolStaff::getStatus, -1)
                .last("limit 1")
                .one();
    }

    /**
     * 更新账号信息
     *
     * @param schoolStaffUpdate SchoolStaffWrapper.SchoolStaffUpdate
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateAccount(SchoolStaffWrapper.SchoolStaffUpdate schoolStaffUpdate) {

        SchoolStaff schoolStaff = schoolStaffService.getById(schoolStaffUpdate.getStaffId());
        if (schoolStaff == null) {
            throw new BizException("用户信息不存在");
        }

        // 用户信息
        SysUser sysUser = sysUserService.queryUserById(schoolStaff.getUserId().intValue());
        if (sysUser == null) {
            throw new BizException("用户信息不存在");
        }

        // 检测手机号是否修改
        if (StringUtils.isNotBlank(schoolStaffUpdate.getPhone()) && !sysUser.getPhone().equals(schoolStaffUpdate.getPhone())) {

            // 检测手机号是否已被使用

            // 用户帐号信息
            SysUser user = teacherDao.getUserWithPhone(schoolStaffUpdate.getPhone());

            if (Objects.nonNull(user)) {
                throw new BizException("手机号已存在");
            }
            sysUser.setPhone(schoolStaffUpdate.getPhone());
            teacherDao.updateUser(sysUser);
        }

        // 检测学校是否修改
        if (schoolStaffUpdate.getCoopId() != null && !schoolStaff.getSchoolId().equals(schoolStaffUpdate.getCoopId())){
            // 删除学校账户
            del(schoolStaff.getId());
            schoolStaff.setSchoolId(schoolStaffUpdate.getCoopId());

            // 创建新学校账户
            SchoolStaffWrapper.SchoolStaff staff = new SchoolStaffWrapper.SchoolStaff();
            staff.setSchoolId(schoolStaffUpdate.getCoopId());
            if (schoolStaffUpdate.getUserType() != null) {
                staff.setUserType(schoolStaffUpdate.getUserType());
            } else {
                staff.setUserType(schoolStaff.getUserType());
            }
            staff.setUsername(sysUser.getUsername());
            staff.setMobile(sysUser.getPhone());
            staff.setCheckSmsCode(false);
            add(staff);
        } else
        // 检测账号类型是否修改
        if (schoolStaffUpdate.getUserType() != null && !schoolStaff.getUserType().equals(schoolStaffUpdate.getUserType())) {


            schoolStaff.setUserType(schoolStaffUpdate.getUserType());
            this.updateById(schoolStaff);

            // 解除im好友关联
            imUserFriendDao.deleteByUserIdAndFriendId(schoolStaff.getUserId());

            // 创建学校员工IM好友
            imUserFriendService.refreshSchoolStaffFriend(schoolStaff.getSchoolId().intValue());

            // 修改群用户类型
            imGroupDao.updateSchoolRoleType(schoolStaff.getUserId(), EFriendRoleType.getByName(schoolStaffUpdate.getUserType().name()));


            if (schoolStaffUpdate.getUserType().equals(ESchoolStaffType.ORCHESTRA_LEADER)) {
                addMusicGroup(SchoolStaffWrapper.SchoolStaff.builder()
                        .schoolId(schoolStaff.getSchoolId())
                        .userId(schoolStaff.getUserId())
                        .userType(schoolStaff.getUserType())
                        .build()
                        , sysUser);
            } else if (schoolStaff.getUserType().equals(ESchoolStaffType.ORCHESTRA_LEADER)) {
                delMusicGroup(SchoolStaffWrapper.SchoolStaff.builder()
                        .schoolId(schoolStaff.getSchoolId())
                        .userId(schoolStaff.getUserId())
                        .userType(schoolStaff.getUserType())
                        .build()
                        , sysUser);
            }

        }


    }

    @Override
    public boolean updateById(SchoolStaff entity) {
        SchoolStaff schoolStaff = getById(entity.getId());
        if (schoolStaff == null) {
            throw new BizException("用户信息不存在");
        }
        // 合作单位禁用下 不可启用
        CooperationOrgan cooperationOrgan = cooperationOrganDao.get(schoolStaff.getSchoolId().intValue());
        if (cooperationOrgan == null || !cooperationOrgan.getIsEnable()) {
            throw new BizException("合作单位已禁用，不可启用");
        }

        SysUser sysUser = sysUserService.queryUserById(schoolStaff.getUserId().intValue());
        if (StringUtils.isNotBlank(entity.getStatus())) {
            if (entity.getStatus().equals("1")) {
                sysUser.setLockFlag(0);
            } else if (entity.getStatus().equals("9")) {
                sysUser.setLockFlag(9);
            }
            teacherDao.updateUser(sysUser);
        }
        sysUserFeignService.exitByPhone("education", sysUser.getPhone());
        return super.updateById(entity);
    }


    @Override
    public IPage<SchoolStaffWrapper.SchoolStaff> selectBaseInfoByPage(IPage<SchoolStaffWrapper.SchoolStaff> page,
                                                                      SchoolStaffWrapper.SchoolStaffQuery query) {
        List<SchoolStaffWrapper.SchoolStaff> schoolStaffs = baseMapper.selectBaseInfoByPage(page, query);
        return page.setRecords(schoolStaffs);
    }
}