package com.ym.mec.biz.service.impl;


import com.alibaba.fastjson.JSON;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.MusicGroupShippingAddressQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.api.CommonResult;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.dto.OrderCreate;
import com.ym.mec.common.dto.PmsProductDto;
import com.ym.mec.common.dto.PmsProductQueryParamDto;
import com.ym.mec.common.dto.ReturnApplyDto;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.mall.MallFeignService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.GroupType.GOODS_SELL;
import static com.ym.mec.biz.dal.enums.GroupType.MUSIC;

@Service
public class SellOrderServiceImpl extends BaseServiceImpl<Integer, SellOrder> implements SellOrderService {
    @Autowired
    private SellOrderDao sellOrderDao;
    @Autowired
    private GoodsDao goodsDao;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private SysPaymentConfigService sysPaymentConfigService;
    @Autowired
    private SysUserCashAccountLogService sysUserCashAccountLogService;
    @Autowired
    private StudentPaymentOrderDetailDao studentPaymentOrderDetailDao;
    @Autowired
    private SporadicChargeInfoDao sporadicChargeInfoDao;
    @Autowired
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;
    @Autowired
    private GoodsSubService goodsSubService;
    @Autowired
    private MallFeignService mallFeignService;
    @Autowired
    private MusicGroupShippingAddressService musicGroupShippingAddressService;

    @Autowired
    private SysAreaService sysAreaService;

    @Autowired
    private StudentRepairDao studentRepairDao;


    @Autowired
    private StudentPaymentOrderAddressService studentPaymentOrderAddressService;
    @Override
    public BaseDAO<Integer, SellOrder> getDAO() {
        return sellOrderDao;
    }

    public SellOrderDao getDao() {
        return sellOrderDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addOrderDetail2SellOrder(List<StudentPaymentOrderDetail> orderDetails,
                                                    StudentPaymentOrder studentPaymentOrder, String musicGroupId) {
        //过滤非商品详情
        List<StudentPaymentOrderDetail> goodsOrderDetails = orderDetails.stream().filter(e -> StringUtils.isNotEmpty(e.getGoodsIdList())).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(goodsOrderDetails)){
            return;
        }
        //获取总金额
        BigDecimal detailTotalPrice = goodsOrderDetails.stream().map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
        //获取比例
        BigDecimal totalRatioAmount = BigDecimal.ZERO;
        if(studentPaymentOrder.getExpectAmount().compareTo(BigDecimal.ZERO) > 0){
            totalRatioAmount = detailTotalPrice.divide(studentPaymentOrder.getExpectAmount(), 32, RoundingMode.DOWN);
        }
        //获取分配的余额
        BigDecimal balancePaymentAmount = studentPaymentOrder.getBalancePaymentAmount().multiply(totalRatioAmount).setScale(2, RoundingMode.DOWN);
        //获取分配的现金
        BigDecimal totalDetailActualAmount = studentPaymentOrder.getActualAmount().multiply(totalRatioAmount).setScale(2, RoundingMode.DOWN);
        //获取总优惠券金额
        BigDecimal detailTotalRemitPrice = goodsOrderDetails.stream().map(StudentPaymentOrderDetail::getRemitFee).reduce(BigDecimal.ZERO, BigDecimal::add);

        List<SellOrder> sellOrders = new ArrayList<>();
        if(balancePaymentAmount.add(totalDetailActualAmount).compareTo(detailTotalPrice) < 0){
            //多出来的金额
            BigDecimal subtract = detailTotalPrice.subtract(balancePaymentAmount).subtract(totalDetailActualAmount);
            if(balancePaymentAmount.compareTo(BigDecimal.ZERO) == 0){
                totalDetailActualAmount = totalDetailActualAmount.add(subtract);
            }else if(totalDetailActualAmount.compareTo(BigDecimal.ZERO) == 0){
                balancePaymentAmount = balancePaymentAmount.add(subtract);
            }else {
                balancePaymentAmount = balancePaymentAmount.add(subtract);
                if(balancePaymentAmount.compareTo(studentPaymentOrder.getBalancePaymentAmount()) > 0){
                    totalDetailActualAmount = totalDetailActualAmount.add(balancePaymentAmount.subtract(studentPaymentOrder.getBalancePaymentAmount()));
                    balancePaymentAmount = studentPaymentOrder.getBalancePaymentAmount();
                }
            }
        }
        //剩余待分配的余额
        BigDecimal subDetailBalance = balancePaymentAmount;
        //剩余待分配的优惠券金额
        BigDecimal subDetailCouponRemitFee = detailTotalRemitPrice;
        //剩余待分配现金
        BigDecimal subDetailActualAmount = totalDetailActualAmount;
        for (int i = 0; i < goodsOrderDetails.size(); i++) {
            StudentPaymentOrderDetail orderDetail = goodsOrderDetails.get(i);
            //获取支付金额比例
            BigDecimal ratioAmount = BigDecimal.ZERO;
            if(detailTotalPrice.compareTo(BigDecimal.ZERO) > 0){
                ratioAmount = orderDetail.getPrice().divide(detailTotalPrice, 32, RoundingMode.DOWN);
            }
            BigDecimal ratioCouponAmount = BigDecimal.ZERO;
            if (detailTotalRemitPrice.compareTo(BigDecimal.ZERO) > 0) {
                //获取优惠券金额比例
                ratioCouponAmount = orderDetail.getRemitFee().divide(detailTotalRemitPrice, 32, RoundingMode.DOWN);
            }
            //获取分配的余额
            BigDecimal detailBalance;
            //获取分配的优惠券金额
            BigDecimal couponRemitFee;
            //获取剩余待分配的现金
            BigDecimal detailActualAmount;
            if (i == goodsOrderDetails.size() - 1) {
                //获取分配的余额
                detailBalance = subDetailBalance;
                //获取分配的优惠券金额
                couponRemitFee = subDetailCouponRemitFee;
                //获取剩余待分配的现金
                detailActualAmount = subDetailActualAmount;
            } else {
                //获取分配的余额
                detailBalance = balancePaymentAmount.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                subDetailBalance = subDetailBalance.subtract(detailBalance);
                //获取分配的优惠券金额
                couponRemitFee = detailTotalRemitPrice.multiply(ratioCouponAmount).setScale(2, RoundingMode.DOWN);
                subDetailCouponRemitFee = subDetailCouponRemitFee.subtract(couponRemitFee);
                //获取剩余待分配的现金
                detailActualAmount = totalDetailActualAmount.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                subDetailActualAmount = subDetailActualAmount.subtract(detailActualAmount);
            }
            List<Integer> goodsIds = Arrays.asList(orderDetail.getGoodsIdList().split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
            sellOrders.addAll(this.initSellOrder(studentPaymentOrder, musicGroupId, goodsIds, orderDetail.getKitGroupPurchaseType(),
                    detailBalance, orderDetail.getPrice(), couponRemitFee, detailActualAmount, false));
        }
        this.batchInsert(sellOrders);
        //同步销售订单到商城
        this.syncSellOrder2Mall(sellOrders, studentPaymentOrder,musicGroupId);
    }

    @Override
    public void batchInsert(List<SellOrder> sellOrders) {
        sellOrderDao.batchInsert(sellOrders);
    }

    @Override
    public List<SellOrder> getSellOrderByParentGoodsId(Long orderId, Integer parentGoodsId) {
        return sellOrderDao.getSellOrderByParentGoodsId(orderId, parentGoodsId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<SellOrder> refund(List<SellOrder> sellOrders, Boolean reBackFee) {
        Date nowDate = new Date();
        String orderNo = sellOrders.get(0).getOrderNo();
        for (SellOrder sellOrder : sellOrders) {
            //1、更改销售列表状态
            sellOrder.setStatus(SellStatus.REFUND);
            sellOrder.setUpdateTime(nowDate);
            sellOrder.setRefundTime(nowDate);
            sellOrderDao.update(sellOrder);

            //2、金额退到余额
            if (reBackFee && sellOrder.getExpectAmount().compareTo(BigDecimal.ZERO) > 0) {
                SysUserCashAccountLog sysUserCashAccountLog = new SysUserCashAccountLog();
                sysUserCashAccountLog.setReturnFeeType(ReturnFeeEnum.GOODS);
                sysUserCashAccountLog.setOrganId(sellOrder.getOrganId());
                sysUserCashAccountLog.setUserId(sellOrder.getUserId());
                sysUserCashAccountLog.setGroupType(GOODS_SELL);
                sysUserCashAccountLog.setComment("商品退费");
                sysUserCashAccountLog.setGroupId(sellOrder.getId().toString());
                sysUserCashAccountLog.setAmount(sysUserCashAccountLogService.musicReturnFeeCharges(sellOrder.getExpectAmount(),0));
                sysUserCashAccountLogService.insert(sysUserCashAccountLog);
//                sysUserCashAccountService.updateBalance(sellOrder.getUserId(), sellOrder.getExpectAmount().multiply(new BigDecimal(0.995)).setScale(CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_HALF_UP), PlatformCashAccountDetailTypeEnum.REFUNDS, "订单：" + sellOrder.getOrderId() + " 商品id：" + sellOrder.getGoodsId() + "退货");
            }
        }
        //3、退货
        /*if (sellOrders.get(0).getAccountType() != null) {
            goodsService.increaseStock(sellOrders, sellOrders.get(0).getAccountType());
        }*/
        //4退货同步到商城
        this.mallRefundByOrderId(sellOrders);
        //5、更改订单状态
        mallFeignService.productUpdateOrderStatus(orderNo,4);
        return sellOrders;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<SellOrder> refundByOrderId(Long orderId, Boolean reBackFee) {
        List<SellOrder> sellOrders = sellOrderDao.getSellOrderByParentGoodsId(orderId, null);
        if (sellOrders.size() <= 0) {
            return sellOrders;
        }
        return refund(sellOrders, reBackFee);
    }

    @Override
    public List<SellOrder> getRefundSellOrder(List<Long> orderIdList) {
        return sellOrderDao.getRefundSellOrder(orderIdList);
    }

    @Override
    public Map<String, BigDecimal> getSellAmount(String orderNo) {
        StudentPaymentOrder order = studentPaymentOrderDao.findOrderByOrderNo(orderNo);
        if (order == null) {
            return null;
        }
        
        List<StudentPaymentOrderDetail> studentPaymentOrderDetailList = studentPaymentOrderDetailService.queryOrderDetail(order.getId());
        BigDecimal cloudIncome = BigDecimal.ZERO;
        BigDecimal cloudBalanceIncome = BigDecimal.ZERO;
        if(studentPaymentOrderDetailList != null) {
        	cloudIncome = studentPaymentOrderDetailList.stream().filter(t -> StringUtils.isNoneBlank(t.getMinuendStockGoodsIdList()) && StringUtils.equals(t.getIncomeItem(), OrderDetailTypeEnum.CLOUD_TEACHER.name())).map(t -> t.getIncome()).reduce(BigDecimal.ZERO, BigDecimal::add);
        	cloudBalanceIncome = studentPaymentOrderDetailList.stream().filter(t -> StringUtils.isNoneBlank(t.getMinuendStockGoodsIdList()) && StringUtils.equals(t.getIncomeItem(), OrderDetailTypeEnum.CLOUD_TEACHER.name())).map(t -> t.getBalanceIncome()).reduce(BigDecimal.ZERO, BigDecimal::add);
        }

        //零星支付除了充值其他都是服务费用
        if (order.getType().equals(OrderTypeEnum.SPORADIC)) {
            SporadicChargeInfo info = sporadicChargeInfoDao.get(Integer.parseInt(order.getMusicGroupId()));
            if (info.getChargeType().equals(SporadicChargeTypeEnum.RECHARGE)) {
                return null;
            }
        }
        Map<String, BigDecimal> sellAmount = new HashMap<>();
        //乐器销售,声部更改,乐器置换
        if (order.getType().equals(OrderTypeEnum.GOODS_SELL) || order.getType().equals(OrderTypeEnum.SUBJECT_CHANGE) || order.getType().equals(OrderTypeEnum.REPLACEMENT)) {
            sellAmount.put("actualAmount", order.getActualAmount().subtract(cloudIncome));
            sellAmount.put("balance", order.getBalancePaymentAmount() == null ? BigDecimal.ZERO : order.getBalancePaymentAmount().subtract(cloudBalanceIncome));
            return sellAmount;
        }
        //乐团报名,乐器维修
        if (order.getType().equals(OrderTypeEnum.APPLY) || order.getType().equals(OrderTypeEnum.REPAIR)) {
            return this.calcSellAmount(order,cloudBalanceIncome, cloudIncome);
        }
        sellAmount.put("actualAmount", BigDecimal.ZERO);
        sellAmount.put("balance", BigDecimal.ZERO);
        return sellAmount;
    }

    @Override
    public List<OrderCreate.OrderItem> convertMallOrder(List<SellOrder> sellOrderList) {
        //按组合商品分组
        List<OrderCreate.OrderItem> orderItems = new ArrayList<>();
        Map<Integer, List<SellOrder>> goodsIdSellOrderMap = sellOrderList.stream().collect(Collectors.groupingBy(SellOrder::getParentGoodsId));
        goodsIdSellOrderMap.keySet().forEach(e -> {
            List<SellOrder> sellOrders = goodsIdSellOrderMap.get(e);
            //按sku分组
            Map<Integer, List<SellOrder>> skuIdSellOrderMap = sellOrders.stream().collect(Collectors.groupingBy(SellOrder::getGoodsSkuId));
            skuIdSellOrderMap.keySet().forEach(skuId -> {
                List<SellOrder> orders = skuIdSellOrderMap.get(skuId);
                OrderCreate.OrderItem orderItemCreate = new OrderCreate.OrderItem();
                orderItemCreate.setProductQuantity(orders.size());
                orderItemCreate.setProductSkuId(skuId.longValue());
                orderItemCreate.setRealAmount(orders.stream().map(SellOrder::getExpectAmount).reduce(BigDecimal.ZERO, BigDecimal::add).divide(new BigDecimal(orders.size()), 2, RoundingMode.DOWN));
                orderItems.add(orderItemCreate);
            });
        });
        return orderItems;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void syncSellOrder2Mall(List<SellOrder> sellOrders, StudentPaymentOrder studentPaymentOrder,String musicGroupId) {
        List<OrderCreate.OrderItem> orderItems = this.convertMallOrder(sellOrders);
        OrderCreate mallOrder = new OrderCreate();
        mallOrder.setOrderItemList(orderItems);
        mallOrder.setOrchestraId(studentPaymentOrder.getMusicGroupId());
        mallOrder.setStatus(1);
        mallOrder.setOrderNo(studentPaymentOrder.getOrderNo());
        mallOrder.setTotalAmount(sellOrders.stream().map(SellOrder::getExpectAmount).reduce(BigDecimal.ZERO, BigDecimal::add));
        String paymentBusinessChannel = studentPaymentOrder.getPaymentBusinessChannel();
        if(StringUtils.isEmpty(paymentBusinessChannel)) {
            if (StringUtils.isEmpty(studentPaymentOrder.getPaymentChannel())){
                mallOrder.setPayType(4);
            }else {
                mallOrder.setPayType(3);
            }
        }else {
            if (StringUtils.endsWithIgnoreCase(paymentBusinessChannel, "wx_pub") ||
                    StringUtils.endsWithIgnoreCase(paymentBusinessChannel, "WECHAT") ||
                    StringUtils.endsWithIgnoreCase(paymentBusinessChannel, "WXPay")) {
                mallOrder.setPayType(2);
            } else if (StringUtils.endsWithIgnoreCase(paymentBusinessChannel, "BALANCE")) {
                mallOrder.setPayType(3);
            }else {
                mallOrder.setPayType(1);
            }
        }
        Integer memberId = studentPaymentOrder.getUserId()==null?studentPaymentOrder.getCooperationId():studentPaymentOrder.getUserId();
        mallOrder.setMemberId(memberId==null?0L:memberId);
        mallOrder.setPlatformType(studentPaymentOrder.getUserId()==null?"SCHOOL":"STUDENT");
        mallOrder.setSourceType(mallOrder.getPlatformType().equals("SCHOOL")?2:1);

        // 乐团学生报名默认收货地址
        if ((studentPaymentOrder.getType() == OrderTypeEnum.APPLY || studentPaymentOrder.getType() == OrderTypeEnum.SUBJECT_CHANGE)
            && StringUtils.isNotBlank(mallOrder.getOrchestraId())) {

            // 查询乐团默认收货地址
            MusicGroupShippingAddressQueryInfo queryInfo = new MusicGroupShippingAddressQueryInfo();
            queryInfo.setMusicGroupId(musicGroupId);
            queryInfo.setPage(1);
            queryInfo.setRows(10);
            List<MusicGroupShippingAddress> rows = musicGroupShippingAddressService.queryPage(queryInfo).getRows();
            if (CollectionUtils.isNotEmpty(rows)) {
                MusicGroupShippingAddress address = rows.get(0);
                mallOrder.setReceiverName(address.getConsigneeName());
                mallOrder.setReceiverPhone(address.getMobile());
                mallOrder.setReceiverProvince(address.getRegionProvince());
                mallOrder.setReceiverCity(address.getRegionCity());
                mallOrder.setReceiverRegion(address.getRegionCounty());
                mallOrder.setReceiverDetailAddress(address.getDetailAddress());
            }
        } else if(studentPaymentOrder.getType() == OrderTypeEnum.GOODS_SELL) {
            StudentPaymentOrderAddress one = studentPaymentOrderAddressService.lambdaQuery()
                .eq(StudentPaymentOrderAddress::getOrderNo, studentPaymentOrder.getOrderNo())
                .last("limit 1")
                .one();
            if(one != null) {
                // 查询地区信息
                List<String> areaIds = new ArrayList<>();
                areaIds.add(one.getProvinceId()==null?null:one.getProvinceId().toString());
                areaIds.add(one.getCityId()==null?null:one.getCityId().toString());
                areaIds.add(one.getCountyId()==null?null:one.getCountyId().toString());
                areaIds = areaIds.stream().filter(StringUtils::isNotBlank).distinct().collect(Collectors.toList());
                Map<String, String> nameMapByIds = sysAreaService.getNameMapByIds(areaIds);
                mallOrder.setReceiverName(one.getContactName());
                mallOrder.setReceiverPhone(one.getContactMobile());
                if (one.getProvinceId() != null) {
                    mallOrder.setReceiverProvince(nameMapByIds.getOrDefault(one.getProvinceId().toString(), ""));
                }
                if (one.getCityId() != null) {
                    mallOrder.setReceiverCity(nameMapByIds.getOrDefault(one.getCityId().toString(), ""));
                }
                if (one.getCountyId() != null) {
                    mallOrder.setReceiverRegion(nameMapByIds.getOrDefault(one.getCountyId().toString(), ""));
                }
                mallOrder.setReceiverDetailAddress(one.getAddress());

            }
        } else if (studentPaymentOrder.getType() == OrderTypeEnum.REPAIR) {
            StudentRepair studentRepair = studentRepairDao.getByOrderNo(studentPaymentOrder.getOrderNo());
            if (studentRepair != null) {
                mallOrder.setReceiverName(studentRepair.getContactName());
                mallOrder.setReceiverPhone(studentRepair.getContactMobile());
                mallOrder.setReceiverDetailAddress(studentRepair.getAddress());
            }
        }

        CommonResult<Boolean> result = mallFeignService.productOrderCreate(mallOrder);
        if (result.getCode() != 200) {
            throw new BizException("同步商城订单失败:{}",result.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void mallRefundByOrderId(List<SellOrder> sellOrders) {
        ReturnApplyDto returnApply = new ReturnApplyDto();
        returnApply.setOrderSn(sellOrders.get(0).getOrderNo());
        returnApply.setReason("管乐迷商品更换退货");
        if(sellOrders.get(0).getGoodsSkuId() != null){
            String skuIds = sellOrders.stream().map(e -> e.getGoodsSkuId() == null ?"":e.getGoodsSkuId().toString()).collect(Collectors.joining(","));
            returnApply.setSkuIds(skuIds);
            CommonResult<Boolean> result = mallFeignService.returnApply(returnApply);
            if (result.getCode() != 200){
                throw new BizException("商城订单退货失败",result.getMessage());
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<SellOrder> initSellOrder(StudentPaymentOrder order, String musicGroupId, List<Integer> goodsIds, KitGroupPurchaseTypeEnum kitGroupPurchaseType,
                                         BigDecimal totalBalance, BigDecimal expectTotalAmount, BigDecimal couponRemitAmount,
                                         BigDecimal actualTotalAmount, Boolean saveFlag) {
        //可用优惠券金额
        BigDecimal subCouponRemitAmount = couponRemitAmount;
        //可用总金额
        BigDecimal subExpectTotalAmount = expectTotalAmount;
        //可用总余额
        BigDecimal subTotalBalance = totalBalance;
        //实际金额
        BigDecimal subTotalActualAmount = actualTotalAmount;
        Integer tenantId = order.getTenantId();
        MusicGroup musicGroup = new MusicGroup();
        if (StringUtils.isNotBlank(musicGroupId)) {
            musicGroup = musicGroupDao.get(musicGroupId);
        }
        List<Goods> goodies = goodsDao.getGoodies(goodsIds);
        BigDecimal goodsTotalPrice = goodies.stream().map(e->order.getGroupType() == MUSIC ?e.getGroupPurchasePrice():e.getDiscountPrice()).
                reduce(BigDecimal.ZERO, BigDecimal::add);

        AccountType accountType = AccountType.INTERNAL;
        if(StringUtils.isNotEmpty(order.getPaymentChannel())){
            accountType  = sysPaymentConfigService.checkAccountType(PaymentChannelEnum.valueOf(order.getPaymentChannel()), order.getMerNos(), order.getTenantId());
        }
        PmsProductQueryParamDto paramDto = new PmsProductQueryParamDto();
        String skuIds = goodsSubService.lambdaQuery().in(GoodsSub::getGoodsId, goodies.stream().map(Goods::getId).collect(Collectors.toList())).list()
                .stream().map(e->e.getSku().toString()).collect(Collectors.joining(","));
        paramDto.setSkuStockIds(skuIds);
        paramDto.setPageSize(1000);
        paramDto.setPageNum(1);
        paramDto.setJson(JSON.toJSONString(paramDto));
        PageInfo<PmsProductDto> productList = mallFeignService.getProductList(paramDto);
        if (org.springframework.util.CollectionUtils.isEmpty(productList.getRows())) {
            throw new BizException("商城商品不存在");
        }
        Map<Integer,PmsProductDto> skuIdProductMap = productList.getRows().stream().collect(Collectors.toMap(PmsProductDto::getSkuStockId, Function.identity()));
        List<SellOrder> sellOrderList = new ArrayList<>();
        for (int i = 0; i < goodies.size(); i++) {
            Goods nowGoods = goodies.get(i);
            //获取比例
            BigDecimal goodsRatioAmount = BigDecimal.ZERO;
            //预计金额
            BigDecimal expectAmount;
            //余额
            BigDecimal goodsBalance;
            //减免金额
            BigDecimal remitAmount;
            //实际金额
            BigDecimal actualAmount;

            if (goodsTotalPrice.compareTo(BigDecimal.ZERO) > 0) {
                goodsRatioAmount = order.getGroupType() == MUSIC ?nowGoods.getGroupPurchasePrice():nowGoods.getDiscountPrice()
                        .divide(goodsTotalPrice, 32, RoundingMode.DOWN);
            }
            if(i == goodies.size() - 1){
                expectAmount = subExpectTotalAmount;
                goodsBalance = subTotalBalance;
                remitAmount = subCouponRemitAmount;
                actualAmount = subTotalActualAmount;
            }else {
                expectAmount = goodsRatioAmount.multiply(expectTotalAmount).setScale(2, RoundingMode.DOWN);
                subExpectTotalAmount = subExpectTotalAmount.subtract(expectAmount);
                goodsBalance = goodsRatioAmount.multiply(totalBalance).setScale(2, RoundingMode.DOWN);
                subTotalBalance = subTotalBalance.subtract(goodsBalance);
                remitAmount = goodsRatioAmount.multiply(couponRemitAmount).setScale(2, RoundingMode.DOWN);
                subCouponRemitAmount = subCouponRemitAmount.subtract(remitAmount);
                actualAmount = expectAmount.subtract(goodsBalance).subtract(remitAmount);
                subTotalActualAmount = subTotalActualAmount.subtract(actualAmount);
            }
            //剩余预计金额
            BigDecimal subExpectAmount = expectAmount;
            //剩余余额
            BigDecimal subGoodsBalance = goodsBalance;
            //剩余减免金额
            BigDecimal subRemitAmount = remitAmount;
            //剩余实际金额
            BigDecimal subActualAmount = actualAmount;
            //租赁、免费的没有销售收入，有销售成本
            if (nowGoods.getType().equals(GoodsType.INSTRUMENT) && kitGroupPurchaseType != null &&
                    (kitGroupPurchaseType.equals(KitGroupPurchaseTypeEnum.LEASE) || kitGroupPurchaseType.equals(KitGroupPurchaseTypeEnum.FREE))) {
                goodsBalance = BigDecimal.ZERO;
                expectAmount = BigDecimal.ZERO;
            }

            List<GoodsSub> goodsSubs = goodsSubService.lambdaQuery().eq(GoodsSub::getGoodsId, nowGoods.getId()).list();
            //商品原始总价
            BigDecimal complementPrice = goodsSubs.stream().map(GoodsSub::getGoodsPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            //生成sellOrder
            for (int j = 0; j < goodsSubs.size(); j++) {
                GoodsSub goodsSub = goodsSubs.get(j);
                PmsProductDto productDto = skuIdProductMap.get(goodsSub.getSku());
                if(Objects.isNull(productDto)){
                    throw new BizException("商品["+nowGoods.getName()+"]子商品["+goodsSub.getSku()+"]不存在");
                }
                SellOrder sellOrder = new SellOrder();
                //获取比例
                BigDecimal ratioAmount = BigDecimal.ZERO;
                if(complementPrice.compareTo(BigDecimal.ZERO) > 0){
                    ratioAmount = goodsSub.getGoodsPrice().divide(complementPrice, 16, RoundingMode.DOWN);
                }
                if (j == goodsSubs.size() - 1){
                    sellOrder.setExpectAmount(subExpectAmount);
                    sellOrder.setBalanceAmount(subGoodsBalance);
                    sellOrder.setActualAmount(subActualAmount);
                    sellOrder.setCouponRemitAmount(subRemitAmount);
                }else {
                    BigDecimal decimal = expectAmount.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                    sellOrder.setExpectAmount(decimal);
                    subExpectAmount = subExpectAmount.subtract(decimal);
                    BigDecimal balance = goodsBalance.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                    sellOrder.setBalanceAmount(balance);
                    subGoodsBalance = subGoodsBalance.subtract(balance);
                    BigDecimal actual = actualAmount.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                    sellOrder.setActualAmount(actual);
                    subActualAmount = subActualAmount.subtract(actual);
                    BigDecimal remit = remitAmount.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                    sellOrder.setCouponRemitAmount(remit);
                    subRemitAmount = subRemitAmount.subtract(remit);
                }
                sellOrder.setParentGoodsId(nowGoods.getId());
                sellOrder.setGoodsSkuId(goodsSub.getSku());
                sellOrder.setNum(1);
                sellOrder.setGoodsName(productDto.getName());
                sellOrder.setAccountType(accountType);
                sellOrder.setOrganSellCost(goodsSub.getGoodsPrice());
                sellOrder.setOrganId(order.getOrganId());
                sellOrder.setCooperationOrganId(order.getCooperationId());
                sellOrder.setEduTeacherId(musicGroup.getEducationalTeacherId());
                sellOrder.setTransNo(order.getTransNo());
                sellOrder.setOrderId(order.getId());
                sellOrder.setOrderNo(order.getOrderNo());
                sellOrder.setUserId(order.getUserId());
                sellOrder.setPaymentChannel(order.getPaymentChannel());
                sellOrder.setMerNo(order.getMerNos());
                sellOrder.setSellTime(order.getCreateTime());
                sellOrder.setTenantId(tenantId);
                if (nowGoods.getType().equals(GoodsType.INSTRUMENT)) {
                    sellOrder.setType(SellTypeEnum.INSTRUMENT);
                } else if (nowGoods.getType().equals(GoodsType.ACCESSORIES)) {
                    sellOrder.setType(SellTypeEnum.ACCESSORIES);
                } else {
                    sellOrder.setType(SellTypeEnum.OTHER);
                }
                sellOrderList.add(sellOrder);
            }
        }
        if (saveFlag && !org.springframework.util.CollectionUtils.isEmpty(sellOrderList)) {
            this.batchInsert(sellOrderList);
            this.syncSellOrder2Mall(sellOrderList,order,musicGroupId);
        }
        return sellOrderList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteByOrderNo(String orderNo) {
        sellOrderDao.deleteByOrderNo(orderNo);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdate(List<SellOrder> sellOrderList) {
        sellOrderDao.batchUpdate(sellOrderList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSellOrder(List<SellOrder> sellOrderList, StudentPaymentOrder studentPaymentOrder) {
        //获取总价
        BigDecimal totalCostPrice = sellOrderList.stream().map(e->e.getExpectAmount()).reduce(BigDecimal.ZERO, BigDecimal::add);
        //待分配金额
        BigDecimal waitAmount = studentPaymentOrder.getExpectAmount();
        for (int i = 0; i < sellOrderList.size(); i++) {
            SellOrder sellOrder = sellOrderList.get(i);
            //获取比例
            BigDecimal proportion = BigDecimal.ZERO;
            if (totalCostPrice.compareTo(BigDecimal.ZERO) > 0) {
                proportion = sellOrder.getExpectAmount().divide(totalCostPrice, 16, RoundingMode.DOWN);
            }
            if (i == sellOrderList.size() - 1) {
                sellOrder.setOrderAmount(waitAmount);
            } else {
                BigDecimal amount = studentPaymentOrder.getExpectAmount().multiply(proportion).setScale(2, RoundingMode.DOWN);
                waitAmount = waitAmount.subtract(amount);
                sellOrder.setOrderAmount(amount);
            }
            sellOrder.setActualAmount(sellOrder.getOrderAmount());
            sellOrder.setExpectAmount(sellOrder.getOrderAmount());
        }
        this.batchUpdate(sellOrderList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<SellOrder> initSellOrder(StudentPaymentOrder order, String musicGroupId, List<Integer> goodsIds,
                                         KitGroupPurchaseTypeEnum kitGroupPurchaseType,Boolean saveFlag) {
        if (CollectionUtils.isEmpty(goodsIds)) {
            return null;
        }
        BigDecimal totalBalance = order.getBalancePaymentAmount();
        if (totalBalance == null) {
            totalBalance = BigDecimal.ZERO;
        }
        BigDecimal expectTotalAmount = order.getExpectAmount();
        BigDecimal couponRemitAmount = order.getCouponRemitFee();
        BigDecimal actualTotalAmount = order.getActualAmount();
        return this.initSellOrder(order, musicGroupId, goodsIds, kitGroupPurchaseType, totalBalance, expectTotalAmount,
                couponRemitAmount, actualTotalAmount,saveFlag==null?true:saveFlag);
    }

    public Map<String, BigDecimal> calcSellAmount(StudentPaymentOrder order,BigDecimal cloudBalanceIncome,BigDecimal cloudIncome) {
        List<StudentPaymentOrderDetail> orderDetails = studentPaymentOrderDetailDao.getOrderDetail(order.getId());
        //总余额支付
        BigDecimal totalBalance = order.getBalancePaymentAmount() != null ? order.getBalancePaymentAmount() : BigDecimal.ZERO;
        //总价格
        BigDecimal totalPrice = order.getExpectAmount();
        //商品总付款
        BigDecimal detailTotalPrice = orderDetails.stream().map(StudentPaymentOrderDetail::getPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
        //商品销售占的余额
        BigDecimal detailTotalBalance = detailTotalPrice.multiply(totalBalance).divide(totalPrice, 2, RoundingMode.DOWN);

        if(detailTotalBalance.subtract(cloudBalanceIncome).compareTo(BigDecimal.ZERO) < 0) {
            detailTotalBalance = cloudBalanceIncome;
        }

        BigDecimal actualAmount = detailTotalPrice.multiply(order.getActualAmount()).divide(totalPrice, 2, RoundingMode.DOWN);

        if(actualAmount.subtract(cloudIncome).compareTo(BigDecimal.ZERO) < 0){
            actualAmount = cloudIncome;
        }
        Map<String, BigDecimal> sellAmount = new HashMap<>(2);
        //返回销售收入
        sellAmount.put("actualAmount", actualAmount.subtract(cloudIncome));
        sellAmount.put("balance", detailTotalBalance.subtract(cloudBalanceIncome));
        return sellAmount;
    }

}
