package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dao.StudentStatisticsDao;
import com.ym.mec.biz.dal.dto.StudentCourseConsumerDetailDto;
import com.ym.mec.biz.dal.dto.StudentCourseConsumerDto;
import com.ym.mec.biz.dal.dto.StudentCourseConsumerSumDto;
import com.ym.mec.biz.dal.page.StudentCourseConsumerQueryInfo;
import com.ym.mec.biz.service.StudentCourseConsumerService;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class StudentCourseConsumerServiceImpl implements StudentCourseConsumerService {

    @Autowired
    private StudentStatisticsDao studentStatisticsDao;
    @Autowired
    private MusicGroupDao musicGroupDao;


    @Override
    public List<StudentCourseConsumerDto> queryCourseConsumer(StudentCourseConsumerQueryInfo queryInfo) {
        return studentStatisticsDao.queryCourseConsumer(queryInfo);
    }

    @Override
    public PageInfo<StudentCourseConsumerDetailDto> queryCourseConsumerDetail(StudentCourseConsumerQueryInfo queryInfo) {
        PageInfo<StudentCourseConsumerDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentCourseConsumerDetailDto> dataList = null;
        int count = studentStatisticsDao.countCourseConsumerDetail(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentStatisticsDao.queryCourseConsumerDetail(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<StudentCourseConsumerDetailDto> findCourseConsumerDetail(Map<String, Object> params) {
        List<StudentCourseConsumerDetailDto> dataList = studentStatisticsDao.queryCourseConsumerDetail(params);
        if(CollectionUtils.isEmpty(dataList)){
            return dataList;
        }
        List<Integer> userIdList = dataList.stream().map(e -> e.getUserId()).collect(Collectors.toList());
        //获取用户关联的乐团主管map
        Map<Integer,String> eduMap = MapUtil.convertMybatisMap(musicGroupDao.queryUserEduMap(userIdList));
        //获取用户关联的合作单位的map
        Map<Integer,String> coopMap = MapUtil.convertMybatisMap(musicGroupDao.queryUserCoopMap(userIdList));
        for (StudentCourseConsumerDetailDto dto : dataList) {
            dto.setEduNames(eduMap.get(dto.getUserId()));
            dto.setCoopNames(coopMap.get(dto.getUserId()));
        }
        return dataList;
    }

    @Override
    public List<StudentCourseConsumerDto> queryPreCourseConsumer(StudentCourseConsumerQueryInfo queryInfo) {
        return studentStatisticsDao.queryPreCourseConsumer(queryInfo);
    }

    @Override
    public PageInfo<StudentCourseConsumerDetailDto> queryPreCourseConsumerDetail(StudentCourseConsumerQueryInfo queryInfo) {
        PageInfo<StudentCourseConsumerDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentCourseConsumerDetailDto> dataList = null;
        int count = studentStatisticsDao.countPreCourseConsumerDetail(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentStatisticsDao.queryPreCourseConsumerDetail(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<StudentCourseConsumerDetailDto> findPreCourseConsumerDetail(Map<String, Object> params) {
        List<StudentCourseConsumerDetailDto> dataList = studentStatisticsDao.queryPreCourseConsumerDetail(params);
        if(CollectionUtils.isEmpty(dataList)){
            return dataList;
        }
        List<Integer> userIdList = dataList.stream().map(e -> e.getUserId()).collect(Collectors.toList());
        //获取用户关联的乐团主管map
        Map<Integer,String> eduMap = MapUtil.convertMybatisMap(musicGroupDao.queryUserEduMap(userIdList));
        //获取用户关联的合作单位的map
        Map<Integer,String> coopMap = MapUtil.convertMybatisMap(musicGroupDao.queryUserCoopMap(userIdList));
        for (StudentCourseConsumerDetailDto dto : dataList) {
            dto.setEduNames(eduMap.get(dto.getUserId()));
            dto.setCoopNames(coopMap.get(dto.getUserId()));
        }
        return dataList;
    }

    @Override
    public List<StudentCourseConsumerSumDto> queryCourseConsumerSum(StudentCourseConsumerQueryInfo queryInfo) {
        List<StudentCourseConsumerSumDto> consumerDtos = studentStatisticsDao.queryCourseConsumerSum(queryInfo);
        if(CollectionUtils.isEmpty(consumerDtos)){
            return consumerDtos;
        }
        Date date = DateUtil.stringToDate(queryInfo.getStartDate(), DateUtil.ISO_YEAR_MONTH_FORMAT);
        Date date2 = DateUtil.stringToDate(queryInfo.getEndDate(), DateUtil.ISO_YEAR_MONTH_FORMAT);
        BigDecimal monthsBetween = new BigDecimal((DateUtil.monthsBetween(date, date2) + 1) * 4);
//#    （2）应有课耗：根据所选时间段计算，若所选时间段在一个自然月内则按在读人数*4，若超出一个月，该时间周期存在几个月就按在读人数*4*月数
//#    （3）实际课耗：所选时间段内所有学生课时数之和，注意：若一节1v2课程又两个学生，则统计为2节课
//#    （4）课耗达标率：实际课耗/应有课耗*100%
//#    （5）达标占比：4节+≥5节的人数之和/在读人数*100%
        BigDecimal bigDecimal = new BigDecimal(100);
        for (StudentCourseConsumerSumDto consumerDto : consumerDtos) {
            BigDecimal courseConsumerNum = consumerDto.getNormalNum().multiply(monthsBetween);
            if(courseConsumerNum.compareTo(BigDecimal.ZERO) != 0){
                consumerDto.setCourseConsumerRate(consumerDto.getCourseSum().divide(courseConsumerNum,2,BigDecimal.ROUND_DOWN).multiply(bigDecimal));
            }
            if(consumerDto.getNormalNum().compareTo(BigDecimal.ZERO) != 0){
                consumerDto.setStandRate(consumerDto.getNum6().divide(consumerDto.getNormalNum(),2,BigDecimal.ROUND_DOWN).multiply(bigDecimal));
            }
            consumerDto.setCourseConsumerNum(courseConsumerNum);
        }
        return consumerDtos;
    }
}