package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.CourseSchedule.CourseScheduleType;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.CourseHomeworkQueryInfo;
import com.ym.mec.biz.dal.page.StudentCourseHomeWorkQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.constant.CommonConstants;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class StudentCourseHomeworkServiceImpl extends BaseServiceImpl<Long, StudentCourseHomework> implements StudentCourseHomeworkService {

    @Resource
    private StudentCourseHomeworkDao studentCourseHomeworkDao;
    @Resource
    private CourseHomeworkService courseHomeworkService;
    @Resource
    private CourseHomeworkDao courseHomeworkDao;
    @Resource
    private CourseScheduleDao courseScheduleDao;
    @Resource
    private ClassGroupDao classGroupDao;
    @Resource
    private SysUserFeignService sysUserFeignService;
    @Resource
    private SysMessageService sysMessageService;
    @Resource
    private CourseScheduleReviewDao courseScheduleReviewDao;
    @Resource
    private TeacherDao teacherDao;
    @Resource
    private StudentDao studentDao;
    @Resource
    private SubjectDao subjectDao;
    @Resource
    private StudentServeService studentServeService;
    @Resource
    private StudentRegistrationDao studentRegistrationDao;
    @Resource
    private VipGroupDao vipGroupDao;
    @Resource
    private PracticeGroupDao practiceGroupDao;
    @Resource
    private ExtracurricularExercisesReplyDao extracurricularExercisesReplyDao;
    @Resource
    private SysConfigDao sysConfigDao;
    @Resource
    private SysMusicScoreDao sysMusicScoreDao;
    @Resource
    private MusicGroupTrainPlanService musicGroupTrainPlanService;
    @Resource
    private StudentExtracurricularExercisesSituationDao studentExtracurricularExercisesSituationDao;
    @Resource
    private ExtracurricularExercisesReplyService extracurricularExercisesReplyService;
    @Resource
    private MusicGroupService musicGroupService;
    @Resource
    private StudentLessonExaminationDao studentLessonExaminationDao;
    @Resource
    private StudentLessonTrainingDetailService studentLessonTrainingDetailService;
    @Resource
    private ExtracurricularExercisesDao extracurricularExercisesDao;

    @Override
    public BaseDAO<Long, StudentCourseHomework> getDAO() {
        return studentCourseHomeworkDao;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public long insert(StudentCourseHomework bean) {
        SysUser user = sysUserFeignService.queryUserInfo();
        bean.setUserId(user.getId());
        bean.setStatus(YesOrNoEnum.YES);
        long insert = super.insert(bean);

        CourseSchedule courseSchedule = courseScheduleDao.get(bean.getCourseScheduleId());

        CourseHomework courseHomework = new CourseHomework();
        courseHomework.setId(bean.getCourseHomeworkId());
        courseHomework.setGroupType(courseSchedule.getGroupType());
//        courseHomework.setCompletedNum(studentCourseHomeworkDao.countCompletedStudentNum(bean.getCourseHomeworkId()));

        courseHomeworkService.update(courseHomework);
        return insert;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void submitHomework(StudentCourseHomework bean) {
        if (Objects.isNull(bean.getAttachments()) || StringUtils.isBlank(bean.getAttachments())) {
            throw new BizException("请选择作业");
        }
        StudentCourseHomework existHomework = studentCourseHomeworkDao.get(bean.getId());
        if (Objects.isNull(bean) || Objects.isNull(bean.getId()) || Objects.isNull(existHomework)) {
            throw new BizException("作业不存在");
        }
        Date now = new Date();

        if (DateUtil.daysBetween(existHomework.getCreateTime(), now) > 7) {
            throw new BizException("该作业已超出可提交时间范围，无法提交作业。");
        }
        CourseSchedule courseSchedule = courseScheduleDao.get(existHomework.getCourseScheduleId());
        //若本周有当前课程所在乐团的服务指标，那么在课程开始当天之后不能再提交作业
        Date classDate = studentExtracurricularExercisesSituationDao.getHomeworkSituations(courseSchedule.getId(),courseSchedule.getMusicGroupId(),"HOMEWORK",
                        DateUtil.getWeekMondayWithDate(now),existHomework.getUserId());
        if(Objects.nonNull(classDate)){
            int i = classDate.compareTo(DateUtil.stringToDate(DateUtil.format(now, DateUtil.ISO_EXPANDED_DATE_FORMAT), DateUtil.ISO_EXPANDED_DATE_FORMAT));
            if(i <= 0){
                throw new BizException("已超过提交作业期限。");
            }
        }
        boolean push = true;
        bean.setSubmitTime(now);
        super.update(bean);
        bean = studentCourseHomeworkDao.get(bean.getId());

        CourseHomework courseHomework = courseHomeworkDao.get(bean.getCourseHomeworkId());
        courseHomework.setGroupType(courseSchedule.getGroupType());
//        courseHomework.setCompletedNum(studentCourseHomeworkDao.countCompletedStudentNum(bean.getCourseHomeworkId()));
        courseHomeworkService.update(courseHomework);
        studentServeService.updateExercisesSituation(courseSchedule.getMusicGroupId(), courseSchedule.getClassDate(), new ArrayList<>(Arrays.asList(bean.getUserId())), courseSchedule.getTeacherId());

        Integer userId = bean.getUserId();
        if (courseSchedule.getType().equals(CourseScheduleType.VIP)) {
            userId = null;
        }

        CourseScheduleReview review = courseScheduleReviewDao.findByCourseScheduleAndStudent(bean.getCourseScheduleId(), userId);
        if (Objects.nonNull(review)) {
            review.setHandHomework(YesOrNoEnum.YES.getCode());
            courseScheduleReviewDao.update(review);
        }

        if (push) {
            Map<Integer, String> userMap = new HashMap<>();
            userMap.put(courseSchedule.getActualTeacherId(), courseSchedule.getActualTeacherId().toString());
            SysUser user = teacherDao.getUser(bean.getUserId());
            String notifyUrl = "9?courseScheduleID=" + bean.getId() + "&userId=" + bean.getUserId() + "&studentCourseHomeworkId=" + bean.getId() + "&extra=0";
            String extra = "dayaedu?courseScheduleID=" + courseSchedule.getId() + "&userId=" + bean.getUserId() + "&studentCourseHomeworkId=" + bean.getId() + "&extra=0";
            String courseDate = DateUtil.dateToString(courseSchedule.getClassDate(), "yyyy年MM月dd日");
            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_HOMEWORK_SUBMIT,
                    userMap, null, 0, notifyUrl, "TEACHER", courseSchedule.getName(), courseDate, user.getUsername());
            MusicGroup musicGroup = musicGroupService.get(courseSchedule.getMusicGroupId());

            sysMessageService.batchSendImMessage(MessageTypeEnum.IM_HOMEWORK_SUBMIT_PUSH, bean.getUserId().toString(), extra,
                    new String[]{courseSchedule.getActualTeacherId().toString()},
                    null, courseDate, courseSchedule.getName());
            if(musicGroup != null && musicGroup.getHomeworkPushFlag() == 1){
                Date date = new Date();
                if (date.before(courseHomework.getExpiryDate())) {
                    ClassGroup classGroup = classGroupDao.findByCourseSchedule(existHomework.getCourseScheduleId().intValue(), 0);
                    if (classGroup != null) {
                        SysUser sysUser = sysUserFeignService.queryUserById(existHomework.getUserId());
                        sysMessageService.batchSendImGroupMessage(MessageTypeEnum.IM_HOMEWORK_SUBMIT_PUSH_GROUP, existHomework.getUserId().toString(), null,
                                new String[]{classGroup.getId().toString()},
                                null, sysUser.getUsername());
                    }
                }
            }
        }
    }

    @Override
    public CourseHomeworkStudentDetailDto findCourseHomeworkStudentDetail(Long courseScheduleID, Long userId) {
        if (Objects.isNull(courseScheduleID)) {
            throw new BizException("作业不存在");
        }
        CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleID);
        
        if(courseSchedule == null){
            throw new BizException("课程不存在");
        }

        MusicGroupTrainPlan musicGroupTrainPlan = null;
        //获取课程规划
        MusicGroupTrainPlanSaveDto schoolTerm = musicGroupTrainPlanService.getSchoolTerm(courseSchedule);
        int courseNumNo = courseScheduleDao.getCourseNumNo(schoolTerm);
        if(courseNumNo > 0){
            List<MusicGroupTrainPlan> planList = musicGroupTrainPlanService.queryPlan(schoolTerm);
            if(planList.size() >= courseNumNo){
                musicGroupTrainPlan = planList.get(courseNumNo - 1);
            }
        }

        CourseHomeworkStudentDetailDto courseHomeworkStudentDetail = studentCourseHomeworkDao.findCourseHomeworkStudentDetail(courseScheduleID, userId);
        if(courseHomeworkStudentDetail == null){
            courseHomeworkStudentDetail = new CourseHomeworkStudentDetailDto();
            courseHomeworkStudentDetail.setMusicGroupTrainPlan(musicGroupTrainPlan);
            return courseHomeworkStudentDetail;
        }
        courseHomeworkStudentDetail.setExpire(DateUtil.isOverdue(courseHomeworkStudentDetail.getSubmitTime(),
                sysConfigDao.findConfigValue("file_expiration_time")));
        courseHomeworkStudentDetail.setMusicGroupTrainPlan(musicGroupTrainPlan);
        
        SysUser sysUser = teacherDao.getUser(userId.intValue());
        courseHomeworkStudentDetail.setStudentId(userId.intValue());
        courseHomeworkStudentDetail.setStudentName(sysUser.getUsername());
        if (Objects.nonNull(courseSchedule)) {
            SysUser teacher = teacherDao.getUser(courseSchedule.getActualTeacherId());
            courseHomeworkStudentDetail.setTeacherId(courseSchedule.getActualTeacherId());
            courseHomeworkStudentDetail.setTeacherName(teacher.getRealName());
        }
        courseHomeworkStudentDetail.setType("HOMEWORK");
        String configValue = sysConfigDao.findConfigValue(SysConfigService.HOMEWORK_OPEN_FLAG);
        if(StringUtils.isEmpty(configValue)){
            configValue = "0";
        }
        courseHomeworkStudentDetail.setHomeworkOpenFlag(Integer.parseInt(configValue));
        if(StringUtils.isNotEmpty(courseHomeworkStudentDetail.getMusicScoreId())){
            List<SysMusicScore> sysMusicScoreList = sysMusicScoreDao.findByIds(courseHomeworkStudentDetail.getMusicScoreId());
            if(CollectionUtils.isNotEmpty(sysMusicScoreList)){
                courseHomeworkStudentDetail.setSysMusicScoreList(sysMusicScoreList);
                SysMusicScore sysMusicScore = sysMusicScoreList.get(0);
                courseHomeworkStudentDetail.setSysMusicScore(sysMusicScore);
                courseHomeworkStudentDetail.setMusicScoreId(sysMusicScore.getId().toString());
                courseHomeworkStudentDetail.setMusicScoreName(sysMusicScore.getName());
            }
        }
        Student student = studentDao.get(userId.intValue());
        courseHomeworkStudentDetail.setHasMember(student.getMemberRankSettingId()==null?0:1);

        Date now = new Date();

        if (DateUtil.daysBetween(courseHomeworkStudentDetail.getCreateTime(), now) > 7) {
            courseHomeworkStudentDetail.setSubmitFlag(false);
        }
        if(courseHomeworkStudentDetail.getSubmitFlag()){
            //若本周有当前课程所在乐团的服务指标，那么在课程开始当天之后不能再提交作业
            Date classDate = studentExtracurricularExercisesSituationDao.getHomeworkSituations(courseSchedule.getId(),courseSchedule.getMusicGroupId(),"HOMEWORK",
                    DateUtil.getWeekMondayWithDate(now),userId.intValue());
            if(Objects.nonNull(classDate)){
                int i = classDate.compareTo(DateUtil.stringToDate(DateUtil.format(now, DateUtil.ISO_EXPANDED_DATE_FORMAT), DateUtil.ISO_EXPANDED_DATE_FORMAT));
                if(i <= 0){
                    courseHomeworkStudentDetail.setSubmitFlag(false);
                }
            }
        }
        return courseHomeworkStudentDetail;
    }

    @Override
    public boolean updateReplyStatus(Integer teacherId, Long courseHomeworkId, Integer studentId) {
        StudentCourseHomework byStudentAndCourseHomewok = studentCourseHomeworkDao.findByStudentAndCourseHomewok(courseHomeworkId, studentId);
        if (Objects.isNull(byStudentAndCourseHomewok)) {
            throw new BizException("作业记录不存在");
        }

        int hours = -1;
        if (Objects.nonNull(byStudentAndCourseHomewok.getSubmitTime())) {
            hours = DateUtil.hoursBetween(byStudentAndCourseHomewok.getSubmitTime(), new Date());
        }
        if (byStudentAndCourseHomewok.getIsReplied() == YesOrNoEnum.YES && Integer.valueOf(1).equals(byStudentAndCourseHomewok.getIsRepliedTimely())) {
            return true;
        }
        if (hours >= 0 && hours <= 24) {
            byStudentAndCourseHomewok.setIsRepliedTimely(1);
        }
        CourseHomework byTeacherAndCourseHomewok = courseHomeworkDao.findByTeacherAndCourseHomewok(courseHomeworkId, teacherId);
        if (Objects.nonNull(byTeacherAndCourseHomewok)) {
            byStudentAndCourseHomewok.setIsReplied(YesOrNoEnum.YES);
            studentCourseHomeworkDao.update(byStudentAndCourseHomewok);

            CourseSchedule courseSchedule = courseScheduleDao.get(byStudentAndCourseHomewok.getCourseScheduleId());

            LocalDate nowDate = LocalDateTime.now(DateUtil.zoneId).toLocalDate();
            LocalDate monDayDate = nowDate.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue());
            LocalDate createDateTime = LocalDateTime.ofInstant(courseSchedule.getClassDate().toInstant(), DateUtil.zoneId).toLocalDate();
//            LocalDate createMonday = createDateTime.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue());

            LocalDate homeworkCreateDateTime = LocalDateTime.ofInstant(byStudentAndCourseHomewok.getCreateTime().toInstant(), DateUtil.zoneId).toLocalDate();
            boolean isOk = false;
            switch (createDateTime.getDayOfWeek()) {
                case SATURDAY:
                    if (createDateTime.until(homeworkCreateDateTime, ChronoUnit.DAYS) < 4) {
                        isOk = true;
                    }
                    break;
                case SUNDAY:
                    if (createDateTime.until(homeworkCreateDateTime, ChronoUnit.DAYS) < 3) {
                        isOk = true;
                    }
                    break;
                default:
                    if (createDateTime.get(DateUtil.weekFields.weekOfYear()) == homeworkCreateDateTime.get(DateUtil.weekFields.weekOfYear())) {
                        isOk = true;
                    }
                    break;
            }
            if (createDateTime.isBefore(monDayDate) && isOk) {
                studentServeService.updateExercisesSituation(byTeacherAndCourseHomewok.getMusicGroupId(), new Date(), new ArrayList<>(Arrays.asList(byStudentAndCourseHomewok.getUserId())), teacherId);
            }
        }
        return true;
    }

    @Override
    public PageInfo<StudentCourseHomework> queryPage(QueryInfo queryInfo) {
        SysUser user = sysUserFeignService.queryUserInfo();
        CourseHomeworkQueryInfo courseHomeworkQueryInfo = (CourseHomeworkQueryInfo) queryInfo;
        courseHomeworkQueryInfo.setUserId(Long.valueOf(user.getId()));
        return super.queryPage(queryInfo);
    }

    @Override
    public List<StudentCourseHomework> findStudentCourseHomeworkByCourse(Long courseSchedule, Long homeworkId, String userName) {
        return studentCourseHomeworkDao.findByCourseSchedule(courseSchedule, homeworkId, userName);
    }

    @Override
    public void homeworkRemind() {
        // 课后作业通知
        List<StudentCourseHomework> list = studentCourseHomeworkDao.queryNotDoneHomeworkList();
        StringBuffer sb;
        for (StudentCourseHomework sch : list) {
            sb = new StringBuffer("homeworkId=").append(sch.getCourseScheduleId()).append("&homeworkType=HOMEWORK");
            Map<Integer, String> receivers = new HashMap<Integer, String>(1);
            receivers.put(sch.getUserId(), sch.getUserId() + "");
            sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_JOB_MESSAGE_V2, receivers, null, 0, "homework?" + sb, "STUDENT");
        }

        // 课外作业通知
        List<ExtracurricularExercisesReply> extraList =  extracurricularExercisesReplyDao.queryNotDoneHomeworkList();
        for (ExtracurricularExercisesReply reply : extraList) {
            sb = new StringBuffer("homeworkId=").append(reply.getExtracurricularExercisesId())
                    .append("&homeworkType=")
                    .append(reply.getExtracurricularExercises().getGroupType());
            Map<Integer, String> receivers = new HashMap<Integer, String>(1);
            receivers.put(reply.getUserId(), reply.getUserId() + "");
            sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_JOB_MESSAGE_EXTRA_V2, receivers, null, 0, "homework?" + sb, "STUDENT");
        }

        //获取即将截止的测评
        String date = DateUtil.dateToString(DateUtil.addDays(new Date(), 1),DateUtil.ISO_EXPANDED_DATE_FORMAT);
        List<StudentLessonExamination> examinations = studentLessonExaminationDao.queryEndList(date);
        if(!CollectionUtils.isEmpty(examinations)){
            Map<Long, List<StudentLessonExamination>> listMap = examinations.stream().collect(Collectors.groupingBy(e -> e.getLessonExaminationId()));
            for (Long lessonExaminationId : listMap.keySet()) {
                List<StudentLessonExamination> studentLessonExaminations = listMap.get(lessonExaminationId);
                Map<Integer, String> receivers = studentLessonExaminations.stream().collect(Collectors.toMap(e -> e.getUserId(), e -> e.getUserId().toString()));
                sb = new StringBuffer("evaluating?evaluateId=").append(lessonExaminationId);
                sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.PUSH_NO_SUBMIT_LESSON_EXAMINATION, receivers, null, 0, sb.toString(), "STUDENT");
            }
        }

    }

    @Override
    public PageInfo<StudentCourseHomeworkDto> findStudentCourseHomeworks(QueryInfo queryInfo) {
        PageInfo<StudentCourseHomeworkDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentCourseHomeworkDto> dataList = null;
        int count = studentCourseHomeworkDao.countStudentCourseHomeworks(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentCourseHomeworkDao.findStudentCourseHomeworks(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<StudentHomeworkRecordDto> findStudentHomeworkRecord(CourseHomeworkQueryInfo queryInfo) {
        PageInfo<StudentHomeworkRecordDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentHomeworkRecordDto> dataList = null;
        int count = studentCourseHomeworkDao.countStudentHomeworkRecord(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentCourseHomeworkDao.findStudentHomeworkRecord(params);
            if (!CollectionUtils.isEmpty(dataList)) {
                List<Integer> teacherIds = dataList.stream().map(StudentHomeworkRecordDto::getTeacherId).collect(Collectors.toList());
                List<SimpleUserDto> teachers = teacherDao.findSimpleTeachers(teacherIds, queryInfo.getTenantId());
                Map<Integer, String> idNameMap = teachers.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, u -> u.getUserName()));
                for (StudentHomeworkRecordDto studentHomeworkRecordDto : dataList) {
                    studentHomeworkRecordDto.setTeacherName(idNameMap.get(studentHomeworkRecordDto.getTeacherId()));
                }
            }
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }

        // 构建返回数据

        // 乐团信息

        // 乐团信息
        Map<String, MusicGroup> musicGroupMap = new HashMap<>();
        List<String> musicGroupIds = dataList.stream().map(e -> e.getMusicGroupId()).collect(Collectors.toList());

        if (!CollectionUtils.isEmpty(musicGroupIds)) {
            List<MusicGroup> musicGroups = musicGroupService.queryListByIds(new HashSet<>(musicGroupIds));
            musicGroupMap = musicGroups.stream().collect(Collectors.toMap(MusicGroup::getId, Function.identity()));
        }


        // 班级名称

        List<Integer> classIds = dataList.stream().map(e -> e.getClassGroupId()).collect(Collectors.toList());
        Map<Integer, ClassGroup> classGroupMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(classIds)) {

            List<ClassGroup> classGroups = classGroupDao.queryByIds(classIds);
            classGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, Function.identity()));
        }


        for (StudentHomeworkRecordDto studentHomeworkRecordDto : dataList) {
            // 乐团信息
            MusicGroup musicGroup = musicGroupMap.get(studentHomeworkRecordDto.getMusicGroupId());
            if (Objects.nonNull(musicGroup)) {
                studentHomeworkRecordDto.setMusicGroupName(musicGroup.getName());
                studentHomeworkRecordDto.setMusicGroupImg(musicGroup.getImg());
            }

            // 班级信息
            ClassGroup classGroup = classGroupMap.get(studentHomeworkRecordDto.getClassGroupId());
            if (Objects.nonNull(classGroup)) {
                studentHomeworkRecordDto.setClassGroupName(classGroup.getName());
            }

            // 设置完成状态
            if (studentHomeworkRecordDto.getStandardFlag() == null) {
                studentHomeworkRecordDto.setFinishStatus(false);
            } else
            if (studentHomeworkRecordDto.getStandardFlag().equals(StandardEnum.STANDARD)
                || studentHomeworkRecordDto.getStandardFlag().equals(StandardEnum.EXCELLENT)){
                studentHomeworkRecordDto.setFinishStatus(true);
            } else {
                studentHomeworkRecordDto.setFinishStatus(false);
            }
            studentHomeworkRecordDto.setType(ELessonTrainingType.HOMEWORK);
        }

        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public void add(StudentCourseHomework studentCourseHomework) {
        studentCourseHomework.setStatus(YesOrNoEnum.YES);
        studentCourseHomeworkDao.insert(studentCourseHomework);
        CourseHomework courseHomework = courseHomeworkDao.get(studentCourseHomework.getCourseHomeworkId());
        Date date = new Date();
        if (date.before(courseHomework.getExpiryDate())) {
            ClassGroup classGroup = classGroupDao.findByCourseSchedule(studentCourseHomework.getCourseScheduleId().intValue(), 0);
            if (classGroup != null) {
                SysUser sysUser = sysUserFeignService.queryUserById(studentCourseHomework.getUserId());
                sysMessageService.batchSendImGroupMessage(MessageTypeEnum.IM_HOMEWORK_SUBMIT_PUSH_GROUP, studentCourseHomework.getUserId().toString(), null,
                        new String[]{classGroup.getId().toString()},
                        null, sysUser.getUsername());
            }
        }
    }

    @Override
    public Map<String, Object> getCourseHomeworkDetail(CourseHomeworkQueryInfo queryInfo) {
        Map<String, Object> result = new HashMap<>();
        CourseHomework courseHomework = courseHomeworkDao.findByCourseSchedule(queryInfo.getCourseScheduleId());

        if (Objects.isNull(courseHomework)) {
            result.put("countInfo", null);
            return result;
        }

        PageInfo<StudentCourseHomeworkDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentCourseHomework> dataList = new ArrayList<>();
        List<StudentCourseHomeworkDto> studentCourseHomeworkDtos = new ArrayList<>();
        int count = studentCourseHomeworkDao.countAll(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentCourseHomeworkDao.queryAll(params);
            Set<Integer> studentIds = dataList.stream().map(StudentCourseHomework::getUserId).collect(Collectors.toSet());
            List<SimpleUserDto> usersSimpleInfo = teacherDao.getUsersSimpleInfo(new ArrayList<>(studentIds));
            List<Student> students = studentDao.findByStudentIds(new ArrayList<>(studentIds));
            Set<Integer> subjectIds = new HashSet<>();
            for (Student student : students) {
                if (StringUtils.isBlank(student.getSubjectIdList())) {
                    continue;
                }
                Set<Integer> studentSubjectIds = Arrays.stream(student.getSubjectIdList().split(",")).mapToInt(Integer::valueOf).boxed().collect(Collectors.toSet());
                subjectIds.addAll(studentSubjectIds);
            }
            List<Subject> subjects = new ArrayList<>();
            if (!CollectionUtils.isEmpty(subjectIds)) {
                subjects = subjectDao.findBySubjectIds(new ArrayList<>(subjectIds));
            }
            Map<Integer, SimpleUserDto> studentInfoMap = usersSimpleInfo.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, s -> s, (s1, s2) -> s1));
            Map<Integer, Student> studentMap = students.stream().collect(Collectors.toMap(Student::getUserId, s -> s, (s1, s2) -> s1));
            for (StudentCourseHomework studentCourseHomework : dataList) {
                StudentCourseHomeworkDto s = new StudentCourseHomeworkDto();
                s.setStudentId(studentCourseHomework.getUserId());
                s.setStudentName(studentInfoMap.containsKey(s.getStudentId()) ? studentInfoMap.get(s.getStudentId()).getNickName() : "");
                s.setPhone(studentInfoMap.containsKey(s.getStudentId()) ? studentInfoMap.get(s.getStudentId()).getPhone() : "");
                s.setSubjectIds(studentMap.containsKey(s.getStudentId()) ? studentMap.get(s.getStudentId()).getSubjectIdList() : "");
                if (StringUtils.isNotBlank(s.getSubjectIds())) {
                    Set<Integer> studentSubjectIds = Arrays.stream(s.getSubjectIds().split(",")).mapToInt(Integer::valueOf).boxed().collect(Collectors.toSet());
                    List<String> subjectNames = subjects.stream().filter(sj -> studentSubjectIds.contains(sj.getId())).map(Subject::getName).collect(Collectors.toList());
                    s.setSubjectNames(StringUtils.join(subjectNames, ","));
                }
                s.setSubmitTime(studentCourseHomework.getSubmitTime());
                s.setIsView(studentCourseHomework.getIsView().getCode());
                s.setIsReplied(studentCourseHomework.getIsReplied().getCode());
                s.setAttachments(studentCourseHomework.getAttachments());
                studentCourseHomeworkDtos.add(s);
            }
        }
        pageInfo.setRows(studentCourseHomeworkDtos);

        if (queryInfo.getPage() == 1) {
            params.put("isReplied", 1);
            int repliedNum = studentCourseHomeworkDao.countAll(params);
            CourseHomeworkDto courseHomeworkDto = new CourseHomeworkDto();
            courseHomeworkDto.setExpectNum(courseHomework.getExpectNum());
            courseHomeworkDto.setCompletedNum(courseHomework.getCompletedNum());
            courseHomeworkDto.setRepliedNum(repliedNum);
            courseHomeworkDto.setRepliedNum(repliedNum);
            result.put("countInfo", courseHomeworkDto);
        }

        result.put("pageInfo", pageInfo);

        return result;
    }

    @Override
    public PageInfo<StudentVisitCourseHomeWorkDto> getStudentHomeWorks(StudentCourseHomeWorkQueryInfo queryInfo) {
        PageInfo<StudentVisitCourseHomeWorkDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<StudentVisitCourseHomeWorkDto> dataList = new ArrayList<>();
        int count = studentCourseHomeworkDao.getStudentCourseHomeWorksCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentCourseHomeworkDao.getStudentCourseHomeWorks(params);

            List<String> musicGroupIds = dataList.stream().filter(e -> e.getGroupType()!=null).filter(e -> e.getGroupType().equals(GroupType.MUSIC)).map(StudentVisitCourseHomeWorkDto::getGroupId).distinct().collect(Collectors.toList());
            List<Long> vipGroupIds = dataList.stream().filter(e -> e.getGroupType()!=null).filter(e -> e.getGroupType().equals(GroupType.VIP)).map(e -> Long.parseLong(e.getGroupId())).distinct().collect(Collectors.toList());
            List<Long> practiceGroupIds = dataList.stream().filter(e -> e.getGroupType()!=null).filter(e -> e.getGroupType().equals(GroupType.PRACTICE)).map(e -> Long.parseLong(e.getGroupId())).distinct().collect(Collectors.toList());

            Map<String, List<MusicGroup>> musicGroupMap = new HashMap<>();
            Map<Long, List<VipGroup>> vipGroupMap = new HashMap<>();
            Map<Long, List<PracticeGroup>> practiceGroupMap = new HashMap<>();
            if (musicGroupIds.size() > 0) {
                musicGroupMap = musicGroupService.getDao().getMusicGroupByIds(musicGroupIds).stream().collect(Collectors.groupingBy(MusicGroup::getId));
            }
            if (vipGroupIds.size() > 0) {
                vipGroupMap = vipGroupDao.getVipGroupByIds(vipGroupIds).stream().collect(Collectors.groupingBy(VipGroup::getId));
            }
            if (practiceGroupIds.size() > 0) {
                practiceGroupMap = practiceGroupDao.getWithIds(practiceGroupIds).stream().collect(Collectors.groupingBy(PracticeGroup::getId));
            }
            for (StudentVisitCourseHomeWorkDto homeWork : dataList) {
                if (GroupType.MUSIC.equals(homeWork.getGroupType()) && musicGroupMap.containsKey(homeWork.getGroupId())) {
                    homeWork.setGroupName(musicGroupMap.get(homeWork.getGroupId()).get(0).getName());
                }
                if (GroupType.VIP.equals(homeWork.getGroupType()) && vipGroupMap.containsKey(Long.parseLong(homeWork.getGroupId()))) {
                    homeWork.setGroupName(vipGroupMap.get(Long.parseLong(homeWork.getGroupId())).get(0).getName());
                }
                if (GroupType.PRACTICE.equals(homeWork.getGroupType()) && practiceGroupMap.containsKey(Long.parseLong(homeWork.getGroupId()))) {
                    homeWork.setGroupName(practiceGroupMap.get(Long.parseLong(homeWork.getGroupId())).get(0).getName());
                }
            }
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<EduHomeworkCardDto> getEduCourseHomeworkList(StudentCourseHomeWorkQueryInfo queryInfo) {
        PageInfo<EduHomeworkCardDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        int count = studentCourseHomeworkDao.countEduCourseHomeworkList(params);
        pageInfo.setTotal(count);
        params.put("offset", pageInfo.getOffset());
        List<EduHomeworkCardDto> dataList = studentCourseHomeworkDao.getEduCourseHomeworkList(params);

        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<EduHomeworkStatDto> queryEduHomeworkStatList(StudentCourseHomeWorkQueryInfo queryInfo) {
        PageInfo<EduHomeworkStatDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        params.put("musicGroupStatus", MusicGroupStatusEnum.PROGRESS.getCode());

        int count = musicGroupService.getDao().queryCount(params);
        pageInfo.setTotal(count);
        params.put("offset", pageInfo.getOffset());
//        StudentService
        List<MusicGroup> musicGroups = musicGroupService.getDao().queryPage(params);

        List<EduHomeworkStatDto> dataList = new ArrayList<>();
        if(!CollectionUtils.isEmpty(musicGroups)){
            List<String> groupIds = musicGroups.stream().map(MusicGroup::getId).collect(Collectors.toList());
            List<StudentRegistration> studentRegistrations = studentRegistrationDao.findByMusicGroupIds(groupIds, StudentMusicGroupStatusEnum.NORMAL);
            Map<String, Long> groupStudentNumMap = studentRegistrations.stream().collect(Collectors.groupingBy(StudentRegistration::getMusicGroupId, Collectors.counting()));

            BigDecimal oneHundred = new BigDecimal(100);
            BigDecimal zero = new BigDecimal("0.00");

            List<EduHomeworkCardDto> courseHomeworkStudentNumList = studentCourseHomeworkDao.countHomeworkStudentInfoWithMusicGroup(groupIds, queryInfo.getStartTime(), queryInfo.getEndTime());
            List<EduHomeworkCardDto> homeworkStudentNumList = extracurricularExercisesReplyDao.countEduHomeworkStudentNumWithMusicGroup(groupIds, queryInfo.getStartTime(), queryInfo.getEndTime(), queryInfo.getTenantId());
            homeworkStudentNumList.addAll(courseHomeworkStudentNumList);
            Map<String, List<EduHomeworkCardDto>> groupStudentListMap = homeworkStudentNumList.stream().collect(Collectors.groupingBy(EduHomeworkCardDto::getGroupId));
            for (MusicGroup mg : musicGroups) {
                EduHomeworkStatDto ehs = new EduHomeworkStatDto();
                ehs.setGroupId(mg.getId());
                ehs.setGroupName(mg.getName());
                if(groupStudentNumMap.containsKey(mg.getId())){
                    ehs.setGroupStudentNum(Math.toIntExact(groupStudentNumMap.get(mg.getId())));
                }else{
                    ehs.setGroupStudentNum(0);
                }
                int exceptStudentNum = 0;
                int submitStudentNum = 0;
                if(groupStudentListMap.containsKey(mg.getId())){
                    List<EduHomeworkCardDto> eduHomeworkCardDtos = groupStudentListMap.get(mg.getId());
                    for (EduHomeworkCardDto eduHomeworkCardDto : eduHomeworkCardDtos) {
                        exceptStudentNum += eduHomeworkCardDto.getExceptStudentNum();
                        submitStudentNum += eduHomeworkCardDto.getSubmitStudentNum();
                    }
                }
                ehs.setExceptStudentNum(exceptStudentNum);
                ehs.setSubmitStudentNum(submitStudentNum);
                if(exceptStudentNum<=0){
                    ehs.setSubmitRate(zero);
                }else{
                    ehs.setSubmitRate(new BigDecimal(submitStudentNum).divide(new BigDecimal(exceptStudentNum), CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_HALF_UP).multiply(oneHundred));
                }
                dataList.add(ehs);
            }
        }

        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<EduHomeworkStatDto> queryEduHomeworkStatDetail(StudentCourseHomeWorkQueryInfo queryInfo) {
        List<EduHomeworkStatDto> dataList = new ArrayList<>();
        MusicGroup musicGroup = musicGroupService.get(queryInfo.getGroupId());
        if(Objects.isNull(musicGroup)){
            return dataList;
        }
        List<StudentRegistration> studentRegistrations = studentRegistrationDao.findByMusicGroupIds(Arrays.asList(queryInfo.getGroupId()), StudentMusicGroupStatusEnum.NORMAL);
        Map<Integer, Long> subjectStudentNumMap = studentRegistrations.stream().collect(Collectors.groupingBy(StudentRegistration::getActualSubjectId, Collectors.counting()));

        Map<Integer, Subject> idSubjectMap = new HashMap<>();
        if(!subjectStudentNumMap.isEmpty()){
            List<Subject> subjects = subjectDao.findBySubjectIds(new ArrayList<>(subjectStudentNumMap.keySet()));
            idSubjectMap = subjects.stream().collect(Collectors.toMap(Subject::getId, s -> s, (s1, s2) -> s1));
        }

        BigDecimal oneHundred = new BigDecimal(100);
        BigDecimal zero = new BigDecimal("0.00");

        List<EduHomeworkCardDto> courseHomeworkStudentNumList = studentCourseHomeworkDao.countHomeworkSubjectStudentInfoWithMusicGroup(queryInfo.getGroupId(), queryInfo.getStartTime(), queryInfo.getEndTime());
        List<EduHomeworkCardDto> homeworkStudentNumList = extracurricularExercisesReplyDao.countEduHomeworkSubjectStudentNumWithMusicGroup(queryInfo.getGroupId(), queryInfo.getStartTime(), queryInfo.getEndTime());
        homeworkStudentNumList.addAll(courseHomeworkStudentNumList);
        Map<Integer, List<EduHomeworkCardDto>> subjectStudentListMap = homeworkStudentNumList.stream().collect(Collectors.groupingBy(EduHomeworkCardDto::getSubjectId));
        for (Map.Entry<Integer, Long> subjectStudentNumEntry : subjectStudentNumMap.entrySet()) {
            EduHomeworkStatDto ehs = new EduHomeworkStatDto();
            if(idSubjectMap.containsKey(subjectStudentNumEntry.getKey())){
                ehs.setSubjectName(idSubjectMap.get(subjectStudentNumEntry.getKey()).getName());
            }
            ehs.setGroupId(queryInfo.getGroupId());
            ehs.setGroupName(musicGroup.getName());
            if(subjectStudentNumMap.containsKey(subjectStudentNumEntry.getKey())){
                ehs.setGroupStudentNum(Math.toIntExact(subjectStudentNumMap.get(subjectStudentNumEntry.getKey())));
            }else{
                ehs.setGroupStudentNum(0);
            }
            int exceptStudentNum = 0;
            int submitStudentNum = 0;
            if(subjectStudentListMap.containsKey(subjectStudentNumEntry.getKey())){
                List<EduHomeworkCardDto> eduHomeworkCardDtos = subjectStudentListMap.get(subjectStudentNumEntry.getKey());
                for (EduHomeworkCardDto eduHomeworkCardDto : eduHomeworkCardDtos) {
                    exceptStudentNum += eduHomeworkCardDto.getExceptStudentNum();
                    submitStudentNum += eduHomeworkCardDto.getSubmitStudentNum();
                }
            }
            ehs.setExceptStudentNum(exceptStudentNum);
            ehs.setSubmitStudentNum(submitStudentNum);
            if(exceptStudentNum<=0){
                ehs.setSubmitRate(zero);
            }else{
                ehs.setSubmitRate(new BigDecimal(submitStudentNum).divide(new BigDecimal(exceptStudentNum), CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_HALF_UP).multiply(oneHundred));
            }
            dataList.add(ehs);
        }
        return dataList;
    }

    /**
     * 根据课程计划id查询学生作业列表
     *
     * @param courseScheduleIds 课程计划id列表
     */
    @Override
    public List<StudentCourseHomework> queryByCourseScheduleIds(List<Integer> courseScheduleIds) {
        return studentCourseHomeworkDao.findByCourseScheduleIds(courseScheduleIds);
    }

    @Override
    public List<StudentCourseHomework> findStudentCourseHomeworkByCourseV2(StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query) {
        List<StudentCourseHomework> studentCourseHomeworkByCourse = studentCourseHomeworkDao.findStudentCourseHomeworkByCourse(query);
        for (StudentCourseHomework studentCourseHomework : studentCourseHomeworkByCourse) {
            studentCourseHomework.setType(ELessonTrainingType.HOMEWORK);
        }
        return getStudentCourseHomeworks(studentCourseHomeworkByCourse);

    }

    @NotNull
    private List<StudentCourseHomework> getStudentCourseHomeworks(List<StudentCourseHomework> studentCourseHomeworkByCourse) {
        if (CollectionUtils.isEmpty(studentCourseHomeworkByCourse)) {
            return Collections.emptyList();
        }

        // 设置是否会员
        // 获取用户id集合
        List<Integer> userIds = studentCourseHomeworkByCourse.stream().map(StudentCourseHomework::getUserId).collect(Collectors.toList());

        // 获取学生列表
        List<Student> students = studentDao.findByStudentIds(userIds);
        if (CollectionUtils.isEmpty(students)) {
            return studentCourseHomeworkByCourse;
        }
        // 转map
        Map<Integer, Student> studentMap = students.stream().collect(Collectors.toMap(Student::getUserId, s -> s, (s1, s2) -> s1));
        for (StudentCourseHomework studentCourseHomework : studentCourseHomeworkByCourse) {
            if (studentMap.containsKey(studentCourseHomework.getUserId())) {
                Student student = studentMap.get(studentCourseHomework.getUserId());
                if ((student.getMembershipEndTime() == null || student.getMembershipEndTime().compareTo(new Date()) < 0) &&
                    (student.getExperienceMembershipEndTime() == null || student.getExperienceMembershipEndTime().compareTo(new Date()) < 0)) {
                    studentCourseHomework.setVipFlag(false);
                } else {
                    studentCourseHomework.setVipFlag(true);
                }
                if (studentCourseHomework.getSubjectId() == null) {
                    studentCourseHomework.setSubjectId(Integer.parseInt(student.getSubjectIdList()));
                }
            }
        }
        // 设置声部
        List<Integer> voicePartIds = studentCourseHomeworkByCourse.stream().map(o -> o.getSubjectId()).collect(Collectors.toList());

        if (CollectionUtils.isEmpty(voicePartIds)) {
            return studentCourseHomeworkByCourse;
        }

        List<Subject> subjectList = subjectDao.findBySubjectIds(voicePartIds);
        // 转map
        Map<Integer, Subject> subjectMap = subjectList.stream().collect(Collectors.toMap(Subject::getId, s -> s, (s1, s2) -> s1));
        for (StudentCourseHomework studentCourseHomework : studentCourseHomeworkByCourse) {
            if (subjectMap.containsKey(studentCourseHomework.getSubjectId())) {
                studentCourseHomework.setSubjectName(subjectMap.get(studentCourseHomework.getSubjectId()).getName());
            }
        }

        // 设置完成状态
        for (StudentCourseHomework studentCourseHomework : studentCourseHomeworkByCourse) {
            if (studentCourseHomework.getStandardFlag() == null) {
                studentCourseHomework.setFinishFlag(false);
            } else
            if( studentCourseHomework.getStandardFlag().equals(StandardEnum.STANDARD) || studentCourseHomework.getStandardFlag().equals(StandardEnum.EXCELLENT)) {
                studentCourseHomework.setFinishFlag(true);
            } else {
                studentCourseHomework.setFinishFlag(false);
            }
        }

        // 设置每个学生的作业完成情况
        List<Integer> studentIds = studentCourseHomeworkByCourse.stream().map(StudentCourseHomework::getUserId).collect(Collectors.toList());

        StudentCourseHomework studentCourseHomework = studentCourseHomeworkByCourse.get(0);
        List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetails = studentCourseHomeworkDao
            .findByStudentIdsAndCourseScheduleId(studentIds, studentCourseHomework.getCourseHomeworkId(),studentCourseHomework.getType());

        if (CollectionUtils.isNotEmpty(studentLessonTrainingDetails)) {
            //设置曲目名称
            studentLessonTrainingDetailService.updateMusicScoreName(studentLessonTrainingDetails);
            // 按userid分组 ，
            Map<Long, List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail>> studentLessonTrainingDetailMap = studentLessonTrainingDetails.stream()
                .collect(Collectors.groupingBy(StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail::getUserId,
                                               LinkedHashMap::new, Collectors.toList()));

            for (StudentCourseHomework studentCourseHomework1 : studentCourseHomeworkByCourse) {
                if (studentLessonTrainingDetailMap.containsKey(studentCourseHomework1.getUserId().longValue())) {
                    List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetails1 = studentLessonTrainingDetailMap.get(studentCourseHomework1.getUserId().longValue());
                    studentCourseHomework1.setStudentLessonTrainingDetail(studentLessonTrainingDetails1);
                }
            }
        }

        return studentCourseHomeworkByCourse;
    }

    @Override
    public List<StudentCourseHomework> findExtraExerciseStudentsV2(StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query) {
        List<StudentCourseHomework> extraExerciseStudents = extracurricularExercisesReplyService.findExtraExerciseStudents(query);

        List<StudentCourseHomework> studentCourseHomeworks = getStudentCourseHomeworks(extraExerciseStudents);
        if (query.getSubjectId() != null) {
            studentCourseHomeworks = studentCourseHomeworks.stream()
            .filter(o ->o.getSubjectId() != null)
            .filter(o -> o.getSubjectId().equals(query.getSubjectId()))
            .collect(Collectors.toList());
        }
        return studentCourseHomeworks;
    }

    /**
     * 学生端查看练习详情
     *
     * @param query
     * @return
     */
    @Override
    public CourseHomeworkWrapper.CourseHomeworkList findStudentHomeworkRecordDetail(CourseHomeworkWrapper.StudentCourseHomeworkQuery query) {
        CourseHomeworkWrapper.CourseHomeworkList courseHomeworkDetail = null;
        if (ELessonTrainingType.HOMEWORK.equals(query.getType())) {
            courseHomeworkDetail= courseHomeworkService.findCourseHomeworkDetail(
                query.getCourseScheduleId());

            StudentCourseHomework homework = studentCourseHomeworkDao
                    .getHomeworkByUserIdAndCourseHomeworkId(query.getUserId(), courseHomeworkDetail.getCourseHomeworkId());
            if (homework == null) {
                throw new BizException("作业不存在");
            }
            courseHomeworkDetail.setSubmitTime(homework.getSubmitTime());
        } else {
            courseHomeworkDetail = courseHomeworkService.findCourseExtraHomeworkDetail(query.getCourseScheduleId());
            ExtracurricularExercisesReply exercisesReply = extracurricularExercisesReplyDao.getByExtraIdAndUserId(courseHomeworkDetail.getCourseHomeworkId(), query.getUserId());
            if (exercisesReply == null) {
                throw new BizException("作业不存在");
            }
            courseHomeworkDetail.setSubmitTime(exercisesReply.getSubmitTime());
        }

        // 获取作业信息
        List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> detailList = studentLessonTrainingDetailService.getBaseLessonTrainingDetail(
            courseHomeworkDetail.getCourseHomeworkId(), query.getType(), query.getUserId(),null);

        long count = detailList.stream().filter(o -> o.getTrainingTimes() < o.getTimes()).count();
        if (count >0) {
            courseHomeworkDetail.setFinishStatus(false);
        } else {
            courseHomeworkDetail.setFinishStatus(true);
        }
        courseHomeworkDetail.setTrainingDetailList(detailList);

        return courseHomeworkDetail;

    }

    @Override
    public List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> findStudentHomeworkCourseware(CourseHomeworkWrapper.StudentCourseHomeworkQuery query) {
        return studentLessonTrainingDetailService.getBaseLessonTrainingDetail(query.getCourseScheduleId().longValue(), query.getType(), query.getUserId(),"VIDEO");
    }

    /**
     * 添加学生作业练习记录
     *
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean addStudentHomeworkRecord(CourseHomeworkWrapper.AddStudentHomeworkRecord record) {

        StudentLessonTrainingDetail studentLessonTrainingDetail = studentLessonTrainingDetailService.getById(record.getId());
        if (studentLessonTrainingDetail == null) {
            throw new BizException("练习记录不存在");
        }
        // 设置个人作业练习详情记录
        studentLessonTrainingDetail.setTrainingTimes(studentLessonTrainingDetail.getTrainingTimes() +1);

        if (studentLessonTrainingDetail.getTrainingTimes() >= studentLessonTrainingDetail.getTimes()) {
            studentLessonTrainingDetail.setTrainingStatus(StandardEnum.STANDARD);
        } else {
            studentLessonTrainingDetail.setTrainingStatus(StandardEnum.NOT_STANDARD);

        }
        studentLessonTrainingDetail.setUpdateTime(new Date());
        studentLessonTrainingDetailService.updateById(studentLessonTrainingDetail);

        // 查询当前作业的其他作业，判断是否全部完成
        List<StudentLessonTrainingDetail> list = studentLessonTrainingDetailService.lambdaQuery()
                                   .eq(StudentLessonTrainingDetail::getCourseHomeworkId,
                                       studentLessonTrainingDetail.getCourseHomeworkId())
                                   .eq(StudentLessonTrainingDetail::getUserId,
                                       record.getUserId())
                                   .eq(StudentLessonTrainingDetail::getType,
                                       studentLessonTrainingDetail.getType())
                                   .in(StudentLessonTrainingDetail::getTrainingStatus,
                                       StandardEnum.NOT_START,
                                       StandardEnum.NOT_STANDARD)
                                   .list();
        Boolean finish = false;
        if (CollectionUtils.isEmpty(list)) {
            finish =  true;
        }

        // 设置个人作业练习记录

        if (studentLessonTrainingDetail.getType().equals(ELessonTrainingType.HOMEWORK)) {

            StudentCourseHomework studentCourseHomework = studentCourseHomeworkDao.getHomeworkByUserIdAndCourseHomeworkId(
                record.getUserId(), studentLessonTrainingDetail.getCourseHomeworkId());

            if (studentCourseHomework == null) {
                throw new BizException("学生作业记录不存在");
            }
            Boolean success = false;
            if (studentCourseHomework.getStandardFlag() == null) {

            } else
            if (studentCourseHomework.getStandardFlag().equals(StandardEnum.STANDARD)
                || studentCourseHomework.getStandardFlag().equals(StandardEnum.EXCELLENT)){
                success =  true;
            }
            // 设置作业完成
            if (Boolean.TRUE.equals(finish)) {
                studentCourseHomework.setStandardFlag(StandardEnum.STANDARD);
            } else {
                studentCourseHomework.setStandardFlag(StandardEnum.NOT_STANDARD);
            }
            studentCourseHomework.setStatus(YesOrNoEnum.YES);
            studentCourseHomework.setUpdateTime(new Date());
            studentCourseHomework.setSubmitTime(new Date());
            studentCourseHomeworkDao.update(studentCourseHomework);

            if (Boolean.TRUE.equals(finish) && Boolean.TRUE.equals(!success)) {
                // 添加完成人数
                CourseHomework courseHomework = courseHomeworkDao.get(studentLessonTrainingDetail.getCourseHomeworkId());
                if (courseHomework == null) {
                    throw new BizException("课程作业不存在");
                }
                if (courseHomework.getExpiryDate().compareTo(new Date()) < 0) {
                    throw new BizException("作业已截止");
                }
                courseHomeworkDao.updateFinishNum(studentLessonTrainingDetail.getCourseHomeworkId());
            }

        } else {
            ExtracurricularExercisesReply exercisesReply = extracurricularExercisesReplyDao.getByExtraIdAndUserId(
                studentLessonTrainingDetail.getCourseHomeworkId(), record.getUserId());

            if (exercisesReply == null) {
                throw new BizException("学生作业记录不存在");
            }

            Boolean success = false;
            if (exercisesReply.getStandardFlag() == null) {

            } else
            if (exercisesReply.getStandardFlag().equals(StandardEnum.STANDARD)
                || exercisesReply.getStandardFlag().equals(StandardEnum.EXCELLENT)){
                success = true;
            }
            // 设置作业完成
            if (Boolean.TRUE.equals(finish)) {
                exercisesReply.setStandardFlag(StandardEnum.STANDARD);
            } else {
                exercisesReply.setStandardFlag(StandardEnum.NOT_STANDARD);
            }
            exercisesReply.setStatus(1);
            exercisesReply.setUpdateTime(new Date());
            exercisesReply.setSubmitTime(new Date());
            extracurricularExercisesReplyDao.update(exercisesReply);
            // 添加完成人数
            if (Boolean.TRUE.equals(finish) && Boolean.TRUE.equals(!success)) {
                ExtracurricularExercises extracurricularExercises = extracurricularExercisesDao.get(studentLessonTrainingDetail.getCourseHomeworkId());
                if (extracurricularExercises == null) {
                    throw new BizException("课外作业不存在");
                }
                if (extracurricularExercises.getExpireDate().compareTo(new Date()) < 0) {
                    throw new BizException("作业已截止");
                }
                extracurricularExercisesDao.updateFinishNum(studentLessonTrainingDetail.getCourseHomeworkId());
            }

        }
        return true;
    }


    /**
     * 老师查看学生作业练习详情
     *
     * @param courseScheduleId 课程计划id
     * @param userId           学生id
     * @param type             作业类型
     */
    @Override
    public StudentLessonTrainingDetailWrapper.StudentLessonTraining findCourseHomeworkStudentDetailPublicV2(Long courseScheduleId, Long userId, ELessonTrainingType type) {

        StudentLessonTrainingDetailWrapper.StudentLessonTraining studentLessonTraining = getUserInfo(userId);

        // 作业信息
        CourseHomeworkStudentDetailDto courseHomeworkStudentDetail = studentCourseHomeworkDao.findCourseHomeworkStudentDetail(
            courseScheduleId, userId);
        if (courseHomeworkStudentDetail == null) {
            throw new BizException("作业不存在");
        }
        studentLessonTraining.setStandardFlag(courseHomeworkStudentDetail.getStandardFlag());

        // 作业完成情况
        if (courseHomeworkStudentDetail.getStandardFlag() == null) {
            studentLessonTraining.setFinishFlag(false);
        } else
        if( courseHomeworkStudentDetail.getStandardFlag().equals(StandardEnum.STANDARD) || courseHomeworkStudentDetail.getStandardFlag().equals(StandardEnum.EXCELLENT)) {
            studentLessonTraining.setFinishFlag(true);
        } else {
            studentLessonTraining.setFinishFlag(false);
        }

        // 练习内容
        List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetails = studentCourseHomeworkDao
            .findByStudentIdsAndCourseScheduleId(Collections.singletonList(userId.intValue()), courseHomeworkStudentDetail.getCourseHomeworkId(),type);

        studentLessonTraining.setStudentLessonTrainingDetail(studentLessonTrainingDetails);
        return studentLessonTraining;

    }

    @Override
    public StudentLessonTrainingDetailWrapper.StudentLessonTraining findExtracurricularExercisesDetailPublicV2(Long courseScheduleId, Long userId, ELessonTrainingType type) {
        StudentLessonTrainingDetailWrapper.StudentLessonTraining studentLessonTraining = getUserInfo(userId);


        // 作业信息
        ExtracurricularExercisesReply exercisesReply = extracurricularExercisesReplyDao.getByExtraIdAndUserId(
            courseScheduleId, userId);
        if (exercisesReply == null) {
            throw new BizException("作业不存在");
        }
        studentLessonTraining.setStandardFlag(exercisesReply.getStandardFlag());

        // 作业完成情况
        if (exercisesReply.getStandardFlag() == null) {
            studentLessonTraining.setFinishFlag(false);
        } else if( exercisesReply.getStandardFlag().equals(StandardEnum.STANDARD) || exercisesReply.getStandardFlag().equals(StandardEnum.EXCELLENT)) {
            studentLessonTraining.setFinishFlag(true);
        } else {
            studentLessonTraining.setFinishFlag(false);
        }

        // 练习内容
        List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetails = studentCourseHomeworkDao
            .findByStudentIdsAndCourseScheduleId(Collections.singletonList(userId.intValue()), exercisesReply.getExtracurricularExercisesId(),type);

        studentLessonTraining.setStudentLessonTrainingDetail(studentLessonTrainingDetails);
        return studentLessonTraining;

    }

    @Override
    @Transactional
    public void pushEndMsg() {
        // 课后作业
        List<CourseHomeworkListDto> courseHomeworks = courseHomeworkDao.findEndHomework();
        StringBuffer sb;
        for (CourseHomeworkListDto courseHomework : courseHomeworks) {
            sb = new StringBuffer("homeworkId=").append(courseHomework.getCourseScheduleId())
                    .append("&homeworkType=HOMEWORK");
            Map<Integer, String> receivers = new HashMap<Integer, String>(1);
            String dateToString = DateUtil.dateToString(courseHomework.getCreateTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
            receivers.put(courseHomework.getTeacherId(), courseHomework.getTeacherId() + "");
            sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_JOB_MESSAGE_V2, receivers, null, 0, "homework?" + sb, "TEACHER",dateToString);
        }
        if (courseHomeworks.size() > 0) {
            // id 集合
            List<Long> ids = courseHomeworks.stream().map(CourseHomeworkListDto::getId).collect(Collectors.toList());
            courseHomeworkDao.updatePushFlag(ids);
        }

        // 课外作业
        List<ExtracurricularExercises> extracurricularExercises = extracurricularExercisesDao.findEndHomework();
        for (ExtracurricularExercises exercises : extracurricularExercises) {
            sb = new StringBuffer("homeworkId=").append(exercises.getId())
                    .append("&homeworkType=")
                    .append(exercises.getGroupType());
            Map<Integer, String> receivers = new HashMap<Integer, String>(1);
            String dateToString = DateUtil.dateToString(exercises.getCreateTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
            receivers.put(exercises.getTeacherId(), exercises.getTeacherId() + "");
            sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_JOB_MESSAGE_EXTRA_V2, receivers, null, 0, "homework?" + sb, "TEACHER",dateToString);
        }
        if (extracurricularExercises.size() > 0) {
            // id 集合
            List<Long> ids = extracurricularExercises.stream().map(ExtracurricularExercises::getId).collect(Collectors.toList());
            extracurricularExercisesDao.updatePushFlag(ids);
        }

    }

    @NotNull
    private StudentLessonTrainingDetailWrapper.StudentLessonTraining getUserInfo(Long userId) {
        SysUser sysUser = sysUserFeignService.queryUserById(userId.intValue());
        if (sysUser == null) {
            throw new BizException("用户不存在");
        }
        StudentLessonTrainingDetailWrapper.StudentLessonTraining studentLessonTraining = new StudentLessonTrainingDetailWrapper.StudentLessonTraining();

        studentLessonTraining.setUserId(userId.longValue());
        studentLessonTraining.setUserName(sysUser.getRealName());
        studentLessonTraining.setAvatar(sysUser.getAvatar());
        Student student = studentDao.get(studentLessonTraining.getUserId().intValue());
        if (student == null) {
            throw new BizException("学生不存在");
        }
        if ((student.getMembershipEndTime() == null || student.getMembershipEndTime().compareTo(new Date()) < 0) &&
            (student.getExperienceMembershipEndTime() == null || student.getExperienceMembershipEndTime().compareTo(new Date()) < 0)) {
            studentLessonTraining.setVipFlag(false);
        } else {
            studentLessonTraining.setVipFlag(true);
        }
        // 声部
        Subject subject = subjectDao.get(Integer.parseInt(student.getSubjectIdList()));
        if (subject == null) {
            throw new BizException("声部不存在");
        }
        studentLessonTraining.setSubjectId(subject.getId());
        studentLessonTraining.setSubjectName(subject.getName());
        return studentLessonTraining;
    }
}
