package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dao.StudentDao;
import com.ym.mec.biz.dal.dao.StudentRegistrationDao;
import com.ym.mec.biz.dal.entity.Student;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.service.StudentServeService;
import com.ym.mec.common.page.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import lombok.extern.slf4j.Slf4j;
import com.ym.mec.biz.dal.entity.StudentCoursewarePlayRecord;
import com.ym.mec.biz.dal.wrapper.StudentCoursewarePlayRecordWrapper;
import com.ym.mec.biz.dal.mapper.StudentCoursewarePlayRecordMapper;
import com.ym.mec.biz.service.StudentCoursewarePlayRecordService;
import org.springframework.transaction.annotation.Transactional;

import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * 学生课件播放统计记录
 * 2024-07-04 14:48:14
 */
@Slf4j
@Service
public class StudentCoursewarePlayRecordServiceImpl extends ServiceImpl<StudentCoursewarePlayRecordMapper, StudentCoursewarePlayRecord> implements StudentCoursewarePlayRecordService {

    @Autowired
    private StudentDao studentDao;

    @Autowired
    private StudentRegistrationDao studentRegistrationDao;

    @Autowired
    private MusicGroupDao musicGroupDao;

    /**
     * 查询详情
     *
     * @param id 详情ID
     * @return StudentCoursewarePlayRecord
     */
    @Override
    public StudentCoursewarePlayRecord detail(Long id) {

        return baseMapper.selectById(id);
    }

    /**
     * 分页查询
     *
     * @param page  IPage<StudentCoursewarePlayRecord>
     * @param query StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecordQuery
     * @return IPage<StudentCoursewarePlayRecord>
     */
    @Override
    public IPage<StudentCoursewarePlayRecord> selectPage(IPage<StudentCoursewarePlayRecord> page, StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecordQuery query) {

        return page.setRecords(baseMapper.selectPage(page, query));
    }

    /**
     * 添加
     *
     * @param studentCoursewarePlayRecord StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecord
     * @return Boolean
     */
    @Override
    public Boolean add(StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecord studentCoursewarePlayRecord) {

        return this.save(JSON.parseObject(studentCoursewarePlayRecord.jsonString(), StudentCoursewarePlayRecord.class));
    }

    /**
     * 更新
     *
     * @param studentCoursewarePlayRecord StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecord
     * @return Boolean
     */
    @Override
    public Boolean update(StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecord studentCoursewarePlayRecord) {

        return this.updateById(JSON.parseObject(studentCoursewarePlayRecord.jsonString(), StudentCoursewarePlayRecord.class));
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(StudentCoursewarePlayRecordWrapper.StudentCoursewarePlayRecordSave studentCoursewarePlayRecord) {
        Integer userId = studentCoursewarePlayRecord.getUserId();
        Student student = studentDao.get(userId);
        if (student == null) {
            return;
        }

        StudentCoursewarePlayRecord record = new StudentCoursewarePlayRecord();
        record.setOrganizationId(studentCoursewarePlayRecord.getOrganizationId());
        record.setUserId(studentCoursewarePlayRecord.getUserId());
        record.setPlayTime(studentCoursewarePlayRecord.getPlayTime());
        record.setCreateDate(new SimpleDateFormat("yyyyMMdd").format(new Date()));
        baseMapper.save(record);
    }

    @Override
    public List<StudentCoursewarePlayRecordWrapper.StatQueryData> statList(StudentCoursewarePlayRecordWrapper.StatQuery statQuery) {
        List<StudentCoursewarePlayRecordWrapper.StatQueryData> list = baseMapper.statList(statQuery);
        list.forEach(n -> n.setAvgPlayTime(n.getAvgPlayTime().setScale(2, RoundingMode.HALF_UP)));
        return list;
    }

    @Override
    public IPage<StudentCoursewarePlayRecordWrapper.StatQueryData> statDetailPage(StudentCoursewarePlayRecordWrapper.StatQuery statQuery) {
        IPage<StudentCoursewarePlayRecordWrapper.StatQueryData> page = QueryInfo.getPage(statQuery);
        List<StudentCoursewarePlayRecordWrapper.StatQueryData> statQueryData = baseMapper.statDetailPage(page, statQuery);
        statQueryData.forEach(n -> n.setAvgPlayTime(n.getAvgPlayTime().setScale(2, RoundingMode.HALF_UP)));
        page.setRecords(statQueryData);
        return page;
    }


}