package com.ym.mec.biz.service.impl;

import com.google.common.collect.Lists;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.EHomeWorkStatus;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.StandardEnum;
import com.ym.mec.biz.dal.enums.TeacherRemindTypeEnum;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.constant.CommonConstants;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class StudentExtracurricularExercisesSituationServiceImpl extends BaseServiceImpl<Long, StudentExtracurricularExercisesSituation> implements StudentExtracurricularExercisesSituationService {
	
	@Autowired
	private StudentExtracurricularExercisesSituationDao studentExtracurricularExercisesSituationDao;
	@Autowired
	private ExtracurricularExercisesReplyDao extracurricularExercisesReplyDao;
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private TeacherRemindDao teacherRemindDao;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private StudentDao studentDao;
	@Autowired
	private SubjectDao subjectDao;
	@Autowired
	private CourseHomeworkDao courseHomeworkDao;
	@Autowired
	private ClassGroupDao classGroupDao;
	@Autowired
	private StudentAttendanceDao studentAttendanceDao;
	@Autowired
	private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private CourseScheduleStatisticsDao courseScheduleStatisticsDao;

    @Autowired
    private MusicGroupService musicGroupService;

    @Autowired
    private ClassGroupService classGroupService;

    @Autowired
    private CooperationOrganService cooperationOrganService;

    @Autowired
    private ExtracurricularExercisesDao extracurricularExercisesDao;

	@Override
	public BaseDAO<Long, StudentExtracurricularExercisesSituation> getDAO() {
		return studentExtracurricularExercisesSituationDao;
	}

	@Override
	public PageInfo<MusicGroupExtracurricularSummaryDto> queryStudentExerciseMusicGroupList(StudentExerciseMusicGroupQueryInfo queryInfo) {
		PageInfo<MusicGroupExtracurricularSummaryDto> pageInfo = new PageInfo<MusicGroupExtracurricularSummaryDto>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);
		
		List<MusicGroupExtracurricularSummaryDto> dataList = null;
		int count = studentExtracurricularExercisesSituationDao.queryStudentExerciseMusicGroupListCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = studentExtracurricularExercisesSituationDao.queryStudentExerciseMusicGroupList(params);
		}
		if (count == 0) {
			dataList = new ArrayList<MusicGroupExtracurricularSummaryDto>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<StudentExercisesSituationDto> findStudentExtracurricularExercisesSituations(StudentExercisesSituationQueryInfo queryInfo) {
		PageInfo<StudentExercisesSituationDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		if (StringUtils.isNotEmpty(queryInfo.getOrganId())){
			List<Integer> organIds = Arrays.stream(queryInfo.getOrganId().split(",")).map(e -> Integer.valueOf(e)).collect(Collectors.toList());
			params.put("organIds",organIds);
		}

		List<StudentExercisesSituationDto> dataList = null;
		int count = studentExtracurricularExercisesSituationDao.countExercisesSituations(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = studentExtracurricularExercisesSituationDao.findExercisesSituations(params);
			List<Integer> organIds = dataList.stream().map(StudentExercisesSituationDto::getOrganId).distinct().collect(Collectors.toList());
			List<String> musicGroupIds = dataList.stream().map(StudentExercisesSituationDto::getMusicGroupId).distinct().collect(Collectors.toList());
			List<Integer> studentIds = dataList.stream().map(StudentExercisesSituationDto::getStudentId).collect(Collectors.toList());
			List<Integer> teacherIds = dataList.stream().map(StudentExercisesSituationDto::getTeacherId).distinct().collect(Collectors.toList());
			List<Map<Integer, Long>> studentVipCoursesMaps = courseScheduleDao.countStudentVipCoursesWithDate(studentIds, queryInfo.getMonday(), queryInfo.getSunday(), GroupType.VIP);
			Map<Integer, Long> studentVipCourseMap = MapUtil.convertIntegerMap(studentVipCoursesMaps);

			List<Map<Integer, Long>> studentPracticeCoursesMaps = courseScheduleDao.countStudentPayPracticeCoursesWithDate(studentIds, queryInfo.getMonday(), queryInfo.getSunday());
			Map<Integer, Long> studentPracticeCourseMap = MapUtil.convertIntegerMap(studentPracticeCoursesMaps);
			Map<Integer, String> teacherNameMap = MapUtil.convertMybatisMap(teacherDao.queryNameByIdList(teacherIds));

			Map<Integer, String> organNameMap = this.getMap("organization", "id_", "name_", organIds, queryInfo.getTenantId(), Integer.class, String.class);
			Map<String, String> groupNameMap = this.getMap("music_group", "id_", "name_", musicGroupIds, queryInfo.getTenantId(), String.class, String.class);
			List<CooperationOrganDto> cooperationOrgans = cooperationOrganService.getDao().queryByGroupIds(musicGroupIds);
			Map<String, CooperationOrganDto> cooperationMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(cooperationOrgans)){
				cooperationMap = cooperationOrgans.stream().collect(Collectors.groupingBy(e -> e.getMusicGroupId(),Collectors.collectingAndThen(Collectors.toList(),v->v.get(0))));
			}
			for (StudentExercisesSituationDto exercisesSituationDto : dataList) {
				Long vipCourses=studentVipCourseMap.get(exercisesSituationDto.getStudentId());
				exercisesSituationDto.setExistVipCourse(Objects.isNull(vipCourses)?0:vipCourses.intValue());
				Long practiceCourses=studentPracticeCourseMap.get(exercisesSituationDto.getStudentId());
				exercisesSituationDto.setExistPracticeCourse(Objects.isNull(practiceCourses)?0:practiceCourses.intValue());
				exercisesSituationDto.setOrganName(organNameMap.get(exercisesSituationDto.getOrganId()));
				exercisesSituationDto.setTeacherName(teacherNameMap.get(exercisesSituationDto.getTeacherId()));
				exercisesSituationDto.setGroupNames(groupNameMap.get(exercisesSituationDto.getMusicGroupId()));
				CooperationOrganDto cooperationOrganDto = cooperationMap.get(exercisesSituationDto.getMusicGroupId());
				if(Objects.nonNull(cooperationOrganDto)){
					exercisesSituationDto.setEducationalTeacherId(cooperationOrganDto.getEducationUserId());
					exercisesSituationDto.setEducationalTeacherName(cooperationOrganDto.getRealName());
				}
//				exercisesSituationDto.setExpectExercisesNum((int) until+1);
				/*List<UserGroupDto> userGroupDtos = userGroupsMap.get(exercisesSituationDto.getStudentId());
				if(!CollectionUtils.isEmpty(userGroupDtos)){
					exercisesSituationDto.setGroupNames(userGroupDtos.stream().filter(ug->StringUtils.isNotBlank(ug.getGroupName())).map(UserGroupDto::getGroupName).sorted().collect(Collectors.joining(",")));
				}*/
				/*List<StudentEduTeacherDto> studentEduTeachers = studentEduTeachersMap.get(exercisesSituationDto.getStudentId());
				if(!CollectionUtils.isEmpty(studentEduTeachers)){
					List<StudentEduTeacherDto> t1 = studentEduTeachers.stream().filter(set -> set.getGroupType().equals(GroupType.VIP.getCode())||set.getGroupType().equals(GroupType.PRACTICE.getCode())).collect(Collectors.toList());
					if(!CollectionUtils.isEmpty(t1)){
						for (StudentEduTeacherDto studentEduTeacherDto : t1) {
							if(StringUtils.isNotBlank(studentEduTeacherDto.getEducationalTeacherName())){
								exercisesSituationDto.setEducationalTeacherId(studentEduTeacherDto.getEducationalTeacherId());
								exercisesSituationDto.setEducationalTeacherName(studentEduTeacherDto.getEducationalTeacherName());
								break;
							}
						}
					}
					if(StringUtils.isNoneBlank(exercisesSituationDto.getEducationalTeacherName())){
						continue;
					}
					List<StudentEduTeacherDto> t2 = studentEduTeachers.stream().filter(set -> set.getGroupType().equals(GroupType.MUSIC.getCode())).collect(Collectors.toList());
					if(!CollectionUtils.isEmpty(t2)){
						for (StudentEduTeacherDto studentEduTeacherDto : t2) {
							if(StringUtils.isNotBlank(studentEduTeacherDto.getEducationalTeacherName())){
								exercisesSituationDto.setEducationalTeacherId(studentEduTeacherDto.getEducationalTeacherId());
								exercisesSituationDto.setEducationalTeacherName(studentEduTeacherDto.getEducationalTeacherName());
								break;
							}
						}
					}
				}*/
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<TeacherServeDto> queryTeacherServeInfo(TeacherServeQueryInfo queryInfo) {
		PageInfo<TeacherServeDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		if(StringUtils.isNotBlank(queryInfo.getOrganId())){
			List<Integer> organIds = Arrays.stream(queryInfo.getOrganId().split(",")).map(e -> Integer.valueOf(e)).collect(Collectors.toList());
			params.put("organIds",organIds);
		}
		List<TeacherServeDto> dataList = null;
		int count = studentExtracurricularExercisesSituationDao.countTeacherServeInfo(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = studentExtracurricularExercisesSituationDao.queryTeacherServeInfo(params);
			List<Integer> teacherIds = dataList.stream().map(TeacherServeDto::getTeacherId).collect(Collectors.toList());
			List<TeacherRemind> teacherReminds = new ArrayList<>();
			if(!CollectionUtils.isEmpty(teacherIds)){
				teacherReminds = teacherRemindDao.findTeachersRemindWithType(TeacherRemindTypeEnum.SERVICE, teacherIds, queryInfo.getMonday(), queryInfo.getSunday());
			}
			Map<String, Map<Integer, List<TeacherRemind>>> mondayTeacherRemindMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(teacherReminds)){
				mondayTeacherRemindMap = teacherReminds.stream().collect(Collectors.groupingBy(e -> DateUtil.dateToString(e.getMonday(), "yyyy-MM-dd"), Collectors.groupingBy(TeacherRemind::getTeacherId)));
			}
			List<StudentExtracurricularExercisesSituation> teacherServeWithDate = studentExtracurricularExercisesSituationDao.findTeacherServeWithDate(queryInfo.getMonday(), queryInfo.getSunday(), teacherIds, null);
			Map<Integer, List<StudentExtracurricularExercisesSituation>> teacherServeMap = teacherServeWithDate.stream().collect(Collectors.groupingBy(StudentExtracurricularExercisesSituation::getTeacherId));
			for (TeacherServeDto teacherServeDto : dataList) {
				teacherServeDto.setHomeworkStuNum(teacherServeDto.getExpectExercisesNum() - teacherServeDto.getExerciseNum());
				List<StudentExtracurricularExercisesSituation> tss = teacherServeMap.get(teacherServeDto.getTeacherId());
				Set<Long> courseIds = new HashSet<>();
				for (StudentExtracurricularExercisesSituation ts : tss) {
					if(teacherServeDto.getMonday().compareTo(ts.getMonday())!=0){
						continue;
					}
					if(StringUtils.isBlank(ts.getCourseIds())){
						continue;
					}
					Set<Long> ids = Arrays.stream(ts.getCourseIds().split(",")).map(s -> Long.valueOf(s)).collect(Collectors.toSet());
					courseIds.addAll(ids);
				}
				teacherServeDto.setHomeworkNum(courseIds.size());
				String mondayStr = DateUtil.dateToString(teacherServeDto.getMonday(), "yyyy-MM-dd");
				if(!mondayTeacherRemindMap.containsKey(mondayStr)){
					continue;
				}
				Map<Integer, List<TeacherRemind>> teacherRemindMap = mondayTeacherRemindMap.get(mondayStr);
				if(!teacherRemindMap.containsKey(teacherServeDto.getTeacherId())){
					continue;
				}
				TeacherRemind lastTeacherRemind = teacherRemindMap.get(teacherServeDto.getTeacherId()).stream().max(Comparator.comparing(TeacherRemind::getCreateTime)).get();
				teacherServeDto.setOperatorName(lastTeacherRemind.getOperatorName());
				teacherServeDto.setRemindDate(lastTeacherRemind.getCreateTime());
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public List<Map<String, Object>> findServiceStudentDetail(StudentServiceDetailQueryInfo queryInfo) {
		PageInfo<StudentServiceDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<StudentServiceDetailDto> dataList = null;
		int count = extracurricularExercisesReplyDao.countStudentHomeworkDetailsInTimeZone(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = extracurricularExercisesReplyDao.findStudentHomeworkDetailsInTimeZone(params);
			Map<String, List<StudentServiceDetailDto>> weekHomeworks=new HashMap<>();
			for (StudentServiceDetailDto studentServiceDetailDto : dataList) {
				if(Objects.isNull(studentServiceDetailDto.getCourseStartTime())){
					continue;
				}
				LocalDate homeworkCreateTime = LocalDateTime.ofInstant(studentServiceDetailDto.getCourseStartTime().toInstant(), DateUtil.zoneId).toLocalDate();
				String mondayStr = homeworkCreateTime.with(DateUtil.weekFields.dayOfWeek(), DayOfWeek.MONDAY.getValue()).toString();
				if(!weekHomeworks.containsKey(mondayStr)){
					weekHomeworks.put(mondayStr, new ArrayList<>());
				}
				weekHomeworks.get(mondayStr).add(studentServiceDetailDto);
			}
			List<Map<String, Object>> results=new ArrayList<>();
			for (Map.Entry<String, List<StudentServiceDetailDto>> stringListEntry : weekHomeworks.entrySet()) {
				Map<String, Object> result=new HashMap<>();
				result.put("monday", stringListEntry.getKey());
				result.put("data", stringListEntry.getValue());
				results.add(result);
			}
			return results;
		}else{
			return null;
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void teacherServeRemindPush(List<TeacherServeDto> teacherServes, Integer operatorId, String operatorName) {
		if(CollectionUtils.isEmpty(teacherServes)){
			return;
		}
		List<TeacherRemind> reminds = new ArrayList<>();
		for (TeacherServeDto teacherServe : teacherServes) {
			if(Objects.isNull(teacherServe.getMonday())||Objects.isNull(teacherServe.getSunday())||Objects.isNull(teacherServe.getTeacherId())){
				continue;
			}
			String mondayStr = DateUtil.dateToString(teacherServe.getMonday(), "yyyy-MM-dd");
			List<StudentExtracurricularExercisesSituation> serves = studentExtracurricularExercisesSituationDao.findWeekServiceWithStudents(mondayStr, teacherServe.getTeacherId(), null);
			if(CollectionUtils.isEmpty(serves)){
				continue;
			}
			TeacherRemind remind = new TeacherRemind();
			remind.setOperatorId(operatorId);
			remind.setOperatorName(operatorName);
			remind.setMonday(teacherServe.getMonday());
			remind.setTeacherId(teacherServe.getTeacherId());
			remind.setType(TeacherRemindTypeEnum.SERVICE);
			remind.setContent("作业布置提醒");
			reminds.add(remind);

			Map<Integer, String> userMap = new HashMap<>();
			userMap.put(teacherServe.getTeacherId(), teacherServe.getTeacherId().toString());
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_SERVE,
					userMap, null, 0, null, "TEACHER", teacherServe.getHomeworkNum(), teacherServe.getExerciseNum());
		}
		if(!CollectionUtils.isEmpty(reminds)){
			teacherRemindDao.batchInsert(reminds);
		}
	}

	@Override
	public PageInfo<TeacherServeHomeworkDto> queryTeacherServeHomeworkDetail(TeacherServeHomeworkQueryInfo queryInfo) {
		if(Objects.isNull(queryInfo.getMonday())||Objects.isNull(queryInfo.getSunday())){
			throw new BizException("请指定课程时间");
		}
		if(Objects.isNull(queryInfo.getTeacherId())){
			throw new BizException("请指定教师");
		}
		PageInfo<TeacherServeHomeworkDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		String mondayStr = DateUtil.dateToString(queryInfo.getMonday(), "yyyy-MM-dd");
		List<StudentExtracurricularExercisesSituation> serves = studentExtracurricularExercisesSituationDao.findWeekServiceWithStudents(mondayStr, queryInfo.getTeacherId(), null);
		if(CollectionUtils.isEmpty(serves)){
			return pageInfo;
		}
		List<StudentExtracurricularExercisesSituation> homeworkServes = serves.stream().filter(s -> "HOMEWORK".equals(s.getServeType())).collect(Collectors.toList());
		if(CollectionUtils.isEmpty(homeworkServes)){
			return pageInfo;
		}
		List<Long> courseIdsArr = new ArrayList<>();
		for (StudentExtracurricularExercisesSituation homeworkServe : homeworkServes) {
			if(StringUtils.isBlank(homeworkServe.getCourseIds())){
				continue;
			}
			List<Long> ids = Arrays.stream(homeworkServe.getCourseIds().split(",")).map(id -> Long.valueOf(id)).collect(Collectors.toList());
			courseIdsArr.addAll(ids);
		}
		if(CollectionUtils.isEmpty(courseIdsArr)){
			return pageInfo;
		}

		Map<Long, Long> courseNumMap = courseIdsArr.stream().collect(Collectors.groupingBy(id -> id, Collectors.counting()));
		Set<Long> courseIds = new HashSet<>(courseIdsArr);

		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		params.put("courseIds", courseIds);

		List<CourseHomework> courseHomeworks = courseHomeworkDao.findByCourseSchedules(new ArrayList<>(courseIds));
		Map<Long, Long> courseHomeworkMap = new HashMap<>();
		if(!CollectionUtils.isEmpty(courseHomeworks)){
			courseHomeworkMap = courseHomeworks.stream().collect(Collectors.groupingBy(CourseHomework::getCourseScheduleId, Collectors.counting()));
		}
		List<TeacherServeHomeworkDto> dataList = new ArrayList<>();
		int count = courseScheduleDao.countByCourseScheduleIds(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			List<CourseSchedule> courseSchedules = courseScheduleDao.queryByCourseScheduleIds(params);
			Map<Integer, ClassGroup> idClassGroupMap = new HashMap<>();
			List<Integer> classGroupIds = courseSchedules.stream().map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
			if(!CollectionUtils.isEmpty(classGroupIds)){
				List<ClassGroup> classGroups = classGroupDao.findByClassGroupIds(classGroupIds, null);
				idClassGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, c -> c, (c1, c2) -> c1));
			}
			List<Integer> subjectIds = new ArrayList<>();
			for (CourseSchedule courseSchedule : courseSchedules) {
				if(idClassGroupMap.containsKey(courseSchedule.getClassGroupId())&&StringUtils.isNotBlank(idClassGroupMap.get(courseSchedule.getClassGroupId()).getSubjectIdList())){
					List<Integer> ids = Arrays.stream(idClassGroupMap.get(courseSchedule.getClassGroupId()).getSubjectIdList().split(",")).map(id -> Integer.valueOf(id)).collect(Collectors.toList());
					subjectIds.addAll(ids);
				}
				TeacherServeHomeworkDto tshd = new TeacherServeHomeworkDto();
				BeanUtils.copyProperties(courseSchedule,tshd);
				tshd.setStudentNum(courseNumMap.get(courseSchedule.getId()).intValue());
				if(courseHomeworkMap.containsKey(courseSchedule.getId())){
					tshd.setHomeworkExist(1);
				}
				dataList.add(tshd);
			}
			List<Subject> subjects = new ArrayList<>();
			if(!CollectionUtils.isEmpty(subjectIds)){
				subjects = subjectDao.findBySubjectIds(new ArrayList<>(subjectIds));
			}
			for (TeacherServeHomeworkDto teacherServeHomeworkDto : dataList) {
				if(idClassGroupMap.containsKey(teacherServeHomeworkDto.getClassGroupId())){
					ClassGroup classGroup = idClassGroupMap.get(teacherServeHomeworkDto.getClassGroupId());
					if(StringUtils.isNotBlank(classGroup.getSubjectIdList())){
						Set<Integer> sids = Arrays.stream(classGroup.getSubjectIdList().split(",")).map(id -> Integer.valueOf(id)).collect(Collectors.toSet());
						List<String> subjectNames = subjects.stream().filter(s -> sids.contains(s.getId())).map(Subject::getName).collect(Collectors.toList());
						teacherServeHomeworkDto.setSubjectName(StringUtils.join(subjectNames, ","));
					}
				}
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo<TeacherServeHomeworkPojo> queryTeacherServeHomeworkDetail1(TeacherServeHomeworkQueryInfo queryInfo) {
//		if(Objects.isNull(queryInfo.getMonth())){
//			throw new BizException("请指定课程时间");
//		}
		PageInfo<TeacherServeHomeworkPojo> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		//获取时间段内有服务指标的课程(课后作业)
		Date firstDayOfMonth;
		Date lastDayOfMonth;
		if(Objects.isNull(queryInfo.getStartTime())){
			firstDayOfMonth = DateUtil.getFirstDayOfMonth(queryInfo.getMonth());
			lastDayOfMonth = DateUtil.getLastDayOfMonth(queryInfo.getMonth());
		}else {
			firstDayOfMonth = queryInfo.getStartTime();
			lastDayOfMonth = queryInfo.getEndTime();
		}
		List<Long> courseIds = studentExtracurricularExercisesSituationDao.queryCourseIdByClassDate(firstDayOfMonth,lastDayOfMonth,queryInfo.getTeacherId(),null);
		if(courseIds == null || courseIds.size() == 0){
			return pageInfo;
		}
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		params.put("courseIds", courseIds);

		List<CourseHomework> courseHomeworks = courseHomeworkDao.findByCourseSchedules(new ArrayList<>(courseIds));
		if(!CollectionUtils.isEmpty(courseHomeworks)){
			Set<Long> collect = courseHomeworks.stream().map(e -> e.getCourseScheduleId()).collect(Collectors.toSet());
			courseIds.removeAll(collect);
		}
		if(courseIds.size() == 0){
			return pageInfo;
		}
		Map<Long,Long> memberNumMap = MapUtil.convertIntegerMap(studentDao.countCourseStudentMemberNum(courseIds));
		Map<Long,String> studentSubjectMap = MapUtil.convertMybatisMap(studentDao.countCourseStudentSubjectName(courseIds));
		Map<Long, Long> studentNumMap = MapUtil.convertIntegerMap(courseScheduleStudentPaymentDao.countCourseStudentNum(courseIds));
		Map<Integer, Long> studentNumCourseMap = MapUtil.convertIntegerMap(studentAttendanceDao.countStudentAttendancesByCourses(courseIds));

		List<TeacherServeHomeworkPojo> homeworkPojos;
		int count = courseScheduleDao.countHomeworkPojoByCourseScheduleIds(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			homeworkPojos = courseScheduleDao.queryHomeworkPojoByCourseScheduleIds(params);
			pageInfo.setRows(homeworkPojos);
			for (TeacherServeHomeworkPojo homeworkPojo : homeworkPojos) {
				Long studentNum = studentNumCourseMap.get(homeworkPojo.getCourseScheduleId().intValue());
				if (Objects.nonNull(studentNum)) {
					homeworkPojo.setStudentAttendanceIsFirstTime(studentNum > 0 ? 0 : 1);
				} else {
					homeworkPojo.setStudentAttendanceIsFirstTime(1);
				}
				Long aLong = memberNumMap.get(homeworkPojo.getCourseScheduleId());
				homeworkPojo.setMemberNum(aLong==null?0:aLong.intValue());
				Long aLong1 = studentNumMap.get(homeworkPojo.getCourseScheduleId());
				homeworkPojo.setStudentNum(aLong1==null?0:aLong1.intValue());
				homeworkPojo.setSubjectName(studentSubjectMap.get(homeworkPojo.getCourseScheduleId()));
			}
		}
		return pageInfo;
	}

	@Override
	public PageInfo<TeacherServeExtraDto> queryTeacherServeExtraDetail(TeacherServeHomeworkQueryInfo queryInfo) {
		if(Objects.isNull(queryInfo.getMonday())||Objects.isNull(queryInfo.getSunday())||Objects.isNull(queryInfo.getTeacherId())){
			throw new BizException("请指定教师");
		}
		PageInfo<TeacherServeExtraDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		String mondayStr = DateUtil.dateToString(queryInfo.getMonday(), "yyyy-MM-dd");
		List<StudentExtracurricularExercisesSituation> serves = studentExtracurricularExercisesSituationDao.findWeekServiceWithStudents(mondayStr, queryInfo.getTeacherId(), null);
		if(CollectionUtils.isEmpty(serves)){
			return pageInfo;
		}
		List<StudentExtracurricularExercisesSituation> homeworkServes = serves.stream().filter(s -> "EXERCISE".equals(s.getServeType())).collect(Collectors.toList());
		if(CollectionUtils.isEmpty(homeworkServes)){
			return pageInfo;
		}

		Set<Integer> studentIds = homeworkServes.stream().map(StudentExtracurricularExercisesSituation::getStudentId).collect(Collectors.toSet());
		Map<Integer, List<StudentExtracurricularExercisesSituation>> studentServeMap = homeworkServes.stream().collect(Collectors.groupingBy(StudentExtracurricularExercisesSituation::getStudentId));

		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		params.put("studentIds", studentIds);

		List<TeacherServeExtraDto> dataList = new ArrayList<>();
		int count = studentDao.countByIds(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			List<Student> students = studentDao.queryByIds(params);
			List<SimpleUserDto> usersSimpleInfo = teacherDao.getUsersSimpleInfo(new ArrayList<>(studentIds));
			Map<Integer, String> idNameMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(usersSimpleInfo)){
				idNameMap = usersSimpleInfo.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, s -> s.getNickName(), (s1, s2) -> s1));
			}
			Set<Integer> subjectIds = new HashSet<>();
			for (Student student : students) {
				if(StringUtils.isNotBlank(student.getSubjectIdList())){
					Set<Integer> ids = Arrays.stream(student.getSubjectIdList().split(",")).map(id -> Integer.valueOf(id)).collect(Collectors.toSet());
					subjectIds.addAll(ids);
				}
				TeacherServeExtraDto tsed = new TeacherServeExtraDto();
				BeanUtils.copyProperties(student,tsed);
				if(studentServeMap.containsKey(student.getUserId())){
					List<StudentExtracurricularExercisesSituation> studentServes = studentServeMap.get(student.getUserId());
					int num = studentServes.stream().mapToInt(StudentExtracurricularExercisesSituation::getActualExercisesNum).reduce(0, Integer::sum);
					if(num>0){
						tsed.setHomeworkExist(1);
					}
				}
				dataList.add(tsed);
			}
			List<Subject> subjects = new ArrayList<>();
			if(!CollectionUtils.isEmpty(subjectIds)){
				subjects = subjectDao.findBySubjectIds(new ArrayList<>(subjectIds));
			}
			for (TeacherServeExtraDto teacherServeExtraDto : dataList) {
				if(idNameMap.containsKey(teacherServeExtraDto.getUserId())){
					teacherServeExtraDto.setUsername(idNameMap.get(teacherServeExtraDto.getUserId()));
				}
				if(StringUtils.isNotBlank(teacherServeExtraDto.getSubjectIdList())){
					Set<Integer> sids = Arrays.stream(teacherServeExtraDto.getSubjectIdList().split(",")).map(id -> Integer.valueOf(id)).collect(Collectors.toSet());
					List<String> subjectNames = subjects.stream().filter(s -> sids.contains(s.getId())).map(Subject::getName).collect(Collectors.toList());
					teacherServeExtraDto.setSubjectNames(StringUtils.join(subjectNames, ","));
				}
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public int countWaitCreateHomeworkNum() {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null) {
			throw new BizException("用户信息获取失败");
		}
		int exerciseNum = studentExtracurricularExercisesSituationDao.countWaitCreateHomeworkNum(sysUser.getId(), "EXERCISE");
		if(exerciseNum > 0){
			return exerciseNum;
		}
		return studentExtracurricularExercisesSituationDao.countWaitCreateHomeworkNum1(sysUser.getId());
	}

	@Override
	public List<TeacherServeDto> queryTeacherPerformanceIndicator(TeacherServeQueryInfo queryInfo) {
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		if(StringUtils.isNotBlank(queryInfo.getOrganId())){
			List<Integer> organIds = Arrays.stream(queryInfo.getOrganId().split(",")).map(e -> Integer.valueOf(e)).collect(Collectors.toList());
			params.put("organIds",organIds);
		}
		List<TeacherServeDto> dataList = null;
		dataList = studentExtracurricularExercisesSituationDao.queryTeacherPerformanceIndicator(params);
		
		if(dataList.size() == 0){
			return dataList;
		}
		
		List<Integer> teacherIds = dataList.stream().map(TeacherServeDto::getTeacherId).collect(Collectors.toList());
		List<StudentExtracurricularExercisesSituation> teacherServeWithDate = studentExtracurricularExercisesSituationDao.findTeacherServeWithDate(queryInfo.getMonday(), queryInfo.getSunday(), teacherIds, null);
		Map<Integer, List<StudentExtracurricularExercisesSituation>> teacherServeMap = teacherServeWithDate.stream().collect(Collectors.groupingBy(StudentExtracurricularExercisesSituation::getTeacherId));
		for (TeacherServeDto teacherServeDto : dataList) {
			List<StudentExtracurricularExercisesSituation> tss = teacherServeMap.get(teacherServeDto.getTeacherId());
			if(tss != null){
				Set<Long> courseIds = new HashSet<>();
				for (StudentExtracurricularExercisesSituation ts : tss) {
					if(teacherServeDto.getMonday().compareTo(ts.getMonday())!=0){
						continue;
					}
					if(StringUtils.isBlank(ts.getCourseIds())){
						continue;
					}
					Set<Long> ids = Arrays.stream(ts.getCourseIds().split(",")).map(s -> Long.valueOf(s)).collect(Collectors.toSet());
					courseIds.addAll(ids);
				}
				teacherServeDto.setHomeworkNum(courseIds.size());
			}
		}
		return dataList;
	}


	@Override
	@Transactional(rollbackFor = Exception.class)
	public void batchInsert(List<StudentExtracurricularExercisesSituation> situations) {
		if (CollectionUtils.isEmpty(situations)){
			return;
		}
		studentExtracurricularExercisesSituationDao.batchInsert(situations);
		String collect = situations.stream().map(e -> e.getCourseIds()).filter(e -> StringUtils.isNotEmpty(e)).distinct().collect(Collectors.joining(","));
		if(StringUtils.isNotEmpty(collect)){
			//转换成int数组
			List<Integer> courseIds = Arrays.stream(collect.split(",")).map(e -> Integer.valueOf(e)).collect(Collectors.toList());
			//更新课程服务指标
			courseScheduleStatisticsDao.updateCourseService(courseIds,1);
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void deleteByMonday(String monday,List<Integer> studentIds) {
		this.delSituations(studentExtracurricularExercisesSituationDao.selectByMonday(monday, studentIds));
		studentExtracurricularExercisesSituationDao.deleteByMonday(monday,studentIds);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void deleteByStudent(Integer studentId, String monday) {
		this.delSituations(studentExtracurricularExercisesSituationDao.selectByStudent(studentId, monday));
		studentExtracurricularExercisesSituationDao.deleteByStudent(studentId,monday);
	}

	@Override
	public TeacherExercisesServiceDto getStatistics(ExtraExercilseQueryInfo queryInfo) {
		TeacherExercisesServiceDto serviceDto = new TeacherExercisesServiceDto();
		List<Integer> teacherIds = new ArrayList();
		teacherIds.add(queryInfo.getTeacherId());
		List<StudentExtracurricularExercisesSituation> teacherServeWithDate =
				studentExtracurricularExercisesSituationDao.findTeacherServeWithDate(queryInfo.getStartTime(), queryInfo.getEndTime(), teacherIds, null);
		if(!CollectionUtils.isEmpty(teacherServeWithDate)){
			//预计训练次数
			double expectExercisesNum = teacherServeWithDate.stream().mapToDouble(e -> e.getExpectExercisesNum()).sum();
			//实际训练次数
			double actualExercisesNum = teacherServeWithDate.stream().mapToInt(e -> e.getActualExercisesNum()).sum();
			//布置率
			BigDecimal oneHundred = new BigDecimal(100);
			if(expectExercisesNum > 0d){
				BigDecimal submitRate = new BigDecimal(actualExercisesNum).divide(new BigDecimal(expectExercisesNum), CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_HALF_UP).multiply(oneHundred).setScale(0);
				serviceDto.setExpectExercisesRate(submitRate + "%");
			}
			//训练提交次数
			double exercisesReplyNum = teacherServeWithDate.stream().mapToInt(e -> e.getExercisesReplyNum()).sum();
			//提交率
			if(actualExercisesNum > 0d){
				BigDecimal submitRate = new BigDecimal(exercisesReplyNum).divide(new BigDecimal(actualExercisesNum), CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_HALF_UP).multiply(oneHundred).setScale(0);
				serviceDto.setExercisesReplyRate(submitRate + "%");
			}
			//点评次数
			double exercisesMessageNum = teacherServeWithDate.stream().mapToInt(e -> e.getExercisesMessageNum()).sum();
			//点评率
			if(exercisesReplyNum > 0d){
				BigDecimal submitRate = new BigDecimal(exercisesMessageNum).divide(new BigDecimal(exercisesReplyNum), CommonConstants.DECIMAL_FINAL_PLACE, BigDecimal.ROUND_HALF_UP).multiply(oneHundred).setScale(0);
				serviceDto.setExercisesMessageRate(submitRate + "%");
			}
		}
		return serviceDto;
	}

    /**
     * 老师端课后作业列表
     *
     * @param queryInfo
     */
    @Override
    public PageInfo<CourseHomeworkWrapper.CourseHomeworkList> queryTeacherServeHomeworkDetailV2(CourseHomeworkWrapper.CourseHomeworkQuery queryInfo) {
        PageInfo<CourseHomeworkWrapper.CourseHomeworkList> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        //获取时间段内有服务指标的课程(课后作业)

        Date firstDayOfMonth = queryInfo.getStartTime();
        Date lastDayOfMonth = queryInfo.getEndTime();
        List<Long> courseIds = new ArrayList<>();
        if (queryInfo.getCourseScheduleId() != null) {
            courseIds.add(queryInfo.getCourseScheduleId().longValue());
        } else {
            courseIds = studentExtracurricularExercisesSituationDao.queryCourseIdByClassDate(firstDayOfMonth,
                                                    lastDayOfMonth, queryInfo.getTeacherId(),queryInfo.getMusicGroupIds());
        }
        if(courseIds == null || courseIds.size() == 0){
            return pageInfo;
        }
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        params.put("versionTag","v2");
		params.put("offset", pageInfo.getOffset());

         int count = courseScheduleDao.countTeacherServeHomeworkDetailV2(params, courseIds);
         if (count == 0) {
             return pageInfo;
         }
        // 查出课程列表
        List<CourseHomeworkWrapper.CourseHomeworkList> courseHomeworkLists = courseScheduleDao.queryTeacherServeHomeworkDetailV2(
            params, courseIds);
        if (CollectionUtils.isEmpty(courseHomeworkLists)) {
            return pageInfo;
        }
        pageInfo.setTotal(count);

        // 乐团信息
        List<String> musicGroupIds = courseHomeworkLists.stream().map(e -> e.getMusicGroupId()).collect(Collectors.toList());

        List<MusicGroup> musicGroups = musicGroupService.queryListByIds(new HashSet<>(musicGroupIds));
        Map<String, MusicGroup> musicGroupMap = musicGroups.stream().collect(Collectors.toMap(MusicGroup::getId, Function.identity()));


        // 班级名称

        List<Integer> classIds = courseHomeworkLists.stream().map(e -> e.getClassGroupId()).collect(Collectors.toList());

        List<ClassGroup> classGroups = classGroupService.queryByIds(classIds);
        Map<Integer, ClassGroup> classGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, Function.identity()));

        for (CourseHomeworkWrapper.CourseHomeworkList courseHomeworkList : courseHomeworkLists) {

            courseHomeworkList.setType(ELessonTrainingType.HOMEWORK);
            // 乐团信息
            MusicGroup musicGroup = musicGroupMap.get(courseHomeworkList.getMusicGroupId());
            if (Objects.nonNull(musicGroup)) {
                courseHomeworkList.setMusicGroupName(musicGroup.getName());
                courseHomeworkList.setMusicGroupImg(musicGroup.getImg());
            }

            // 班级信息
            ClassGroup classGroup = classGroupMap.get(courseHomeworkList.getClassGroupId());
            if (Objects.nonNull(classGroup)) {
                courseHomeworkList.setClassGroupName(classGroup.getName());
            }

            // 设置老师信息
            Integer teacherId = courseHomeworkList.getTeacherId();
            if (teacherId != null) {
                SysUser sysUser = sysUserFeignService.queryUserById(teacherId);
                courseHomeworkList.setTeacherImg(sysUser.getAvatar());
                courseHomeworkList.setTeacherName(sysUser.getRealName());
            }

            // 格式时间
            courseHomeworkList.setStartTime(DateUtil.startDateAndEndTime(courseHomeworkList.getClassDate(), courseHomeworkList.getStartTime()));
            courseHomeworkList.setEndTime(DateUtil.startDateAndEndTime(courseHomeworkList.getClassDate(), courseHomeworkList.getEndTime()));

        }


        pageInfo.setRows(courseHomeworkLists);
        return pageInfo;
    }

    /**
     * 老师课外作业列表
     *
     * @param queryInfo
     */
    @Override
    public PageInfo<CourseHomeworkWrapper.CourseHomeworkList> queryTeacherExtraHomeworkDetailV2(CourseHomeworkWrapper.CourseHomeworkQuery queryInfo) {

        // 获取时间段内有服务指标的乐团(课外作业)
        Map<String,String> groupIds = new HashMap<>();
        if (queryInfo.getCourseScheduleId() == null) {
            if (queryInfo.getType() == null || queryInfo.getType().equals(ELessonTrainingType.EXTRACURRICULAR)) {
                if (queryInfo.getHomeWorkStatus() == null || queryInfo.getHomeWorkStatus().equals(EHomeWorkStatus.UNASSIGNED)) {
                    StudentExerciseMusicGroupQueryInfo query = new StudentExerciseMusicGroupQueryInfo();
                    query.setMondayDate(DateUtil.dateToString(queryInfo.getStartTime()));
                    query.setSundayDate(DateUtil.dateToString(queryInfo.getEndTime()));
                    query.setTeacherId(queryInfo.getTeacherId());
                    query.setPage(1);
                    query.setRows(10000);
                    PageInfo<MusicGroupExtracurricularSummaryDto> groupList = queryStudentExerciseMusicGroupList(query);

                    if (!CollectionUtils.isEmpty(groupList.getRows())) {
                        groupIds = groupList.getRows().stream().collect(Collectors.toMap(o -> o.getMusicGroupId(),o -> o.getStudentIds(),(k1,k2)->k1));
                    }
                }
            }
        }
        PageInfo<CourseHomeworkWrapper.CourseHomeworkList> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

        Set<String> subjectIds =groupIds.keySet();
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        if (StringUtils.isNotBlank(queryInfo.getMusicGroupId()) && subjectIds.contains(queryInfo.getMusicGroupId())) {
            subjectIds = new HashSet<>();
            subjectIds.add(queryInfo.getMusicGroupId());
        }
        params.put("subjectIds", subjectIds);
        params.put("versionTag","v2");

        int count = extracurricularExercisesDao.countTeacherExtraHomeworkDetailV2(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            List<CourseHomeworkWrapper.CourseHomeworkList> courseHomeworkLists  = extracurricularExercisesDao.queryTeacherExtraHomeworkDetailV2(params);


            // 乐团信息
            List<String> musicGroupIds = courseHomeworkLists.stream().map(e -> e.getMusicGroupId())
                                                            .filter(Objects::nonNull)
                                                            .collect(Collectors.toList());
            Map<String, MusicGroup> musicGroupMap = new HashMap<>();
            if (!CollectionUtils.isEmpty(musicGroupIds)) {
                List<MusicGroup> musicGroups = musicGroupService.queryListByIds(new HashSet<>(musicGroupIds));
                musicGroupMap = musicGroups.stream()
                                                                   .collect(Collectors.toMap(MusicGroup::getId, Function.identity()));
            }


            // 班级名称

            List<Integer> classIds = courseHomeworkLists.stream().map(e -> e.getClassGroupId())
                                                        .filter(Objects::nonNull)
                                                        .collect(Collectors.toList());
            Map<Integer, ClassGroup> classGroupMap = new HashMap<>();
            if (!CollectionUtils.isEmpty(classIds)) {
                List<ClassGroup> classGroups = classGroupService.queryByIds(classIds);
                if (!CollectionUtils.isEmpty(classGroups)) {
                    classGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, Function.identity()));
                }
            }
            for (CourseHomeworkWrapper.CourseHomeworkList courseHomeworkList : courseHomeworkLists) {
                courseHomeworkList.setAssignFlag(true);
                // 如果是未布置的 设置时间
                if(courseHomeworkList.getHomeWorkStatus().equals(EHomeWorkStatus.UNASSIGNED)){
                    courseHomeworkList.setAssignTime(queryInfo.getStartTime());
                    courseHomeworkList.setStudentIdList(groupIds.get(courseHomeworkList.getMusicGroupId()));
                    // 如果结束时间小于当前时间  设置为已过期
                    LocalDate now = LocalDate.now();
                    LocalDate endTime = LocalDate.parse(DateUtil.dateToString(queryInfo.getEndTime()), DateTimeFormatter.ofPattern("yyyy-MM-dd"));
                    if(now.isAfter(endTime)){
                        courseHomeworkList.setAssignFlag(false);
                    }
                }

                // 乐团信息
                MusicGroup musicGroup = musicGroupMap.get(courseHomeworkList.getMusicGroupId());
                if (Objects.nonNull(musicGroup)) {
                    courseHomeworkList.setMusicGroupName(musicGroup.getName());
                    courseHomeworkList.setMusicGroupImg(musicGroup.getImg());
                }

                // 班级信息
                ClassGroup classGroup = classGroupMap.get(courseHomeworkList.getClassGroupId());
                if (Objects.nonNull(classGroup)) {
                    courseHomeworkList.setClassGroupName(classGroup.getName());
                }

                // 如果是给学生布置的作业  名 ： xx等多少名学员
                if (courseHomeworkList.getClassGroupId() == null) {
                    String studentIdList = courseHomeworkList.getStudentIdList();
                    if (StringUtils.isNotBlank(studentIdList)) {

                        String[] split = studentIdList.split(",");
                        String userId = split[0];
                        SysUser sysUser = sysUserFeignService.queryUserById(Integer.parseInt(userId));
                        if (split.length == 1) {
                            courseHomeworkList.setClassGroupName(sysUser.getUsername());
                        } else {
                            courseHomeworkList.setClassGroupName(sysUser.getUsername() + "等" + split.length + "名学员");
                        }
                    }
                }

                if (courseHomeworkList.getTeacherId() != null) {
                    // 设置老师信息
                    SysUser sysUser = sysUserFeignService.queryUserById(courseHomeworkList.getTeacherId());
                    if (Objects.nonNull(sysUser)) {
                        courseHomeworkList.setTeacherName(sysUser.getRealName());
                        courseHomeworkList.setTeacherImg(sysUser.getAvatar());
                    }
                }
            }

            pageInfo.setRows(courseHomeworkLists);
        }


        return pageInfo;
    }

    @Override
	@Transactional(rollbackFor = Exception.class)
    public void updateNum(LessonExamination lessonExamination, List<Integer> studentIds) {
		courseScheduleStatisticsDao.updateNum(lessonExamination.getCourseScheduleId(),lessonExamination.getTeacherId(),studentIds);
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateLastSubmitTime(LessonExamination lessonExamination, StudentLessonExamination studentLessonExamination) {
		courseScheduleStatisticsDao.updateLastSubmitTime(lessonExamination.getCourseScheduleId(),lessonExamination.getTeacherId(),
				studentLessonExamination.getUserId());
	}

	@Transactional(rollbackFor = Exception.class)
	public void delSituations(List<StudentExtracurricularExercisesSituation> situations){
		if (CollectionUtils.isEmpty(situations)){
			return;
		}
//		List<Long> collect1 = situations.stream().map(e -> e.getId()).collect(Collectors.toList());
//		studentExtracurricularExercisesSituationDao.batchDelete(collect1);
		String collect = situations.stream().map(e -> e.getCourseIds()).filter(e -> StringUtils.isNotEmpty(e)).distinct().collect(Collectors.joining(","));
		if(StringUtils.isNotEmpty(collect)){
			List<Integer> courseIds = Arrays.stream(collect.split(",")).map(e -> Integer.valueOf(e)).collect(Collectors.toList());
			//更新课程服务指标
			courseScheduleStatisticsDao.updateCourseService(courseIds,0);
		}
	}
}
