package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.dao.LessonExaminationDao;
import com.ym.mec.biz.dal.dao.StudentExtracurricularExercisesSituationDao;
import com.ym.mec.biz.dal.dao.StudentLessonExaminationDao;
import com.ym.mec.biz.dal.dao.StudentLessonExaminationDetailDao;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.LessonExamination;
import com.ym.mec.biz.dal.entity.StudentLessonExamination;
import com.ym.mec.biz.dal.entity.StudentLessonExaminationDetail;
import com.ym.mec.biz.dal.entity.StudentLessonTrainingDetail;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 学生进度评测表(StudentLessonExamination)表服务实现类
 *
 * @author zx
 * @since 2023-04-03 18:21:30
 */
@Service("studentLessonExaminationService")
public class StudentLessonExaminationServiceImpl extends ServiceImpl<StudentLessonExaminationDao, StudentLessonExamination> implements StudentLessonExaminationService {

    private final static Logger log = LoggerFactory.getLogger(StudentLessonExaminationServiceImpl.class);

    @Autowired
    private StudentLessonExaminationDetailService studentLessonExaminationDetailService;
    @Autowired
    private LessonExaminationDao lessonExaminationDao;
    @Autowired
    private LessonExaminationService lessonExaminationService;
    @Autowired
    private StudentExtracurricularExercisesSituationService studentExtracurricularExercisesSituationService;

    @Autowired
    private StudentLessonTrainingDetailService studentLessonTrainingDetailService;

    @Override
    public StudentLessonExaminationDao getDao() {
        return this.baseMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<Integer> save(Long lessonExaminationId, List<StudentLessonExaminationSaveDto> studentLessonExaminationSaveDtos) {
        List<StudentLessonExamination> studentLessonExaminations = new ArrayList<>();
        List<Integer> studentIdList = new ArrayList<>();
        for (StudentLessonExaminationSaveDto dto : studentLessonExaminationSaveDtos) {
            String[] studentIds = dto.getStudentIds().split(",");
            for (String studentId : studentIds) {
                StudentLessonExamination studentLessonExamination = new StudentLessonExamination();
                studentLessonExamination.setLessonExaminationId(lessonExaminationId);
                Integer parseInt = Integer.parseInt(studentId);
                studentIdList.add(parseInt);
                studentLessonExamination.setUserId(parseInt);
//                studentLessonExamination.setSubjectId(dto.getSubjectId());
                studentLessonExaminations.add(studentLessonExamination);
            }
        }
        LessonExamination examination = lessonExaminationService.getById(lessonExaminationId);

        // 学生ID集合
        Set<Integer> userIdList = studentLessonExaminations.stream()
                .map(StudentLessonExamination::getUserId)
                .filter(Objects::nonNull)
                .collect(Collectors.toSet());
        // 设置声部
        Map<Integer, StudentSubjectDto> subjectDtoMap = studentLessonTrainingDetailService
                .getStudentMusicGroupSubject(new ArrayList<>(userIdList), examination.getMusicGroupId());

        studentLessonExaminations.forEach(studentLessonTrainingDetail -> {
            if (studentLessonTrainingDetail.getUserId() == null) {
                return;
            }
            StudentSubjectDto studentSubjectDto = subjectDtoMap.get(studentLessonTrainingDetail.getUserId());
            if (Objects.nonNull(studentSubjectDto)) {
                studentLessonTrainingDetail.setSubjectId(studentSubjectDto.getSubjectId());
            }
        });

        baseMapper.insertBatch(studentLessonExaminations);
        return studentIdList;
    }

    @Override
    public PageInfo<StudentLessonExaminationDto> queryPage(StudentLessonExaminationQueryInfo queryInfo) {
        PageInfo<StudentLessonExaminationDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        List<StudentLessonExaminationDto> dataList = null;
        int count = baseMapper.findCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = baseMapper.queryPage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void submit(LessonExaminationSubmitDto submitDto) {
        Long studentLessonExaminationDetailId = submitDto.getStudentLessonExaminationDetailId();
        StudentLessonExaminationDetail detail = studentLessonExaminationDetailService.getById(studentLessonExaminationDetailId);
        if(Objects.isNull(detail)){
            throw new BizException("训练内容不存在");
        }
        Long lessonExaminationId = detail.getLessonExaminationId();
        Integer userId = detail.getUserId();
        LessonExamination lessonExamination = lessonExaminationDao.selectById(lessonExaminationId);
        if(Objects.isNull(lessonExamination)){
            throw new BizException("训练内容不存在");
        }
        Date now = new Date();
        if(now.after(lessonExamination.getExpireDate())){
            throw new BizException("训练已截止");
        }
        StudentLessonExamination studentLessonExamination = this.lambdaQuery().eq(StudentLessonExamination::getLessonExaminationId, lessonExaminationId)
                .eq(StudentLessonExamination::getUserId, userId).one();
        if(studentLessonExamination.getTrainingTime() == null){
            //训练人数加一
            lessonExaminationDao.updateTrainingNum(lessonExaminationId);
        }
        studentLessonExamination.setTrainingFlag(true);
        studentLessonExamination.setTrainingTime(now);
        this.updateById(studentLessonExamination);
        //是否达标
        if(submitDto.getScore() >= detail.getStandardScore()){
            detail.setStandardFlag(true);
        }
        //训练分数是否比上一次高
        Boolean flag = submitDto.getScore() > detail.getTrainingScore();
        if(flag){
            detail.setTrainingScore(submitDto.getScore());
            //计算分数
            Double actualScore = detail.getTrainingScore() * 1D / detail.getStandardScore() * detail.getAvgScore();
            //更新分数
            detail.setActualAvgScore(actualScore.intValue() > detail.getAvgScore() ? detail.getAvgScore():actualScore.intValue());
        }
        //更新详情
        detail.setTrainingTime(now);
        studentLessonExaminationDetailService.updateById(detail);
        //更新达标人数
        lessonExaminationDao.updateStandardNum(lessonExaminationId);
        //更新总分
        if(flag){
            baseMapper.updateTrainingScore(lessonExaminationId,userId);
        }
        //更新服务指标
        studentExtracurricularExercisesSituationService.updateLastSubmitTime(lessonExamination,studentLessonExamination);
    }

    @Override
    public List<StudentLessonExaminationDto1> queryStudentPage(Long lessonExaminationId,Integer subjectId) {
        return baseMapper.queryStudentPage(lessonExaminationId,subjectId);
    }
}

