package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.SporadicOrderQueryInfo;
import com.ym.mec.biz.dal.page.StudentPaymentOrderQueryInfo;
import com.ym.mec.biz.dal.vo.StudentPaymentOrderVo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.entity.MallCreateOrderModel;
import com.ym.mec.common.entity.OrderCancelModel;
import com.ym.mec.common.entity.OrderSkuSync;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.page.WrapperUtil;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.adapay.ConfigInit;
import com.ym.mec.thirdparty.adapay.Payment;
import com.ym.mec.thirdparty.adapay.entity.BaseResult;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import com.ym.mec.thirdparty.yeepay.YeepayPaymentService;
import com.ym.mec.thirdparty.yqpay.*;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.DealStatusEnum.CLOSE;
import static com.ym.mec.biz.dal.enums.DealStatusEnum.SUCCESS;

@Service
public class StudentPaymentOrderServiceImpl extends BaseServiceImpl<Long, StudentPaymentOrder> implements StudentPaymentOrderService {

    @Resource
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Resource
    private YqPayFeignService yqPayFeignService;
    @Resource
    private StudentRegistrationService studentRegistrationService;
    @Resource
    @Lazy
    private VipGroupService vipGroupService;
    @Resource
    private MusicGroupService musicGroupService;
    @Resource
    private SporadicChargeInfoService sporadicChargeInfoService;
    @Resource
    private PracticeGroupService practiceGroupService;
    @Resource
    private StudentRepairService studentRepairService;
    @Resource
    private SysUserCashAccountService sysUserCashAccountService;
    @Resource
    private SubjectChangeService subjectChangeService;
    @Resource
    private DegreeRegistrationService degreeRegistrationService;
    @Resource
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;
    @Resource
    private IdGeneratorService idGeneratorService;
    @Resource
    private SysConfigDao sysConfigDao;
    @Resource
    private GoodsService goodsService;
    @Resource
    private StudentInstrumentService studentInstrumentService;
    @Resource
    private ReplacementInstrumentActivityService replacementInstrumentActivityService;
    @Resource
    private ChildrenDayReserveService childrenDayReserveService;
    @Resource
    private MemberRankSettingService memberRankSettingService;
    @Resource
    private PayService payService;
    @Resource
    private StudentPaymentRouteOrderService studentPaymentRouteOrderService;
    @Resource
    private CloudTeacherOrderService cloudTeacherOrderService;
    @Resource
    private SysCouponCodeService sysCouponCodeService;
    @Resource
    private VipGroupActivityService vipGroupActivityService;
    @Resource
    private ActivityUserMapperService activityUserMapperService;
    @Resource
    private MusicGroupSubjectPlanService musicGroupSubjectPlanService;
    @Resource
    private SysUserFeignService sysUserFeignService;
    @Resource
    private TenantConfigService tenantConfigService;
    @Resource
    private GoodsProcurementDao goodsProcurementDao;
    @Resource
    private SysPaymentConfigService sysPaymentConfigService;
    @Resource
    private SellOrderDao sellOrderDao;
    @Lazy
    @Resource
    private ContractService contractService;
    @Resource
    private HfMerchantConfigService hfMerchantConfigService;
    @Resource
    private StudentDao studentDao;
    @Resource
    private SysUserCashAccountDetailService sysUserCashAccountDetailService;
    @Resource
    private YeepayPaymentService yeepayPaymentService;
    @Resource
    private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
    @Resource
    private StudentRepairDao studentRepairDao;
    @Resource
    private MusicGroupDao musicGroupDao;
    @Resource
    private StudentGoodsSellDao studentGoodsSellDao;
    @Resource
    private StudentPaymentOrderService studentPaymentOrderService;
    @Resource
    private TempDirectorTrainingCampDetailService tempDirectorTrainingCampDetailService;
    @Resource
    private MusicGroupPaymentCalenderAddressService musicGroupPaymentCalenderAddressService;

    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    @Override
    public BaseDAO<Long, StudentPaymentOrder> getDAO() {
        return studentPaymentOrderDao;
    }

    private final static Logger LOGGER = LoggerFactory.getLogger(StudentPaymentOrderServiceImpl.class);

    @Override
    public StudentPaymentOrderDao getDao() {
        return studentPaymentOrderDao;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public Map createOrder(StudentPaymentOrder studentPaymentOrder, List<StudentPaymentOrderDetail> details, String payReceiver) throws Exception {

        BigDecimal cashAmount = studentPaymentOrder.getActualAmount();
        BigDecimal expectAmount = studentPaymentOrder.getExpectAmount();
        BigDecimal balance = studentPaymentOrder.getBalancePaymentAmount();

        if (cashAmount.add(balance).compareTo(expectAmount) != 0) {
            throw new BizException("支付金额不正确，请重试");
        }

        Date date = new Date();
        studentPaymentOrder.setUpdateTime(date);

        StudentPaymentOrder orignStudentPaymentOrder = get(studentPaymentOrder.getId());
        if (orignStudentPaymentOrder == null) {

            String orderNo = idGeneratorService.generatorId("payment") + "";
            studentPaymentOrder.setOrderNo(orderNo);
            studentPaymentOrder.setCreateTime(date);
            studentPaymentOrder.setUpdateTime(date);
            studentPaymentOrder.setVersion(0);
            insert(studentPaymentOrder);
        }

        if (CollectionUtils.isNotEmpty(details)) {
            for (StudentPaymentOrderDetail spod : details) {
                spod.setPaymentOrderId(studentPaymentOrder.getId());
                spod.setCreateTime(date);
                spod.setUpdateTime(date);
            }

            studentPaymentOrderDetailService.allocateAndAdd(details, studentPaymentOrder.getCouponRemitFee());
        }

        String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);

        //logger.info("XXX-createOrder ---> cashAmount:{}, balance:{}", cashAmount, cashAmount.compareTo(BigDecimal.ZERO));
        if (cashAmount.compareTo(BigDecimal.ZERO) == 0) {

            studentPaymentRouteOrderService.addRouteOrder(studentPaymentOrder.getOrderNo(), studentPaymentOrder.getOrganId(), balance);

            Map<String, String> notifyMap = new HashMap<>(4);
            notifyMap.put("tradeState", "1");
            notifyMap.put("merOrderNo", studentPaymentOrder.getOrderNo());
            notifyMap.put("channelType", "");
            notifyMap.put("orderNo", "");
            updateOrder(notifyMap);
            return notifyMap;
        }

        Map<String, Object> payMap = payService.getPayMap(cashAmount, balance, studentPaymentOrder.getOrderNo()
                , baseApiUrl
                + "/api-student/studentOrder/paymentResult?orderNo=" + studentPaymentOrder.getOrderNo(), studentPaymentOrder.getMemo(), studentPaymentOrder.getMemo(),
                studentPaymentOrder.getOrganId(), payReceiver);

        studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
        studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
        studentPaymentOrder.setUpdateTime(date);
        update(studentPaymentOrder);
        return payMap;
    }

    @Override
    public List<StudentPaymentOrder> queryByCondition(GroupType groupType, String musicGroupId, Integer userId, DealStatusEnum status, OrderTypeEnum type) {
        return studentPaymentOrderDao.queryByCondition(groupType, musicGroupId, userId, status, type);
    }

    @Override
    public List<StudentPaymentOrder> findMusicGroupApplyOrderByStatus(Integer userId, String musicGroupId, DealStatusEnum status) {
        return studentPaymentOrderDao.findMusicGroupApplyOrderByStatus(userId, musicGroupId, status);
    }

    @Override
    public StudentPaymentOrder findOrderByOrderNo(String orderNo) {
        return studentPaymentOrderDao.findOrderByOrderNo(orderNo);
    }

    @Override
	public List<StudentPaymentOrder> findOrderByOrderNos(List<String> orderNoList) {
        return studentPaymentOrderDao.findOrderByOrderNos(orderNoList);
	}

	@Override
    public List<StudentPaymentOrder> queryByBatchNo(Integer userId, String batchNo, DealStatusEnum status) {
        return studentPaymentOrderDao.queryByBatchNo(userId, batchNo, status);
    }

    @Override
    public List<StudentPaymentOrder> findOrdersByStatus(DealStatusEnum status) {
        return studentPaymentOrderDao.findOrdersByStatus(status);
    }

    @Override
    public List<StudentPaymentOrder> findOrdersOverTime(List<String> orderNoList, DealStatusEnum status, Date beforeTime) {
        return studentPaymentOrderDao.findOrdersOverTime(orderNoList, status, beforeTime);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public void queryOrderStatus() throws Exception {
        //yqPayQuery(null);
        adaPayQuery(null);
        //reConfirmOrder(null, null);
    }

    private void yqPayQuery(List<StudentPaymentOrder> payingOrders) throws Exception {
        if (payingOrders == null) {
            payingOrders = findOrdersByStatus(DealStatusEnum.ING);
        }
        if (payingOrders.size() == 0) {
            return;
        }
        List<String> orderNoList = payingOrders.stream().map(StudentPaymentOrder::getOrderNo).collect(Collectors.toList());
        String merOrderNos = payingOrders.stream().map(StudentPaymentOrder::getOrderNo).collect(Collectors.joining(","));

//        String notifyUrl = ""; //回调地址
        Map<String, Object> resultMap = new LinkedHashMap<>();
        resultMap.put("merOrderNoList", merOrderNos);
        Map<String, Object> requestMap = YqPayUtil.getRequestMap(resultMap);

        RsqMsg rsqMsg = new RsqMsg(requestMap);

        Msg queryRs = yqPayFeignService.orderQuery(rsqMsg);

        if (queryRs.getCode().equals("88")) {
            //更新订单状态
            String[] statusArr = {"0", "1", "7"};
            String responseParameters = queryRs.getResponseParameters();
            List<Map<String, String>> responseList = JSON.parseObject(responseParameters, List.class);
            for (Map<String, String> response : responseList) {
                Map<String, String> rpMap = response;
                String channelType = rpMap.get("channelType").equals("1") ? "WXPay" : (rpMap.get("channelType").equals("2") ? "Alipay" : "quickPay");
                rpMap.put("channelType", channelType);

                if (orderNoList.contains(rpMap.get("merOrderNo"))) {
                    orderNoList.remove(rpMap.get("merOrderNo"));
                }

                if (Arrays.asList(statusArr).contains(rpMap.get("tradeState"))) {
                    try {
                        updateOrder(rpMap); //更新订单
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    continue;
                }
            }
            closeOrders(orderNoList); //关闭订单
        }
    }

    private void adaPayQuery(List<StudentPaymentOrder> payingOrders) throws Exception {
        if (payingOrders == null) {
            payingOrders = findOrdersByStatus(DealStatusEnum.ING);
        }

        Date date = new Date();

        if (payingOrders.size() == 0) {
            return;
        }
        
        List<String> orderNoList = new ArrayList<String>();

        for (StudentPaymentOrder payingOrder : payingOrders) {
            //if (payingOrder.getTransNo() == null && "205".equals(payingOrder.getPaymentAccountNo()) && DateUtil.isSameDay(payingOrder.getCreateTime(), new Date())) {
            if (payingOrder.getTransNo() == null && "205".equals(payingOrder.getPaymentAccountNo())) {
                continue;
            }
            if (payingOrder.getTransNo() == null) {
                orderNoList.add(payingOrder.getOrderNo());
                continue;
            }
            //查询支付超过10分钟的
            if (DateUtil.minutesBetween(payingOrder.getCreateTime(), date) < 10) {
                continue;
            }
            try {
            	updateOrderStatus(date, payingOrder);
            }catch (Exception e) {
				logger.error("执行[订单状态查询]任务失败", e);
			}
        }
        closeOrders(orderNoList);
    }

    @Override
    @Transactional
    public OrderCancelModel updateOrderStatus(Date date, StudentPaymentOrder payingOrder) throws Exception {
    	
        HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(payingOrder.getTenantId(), payingOrder.getPaymentChannel());
        if(hfMerchantConfig == null){
            throw new BizException("机构[{}]汇付商户信息找不到", payingOrder.getTenantId());
        }
        
        PaymentChannelEnum paymentChannel = PaymentChannelEnum.codeOf(payingOrder.getPaymentChannel());
        
        OrderCancelModel  model = new OrderCancelModel();
        Map<String, Object> payment = new HashMap<String, Object>();

        if(paymentChannel == PaymentChannelEnum.ADAPAY) {
        	payment = Payment.queryPayment(payingOrder.getTransNo(), hfMerchantConfig.getMerKey());
        }else if(paymentChannel == PaymentChannelEnum.YEEPAY) {
        	Map<String, Object> orderInfo = yeepayPaymentService.queryOrder(hfMerchantConfig, payingOrder.getMerNos(), payingOrder.getOrderNo());
        	payment.put("pay_channel", orderInfo.get("channel"));
        	
        	String respStatus = orderInfo.get("status").toString();
        	
        	if(StringUtils.equals("SUCCESS", respStatus)) {
            	payment.put("status", "succeeded");
        	}else if(StringUtils.equals("FAIL", respStatus) || StringUtils.equals("TIME_OUT", respStatus) || StringUtils.equals("CLOSE", respStatus)) {
            	payment.put("status", "failed");
        	}else {
        		payment.put("status", "pending");
        	}
        	payment.put("error_msg", orderInfo.get("failReason"));
        }else {
        	throw new BizException("[{}]支付渠道不支持", payingOrder.getPaymentChannel());
        }
        
        LOGGER.warn("订单[{}]交易主动查询接口返回：{}", payingOrder.getTransNo(), payment);

        Map<String, String> rpMap = new HashMap<>();
        rpMap.put("merOrderNo", payingOrder.getOrderNo());
        rpMap.put("orderNo", payingOrder.getTransNo());
        rpMap.put("channelType", (String) payment.get("pay_channel"));
        model.setPayChannel((String) payment.get("pay_channel"));
        String status = (String) payment.get("status");
        if (payment.containsKey("error_msg")) {
            rpMap.put("remarks", (String) payment.get("error_msg"));
        }
        if (status.equals("succeeded")) {
            rpMap.put("tradeState", "1");
            model.setStatus(true);
        } else if (status.equals("failed")
                || (DateUtil.minutesBetween(DateUtil.addHours(payingOrder.getUpdateTime(), 2),date) > 0)) {
            rpMap.put("tradeState", "0");
            model.setStatus(false);
        }

        if (status.equals("succeeded") ||
                (status.equals("failed") || payingOrder.getCreateTime().before(DateUtil.addMinutes(new Date(), -35)))) {
            try {
                updateOrder(rpMap); //更新订单
                model.setSuccess(true);
            } catch (Exception e) {
                e.printStackTrace();
                model.setSuccess(false);
                return model;
            }
        }
        return model;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void quitCouponAndBalance(StudentPaymentOrder order) {
        //增加用户余额
        if (order.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
            sysUserCashAccountService.updateBalance(order.getUserId(), order.getBalancePaymentAmount(),
                    PlatformCashAccountDetailTypeEnum.REFUNDS, order.getMemo() + ",订单号:" + order.getOrderNo());
        }
        //退还优惠券
        sysCouponCodeService.quit(order.getCouponCodeId());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateOrder(Map<String, String> rpMap) throws Exception {
        DealStatusEnum status = "1".equals(rpMap.get("tradeState")) ? SUCCESS : DealStatusEnum.FAILED;
        StudentPaymentOrder order = findOrderByOrderNo(rpMap.get("merOrderNo"));
        //关闭或失败的订单查询订单成功,订单改成成功，钱退到余额
        if (order != null && (order.getStatus().equals(CLOSE) || order.getStatus().equals(DealStatusEnum.FAILED)) && status.equals(SUCCESS)) {
            String memo = order.getStatus().equals(CLOSE) ? "关闭订单" : "失败订单";
            memo = memo + ",实际支付成功,退到用户余额";
            //更新订单状态
            order.setStatus(status);
            order.setTransNo(rpMap.get("orderNo"));
            order.setPaymentBusinessChannel(rpMap.get("channelType"));
            order.setPayTime(new Date());
            order.setMemo(memo);
            int updateCount = this.update(order);
            if (updateCount <= 0) {
                throw new BizException("订单更新失败");
            }
            sysUserCashAccountService.updateBalance(order.getUserId(), order.getActualAmount(),
                    PlatformCashAccountDetailTypeEnum.REFUNDS, order.getMemo() + ",订单号:" + order.getOrderNo());

            if (StringUtils.isNotBlank(order.getPaymentChannel())
                    && (order.getPaymentChannel().equals(PaymentChannelEnum.ADAPAY.getCode()) || order.getPaymentChannel().equals(PaymentChannelEnum.YEEPAY.getCode()))
                    && !rpMap.containsKey("simulation")) {
                confirmOrder(order);
            }
            return;
        }

        if (order == null || !order.getStatus().equals(DealStatusEnum.ING)) {
            return;
        }

        if (status.equals(SUCCESS)) {
            order.setPayTime(new Date());
        } else {
            order.setMemo(rpMap.get("remarks"));
        }
        // 设置订单状态
        order.setStatus(status);
        if (StringUtils.isBlank(order.getTransNo())) {
            order.setTransNo(rpMap.get("orderNo"));
        }

        order.setPaymentBusinessChannel(rpMap.get("channelType"));
        order.setTenantId(this.get(order.getId()).getTenantId());

        TenantContextHolder.setTenantId(order.getTenantId());

        callOrderCallBack(order);

        if (status.equals(SUCCESS)
                && StringUtils.isNotBlank(order.getPaymentChannel())
                && (order.getPaymentChannel().equals(PaymentChannelEnum.ADAPAY.getCode()) || order.getPaymentChannel().equals(PaymentChannelEnum.YEEPAY.getCode()))
                && !rpMap.containsKey("simulation")) {
            confirmOrder(order);
        }
        TenantContextHolder.clearTenantId();
    }

    @Transactional(rollbackFor = Exception.class)
    public Boolean confirmOrder(StudentPaymentOrder order) {
        // 只有大雅延迟分账，其他实时分账
        if (order.getTenantId() != 1) {
            return true;
        }
        Date date = new Date();
        
        HfMerchantConfig hfMerchantConfig = null;
        
        List<StudentPaymentRouteOrder> routeOrders = studentPaymentRouteOrderService.getDao().getRouteOrders(order.getOrderNo());
        
        PaymentChannelEnum paymentChannel = PaymentChannelEnum.codeOf(order.getPaymentChannel());
        

    	if(hfMerchantConfig == null){
    		hfMerchantConfig = hfMerchantConfigService.queryByTenantId(order.getTenantId(), order.getPaymentChannel());
    	}
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}][{}]商户信息找不到", order.getTenantId(), order.getPaymentChannel());
        }
        
        StudentPaymentRouteOrder studentPaymentRouteOrder = null;
        
        if(paymentChannel == PaymentChannelEnum.ADAPAY) {
        	
            for (StudentPaymentRouteOrder routeOrder : routeOrders) {
                
                Map<String, Object> divMember = new HashMap<>();
                divMember.put("member_id", routeOrder.getMerNo());//分佣账户
                divMember.put("amount", routeOrder.getRouteAmount().setScale(2, BigDecimal.ROUND_HALF_UP));//分佣金额
                divMember.put("fee_flag", "Y");
                if (routeOrder.getMerNo().equals(ConfigInit.merNo)) {
                    divMember.put("member_id", 0);
                }
                List<Map<String, Object>> divMembers = new ArrayList<>();
                divMembers.add(divMember);

                Map<String, Object> confirm = new HashMap<>();
                confirm.put("payment_id", order.getTransNo());
                confirm.put("order_no", idGeneratorService.generatorId("payment"));
                confirm.put("confirm_amt", routeOrder.getRouteAmount());
                confirm.put("div_members", divMembers);
                try {
                    Map<String, Object> map = Payment.createConfirm(confirm, hfMerchantConfig.getMerKey());
                    LOGGER.info("分账信息返回：{}", map);
                    routeOrder.setServiceFee(new BigDecimal(map.get("fee_amt").toString()));
                    routeOrder.setUpdateTime(date);
                    studentPaymentRouteOrderService.getDao().update(routeOrder);
                } catch (Exception e) {
                	LOGGER.error("分账失败 orderId:{}", order.getId(), e);
                    continue;
                }
            }
			
		} else if (paymentChannel == PaymentChannelEnum.YEEPAY) {
			try {
				Map<String, Object> params = new HashMap<String, Object>();
				params.put("parentMerchantNo", hfMerchantConfig.getPlatformPayeeMemberId());
				params.put("merchantNo", hfMerchantConfig.getPlatformPayeeMemberId());
				params.put("orderId", order.getOrderNo());
				params.put("uniqueOrderNo", order.getTransNo());
				params.put("divideRequestId", idGeneratorService.generatorId("payment") + "");

				Map<String, Object> orderInfo = yeepayPaymentService.queryOrder(hfMerchantConfig, order.getMerNos(), order.getOrderNo());
				if (orderInfo == null) {
					throw new BizException("[{}][{}]订单查询失败", paymentChannel, order.getOrderNo());
				}
				BigDecimal splitAmount = new BigDecimal(orderInfo.get("unSplitAmount").toString());

				BigDecimal divideAmount = BigDecimal.ZERO;
				BigDecimal dividedAmount = BigDecimal.ZERO;
				List<Map<String,String>> divides = new ArrayList<Map<String,String>>();
				for (int i = 0; i < routeOrders.size(); i++) {

					studentPaymentRouteOrder = routeOrders.get(i);
					
					Map<String, String> divide = new HashMap<String, String>();

					divide.put("ledgerNo", studentPaymentRouteOrder.getMerNo());

					if (i + 1 == routeOrders.size()) {
						divideAmount = splitAmount.subtract(dividedAmount);
						divide.put("amount", divideAmount.toPlainString());
					} else {
						divideAmount = studentPaymentRouteOrder.getRouteAmount().multiply(splitAmount).divide(order.getActualAmount(), 2,
								BigDecimal.ROUND_HALF_UP);
						divide.put("amount", divideAmount.toPlainString());
					}
					dividedAmount = dividedAmount.add(divideAmount);

					studentPaymentRouteOrder.setServiceFee(studentPaymentRouteOrder.getRouteAmount().subtract(divideAmount));
					studentPaymentRouteOrder.setUpdateTime(date);
					
					divides.add(divide);
				}
				params.put("divideDetail", JSON.toJSONString(divides));

				Map<String, Object> result = yeepayPaymentService.divide(hfMerchantConfig, params);
				LOGGER.info("分账信息返回：{}", JSON.toJSONString(result));
			} catch (IOException e) {
				throw new BizException("调用易宝接口报错", e);
			}

			for (StudentPaymentRouteOrder spro : routeOrders) {

                studentPaymentRouteOrderService.getDao().update(spro);
			}
		}
        
        return true;
    }

    //调用相应业务回调接口
    @Override
    public void callOrderCallBack(StudentPaymentOrder order) throws Exception {
        if(order.getGroupType() == GroupType.ACTIVITY){
            vipGroupActivityService.orderCallback(order);
        }else {
            if (order.getType().equals(OrderTypeEnum.APPLY)) { //报名订单
                studentRegistrationService.updateApplyOrder(order);
            } else if (order.getType().equals(OrderTypeEnum.SMALL_CLASS_TO_BUY) || order.getType().equals(OrderTypeEnum.LIVE_GROUP_BUY)) {
                vipGroupService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.RENEW) || order.getType().equals(OrderTypeEnum.ADD_STUDENT)) {
                musicGroupService.renewForCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.SPORADIC) || order.getType().equals(OrderTypeEnum.LUCK) || order.getType().equals(OrderTypeEnum.DOUBLE_ELEVEN2020)) {
                sporadicChargeInfoService.renewForCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.PRACTICE_GROUP_BUY) || order.getType().equals(OrderTypeEnum.PRACTICE_GROUP_RENEW)) {
                practiceGroupService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.REPAIR)) {
                studentRepairService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.GOODS_SELL)) {
                studentRepairService.goodsSellOrderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.SUBJECT_CHANGE)) {
                subjectChangeService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.DEGREE_REGISTRATION)) {
                degreeRegistrationService.updateStatus(order);
            } else if (order.getType().equals(OrderTypeEnum.MAINTENANCE)) {
                studentInstrumentService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.REPLACEMENT)) {
                replacementInstrumentActivityService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.DEGREE)) {
                childrenDayReserveService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.MEMBER)) {
                //购买团练宝
                memberRankSettingService.orderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.ACTIVITY)) {
                //购买活动
                practiceGroupService.activityOrderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.LIVE_BUY)) {
                //直播课购买回调
                memberRankSettingService.liveBuyOrderCallback(order);
            } else if (order.getType().equals(OrderTypeEnum.MALL_BUY)) {
                // 商城购买订单回调
                studentPaymentOrderService.mallBuyOrderCallback(order);
            }else if (order.getType().equals(OrderTypeEnum.DIRECTOR_TRAINING_CAMP)) {
                tempDirectorTrainingCampDetailService.orderCallback(order);
            }
        }
    }

    private void closeOrders(List<String> orderNoList) throws Exception {
        if (orderNoList.size() == 0) {
            return;
        }

        Calendar beforeTime = Calendar.getInstance();
        beforeTime.add(Calendar.MINUTE, -35);// 30分钟之前的时间
        Date beforeDate = beforeTime.getTime();

        List<StudentPaymentOrder> ordersOverTime = findOrdersOverTime(orderNoList, DealStatusEnum.ING, beforeDate);
        for (StudentPaymentOrder order : ordersOverTime) {
            try {
                TenantContextHolder.setTenantId(order.getTenantId());
                order.setStatus(DealStatusEnum.FAILED);
                order.setMemo("超时未支付关闭");
                callOrderCallBack(order);
                TenantContextHolder.clearTenantId();
            } catch (Exception e) {
                e.printStackTrace();
                continue;
            }
        }

    }

    @Override
    public List<StudentPaymentOrderExportDto> ExportQueryPage(Map<String, Object> params) {
        return studentPaymentOrderDao.ExportQueryPage(params);
    }

    @Override
    public PageInfo<SporadicChargeInfoDto> sporadicQueryPage(SporadicOrderQueryInfo queryInfo) {
        PageInfo<SporadicChargeInfoDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<SporadicChargeInfoDto> dataList = null;
        int count = studentPaymentOrderDao.countSporadicPage(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentPaymentOrderDao.sporadicQueryPage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfoOrder<StudentPaymentOrderDto> OrderQueryPage(StudentPaymentOrderQueryInfo queryInfo) {
        PageInfoOrder<StudentPaymentOrderDto> pageInfo = new PageInfoOrder<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentPaymentOrderDto> dataList = new ArrayList<>();
        
        int jmCount = 0;
        if(queryInfo.getTenantId() == 28){
        	queryInfo.setOrganId(null);
        	jmCount = findCount(params);
            params.put("offset", pageInfo.getOffset());
        	dataList = studentPaymentOrderDao.queryPageOrder(params);
        	
        	queryInfo.setTenantId(1);
        	if(StringUtils.isBlank(queryInfo.getMerNos())){
        		queryInfo.setMerNos("H004236");
        	}
        	params.clear();
        	MapUtil.populateMap(params, queryInfo);
        }
        
        int count = this.findCount(params);
        count = count + jmCount;

        BigDecimal totalUserBalance = sysUserCashAccountService.getDao().getTotalUserBalance(params);
        pageInfo.setTotalUserBalance(totalUserBalance);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            
            if(dataList == null){
            	dataList = studentPaymentOrderDao.queryPageOrder(params);
            }else{
            	dataList.addAll(studentPaymentOrderDao.queryPageOrder(params));
            }

            StudentPaymentOrder orderMoneyAmount = studentPaymentOrderDao.getOrderMoneyAmount(params);
            BigDecimal totalExpectAmount = orderMoneyAmount.getExpectAmount() != null ? orderMoneyAmount.getExpectAmount() : BigDecimal.ZERO;
            BigDecimal totalActualAmount = orderMoneyAmount.getActualAmount() != null ? orderMoneyAmount.getActualAmount() : BigDecimal.ZERO;
            BigDecimal totalBalancePaymentAmount = orderMoneyAmount.getBalancePaymentAmount() != null ? orderMoneyAmount.getBalancePaymentAmount() : BigDecimal.ZERO;

            params.put("chargeType", SporadicChargeTypeEnum.RECHARGE);
            BigDecimal totalChargeAmount = studentPaymentOrderDao.getChargeAmount(params);
            totalChargeAmount = totalChargeAmount != null ? totalChargeAmount : BigDecimal.ZERO;
            BigDecimal totalAdvanceAmount = totalChargeAmount.subtract(totalBalancePaymentAmount);

            pageInfo.setTotalAdvanceAmount(totalAdvanceAmount);
            pageInfo.setTotalExpectAmount(totalExpectAmount);
            pageInfo.setTotalActualAmount(totalActualAmount);
            pageInfo.setTotalRevenueAmount(totalActualAmount.subtract(totalAdvanceAmount));
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public Boolean getOrderStatus(Long id) throws Exception {
        StudentPaymentOrder studentPaymentOrder = studentPaymentOrderDao.get(id);
        List<StudentPaymentOrder> orderList = new ArrayList<>();
        orderList.add(studentPaymentOrder);
        if (studentPaymentOrder.getPaymentChannel().equals(PaymentChannelEnum.ADAPAY.getCode()) || studentPaymentOrder.getPaymentChannel().equals(PaymentChannelEnum.YEEPAY.getCode())) {
            adaPayQuery(orderList);
        } else if (studentPaymentOrder.getPaymentChannel().equals("YQPAY")) {
            yqPayQuery(orderList);
        }
        return true;
    }

    @Override
    public Integer findOrganPayOrderNum(Integer organId) {
        return studentPaymentOrderDao.findOrganPayOrderNum(organId);
    }

    @Override
    public List<StudentPaymentOrder> getUserOrderByType(Integer userId, OrderTypeEnum type, DealStatusEnum status) {
        return studentPaymentOrderDao.getUserOrderByType(userId, type, status);
    }

    @Override
    public List<StudentPaymentOrder> reConfirmOrder(Date startTime, Date endTime) {
        if (startTime == null) {
            endTime = new Date();
            startTime = DateUtil.addMinutes(endTime, -20);
        }

        List<StudentPaymentOrder> orders = studentPaymentOrderDao.getAdaPayOrdersByTime(startTime, endTime);
        String tips = "";
        for (StudentPaymentOrder order : orders) {
            try {
                this.confirmOrder(order);
            } catch (Exception e) {
                tips += order.getOrderNo() + ":" + e.getMessage() + ",";
                continue;
            }
        }
        if (StringUtils.isNotBlank(tips)) {
            throw new BizException(tips);
        }
        return orders;
    }

    @Override
    public List<StudentPaymentOrderExportDto> getUserApplyOrders(Integer studentId, String musicGroupId) {
        List<StudentPaymentOrderExportDto> orders = studentPaymentOrderDao.getUserApplyOrders(studentId, musicGroupId);
        StudentRegistration studentRegister = studentRegistrationService.getStudentRegister(musicGroupId, studentId);
        for (StudentPaymentOrderExportDto order : orders) {
            if ((DealStatusEnum.ING.equals(order.getStatus()) || SUCCESS.equals(order.getStatus())) &&
                    studentRegister.getPayingStatus().equals(2)) {
                order.setPayingStatus(2);
            }
            if (order.getOrderDetailList() == null) {
                continue;
            }
            
            BigDecimal organShareProfit = BigDecimal.ZERO;
            for (StudentPaymentOrderDetail studentPaymentOrderDetail : order.getOrderDetailList()) {
                if (studentPaymentOrderDetail.getGoodsList() == null) {
                    continue;
                }

                String childGoodIds = "";
                for (Goods goods : studentPaymentOrderDetail.getGoodsList()) {
                    if (StringUtils.isNotBlank(goods.getComplementGoodsIdList())) {
                        childGoodIds += StringUtils.isNotBlank(childGoodIds) ? "," : "" + goods.getComplementGoodsIdList();
                    }
                }
                if (StringUtils.isNotBlank(childGoodIds)) {
                    studentPaymentOrderDetail.setChildGoodsList(goodsService.getGoodsDao().findGoodsByIds(childGoodIds));
                }
                if (studentPaymentOrderDetail.getType() == OrderDetailTypeEnum.CLOUD_TEACHER || studentPaymentOrderDetail.getType() == OrderDetailTypeEnum.CLOUD_TEACHER_PLUS) {
                    studentPaymentOrderDetail.setCloudTeacherOrderDto(cloudTeacherOrderService.queryOrderInfoByOrderId(studentPaymentOrderDetail.getPaymentOrderId()));
                }
                
                if (studentPaymentOrderDetail.getType() == OrderDetailTypeEnum.ORGAN_SHARE_PROFIT){
                	organShareProfit = organShareProfit.add(studentPaymentOrderDetail.getPrice());
                }
            }
            
            StudentPaymentOrderDetail spod = null;
            Iterator<StudentPaymentOrderDetail> iterator = order.getOrderDetailList().iterator();
            while(iterator.hasNext()){
            	spod = iterator.next();
            	if(spod.getType() == OrderDetailTypeEnum.MUSICAL){
            		spod.setPrice(spod.getPrice().add(organShareProfit));
            	}else if(spod.getType() == OrderDetailTypeEnum.ORGAN_SHARE_PROFIT){
            		iterator.remove();
            	}
            }
        }
        return orders;
    }

    @Override
    public StudentPaymentOrder getUserReplacementIngOrder(Integer userId, Integer replacementId) {
        return studentPaymentOrderDao.getUserReplacementIngOrder(userId, replacementId);
    }

    @Override
    public int batchUpdate(List<StudentPaymentOrder> studentPaymentOrderList) {
        return studentPaymentOrderDao.batchUpdate(studentPaymentOrderList);
    }

    @Override
    public PayStatus queryPayStatus(StudentPaymentOrder studentPaymentOrder) throws Exception {
    	
    	String paymentChannel = studentPaymentOrder.getPaymentChannel();
    	String orderNo = studentPaymentOrder.getOrderNo();
    	String transNo = studentPaymentOrder.getTransNo();
    	
        if (StringUtils.isBlank(transNo)) {
            return PayStatus.FAILED;
        }
        
    	HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(studentPaymentOrder.getTenantId(), studentPaymentOrder.getPaymentChannel());
        if(hfMerchantConfig == null){
        	throw new BizException("机构[{}]汇付商户信息找不到", studentPaymentOrder.getTenantId());
        }
        
        if (StringUtils.equals(paymentChannel, "ADAPAY")) {
            Map<String, Object> payment = Payment.queryPayment(transNo, hfMerchantConfig.getMerKey());
            String status = (String) payment.get("status");
            if (status.equals("succeeded")) {
                return PayStatus.SUCCESSED;
            } else if (status.equals("failed")) {
                return PayStatus.FAILED;
            } else {
                return PayStatus.PAYING;
            }
        } else if (StringUtils.equals(paymentChannel, "YQPAY")) {
//            String notifyUrl = ""; // 回调地址
            Map<String, Object> resultMap = new LinkedHashMap<>();
            resultMap.put("merOrderNoList", orderNo);
            Map<String, Object> requestMap = YqPayUtil.getRequestMap(resultMap);

            RsqMsg rsqMsg = new RsqMsg(requestMap);

            Msg queryRs = yqPayFeignService.orderQuery(rsqMsg);

            if (queryRs.getCode().equals("88")) {
                // 更新订单状态
                String responseParameters = queryRs.getResponseParameters();
                List<Map<String, String>> responseList = JSON.parseObject(responseParameters, List.class);
                for (Map<String, String> response : responseList) {

                    if (StringUtils.equals(response.get("tradeState"), "1")) {
                        return PayStatus.SUCCESSED;
                    } else if (StringUtils.equals(response.get("tradeState"), "0") || StringUtils.equals(response.get("tradeState"), "7")) {
                        return PayStatus.FAILED;
                    } else {
                        return PayStatus.PAYING;
                    }
                }
            }
        }else if (StringUtils.equals(paymentChannel, "YEEPAY")) {
        	Map<String, Object> result = yeepayPaymentService.queryOrder(hfMerchantConfig, studentPaymentOrder.getMerNos(), orderNo);
        	
        	if("OPR00000".equals(result.get("code").toString())) {

            	String status = result.get("status").toString();
            	if (status.equals("SUCCESS")) {
                    return PayStatus.SUCCESSED;
                } else if (status.equals("TIME_OUT") || status.equals("FAIL") || status.equals("CLOSE")) {
                    return PayStatus.FAILED;
                } else {
                    return PayStatus.PAYING;
                }
        	}else if("OPR12002".equals(result.get("code").toString())) {
        		//订单不存在
        		return PayStatus.FAILED;
        	}
            throw new BizException(result.get("message").toString());
        }
        throw new BizException("支付渠道错误");
    }

    @Override
    public Integer getOrderIdByMusical(String musicGroupId, Integer studentId) {
        return studentPaymentOrderDao.getOrderIdByMusical(musicGroupId, studentId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ActivityUserMapper activityGive(Integer vipGroupActivityId, StudentPaymentOrder studentPaymentOrder, String vipGroupId, String practiceGroupId) {
        ActivityUserMapper activityUserMapper = null;
        if (vipGroupActivityId != null) {
            VipGroupActivity activity = vipGroupActivityService.get(vipGroupActivityId);
            activityUserMapper = new ActivityUserMapper();
            activityUserMapper.setVipGroupId(vipGroupId);
            activityUserMapper.setPracticeGroupId(practiceGroupId);
            activityUserMapper.setPaymentOrderId(studentPaymentOrder.getId());
            activityUserMapper.setUserId(studentPaymentOrder.getUserId());
            activityUserMapper.setActivityId(vipGroupActivityId);
            activityUserMapper.setActualPrice(studentPaymentOrder.getExpectAmount());
            activityUserMapper.setAddMemo(activity.getName());
            activityUserMapper.setOrganId(studentPaymentOrder.getOrganId());
            //如果是课程活动
            if(activity.getActivityType().equals(0)){
                activityUserMapper.setSubNoCoursePrice(activityUserMapper.getActualPrice());
                activityUserMapper.setTotalCourseNum(studentPaymentOrder.getActivityBuyNum());
                activityUserMapper.setSubCourseNum(studentPaymentOrder.getActivityBuyNum());
                activityUserMapper.setSingleCourseTime(activity.getSingleCourseTime());
                //vip课
                if(StringUtils.isNotEmpty(activity.getVipGroupCategoryIdList())){
                    activityUserMapper.setVipFlag(1);
                    activityUserMapper.setCategoryId(Integer.parseInt(activity.getVipGroupCategoryIdList()));
                }else {
                    //网管课
                    activityUserMapper.setPracticeFlag(1);
                }
                if (vipGroupId != null) {
                    activityUserMapper.setVipFlag(2);
                    activityUserMapper.setSubNoCoursePrice(BigDecimal.ZERO);
                    activityUserMapper.setSubCourseNum(0);
                }
                if(practiceGroupId != null) {
                    activityUserMapper.setPracticeFlag(2);
                    activityUserMapper.setSubNoCoursePrice(BigDecimal.ZERO);
                    activityUserMapper.setSubCourseNum(0);
                }
            }else {
                CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
                cloudTeacherOrder.setStatus(1);
                cloudTeacherOrder.setAmount(activityUserMapper.getActualPrice());
                cloudTeacherOrder.setOrderId(studentPaymentOrder.getId());
                cloudTeacherOrder.setRemark(studentPaymentOrder.getMemo());
                cloudTeacherOrder.setLevel(activity.getMemberRankId());
                cloudTeacherOrder.setStudentId(studentPaymentOrder.getUserId());
                cloudTeacherOrder.setType(activity.getPeriodEnum());
                cloudTeacherOrder.setOrganId(studentPaymentOrder.getOrganId());
                cloudTeacherOrder.setTime(studentPaymentOrder.getActivityBuyNum());
                cloudTeacherOrderService.save(cloudTeacherOrder,tenantConfigService.autoActivationFlag(cloudTeacherOrder.getTenantId()));
                activityUserMapper.setMemberFlag(1);
                activityUserMapper.setMemberOrderId(cloudTeacherOrder.getId());
            }

            //是否达到满赠标准
            if (activity.getFullMinusCourseTimes() == -1 || studentPaymentOrder.getActivityBuyNum() < activity.getFullMinusCourseTimes()) {
                if(activity.getActivityType().equals(1)){
                    return null;
                }
                activityUserMapperService.insert(activityUserMapper);
                return activityUserMapper;
            }
            //是否赠送课程
            if (activity.getGiveCourseNum() > 0) {
                if (activity.getGiveCourseType().equals("VIP")) {
                    activityUserMapper.setGiveVipFlag(1);
                } else {
                    activityUserMapper.setGivePracticeFlag(1);
                }
                activityUserMapperService.copyProperty(activityUserMapper,activity,true,false);
            }
            //是否赠送优惠券
            if (activity.getGiveCouponNum() != null && activity.getGiveCouponNum() > 0) {
                activityUserMapper.setGiveCouponId(activity.getGiveCouponId());
            }
            //是否赠送会员(会员立即生效)
            if (activity.getGiveMemberTime() > 0) {
                // 查询会员订单信息
                CloudTeacherOrder cloudTeacherOrder = new CloudTeacherOrder();
                cloudTeacherOrder.setStatus(1);
                cloudTeacherOrder.setAmount(BigDecimal.ZERO);
                cloudTeacherOrder.setOrderId(studentPaymentOrder.getId());
                cloudTeacherOrder.setRemark(studentPaymentOrder.getMemo() + "赠送会员");
                cloudTeacherOrder.setLevel(activity.getGiveMemberRankId());
                cloudTeacherOrder.setStudentId(studentPaymentOrder.getUserId());
                cloudTeacherOrder.setType(activity.getGivePeriodEnum());
                cloudTeacherOrder.setOrganId(studentPaymentOrder.getOrganId());
                cloudTeacherOrder.setTime(activity.getGiveMemberTime());
                cloudTeacherOrderService.save(cloudTeacherOrder,tenantConfigService.autoActivationFlag(cloudTeacherOrder.getTenantId()));
                // 添加会员有效时长
                activityUserMapper.setGiveMemberFlag(1);
                activityUserMapper.setGiveMemberOrderId(cloudTeacherOrder.getId());
            }
            activityUserMapperService.insert(activityUserMapper);
        }
        return activityUserMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult checkRepeatPay(StudentPaymentOrder order, Boolean repeatPay) throws Exception {
        // 查询订单状态
        PayStatus payStatus = this.queryPayStatus(order);
        if (payStatus == PayStatus.SUCCESSED) {
            throw new BizException("订单已支付成功，请勿重复支付");
        }
        if (!repeatPay && payStatus == PayStatus.PAYING) {
            return BaseController.failed(HttpStatus.CONTINUE, "您有支付中的订单,是否继续支付");
        }
        //处理关闭订单
        order.setStatus(CLOSE);
        order.setMemo("关闭订单");

        
        Date nowDate = new Date();
        //增加商品库存
        List<StudentPaymentOrderDetail> studentPaymentOrderDetailList = studentPaymentOrderDetailService.queryOrderDetail(order.getId());
        String goodsIds = studentPaymentOrderDetailList.stream().filter(t -> StringUtils.isNotBlank(t.getMinuendStockGoodsIdList())).map(t -> t.getMinuendStockGoodsIdList()).collect(Collectors.joining(","));
        if(StringUtils.isNotBlank(goodsIds)){
        	List<Goods> goodsList = goodsService.getGoodsWithLocked(goodsIds);
        	Map<Integer,Goods> goodsMap = goodsList.stream().collect(Collectors.toMap(Goods :: getId, t -> t));
        	Goods goods = null;
			Map<Integer, Goods> batchUpdateGoodsMap = new HashMap<Integer, Goods>();

        	for(String goodsIdStr : goodsIds.split(",")){
        		if(StringUtils.isBlank(goodsIdStr)){
					continue;
				}
				goods = goodsMap.get(Integer.parseInt(goodsIdStr));
				if(batchUpdateGoodsMap.get(goods.getId()) != null){
					goods = batchUpdateGoodsMap.get(goods.getId());
				}
				goods.setSellCount(new AtomicInteger(goods.getSellCount()).decrementAndGet());
				goods.setUpdateTime(nowDate);
				
				batchUpdateGoodsMap.put(goods.getId(), goods);
        	}
            if(batchUpdateGoodsMap.size() > 0){
            	goodsService.batchUpdate(new ArrayList<Goods>(batchUpdateGoodsMap.values()));
            }
        }
        
        //如果是报名订单，订单关闭时需要更改已缴费学员数
        cutSubjectPlan(order);
        if (order.getBalancePaymentAmount() != null && order.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
            sysUserCashAccountService.updateBalance(order.getUserId(), order.getBalancePaymentAmount(),
                    PlatformCashAccountDetailTypeEnum.REFUNDS, "订单主动关闭退费");
        }
        studentPaymentOrderDao.update(order);
        sysCouponCodeService.quit(order.getCouponCodeId());
        return BaseController.succeed();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void cutSubjectPlan(StudentPaymentOrder order) {
        if (order.getType() == OrderTypeEnum.APPLY) {
            //如果是报名订单，订单关闭时需要更改已缴费学员数
            String musicGroupId = order.getMusicGroupId();
            Integer userId = order.getUserId();
            StudentRegistration studentRegistration = studentRegistrationService.getStudentRegister(musicGroupId, userId);
            //获取学员在乐团的状态，如果是在读，那么不处理人数
            if (studentRegistration.getMusicGroupStatus() == StudentMusicGroupStatusEnum.APPLY) {
                List<StudentPaymentOrder> oldStudentPaymentOrderList = this.queryByCondition(GroupType.MUSIC, musicGroupId, userId, SUCCESS, OrderTypeEnum.APPLY);
                //判断是否有成功的订单，如果有，表示已经处理过人数
                if (oldStudentPaymentOrderList == null || oldStudentPaymentOrderList.size() == 0) {
                    List<StudentPaymentOrderDetail> orderDetail = studentPaymentOrderDetailService.getOrderDetail(order.getId());
                    if (orderDetail != null && orderDetail.size() > 0) {
                        long count = orderDetail.stream().filter(e -> e.getType() == OrderDetailTypeEnum.MUSICAL || e.getType() == OrderDetailTypeEnum.CLOUD_TEACHER).count();
                        MusicGroup musicGroup = musicGroupService.get(musicGroupId);
                        //如果购买了乐器或者云教练，或者不是会员团，那么关闭订单时人数减一
                        if (count > 0 || musicGroup.getCourseViewType() != CourseViewTypeEnum.MEMBER_FEE) {
                            MusicGroupSubjectPlan musicOneSubjectClassPlan = musicGroupSubjectPlanService.getMusicOneSubjectClassPlan(musicGroupId, studentRegistration.getActualSubjectId());
                            musicOneSubjectClassPlan.setPaidStudentNum(musicOneSubjectClassPlan.getPaidStudentNum() == null ? 0 : musicOneSubjectClassPlan.getPaidStudentNum() - 1);
                            musicGroupSubjectPlanService.update(musicOneSubjectClassPlan);
                        }
                    }
                }
            }
        }
    }

    /**
     * 只查询成功的订单
     *
     * @param param
     */
    @Override
    public PageInfo<StudentPaymentOrderVo> queryOrder(Map<String, Object> param) throws Exception {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (Objects.isNull(user)) {
            throw new BizException("未查询到学员信息！");
        }
        param.put("userId", user.getId());
        String queryDateStr = WrapperUtil.toStr(param, "queryDate", "请选择查询日期！");
        Date startDate = DateUtils.parseDate(queryDateStr + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        Date endDate = DateUtils.addMonths(startDate, 1);
        param.put("startDate", startDate);
        param.put("endDate", endDate);
        Page<StudentPaymentOrderVo> pageInfo = PageUtil.getPageInfo(param);
        pageInfo.setDesc("a.create_time_");
        return PageUtil.pageInfo(studentPaymentOrderDao.queryStudentOrderPage(pageInfo, param));
    }

    /**
     * 查询订单明细
     *
     * @param orderNo 订单编号
     */
    @Override
    public StudentPaymentOrderVo queryOrderDetail(String orderNo) {
        Optional.ofNullable(orderNo)
                .map(this::findOrderByOrderNo)
                .orElseThrow(() -> new BizException("未查询到该订单数据!"));
        StudentPaymentOrderVo voList = studentPaymentOrderDao.queryOrderDetail(orderNo);
        if (Objects.nonNull(voList) && CollectionUtils.isNotEmpty(voList.getList())) {
            List<StudentPaymentOrderVo.StudentPaymentDetailVo> collect = voList.getList().stream()
                    .filter(d -> Objects.nonNull(d.getGoodsName()))
                    .collect(Collectors.toList());
            if (CollectionUtils.isEmpty(collect)) {
                voList.setList(null);
            } else {
                voList.setList(collect);
            }
        }
        return voList;
    }

    @Override
    public List<StudentPaymentOrder> queryByDealStatus(String musicGroupId, OrderTypeEnum orderType, DealStatusEnum status) {
        return studentPaymentOrderDao.queryByDealStatus(musicGroupId, orderType, status);
    }

    @Override
    public StudentPaymentOrder lockOrder(Long paymentOrderId) {
        return studentPaymentOrderDao.lockOrder(paymentOrderId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void returnBalanceAndCoupon(StudentPaymentOrder studentPaymentOrder, String memo) {
        if (studentPaymentOrder.getBalancePaymentAmount() != null && studentPaymentOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
            sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(),
                    PlatformCashAccountDetailTypeEnum.REFUNDS, memo);
        }
        sysCouponCodeService.quit(studentPaymentOrder.getCouponCodeId());
    }

    @Override
    public StudentPaymentOrder getMemberIngOrder(OrderTypeEnum member, DealStatusEnum status) {
        return studentPaymentOrderDao.getMemberIngOrder(member, status);
    }

    @Override
    @Transactional
    public Map createOrder(MallCreateOrderModel model) throws Exception {

        if(model == null ||model.getUserId() == null) {
            throw new BizException("参数错误");
        }
        StudentGoodsSell studentGoodsSell;
        if (StringUtils.isNotEmpty(model.getOldOrderNo())) {
            studentGoodsSell = studentGoodsSellDao.findByOrderNo(model.getOldOrderNo());
        }  else {
            studentGoodsSell = new StudentGoodsSell();
        }
        studentDao.lockUser(model.getUserId());
        SysUser student = studentDao.getUser(model.getUserId());
        Map payMap = new HashMap();
        if (student == null) {
            payMap.put("mallStatus",0);
            return payMap;
        }
        studentGoodsSell.setUserId(model.getUserId());
        studentGoodsSell.setTeacherId(model.getTeacherId() >0 ? model.getTeacherId():studentGoodsSell.getTeacherId());
        studentGoodsSell.setAuthorUser(model.getTeacherId() >0 ? model.getTeacherId():studentGoodsSell.getAuthorUser());
        //如果教务老师为空，代表学员自己创建的订单
        if (model.getTeacherId() == null && studentGoodsSell.getTeacherId() == null) {
            //获取学生关联的所有教务老师列表
            List<Integer> stuEducation = studentRepairDao.countStuEducation(model.getUserId());
            if (stuEducation != null && stuEducation.size() == 1) {
                //获取学员乐团关联的教务
                MusicGroup musicGroup = musicGroupDao.getStuMusic(model.getUserId());
                studentGoodsSell.setTeacherId(stuEducation.get(0));
                if (musicGroup != null) {
                    studentGoodsSell.setCooperationOrganId(musicGroup.getCooperationOrganId());
                }
            }
        } else if (studentGoodsSell.getCooperationOrganId() == null) {
            //获取教务老师，学员关联的乐团
            MusicGroup musicGroup = musicGroupDao.getStuEduMusic(model.getUserId(), studentGoodsSell.getTeacherId());
            if (musicGroup != null) {
                studentGoodsSell.setCooperationOrganId(musicGroup.getCooperationOrganId());
            }
        }


        // 余额支付
        BigDecimal amount =  BigDecimal.ZERO;
        BigDecimal balance = BigDecimal.ZERO;

        if (StringUtils.isNotEmpty(model.getOldOrderNo())) {
            StudentPaymentOrder orderByOrderNo = this.findOrderByOrderNo(model.getOldOrderNo());
            if (orderByOrderNo == null) {
                throw new BizException("未找到原订单");
            }
            amount = orderByOrderNo.getActualAmount();
            balance = orderByOrderNo.getBalancePaymentAmount();
            // 已有订单，先关单
            orderByOrderNo.setStatus(CLOSE);
            orderByOrderNo.setMemo("关闭订单");
            this.update(orderByOrderNo);
            //退优惠券
            sysCouponCodeService.quit(orderByOrderNo.getCouponCodeId());
        }


        Date date = new Date();
        StudentPaymentOrder studentPaymentOrder = sysCouponCodeService.use(model.getCouponIdList(), model.getAmount(),true);

        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        if (studentPaymentOrder.getActualAmount().compareTo(BigDecimal.ZERO) == 0) {
            studentPaymentOrder.setStatus(SUCCESS);
        }
        studentPaymentOrder.setGroupType(GroupType.MALL_SELL);
        studentPaymentOrder.setUserId(model.getUserId());
        studentPaymentOrder.setType(OrderTypeEnum.MALL_BUY);
        studentPaymentOrder.setOrderNo(model.getOrderNo());
        studentPaymentOrder.setCreateTime(date);
        studentPaymentOrder.setUpdateTime(date);
        if (model.isUseBalance()) {
            studentPaymentOrder.setPaymentChannel("BALANCE");
        } else {
            studentPaymentOrder.setPaymentChannel("ADAPAY");
        }
        studentPaymentOrder.setVersion(0);
        studentPaymentOrder.setOrganId(student.getOrganId());
        studentPaymentOrder.setRoutingOrganId(student.getOrganId());
        studentPaymentOrder.setTenantId(student.getTenantId());
        this.insert(studentPaymentOrder);
        studentPaymentOrder.setVersion(studentPaymentOrder.getVersion());

        // 余额支付
        if (model.isUseBalance() && studentPaymentOrder.getActualAmount().compareTo(BigDecimal.ZERO) > 0 ) {
            if (!StringUtils.isEmpty(model.getOldOrderNo())) {
                studentPaymentOrder.setActualAmount(amount);
                studentPaymentOrder.setBalancePaymentAmount(balance);
            } else {
                SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(model.getUserId());
                if (userCashAccount == null) {
                    throw new BizException("用户账户不存在");
                }
                amount = studentPaymentOrder.getActualAmount();
                if (userCashAccount.getBalance() != null && userCashAccount.getBalance().compareTo(BigDecimal.ZERO) > 0) {
                    balance = amount.compareTo(userCashAccount.getBalance()) >= 0 ? userCashAccount.getBalance() : amount;
                    amount = amount.subtract(balance);
                    studentPaymentOrder.setActualAmount(amount);
                    studentPaymentOrder.setBalancePaymentAmount(balance);
                    sysUserCashAccountService.updateBalance(model.getUserId(), balance.negate(),
                                                            PlatformCashAccountDetailTypeEnum.PAY_FEE, "商品销售");
                }
            }
        }
        this.update(studentPaymentOrder);
        studentPaymentOrder.setVersion(studentPaymentOrder.getVersion() + 1);

        studentGoodsSell.setCouponMarketAmount(studentPaymentOrder.getCouponRemitFee());
        studentGoodsSell.setOrganId(student.getOrganId());
        studentGoodsSell.setTotalAmount(studentPaymentOrder.getActualAmount());
        studentGoodsSell.setGoodsType("MALL");
        studentGoodsSell.setGoodsJson(model.getOrderInfo());
        studentGoodsSell.setOrderNo(model.getOrderNo());
        List<Integer> couponIdList = model.getCouponIdList();
        if (couponIdList != null && couponIdList.size() > 0) {
            studentGoodsSell.setCouponIds(StringUtils.join(couponIdList, ","));
        }
        studentGoodsSell.setTenantId(student.getTenantId());
        studentGoodsSellDao.insert(studentGoodsSell);


        if (studentPaymentOrder.getActualAmount().compareTo(BigDecimal.ZERO) == 0) {
            studentPaymentRouteOrderService.addRouteOrder(model.getOrderNo(), student.getOrganId(), studentPaymentOrder.getBalancePaymentAmount());
            Map<String, String> notifyMap = new HashMap<>();
            notifyMap.put("tradeState", "1");
            notifyMap.put("balanceAmount",studentPaymentOrder.getBalancePaymentAmount() == null?"0":studentPaymentOrder.getBalancePaymentAmount().toString());
            notifyMap.put("merOrderNo", studentPaymentOrder.getOrderNo());
            this.updateOrder(notifyMap);
            notifyMap.put("orderNo", model.getOrderNo());
            return notifyMap;
        }

        String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
        //获取支付渠道
        /*String paymentChannelStr = sysConfigDao.findConfigValue(SysConfigService.PAYMENT_CHANNEL);
        if(StringUtils.isBlank(paymentChannelStr)) {
            paymentChannelStr = PaymentChannelEnum.ADAPAY.getCode();
        }
        PaymentChannelEnum paymentChannel = PaymentChannelEnum.codeOf(paymentChannelStr);
        String notifyUrl = "";
        if (PaymentChannelEnum.YEEPAY == paymentChannel){
            notifyUrl = baseApiUrl + "/api-mall-portal/payment/callback_yee/" + studentPaymentOrder.getTenantId();
        }else if (PaymentChannelEnum.ADAPAY == paymentChannel){
            notifyUrl = baseApiUrl + "/api-mall-portal/payment/callback";
        }else {
            throw new BizException("支付渠道错误");
        }*/
        //  receiver类型
        payMap = payService.getPayMap(
                studentPaymentOrder.getActualAmount(),
                studentPaymentOrder.getBalancePaymentAmount(),
                model.getOrderNo(),
                baseApiUrl + "/api-student/studentOrder/paymentResult?type=edu&orderNo=" + model.getOrderNo(),
                "商品销售",
                "商品销售",
                student.getOrganId(),
                "mall_buy"
        );
        payMap.put("balanceAmount",studentPaymentOrder.getBalancePaymentAmount() == null?BigDecimal.ZERO:studentPaymentOrder.getBalancePaymentAmount());
        studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
        studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
        this.update(studentPaymentOrder);
        payMap.put("mallCouponAmount",studentPaymentOrder.getCouponRemitFee() == null?BigDecimal.ZERO:studentPaymentOrder.getCouponRemitFee());
        payMap.put("mallStatus",1);
        return payMap;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void mallBuyOrderCallback(StudentPaymentOrder studentPaymentOrder) {
        Date nowDate = new Date();
        //更新订单信息
        studentPaymentOrder.setUpdateTime(nowDate);
        int updateCount = this.update(studentPaymentOrder);
        if (updateCount <= 0) {
            throw new BizException("订单更新失败");
        }

        Integer userId = studentPaymentOrder.getUserId();

        Map<Integer, String> map = new HashMap<>();
        map.put(userId, userId.toString());

        if (studentPaymentOrder.getStatus() == SUCCESS) {
            try {
                contractService.transferProduceContract(userId, null, studentPaymentOrder.getType());
            } catch (Exception e) {
                logger.error("产品协议生成失败", e);
            }
            Long paymentOrderId = studentPaymentOrder.getId();
            Integer organId = studentPaymentOrder.getOrganId();
            String transNo = studentPaymentOrder.getTransNo();
            String orderNo = studentPaymentOrder.getOrderNo();
            BigDecimal balancePaymentAmount = studentPaymentOrder.getBalancePaymentAmount();
            //解析student_goods_sell字段goodsJson，生成订单详情和sellOrder
            StudentGoodsSellDto studentGoodsSellDto = studentGoodsSellDao.getStudentGoodsSellDto(studentPaymentOrder.getOrderNo());
            if(Objects.nonNull(studentGoodsSellDto)){
                if(StringUtils.isNotEmpty(studentGoodsSellDto.getGoodsJson())){
                    JSONObject jsonObject = JSON.parseObject(studentGoodsSellDto.getGoodsJson());
                    String orderItemList = jsonObject.getString("orderItemList");
                    if(StringUtils.isNotEmpty(orderItemList)){
                        List<GoodsOrderItemVO> goodsOrderItemVOS = JSON.parseArray(orderItemList, GoodsOrderItemVO.class);
                        long count = goodsOrderItemVOS.stream().filter(e -> StringUtils.isEmpty(e.getOrderSn())).count();
                        //如果有空的商品货号，不处理
                        if(count == 0l){
                            List<String> productSns = goodsOrderItemVOS.stream().map(e -> e.getProductSn()).distinct().collect(Collectors.toList());
                            List<Goods> goodsList = goodsService.getGoodsDao().findBySns(productSns);
                            //如果有不匹配的商品货号，不处理
                            if(CollectionUtils.isEmpty(goodsList) || goodsList.size() < productSns.size()){
                                return;
                            }
                            Map<String, Goods> goodsSnMap = goodsList.stream().collect(Collectors.groupingBy(e -> e.getSn(),
                                    Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
                            List<StudentPaymentOrderDetail> orderDetails = new ArrayList<>();
                            List<SellOrder> sellOrders = new ArrayList<>();
                            BigDecimal subjectBalance = balancePaymentAmount;
                            for (int i = 0; i < goodsOrderItemVOS.size(); i++) {
                                GoodsOrderItemVO goodsVo = goodsOrderItemVOS.get(i);
                                Goods goods = goodsSnMap.get(goodsVo.getProductSn());
                                for (int j = 0; j < goodsVo.getProductQuantity(); j++) {
                                    StudentPaymentOrderDetail detail = new StudentPaymentOrderDetail();
                                    detail.setType(OrderDetailTypeEnum.valueOf(goods.getType() == GoodsType.INSTRUMENT?"MUSICAL":goods.getType().getCode()));
                                    detail.setGoodsIdList(goods.getId().toString());
                                    detail.setPrice(goodsVo.getRealAmount());
                                    detail.setRemitFee(BigDecimal.ZERO);
                                    detail.setPaymentOrderId(paymentOrderId);
                                    orderDetails.add(detail);
                                }

                                BigDecimal totalAmount = goodsVo.getRealAmount().multiply(new BigDecimal(goodsVo.getProductQuantity()));
                                SellOrder sellOrder = new SellOrder();
                                sellOrder.setOrganId(organId);
                                sellOrder.setTransNo(transNo);
                                sellOrder.setOrderId(paymentOrderId);
                                sellOrder.setOrderNo(orderNo);
                                sellOrder.setExpectAmount(totalAmount);
                                if(goodsOrderItemVOS.size() - 1 == i){
                                    sellOrder.setBalanceAmount(subjectBalance);
                                }else {
                                    //获取比例
                                    BigDecimal ratioAmount = totalAmount.divide(studentGoodsSellDto.getExpectAmount(), 16, RoundingMode.DOWN);
                                    BigDecimal multiply = balancePaymentAmount.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                                    subjectBalance = subjectBalance.subtract(multiply);
                                    sellOrder.setBalanceAmount(multiply);
                                }
                                sellOrder.setActualAmount(totalAmount.subtract(sellOrder.getBalanceAmount()));

                                AccountType accountType = sysPaymentConfigService.checkAccountType(PaymentChannelEnum.valueOf(studentPaymentOrder.getPaymentChannel()), studentPaymentOrder.getMerNos(), studentPaymentOrder.getTenantId());
                                GoodsProcurement goodsProcurement = null;
                                if(StockType.INTERNAL.equals(goods.getStockType())||(StockType.ALL.equals(goods.getStockType())&&AccountType.INTERNAL.equals(accountType))){
                                    goodsProcurement = goodsProcurementDao.getWithStockSurplusProcurement(goods.getId());
                                    if(Objects.nonNull(goodsProcurement)){
                                        goodsProcurement.setStockSoldNum(new AtomicInteger(goodsProcurement.getStockSoldNum()).incrementAndGet());
                                    }
                                }else if(StockType.EXTERNAL.equals(goods.getStockType())||(StockType.ALL.equals(goods.getStockType())&&AccountType.EXTERNAL.equals(accountType))){
                                    goodsProcurement = goodsProcurementDao.getWithTaxStockSurplusProcurement(goods.getId());
                                    if(Objects.nonNull(goodsProcurement)){
                                        goodsProcurement.setTaxStockSoldNum(new AtomicInteger(goodsProcurement.getTaxStockSoldNum()).incrementAndGet());
                                    }
                                }

                                //如果有采购清单，初始化总部成本价
                                if(Objects.nonNull(goodsProcurement)){
                                    goodsProcurementDao.update(goodsProcurement);
                                    sellOrder.setBatchNo(goodsProcurement.getBatchNo());
                                    sellOrder.setSellCost(goodsProcurement.getDiscountPrice());
                                    Map<String, BigDecimal> CostMap = new HashMap<>();
                                    CostMap.put("sellCost", goodsProcurement.getDiscountPrice());
                                    if (Objects.nonNull(goodsProcurement.getAgreeCostPrice())) {
                                        CostMap.put("SellCost2", goodsProcurement.getAgreeCostPrice());
                                    }
                                    sellOrder.setSellCost2(JSON.toJSONString(CostMap));
                                    sellOrder.setBatchNo(goodsProcurement.getBatchNo());
                                    sellOrder.setOrganSellCost(goods.getOrganCostPrice());
                                }
                                sellOrder.setNum(goodsVo.getProductQuantity());
                                sellOrder.setUserId(userId);
                                sellOrder.setPaymentChannel(studentPaymentOrder.getPaymentChannel());
                                sellOrder.setMerNo(studentPaymentOrder.getMerNos());
                                sellOrder.setStockType(goods.getStockType());
                                sellOrder.setAccountType(accountType);
                                sellOrder.setStatus(SellStatus.NORMAL);
                                sellOrder.setSellTime(studentPaymentOrder.getPayTime());
                                sellOrder.setGoodsId(goods.getId());
                                sellOrder.setGoodsName(goods.getName());
                                sellOrder.setType(SellTypeEnum.valueOf(goods.getType().getCode()));
                                sellOrders.add(sellOrder);
                            }
                            sellOrderDao.batchInsert(sellOrders);
                            studentPaymentOrderDetailService.batchAdd(orderDetails);
                        }
                    }
                }
            }
            // 完全余额支付 不生成下面的记录
            if (BigDecimal.ZERO.compareTo(studentPaymentOrder.getActualAmount()) == 0) {
                return;
            }

            //插入交易明细
            BigDecimal amount = studentPaymentOrder.getActualAmount();
            SysUserCashAccount cashAccount = sysUserCashAccountService.get(userId);
            //充值
            SysUserCashAccountDetail rechargeDetail = new SysUserCashAccountDetail();
            rechargeDetail.setAmount(amount);
            rechargeDetail.setBalance(cashAccount.getBalance().add(amount));
            rechargeDetail.setComment("缴费前充值");
            rechargeDetail.setStatus(SUCCESS);
            rechargeDetail.setTransNo(studentPaymentOrder.getTransNo());
            rechargeDetail.setType(PlatformCashAccountDetailTypeEnum.RECHARGE);
            rechargeDetail.setUserId(userId);
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            if (studentPaymentOrder.getComAmount() != null) {
                rechargeDetail.setComAmount(studentPaymentOrder.getComAmount().negate());
                rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount().negate());
            }
            sysUserCashAccountDetailService.insert(rechargeDetail);
            //缴费
            SysUserCashAccountDetail paymentDetail = new SysUserCashAccountDetail();
            paymentDetail.setAmount(amount.negate());
            paymentDetail.setBalance(cashAccount.getBalance());
            paymentDetail.setComment("商品销售");
            paymentDetail.setStatus(SUCCESS);
            paymentDetail.setTransNo(studentPaymentOrder.getTransNo());
            paymentDetail.setType(PlatformCashAccountDetailTypeEnum.GOODS_SELL);
            paymentDetail.setUserId(userId);
            sysUserCashAccountDetailService.insert(paymentDetail);
        } else if (studentPaymentOrder.getStatus() == CLOSE || studentPaymentOrder.getStatus() == DealStatusEnum.FAILED) {

            if (studentPaymentOrder.getBalancePaymentAmount() != null && studentPaymentOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
                sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(), PlatformCashAccountDetailTypeEnum.REFUNDS, "商城购买支付失败");
            }
            sysCouponCodeService.quit(studentPaymentOrder.getCouponCodeId());
        }
    }

    @Override
    @Transactional
    public void mallBuyOrderCallback1(Long orderId) {
        StudentPaymentOrder studentPaymentOrder = this.get(orderId);

        Integer userId = studentPaymentOrder.getUserId();

        Map<Integer, String> map = new HashMap<>();
        map.put(userId, userId.toString());

            try {
                contractService.transferProduceContract(userId, null, studentPaymentOrder.getType());
            } catch (Exception e) {
                logger.error("产品协议生成失败", e);
            }
            Long paymentOrderId = studentPaymentOrder.getId();
            Integer organId = studentPaymentOrder.getOrganId();
            String transNo = studentPaymentOrder.getTransNo();
            String orderNo = studentPaymentOrder.getOrderNo();
            BigDecimal balancePaymentAmount = studentPaymentOrder.getBalancePaymentAmount();
            //解析student_goods_sell字段goodsJson，生成订单详情和sellOrder
            StudentGoodsSellDto studentGoodsSellDto = studentGoodsSellDao.getStudentGoodsSellDto(studentPaymentOrder.getOrderNo());
            if(Objects.nonNull(studentGoodsSellDto)){
                if(StringUtils.isNotEmpty(studentGoodsSellDto.getGoodsJson())){
                    JSONObject jsonObject = JSON.parseObject(studentGoodsSellDto.getGoodsJson());
                    String orderItemList = jsonObject.getString("orderItemList");
                    if(StringUtils.isNotEmpty(orderItemList)){
                        List<GoodsOrderItemVO> goodsOrderItemVOS = JSON.parseArray(orderItemList, GoodsOrderItemVO.class);
                        long count = goodsOrderItemVOS.stream().filter(e -> StringUtils.isEmpty(e.getOrderSn())).count();
                        //如果有空的商品货号，不处理
                        if(count == 0l){
                            List<String> productSns = goodsOrderItemVOS.stream().map(e -> e.getProductSn()).distinct().collect(Collectors.toList());
                            List<Goods> goodsList = goodsService.getGoodsDao().findBySns(productSns);
                            //如果有不匹配的商品货号，不处理
                            if(CollectionUtils.isEmpty(goodsList) || goodsList.size() < productSns.size()){
                                return;
                            }
                            Map<String, Goods> goodsSnMap = goodsList.stream().collect(Collectors.groupingBy(e -> e.getSn(),
                                    Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
                            List<StudentPaymentOrderDetail> orderDetails = new ArrayList<>();
                            List<SellOrder> sellOrders = new ArrayList<>();
                            BigDecimal subjectBalance = balancePaymentAmount;
                            for (int i = 0; i < goodsOrderItemVOS.size(); i++) {
                                GoodsOrderItemVO goodsVo = goodsOrderItemVOS.get(i);
                                Goods goods = goodsSnMap.get(goodsVo.getProductSn());
                                for (int j = 0; j < goodsVo.getProductQuantity(); j++) {
                                    StudentPaymentOrderDetail detail = new StudentPaymentOrderDetail();
                                    detail.setType(OrderDetailTypeEnum.valueOf(goods.getType() == GoodsType.INSTRUMENT?"MUSICAL":goods.getType().getCode()));
                                    detail.setGoodsIdList(goods.getId().toString());
                                    detail.setPrice(goodsVo.getRealAmount());
                                    detail.setRemitFee(BigDecimal.ZERO);
                                    detail.setPaymentOrderId(paymentOrderId);
                                    orderDetails.add(detail);
                                }

                                BigDecimal totalAmount = goodsVo.getRealAmount().multiply(new BigDecimal(goodsVo.getProductQuantity()));
                                SellOrder sellOrder = new SellOrder();
                                sellOrder.setOrganId(organId);
                                sellOrder.setTransNo(transNo);
                                sellOrder.setOrderId(paymentOrderId);
                                sellOrder.setOrderNo(orderNo);
                                sellOrder.setExpectAmount(totalAmount);
                                if(goodsOrderItemVOS.size() - 1 == i){
                                    sellOrder.setBalanceAmount(subjectBalance);
                                }else {
                                    //获取比例
                                    BigDecimal ratioAmount = totalAmount.divide(studentGoodsSellDto.getExpectAmount(), 16, RoundingMode.DOWN);
                                    BigDecimal multiply = balancePaymentAmount.multiply(ratioAmount).setScale(2, RoundingMode.DOWN);
                                    subjectBalance = subjectBalance.subtract(multiply);
                                    sellOrder.setBalanceAmount(multiply);
                                }
                                sellOrder.setActualAmount(totalAmount.subtract(sellOrder.getBalanceAmount()));

                                AccountType accountType = sysPaymentConfigService.checkAccountType(PaymentChannelEnum.valueOf(studentPaymentOrder.getPaymentChannel()), studentPaymentOrder.getMerNos(), studentPaymentOrder.getTenantId());
                                GoodsProcurement goodsProcurement = null;
                                if(StockType.INTERNAL.equals(goods.getStockType())||(StockType.ALL.equals(goods.getStockType())&&AccountType.INTERNAL.equals(accountType))){
                                    goodsProcurement = goodsProcurementDao.getWithStockSurplusProcurement(goods.getId());
                                    if(Objects.nonNull(goodsProcurement)){
                                        goodsProcurement.setStockSoldNum(new AtomicInteger(goodsProcurement.getStockSoldNum()).incrementAndGet());
                                    }
                                }else if(StockType.EXTERNAL.equals(goods.getStockType())||(StockType.ALL.equals(goods.getStockType())&&AccountType.EXTERNAL.equals(accountType))){
                                    goodsProcurement = goodsProcurementDao.getWithTaxStockSurplusProcurement(goods.getId());
                                    if(Objects.nonNull(goodsProcurement)){
                                        goodsProcurement.setTaxStockSoldNum(new AtomicInteger(goodsProcurement.getTaxStockSoldNum()).incrementAndGet());
                                    }
                                }

                                //如果有采购清单，初始化总部成本价
                                if(Objects.nonNull(goodsProcurement)){
                                    goodsProcurementDao.update(goodsProcurement);
                                    sellOrder.setBatchNo(goodsProcurement.getBatchNo());
                                    sellOrder.setSellCost(goodsProcurement.getDiscountPrice());
                                    Map<String, BigDecimal> CostMap = new HashMap<>();
                                    CostMap.put("sellCost", goodsProcurement.getDiscountPrice());
                                    if (Objects.nonNull(goodsProcurement.getAgreeCostPrice())) {
                                        CostMap.put("SellCost2", goodsProcurement.getAgreeCostPrice());
                                    }
                                    sellOrder.setSellCost2(JSON.toJSONString(CostMap));
                                    sellOrder.setBatchNo(goodsProcurement.getBatchNo());
                                    sellOrder.setOrganSellCost(goods.getOrganCostPrice());
                                }
                                sellOrder.setNum(goodsVo.getProductQuantity());
                                sellOrder.setUserId(userId);
                                sellOrder.setPaymentChannel(studentPaymentOrder.getPaymentChannel());
                                sellOrder.setMerNo(studentPaymentOrder.getMerNos());
                                sellOrder.setStockType(goods.getStockType());
                                sellOrder.setAccountType(accountType);
                                sellOrder.setStatus(SellStatus.NORMAL);
                                sellOrder.setSellTime(studentPaymentOrder.getPayTime());
                                sellOrder.setGoodsId(goods.getId());
                                sellOrder.setGoodsName(goods.getName());
                                sellOrder.setType(SellTypeEnum.valueOf(goods.getType().getCode()));
                                sellOrders.add(sellOrder);
                            }
                            sellOrderDao.batchInsert(sellOrders);
                            studentPaymentOrderDetailService.batchAdd(orderDetails);
                        }
                    }
                }
            }
    }

    @Override
    @Transactional
    public void mallSaveOrderInfo(MallCreateOrderModel model) {

        StudentGoodsSell studentGoodsSell = studentGoodsSellDao.findByOrderNo(model.getOrderNo());
        if (studentGoodsSell == null) {
            throw new BizException("未找到订单信息");
        }
        studentGoodsSell.setGoodsJson(model.getOrderInfo());
        studentGoodsSell.setGoodsType("MALL");
        studentGoodsSellDao.update(studentGoodsSell);
    }

    @Override
    @Transactional
    public OrderCancelModel cancelOrder(StudentPaymentOrder order, String reason) {
        OrderCancelModel model = new OrderCancelModel();
        model.setSuccess(true);
        try {
            HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(order.getTenantId(), order.getPaymentChannel());
            if(hfMerchantConfig == null){
                throw new BizException("机构[{}]汇付商户信息找不到", order.getTenantId());
            }
            Map<String, Object> payment = new HashMap<>();
            if(StringUtils.equals(order.getPaymentChannel(), PaymentChannelEnum.ADAPAY.getCode())) {
                payment = Payment.queryPayment(order.getTransNo(), hfMerchantConfig.getMerKey());
            }else if(StringUtils.equals(order.getPaymentChannel(), PaymentChannelEnum.YEEPAY.getCode())) {
                Map<String, Object> orderInfo = yeepayPaymentService.queryOrder(hfMerchantConfig, order.getMerNos(), order.getOrderNo());
                payment.put("pay_channel", orderInfo.get("channel"));

                String respStatus = orderInfo.get("status").toString();

                if(StringUtils.equals("SUCCESS", respStatus)) {
                    payment.put("status", "succeeded");
                }else if(StringUtils.equals("FAIL", respStatus) || StringUtils.equals("TIME_OUT", respStatus) || StringUtils.equals("CLOSE", respStatus)) {
                    payment.put("status", "failed");
                }else {
                    payment.put("status", "pending");
                }
                payment.put("error_msg", orderInfo.get("failReason"));
            }else {
                throw new BizException("[{}]支付渠道不支持", order.getPaymentChannel());
            }
            LOGGER.info("订单[{}]交易主动查询接口返回：{}", order.getTransNo(), payment);

            model.setPayChannel((String) payment.get("pay_channel"));
            String status = (String) payment.get("status");
            if (status.equals("succeeded")) {
                model.setStatus(true);
                order.setStatus(SUCCESS);
            } else if (status.equals("failed") ) {
                model.setStatus(false);
                order.setStatus(CLOSE);
                order.setMemo(reason);
            } else if (status.equals("pending")){
                // 执行关单操作
                Map<String, String> closeInfo = new HashMap<>();
                if(StringUtils.equals(order.getPaymentChannel(), PaymentChannelEnum.ADAPAY.getCode())) {
                    closeInfo = Payment.closeWithKey(order.getTransNo(), "", hfMerchantConfig.getMerKey());
                }else if(StringUtils.equals(order.getPaymentChannel(), PaymentChannelEnum.YEEPAY.getCode())) {
                    closeInfo = yeepayPaymentService.close(hfMerchantConfig, order.getTransNo(), "", order.getOrderNo());
                }
                boolean success = "success".equals(closeInfo.get("status"));
                model.setStatus(false);
                order.setStatus(CLOSE);
                model.setSuccess(success);
                model.setMessage(closeInfo.get("msg"));
            }
            if (model.getSuccess()) {
                TenantContextHolder.setTenantId(order.getTenantId());
                callOrderCallBack(order);
                TenantContextHolder.clearTenantId();
            }
        }catch (Exception e) {
            model.setSuccess(false);
            model.setMessage(e.getMessage());
            logger.error("订单[{}]取消失败", order.getTransNo(), e);
        }
        return model;
    }

    @Override
    @Transactional
    public BaseResult<Map<String, Object>> refund(String orderNo, String notifyUrl, String refundNo) {

        StudentPaymentOrder orderByOrderNo = this.findOrderByOrderNo(orderNo);
        if (orderByOrderNo == null) {
            throw new BizException("为找到订单");
        }
        if (orderByOrderNo.getActualAmount().compareTo(BigDecimal.ZERO) ==0) {
            throw new BizException("0元订单");
        }

        HfMerchantConfig hfMerchantConfig = hfMerchantConfigService.queryByTenantId(orderByOrderNo.getTenantId(), orderByOrderNo.getPaymentChannel());
        if(hfMerchantConfig == null){
            throw new BizException("请配置机构的汇付商户信息");
        }
        return Payment.reversePayment(orderByOrderNo.getTransNo(),hfMerchantConfig.getAppId(),refundNo,
                               orderByOrderNo.getActualAmount().setScale(2,BigDecimal.ROUND_HALF_UP).toString(),notifyUrl);
    }

    @Override
    public Map couponAmount(MallCreateOrderModel model) {
        Map payMap = new HashMap();
        StudentPaymentOrder studentPaymentOrder = sysCouponCodeService.use(model.getCouponIdList(), model.getAmount(),false);
        payMap.put("mallCouponAmount",studentPaymentOrder.getCouponRemitFee() == null?BigDecimal.ZERO:studentPaymentOrder.getCouponRemitFee());
        payMap.put("mallStatus",1);
        return payMap;
    }

    @Override
    @Transactional
    public void receive(String orderNo, boolean autoReceive) {

        StudentPaymentOrder orderByOrderNo = this.findOrderByOrderNo(orderNo);
        if (orderByOrderNo == null) {
            return;
        }
        // MANUAL_RECEIVE 手动确认,AUTO_RECEIVE 自动确认
        if (autoReceive) {
            orderByOrderNo.setReceiveStatus("AUTO_RECEIVE");
        } else {
            orderByOrderNo.setReceiveStatus("MANUAL_RECEIVE");
        }
        studentPaymentOrderDao.update(orderByOrderNo);
    }

    @Override
    public BigDecimal balance(Long userId) {
        SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(userId.intValue());
        if (userCashAccount == null) {
            throw new BizException("用户账户不存在");
        }
        return userCashAccount.getBalance() == null?BigDecimal.ZERO:userCashAccount.getBalance();
    }

    @Override
    public List<OperatingTotalIncomeDto> queryOperatingSummaryIncome(Map<String, Object> params) {
        List<StudentPaymentOrderExportDto> studentPaymentOrderExportDtos = studentPaymentOrderDao.ExportQueryPage1(params);
        studentPaymentOrderExportDtos.addAll(studentPaymentOrderDao.ExportQueryPage2(params));
        long i = 1;
        //获取机构费率
        List<Long> otherOrderIds = studentPaymentOrderExportDtos.stream().filter(e -> e.getGroupType() != GroupType.GOODS_SELL
                && e.getGroupType() != GroupType.REPLACEMENT
                && e.getGroupType() != GroupType.PRACTICE
                && e.getGroupType() != GroupType.SPORADIC
                && e.getGroupType() != GroupType.MEMBER
                && e.getGroupType() != GroupType.ACTIVITY
                && e.getGroupType() != GroupType.VIP
                && e.getGroupType() != GroupType.LIVE).map(e -> e.getId()).collect(Collectors.toList());
        Map<Long, List<StudentPaymentOrderExportDto>> feeByTypeMap = new HashMap<>();
        Map<Long, BigDecimal> childRepairMap = new HashMap<>();
        if (!org.springframework.util.CollectionUtils.isEmpty(otherOrderIds)) {
            List<StudentPaymentOrderExportDto> feeByType = sellOrderDao.queryFeeByType(otherOrderIds);
            feeByTypeMap = feeByType.stream().collect(Collectors.groupingBy(StudentPaymentOrderExportDto::getId));
            childRepairMap = MapUtil.convertIntegerMap(sellOrderDao.queryChildRepair(otherOrderIds));
        }
        //获取外部订单,将学校商品采购划分为大件乐器
        Map<String, StudentPaymentRouteOrder> routeOrderMap = new HashMap<>();
        List<String> outOrderNos = studentPaymentOrderExportDtos.stream().filter(e -> e.getGroupType() == GroupType.OUTORDER && e.getType() == OrderTypeEnum.SCHOOL)
        .map(e -> e.getOrderNo()).collect(Collectors.toList());
        if(CollectionUtils.isNotEmpty(outOrderNos)){
            List<StudentPaymentRouteOrder> routeOrders = studentPaymentRouteOrderService.getDao().queryByOrderNos(outOrderNos);
            if (CollectionUtils.isNotEmpty(routeOrders)){
                routeOrderMap = routeOrders.stream().filter(e->e.getSaleAmount().compareTo(BigDecimal.ZERO) > 0)
                        .collect(Collectors.toMap(StudentPaymentRouteOrder::getOrderNo, Function.identity()));
            }
        }
        //阳光、善学、坚韧、做更好的自己
//        List<Integer> organIds = studentPaymentOrderExportDtos.stream().map(e -> e.getOrganId()).distinct().collect(Collectors.toList());
        Map<Integer, String> userOrganNameMap = this.getMap("organization","id_","name_",true,1,Integer.class,String.class);
        //按分部分组
        Map<Integer, List<StudentPaymentOrderExportDto>> listMap = studentPaymentOrderExportDtos.stream().collect(Collectors.groupingBy(e -> e.getOrganId()));
        List<OperatingTotalIncomeDto> incomeDtos = new ArrayList<>();
        //承担课程的云教练收入
        String month = params.get("month").toString();
        String firstDayOfMonth = DateUtil.dateToString(DateUtil.getFirstDayOfMonth(DateUtil.stringToDate(month,DateUtil.ISO_YEAR_MONTH_FORMAT)),DateUtil.ISO_EXPANDED_DATE_FORMAT);
        String lastDayOfMonth = DateUtil.dateToString(DateUtil.getLastDayOfMonth(DateUtil.stringToDate(month,DateUtil.ISO_YEAR_MONTH_FORMAT)),DateUtil.ISO_EXPANDED_DATE_FORMAT);

        String cutMonth = DateUtil.dateToString(DateUtil.addMonths(DateUtil.stringToDate(month,DateUtil.ISO_YEAR_MONTH_FORMAT),-1),DateUtil.ISO_YEAR_MONTH_FORMAT);
        String cutFirstDayOfMonth = DateUtil.dateToString(DateUtil.getFirstDayOfMonth(DateUtil.stringToDate(cutMonth,DateUtil.ISO_YEAR_MONTH_FORMAT)),DateUtil.ISO_EXPANDED_DATE_FORMAT);
        String cutLastDayOfMonth = DateUtil.dateToString(DateUtil.getLastDayOfMonth(DateUtil.stringToDate(cutMonth,DateUtil.ISO_YEAR_MONTH_FORMAT)),DateUtil.ISO_EXPANDED_DATE_FORMAT);
        Map<Integer, BigDecimal> groupCloudAmount = MapUtil.convertIntegerMap(cloudTeacherOrderService.getDao().sumGroupCloudAmount(month,firstDayOfMonth,lastDayOfMonth));
        //不承担课程的云教练收入
        Map<Integer, BigDecimal> personalCloudAmount = MapUtil.convertIntegerMap(cloudTeacherOrderService.getDao().sumPersonalCloudAmount(month,firstDayOfMonth,lastDayOfMonth));
        List<Map> courseFeeStat = courseScheduleStudentPaymentDao.statCourseFee(cutFirstDayOfMonth,cutLastDayOfMonth);
        Map<Integer, BigDecimal> courseIncome = courseFeeStat.stream().collect(Collectors.toMap(s -> Integer.parseInt(s.get("key").toString()), s -> BigDecimal.valueOf(Double.parseDouble(s.get("value").toString()))));
        for (Integer organId : userOrganNameMap.keySet()) {
            OperatingTotalIncomeDto incomeDto = new OperatingTotalIncomeDto();
            incomeDto.setOrganId(organId);
            incomeDto.setOrganName(userOrganNameMap.get(organId));
            List<StudentPaymentOrderExportDto> rows = listMap.get(organId);
            if(CollectionUtils.isNotEmpty(rows)){
                for (StudentPaymentOrderExportDto row : rows) {
                    if (row.getActualAmount() == null) {
                        row.setActualAmount(BigDecimal.ZERO);
                    }
                    if (row.getBalancePaymentAmount() == null) {
                        row.setBalancePaymentAmount(BigDecimal.ZERO);
                    }
                    if (row.getOrderDetailList() != null && GroupType.GOODS_SELL != row.getGroupType()) {
                        for (StudentPaymentOrderDetail orderDetail : row.getOrderDetailList()) {
                            switch (orderDetail.getType()) {
                                case COURSE:
                                case CLASSROOM:
                                case SINGLE:
                                case MIX:
                                case COMPREHENSIVE:
                                case ENLIGHTENMENT:
                                case TRAINING_SINGLE:
                                case TRAINING_MIX:
                                case MUSIC_NETWORK:
                                case HIGH:
                                case HIGH_ONLINE:
                                case HIGH_ONLINE_COURSE:
                                    row.setMusicGroupCourseFee(row.getMusicGroupCourseFee().add(orderDetail.getPrice()));
                                    break;
                                case MUSICAL:
                                    if (orderDetail.getKitGroupPurchaseType() != null && orderDetail.getKitGroupPurchaseType().equals(KitGroupPurchaseTypeEnum.LEASE)) {
                                        row.setLeaseFee(getActualAmount(orderDetail.getPrice(), row));
                                    } else {
                                        row.setMusicalFee(orderDetail.getPrice());
                                    }
                                    break;
                                case ACCESSORIES:
                                case TEACHING:
                                    row.setTeachingFee(row.getTeachingFee().add(orderDetail.getPrice()));
                                    break;
                                case OTHER:
                                    row.setOtherFee(row.getOtherFee().add(orderDetail.getPrice()));
                                    break;
                                case MAINTENANCE:
                                    row.setMaintenanceFee(getActualAmount(orderDetail.getPrice(), row));
                                    break;
                                case CLOUD_TEACHER:
                                case CLOUD_TEACHER_PLUS:
                                    row.setCloudTeacherFee(getActualAmount(orderDetail.getPrice(), row));
                                    break;
                                case DEGREE_REGISTRATION:
                                    row.setDegreeFee(getActualAmount(orderDetail.getPrice(), row));
                                    break;
                                case PRACTICE:
                                    row.setPracticeCourseFee(getActualAmount(orderDetail.getPrice(), row));
                                    break;
                                case VIP:
                                    row.setVipCourseFee(getActualAmount(orderDetail.getPrice(), row));
                                    break;
                                case THEORY_COURSE:
                                    row.setTheoryCourseFee(getActualAmount(orderDetail.getPrice(), row));
                                    break;
                                case REPAIR:
                                    row.setRepairFee(getActualAmount(orderDetail.getPrice(), row));
                                    break;
                                default:
                                    break;
                            }
                        }
                    }
                    //专业
                    if (row.getGroupType().equals(GroupType.VIP)) {
                        row.setVipCourseFee(row.getActualAmount());
                    } else if (row.getGroupType() == GroupType.ACTIVITY) {
                        row.setVipCourseFee(row.getActualAmount());
                    } else if (row.getGroupType() == GroupType.LIVE) {
                        row.setLiveCourseFee(row.getActualAmount());
                    } else if (row.getGroupType() == GroupType.MEMBER) {
                        row.setCloudTeacherFee(row.getActualAmount());
                    } else if (row.getGroupType().equals(GroupType.SPORADIC)) {
                        //考级报名
                        if (row.getType().equals(OrderTypeEnum.DEGREE_REGISTRATION)) {
                            if(StringUtils.isEmpty(row.getMemo())){
                                row.setDegreeFee(row.getActualAmount());
                            }else {
                                String[] feeTypes = row.getMemo().split("；");
                                List<String> feeTypeList = Arrays.asList(feeTypes);
                                for (String feeTypeStr : feeTypeList) {
                                    int index = feeTypeStr.lastIndexOf("：");
                                    if (index < 0) {
                                        continue;
                                    }
                                    BigDecimal typeFee = new BigDecimal(feeTypeStr.substring(index + 1));
                                    if (row.getExpectAmount().compareTo(BigDecimal.ZERO) > 0) {
                                        typeFee = typeFee.multiply(row.getActualAmount()).divide(row.getExpectAmount(), 2, BigDecimal.ROUND_DOWN);
                                    }
                                    if (feeTypeStr.contains("VIP")) {
                                        row.setVipCourseFee(typeFee);
                                    } else if (feeTypeStr.contains("网管课")) {
                                        row.setPracticeCourseFee(typeFee);
                                    } else if (feeTypeStr.contains("小组课")) {
                                        row.setTheoryCourseFee(typeFee);
                                    }
                                }
                                row.setDegreeFee(row.getActualAmount().subtract(row.getVipCourseFee()).subtract(row.getPracticeCourseFee()).subtract(row.getTheoryCourseFee()));
                            }
                        } else {
                            if (row.getChargeType() != null && SporadicChargeTypeEnum.LEVEL.getCode().equals(row.getChargeType())) {
                                row.setDegreeFee(row.getActualAmount());
                            } else if (row.getChargeType() != null && SporadicChargeTypeEnum.PRACTICE_GROUP_BUY.getCode().equals(row.getChargeType())) {
                                row.setPracticeCourseFee(row.getActualAmount());
                            } else if (row.getChargeType() != null && SporadicChargeTypeEnum.MUSIC_UPKEEP.getCode().equals(row.getChargeType())) {
                                row.setMaintenanceFee(row.getActualAmount());
                            } else if (row.getChargeType() != null && SporadicChargeTypeEnum.VISITING_FEE.getCode().equals(row.getChargeType())) {
                                row.setVisitFee(row.getActualAmount());
                            } else if (row.getChargeType() != null && (SporadicChargeTypeEnum.RECHARGE.getCode().equals(row.getChargeType()) ||
                                    SporadicChargeTypeEnum.HIGH_ONLINE_ACTIVITY.getCode().equals(row.getChargeType()))) {
                                row.setRechargeFee(row.getActualAmount());
                            } else if (row.getChargeType() != null && SporadicChargeTypeEnum.CLOUD_TEACHER_BUY.getCode().equals(row.getChargeType())) {
                                row.setCloudTeacherFee(row.getActualAmount());
                            } else {
                                row.setOtherFee(row.getActualAmount());
                            }
                        }
                    } else if (row.getGroupType().equals(GroupType.PRACTICE)) {
                        row.setPracticeCourseFee(row.getActualAmount());
                    } else if (row.getGroupType().equals(GroupType.REPLACEMENT)) {
                        row.setMusicalFee(row.getActualAmount());
                    } else if (row.getGroupType().equals(GroupType.GOODS_SELL)) {
                        row.setRetailGoodsFee(row.getActualAmount());
                    }else {
                        List<StudentPaymentOrderExportDto> exportDtoList = feeByTypeMap.get(row.getId());
                        if (!org.springframework.util.CollectionUtils.isEmpty(exportDtoList)) {
                            StudentPaymentOrderExportDto feeByType = exportDtoList.get(0);
                            BigDecimal childRepairFee = childRepairMap.get(row.getId()) == null ? BigDecimal.ZERO : childRepairMap.get(row.getId());
                            row.setMusicalFee(feeByType.getMusicalFee());
                            row.setTeachingFee(feeByType.getTeachingFee().subtract(childRepairFee));
                            row.setMaintenanceProductFee(feeByType.getRepairFee().add(childRepairFee));
                            row.setOtherFee(feeByType.getOtherFee());
                            if (row.getGroupType().equals(GroupType.MUSIC)) {
                                row.setMusicGroupCourseFee(feeByType.getActualAmount().subtract(feeByType.getMusicalFee()).
                                        subtract(row.getTeachingFee()).subtract(row.getMaintenanceFee()).subtract(row.getMaintenanceProductFee()).
                                        subtract(feeByType.getOtherFee()).subtract(row.getLeaseFee()).subtract(row.getCloudTeacherFee()).
                                        subtract(row.getVipCourseFee()).subtract(row.getPracticeCourseFee()));
                            } else if (StringUtils.equals(row.getGroupType().getCode(),"REPAIR")) {
                                row.setRepairFee(feeByType.getActualAmount().subtract(feeByType.getMusicalFee()).subtract(feeByType.getTeachingFee()).subtract(feeByType.getOtherFee()));
                            } else if (row.getGroupType() == GroupType.OUTORDER) {
                                //销售收入
                                BigDecimal add = feeByType.getMusicalFee().add(row.getTeachingFee()).add(row.getMaintenanceProductFee()).add(feeByType.getOtherFee());
                                //服务收入
                                BigDecimal subtract = feeByType.getActualAmount().subtract(add);
                                //如果是淘器微信那么就是零售
                                if (Objects.equals("淘器微信", row.getMerNos()) && add.compareTo(BigDecimal.ZERO) > 0) {
                                    //如果有服务收入那么是课程学校采买
                                    row.setMusicalFee(BigDecimal.ZERO);
                                    row.setTeachingFee(BigDecimal.ZERO);
                                    row.setMaintenanceProductFee(BigDecimal.ZERO);
                                    row.setOtherFee(subtract);
                                    row.setRetailGoodsFee(add);
                                } else {
                                    //拆分导入订单
                                    if (row.getType() == OrderTypeEnum.OUTORDER || row.getType() == OrderTypeEnum.SCHOOL) {
                                        if(routeOrderMap.get(row.getOrderNo()) != null){
                                            row.setLargeMusicalFee(subtract);
                                        }else {
                                            if (row.getType() == OrderTypeEnum.SCHOOL) {
                                                row.setMusicalFee(BigDecimal.ZERO);
                                                row.setTeachingFee(BigDecimal.ZERO);
                                                row.setMaintenanceProductFee(BigDecimal.ZERO);
                                                row.setOtherFee(BigDecimal.ZERO);
                                                row.setLargeMusicalFee(add);
                                            }
                                            row.setCourseSchoolBuyAmount(subtract);
                                        }
                                    } else if (row.getType() == OrderTypeEnum.OTHER) {
                                        row.setOtherFee(row.getOtherFee().add(subtract));
                                    } else {
                                        row.setMusicalFee(BigDecimal.ZERO);
                                        row.setTeachingFee(BigDecimal.ZERO);
                                        row.setMaintenanceProductFee(BigDecimal.ZERO);
                                        row.setOtherFee(BigDecimal.ZERO);
                                        row.setRetailGoodsFee(add);
                                        row.setMusicGroupCourseFee(subtract);
                                    }
                                }
                                row.setTransferFee(BigDecimal.ZERO);
                                row.setPlatformFee(BigDecimal.ZERO);
                            }
                        }
                    }
                    row.setId(i);
                    row.setRepairFee(row.getRepairFee().add(row.getMaintenanceProductFee()));
                    row.setOrderAmount(row.getExpectAmount().add(row.getCouponRemitFee()));
                    i++;
                    //学校课程采买
                    incomeDto.setCoursePurchaseIncome(incomeDto.getCoursePurchaseIncome()
                            .add(row.getCourseSchoolBuyAmount()));
                    //考级
                    incomeDto.setExamIncome(incomeDto.getExamIncome()
                            .add(row.getDegreeFee()));
                    //维修费用
                    incomeDto.setMaintenanceCost(incomeDto.getMaintenanceCost()
                            .add(row.getRepairFee()));
                    //其他服务收入
                    incomeDto.setOtherServiceIncome(incomeDto.getOtherServiceIncome()
                            .add(row.getLeaseFee())
                            .add(row.getVisitFee())
                            .add(row.getRechargeFee())
                            .add(row.getOtherFee())
                    );
                    //团购乐器
                    incomeDto.setGroupPurchaseInstruments(incomeDto.getGroupPurchaseInstruments()
                            .add(row.getMusicalFee()));
                    //零售乐器,计入其他
                    incomeDto.setOtherIncome(incomeDto.getOtherIncome()
                            .add(row.getRetailGoodsFee()));
                    if(row.getGroupType() == GroupType.GOODS_SELL){
                        //零售教辅,计入其他
                        incomeDto.setOtherIncome(incomeDto.getOtherIncome()
                                .add(row.getTeachingFee()));
                    }else {
                        //团购教辅
                        incomeDto.setGroupPurchaseTeachingAids(incomeDto.getGroupPurchaseTeachingAids()
                                .add(row.getTeachingFee()));
                    }
                    //区分零售月保
                    if(row.getGroupType() == GroupType.MAINTENANCE){
                        //零售月保,计入其他
                        incomeDto.setOtherIncome(incomeDto.getOtherIncome()
                                .add(row.getMaintenanceFee()));
                    }else {
                        //团购月保
                        incomeDto.setInstrumentInsuranceCost(incomeDto.getInstrumentInsuranceCost()
                                .add(row.getMaintenanceFee()));
                    }

                    //大件乐器
                    incomeDto.setLargeInstruments(incomeDto.getLargeInstruments()
                            .add(row.getLargeMusicalFee()));
                }
            }
            //学员指导课费
            incomeDto.setStudentGuidanceIncome(incomeDto.getStudentGuidanceIncome().add(courseIncome.getOrDefault(organId,BigDecimal.ZERO)));
            incomeDto.setCloudCoachIncome(incomeDto.getCloudCoachIncome().add(groupCloudAmount.getOrDefault(organId,BigDecimal.ZERO)));
            incomeDto.setSaleCloudCoachIncome(incomeDto.getSaleCloudCoachIncome().add(personalCloudAmount.getOrDefault(organId,BigDecimal.ZERO)));
            //服务收入
            incomeDto.setServiceIncome(incomeDto.getStudentGuidanceIncome()
                    .add(incomeDto.getCoursePurchaseIncome())
                    .add(incomeDto.getExamIncome())
                    .add(incomeDto.getMaintenanceCost())
                    .add(incomeDto.getInstrumentInsuranceCost())
                    .add(incomeDto.getOtherServiceIncome())
                    .add(incomeDto.getCloudCoachIncome())
            );
            incomeDto.setSalesIncome(incomeDto.getSalesIncome()
                    .add(incomeDto.getSaleCloudCoachIncome())
                    .add(incomeDto.getGroupPurchaseInstruments())
                    .add(incomeDto.getGroupPurchaseTeachingAids())
                    .add(incomeDto.getLargeInstruments())
                    .add(incomeDto.getOtherIncome())
            );
            incomeDtos.add(incomeDto);
        }
        return incomeDtos;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateShippedStatus(List<OrderSkuSync> orderSkuSyncs) {
        if (CollectionUtils.isEmpty(orderSkuSyncs)) {
            return;
        }
        List<String> orderNos = orderSkuSyncs.stream().map(e -> e.getOrderNo()).collect(Collectors.toList());
        List<SellOrder> sellOrders = sellOrderDao.queryByOrganNos(orderNos);
        if (CollectionUtils.isEmpty(sellOrders)) {
            return;
        }
        sellOrders = sellOrders.stream().filter(e -> e.getDeliveryTime() == null).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(sellOrders)){
            return;
        }
        String format = DateUtil.format(new Date(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
        Map<String,List<SellOrder>> sellOrderMap = sellOrders.stream().collect(Collectors.groupingBy(SellOrder::getOrderNo));
        for (OrderSkuSync orderSkuSync : orderSkuSyncs) {
            List<SellOrder> dtos = sellOrderMap.get(orderSkuSync.getOrderNo());
            if(CollectionUtils.isEmpty(dtos)){
                continue;
            }
            Map<Long,List<SellOrder>> goodsSkuMap = dtos.stream().collect(Collectors.groupingBy(e->e.getGoodsSkuId().longValue()));
            for (OrderSkuSync.SkuSync skuSync : orderSkuSync.getSkuSyncList()) {
                List<SellOrder> sellOrderList = goodsSkuMap.get(skuSync.getSku());
                for (int i = 0; i < skuSync.getCount(); i++) {
                    SellOrder sellOrder = this.getSellorder(sellOrderList,i);
                    sellOrder.setDeliveryTime(format);
                    sellOrder.setSellCost(skuSync.getPrice());
                    sellOrder.setStockType(StringUtils.equalsIgnoreCase(skuSync.getType(),"INTERNAL")?StockType.INTERNAL:StockType.EXTERNAL);
                }
            }
        }
        sellOrderDao.batchUpdate(sellOrders);
        musicGroupPaymentCalenderAddressService.lambdaUpdate().eq(MusicGroupPaymentCalenderAddress::getOrderNo,sellOrders.get(0).getOrderNo())
                .set(MusicGroupPaymentCalenderAddress::getDeliveryFlag,true).update();
    }

    private SellOrder getSellorder(List<SellOrder> sellOrderList,Integer index){
        if(index >= sellOrderList.size()){
            return new SellOrder();
        }
        SellOrder sellOrder = sellOrderList.get(index);
        if(StringUtils.isNotEmpty(sellOrder.getDeliveryTime())){
            return getSellorder(sellOrderList,index+1);
        }else {
            return sellOrder;
        }
    }

    public BigDecimal getActualAmount(BigDecimal price, StudentPaymentOrderExportDto row) {
        if (row.getExpectAmount().compareTo(BigDecimal.ZERO) > 0) {
            return price.multiply(row.getActualAmount()).divide(row.getExpectAmount(), 2, RoundingMode.DOWN);
        }
        return BigDecimal.ZERO;
    }
}
