package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.StudentPaymentOrderQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import com.ym.mec.util.ini.IniFileUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.Map.Entry;
import java.util.stream.Collectors;

@Service
public class StudentPaymentRouteOrderServiceImpl extends BaseServiceImpl<Long, StudentPaymentRouteOrder> implements StudentPaymentRouteOrderService {

    @Resource
    private StudentPaymentRouteOrderDao studentPaymentRouteOrderDao;
    @Resource
    private SysUserCashAccountDao sysUserCashAccountDao;
    @Resource
    private StudentPaymentOrderService studentPaymentOrderService;
    @Resource
    private StudentPaymentOrderDetailService studentPaymentOrderDetailService;
    @Resource
    private IdGeneratorService idGeneratorService;
    @Resource
    private GoodsDao goodsDao;
    @Resource
    private SellOrderService sellOrderService;
    @Resource
    private SysUserFeignService sysUserFeignService;
    @Resource
    private TeacherDao teacherDao;
    @Resource
    private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
    @Resource
    private MusicGroupPaymentCalenderDetailDao musicGroupPaymentCalenderDetailDao;
    @Resource
    private MusicGroupCalenderRefundPeriodService musicGroupCalenderRefundPeriodService;
    @Resource
    private CooperationOrganDao cooperationOrganDao;

    @Override
    public BaseDAO<Long, StudentPaymentRouteOrder> getDAO() {
        return studentPaymentRouteOrderDao;
    }

    @Override
    public StudentPaymentRouteOrderDao getDao() {
        return studentPaymentRouteOrderDao;
    }

    @Override
    public PageInfo<StudentPaymentRouteOrder> queryAuditList(QueryInfo queryInfo) {
        PageInfo<StudentPaymentRouteOrder> pageInfo = new PageInfo<StudentPaymentRouteOrder>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentPaymentRouteOrder> dataList = null;
        int count = studentPaymentRouteOrderDao.queryAuditCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentPaymentRouteOrderDao.queryAuditPage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<StudentPaymentRouteOrder>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfoOrder<StudentPaymentRouteOrderDto> OrderQueryPage(StudentPaymentOrderQueryInfo queryInfo) {
        PageInfoOrder<StudentPaymentRouteOrderDto> pageInfo = new PageInfoOrder<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentPaymentRouteOrderDto> dataList = new ArrayList<>();
        int count = this.findCount(params);

        BigDecimal totalUserBalance = sysUserCashAccountDao.getTotalUserBalance(params);
        pageInfo.setTotalUserBalance(totalUserBalance);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentPaymentRouteOrderDao.queryPageOrder(params);

            StudentPaymentRouteOrderDto orderMoneyAmount = studentPaymentRouteOrderDao.getOrderMoneyAmount(params);
            BigDecimal totalActualAmount = orderMoneyAmount.getActualAmount() != null ? orderMoneyAmount.getActualAmount() : BigDecimal.ZERO;
            BigDecimal totalBalancePaymentAmount = orderMoneyAmount.getBalancePaymentAmount() != null ? orderMoneyAmount.getBalancePaymentAmount() : BigDecimal.ZERO;

            params.put("chargeType", SporadicChargeTypeEnum.RECHARGE);
            BigDecimal totalChargeAmount = studentPaymentRouteOrderDao.getChargeAmount(params);
            totalChargeAmount = totalChargeAmount != null ? totalChargeAmount : BigDecimal.ZERO;
            BigDecimal totalAdvanceAmount = totalChargeAmount.subtract(totalBalancePaymentAmount);

            pageInfo.setTotalAdvanceAmount(totalAdvanceAmount);
            pageInfo.setTotalActualAmount(totalActualAmount);
            pageInfo.setTotalRevenueAmount(totalActualAmount.subtract(totalAdvanceAmount));
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public StudentPaymentRouteOrder addOutOrder(StudentPaymentRouteOrder studentPaymentRouteOrder) {
        if (studentPaymentRouteOrder.getCalenderId() != null) {
            MusicGroupPaymentCalender musicGroupPaymentCalender = musicGroupPaymentCalenderDao.getSchoolCalender(studentPaymentRouteOrder.getCalenderId());
            if (musicGroupPaymentCalender == null) {
                throw new BizException("学校缴费单号（审核通过）不存在，请核查");
            }
            if(musicGroupPaymentCalender.getPaymentType() == MusicGroupPaymentCalender.PaymentType.GOODS_PURCHASE){
                studentPaymentRouteOrder.setSaleAmount(studentPaymentRouteOrder.getRouteAmount());
            }else {
                studentPaymentRouteOrder.setServiceAmount(studentPaymentRouteOrder.getRouteAmount());
            }
        }
        Date nowDate = new Date();
        String orderNo = idGeneratorService.generatorId("payment") + "";
        studentPaymentRouteOrder.setOrderNo(orderNo);
        studentPaymentRouteOrder.setAuditStatus(AuditStatusEnum.ING);
        studentPaymentRouteOrder.setCreateTime(nowDate);
        studentPaymentRouteOrder.setUpdateTime(nowDate);
        studentPaymentRouteOrder.setRouteAmount(studentPaymentRouteOrder.getSaleAmount().add(studentPaymentRouteOrder.getServiceAmount()));
        studentPaymentRouteOrder.setRouteBalanceAmount(BigDecimal.ZERO);
        studentPaymentRouteOrder.setFeeFlag("N");

        studentPaymentRouteOrderDao.insert(studentPaymentRouteOrder);

        return studentPaymentRouteOrder;
    }

    @Override
    public StudentPaymentRouteOrder updateOutOrder(StudentPaymentRouteOrder studentPaymentRouteOrder) {
        Date nowDate = new Date();
        studentPaymentRouteOrder.setAuditStatus(AuditStatusEnum.ING);
        studentPaymentRouteOrder.setUpdateTime(nowDate);
        studentPaymentRouteOrder.setRouteAmount(studentPaymentRouteOrder.getSaleAmount().add(studentPaymentRouteOrder.getServiceAmount()));
        studentPaymentRouteOrder.setRouteBalanceAmount(BigDecimal.ZERO);
        studentPaymentRouteOrder.setFeeFlag("N");

        studentPaymentRouteOrderDao.update(studentPaymentRouteOrder);

        return studentPaymentRouteOrder;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public boolean auditOutOrder(Long studentPaymentRouteOrderId, AuditStatusEnum auditStatus, String memo, Integer operatorId) {
        StudentPaymentRouteOrder studentPaymentRouteOrder = studentPaymentRouteOrderDao.get(studentPaymentRouteOrderId);
        if (studentPaymentRouteOrder == null) {
            throw new BizException("查询订单信息失败");
        }

        if (studentPaymentRouteOrder.getAuditStatus() != AuditStatusEnum.ING) {
            throw new BizException("只有“审核中”的订单才能审核");
        }

        Date nowDate = new Date();
        String orderNo = idGeneratorService.generatorId("payment") + "";

        studentPaymentRouteOrder.setUpdateBy(operatorId);
        studentPaymentRouteOrder.setUpdateTime(nowDate);
        studentPaymentRouteOrder.setAuditComment(memo);
        studentPaymentRouteOrder.setAuditStatus(auditStatus);
        studentPaymentRouteOrder.setOrderNo(orderNo);
        studentPaymentRouteOrderDao.update(studentPaymentRouteOrder);
        if (auditStatus == AuditStatusEnum.PASS) {
            MusicGroupPaymentCalender musicGroupPaymentCalender = musicGroupPaymentCalenderDao.get(studentPaymentRouteOrder.getCalenderId());
            StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
            studentPaymentOrder.setActualAmount(studentPaymentRouteOrder.getRouteAmount());
            if(studentPaymentRouteOrder.getSchoolId() != null) {
                studentPaymentOrder.setCooperationId(studentPaymentRouteOrder.getSchoolId());
            }
            MusicGroupPaymentCalender calender = null;
            if(studentPaymentRouteOrder.getCalenderId() != null){
                calender = musicGroupPaymentCalenderDao.get(studentPaymentRouteOrder.getCalenderId());
                studentPaymentOrder.setMusicGroupId(calender.getMusicGroupId());
                studentPaymentOrder.setCalenderId(studentPaymentRouteOrder.getCalenderId());
            }
            studentPaymentOrder.setType(studentPaymentRouteOrder.getType());
            studentPaymentOrder.setMerNos(studentPaymentRouteOrder.getMerNo());
            studentPaymentOrder.setOrderNo(studentPaymentRouteOrder.getOrderNo());
            studentPaymentOrder.setGroupType(GroupType.OUTORDER);
            studentPaymentOrder.setOrganId(studentPaymentRouteOrder.getRouteOrganId());
            studentPaymentOrder.setRoutingOrganId(studentPaymentRouteOrder.getRouteOrganId());
            studentPaymentOrder.setExpectAmount(studentPaymentRouteOrder.getRouteAmount());
            studentPaymentOrder.setStatus(DealStatusEnum.SUCCESS);
            studentPaymentOrder.setCreateTime(studentPaymentRouteOrder.getPayTime());
            studentPaymentOrder.setUpdateTime(nowDate);
            studentPaymentOrder.setMemo(studentPaymentRouteOrder.getMemo());
            studentPaymentOrder.setUserId(studentPaymentRouteOrder.getUserId());
            studentPaymentOrder.setPayTime(studentPaymentRouteOrder.getPayTime());
            studentPaymentOrder.setTransNo(studentPaymentRouteOrder.getTransNo());
            studentPaymentOrder.setOrderNo(orderNo);

            studentPaymentRouteOrderDao.insertOrder(studentPaymentOrder);


            if (StringUtils.isNoneBlank(studentPaymentRouteOrder.getGoodsJson())) {
                JSONObject jsonObj = JSON.parseObject(studentPaymentRouteOrder.getGoodsJson());
                List<GoodsSellDto> goodsSellDtos = new ArrayList<>();
                for (Entry<String, Object> goodsMap : jsonObj.entrySet()) {
                    GoodsSellDto goodsSellDto = new GoodsSellDto();
                    goodsSellDto.setGoodsId(Integer.parseInt(goodsMap.getKey()));
                    goodsSellDto.setGoodsNum(Integer.parseInt(goodsMap.getValue().toString()));
                    goodsSellDtos.add(goodsSellDto);
                }
                List<StudentPaymentOrderDetail> orderDetails = studentPaymentOrderDetailService.initStudentPaymentOrderDetail(studentPaymentOrder, goodsSellDtos);
                sellOrderService.addOrderDetail2SellOrder(orderDetails, studentPaymentOrder, studentPaymentOrder.getMusicGroupId());
            }
            if (studentPaymentRouteOrder.getCalenderId() == null) {
                return true;
            }

            //根据学校缴费项id，确定缴费状态
//            BigDecimal amount = studentPaymentRouteOrderDao.sumAmountByCalenderId(studentPaymentRouteOrder.getCalenderId());
            //修改学员缴费回款周期
            Boolean aBoolean = musicGroupCalenderRefundPeriodService.calcRefundAmount(studentPaymentRouteOrder.getCalenderId(),studentPaymentRouteOrder.getOrderNo(), studentPaymentRouteOrder.getRouteAmount());

            //是否已完成回款
            if(aBoolean){
                List<MusicGroupPaymentCalenderDetail> calenderDetails = musicGroupPaymentCalenderDetailDao.getCalenderDetailWithCalender(studentPaymentRouteOrder.getCalenderId());
                if (studentPaymentRouteOrder.getRouteAmount().compareTo(musicGroupPaymentCalender.getCurrentTotalAmount()) >= 0) {
                    musicGroupPaymentCalender.setActualNum(musicGroupPaymentCalender.getExpectNum());
                    musicGroupPaymentCalender.setStatus(MusicGroupPaymentCalender.PaymentCalenderStatusEnum.PAID);
                    musicGroupPaymentCalenderDao.update(musicGroupPaymentCalender);

                    for (MusicGroupPaymentCalenderDetail calenderDetail : calenderDetails) {
                        calenderDetail.setPaymentStatus(MusicGroupStudentFee.PaymentStatus.PAID_COMPLETED);
                        calenderDetail.setPayTime(nowDate);
                    }
                    if (CollectionUtils.isNotEmpty(calenderDetails)) {
                        musicGroupPaymentCalenderDetailDao.batchUpdate(calenderDetails);
                    }
                }
            }
        }
        return true;
    }

    @Override
    public StudentPaymentRouteOrder addRouteOrder(String orderNo, Integer routeOrganId, BigDecimal routeBalanceAmount) {
        StudentPaymentRouteOrder studentPaymentRouteOrder = new StudentPaymentRouteOrder();
        studentPaymentRouteOrder.setOrderNo(orderNo);
        studentPaymentRouteOrder.setRouteOrganId(routeOrganId);
        studentPaymentRouteOrder.setRouteAmount(BigDecimal.ZERO);
        studentPaymentRouteOrder.setRouteBalanceAmount(routeBalanceAmount);
        studentPaymentRouteOrder.setFeeFlag("N");
        if (studentPaymentRouteOrderDao.insert(studentPaymentRouteOrder) <= 0) {
            throw new BizException("插入订单失败");
        }
        return studentPaymentRouteOrder;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteRouteOrder(Long orderId) {
        StudentPaymentOrder studentPaymentOrder = studentPaymentOrderService.get(orderId);
        if (studentPaymentOrder == null) {
            throw new BizException("订单不存在,请核查");
        }
        studentPaymentOrderService.delete(orderId);
        studentPaymentRouteOrderDao.deleteByOrderNo(studentPaymentOrder.getOrderNo());
        //归还库存
        List<SellOrder> sellOrders = sellOrderService.getSellOrderByParentGoodsId(orderId, null);
        if (CollectionUtils.isNotEmpty(sellOrders)) {
            sellOrderService.getDao().deleteByOrderId(orderId);
        }
        return true;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult<List<StudentPaymentRouteOrder>> importRouteOrder(MultipartFile file) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null || sysUser.getId() == null) {
            throw new BizException("用户信息获取失败");
        }
        Map<String, List<Map<String, Object>>> sheetsListMap = POIUtil.importExcel(new ByteArrayInputStream(file.getBytes()), 2, file.getOriginalFilename());
        InputStream inputStream = new ClassPathResource("columnMapper.ini").getInputStream();
        Map<String, String> columns = IniFileUtil.readIniFile(inputStream, TemplateTypeEnum.ROUTE_ORDER.getMsg());
        List<StudentPaymentRouteOrder> studentPaymentRouteOrders = new ArrayList<>();
        Integer tenantId = TenantContextHolder.getTenantId();
        Map<String, Integer> organMap = getMap("organization", "name_", "id_", true,tenantId, String.class, Integer.class);
        Map<String, Integer> cooperationOrganMap = getMap("cooperation_organ", "name_", "id_", true,tenantId, String.class, Integer.class);
        StringBuffer sb = new StringBuffer();
        for (String e : sheetsListMap.keySet()) {
            List<Map<String, Object>> sheet = sheetsListMap.get(e);
            List<Object> collect = sheet.stream().map(m -> m.get("交易流水号")).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(collect)) {
                continue;
            }
            Date nowDate = new Date();

            List<Object> calenderIds = sheet.stream().map(m -> m.get("缴费单号")).collect(Collectors.toList());
            List<MusicGroupPaymentCalender> calenders = musicGroupPaymentCalenderDao.getPaymentCalenderWithCalenderIdsAndPayUserType(calenderIds, MusicGroupPaymentCalender.PayUserType.SCHOOL);
            Map<String, Long> schoolCalender = calenders.stream().collect(Collectors.toMap(m -> m.getId().toString(), MusicGroupPaymentCalender::getId));

            valueIsNull:
            for (int j = 0; j < sheet.size(); j++) {
                int rowNum = j + 2;
                Map<String, Object> row = sheet.get(j);
                if (row == null || row.isEmpty()) {
                    continue;
                }
                JSONObject objectMap = new JSONObject();
                Iterator<String> iterator = row.keySet().iterator();
                for (int i = 0; i < row.keySet().size(); i++) {
                    String s = iterator.next();
                    if (!columns.containsKey(s)) {
                        continue;
                    }
                    String columnValue = columns.get(s);
                    if (columnValue.equals("incomeType")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第").append(rowNum).append("行数据导入失败：收入类型不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("incomeType", row.get(s));
                        }
                    }
                    if (columnValue.equals("type")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第").append(rowNum).append("行数据导入失败：订单类型不可为空;");
                            continue valueIsNull;
                        } else {
                            for (OrderTypeEnum orderTypeEnum : OrderTypeEnum.values()) {
                                if (orderTypeEnum.getMsg().equals(row.get(s).toString())) {
                                    objectMap.put(columnValue, orderTypeEnum);
                                    break;
                                }
                            }
                            continue;
                        }
                    }
                    if (columnValue.equals("transNo")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第").append(rowNum).append("行数据导入失败：交易流水号不可为空;");
                            continue valueIsNull;
                        }
                    }
                    if (columnValue.equals("payTime") && StringUtils.isEmpty(row.get(s).toString())) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第").append(rowNum).append("行数据导入失败：交易日期不可为空;");
                            continue valueIsNull;
                        }
                    }
                    if (columnValue.equals("merNo")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第").append(rowNum).append("行数据导入失败：收款账户不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("merNo", row.get(s));
                        }
                        continue;
                    }
                    if (columnValue.equals("organName") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        Integer integer = organMap.get(row.get(s));
                        if (integer == null) {
                            sb.append("第").append(rowNum).append("行数据导入失败：分部名称不存在;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("routeOrganId", integer);
                        }
                        continue;
                    }
                    if (columnValue.equals("cooperationOrganName") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        Integer integer = cooperationOrganMap.get(row.get(s));
                        if (integer != null) {
                            objectMap.put("schoolId", integer);
                        } else {
                            sb.append("第").append(rowNum).append("行数据导入失败：所属学校名称不存在;");
                            continue valueIsNull;
                        }
                        continue;
                    }
                    if (columnValue.equals("userId") && StringUtils.isNotEmpty(row.get(s).toString())) {
                        Boolean flag = teacherDao.existUser(row.get(s));
                        if (flag) {
                            objectMap.put("userId", row.get(s));
                        } else {
                            sb.append("第").append(rowNum).append("行数据导入失败：学员编号不存在;");
                            continue valueIsNull;
                        }
                        continue;
                    }
                    if (columnValue.equals("payTime")) {
                        String toString = row.get(s).toString();
                        Date date = DateUtil.stringToDate(toString, DateUtil.DEFAULT_PATTERN);
                        if (date == null) {
                            sb.append("第").append(rowNum).append("行数据导入失败：交易日期格式错误;");
                            continue valueIsNull;
                        } else if (date.after(nowDate)) {
                            sb.append("第").append(rowNum).append("行数据导入失败：交易日期不能大于当前日期");
                            continue valueIsNull;
                        } else {
                            objectMap.put(columnValue, DateUtil.format(date, DateUtil.DEFAULT_PATTERN));
                        }
                        continue;
                    }
                    if (columnValue.equals("calenderId")) {
                        if (StringUtils.isNotBlank(row.get(s).toString()) && !schoolCalender.containsKey(row.get(s).toString())) {
                            sb.append("第").append(rowNum).append("行数据导入失败：学校缴费单号（审核通过）不存在;");
                            continue valueIsNull;
                        } else {
                            objectMap.put("calenderId", row.get(s));
                        }
                        continue;
                    }
                    objectMap.put(columnValue, row.get(s));
                }
                StudentPaymentRouteOrder studentPaymentRouteOrder = JSONObject.parseObject(objectMap.toJSONString(), StudentPaymentRouteOrder.class);
                //销售收入
                if ("销售收入".equals(studentPaymentRouteOrder.getIncomeType())) {
                    boolean calenderFlag = studentPaymentRouteOrder.getCalenderId()==null;
                    if (studentPaymentRouteOrder.getSaleAmount() == null) {
                        throw new BizException("导入失败：交易流水号{},收入类型为销售收入时，销售金额不可为空", studentPaymentRouteOrder.getTransNo());
                    }
                    studentPaymentRouteOrder.setServiceAmount(BigDecimal.ZERO);
                    String goodsIds = studentPaymentRouteOrder.getGoodsIds();
                    String goodsNums = studentPaymentRouteOrder.getGoodsNums();

                    if (calenderFlag && StringUtils.isEmpty(goodsNums)) {
                        throw new BizException("导入失败：有销售金额时商品数量必填");
                    }
                    if (calenderFlag && StringUtils.isEmpty(goodsIds)) {
                        throw new BizException("导入失败：有销售金额时商品编号必填");
                    }
                    if(StringUtils.isNotEmpty(goodsNums) && StringUtils.isNotEmpty(goodsIds)){
                        List<String> goodsIdList = Arrays.asList(goodsIds.split(","));
                        List<String> goodsNumList = Arrays.asList(goodsNums.split(","));
                        if (goodsIdList.size() != goodsNumList.size()) {
                            throw new BizException("导入失败：商品编号和商品数量不一致");
                        }
                        JSONObject goodsJSON = new JSONObject();
                        for (int i = 0; i < goodsIdList.size(); i++) {
                            if (!goodsDao.findByIdAndStatus(goodsIdList.get(i), 1)) {
                                throw new BizException("导入失败：商品编号{}不存在", goodsIdList.get(i));
                            }
                            try {
                                int goodsNum = Integer.parseInt(goodsNumList.get(i));
                                goodsJSON.put(goodsIdList.get(i), goodsNum);
                            } catch (Exception e1) {
                                throw new BizException("导入失败：商品数量{}填写错误", goodsNumList.get(i));
                            }
                        }
                        studentPaymentRouteOrder.setGoodsJson(goodsJSON.toJSONString());
                    }
                    //服务收入
                } else {
                    studentPaymentRouteOrder.setSaleAmount(BigDecimal.ZERO);
                    if (studentPaymentRouteOrder.getServiceAmount() == null) {
                        throw new BizException("导入失败：交易流水号{},收入类型为服务收入时，服务金额不可为空", studentPaymentRouteOrder.getTransNo());
                    }
                }
                studentPaymentRouteOrder.setRouteBalanceAmount(BigDecimal.ZERO);
                studentPaymentRouteOrder.setAuditStatus(AuditStatusEnum.ING);
                studentPaymentRouteOrder.setRouteAmount(studentPaymentRouteOrder.getSaleAmount().add(studentPaymentRouteOrder.getServiceAmount()));
                studentPaymentRouteOrder.setCreateBy(sysUser.getId());
                studentPaymentRouteOrder.setUpdateBy(sysUser.getId());
                //studentPaymentRouteOrder.setUserId(sysUser.getId());
                studentPaymentRouteOrders.add(studentPaymentRouteOrder);
            }
        }
        if(sb.length() > 0){
            throw new BizException(sb.toString());
        }
        if (CollectionUtils.isNotEmpty(studentPaymentRouteOrders)) {
            studentPaymentRouteOrderDao.batchAdd(studentPaymentRouteOrders);
        } else {
            throw new BizException(sb.length() > 0 ? sb.toString() : "导入失败：没有可以录入的数据");
        }
        HttpResponseResult<List<StudentPaymentRouteOrder>> httpResponseResult = new HttpResponseResult<List<StudentPaymentRouteOrder>>();
        httpResponseResult.setData(studentPaymentRouteOrders);
        httpResponseResult.setMsg(sb.toString());
        if (StringUtils.isNotEmpty(sb.toString())) {
            httpResponseResult.setCode(0);
        } else {
            httpResponseResult.setCode(200);
        }
        return httpResponseResult;
    }

    @Override
    public OutOrderInfoDto getRouteOrder(Long orderId) {
        OutOrderInfoDto outOrderInfoDto = new OutOrderInfoDto();
        StudentPaymentOrder order = studentPaymentOrderService.get(orderId);
        outOrderInfoDto.setStudentPaymentOrder(order);
        StudentPaymentRouteOrder routeOrder = studentPaymentRouteOrderDao.getByOrderNo(order.getOrderNo());
        String goodsJson = routeOrder.getGoodsJson();
        if(StringUtils.isNotEmpty(goodsJson)){
            JSONObject jsonObj = JSON.parseObject(goodsJson);
            List<GoodsNameDto> goodsSellDtos = new ArrayList<>();
            for (Map.Entry<String, Object> goodsMap : jsonObj.entrySet()) {
                GoodsNameDto goodsSellDto = new GoodsNameDto();
                goodsSellDto.setGoodsId(Integer.parseInt(goodsMap.getKey()));
                goodsSellDto.setGoodsNum(Integer.parseInt(goodsMap.getValue().toString()));
                goodsSellDtos.add(goodsSellDto);
            }
            List<Integer> goosIds = goodsSellDtos.stream().map(e -> e.getGoodsId()).collect(Collectors.toList());
            List<Goods> goodies = goodsDao.getGoodies(goosIds);
            if(CollectionUtils.isNotEmpty(goodies)){
                Map<Integer, String> goodsMap = goodies.stream().collect(Collectors.toMap(Goods::getId, Goods::getName));
                goodsSellDtos.forEach(e -> e.setGoodsName(goodsMap.get(e.getGoodsId())));
            }
            outOrderInfoDto.setGoodsSellDtos(goodsSellDtos);
        }
        outOrderInfoDto.setCalenderId(routeOrder.getCalenderId());
        if(order.getCooperationId() != null){
            CooperationOrgan cooperationOrgan = cooperationOrganDao.get(order.getCooperationId());
            if (cooperationOrgan != null) {
                outOrderInfoDto.setCoopName(cooperationOrgan.getName());
            }
        }
        BigDecimal goodsAmount = BigDecimal.ZERO;
        List<SellOrder> sellOrders = sellOrderService.getDao().getOrderSellOrder(orderId);
        if(CollectionUtils.isNotEmpty(sellOrders)){
            goodsAmount = sellOrders.stream().map(SellOrder::getActualAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
            outOrderInfoDto.setSellOrders(sellOrders);
            outOrderInfoDto.setGoodsAmount(goodsAmount);
        }
        BigDecimal serviceAmount = outOrderInfoDto.getStudentPaymentOrder().getActualAmount().subtract(goodsAmount);
        outOrderInfoDto.setServiceAmount(serviceAmount);
        return outOrderInfoDto;
    }

}
