package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.StudentTeacherMapperDao;
import com.ym.mec.biz.dal.entity.CourseSchedule;
import com.ym.mec.biz.dal.entity.StudentTeacherMapper;
import com.ym.mec.biz.service.StudentTeacherMapperService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.entity.CourseSchedule.CourseScheduleType.*;

@Service
public class StudentTeacherMapperServiceImpl extends BaseServiceImpl<Integer,StudentTeacherMapper> implements StudentTeacherMapperService{

    @Autowired
    private StudentTeacherMapperDao studentTeacherMapperDao;

    @Override
    public BaseDAO<Integer, StudentTeacherMapper> getDAO() {
        return studentTeacherMapperDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void init() {
        studentTeacherMapperDao.delAll();
        studentTeacherMapperDao.create();
        studentTeacherMapperDao.updatePracticeTeacher();
        studentTeacherMapperDao.updateVipTeacher();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateStudentTeacherMapper(List<CourseSchedule> courseScheduleList) {
        if(CollectionUtils.isEmpty(courseScheduleList)){
            return;
        }
        List<Long> collect = courseScheduleList.stream().filter(e -> e.getType() == VIP || e.getType() == PRACTICE).map(e -> e.getId()).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(collect)){
            return;
        }
        List<StudentTeacherMapper> list = studentTeacherMapperDao.findByCourseIds(collect);
        if(CollectionUtils.isEmpty(list)){
            return;
        }
        Map<String, List<StudentTeacherMapper>> listMap = list.stream().collect(Collectors.groupingBy(StudentTeacherMapper::getTeacherType));
        for (String teacherType : listMap.keySet()) {
            List<Integer> studentIds = listMap.get(teacherType).stream().map(e -> e.getStudentId()).collect(Collectors.toList());
            studentTeacherMapperDao.delByStudent(studentIds,teacherType);
        }
        studentTeacherMapperDao.batchInsert(list);
    }
}