package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.StudentPreVisitQueryInfo;
import com.ym.mec.biz.dal.page.StudentVisitQueryInfo;
import com.ym.mec.biz.service.StudentVisitService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.excel.POIUtil;
import com.ym.mec.util.ini.IniFileUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class StudentVisitServiceImpl extends BaseServiceImpl<Integer, StudentVisit> implements StudentVisitService {

    @Autowired
    private StudentVisitDao studentVisitDao;
    @Autowired
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private StudentRepairDao studentRepairDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
    private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private StudentAttendanceDao studentAttendanceDao;
    @Autowired
    private StudentOperatingVisitDao studentOperatingVisitDao;
    @Autowired
    private SysUserService sysUserService;


    @Override
    public BaseDAO<Integer, StudentVisit> getDAO() {
        return studentVisitDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentVisit addVisit(StudentVisit studentVisit) {
        Date now = new Date();
        SysUser user = teacherDao.getUser(studentVisit.getStudentId());
        studentVisit.setOrganId(user.getOrganId());
        studentVisit.setCreateTime(now);
        if(Objects.nonNull(studentVisit.getObjectId())){
            if("考勤申诉".equals(studentVisit.getPurpose())){
                CourseScheduleStudentPayment courseScheduleStudentPayment = courseScheduleStudentPaymentDao.get(studentVisit.getObjectId());
                if(Objects.isNull(courseScheduleStudentPayment)){
                    throw new BizException("学员课程信息不存在");
                }
                StudentAttendance studentAttendance = studentAttendanceDao.findByStatusAndCourseScheduleId(courseScheduleStudentPayment.getUserId(), courseScheduleStudentPayment.getCourseScheduleId().intValue());
                if (studentAttendance == null) {
                    studentAttendance = new StudentAttendance();
                    studentAttendance.setClassGroupId(courseScheduleStudentPayment.getClassGroupId());
                    studentAttendance.setCourseScheduleId(courseScheduleStudentPayment.getCourseScheduleId());
                    studentAttendance.setGroupType(courseScheduleStudentPayment.getGroupType());
                    studentAttendance.setMusicGroupId(courseScheduleStudentPayment.getMusicGroupId());
                    studentAttendance.setUserId(courseScheduleStudentPayment.getUserId());
                    studentAttendance.setCreateTime(now);
                    studentAttendance.setUpdateTime(now);
                    studentAttendance.setVisitFlag(YesOrNoEnum.YES);
                    studentAttendanceDao.insert(studentAttendance);
                } else {
                    studentAttendance.setVisitFlag(YesOrNoEnum.YES);
                    studentAttendance.setUpdateTime(now);
                    studentAttendanceDao.update(studentAttendance);
                }
                CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleStudentPayment.getCourseScheduleId());
                studentVisit.setOrganId(courseSchedule.getOrganId());
            }else if("异常请假回访".equals(studentVisit.getPurpose())){
                //处理所有的请假考勤为已回访
                studentAttendanceDao.updateLeaveVisitFlag(user.getId());
            }
        }
        studentVisitDao.insert(studentVisit);
        //更新学员回访统计数据
        studentOperatingVisitDao.updateVisit(studentVisit.getStudentId(),DateUtil.format(studentVisit.getVisitTime(),DateUtil.ISO_YEAR_MONTH_FORMAT));
        return studentVisit;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchAddVisit(List<StudentVisit> studentVisit) {
        Integer userId = sysUserService.getUserId();
        StudentVisit.VisiterTypeEnum visitType = StudentVisit.VisiterTypeEnum.EDU_TEACHER;
        Map<String, List<StudentVisit>> musicMap = studentVisit.stream().collect(Collectors.groupingBy(e -> e.getMusicGroupId()));
        for (String musicGroupId : musicMap.keySet()) {
            MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
            if(musicGroup.getStatus() != MusicGroupStatusEnum.PAY){
                throw new BizException("当前乐团状态无需批量回访");
            }
            List<StudentVisit> studentVisits = musicMap.get(musicGroupId);
            List<Integer> studentIds = studentVisits.stream().map(e -> e.getStudentId()).collect(Collectors.toList());
            List<SimpleUserDto> userDtos = teacherDao.getUsersSimpleInfo(studentIds);
            Date now = new Date();
            Map<Integer, SimpleUserDto> userDtoMap = userDtos.stream().collect(Collectors.groupingBy(e -> e.getUserId(), Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
            Map<String,List<Integer>> visitMap = new HashMap<>();
            for (StudentVisit visit : studentVisits) {
                visit.setType("其它");
                visit.setPurpose("其它");
                visit.setTeacherId(userId);
                visit.setVisiterType(visitType);
                SimpleUserDto userDto = userDtoMap.get(visit.getStudentId());
                if (Objects.isNull(userDto)){
                    throw new BizException("学员编号{}用户信息不存在",visit.getStudentId());
                }
                visit.setOrganId(Integer.parseInt(userDto.getOrganIds()));
                visit.setCreateTime(now);
                String month = DateUtil.format(visit.getVisitTime(), DateUtil.ISO_YEAR_MONTH_FORMAT);
                List<Integer> userIds;
                if(visitMap.containsKey(month)){
                    userIds = visitMap.get(month);
                }else {
                    userIds = new ArrayList<>();
                }
                userIds.add(visit.getStudentId());
                visitMap.put(month,userIds);
            }
            studentVisitDao.batchInsert(studentVisits);
            //更新学员回访统计数据
            for (String month : visitMap.keySet()) {
                studentOperatingVisitDao.batchUpdateVisit(visitMap.get(month),month);
            }
        }
    }

    @Override
    public StudentVisitDto getInfo(Integer id) {
        return studentVisitDao.getInfo(id);
    }

    @Override
    public String updateProbStatus(Integer id, Integer probStatus) {
        Integer status = studentVisitDao.updateProbStatus(id, probStatus);
        return status >0 ?"成功":"失败";

    }

    @Override
    public PageInfo<StudentVisitDto> getPageList(StudentVisitQueryInfo queryInfo) {
        if (queryInfo.getEndTime() != null) {
            queryInfo.setEndTime(DateUtil.getLastTimeWithDay(queryInfo.getEndTime()));
        }
        PageInfo<StudentVisitDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentVisitDto> dataList = new ArrayList<>();
        int count = this.findCount(params);

        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentVisitDao.getPageList(params);
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<StudentRegistration> getMusicGroupStu(String musicGroupId, Integer teacherId) {
        return studentRegistrationDao.getMusicGroupStu4Teacher(musicGroupId, teacherId);
    }

    @Override
    public List<MusicGroup> getMusicGroup(Integer teacherId) {
        return musicGroupDao.getTeacherMusicGroup(teacherId);
    }

    @Override
    public PageInfo<BasicUserDto> getStudents(StudentVisitQueryInfo queryInfo) {
        PageInfo<BasicUserDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<BasicUserDto> dataList = new ArrayList<>();
        int count = studentVisitDao.getStudentsCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentVisitDao.getStudents(params);
            Set<Integer> userIds = dataList.stream().map(BasicUserDto::getUserId).collect(Collectors.toSet());
            List<MusicGroup> musicGroups = studentRepairDao.getUserMusicGroup(userIds);
            Map<Integer, List<MusicGroup>> userMusicGroup = musicGroups.stream().collect(Collectors.groupingBy(MusicGroup::getRepairUserId));
            for (BasicUserDto basicUserDto : dataList) {
                if (!userMusicGroup.containsKey(basicUserDto.getUserId())) {
                    continue;
                }
                MusicGroup musicGroup = userMusicGroup.get(basicUserDto.getUserId()).get(0);
                basicUserDto.setMusicGroupId(musicGroup.getId());
                basicUserDto.setMusicGroupName(musicGroup.getName());
            }
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<StudentPreVisitDto> queryWaitVisit(StudentPreVisitQueryInfo queryInfo) {
        PageInfo<StudentPreVisitDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<StudentPreVisitDto> dataList = new ArrayList<>();
        int count = studentVisitDao.countWaitVisit(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentVisitDao.queryWaitVisit(params);
            Set<Integer> collect = dataList.stream().map(e -> e.getOrganId()).collect(Collectors.toSet());
            Map<Integer,String> organMap = getMap("organization", "id_", "name_", collect, TenantContextHolder.getTenantId(), Integer.class, String.class);
            for (StudentPreVisitDto visitDto : dataList) {
                visitDto.setOrganName(organMap.get(visitDto.getOrganId()));
            }
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<String> importMusicVisit(MultipartFile file) throws Exception{
        Map<String, List<Map<String, Object>>> sheetsListMap = POIUtil.importExcel(new ByteArrayInputStream(file.getBytes()), 2, file.getOriginalFilename());
        InputStream inputStream = new ClassPathResource("columnMapper.ini").getInputStream();
        Map<String, String> columns = IniFileUtil.readIniFile(inputStream, TemplateTypeEnum.PAYMENT_MUSIC_VISIT_IMPORT.getMsg());
        List<StudentVisit> importDtos = new ArrayList<>();
        List<String> errMsg = new ArrayList();
        StringBuffer sb = new StringBuffer();
        for (String sheetName : sheetsListMap.keySet()) {
            List<Map<String, Object>> sheet = sheetsListMap.get(sheetName);
            for (int i = 0; i < sheet.size(); i++) {
                int rowNum = i + 2;
                Map<String, Object> row = sheet.get(i);
                if (row.size() == 0) {
                    continue;
                }
                JSONObject objectMap = new JSONObject();
                valueIsNull: for (String s : row.keySet()) {
                    String columnValue = columns.get(s);
                    if (columnValue.equals("musicGroupId")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：乐团编号不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put(columnValue, row.get(s));
                        }
                    } else if (columnValue.equals("phone")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：学员手机号不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put(columnValue, row.get(s));
                        }
                    } else if (columnValue.equals("visitTime")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：回访日期不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put(columnValue, row.get(s));
                        }
                    } else if (columnValue.equals("overview")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：学员情况不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put(columnValue, row.get(s));
                        }
                    } else if (columnValue.equals("feedback")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：家长反馈不可为空;");
                            continue valueIsNull;
                        } else {
                            objectMap.put(columnValue, row.get(s));
                        }
                    } else if (columnValue.equals("probStatus")) {
                        if (StringUtils.isEmpty(row.get(s).toString())) {
                            sb.append("第" + rowNum).append("行数据导入失败：问题状态不可为空;");
                            continue valueIsNull;
                        } else {
                            if ("已解决".equals(row.get(s).toString())) {
                                objectMap.put(columnValue, 1);
                            }
                        }
                    }
                }
                try {
                    StudentVisit cost = JSONObject.parseObject(objectMap.toJSONString(), StudentVisit.class);
                    SysUser user = teacherDao.getUserWithPhone(cost.getPhone());
                    if(Objects.isNull(user) || !user.getUserType().contains("STUDENT")){
                        throw new BizException("手机号{}不存在",cost.getPhone());
                    }
                    cost.setStudentId(user.getId());
                    StudentRegistration studentRegistration = studentRegistrationDao.getStudentRegister(cost.getMusicGroupId(),cost.getStudentId());
                    if(Objects.isNull(studentRegistration)){
                        throw new BizException("学员{}不在乐团{}中，无需回访",cost.getStudentId(),cost.getMusicGroupId());
                    }
                    if(studentRegistration.getMusicGroupStatus() != StudentMusicGroupStatusEnum.APPLY){
                        throw new BizException("学员{}已缴费无需回访",studentRegistration.getUserId());
                    }
                    Integer visitStudent = studentVisitDao.getMusicVisitStudent(studentRegistration.getMusicGroupId(), studentRegistration.getUserId());
                    if(Objects.nonNull(visitStudent)){
                        throw new BizException("学员{}已回访",visitStudent);
                    }
                    importDtos.add(cost);
                } catch (Exception ex) {
                    throw new BizException("导入数据出错", ex);
                }
            }
        }
        if (!org.springframework.util.CollectionUtils.isEmpty(errMsg)) {
            return errMsg;
        }
        if(!org.springframework.util.CollectionUtils.isEmpty(importDtos)){
            this.batchAddVisit(importDtos);
        }
        return null;
    }
}
