package com.ym.mec.biz.service.impl;

import com.dayaedu.cbs.openfeign.client.MusicFeignClientService;
import com.dayaedu.cbs.openfeign.wrapper.musicInstrument.CbsMusicalInstrumentWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dao.StudentRegistrationDao;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dao.SubjectGoodsMapperDao;
import com.ym.mec.biz.dal.dto.ConditionDto;
import com.ym.mec.biz.dal.dto.StudentSubjectDto;
import com.ym.mec.biz.dal.dto.SubFeeSettingDto;
import com.ym.mec.biz.dal.dto.SubjectApplyDetailDto;
import com.ym.mec.biz.dal.entity.Instrument;
import com.ym.mec.biz.dal.entity.Subject;
import com.ym.mec.biz.dal.entity.SubjectGoodsMapper;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.SubjectQueryInfo;
import com.ym.mec.biz.dal.wrapper.InstrumentWrapper;
import com.ym.mec.biz.dal.wrapper.SubjectWrapper;
import com.ym.mec.biz.service.InstrumentService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class SubjectServiceImpl extends BaseServiceImpl<Integer, Subject> implements SubjectService {

    @Resource
    private SubjectDao subjectDao;
    @Resource
    private StudentRegistrationDao studentRegistrationDao;
    @Resource
    private SubjectGoodsMapperDao subjectGoodsMapperDao;
    @Resource
    private InstrumentService instrumentService;
    @Resource
    private MusicFeignClientService musicFeignClientService;

    @Override
    public BaseDAO<Integer, Subject> getDAO() {
        return subjectDao;
    }

    public SubjectDao getDao() {
        return subjectDao;
    }

    @Override
    public List<Subject> findSubByMusicGroupId(String musicGroupId) {
        return subjectDao.findSubByMusicGroupId(musicGroupId);
    }

    @Override
    public PageInfo<Subject> queryPageTree(SubjectQueryInfo queryInfo) {
        PageInfo<Subject> pageInfo = queryPage(queryInfo);
        for (Subject subject : pageInfo.getRows()) {
            subject = getTree(subject, queryInfo.getDelFlag(), queryInfo.getTenantId());
        }
        return pageInfo;
    }

    @Override
    public List<SubjectApplyDetailDto> findSubApplyDetail(String musicGroupId) {
        List<SubjectApplyDetailDto> subApplyDetail = subjectDao.findSubApplyDetail(musicGroupId);
        
        List<SubjectApplyDetailDto> registers = studentRegistrationDao.queryStudentApplyDetail(musicGroupId);
        
        Map<Integer,SubjectApplyDetailDto> map = registers.stream().collect(Collectors.toMap(SubjectApplyDetailDto :: getSubjectId, t -> t));

        Map<Long,BigDecimal> maps = MapUtil.convertIntegerMap(studentRegistrationDao.querySubjectAmount(musicGroupId));
        subApplyDetail.forEach(detail -> {
            SubjectApplyDetailDto dto = new SubjectApplyDetailDto();
            if(map.containsKey(detail.getSubjectId())){
                dto = map.get(detail.getSubjectId());
            }
            detail.setTotalAmount(maps.get(detail.getSubjectId().longValue()));
            detail.setPayNum(dto.getPayNum());
            detail.setApplyStudentNum(dto.getApplyStudentNum());
            detail.setPayingNum(dto.getPayingNum());
            detail.setCheckNum(dto.getCheckNum());
            detail.setNormalNum(dto.getNormalNum());
            detail.setBuyCloudTeacherNum(dto.getBuyCloudTeacherNum());
            detail.setCloudTeacherStudentNumOfApply(dto.getCloudTeacherStudentNumOfApply());
            detail.setNoCloudTeacherStudentNumOfNormal(dto.getNoCloudTeacherStudentNumOfNormal());
        });
        return subApplyDetail;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void upSetSubject(Subject subject) {
        if (subject.getDelFlag() == YesOrNoEnum.YES) {
            subjectDao.delete(subject.getId());
            instrumentService.lambdaUpdate().eq(Instrument::getSubjectId, subject.getId()).remove();
            return;
        }
        Integer parentId = upset(subject, null);
        List<Subject> subjects = subject.getSubjects();
        if (subjects != null && subjects.size() > 0) {
            subjects.forEach(e -> {
                upset(e, parentId);
            });
        }
    }

    private Integer upset(Subject subject, Integer parentId) {
        if (parentId != null) {
            subject.setParentSubjectId(parentId);
        }
        if (subject.getId() != null) {
            subject.setUpdateTime(new Date());
            subjectDao.update(subject);
        } else {
            subjectDao.insert(subject);
        }
        //保存乐器关联
        this.saveInstrument(subject);
        return subject.getId();
    }

    private void saveInstrument(Subject subject) {
        if (StringUtils.isNotEmpty(subject.getInstrumentIds())) {
            instrumentService.lambdaUpdate().eq(Instrument::getSubjectId, subject.getId()).remove();
            List<Instrument> instruments = new ArrayList<>();
            String[] split = subject.getInstrumentIds().split(",");
            for (String s : split) {
                Instrument instrument = new Instrument();
                instrument.setSubjectId(subject.getId());
                instrument.setId(Integer.valueOf(s));
                instrument.setEnableFlag(true);
                instrument.setDelFlag(false);
                instruments.add(instrument);
            }
            instrumentService.saveBatch(instruments);
        }
    }

    @Override
    public void markGoods(List<SubjectGoodsMapper> subjectGoodsMappers) {
        subjectGoodsMappers.forEach(e -> {
            upsetGoods(e);
        });
    }

    private void upsetGoods(SubjectGoodsMapper subjectGoodsMapper) {
        if (subjectGoodsMapper.getId() != null) {
            if (subjectGoodsMapper.getDelFlag()) {
                subjectGoodsMapperDao.delete(subjectGoodsMapper.getId());
                return;
            }
            subjectGoodsMapper.setUpdateTime(new Date());
            subjectGoodsMapperDao.update(subjectGoodsMapper);
        } else {
            subjectGoodsMapperDao.insert(subjectGoodsMapper);
        }
    }


    private Subject getTree(Subject sub, YesOrNoEnum yesOrNoEnum, Integer tenantId) {
        //得到根节点对象
        //获取子节点list
        List<Subject> subjects = subjectDao.findByParentId(sub.getId(), yesOrNoEnum, tenantId);
        //如果存在子节点
        if (subjects != null && subjects.size() > 0) {
            //将子节点list放入父节点对象
            sub.setSubjects(subjects);
            //遍历子节点....
            for (Subject subject : subjects) {
                getTree(subject, yesOrNoEnum, tenantId);
            }
        }
        return sub;
    }

    @Override
    public List<Subject> findTeacherSubjects(Integer userId) {
        return subjectDao.findTeacherSubjects(userId);
    }

    @Override
    public List<ConditionDto> findSubjectConditions() {
        return subjectDao.findSubjectConditions();
    }

    @Override
    public Map<Long, String> findSubjectMapBySubjectIdsList(List<String> subjectIdsList) {
        if (CollectionUtils.isEmpty(subjectIdsList)) {
            return new HashMap<>();
        }
        String subjectIdsString = StringUtils.join(subjectIdsList, ",");
        String[] split = subjectIdsString.split(",");
        List<String> subjectIds = new ArrayList<>(Arrays.asList(split));
        subjectIds = subjectIds.stream().distinct().collect(Collectors.toList());
        List<Map<Long, String>> bySubjecIds = subjectDao.findBySubjecIds(StringUtils.join(subjectIds, ","));
        Map<Long, String> subjectIdAndName = MapUtil.convertMybatisMap(bySubjecIds);
        return subjectIdAndName;
    }

    @Override
    public List<Subject> findSubSubjects(Integer parentSubjectId,Boolean cbsFlag) {
        return subjectDao.findSubSubjects(parentSubjectId,cbsFlag);
    }

    @Override
    public List<Subject> findBySubjectByIdList(List<Integer> subjectIdList) {
        return subjectDao.findBySubjectByIdList(StringUtils.join(subjectIdList, ","));
    }

    @Override
    public List<Subject> findBySubjectByIdList(String subjectIdList) {
        return subjectDao.findBySubjectByIdList(subjectIdList);
    }

    @Override
    public List<String> findBySubIds(String subIds) {
        return subjectDao.findBySubIds(subIds);
    }

    @Override
    public SubFeeSettingDto setSubjectInfo(Integer chargeTypeId) {
        return null;
    }

    @Override
    public PageInfo<Subject> queryPageByIdOrName(SubjectQueryInfo queryInfo) {
        PageInfo<Subject> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<Subject> dataList = null;
        int count = subjectDao.queryPageByIdOrNameCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = subjectDao.queryPageByIdOrName(params);
            List<Integer> subjectIds = dataList.stream().map(Subject::getId).collect(Collectors.toList());
            List<Instrument> list = instrumentService.lambdaQuery().in(Instrument::getSubjectId, subjectIds).list();
            Map<Integer, List<Instrument>> groupBySubjectId = list.stream().collect(Collectors.groupingBy(Instrument::getSubjectId));
            List<Integer> instrumentIds = list.stream().map(Instrument::getId).collect(Collectors.toList());
            CbsMusicalInstrumentWrapper.MusicalInstrumentQuery query = new CbsMusicalInstrumentWrapper.MusicalInstrumentQuery();
            query.setIds(instrumentIds);
            query.setPage(1);
            query.setRows(instrumentIds.size());
            R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto>> infoR =
                    musicFeignClientService.musicalInstrumentPage(query);
            if (infoR.getCode() != 200){
                throw new BizException("调用内容平台失败 : {}",infoR.getMessage());
            }
            List<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> rows = infoR.getData().getRows();
            Map<Integer,CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> map = rows.stream()
                    .collect(Collectors.toMap(CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto::getId, Function.identity()));
            for (Subject subject : dataList) {
                List<Instrument> instruments = groupBySubjectId.get(subject.getId());
                if(!CollectionUtils.isEmpty(instruments)){
                    subject.setInstrumentIds(instruments.stream().map(Instrument::getId).map(String::valueOf).collect(Collectors.joining(",")));
                    StringBuffer sb  = new StringBuffer();
                    for (Instrument instrument : instruments) {
                        CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto dto = map.get(instrument.getId());
                        if (dto != null) {
                            if(sb.length() > 0){
                                sb.append(",");
                            }
                            sb.append(dto.getName());
                        }
                    }
                    subject.setInstrumentName(sb.toString());
                }
            }
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<Subject> list() {
        return subjectDao.list();
    }

    @Override
    public Map<Integer, Subject> getMapByIds(List<Integer> subjectIds) {
        if (CollectionUtils.isEmpty(subjectIds)) {
            return new HashMap<>();
        }

        List<Subject> subjectList = subjectDao.findBySubjectIds(subjectIds);
        if (CollectionUtils.isEmpty(subjectList)) {
            return new HashMap<>();
        }

        return subjectList.stream().collect(Collectors.toMap(Subject::getId, Function.identity()));
    }

    @Override
    public Map<Integer, StudentSubjectDto> getSubjectByStudentId(Set studentIds) {

        if (CollectionUtils.isEmpty(studentIds)) {
            return new HashMap<>();
        }
        List<StudentSubjectDto> subjects = subjectDao.getSubjectByStudentId(studentIds);
        Map<Integer, StudentSubjectDto> map = new HashMap<>();
        if (!CollectionUtils.isEmpty(subjects)) {
            map = subjects.stream().collect(Collectors.toMap(StudentSubjectDto::getStudentId, Function.identity()));
        }
        return map;
    }

    @Override
    public List<SubjectWrapper.Subject> queryList(SubjectWrapper.SubjectQuery query) {
        query.setPage(1);
        query.setRows(9999);
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, query);

        PageInfo<SubjectWrapper.Subject> subjectIPage = this.selectPage(query);
        List<SubjectWrapper.Subject> records = subjectIPage.getRows();
        if (CollectionUtils.isEmpty(records)) {
            return records;
        }
        String instrumentIds = query.getInstrumentIds();
        List<Integer> instrumentIdList = new ArrayList<>();
        if (StringUtils.isNotEmpty(instrumentIds)) {
            List<Integer> collect = Arrays.stream(instrumentIds.split(",")).map(Integer::valueOf).distinct().collect(Collectors.toList());
            instrumentIdList.addAll(collect);
        }
        if (!instrumentIdList.isEmpty()) {
            records.forEach(record -> {
                List<InstrumentWrapper.Instrument> instruments = record.getInstruments();
                instruments.removeIf(next -> !instrumentIdList.contains(next.getId()));
            });
        }
        return records;
    }

    @Override
    public PageInfo<SubjectWrapper.Subject> selectPage(SubjectWrapper.SubjectQuery query) {
        PageInfo<SubjectWrapper.Subject> pageInfo = new PageInfo<>(query.getPage(), query.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, query);

        List<SubjectWrapper.Subject> dataList = null;
        int count = subjectDao.findCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = subjectDao.findPage(params);
            // 设置声部下的乐器信息
            List<Integer> subjectIds = dataList.stream().map(SubjectWrapper.Subject::getId).distinct().collect(Collectors.toList());
            Map<Integer, List<InstrumentWrapper.Instrument>> groupBySubjectId = instrumentService.getGroupBySubjectId(subjectIds, query.getEnableFlag());
            Map<Integer, Subject> map = this.findBySubjectByIdList(dataList.stream().map(SubjectWrapper.Subject::getParentSubjectId).collect(Collectors.toList())).stream()
                    .collect(Collectors.toMap(Subject::getId, t -> t));
            dataList.forEach(e -> {
                if(e.getParentSubjectId() != null && e.getParentSubjectId() > 0) {
                    e.setParentSubjectName(map.get(e.getParentSubjectId()).getName());
                }
                List<InstrumentWrapper.Instrument> instruments = groupBySubjectId.get(e.getId());
                if (org.apache.commons.collections.CollectionUtils.isNotEmpty(instruments)) {
                    e.setInstruments(instruments);
                }
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }
}
