package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayaedu.cbs.common.enums.EAppKey;
import com.dayaedu.cbs.common.enums.EClientType;
import com.dayaedu.cbs.common.enums.message.EMessageSendMode;
import com.dayaedu.cbs.common.enums.message.ESendStatus;
import com.dayaedu.cbs.openfeign.client.MessageFeignClientService;
import com.dayaedu.cbs.openfeign.wrapper.message.CbsMessageWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dao.SysConfigDao;
import com.ym.mec.biz.dal.dao.SysMessageDao;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dao.WaitSendMessageDao;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.dto.SysMessageVo;
import com.ym.mec.biz.dal.entity.ImSendGroupMessage;
import com.ym.mec.biz.dal.entity.SysMessage;
import com.ym.mec.biz.dal.entity.SysMessageConfig;
import com.ym.mec.biz.dal.entity.WaitSendMessage;
import com.ym.mec.biz.dal.enums.MessageSendMode;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.SendStatusEnum;
import com.ym.mec.biz.dal.page.SysMessageQueryInfo;
import com.ym.mec.biz.dal.wrapper.SysMessageWrapper;
import com.ym.mec.biz.service.SysMessageConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImMessageDto;
import com.ym.mec.common.entity.ImPrivateMessage;
import com.ym.mec.common.entity.ImSendMessageUserInfo;
import com.ym.mec.common.entity.ImTxtMessage;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.redis.service.RedisCache;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.string.MessageFormatter;
import net.coobird.thumbnailator.Thumbnails;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.MessageTypeEnum.STUDENT_PUSH_VIP_BUY;

@Service
public class SysMessageServiceImpl extends BaseServiceImpl<Long, SysMessage> implements SysMessageService {

	private final static Logger LOGGER = LoggerFactory.getLogger(SysMessageServiceImpl.class);

	@Resource
	private SysMessageDao sysMessageDao;
	@Resource
	private SysMessageConfigService sysMessageConfigService;
	@Resource
	private SysUserService sysUserService;
	@Resource
	private TeacherDao teacherDao;
	@Resource
	private WaitSendMessageDao waitSendMessageDao;
	@Resource
	private SysConfigDao sysConfigDao;
	@Resource
	private ImFeignService imFeignService;
	@Resource
	private RedisCache<String, Object> redisCache;
	@Resource
	private MessageFeignClientService messageFeignClientService;
	// 验证码有效期
	public static final int CODE_EXPIRE = 60 * 10;
	// 发送验证码的间隔时间
	public static final int CODE_INTERVAL_TIME = 60;

	private final int DEFAULT_CODE = 666666;

	@Value("${message.debugMode}")
	private boolean debugMode;

	@Override
	public BaseDAO<Long, SysMessage> getDAO() {
		return sysMessageDao;
	}



    /**
     * 分页查询
     * @param page IPage<SysMessageWrapper.SysMessageList>
     * @param query SysMessageWrapper.SysMessageQuery
     * @return IPage<SysMessageWrapper.SysMessage>
     */
    @Override
    public PageInfo<SysMessageWrapper.Message> queryPage(SysMessageQueryInfo query){

        CbsMessageWrapper.SysMessageQuery cbsQuery = getSysMessageQuery(query);

        PageInfo<SysMessageWrapper.Message> pageInfo = new PageInfo<>(query.getPage(), query.getRows());

        cbsQuery.setPage(query.getPage());
        cbsQuery.setRows(query.getRows());
        com.microsvc.toolkit.common.response.paging.PageInfo<CbsMessageWrapper.SysMessage> sysMessagePageInfo = messageFeignClientService
            .sysMessagePage(cbsQuery).feignData();
        List<CbsMessageWrapper.SysMessage> rows = sysMessagePageInfo.getRows();
        if (CollectionUtils.isEmpty(rows)) {
            return pageInfo;
        }
        List<SysMessage> sysMessages = rows.stream().map(this::convert).collect(Collectors.toList());
        pageInfo.setRows(JSON.parseArray(JSON.toJSONString(sysMessages), SysMessageWrapper.Message.class));
        pageInfo.setTotal(sysMessagePageInfo.getTotal());
        return pageInfo;
    }

    private SysMessage convert(CbsMessageWrapper.SysMessage sysMessage) {
        SysMessage message = new SysMessage();
        message.setId(sysMessage.getId().toString());
        if (sysMessage.getUserId() != null) {
            message.setUserId(sysMessage.getUserId().intValue());
        }
        if (sysMessage.getMessageConfigId() != null) {
            message.setMessageConfigId(sysMessage.getMessageConfigId().intValue());
        }
        message.setTitle(sysMessage.getTitle());
        message.setContent(sysMessage.getContent());
        message.setType(MessageSendMode.valueOf(sysMessage.getSendMode() !=null?sysMessage.getSendMode().name():"PUSH"));
        message.setStatus(SendStatusEnum.valueOf(Optional.ofNullable(sysMessage.getStatus()).map(o->o.name()).orElse(SendStatusEnum.FAILED.name())));
        message.setReceiver(sysMessage.getReceiver());
        message.setSendTime(sysMessage.getSendTime());
        message.setErrorMsg(sysMessage.getErrorMsg());
        message.setMemo(sysMessage.getMemo());
        message.setReadStatus(sysMessage.getReadStatus()?1:0);
        message.setGroup(sysMessage.getGroup());
        message.setCreateOn(sysMessage.getCreateTime());
        message.setModifyOn(sysMessage.getUpdateTime());
        message.setJpushType(sysMessage.getSendMode().name());
        return message;
    }

    private CbsMessageWrapper.SysMessageQuery getSysMessageQuery(SysMessageQueryInfo query) {
        CbsMessageWrapper.SysMessageQuery cbsQuery = new CbsMessageWrapper.SysMessageQuery();
        if (query.getUserId() != null) {
            cbsQuery.setUserId(query.getUserId().longValue());
        }
        cbsQuery.setAppKey(EAppKey.GYM);
        if (query.getJpushType() !=null) {
            cbsQuery.setClientType(getClientType(query.getJpushType()));
        }
        if (query.getStatus() !=null) {
            switch (query.getStatus()) {
                case 0:
                    cbsQuery.setStatus(ESendStatus.WAIT);
                    break;
                case 1:
                    cbsQuery.setStatus(ESendStatus.SENDING);
                    break;
                case 2:
                    cbsQuery.setStatus(ESendStatus.SUCCESSED);
                    break;
            }
        }
        cbsQuery.setTitle(query.getTitle());
        if (query.getType() != null) {
            switch (query.getType()) {
                case 1:
                    cbsQuery.setSendMode(EMessageSendMode.SMS);
                    break;
                case 2:
                    cbsQuery.setSendMode(EMessageSendMode.EMAIL);
                    break;
                case 3:
                    cbsQuery.setSendMode(EMessageSendMode.PUSH);
                    break;
            }
        }
        if (StringUtils.isNotBlank(query.getGroup())) {
            cbsQuery.setGroup(query.getGroup());
        }
        if (query.getReadStatus() != null) {
            cbsQuery.setReadStatus(query.getReadStatus() == 1);
        }
        return cbsQuery;
    }

	@Override
	public R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMessageWrapper.SysMessage>> queryListPage(SysMessageQueryInfo queryInfo) {
		CbsMessageWrapper.SysMessageQuery query = new CbsMessageWrapper.SysMessageQuery();
		query.setPage(queryInfo.getPage());
		query.setRows(queryInfo.getRows());
		query.setUserId(queryInfo.getUserId().longValue());
//		query.setStatus(ESendStatus.SUCCESSED);
		query.setAppKey(EAppKey.GYM);
		query.setGroup(queryInfo.getGroup());
		query.setSendMode(getMessageSendMode(queryInfo.getType()));
		query.setTitle(queryInfo.getTitle());
		query.setClientType(getClientType(queryInfo.getJpushType()));
		query.setReadStatus(queryInfo.getReadStatus() == null?null:queryInfo.getReadStatus()==1);
		return messageFeignClientService.sysMessagePage(query);
	}

	private EMessageSendMode getMessageSendMode(Integer type){
		if(type == null){
			return null;
		}else if(type == 1){
			return EMessageSendMode.SMS;
		}else if(type == 2){
			return EMessageSendMode.EMAIL;
		}else if(type == 3){
			return EMessageSendMode.PUSH;
		}else {
			return EMessageSendMode.SEO;
		}
	}

	public EClientType getClientType(String jpushType){
		if(StringUtils.isEmpty(jpushType)){
			return null;
		}else if(StringUtils.equals(jpushType, "STUDENT")){
			return EClientType.STUDENT;
		}else if(StringUtils.equals(jpushType, "TEACHER")){
			return EClientType.TEACHER;
		}else if(StringUtils.equals(jpushType, "SCHOOL")){
			return EClientType.SCHOOL;
		}else {
			return EClientType.BACKEND;
		}
	}

	@Override
	public boolean batchSeoMessage(Set<Integer> userIds, MessageTypeEnum messageType,String memo, Object... args) {
		if(userIds != null){
			userIds.removeAll(Collections.singleton(null));
		}
		if (CollectionUtils.isEmpty(userIds)) {
			LOGGER.error("接收地址不能为空");
			return false;
		}
		List<SimpleUserDto> userDtoList = teacherDao.getUsersSimpleInfo(new ArrayList<>(userIds));
		List<CbsMessageWrapper.MessageParam> messageParams = new ArrayList<>();
		for (SimpleUserDto user : userDtoList) {
			CbsMessageWrapper.MessageParam messageParam = new CbsMessageWrapper.MessageParam();
			messageParam.setUserId(user.getUserId().longValue());
			messageParam.setAppKey(EAppKey.GYM);
			messageParam.setClientType(EClientType.BACKEND);
			messageParam.setClientId(messageParam.getClientType());
			messageParam.setType(messageType.getCode());
			Map<Long, String> params = new HashMap<>(1);
			params.put(user.getUserId().longValue(), user.getPhone());
			messageParam.setReceivers(params);
			messageParam.setRealSend(!debugMode);
			messageParam.setUrl(memo);
			if(args != null && args.length > 0){
				String[] argsStr = new String[args.length];
				for (int i = 0; i < args.length; i++) {
					if (args[i]!=null){
						argsStr[i] = args[i].toString();
					}else {
						argsStr[i] = "";
					}
				}
				messageParam.setContentArgs(argsStr);
			}
			messageParams.add(messageParam);
		}
		messageFeignClientService.sysMessageBatchSendMessage(messageParams);
		return true;
	}

	@Override
	public void batchSendImMessage(MessageTypeEnum type, String senderId,String extra, String[] targetIds, String url, Object... args) {
		String messageContent = checkMessageConfig(type);
		String content = getContent(messageContent,url, args);
		//ExecutorService executor = Executors.newCachedThreadPool();
		CompletableFuture.runAsync(()-> imFeignService.privateSend(getImGroupMessage("RC:TxtMsg",senderId,targetIds,content,extra)));
	}

	@Override
	public void batchSendImGroupMessage(MessageTypeEnum type, String senderId,String extra, String[] targetIds, String url, Object... args) {
		String messageContent = checkMessageConfig(type);
		String content = getContent(messageContent,url, args);
		ImMessageDto imGroupMessage = getImGroupMessage("RC:TxtMsg", senderId, targetIds, content, extra);
		this.sendGroupMessage(imGroupMessage);
	}

	@Override
	public void batchSendImGroupTxtMessage(ImSendGroupMessage messageDto) {
		messageDto.setObjectName("RC:TxtMsg");
        if (StringUtils.isNotBlank(messageDto.getTargetIds())) {
            ImMessageDto imGroupMessage = this.getImGroupMessage(messageDto);
            this.sendGroupMessage(imGroupMessage);
        }
        if (StringUtils.isNotBlank(messageDto.getUserTargetIds())) {
            messageDto.setTargetIds(messageDto.getUserTargetIds());
            ImMessageDto imGroupMessage = this.getImGroupMessage(messageDto);
            this.sendImMessage(imGroupMessage);
        }
	}

	@Override
	public void batchSendImGroupMessage(String messageContent, String senderId,String extra, String[] targetIds,String objectName) {
		//ExecutorService executor = Executors.newCachedThreadPool();
		CompletableFuture.runAsync(()-> imFeignService.groupSend(getImGroupMessage(objectName,senderId,targetIds,messageContent,extra)));
	}

	public static String imageToBase64(BufferedImage bufferedImage,String suffix) {
		Base64 encoder = new Base64();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			ImageIO.write(bufferedImage, suffix, baos);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return new String(encoder.encode((baos.toByteArray())));
	}

	@Override
	public void batchSendImGroupImgMessage(ImSendGroupMessage messageDto) throws Exception {
		LOGGER.info("batchSendImGroupImgMessage messageDto={}", messageDto);
		URL url = new URL(messageDto.getFileUrl());
		BufferedImage bufferedImage = Thumbnails.of(url).scale(0.1f).outputQuality(0.25f).asBufferedImage();
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		ImageIO.write(bufferedImage, "png", outputStream);
		byte[] bytes = outputStream.toByteArray();
		outputStream.close();
		int length = bytes.length;
		//文件大小不能超过393216 Bytes
		if (length > 393216) {
			throw new BizException("文件过大请调整 像素 或 文件大小");
		}
		messageDto.setObjectName("RC:ImgMsg");
        if (StringUtils.isNotBlank(messageDto.getTargetIds())) {
            ImMessageDto imMessageDto = this.getImGroupMessage(messageDto);
            //imMessageDto.setMessageContent(imageToBase64(bufferedImage, "png"));
            imMessageDto.setFileUrl(messageDto.getFileUrl());
            this.sendGroupMessage(imMessageDto);
        }
        if (StringUtils.isNotBlank(messageDto.getUserTargetIds())) {
            String targetIds = messageDto.getTargetIds();
            messageDto.setTargetIds(messageDto.getUserTargetIds());
            ImMessageDto imMessageDto = this.getImGroupMessage(messageDto);
            messageDto.setTargetIds(targetIds);
            //imMessageDto.setMessageContent(imageToBase64(bufferedImage, "png"));
            imMessageDto.setFileUrl(messageDto.getFileUrl());
            imMessageDto.setObjectName(messageDto.getObjectName());
            this.sendImMessage(imMessageDto);
        }
	}

	public void sendGroupMessage(ImMessageDto imMessageDto){
		LOGGER.info("sendGroupMessage groupSend ret={}", JSON.toJSONString(imMessageDto));
		//ExecutorService executor = Executors.newCachedThreadPool();
		CompletableFuture.runAsync(()->{
			Object ret;
			int maxRetry = 5;
			do {
				// 若发送失败，重试5次后放弃
				ret = imFeignService.groupSend(imMessageDto);
				LOGGER.info("sendGroupMessage groupSend retry={}, ret={}", maxRetry, JSON.toJSONString(ret));
			} while (ret == null && maxRetry-- > 0);
		});
	}


    public void sendImMessage(ImMessageDto imMessageDto){
		LOGGER.info("sendImMessage privateSend ret={}", JSON.toJSONString(imMessageDto));
        CompletableFuture.runAsync(()-> {
			Object ret;
			int maxRetry = 5;
			do {
				// 若发送失败，重试5次后放弃
				ret = imFeignService.privateSend(imMessageDto);
				LOGGER.info("sendGroupMessage groupSend retry={}, ret={}", maxRetry, JSON.toJSONString(ret));
			} while (ret == null && maxRetry-- > 0);
		});
    }

	@Override
	public void batchSendImGroupFileMessage(ImSendGroupMessage messageDto) {
		messageDto.setObjectName("RC:FileMsg");
        if (StringUtils.isNotBlank(messageDto.getTargetIds())) {
            ImMessageDto imMessageDto = this.getImGroupMessage(messageDto);
            imMessageDto.setMessageContent("");
            imMessageDto.setFileUrl(messageDto.getFileUrl());
            imMessageDto.setFileSize(messageDto.getFileSize());
            imMessageDto.setFileName(messageDto.getFileName());
            this.sendGroupMessage(imMessageDto);
        }
        if (StringUtils.isNotBlank(messageDto.getUserTargetIds())) {
            messageDto.setTargetIds(messageDto.getUserTargetIds());
            ImMessageDto imMessageDto = this.getImGroupMessage(messageDto);
            imMessageDto.setMessageContent("");
            imMessageDto.setFileUrl(messageDto.getFileUrl());
            imMessageDto.setFileSize(messageDto.getFileSize());
            imMessageDto.setFileName(messageDto.getFileName());
            this.sendImMessage(imMessageDto);
        }
	}

	private String checkMessageConfig(MessageTypeEnum type){
		SysMessageConfig messageConfig = sysMessageConfigService.queryByType(type);
		if (messageConfig == null) {
			throw new BizException("消息类型错误");
		}else if(messageConfig.getSendFlag() == null || messageConfig.getSendFlag() == 0){
			throw new BizException("消息类型未开启");
		}
		return messageConfig.getContent();
	}

	private String getContent(String messageContent, String url, Object... args) {
		String content = MessageFormatter.arrayFormat(messageContent, args);
		if(StringUtils.isNotEmpty(url)){
			content += url;
		}
		return content;
	}

	private ImMessageDto getImGroupMessage(ImSendGroupMessage messageDto) {
		ImMessageDto imMessageDto = new ImMessageDto();
		if(messageDto.getIsIncludeSender() == 0){
			//发送系统消息用指定的用户
			String userId = sysConfigDao.findConfigValue("system_message_user");
			imMessageDto.setSenderId(userId);
			SimpleUserDto simpleUser = teacherDao.getSimpleUser(Integer.parseInt(userId));
			ImSendMessageUserInfo userInfo = new ImSendMessageUserInfo();
			userInfo.setIcon(simpleUser.getAvatar());
			userInfo.setId(userId);
			userInfo.setName(simpleUser.getUserName());
			imMessageDto.setUserInfo(userInfo);
		}else {
			imMessageDto.setSenderId(messageDto.getSenderId());
		}
		imMessageDto.setObjectName(messageDto.getObjectName());
		imMessageDto.setTargetIds(messageDto.getTargetIds().split(","));
		imMessageDto.setExtra(messageDto.getExtra());
		imMessageDto.setMessageContent(messageDto.getMessageContent());
		imMessageDto.setIsIncludeSender(messageDto.getIsIncludeSender());
		return imMessageDto;
	}
	private ImMessageDto getImGroupMessage(String objectName, String senderId, String[] targetIds, String content, String extra) {
		ImMessageDto imMessageDto = new ImMessageDto();
		imMessageDto.setObjectName(objectName);
		imMessageDto.setTargetIds(targetIds);
		imMessageDto.setSenderId(senderId);
		imMessageDto.setExtra(extra);
		imMessageDto.setMessageContent(content);
		return imMessageDto;
	}

	@Override
	public void sendPrivateMessage(String studentId, String content, String extra) {
		Integer userId = sysUserService.getUserId();
		//ExecutorService executor = Executors.newCachedThreadPool();
		CompletableFuture.runAsync(()->{
			imFeignService.privateSend(getImGroupMessage("RC:TxtMsg", userId.toString(),
				studentId.split(","), content, extra));
		});
		//executor.shutdown();
	}

	@Override
	public void sendNoAuthPrivateMessage(String sender, String receiver, String content) {
		ImPrivateMessage privateMessage = new ImPrivateMessage();
		privateMessage.setObjectName("RC:TxtMsg");
		privateMessage.setTargetId(receiver.split(","));
		privateMessage.setSenderId(sender);
		privateMessage.setContent(new ImTxtMessage(content,null));
//		imFeignService.privateSend(privateMessage);
	}

	@Override
	public void pushWaitSendMessageTask() {
		List<WaitSendMessage> all = waitSendMessageDao.findAll(null);
		for (WaitSendMessage waitSendMessage : all) {
			Map<Integer,String> map = (Map<Integer,String>) JSON.parseObject(waitSendMessage.getReceivers(), Map.class);
			String[] args = null;
			if(waitSendMessage.getAgrs() != null){
				List<String> strings = JSON.parseArray(waitSendMessage.getAgrs(), String.class);
				String[] str = new String[strings.size()];
				args = strings.toArray(str);
			}
			batchSendMessage(MessageSender.valueOf(waitSendMessage.getMessageSender()),
					MessageTypeEnum.valueOf(waitSendMessage.getMessageType()),map,null,0,
					waitSendMessage.getUrl(),waitSendMessage.getJpushType(),args);
			waitSendMessageDao.delete(waitSendMessage.getId());
		}
	}

    @Override
    public R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMessageWrapper.SysMessage>> queryMessageCode(SysMessageVo.SysMessageQuery queryInfo) {
		CbsMessageWrapper.SysMessageQuery query = new CbsMessageWrapper.SysMessageQuery();
		query.setPage(queryInfo.getPage());
		query.setRows(queryInfo.getRows());
//		query.setStatus(ESendStatus.SUCCESSED);
		query.setAppKey(EAppKey.GYM);
		query.setTitle(queryInfo.getTitle());
		query.setSendMode(EMessageSendMode.SMS);
		query.setSendTimeStart(queryInfo.getSendTime() == null?null:DateUtil.stringToDate(queryInfo.getSendTime()+" 00:00:00"));
		return messageFeignClientService.sysMessagePage(query);
	}

    @Override
	public void batchSendMessage(Integer senderTenantId, MessageSender messageSender, MessageTypeEnum type, Map<Integer,String> receivers, Date triggerTime, Integer readStatus,
								 String url, String jpushType, Object... args) {
		LOGGER.info("batchSendMessage tenantId {} msgSender {}, type {}, receivers {} ", senderTenantId, messageSender, type, receivers);
		if (receivers == null || receivers.isEmpty()) {
			LOGGER.error("接收地址不能为空");
			return;
		}
		if(messageSender == MessageSender.JIGUANG && "STUDENT".equals(jpushType) && type != STUDENT_PUSH_VIP_BUY){
			//如果不是缴费信息
			int hour = DateUtil.getHour(new Date());
			//如果当前时间在22点之后
			String messageTime = sysConfigDao.findConfigValue("wait_send_message_time");
			if(StringUtils.isEmpty(messageTime)){
				messageTime = "22";
			}
			if(messageTime.contains(":")){
				messageTime = messageTime.substring(0,messageTime.indexOf(":"));
			}
			if(hour > Integer.parseInt(messageTime)-1){
				//保存推送信息
				WaitSendMessage waitSendMessage = new WaitSendMessage();
				if(args != null){
					waitSendMessage.setAgrs(JSON.toJSONString(args));
				}
				waitSendMessage.setJpushType(jpushType);
				waitSendMessage.setMessageSender(messageSender.name());
				waitSendMessage.setMessageType(type.name());
				waitSendMessage.setReceivers(JSON.toJSONString(receivers));
				waitSendMessage.setUrl(url);
				waitSendMessageDao.insert(waitSendMessage);
				return;
			}
		}
		if(StringUtils.isEmpty(jpushType)){
			jpushType = "STUDENT";
		}
		// 处理注册验证码，没有用户ID的情况导致没有发送短息
		List<CbsMessageWrapper.MessageParam> messageParams = new ArrayList<>();
		List<SimpleUserDto> userDtoList = teacherDao.getUsersSimpleInfo(new ArrayList<>(receivers.keySet()));
		Map<Integer, SimpleUserDto> userMap = userDtoList.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, Function.identity()));
		for (Map.Entry<Integer, String> receive : receivers.entrySet()) {
			Long userId = Long.valueOf(Optional.ofNullable(receive.getKey()).orElse(-1));
			SimpleUserDto user = userMap.getOrDefault(userId.intValue(), new SimpleUserDto());
			CbsMessageWrapper.MessageParam messageParam = new CbsMessageWrapper.MessageParam();
			messageParam.setUserId(userId);
			messageParam.setAppKey(EAppKey.GYM);
			messageParam.setClientType(getClientType(jpushType));
			messageParam.setClientId(getClientType(jpushType));
			messageParam.setType(type.getCode());
			messageParam.setUrl(url);
			Map<Long, String> params = new HashMap<>();
			if(messageSender == MessageSender.EMAIL){
				//如果是极光推送，没有用户ID的情况下，使用手机号
				params.put(userId, Optional.ofNullable(receive.getValue()).orElse(user.getPhone()));
			}else {
				params.put(userId, Optional.ofNullable(user.getPhone()).orElse(receive.getValue()));
			}
			messageParam.setReceivers(params);
			messageParam.setRealSend(!debugMode);
			if(args != null && args.length > 0){
				String[] argsStr = new String[args.length];
				for (int i = 0; i < args.length; i++) {
					if (args[i]!=null){
						argsStr[i] = args[i].toString();
					}else {
						argsStr[i] = "";
					}
				}
				messageParam.setContentArgs(argsStr);
			}
			messageParams.add(messageParam);
		}
		messageFeignClientService.sysMessageBatchSendMessage(messageParams);
	}

    @Override
	public void batchSendMessage(MessageSender messageSender, MessageTypeEnum type, Map<Integer,String> receivers, Date triggerTime, Integer readStatus,
			String url,String jpushType, Object... args) {
		batchSendMessage(TenantContextHolder.getTenantId(), messageSender, type, receivers, triggerTime, readStatus, url, jpushType, args);
	}


	@Override
	public void batchPushMessage(MessageTypeEnum type, Map<Integer,String> receivers, Date triggerTime, Integer readStatus,
			String url,String jpushType,String sound,String channelId, Object... args) {
		if (receivers == null || receivers.isEmpty()) {
			LOGGER.error("接收地址不能为空");
			return;
		}
		if("STUDENT".equals(jpushType) && type != STUDENT_PUSH_VIP_BUY){
			int hour = DateUtil.getHour(new Date());
			//如果当前时间在22点之后
			String messageTime = sysConfigDao.findConfigValue("wait_send_message_time");
			if(StringUtils.isEmpty(messageTime)){
				messageTime = "22";
			}
			if(messageTime.contains(":")){
				messageTime = messageTime.substring(0,messageTime.indexOf(":"));
			}
			if(hour > Integer.parseInt(messageTime)-1){
				//保存推送信息
				WaitSendMessage waitSendMessage = new WaitSendMessage();
				if(args != null){
					waitSendMessage.setAgrs(JSON.toJSONString(args));
				}
				waitSendMessage.setJpushType(jpushType);
				waitSendMessage.setMessageSender(MessageSender.JIGUANG.name());
				waitSendMessage.setMessageType(type.name());
				waitSendMessage.setReceivers(JSON.toJSONString(receivers));
				waitSendMessage.setUrl(url);
				waitSendMessageDao.insert(waitSendMessage);
				return;
			}
		}
		if(StringUtils.isEmpty(jpushType)){
			jpushType = "STUDENT";
		}
		List<SimpleUserDto> userDtoList = teacherDao.getUsersSimpleInfo(new ArrayList<>(receivers.keySet()));
		List<CbsMessageWrapper.MessageParam> messageParams = new ArrayList<>();
		for (SimpleUserDto user : userDtoList) {
			CbsMessageWrapper.MessageParam messageParam = new CbsMessageWrapper.MessageParam();
			messageParam.setUserId(user.getUserId().longValue());
			messageParam.setAppKey(EAppKey.GYM);
			messageParam.setClientType(EClientType.valueOf(jpushType));
			messageParam.setClientId(messageParam.getClientType());
			messageParam.setType(type.getCode());
			messageParam.setUrl(url);
			Map<Long, String> params = new HashMap<>();
			params.put(user.getUserId().longValue(), user.getPhone());
			messageParam.setReceivers(params);
			messageParam.setRealSend(!debugMode);
			if(args != null && args.length > 0){
				String[] argsStr = new String[args.length];
				for (int i = 0; i < args.length; i++) {
					if (args[i]!=null){
						argsStr[i] = args[i].toString();
					}else {
						argsStr[i] = "";
					}
				}
				messageParam.setContentArgs(argsStr);
			}
			messageParams.add(messageParam);
		}
		messageFeignClientService.sysMessageBatchSendMessage(messageParams);
	}


	@Override
	public boolean sendSecurityCode(MessageSender messageSender, Integer userId, MessageTypeEnum messageType, String receiver,String jpushType) {
		String key1 = getVerificationCode1CacheKey(MessageTypeEnum.SMS_VERIFY_CODE_LOGIN, receiver);
		if (redisCache.get(key1) != null) {
			throw new BizException("请勿频繁操作，获取验证码间隔时间为60秒");
		}

		String key = getVerificationCodeCacheKey(MessageTypeEnum.SMS_VERIFY_CODE_LOGIN, receiver);
		int code = DEFAULT_CODE;
		if (!debugMode) {
			code = getRandomCode(MessageTypeEnum.SMS_VERIFY_CODE_LOGIN, receiver);
		}
		Map<Integer, String> receivers = new HashMap<>(1);
		receivers.put(userId, receiver);
		batchSendMessage(messageSender, messageType, receivers, null, 1, "",jpushType, Integer.toString(code));
		redisCache.put(key, code + "", CODE_EXPIRE);
		redisCache.put(key1, code + "", CODE_INTERVAL_TIME);
		return true;
	}

	@Override
	public String getSendedVerificationCode(MessageTypeEnum type, String mobileNOOrEmailAddr) {
		if (debugMode) {
			return DEFAULT_CODE + "";
		}
		String key = getVerificationCodeCacheKey(type, mobileNOOrEmailAddr);
		Object object = redisCache.get(key);
		return object == null ? null : object.toString();
	}


	@Override
	public Map<String, Integer> queryCountOfUnread(MessageSendMode type, Integer userId, String jpushType, Integer tenantId) {
		CbsMessageWrapper.SysMessageQuery query = new CbsMessageWrapper.SysMessageQuery();
		query.setUserId(userId.longValue());
		query.setClientType(getClientType(jpushType));
//		query.setStatus(ESendStatus.SUCCESSED);
		query.setAppKey(EAppKey.GYM);
		query.setSendMode(getMessageSendMode(type.getCode()));
		query.setReadStatus(false);
		List<CbsMessageWrapper.MessageUnreadStat> messageUnreadStats = messageFeignClientService.messageUnreadStat(query).feignData();
//		List<Mapper> mappers = sysMessageDao.queryCountOfUnread(type, userId, jpushType, tenantId);

		Map<String, Integer> map = new HashMap<String, Integer>();

		for (CbsMessageWrapper.MessageUnreadStat mapper : messageUnreadStats) {
			map.put(mapper.getGroup(), mapper.getNumber());
		}

		return map;
	}

	public void setDebugMode(boolean debugMode) {
		this.debugMode = debugMode;
	}

	private int getRandomCode(MessageTypeEnum type, String mobileNo) {
		int min = 100000;
		int max = 999999;
		Random random = new Random();

		return random.nextInt(max) % (max - min + 1) + min;
	}

	private static String getVerificationCodeCacheKey(MessageTypeEnum type, String mobileNOOrEmailAddr) {
		return "verificationCode" + type.name() + mobileNOOrEmailAddr;
	}

	private static String getVerificationCode1CacheKey(MessageTypeEnum type, String mobileNOOrEmailAddr) {
		return "verificationCode" + mobileNOOrEmailAddr;
	}

	/**
	 *
	 * @param userId 用户id数组
	 * @param status 状态/1 已读,0未读
	 * @return 修改的条数
	 */
	@Override
	public Boolean updateStatus(Integer userId, int status,String jpushType) {
		CbsMessageWrapper.MessageRead messageRead = new CbsMessageWrapper.MessageRead();
		messageRead.setUserId(userId.longValue());
		messageRead.setAppKey(EAppKey.GYM);
		messageRead.setClientType(getClientType(jpushType));
		return messageFeignClientService.batchSetRead(messageRead).feignData();
	}

	@Override
	public Boolean updateOneStatus(Long id) {
		return messageFeignClientService.setRead(id).feignData();
	}

	@Override
	public List<CbsMessageWrapper.SysMessage> queryUserInRecentMinList(String mobile, int recentMin, MessageSendMode type) {
		CbsMessageWrapper.UserRecentMessageQuery query = new CbsMessageWrapper.UserRecentMessageQuery();
		query.setPhone(mobile);
		query.setRecentMin(recentMin);
		query.setSendMode(EMessageSendMode.SMS);
		query.setAppKey(EAppKey.GYM);
		return messageFeignClientService.userRecentMessageRecords(query).feignData();
	}
}
