package com.ym.mec.biz.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.dayaedu.cbs.common.enums.EClientType;
import com.dayaedu.cbs.common.enums.music.EAudioType;
import com.dayaedu.cbs.common.enums.music.EMusicAvailableType;
import com.dayaedu.cbs.common.enums.music.EMusicPlayMode;
import com.dayaedu.cbs.common.enums.music.EMusicSourceType;
import com.dayaedu.cbs.common.enums.school.EMusicSheetType;
import com.dayaedu.cbs.domain.wrapper.MusicSheetWrapper;
import com.dayaedu.cbs.openfeign.client.MusicFeignClientService;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetApplicationExtendClientWrapper;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.dayaedu.cbs.openfeign.wrapper.musicInstrument.CbsMusicalInstrumentWrapper;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.biz.dal.dao.SysMusicScoreAccompanimentDao;
import com.ym.mec.biz.dal.dao.SysMusicScoreCategoriesDao;
import com.ym.mec.biz.dal.dao.SysMusicScoreDao;
import com.ym.mec.biz.dal.dto.ReasonDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.SysExamSongQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
public class SysMusicScoreServiceImpl extends BaseServiceImpl<Integer, SysMusicScore> implements SysMusicScoreService {

    @Resource
    private SysMusicScoreDao sysMusicScoreDao;
    @Resource
    private SysMusicScoreCategoriesDao sysMusicScoreCategoriesDao;
    @Resource
    private CbsMusicScoreCategoriesService cbsMusicScoreCategoriesService;
    @Resource
    private SubjectService subjectService;
    @Resource
    private SysMusicScoreAccompanimentDao sysMusicScoreAccompanimentDao;
    @Resource
    private SysMusicScoreAccompanimentService sysMusicScoreAccompanimentService;
    @Resource
    private MusicFeignClientService musicFeignClientService;
    @Resource
    private InstrumentService instrumentService;

    @Value("${openfeign-client.app-id:1745637981387108354}")
    public Long applicationId;

    @Override
    public BaseDAO<Integer, SysMusicScore> getDAO() {
        return sysMusicScoreDao;
    }

    @Override
    public SysMusicScoreDao getDao() {
        return sysMusicScoreDao;
    }

    private String getParentTreeName(Integer categoriesId) {
        StringBuffer sb = new StringBuffer();
        while (true) {
            SysMusicScoreCategories categories = sysMusicScoreCategoriesDao.get(categoriesId);
            if (sb.length() == 0) {
                sb.append(categories.getName());
            } else {
                sb.insert(0, "/");
                sb.insert(0, categories.getName());
            }
            if (categories.getParentId() == 0) {
                return sb.toString();
            }
            categoriesId = categories.getParentId();
        }
    }

    @Override
    public CbsMusicSheetWrapper.MusicSheetApplicationQuery getMusicSheetApplicationQuery() {
        CbsMusicSheetWrapper.MusicSheetApplicationQuery query = new CbsMusicSheetWrapper.MusicSheetApplicationQuery();
        query.setDetailFlag(true);
        query.setDelFlag(true);
        query.setPage(1);
        query.setAudioPlayTypeFlag(false);
        query.setApplicationId(applicationId);
        return query;
    }

    @Override
    public List<CbsMusicSheetWrapper.MusicSheetApplication> queryCbsMusicSheetApplication(CbsMusicSheetWrapper.MusicSheetApplicationQuery query) {
        R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMusicSheetWrapper.MusicSheetApplication>> pageInfoR =
                musicFeignClientService.musicSheetPageByApplication(query);
        if(pageInfoR.getCode() != 200){
            throw new BizException("获取曲目信息失败");
        }
        com.microsvc.toolkit.common.response.paging.PageInfo<CbsMusicSheetWrapper.MusicSheetApplication> pageInfo = pageInfoR.getData();
        if(pageInfo != null && CollectionUtils.isNotEmpty(pageInfo.getRows())){
            return pageInfo.getRows();
        }
        return new ArrayList<>();
    }

    @Override
    public PageInfo<CbsMusicSheetWrapper.MusicSheetAccApplication> queryCbsMusicSheetSoundApplication(CbsMusicSheetWrapper.MusicSheetApplicationQuery query) {
        R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMusicSheetWrapper.MusicSheetAccApplication>> pageInfoR =
                musicFeignClientService.musicSheetAccPageByApplication(query);
        if(pageInfoR.getCode() != 200){
            throw new BizException("获取曲目信息失败");
        }
        com.microsvc.toolkit.common.response.paging.PageInfo<CbsMusicSheetWrapper.MusicSheetAccApplication> pageInfo = pageInfoR.getData();
        return PageUtil.pageInfo(pageInfo);
    }

    @Override
    public List<CbsMusicSheetWrapper.MusicSheetApplication> queryCbsMusicSheetApplication(List<String> cbsMusicSheetIds){
        CbsMusicSheetWrapper.MusicSheetApplicationQuery query = this.getMusicSheetApplicationQuery();
        query.setMusicSheetIds(cbsMusicSheetIds.stream().map(Long::parseLong).collect(Collectors.toList()));
        query.setRows(cbsMusicSheetIds.size());
        return this.queryCbsMusicSheetApplication(query);
    }

    //获取乐器名称
    @Override
    public List<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> queryMusicalInstrument(List<Integer> musicalInstrumentIds){
        //获取乐器名称
        CbsMusicalInstrumentWrapper.MusicalInstrumentQuery musicalInstrumentQuery = new CbsMusicalInstrumentWrapper.MusicalInstrumentQuery();
        musicalInstrumentQuery.setIds(musicalInstrumentIds);
        musicalInstrumentQuery.setPage(1);
        musicalInstrumentQuery.setRows(musicalInstrumentIds.size());
        R<com.microsvc.toolkit.common.response.paging.PageInfo<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto>> infoR =
                musicFeignClientService.musicalInstrumentPage(musicalInstrumentQuery);
        if(infoR.getCode() != 200){
            throw new com.microsvc.toolkit.common.webportal.exception.BizException("获取乐器信息失败,{}",infoR.getMessage());
        }
        return infoR.getData().getRows();
    }

    private void dealMusicScoreData(List<SysMusicScore> records){
        if (CollectionUtils.isNotEmpty(records)) {
            List<String> cbsMusicSheetIds = records.stream().map(SysMusicScore::getCbsMusicSheetId).collect(Collectors.toList());
            List<CbsMusicSheetWrapper.MusicSheetApplication> applications = this.queryCbsMusicSheetApplication(cbsMusicSheetIds);
            if(CollectionUtils.isNotEmpty(applications)){
                //获取原音
                List<Integer> musicSheetIds = records.stream().map(SysMusicScore::getId).collect(Collectors.toList());
                List<SysMusicScoreAccompaniment> accompaniments = sysMusicScoreAccompanimentService.queryAccPage(musicSheetIds,applications);

                Map<Long, CbsMusicSheetWrapper.MusicSheetApplication> musicSheetApplicationMap = applications
                        .stream().collect(Collectors.toMap(CbsMusicSheetWrapper.MusicSheetApplication::getId, Function.identity()));

                //获取乐器名称
                String instrumentIds = applications.stream().map(CbsMusicSheetWrapper.MusicSheetApplication::getMusicalInstrumentIds)
                        .filter(StringUtils::isNotEmpty).collect(Collectors.joining(","));
                Map<Integer,CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> musicalInstrumentQueryMap = new HashMap<>();
                if(StringUtils.isNotEmpty(instrumentIds)){
                    List<CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> musicalInstrumentQueryDtos =
                            this.queryMusicalInstrument(Arrays.stream(instrumentIds.split(",")).map(Integer::parseInt).distinct().collect(Collectors.toList()));
                    musicalInstrumentQueryMap = musicalInstrumentQueryDtos.stream().collect(Collectors.toMap(CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto::getId, o -> o, (o1, o2) -> o1));
                }
                Map<Integer, List<SysMusicScoreAccompaniment>> accompanimentMap = new HashMap<>();
                if(CollectionUtils.isNotEmpty(accompaniments)){
                    accompanimentMap = accompaniments.stream().collect(Collectors.groupingBy(SysMusicScoreAccompaniment::getExamSongId));
                }
                //获取声部乐器关联
                List<Instrument> list = instrumentService.lambdaQuery().list();
                Map<Integer, Instrument> instrumentMap = list.stream().collect(Collectors.toMap(Instrument::getSubjectId, o -> o));
                for (SysMusicScore record : records) {
                    CbsMusicSheetWrapper.MusicSheetApplication musicSheetApplication = musicSheetApplicationMap.get(Long.parseLong(record.getCbsMusicSheetId()));
                    if(musicSheetApplication != null){
                        this.initMusicSheetVo(record,musicSheetApplication,musicalInstrumentQueryMap,instrumentMap);
                    }
                    List<SysMusicScoreAccompaniment> accompanimentList = accompanimentMap.get(record.getId());
                    if(CollectionUtils.isNotEmpty(accompanimentList)){
                        record.setBackground(accompanimentList);
                    }
                }
            }
        }
    }

    public void initMusicSheetVo(SysMusicScore record,CbsMusicSheetWrapper.MusicSheetApplication musicSheetApplication
            ,Map<Integer,CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto> musicalInstrumentQueryMap,
                                 Map<Integer, Instrument> instrumentMap){
        record.setRankIds(StringUtils.equals("FREE",musicSheetApplication.getPaymentType()) ? "" : "1");
        record.setCbsMusicSheetId(musicSheetApplication.getId().toString());
        record.setPlayMode(SysMusicScore.PlayMode.valueOf(musicSheetApplication.getPlayMode().getCode()));
        record.setExtConfigJson(musicSheetApplication.getExtConfigJson());
        record.setExtStyleConfigJson(musicSheetApplication.getExtStyleConfigJson());
        record.setShowFlag(musicSheetApplication.getStatus()?1:0);
//        if(musicSheetApplication.getMusicSheetCategoryId() != null){
//            record.setMusicScoreCategoriesId(musicSheetApplication.getMusicSheetCategoryId().intValue());
//            record.setCategoriesId(musicSheetApplication.getMusicSheetCategoryId().intValue());
//        }
        record.setName(musicSheetApplication.getName());
        record.setCategoriesName(musicSheetApplication.getMusicSheetCategoryName());
        record.setSpeed(musicSheetApplication.getPlaySpeed());
        record.setIsOpenMetronome(musicSheetApplication.getIsUseSystemBeat());
        record.setIsMixBeat(musicSheetApplication.getIsMixBeat());
        record.setMusicPdfUrl(musicSheetApplication.getMusicPdfUrl());
        record.setFirstPdfUrl(musicSheetApplication.getFirstPdfUrl());
        record.setJianPdfUrl(musicSheetApplication.getJianPdfUrl());
        List<CbsMusicSheetWrapper.MusicSheetAccompaniment> accompanimentList = musicSheetApplication.getMusicSheetAccompanimentList();
        if (CollectionUtils.isNotEmpty(accompanimentList)) {
            CbsMusicSheetWrapper.MusicSheetAccompaniment accompaniment = accompanimentList.get(0);
            record.setMetronomeUrl(accompaniment.getAudioFileUrl());
            record.setUrl(accompaniment.getAudioFileUrl());
            record.setAccMusicPdfUrl(accompaniment.getMusicPdfUrl());
            record.setAccFirstPdfUrl(accompaniment.getFirstPdfUrl());
            record.setAccJianPdfUrl(accompaniment.getJianPdfUrl());
        }
        List<CbsMusicSheetWrapper.MusicSheetSound> soundList = musicSheetApplication.getMusicSheetSoundList();
        if (CollectionUtils.isNotEmpty(soundList)) {
            CbsMusicSheetWrapper.MusicSheetSound sheetSound = soundList.get(0);
            record.setAccompanimentMetronomeUrl(sheetSound.getAudioFileUrl());
            record.setAccompanimentUrl(sheetSound.getAudioFileUrl());
            record.setSoundMusicPdfUrl(sheetSound.getMusicPdfUrl());
            record.setSoundFirstPdfUrl(sheetSound.getFirstPdfUrl());
            record.setSoundJianPdfUrl(sheetSound.getJianPdfUrl());
        }
        record.setXmlUrl(musicSheetApplication.getXmlFileUrl());
        record.setMidiUrl(musicSheetApplication.getMidiFileUrl());
        record.setOrder(musicSheetApplication.getSortNo());
        record.setEnableEvaluation(musicSheetApplication.getIsEvaluated());
        record.setIsConvertibleScore(musicSheetApplication.getIsConvertibleScore());
        record.setScoreType(musicSheetApplication.getScoreType().getCode());
        if(StringUtils.isNotEmpty(record.getSubjectIds())){
            //获取第一个声部
            String[] subjectIds = record.getSubjectIds().split(",");
            record.setSubjectId(Integer.parseInt(subjectIds[0]));
            if(StringUtils.isNotEmpty(musicSheetApplication.getMusicalInstrumentIds())){
                Set<String> subjectNames = new HashSet<>();
                for (String subjectId : record.getSubjectIds().split(",")) {
                    Instrument instrument = instrumentMap.get(Integer.parseInt(subjectId));
                    if(instrument != null){
                        CbsMusicalInstrumentWrapper.MusicalInstrumentQueryDto musicalInstrumentQueryDto = musicalInstrumentQueryMap.get(instrument.getId());
                        if(Objects.nonNull(musicalInstrumentQueryDto)) {
                            subjectNames.add(musicalInstrumentQueryDto.getName());
                        }
                    }
                }
                record.setSubjectName(String.join(",",subjectNames));
            }
        }
    }

    @Override
    public List<SysMusicScore> queryMusicScoreAll(SysExamSongQueryInfo queryInfo) {
        //封装查询参数
//        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        if(queryInfo.getSubjectId() != null){
            queryInfo.setSubjectIds(queryInfo.getSubjectId().toString());
        }
        this.initCategoryIds(queryInfo);
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<SysMusicScore> rows = findAll(params);
        //处理查询结果
        dealMusicScoreData(rows);
        return rows;
    }

    @Override
    public PageInfo<SysMusicScore> queryMusicScorePage(SysExamSongQueryInfo queryInfo) {
        //封装查询参数
//        queryInfo.setOrganId(organizationService.getEmployeeOrgan(queryInfo.getOrganId()));
        if(queryInfo.getSubjectId() != null){
            queryInfo.setSubjectIds(queryInfo.getSubjectId().toString());
        }
        this.initCategoryIds(queryInfo);

        PageInfo<SysMusicScore> sysMusicScorePageInfo = queryPage(queryInfo);
        List<SysMusicScore> rows = sysMusicScorePageInfo.getRows();
        //处理查询结果
        dealMusicScoreData(rows);
        return sysMusicScorePageInfo;
    }

    private void initCategoryIds(SysExamSongQueryInfo queryInfo){
        if(StringUtils.isNotEmpty(queryInfo.getCategoriesId())){
            List<Integer> categoriesIdList = cbsMusicScoreCategoriesService.getAllCategoryIdList(queryInfo.getCategoriesId());
            queryInfo.setCategoriesIdList(categoriesIdList);
        }
    }


    @Override
    public PageInfo<SysMusicScore> queryMusicScorePageInfo(SysExamSongQueryInfo queryInfo) {
        this.initCategoryIds(queryInfo);
        if(queryInfo.getSubjectId() != null){
            if(queryInfo.getSubjectId() == 5){
                queryInfo.setSubjectId(6);
                queryInfo.setSubjectIds("6");
            }else {
                queryInfo.setSubjectIds(queryInfo.getSubjectId().toString());
            }
        }
        if(CollectionUtils.isEmpty(queryInfo.getCategoriesIdList())){
            return new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        }
        PageInfo<SysMusicScore> pageInfo = this.queryPage(queryInfo);
        List<SysMusicScore> rows = pageInfo.getRows();
        dealMusicScoreData(rows);
        return pageInfo;
    }

    @Override
    public Map<Long, SysMusicScore> getMapByIds(List<Long> musicSheetIds) {
        if (CollectionUtils.isEmpty(musicSheetIds)) {
            return new HashMap<>();
        }
        List<SysMusicScore> musicScoreList = sysMusicScoreDao.findByIds(musicSheetIds.stream().map(String::valueOf).collect(Collectors.joining(",")));
        if (CollectionUtils.isEmpty(musicScoreList)) {
            return new HashMap<>();
        }
        dealMusicScoreData(musicScoreList);
        return musicScoreList.stream().collect(Collectors.toMap(o -> o.getId().longValue(), e -> e));
    }

    @Override
    public Boolean stateList(ReasonDto reasonDto) {
        List<Integer> musicSheets = Arrays.stream(reasonDto.getMusicSheetIds().split(",")).map(Integer::parseInt).collect(Collectors.toList());
        List<SysMusicScore> musicSheetList = sysMusicScoreDao.findByIds(musicSheets.stream().map(String::valueOf).collect(Collectors.joining(",")));

        List<CbsMusicSheetWrapper.MusicSheetApplication> rows =
                this.queryCbsMusicSheetApplication(musicSheetList.stream().map(e -> e.getCbsMusicSheetId()).collect(Collectors.toList()));
        String applicationExtendIds = rows.stream().map(e -> e.getApplicationExtendId().toString()).collect(Collectors.joining(","));
        CbsMusicSheetApplicationExtendClientWrapper.MusicSheetApplicationExtendStatus extendStatus = new CbsMusicSheetApplicationExtendClientWrapper.MusicSheetApplicationExtendStatus();
        extendStatus.setIds(applicationExtendIds);
        extendStatus.setStatus(reasonDto.getState()== YesOrNoEnum.YES);
        R<JSONObject> jsonObjectR = musicFeignClientService.musicSheetApplicationExtendStatus(extendStatus);
        if (jsonObjectR.getCode() != 200) {
            throw new BizException("修改曲目状态失败,{}", jsonObjectR.getMessage());
        }
        return true;
    }

    @Override
    public Boolean state(Integer id) {
        SysMusicScore musicScore = sysMusicScoreDao.get(id);
        if (musicScore == null) {
            throw new BizException("曲目信息不存在");
        }
        List<CbsMusicSheetWrapper.MusicSheetApplication> rows =
                this.queryCbsMusicSheetApplication(Collections.singletonList(musicScore.getCbsMusicSheetId()));
        String applicationExtendIds = rows.stream().map(e -> e.getApplicationExtendId().toString()).collect(Collectors.joining(","));
        CbsMusicSheetApplicationExtendClientWrapper.MusicSheetApplicationExtendStatus extendStatus = new CbsMusicSheetApplicationExtendClientWrapper.MusicSheetApplicationExtendStatus();
        extendStatus.setIds(applicationExtendIds);
        extendStatus.setStatus(musicScore.getClientShowFlag() != 1);
        R<JSONObject> jsonObjectR = musicFeignClientService.musicSheetApplicationExtendStatus(extendStatus);
        if (jsonObjectR.getCode() != 200) {
            throw new BizException("修改曲目状态失败,{}", jsonObjectR.getMessage());
        }
        return true;
    }

    @Override
    public void updateCbs(MusicSheetWrapper.MusicSheetApplicationExtendWrapper addMusicSheetApplicationExtend) {
        CbsMusicSheetApplicationExtendClientWrapper.AddMusicSheetApplicationExtend extend =
                JSONObject.parseObject(JSONObject.toJSONString(addMusicSheetApplicationExtend), CbsMusicSheetApplicationExtendClientWrapper.AddMusicSheetApplicationExtend.class);
        extend.setApplicationId(applicationId);
        R<JSONObject> jsonObjectR = musicFeignClientService.musicSheetApplicationExtendUpdate(extend);
        if (jsonObjectR.getCode() != 200) {
            throw new BizException("修改失败,{}",jsonObjectR.getMessage());
        }
    }

    @Override
    public void sync() {
        ThreadPoolExecutor executorService =  new ThreadPoolExecutor(50, 50,
                0L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<>());
        List<SysMusicScore> records = sysMusicScoreDao.selectSyncPage();
        if (CollectionUtils.isEmpty(records)) {
            return;
        }
        for (SysMusicScore record : records) {
            executorService.submit(() ->
                    {
                        try {
                            syncMusicSheet(record);
                        }catch (Exception e) {
                            log.error("曲目同步异常，曲目ID：{}", record.getId(), e);
                        }
                    }
            );
        }
        while (!executorService.getQueue().isEmpty()) {
            try {
                log.info("线程池中正在执行的任务数量：{},等待数量：{}", executorService.getActiveCount(),executorService.getQueue().size());
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                log.error("线程等待异常", e);
            }
        }
        log.info("-------------------- 曲目数据同步成功! -------------------------------");
        executorService.shutdown();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void del(Long id) {
        SysMusicScore musicSheet = sysMusicScoreDao.getCbsId(id);
        if (musicSheet == null) {
            throw new BizException("曲目不存在");
        }
        CbsMusicSheetWrapper.MusicSheetApplicationQuery query = this.getMusicSheetApplicationQuery();
        query.setMusicSheetIds(Collections.singletonList(Long.parseLong(musicSheet.getCbsMusicSheetId())));
        query.setRows(1);
        List<CbsMusicSheetWrapper.MusicSheetApplication> rows = this.queryCbsMusicSheetApplication(query);
        if (CollectionUtils.isEmpty(rows)) {
            throw new BizException("获取曲目信息失败");
        }
        CbsMusicSheetWrapper.MusicSheetApplication musicSheetApplication = rows.get(0);
        R<JSONObject> jsonObjectR = musicFeignClientService.musicSheetApplicationExtendDel(musicSheetApplication.getApplicationExtendId());
        if (jsonObjectR.getCode() != 200) {
            throw new BizException("曲目删除失败,{}", jsonObjectR.getMessage());
        }
    }

    private void syncMusicSheet(SysMusicScore record) {
        List<SysMusicScoreAccompaniment> list = sysMusicScoreAccompanimentDao.findByMusicScoreId(record.getId());

        CbsMusicSheetWrapper.AddMusicSheet addMusicSheet = this.musicToAddMusicSheet(record,list);

        R<CbsMusicSheetWrapper.MusicSheet> musicSheetR = musicFeignClientService.musicCbsSync(addMusicSheet);
        try {
            CbsMusicSheetWrapper.MusicSheet musicSheet = musicSheetR.feignData();
            if (musicSheet == null) {
                log.error("曲目同步失败，曲目ID：{}", record.getId());
            }
        } catch (Exception e){
            log.error("曲目同步失败，曲目ID：{}", record.getId(),e);
        }

    }

    //测试环境合奏分类
//    private static String categoryIds = "42,43,44,45,46,91,92,93,94,95,7,11,15,180";
    private static String prodCategoryIds = "42,43,44,45,46,91,92,93,94,95,185,186,7,11,15,180";

    private CbsMusicSheetWrapper.AddMusicSheet musicToAddMusicSheet(SysMusicScore from, List<SysMusicScoreAccompaniment> accList) {
        CbsMusicSheetWrapper.AddMusicSheet addMusicSheet = new CbsMusicSheetWrapper.AddMusicSheet();
        addMusicSheet.setId(Long.parseLong(from.getCbsMusicSheetId()));
        addMusicSheet.setName(from.getName());
        if (StringUtils.isNotBlank(from.getSubjectIds())) {
            List<Integer> subjectIds = Arrays.stream(from.getSubjectIds().split(","))
                    .map(Integer::parseInt).distinct().collect(Collectors.toList());
            List<Subject> list = subjectService.findBySubjectByIdList(subjectIds);
            if (CollectionUtils.isNotEmpty(list)) {
                addMusicSheet.setSubjectIds(list.stream().filter(e->e.getCbsSubjectId() != null).map(o->o.getCbsSubjectId().toString()).collect(Collectors.joining(",")));
                //获取关联的乐器
                List<Instrument> list1 = instrumentService.lambdaQuery().in(Instrument::getSubjectId, subjectIds).list();
                if (CollectionUtils.isNotEmpty(list1)) {
                    addMusicSheet.setMusicalInstrumentIds(list1.stream().map(o -> o.getId().toString()).collect(Collectors.joining(",")));
                }
            }
        }
        addMusicSheet.setSendAppFlag(false);
        addMusicSheet.setMusicCover("https://oss.dayaedu.com/gyt/07/1690789356356.png");
        addMusicSheet.setAppAuditFlag(false);
        addMusicSheet.setRemark(from.getMuseScoreMemo());
        addMusicSheet.setIsPlayBeat(true);

        String extConfigJson = from.getExtConfigJson();
        if(StringUtils.isNotEmpty(extConfigJson)){
            Integer multitrack = JSONObject.parseObject(extConfigJson).getInteger("multitrack");
            if(multitrack != null && multitrack > 1){
                addMusicSheet.setMusicSheetType(EMusicSheetType.SINGLE);
            }
        }
        if(addMusicSheet.getMusicSheetType() == null){
            if(Arrays.stream(prodCategoryIds.split(",")).anyMatch(e->e.equals(from.getCbsMusicCategoriesId().toString()))){
                addMusicSheet.setMusicSheetType(EMusicSheetType.CONCERT);
            }else {
                addMusicSheet.setMusicSheetType(EMusicSheetType.SINGLE);
            }
        }
        addMusicSheet.setIsEvaluated(from.isEnableEvaluation());
        addMusicSheet.setIsConvertibleScore(true);
        addMusicSheet.setPlayMode(from.getPlayMode() == SysMusicScore.PlayMode.MP3 ? EMusicPlayMode.MP3 : EMusicPlayMode.MIDI);
        addMusicSheet.setMidiFileUrl(from.getMidiUrl());
        addMusicSheet.setExtConfigJson(from.getExtConfigJson());
        addMusicSheet.setCreateTime(from.getCreateTime());
        addMusicSheet.setAudioType(EAudioType.HOMEMODE);
        addMusicSheet.setAvailableType(EMusicAvailableType.ORG);
        addMusicSheet.setSourceType(EMusicSourceType.PLATFORM);
        addMusicSheet.setStatus(from.getShowFlag() == 1);

        String tracks = "";
        // 伴奏，原音 扩展信息
        if (addMusicSheet.getPlayMode() == EMusicPlayMode.MP3) {
            if (CollectionUtils.isNotEmpty(accList)) {
                List<CbsMusicSheetWrapper.MusicSheetSound> musicSheetSounds = new ArrayList<>();
                int i = 0;
                for (SysMusicScoreAccompaniment sheetAccompaniment : accList) {
                    CbsMusicSheetWrapper.MusicSheetSound musicSheetSound = new CbsMusicSheetWrapper.MusicSheetSound();
                    if (sheetAccompaniment.getIsOpenMetronome() != null && sheetAccompaniment.getIsOpenMetronome()){
                        musicSheetSound.setAudioFileUrl(sheetAccompaniment.getMp3Url());
                    } else {
                        musicSheetSound.setAudioFileUrl(sheetAccompaniment.getMetronomeMp3Url());
                    }
                    musicSheetSound.setTrack(sheetAccompaniment.getTrack());
                    if (sheetAccompaniment.getSubjectId() != null) {
                        Subject subject = subjectService.get(Integer.parseInt(sheetAccompaniment.getSubjectId()));
                        if (Objects.nonNull(subject)) {
                            //获取关联的乐器
                            Instrument one = instrumentService.lambdaQuery().eq(Instrument::getSubjectId, subject.getId()).one();
                            if (Objects.nonNull(one)) {
                                musicSheetSound.setMusicalInstrumentId(one.getId().toString());
                            }
                        }
                    }
                    musicSheetSound.setSortNumber(i++);
                    if (StringUtils.isNotBlank(musicSheetSound.getAudioFileUrl())) {
                        //xml
                        addMusicSheet.setXmlFileUrl(sheetAccompaniment.getXmlUrl());
                        //是否显示指法
                        addMusicSheet.setIsShowFingering(sheetAccompaniment.getIsShowFingering());
                        //是否使用系统节拍器
                        addMusicSheet.setIsUseSystemBeat(sheetAccompaniment.getIsOpenMetronome());
                        musicSheetSounds.add(musicSheetSound);
                    }
                    addMusicSheet.setPlaySpeed(sheetAccompaniment.getSpeed());
                }
                tracks = musicSheetSounds.stream().map(CbsMusicSheetWrapper.MusicSheetSound::getTrack).collect(Collectors.joining(","));
                addMusicSheet.setMusicSheetSoundList(musicSheetSounds);
            }
            CbsMusicSheetWrapper.MusicSheetAccompaniment musicSheetAccompaniment = new CbsMusicSheetWrapper.MusicSheetAccompaniment();
            if (addMusicSheet.getIsUseSystemBeat() != null && addMusicSheet.getIsUseSystemBeat()) {
                musicSheetAccompaniment.setAudioFileUrl(from.getUrl());
            }else {
                musicSheetAccompaniment.setAudioFileUrl(from.getMetronomeUrl());
            }
            if (StringUtils.isNotBlank(musicSheetAccompaniment.getAudioFileUrl())) {
                addMusicSheet.setMusicSheetAccompanimentList(Lists.newArrayList(musicSheetAccompaniment));
            }
        }
        addMusicSheet.setMultiTracksSelection(tracks);

        CbsMusicSheetWrapper.MusicSheetExtend musicSheetExtend = new CbsMusicSheetWrapper.MusicSheetExtend();
        musicSheetExtend.setApplicationId(applicationId);
        musicSheetExtend.setClientType(EClientType.BACKEND.name());
        addMusicSheet.setMusicSheetExtend(musicSheetExtend);

        // 应用扩展信息
        CbsMusicSheetApplicationExtendClientWrapper.AddMusicSheetApplicationExtend addMusicSheetApplicationExtend = new CbsMusicSheetApplicationExtendClientWrapper.AddMusicSheetApplicationExtend();
        addMusicSheetApplicationExtend.setApplicationId(applicationId);
        addMusicSheetApplicationExtend.setStatus(addMusicSheet.getStatus());
        addMusicSheetApplicationExtend.setSortNo(from.getOrder());
        addMusicSheetApplicationExtend.setPaymentType(StringUtils.isNotEmpty(from.getRankIds()) ? "VIP" : "FREE");
        addMusicSheetApplicationExtend.setAvailableType(EMusicAvailableType.ORG);
        addMusicSheetApplicationExtend.setMusicSheetCategoryId(from.getCbsMusicCategoriesId().longValue());
        addMusicSheet.setMusicCategoryId(from.getCbsMusicCategoriesId().longValue());
        addMusicSheet.setAddMusicSheetApplicationExtend(Lists.newArrayList(addMusicSheetApplicationExtend));
        addMusicSheet.setIsConvertibleScore(true);
        return addMusicSheet;
    }

    private void getAllCategoryIdList(List<Integer> categoryIdList, List<SysMusicScoreCategories> sysMusicScoreCategoriesList) {

        if (CollectionUtils.isNotEmpty(sysMusicScoreCategoriesList)) {
            for (SysMusicScoreCategories sysMusicScoreCategories : sysMusicScoreCategoriesList) {
                categoryIdList.add(sysMusicScoreCategories.getId());
                getAllCategoryIdList(categoryIdList, sysMusicScoreCategories.getSysMusicScoreCategoriesList());
            }
        }

    }

}