package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.gson.JsonObject;
import com.ym.mec.biz.dal.dao.ClassGroupDao;
import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dao.MusicGroupQuitDao;
import com.ym.mec.biz.dal.dao.SysNewsInformationDao;
import com.ym.mec.biz.dal.entity.ClassGroup;
import com.ym.mec.biz.dal.entity.ClassGroupDto;
import com.ym.mec.biz.dal.entity.SysNewsInformation;
import com.ym.mec.biz.dal.school.dto.NewsInformationDto;
import com.ym.mec.biz.service.SysNewsInformationService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.tenant.TenantContextHolder;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 新闻资讯表(SysNewsInformation)表服务实现类
 *
 * @author zx
 * @since 2023-05-11 16:08:28
 */
@Service("sysNewsInformationService")
public class SysNewsInformationServiceImpl extends ServiceImpl<SysNewsInformationDao, SysNewsInformation> implements SysNewsInformationService {

    private final static Logger log = LoggerFactory.getLogger(SysNewsInformationServiceImpl.class);

    @Override
    public SysNewsInformationDao getDao(){
            return this.baseMapper;
    }

    @Resource
    private MusicGroupQuitDao musicGroupQuitDao;
    @Resource
    private MusicGroupDao musicGroupDao;

    @Autowired
    private ClassGroupDao classGroupDao;

    @Override
    public List<NewsInformationDto> queryPage(Integer type,Integer coopId) throws Exception {
        List<SysNewsInformation> dtos = this.lambdaQuery()
                .eq(SysNewsInformation::getType, type)
                .eq(SysNewsInformation::getStatus, 1)
                .eq(SysNewsInformation::getDelFlag, false)
                .orderByDesc(SysNewsInformation::getOrder).list();
        List<NewsInformationDto> resultList = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(dtos)){
            List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
            for (SysNewsInformation dto : dtos) {
                NewsInformationDto informationDto = JSON.parseObject(JSON.toJSONString(dto), NewsInformationDto.class);
                if(CollectionUtils.isNotEmpty(musicGroupIds)){
                    if(informationDto.getRedPoint() != null){
                        //统计是否有红点
                        switch (informationDto.getRedPoint()) {
                            //待办事项
                            case BACKLOG:
                                informationDto.setRedPointFlag(musicGroupQuitDao.countRedPointBySchool(musicGroupIds));
                                break;
                            case SITE_MANAGE:
                                informationDto.setRedPointFlag(classGroupDao.teachingPointRemind(coopId)>0);
                                break;
                        }
                    }
                }
                resultList.add(informationDto);
            }
        }
        return resultList;
    }

}

