package com.ym.mec.biz.service.impl;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ym.mec.biz.dal.dao.HfMemberDao;
import com.ym.mec.biz.dal.dao.SysPaymentConfigDao;
import com.ym.mec.biz.dal.entity.HfMember;
import com.ym.mec.biz.dal.entity.SysPaymentConfig;
import com.ym.mec.biz.dal.enums.AccountType;
import com.ym.mec.biz.dal.enums.PaymentChannelEnum;
import com.ym.mec.biz.service.SysPaymentConfigService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;

@Service
public class SysPaymentConfigServiceImpl extends BaseServiceImpl<Integer, SysPaymentConfig> implements SysPaymentConfigService {
    @Autowired
    private SysPaymentConfigDao sysPaymentConfigDao;
    
    @Autowired
    private HfMemberDao hfMemberDao;

    @Override
    public BaseDAO<Integer, SysPaymentConfig> getDAO() {
        return sysPaymentConfigDao;
    }

    @Override
	public SysPaymentConfig findPaymentConfigByOrganId(Integer organId) {
		return sysPaymentConfigDao.findPaymentConfigByOrganId(organId);
	}

	@Override
    public SysPaymentConfig findPaymentConfigByOrganId(PaymentChannelEnum paymentChannel, Integer organId) {
    	SysPaymentConfig sysPaymentConfig =  sysPaymentConfigDao.findPaymentConfigByOrganId(organId);
    	
    	if(sysPaymentConfig != null) {
    		HfMember hfMember = hfMemberDao.getByOrganId(organId, paymentChannel.getCode());
    		if(hfMember == null) {
        		throw new BizException("[{}]分部[{}]没有配置商户号", paymentChannel.getCode(), organId);
        	}
    		sysPaymentConfig.setHfMerNo(hfMember.getMemberId());
    		sysPaymentConfig.setYqMerNo(hfMember.getMemberId());
    		sysPaymentConfig.setPayType(PaymentChannelEnum.codeOf(hfMember.getPayerName()));
    	}
    	
    	return sysPaymentConfig;
    }

    @Override
    public List<SysPaymentConfig> findPaymentConfigByOrganIds(PaymentChannelEnum paymentChannel, List<Integer> organIds) {
    	List<SysPaymentConfig> sysPaymentConfigs = sysPaymentConfigDao.findPaymentConfigByOrganIds(organIds);
    	
    	for(SysPaymentConfig sysPaymentConfig : sysPaymentConfigs) {
    		HfMember hfMember = hfMemberDao.getByOrganId(sysPaymentConfig.getOrganId(), paymentChannel.getCode());
    		if(hfMember == null) {
        		throw new BizException("[{}]分部[{}]没有配置商户号", paymentChannel.getCode(), sysPaymentConfig.getOrganId());
        	}
    		sysPaymentConfig.setHfMerNo(hfMember.getMemberId());
    		sysPaymentConfig.setYqMerNo(hfMember.getMemberId());
    		sysPaymentConfig.setPayType(PaymentChannelEnum.codeOf(hfMember.getPayerName()));
    	}
    	
        return sysPaymentConfigs;
    }

    @Override
    public List<SysPaymentConfig> getPaymentConfigs(PaymentChannelEnum payType, Integer tenantId) {
        return sysPaymentConfigDao.getPaymentConfigs(payType,tenantId);
    }

    @Override
    public AccountType checkAccountType(PaymentChannelEnum payType, String merNos, Integer tenantId) {
        if (payType == null || (!payType.equals(PaymentChannelEnum.ADAPAY) && !payType.equals(PaymentChannelEnum.YQPAY)) || StringUtils.isBlank(merNos)) {
            return AccountType.INTERNAL;
        }
        List<SysPaymentConfig> accounts = sysPaymentConfigDao.getOutAccounts(payType,tenantId);
        String[] merNoArr = merNos.split(",");
        AccountType accountType = AccountType.INTERNAL;
        NODE1:
        for (String merNo : merNoArr) {
            for (SysPaymentConfig account : accounts) {
                String accountNo = "";
                if (payType.equals(PaymentChannelEnum.ADAPAY)) {
                    accountNo = account.getHfMerNo();
                } else {
                    accountNo = account.getYqMerNo();
                }
                if (accountNo.equals(merNo)) {
                    accountType = AccountType.EXTERNAL;
                    break NODE1;
                }
            }
        }
        return accountType;
    }

    @Override
    public SysPaymentConfig findPaymentConfigByMerNo(String merNo) {
        return sysPaymentConfigDao.findPaymentConfigByMerNo(merNo);
    }
}
