package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.dayaedu.cbs.common.enums.EAppKey;
import com.dayaedu.cbs.common.enums.EClientType;
import com.dayaedu.cbs.openfeign.client.MessageFeignClientService;
import com.dayaedu.cbs.openfeign.wrapper.message.CbsMessageWrapper;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.common.response.template.R;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.entity.SysConfig;
import com.ym.mec.biz.dal.entity.Teacher;
import com.ym.mec.biz.dal.enums.ClientEnum;
import com.ym.mec.biz.dal.wrapper.UserInfoWrapper;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.biz.service.im.ImGroupCoreService;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.Nullable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.*;

@Slf4j
@Service
public class SysUserServiceImpl implements SysUserService {

    @Resource
    private SysUserFeignService sysUserFeignService;
    @Resource
    private TeacherDao teacherDao;
    @Resource
    private SysConfigService sysConfigService;
    @Resource
    private ImGroupCoreService imGroupCoreService;
    @Resource
    private MessageFeignClientService messageFeignClientService;

    @Override
    public Integer getUserId(){
        return Optional.ofNullable(sysUserFeignService.queryUserInfo())
                .map(SysUser::getId)
                .orElseThrow(() -> new BizException("请登录"));
    }

    @Override
    public SysUser getUser(){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Optional.ofNullable(sysUser)
                .map(SysUser::getId)
                .orElseThrow(() -> new BizException("请登录"));
        return sysUser;
    }

    @Override
    public SysUser queryUserByPhone(String phone) {
        SysUser user = sysUserFeignService.queryUserByMobile(phone);
        Optional.ofNullable(user)
                .map(SysUser::getId)
                .orElseThrow(() -> new BizException("用户信息获取失败"));
        return user;
    }
    @Override
    public SysUser queryUserById(Integer userId) {
        SysUser user = sysUserFeignService.queryUserById(userId);
        Optional.ofNullable(user)
                .map(SysUser::getId)
                .orElseThrow(() -> new BizException("用户信息获取失败"));
        return user;
    }

    @Override
    public Boolean updateSysUser(SysUser user) {
        return sysUserFeignService.updateSysUser(user);
    }

    @Override
    public String getImName(SysUser sysUser) {
        if (sysUser == null) {
            return "";
        }
        String username = sysUser.getUsername();
        List<String> userTypes = Lists.newArrayList(sysUser.getUserType().split(","));
        if (userTypes.size() > 1
            || userTypes.contains(ClientEnum.TEACHER.getCode())
            || userTypes.contains(ClientEnum.SYSTEM.getCode())) {
            username = Optional.ofNullable(sysUser.getRealName()).filter(StringUtils::isNotBlank).orElse(username);
        }
        return username;
    }

    @Override
    public String getImAvatar(SysUser sysUser) {
        if (sysUser == null) {
            return "";
        }
        String avatar = sysUser.getAvatar();
        if (StringUtils.isNotBlank(avatar)) {
            return avatar;
        }
        List<String> userTypes = Lists.newArrayList(sysUser.getUserType().split(","));
        if (userTypes.size() == 1 && userTypes.contains(ClientEnum.STUDENT.getCode())) {
            return Optional.ofNullable(sysConfigService.findByParamName(SysConfigService.STUDENT_DEFAULT_AVATAR)).orElse(new SysConfig()).getParanValue();
        } else {
            return Optional.ofNullable(sysConfigService.findByParamName(SysConfigService.TEACHER_DEFAULT_AVATAR)).orElse(new SysConfig()).getParanValue();
        }
    }

    /**
     * 注册用户三方IM
     *
     * @param userId 用户id
     */
    @Override
    public void registerImUserInfo(Integer userId) {
        SysUser userInfo = teacherDao.getUser(userId);
        if (Objects.nonNull(userInfo)) {

            try {
                String username = userInfo.getUsername();
                List<String> userTypes = Lists.newArrayList(userInfo.getUserType().split(","));
                if (userTypes.size() > 1
                    || userTypes.contains(ClientEnum.TEACHER.getCode())
                    || userTypes.contains(ClientEnum.SYSTEM.getCode())) {
                    username = Optional.ofNullable(userInfo.getRealName()).filter(StringUtils::isNotBlank).orElse(username);
                }

                imGroupCoreService.register(String.valueOf(userInfo.getId()), "", username, getImAvatar(userInfo));
            } catch (Exception e) {
                log.error("用户更新信息同步到三方失败，userId：{}", userInfo.getId(), e);
            }
        }
    }

    @Override
    public Map<Long, SysUser> getMapByIds(Collection<? extends Serializable> studentIds) {
        if (CollectionUtils.isEmpty(studentIds)) {
            return new HashMap<>();
        }

        List<SysUser> userList = teacherDao.getUserList(studentIds);
        return userList.stream().collect(HashMap::new, (m, v) -> m.put(v.getId().longValue(), v), HashMap::putAll);
    }



    @Override
    public void imDeviceId(UserInfoWrapper.UpdateUser info) {
        CbsMessageWrapper.UpdateDevice updateDevice = new CbsMessageWrapper.UpdateDevice();
        updateDevice.setUserId(info.getUserId().longValue());
        updateDevice.setDeviceIds(Lists.newArrayList(info.getImDeviceId()));
        updateDevice.setClientType(getClientType(info.getClient().getCode()));
        updateDevice.setAppKey(EAppKey.GYM);
        R<Boolean> booleanR = messageFeignClientService.updateDevice(updateDevice);
        if (booleanR != null && booleanR.feignData()) {
            teacherDao.updateImDeviceId(info.getImDeviceId(), info.getClient().getCode(), info.getUserId());
        }
        log.error("更新用户设备信息失败，userId：{}", info.getUserId());
    }

    public EClientType getClientType(String jpushType){
        if(StringUtils.isEmpty(jpushType)){
            return null;
        }else if(StringUtils.equals(jpushType, "STUDENT")){
            return EClientType.STUDENT;
        }else if(StringUtils.equals(jpushType, "TEACHER")){
            return EClientType.TEACHER;
        }else if(StringUtils.equals(jpushType, "SCHOOL")){
            return EClientType.SCHOOL;
        }else {
            return EClientType.BACKEND;
        }
    }

    @Nullable
    private String getTeacherDeviceId(UserInfoWrapper.UpdateUser info, String teacherDeviceId) {
        Teacher teacher1 = teacherDao.get(info.getUserId());
        if (teacher1 != null) {
            if (org.springframework.util.StringUtils.isEmpty(teacher1.getImDeviceId())) {
                return null;
            }
            teacherDeviceId = teacher1.getImDeviceId();
        }
        return teacherDeviceId;
    }
}