package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.TeacherAttendanceComplaintsQueryInfo;
import com.ym.mec.biz.dal.page.TeacherCloseQueryInfo;
import com.ym.mec.biz.dal.school.dto.TeacherAttendanceListDto;
import com.ym.mec.biz.dal.school.dto.TeacherAttendanceSumDto;
import com.ym.mec.biz.dal.school.page.TeacherAttendanceQueryInfo;
import com.ym.mec.biz.dal.wrapper.SchoolStaffWrapper;
import com.ym.mec.biz.dal.wrapper.TeacherAttendanceWrapper;
import com.ym.mec.biz.event.source.SendMessageSource;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.redis.service.RedisCache;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.string.MessageFormatter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import static com.ym.mec.biz.dal.enums.ComplaintsStatusEnum.REPEAL;
import static com.ym.mec.biz.dal.enums.SignInStatusEnum.*;

@Service
public class TeacherAttendanceServiceImpl extends BaseServiceImpl<Long, TeacherAttendance>  implements TeacherAttendanceService {

	@Autowired
	private TeacherAttendanceDao teacherAttendanceDao;
	@Autowired
	private StudentAttendanceDao studentAttendanceDao;
	@Autowired
	private CourseScheduleDao courseScheduleDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private CourseHomeworkService courseHomeworkService;
	@Autowired
	private StudentCourseHomeworkDao studentCourseHomeworkDao;
	@Autowired
	private SysUserFeignService sysUserFeignService;
	@Autowired
	private CourseScheduleService courseScheduleService;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private SendMessageSource sendMessageSource;
	@Autowired
	private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
	@Autowired
	private ClassGroupDao classGroupDao;
	@Autowired
	private SubjectDao subjectDao;
	@Autowired
	private SchoolDao schoolDao;
	@Autowired
	private RedisCache<String,Object> redisCache;
	@Autowired
	private CourseHomeworkDao courseHomeworkDao;
	@Autowired
	private StudentServeService studentServeService;
	@Autowired
	private SysTenantConfigService sysTenantConfigService;
	@Autowired
	private StudentService studentService;
	@Autowired
	private LessonExaminationService lessonExaminationService;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private StudentLessonTrainingDetailService studentLessonTrainingDetailService;

    @Autowired
    private SchoolStaffService schoolStaffService;

    @Autowired
    private SysMessageConfigDao sysMessageConfigDao;

	@Override
	public BaseDAO<Long, TeacherAttendance> getDAO() {
		return teacherAttendanceDao;
	}

    @Override
    public TeacherAttendanceDao getDao() {
        return teacherAttendanceDao;
    }

    @Transactional(rollbackFor = Exception.class)
	@Override
	public void saveStudentStandard(List<StudentStandardDto> studentStandardDtos,Long courseScheduleId){
		if(studentStandardDtos == null || studentStandardDtos.size() == 0){
			return;
		}
		List<StudentStandardDto> noQualified = studentStandardDtos.stream().filter(e -> e.getQualifiedFlag() == 0).collect(Collectors.toList());
		CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleId);
		CourseSchedule.CourseScheduleType type = courseSchedule.getType();
		String courseDate = DateUtil.format(courseSchedule.getClassDate(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
		if(noQualified.size() > 0){
			List<Integer> studentIds = noQualified.stream().map(e -> e.getUserId()).collect(Collectors.toList());
			studentAttendanceDao.updateQualified(courseScheduleId,studentIds,0);
			this.sendMessage(studentIds,type,courseDate,MessageTypeEnum.SMS_STANDARD_NO_QUALIFIED);
		}
		List<StudentStandardDto> qualified = studentStandardDtos.stream().filter(e -> e.getQualifiedFlag() == 1).collect(Collectors.toList());
		if(qualified.size() > 0){
			List<Integer> studentIds = qualified.stream().map(e -> e.getUserId()).collect(Collectors.toList());
			studentAttendanceDao.updateQualified(courseScheduleId,studentIds,1);
			this.sendMessage(studentIds,type,courseDate,MessageTypeEnum.SMS_STANDARD_QUALIFIED);
		}
		//记录学员是否需要维修
		List<Integer> repairUser = studentStandardDtos.stream().filter(e -> e.getRepairFlag()).map(e -> e.getUserId()).collect(Collectors.toList());
		List<Integer> notRepairUser = studentStandardDtos.stream().filter(e -> !e.getRepairFlag()).map(e -> e.getUserId()).collect(Collectors.toList());
		studentService.updateRepairFlag(repairUser,true);
		studentService.updateRepairFlag(notRepairUser,false);
		//标记当前课程为已评价
		courseScheduleDao.updateEvaluate(courseScheduleId);
	}

	public void sendMessage(List<Integer> studentIds,CourseSchedule.CourseScheduleType type,String courseDate,MessageTypeEnum messageTypeEnum){
		Map<Integer, String> receivers = studentIds.stream().collect(Collectors.toMap(e -> e, e -> e.toString(), (k1, k2) -> k1));
		sendMessageSource.batchSendMessage(MessageSender.JIGUANG,messageTypeEnum,
				receivers,null,"STUDENT",courseDate,type.getMsg());
	}

	@Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
	@Override
	public Map<String, Object> addTeacherAttendanceRecord(TeacherSignOutDto teacherSignOutDto) {
		SysUser user = sysUserFeignService.queryUserInfo();
		teacherDao.getLocked(user.getId());

        if(Objects.isNull(teacherSignOutDto.getTeacherAttendanceInfo().getCourseScheduleId())){
			throw new BizException("请选择课程!");
		}
		Integer tenantId = TenantContextHolder.getTenantId();

		Map<String,Object> result=new HashMap<>();

		TeacherAttendance teacherAttendance=teacherAttendanceDao.findByTeacherAttendanceInfo(user.getId().longValue(),teacherSignOutDto.getTeacherAttendanceInfo().getCourseScheduleId());

		Date date = new Date();
		if(Objects.isNull(teacherAttendance)){
			teacherAttendance=teacherSignOutDto.getTeacherAttendanceInfo();
			teacherAttendance.setTeacherId(user.getId());
			teacherAttendance.setCreateTime(date);
		}else if(teacherAttendance.getSignInTime() != null && teacherAttendance.getSignOutTime() != null){
			throw new BizException("当前课程已签到");
		}

		TeacherAttendanceDto currentCourseDetail = courseScheduleDao.getCurrentCourseDetail(teacherSignOutDto.getTeacherAttendanceInfo().getCourseScheduleId(),user.getId());
		if(Objects.isNull(currentCourseDetail)){
			throw new BizException("课程不存在");
		}

		teacherAttendance.setMusicGroupId(currentCourseDetail.getMusicGroupId());
		teacherAttendance.setClassGroupId(currentCourseDetail.getClassId());
		teacherAttendance.setRemark(teacherSignOutDto.getTeacherAttendanceInfo().getRemark());
		teacherAttendance.setSignOutRemark(teacherSignOutDto.getTeacherAttendanceInfo().getSignOutRemark());
		int advanceSignMinutes = 20;
		String tenantConfigValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.OFFLINE_ADVANCE_SIGN_IN_MINUTES, user.getTenantId());
		if(StringUtils.isNotEmpty(tenantConfigValue)){
			advanceSignMinutes = Integer.parseInt(tenantConfigValue);
		}

//		int advanceSignOutMinutes = Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.ADVANCE_SIGN_OUT_MINUTES));
		String offlineSignInEarlyForwardTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.OFFLINE_SIGN_IN_EARLY_FORWARD_TIME, user.getTenantId());
		if(StringUtils.isEmpty(offlineSignInEarlyForwardTime)){
			offlineSignInEarlyForwardTime = "60";
		}

		CourseSchedule courseSchedule = courseScheduleDao.get(teacherAttendance.getCourseScheduleId());
		teacherAttendance.setGroupType(courseSchedule.getGroupType());
		if(courseSchedule == null){
			throw new BizException("课程信息不存在");
		}

		Date tomorrow = DateUtil.addDays1(date, 1);

		if(tomorrow.before(courseSchedule.getStartClassTime())){
			throw new BizException("明天的课程不能进行签到");
		}
		//处理学员学习标准是否达标
		this.saveStudentStandard(teacherSignOutDto.getStudentStandardDtos(),teacherAttendance.getCourseScheduleId());

		School school = schoolDao.get(courseSchedule.getSchoolId());

		String classDate = DateUtil.format(courseSchedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
		String startClassTime = DateUtil.format(courseSchedule.getStartClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
		String endClassTime = DateUtil.format(courseSchedule.getEndClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
		//上课时间
		Date classStartDateTime = DateUtil.stringToDate(classDate + " " + startClassTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
		Date classEndDateTime = DateUtil.stringToDate(classDate + " " + endClassTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
		String configValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.ADVANCE_SIGN_OUT_MINUTES, user.getTenantId());
		if(StringUtils.isEmpty(configValue)){
			configValue = "360";
		}
		Date classEndDateAdd60Minutes = DateUtil.addMinutes(classEndDateTime, Integer.parseInt(configValue));
		Date add60Minutes = DateUtil.addMinutes(classStartDateTime, Integer.parseInt(offlineSignInEarlyForwardTime) * -1);
		Date add20Minutes = DateUtil.addMinutes(classStartDateTime, advanceSignMinutes * -1);
		if(teacherAttendance.getSignInTime() == null && teacherSignOutDto.getTeacherAttendanceInfo().getStatus().equals(SignStatusEnum.SIGN_IN.getCode())){

			if(StringUtils.isBlank(teacherSignOutDto.getTeacherAttendanceInfo().getSignInLongitudeLatitude())){
				throw new BizException("未获取到您的位置");
			}

			//是否在范围内
			boolean isInScore = true;
			if(StringUtils.isBlank(school.getLongitudeLatitude())){
				if(teacherSignOutDto.getTeacherAttendanceInfo().getUpdate().equals(YesOrNoEnum.YES.getCode())){
					school.setLongitudeLatitude(teacherSignOutDto.getTeacherAttendanceInfo().getSignInLongitudeLatitude());
					schoolDao.update(school);
				}else{
					isInScore = false;
				}
			}else{
				//签到GPS范围
				String gpsRange = sysTenantConfigService.getTenantConfigValue(SysConfigService.ATTENDANCE_RANGE,tenantId);
				if(StringUtils.isEmpty(gpsRange)){
					isInScore = true;
				}else {
					double attendanceRange = Double.parseDouble(gpsRange);
					double distance = MapUtil.distance(teacherSignOutDto.getTeacherAttendanceInfo().getSignInLongitudeLatitude(),
							school.getLongitudeLatitude());
					if(distance>attendanceRange){
						isInScore=false;
					}
				}
			}

			teacherAttendance.setCurrentClassTimes(courseScheduleDao.countCurrentTimes(teacherAttendance.getClassGroupId(),courseSchedule.getStartClassTime()) + 1);
			teacherAttendance.setSignInLongitudeLatitude(teacherSignOutDto.getTeacherAttendanceInfo().getSignInLongitudeLatitude());
			if (currentCourseDetail.getEndClassTime().before(date)){
				throw new BizException("该课程已结束，不能进行签到!");
			}

			boolean enableIgnoreAttendanceTime = false;
			if(Objects.nonNull(courseSchedule.getSchoolId())){
				enableIgnoreAttendanceTime = courseScheduleService.enableIgnoreAttendanceTime(currentCourseDetail.getStartClassTime(),
						user.getId(),
						currentCourseDetail.getSchoolId().intValue());
			}
			teacherAttendance.setSignInTime(date);
			teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
			result.put("signInStatus",0);
			if(!isInScore){
				teacherAttendance.setSignInStatus(YesOrNoEnum.NO);
			}else if(enableIgnoreAttendanceTime){
				//连堂课，可忽略正常时间签到范围
			}else if(date.compareTo(add20Minutes) <= 0&&date.compareTo(add60Minutes)>=0){
				//正常签到范围(开始前60分钟  ~  开始之前20分钟)
				if(!isInScore){
					teacherAttendance.setSignInStatus(YesOrNoEnum.NO);
				}
			}else if(date.compareTo(add20Minutes) > 0){
				//异常签到范围(开始之后  ~  结束之前)
				teacherAttendance.setSignInStatus(YesOrNoEnum.NO);
				result.put("signInStatus",1);
			}else {
				throw new BizException("签到时间异常");
			}
			if(YesOrNoEnum.NO.equals(teacherAttendance.getSignInStatus())&&StringUtils.isEmpty(teacherAttendance.getRemark())&&!teacherSignOutDto.isNotRturnErrorInfo()){
				//当前为异常签到，请填写原因!
				result.put("status",2);
				return result;
			}
            teacherAttendance.setSignInAttachments(teacherSignOutDto.getTeacherAttendanceInfo().getSignInAttachments());
			teacherAttendance.setSignInDeviceNo(teacherSignOutDto.getTeacherAttendanceInfo().getDeviceNum());
			if(Objects.nonNull(teacherAttendance.getId())){
				teacherAttendanceDao.update(teacherAttendance);
			}else{
				teacherAttendanceDao.insert(teacherAttendance);
			}
            // 推送异常考勤消息
            if(YesOrNoEnum.NO.equals(teacherAttendance.getSignInStatus())){

                // 乐团信息
                MusicGroup musicGroup = musicGroupDao.get(courseSchedule.getMusicGroupId());

                IPage<SchoolStaffWrapper.SchoolStaff> page = schoolStaffService.selectPage(new Page<>(1, -1),
                        SchoolStaffWrapper.SchoolStaffQuery.builder().schoolId(musicGroup.getCooperationOrganId().longValue()).build());
                List<SchoolStaffWrapper.SchoolStaff> records = page.getRecords();
                if (CollectionUtils.isNotEmpty(records)) {

                    Map<Integer, String> userMap = new HashMap<>();
                    records.forEach(o -> userMap.put(o.getUserId().intValue(), o.getUserId().toString()));

                    SysMessageConfig sysMessageConfig = sysMessageConfigDao.queryByType(MessageTypeEnum.JIGUANG_TEACHER_SIGN_IN_EXCEPTION.getCode());
                    String url = MessageFormatter.arrayFormat(sysMessageConfig.getUrl(),courseSchedule.getActualTeacherId(), courseSchedule.getClassGroupId());

                    // 课程时间
                    String courseTime = classDate +" " + startClassTime + "-" + endClassTime;
                    sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.JIGUANG_TEACHER_SIGN_IN_EXCEPTION,
                            userMap, null, 0, url,
                            "SCHOOL",musicGroup.getName(),user.getRealName(),courseTime,courseSchedule.getName());
                }
            }



			return result;
		}else if(teacherAttendance.getSignOutTime() == null && teacherSignOutDto.getTeacherAttendanceInfo().getStatus().equals(SignStatusEnum.SIGN_OUT.getCode())){
			int i = studentAttendanceDao.countByCourseSchedule(courseSchedule.getId());
			if(i<=0){
				throw new BizException("请先点名");
			}

			//是否在范围内
			boolean isInScore = false;
			if(StringUtils.isBlank(school.getLongitudeLatitude())){
				if(teacherSignOutDto.getTeacherAttendanceInfo().getUpdate().equals(YesOrNoEnum.YES.getCode())){
					school.setLongitudeLatitude(teacherSignOutDto.getTeacherAttendanceInfo().getSignInLongitudeLatitude());
					schoolDao.update(school);
				}
			}else if(StringUtils.isNotBlank(teacherSignOutDto.getTeacherAttendanceInfo().getSignOutLongitudeLatitude())){
			    teacherAttendance.setSignOutLongitudeLatitude(teacherSignOutDto.getTeacherAttendanceInfo().getSignOutLongitudeLatitude());
				String gpsRange = sysTenantConfigService.getTenantConfigValue(SysConfigService.ATTENDANCE_RANGE,tenantId);
				if(StringUtils.isNotEmpty(gpsRange)){
					double attendanceRange = Double.parseDouble(gpsRange);
					double distance = MapUtil.distance(teacherSignOutDto.getTeacherAttendanceInfo().getSignOutLongitudeLatitude(),
							school.getLongitudeLatitude());
					if(distance <= attendanceRange){
						isInScore = true;
					}
				}else {
					isInScore = true;
				}
			}

			if(Objects.isNull(teacherAttendance.getSignInTime())){
				throw new BizException("未签到不可点名和签退");
			}

			//正常签退范围（结束后1小时内）
			teacherAttendance.setSignOutTime(date);
			CourseHomework courseHomeworkInfo = teacherSignOutDto.getCourseHomeworkInfo();
			teacherAttendance.setSignOutAttachments(courseHomeworkInfo.getAttachments());
			ClassGroup classGroup = classGroupDao.get(courseSchedule.getClassGroupId());
			teacherAttendance.setCurrentClassTimes(classGroup.getCurrentClassTimes() + 1);
			if(!isInScore){
				teacherAttendance.setSignOutStatus(YesOrNoEnum.NO);
				result.put("signOutStatus",1);
			}else if(date.compareTo(classEndDateTime) >= 0 && date.compareTo(classEndDateAdd60Minutes) <= 0 && DateUtil.isSameDay(date, classEndDateTime)){
				teacherAttendance.setSignOutStatus(YesOrNoEnum.YES);
				result.put("signOutStatus",0);
			}else if(date.compareTo(classEndDateAdd60Minutes) > 0 && DateUtil.isSameDay(date, classEndDateTime)){
				teacherAttendance.setSignOutStatus(YesOrNoEnum.NO);
				result.put("signOutStatus",1);
			}else {
				throw new BizException("签退时间异常");
			}
			if(YesOrNoEnum.NO.equals(teacherAttendance.getSignOutStatus())&&StringUtils.isEmpty(teacherAttendance.getSignOutRemark())&&!teacherSignOutDto.isNotRturnErrorInfo()){
				//当前为异常签退，请填写原因!
				result.put("status",2);
				return result;
			}
			teacherAttendance.setSignOutDeviceNo(teacherSignOutDto.getTeacherAttendanceInfo().getDeviceNum());
			if(Objects.nonNull(teacherAttendance.getId())){
				teacherAttendanceDao.update(teacherAttendance);
			}else{
				teacherAttendanceDao.insert(teacherAttendance);
			}

            if(CollectionUtils.isNotEmpty(courseHomeworkInfo.getMusicScoreSubjectDto())){
				CourseHomework courseHomework = courseHomeworkDao.findByCourseSchedule(teacherAttendance.getCourseScheduleId());
				if(Objects.isNull(courseHomework)){
					//新增课堂作业
					courseHomework = courseHomeworkInfo;
					courseHomework.setCourseScheduleId(teacherAttendance.getCourseScheduleId());
					courseHomework.setMusicGroupId(currentCourseDetail.getMusicGroupId());
					courseHomework.setGroupType(teacherAttendance.getGroupType());
					courseHomework.setClassGroupId(currentCourseDetail.getClassId());

                    LocalDateTime localDateTime=LocalDateTime.now();
                    localDateTime=localDateTime.plusDays(Integer.parseInt(sysConfigService.findByParamName("homework_expire_time").getParanValue()))
                            .withHour(21).withMinute(0).withSecond(0).withNano(0);
					courseHomework.setExpiryDate(Date.from(localDateTime.atZone(DateUtil.zoneId).toInstant()));

					courseHomeworkService.insert(courseHomework);
					List<StudentCourseHomework> studentCourseHomeworks = new ArrayList<>();

                    String versionTag = "v1";
                    // 作业详情
                    List<StudentLessonTrainingDetail> studentLessonTrainingDetails = new ArrayList<>();

					//获取按学员声部发放的作业
					List<MusicScoreSubjectDto> musicScoreSubjectDto = courseHomework.getMusicScoreSubjectDto();
					if(musicScoreSubjectDto != null && musicScoreSubjectDto.size() > 0){
						for (MusicScoreSubjectDto scoreSubjectDto : musicScoreSubjectDto) {
                            List<Integer> userIdList = scoreSubjectDto.getUserIdList();
                            studentCourseHomeworks.addAll(studentCourseHomeworkDao.constructInitialStudentHomeworkRecordsWithPayment(teacherAttendance.getCourseScheduleId(),
                                             courseHomework.getId(),
                                             StringUtils.join(scoreSubjectDto.getMusicScoreIdList(),","),
                                             JSONObject.toJSONString(scoreSubjectDto.getMusicScoreHomeworkDtoList()),
                                             userIdList));
                            if (CollectionUtils.isEmpty(scoreSubjectDto.getStudentLessonTrainingDetails())) {
                                throw new BizException("有部分声部未选择曲目");
                            }

                            for (StudentLessonTrainingDetailWrapper.AddStudentLessonTrainingDetail studentLessonTrainingDetail : scoreSubjectDto.getStudentLessonTrainingDetails()) {
                                studentLessonTrainingDetail.setSubjectId(0);
                            }
                            studentLessonTrainingDetails.addAll(studentLessonTrainingDetailService
                                                                    .homeWorkDetail(scoreSubjectDto.getStudentLessonTrainingDetails(), userIdList,
                                                                            courseHomework.getId(),ELessonTrainingType.HOMEWORK, scoreSubjectDto.getGroup()));

						}
					}else {
						studentCourseHomeworks.addAll(studentCourseHomeworkDao.constructInitialStudentHomeworkRecordsWithPayment(teacherAttendance.getCourseScheduleId(),
								courseHomework.getId(),
								courseHomework.getMusicScoreId(),
								"",
								null));

                        List<Integer> userIdList = studentCourseHomeworks.stream()
                                                                      .map(StudentCourseHomework::getUserId)
                                                                      .collect(Collectors.toList());
                        studentLessonTrainingDetails.addAll(studentLessonTrainingDetailService
                                                                .homeWorkDetail(courseHomeworkInfo.getStudentLessonTrainingDetails(), userIdList,courseHomework.getId(),ELessonTrainingType.HOMEWORK,null));

					}
					if(CollectionUtils.isEmpty(studentCourseHomeworks)){
						throw new BizException("此课程没有学生");
					}
                    for (StudentCourseHomework studentCourseHomework : studentCourseHomeworks) {
                        studentCourseHomework.setGroupType(teacherAttendance.getGroupType());
                    }

                    if (!CollectionUtils.isEmpty(studentLessonTrainingDetails)) {
                        versionTag ="v2";
                    }
					courseHomework.setExpectNum(studentCourseHomeworks.size());
                    courseHomework.setVersionTag(versionTag);

					Teacher teacher = teacherDao.get(courseSchedule.getActualTeacherId());
					String dateStr = DateUtil.dateToString(DateUtil.addDays(date, 3), "MM月dd日");

					if(!CollectionUtils.isEmpty(studentCourseHomeworks)){
						courseHomeworkService.update(courseHomework);

						studentCourseHomeworkDao.batchInsertStudentCourseHomeworkRecord(studentCourseHomeworks);
						List<Integer> studentIds = studentCourseHomeworks.stream().map(StudentCourseHomework::getUserId).collect(Collectors.toList());
						studentServeService.updateExercisesSituation(courseSchedule.getMusicGroupId(), courseSchedule.getClassDate(), studentIds, teacherAttendance.getTeacherId());

						for (StudentCourseHomework studentCourseHomework : studentCourseHomeworks) {
                            if (versionTag.equals("v2")) {
                                Map<Integer, String> userMap = new HashMap<>();
                                userMap.put(studentCourseHomework.getUserId(), studentCourseHomework.getUserId().toString());
                                String notifyUrl = "homeworkId=" + courseHomework.getCourseScheduleId() + "&homeworkType=" + ELessonTrainingType.HOMEWORK.getCode();
                                String extra = "dayaedu?type=homework&" + notifyUrl ;

                                sysMessageService.batchSendImMessage(MessageTypeEnum.IM_HOMEWORK_REMIND_V2, courseSchedule.getActualTeacherId().toString(), extra,
                                        new String[]{studentCourseHomework.getUserId().toString()},
                                        null, courseSchedule.getName(), dateStr);

                                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_HOMEWORK_REMIND_V2,
                                        userMap, null, 0, "homework?" + notifyUrl, "STUDENT", teacher.getRealName());

                            } else {
                                Map<Integer, String> userMap = new HashMap<>();
                                userMap.put(studentCourseHomework.getUserId(), studentCourseHomework.getUserId().toString());
                                String notifyUrl = "?courseScheduleID=" + courseSchedule.getId() + "&studentCourseHomeworkId=" + studentCourseHomework.getId() + "&extra=0";
                                String extra = "dayaedu" + notifyUrl + "&userId=" + studentCourseHomework.getUserId();

                                sysMessageService.batchSendImMessage(MessageTypeEnum.IM_HOMEWORK_REMIND, courseSchedule.getActualTeacherId().toString(), extra,
                                        new String[]{studentCourseHomework.getUserId().toString()},
                                        null, courseSchedule.getName(), dateStr, courseHomework.getContent());

                                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_HOMEWORK_REMIND,
                                        userMap, null, 0, 3 + notifyUrl, "STUDENT", teacher.getRealName(), courseSchedule.getName(), dateStr);
                            }
						}
					}
                    if (CollectionUtils.isNotEmpty(studentLessonTrainingDetails)) {
                        // 学生ID集合
                        Set<Integer> userIdList = studentLessonTrainingDetails.stream()
                                                                              .map(StudentLessonTrainingDetail::getUserId)
                                                                                .filter(Objects::nonNull)
                                                                                .map(Long::intValue)
                                                                              .collect(Collectors.toSet());
                        // 设置声部
                        Map<Integer, StudentSubjectDto> subjectDtoMap = studentLessonTrainingDetailService
                                .getStudentMusicGroupSubject(new ArrayList<>(userIdList), classGroup.getMusicGroupId());

                        studentLessonTrainingDetails.forEach(studentLessonTrainingDetail -> {
                            if (studentLessonTrainingDetail.getUserId() == null) {
                                return;
                            }
                            StudentSubjectDto studentSubjectDto = subjectDtoMap.get(studentLessonTrainingDetail.getUserId().intValue());
                            if (Objects.nonNull(studentSubjectDto)) {
                                studentLessonTrainingDetail.setSubjectId(studentSubjectDto.getSubjectId().longValue());
                            }
                        });
                        studentLessonTrainingDetailService.saveBatch(studentLessonTrainingDetails);
                    }
				}else{
					courseHomework.setContent(teacherSignOutDto.getCourseHomeworkInfo().getContent());
					courseHomework.setAttachments(teacherSignOutDto.getCourseHomeworkInfo().getAttachments());
					courseHomeworkService.update(courseHomework);
				}
			}

			//布置课后评测
			LessonExaminationSaveDto examinationSaveDto = teacherSignOutDto.getCourseHomeworkInfo().getLessonExaminationSaveDto();
			if(Objects.nonNull(examinationSaveDto)){
                examinationSaveDto.setTeacherId(teacherAttendance.getTeacherId());
                examinationSaveDto.setClassGroupId(teacherAttendance.getClassGroupId());
                examinationSaveDto.setCourseScheduleId(teacherAttendance.getCourseScheduleId());
                examinationSaveDto.setMusicGroupId(teacherAttendance.getMusicGroupId());
				lessonExaminationService.publish(examinationSaveDto);
			}


            // 推送异常考勤消息
            if(YesOrNoEnum.NO.equals(teacherAttendance.getSignOutStatus())){

                // 乐团信息
                MusicGroup musicGroup = musicGroupDao.get(courseSchedule.getMusicGroupId());

                IPage<SchoolStaffWrapper.SchoolStaff> page = schoolStaffService.selectPage(new Page<>(1, -1),
                        SchoolStaffWrapper.SchoolStaffQuery.builder().schoolId(musicGroup.getCooperationOrganId().longValue()).build());
                List<SchoolStaffWrapper.SchoolStaff> records = page.getRecords();
                if (CollectionUtils.isNotEmpty(records)) {

                    Map<Integer, String> userMap = new HashMap<>();
                    records.forEach(o -> userMap.put(o.getUserId().intValue(), o.getUserId().toString()));

                    SysMessageConfig sysMessageConfig = sysMessageConfigDao.queryByType(MessageTypeEnum.JIGUANG_TEACHER_SIGN_OUT_EXCEPTION.getCode());
                    String url = MessageFormatter.arrayFormat(sysMessageConfig.getUrl(),courseSchedule.getActualTeacherId(), courseSchedule.getClassGroupId());

                    // 课程时间
                    String courseTime = classDate +" " + startClassTime + "-" + endClassTime;
                    sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.JIGUANG_TEACHER_SIGN_OUT_EXCEPTION,
                            userMap, null, 0,url,
                            "SCHOOL",musicGroup.getName(),user.getRealName(),courseTime,courseSchedule.getName());
                }
            }
			return result;
		}

		result.put("status",0);
		result.put("currentTime",date);

		return result;
	}


	@Override
	@Transactional(rollbackFor = Exception.class)
	public void addTeacherAttendanceSignOut(Long firstCourseId, Integer userId,String deviceNum) {
		Date date = new Date();
		//签退
		CourseSchedule courseSchedule = courseScheduleDao.get(firstCourseId);
		List<CourseSchedule> courseSchedules = new ArrayList<>();
		courseSchedules.add(courseSchedule);
		CourseSchedule cs = courseSchedule;
		//是否是连堂课
		String continueCourseTime = null;
		if(courseSchedule.getTeachMode() == TeachModeEnum.OFFLINE){
			continueCourseTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.OFFLINE_CONTINUE_COURSE_MINUTES,courseSchedule.getTenantId());
		}else {
			continueCourseTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.ONLINE_CONTINUE_COURSE_TIME,courseSchedule.getTenantId());
		}
		if (StringUtils.isEmpty(continueCourseTime)) {
			continueCourseTime = "5";
		}
		while (true){
			//获取当前课程的所有连堂课列表
			String courseClassDate = DateUtil.format(cs.getClassDate(), DateUtil.DEFAULT_PATTERN);
			String courseEndDateTime = DateUtil.format(cs.getEndClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
			cs = courseScheduleDao.queryTeacherContinueCourse(cs,continueCourseTime,courseClassDate + " " + courseEndDateTime);
			//存在连堂课
			if(cs != null){
				courseSchedules.add(cs);
			}else {
				break;
			}
		}
		String courseClassDate;
		String courseStartDateTime;
		String courseEndDateTime;
		Boolean isBreak = false;
		for (int i = courseSchedules.size() - 1; i >= 0; i--) {
			if(isBreak){
				break;
			}
			CourseSchedule schedule = courseSchedules.get(i);
			courseClassDate = DateUtil.format(schedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
			courseStartDateTime = DateUtil.format(schedule.getStartClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
			Date startDateTime = DateUtil.stringToDate(courseClassDate + " " + courseStartDateTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
			courseEndDateTime = DateUtil.format(schedule.getEndClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
			Date endDateTime = DateUtil.stringToDate(courseClassDate + " " + courseEndDateTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
			//不是最后一节课
			if(i == courseSchedules.size() - 1){
				if(date.after(endDateTime)){
					isBreak = upsetAttendance(userId,startDateTime,date,YesOrNoEnum.YES,schedule,firstCourseId,deviceNum);
				}else {
					isBreak = upsetAttendance(userId,startDateTime,date,YesOrNoEnum.NO,schedule,firstCourseId,deviceNum);
				}
			}else {
				isBreak = upsetAttendance(userId,startDateTime,endDateTime,YesOrNoEnum.YES,schedule,firstCourseId,deviceNum);
			}
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void addTeacherAttendanceSignIn(Long firstCourseId, Integer userId, Long courseScheduleId,String deviceNum) {
		Integer tenantId = TenantContextHolder.getTenantId();
		ClassGroup classGroup = classGroupDao.findByCourseSchedule(courseScheduleId.intValue(),0);
		TeacherAttendance teacherAttendance=teacherAttendanceDao.findByTeacherAttendanceInfo(userId.longValue(),courseScheduleId);
		if(Objects.isNull(teacherAttendance)){
			teacherAttendance=new TeacherAttendance();
			teacherAttendance.setGroupType(classGroup.getGroupType());
			teacherAttendance.setCourseScheduleId(courseScheduleId);
			teacherAttendance.setTeacherId(userId);
			teacherAttendance.setMusicGroupId(classGroup.getMusicGroupId());
			teacherAttendance.setClassGroupId(classGroup.getId());
			teacherAttendance.setTeacherId(userId);
			teacherAttendance.setSignInDeviceNo(deviceNum);
			teacherAttendance.setCurrentScheduleId(firstCourseId);
			teacherAttendanceDao.insert(teacherAttendance);
		}else {
			if(StringUtils.isEmpty(teacherAttendance.getSignInDeviceNo())){
				teacherAttendance.setSignInDeviceNo(deviceNum);
			}
			teacherAttendance.setCurrentScheduleId(firstCourseId);
			teacherAttendance.setGroupType(classGroup.getGroupType());
			teacherAttendance.setTeacherId(userId);
			teacherAttendance.setClassGroupId(classGroup.getId());
			teacherAttendance.setMusicGroupId(classGroup.getMusicGroupId());
		}
		//签到
		if(teacherAttendance.getSignInTime() == null){
			Date date = new Date();
			CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleId);
			String classDate = DateUtil.format(courseSchedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
			String startClassTime = DateUtil.format(courseSchedule.getStartClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
			Date classStartDateTime = DateUtil.stringToDate(classDate + " " + startClassTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
			YesOrNoEnum yesOrNoEnum = courseScheduleService.enableOnlyNormalAttendance(courseSchedule.getStartClassTime(),
					userId.longValue(),
					false,
					null);
			teacherAttendance.setSignInTime(date);
			teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
			teacherAttendance.setCurrentClassTimes(classGroup.getCurrentClassTimes() + 1);
			String onlineAdvanceSignInMinutesStr = sysTenantConfigService.getTenantConfigValue(SysConfigService.ONLINE_ADVANCE_SIGN_IN_MINUTES, tenantId);
			Integer onlineAdvanceSignInMinutes = 0;
			if(StringUtils.isNotEmpty(onlineAdvanceSignInMinutesStr)){
				onlineAdvanceSignInMinutes = Integer.parseInt(onlineAdvanceSignInMinutesStr);
			}
			Date add1Minutes = DateUtil.addMinutes(classStartDateTime, ~--onlineAdvanceSignInMinutes);
			int secondsBetween = DateUtil.secondsBetween(date, add1Minutes);

			if(yesOrNoEnum != YesOrNoEnum.YES && secondsBetween <= 0){
				teacherAttendance.setSignInStatus(YesOrNoEnum.NO);
			}
			teacherAttendanceDao.update(teacherAttendance);
			if(courseSchedule.getTeachMode() == TeachModeEnum.ONLINE){
				//获取所有当前课程的学生列表
				Map<Integer,String> userMap = JSONObject.parseObject(JSONObject.toJSONString(MapUtil.convertMybatisMap(courseScheduleDao.findStudentMap(courseScheduleId))),HashMap.class);
				if(userMap != null && userMap.size() > 0){
					String name = subjectDao.findNames(classGroup.getSubjectIdList());
					sysMessageService.batchSendMessage(MessageSender.JIGUANG,MessageTypeEnum.STUDENT_PUSH_ONLINE_COURSE_ACTION,userMap,
							null,0,"6?sealClassId=" + courseScheduleId + "&subjectName=" + name ,"STUDENT",courseSchedule.getName());
				}
			}
		}else if(StringUtils.isNotEmpty(teacherAttendance.getSignInDeviceNo())){
			teacherAttendance.setSignInDeviceNo(deviceNum);
			teacherAttendanceDao.update(teacherAttendance);
		}
	}

	public Boolean upsetAttendance(Integer userId,Date signInTime,Date signOutTime,YesOrNoEnum status,CourseSchedule courseSchedule,Long courseScheduleId,String deviceNum){
		TeacherAttendance teacherAttendance = teacherAttendanceDao.findByTeacherAttendanceInfo(userId.longValue(),courseSchedule.getId());
		Boolean flag = false;
		Long courseId = teacherAttendanceDao.findFirstSign(courseScheduleId,userId);
		//如果当前课程是第一次签到的课程那么不更新签到时间，并且退出下次循环
		if(courseId != null && courseId.equals(courseSchedule.getId())){
			flag = true;
		}
		if(teacherAttendance == null){
			teacherAttendance = new TeacherAttendance();
			teacherAttendance.setGroupType(courseSchedule.getGroupType());
			teacherAttendance.setTeacherId(userId);
			teacherAttendance.setMusicGroupId(courseSchedule.getMusicGroupId());
			teacherAttendance.setClassGroupId(courseSchedule.getClassGroupId());
			teacherAttendance.setCourseScheduleId(courseSchedule.getId());
			teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
			teacherAttendance.setSignOutStatus(status);
			teacherAttendance.setSignOutTime(signOutTime);
			teacherAttendance.setSignInTime(signInTime);
			teacherAttendance.setCurrentScheduleId(courseScheduleId);
			teacherAttendance.setSignInDeviceNo(deviceNum);
			teacherAttendance.setSignOutDeviceNo(deviceNum);
			teacherAttendanceDao.insert(teacherAttendance);
		}else {
			teacherAttendance.setSignOutDeviceNo(deviceNum);
			teacherAttendance.setSignOutStatus(status);
			teacherAttendance.setSignOutTime(signOutTime);
			teacherAttendance.setCurrentScheduleId(courseScheduleId);
			if(!flag){
				teacherAttendance.setSignInTime(signInTime);
				teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
			}
			teacherAttendanceDao.update(teacherAttendance);
		}
		return flag;
	}

	@Override
	public PageInfo getTeacherPersonalAttendances(TeacherCloseQueryInfo queryInfo) {
		PageInfo<TeacherPersonalAttendanceDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<TeacherPersonalAttendanceDto> dataList = null;
		int count = teacherAttendanceDao.getTeacherPersonalAttendancesCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherAttendanceDao.getTeacherPersonalAttendances(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public PageInfo queryTeacherAttendances(TeacherCloseQueryInfo queryInfo) {
		PageInfo<TeacherAttendancesDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);

		List<TeacherAttendancesDto> dataList = null;
		int count = teacherAttendanceDao.countTeacherAttendances(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherAttendanceDao.queryTeacherAttendances(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public boolean pushNoSignOutMessage(Integer tenantId) {
		String configValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.SIGN_OUT_MESSAGE_PUSH_MINUTE, tenantId);
		if(StringUtils.isEmpty(configValue)){
			return true;
		}
		Integer minutes = Integer.parseInt(configValue);
		List<Mapper> list = teacherAttendanceDao.queryNoSignOutListByOverMinutes(minutes, tenantId);
		if(CollectionUtils.isEmpty(list)){
			return true;
		}
		List<Long> courseScheduleIds = list.stream().map(m -> m.getCourseScheduleId().longValue()).collect(Collectors.toList());

		List<CourseSchedule> courseSchedules = courseScheduleDao.findByCourseScheduleIds(courseScheduleIds);
		Map<Long, CourseSchedule> idCourseMap = courseSchedules.stream().collect(Collectors.toMap(CourseSchedule::getId, c -> c, (c1, c2) -> c1));

		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalarys = courseScheduleTeacherSalaryService.findByCourseSchedules(courseScheduleIds);
		Map<Long, List<CourseScheduleTeacherSalary>> courseTeacherSalaryMap = new HashMap<>();
		if(!CollectionUtils.isEmpty(courseScheduleTeacherSalarys)){
			courseTeacherSalaryMap = courseScheduleTeacherSalarys.stream().collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId));
		}

		for (Mapper mapper : list) {
			Map<Integer, String> receivers = new HashMap<Integer, String>();
			Integer userId = (Integer) mapper.getKey();
			receivers.put(userId, userId + "");

			String key = "SignOut_" + userId + "_" + mapper.getCourseScheduleId();

			CourseSchedule courseSchedule = idCourseMap.get(mapper.getCourseScheduleId().longValue());

			List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseTeacherSalaryMap.get(mapper.getCourseScheduleId().longValue());
			TeachTypeEnum teachType = TeachTypeEnum.BISHOP;
			if(!CollectionUtils.isEmpty(courseScheduleTeacherSalaries)){
				for (CourseScheduleTeacherSalary courseScheduleTeacherSalary : courseScheduleTeacherSalaries) {
					if(mapper.getKey().equals(courseScheduleTeacherSalary.getUserId())){
						teachType = courseScheduleTeacherSalary.getTeacherRole();
					}
				}
			}

			if (!redisCache.exists(key)) {
				sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_SIGN_OUT_MESSAGE, receivers, null, 0,
						"5?courseScheduleId=" + mapper.getCourseScheduleId() + "&teacherRole=" + teachType.getCode(), "TEACHER",
						mapper.getValue() + (Objects.isNull(courseSchedule)?"":("(" + DateUtil.format(courseSchedule.getStartClassTime(), "yyyy年MM月dd日 HH时mm分") + ")")));
				redisCache.put(key, 1, (int) DateUtil.getTomorrowZeroSeconds());
			}
		}
		return true;
	}

	@Override
	public TeacherSignOutDto getTeacherSignDetail(Integer courseScheduleId, Integer userId) {
		if(Objects.isNull(courseScheduleId)){
			throw new BizException("请指定课程");
		}
		TeacherSignOutDto teacherSignDetail = new TeacherSignOutDto();
		TeacherAttendance teacherAttendanceInfo = teacherAttendanceDao.findByTeacherAttendanceInfo(userId.longValue(), courseScheduleId.longValue());
		teacherSignDetail.setTeacherAttendanceInfo(teacherAttendanceInfo);
		CourseHomework courseHomewok = courseHomeworkDao.findByCourseSchedule(courseScheduleId.longValue());
		if(Objects.nonNull(teacherAttendanceInfo)&&Objects.nonNull(courseHomewok)){
			courseHomewok.setAttachments(teacherAttendanceInfo.getSignOutAttachments());
		}
		teacherSignDetail.setCourseHomeworkInfo(courseHomewok);
		return teacherSignDetail;
	}

	@Override
	public TeacherAttendance getTeacherPersonalAttendance(Integer courseScheduleId) {
		return teacherAttendanceDao.getMainTeacherAttendance(courseScheduleId);
	}

	@Override
	public void updateTeacherAttendance(TeacherAttendance teacherAttendanceInfo) {
		if(Objects.isNull(teacherAttendanceInfo.getCourseScheduleId())){
			throw new BizException("请指定课程");
		}
		if(Objects.isNull(teacherAttendanceInfo.getSignInStatus())){
			throw new BizException("请指定签到状态");
		}
		if(Objects.isNull(teacherAttendanceInfo.getSignOutStatus())){
			throw new BizException("请指定签退状态");
		}
		CourseSchedule courseSchedule = courseScheduleDao.get(teacherAttendanceInfo.getCourseScheduleId());
		if(Objects.isNull(courseSchedule)){
			throw new BizException("课程不存在");
		}

		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryService.findByCourseSchedule(teacherAttendanceInfo.getCourseScheduleId());
		if(!CollectionUtils.isEmpty(courseScheduleTeacherSalaries)&&Objects.nonNull(courseScheduleTeacherSalaries.get(0).getSettlementTime())){
			throw new BizException("此课程已结算");
		}

		if(Objects.isNull(teacherAttendanceInfo.getTeacherId())||teacherAttendanceInfo.getTeacherId()<=0){
			if(Objects.isNull(courseSchedule.getActualTeacherId())){
				throw new BizException("请指定教师");
			}
			teacherAttendanceInfo.setTeacherId(courseSchedule.getActualTeacherId());
		}
		TeacherAttendance teacherAttendance=teacherAttendanceDao.findByTeacherAttendanceInfo(teacherAttendanceInfo.getTeacherId().longValue(),teacherAttendanceInfo.getCourseScheduleId());

		Date date = new Date();
		if(Objects.isNull(teacherAttendance)){
			teacherAttendance=new TeacherAttendance();
			teacherAttendance.setGroupType(courseSchedule.getGroupType());
			teacherAttendance.setMusicGroupId(courseSchedule.getMusicGroupId());
			teacherAttendance.setClassGroupId(courseSchedule.getClassGroupId());
			teacherAttendance.setCourseScheduleId(courseSchedule.getId());
			teacherAttendance.setTeacherId(teacherAttendanceInfo.getTeacherId());
			teacherAttendance.setCreateTime(date);
		}

		if(Objects.isNull(teacherAttendance.getSignInStatus())||teacherAttendance.getSignInStatus().equals(YesOrNoEnum.NO)){
			int advanceSignMinutes = 20;
			String tenantConfigValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.OFFLINE_ADVANCE_SIGN_IN_MINUTES, courseSchedule.getTenantId());
			if(StringUtils.isNotEmpty(tenantConfigValue)){
				advanceSignMinutes = Integer.parseInt(tenantConfigValue);
			}
			Date add20Minutes = DateUtil.addMinutes(courseSchedule.getStartClassTime(), advanceSignMinutes * -1);
			teacherAttendance.setSignInTime(add20Minutes);
			teacherAttendance.setSignInStatus(teacherAttendanceInfo.getSignInStatus());
		}

		if(Objects.isNull(teacherAttendance.getSignOutStatus())||teacherAttendance.getSignOutStatus().equals(YesOrNoEnum.NO)) {
			teacherAttendance.setSignOutTime(courseSchedule.getEndClassTime());
			teacherAttendance.setSignOutStatus(teacherAttendanceInfo.getSignOutStatus());
		}

		SysUser sysUser = sysUserFeignService.queryUserInfo();
		StringBuilder remark=new StringBuilder("补签到,");
		remark.append(sysUser.getRealName());
		teacherAttendance.setRemark(remark.toString());

		if(Objects.isNull(teacherAttendance.getId())){
			teacherAttendanceDao.insert(teacherAttendance);
		}else{
			teacherAttendanceDao.update(teacherAttendance);
		}
	}

	@Override
	public TeacherAttendanceDetailDto getTeacherPersonalAttendanceDetail(Integer courseScheduleId) {
		TeacherAttendance teacherAttendance = teacherAttendanceDao.getMainTeacherAttendance(courseScheduleId);
		CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleId.longValue());
		TeacherAttendanceDetailDto result = new TeacherAttendanceDetailDto();
		result.setStatus(courseSchedule.getStatus());
		result.setTeachMode(courseSchedule.getTeachMode());
		result.setType(courseSchedule.getType());
		result.setTeacherName(teacherDao.queryNameById(courseSchedule.getActualTeacherId()));
		result.setAttendClassTime(teacherAttendanceDao.getAttendClassTime(courseScheduleId));
		result.setSchool(schoolDao.get(courseSchedule.getSchoolId()));

        // 设置课后作业版本
        CourseHomework homework = courseHomeworkDao.getByCourseScheduleId(courseScheduleId);
        if (homework != null) {
            result.setVersionTag(homework.getVersionTag());
        }
        //获取异常考勤学员数量
		result.setErrorAttendanceNum(studentAttendanceDao.countErrorAttendance(courseScheduleId));
		if(teacherAttendance != null){
			result.setIsSignIn(Objects.isNull(teacherAttendance.getSignInStatus())?3:teacherAttendance.getSignInStatus().getCode());
			result.setIsSignOut(Objects.isNull(teacherAttendance.getSignOutStatus())?3:teacherAttendance.getSignOutStatus().getCode());
			result.setSignInStatusEnum(result.getIsSignIn() == 1?NORMAL:result.getIsSignIn() == 0?UNUSUAL:NO);
			result.setSignOutStatusEnum(result.getIsSignOut() == 1?SignOutStatusEnum.NORMAL:result.getIsSignOut() == 0? SignOutStatusEnum.UNUSUAL: SignOutStatusEnum.NO);
			result.setSignInTime(teacherAttendance.getSignInTime());
			result.setSignOutTime(teacherAttendance.getSignOutTime());
			result.setRemark(teacherAttendance.getRemark());
			int studentNum = studentAttendanceDao.countByCourseSchedule(courseScheduleId.longValue());
			if (Objects.nonNull(studentNum)) {
				result.setIsCallNames(studentNum > 0 ? YesOrNoEnum.YES : YesOrNoEnum.NO);
			} else {
				result.setIsCallNames(YesOrNoEnum.NO);
			}
		}
		return result;
	}

	@Override
	public void pushTeacherExceptionAttendanceTask() {
		Date date = new Date();
		String format = DateUtil.format(date, DateUtil.DEFAULT_PATTERN);
		Set<Integer> teacherId = teacherAttendanceDao.queryTeacherExceptionAttendance(format);
		if(teacherId == null || teacherId.size() == 0){
			return;
		}
		Map<Integer,String> receivers = new HashMap(teacherId.size());
		teacherId.forEach(e->{
			receivers.put(e,e.toString());
		});
		String url = new StringBuffer("10?date=").append(format).toString();
		sysMessageService.batchSendMessage(MessageSender.JIGUANG,MessageTypeEnum.TEACHER_PUSH_EXCEPTION_ATTENDANCE,receivers,null,0,url,"TEACHER",format);
	}

	//校验考勤申诉
	void checkComplaints(Integer tenantId,Date classDate,Boolean isWeb){
		Date date = new Date();
		if(isWeb){
			String salary = sysTenantConfigService.getTenantConfigValue(SysConfigService.WEB_TEACHER_COMPLAINTS_SALARY,tenantId);
			if(StringUtils.isNotEmpty(salary)){
				Integer day = Integer.parseInt(salary);
				if(day <= 0){
					return;
				}
				//获取上课时间下个月5号
				date = DateUtil.addDays(DateUtil.getFirstDayOfMonth(DateUtil.addMonths(classDate, 1)), day);
				int i = DateUtil.daysBetween(date,new Date());
				if(i > 0){
					throw new BizException("超出可申述时间");
				}
			}
		}else {
			String salary = sysTenantConfigService.getTenantConfigValue(SysConfigService.TEACHER_APP_SALARY_COMPLAINTS_TIME,tenantId);
			if(StringUtils.isNotEmpty(salary)) {
				Integer day = Integer.parseInt(salary);
				if (day <= 0) {
					return;
				}
				//如果课程时间在两天前
				int i = DateUtil.daysBetween(classDate, DateUtil.stringToDate(DateUtil.format(date, DateUtil.DEFAULT_PATTERN), DateUtil.DEFAULT_PATTERN));
				if (i > day) {
					throw new BizException("超出可申述时间");
				}
			}
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void addComplaints(Long courseScheduleId, String content,String url,Integer userId, UpdateAttendanceEnum complaintsType) {
		Boolean isWeb = true;
		if(userId == null){
			isWeb = false;
			SysUser user = sysUserFeignService.queryUserInfo();
			if(Objects.isNull(user)){
				throw new BizException("请登录");
			}
			userId = user.getId();
		}
		TeacherAttendance teacherAttendance = teacherAttendanceDao.findByTeacherAttendanceInfo(userId.longValue(), courseScheduleId);
		CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleId);
		if(courseSchedule == null){
			throw new BizException("课程不存在");
		}
		if(teacherAttendance == null){
			teacherAttendance = new TeacherAttendance();
			teacherAttendance.setMusicGroupId(courseSchedule.getMusicGroupId());
			teacherAttendance.setTeacherId(userId);
			teacherAttendance.setClassGroupId(courseSchedule.getClassGroupId());
			teacherAttendance.setGroupType(courseSchedule.getGroupType());
			teacherAttendance.setCourseScheduleId(courseScheduleId);
			teacherAttendanceDao.insert(teacherAttendance);
		}else if(teacherAttendance.getComplaintsStatus() != null && teacherAttendance.getComplaintsStatus() != REPEAL){
            throw new BizException("请勿重复提交考勤申述");
        }
		if(teacherAttendance.getSignInStatus() == YesOrNoEnum.YES && teacherAttendance.getSignOutStatus() == YesOrNoEnum.YES){
			throw new BizException("考勤正常，无需提交申诉");
		}
		Teacher teacher = teacherDao.get(userId);
		if(teacher.getTenantId() == null){
			throw new BizException("用户机构信息异常");
		}
		checkComplaints(teacher.getTenantId(),courseSchedule.getClassDate(),isWeb);
		if(complaintsType == null){
			complaintsType = UpdateAttendanceEnum.ALL;
		}
		teacherAttendance.setComplaintsType(complaintsType);
		teacherAttendance.setIsComplaints(1);
		teacherAttendance.setComplaintsStatus(ComplaintsStatusEnum.WAIT);
		teacherAttendance.setComplaintsContent(content);
		teacherAttendance.setComplaintsTime(new Date());
		teacherAttendance.setUrl(url);
		teacherAttendanceDao.update(teacherAttendance);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void repealComplaints(Long courseScheduleId,Integer userId) {
		TeacherAttendance teacherAttendance = teacherAttendanceDao.findByTeacherAttendanceInfo(userId.longValue(), courseScheduleId);
		if(teacherAttendance == null){
			throw new BizException("考勤信息不存在");
		}
		if(teacherAttendance.getComplaintsStatus() == ComplaintsStatusEnum.WAIT){
			teacherAttendance.setComplaintsStatus(REPEAL);
			teacherAttendanceDao.update(teacherAttendance);
		}else {
			throw new BizException("操作失败: 当前申述状态不允许撤销");
		}
	}

	@Override
	public PageInfo<TeacherAttendanceComplaintsDto> queryTeacherAttendanceComplaints(TeacherAttendanceComplaintsQueryInfo queryInfo) {
		PageInfo<TeacherAttendanceComplaintsDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		List<TeacherAttendanceComplaintsDto> dataList = null;
		int count = teacherAttendanceDao.countTeacherAttendanceComplaints(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherAttendanceDao.queryTeacherAttendanceComplaints(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void agreeTeacherAttendanceComplaints(long teacherAttendanceId,String content) {
		TeacherAttendance teacherAttendance = teacherAttendanceDao.get(teacherAttendanceId);
		if(teacherAttendance == null){
			throw new BizException("考勤信息不存在");
		}
		UpdateAttendanceEnum complaintsType = teacherAttendance.getComplaintsType();
		if(teacherAttendance.getComplaintsStatus() == ComplaintsStatusEnum.WAIT){
			teacherAttendance.setUpdateAttendanceEnum(complaintsType);
			StringBuffer remark = new StringBuffer("同意异常考勤申述");
			CourseSchedule courseSchedule = courseScheduleDao.get(teacherAttendance.getCourseScheduleId());
			String classDate = DateUtil.format(courseSchedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
			if(complaintsType != UpdateAttendanceEnum.SIGN_OUT && teacherAttendance.getSignInStatus() != YesOrNoEnum.YES){
				if(teacherAttendance.getSignInTime() != null){
					remark.append("  原签到时间: ").append(DateUtil.format(teacherAttendance.getSignInTime(),DateUtil.EXPANDED_DATE_TIME_FORMAT));
				}
				String startTime = DateUtil.format(courseSchedule.getStartClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
				Date date = DateUtil.stringToDate(classDate + ' ' + startTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
				teacherAttendance.setSignInTime(DateUtil.addMinutes(date,-1));
				teacherAttendance.setSignInStatus(YesOrNoEnum.YES);
			}
			if(complaintsType != UpdateAttendanceEnum.SIGN_IN && teacherAttendance.getSignOutStatus() != YesOrNoEnum.YES){
				if(teacherAttendance.getSignOutTime() != null){
					remark.append("  原签退时间: ").append(DateUtil.format(teacherAttendance.getSignOutTime(),DateUtil.EXPANDED_DATE_TIME_FORMAT));
				}
				String endTime = DateUtil.format(courseSchedule.getEndClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
				Date date = DateUtil.stringToDate(classDate + ' ' + endTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
				teacherAttendance.setSignOutTime(date);
				teacherAttendance.setSignOutStatus(YesOrNoEnum.YES);
			}
			teacherAttendance.setDisposeContent(content);
			SysUser user = sysUserFeignService.queryUserInfo();
			if(user != null && user.getId() != null){
				teacherAttendance.setOperator(user.getId());
			}
			teacherAttendance.setComplaintsStatus(ComplaintsStatusEnum.PASS);
			teacherAttendance.setRemark(remark.toString());
			teacherAttendanceDao.update(teacherAttendance);
			String format = DateUtil.format(courseSchedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
			Map<Integer, String> receivers = new HashMap<>(1);
			receivers.put(teacherAttendance.getTeacherId(),teacherAttendance.getTeacherId().toString());
			sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_ATTENDANCE_COMPLAINTS_RESULT, receivers, null, 0, null, "TEACHER",
					format,courseSchedule.getName(),"通过");
		}else {
			throw new BizException("操作失败: 当前申述状态不允许该操作");
		}
	}

	@Override
	public void rejectTeacherAttendanceComplaints(long teacherAttendanceId,String content) {
		SysUser user = sysUserFeignService.queryUserInfo();
		if(Objects.isNull(user)){
			throw new BizException("请登录");
		}
		TeacherAttendance teacherAttendance = teacherAttendanceDao.get(teacherAttendanceId);
		if(teacherAttendance == null){
			throw new BizException("考勤信息不存在");
		}
		if(teacherAttendance.getComplaintsStatus() == ComplaintsStatusEnum.WAIT){
			teacherAttendance.setDisposeContent(content);
			teacherAttendance.setOperator(user.getId());
			teacherAttendance.setComplaintsStatus(ComplaintsStatusEnum.REJECT);
			teacherAttendanceDao.update(teacherAttendance);
			CourseSchedule courseSchedule = courseScheduleDao.get(teacherAttendance.getCourseScheduleId());
			String format = DateUtil.format(courseSchedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
			Map<Integer, String> receivers = new HashMap<>(1);
			receivers.put(teacherAttendance.getTeacherId(),teacherAttendance.getTeacherId().toString());
			sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_ATTENDANCE_COMPLAINTS_RESULT, receivers, null, 0, null, "TEACHER",
					format,courseSchedule.getName(),"拒绝");
		}else {
			throw new BizException("操作失败: 当前申述状态不允许该操作");
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateDeviceNum(Integer courseId, String userId, String signInDeviceNum, String signOutDeviceNum) {
		teacherAttendanceDao.updateDeviceNum(courseId,userId,signInDeviceNum,signOutDeviceNum);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void addTeacherAttendance(List<CourseSchedule> courseScheduleList) {
		List<TeacherAttendance> teacherAttendanceList = new ArrayList<>();
		for (CourseSchedule courseSchedule : courseScheduleList) {
			TeacherAttendance teacherAttendance = new TeacherAttendance();
			teacherAttendance.setGroupType(courseSchedule.getGroupType());
			teacherAttendance.setTeacherId(courseSchedule.getActualTeacherId());
			teacherAttendance.setMusicGroupId(courseSchedule.getMusicGroupId());
			teacherAttendance.setClassGroupId(courseSchedule.getClassGroupId());
			teacherAttendance.setCourseScheduleId(courseSchedule.getId());
			teacherAttendanceList.add(teacherAttendance);
		}
		teacherAttendanceDao.batchInsert(teacherAttendanceList);
	}

    @Override
    public List<TeacherAttendanceSumDto> queryTeacherAttendance(TeacherAttendanceQueryInfo queryInfo) {
		//获取合作单位关联的所有乐团列表
		List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(queryInfo.getCoopId());
		if(CollectionUtils.isEmpty(musicGroupIds)){
			return Collections.EMPTY_LIST;
		}
		//获取当前学期
		Date date = new Date();
		String startTerm = DateUtil.getStartTerm(date);
		String endTerm = DateUtil.getEndTerm(date);
		queryInfo.setStartTerm(startTerm);
		queryInfo.setEndTerm(endTerm);
		queryInfo.setMusicGroupIds(musicGroupIds);
		List<TeacherAttendanceSumDto> dtoList = teacherAttendanceDao.queryTeacherAttendance(queryInfo);
		if(CollectionUtils.isNotEmpty(dtoList)){
			//获取本周老师考勤状态
			Date now = new Date();
			String monday = DateUtil.dateToString(DateUtil.getWeekDayWithDate(now, Calendar.MONDAY), DateUtil.ISO_EXPANDED_DATE_FORMAT);
			String sunday = DateUtil.dateToString(DateUtil.getWeekDayWithDate(now, Calendar.SUNDAY), DateUtil.ISO_EXPANDED_DATE_FORMAT);
			//获取本周考勤列表
			TeacherAttendanceQueryInfo info = new TeacherAttendanceQueryInfo();
			info.setStartTerm(monday);
			info.setEndTerm(sunday);
			info.setMusicGroupIds(queryInfo.getMusicGroupIds());
			List<TeacherAttendanceWrapper.CourseTeacherAttendance> courseTeacherAttendances = teacherAttendanceDao.queryTeacherAttendanceForWeek(info);
			//获取本学期考勤
			info.setStartTerm(startTerm);
			info.setEndTerm(endTerm);
			info.setMusicGroupIds(queryInfo.getMusicGroupIds());
			List<TeacherAttendanceWrapper.CourseTeacherAttendance> termCourseTeacherAttendances = teacherAttendanceDao.queryTeacherAttendanceForWeek(info);
			Map<Integer, List<TeacherAttendanceWrapper.CourseTeacherAttendance>> attendanceMap = new HashMap<>(dtoList.size());
			Map<Integer, List<TeacherAttendanceWrapper.CourseTeacherAttendance>> termAttendanceMap = new HashMap<>(dtoList.size());
			Map<Integer, School> idSchoolMap = new HashMap<>();
			Set<Integer> schoolIds = new HashSet<>();
			if(CollectionUtils.isNotEmpty(courseTeacherAttendances)){
				attendanceMap = courseTeacherAttendances.stream().collect(Collectors.groupingBy(e -> e.getActualTeacherId()));
				schoolIds = courseTeacherAttendances.stream().map(TeacherAttendanceWrapper.CourseTeacherAttendance::getSchoolId).collect(Collectors.toSet());
			}
			if(CollectionUtils.isNotEmpty(termCourseTeacherAttendances)){
				termAttendanceMap = termCourseTeacherAttendances.stream().collect(Collectors.groupingBy(e -> e.getActualTeacherId()));
				schoolIds.addAll(termCourseTeacherAttendances.stream().map(TeacherAttendanceWrapper.CourseTeacherAttendance::getSchoolId).collect(Collectors.toSet()));
			}
			if(CollectionUtils.isNotEmpty(schoolIds)){
				List<School> schools = schoolDao.getSchools(new ArrayList<>(schoolIds));
				if(!org.springframework.util.CollectionUtils.isEmpty(schools)){
					idSchoolMap = schools.stream().collect(Collectors.toMap(School::getId, s->s, (s1, s2)->s1));
				}
			}
			//定位范围
			String attendanceRange = sysTenantConfigService.getTenantConfigValue(SysConfigService.ATTENDANCE_RANGE,queryInfo.getTenantId());

			for (TeacherAttendanceSumDto dto : dtoList) {
				List<TeacherAttendanceWrapper.CourseTeacherAttendance> attendances = attendanceMap.get(dto.getTeacherId());
				if(CollectionUtils.isNotEmpty(attendances)){
					Map<Integer, List<TeacherAttendanceWrapper.CourseTeacherAttendance>> classMap = attendances.stream().collect(Collectors.groupingBy(e -> e.getClassGroupId()));
					List<TeacherAttendanceWrapper.CourseTeacherAttendance> attendanceList = classMap.get(dto.getClassGroupId());
					if(CollectionUtils.isNotEmpty(attendanceList)){
						dto.setTeacherAttendanceStatus(TeacherAttendanceQueryInfo.TeacherAttendanceEnum.NORMAL);
						att:for (TeacherAttendanceWrapper.CourseTeacherAttendance attendance : attendanceList) {
							if(attendance.getSignInStatus() == 0 || attendance.getSignOutStatus() == 0){
								dto.setTeacherAttendanceStatus(TeacherAttendanceQueryInfo.TeacherAttendanceEnum.ERR);
								break att;
							}
							if(attendance.getTeachMode() == TeachModeEnum.OFFLINE){
								if(StringUtils.isEmpty(attendance.getSignInLongitudeLatitude()) || StringUtils.isEmpty(attendance.getSignOutLongitudeLatitude())){
									dto.setTeacherAttendanceStatus(TeacherAttendanceQueryInfo.TeacherAttendanceEnum.ERR);
									break att;
								}else {
									//计算考勤范围
									School school = idSchoolMap.get(attendance.getSchoolId());
									if(school != null){
										Boolean signRange = courseScheduleTeacherSalaryService.getSignRange(school.getLongitudeLatitude(), attendance.getSignInLongitudeLatitude(), attendanceRange);
										if(!signRange){
											dto.setTeacherAttendanceStatus(TeacherAttendanceQueryInfo.TeacherAttendanceEnum.ERR);
											break att;
										}
										signRange = courseScheduleTeacherSalaryService.getSignRange(school.getLongitudeLatitude(), attendance.getSignOutLongitudeLatitude(), attendanceRange);
										if(!signRange){
											dto.setTeacherAttendanceStatus(TeacherAttendanceQueryInfo.TeacherAttendanceEnum.ERR);
											break att;
										}
									}
								}
							}
						}
					}
				}
				List<TeacherAttendanceWrapper.CourseTeacherAttendance> termAttendances = termAttendanceMap.get(dto.getTeacherId());
				if(CollectionUtils.isNotEmpty(termAttendances)){
					Map<Integer, List<TeacherAttendanceWrapper.CourseTeacherAttendance>> classMap = termAttendances.stream().collect(Collectors.groupingBy(e -> e.getClassGroupId()));
					List<TeacherAttendanceWrapper.CourseTeacherAttendance> attendanceList = classMap.get(dto.getClassGroupId());
					if(CollectionUtils.isNotEmpty(attendanceList)){
						int errCourseNum = 0;
						att:for (TeacherAttendanceWrapper.CourseTeacherAttendance attendance : attendanceList) {
							if(attendance.getSignInStatus() == 0 || attendance.getSignOutStatus() == 0){
								errCourseNum++;
								continue att;
							}
							if(attendance.getTeachMode() == TeachModeEnum.OFFLINE){
								if(StringUtils.isEmpty(attendance.getSignInLongitudeLatitude()) || StringUtils.isEmpty(attendance.getSignOutLongitudeLatitude())){
									errCourseNum++;
								}else {
									//计算考勤范围
									School school = idSchoolMap.get(attendance.getSchoolId());
									if(school != null){
										Boolean signRange = courseScheduleTeacherSalaryService.getSignRange(school.getLongitudeLatitude(), attendance.getSignInLongitudeLatitude(), attendanceRange);
										if(!signRange){
											errCourseNum++;
											continue att;
										}
										signRange = courseScheduleTeacherSalaryService.getSignRange(school.getLongitudeLatitude(), attendance.getSignOutLongitudeLatitude(), attendanceRange);
										if(!signRange){
											errCourseNum++;
										}
									}
								}
							}
						}
						dto.setErrCourseNum(errCourseNum);
					}
				}
			}
			if(queryInfo.getTeacherAttendanceStatus() != null){
				dtoList = dtoList.stream().filter(e->e.getTeacherAttendanceStatus() == queryInfo.getTeacherAttendanceStatus()).collect(Collectors.toList());
			}
		}
		return dtoList;
    }

	@Override
	public List<TeacherAttendanceListDto> getClassTeacherAttendance(TeacherAttendanceWrapper.ClassTeacherAttendanceQuery query) {
		List<TeacherAttendanceListDto> attendanceDtos = teacherAttendanceDao.getClassTeacherAttendance(query);
		Integer tenantId = TenantContextHolder.getTenantId();
		//计算考勤范围
		//定位范围
		String attendanceRange = sysTenantConfigService.getTenantConfigValue(SysConfigService.ATTENDANCE_RANGE,tenantId);
		if(CollectionUtils.isNotEmpty(attendanceDtos)){
			for (TeacherAttendanceListDto attendanceDto : attendanceDtos) {
				if(attendanceDto.getTeachMode() == TeachModeEnum.OFFLINE){
					if(StringUtils.isNotEmpty(attendanceDto.getSignInLongitudeLatitude())){
						boolean signRange = courseScheduleTeacherSalaryService.getSignRange(attendanceDto.getSchoolLongitudeLatitude(), attendanceDto.getSignInLongitudeLatitude(), attendanceRange);
						attendanceDto.setSignInAddressStatus(signRange);
					}
					if(StringUtils.isNotEmpty(attendanceDto.getSignOutLongitudeLatitude())){
						boolean signRange1 = courseScheduleTeacherSalaryService.getSignRange(attendanceDto.getSchoolLongitudeLatitude(), attendanceDto.getSignOutLongitudeLatitude(), attendanceRange);
						attendanceDto.setSignOutAddressStatus(signRange1);
					}
				}
			}
			if(query.getSignStatus() != null){
				if(query.getSignStatus()){
					attendanceDtos = attendanceDtos.stream().filter(e->e.getSignInStatus() && e.getSignOutStatus() && e.getSignInAddressStatus() && e.getSignOutAddressStatus()).collect(Collectors.toList());
				}else {
					attendanceDtos = attendanceDtos.stream().filter(e->!e.getSignInStatus() || !e.getSignOutStatus() || !e.getSignInAddressStatus() || !e.getSignOutAddressStatus()).collect(Collectors.toList());
				}
			}

		}
		return attendanceDtos;
	}

	@Override
	public com.ym.mec.biz.dal.school.dto.TeacherAttendanceDto getCourseTeacherAttendance(Long courseId, Integer teacherId) {
		com.ym.mec.biz.dal.school.dto.TeacherAttendanceDto attendanceDto = teacherAttendanceDao.getCourseTeacherAttendance(courseId,teacherId);
		if(attendanceDto.getTeachMode() == TeachModeEnum.OFFLINE){
			Integer tenantId = TenantContextHolder.getTenantId();
			String attendanceRange = sysTenantConfigService.getTenantConfigValue(SysConfigService.ATTENDANCE_RANGE,tenantId);
			if(StringUtils.isNotEmpty(attendanceDto.getSignInLongitudeLatitude())){
				boolean signRange = courseScheduleTeacherSalaryService.getSignRange(attendanceDto.getSchoolLongitudeLatitude(), attendanceDto.getSignInLongitudeLatitude(), attendanceRange);
				attendanceDto.setSignInAddressStatus(signRange);
			}
			if(StringUtils.isNotEmpty(attendanceDto.getSignOutLongitudeLatitude())){
				boolean signRange1 = courseScheduleTeacherSalaryService.getSignRange(attendanceDto.getSchoolLongitudeLatitude(), attendanceDto.getSignOutLongitudeLatitude(), attendanceRange);
				attendanceDto.setSignOutAddressStatus(signRange1);
			}
		}
		CourseSchedule courseSchedule = courseScheduleDao.get(courseId);
		attendanceDto.setCourseStatus(courseSchedule.getStatus());
		return attendanceDto;
	}

    @Override
    public Map<Long, TeacherAttendance> getMapByCourseIds(List<Long> courseIds) {
        if (CollectionUtils.isEmpty(courseIds)) {
            return Collections.emptyMap();
        }
        List<TeacherAttendance> attendanceList = teacherAttendanceDao.findByCourseSchedules(courseIds);
        if (CollectionUtils.isEmpty(attendanceList)) {
            return Collections.emptyMap();
        }
        return attendanceList.stream().collect(Collectors.toMap(TeacherAttendance::getCourseScheduleId, Function.identity(), (k1, k2) -> k1));

    }
}
