package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.TeacherCloudCourseReportDao;
import com.ym.mec.biz.dal.entity.TeacherCloudCourseReport;
import com.ym.mec.biz.dal.page.TeacherCloudCourseReportQueryInfo;
import com.ym.mec.biz.dal.vo.TeacherCloudCourseReportDto;
import com.ym.mec.biz.service.TeacherCloudCourseReportService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
public class TeacherCloudCourseReportServiceImpl extends BaseServiceImpl<Long, TeacherCloudCourseReport>  implements TeacherCloudCourseReportService {
	
	@Autowired
	private TeacherCloudCourseReportDao teacherCloudCourseReportDao;

	@Override
	public BaseDAO<Long, TeacherCloudCourseReport> getDAO() {
		return teacherCloudCourseReportDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void taskReport() {
		String classDate = DateUtil.format(DateUtil.addDays(new Date(),-1), DateUtil.DEFAULT_PATTERN);
		teacherCloudCourseReportDao.deleteByClassDate(classDate);
		teacherCloudCourseReportDao.batchSaveBySelect(classDate);
	}

	@Override
	public PageInfo<TeacherCloudCourseReportDto> queryReportPage(TeacherCloudCourseReportQueryInfo queryInfo) {
		PageInfo<TeacherCloudCourseReportDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<TeacherCloudCourseReportDto> dataList = null;
		int count = teacherCloudCourseReportDao.countReport(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = teacherCloudCourseReportDao.queryReport(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}
}