package com.ym.mec.biz.service.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.TeacherDao;
import com.ym.mec.biz.dal.dao.TempTestDao;
import com.ym.mec.biz.dal.entity.TempTest;
import com.ym.mec.biz.service.TempTestService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.date.DateUtil;

@Service
public class TempTestServiceImpl extends BaseServiceImpl<Long, TempTest> implements TempTestService {

	@Autowired
	private TempTestDao tempTestDao;

	@Autowired
	private TeacherDao teacherDao;

	@Override
	public BaseDAO<Long, TempTest> getDAO() {
		return tempTestDao;
	}

	@Override
	public Integer queryUserId(String mobile, String idCard, String realName) {
		return null;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean processBiz(List<Map<String, Object>> datas) {

		String mobile, idCard, realName, orderNo;
		SysUser user = null;
		Integer userId = null;
		Date classDate = null;
		
		BigDecimal courseFee = null;
		
		List<UserSalary> userSalaryList = new ArrayList<UserSalary>();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
		UserSalary userSalary = null;
		
		//数据整合
		for (Map<String, Object> row : datas) {
			orderNo = row.get("订单号").toString();
			mobile = row.get("收款人手机号").toString();
			idCard = row.get("收款人身份证").toString();
			realName = row.get("收款人姓名").toString();
			courseFee = new BigDecimal(row.get("收款金额").toString());
			classDate = DateUtil.stringToDate(row.get("提交订单时间").toString(), "yyyy/MM/dd HH:mm:ss");
			classDate = DateUtil.addMonths(classDate, -1);
			
			user = teacherDao.getUserWithIdCard(idCard);
			
			if (user == null) {
				user = teacherDao.getUserWithRealName(realName);
			}
			if (user == null) {
				user = teacherDao.getUserWithPhone(mobile);
			}
			
			if(user == null){
				System.out.println("订单号[" + orderNo + "]用户找不到");
				continue;
			}
			
			userSalary = new UserSalary(mobile, user, sdf.format(classDate), courseFee, orderNo);
			userSalaryList.add(userSalary);
		}
		
		Map<String,UserSalary> userSalaryMap = new HashMap<String, UserSalary>();
		
		for(UserSalary us : userSalaryList){
			if(userSalaryMap.containsKey(us.getBatchNo())){
				userSalary = userSalaryMap.get(us.getBatchNo());
				userSalary.setCourseFee(userSalary.getCourseFee().add(us.getCourseFee()));

				userSalaryMap.put(us.getBatchNo(), userSalary);
			}else{
				userSalaryMap.put(us.getBatchNo(), us);
			}
		}
		
		
		
		for (Entry<String, UserSalary> entry : userSalaryMap.entrySet()) {

			userSalary = entry.getValue();
			userId = userSalary.getUserId();
			
			//查询课程，优先vip课
			List<TempTest> tempTestList = tempTestDao.queryCourseScheduleInfo(userId, userSalary.getClassMonth());
			
			tempTestList = extractCourseSchedule(tempTestList, userSalary.getCourseFee());
			if(tempTestList.size() > 0){
				try{
					tempTestDao.batchInsert(tempTestList);
				}catch(Exception e){
					System.out.println("userId:" + userId + "  mobile:" + entry.getValue().getMobile() + " month:" + entry.getValue().getClassMonth() + " message:" + e.getMessage());
				}
			}
			
		}
		return true;
	}

	@Override
	public boolean processBiz1(List<Map<String, Object>> datas) {

		String mobile, idCard, realName, orderNo;
		SysUser user = null;
		Integer userId = null;
		Date classDate = null;
		
		BigDecimal courseFee = null;
		
		List<UserSalary> userSalaryList = new ArrayList<UserSalary>();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
		UserSalary userSalary = null;
		
		//数据整合
		for (Map<String, Object> row : datas) {
			realName = row.get("收款人姓名").toString();
			courseFee = new BigDecimal(row.get("收款金额").toString());
			classDate = DateUtil.stringToDate(row.get("月份").toString(), "yyyy/MM/dd");
			classDate = DateUtil.addMonths(classDate, -1);
			
			user = teacherDao.getUserWithRealName(realName);
			
			if(user == null){
				System.out.println("[" + realName + "]用户找不到");
				continue;
			}
			
			userSalary = new UserSalary(null, user, sdf.format(classDate), courseFee, null);
			userSalaryList.add(userSalary);
		}
		
		Map<String,UserSalary> userSalaryMap = new HashMap<String, UserSalary>();
		
		for(UserSalary us : userSalaryList){
			if(userSalaryMap.containsKey(us.getBatchNo())){
				userSalary = userSalaryMap.get(us.getBatchNo());
				userSalary.setCourseFee(userSalary.getCourseFee().add(us.getCourseFee()));

				userSalaryMap.put(us.getBatchNo(), userSalary);
			}else{
				userSalaryMap.put(us.getBatchNo(), us);
			}
		}
		
		
		
		for (Entry<String, UserSalary> entry : userSalaryMap.entrySet()) {

			userSalary = entry.getValue();
			userId = userSalary.getUserId();
			
			//查询课程，优先vip课
			List<TempTest> tempTestList = tempTestDao.queryCourseScheduleInfo(userId, userSalary.getClassMonth());
			
			tempTestList = extractCourseSchedule(tempTestList, userSalary.getCourseFee());
			if(tempTestList.size() > 0){
				try{
					tempTestDao.batchInsert(tempTestList);
				}catch(Exception e){
					System.out.println("userId:" + userId + "  mobile:" + entry.getValue().getMobile() + " month:" + entry.getValue().getClassMonth() + " message:" + e.getMessage());
				}
			}
			
		}
		return true;
	}

	private List<TempTest> extractCourseSchedule(List<TempTest> srcTempTestList, BigDecimal limitSalary){
		
		BigDecimal courseSalary = BigDecimal.ZERO;
		List<TempTest> result = new ArrayList<TempTest>();
		
		for(TempTest tt : srcTempTestList){
			if(tt.getSalary().compareTo(BigDecimal.ZERO) <= 0){
				continue;
			}
			if(limitSalary.compareTo(courseSalary.add(tt.getSalary())) > 0){
				courseSalary = courseSalary.add(tt.getSalary());
				result.add(tt);
			}else{
				tt.setSalary(limitSalary.subtract(courseSalary));
				result.add(tt);
				break;
			}
		}
		
		return result;
	}
}

class UserSalary{
	
	private Integer userId;
	
	private SysUser user;
	
	private String classMonth;
	
	private String orderNo;
	
	private BigDecimal courseFee;
	
	private String batchNo;
	
	private String mobile;
	
	public UserSalary(String mobile, SysUser user, String classMonth,BigDecimal courseFee,String orderNo) {
		this.mobile = mobile;
		this.user = user;
		this.classMonth = classMonth;
		this.courseFee = courseFee;
		this.orderNo = orderNo;
		this.batchNo = user.getId() + "#" + classMonth;
		this.userId = user.getId();
	}

	public Integer getUserId() {
		return userId;
	}

	public void setUserId(Integer userId) {
		this.userId = userId;
	}

	public SysUser getUser() {
		return user;
	}

	public void setUser(SysUser user) {
		this.user = user;
	}

	public String getClassMonth() {
		return classMonth;
	}

	public void setClassMonth(String classMonth) {
		this.classMonth = classMonth;
	}

	public String getOrderNo() {
		return orderNo;
	}

	public void setOrderNo(String orderNo) {
		this.orderNo = orderNo;
	}

	public BigDecimal getCourseFee() {
		return courseFee;
	}

	public void setCourseFee(BigDecimal courseFee) {
		this.courseFee = courseFee;
	}

	public String getBatchNo() {
		return batchNo;
	}

	public void setBatchNo(String batchNo) {
		this.batchNo = batchNo;
	}

	public String getMobile() {
		return mobile;
	}

	public void setMobile(String mobile) {
		this.mobile = mobile;
	}
}