package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alipay.service.schema.util.StringUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.microsvc.toolkit.common.response.paging.QueryInfo;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ClientEnum;
import com.ym.mec.biz.dal.mapper.UserMusicMapper;
import com.ym.mec.biz.dal.mapper.UserMusicStarMapper;
import com.ym.mec.biz.dal.wrapper.UserMusicStarWrapper;
import com.ym.mec.biz.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 用户曲目作品点赞记录
 * 2023-10-30 14:59:53
 */
@Slf4j
@Service
public class UserMusicStarServiceImpl extends ServiceImpl<UserMusicStarMapper, UserMusicStar> implements UserMusicStarService {

    @Autowired
    private UserMusicMapper userMusicMapper;

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private TeacherService teacherService;

    @Autowired
    private SysConfigService sysConfigService;
	/**
     * 查询详情
     * @param id 详情ID
     * @return UserMusicStar
     */
	@Override
    public UserMusicStar detail(Long id) {

        return baseMapper.selectById(id);
    }

    /**
     * 分页查询
     * @param page IPage<UserMusicStar>
     * @param query UserMusicStarWrapper.UserMusicStarQuery
     * @return IPage<UserMusicStar>
     */
    @Override
    public IPage<UserMusicStarWrapper.UserMusicStar> selectPage(IPage<UserMusicStar> page, UserMusicStarWrapper.UserMusicStarQuery query) {

        IPage<UserMusicStar> userMusicStarIPage = page.setRecords(baseMapper.selectPage(page, query));
        IPage<UserMusicStarWrapper.UserMusicStar> convert = userMusicStarIPage.convert(o -> JSON.parseObject(JSON.toJSONString(o), UserMusicStarWrapper.UserMusicStar.class));


        List<UserMusicStarWrapper.UserMusicStar> records = convert.getRecords();

        List<UserMusicStarWrapper.UserMusicStar> userMusicStars = new ArrayList<>();
        if (CollectionUtils.isEmpty(records)){
            return convert;
        }

        userMusicStars = JSON.parseArray(JSON.toJSONString(records), UserMusicStarWrapper.UserMusicStar.class);

        // 学生ID集合
        List<Integer> studentIds = records.stream().map(UserMusicStarWrapper.UserMusicStar::getUserId).map(Long::intValue).collect(Collectors.toList());

        Map<Long, SysUser> userMap = sysUserService.getMapByIds(studentIds);
        studentIds = records.stream().filter(o ->o.getClientType() == ClientEnum.STUDENT)
            .map(UserMusicStarWrapper.UserMusicStar::getUserId).map(Long::intValue).collect(Collectors.toList());

        Map<Integer, Student> studentMap = studentService.getMapByIds(studentIds);


        studentIds = records.stream().filter(o ->o.getClientType() == ClientEnum.TEACHER)
            .map(UserMusicStarWrapper.UserMusicStar::getUserId).map(Long::intValue).collect(Collectors.toList());

        SysConfig userAvatar = sysConfigService.findByParamName(SysConfigService.USER_DEFAULT_AVATAR);
        Map<Integer, Teacher> teacherMap = teacherService.getMapByIds(studentIds);
        for (UserMusicStarWrapper.UserMusicStar userMusicStar : userMusicStars) {
            if (userMusicStar.getClientType() == ClientEnum.STUDENT) {
                Student student = studentMap.get(userMusicStar.getUserId().intValue());
                if (student != null) {
                    userMusicStar.setSubjectId(student.getSubjectIdList());
                    userMusicStar.setCurrentClass(student.getCurrentClass());
                    userMusicStar.setCurrentGradeNum(student.getCurrentGradeNum());
                }
            } else {
                Teacher teacher = teacherMap.get(userMusicStar.getUserId().intValue());
                if (teacher != null) {
                    userMusicStar.setSubjectId(teacher.getSubjectId());
                }

            }
            SysUser sysUser = userMap.get(userMusicStar.getUserId());
            if (sysUser != null) {
                userMusicStar.setUserName(sysUser.getUsername());
                userMusicStar.setUserAvatar(sysUser.getAvatar());
            }
            if (StringUtil.isEmpty(userMusicStar.getUserAvatar())) {
                userMusicStar.setUserAvatar(userAvatar.getParanValue());
            }
        }

        // 声部ID集合
        List<Integer> subjectIds = userMusicStars.stream().map(UserMusicStarWrapper.UserMusicStar::getSubjectId)
            .filter(StringUtils::isNotBlank).map(Integer::parseInt).collect(Collectors.toList());

        Map<Integer, Subject> subjectMap =
            subjectService.getMapByIds(subjectIds);

        for (UserMusicStarWrapper.UserMusicStar userMusicStar : userMusicStars) {


            if (!StringUtil.isEmpty(userMusicStar.getSubjectId())) {
                String[] split = userMusicStar.getSubjectId().split(",");
                List<String> subjectNameList = new ArrayList<>();
                for (String s : split) {
                    Subject subject = subjectMap.get(Integer.parseInt(s));
                    if (subject != null) {
                        subjectNameList.add(subject.getName());
                    }
                }
                userMusicStar.setSubjectName(String.join(",", subjectNameList));

            }
        }
        return convert.setRecords(userMusicStars);
    }

    /**
     * 添加
     * @param userMusicStar UserMusicStarWrapper.UserMusicStar
     * @return Boolean
     */
    @Override
    public Boolean add(UserMusicStarWrapper.UserMusicStar userMusicStar) {

        return this.save(JSON.parseObject(userMusicStar.jsonString(), UserMusicStar.class));
    }

    /**
     * 更新
     * @param userMusicStar UserMusicStarWrapper.UserMusicStar
     * @return Boolean
     */
    @Override
    public Boolean update(UserMusicStarWrapper.UserMusicStar userMusicStar){

        return this.updateById(JSON.parseObject(userMusicStar.jsonString(), UserMusicStar.class));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void star(UserMusicStarWrapper.Star star) {

        this.lambdaUpdate()
            .eq(UserMusicStar::getUserId, star.getUserId())
            .eq(UserMusicStar::getUserMusicId, star.getUserMusicId())
            .eq(UserMusicStar::getClientType, star.getClientType())
            .remove();

        if (star.getStar()) {
            UserMusicStar userMusicStar = new UserMusicStar();
            userMusicStar.setClientType(star.getClientType());
            userMusicStar.setUserId(star.getUserId());
            userMusicStar.setUserMusicId(star.getUserMusicId());
            this.save(userMusicStar);
        }
        userMusicMapper.updateStar(star.getUserMusicId());


    }

    @Override
    public Map<Long, Integer> getStarCountMapByUserMusicIds(List<Long> userMusicIds) {
        if (CollectionUtils.isEmpty(userMusicIds)) {
            return new HashMap<>();
        }
        List<UserMusicStarWrapper.StarCount> starCounts = baseMapper.getStarCountMapByUserMusicIds(userMusicIds);
        Map<Long, Integer> starCountMap = new HashMap<>();
        for (UserMusicStarWrapper.StarCount starCount : starCounts) {
            starCountMap.put(starCount.getUserMusicId(), starCount.getStarCount());
        }
        return starCountMap;
    }

    @Override
    public Map<Long, Boolean> getStarFlagMapByUserMusicIds(List<Long> userMusicIds, String clientType, String userId) {
        if (CollectionUtils.isEmpty(userMusicIds)) {
            return new HashMap<>();
        }
        if (StringUtil.isEmpty(clientType)) {
            return new HashMap<>();
        }
        if (StringUtil.isEmpty(userId)) {
            return new HashMap<>();
        }
        List<UserMusicStar> list = this.lambdaQuery()
            .eq(UserMusicStar::getClientType, clientType)
            .eq(UserMusicStar::getUserId, userId)
            .in(UserMusicStar::getUserMusicId, userMusicIds)
            .list();
        if (CollectionUtils.isEmpty(list)) {
            return new HashMap<>();
        }
        return list.stream().collect(Collectors.toMap(UserMusicStar::getUserMusicId, o->true, (k1, k2) -> k1));
    }
}
