package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.ActivityStudentQueryInfo;
import com.ym.mec.biz.dal.page.DoubleEleven2021QueryInfo;
import com.ym.mec.biz.dal.page.VipGroupActivityQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.string.MessageFormatter;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.SysUserRoleEnum.ORGAN_MANAGER;
import static java.math.BigDecimal.ROUND_UP;
import static java.math.BigDecimal.ZERO;

@Service
public class VipGroupActivityServiceImpl extends BaseServiceImpl<Integer, VipGroupActivity>  implements VipGroupActivityService {

    private static final Logger LOGGER = LoggerFactory.getLogger(VipGroupActivityServiceImpl.class);
	
	@Autowired
	private VipGroupActivityDao vipGroupActivityDao;
	@Autowired
	private ActivityApplyStudentTypeDao activityApplyStudentTypeDao;
	@Autowired
	private TeacherDao teacherDao;
	@Autowired
	private VipGroupCategoryDao vipGroupCategoryDao;
	@Autowired
	private StudentPaymentOrderService studentPaymentOrderService;
	@Autowired
	private ActivityUserMapperService activityUserMapperService;
	@Autowired
	private SysCouponCodeService sysCouponCodeService;
	@Autowired
	private MemberFeeSettingService memberFeeSettingService;
	@Autowired
	private PracticeGroupSellPriceService practiceGroupSellPriceService;
	@Autowired
	private VipGroupDefaultClassesUnitPriceService vipGroupDefaultClassesUnitPriceService;
	@Autowired
	private IdGeneratorService idGeneratorService;
	@Autowired
	private SysUserCashAccountService sysUserCashAccountService;
	@Autowired
	private SysUserCashAccountDetailService sysUserCashAccountDetailService;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private PayService payService;
	@Autowired
	private StudentRegistrationDao studentRegistrationDao;
	@Autowired
	private SysEmployeePositionService employeePositionService;
	@Autowired
	private SysMessageService sysMessageService;
	@Autowired
	private StudentDao studentDao;
	@Autowired
	private TenantConfigService tenantConfigService;
	@Autowired
	private LuckDrawCountService luckDrawCountService;
	@Autowired
	private TempBuyDoubleElevenUserLogDao tempBuyDoubleElevenUserLogDao;
	@Autowired
	private StudentTeacherMapperDao studentTeacherMapperDao;
    @Autowired
    @Lazy
    private ContractService contractService;

	@Override
	public BaseDAO<Integer, VipGroupActivity> getDAO() {
		return vipGroupActivityDao;
	}

	private void checkSettlement(VipGroupSalarySettlementTypeDto settlementTypeDto){
		if(Objects.nonNull(settlementTypeDto) && settlementTypeDto.getSalarySettlementType() != null){
			switch (settlementTypeDto.getSalarySettlementType()){
				case TEACHER_DEFAULT:
					break;
				case FIXED_SALARY:
					if(Objects.isNull(settlementTypeDto.getSettlementValue())){
						throw new BizException("请指定固定课酬");
					}
					break;
				default:
					throw new BizException("错误的课酬结算标准!");
			}
		}
	}

    @Override
    public VipGroupActivityDao getDao() {
        return vipGroupActivityDao;
    }

	@Override
	public PageInfo<VipGroupActivity> queryPage(VipGroupActivityQueryInfo queryInfo){
		PageInfo<VipGroupActivity> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<VipGroupActivity> dataList = null;
		int count = this.findCount(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = this.getDAO().queryPage(params);
			String organIds = dataList.stream().map(e -> e.getOrganId()).filter(e -> StringUtils.isNotEmpty(e)).collect(Collectors.joining(","));
			Set<Integer> organIdList = Arrays.stream(organIds.split(",")).map(e -> Integer.parseInt(e)).collect(Collectors.toSet());
			Map<Integer, String> organNameMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(organIdList)){
				organNameMap = this.getMap("organization", "id_", "name_", organIdList, queryInfo.getTenantId(), Integer.class, String.class);
			}
			for (VipGroupActivity activity : dataList) {
				if(StringUtils.isNotEmpty(activity.getOrganId())){
					StringBuffer sb = new StringBuffer();
					for (String organId : activity.getOrganId().split(",")) {
						String s = organNameMap.get(Integer.parseInt(organId));
						if(StringUtils.isNotEmpty(s)){
							if(sb.length() > 0){
								sb.append(",");
							}
							sb.append(s);
						}
					}
					activity.setOrganName(sb.toString());
				}
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

    @Override
	public VipGroupActivityAddDto addVipGroupActivity(VipGroupActivityAddDto vipGroupActivityAddDto) {

		if(Objects.isNull(vipGroupActivityAddDto.getOrganId())){
			throw new BizException("请指定部门!");
		}
		VipGroupSalarySettlementDto vipGroupSalarySettlement = vipGroupActivityAddDto.getVipGroupSalarySettlement();
		if(vipGroupSalarySettlement != null){
			checkSettlement(vipGroupSalarySettlement.getOfflineSalarySettlement());
			checkSettlement(vipGroupSalarySettlement.getGiveVipOfflineSalarySettlement());
			checkSettlement(vipGroupSalarySettlement.getPracticeSalarySettlement());
			checkSettlement(vipGroupSalarySettlement.getGivePracticeSalarySettlement());
		}
		vipGroupActivityAddDto.setSalarySettlementJson(JSON.toJSONString(vipGroupActivityAddDto.getVipGroupSalarySettlement()));
		ActivityApplyStudentType applyToStudentType = getApplyToStudentType(vipGroupActivityAddDto.getApplyToStudentType());
//		vipGroupActivityAddDto.setApplyToStudentType(JSONObject.toJSONString(applyToStudentType));
		super.insert(vipGroupActivityAddDto);
		applyToStudentType.setActivityId(vipGroupActivityAddDto.getId());
		activityApplyStudentTypeDao.insert(applyToStudentType);
		return vipGroupActivityAddDto;
	}

	private ActivityApplyStudentType getApplyToStudentType(String applyToStudentType){
		ActivityApplyStudentType activityApplyStudentType = new ActivityApplyStudentType();
		if(StringUtils.isNotEmpty(applyToStudentType)){
			List<Integer> collect = Arrays.stream(applyToStudentType.split(",")).mapToInt(Integer::valueOf).boxed().collect(Collectors.toList());
			//0：老用户；1：新用户，2会员，3非会员
			if(collect.contains(0) && collect.contains(1)){
				activityApplyStudentType.setNewStudent(-1);
			}else {
				if(collect.contains(0)){
					activityApplyStudentType.setNewStudent(0);
				}
				if(collect.contains(1)){
					activityApplyStudentType.setNewStudent(1);
				}
			}
			if(collect.contains(2) && collect.contains(3)){
				activityApplyStudentType.setMemberFlag(-1);
			}else {
				if(collect.contains(2)){
					activityApplyStudentType.setMemberFlag(1);
				}
				if(collect.contains(3)){
					activityApplyStudentType.setMemberFlag(0);
				}
			}
		}
		return activityApplyStudentType;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public VipGroupActivityAddDto updateVipGroupActivity(VipGroupActivityAddDto vipGroupActivityAddDto, Integer operatorId) {
		VipGroupActivity oldVipGroupActivity = vipGroupActivityDao.get(vipGroupActivityAddDto.getId());
		if(Objects.isNull(oldVipGroupActivity)){
			throw new BizException("活动不存在!");
		}
		if("PROGRESS".equals(oldVipGroupActivity.getStatus())){
			oldVipGroupActivity.setOrganId(vipGroupActivityAddDto.getOrganId());
			oldVipGroupActivity.setCoursesStartTime(vipGroupActivityAddDto.getCoursesStartTime());
			oldVipGroupActivity.setCoursesEndTime(vipGroupActivityAddDto.getCoursesEndTime());
			oldVipGroupActivity.setStartTime(vipGroupActivityAddDto.getStartTime());
			oldVipGroupActivity.setEndTime(vipGroupActivityAddDto.getEndTime());
			oldVipGroupActivity.setApplyToStudentType(vipGroupActivityAddDto.getApplyToStudentType());
			oldVipGroupActivity.setStudentMaxUsedTimes(vipGroupActivityAddDto.getStudentMaxUsedTimes());
			super.update(oldVipGroupActivity);
		}else {
			VipGroupSalarySettlementDto vipGroupSalarySettlement = vipGroupActivityAddDto.getVipGroupSalarySettlement();
			if(vipGroupSalarySettlement != null){
				checkSettlement(vipGroupSalarySettlement.getOfflineSalarySettlement());
				checkSettlement(vipGroupSalarySettlement.getGiveVipOfflineSalarySettlement());
				checkSettlement(vipGroupSalarySettlement.getPracticeSalarySettlement());
				checkSettlement(vipGroupSalarySettlement.getGivePracticeSalarySettlement());
			}
			vipGroupActivityAddDto.setSalarySettlementJson(JSON.toJSONString(vipGroupActivityAddDto.getVipGroupSalarySettlement()));
			super.update(vipGroupActivityAddDto);
		}
		ActivityApplyStudentType applyToStudentType = getApplyToStudentType(vipGroupActivityAddDto.getApplyToStudentType());
		applyToStudentType.setActivityId(vipGroupActivityAddDto.getId());
		activityApplyStudentTypeDao.updateByActivity(applyToStudentType);
		return vipGroupActivityAddDto;
	}

	@Override
	public List<VipGroupActivity> findByVipGroupCategory(ActivityQueryDto activityQueryDto) {
		if(activityQueryDto.getTeacherId() != null){
			Teacher teacher = teacherDao.get(activityQueryDto.getTeacherId());
			if(Objects.isNull(teacher)){
				throw new BizException("未找到指定教师");
			}
			String organId = activityQueryDto.getOrganId();
			if(StringUtils.isBlank(organId)){
				organId = teacher.getTeacherOrganId().toString();
				if (StringUtils.isNotEmpty(teacher.getFlowOrganRange())) {
					organId += "," + teacher.getFlowOrganRange();
				}
				activityQueryDto.setOrganId(organId);
			}
		}
		List<VipGroupActivity> activityList = vipGroupActivityDao.findByCategory(activityQueryDto);
		if(!CollectionUtils.isEmpty(activityList) && activityQueryDto.getActivityChannel().equals(3)){
			Integer organId = Integer.parseInt(activityQueryDto.getOrganId());
			BigDecimal buyNum = new BigDecimal(1);
			activityList.stream().forEach(e->e.setSinglePrice(this.getActivityPrice(e,organId,buyNum)));
		}
		return activityList;
	}

	@Override
	public BigDecimal getActivityPrice(VipGroupActivity vipGroupActivity, Integer organId, BigDecimal buyNum){
		BigDecimal actualAmount;
		if(vipGroupActivity.getActivityType().equals(1)){
			//获取团练宝价格
			actualAmount = memberFeeSettingService.getAmount(organId,1,
					vipGroupActivity.getPeriodEnum(),buyNum,
					MemberFeeSettingServiceImpl::getCurrentFee);
		}else {
			if (StringUtils.isEmpty(vipGroupActivity.getVipGroupCategoryIdList())){
				//获取分部网管课价格
				actualAmount = practiceGroupSellPriceService.getAmount(organId,buyNum);
			}else {
				//获取分部vip课价格
				actualAmount = vipGroupDefaultClassesUnitPriceService.getAmount(Integer.parseInt(vipGroupActivity.getVipGroupCategoryIdList()),organId,buyNum);
			}
		}
		return actualAmount;
	}

	@Override
	public List<ExportVipGroupActivityDto> exportVipGroupActivity(VipGroupActivityQueryInfo queryInfo) {
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);
		return vipGroupActivityDao.exportVipGroupActivity(params);
	}

    @Override
    public List<WaitCourseActivityDto> queryWaitCourseActivity(Integer teacherId,String organId) {
		return vipGroupActivityDao.queryWaitCourseActivity(teacherId,organId, TenantContextHolder.getTenantId());
    }

    @Override
    public ActivityWaitCourseStudentNumDto getActivityWaitCourseStudentNum(Integer userId, Integer activityId) {
		ActivityWaitCourseStudentNumDto activityWaitCourseStudentNum = vipGroupActivityDao.getActivityWaitCourseStudentNum(userId, activityId);
		VipGroupActivity activity = vipGroupActivityDao.get(activityId);
		if(StringUtils.isNotEmpty(activity.getVipGroupCategoryIdList())){
			VipGroupCategory category = vipGroupCategoryDao.get(Integer.parseInt(activity.getVipGroupCategoryIdList().split(",")[0]));
			activity.setVipGroupCategoryNames(category.getName());
			activity.setVipGroupCategoryNum(category.getStudentNum());
		}
		if(StringUtils.isNotEmpty(activity.getGiveCategoryId())){
			VipGroupCategory category = vipGroupCategoryDao.get(Integer.parseInt(activity.getGiveCategoryId()));
			activity.setGiveCategoryName(category.getName());
			activity.setGiveCategoryNum(category.getStudentNum());
		}
		activityWaitCourseStudentNum.setVipGroupActivity(activity);
		return activityWaitCourseStudentNum;
    }

	@Override
	public PageInfo<ActivityWaitCourseStudentNumDto> getActivityStudentCanCourseNum(ActivityStudentQueryInfo queryInfo) {
		PageInfo<ActivityWaitCourseStudentNumDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<ActivityWaitCourseStudentNumDto> dataList = null;
		int count = vipGroupActivityDao.countActivityStudentCanCourseNum(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupActivityDao.getActivityStudentCanCourseNum(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	@Override
	public List<DoubleEleven2021Dto> organDoubleEleven2022Statis(OrganDoubleEleven2021StatisDto queryInfo) {
		List<DoubleEleven2021Dto> doubleEleven2022Dtos = vipGroupActivityDao.countOrganDoubleEleven2022Statis(queryInfo.getOrganId());
		return this.sumDoubleEleven(doubleEleven2022Dtos,queryInfo,
				vipGroupActivityDao::countOrgan2022ActivityStatis,
				null);
	}

	@Override
	public DoubleEleven2021Dto doubleEleven2022Statis(String organId) {
		DoubleEleven2021Dto doubleEleven2022Dto = vipGroupActivityDao.countDoubleEleven2022Statis(organId);
		if(doubleEleven2022Dto.getTotalBuyAmount().compareTo(ZERO) > 0){
			doubleEleven2022Dto.setAvgBuyAmount(
					doubleEleven2022Dto.getTotalBuyAmount().divide(new BigDecimal(doubleEleven2022Dto.getTotalBuyNum()),2,BigDecimal.ROUND_CEILING));
		}
		//获取活动成交数据
		this.groupTypeConvert(vipGroupActivityDao.count2022ActivityStatis(organId),doubleEleven2022Dto);
		//获取赠送会员数
		Integer giveMemberNum = tempBuyDoubleElevenUserLogDao.countGiveNum(null);
		doubleEleven2022Dto.setGiveMemberNum(giveMemberNum==null?0l:giveMemberNum.longValue());
		return doubleEleven2022Dto;
	}

	@Override
    public List<DoubleEleven2021Dto> organDoubleEleven2021Statis(OrganDoubleEleven2021StatisDto statisDto) {
		List<DoubleEleven2021Dto> doubleEleven2021Dtos = vipGroupActivityDao.countOrganDoubleEleven2021Statis(statisDto.getOrganId());
		return this.sumDoubleEleven(doubleEleven2021Dtos,statisDto,
				vipGroupActivityDao::countOrganActivityStatis,
				vipGroupActivityDao::countOrganGiveMemberNum);
    }

	private List<DoubleEleven2021Dto> sumDoubleEleven(List<DoubleEleven2021Dto> doubleEleven2021Dtos,
													  OrganDoubleEleven2021StatisDto statisDto,
													  Function<List<Integer>,List<DoubleEleven2021ActivityDto>> funOne,
													  Function<List<Integer>,List<Map<Long,Long>>> funTwo){
		if(!CollectionUtils.isEmpty(doubleEleven2021Dtos)){
			String order = statisDto.getOrder();
			String sort = statisDto.getSort();
			List<Integer> organIdList = doubleEleven2021Dtos.stream().map(e -> e.getOrganId()).collect(Collectors.toList());

			List<DoubleEleven2021ActivityDto> activityDtoList = funOne.apply(organIdList);;
			Map<Integer, List<DoubleEleven2021ActivityDto>> collect = new HashMap<>();
			if(activityDtoList != null && activityDtoList.size() > 0){
				collect = activityDtoList.stream().collect(Collectors.groupingBy(DoubleEleven2021ActivityDto::getOrganId));
			}
			Map<Long, Long> maps = new HashMap<>();
			if(funTwo != null){
				maps = MapUtil.convertIntegerMap(funTwo.apply(organIdList));
			}
			for (DoubleEleven2021Dto doubleEleven2021Dto : doubleEleven2021Dtos) {
				if(doubleEleven2021Dto.getTotalBuyAmount().compareTo(ZERO) > 0){
					doubleEleven2021Dto.setAvgBuyAmount(
							doubleEleven2021Dto.getTotalBuyAmount().divide(new BigDecimal(doubleEleven2021Dto.getTotalBuyNum()),2,BigDecimal.ROUND_CEILING));
				}
				List<DoubleEleven2021ActivityDto> activityDtos = collect.get(doubleEleven2021Dto.getOrganId());
				if(activityDtos != null && activityDtos.size() > 0){
					for (DoubleEleven2021ActivityDto activityDto : activityDtos) {
						if(activityDto.getType() == 1){
							doubleEleven2021Dto.setVip1V120Amount(activityDto.getBuyAmount());
							doubleEleven2021Dto.setVip1V120Num(activityDto.getBuyNum());
						}else if (activityDto.getType() == 2){
							doubleEleven2021Dto.setVip1V140Amount(activityDto.getBuyAmount());
							doubleEleven2021Dto.setVip1V140Num(activityDto.getBuyNum());
						}else if (activityDto.getType() == 3){
							doubleEleven2021Dto.setVip1V220Amount(activityDto.getBuyAmount());
							doubleEleven2021Dto.setVip1V220Num(activityDto.getBuyNum());
						}else if (activityDto.getType() == 4){
							doubleEleven2021Dto.setVip1V240Amount(activityDto.getBuyAmount());
							doubleEleven2021Dto.setVip1V240Num(activityDto.getBuyNum());
						}else if (activityDto.getType() == 5){
							doubleEleven2021Dto.setMusicTheoryAmount(activityDto.getBuyAmount());
							doubleEleven2021Dto.setMusicTheoryNum(activityDto.getBuyNum());
						}
					}
				}
				Long giveNum = maps.get(doubleEleven2021Dto.getOrganId().longValue());
				doubleEleven2021Dto.setGiveMemberNum(giveNum == null?0l:giveNum);
			}
			if(sort.equals("totalBuyAmount")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getTotalBuyAmount)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getTotalBuyAmount).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("totalBuyNum")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getTotalBuyNum)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getTotalBuyNum).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("avgBuyAmount")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getAvgBuyAmount)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getAvgBuyAmount).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("giveMemberNum")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getGiveMemberNum)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getGiveMemberNum).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v120Amount")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V120Amount)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V120Amount).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v140Amount")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V140Amount)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V140Amount).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v220Amount")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V220Amount)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V220Amount).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v240Amount")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V240Amount)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V240Amount).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("musicTheoryAmount")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getMusicTheoryAmount)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getMusicTheoryAmount).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v120Num")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V120Num)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V120Num).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v140Num")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V140Num)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V140Num).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v220Num")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V220Num)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V220Num).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("vip1v240Num")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V240Num)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getVip1V240Num).reversed()).collect(Collectors.toList());
				}
			}else if(sort.equals("musicTheoryNum")){
				if(order.equals("ASC")){
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getMusicTheoryNum)).collect(Collectors.toList());
				}else {
					doubleEleven2021Dtos = doubleEleven2021Dtos.stream().sorted(Comparator.comparing(DoubleEleven2021Dto::getMusicTheoryNum).reversed()).collect(Collectors.toList());
				}
			}
		}
		return doubleEleven2021Dtos;
	}

	@Override
	public DoubleEleven2021Dto doubleEleven2021Statis() {
		//获取总成交数据
		DoubleEleven2021Dto doubleEleven2021Dto = vipGroupActivityDao.countDoubleEleven2021Statis();
		if(doubleEleven2021Dto.getTotalBuyAmount().compareTo(ZERO) > 0){
			doubleEleven2021Dto.setAvgBuyAmount(
					doubleEleven2021Dto.getTotalBuyAmount().divide(new BigDecimal(doubleEleven2021Dto.getTotalBuyNum()),2,BigDecimal.ROUND_CEILING));
		}
		//获取活动成交数据
		this.groupTypeConvert(vipGroupActivityDao.countActivityStatis(),doubleEleven2021Dto);
		//获取赠送会员数
		Integer giveMemberNum = vipGroupActivityDao.countGiveMemberNum();
		doubleEleven2021Dto.setGiveMemberNum(giveMemberNum==null?0l:giveMemberNum.longValue());
		return doubleEleven2021Dto;
	}

	private void groupTypeConvert(List<DoubleEleven2021ActivityDto> activityDtos,DoubleEleven2021Dto doubleEleven2021Dto){
		if(!CollectionUtils.isEmpty(activityDtos)){
			for (DoubleEleven2021ActivityDto activityDto : activityDtos) {
				if(activityDto.getType() == 1){
					doubleEleven2021Dto.setVip1V120Amount(activityDto.getBuyAmount());
					doubleEleven2021Dto.setVip1V120Num(activityDto.getBuyNum());
				}else if (activityDto.getType() == 2){
					doubleEleven2021Dto.setVip1V140Amount(activityDto.getBuyAmount());
					doubleEleven2021Dto.setVip1V140Num(activityDto.getBuyNum());
				}else if (activityDto.getType() == 3){
					doubleEleven2021Dto.setVip1V220Amount(activityDto.getBuyAmount());
					doubleEleven2021Dto.setVip1V220Num(activityDto.getBuyNum());
				}else if (activityDto.getType() == 4){
					doubleEleven2021Dto.setVip1V240Amount(activityDto.getBuyAmount());
					doubleEleven2021Dto.setVip1V240Num(activityDto.getBuyNum());
				}else if (activityDto.getType() == 5){
					doubleEleven2021Dto.setMusicTheoryAmount(activityDto.getBuyAmount());
					doubleEleven2021Dto.setMusicTheoryNum(activityDto.getBuyNum());
				}
			}
		}
	}

	@Override
	public PageInfo<DoubleEleven2021OrderDetailDto> doubleEleven2021OrderDetail(DoubleEleven2021QueryInfo queryInfo) {
		PageInfo<DoubleEleven2021OrderDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<String, Object>();
		MapUtil.populateMap(params, queryInfo);

		List<DoubleEleven2021OrderDetailDto> dataList = null;
		int count = vipGroupActivityDao.countDoubleEleven2021OrderDetail(params);
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			dataList = vipGroupActivityDao.queryDoubleEleven2021OrderDetail(params);
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}
		pageInfo.setRows(dataList);
		return pageInfo;
	}

	private VipGroupActivity checkActivityBuy(ActivityPayParamDto payParam){
		VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(payParam.getVipGroupActivityId());
		if (Objects.nonNull(vipGroupActivity.getStudentMaxUsedTimes()) && vipGroupActivity.getStudentMaxUsedTimes() != -1) {
			int useNum = activityUserMapperService.countActivityBuyNum(vipGroupActivity.getId(), payParam.getUserId());
			if (useNum >= vipGroupActivity.getStudentMaxUsedTimes()) {
				throw new BizException("您已超过该活动购买限制{}次", vipGroupActivity.getStudentMaxUsedTimes());
			}
		}
		if (Objects.nonNull(vipGroupActivity.getMinCourseNum())
				&& vipGroupActivity.getMinCourseNum() != -1 && Objects.nonNull(vipGroupActivity.getMaxCourseNum())
				&& vipGroupActivity.getMaxCourseNum() != -1) {
			if (payParam.getBuyNum().compareTo(vipGroupActivity.getMinCourseNum()) < 0 || payParam.getBuyNum().compareTo(vipGroupActivity.getMaxCourseNum()) > 0) {
				throw new BizException("该活动课时数为{}节~{}节", vipGroupActivity.getMinCourseNum(), vipGroupActivity.getMaxCourseNum());
			}
		}
		return vipGroupActivity;
	}

    @Override
	@Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
    public HttpResponseResult buy(ActivityPayParamDto payParam) throws Exception {
		//校验购买资格
		VipGroupActivity vipGroupActivity = this.checkActivityBuy(payParam);
		//判断购买类型
		OrderTypeEnum type;
		CouponDetailTypeEnum couponType;
		if(vipGroupActivity.getActivityType().equals(1)){
			type = OrderTypeEnum.MEMBER;
			couponType = CouponDetailTypeEnum.MEMBER;
		}else {
			if (StringUtils.isEmpty(vipGroupActivity.getVipGroupCategoryIdList())){
				type = OrderTypeEnum.PRACTICE_GROUP_BUY;
				couponType = CouponDetailTypeEnum.PRACTICE;
			}else {
				type = OrderTypeEnum.SMALL_CLASS_TO_BUY;
				couponType = CouponDetailTypeEnum.VIP;
			}
		}
		//计算支付金额
		BigDecimal actualAmount;
		if(vipGroupActivity.getFixedCourseNumFlag()){
			actualAmount = vipGroupActivity.getMarketPrice();
		}else {
			actualAmount = this.getActivityPrice(vipGroupActivity,payParam.getOrganId(),new BigDecimal(1));
			actualAmount = actualAmount.multiply(vipGroupActivity.getDiscount()).divide(new BigDecimal(100), 0, ROUND_UP);
			actualAmount = actualAmount.multiply(new BigDecimal(payParam.getBuyNum()));
		}
		// 判断是否存在支付中的记录
		List<StudentPaymentOrder> list = studentPaymentOrderService
				.queryByCondition(GroupType.ACTIVITY,
						payParam.getVipGroupActivityId().toString(),
						payParam.getUserId(),
						DealStatusEnum.ING,type);

		if (list.size() > 0) {
			StudentPaymentOrder applyOrder = list.get(list.size() - 1);
			//校验重复支付
			HttpResponseResult result = studentPaymentOrderService.checkRepeatPay(applyOrder, payParam.getRepeatPay());
			if (result.getCode() != 200) {
				return result;
			}
		}

		//校验优惠券
		String[] checkCoupon = CouponDetailTypeEnum.getAllowType(couponType);
		StudentPaymentOrder studentPaymentOrder = sysCouponCodeService.use(payParam.getCouponIdList(), actualAmount, true, checkCoupon);
		if(payParam.getAmount().compareTo(studentPaymentOrder.getActualAmount()) != 0){
			throw new BizException("非法请求");
		}
		studentPaymentOrder.setUserId(payParam.getUserId());
		studentPaymentOrder.setGroupType(GroupType.ACTIVITY);
		String orderNo = idGeneratorService.generatorId("payment") + "";
		studentPaymentOrder.setOrderNo(orderNo);
		studentPaymentOrder.setStatus(DealStatusEnum.ING);
		studentPaymentOrder.setType(type);
		studentPaymentOrder.setMusicGroupId(payParam.getVipGroupActivityId().toString());
		studentPaymentOrder.setOrganId(payParam.getOrganId());
		studentPaymentOrder.setRoutingOrganId(payParam.getOrganId());
		studentPaymentOrder.setVersion(0);
		studentPaymentOrder.setActivityBuyNum(payParam.getBuyNum());
		studentPaymentOrder.setActivityId(payParam.getVipGroupActivityId().toString());
		studentPaymentOrder.setMemo("学员活动购买");
		studentPaymentOrderService.insert(studentPaymentOrder);

		//余额消耗
		Boolean success = sysUserCashAccountService.use(payParam,studentPaymentOrder);
		if(success){
			studentPaymentOrder.setPayTime(new Date());
			this.orderCallback(studentPaymentOrder);
			Map<String, Object> result = new HashMap<>();
			result.put("orderNo", studentPaymentOrder.getOrderNo());
			return BaseController.succeed(result);
		}

		BigDecimal amount = studentPaymentOrder.getActualAmount();
		String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
		//分类费用 course，instrument，accessories，other
		Map<String, BigDecimal> classFee = new HashMap<>();
		classFee.put("course", amount);
		classFee.put("instrument", ZERO);
		classFee.put("accessories", ZERO);
		classFee.put("other", ZERO);
		Map<String, Object> payMap = payService.getPayMap(
				amount,
				studentPaymentOrder.getBalancePaymentAmount(),
				orderNo,
				baseApiUrl + "/api-student/studentOrder/paymentResult?orderNo=" + orderNo,
				"学员活动购买",
				vipGroupActivity.getName(),
				payParam.getOrganId(),
				"activityBuy"
		);

		studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
		studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
		studentPaymentOrderService.update(studentPaymentOrder);
		return BaseController.succeed(payMap);
    }


	@Override
	@Transactional(rollbackFor = Exception.class, isolation = Isolation.READ_COMMITTED)
	public void orderCallback(StudentPaymentOrder studentPaymentOrder) {
		//更新订单状态
		int updateCount = studentPaymentOrderService.update(studentPaymentOrder);
		if (updateCount <= 0) {
			throw new BizException("订单更新失败");
		}
		if (studentPaymentOrder.getStatus() == DealStatusEnum.SUCCESS) {
			//处理双十一活动特殊赠送
			boolean elevenFlag = this.doubleElevenGive(studentPaymentOrder);
			ActivityUserMapper activityUserMapper = null;
			if(!elevenFlag){
				activityUserMapper = studentPaymentOrderService.activityGive(Integer.parseInt(studentPaymentOrder.getActivityId()), studentPaymentOrder,
						null, null);
			}
			//保存账户信息
			sysUserCashAccountDetailService.saveSysUserCashAccountDetail(studentPaymentOrder,studentPaymentOrder.getMemo());
            try {
                contractService.transferVipGroupCoursesContract(studentPaymentOrder.getUserId(), null, studentPaymentOrder.getType());
            } catch (Exception e) {
                LOGGER.error(MessageFormatter.arrayFormat("小课活动[{}]购买协议错误:{}", studentPaymentOrder.getActivityId(), e.getMessage()), e);
            }
			//发送短信
			this.sendSuccessMsg(studentPaymentOrder,activityUserMapper);
		}else {
			if (Objects.nonNull(studentPaymentOrder.getBalancePaymentAmount()) && studentPaymentOrder.getBalancePaymentAmount().compareTo(ZERO) > 0) {
				sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(), PlatformCashAccountDetailTypeEnum.REFUNDS, studentPaymentOrder.getMemo() + "支付失败，退还余额");
			}
			sysCouponCodeService.quit(studentPaymentOrder.getCouponCodeId());
			return;
		}
	}

	//处理双十一活动特殊赠送
	private boolean doubleElevenGive(StudentPaymentOrder studentPaymentOrder){
		String[] split = studentPaymentOrder.getActivityId().split(",");
		String activeConfig = sysConfigDao.findConfigValue("2022_double_eleven_give_config");
		String giveStartTime = sysConfigDao.findConfigValue("2021_double_eleven_give_member_start_time");
		String giveEndTime = sysConfigDao.findConfigValue("2021_double_eleven_give_member_end_time");
		Boolean doubleElevenFlag = false;
		for (String activityId : split) {
			List<SysConfigJsonDto> cloud2022ActivityDtos = JSONArray.parseArray(activeConfig, SysConfigJsonDto.class);
			List<SysConfigJsonDto> activityDtos = cloud2022ActivityDtos.stream().
					filter(e -> Arrays.stream(e.getKey().split(",")).collect(Collectors.toList()).
							contains(activityId)).collect(Collectors.toList());
			if(!CollectionUtils.isEmpty(activityDtos)){
				doubleElevenFlag = true;
				SysConfigJsonDto jsonDto = activityDtos.get(0);
				//处理双十一活动特殊处理
				Date now = new Date();
				TempBuyDoubleElevenUserLog doubleElevenUserLog = new TempBuyDoubleElevenUserLog();
				doubleElevenUserLog.setOrderId(studentPaymentOrder.getId());
				doubleElevenUserLog.setUserId(studentPaymentOrder.getUserId());
				doubleElevenUserLog.setCreateTime(now);
				doubleElevenUserLog.setOrganId(studentPaymentOrder.getOrganId());
				if(DateUtil.contents(giveStartTime,giveEndTime,now)){
					if(studentPaymentOrder.getOrganId().equals(4)){
						//赠送抽奖机会
						luckDrawCountService.freeGive(studentPaymentOrder.getUserId().longValue(),Integer.parseInt(jsonDto.getValue()),4);
					}else {
						luckDrawCountService.freeGive(studentPaymentOrder.getUserId().longValue(),Integer.parseInt(jsonDto.getValue()),3);
						Integer giveNum = tempBuyDoubleElevenUserLogDao.countGiveNum(studentPaymentOrder.getUserId());
						if(giveNum == null || giveNum == 0){
							//符合赠送条件，赠送乐保套装
							doubleElevenUserLog.setGiveFlag(true);
						}
					}
				}
				tempBuyDoubleElevenUserLogDao.insert(doubleElevenUserLog);

			}
		}
		if (doubleElevenFlag){
			//处理活动特殊排课资格
			activityUserMapperService.saveToOrder(studentPaymentOrder);
		}
		return doubleElevenFlag;
	}

    @Override
    public void setActivityType(VipGroupActivity activity) {
		//如果是会员活动
		if(activity.getActivityType().equals(1)){
			activity.setActivityTypeEnum("MEMBER");
		}else {
			if(StringUtils.isEmpty(activity.getVipGroupCategoryIdList())){
				activity.setActivityTypeEnum("PRACTICE");
			}else {
				activity.setActivityTypeEnum(activity.getVipGroupCategoryNames());
				/*if(StringUtils.equals(activity.getVipGroupCategoryNames(),"小组课")){
					activity.setActivityTypeEnum("THEORY");
				}else{
					activity.setActivityTypeEnum(activity.getVipGroupCategoryNames());
				}*/
			}
		}
		/*String giveCourseType = activity.getGiveCourseType();
		if(StringUtils.isNotEmpty(giveCourseType)){
			if(StringUtils.equals(giveCourseType,"VIP")){
				if(StringUtils.equals(activity.getGiveCategoryName(),"小组课")){
					activity.setGiveCourseType("THEORY");
				}
			}
		}*/
    }

	@Override
	@Transactional( rollbackFor = Exception.class,isolation = Isolation.READ_COMMITTED)
	public HttpResponseResult buyDoubleEleven(BuyDoubleEleven2022Dto payParam) throws Exception {
		String activityIdList = payParam.getActivityIdList();
		List<VipGroupActivity> vipGroupActivities = vipGroupActivityDao.queryByIds(activityIdList);
		Student student = studentDao.getLocked(payParam.getUserId());
		BigDecimal activityFee = ZERO;
		for (VipGroupActivity activity : vipGroupActivities) {
			//校验活动购买条件
			this.checkActivity(activity,payParam.getUserId(),payParam.getOrganId());
			activityFee = activityFee.add(activity.getMarketPrice());
		}
		// 判断是否存在支付中的记录
		List<StudentPaymentOrder> list = studentPaymentOrderService.queryByCondition(GroupType.ACTIVITY,null,student.getUserId(),DealStatusEnum.ING,OrderTypeEnum.SMALL_CLASS_TO_BUY);
		if (list.size() > 0) {
			StudentPaymentOrder applyOrder = list.get(list.size() - 1);
			//校验重复支付
			HttpResponseResult result = studentPaymentOrderService.checkRepeatPay(applyOrder, payParam.getRepeatPay());
			if (result.getCode() != 200) {
				return result;
			}
		}

		//计算订单金额
		StudentPaymentOrder studentPaymentOrder = sysCouponCodeService.use(payParam.getCouponIdList(), activityFee, true);
		if (studentPaymentOrder.getActualAmount().compareTo(payParam.getAmount()) != 0) {
			throw new BizException("非法请求");
		}
		String orderNo = idGeneratorService.generatorId("payment") + "";
		studentPaymentOrder.setMemo("2022双十一活动购买");
		studentPaymentOrder.setPaymentChannel("BALANCE");
		studentPaymentOrder.setUserId(student.getUserId());
		studentPaymentOrder.setGroupType(GroupType.ACTIVITY);
		studentPaymentOrder.setOrderNo(orderNo);
		studentPaymentOrder.setType(OrderTypeEnum.SMALL_CLASS_TO_BUY);
		studentPaymentOrder.setStatus(DealStatusEnum.ING);
		studentPaymentOrder.setRemitFee(ZERO);
		studentPaymentOrder.setCourseRemitFee(ZERO);
		studentPaymentOrder.setOrganId(payParam.getOrganId());
		studentPaymentOrder.setRoutingOrganId(payParam.getOrganId());
		studentPaymentOrder.setActivityId(activityIdList);
		studentPaymentOrder.setMusicGroupId(activityIdList);
		studentPaymentOrder.setVersion(0);
		studentPaymentOrderService.insert(studentPaymentOrder);

		//消耗余额
		Boolean success = sysUserCashAccountService.use(payParam,studentPaymentOrder);
		if(success){
			this.orderCallback(studentPaymentOrder);
			Map<String, Object> result = new HashMap<>();
			result.put("orderNo", studentPaymentOrder.getOrderNo());
			return BaseController.succeed(result);
		}

		String orderSubject = "2022双十一活动";
		String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
		Map payMap = payService.getPayMap(
				studentPaymentOrder.getActualAmount(),
				studentPaymentOrder.getBalancePaymentAmount(),
				orderNo,
				baseApiUrl + "/api-student/studentOrder/paymentResult?orderNo=" + orderNo,
				orderSubject,
				orderSubject,
				studentPaymentOrder.getOrganId(),
				"vip"
		);

		studentPaymentOrder.setOrganId(studentPaymentOrder.getOrganId());
		studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
		studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
		studentPaymentOrderService.update(studentPaymentOrder);
		return BaseController.succeed(payMap);
	}

	@Override
	public void checkActivity(VipGroupActivity activity,Integer userId,Integer organId){
		//校验分部
		String activityOrganIds = activity.getOrganId();
		if(StringUtils.isNotEmpty(activityOrganIds) && !activityOrganIds.contains(organId.toString())){
			throw new BizException("当前分部暂不支持此活动");
		}
		//校验活动有效时间
		SysConfigService.checkActivityDate(activity.getStartTime(),activity.getEndTime());
		Integer studentMaxUsedTimes = activity.getStudentMaxUsedTimes();
		if(studentMaxUsedTimes != -1){
			//获取活动购买次数
			int activityBuyNum = activityUserMapperService.countActivityBuyNum(activity.getId(),userId);
			if(activityBuyNum >= studentMaxUsedTimes){
				throw new BizException("{}活动最多课购买{}次，感谢您的参与",activity.getName(),studentMaxUsedTimes);
			}
		}
	}

    private void sendSuccessMsg(StudentPaymentOrder studentPaymentOrder,ActivityUserMapper activityUserMapper){
		try {
			Integer userId = studentPaymentOrder.getUserId();
			String[] split = studentPaymentOrder.getActivityId().split(",");
			for (String activityId : split) {
				VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(Integer.parseInt(activityId));
				String activityName = vipGroupActivity.getName();
				SysUser user = teacherDao.getUser(userId);
				String username = StringUtils.isEmpty(user.getUsername()) ? user.getRealName() : user.getUsername();
				mark:if(vipGroupActivity.getActivityType().equals(0)) {
					Map<Integer, String> map = new HashMap<>(1);
					map.put(userId, userId.toString());
					sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.PUSH_STUDENT_BUY_ACTIVITY, map, null, 0, null, "STUDENT",
							activityName);
					if(studentPaymentOrder.getType() == OrderTypeEnum.SMALL_CLASS_TO_BUY
							|| studentPaymentOrder.getType() == OrderTypeEnum.PRACTICE_GROUP_BUY
							|| studentPaymentOrder.getType() == OrderTypeEnum.PRACTICE_GROUP_RENEW){
						List<Integer> teacherIds = studentTeacherMapperDao.queryTeacherIds(userId, studentPaymentOrder.getType() == OrderTypeEnum.SMALL_CLASS_TO_BUY ? "VIP" : "PRACTICE");
						if(!CollectionUtils.isEmpty(teacherIds)){
							Map<Integer, String> teacherMap = new HashMap<>(teacherIds.size());
							teacherIds.stream().forEach(e->teacherMap.put(e,e.toString()));
							sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.PUSH_TEACHER_ACTIVITY_COURSE,
									teacherMap, null, 0, null, "TEACHER", username, activityName);
							break mark;
						}
					}
					//获取乐团主管
					Map<Integer, String> eduMap = MapUtil.convertMybatisMap(studentRegistrationDao.queryStudentEduMap(userId));
					if (CollectionUtils.isEmpty(eduMap)) {
						Set<Integer> integers = employeePositionService.queryUserIdByRoleId(new SysUserRoleEnum[]{ORGAN_MANAGER}, studentPaymentOrder.getOrganId());
						eduMap = new HashMap<>(integers.size());
						if (!CollectionUtils.isEmpty(integers)) {
							for (Integer integer : integers) {
								eduMap.put(integer, integer.toString());
							}
						}
					}
					if (!CollectionUtils.isEmpty(eduMap)) {
						sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.PUSH_SYSTEM_ACTIVITY_COURSE,
								eduMap, null, 0, null, "SYSTEM", username, activityName);
					}
				}else {
					Map<Integer, String> map = new HashMap<>(1);
					map.put(userId, userId.toString());
					sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.PUSH_STUDENT_BUY_MEMBER_ACTIVITY, map, null, 0, null, "STUDENT",
							activityName);
					if(!tenantConfigService.autoActivationFlag(user.getTenantId())){
						//获取乐团主管
						Map<Integer, String> eduMap = MapUtil.convertMybatisMap(studentRegistrationDao.queryStudentEduMap(userId));
						if (CollectionUtils.isEmpty(eduMap)) {
							Set<Integer> integers = employeePositionService.queryUserIdByRoleId(new SysUserRoleEnum[]{ORGAN_MANAGER}, studentPaymentOrder.getOrganId());
							eduMap = new HashMap<>(integers.size());
							if (!CollectionUtils.isEmpty(integers)) {
								for (Integer integer : integers) {
									eduMap.put(integer, integer.toString());
								}
							}
						}
						if (!CollectionUtils.isEmpty(eduMap)) {
							sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.PUSH_SYSTEM_ACTIVITY_MEMBER,
									eduMap, null, 0, null, "SYSTEM", username, activityName);
						}
					}
				}
				mark:if(activityUserMapper != null){
					String teacherType = null;
					Integer giveVipFlag = activityUserMapper.getGiveVipFlag();
					if(giveVipFlag == 1){
						teacherType = "VIP";
					}
					Integer givePracticeFlag = activityUserMapper.getGivePracticeFlag();
					if(givePracticeFlag == 1){
						teacherType = "PRACTICE";
					}
					if(teacherType == null){
						continue;
					}
					if(studentPaymentOrder.getType() == OrderTypeEnum.SMALL_CLASS_TO_BUY
							|| studentPaymentOrder.getType() == OrderTypeEnum.PRACTICE_GROUP_BUY
							|| studentPaymentOrder.getType() == OrderTypeEnum.PRACTICE_GROUP_RENEW){
						List<Integer> teacherIds = studentTeacherMapperDao.queryTeacherIds(userId,teacherType);
						if(!CollectionUtils.isEmpty(teacherIds)){
							Map<Integer, String> teacherMap = new HashMap<>(teacherIds.size());
							teacherIds.stream().forEach(e->teacherMap.put(e,e.toString()));
							sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.PUSH_TEACHER_ACTIVITY_COURSE,
									teacherMap, null, 0, null, "TEACHER", username, activityName);
							break mark;
						}
					}
					//获取乐团主管
					Map<Integer, String> eduMap = MapUtil.convertMybatisMap(studentRegistrationDao.queryStudentEduMap(userId));
					if (CollectionUtils.isEmpty(eduMap)) {
						Set<Integer> integers = employeePositionService.queryUserIdByRoleId(new SysUserRoleEnum[]{ORGAN_MANAGER}, studentPaymentOrder.getOrganId());
						eduMap = new HashMap<>(integers.size());
						if (!CollectionUtils.isEmpty(integers)) {
							for (Integer integer : integers) {
								eduMap.put(integer, integer.toString());
							}
						}
					}
					if (!CollectionUtils.isEmpty(eduMap)) {
						sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.PUSH_SYSTEM_ACTIVITY_COURSE,
								eduMap, null, 0, null, "SYSTEM", username, activityName);
					}
				}
			}
		}catch (Exception e){
			e.printStackTrace();
		}
	}
}
