package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.websocket.WsConnectRecordInfoMapper;
import com.ym.mec.biz.dal.entity.websocket.WsConnectRecordInfo;
import com.ym.mec.biz.dal.entity.websocket.WsConnectRecordInfoExample;
import com.ym.mec.biz.service.WsConnectService;
import org.apache.commons.collections.CollectionUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created by Eric.Shang on 2022/10/31.
 */
@Service
public class WsConnectServiceImpl implements WsConnectService {

    @Autowired
    private WsConnectRecordInfoMapper wsConnectRecordInfoMapper;


    /**
     * 保存用户ws连接记录信息
     *
     * @param sessionId  会放ID
     * @param roomId     房间ID
     * @param userid     用户ID
     * @param clientType 客户端类型
     * @return int
     */
    @Override
    public int saveWsConnectRecordInfo(String sessionId, String roomId, String userid, String clientType) {

        Date date = DateTime.now().toDate();
        // 连接记录信息
        WsConnectRecordInfo record = new WsConnectRecordInfo();
        record.setSessionId(sessionId);
        record.setRoomId(roomId);
        record.setUserId(Long.parseLong(userid));
        record.setClientType(clientType);
        record.setConnectTime(date);
        record.setCreatedTime(date);

        // 保存连接记录信息
        return wsConnectRecordInfoMapper.insertSelective(record);
    }

    /**
     * 更新用户ws连接断开时间
     * @param sessionId  会话ID
     * @param roomId     房间ID
     * @param userId     用户
     * @param clientType 客户端类型
     * @return int
     */
    @Override
    public int updateWsDisconnectRecordInfo(String sessionId, String roomId, String userId, String clientType) {

        // 查询连接记录信息
        WsConnectRecordInfoExample example = new WsConnectRecordInfoExample();
        example.or()
                .andSessionIdEqualTo(sessionId)
                .andRoomIdEqualTo(roomId)
                .andUserIdEqualTo(Long.parseLong(userId))
                .andClientTypeEqualTo(clientType);

        List<WsConnectRecordInfo> recordInfos = wsConnectRecordInfoMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(recordInfos)) {
            return 0;
        }

        // 连接记录信息
        WsConnectRecordInfo record = recordInfos.get(0);

        // 更新数据
        WsConnectRecordInfo recordInfo = new WsConnectRecordInfo();
        recordInfo.setId(record.getId());
        recordInfo.setDisconnectTime(DateTime.now().toDate());
        recordInfo.setOnlineTime(recordInfo.getDisconnectTime().getTime() - record.getConnectTime().getTime());
        // 如果断开时间小于0，则设置为0
        if (recordInfo.getOnlineTime() < 0) {
            recordInfo.setOnlineTime(0L);
        }

        return wsConnectRecordInfoMapper.updateByPrimaryKeySelective(recordInfo);
    }
}
