package com.ym.mec.mall;

import com.ym.mec.common.api.CommonResult;
import com.ym.mec.common.config.FeignConfiguration;
import com.ym.mec.common.dto.*;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.mall.fallback.MallFeignServiceFallback;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@FeignClient(name = "${app-config.open-feign.mall-server.name:mall-server}",
		contextId = "MallFeignService",
		url = "${app-config.open-feign.mall-server.url:}",
		configuration = FeignConfiguration.class, fallback = MallFeignServiceFallback.class)
public interface MallFeignService {

	@PostMapping(value = "open/product/list")
	@ResponseBody
	PageInfo<PmsProductDto> getProductList(@RequestBody PmsProductQueryParamDto productQueryParam);


    // 同步完成订单
    @PostMapping(value = "/open/product/orderCreate", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    CommonResult<Boolean> productOrderCreate(@RequestBody OrderCreate order) ;

    // 同步订单状态，
    /**
     * 查询商品订单状态
     * @param orderNo 订单编号
     * @return status 订单状态  0->待付款；1->待发货；2->已发货；3->已完成；4->已关闭；5->无效订单
     */
    @GetMapping(value = "/open/product/updateOrderStatus", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    CommonResult<Boolean> productUpdateOrderStatus(@RequestParam("orderNo") String orderNo,@RequestParam("status") Integer status);


    //根据品牌名称分页获取品牌列表
    @RequestMapping(value = "/open/brand/list", method = RequestMethod.GET)
    List<BrandDto> getList();


    // 查询所有一级分类及子分类
    @RequestMapping(value = "/open/productCategory/list/withChildren", method = RequestMethod.GET)
    @ResponseBody
    List<ProductCategoryDto> listWithChildren();


//    @ApiOperation("分页获取所有商品属性分类")
    @RequestMapping(value = "/open/productAttribute/category/list", method = RequestMethod.GET)
    List<ProductAttributeCategoryDto> getProductAttributeCategoryList();




    //    @ApiOperation("分页获取所有商品属性分类")
    @RequestMapping(value = "/open/product/returnApply", method = RequestMethod.POST)
    CommonResult<Boolean> returnApply(ReturnApplyDto returnApplyDto);
}
