package com.ym.mec.web;

import com.ym.mec.common.config.FeignConfiguration;
import com.ym.mec.common.dto.SchoolDto;
import com.ym.mec.common.entity.*;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.thirdparty.adapay.entity.BaseResult;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import com.ym.mec.web.fallback.WebFeignServiceFallback;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

@FeignClient(name = "${app-config.open-feign.web-server.name:web-server}",
		contextId = "WebFeignService",
		url = "${app-config.open-feign.web-server.url:}",
		configuration = FeignConfiguration.class, fallback = WebFeignServiceFallback.class)
public interface WebFeignService {

	@RequestMapping(value = "api/createCashAccount")
	Boolean createCashAccount(@RequestParam("userId") Integer userId,@RequestParam("tenantId") Integer tenantId);

	@RequestMapping(value = "api/queryTeacherOrganId")
	Integer queryTeacherOrganId(@RequestParam("userId") Integer userId);

	@RequestMapping(value = "api/isPurchasedPracticeCourse")
	Boolean isPurchasedPracticeCourse(@RequestParam("userId") Integer userId);

	@RequestMapping(value = "api/updateNickName")
	int updateNickName(@RequestParam("userId") Integer userId, @RequestParam("nickName") String nickName);

	@RequestMapping(value = "api/queryOrganIdByUserId")
	String queryOrganIdByUserId(@RequestParam("userId") Integer userId, @RequestParam("clientId") String clientId);

	@RequestMapping(value = "api/getTenantConfigValue")
	String getTenantConfigValue(@RequestParam("tenantId") Integer tenantId, @RequestParam("paramName") String paramName);

	@PostMapping(value = "api/queryVipPracticeGroups")
	Boolean queryVipPracticeGroups();

	@GetMapping(value = "/hfMerchantConfig/queryByTenantId")
	HttpResponseResult<HfMerchantConfig> queryByTenantId(@RequestParam("payerName") String payerName,@RequestParam("id") Integer id);

	@PostMapping("/api/mallCreateOrder")
	HttpResponseResult<Map> mallCreateOrder(@RequestBody MallCreateOrderModel model);

	@PostMapping("/api/executePayment")
	HttpResponseResult executePayment(@RequestBody Map paymentParam);

	@PostMapping("/api/updateOrder")
	HttpResponseResult updateOrder(@RequestBody Map map);

	//商城发货状态同步
	@PostMapping("/api/updateShippedStatus")
	HttpResponseResult updateShippedStatus(@RequestBody List<OrderSkuSync> orderSkuSyncs);

	@PostMapping("/api/cancelOrder")
	HttpResponseResult<OrderCancelModel> cancelOrder(@RequestParam("orderNo") String orderNo,@RequestParam("reason") String reason);

	@PostMapping("/api/mallSaveOrderInfo")
	HttpResponseResult<Map> mallSaveOrderInfo(@RequestBody MallCreateOrderModel model);

	// 商城获取 退款数据
	@GetMapping("/api/getRefundInfo")
	HttpResponseResult<Map<String,String>> getBaseUrl(@RequestParam("orderNo") String orderNo);

	// 商城获取 退款数据
	@PostMapping("/api/refund")
	HttpResponseResult<BaseResult<Map<String, Object>>> refund(@RequestBody RefundModel refundModel);

	// 商城订单优惠券金额
	@PostMapping("/api/couponAmount")
	HttpResponseResult<Map> couponAmount(@RequestBody MallCreateOrderModel model);

	// 商城获取 退款数据  只能全额退款
	@PostMapping("/api/receive")
	HttpResponseResult receive(@RequestParam("orderNo") String orderNo,@RequestParam("autoReceive") boolean autoReceive);

	// 商城获取 收货状态
	@PostMapping("/api/balance")
	HttpResponseResult<BigDecimal> balance(@RequestParam("userId") Long userId);

	// 商城商品状态更新，同步商品状态
	@PostMapping("/api/goodsStatusSynchronize")
	HttpResponseResult<Boolean> goodsStatusSynchronize(@RequestBody List<GoodsSubModel> goodsSubModelList);

	// 商城商品库存更新，同步商品库存
	@PostMapping("/api/goodsStockCountSynchronize")
	HttpResponseResult<Boolean> goodsStockCountSynchronize(@RequestBody List<GoodsSubStockModel> goodsSubStockModels);


    // 获取学校信息
    @PostMapping("/api/schoolListByIds")
    HttpResponseResult<List<SchoolDto>> schoolListByIds(@RequestBody List<Integer> schoolIds);
}
