package com.yonge.log.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.yonge.log.dal.dao.HistoryMessageDao;
import com.yonge.log.dal.model.HistoryMessage;
import com.yonge.log.service.HistoryMessageService;
import com.yonge.mongodb.PageInfo;
import com.yonge.mongodb.dao.BaseDaoWithMongo;
import com.yonge.mongodb.service.impl.BaseServiceImplWithMongo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

@Service
public class HistoryMessageServiceImpl extends BaseServiceImplWithMongo<String, HistoryMessage> implements HistoryMessageService {

    @Autowired
    private HistoryMessageDao historyMessageDao;

    @Autowired
    private MongoTemplate mongoTemplate;

	@Override
	public BaseDaoWithMongo<String, HistoryMessage> getDAO() {
		return historyMessageDao;
	}

    @Override
    public void saveImHistoryMessage(File file) throws Exception {
//		long length = file.length();
//		if(length > 1048576){
        doNioReadFile(file);
//		}else {
//			doBufferReadFile(file);
//		}
    }

    @Override
    public long selectCount(int status, Date date, int targetType) {
        Query query = new Query();

        // 查询status 为null or 0
        // 查询时间大于date

        // date 转 LocalDateTime
        LocalDateTime ldt = LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault());

        String dateFormat = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"));


        Criteria criteria = Criteria.where("dateTime").gt(dateFormat).orOperator(Criteria.where("status").is(0),Criteria.where("status").is(null));
        criteria.and("targetType").is(targetType);
        criteria.and("classname").in("RC:ImgMsg", "RC:GIFMsg", "RC:HQVCMsg", "RC:FileMsg", "RC:SightMsg", "RC:LBSMsg", "RC:ImgTextMsg","RC:TxtMsg");
        query.addCriteria(criteria);
        return mongoTemplate.count(query,HistoryMessage.class);
    }

    @Override
    public PageInfo<HistoryMessage> selectPage(int page, int size, int status, Date date, int targetType) {
        PageInfo<HistoryMessage> pageInfo = new PageInfo<HistoryMessage>(page, size);

        // date 转 LocalDateTime
        LocalDateTime ldt = LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault());

        String dateFormat = ldt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS"));
        List<HistoryMessage> dataList = null;
        long total = this.selectCount(status,date, targetType);
        if (total > 0) {
            pageInfo.setTotal(total);

            Query query = new Query();
            Criteria criteria = Criteria.where("dateTime").gt(dateFormat).orOperator(Criteria.where("status").is(0),Criteria.where("status").is(null));
            criteria.and("targetType").is(targetType);
            criteria.and("classname").in("RC:ImgMsg", "RC:GIFMsg", "RC:HQVCMsg", "RC:FileMsg", "RC:SightMsg", "RC:LBSMsg", "RC:ImgTextMsg","RC:TxtMsg");
            query.addCriteria(criteria);
            query.skip(pageInfo.getOffset()).limit(pageInfo.getLimit());
            dataList = mongoTemplate.find(query, HistoryMessage.class);
        }
        if (total == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public void updateStatus(String id, Integer status) {
        Criteria criteria = Criteria.where("_id").is(id);
        Query query = new Query(criteria);

        Update update = new Update();
        update.set("status", status); // 设置新的 status 值

        mongoTemplate.updateMulti(query, update, HistoryMessage.class);
    }

    @Override
    public void updateAllStatus(int status) {

        Query query = new Query();

        Update update = new Update();
        update.set("status", status); // 设置新的 status 值

        mongoTemplate.updateMulti(query, update, HistoryMessage.class);
    }

    @Override
    public long querySyncCount() {
        Query query = new Query();

        Criteria criteria = new Criteria().orOperator(Criteria.where("syncFlag").is(0),Criteria.where("syncFlag").is(null));
        query.addCriteria(criteria);
        return mongoTemplate.count(query,HistoryMessage.class);
    }

    @Override
    public PageInfo<HistoryMessage> getImToTencent(long page, int size) {
        PageInfo<HistoryMessage> pageInfo = new PageInfo<HistoryMessage>(Long.valueOf(page).intValue(), size);

        long total = this.querySyncCount();
        List<HistoryMessage> dataList = null;
        if (total > 0) {
            pageInfo.setTotal(total);

            Query query = new Query();
            Criteria criteria = new Criteria().orOperator(Criteria.where("syncFlag").is(0),Criteria.where("syncFlag").is(null));
            query.addCriteria(criteria);
            query.skip(pageInfo.getOffset()).limit(pageInfo.getLimit());
            dataList = mongoTemplate.find(query, HistoryMessage.class);
        }
        if (total == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public void updateSyncFlag(String id, int syncFlag) {

        Criteria criteria = Criteria.where("_id").is(id);
        Query query = new Query(criteria);

        Update update = new Update();
        update.set("syncFlag", syncFlag); // 设置新的 status 值

        mongoTemplate.updateMulti(query, update, HistoryMessage.class);
    }

    @Override
    public void updateMsg(String msgUID, Integer msgSeq, Integer msgRandom, Long msgTimeStamp) {

        Criteria criteria = Criteria.where("_id").is(msgUID);
        Query query = new Query(criteria);

        Update update = new Update();
        update.set("msgSeq", msgSeq); // 设置新的 status 值
        update.set("msgRandom", msgRandom); // 设置新的 status 值
        update.set("msgTimeStamp", msgTimeStamp); // 设置新的 status 值

        mongoTemplate.updateMulti(query, update, HistoryMessage.class);
    }


//	public static void main(String[] args) throws Exception {
//		doNioReadFile1(new File("/Users/chenxiaoyu/Documents/77fe9ce6-7d91-4568-afe3-9e8ac351e87f.zip"));
//	}

    public void doNioReadFile(File file) {
        ZipInputStream zin = null;
        try {
            zin = new ZipInputStream(new FileInputStream(file), StandardCharsets.UTF_8);
            ZipFile zf = new ZipFile(file);
            ZipEntry ze;
            Set<HistoryMessage> historyMessages = new HashSet<>();
            while ((ze = zin.getNextEntry()) != null) {
                BufferedReader br = new BufferedReader(new InputStreamReader(zf.getInputStream(ze)));
                String line;
                while ((line = br.readLine()) != null) {
                    try {
                        historyMessages.add(JSONObject.parseObject(line.substring(line.indexOf("{")), HistoryMessage.class));
//						if(historyMessages.size() >= 2000){
//							historyMessageDao.batchInsert(new ArrayList<>(historyMessages),HistoryMessage.class);
//							historyMessages.clear();
//						}
                    }catch (Exception e){
                        e.printStackTrace();
                    }
                }
                br.close();
                break;
            }
            if(historyMessages.size() > 0){
                historyMessageDao.batchInsert(historyMessages.stream().sorted(Comparator.comparing(HistoryMessage::getDateTime)).collect(Collectors.toList()),HistoryMessage.class);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (zin != null) {
                try {
                    zin.closeEntry();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

//	public void doBufferReadFile(File file) throws Exception
//	{
//		BufferedReader reader = null;
//		try {
//
////			InputStreamReader isr = new InputStreamReader(new FileInputStream(file), "GBK");
//			reader = new BufferedReader(new FileReader(file));
//			String line = null;
//			int num = 0;
//			List<HistoryMessage> historyMessages = new ArrayList<>();
//			while ((line = reader.readLine()) != null){
//				try {
//					historyMessages.add(JSONObject.parseObject(line.substring(line.indexOf("{")), HistoryMessage.class));
//					if(historyMessages.size() >= 2000){
//						historyMessageDao.batchInsert(historyMessages,HistoryMessage.class);
//						historyMessages.clear();
//					}
//				}catch (Exception e){
//					e.printStackTrace();
//				}
//				num++;
//			}
//			if(historyMessages.size() > 0){
//				historyMessageDao.batchInsert(historyMessages,HistoryMessage.class);
//			}
//		} finally {
//			reader.close();
//		}
//	}


//	public static void doNioReadFile(File file) throws Exception
//	{
//		String enterStr = "\n";
//
//		FileChannel inChannel = new FileInputStream(file).getChannel();
//		ByteBuffer buffer = ByteBuffer.allocate(1048576);
//
//		StringBuilder newlinesBui = new StringBuilder();
//		while (inChannel.read(buffer) != -1)
//		{
//			buffer.flip();
//			//数据组合.
//			String content = new String(buffer.array());
//			newlinesBui.append(content).toString();
//
//			int fromIndex = 0;
//			int endIndex = -1;
//			//循环找到 \n
//			List<HistoryMessage> historyMessages = new ArrayList<>();
//			while ((endIndex = newlinesBui.indexOf(enterStr, fromIndex)) > -1)
//			{
//				//得到一行
//				String line = newlinesBui.substring(fromIndex, endIndex);
//				try {
//					historyMessages.add(JSONObject.parseObject(line.substring(line.indexOf("{")), HistoryMessage.class));
//					if(historyMessages.size() >= 2000){
////						historyMessageDao.batchInsert(historyMessages,HistoryMessage.class);
//						historyMessages.clear();
//					}
//				}catch (Exception e){
//					e.printStackTrace();
//				}
//				fromIndex = endIndex + 1;
//			}
//			if(historyMessages.size() > 0){
////				historyMessageDao.batchInsert(historyMessages,HistoryMessage.class);
//			}
//			newlinesBui.delete(0, fromIndex);
//			buffer.clear();
//		}
//	}
}
