package com.yonge.log.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.yonge.log.dal.dao.HistoryMessageTenantDao;
import com.yonge.log.dal.dao.OperationLogDao;
import com.yonge.log.dal.model.HistoryMessage;
import com.yonge.log.dal.model.HistoryMessageTencent;
import com.yonge.log.dal.model.OperationLog;
import com.yonge.log.service.HistoryMessageTenantService;
import com.yonge.log.service.OperationLogService;
import com.yonge.log.wrapper.HistoryMessageTencentWrapper;
import com.yonge.mongodb.PageInfo;
import com.yonge.mongodb.dao.BaseDaoWithMongo;
import com.yonge.mongodb.service.BaseServiceWithMongo;
import com.yonge.mongodb.service.impl.BaseServiceImplWithMongo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

@Service
public class HistoryMessageTenantServiceImpl extends BaseServiceImplWithMongo<String, HistoryMessageTencent> implements HistoryMessageTenantService {

    @Autowired
    private HistoryMessageTenantDao historyMessageTenantDao;

    @Autowired
    private MongoTemplate mongoTemplate;
    @Override
    public BaseDaoWithMongo<String, HistoryMessageTencent> getDAO() {
        return historyMessageTenantDao;
    }

    @Override
    public PageInfo<HistoryMessageTencentWrapper.HistoryMessageTencent> historyMessage(HistoryMessageTencentWrapper.HistoryMessageTencentQuery param) {
        PageInfo<HistoryMessageTencentWrapper.HistoryMessageTencent> pageInfo = new PageInfo<>(param.getPage(), param.getRows());


        Query query = getQuery(param);

        List<HistoryMessageTencentWrapper.HistoryMessageTencent> dataList = new ArrayList<>();

        long total = mongoTemplate.count(query,HistoryMessageTencent.class);
        if (total > 0) {
            pageInfo.setTotal(total);

            query.skip(pageInfo.getOffset()).limit(pageInfo.getLimit());
            List<HistoryMessageTencent> historyMessageTencents = mongoTemplate.find(query, HistoryMessageTencent.class);
            dataList = JSON.parseArray(JSON.toJSONString(historyMessageTencents), HistoryMessageTencentWrapper.HistoryMessageTencent.class);
        }
        if (total == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public void deleteOld() {
        Query query = new Query();
        Criteria criteria = Criteria.where("app_service").is("MEC");
        query.addCriteria(criteria);
        mongoTemplate.remove(query,HistoryMessageTencent.class);
    }

    @Override
    public int initMsgBodyJsonCount() {
        Query query = new Query();
        Criteria criteria = Criteria.where("msg_body_json").is(null).orOperator(Criteria.where("json_flag").is(0),Criteria.where("json_flag").is(null));
        query.addCriteria(criteria);
        return Long.valueOf(mongoTemplate.count(query,HistoryMessageTencent.class)).intValue();
    }

    @Override
    public void initMsgBodyJson(int page, int size) {
        Query query = new Query();
        Criteria criteria = Criteria.where("msg_body_json").is(null).orOperator(Criteria.where("json_flag").is(0),Criteria.where("json_flag").is(null));
        query.addCriteria(criteria);
        query.skip(page).limit(size);
        List<HistoryMessageTencent> historyMessageTencents = mongoTemplate.find(query, HistoryMessageTencent.class);
        for (HistoryMessageTencent historyMessageTencent : historyMessageTencents) {
            historyMessageTencent.setJsonFlag(1);
            try {
                historyMessageTencent.setMsgBodyJson(JSON.parseArray(historyMessageTencent.getMsgBody()));
            } catch (Exception e) {
                historyMessageTencent.setMsgBodyJson(new JSONArray());
            }
            mongoTemplate.save(historyMessageTencent);
        }
    }

    private static Query getQuery(HistoryMessageTencentWrapper.HistoryMessageTencentQuery param) {
        Query query = new Query();
        Criteria criteria = new Criteria();

        if (param.getStartTime() != null) {
            if (param.getEndTime() != null) {
                criteria.and("msg_timestamp").gte(param.getStartTime().getTime() / 1000).lte(param.getEndTime().getTime() / 1000);
            } else {
                criteria.and("msg_timestamp").gte(param.getStartTime().getTime() / 1000);
            }
        }
        if (param.getEndTime() != null) {
            if (param.getStartTime() == null) {
                criteria.and("msg_timestamp").lte(param.getEndTime().getTime()/1000);
            }
        }
        if (StringUtils.isNotBlank(param.getChatType())) {
            criteria.and("chat_type").is(param.getChatType());
        }
        if (StringUtils.isNotBlank(param.getFromAccount())) {
            criteria.and("from_account").is(param.getFromAccount());
        }
        if (StringUtils.isNotBlank(param.getToAccount())) {
            criteria.and("to_account").is(param.getToAccount());
        }
        if (StringUtils.isNotBlank(param.getGroupId())) {
            criteria.and("group_id").is(param.getGroupId());
        }
        if (StringUtils.isNotBlank(param.getAppService())) {
            criteria.and("app_service").is(param.getAppService());
        }
        Pattern pattern = Pattern.compile(".*", Pattern.CASE_INSENSITIVE);
        if (StringUtils.isNotBlank(param.getKeyword())) {
            pattern = Pattern.compile(".*" + param.getKeyword() + ".*", Pattern.CASE_INSENSITIVE);
            criteria.and("msg_body_json").elemMatch(
                Criteria.where("MsgContent.Text").regex(pattern)
                    .and("MsgType").is("TIMTextElem")
            );
        }


        query.addCriteria(criteria);
        query.with(Sort.by(Sort.Order.desc("msg_timestamp"))); // 以升序方式按字段排序

        return query;
    }
}
