package com.ym.controller;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.middleware.common.http.ImageUtil;
import com.microsvc.toolkit.middleware.im.ImPluginContext;
import com.microsvc.toolkit.middleware.im.impl.RongCloudImPlugin;
import com.microsvc.toolkit.middleware.im.impl.TencentCloudImPlugin;
import com.microsvc.toolkit.middleware.im.message.ETencentMessage;
import com.microsvc.toolkit.middleware.im.message.MessageWrapper;
import com.microsvc.toolkit.middleware.im.message.TencentRequest;
import com.ym.mec.common.entity.ImMessageDto;
import com.ym.pojo.NoticeMessage;
import com.ym.service.GroupService;
import com.ym.service.MessageService;
import io.rong.messages.FileMessage;
import io.rong.messages.ImgMessage;
import io.rong.messages.TxtMessage;
import io.rong.messages.UserInfo;
import io.rong.models.group.GroupMember;
import io.rong.models.group.GroupModel;
import io.rong.models.group.UserGroup;
import io.rong.models.message.GroupMessage;
import io.rong.models.message.MentionMessage;
import io.rong.models.message.RecallMessage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by weiqinxiao on 2019/2/25.
 */
@Slf4j
@RestController
@RequestMapping("/group")
public class GroupController{

    @Autowired
    MessageService messageService;
    @Autowired
    GroupService groupService;
    @Autowired
    private ImPluginContext imPluginContext;

    @RequestMapping(value = "/send", method = RequestMethod.POST)
    public Object send(@RequestBody ImMessageDto imMessageDto) throws Exception {

        // 群组消息
        MessageWrapper.GroupMessage build = MessageWrapper.GroupMessage.builder()
            .senderId(imMessageDto.getSenderId())
            .objectName(imMessageDto.getObjectName())
            //.targetIds()
            //.groupId()
            .pushExt("")
            .includeSender(imMessageDto.getIsIncludeSender())
            .cloudCustomData(imMessageDto.getExtra())
            .build();

        String objectName = imMessageDto.getObjectName();
        if (RongCloudImPlugin.PLUGIN_NAME.equals(imPluginContext.defaultService())) {

            // 融云消息
            GroupMessage groupMessage = new GroupMessage();
            UserInfo userInfo = new UserInfo();
            if(imMessageDto.getUserInfo() != null){
                BeanUtils.copyProperties(imMessageDto.getUserInfo(),userInfo);
            }
            if("DY:NoticeMsg".equals(objectName)){
                NoticeMessage noticeMessage = new NoticeMessage(imMessageDto.getMessageContent(),imMessageDto.getExtra());
                noticeMessage.setUser(userInfo);
                groupMessage.setContent(noticeMessage);
            }else if("RC:ImgMsg".equals(objectName)){
                ImgMessage imgMessage = new ImgMessage(imMessageDto.getMessageContent(),imMessageDto.getExtra(),imMessageDto.getFileUrl());
                imgMessage.setUser(userInfo);
                groupMessage.setContent(imgMessage);
            }else if("RC:FileMsg".equals(objectName)){
                FileMessage imgMessage = new FileMessage(imMessageDto.getFileName(),imMessageDto.getFileSize(),imMessageDto.getFileUrl(),imMessageDto.getExtra());
                groupMessage.setContent(imgMessage);
            }else {
                TxtMessage txtMessage = new TxtMessage(imMessageDto.getMessageContent(),imMessageDto.getExtra());
                txtMessage.setUser(userInfo);
                groupMessage.setContent(txtMessage);
            }
            groupMessage.setSenderId(imMessageDto.getSenderId());
            groupMessage.setObjectName(imMessageDto.getObjectName());
            groupMessage.setIsIncludeSender(imMessageDto.getIsIncludeSender());

            build.rongCloueMessage(groupMessage.getContent());
        } else {
            // 腾讯云消息
            if ("RC:ImgMsg".equals(objectName)) {
                // 图片消息
                ImageUtil.ImageInfo imageReq = ImageUtil.ImageInfo.builder()
                    .imgUrl(imMessageDto.getFileUrl())
                    .suffix(imMessageDto.getFileUrl().substring(imMessageDto.getFileUrl().lastIndexOf(".") + 1))
                    .build();

                // 获取图片基本信息
                ImageUtil.imageToBase64(imageReq);

                // 图片消息
                build.tencentMessage(TencentRequest.getTIMImageElem(imageReq));
            } else if ("RC:FileMsg".equals(objectName)) {
                // 文件消息
                build.tencentMessage(TencentRequest.getTIMFileElem(imMessageDto.getFileUrl(), imMessageDto.getFileSize().intValue(), imMessageDto.getFileName()));
            } else {
                // DY:NoticeMsg 当作普通文本消息发送
                // 文本消息
                build.tencentMessage(TencentRequest.MessageBody.builder()
                    .msgType(ETencentMessage.TIMTextElem.name())
                    .msgContent(TencentRequest.TextMessageBody.builder().text(imMessageDto.getMessageContent()).build())
                    .build());

                // 普通文本消息，若是超链接地址，则需要转换成腾讯云的消息格式
                if (imMessageDto.getMessageContent().startsWith("http") && StringUtils.isBlank(imMessageDto.getExtra())) {
                    // 自动填充为拓展消息内容
                    build.setCloudCustomData(imMessageDto.getMessageContent());
                }
            }
        }

        //融云只支持3个群以内的群发
        String[] targetIds = imMessageDto.getTargetIds();
        List<String> imGroupIds = Arrays.stream(targetIds).collect(Collectors.toList());
        List<List<String>> partition = Lists.partition(imGroupIds, 3);
        //Result result = null;
        for (List<String> integers : partition) {
            /*String[] strings = integers.toArray(new String[integers.size()]);
            groupMessage.setTargetId(strings);
            result = messageService.groupSend(groupMessage);*/

            if (TencentCloudImPlugin.PLUGIN_NAME.equals(imPluginContext.defaultService())) {
                // 腾讯云消息
                for (String groupId : integers) {
                    MessageWrapper.GroupMessage groupMessage = JSON.parseObject(JSON.toJSONString(build.groupId(groupId)),
                        MessageWrapper.GroupMessage.class);
                    String ret = imPluginContext.getPluginService().sendGroupMessage(groupMessage);
                    log.info("batchSendPrivateMessage GROUP tencentCloud senderId={}, ret={}, message={}", build.getSenderId(), ret, JSON.toJSONString(groupMessage));

                }
            } else {

                MessageWrapper.GroupMessage groupMessage = JSON.parseObject(build.targetIds(integers).toString(),
                    MessageWrapper.GroupMessage.class);
                // 融云消息
                String ret = imPluginContext.getPluginService().sendGroupMessage(groupMessage);
                log.info("batchSendPrivateMessage GROUP rongCloud senderId={}, ret={}", build.getSenderId(), ret);
            }

        }
        return true;
    }

    @RequestMapping(value = "/sendDirection", method = RequestMethod.POST)
    public Object sendDirection(@RequestBody GroupMessage groupMessage) throws Exception {
        return messageService.groupSendDirection(groupMessage);
    }

    @RequestMapping(value = "/sendMention", method = RequestMethod.POST)
    public Object sendMention(@RequestBody MentionMessage mentionMessage) throws Exception {
        return messageService.groupSendMention(mentionMessage);
    }

    @RequestMapping(value = "/recall", method = RequestMethod.POST)
    public Object recall(@RequestBody RecallMessage recallMessage) throws Exception {
        return messageService.groupRecall(recallMessage);
    }

    @RequestMapping(value = "/sync", method = RequestMethod.POST)
    public Object sync(@RequestBody UserGroup userGroup) throws Exception {
        return groupService.groupSync(userGroup);
    }

    @RequestMapping(value = "/create", method = RequestMethod.POST)
    public Object create(@RequestBody GroupModel groupModel) throws Exception {
        return groupService.groupCreate(groupModel);
    }

    @RequestMapping(value = "/batchCreate", method = RequestMethod.POST)
    public void batchCreate(@RequestBody List<GroupModel> groupModels){
        groupService.batchCreate(groupModels);
    }

    @RequestMapping(value = "/get", method = RequestMethod.POST)
    public Object get(@RequestBody GroupModel groupModel) throws Exception {
        return groupService.groupGet(groupModel);
    }

    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public Object update(@RequestBody GroupModel groupModel) throws Exception {
        return groupService.groupUpdate(groupModel);
    }

    @RequestMapping(value = "/join", method = RequestMethod.POST)
    public Object join(@RequestBody GroupModel groupModel) throws Exception {
        return groupService.groupJoin(groupModel);
    }

    @RequestMapping(value = "/batchJoin", method = RequestMethod.POST)
    public void batchJoin(@RequestBody List<GroupModel> groupModels) throws Exception {
        groupService.groupBatchJoin(groupModels);
    }

    @RequestMapping(value = "/quit", method = RequestMethod.POST)
    public Object quit(@RequestBody GroupModel groupModel) throws Exception {
        return groupService.groupQuit(groupModel);
    }

    @RequestMapping(value = "/batchQuit", method = RequestMethod.POST)
    public void batchQuit(@RequestBody List<GroupModel> groupModels) throws Exception {
        groupService.groupBatchQuit(groupModels);
    }

    @RequestMapping(value = "/dismiss", method = RequestMethod.POST)
    public Object dismiss(@RequestBody GroupModel groupModel) throws Exception {
        GroupMember[] groupMember = {new GroupMember("1",null,null)};
        groupModel.setMembers(groupMember);
        return groupService.groupDismiss(groupModel);
    }

    @RequestMapping(value = "/batchDismiss", method = RequestMethod.POST)
    public void batchDismiss(@RequestBody List<GroupModel> groupModels){
        groupService.groupBatchDismiss(groupModels);
    }
}
