package com.ym.pojo;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.microsvc.toolkit.middleware.rtc.message.RTCRoomConfig;
import com.ym.mec.biz.dal.entity.CourseScheduleStudentMusicScore;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * Created by weiqinxiao on 2019/2/28.
 */
@Getter
@Setter
public class RoomResult {
    private @Getter @Setter String roomId;
    //课程结束后是否自动关闭课程
    private @Getter @Setter Boolean autoCloseFlag = true;
    //课程结束后多少分钟关闭网络教室
    private @Getter @Setter String autoCloseNetworkRoomTime;
    private @Getter @Setter Date startTime;
    private @Getter @Setter int surplusTime;
    //伴奏音量
    private @Getter @Setter int soundVolume = 100;
    private @Getter @Setter String imToken;
    private @Getter @Setter String authorization;
    private @Getter List<MemberResult> members = new ArrayList<>();
    private @Getter @Setter String display;
    private @Getter @Setter List<WhiteboardResult> whiteboards = new ArrayList<>();
    private @Getter @Setter MemberResult userInfo;
    private @Getter @Setter String randomNumeric = "0";

    @ApiModelProperty("RTC接入参数")
    private @Getter @Setter RTCRoomConfig rtcRoomConfig;
    @ApiModelProperty("直播间用户签名")
    private @Getter @Setter String userSig;
    @ApiModelProperty("群组id")
    private @Getter @Setter String groupId;
    @ApiModelProperty("服务提供商")
    private @Getter @Setter String serviceProvider;
    @ApiModelProperty("课程人数")
    private @Getter @Setter Integer studentNums;

    @ApiModelProperty("直播间编号")
    private String liveRoomId;

    @ApiModelProperty("声部编号")
    private String subjectId;

    @ApiModelProperty("os")
    private String os;
    public RoomResult rtcRoomConfig(RTCRoomConfig rtcRoomConfig) {
        this.rtcRoomConfig = rtcRoomConfig;
        return this;
    }

    public RoomResult userSig(String userSig) {
        this.userSig = userSig;
        return this;
    }

    public RoomResult subjectId(String subjectId) {
        this.subjectId = subjectId;
        return this;
    }

    @Data
    public static class MemberResult {
        String userId;
        String userName;
        String headUrl;
        int role;
        Date joinTime;
        boolean camera;
        boolean microphone;
        boolean handUpOn;
        ExamSongDownloadData examSongDownloadJson;
        List<CourseScheduleStudentMusicScore> scheduleStudentMusicScores;
        CustomMessage playMidiJson;
    }

    @Data
    public static class WhiteboardResult {
        String whiteboardId;
        String name;
        int curPg;
    }

    public void setMembers(List<RoomMember> roomMemberList, Map<Integer,String> midiMap,Map<Integer,String> examSongMap,List<CourseScheduleStudentMusicScore> scheduleStudentMusicScores) {
        for (RoomMember member : roomMemberList) {
            MemberResult result = new MemberResult();
            result.setUserId(member.getUid());
            result.setJoinTime(member.getJoinDt());
            result.setRole(member.getRole());
            result.setMicrophone(member.isMic());
            result.setCamera(member.isCamera());
            result.setUserName(member.getName());
            result.setHandUpOn(member.isHand());
            result.setHeadUrl(member.getHeadUrl());
            result.setPlayMidiJson(JSONObject.parseObject(midiMap.get(Integer.parseInt(member.getUid())),CustomMessage.class));
            if(CollectionUtils.isNotEmpty(scheduleStudentMusicScores)){
                result.setScheduleStudentMusicScores(scheduleStudentMusicScores.stream()
                        .filter(e->e.getUserId().equals(Integer.parseInt(result.getUserId()))).collect(Collectors.toList()));
            }
            ExamSongDownloadData examSongDownloadData;
            String json = examSongMap.get(Integer.parseInt(member.getUid()));
            if(StringUtils.isEmpty(json)){
                examSongDownloadData = new ExamSongDownloadData();
            }else {
                examSongDownloadData = JSON.parseObject(json,ExamSongDownloadData.class);
            }
            result.setExamSongDownloadJson(examSongDownloadData);
            members.add(result);
        }
    }

    public void setWhiteboards(List<Whiteboard> whiteboardList) {
        for (Whiteboard wb : whiteboardList) {
            WhiteboardResult r = new WhiteboardResult();
            r.setName(wb.getName());
            r.setWhiteboardId(wb.getWbid());
            r.setCurPg(wb.getCurPg());
            whiteboards.add(r);
        }
    }


}
