package com.yonge.cooleshow.admin.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.yonge.cooleshow.admin.dto.PmsProductParam;
import com.yonge.cooleshow.admin.dto.PmsProductQueryParam;
import com.yonge.cooleshow.admin.dto.PmsProductResult;
import com.yonge.cooleshow.admin.service.PmsProductService;
import com.yonge.cooleshow.admin.service.PmsProductSkuStockRecordService;
import com.yonge.cooleshow.admin.wrapper.PmsProductSkuStockRecordWrapper;
import com.yonge.cooleshow.mall.common.api.CommonPage;
import com.yonge.cooleshow.mall.common.api.CommonResult;
import com.yonge.cooleshow.mall.common.api.ResultCode;
import com.yonge.cooleshow.mbg.model.PmsProduct;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * 商品管理Controller
 * Created by macro on 2018/4/26.
 */
@RestController
@Api(tags = "PmsProductSkuStockRecordController", description = "商品库存入库记录")
@RequestMapping("/PmsProductSkuStockRecord")
public class PmsProductSkuStockRecordController {
    @Autowired
    private PmsProductSkuStockRecordService pmsProductSkuStockRecordService;

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @ApiOperation("商品库存入库记录")
    @PostMapping(value = "/list")
    public CommonResult<CommonPage<PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord>> getList(@RequestBody
                                      @Validated PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecordQuery query) {
        List<PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord> productList = pmsProductSkuStockRecordService.list(query);
        return CommonResult.success(CommonPage.restPage(productList));
    }

    @ApiOperation("新增商品库存入库记录")
    @RequestMapping(value = "/create", method = RequestMethod.POST)
    @ResponseBody
    public CommonResult create(@RequestBody @Validated PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord pmsProductSkuStockRecord) {

        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null || sysUser.getId() == null) {
            return CommonResult.failed(ResultCode.FORBIDDEN, "请登录");
        }
        pmsProductSkuStockRecord.setCreateBy(sysUser.getId().longValue());
        int count = pmsProductSkuStockRecordService.create(pmsProductSkuStockRecord);
        if (count > 0) {
            return CommonResult.success(count);
        } else {
            return CommonResult.failed();
        }
    }


    @ApiOperation("新增商品库存入库记录")
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    @ResponseBody
    public CommonResult update(@RequestBody @Validated PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord pmsProductSkuStockRecord) {

        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null || sysUser.getId() == null) {
            return CommonResult.failed(ResultCode.FORBIDDEN, "请登录");
        }
        pmsProductSkuStockRecord.setCreateBy(sysUser.getId().longValue());
        pmsProductSkuStockRecord.setCreateTime(new Date());
        int count = pmsProductSkuStockRecordService.update(pmsProductSkuStockRecord);
        if (count > 0) {
            return CommonResult.success(count);
        } else {
            return CommonResult.failed();
        }
    }
}
