package com.yonge.cooleshow.admin.controller.open;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.api.R;
import com.ym.mec.common.dto.PmsProductDto;
import com.ym.mec.common.dto.PmsProductQueryParamDto;
import com.ym.mec.common.dto.ReturnApplyDto;
import com.ym.mec.common.page.PageInfo;
import com.yonge.cooleshow.admin.controller.OmsOrderReturnApplyController;
import com.yonge.cooleshow.admin.dto.OrderCreate;
import com.yonge.cooleshow.admin.dto.PmsProductCategoryWithChildrenItem;
import com.yonge.cooleshow.admin.service.*;
import com.yonge.cooleshow.mall.common.api.CommonPage;
import com.yonge.cooleshow.mall.common.api.CommonResult;
import com.yonge.cooleshow.mall.common.enums.OrderCacheEnum;
import com.yonge.cooleshow.mbg.model.PmsBrand;
import com.yonge.cooleshow.mbg.model.PmsProductAttributeCategory;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * Description
 *
 * @author liujunchi
 * @date 2022-07-28
 */
@RestController
@RequestMapping("/open")
public class OpenController {
    @Autowired
    private OmsOrderReturnApplyService returnApplyService;


    @Value("${message.debugMode}")
    private boolean debugMode;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private PmsProductService productService;

    @Autowired
    private PmsBrandService brandService;

    @Autowired
    private PmsProductCategoryService productCategoryService;

    @Autowired
    private PmsProductAttributeCategoryService productAttributeCategoryService;

    @Autowired
    private OmsOrderService orderService;


    private static final Logger log = LoggerFactory.getLogger(OmsOrderReturnApplyController.class);

    @ApiOperation("完成退货测试接口")
    @GetMapping(value = "/test/returnApply/{id}")
    @ResponseBody
    public CommonResult test(@PathVariable Long id) {
        if (!debugMode) {
            return CommonResult.failed("当前环境不可使用");
        }
        RLock lock = redissonClient.getLock(OrderCacheEnum.LOCK_REFUND_ORDER_MALL + ":refundOrder:" + id);
        try {
            boolean b = lock.tryLock(60, 60, TimeUnit.SECONDS);
            if (b) {
                returnApplyService.setTestReturnApply(id);
                return CommonResult.success(true);
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            if (lock.getHoldCount() >0) {
                lock.unlock();
            }
        }

        return CommonResult.failed();
    }



    /**
     * 查询商品订单生成
     * @param order ProductWrapper.OrderCreate
     * @return String
     */
    @PostMapping(value = "/product/orderCreate", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public CommonResult<Boolean> productOrderCreate(@RequestBody OrderCreate order) {

        log.info("创建管乐迷订单：{}", JSON.toJSONString(order));

        if (!CollectionUtils.isEmpty(order.getCloseOrderNoList())) {

            RLock lock = redissonClient.getLock(OrderCacheEnum.LOCK_REFUND_ORDER_MALL + ":delivery");
            try {
                boolean b = lock.tryLock(60, 60, TimeUnit.SECONDS);
                if (b) {
                    orderService.mecProductOrderCreate(order);

                    return CommonResult.success(true);


                }
            } catch (InterruptedException e) {
                log.error("批量修改订单", e);
            } finally {
                if (lock.getHoldCount() >0) {
                    lock.unlock();
                }
            }
        } else {
            orderService.mecProductOrderCreate(order);


        }



        return CommonResult.success(true);
    }


    /**
     * 退货
     */
    @PostMapping(value = "/product/returnApply", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public CommonResult<Boolean> returnApply(@RequestBody ReturnApplyDto returnApplyDto) {

        log.info("创建管乐迷退款：{}", JSON.toJSONString(returnApplyDto));

        returnApplyService.returnApply(returnApplyDto);


        return CommonResult.success(true);
    }


    // 同步订单状态，
    /**
     * 查询商品订单状态
     * @param orderNo 订单编号
     * @return status 订单状态  0->待付款；1->待发货；2->已发货；3->已完成；4->已关闭；5->无效订单
     */
    @GetMapping(value = "/product/updateOrderStatus", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public CommonResult<Boolean> productUpdateOrderStatus(@RequestParam("orderNo") String orderNo,@RequestParam("status") Integer status) {
        log.info("同步订单状态：{}，{}", orderNo, status);
        // 已发货订单不可修改状态

        if (status == 4) {

            RLock lock = redissonClient.getLock(OrderCacheEnum.LOCK_REFUND_ORDER_MALL + ":delivery");
            try {
                boolean b = lock.tryLock(60, 60, TimeUnit.SECONDS);
                if (b) {
                    orderService.productUpdateOrderStatus(orderNo, status);

                    return CommonResult.success(true);


                }
            } catch (InterruptedException e) {
                log.error("批量修改订单", e);
            } finally {
                if (lock.getHoldCount() >0) {
                    lock.unlock();
                }
            }
        } else {
            orderService.productUpdateOrderStatus(orderNo, status);


        }

        return CommonResult.success(true);
    }

    @PostMapping(value = "/product/list")
    @ResponseBody
    public PageInfo<PmsProductDto> getProductList(@RequestBody @Validated PmsProductQueryParamDto productQueryParam){
        String json = productQueryParam.getJson();
        productQueryParam = JSON.parseObject(json,PmsProductQueryParamDto.class);
        List<PmsProductDto> feignProductList = productService.getFeignProductList(productQueryParam);
        CommonPage<PmsProductDto> dtoCommonPage = CommonPage.restPage(feignProductList);
        PageInfo<PmsProductDto> pageInfo = new PageInfo<>(productQueryParam.getPageNum(),productQueryParam.getPageSize());
        pageInfo.setRows(feignProductList);
        pageInfo.setTotal(dtoCommonPage.getTotal().intValue());
        return pageInfo;

    }


    @ApiOperation(value = "根据品牌名称分页获取品牌列表")
    @RequestMapping(value = "/brand/list", method = RequestMethod.GET)
    @ResponseBody
    public List<PmsBrand> getList() {
        List<PmsBrand> brandList = brandService.listBrand("", 1, 9999);
        return brandList;
    }

    @ApiOperation("查询所有一级分类及子分类")
    @RequestMapping(value = "/productCategory/list/withChildren", method = RequestMethod.GET)
    @ResponseBody
    public List<PmsProductCategoryWithChildrenItem> listWithChildren() {
        List<PmsProductCategoryWithChildrenItem> list = productCategoryService.listWithChildren();
        return list;
    }


    @ApiOperation("分页获取所有商品属性分类")
    @RequestMapping(value = "productAttribute/category/list", method = RequestMethod.GET)
    public List<PmsProductAttributeCategory> getProductAttributeCategoryList() {
        List<PmsProductAttributeCategory> productAttributeCategoryList = productAttributeCategoryService.getList(9999, 1);
        CommonPage<PmsProductAttributeCategory> pmsProductAttributeCategoryCommonPage = CommonPage.restPage(productAttributeCategoryList);
        return pmsProductAttributeCategoryCommonPage.getList();
    }
}
