package com.yonge.cooleshow.admin.service;

import com.ym.mec.common.dto.PmsProductDto;
import com.ym.mec.common.dto.PmsProductQueryParamDto;
import com.yonge.cooleshow.admin.dto.HomeStatistical;
import com.yonge.cooleshow.admin.dto.PmsProductParam;
import com.yonge.cooleshow.admin.dto.PmsProductQueryParam;
import com.yonge.cooleshow.admin.dto.PmsProductResult;
import com.yonge.cooleshow.mbg.model.PmsProduct;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * 商品管理Service
 * Created by macro on 2018/4/26.
 */
public interface PmsProductService {
    /**
     * 创建商品
     */
    @Transactional(isolation = Isolation.DEFAULT,propagation = Propagation.REQUIRED)
    int create(PmsProductParam productParam);

    /**
     * 根据商品编号获取更新信息
     */
    PmsProductResult getUpdateInfo(Long id);

    /**
     * 更新商品
     */
    @Transactional
    int update(Long id, PmsProductParam productParam);

    /**
     * 分页查询商品
     */
    List<PmsProduct> list(PmsProductQueryParam productQueryParam, Integer pageSize, Integer pageNum);

    /**
     * 批量修改审核状态
     * @param ids 产品id
     * @param verifyStatus 审核状态
     * @param detail 审核详情
     */
    @Transactional
    int updateVerifyStatus(List<Long> ids, Integer verifyStatus, String detail);

    /**
     * 批量修改商品上架状态
     */
    @Transactional
    int updatePublishStatus(List<Long> ids, Integer publishStatus);

    /**
     * 批量修改商品推荐状态
     */
    @Transactional
    int updateRecommendStatus(List<Long> ids, Integer recommendStatus);

    /**
     * 批量修改新品状态
     */
    @Transactional
    int updateNewStatus(List<Long> ids, Integer newStatus);

    /**
     * 批量删除商品
     */
    @Transactional
    int updateDeleteStatus(List<Long> ids, Integer deleteStatus);

    /**
     * 根据商品名称或者货号模糊查询
     */
    List<PmsProduct> list(String keyword);

    /**
     * 获取缺货数量
     */
    HomeStatistical countLowStockCount();

    /**
     * 统计 已下架 已上架 全部商品
     *
     * @return
     */
    HomeStatistical countProductShowStatus();

    Map<Long, PmsProduct> getMapByIds(List<Long> productIds);

    /**
     * 获取商品SKU列表
     *
     */
    List<PmsProductDto> getFeignProductList(PmsProductQueryParamDto productQueryParam);
}
