package com.yonge.cooleshow.admin.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.ym.mec.common.dto.ReturnApplyDto;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.IdGeneratorService;
import com.yonge.cooleshow.admin.dao.OmsOrderReturnApplyDao;
import com.yonge.cooleshow.admin.dto.*;
import com.yonge.cooleshow.admin.service.OmsOrderService;
import com.yonge.cooleshow.admin.service.PmsProductService;
import com.yonge.cooleshow.mall.common.exception.ApiException;
import com.yonge.cooleshow.mall.common.service.RedisService;
import com.yonge.cooleshow.mbg.mapper.*;
import com.yonge.cooleshow.mbg.model.*;
import com.yonge.cooleshow.admin.service.OmsOrderReturnApplyService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 订单退货管理Service实现类
 * Created by macro on 2018/10/18.
 */
@Service
public class OmsOrderReturnApplyServiceImpl implements OmsOrderReturnApplyService {

    private static final Logger log = LoggerFactory.getLogger(OmsOrderReturnApplyService.class);

    @Autowired
    private OmsOrderReturnApplyDao returnApplyDao;
    @Autowired
    private OmsOrderReturnApplyMapper returnApplyMapper;

    @Autowired
    private OmsOrderService omsOrderService;
    @Autowired
    private OmsOrderMapper omsOrderMapper;

    @Autowired
    private OmsOrderItemMapper orderItemMapper;

    @Autowired
    private PmsProductMapper productMapper;
    @Autowired
    private PmsBrandMapper brandMapper;

    @Autowired
    private RedisService redisService;
    private String REDIS_KEY_ORDER_ID= "redis.key.orderId";
    private String REDIS_DATABASE = "redis.database";

    @Autowired
    private UserOrderPaymentMapper userOrderPaymentMapper;

    @Autowired
    private IdGeneratorService idGeneratorService;

    @Autowired
    private UserOrderRefundMapper userOrderRefundMapper;

    @Override
    public List<OmsOrderReturnApply> list(OmsReturnApplyQueryParam queryParam, Integer pageSize, Integer pageNum) {
        PageHelper.startPage(pageNum, pageSize);
        return returnApplyDao.getList(queryParam);
    }

    @Override
    public int delete(List<Long> ids) {
        OmsOrderReturnApplyExample example = new OmsOrderReturnApplyExample();
        example.createCriteria().andIdIn(ids).andStatusEqualTo(3);
        List<OmsOrderReturnApply> omsOrderReturnApplies = returnApplyMapper.selectByExample(example);
        if (omsOrderReturnApplies.size() != ids.size()) {
            throw new BizException("只能删除已拒绝的退货订单");
        }
        return returnApplyMapper.deleteByExample(example);
    }

    @Override
    public int updateStatus(Long id, OmsUpdateStatusParam statusParam) {
        Integer status = statusParam.getStatus();
        OmsOrderReturnApply omsOrderReturnApply = returnApplyMapper.selectByPrimaryKey(id);
        if (omsOrderReturnApply == null) {
            throw new BizException("退货订单不存在或被撤销");
        }
        if (omsOrderReturnApply.getProductRealPrice().multiply(BigDecimal.valueOf(omsOrderReturnApply.getProductCount())).compareTo(statusParam.getReturnAmount()) <0) {
            throw new BizException("退款金额不能大于商品金额");
        }
        if (omsOrderReturnApply.getStatus().equals(statusParam.getStatus())) {
            return 1;
        }
        OmsOrderReturnApply returnApply = new OmsOrderReturnApply();
        if (status.equals(1)) {
            //确认退货
            returnApply.setId(id);
            returnApply.setStatus(1);
            returnApply.setReturnAmount(statusParam.getReturnAmount());
            returnApply.setCompanyAddressId(statusParam.getCompanyAddressId());
            returnApply.setHandleTime(new Date());
            returnApply.setHandleMan(statusParam.getHandleMan());
            returnApply.setHandleNote(statusParam.getHandleNote());
        } else if (status.equals(2)) {
            //完成退货
            returnApply.setId(id);
            returnApply.setStatus(2);
            returnApply.setReceiveTime(new Date());
            returnApply.setReceiveMan(statusParam.getReceiveMan());
            returnApply.setReceiveNote(statusParam.getReceiveNote());
            if (omsOrderReturnApply.getReturnAmount().compareTo(BigDecimal.ZERO) >0) {
                // refundAmount(omsOrderReturnApply);
            }

            // 确认退款 取消平台收入和 分润
            // setRecord(omsOrderReturnApply);

        } else if (status.equals(3)) {
            //拒绝退货
            returnApply.setId(id);
            returnApply.setStatus(3);
            returnApply.setHandleTime(new Date());
            returnApply.setHandleMan(statusParam.getHandleMan());
            returnApply.setHandleNote(statusParam.getHandleNote());
        } else {
            return 0;
        }
        return returnApplyMapper.updateByPrimaryKeySelective(returnApply);
    }


    @Override
    public void setTestReturnApply(Long id) {
        OmsOrderReturnApply returnApply = returnApplyMapper.selectByPrimaryKey(id);
        returnApply.setId(id);
        returnApply.setStatus(2);
        returnApply.setReceiveTime(new Date());
        returnApply.setReceiveMan("测试退货");
        returnApply.setReceiveNote("测试退货");

        // 确认退款 取消平台收入和 分润
        // setRecord(returnApply);
        returnApplyMapper.updateByPrimaryKeySelective(returnApply);
    }

    @Override
    public OmsOrderReturnApplyResult getItem(Long id) {
        return returnApplyDao.getDetail(id);
    }


    @Override
    public HomeStatistical countReturnStatus() {

        return returnApplyDao.selectReturnStatusCount();
    }

    @Override
    @Transactional
    public void returnApply(ReturnApplyDto returnApplyDto) {

        // 查询订单
        OmsOrder byOrderSn = omsOrderMapper.getByOrderSn(returnApplyDto.getOrderSn());
        if (byOrderSn == null) {
            throw new ApiException("未找到订单信息");
        }
        OmsOrderItemExample example = new OmsOrderItemExample();
        example.createCriteria().andOrderIdEqualTo(byOrderSn.getId());
        List<OmsOrderItem> omsOrderItems = orderItemMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(omsOrderItems)) {
            throw new ApiException("未找到订单商品信息");
        }
        if (StringUtils.isEmpty(returnApplyDto.getSkuIds())) {
            throw new ApiException("未找到订单商品信息");
        }
        List<OmsOrderReturnApply> returnApplyList = new ArrayList<>();

        Arrays.stream(returnApplyDto.getSkuIds().split(",")).forEach(skuId -> {
            OmsOrderItem orderItem = omsOrderItems.stream().filter(omsOrderItem -> omsOrderItem.getProductSkuId().equals(Long.valueOf(skuId)))
                .findFirst().orElseThrow(() -> new ApiException("未找到订单商品信息"));
            PmsProduct pmsProduct = productMapper.selectByPrimaryKey(orderItem.getProductId());

            if (pmsProduct == null) {
                throw new ApiException("未找到商品信息");
            }
            OmsOrderReturnApply returnApply = new OmsOrderReturnApply();
            returnApply.setOrderId(byOrderSn.getId());
            returnApply.setProductId(pmsProduct.getId());
            returnApply.setOrderItemId(orderItem.getId());
            returnApply.setOrderSn(byOrderSn.getOrderSn());
            returnApply.setCreateTime(new Date());
            returnApply.setMemberUsername(byOrderSn.getMemberUsername());
            returnApply.setReturnAmount(BigDecimal.ZERO);
            returnApply.setStatus(2);
            returnApply.setHandleTime(new Date());
            returnApply.setProductPic(pmsProduct.getPic());
            returnApply.setProductName(pmsProduct.getName());
            if (pmsProduct.getBrandId() != null) {
                returnApply.setProductBrand(brandMapper.selectByPrimaryKey(pmsProduct.getBrandId()).getName());
            }
            returnApply.setProductAttr(orderItem.getProductAttr());
            returnApply.setProductCount(orderItem.getProductQuantity());
            returnApply.setProductPrice(orderItem.getProductPrice());
            returnApply.setProductRealPrice(orderItem.getRealAmount());
            returnApply.setReason(returnApplyDto.getReason());
            returnApplyList.add(returnApply);
        });

        if (!CollectionUtils.isEmpty(returnApplyList)) {
            for (OmsOrderReturnApply omsOrderReturnApply : returnApplyList) {
                returnApplyMapper.insert(omsOrderReturnApply);

            }
        }


    }


    /**
     * 生成18位退款订单编号:8位日期+2位平台号码+2位支付方式+6位以上自增id
     */
    private String generateOrderSn(OmsOrderReturnApply returnApply) {
        return String.valueOf(idGeneratorService.generatorId("refund"));
    }
}
