package com.yonge.cooleshow.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.google.common.collect.Lists;
import com.ym.mec.common.dto.SchoolDto;
import com.ym.mec.common.entity.GoodsSubStockModel;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.entity.OrderSkuSync;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.web.WebFeignService;
import com.yonge.cooleshow.admin.dao.OmsOrderDao;
import com.yonge.cooleshow.admin.dao.OmsOrderOperateHistoryDao;
import com.yonge.cooleshow.admin.dto.*;
import com.yonge.cooleshow.admin.dto.search.OrderStatisticalSearch;
import com.yonge.cooleshow.admin.dto.search.UserStatisticalSearch;
import com.yonge.cooleshow.admin.enums.OrderStatisticalEnum;
import com.yonge.cooleshow.admin.service.*;
import com.yonge.cooleshow.admin.wrapper.OmsOrderWrapper;
import com.yonge.cooleshow.mbg.mapper.*;
import com.yonge.cooleshow.mbg.model.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.WeekFields;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 订单管理Service实现类
 * Created by macro on 2018/10/11.
 */
@Service
@Slf4j
public class OmsOrderServiceImpl implements OmsOrderService {
    @Autowired
    private OmsOrderMapper orderMapper;
    @Autowired
    private OmsOrderDao orderDao;
    @Autowired
    private OmsOrderOperateHistoryDao orderOperateHistoryDao;
    @Autowired
    private OmsOrderOperateHistoryMapper orderOperateHistoryMapper;

    @Autowired
    private OmsOrderReturnApplyService orderReturnApplyService;

    @Autowired
    private PmsProductService productService;

    @Autowired
    private OmsOrderItemMapper omsOrderItemMapper;

    @Autowired
    private PmsProductSkuStockRecordMapper pmsProductSkuStockRecordMapper;

    @Autowired
    private WebFeignService webFeignService;

    @Autowired
    private PmsSkuStockService pmsSkuStockService;


    @Autowired
    private OmsOrderSettingMapper orderSettingMapper;

    @Autowired
    private UmsMemberService umsMemberService;

    @Autowired
    private PmsSkuStockMapper skuStockMapper;

    @Autowired
    private PmsProductMapper pmsProductMapper;



    @Override
    public List<OrderVo> list(OmsOrderQueryParam queryParam, Integer pageSize, Integer pageNum) {
        PageHelper.startPage(pageNum, pageSize);
        List<OrderVo> list = orderDao.getList(queryParam);

        // 过滤出学校订单，查询学校信息
        List<Integer> schoolIds = list.stream().filter(o->o.getSourceType() ==2).map(e->e.getMemberId().intValue()).collect(Collectors.toList());
        for (OrderVo orderVo : list) {
            if (orderVo.getSourceType() == 2 ||orderVo.getSourceType() == 4) {
                orderVo.setPhone("");
                orderVo.setUsername("");
            }
        }
        if (!CollectionUtils.isEmpty(schoolIds)) {
            try {
                HttpResponseResult<List<SchoolDto>> schoolDtoHttpResponseResult = webFeignService.schoolListByIds(schoolIds);
                if (schoolDtoHttpResponseResult.getCode() == 200 && schoolDtoHttpResponseResult.getStatus()) {
                    Map<Integer, SchoolDto> schoolDtoMap = schoolDtoHttpResponseResult.getData().stream().collect(Collectors.toMap(SchoolDto::getId, o -> o));
                    list.forEach(o->{
                        if (o.getSourceType() == 2) {
                            SchoolDto schoolDto = schoolDtoMap.get(o.getMemberId().intValue());
                            if (schoolDto != null) {
                                o.setUsername(schoolDto.getName());
                            } else {
                                o.setUsername("");
                            }
                        }
                    });
                }
            } catch (Exception e) {
                log.error("查询学校信息失败", e);
            }

        }


        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int delivery(List<OmsOrderDeliveryParam> deliveryParamList) {
        //批量发货
        int count = orderDao.delivery(deliveryParamList);

        // 扣减库存
        //库存扣减规则更新
        //扣减顺序：内部库存>税务库存>进货批次
        //先把所有批次的内部库存扣完，再扣税务库存，例如：
        //第一批   内部0     税务10
        //第二批   内部10   税务10
        //这个时候发货10个商品，扣减的是第二批的内部10，再发货10个商品扣减的是第一批的税务10

        // 查询出所有订单关联的skuId
        // 订单ID集合
        List<Long> orderIds = deliveryParamList.stream().map(OmsOrderDeliveryParam::getOrderId).collect(Collectors.toList());
        updateSkuStock( orderIds);


        //添加操作记录
        List<OmsOrderOperateHistory> operateHistoryList = deliveryParamList.stream()
            .map(omsOrderDeliveryParam -> {
                OmsOrderOperateHistory history = new OmsOrderOperateHistory();
                history.setOrderId(omsOrderDeliveryParam.getOrderId());
                history.setCreateTime(new Date());
                history.setOperateMan("后台管理员");
                history.setOrderStatus(2);
                history.setNote("完成发货");
                return history;
            }).collect(Collectors.toList());
        orderOperateHistoryDao.insertList(operateHistoryList);

        return count;
    }

    /**
     * 库存更新
     */
//    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSkuStock(List<Long> orderIds) {

        // 查询订单详情
        OmsOrderItemExample example = new OmsOrderItemExample();
        example.createCriteria().andOrderIdIn(orderIds);
        List<OmsOrderItem> omsOrderItems = omsOrderItemMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(omsOrderItems)) {
            throw new BizException("订单详情不存在");
        }
        // skuID 商品名称 map
        Map<Long,String > skuMap = omsOrderItems.stream().collect(Collectors.toMap(OmsOrderItem::getProductSkuId, OmsOrderItem::getProductName,(o1, o2)->o1));

        // skuID集合
        List<Long> skuIds = omsOrderItems.stream().map(OmsOrderItem::getProductSkuId).distinct().collect(Collectors.toList());
        // 查询库存
        PmsProductSkuStockRecordExample stockRecordExample = new PmsProductSkuStockRecordExample();
        stockRecordExample.createCriteria().andProductSkuIdIn(skuIds);
        List<PmsProductSkuStockRecord> productSkuStockRecords = pmsProductSkuStockRecordMapper.selectByExample(stockRecordExample);
        if (CollectionUtils.isEmpty(productSkuStockRecords)) {
            ArrayList<String> list = new ArrayList<>(skuMap.values());
            throw new BizException(list.get(0) +"库存不足");
        }
        //id 集合
        List<Long> stockRecordIds = productSkuStockRecords.stream().map(PmsProductSkuStockRecord::getId).collect(Collectors.toList());

        // 根据ID锁定库存
        pmsProductSkuStockRecordMapper.lockStock(stockRecordIds);

        // 根据 skuID 分组
        Map<Long, List<PmsProductSkuStockRecord>> skuStockRecordMap = productSkuStockRecords.stream().collect(Collectors.groupingBy(PmsProductSkuStockRecord::getProductSkuId));
        // 扣减库存

        Map<Long,PmsProductSkuStockRecord> updateMap = new HashMap<>();
        Map<String, List<OmsOrderItem>> map = omsOrderItems.stream().collect(Collectors.groupingBy(o -> o.getOrderSn()));
        List<OrderSkuSync> skuSyncList = new ArrayList<>();
        List<OmsOrderItem> updateItems = new ArrayList<>();
        map.forEach((sn,v) -> {
            OrderSkuSync skuSync = new OrderSkuSync();
            skuSync.setOrderNo(sn);
            List<OrderSkuSync.SkuSync> skuSyncs = new ArrayList<>();
            skuSync.setSkuSyncList(skuSyncs);
            v.stream().forEach(o -> {

                List<PmsProductSkuStockRecord> skuStockRecords = skuStockRecordMap.get(o.getProductSkuId());
                if (CollectionUtils.isEmpty(skuStockRecords)) {
                    throw new BizException(skuMap.get(o.getProductSkuId()) +"库存不足");
                }
                List<OrderSkuSync.SkuSync> itemSkus = new ArrayList<>();
                int count1 = o.getProductQuantity();
                // 扣减内部库存
                for (PmsProductSkuStockRecord skuStockRecord : skuStockRecords) {
                    if (count1 <= 0) {
                        break;
                    }
                    if (skuStockRecord.getInternalStock() > 0) {
                        OrderSkuSync.SkuSync sync = new OrderSkuSync.SkuSync();
                        sync.setSku(skuStockRecord.getProductSkuId());
                        sync.setType("INTERNAL");


                        int stock = skuStockRecord.getInternalStock() - skuStockRecord.getInternalSaleStock();
                        if (stock > 0) {
                            sync.setPrice(skuStockRecord.getPrice());
                            sync.setRecordId(skuStockRecord.getId());

                            if (stock >= count1) {
                                skuStockRecord.setInternalSaleStock(skuStockRecord.getInternalSaleStock() + count1);
                                sync.setCount(count1);
                                count1 = 0;
                            } else {
                                count1 = count1 - stock;
                                skuStockRecord.setInternalSaleStock(skuStockRecord.getInternalStock());
                                sync.setCount(stock);
                            }
                            updateMap.put(skuStockRecord.getId(),skuStockRecord);
                            skuSyncs.add(sync);
                            itemSkus.add(sync);
                        }
                    }
                }
                // 扣减税务库存
                for (PmsProductSkuStockRecord skuStockRecord : skuStockRecords) {
                    if (count1 <= 0) {
                        break;
                    }
                    if (skuStockRecord.getTaxStock() > 0) {

                        OrderSkuSync.SkuSync sync = new OrderSkuSync.SkuSync();
                        sync.setSku(skuStockRecord.getProductSkuId());
                        sync.setPrice(skuStockRecord.getPrice());
                        sync.setRecordId(skuStockRecord.getId());
                        sync.setType("TAX");

                        int stock = skuStockRecord.getTaxStock() - skuStockRecord.getTaxSaleStock();
                        if (stock > 0) {
                            if (stock >= count1) {
                                skuStockRecord.setTaxSaleStock(skuStockRecord.getTaxSaleStock() + count1);
                                sync.setCount(count1);
                                count1 = 0;
                            } else {
                                count1 = count1 - stock;
                                skuStockRecord.setTaxSaleStock(skuStockRecord.getTaxStock());
                                sync.setCount(stock);
                            }
                            updateMap.put(skuStockRecord.getId(),skuStockRecord);
                            skuSyncs.add(sync);
                            itemSkus.add(sync);
                        }
                    }
                }
                if (count1 > 0) {
                    throw new BizException(skuMap.get(o.getProductSkuId())+"库存不足");
                }
                if (CollUtil.isNotEmpty(itemSkus)) {

                    OmsOrderItem orderItem = new OmsOrderItem();
                    orderItem.setId(o.getId());
                    orderItem.setStockJson(JSON.toJSONString(itemSkus));
                    updateItems.add(orderItem);
                }
            });
            skuSyncList.add(skuSync);
        });
        for (Map.Entry<Long, PmsProductSkuStockRecord> entry : updateMap.entrySet()) {
            PmsProductSkuStockRecord v = entry.getValue();
            PmsProductSkuStockRecord update = new PmsProductSkuStockRecord();
            update.setId(v.getId());
            update.setInternalSaleStock(v.getInternalSaleStock());
            update.setTaxSaleStock(v.getTaxSaleStock());
            pmsProductSkuStockRecordMapper.updateByPrimaryKeySelective(update);
        }
        // 更新订单详情
        for (OmsOrderItem updateItem : updateItems) {
            omsOrderItemMapper.updateByPrimaryKeySelective(updateItem);
        }

        // 调用三方发货通知接口
        OmsOrderExample orderExample = new OmsOrderExample();
        orderExample.createCriteria().andIdIn(orderIds);
        try {
            HttpResponseResult httpResponseResult = webFeignService.updateShippedStatus(skuSyncList);
            if (httpResponseResult.getCode() != 200 && !httpResponseResult.getStatus()) {
                throw new BizException("发货通知失败:"+httpResponseResult.getMsg());
            }
        } catch (Exception e) {
            throw new BizException("发货通知失败");
        }

        // 调用三方库存更新接口
        syncSkuStock(skuIds);
    }

    @Override
    public void syncSkuStock(List<Long> skuIds) {
        Map<Long, PmsSkuStock> skuStockMap = pmsSkuStockService.getMapByIds(skuIds);
        List<GoodsSubStockModel> modelList = skuStockMap.values().stream().map(skuStock -> {
            GoodsSubStockModel stockModel = new GoodsSubStockModel();
            stockModel.setMallGoodsId(skuStock.getProductId().intValue());
            stockModel.setSku(skuStock.getId().intValue());
            stockModel.setStock(skuStock.getStock());
            return stockModel;
        }).collect(Collectors.toList());
        try {
            log.info("同步商品库存:{}",modelList);
            HttpResponseResult<Boolean> booleanHttpResponseResult = webFeignService.goodsStockCountSynchronize(modelList);
            if (booleanHttpResponseResult.getCode() != 200 && !booleanHttpResponseResult.getStatus()) {
                throw new BizException("库存同步失败");
            }
        }catch (Exception e) {
            throw new BizException("库存同步失败");
        }

    }

    @Override
    public int close(List<Long> ids, String note) {
        OmsOrder record = new OmsOrder();
        record.setStatus(4);
        OmsOrderExample example = new OmsOrderExample();
        example.createCriteria().andDeleteStatusEqualTo(0).andIdIn(ids);

        List<OmsOrder> omsOrders = orderMapper.selectByExample(example);
        long count1 = omsOrders.stream()
                               .filter(omsOrder -> omsOrder.getStatus() != 0 && omsOrder.getStatus() != 6)
                               .count();
        if (count1 != 0) {
            throw new BizException("订单不处于待支付状态，不能关闭");
        }
        int count = orderMapper.updateByExampleSelective(record, example);
        List<OmsOrderOperateHistory> historyList = ids.stream().map(orderId -> {
            OmsOrderOperateHistory history = new OmsOrderOperateHistory();
            history.setOrderId(orderId);
            history.setCreateTime(new Date());
            history.setOperateMan("后台管理员");
            history.setOrderStatus(4);
            history.setNote("订单关闭:"+note);
            return history;
        }).collect(Collectors.toList());
        orderOperateHistoryDao.insertList(historyList);
        return count;
    }

    @Override
    public int delete(List<Long> ids) {
        OmsOrder record = new OmsOrder();
        record.setDeleteStatus(1);
        OmsOrderExample example = new OmsOrderExample();
        example.createCriteria().andDeleteStatusEqualTo(0).andIdIn(ids);
        return orderMapper.updateByExampleSelective(record, example);
    }

    @Override
    public OmsOrderDetail detail(Long id) {
        OmsOrderDetail detail = orderDao.getDetail(id);

        // 获取订单详情
        OmsOrderItemExample example = new OmsOrderItemExample();
        example.createCriteria().andOrderIdEqualTo(id);
        List<OmsOrderItem> omsOrderItems = omsOrderItemMapper.selectByExample(example);
        detail.setOrderItemList(omsOrderItems);

        // 学校订单查询学校信息
        try {
            if (detail.getSourceType() == 2) {
                HttpResponseResult<List<SchoolDto>> schoolDtoHttpResponseResult = webFeignService.schoolListByIds(Lists.newArrayList(detail.getMemberId().intValue()));
                if (schoolDtoHttpResponseResult.getCode() == 200 && schoolDtoHttpResponseResult.getStatus()) {
                    List<SchoolDto> data = schoolDtoHttpResponseResult.getData();
                    if (!CollectionUtils.isEmpty(data)) {
                        detail.setMemberUsername(data.get(0).getName());
                    }
                }
            }
        } catch (Exception e) {
            log.error("查询学校信息失败", e);
        }

        return detail;
    }

    @Override
    public int updateReceiverInfo(OmsReceiverInfoParam receiverInfoParam) {
        OmsOrder order = new OmsOrder();
        order.setId(receiverInfoParam.getOrderId());
        order.setReceiverName(receiverInfoParam.getReceiverName());
        order.setReceiverPhone(receiverInfoParam.getReceiverPhone());
        order.setReceiverPostCode(receiverInfoParam.getReceiverPostCode());
        order.setReceiverDetailAddress(receiverInfoParam.getReceiverDetailAddress());
        order.setReceiverProvince(receiverInfoParam.getReceiverProvince());
        order.setReceiverCity(receiverInfoParam.getReceiverCity());
        order.setReceiverRegion(receiverInfoParam.getReceiverRegion());
        order.setModifyTime(new Date());
        int count = orderMapper.updateByPrimaryKeySelective(order);
        //插入操作记录
        OmsOrderOperateHistory history = new OmsOrderOperateHistory();
        history.setOrderId(receiverInfoParam.getOrderId());
        history.setCreateTime(new Date());
        history.setOperateMan("后台管理员");
        history.setOrderStatus(receiverInfoParam.getStatus());
        history.setNote("修改收货人信息");
        orderOperateHistoryMapper.insert(history);
        return count;
    }

    @Override
    public int updateMoneyInfo(OmsMoneyInfoParam moneyInfoParam) {
        OmsOrder order = new OmsOrder();
        order.setId(moneyInfoParam.getOrderId());
        order.setFreightAmount(moneyInfoParam.getFreightAmount());
        order.setDiscountAmount(moneyInfoParam.getDiscountAmount());
        order.setModifyTime(new Date());
        int count = orderMapper.updateByPrimaryKeySelective(order);
        //插入操作记录
        OmsOrderOperateHistory history = new OmsOrderOperateHistory();
        history.setOrderId(moneyInfoParam.getOrderId());
        history.setCreateTime(new Date());
        history.setOperateMan("后台管理员");
        history.setOrderStatus(moneyInfoParam.getStatus());
        history.setNote("修改费用信息");
        orderOperateHistoryMapper.insert(history);
        return count;
    }

    @Override
    public int updateNote(Long id, String note, Integer status) {
        OmsOrder order = new OmsOrder();
        order.setId(id);
        order.setNote(note);
        order.setModifyTime(new Date());
        int count = orderMapper.updateByPrimaryKeySelective(order);
        OmsOrderOperateHistory history = new OmsOrderOperateHistory();
        history.setOrderId(id);
        history.setCreateTime(new Date());
        history.setOperateMan("后台管理员");
        history.setOrderStatus(status);
        history.setNote("修改备注信息："+note);
        orderOperateHistoryMapper.insert(history);
        return count;
    }

    @Override
    public void userStatistical(UserStatisticalSearch search) {
        // orderDao.userStatistical(search);
    }

    @Override
    public List<Map<String,Object>> userOrderInfoList(Map<String, Object> param) {
        return orderDao.getUserOrderInfoList(param);
    }

    @Override
    public HomeStatistical countStatistical() {
        // 订单状态
        HomeStatistical homeStatistical = orderDao.selectOrderStatusNum();
        if (homeStatistical == null) {
            homeStatistical = new HomeStatistical();
        }

        // 缺货登记   库存紧张
        HomeStatistical stock = productService.countLowStockCount();
        if (stock == null) {
            stock = new HomeStatistical();
        }

        homeStatistical.setOutStock(stock.getOutStock());
        homeStatistical.setOutStockProduct(stock.getOutStockProduct());

        // 待处理退款申请  待处理退货订单
        HomeStatistical returnInfo = orderReturnApplyService.countReturnStatus();
        if (returnInfo == null) {
            returnInfo = new HomeStatistical();
        }

        homeStatistical.setWaitDisposeRefund(returnInfo.getWaitDisposeRefund());
        homeStatistical.setWaitDisposeRefundOrder(returnInfo.getWaitDisposeRefundOrder());

        // 已下架 已上架 全部商品
        HomeStatistical product = productService.countProductShowStatus();

        if (product == null) {
            product = new HomeStatistical();
        }

        homeStatistical.setShelves(product.getShelves());
        homeStatistical.setDelisted(product.getDelisted());
        homeStatistical.setProductNum(product.getProductNum());

        Date endTime = Date.from(LocalDateTime.now().atZone(ZoneId.systemDefault()).toInstant());
        LocalDateTime localDateTime = LocalDateTime.now().plusDays(-1);
        LocalDate localDate = localDateTime.toLocalDate();
        Date startTime = Date.from(localDate.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());

        // 今日、昨日 订单总数和销售总额
        List<OrderStatistical> orderStatistical = orderDao
                .selectOrderCountAndAmount(startTime, endTime, OrderStatisticalEnum.DAY.name());

        Map<String, List<OrderStatistical>> collect = orderStatistical.stream()
                              .collect(Collectors.groupingBy(OrderStatistical::getDate));
        OrderStatistical statistical = getOrderStatistical(collect, DateUtil.format(startTime, "yyyy-MM-dd"));
        homeStatistical.setYesterdayOrderNum(statistical.getOrderNum());
        homeStatistical.setYesterdayOrderAmount(statistical.getOrderAmount());

        statistical = getOrderStatistical(collect, DateUtil.format(endTime, "yyyy-MM-dd"));
        homeStatistical.setTodayOrderNum(statistical.getOrderNum());
        homeStatistical.setTodayOrderAmount(statistical.getOrderAmount());

        return homeStatistical;
    }

    @Override
    public HomeOrderStatistical orderStatistical(OrderStatisticalSearch search) {

        HomeOrderStatistical homeOrderStatistical = new HomeOrderStatistical();

        // 整理出时间数据
        LocalDate start = LocalDate.parse(search.getStartDate());
        Date startTime = Date.from(start.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
        LocalDate end = LocalDate.parse(search.getEndDate());
        LocalDateTime localDateTime = end.plusDays(1).atStartOfDay();
        Date endTime = Date.from( localDateTime.plusSeconds(-1).atZone(ZoneId.systemDefault()).toInstant());

        // 查询每天的订单数据
        List<OrderStatistical> orderStatistical = orderDao
                .selectOrderCountAndAmount(startTime, endTime, OrderStatisticalEnum.DAY.name());

        Map<String, List<OrderStatistical>> map = orderStatistical.stream()
                                      .collect(Collectors.groupingBy(OrderStatistical::getDate));

        // 整理统计数据
        List<OrderStatistical> statisticalList = new ArrayList<>();
        while (start.compareTo(end) <=0) {
            String date = start.format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            OrderStatistical statisticalInfo = getOrderStatistical(map, date);
            statisticalList.add(statisticalInfo);
            start = start.plusDays(1);
        }
        homeOrderStatistical.setOrderStatisticalList(statisticalList);

        // 本月订单

        LocalDate now = LocalDate.now();
        endTime =  Date.from( now.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
        startTime = Date.from( now.plusMonths(-1).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());
        // 查询统计数据
        orderStatistical = orderDao.selectOrderCountAndAmount(startTime, endTime, OrderStatisticalEnum.MONTH.name());

        map = orderStatistical.stream().collect(Collectors.groupingBy(OrderStatistical::getDate));

        // 本月订单统计数据
        String date = DateUtil.format(endTime, "yyyy-MM");
        OrderStatistical statistical = getOrderStatistical(map, date);
        homeOrderStatistical.setMonthOrderAmount(statistical.getOrderAmount());
        homeOrderStatistical.setMonthOrderCount(statistical.getOrderNum());

        // 上月订单统计数据
        date = DateUtil.format(startTime, "yyyy-MM");
        statistical = getOrderStatistical(map, date);

        // 月 同比 金额
        homeOrderStatistical.setMonthOrderAmountProportion(getProportion(homeOrderStatistical.getMonthOrderAmount(),
                                                                         statistical.getOrderAmount()));
        // 月同比 数量
        homeOrderStatistical.setMonthOrderCountProportion(getProportion(new BigDecimal(homeOrderStatistical.getMonthOrderCount()),
                                                          new BigDecimal(statistical.getOrderNum())));

        // 周数据
        startTime = Date.from( now.plusDays(-7).atStartOfDay().atZone(ZoneId.systemDefault()).toInstant());

        // 查询统计数据
        orderStatistical = orderDao.selectOrderCountAndAmount(startTime, endTime, OrderStatisticalEnum.WEEK.name());

        map = orderStatistical.stream().collect(Collectors.groupingBy(obj -> obj.getDate().substring(4)));

        // 本周订单统计数据
        WeekFields weekFields = WeekFields.ISO;
        int i = now.get(weekFields.weekOfWeekBasedYear());
        if (i <10) {
            date =  "0" + i;
        } else {
            date =  "" + i;
        }
        statistical = getOrderStatistical(map, date);
        homeOrderStatistical.setWeekOrderAmount(statistical.getOrderAmount());
        homeOrderStatistical.setWeekOrderCount(statistical.getOrderNum());

        // 上周订单统计数据
        i = now.plusDays(-7).get(weekFields.weekOfWeekBasedYear());
        if (i <10) {
            date =  "0" + i;
        } else {
            date =  "" + i;
        }
        statistical = getOrderStatistical(map, date);

        // 月 同比 金额
        homeOrderStatistical.setWeekOrderAmountProportion(getProportion(homeOrderStatistical.getWeekOrderAmount(),
                                                                         statistical.getOrderAmount()));
        // 月同比 数量
        homeOrderStatistical.setWeekOrderCountProportion(getProportion(new BigDecimal(homeOrderStatistical.getWeekOrderCount()),
                                                                        new BigDecimal(statistical.getOrderNum())));
        return homeOrderStatistical;

    }

    @Override
    public List<OrderDetailVo> queryOrderDetail(OmsOrderQueryParam queryParam) {
        return orderDao.queryOrderDetail(queryParam);
    }

    private Integer getProportion(BigDecimal now, BigDecimal last) {
        if (now.compareTo(BigDecimal.ZERO) == 0) {
            if(last.compareTo(BigDecimal.ZERO) == 0) {
                return 0;
            }  else {
                return -100;
            }
        } else {
            if(last.compareTo(BigDecimal.ZERO) == 0) {
                return 100;
            }  else {
                double monthAmount = now.doubleValue();
                double lastMonthAmount =last.doubleValue();
                double v = 100 * (monthAmount / lastMonthAmount - 1);
                return Double.valueOf(v).intValue();
            }
        }
    }

    private OrderStatistical getOrderStatistical(Map<String, List<OrderStatistical>> map, String date) {
        OrderStatistical statisticalInfo = new OrderStatistical();
        statisticalInfo.setDate(date);
        List<OrderStatistical> orderStatisticals = map.get(date);
        if (CollectionUtils.isEmpty(orderStatisticals)) {
            statisticalInfo.setOrderAmount(BigDecimal.ZERO);
            statisticalInfo.setOrderNum(0);
        } else {
            statisticalInfo.setOrderAmount(orderStatisticals.get(0).getOrderAmount());
            statisticalInfo.setOrderNum(orderStatisticals.get(0).getOrderNum());
        }
        return statisticalInfo;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void mecProductOrderCreate(OrderCreate order) {


        // 查询订单是否已存在
        OmsOrder detail = orderMapper.getByOrderSn(order.getOrderNo());
        if (detail != null) {
            throw new BizException("订单已存在，请勿重复提交");
        }
        UmsMember umsMember = null;
        // 学校订单和其他订单没有个人信息
        if (order.getPlatformType().equals("STUDENT")) {
            umsMember = umsMemberService.getById(order.getMemberId());
        }


        // 创建订单

        //设置自动收货天数
        List<OmsOrderSetting> orderSettings = orderSettingMapper.selectByExample(new OmsOrderSettingExample());


        detail = new OmsOrder();
        detail.setOrchestraId(order.getOrchestraId());
        detail.setOrderFormType(order.getOrderFormType());
        detail.setMemberId(order.getMemberId());
        detail.setPlatformType(order.getPlatformType());
        detail.setOrderSn(order.getOrderNo());
        detail.setCreateTime(new Date());
        if (umsMember !=null) {
            detail.setMemberUsername(umsMember.getUsername());
        }
        detail.setTotalAmount(order.getTotalAmount());
        detail.setPayAmount(order.getTotalAmount());
        detail.setFreightAmount(BigDecimal.ZERO);
        detail.setPromotionAmount(BigDecimal.ZERO);
        detail.setIntegrationAmount(BigDecimal.ZERO);
        detail.setCouponAmount(BigDecimal.ZERO);
        detail.setDiscountAmount(BigDecimal.ZERO);
        detail.setPayType(order.getPayType());
        detail.setConfirmStatus(0);
        detail.setSourceType(order.getSourceType());
        detail.setStatus(order.getStatus());
        detail.setOrderType(0);
        if (CollUtil.isNotEmpty(orderSettings)) {
            detail.setAutoConfirmDay(orderSettings.get(0).getConfirmOvertime());
        }
        detail.setReceiverName(order.getReceiverName());
        detail.setReceiverPhone(order.getReceiverPhone());
        detail.setReceiverPostCode(order.getReceiverPostCode());
        detail.setReceiverProvince(order.getReceiverProvince());
        detail.setReceiverCity(order.getReceiverCity());
        detail.setReceiverRegion(order.getReceiverRegion());
        detail.setReceiverDetailAddress(order.getReceiverDetailAddress());
        detail.setDeleteStatus(0);
        detail.setPaymentTime(new Date());
        detail.setAfterSale(1);
        detail.setModifyTime(new Date());
        orderMapper.insert(detail);

        List<OmsOrderItem> orderItemList = new ArrayList<>();
        // 创建订单详情
        for (OrderCreate.OrderItem orderItem : order.getOrderItemList()) {

            // 获取商品信息
            PmsSkuStock pmsSkuStock = skuStockMapper.selectByPrimaryKey(orderItem.getProductSkuId());
            if (pmsSkuStock == null) {
                throw new BizException("商品SKU已删除，请重新申请");
            }
            PmsProduct pmsProduct = pmsProductMapper.selectByPrimaryKey(pmsSkuStock.getProductId());

            if (pmsProduct == null) {
                throw new BizException("商品货号已变更，请重新申请");
            }

            OmsOrderItem omsOrderItem = new OmsOrderItem();
            omsOrderItem.setOrderId(detail.getId());
            omsOrderItem.setOrderSn(detail.getOrderSn());
            omsOrderItem.setProductId(pmsProduct.getId());
            omsOrderItem.setProductSn(pmsProduct.getProductSn());
            omsOrderItem.setProductPic(pmsProduct.getPic());
            omsOrderItem.setProductName(pmsProduct.getName());
            omsOrderItem.setProductBrand(pmsProduct.getBrandName());
            omsOrderItem.setProductPrice(pmsProduct.getPrice());
            omsOrderItem.setProductQuantity(orderItem.getProductQuantity());
            omsOrderItem.setProductSkuId(pmsSkuStock.getId());
            omsOrderItem.setProductSkuCode(pmsSkuStock.getSkuCode());
            omsOrderItem.setProductCategoryId(pmsProduct.getProductCategoryId());
            omsOrderItem.setPromotionAmount(BigDecimal.ZERO);
            omsOrderItem.setCouponAmount(BigDecimal.ZERO);
            omsOrderItem.setIntegrationAmount(BigDecimal.ZERO);
            omsOrderItem.setRealAmount(orderItem.getRealAmount());
            omsOrderItem.setProductAttr(pmsSkuStock.getSpData());
            omsOrderItem.setReturnStatus(-1);

            orderItemList.add(omsOrderItem);
        }
        // 判断是否存在被删除的商品 和sku 如果有被删除的商品或者sku则不允许下单
        List<Long> productIds = orderItemList.stream().map(OmsOrderItem::getProductId).collect(Collectors.toList());
        PmsProductExample pmsProductExample = new PmsProductExample();
        pmsProductExample.createCriteria().andIdIn(productIds).andDeleteStatusEqualTo(1);
        List<PmsProduct> pmsProducts = pmsProductMapper.selectByExample(pmsProductExample);
        if (CollUtil.isNotEmpty(pmsProducts)) {
            throw new BizException("商品已下架，请重新申请");
        }
        List<Long> skuIds = orderItemList.stream().map(OmsOrderItem::getProductSkuId).collect(Collectors.toList());
        PmsSkuStockExample pmsSkuStockExample = new PmsSkuStockExample();
        pmsSkuStockExample.createCriteria().andIdIn(skuIds).andDeleteStatusEqualTo(1);
        List<PmsSkuStock> pmsSkuStocks = skuStockMapper.selectByExample(pmsSkuStockExample);
        if (CollUtil.isNotEmpty(pmsSkuStocks)) {
            throw new BizException("商品已下架，请重新申请");
        }

        // 保存数据
        if (!CollectionUtils.isEmpty(orderItemList)) {
            omsOrderItemMapper.insertList(orderItemList);
        }

        // 关闭需要关闭的订单
        if(!CollectionUtils.isEmpty(order.getCloseOrderNoList())) {

            OmsOrderExample example = new OmsOrderExample();
            example.createCriteria().andOrderSnIn(order.getCloseOrderNoList());
            List<OmsOrder> omsOrders = orderMapper.selectByExample(example);
            if (!CollectionUtils.isEmpty(omsOrders)) {
                List<Long> orderIds = omsOrders.stream().map(OmsOrder::getId).collect(Collectors.toList());
                OmsOrderWrapper.OrderStatusUpdate orderStatusUpdate = new OmsOrderWrapper.OrderStatusUpdate();
                orderStatusUpdate.setIds(orderIds);
                orderStatusUpdate.setStatus(4);
                updateStatus(orderStatusUpdate);
            }

        }

        // 如果订单状态是已发货，扣减库存
        if (order.getStatus() ==2 || order.getStatus() == 3) {

            updateSkuStock(Lists.newArrayList(detail.getId()));
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean productUpdateOrderStatus(String orderNo, Integer status) {

        OmsOrder detail = orderMapper.getByOrderSn(orderNo);
        if (detail == null) {
            throw new BizException("订单不存在");
        }
        if (status == 4) {
            OmsOrderExample example = new OmsOrderExample();
            example.createCriteria().andOrderSnEqualTo(orderNo);
            List<OmsOrder> omsOrders = orderMapper.selectByExample(example);
            if (!CollectionUtils.isEmpty(omsOrders)) {
                List<Long> orderIds = omsOrders.stream().map(OmsOrder::getId).collect(Collectors.toList());
                OmsOrderWrapper.OrderStatusUpdate orderStatusUpdate = new OmsOrderWrapper.OrderStatusUpdate();
                orderStatusUpdate.setIds(orderIds);
                orderStatusUpdate.setStatus(4);
                updateStatus(orderStatusUpdate);
            }
        } else {
            OmsOrder omsOrder = new OmsOrder();
            omsOrder.setId(detail.getId());
            omsOrder.setStatus(status);
            omsOrder.setModifyTime(new Date());
            orderMapper.updateByPrimaryKeySelective(omsOrder);
        }


        return true;
    }

    // 直接修改订单状态为完成/关闭
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateStatus(OmsOrderWrapper.OrderStatusUpdate orderStatusUpdate) {


        // 如果时关单 并且 是已发货状态 则需要退回库存
        OmsOrderExample orderExample = new OmsOrderExample();
        orderExample.createCriteria().andIdIn(orderStatusUpdate.getIds());
        List<OmsOrder> omsOrders = orderMapper.selectByExample(orderExample);
        List<Long> orderIds = omsOrders.stream().filter(o -> o.getStatus() == 2 || o.getStatus() == 3)
            .map(OmsOrder::getId).collect(Collectors.toList());
        if (orderStatusUpdate.getStatus() ==4 && !CollectionUtils.isEmpty(orderIds)) {
            OmsOrderItemExample orderItemExample = new OmsOrderItemExample();
            orderItemExample.createCriteria().andOrderIdIn(orderIds);
            List<OmsOrderItem> omsOrderItems = omsOrderItemMapper.selectByExample(orderItemExample);
            List<String> stockJsons = omsOrderItems.stream().map(OmsOrderItem::getStockJson)
                .filter(o -> !StringUtils.isEmpty(o)).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(stockJsons)) {
                List<OrderSkuSync.SkuSync> skuSyncList = new ArrayList<>();
                for (String stockJson : stockJsons) {
                    List<OrderSkuSync.SkuSync> skuSyncs = JSON.parseArray(stockJson, OrderSkuSync.SkuSync.class);
                    skuSyncList.addAll(skuSyncs);
                }
                Map<Long, Integer> internalMap = skuSyncList.stream().filter(o -> "INTERNAL".equals(o.getType()))
                    .collect(Collectors.groupingBy(OrderSkuSync.SkuSync::getRecordId, Collectors.summingInt(OrderSkuSync.SkuSync::getCount)));
                Map<Long, Integer> taxMap = skuSyncList.stream().filter(o -> "TAX".equals(o.getType()))
                    .collect(Collectors.groupingBy(OrderSkuSync.SkuSync::getRecordId, Collectors.summingInt(OrderSkuSync.SkuSync::getCount)));
                List<PmsProductSkuStockRecord> updateList = new ArrayList<>();
                for (Map.Entry<Long, Integer> entry : internalMap.entrySet()) {
                    PmsProductSkuStockRecord skuStockRecord = new PmsProductSkuStockRecord();
                    skuStockRecord.setId(entry.getKey());
                    skuStockRecord.setInternalSaleStock(entry.getValue());
                    updateList.add(skuStockRecord);
                }
                for (Map.Entry<Long, Integer> entry : taxMap.entrySet()) {
                    PmsProductSkuStockRecord skuStockRecord = new PmsProductSkuStockRecord();
                    skuStockRecord.setId(entry.getKey());
                    skuStockRecord.setTaxSaleStock(entry.getValue());
                    updateList.add(skuStockRecord);
                }
                for (PmsProductSkuStockRecord pmsProductSkuStockRecord : updateList) {
                    pmsProductSkuStockRecordMapper.updateStock(pmsProductSkuStockRecord,true);
                }
                List<Long> skuIds = skuSyncList.stream().map(OrderSkuSync.SkuSync::getSku).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(skuIds)) {
                    syncSkuStock(skuIds);
                }


            }
        }

        OmsOrder record = new OmsOrder();
        record.setStatus(orderStatusUpdate.getStatus());
        OmsOrderExample example = new OmsOrderExample();
        example.createCriteria().andIdIn(orderStatusUpdate.getIds());
        orderMapper.updateByExampleSelective(record, example);

        //添加操作记录
        List<OmsOrderOperateHistory> operateHistoryList = orderStatusUpdate.getIds().stream()
            .map(id -> {
                OmsOrderOperateHistory history = new OmsOrderOperateHistory();
                history.setOrderId(id);
                history.setCreateTime(new Date());
                history.setOperateMan("后台管理员");
                history.setOrderStatus(orderStatusUpdate.getStatus());
                history.setNote("修改订单状态");
                return history;
            }).collect(Collectors.toList());
        return orderOperateHistoryDao.insertList(operateHistoryList);
    }
}
