package com.yonge.cooleshow.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.github.pagehelper.PageHelper;
import com.ym.mec.common.dto.PmsProductDto;
import com.ym.mec.common.dto.PmsProductQueryParamDto;
import com.ym.mec.common.entity.GoodsSubModel;
import com.ym.mec.common.entity.GoodsSubStockModel;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.web.WebFeignService;
import com.yonge.cooleshow.admin.dao.*;
import com.yonge.cooleshow.admin.dto.HomeStatistical;
import com.yonge.cooleshow.admin.dto.PmsProductParam;
import com.yonge.cooleshow.admin.dto.PmsProductQueryParam;
import com.yonge.cooleshow.admin.dto.PmsProductResult;
import com.yonge.cooleshow.admin.service.PmsBrandService;
import com.yonge.cooleshow.admin.service.PmsProductCategoryService;
import com.yonge.cooleshow.admin.service.PmsProductService;
import com.yonge.cooleshow.mbg.dto.PmsSkuStockDto;
import com.yonge.cooleshow.mbg.mapper.*;
import com.yonge.cooleshow.mbg.model.*;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 商品管理Service实现类
 * Created by macro on 2018/4/26.
 */
@Service
@Slf4j
public class PmsProductServiceImpl implements PmsProductService {
    private static final Logger LOGGER = LoggerFactory.getLogger(PmsProductServiceImpl.class);
    @Autowired
    private PmsProductMapper productMapper;
    @Autowired
    private PmsMemberPriceDao memberPriceDao;
    @Autowired
    private PmsMemberPriceMapper memberPriceMapper;
    @Autowired
    private PmsProductLadderDao productLadderDao;
    @Autowired
    private PmsProductLadderMapper productLadderMapper;
    @Autowired
    private PmsProductFullReductionDao productFullReductionDao;
    @Autowired
    private PmsProductFullReductionMapper productFullReductionMapper;
    @Autowired
    private PmsSkuStockDao skuStockDao;
    @Autowired
    private PmsSkuStockMapper skuStockMapper;
    @Autowired
    private PmsProductAttributeValueDao productAttributeValueDao;
    @Autowired
    private PmsProductAttributeValueMapper productAttributeValueMapper;
    @Autowired
    private CmsSubjectProductRelationDao subjectProductRelationDao;
    @Autowired
    private CmsSubjectProductRelationMapper subjectProductRelationMapper;
    @Autowired
    private CmsPreferenceAreaProductRelationDao prefrenceAreaProductRelationDao;
    @Autowired
    private CmsPrefrenceAreaProductRelationMapper prefrenceAreaProductRelationMapper;
    @Autowired
    private PmsProductDao productDao;
    @Autowired
    private PmsProductVertifyRecordDao productVertifyRecordDao;
    @Autowired
    private WebFeignService webFeignService;

    @Autowired
    private PmsBrandService brandService;

    @Autowired
    private PmsProductCategoryService productCategoryService;

    @Override
    public int create(PmsProductParam productParam) {
        if (productParam.getPrice().compareTo(BigDecimal.ZERO) <=0 ) {
            throw new BizException("商品价格不能小于等于0");
        }
        for (PmsSkuStock pmsSkuStock : productParam.getSkuStockList()) {
            if (pmsSkuStock.getPrice().compareTo(BigDecimal.ZERO) <= 0) {
                throw new BizException("商品价格不能小于等于0");
            }
        }
        int count;
        //创建商品
        productParam.setId(null);
        productParam.setVerifyStatus(1);
        productParam.setLockStock(0);
        productMapper.insertSelective(productParam);
        //根据促销类型设置价格：会员价格、阶梯价格、满减价格
        Long productId = productParam.getId();
        //会员价格
        relateAndInsertList(memberPriceDao, productParam.getMemberPriceList(), productId);
        //阶梯价格
        relateAndInsertList(productLadderDao, productParam.getProductLadderList(), productId);
        //满减价格
        relateAndInsertList(productFullReductionDao, productParam.getProductFullReductionList(), productId);
        //处理sku的编码
        handleSkuStockCode(productParam.getSkuStockList(),productId);
        //添加sku库存信息
        relateAndInsertList(skuStockDao, productParam.getSkuStockList(), productId);
        //添加商品参数,添加自定义商品规格
        relateAndInsertList(productAttributeValueDao, productParam.getProductAttributeValueList(), productId);
        //关联专题
        relateAndInsertList(subjectProductRelationDao, productParam.getSubjectProductRelationList(), productId);
        //关联优选
        relateAndInsertList(prefrenceAreaProductRelationDao, productParam.getPrefrenceAreaProductRelationList(), productId);
        count = 1;
        return count;
    }

    private void handleSkuStockCode(List<PmsSkuStock> skuStockList, Long productId) {
        if(CollectionUtils.isEmpty(skuStockList))return;
        for(int i=0;i<skuStockList.size();i++){
            PmsSkuStock skuStock = skuStockList.get(i);
            if(StringUtils.isEmpty(skuStock.getSkuCode())){
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                StringBuilder sb = new StringBuilder();
                //日期
                sb.append(sdf.format(new Date()));
                //四位商品id
                sb.append(String.format("%04d", productId));
                //三位索引id
                sb.append(String.format("%03d", i+1));
                skuStock.setSkuCode(sb.toString());
            }
        }
    }

    @Override
    public PmsProductResult getUpdateInfo(Long id) {
        PmsProductResult updateInfo = productDao.getUpdateInfo(id);
        updateProductStock(Collections.singletonList(updateInfo));
        updateProductSkuStock(updateInfo.getSkuStockList());

        return updateInfo;
    }



    public List<PmsSkuStock> updateProductSkuStock(List<PmsSkuStock> skuStocks) {
        if (CollectionUtils.isEmpty(skuStocks)) {
            return new ArrayList<>();
        }
        // 从进货清单中 获取剩余库存数量
        List<Long> productIds = skuStocks.stream().map(PmsSkuStock::getProductId).distinct().collect(Collectors.toList());

        // 计算库存数据
        List<PmsSkuStockDto.PmsSkuStockStock> skuStockStocks = skuStockMapper.getSkuStockByProductIds(productIds);

        // Convert to map
        Map<Long, Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>>> map = skuStockStocks.stream()
            .collect(Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getProductId, Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getId)));

        // Merge
        for (PmsSkuStock skuStock : skuStocks) {
            Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>> skuStockStock = map.get(skuStock.getProductId());
            if (!CollectionUtils.isEmpty(skuStockStock)) {
                List<PmsSkuStockDto.PmsSkuStockStock> pmsSkuStockStocks = skuStockStock.get(skuStock.getId());
                if (CollectionUtils.isEmpty(pmsSkuStockStocks)) {
                    skuStock.setLockStock(0);
                    skuStock.setStock(0);
                    continue;
                }
                skuStock.setStock(pmsSkuStockStocks.stream().mapToInt(item -> item.getInternalStock() + item.getTaxStock()).sum());
                skuStock.setLockStock(0);
            }

        }
        return skuStocks;
    }

    public List<PmsProduct> updateProductStock(List<PmsProduct> pmsProducts) {
        if (CollectionUtils.isEmpty(pmsProducts)) {
            return new ArrayList<>();
        }
        // 从进货清单中 获取剩余库存数量
        List<Long> productIds = pmsProducts.stream().map(PmsProduct::getId).collect(Collectors.toList());

        // 计算库存数据
        List<PmsSkuStockDto.PmsSkuStockStock> skuStockStocks = skuStockMapper.getSkuStockByProductIds(productIds);

        // Convert to map
        Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>> skuStockStockMap = skuStockStocks.stream().collect(Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getProductId));

        // Merge
        for (PmsProduct pmsProduct : pmsProducts) {
            List<PmsSkuStockDto.PmsSkuStockStock> skuStockStock = skuStockStockMap.get(pmsProduct.getId());
            if (!CollectionUtils.isEmpty(skuStockStock)) {
                pmsProduct.setStock(skuStockStock.stream().mapToInt(PmsSkuStockDto.PmsSkuStockStock::getInternalStock).sum() + skuStockStock.stream().mapToInt(PmsSkuStockDto.PmsSkuStockStock::getTaxStock).sum());
                pmsProduct.setLockStock(0);
            } else {
                pmsProduct.setStock(0);
                pmsProduct.setLockStock(0);
            }
        }
        return pmsProducts;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int update(Long id, PmsProductParam productParam) {
        if (productParam.getPrice().compareTo(BigDecimal.ZERO) <=0 ) {
            throw new BizException("商品价格不能小于等于0");
        }
        for (PmsSkuStock pmsSkuStock : productParam.getSkuStockList()) {
            if (pmsSkuStock.getPrice().compareTo(BigDecimal.ZERO) <= 0) {
                throw new BizException("商品价格不能小于等于0");
            }
        }
        int count;
        //更新商品信息
        PmsProduct product = productParam;
        product.setId(id);
        productMapper.updateByPrimaryKeySelective(product);
        //会员价格
        PmsMemberPriceExample pmsMemberPriceExample = new PmsMemberPriceExample();
        pmsMemberPriceExample.createCriteria().andProductIdEqualTo(id);
        memberPriceMapper.deleteByExample(pmsMemberPriceExample);
        relateAndInsertList(memberPriceDao, productParam.getMemberPriceList(), id);
        //阶梯价格
        PmsProductLadderExample ladderExample = new PmsProductLadderExample();
        ladderExample.createCriteria().andProductIdEqualTo(id);
        productLadderMapper.deleteByExample(ladderExample);
        relateAndInsertList(productLadderDao, productParam.getProductLadderList(), id);
        //满减价格
        PmsProductFullReductionExample fullReductionExample = new PmsProductFullReductionExample();
        fullReductionExample.createCriteria().andProductIdEqualTo(id);
        productFullReductionMapper.deleteByExample(fullReductionExample);
        relateAndInsertList(productFullReductionDao, productParam.getProductFullReductionList(), id);
        //修改sku库存信息
        handleUpdateSkuStockList(id, productParam);
        //修改商品参数,添加自定义商品规格
        PmsProductAttributeValueExample productAttributeValueExample = new PmsProductAttributeValueExample();
        productAttributeValueExample.createCriteria().andProductIdEqualTo(id);
        productAttributeValueMapper.deleteByExample(productAttributeValueExample);
        relateAndInsertList(productAttributeValueDao, productParam.getProductAttributeValueList(), id);
        //关联专题
        CmsSubjectProductRelationExample subjectProductRelationExample = new CmsSubjectProductRelationExample();
        subjectProductRelationExample.createCriteria().andProductIdEqualTo(id);
        subjectProductRelationMapper.deleteByExample(subjectProductRelationExample);
        relateAndInsertList(subjectProductRelationDao, productParam.getSubjectProductRelationList(), id);
        //关联优选
        CmsPrefrenceAreaProductRelationExample prefrenceAreaExample = new CmsPrefrenceAreaProductRelationExample();
        prefrenceAreaExample.createCriteria().andProductIdEqualTo(id);
        prefrenceAreaProductRelationMapper.deleteByExample(prefrenceAreaExample);
        relateAndInsertList(prefrenceAreaProductRelationDao, productParam.getPrefrenceAreaProductRelationList(), id);
        count = 1;
        return count;
    }

    private void handleUpdateSkuStockList(Long id, PmsProductParam productParam) {
        //当前的sku信息
        List<PmsSkuStock> currSkuList = productParam.getSkuStockList();
        //当前没有sku直接删除
        if(CollUtil.isEmpty(currSkuList)){
            syncProductStatus(Collections.singletonList(id), 0);
            PmsSkuStockExample skuStockExample = new PmsSkuStockExample();
            skuStockExample.createCriteria().andProductIdEqualTo(id);
            skuStockMapper.deleteByExample(skuStockExample);
            return;
        }
        //获取初始sku信息
        PmsSkuStockExample skuStockExample = new PmsSkuStockExample();
        skuStockExample.createCriteria().andProductIdEqualTo(id).andDeleteStatusEqualTo(0);
        List<PmsSkuStock> oriStuList = skuStockMapper.selectByExample(skuStockExample);
        //获取新增sku信息
        List<PmsSkuStock> insertSkuList = currSkuList.stream().filter(item->item.getId()==null).collect(Collectors.toList());
        //获取需要更新的sku信息
        List<PmsSkuStock> updateSkuList = currSkuList.stream().filter(item->item.getId()!=null).collect(Collectors.toList());
        List<Long> updateSkuIds = updateSkuList.stream().map(PmsSkuStock::getId).collect(Collectors.toList());
        //获取需要删除的sku信息
        List<PmsSkuStock> removeSkuList = oriStuList.stream().filter(item-> !updateSkuIds.contains(item.getId())).collect(Collectors.toList());
        handleSkuStockCode(insertSkuList,id);
        handleSkuStockCode(updateSkuList,id);

        List<PmsSkuStock> adds= new ArrayList<>();
        adds.addAll(insertSkuList);
        adds.addAll(updateSkuList);

        // 判断sku编码是否重复
        List<String> skuCodes = adds.stream().map(PmsSkuStock::getSkuCode).collect(Collectors.toList());
        long distinctCount = skuCodes.stream().distinct().count();
        if (distinctCount != skuCodes.size()) {
            throw new BizException("SKU编码重复");
        }
        //新增sku
        if(CollUtil.isNotEmpty(insertSkuList)){
            relateAndInsertList(skuStockDao, insertSkuList, id);
        }
        //删除sku
        if(CollUtil.isNotEmpty(removeSkuList)){
            syncSkuStatus(0, removeSkuList);

            List<Long> removeSkuIds = removeSkuList.stream().map(PmsSkuStock::getId).collect(Collectors.toList());
            PmsSkuStockExample removeExample = new PmsSkuStockExample();
            removeExample.createCriteria().andIdIn(removeSkuIds);
            skuStockMapper.deleteByExample(removeExample);
        }
        //修改sku
        if(CollUtil.isNotEmpty(updateSkuList)){

            for (PmsSkuStock pmsSkuStock : updateSkuList) {
                pmsSkuStock.setLockStock(null);
                skuStockMapper.updateByPrimaryKeySelective(pmsSkuStock);
            }
        }

    }

    @Override
    public List<PmsProduct> list(PmsProductQueryParam productQueryParam, Integer pageSize, Integer pageNum) {
        PageHelper.startPage(pageNum, pageSize);
        return productDao.list(productQueryParam);

    }

    @Override
    public int updateVerifyStatus(List<Long> ids, Integer verifyStatus, String detail) {
        PmsProduct product = new PmsProduct();
        product.setVerifyStatus(verifyStatus);
        PmsProductExample example = new PmsProductExample();
        example.createCriteria().andIdIn(ids);
        List<PmsProductVertifyRecord> list = new ArrayList<>();
        int count = productMapper.updateByExampleSelective(product, example);
        //修改完审核状态后插入审核记录
        for (Long id : ids) {
            PmsProductVertifyRecord record = new PmsProductVertifyRecord();
            record.setProductId(id);
            record.setCreateTime(new Date());
            record.setDetail(detail);
            record.setStatus(verifyStatus);
            record.setVertifyMan("test");
            list.add(record);
        }
        productVertifyRecordDao.insertList(list);
        return count;
    }

    @Override
    public int updatePublishStatus(List<Long> ids, Integer publishStatus) {

        syncProductStatus(ids, publishStatus);

        PmsProduct record = new PmsProduct();
        record.setPublishStatus(publishStatus);
        PmsProductExample example = new PmsProductExample();
        example.createCriteria().andIdIn(ids);
        return productMapper.updateByExampleSelective(record, example);
    }

    private void syncProductStatus(List<Long> ids, Integer publishStatus) {

        // 查询商品的sku
        PmsSkuStockExample skuStockExample = new PmsSkuStockExample();
        skuStockExample.createCriteria().andProductIdIn(ids).andDeleteStatusEqualTo(0);
        List<PmsSkuStock> pmsSkuStocks = skuStockMapper.selectByExample(skuStockExample);
        syncSkuStatus(publishStatus, pmsSkuStocks);
    }

    private void syncSkuStatus(Integer publishStatus, List<PmsSkuStock> pmsSkuStocks) {
        if (CollectionUtils.isEmpty(pmsSkuStocks)) {
            return;
        }
        // 商品状态同步
        List<GoodsSubModel> collect = pmsSkuStocks.stream().map(o -> {
            GoodsSubModel goodsSubModel = new GoodsSubModel();
            goodsSubModel.setMallGoodsId(o.getProductId().intValue());
            goodsSubModel.setSkuId(o.getId().intValue());
            goodsSubModel.setGoodsStatus(publishStatus == 1);
            return goodsSubModel;
        }).collect(Collectors.toList());
        log.info("同步商品状态:{}",collect);
        HttpResponseResult<Boolean> booleanHttpResponseResult = webFeignService.goodsStatusSynchronize(collect);
        if (booleanHttpResponseResult.getCode() != 200) {
            throw new BizException("商品状态同步失败");
        }
    }

    @Override
    public int updateRecommendStatus(List<Long> ids, Integer recommendStatus) {
        PmsProduct record = new PmsProduct();
        record.setRecommandStatus(recommendStatus);
        PmsProductExample example = new PmsProductExample();
        example.createCriteria().andIdIn(ids);
        return productMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateNewStatus(List<Long> ids, Integer newStatus) {
        PmsProduct record = new PmsProduct();
        record.setNewStatus(newStatus);
        PmsProductExample example = new PmsProductExample();
        example.createCriteria().andIdIn(ids);
        return productMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateDeleteStatus(List<Long> ids, Integer deleteStatus) {
        if (1 == deleteStatus) {
            syncProductStatus(ids, 0);
            // 更新库存
            PmsSkuStockExample example = new PmsSkuStockExample();
            example.createCriteria().andProductIdIn(ids).andDeleteStatusEqualTo(0);
            List<PmsSkuStock> pmsSkuStocks = skuStockMapper.selectByExample(example);
            if (!CollectionUtils.isEmpty(pmsSkuStocks)) {
                List<GoodsSubStockModel> stockModels = pmsSkuStocks.stream().map(next -> {
                    GoodsSubStockModel stockModel = new GoodsSubStockModel();
                    stockModel.setMallGoodsId(next.getProductId().intValue());
                    stockModel.setSku(next.getId().intValue());
                    stockModel.setStock(0);
                    return stockModel;
                }).collect(Collectors.toList());

                log.info("同步商品库存:{}",stockModels);
                webFeignService.goodsStockCountSynchronize(stockModels);
            }
        }
        PmsProduct record = new PmsProduct();
        record.setDeleteStatus(deleteStatus);
        PmsProductExample example = new PmsProductExample();
        example.createCriteria().andIdIn(ids);
        return productMapper.updateByExampleSelective(record, example);
    }

    @Override
    public List<PmsProduct> list(String keyword) {
        PmsProductExample productExample = new PmsProductExample();
        PmsProductExample.Criteria criteria = productExample.createCriteria();
        criteria.andDeleteStatusEqualTo(0);
        if(!StringUtils.isEmpty(keyword)){
            criteria.andNameLike("%" + keyword + "%");
            productExample.or().andDeleteStatusEqualTo(0).andProductSnLike("%" + keyword + "%");
        }
        return productMapper.selectByExample(productExample);
    }

    @Override
    public HomeStatistical countLowStockCount() {

        return productDao.selectLowStockCount();

    }

    @Override
    public HomeStatistical countProductShowStatus() {
        return productDao.selectProductShowStatusCount();
    }

    @Override
    public Map<Long, PmsProduct> getMapByIds(List<Long> productIds) {
        if (CollectionUtils.isEmpty(productIds)) {
            return new HashMap<>();
        }
        PmsProductExample example = new PmsProductExample();
        example.createCriteria().andIdIn(productIds);
        List<PmsProduct> pmsProducts = productMapper.selectByExample(example);
        return pmsProducts.stream().collect(Collectors.toMap(PmsProduct::getId, pmsProduct -> pmsProduct));
    }

    @Override
    public List<PmsProductDto> getFeignProductList(PmsProductQueryParamDto productQueryParam) {
        PageHelper.startPage(productQueryParam.getPageNum(), productQueryParam.getPageSize());
        List<PmsProductDto> feignProductList = productDao.getFeignProductList(productQueryParam);
        if (CollectionUtils.isEmpty(feignProductList)) {
            return new ArrayList<>();
        }

        // 设置库存
        // 从进货清单中 获取剩余库存数量
        List<Long> productIds = feignProductList.stream().map(PmsProductDto::getGoodsId).collect(Collectors.toList());

        // 计算库存数据
        List<PmsSkuStockDto.PmsSkuStockStock> skuStockStocks = skuStockMapper.getSkuStockByProductIds(productIds);

        // Convert to map
        Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>> skuStockStockMap = skuStockStocks.stream().collect(Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getId));

        // Merge
        for (PmsProductDto pmsProduct : feignProductList) {
            List<PmsSkuStockDto.PmsSkuStockStock> skuStockStock = skuStockStockMap.get(pmsProduct.getSkuStockId().longValue());
            if (!CollectionUtils.isEmpty(skuStockStock)) {
                pmsProduct.setStock(skuStockStock.stream().mapToInt(PmsSkuStockDto.PmsSkuStockStock::getInternalStock).sum() + skuStockStock.stream().mapToInt(PmsSkuStockDto.PmsSkuStockStock::getTaxStock).sum());
            } else {
                pmsProduct.setStock(0);
            }
        }

        // 品牌ID集合
        List<Long> brandIds = feignProductList.stream().map(PmsProductDto::getBrandId).collect(Collectors.toList());

        Map<Long, PmsBrand> brandMap = brandService.getMapByIds(brandIds);

        // 商品分类ID集合
        List<Long> categoryIds = feignProductList.stream().map(PmsProductDto::getProductCategoryId).collect(Collectors.toList());

        Map<Long, PmsProductCategory> categoryMap = productCategoryService.getMapByIds(categoryIds);

        // merge
        for (PmsProductDto pmsProductDto : feignProductList) {

            PmsBrand pmsBrand = brandMap.get(pmsProductDto.getBrandId());
            if (pmsBrand != null) {
                pmsProductDto.setBrandName(pmsBrand.getName());
            }
            PmsProductCategory pmsProductCategory = categoryMap.get(pmsProductDto.getProductCategoryId());
            if (pmsProductCategory != null) {
                pmsProductDto.setProductCategoryName(pmsProductCategory.getName());
            }
        }

        return feignProductList;
    }

    /**
     * 建立和插入关系表操作
     *
     * @param dao       可以操作的dao
     * @param dataList  要插入的数据
     * @param productId 建立关系的id
     */
    private void relateAndInsertList(Object dao, List dataList, Long productId) {
        try {
            if (CollectionUtils.isEmpty(dataList)) return;
            for (Object item : dataList) {
                Method setId = item.getClass().getMethod("setId", Long.class);
                setId.invoke(item, (Long) null);
                Method setProductId = item.getClass().getMethod("setProductId", Long.class);
                setProductId.invoke(item, productId);
            }
            Method insertList = dao.getClass().getMethod("insertList", List.class);
            insertList.invoke(dao, dataList);
        } catch (Exception e) {
            LOGGER.warn("创建产品出错:{}", e.getMessage());
            throw new RuntimeException(e.getMessage());
        }
    }

}
