package com.yonge.cooleshow.admin.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.google.common.collect.Lists;
import com.ym.mec.common.exception.BizException;
import com.yonge.cooleshow.admin.dao.PmsProductSkuStockRecordDao;
import com.yonge.cooleshow.admin.service.OmsOrderService;
import com.yonge.cooleshow.admin.service.PmsProductService;
import com.yonge.cooleshow.admin.service.PmsProductSkuStockRecordService;
import com.yonge.cooleshow.admin.service.PmsSkuStockService;
import com.yonge.cooleshow.admin.wrapper.PmsProductSkuStockRecordWrapper;
import com.yonge.cooleshow.mall.common.util.StringUtil;
import com.yonge.cooleshow.mbg.mapper.PmsProductMapper;
import com.yonge.cooleshow.mbg.mapper.PmsProductSkuStockRecordMapper;
import com.yonge.cooleshow.mbg.mapper.PmsSkuStockMapper;
import com.yonge.cooleshow.mbg.model.PmsProduct;
import com.yonge.cooleshow.mbg.model.PmsProductSkuStockRecord;
import com.yonge.cooleshow.mbg.model.PmsSkuStock;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class PmsProductSkuStockRecordServiceImpl implements PmsProductSkuStockRecordService {

    @Autowired
    private PmsProductSkuStockRecordMapper pmsProductSkuStockRecordMapper;

    @Autowired
    private PmsProductSkuStockRecordDao pmsProductSkuStockRecordDao;

    @Autowired
    private PmsSkuStockService skuStockService;


    @Autowired
    private PmsSkuStockMapper pmsSkuStockMapper;

    @Autowired
    private PmsProductMapper pmsProductMapper;

    @Autowired
    private PmsProductService pmsProductService;

    @Autowired
    private OmsOrderService omsOrderService;

    /**
     * 列表分页
     *
     * @param query
     */
    @Override
    public List<PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord> list(PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecordQuery query) {
        PageHelper.startPage(query.getPageNum(), query.getPageSize());
        List<PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord> list = pmsProductSkuStockRecordDao.list(query);
        if (CollectionUtils.isEmpty(list)) {
            return list;
        }
        // sku id 集合
        List<Long> skuIds = list.stream().map(PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord::getProductSkuId).distinct().collect(Collectors.toList());

        // 商品ID 集合
        List<Long> productIds = list.stream().map(PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord::getProductId).distinct().collect(Collectors.toList());
        // 获取商品信息
        Map<Long, PmsProduct> productMap = pmsProductService.getMapByIds(productIds);

        Map<Long, PmsSkuStock> skuStockMap = skuStockService.getMapByIds(skuIds);
        list.forEach(item -> {
            PmsSkuStock skuStock = skuStockMap.get(item.getProductSkuId());
            if (skuStock != null) {
                item.setProductSkuCode(skuStock.getSkuCode());
                item.setSpData(skuStock.getSpData());

                //规格 信息
                if (StringUtil.isEmpty(item.getSpData())) {
                    item.setSpData("无");
                } else {
                    try {

                        JSONArray objects = JSON.parseArray(item.getSpData());
                        List<String> list1 = Lists.newArrayList();
                        for (Object object : objects) {
                            list1.add(((JSONObject)object).getString("key")+":"+((JSONObject)object).getString("value"));
                        }
                        item.setSpData(list1.stream().collect(Collectors.joining(";")));
                    } catch (Exception e) {
                        item.setSpData("无");
                    }
                }
            }
            PmsProduct pmsProduct = productMap.get(item.getProductId());
            if (pmsProduct != null) {
                item.setProductName(pmsProduct.getName());
                item.setProductSn(pmsProduct.getProductSn());
            }
        });

        return list;

    }

    /**
     * 新增
     *
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int create(PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord pmsProductSkuStockRecord) {

        //判断商品存在
        PmsProduct pmsProduct = pmsProductMapper.selectByPrimaryKey(pmsProductSkuStockRecord.getProductId());
        if (pmsProduct == null) {
            throw new BizException("商品不存在");
        }
        //判断sku存在
        PmsSkuStock pmsSkuStock = pmsSkuStockMapper.selectByPrimaryKey(pmsProductSkuStockRecord.getProductSkuId());
        if (pmsSkuStock == null) {
            throw new BizException("sku不存在");
        }

        pmsProductSkuStockRecordMapper.insertSelective(JSON.parseObject(JSON.toJSONString(pmsProductSkuStockRecord), PmsProductSkuStockRecord.class));

        // 同步库存数据
        omsOrderService.syncSkuStock(Lists.newArrayList(pmsProductSkuStockRecord.getProductSkuId()));

        return 1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int update(PmsProductSkuStockRecordWrapper.PmsProductSkuStockRecord pmsProductSkuStockRecord) {
        // 没有扣减过库存的记录才能修改
        if (pmsProductSkuStockRecord.getId() == null) {
            throw new BizException("批次号不能为空");
        }
        int i = pmsProductSkuStockRecordMapper.updateByIdAndStockSelective(JSON.parseObject(JSON.toJSONString(pmsProductSkuStockRecord), PmsProductSkuStockRecord.class));
        if (i > 0) {
            // 同步库存数据
            omsOrderService.syncSkuStock(Lists.newArrayList(pmsProductSkuStockRecord.getProductSkuId()));
        } else {
            throw new BizException("修改失败");
        }
        return i;
    }
}
