package com.yonge.cooleshow.admin.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.ym.mec.common.exception.BizException;
import com.yonge.cooleshow.admin.dao.PmsSkuStockDao;
import com.yonge.cooleshow.admin.wrapper.PmsSkuStockWrapper;
import com.yonge.cooleshow.mbg.dto.PmsSkuStockDto;
import com.yonge.cooleshow.mbg.mapper.PmsSkuStockMapper;
import com.yonge.cooleshow.mbg.model.PmsSkuStock;
import com.yonge.cooleshow.mbg.model.PmsSkuStockExample;
import com.yonge.cooleshow.admin.service.PmsSkuStockService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 商品SKU库存管理Service实现类
 * Created by macro on 2018/4/27.
 */
@Service
public class PmsSkuStockServiceImpl implements PmsSkuStockService {
    @Autowired
    private PmsSkuStockMapper skuStockMapper;
    @Autowired
    private PmsSkuStockDao skuStockDao;

    @Override
    public List<PmsSkuStockWrapper.PmsSkuStock> getList(Long pid, String keyword) {
        PmsSkuStockExample example = new PmsSkuStockExample();
        PmsSkuStockExample.Criteria criteria = example.createCriteria().andProductIdEqualTo(pid).andDeleteStatusEqualTo(0);
        if (!StringUtils.isEmpty(keyword)) {
            criteria.andSkuCodeLike("%" + keyword + "%");
        }
        List<PmsSkuStock> pmsSkuStocks = skuStockMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(pmsSkuStocks)) {
            return new ArrayList<>();
        }

        // Convert to wrapper
        List<PmsSkuStockWrapper.PmsSkuStock> stockList = JSON.parseArray(JSON.toJSONString(pmsSkuStocks), PmsSkuStockWrapper.PmsSkuStock.class);

        // 计算库存数据
        List<PmsSkuStockDto.PmsSkuStockStock> skuStockStocks =  skuStockMapper.getSkuStockByProductIds(Lists.newArrayList(pid));

        // Convert to map
        Map<Long, PmsSkuStockDto.PmsSkuStockStock> skuStockStockMap = skuStockStocks.stream().collect(Collectors.toMap(PmsSkuStockDto.PmsSkuStockStock::getId, skuStock -> skuStock));

        // Merge
        for (PmsSkuStockWrapper.PmsSkuStock stock : stockList) {
            PmsSkuStockDto.PmsSkuStockStock skuStockStock = skuStockStockMap.get(stock.getId());
            if (skuStockStock != null) {
                stock.setInternalStock(skuStockStock.getInternalStock());
                stock.setTaxStock(skuStockStock.getTaxStock());
            } else {
                stock.setInternalStock(0);
                stock.setTaxStock(0);
            }
        }

        return stockList;
    }

    @Override
    public int update(Long pid, List<PmsSkuStock> skuStockList) {

        // 判断sku code是否重复
        // 判断sku编码是否重复
        List<String> skuCodes = skuStockList.stream().map(PmsSkuStock::getSkuCode).collect(Collectors.toList());
        long distinctCount = skuCodes.stream().distinct().count();
        if (distinctCount != skuCodes.size()) {
            throw new BizException("SKU编码重复");
        }

        return skuStockDao.replaceList(skuStockList);
    }

    @Override
    public Map<Long, PmsSkuStock> getMapByIds(List<Long> skuIds) {
        if (CollectionUtils.isEmpty(skuIds)) {
            return new HashMap<>();
        }
        PmsSkuStockExample example = new PmsSkuStockExample();
        example.createCriteria().andIdIn(skuIds);
        List<PmsSkuStock> skuStockList = skuStockMapper.selectByExample(example);
        skuStockList = updateProductSkuStock(skuStockList);
        if (CollectionUtils.isEmpty(skuStockList)) {
            return new HashMap<>();
        }
        return skuStockList.stream().collect(Collectors.toMap(PmsSkuStock::getId, skuStock -> skuStock));
    }

    public List<PmsSkuStock> updateProductSkuStock(List<PmsSkuStock> skuStocks) {
        if (CollectionUtils.isEmpty(skuStocks)) {
            return new ArrayList<>();
        }
        // 从进货清单中 获取剩余库存数量
        List<Long> productIds = skuStocks.stream().map(PmsSkuStock::getProductId).distinct().collect(Collectors.toList());

        // 计算库存数据
        List<PmsSkuStockDto.PmsSkuStockStock> skuStockStocks = skuStockMapper.getSkuStockByProductIds(productIds);

        // Convert to map
        Map<Long, Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>>> map = skuStockStocks.stream()
            .collect(Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getProductId, Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getId)));

        // Merge
        for (PmsSkuStock skuStock : skuStocks) {
            Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>> skuStockStock = map.get(skuStock.getProductId());
            if (!CollectionUtils.isEmpty(skuStockStock)) {
                List<PmsSkuStockDto.PmsSkuStockStock> pmsSkuStockStocks = skuStockStock.get(skuStock.getId());
                if (CollectionUtils.isEmpty(pmsSkuStockStocks)) {
                    skuStock.setLockStock(0);
                    skuStock.setStock(0);
                    continue;
                }
                skuStock.setStock(pmsSkuStockStocks.stream().mapToInt(item -> item.getInternalStock() + item.getTaxStock()).sum());
                skuStock.setLockStock(0);
            }

        }
        return skuStocks;
    }
}
