package com.yonge.cooleshow.admin.wrapper;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.format.DateTimeFormat;
import com.alibaba.excel.annotation.format.NumberFormat;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

public class PmsProductSkuStockRecordWrapper {


    @Data
    @ApiModel("PmsProductSkuStockRecordQuery-商品库存入库记录")
    public static class PmsProductSkuStockRecordQuery {

        @ApiModelProperty("当前页")
        private Integer pageNum =1;

        @ApiModelProperty("分页行数")
        private Integer pageSize =10;

        @ApiModelProperty(value = "商品ID",required = true)
        @NotNull(message = "商品ID不能为空")
        private Long productId;

        @ApiModelProperty("批次号")
        private String id;

        @ApiModelProperty("备查货号")
        private String selectProductSn;

        @ApiModelProperty("入库开始时间")
        private Date startTime;

        @ApiModelProperty("入库结束时间")
        private Date endTime;

        private List<Long> productIds;

    }



    @Data
    @ApiModel("PmsProductSkuStockRecord-商品库存入库记录")
    public static class PmsProductSkuStockRecord {
        /**
         * id，批次号
         */
        @ApiModelProperty("批次号")

        @ExcelProperty(value = "批次号",order = 1)
        @ColumnWidth(20)
        private Long id;

        /**
         * 商品ID
         */
        @ApiModelProperty("商品ID")
        @NotNull(message = "商品ID不能为空")
        @ExcelIgnore
        private Long productId;

        @ApiModelProperty("商品货号")
        @ExcelProperty(value = "商品货号",order = 2)
        private String productSn;


        @ApiModelProperty("商品名称")
        @ExcelProperty(value = "商品名称",order = 2)
        private String productName;

        /**
         * 商品skuId
         */
        @ApiModelProperty("商品skuId")
        @NotNull(message = "商品skuId不能为空")
        @ColumnWidth(20)
        @ExcelProperty(value = "skuId",order = 3)
        private Long productSkuId;


        /**
         * 商品skuId
         */
        @ApiModelProperty("商品sku编号")
        @ExcelProperty(value = "sku编号",order = 3)
        private String productSkuCode;


        /**
         * 商品skuId
         */
        @ApiModelProperty("sku规格")
        @ExcelProperty(value = "sku规格",order = 3)
        private String spData;

        /**
         * 备查货号
         */
        @ApiModelProperty("备查货号")
        @NotNull(message = "备查货号不能为空")
        @ExcelProperty(value = "备查货号",order = 2)
        private String selectProductSn;

        /**
         * 内部库存
         */
        @ApiModelProperty("内部库存")
        @NotNull(message = "内部库存不能为空")
        @ExcelProperty(value = "内部库存",order = 4)
        private Integer internalStock;

        /**
         * 内部销售库存
         */
        @ApiModelProperty("内部销售库存")
        @ExcelProperty(value = "内部销售库存",order = 5)
        private Integer internalSaleStock;

        /**
         * 税务库存
         */
        @ApiModelProperty("税务库存")
        @NotNull(message = "税务库存不能为空")
        @ExcelProperty(value = "税务库存",order = 6)
        private Integer taxStock;

        /**
         * 税务库存
         */
        @ApiModelProperty("税务销售库存")
        @ExcelProperty(value = "税务销售库存",order = 7)
        private Integer taxSaleStock;

        /**
         * 采购价
         */
        @ApiModelProperty("采购价")
        @NotNull(message = "采购价不能为空")
        @ExcelProperty(value = "采购价",order = 8)
//        @NumberFormat("#.##")
        private BigDecimal price;


        /**
         * 创建时间
         */
        @ApiModelProperty("创建时间")
        @ColumnWidth(30)
        @ExcelProperty(value = "创建时间",order = 9)
        @DateTimeFormat("yyyy-MM-dd HH:mm:ss")
        private Date createTime;

        @ExcelIgnore
        private Long createBy;
    }
}
