package com.yonge.cooleshow.portal.controller;

import cn.hutool.core.map.MapUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huifu.adapay.core.AdapayCore;
import com.huifu.adapay.core.util.AdapaySign;
import com.yeepay.g3.sdk.yop.encrypt.DigitalEnvelopeDTO;
import com.yeepay.g3.sdk.yop.utils.DigitalEnvelopeUtils;
import com.yeepay.g3.sdk.yop.utils.RSAKeyUtils;
import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.thirdparty.adapay.entity.HfMerchantConfig;
import com.ym.mec.web.WebFeignService;
import com.yonge.cooleshow.mall.common.enums.OrderCacheEnum;
import com.yonge.cooleshow.mbg.model.UserOrderPayment;
import com.yonge.cooleshow.portal.dto.OrderPayRes;
import com.yonge.cooleshow.portal.service.OmsPortalOrderService;
import com.yonge.cooleshow.portal.service.UserOrderPaymentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author: liweifan
 * @Data: 2022/3/11 18:16
 */
@RestController
@RequestMapping("/payment")
@Api(value = "付款接口", tags = "付款接口")
public class PaymentController extends BaseController {
    private final static Logger log = LoggerFactory.getLogger(PaymentController.class);

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private OmsPortalOrderService orderService;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private UserOrderPaymentService userOrderPaymentService;

    @Autowired
    private WebFeignService webFeignService;


    /***
     * 支付回调
     * @author liweifan
     * @param: request
     * @updateTime 2022/3/11 18:35
     */
    @RequestMapping("/callback")
    public Object callback( HttpServletRequest request) {
        try {
            // 验签请参data
            String data = request.getParameter("data");
            // 验签请参sign
            String sign = request.getParameter("sign");

            log.info("callback request param:{}", data);

            // 验签
            if (AdapaySign.verifySign(data, sign, AdapayCore.PUBLIC_KEY)) {

                JSONObject dataObj = JSON.parseObject(data);

                String transType = request.getParameter("type");


                String paymentOrderNo = dataObj.getString("order_no");

                UserOrderPayment userOrderPayment = userOrderPaymentService.getByAdapayNo(paymentOrderNo);
                if (userOrderPayment == null) {
                    throw new BizException("订单不存在");
                }

                RLock lock = redissonClient.getLock(OrderCacheEnum.LOCK_ORDER_NO_MALL.getRedisKey(userOrderPayment.getOrderNo()));
                try {
                    boolean b = lock.tryLock(60, TimeUnit.SECONDS);

                    if (b) {

                        switch (transType) {
                            case "payment.succeeded":// 支付成功

                                orderService.paymentSucceededHandle(paymentOrderNo,
                                                   dataObj.getBigDecimal("pay_amt"),dataObj.getString("pay_channel"));

                                break;

                            case "payment.failed":// 支付失败

                                Map<String,Object> notifyMap = new HashMap<>();

                                notifyMap.put("channelType", dataObj.getString("pay_channel"));
                                notifyMap.put("tradeState", "0");
                                notifyMap.put("totalMoney", dataObj.getString("pay_amt"));
                                notifyMap.put("merOrderNo", paymentOrderNo);
                                orderService.paymentFailedHandle(paymentOrderNo,notifyMap);

                                break;
                            case "payment_reverse.succeeded" :

                                orderService.paymentRefundSucceededHandle(dataObj.getString("payment_id")
                                                , dataObj.getString("reverse_amt"), dataObj.getString("id"), BigDecimal.ZERO.toString());
                                break;
                            case "payment_reverse.failed" :

                                orderService.paymentRefundFailedHandle(dataObj.getString("payment_id")
                                        ,dataObj.getString("error_msg"),dataObj.getString("id"));
                            default:
                                break;
                        }
                    } else {
                        return failed("请求超时");
                    }
                }catch (Exception e) {
                    e.printStackTrace();
                    return failed("请求超时");
                }
            }else {
                return "验签失败";
            }
        } catch (Exception e) {
            return e.getMessage();
        }

        return "succeeded";
    }

    /***
     * 易宝支付回调
     * @author liweifan
     * @param: request
     * @updateTime 2022/3/11 18:35
     */
    @RequestMapping("/callback_yee/{tenantId}")
    public String callback_yee(@PathVariable("tenantId") Integer tenantId, HttpServletRequest request) {
        try {
            String content = request.getParameter("response");
            log.info("易宝支付回调信息：content:{}", content);

            HfMerchantConfig hfMerchantConfig = webFeignService.queryByTenantId("YEEPAY",tenantId).getData();
            if(hfMerchantConfig == null){
                throw new BizException("机构[{}][{}]商户信息找不到", tenantId, "YEEPAY");
            }

            // 构造结果通知请求对象
            DigitalEnvelopeDTO dto = new DigitalEnvelopeDTO();
            dto.setCipherText(content);
            PrivateKey privateKey = RSAKeyUtils.string2PrivateKey(hfMerchantConfig.getRsaPrivateKey());
            PublicKey publicKey = RSAKeyUtils.string2PublicKey(hfMerchantConfig.getRsaPublicKey());

            dto = DigitalEnvelopeUtils.decrypt(dto, privateKey, publicKey);

            log.info("易宝支付回调信息：plaintText:{}", dto.getPlainText());
            JSONObject dataObj = JSON.parseObject(dto.getPlainText());

            String orderNo = dataObj.getString("orderId");
            UserOrderPayment userOrderPayment = userOrderPaymentService.getByAdapayNo(orderNo);
            if (userOrderPayment == null) {
                throw new BizException("订单不存在");
            }
            if("SUCCESS".equals(dataObj.getString("status"))) {
                orderService.paymentSucceededHandle(orderNo,
                        dataObj.getBigDecimal("orderAmount"),dataObj.getString("channel"));
            }else {
                Map<String,Object> notifyMap = new HashMap<>();
                notifyMap.put("channelType", dataObj.getString("channel"));
                notifyMap.put("tradeState", "0");
                notifyMap.put("totalMoney", dataObj.getString("orderAmount"));
                notifyMap.put("merOrderNo", orderNo);
                orderService.paymentFailedHandle(orderNo,notifyMap);
            }
        } catch (Exception e) {
            return e.getMessage();
        }
        return "SUCCESS";
    }

}
