package com.yonge.cooleshow.portal.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageHelper;
import com.ym.mec.auth.api.entity.SysUser;
import com.yonge.cooleshow.mall.common.exception.Asserts;
import com.yonge.cooleshow.mbg.dto.PmsSkuStockDto;
import com.yonge.cooleshow.mbg.mapper.*;
import com.yonge.cooleshow.mbg.model.*;
import com.yonge.cooleshow.portal.dao.PortalProductDao;
import com.yonge.cooleshow.portal.domain.PmsPortalProductDetail;
import com.yonge.cooleshow.portal.domain.PmsProductCategoryNode;
import com.yonge.cooleshow.portal.dto.ProductCategorySmallVo;
import com.yonge.cooleshow.portal.dto.ProductSearchConditionVo;
import com.yonge.cooleshow.portal.dto.ShareProductVo;
import com.yonge.cooleshow.portal.dto.StockOperateModel;
import com.yonge.cooleshow.portal.service.PmsPortalProductService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 前台订单管理Service实现类
 * Created by macro on 2020/4/6.
 */
@Service
public class PmsPortalProductServiceImpl implements PmsPortalProductService {
    @Autowired
    private PmsProductMapper productMapper;
    @Autowired
    private PmsProductCategoryMapper productCategoryMapper;
    @Autowired
    private PmsBrandMapper brandMapper;
    @Autowired
    private PmsProductAttributeMapper productAttributeMapper;
    @Autowired
    private PmsProductAttributeValueMapper productAttributeValueMapper;
    @Autowired
    private PmsSkuStockMapper skuStockMapper;
    @Autowired
    private PmsProductLadderMapper productLadderMapper;
    @Autowired
    private PmsProductFullReductionMapper productFullReductionMapper;
    @Autowired
    private PortalProductDao portalProductDao;

    @Autowired
    private PmsProductMapper pmsProductMapper;

    @Autowired
    private PmsProductAttributeCategoryMapper productAttributeCategoryMapper;

    @Override
    public List<PmsProduct> search(String keyword, Long brandId, Long productCategoryId, Long productAttributeCategoryId, Integer pageNum, Integer pageSize, Integer sort) {
        PageHelper.startPage(pageNum, pageSize);
        PmsProductExample example = new PmsProductExample();
        PmsProductExample.Criteria criteria = example.createCriteria();
        criteria.andDeleteStatusEqualTo(0);
        criteria.andPublishStatusEqualTo(1);
        if (StrUtil.isNotEmpty(keyword)) {
            criteria.andNameLike("%" + keyword + "%");
        }
        if (brandId != null) {
            criteria.andBrandIdEqualTo(brandId);
        }
        if (productCategoryId != null) {
            List<Long> productCategoryIds = getProductCategoryIds(productCategoryId);
            criteria.andProductCategoryIdIn(productCategoryIds);
        }
        if (productAttributeCategoryId != null) {
            criteria.andProductAttributeCategoryIdEqualTo(productAttributeCategoryId);
        }
        //1->按新品；2->按销量；3->价格从低到高；4->价格从高到低
        if (sort == 1) {
            example.setOrderByClause("id desc");
        } else if (sort == 2) {
            example.setOrderByClause("sale desc,id desc");
        } else if (sort == 3) {
            example.setOrderByClause("price asc,id desc");
        } else if (sort == 4) {
            example.setOrderByClause("price desc,id desc");
        } else {
            example.setOrderByClause("sort desc,id desc");
        }
        List<PmsProduct> pmsProducts = productMapper.selectByExample(example);
        List<Long> productIdList = pmsProducts.stream().map(PmsProduct::getId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(productIdList)) {
            return pmsProducts;
        }
        PmsSkuStockExample skuExample = new PmsSkuStockExample();
        skuExample.createCriteria().andProductIdIn(productIdList).andDeleteStatusEqualTo(0);
        List<PmsSkuStock> skuStockList = skuStockMapper.selectByExample(skuExample);
        skuStockList = updateProductSkuStock(skuStockList);
        Map<Long, List<PmsSkuStock>> collect = skuStockList.stream()
                                                           .collect(Collectors.groupingBy(PmsSkuStock::getProductId));
        for (PmsProduct pmsProduct : pmsProducts) {
            pmsProduct.setSkuStockList(collect.get(pmsProduct.getId()));
        }

        return updateProductStock(pmsProducts);

    }

    @Override
    public List<PmsSkuStock> updateProductSkuStock(List<PmsSkuStock> skuStocks) {
        if (CollectionUtils.isEmpty(skuStocks)) {
            return new ArrayList<>();
        }
        // 从进货清单中 获取剩余库存数量
        List<Long> productIds = skuStocks.stream().map(PmsSkuStock::getProductId).distinct().collect(Collectors.toList());

        // 计算库存数据
        List<PmsSkuStockDto.PmsSkuStockStock> skuStockStocks = skuStockMapper.getSkuStockByProductIds(productIds);

        // Convert to map
        Map<Long, Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>>> map = skuStockStocks.stream()
            .collect(Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getProductId, Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getId)));

        // Merge
        for (PmsSkuStock skuStock : skuStocks) {
            Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>> skuStockStock = map.get(skuStock.getProductId());
            if (!CollectionUtils.isEmpty(skuStockStock)) {
                List<PmsSkuStockDto.PmsSkuStockStock> pmsSkuStockStocks = skuStockStock.get(skuStock.getId());
                if (CollectionUtils.isEmpty(pmsSkuStockStocks)) {
                    skuStock.setLockStock(0);
                    skuStock.setStock(0);
                    continue;
                }
                skuStock.setStock(pmsSkuStockStocks.stream().mapToInt(item -> item.getInternalStock() + item.getTaxStock()).sum());
                skuStock.setLockStock(0);
            } else {
                skuStock.setStock(0);
                skuStock.setLockStock(0);
            }

        }
        return skuStocks;
    }

    @Override
    public List<PmsProduct> updateProductStock(List<PmsProduct> pmsProducts) {
        if (CollectionUtils.isEmpty(pmsProducts)) {
            return new ArrayList<>();
        }
        // 从进货清单中 获取剩余库存数量
        List<Long> productIds = pmsProducts.stream().map(PmsProduct::getId).collect(Collectors.toList());

        // 计算库存数据
        List<PmsSkuStockDto.PmsSkuStockStock> skuStockStocks = skuStockMapper.getSkuStockByProductIds(productIds);

        // Convert to map
        Map<Long, List<PmsSkuStockDto.PmsSkuStockStock>> skuStockStockMap = skuStockStocks.stream().collect(Collectors.groupingBy(PmsSkuStockDto.PmsSkuStockStock::getProductId));

        // Merge
        for (PmsProduct pmsProduct : pmsProducts) {
            List<PmsSkuStockDto.PmsSkuStockStock> skuStockStock = skuStockStockMap.get(pmsProduct.getId());
            if (!CollectionUtils.isEmpty(skuStockStock)) {
                pmsProduct.setStock(skuStockStock.stream().mapToInt(PmsSkuStockDto.PmsSkuStockStock::getInternalStock).sum() + skuStockStock.stream().mapToInt(PmsSkuStockDto.PmsSkuStockStock::getTaxStock).sum());
                pmsProduct.setLockStock(0);
            } else {
                pmsProduct.setStock(0);
                pmsProduct.setLockStock(0);
            }
        }
        return pmsProducts;
    }

    /**
     * 获取当前分类和下级分类 id
     *
     * @param productCategoryId
     * @return
     */
    private List<Long> getProductCategoryIds(Long productCategoryId) {
        PmsProductCategoryExample productCategoryExample = new PmsProductCategoryExample();
        productCategoryExample.setOrderByClause("sort desc");
        productCategoryExample.createCriteria().andParentIdEqualTo(productCategoryId)
               .andShowStatusEqualTo(1);
        List<PmsProductCategory> pmsProductCategories = productCategoryMapper.selectByExample(productCategoryExample);
        List<Long> productCategoryIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(pmsProductCategories)) {
            productCategoryIds = pmsProductCategories
                    .stream()
                    .map(PmsProductCategory::getId)
                    .collect(Collectors.toList());
        }
        productCategoryIds.add(productCategoryId);
        return productCategoryIds;
    }

    @Override
    public List<PmsProductCategoryNode> categoryTreeList() {
        PmsProductCategoryExample example = new PmsProductCategoryExample();
        List<PmsProductCategory> allList = productCategoryMapper.selectByExample(example);
        return allList.stream()
                      .filter(item -> item.getParentId().equals(0L))
                      .map(item -> covert(item, allList)).collect(Collectors.toList());
    }

    @Override
    public PmsPortalProductDetail detail(Long id) {
        PmsPortalProductDetail result = new PmsPortalProductDetail();
        //获取商品信息
        PmsProduct product = productMapper.selectByPrimaryKey(id);
        if (product == null) {
            Asserts.fail("商品不存在");
        }
        List<PmsProduct> pmsProducts = updateProductStock(CollUtil.newArrayList(product));
        result.setProduct(pmsProducts.get(0));
        //获取品牌信息
        PmsBrand brand = brandMapper.selectByPrimaryKey(product.getBrandId());
        result.setBrand(brand);
        //获取商品属性信息
        PmsProductAttributeExample attributeExample = new PmsProductAttributeExample();
        attributeExample.createCriteria().andProductAttributeCategoryIdEqualTo(product.getProductAttributeCategoryId());
        List<PmsProductAttribute> productAttributeList = productAttributeMapper.selectByExample(attributeExample);
        result.setProductAttributeList(productAttributeList);
        //获取商品属性值信息
        if(CollUtil.isNotEmpty(productAttributeList)){
            List<Long> attributeIds = productAttributeList.stream().map(PmsProductAttribute::getId).collect(Collectors.toList());
            PmsProductAttributeValueExample attributeValueExample = new PmsProductAttributeValueExample();
            attributeValueExample.createCriteria().andProductIdEqualTo(product.getId())
                    .andProductAttributeIdIn(attributeIds);
            List<PmsProductAttributeValue> productAttributeValueList = productAttributeValueMapper.selectByExample(attributeValueExample);
            result.setProductAttributeValueList(productAttributeValueList);
        }
        //获取商品SKU库存信息
        PmsSkuStockExample skuExample = new PmsSkuStockExample();
        skuExample.createCriteria().andProductIdEqualTo(product.getId()).andDeleteStatusEqualTo(0);
        List<PmsSkuStock> skuStockList = skuStockMapper.selectByExample(skuExample);
        skuStockList = updateProductSkuStock(skuStockList);
        result.setSkuStockList(skuStockList);
        //商品阶梯价格设置
        if(product.getPromotionType()==3){
            PmsProductLadderExample ladderExample = new PmsProductLadderExample();
            ladderExample.createCriteria().andProductIdEqualTo(product.getId());
            List<PmsProductLadder> productLadderList = productLadderMapper.selectByExample(ladderExample);
            result.setProductLadderList(productLadderList);
        }
        //商品满减价格设置
        if(product.getPromotionType()==4){
            PmsProductFullReductionExample fullReductionExample = new PmsProductFullReductionExample();
            fullReductionExample.createCriteria().andProductIdEqualTo(product.getId());
            List<PmsProductFullReduction> productFullReductionList = productFullReductionMapper.selectByExample(fullReductionExample);
            result.setProductFullReductionList(productFullReductionList);
        }
        //商品可用优惠券
        result.setCouponList(portalProductDao.getAvailableCouponList(product.getId(),product.getProductCategoryId()));
        return result;
    }

    @Override
    public ProductSearchConditionVo searchCondition() {
        ProductSearchConditionVo result = new ProductSearchConditionVo();
        // 设置品牌
        result.setBrandList(getBrandList());

        // 设置分类
        result.setProductCategorySmallVoList(getProductCategoryList());

        // 设置类别
        result.setProductAttributeCategoryList(getHomeProductAttributeCategory());
        return result;
    }

    @Override
    public ShareProductVo shareProductProfit(SysUser sysUser, Long productId) {
        PmsPortalProductDetail detail = detail(productId);
        ShareProductVo  shareProductVo = new ShareProductVo();
        shareProductVo.setProductDetail(detail);
        shareProductVo.setAvatar(sysUser.getAvatar());
        shareProductVo.setTeacherId(sysUser.getId().longValue());
        shareProductVo.setName(sysUser.getUsername());
        // String url = adminFeignService.findConfigValue(SysConfigConstant.TEACHER_PRODUCT_SHARE_PROFIT_URL).getData();

        // shareProductVo.setUrl(MessageFormatter.arrayFormat(url, detail.getProduct().getId(), sysUser.getId()));

        return shareProductVo;
    }

    @Override
    @Transactional
    public void reduceStock(List<StockOperateModel> model, String stockType) {
//
//        // 商品库存变动
//        List<ProductStock> productList = new ArrayList<>();
//        Map<Long, List<StockOperateModel>> collect = model.stream().collect(Collectors.groupingBy(
//                StockOperateModel::getProductId));
//        for (Map.Entry<Long, List<StockOperateModel>> entry : collect.entrySet()) {
//            ProductStock productStock = new ProductStock();
//            List<StockOperateModel> value = entry.getValue();
//            if (CollectionUtils.isEmpty(value)) {
//                continue;
//            }
//            int sum = value.stream().mapToInt(StockOperateModel::getCount).sum();
//            productStock.setProductId(entry.getKey());
//            productStock.setStock(sum);
//            productList.add(productStock);
//        }
//        switch (stockType) {
//            case "lock" :
//                // sku库存锁定
//                for (StockOperateModel stockOperateModel : model) {
//                    int i = skuStockMapper.lockStock(stockOperateModel.getProductSkuId(), stockOperateModel.getCount());
//                    if (i ==0) {
//                        Asserts.fail("库存不足");
//                    }
//                }
//                // 商品扣减锁定
//                for (ProductStock product : productList) {
//                    if (product.getStock() > 0) {
//                        int i = pmsProductMapper.lockStock(product.getProductId(), product.getStock());
//                        if (i == 0) {
//                            Asserts.fail("库存不足");
//                        }
//                    }
//                }
//                break;
//            case "unlock" :
//                // 解锁sku库存
//                portalProductDao.unlockSku(model);
//
//                // 解锁商品库存
//                portalProductDao.unlockProduct(productList);
//                break;
//            case "reduce" :
//                // 扣减sku库存
//                portalProductDao.reduceSku(model);
//
//                // 扣减商品库存
//                portalProductDao.updateProductStock(productList);
//                break;
//            default: break;
//
//        }


    }


    private  List<PmsProductAttributeCategory>  getHomeProductAttributeCategory() {
        PmsProductAttributeCategoryExample example = new PmsProductAttributeCategoryExample();
        example.setOrderByClause("sort desc,id desc");
        List<PmsProductAttributeCategory> pmsProductCategories = productAttributeCategoryMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(pmsProductCategories)) {
            return new ArrayList<>();
        }
        return pmsProductCategories;
    }

    private List<ProductCategorySmallVo> getProductCategoryList() {
        PmsProductCategoryExample example = new PmsProductCategoryExample();
        example.setOrderByClause("sort desc");
        example.createCriteria()
               .andShowStatusEqualTo(1);
        List<PmsProductCategory> pmsProductCategories = productCategoryMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(pmsProductCategories)) {
            return new ArrayList<>();
        }
        List<ProductCategorySmallVo> collect = pmsProductCategories.stream().map(pmsProductCategory -> {
            ProductCategorySmallVo productCategorySmallVo = new ProductCategorySmallVo();
            BeanUtils.copyProperties(pmsProductCategory, productCategorySmallVo);
            return productCategorySmallVo;
        }).collect(Collectors.toList());

        List<ProductCategorySmallVo> tree = new ArrayList<>();
        for (ProductCategorySmallVo categorySmallVo : collect) {
            if (categorySmallVo.getParentId() == null || categorySmallVo.getParentId() == 0) {
                tree.add(categorySmallVo);
            }
            for (ProductCategorySmallVo tagVo : collect) {
                if (tagVo.getParentId() != null && tagVo.getParentId() != 0 && tagVo.getParentId().equals(categorySmallVo.getId())) {
                    if (categorySmallVo.getChildren() == null) {
                        categorySmallVo.setChildren(new ArrayList<>());
                    }
                    categorySmallVo.getChildren().add(tagVo);
                }

            }
        }
        return tree;
    }

    private List<PmsBrand> getBrandList() {
        PmsBrandExample pmsBrandExample = new PmsBrandExample();
        pmsBrandExample.createCriteria().andShowStatusEqualTo(1);
        pmsBrandExample.setOrderByClause("sort desc");
        return brandMapper.selectByExample(pmsBrandExample);
    }


    /**
     * 初始对象转化为节点对象
     */
    private PmsProductCategoryNode covert(PmsProductCategory item, List<PmsProductCategory> allList) {
        PmsProductCategoryNode node = new PmsProductCategoryNode();
        BeanUtils.copyProperties(item, node);
        List<PmsProductCategoryNode> children = allList.stream()
                .filter(subItem -> subItem.getParentId().equals(item.getId()))
                .map(subItem -> covert(subItem, allList)).collect(Collectors.toList());
        node.setChildren(children);
        return node;
    }
}
