package com.ym.mec.web.config;

import com.corundumstudio.socketio.SocketConfig;
import com.corundumstudio.socketio.SocketIOServer;
import com.corundumstudio.socketio.Transport;
import com.corundumstudio.socketio.store.RedissonStoreFactory;
import com.corundumstudio.socketio.store.pubsub.PubSubStore;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Slf4j
@Configuration
public class SocketIoConfig {
    @Value("${socket.server.host:127.0.0.1}")
    private String host;

    @Value("${socket.server.port:3002}")
    private Integer port;

    private final RedissonClient redissonClient;

    @Autowired
    public SocketIoConfig(RedissonClient redissonClient) {
        this.redissonClient = redissonClient;
    }

    @Bean
    public SocketIOServer socketIOServer() {
        com.corundumstudio.socketio.Configuration config = new com.corundumstudio.socketio.Configuration();
        config.setHostname(host);
        config.setPort(port);
        config.setPingInterval(25000);
        config.setPingTimeout(60000);
        config.setWorkerThreads(100);
        config.setRandomSession(true);
        //设置最大每帧处理数据的长度，防止他人利用大数据来攻击服务器
        config.setMaxFramePayloadLength(1024 * 1024);
        //设置http交互最大内容长度
        config.setMaxHttpContentLength(1024 * 1024);
        //授权
        config.setAuthorizationListener(data -> true);
        config.setTransports(Transport.WEBSOCKET, Transport.POLLING);
        // 推荐使用redisson
        config.setStoreFactory(new RedissonStoreFactory(redissonClient));
        // 配置一个统一的URL，nginx做后端请求转发; ****注意"/"不能少****
        config.setContext("/ws-server");
        log.info("--------SocketIO------- context={}", config.getContext());

        // 服务端配置
        SocketConfig socketConfig = new SocketConfig();
        socketConfig.setReuseAddress(true);
        socketConfig.setTcpNoDelay(true);
        socketConfig.setSoLinger(0);
        config.setSocketConfig(socketConfig);

        return new SocketIOServer(config);
    }

    /**
     * 解决分布式服务器实例环境
     * @param socketServer SocketIOServer
     * @return PubSubStore
     */
    @Bean
    public PubSubStore pubSubStore(SocketIOServer socketServer) {

        return socketServer.getConfiguration().getStoreFactory().pubSubStore();
    }
}
