package com.ym.mec.web.config;

import com.ym.mec.common.config.EnumConverterFactory;
import com.ym.mec.common.config.LocalFastJsonHttpMessageConverter;
import com.ym.mec.common.filters.EmojiEncodingFilter;
import com.ym.mec.web.interceptor.OperationLogInterceptor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.format.FormatterRegistry;
import org.springframework.http.MediaType;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import javax.servlet.Filter;
import java.util.ArrayList;
import java.util.List;

@Configuration
public class WebMvcConfig implements WebMvcConfigurer {

	@Autowired
	private OperationLogInterceptor operationLogInterceptor;

	/**
	 * 枚举类的转换器 addConverterFactory
	 */
	@Override
	public void addFormatters(FormatterRegistry registry) {
		registry.addConverterFactory(new EnumConverterFactory());
	}

	@Override
	public void addInterceptors(InterceptorRegistry registry) {

		registry.addInterceptor(operationLogInterceptor).addPathPatterns("/**").excludePathPatterns("/login");
	}

	@Override
	public void addCorsMappings(CorsRegistry registry) {
		registry.addMapping("/**").allowedOrigins("*")
				// 是否允许证书
				.allowCredentials(true)
				// 设置允许的方法
				.allowedMethods("*")
				// 设置允许的header属性
				.allowedHeaders("*")
				// 跨域允许时间
				.maxAge(3600);
	}

	@Bean
	public HttpMessageConverters fastJsonHttpMessageConverters() {
		LocalFastJsonHttpMessageConverter converter = new LocalFastJsonHttpMessageConverter();
		List<MediaType> fastMediaTypes = new ArrayList<>();
		fastMediaTypes.add(MediaType.APPLICATION_JSON_UTF8);
		converter.setSupportedMediaTypes(fastMediaTypes);
		return new HttpMessageConverters(converter);
	}

	/**
	 * 注册filter
	 * @return FilterRegistrationBean<Filter>
	 */
	@Bean
	public FilterRegistrationBean<Filter> filterRegistrationBean() {
		FilterRegistrationBean<Filter> registration = new FilterRegistrationBean<Filter>();
		// 注入过滤器
		registration.setFilter(new EmojiEncodingFilter());
		// 过滤器名称
		registration.setName("emojiEncodingFilter");
		// 拦截规则
		registration.addUrlPatterns("/*");
		// 过滤器顺序(值越小，优先级越高)
		registration.setOrder(1);
		return registration;
	}
}
