package com.ym.mec.web.support.socket;

import com.corundumstudio.socketio.SocketIONamespace;
import com.corundumstudio.socketio.SocketIOServer;
import com.ym.mec.web.support.anno.OnNamespace;
import jodd.util.StringUtil;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;

@Component
public class NamespaceFactoryBean implements FactoryBean<SocketIONamespace> {

    private Class<?> originalBeanClass;
    private final SocketIOServer server;

    @Autowired
    public NamespaceFactoryBean(SocketIOServer server) {
        this.server = server;
    }

    @Override
    public SocketIONamespace getObject() {

        OnNamespace onNamespace = AnnotationUtils.findAnnotation(originalBeanClass, OnNamespace.class);
        if (onNamespace == null || StringUtil.isBlank(onNamespace.value())) {
            return server.getNamespace("");
        }

        String namespace = onNamespace.value();
        if (!namespace.startsWith("/")) {
            namespace = "/" + namespace;
        }
        return server.addNamespace(namespace);
    }

    @Override
    public Class<?> getObjectType() {
        return SocketIONamespace.class;
    }

    public void setOriginalBeanClass(Class<?> originalBeanClass) {
        this.originalBeanClass = originalBeanClass;
    }
}
