package com.ym.mec.web.support.socket;

import com.corundumstudio.socketio.SocketIOServer;
import com.corundumstudio.socketio.namespace.Namespace;
import com.corundumstudio.socketio.namespace.NamespacesHub;
import com.corundumstudio.socketio.store.pubsub.DispatchMessage;
import com.corundumstudio.socketio.store.pubsub.PubSubStore;
import com.corundumstudio.socketio.store.pubsub.PubSubType;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import java.util.Objects;

@Slf4j
@Component
public class ServerRunner implements CommandLineRunner {

    private final SocketIOServer server;
    private final PubSubStore pubSubStore;
    private final NamespacesHub namespacesHub;

    @Autowired
    public ServerRunner(SocketIOServer server, PubSubStore pubSubStore) {
        this.server = server;
        this.pubSubStore = pubSubStore;
        this.namespacesHub = new NamespacesHub(server.getConfiguration());
    }

    @Override
    public void run(String... args) {

        // 启动服务
        server.start();

        // 订阅消息
        pubSubStore.subscribe(PubSubType.DISPATCH, message -> {

            // 分布式服务空间请求分发
            Namespace namespace = namespacesHub.get(message.getNamespace());
            if (Objects.nonNull(namespace)) {
                log.info("PubSubType.DISPATCH SYNC_MESSAGE room={}, ns={}, nodeId={}", message.getRoom(), message.getNamespace(), message.getNodeId());
                namespace.dispatch(message.getRoom(), message.getPacket());
            }

        }, DispatchMessage.class);

        log.info("--------SocketIO------- SERVER.START PORT={}", server.getConfiguration().getPort());
    }
}
