package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.microsvc.toolkit.middleware.live.LivePluginContext;
import com.microsvc.toolkit.middleware.live.LivePluginService;
import com.microsvc.toolkit.middleware.live.message.LiveRoomMessage;
import com.microsvc.toolkit.middleware.rtc.RTCRoomPluginContext;
import com.microsvc.toolkit.middleware.rtc.RTCRoomPluginService;
import com.microsvc.toolkit.middleware.rtc.impl.TencentCloudRTCPlugin;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.*;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.entity.CourseSchedule.CourseScheduleType;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.mapper.CourseSchedulePlusMapper;
import com.ym.mec.biz.dal.mapper.LiveGroupPlusMapper;
import com.ym.mec.biz.dal.page.*;
import com.ym.mec.biz.dal.school.dto.ClassesForDayDto;
import com.ym.mec.biz.dal.school.dto.CourseStudentDto;
import com.ym.mec.biz.dal.vo.CourseScheduleWrapper;
import com.ym.mec.biz.dal.vo.ImLiveRoomVideoVo;
import com.ym.mec.biz.dal.wrapper.DailySummaryOfClassesForTheCurrentSemesterWrapper;
import com.ym.mec.biz.dal.wrapper.LiveGroupWrapper;
import com.ym.mec.biz.dal.wrapper.TeachingPointWrapper;
import com.ym.mec.biz.event.source.CourseEventSource;
import com.ym.mec.biz.event.source.SendSeoMessageSource;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.QueryInfo;
import com.ym.mec.common.page.WrapperUtil;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext.MessageSender;
import com.ym.mec.util.collection.ListUtil;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateConvertor;
import com.ym.mec.util.date.DateUtil;
import com.ym.mec.util.json.JsonUtil;

import org.apache.commons.collections.CollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.ListUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.Nullable;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.text.MessageFormat;
import java.text.ParseException;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.timevale.tgtext.text.html.b.i;
import static com.ym.mec.biz.dal.entity.CourseSchedule.CourseScheduleType.MUSIC_NETWORK;
import static com.ym.mec.biz.dal.enums.GroupType.*;
import static com.ym.mec.biz.dal.enums.PracticeGroupType.FREE;
import static com.ym.mec.biz.dal.enums.PracticeGroupType.TRIAL;
import static com.ym.mec.biz.dal.enums.SysUserRoleEnum.ORGAN_MANAGER;

@Slf4j
@Service
public class CourseScheduleServiceImpl extends BaseServiceImpl<Long, CourseSchedule> implements CourseScheduleService {

    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private CourseScheduleTeacherSalaryDao courseScheduleTeacherSalaryDao;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private CourseScheduleModifyLogService courseScheduleModifyLogService;
    @Autowired
    private CourseScheduleComplaintsDao courseScheduleComplaintsDao;
    @Autowired
    private CourseScheduleEvaluateDao courseScheduleEvaluateDao;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private SysTenantConfigService sysTenantConfigService;
    @Autowired
    private StudentAttendanceDao studentAttendanceDao;
    @Autowired
    private ClassGroupStudentMapperDao classGroupStudentMapperDao;
    @Autowired
    private TeacherDefaultPracticeGroupSalaryDao teacherDefaultPracticeGroupSalaryDao;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    @Lazy
    private CourseScheduleTeacherSalaryService courseScheduleTeacherSalaryService;
    @Autowired
    private VipGroupDao vipGroupDao;
	@Autowired
	private CoursesGroupDao coursesGroupDao;
    @Autowired
    private TeacherDefaultVipGroupSalaryDao teacherDefaultVipGroupSalaryDao;
    @Autowired
    private TeacherDefaultMusicGroupSalaryDao teacherDefaultMusicGroupSalaryDao;
    @Autowired
    private ClassGroupTeacherMapperService classGroupTeacherMapperService;
    @Autowired
    private CourseScheduleStudentPaymentService courseScheduleStudentPaymentService;
    @Autowired
    private CourseScheduleStudentPaymentDao courseScheduleStudentPaymentDao;
    @Autowired
    private TeacherAttendanceDao teacherAttendanceDao;
    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private ClassGroupDao classGroupDao;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private SubjectDao subjectDao;
    @Autowired
    private ClassGroupTeacherSalaryDao classGroupTeacherSalaryDao;
    @Autowired
    private PracticeGroupDao practiceGroupDao;
    @Autowired
    @Lazy
    private VipGroupService vipGroupService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private SchoolDao schoolDao;
    @Autowired
	private ImGroupMemberService imGroupMemberService;
    @Autowired
	private ImUserFriendService imUserFriendService;
    @Autowired
    private CourseHomeworkService courseHomeworkService;
    @Autowired
    private GroupDao groupDao;
    @Autowired
    private TeacherDao teacherDao;
    @Autowired
	private CourseScheduleReviewDao courseScheduleReviewDao;
    @Autowired
	private PracticeGroupService practiceGroupService;
    @Autowired
	private CoursesGroupModifyLogDao coursesGroupModifyLogDao;
    @Autowired
	private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;
    @Autowired
	private CourseEventSource courseEventSource;
    @Autowired
	private StudentServeService studentServeService;
    @Autowired
	private SendSeoMessageSource sendSeoMessageSource;
    @Autowired
	private VipGroupActivityDao vipGroupActivityDao;
    @Autowired
	private ActivityUserMapperDao activityUserMapperDao;
    @Autowired
	private MusicGroupTrainPlanService musicGroupTrainPlanService;
    @Autowired
	private PracticeGroupSellPriceDao practiceGroupSellPriceDao;
    @Autowired
	private TenantAssetsInfoService tenantAssetsInfoService;
    @Autowired
	private TenantConfigService tenantConfigService;
    @Autowired
	private VipGroupCategoryDao vipGroupCategoryDao;
    @Autowired
	private StudentTeacherMapperService studentTeacherMapperService;
    @Autowired
	private CourseScheduleConvertDao courseScheduleConvertDao;
    @Autowired
    private CourseSchedulePlusMapper courseSchedulePlusMapper;
    @Autowired
    private TeacherService teacherService;
    @Autowired
    private TeacherAttendanceService teacherAttendanceService;
    @Autowired
    private LessonExaminationService lessonExaminationService;
	@Autowired
	private ImLiveBroadcastRoomService imLiveBroadcastRoomService;
	@Autowired
	private LivePluginContext livePluginContext;
    @Autowired
    private ImLiveRoomVideoService imLiveRoomVideoService;

    @Autowired
    private RTCRoomPluginContext rtcRoomPluginContext;
    @Autowired
    private ImLiveRoomVideoDao imLiveRoomVideoDao;

    @Autowired
    private LiveGroupPlusMapper liveGroupPlusMapper;

    @Autowired
    private TenantInfoService tenantInfoService;
    @Autowired
    private CooperationOrganService cooperationOrganService;
    @Autowired
    private OrganizationService organizationService;

    private final Logger businessLogger = LoggerFactory
            .getLogger(this.getClass());

    @Override
    public BaseDAO<Long, CourseSchedule> getDAO() {
        return courseScheduleDao;
    }

    @Override
    public CourseScheduleDao getDao() {
        return courseScheduleDao;
    }

    @Override
	public CourseSchedule queryById(Long courseScheduleId) {
    	CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleId);
    	if(courseSchedule != null){
    		//查询助教列表
    		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList =  courseScheduleTeacherSalaryDao.findByCourseSchedule(courseScheduleId);
    		if(courseScheduleTeacherSalaryList != null){
    			List<Integer> teacherIdList = courseScheduleTeacherSalaryList.stream().filter(t -> TeachTypeEnum.TEACHING.equals(t.getTeacherRole())).map(CourseScheduleTeacherSalary::getUserId).collect(Collectors.toList());
    			courseSchedule.setTeachingTeacherIdList(teacherIdList);
    		}
    	}
		return courseSchedule;
	}

	@Override
    public TeacherAttendanceDto getCurrentCourseDetail(Long courseID) {
        SysUser user = sysUserService.getUser();
        if (null == user.getTenantId()) {
            throw new BizException("用户机构信息异常，请联系管理员");
        }
        if (Objects.isNull(courseID)) {
            throw new BizException("参数错误!");
        }
        TeacherAttendanceDto currentCourseDetail = courseScheduleDao.getCurrentCourseDetail(courseID, user.getId());
        if (Objects.isNull(currentCourseDetail)) {
            throw new BizException("课程不存在");
        }
        if(Objects.nonNull(currentCourseDetail.getSchoolId())){
            boolean enableIgnoreAttendanceTime = enableIgnoreAttendanceTime(currentCourseDetail.getStartClassTime(),
                    user.getId(),
                    currentCourseDetail.getSchoolId().intValue());
            currentCourseDetail.setOnlyNormal(enableIgnoreAttendanceTime?YesOrNoEnum.YES:YesOrNoEnum.NO);
        }else{
            currentCourseDetail.setOnlyNormal(YesOrNoEnum.NO);
        }

		Set<Long> teacherServeCourseIds = studentServeService.getTeacherServeCourseIds(user.getId());
        if(teacherServeCourseIds.contains(courseID)){
			currentCourseDetail.setEnableAssignHomework(1);
		}

		currentCourseDetail.setCurrentClassTimes(courseScheduleDao.countClassGroupOverCourseNum(currentCourseDetail.getClassId()));
//        SysConfig sysConfig = sysConfigService.findByParamName(SysConfigService.ENABLE_STUDENT_ATTENDANCE_TIME_RANGE);
        String sysConfigVip = sysTenantConfigService.getTenantConfigValue(SysConfigService.ENABLE_STUDENT_ATTENDANCE_TIME_RANGE_VIP,user.getTenantId());
//        currentCourseDetail.setEnableStudentAttendanceTimeRange(sysConfig.getParanValue(Integer.class));
        currentCourseDetail.setEnableStudentAttendanceTimeRangeVip(StringUtils.isEmpty(sysConfigVip)?20:Integer.parseInt(sysConfigVip));
        int studentNum = studentAttendanceDao.countByCourseSchedule(courseID);
        if (Objects.nonNull(studentNum)) {
            currentCourseDetail.setStudentAttendanceIsFirstTime(studentNum > 0 ? 0 : 1);
        } else {
            currentCourseDetail.setStudentAttendanceIsFirstTime(1);
        }

        int normalRemindNum = studentAttendanceDao.countNormalRemindNum(courseID);
        currentCourseDetail.setNormalRemind(normalRemindNum<=0?0:1);
		currentCourseDetail.setCurrentTime(new Date());
		String tenantConfigValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.OFFLINE_ADVANCE_SIGN_IN_MINUTES, user.getTenantId());
		if(StringUtils.isEmpty(tenantConfigValue)){
			tenantConfigValue = "20";
		}
		currentCourseDetail.setAdvanceSignInMinutes(Integer.parseInt(tenantConfigValue));
		String gpsRange = sysTenantConfigService.getTenantConfigValue(SysConfigService.ATTENDANCE_RANGE,user.getTenantId());
		Integer attendanceRange = 0;
		if(StringUtils.isNotEmpty(gpsRange)){
			attendanceRange = Integer.parseInt(gpsRange);
		}
		currentCourseDetail.setAttendanceRange(attendanceRange);
		String configValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.ADVANCE_SIGN_OUT_MINUTES, user.getTenantId());
		if(StringUtils.isEmpty(configValue)){
			configValue = "360";
		}
		currentCourseDetail.setAdvanceSignOutMinutes(Integer.parseInt(configValue));
		CourseSchedule courseSchedule = courseScheduleDao.get(courseID);
		//获取课程规划
		MusicGroupTrainPlanSaveDto schoolTerm = musicGroupTrainPlanService.getSchoolTerm(courseSchedule);
		int courseNumNo = courseScheduleDao.getCourseNumNo(schoolTerm);
		List<MusicGroupTrainPlan> planList = musicGroupTrainPlanService.queryPlan(schoolTerm);
		if(planList.size() >= courseNumNo){
			currentCourseDetail.setCoursePlan(planList.get(courseNumNo - 1).getPlan());
		}
		return currentCourseDetail;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
	public void batchDeleteCourseSchedules(List<Long> courseScheduleIds) {

		List<CourseSchedule> courseScheduleList = courseScheduleDao.findByCourseScheduleIds(courseScheduleIds);

		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseScheduleIds);
		Set<Long> settlementCourseIds = new HashSet<>();
		if(!CollectionUtils.isEmpty(courseScheduleTeacherSalaries)){
			settlementCourseIds = courseScheduleTeacherSalaries.stream().filter(c->Objects.nonNull(c.getSettlementTime())).map(CourseScheduleTeacherSalary::getCourseScheduleId).collect(Collectors.toSet());
		}

		Date date = new Date();
		if (courseScheduleList != null) {
			for (CourseSchedule cs : courseScheduleList) {
				if(cs.getStartClassTime().compareTo(date)<0){
					throw new BizException("删除失败，进行中或已结束的课程不能删除");
				}
				if(settlementCourseIds.contains(cs.getId())){
					throw new BizException("删除失败，已结算的课程不能删除");
				}
				if (CourseStatusEnum.UNDERWAY.equals(cs.getStatus())) {
					throw new BizException("删除失败，进行中的课程不能删除");
				}
				if (Objects.nonNull(cs.getIsLock())&&cs.getIsLock()==1) {
					throw new BizException("删除失败，被锁定的课程不能删除");
				}
				if(Objects.nonNull(cs.getNewCourseId())){
					throw new BizException("删除失败，被合并课程不能删除");
				}
			}
		}
		courseScheduleDao.batchDeleteAllCourseSchedules(courseScheduleIds);
		courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
		courseScheduleStudentPaymentDao.deleteByCourseSchedule(courseScheduleIds);
		// 删除考勤
		teacherAttendanceDao.batchDeleteByCourseSchedules(courseScheduleIds);
		studentAttendanceDao.deleteByCourseSchedules(courseScheduleIds);

		// 删除作业
		courseHomeworkService.delHomwworkByCourseScheduleId(courseScheduleIds);

		List<Integer> classGroupIdList = courseScheduleList.stream().map(t -> t.getClassGroupId()).collect(Collectors.toList());
		// 更新班级信息
		if (classGroupIdList != null && classGroupIdList.size() > 0) {
			for (Integer classGroupId : classGroupIdList) {
				classGroupService.updateClassGroupInfo(classGroupId);
			}
		}
	}

	@Override
	public void batchDeleteCourseSchedulesWithoutCheck(List<CourseSchedule> courseScheduleList) {
    	if(CollectionUtils.isEmpty(courseScheduleList)){
    		return;
		}
		List<Long> courseScheduleIds = courseScheduleList.stream().map(CourseSchedule::getId).collect(Collectors.toList());

		courseScheduleDao.batchDelete(courseScheduleIds);
		courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
		courseScheduleStudentPaymentDao.deleteByCourseSchedule(courseScheduleIds);
		// 删除考勤
		teacherAttendanceDao.batchDeleteByCourseSchedules(courseScheduleIds);
		studentAttendanceDao.deleteByCourseSchedules(courseScheduleIds);

		// 删除作业
		courseHomeworkService.delHomwworkByCourseScheduleId(courseScheduleIds);

		List<Integer> classGroupIdList = courseScheduleList.stream().map(t -> t.getClassGroupId()).collect(Collectors.toList());
		// 更新班级信息
		if (classGroupIdList != null && classGroupIdList.size() > 0) {
			for (Integer classGroupId : classGroupIdList) {
				classGroupService.updateClassGroupInfo(classGroupId);
			}
		}
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public void batchDeleteMusicGroupNotStartCourse(String musicGroupId, GroupType groupType) {
        if (StringUtils.isBlank(musicGroupId)) {
            throw new BizException("请指定乐团");
        }
        List<CourseSchedule> musicGroupCourseSchedules = courseScheduleDao.findMusicGroupCourseSchedules(musicGroupId, groupType.getCode(), CourseStatusEnum.NOT_START.getCode());
        if (CollectionUtils.isEmpty(musicGroupCourseSchedules)) {
            throw new BizException("此乐团不存在未上课程");
        }
        List<Long> courseScheduleIds = musicGroupCourseSchedules.stream()
                .map(CourseSchedule::getId)
                .collect(Collectors.toList());
        batchDeleteCourseSchedules(courseScheduleIds);
    }

	@Override
	public void batchDeleteMusicGroup(String musicGroupId, GroupType groupType) {
		if (StringUtils.isBlank(musicGroupId)) {
			throw new BizException("请指定乐团");
		}
		List<CourseSchedule> musicGroupCourseSchedules = courseScheduleDao.findMusicGroupAllCourseSchedules(musicGroupId, groupType.getCode());
		if (CollectionUtils.isEmpty(musicGroupCourseSchedules)) {
			throw new BizException("此乐团不存在未上课程");
		}
		batchDeleteCourseSchedulesWithoutCheck(musicGroupCourseSchedules);
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public void batchDeleteMusicGroupCourseWithStudent(String musicGroupId, Integer userId, GroupType groupType) {
        if (StringUtils.isBlank(musicGroupId)) {
            throw new BizException("请指定课程组");
        }
        List<CourseSchedule> musicGroupCourseSchedules = courseScheduleDao.findMusicGroupCourseSchedulesWithStudent(musicGroupId, groupType.getCode(), CourseStatusEnum.NOT_START.getCode(), userId);
        if (!CollectionUtils.isEmpty(musicGroupCourseSchedules)) {
            List<Long> courseScheduleIds = musicGroupCourseSchedules.stream()
                    .map(courseSchedule -> courseSchedule.getId())
                    .collect(Collectors.toList());
            courseScheduleDao.deleteMusicGroupCourseSchedulesWithStudent(courseScheduleIds, userId);
			studentAttendanceDao.deleteByCourseAndUser(courseScheduleIds,userId);
        }
    }

    @Override
    public YesOrNoEnum enableOnlyNormalAttendance(Date startClassTime, Long userId, boolean checkSchool, Integer schoolId) {
        Date endTime = startClassTime;
        Date startTime = DateUtil.addHours(endTime, -1);
        List<TeacherAttendanceDto> teacherCourseByDateSpeeds = courseScheduleDao.getTeacherCourseByDateSpeed(startTime, endTime, userId, schoolId);
        if (CollectionUtils.isEmpty(teacherCourseByDateSpeeds)) {
            return YesOrNoEnum.NO;
        } else {
            if (checkSchool
                    && Objects.nonNull(schoolId)) {
                return YesOrNoEnum.YES;
            } else {
                return YesOrNoEnum.NO;
            }
        }
    }

	@Override
	public boolean enableIgnoreAttendanceTime(Date startClassTime, Integer teacherId, Integer schoolId) {
		CourseSchedule prevCourse = courseScheduleDao.getTeacherAttendanceCourseBeforeStartTime(startClassTime, teacherId);
		if(Objects.isNull(prevCourse)||Objects.isNull(prevCourse.getSchoolId())){
			return false;
		}
		if(!schoolId.equals(prevCourse.getSchoolId())){
			return true;
		}
		int minutesBetween = DateUtil.minutesBetween(prevCourse.getEndClassTime(), startClassTime);
		if(minutesBetween>60){
			return false;
		}else{
			return true;
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean batchAddCourseSchedule(Integer classGroupId, int coursesTimes, Date startDate, Date groupEndDate, Date groupStartDate, List<CourseTimeDto> teachingArrangementList,
			TeachModeEnum teachMode, CourseScheduleType type, Integer schoolId, boolean isJumpHoliday, List<CourseScheduleTeacherSalary> oldCourseScheduleTeacherSalaries) {
		ClassGroup classGroup = classGroupService.get(classGroupId);
		if (classGroup == null) {
			throw new BizException("排课失败，班级信息不存在");
		}

		if (teachMode == TeachModeEnum.OFFLINE && schoolId == null) {
			throw new BizException("线下课请指定教学点");
		}

		if (type == null) {
			throw new BizException("请指定课程类型");
		}

		if (type == CourseScheduleType.VIP) {
			throw new BizException("暂不支持vip课加课");
		}

		if(type == CourseSchedule.CourseScheduleType.HIGH){
        	type = CourseScheduleType.HIGH_ONLINE;
        }

		if (teachingArrangementList == null || teachingArrangementList.size() == 0) {
			throw new BizException("请设置课时安排");
		}

		if(!CollectionUtils.isEmpty(oldCourseScheduleTeacherSalaries)){
			oldCourseScheduleTeacherSalaries.sort(Comparator.comparing(c->c.getCourseSchedule().getStartClassTime()));
		}

		BigDecimal subSidy = new BigDecimal(0);
		// 查询教学点补贴
		if (schoolId != null) {
			School school = schoolDao.get(schoolId);
			if (school == null) {
				throw new BizException("指定的教学点不存在");
			}
			subSidy = school.getSubsidy();
		}

		//查询声部
		String subjectName = subjectDao.findNames(classGroup.getSubjectIdList());

		// 查询班级老师信息
		List<ClassGroupTeacherMapper> classGroupTeacherMapperList = classGroupTeacherMapperService.getClassGroupTeachers(classGroupId);

		// 查询主教老师
		Integer teacherId = null;
		for (ClassGroupTeacherMapper cgtm : classGroupTeacherMapperList) {
			if (cgtm.getTeacherRole() == TeachTypeEnum.BISHOP) {
				teacherId = cgtm.getUserId();
				break;
			}
		}

		if (teacherId == null) {
			throw new BizException("班级缺少主教老师");
		}

		List<Integer> teacherIdList = classGroupTeacherMapperList.stream().map(ClassGroupTeacherMapper::getUserId).collect(Collectors.toList());

		// 查询老师列表
		List<Teacher> teacherList = teacherDao.findByTeacherIds(teacherIdList);
		Map<Integer, Teacher> teacherMap = teacherList.stream().collect(Collectors.toMap(Teacher::getId, t -> t));

		Map<Integer, BigDecimal> teacherPracticeSalaryMap = new HashMap<Integer, BigDecimal>();
		Map<Integer, TeacherDefaultMusicGroupSalary> teacherMusicSalaryMap = new HashMap<Integer, TeacherDefaultMusicGroupSalary>();

		Date entryDate = null;
		String settlementType = null;
		Integer organId = null;

		// 查询老师默认课酬
		if (type == CourseScheduleType.PRACTICE) {
			PracticeGroup practiceGroup = practiceGroupDao.get(Long.valueOf(classGroup.getMusicGroupId()));
			if(Objects.isNull(practiceGroup)){
				throw new BizException("找不到网管课信息");
			}
			if(!GroupStatusEnum.NORMAL.equals(practiceGroup.getGroupStatus())){
				throw new BizException("非进行中网管课不支持加课");
			}
			groupEndDate = practiceGroup.getCoursesExpireDate();
			organId=practiceGroup.getOrganId();

			List<TeacherDefaultPracticeGroupSalary> teacherDefaultPracticeGroupSalaryList = teacherDefaultPracticeGroupSalaryDao
					.queryByUserIdList(teacherIdList);

			teacherPracticeSalaryMap = teacherDefaultPracticeGroupSalaryList.stream().collect(
					Collectors.toMap(TeacherDefaultPracticeGroupSalary::getUserId, TeacherDefaultPracticeGroupSalary::getMainTeacherSalary));

		} else {
			MusicGroup musicGroup = musicGroupDao.get(classGroup.getMusicGroupId());
			if (musicGroup == null) {
				throw new BizException("找不到乐团信息");
			}
			organId=musicGroup.getOrganId();
			// 乐团结算方式
			settlementType = musicGroup.getSettlementType().getCode();

			// 所有老师默认乐团课酬
			List<TeacherDefaultMusicGroupSalary> teacherDefaultMusicGroupSalaries = teacherDefaultMusicGroupSalaryDao.findByTeacher(teacherIdList);

			for (TeacherDefaultMusicGroupSalary tdms : teacherDefaultMusicGroupSalaries) {
				if (tdms.getCourseScheduleType() == type) {
					teacherMusicSalaryMap.put(tdms.getUserId(), tdms);
				}
			}
		}

		// 查询班级学生信息
		List<ClassGroupStudentMapper> classGroupStudentMapperList = classGroupStudentMapperDao.findByClassGroup(classGroupId);
		String usernameList = classGroupStudentMapperList.stream().map(ClassGroupStudentMapper::getUserName).collect(Collectors.joining("、"));

		Map<Integer, List<CourseTimeDto>> dayOfWeekMap = new HashMap<Integer, List<CourseTimeDto>>();
		for(CourseTimeDto dto : teachingArrangementList){
			//检查是否有开始时间
			if (StringUtils.isBlank(dto.getStartClassTime())) {
				throw new BizException("必须设置排课开始时间");
			}
			List<CourseTimeDto> list = dayOfWeekMap.get(dto.getDayOfWeek());
			if(list == null){
				list = new ArrayList<CourseTimeDto>();
			}
			list.add(dto);
			dayOfWeekMap.put(dto.getDayOfWeek(), list);
		}

        Integer tenantId = TenantContextHolder.getTenantId();
        Calendar calendar = Calendar.getInstance();
		Date now = new Date();
		List<CourseTimeDto> courseTimeDtoList = null;
		List<CourseSchedule> courseScheduleList = new ArrayList<CourseSchedule>();
		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = new ArrayList<CourseScheduleTeacherSalary>();
		List<CourseScheduleStudentPayment> courseScheduleStudentPayments = new ArrayList<CourseScheduleStudentPayment>();
		List<TeacherAttendance> teacherAttendances = new ArrayList<TeacherAttendance>();
		Set<String> holidayDays = new HashSet<>();
		if (isJumpHoliday) {
			String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING,tenantId);
			if(StringUtils.isNotBlank(holidaySetting)){
				holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
			}
		}
		calendar.setTime(startDate);

		int currentCourseTimes = 0;
		while (coursesTimes != currentCourseTimes) {
            if (holidayDays.contains(DateUtil.format(calendar.getTime(), "yyyy-MM-dd"))) {
                calendar.add(Calendar.DATE, 1);
                continue;
            }
			// 查询当前日期是周几
			int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
			if (dayOfWeek == 1) {
				dayOfWeek = 7;
			} else {
				dayOfWeek = dayOfWeek - 1;
			}
			courseTimeDtoList = dayOfWeekMap.get(dayOfWeek);
			if (courseTimeDtoList != null) {
				for(CourseTimeDto courseTimeDto : courseTimeDtoList){

					// 课表
					CourseSchedule courseSchedule = new CourseSchedule();
					courseSchedule.setMusicGroupId(classGroup.getMusicGroupId());
					courseSchedule.setClassGroupId(classGroup.getId());
					courseSchedule.setStatus(CourseStatusEnum.NOT_START);
					courseSchedule.setClassDate(calendar.getTime());

					String dateYmdStr = DateUtil.dateToString(calendar.getTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);

					String startTimeStr = dateYmdStr + " " + courseTimeDto.getStartClassTime();
					Date courseStartTime = DateUtil.stringToDate(startTimeStr, "yyyy-MM-dd HH:mm");
					courseSchedule.setStartClassTime(courseStartTime);

					if (StringUtils.isNotBlank(courseTimeDto.getEndClassTime())) {
						String endTimeStr = dateYmdStr + " " + courseTimeDto.getEndClassTime();
						Date courseEndTime = DateUtil.stringToDate(endTimeStr, "yyyy-MM-dd HH:mm");
						courseSchedule.setEndClassTime(courseEndTime);

						if (courseEndTime.before(courseStartTime)) {
							throw new BizException("课时上课时间安排错误");
						}
					} else {
						if (type == CourseScheduleType.PRACTICE) {
							courseSchedule.setEndClassTime(DateUtil.addMinutes(courseStartTime, 25));
						} else {
							throw new BizException("必须设置上课结束时间");
						}
					}

					if (!DateUtil.isSameDay(courseSchedule.getStartClassTime(), courseSchedule.getEndClassTime())) {
						throw new BizException("上课时间不允许跨天，请重新调整上课时间");
					}

					courseSchedule.setTeacherId(teacherId);
					courseSchedule.setActualTeacherId(teacherId);
					courseSchedule.setCreateTime(now);
					courseSchedule.setUpdateTime(now);
					courseSchedule.setTeachMode(teachMode);
					courseSchedule.setType(type);
					courseSchedule.setGroupType(classGroup.getGroupType());
					courseSchedule.setName(classGroup.getName());
					courseSchedule.setSchoolId(schoolId);
					courseSchedule.setSubsidy(subSidy);
					courseSchedule.setOrganId(organId);
					if (type == CourseScheduleType.PRACTICE || type == CourseScheduleType.VIP) {
						courseSchedule.setName(subjectName + "•" + usernameList);
					} else {
						courseSchedule.setName(type.getMsg());
					}

					// 判断课程时间是否超过排课结束时间
					if(groupStartDate != null){
						if(groupStartDate.after(calendar.getTime())){
							throw new BizException("调整时间不得早于开课时间（{}）",groupStartDate);
						}
					}
					/*if (groupEndDate != null) {
						if (calendar.getTime().after(groupEndDate)) {
							throw new BizException("排课时间超出排课截止日期（{}）", DateUtil.dateToString(groupEndDate, DateUtil.DATE_FORMAT_MIN));
						}
					}*/

					courseScheduleDao.insert(courseSchedule);
					courseScheduleList.add(courseSchedule);

					CourseScheduleTeacherSalary oldCourseMainTeacherSalary = null;
					if(!CollectionUtils.isEmpty(oldCourseScheduleTeacherSalaries)){
						oldCourseMainTeacherSalary = oldCourseScheduleTeacherSalaries.get(courseScheduleList.size()-1);
					}

					// 课程与老师薪水表
					for (ClassGroupTeacherMapper cgtm : classGroupTeacherMapperList) {
						CourseScheduleTeacherSalary courseScheduleTeacherSalary = new CourseScheduleTeacherSalary();
						courseScheduleTeacherSalary.setCourseScheduleId(courseSchedule.getId());
						courseScheduleTeacherSalary.setGroupType(classGroup.getGroupType());
						courseScheduleTeacherSalary.setMusicGroupId(classGroup.getMusicGroupId());
						courseScheduleTeacherSalary.setTeacherRole(cgtm.getTeacherRole());
						courseScheduleTeacherSalary.setUserId(cgtm.getUserId());
						if(TeachTypeEnum.BISHOP.equals(cgtm.getTeacherRole()) && Objects.nonNull(oldCourseMainTeacherSalary) && !oldCourseMainTeacherSalary.getEnableChangeSalary()){
							courseScheduleTeacherSalary.setExpectSalary(oldCourseMainTeacherSalary.getExpectSalary());
							courseScheduleTeacherSalary.setSubsidy(oldCourseMainTeacherSalary.getSubsidy());
							courseScheduleTeacherSalary.setEnableChangeSalary(oldCourseMainTeacherSalary.getEnableChangeSalary());
						}else if (type == CourseScheduleType.PRACTICE) {
							PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(classGroup.getMusicGroupId()));
							if(practiceGroup == null){
								throw new BizException("课程组不存在");
							}else if (FREE.equals(practiceGroup.getType())){
								BigDecimal salary = teacherPracticeSalaryMap.get(cgtm.getUserId());
								if (salary == null) {
									throw new BizException("请设置老师网管课课酬");
								}
								BigDecimal teacherSalary = BigDecimal.ZERO;
								if(salary.compareTo(new BigDecimal(0))>0){
									SysConfig practiceCourseSalaryConfig = sysConfigService.findByParamName(SysConfigService.PRACTICE_COURSE_SALARY);
									teacherSalary = new BigDecimal(practiceCourseSalaryConfig.getParanValue());
								}
								courseScheduleTeacherSalary.setExpectSalary(teacherSalary);
							}else if (practiceGroup.getType() == TRIAL){
								courseScheduleTeacherSalary.setExpectSalary(new BigDecimal(0));
							}else {
								SysConfig practiceCourseSalaryConfig = sysConfigService.findByParamName(SysConfigService.PRACTICE_COURSE_SALARY);
								BigDecimal freeTeacherSalary = new BigDecimal(practiceCourseSalaryConfig.getParanValue());

								if(Objects.nonNull(oldCourseMainTeacherSalary)&&freeTeacherSalary.compareTo(oldCourseMainTeacherSalary.getExpectSalary())==0){
									courseScheduleTeacherSalary.setExpectSalary(freeTeacherSalary);
								}else{
									BigDecimal salary = teacherPracticeSalaryMap.get(cgtm.getUserId());
									if (salary == null) {
										throw new BizException("请设置老师网管课课酬");
									}
									courseScheduleTeacherSalary.setExpectSalary(salary);
								}
							}
							Integer vipGroupActivityId = practiceGroup.getVipGroupActivityId();
							if(vipGroupActivityId != null){
								ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(practiceGroup.getId(),"PRACTICE",null);
								if(activityUserMapper != null){
									Map<String, BigDecimal> map = practiceGroupService.countPracticeGroupPredictFee1(practiceGroup, cgtm.getUserId());
									if(practiceGroup.getId().equals(activityUserMapper.getPracticeGroupId())){
										courseScheduleTeacherSalary.setExpectSalary(map.get("onlineTeacherSalary"));
									}else {
										courseScheduleTeacherSalary.setExpectSalary(map.get("giveTeacherDefaultSalary"));
									}
								}
							}
						} else if (type == CourseScheduleType.VIP) {

						} else {
							TeacherDefaultMusicGroupSalary tdms = teacherMusicSalaryMap.get(cgtm.getUserId());

							int newMinutes = DateUtil.minutesBetween(courseSchedule.getStartClassTime(), courseSchedule.getEndClassTime());

							int unitMinutes = 30;

							// 如果当前教师是在指定日期之后入职的，则按照3.0的方式结算
							if (type != CourseScheduleType.CLASSROOM) {
								Teacher teacher = teacherMap.get(teacherId);
								if (Objects.nonNull(teacher.getEntryDate())
										&& (teacher.getEntryDate().after(entryDate) || teacher.getEntryDate().getTime() == entryDate.getTime())) {
									settlementType = SalarySettlementTypeEnum.GRADIENT_SALARY.getCode();
								}
							}
							if (StringUtils.equals(settlementType, "TEACHER_DEFAULT")) {
								if (courseScheduleTeacherSalary.getTeacherRole() == TeachTypeEnum.BISHOP) {
									courseScheduleTeacherSalary.setExpectSalary(new BigDecimal(tdms.getMainTeacher30MinSalary().doubleValue() * newMinutes
											/ unitMinutes));
								} else {
									courseScheduleTeacherSalary.setExpectSalary(new BigDecimal(tdms.getAssistantTeacher30MinSalary().doubleValue() * newMinutes
											/ unitMinutes));
								}
							} else if (StringUtils.equals(settlementType, "GRADIENT_SALARY")) {
								unitMinutes = 90;
								if (courseScheduleTeacherSalary.getTeacherRole() == TeachTypeEnum.BISHOP) {
									courseScheduleTeacherSalary.setExpectSalary(new BigDecimal(tdms.getMainTeacher90MinSalary().doubleValue() * newMinutes
											/ unitMinutes));
								} else {
									courseScheduleTeacherSalary.setExpectSalary(new BigDecimal(tdms.getAssistantTeacher90MinSalary().doubleValue() * newMinutes
											/ unitMinutes));
								}
							}

							//课堂课课酬
							if (type == CourseScheduleType.CLASSROOM) {
								BigDecimal salary = courseScheduleTeacherSalary.getTeacherRole().equals(TeachTypeEnum.BISHOP) ? tdms.getMainTeacher90MinSalary() : tdms.getAssistantTeacher90MinSalary();
								courseScheduleTeacherSalary.setExpectSalary(salary);
							}
							//基础技能提高课
							if(type == CourseScheduleType.HIGH_ONLINE || type == CourseScheduleType.HIGH){

								if(StringUtils.isBlank(tdms.getSalaryRuleJson())){
				            		throw new BizException("线上基础技能课课酬设置异常");
				                }
								int studentNum = courseScheduleStudentPaymentDao.countCourseOnlyStudentNum(courseSchedule.getId());
								if(studentNum<=0){
									studentNum = classGroupStudentMapperDao.countClassGroupNormalStudentNum(courseSchedule.getClassGroupId());
								}
								BigDecimal salary = JSON.parseObject(tdms.getSalaryRuleJson()).getBigDecimal(studentNum+"");
								if(studentNum<=0){
									salary = new BigDecimal(0);
								}
								courseScheduleTeacherSalary.setExpectSalary(salary);
							}

							if (type.equals(CourseSchedule.CourseScheduleType.MUSIC_NETWORK)) {
								courseScheduleTeacherSalary.setExpectSalary(JSON.parseObject(tdms.getSalaryRuleJson()).getBigDecimal("1"));
							}
						}
						courseScheduleTeacherSalary.setCreateTime(now);
						courseScheduleTeacherSalary.setUpdateTime(now);
						courseScheduleTeacherSalary.setClassGroupId(classGroup.getId());
						courseScheduleTeacherSalary.setSubsidy(subSidy);
						courseScheduleTeacherSalaries.add(courseScheduleTeacherSalary);
					}

					PracticeGroupSellPrice practiceGroupSellPrice = practiceGroupSellPriceDao.get(organId);
					if(practiceGroupSellPrice == null){
						throw new BizException("请配置网管课价格");
					}
					// 学生缴费记录
					for (ClassGroupStudentMapper cgsm : classGroupStudentMapperList) {
						CourseScheduleStudentPayment courseScheduleStudentPayment = new CourseScheduleStudentPayment();
						courseScheduleStudentPayment.setGroupType(classGroup.getGroupType());
						courseScheduleStudentPayment.setMusicGroupId(classGroup.getMusicGroupId());
						courseScheduleStudentPayment.setCourseScheduleId(courseSchedule.getId());
						courseScheduleStudentPayment.setUserId(cgsm.getUserId());
						courseScheduleStudentPayment.setExpectPrice(BigDecimal.ZERO);
						courseScheduleStudentPayment.setActualPrice(BigDecimal.ZERO);
						courseScheduleStudentPayment.setClassGroupId(classGroup.getId());
						courseScheduleStudentPayment.setCreateTime(now);
						courseScheduleStudentPayment.setUpdateTime(now);
						courseScheduleStudentPayment.setOriginalPrice(practiceGroupSellPrice.getSingleClassMinutesPrice());
						courseScheduleStudentPayments.add(courseScheduleStudentPayment);
					}

					// 教师签到记录
					for (ClassGroupTeacherMapper cgtm : classGroupTeacherMapperList) {
						TeacherAttendance teacherAttendance = new TeacherAttendance();
						teacherAttendance.setMusicGroupId(classGroup.getMusicGroupId());
						teacherAttendance.setTeacherId(cgtm.getUserId());
						teacherAttendance.setClassGroupId(classGroup.getId());
						teacherAttendance.setGroupType(classGroup.getGroupType());
						teacherAttendance.setCourseScheduleId(courseSchedule.getId());
						teacherAttendance.setCreateTime(now);
						teacherAttendances.add(teacherAttendance);
					}
	                ++currentCourseTimes;
	                if(coursesTimes == currentCourseTimes){
	                	break;
	                }
				}
			}
            calendar.add(Calendar.DATE, 1);
		}

		//检查冲突
		checkNewCourseSchedules(courseScheduleList, false,false);

		if (courseScheduleTeacherSalaries.size() > 0) {
			courseScheduleTeacherSalaryDao.batchInsert(courseScheduleTeacherSalaries);
		}

		if (courseScheduleStudentPayments.size() > 0) {
			courseScheduleStudentPaymentDao.batchInsert(courseScheduleStudentPayments);
		}

		if (teacherAttendances.size() > 0) {
			teacherAttendanceDao.batchInsert(teacherAttendances);
		}

		// 更新班级信息
		classGroupService.updateClassGroupInfo(classGroupId);
		return true;
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public void createCourseSchedules(CreateCourseScheduleDto createCourseScheduleDto) {
        List<CourseSchedule> courseSchedules = createCourseScheduleDto.getCourseSchedules();

        if(!CollectionUtils.isEmpty(courseSchedules)){
			List<ClassGroupTeacherMapper> byMusicGroup = classGroupTeacherMapperDao.findAllByMusicGroup(createCourseScheduleDto.getMusicGroupID(),courseSchedules.get(0).getGroupType(),null);
			Map<Integer, List<ClassGroupTeacherMapper>> teacherByClassGroup = byMusicGroup.stream().collect(Collectors.groupingBy(ClassGroupTeacherMapper::getClassGroupId));

			Map<Integer, List<CourseSchedule>> classGroupCourseSchedules = courseSchedules.stream().collect(Collectors.groupingBy(CourseSchedule::getClassGroupId));
			for (Map.Entry<Integer, List<CourseSchedule>> classGroupCoursesEntry : classGroupCourseSchedules.entrySet()) {
				List<ClassGroupTeacherMapper> classGroupTeacherMappers = teacherByClassGroup.get(classGroupCoursesEntry.getKey());
				Set<Integer> teachingTeacherIds = classGroupTeacherMappers.stream().filter(t -> TeachTypeEnum.TEACHING.equals(t.getTeacherRole())).map(ClassGroupTeacherMapper::getUserId).collect(Collectors.toSet());
				classGroupCoursesEntry.getValue().sort(Comparator.comparing(CourseSchedule::getStartClassTime));
				for(int i=0;i<classGroupCoursesEntry.getValue().size();i++){
					classGroupCoursesEntry.getValue().get(i).setTeachingTeacherIdList(new ArrayList<>(teachingTeacherIds));
					if(i==0){
						continue;
					}
					CourseSchedule c1=classGroupCoursesEntry.getValue().get(i-1);
					CourseSchedule c2=classGroupCoursesEntry.getValue().get(i);
					if(c1.getStartClassTime().compareTo(c2.getEndClassTime())<0
						&&c1.getEndClassTime().compareTo(c2.getStartClassTime())>0){
						throw new BizException("排课循环周期时间存在重叠");
					}
				}
//				for (CourseSchedule courseSchedule : classGroupCoursesEntry.getValue()) {
//					courseSchedule.setTeachingTeacherIdList(new ArrayList<>(teachingTeacherIds));
//				}
				//添加课程计划
				batchAddCourseSchedule(classGroupCoursesEntry.getValue());
				//创建学生单节课的缴费记录，乐团课的缴费为0
				courseScheduleStudentPaymentService.createCourseScheduleStudentPaymentByCourseSchedules(classGroupCoursesEntry.getValue());
				classGroupTeacherMapperService.initCourseScheduleTeacherSalary(classGroupCoursesEntry.getValue(),classGroupTeacherMappers);
			}
			List<Long> courseScheduleIds = courseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());
			courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
            //更新课次
            List<Integer> classGroupIds = courseSchedules.stream().map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
            classGroupService.batchUpdateClassCourseTimes(classGroupIds);
        }

        MusicGroup musicGroup = musicGroupService.get(createCourseScheduleDto.getMusicGroupID());

        if (musicGroup.getStatus().equals(MusicGroupStatusEnum.PROGRESS)) {
            classGroupTeacherMapperService.createMusicGroupTeacherMapper(createCourseScheduleDto.getCourseSchedules(), musicGroup.getId());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
	public boolean batchUpdateCourseSchedule(BatchInsertCoursesDto batchInsertCoursesDto) {

		List<CourseSchedule> courseScheduleList = courseScheduleDao.findByCourseScheduleIds(batchInsertCoursesDto.getCourseScheduleIdList());
		List<Integer> classGroupList = courseScheduleList.stream().map(CourseSchedule::getClassGroupId).distinct().collect(Collectors.toList());

		if (classGroupList == null || classGroupList.size() != 1) {
			throw new BizException("修改失败，只支持同一个班级的课程批量调整");
		}

		Integer classGroupId = classGroupList.get(0);
		String musicGroupId = courseScheduleList.get(0).getMusicGroupId();

		// 批量删课
		List<Long> courseScheduleIds = batchInsertCoursesDto.getCourseScheduleIdList();

		// 学生已点名不能调整（请假可以调整）
		for (Long courseScheduleId : courseScheduleIds) {
			List<StudentAttendance> studentAttendanceList = studentAttendanceDao.findByCourseId(courseScheduleId);

			for (StudentAttendance studentAttendance : studentAttendanceList) {
				if (studentAttendance.getStatus() != null && studentAttendance.getStatus() != StudentAttendanceStatusEnum.LEAVE) {
					throw new BizException("操作失败，存在已点名课程");
				}
			}
		}

		Map<Long, CourseSchedule> idCourseMap = courseScheduleList.stream().collect(Collectors.toMap(CourseSchedule::getId, c -> c, (c1, c2) -> c1));
		List<CourseScheduleTeacherSalary> oldCourseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseScheduleIds);
		oldCourseScheduleTeacherSalaries = oldCourseScheduleTeacherSalaries.stream().filter(c->TeachTypeEnum.BISHOP.equals(c.getTeacherRole())).collect(Collectors.toList());
		oldCourseScheduleTeacherSalaries.forEach(o->o.setCourseSchedule(idCourseMap.get(o.getCourseScheduleId())));

		courseScheduleDao.batchDeleteAllCourseSchedules(courseScheduleIds);
		courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIds);
		courseScheduleStudentPaymentDao.deleteByCourseSchedule(courseScheduleIds);
		// 删除考勤
		teacherAttendanceDao.batchDeleteByCourseSchedules(courseScheduleIds);
		studentAttendanceDao.deleteByCourseSchedules(courseScheduleIds);

		// 删除作业
		courseHomeworkService.delHomwworkByCourseScheduleId(courseScheduleIds);
		//删除评论
		courseScheduleReviewDao.delByCourseScheduleIds(courseScheduleIds);
		//删除申述
		courseScheduleComplaintsDao.delByCourseScheduleIds(courseScheduleIds);

		Date endDate = null;
		Date startDate = null;
		if (batchInsertCoursesDto.getType() == CourseScheduleType.PRACTICE) {
			PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(musicGroupId));
			if (practiceGroup == null) {
				throw new BizException("找不到课程组[{}]信息", musicGroupId);
			}
			endDate = practiceGroup.getCoursesExpireDate();
			startDate = practiceGroup.getCoursesStartDate();
		}

		// 批量加课
		batchAddCourseSchedule(classGroupId, batchInsertCoursesDto.getCourseScheduleIdList().size(), batchInsertCoursesDto.getStartDate(), endDate, startDate,
				batchInsertCoursesDto.getTeachingArrangementList(), batchInsertCoursesDto.getTeachMode(), batchInsertCoursesDto.getType(),
				batchInsertCoursesDto.getSchoolId(), batchInsertCoursesDto.getIsJumpHoliday(), oldCourseScheduleTeacherSalaries);

		return true;
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public void batchAddCourseSchedule1(List<CourseSchedule> courseSchedules) {
        if (Objects.isNull(courseSchedules) && courseSchedules.size() <= 0) {
            throw new BizException("参数错误!");
        }
//        checkNewCourseSchedules(courseSchedules, false,false);
        createCourseScheduleName(courseSchedules);
		this.batchAddCourse(courseSchedules);
		courseScheduleModifyLogService.pushSchoolCourseCreate(courseSchedules.get(0));
    }

	@Override
    @Transactional(rollbackFor = Exception.class)
    public void batchAddCourseSchedule(List<CourseSchedule> courseSchedules) {
        if (Objects.isNull(courseSchedules) && courseSchedules.size() <= 0) {
            throw new BizException("参数错误!");
        }
        checkNewCourseSchedules(courseSchedules, false,false);
        createCourseScheduleName(courseSchedules);
		this.batchAddCourse(courseSchedules);
		courseScheduleModifyLogService.pushSchoolCourseCreate(courseSchedules.get(0));
    }

	@Override
	public void batchAddCourse(List<CourseSchedule> courseSchedules) {
		CourseSchedule courseSchedule = courseSchedules.get(0);
		if(courseSchedule.getId() == null){
			courseScheduleDao.batchAddCourseSchedules(courseSchedules);
		}else {
			courseScheduleDao.batchAddCourseSchedulesHasId(courseSchedules);
		}
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdateCourseSchedule(List<CourseSchedule> courseSchedules, String musicGroupID) {
        if (Objects.isNull(courseSchedules) && courseSchedules.size() <= 0) {
            throw new BizException("请选择需要调整的课程!");
        }
        if (Objects.isNull(musicGroupID)) {
            throw new BizException("请指定乐团!");
        }
        checkNewCourseSchedules(courseSchedules, false,false);
        courseScheduleDao.deleteCourseSchedulesByMusicGroupID(musicGroupID, GroupType.MUSIC.getCode());
		courseScheduleModifyLogService.pushSchoolCourseChange(courseSchedules.get(0));
    }

    @Override
    public List<Date> getCourseScheduleDateByMonth(Long musicGroupID, Date month) {
        if (Objects.isNull(month) || Objects.isNull(musicGroupID)) {
            throw new BizException("请选择日期并指定乐团!");
        }
		String firstDayOfMonth = DateUtil.format(DateUtil.getFirstDayOfMonth(month),DateUtil.DEFAULT_PATTERN);
		String lastDayOfMonth = DateUtil.format(DateUtil.getLastDayOfMonth(month),DateUtil.DEFAULT_PATTERN);
		return courseScheduleDao.getCourseScheduleDateByMonth(musicGroupID, firstDayOfMonth,lastDayOfMonth);
    }

    @Override
    public List<Date> getCourseScheduleDateByStudent(Date month, Integer isAttend) {
        return courseScheduleDao.getStudentCourseScheduleDate(sysUserService.getUserId().longValue(), month, isAttend);
    }

    @Override
    public List<Date> getCourseScheduleDateByTeacher(Date month, String type) {
        return courseScheduleDao.getTeacherCourseScheduleDate(sysUserService.getUserId().longValue(), month, type);
    }

    @Override
    public List<Date> getHistoryCourseScheduleDateByTeacher(Integer userId, Date month) {
        return courseScheduleDao.getTeacherHistoryCourseScheduleDate(userId, month);
    }

    @Override
    public List<Date> getCourseScheduleDates(Date month, String organIdList, String type, Integer transactionTeacherId) {
		List<Integer> organIds = new ArrayList<>();
		if(StringUtils.isNotBlank(organIdList)){
			organIds = Arrays.stream(organIdList.split(",")).map(s->Integer.valueOf(s)).collect(Collectors.toList());
		}
		return courseScheduleDao.getCourseScheduleDateWithOrgan(organIds, month, type, transactionTeacherId, TenantContextHolder.getTenantId());
    }

    @Override
    public Map getStudentCourseSchedulesWithDate(Date classDate) {
        if (Objects.isNull(classDate)) {
            throw new BizException("请选择上课日期!");
        }
        SysUser user = sysUserService.getUser();
        if (null == user) {
            throw new BizException("请登录");
        }
        if (null == user.getTenantId()) {
            throw new BizException("用户机构信息异常");
        }
        Date now = new Date();
//		String configValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.ENABLE_STUDENT_ATTENDANCE_TIME_RANGE_VIP, user.getTenantId());
		String courseBeforeBufferTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_BEFORE_BUFFER_TIME, user.getTenantId());
		String courseAfterBufferTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_AFTER_BUFFER_TIME, user.getTenantId());
        Map<String, Object> result = new HashMap<>(7);
        result.put("vipAppealDaysRange", sysConfigDao.findConfigValue(SysConfigService.VIP_APPEAL_DAYS_RANGE));
		String tenantConfigValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.ADVANCE_LEAVE_HOURS, user.getTenantId());
		Integer advanceLeaveHours = 0;
		if(StringUtils.isNotEmpty(tenantConfigValue)){
			advanceLeaveHours = Integer.parseInt(tenantConfigValue);
		}
		result.put("advanceLeaveHours", advanceLeaveHours);
        result.put("earliestTimeForLeave", Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.EARLIEST_TIME_FOR_LEAVE)));
		//网络教室可以提前进入时间：对内和对外开课时间 (对内：joinRoomBeforeTime 对外：courseBeforeBufferTime)
        result.put("joinRoomBeforeTime",StringUtils.isEmpty(courseBeforeBufferTime)?10:Integer.parseInt(courseBeforeBufferTime));
		result.put("courseBeforeBufferTime", StringUtils.isEmpty(courseBeforeBufferTime)?10:Integer.parseInt(courseBeforeBufferTime));
		result.put("courseAfterBufferTime", StringUtils.isEmpty(courseAfterBufferTime)?10:Integer.parseInt(courseAfterBufferTime));
//		List<Integer> quitClassGroupIds=classGroupStudentMapperDao.findClassGroupIdByUserAndCourseDateAndStatus(user.getId(),classDate, ClassGroupStudentStatusEnum.QUIT.getCode());
        List<CourseScheduleDto> studentCourseSchedulesWithDate = courseScheduleDao.getStudentCourseSchedulesWithDate(classDate, user.getId().longValue(), null);
        List<Long> courseScheduleIds = studentCourseSchedulesWithDate.stream()
                .filter(cs -> cs.getType().equals(CourseSchedule.CourseScheduleType.VIP)
						|| cs.getType().equals(CourseSchedule.CourseScheduleType.LIVE)
						|| cs.getType().equals(CourseSchedule.CourseScheduleType.DEMO)
						|| cs.getType().equals(CourseScheduleType.PRACTICE)
						|| cs.getType().equals(CourseScheduleType.COMM))
                .map(CourseScheduleDto::getId).collect(Collectors.toList());

        courseDataFormat(user, now, studentCourseSchedulesWithDate, courseScheduleIds);
        result.put("list", studentCourseSchedulesWithDate);
        return result;
    }

    private void courseDataFormat(SysUser user, Date now, List<CourseScheduleDto> studentCourseSchedulesWithDate, List<Long> courseScheduleIds) {
        // 直播课设置直播间信息
        Map<String, ImLiveBroadcastRoomDto> liveRoomMap = getLiveRoomMap(studentCourseSchedulesWithDate);

        // 直播间记录回放记录

        // 直播间房间号集合
        List<String> roomIds = studentCourseSchedulesWithDate.stream()
                .filter(cs -> cs.getType().equals(CourseSchedule.CourseScheduleType.LIVE))
                .map(CourseScheduleDto::getLiveRoomId).collect(Collectors.toList());

        Map<String, List<ImLiveRoomVideoVo>> imLiveRoomVideoVoMap = getLiveRecord(roomIds);

        for (CourseScheduleDto courseScheduleDto : studentCourseSchedulesWithDate) {
            if (StringUtils.isNotEmpty(courseScheduleDto.getLiveRoomId())) {
                List<ImLiveRoomVideoVo> imLiveRoomVideoVoList = imLiveRoomVideoVoMap.get(courseScheduleDto.getLiveRoomId());
                if (CollectionUtils.isNotEmpty(imLiveRoomVideoVoList)) {
                    courseScheduleDto.setHasVideoRecord(true);
                    continue;
                }
                courseScheduleDto.setHasVideoRecord(false);
            }
        }
        Map<Integer, String> subjectNameCourseMap = new HashMap<>();
        Map<Integer, String> subjectIdCourseMap = new HashMap<>();
        Map<Integer, Integer> reviewIdMap = new HashMap<>();
        Map<Long, Long> StudentReviewIdMap = new HashMap<>();

        if (!CollectionUtils.isEmpty(courseScheduleIds)) {
            List<Map<Integer, String>> subjectNameCourseMaps = subjectDao.findSubjectNameCourseMaps(courseScheduleIds);
            List<Map<Integer, String>> subjectIdCourseMaps = subjectDao.findClassGroupSubjectId(courseScheduleIds);
            subjectNameCourseMap = MapUtil.convertMybatisMap(subjectNameCourseMaps);
            subjectIdCourseMap = MapUtil.convertMybatisMap(subjectIdCourseMaps);
			List<CourseScheduleReview> reviews = courseScheduleReviewDao.findByCourseSchedulesAndStudentId(courseScheduleIds, user.getId());
			reviewIdMap = reviews.stream().collect(Collectors.toMap(CourseScheduleReview::getCourseScheduleId, CourseScheduleReview::getId));
			List<CourseScheduleComplaints> studentReviews = courseScheduleComplaintsDao.findByCourseIdAndUserId(courseScheduleIds, user.getId());
			StudentReviewIdMap = studentReviews.stream().collect(Collectors.toMap(CourseScheduleComplaints::getCourseScheduleId, CourseScheduleComplaints::getId));
		}

        Set<Long> practiceGroupIds = studentCourseSchedulesWithDate.stream().filter(cs -> CourseScheduleType.PRACTICE.equals(cs.getType())&&Objects.nonNull(cs.getMusicGroupId())).map(cs->Long.valueOf(cs.getMusicGroupId())).collect(Collectors.toSet());

        Map<Long, PracticeGroupType> practiceGroupTypeMap = new HashMap<>();
        if(!CollectionUtils.isEmpty(practiceGroupIds)){
			List<PracticeCourseDto> practiceGroups = practiceGroupDao.getPracticeGroupByIds(new ArrayList<>(practiceGroupIds));
			practiceGroupTypeMap = practiceGroups.stream().collect(Collectors.toMap(PracticeCourseDto::getId, PracticeCourseDto::getType));
		}

        for (CourseScheduleDto courseScheduleDto : studentCourseSchedulesWithDate) {
//            if (now.before(courseScheduleDto.getStartClassTime())) {
//                courseScheduleDto.setStatus(CourseStatusEnum.NOT_START);
//            } else if (now.after(courseScheduleDto.getEndClassTime())) {
//                courseScheduleDto.setStatus(CourseStatusEnum.OVER);
//            } else {
//                courseScheduleDto.setStatus(CourseStatusEnum.UNDERWAY);
//            }
            courseScheduleDto.setSubjectName(MapUtils.getString(subjectNameCourseMap, courseScheduleDto.getId()));
            courseScheduleDto.setSubjectId(MapUtils.getString(subjectIdCourseMap, courseScheduleDto.getId()));
			courseScheduleDto.setReviewId(reviewIdMap.get(courseScheduleDto.getId().intValue()));
			courseScheduleDto.setStudentReviewId(StudentReviewIdMap.get(courseScheduleDto.getId()));
			if(CourseScheduleType.PRACTICE.equals(courseScheduleDto.getType())&&Objects.nonNull(courseScheduleDto.getMusicGroupId())){
				courseScheduleDto.setPracticeType(practiceGroupTypeMap.get(Long.valueOf(courseScheduleDto.getMusicGroupId())));
			}

			// 设置直播间配置
			courseScheduleDto.setLiveBroadcastRoom(liveRoomMap.get(courseScheduleDto.getMusicGroupId()));
			if (Objects.nonNull(courseScheduleDto.getLiveBroadcastRoom())) {
				courseScheduleDto.setOs(courseScheduleDto.getLiveBroadcastRoom().getOs());
			}
        }

        // 直播课判断时间，提前30分钟将课程改为进行中

        Integer studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());
        for (CourseScheduleDto record : studentCourseSchedulesWithDate) {
            if (CourseScheduleType.LIVE.equals(record.getType()) && CourseStatusEnum.NOT_START.equals(record.getStatus())) {
                Date date = DateUtil.addMinutes(new Date(), studentRemindTime);
                if (date.after(record.getStartClassTime())) {
                    record.setStatus(CourseStatusEnum.UNDERWAY);
                }
            }
        }
    }

    private Map<String, List<ImLiveRoomVideoVo>> getLiveRecord(List<String> roomIds) {

        if (CollectionUtils.isEmpty(roomIds)) {
            return new HashMap<>();
        }
        List<ImLiveRoomVideoVo> imLiveRoomVideoVos = imLiveRoomVideoDao.queryByRoomIds(roomIds);
        if (CollectionUtils.isEmpty(imLiveRoomVideoVos)) {
            return new HashMap<>();
        }
        return imLiveRoomVideoVos.stream().collect(Collectors.groupingBy(ImLiveRoomVideoVo::getRoomUid));

    }

    /**
	 * 直播课设置直播间信息
	 * @param studentCourseSchedulesWithDate List<CourseScheduleDto>
	 * @return Map<String, ImLiveBroadcastRoomDto>
	 */
	private Map<String, ImLiveBroadcastRoomDto> getLiveRoomMap(List<CourseScheduleDto> studentCourseSchedulesWithDate) {

		Map<String, ImLiveBroadcastRoomDto> liveRoomMap = Maps.newHashMap();
		{
			Map<Long, String> collect = studentCourseSchedulesWithDate.stream()
					.filter(x -> StringUtils.isNotEmpty(x.getMusicGroupId()))
					.filter(x -> x.getType().equals(CourseScheduleType.LIVE))
					.collect(Collectors.toMap(CourseSchedule::getId, CourseSchedule::getMusicGroupId, (o, n) -> n));

			if (MapUtils.isNotEmpty(collect)) {

				List<Long> liveGroupIds = collect.values().stream().map(Long::parseLong).distinct().collect(Collectors.toList());

				liveRoomMap = vipGroupDao.getVipGroupByIds(liveGroupIds).stream()
						.filter(x -> Objects.nonNull(x.getLiveConfigJson()))
						.collect(Collectors.toMap(x -> String.valueOf(x.getId()),
								x -> JSON.parseObject(x.getLiveConfigJson(), ImLiveBroadcastRoomDto.class), (o, n) -> n));

			}

		}
		return liveRoomMap;
	}

	@Override
    public PageInfo getCourseSchedulesWithDateByEdu(CourseScheduleQueryInfo queryInfo) {
//        queryInfo.setOrganId(Long.parseLong(user.getOrganId()));
        PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

        if (Objects.isNull(queryInfo.getDate())) {
            queryInfo.setDate(new Date());
        }

		List<Integer> organIds = new ArrayList<>();
		if(StringUtils.isNotBlank(queryInfo.getOrganIdList())){
			organIds = Arrays.stream(queryInfo.getOrganIdList().split(",")).map(s->Integer.valueOf(s)).collect(Collectors.toList());
		}

        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
		params.put("organIds", organIds);

        List<CourseScheduleDto> dataList = null;
        int count = courseScheduleDao.countCourseSchedulesWithDate(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = courseScheduleDao.getCourseSchedulesWithDate(params);
            List<Long> allCourseScheduleIds = dataList.stream().map(CourseScheduleDto::getId).collect(Collectors.toList());
            List<Map<Long, Long>> courseStudentNumMaps = courseScheduleStudentPaymentDao.countCourseStudentNum(allCourseScheduleIds);
            Map<Long, Long> courseStudentNumMap = MapUtil.convertIntegerMap(courseStudentNumMaps);

			List<StudentAttendance> studentAttendances = studentAttendanceDao.getWithCoursesAndExcludeQuitStudent(allCourseScheduleIds);
			Map<Long, List<StudentAttendance>> courseStudentAttendanceMap = studentAttendances.stream().collect(Collectors.groupingBy(StudentAttendance::getCourseScheduleId));

			List<Integer> teacherIds=new ArrayList<>();
            for (CourseScheduleDto courseScheduleEndDto : dataList) {
                teacherIds.add(courseScheduleEndDto.getActualTeacherId());
            }

			List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedules(allCourseScheduleIds);
            Map<Long, Set<Integer>> courseTeachingTeacherIdsMap = new HashMap<>();
            if(!CollectionUtils.isEmpty(courseScheduleTeacherSalaries)){
				courseTeachingTeacherIdsMap = courseScheduleTeacherSalaries.stream().filter(c->TeachTypeEnum.TEACHING.equals(c.getTeacherRole())).collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId, Collectors.mapping(CourseScheduleTeacherSalary::getUserId, Collectors.toSet())));
			}

			List<Integer> classGroupIds = dataList.stream().map(CourseScheduleDto::getClassGroupId).collect(Collectors.toList());
			List<Group> groupsT = groupDao.findByClassGroupIds(classGroupIds);
            Map<Integer, Group> idGroupMap = groupsT.stream().collect(Collectors.toMap(Group::getClassGroupId, group -> group));

            for (int i=0;i<dataList.size();i++) {
                Long studentNum=courseStudentNumMap.get(dataList.get(i).getId());
                dataList.get(i).setTotalStudentNum(Objects.isNull(studentNum)?0:studentNum.intValue());
                Group group = idGroupMap.get(dataList.get(i).getClassGroupId());
                if(Objects.nonNull(group)){
                    dataList.get(i).setGroupName(group.getGroupName());
                }
				dataList.get(i).setLeaveStudentNum(0);
				dataList.get(i).setStudentNum(0);
                if(courseStudentAttendanceMap.containsKey(dataList.get(i).getId())){
					List<StudentAttendance> courseStudentAttendances = courseStudentAttendanceMap.get(dataList.get(i).getId());
					dataList.get(i).setLeaveStudentNum((int) courseStudentAttendances.stream().filter(s->StudentAttendanceStatusEnum.LEAVE.equals(s.getStatus())).count());
					dataList.get(i).setStudentNum((int) courseStudentAttendances.stream().filter(s->StudentAttendanceStatusEnum.NORMAL.equals(s.getStatus())).count());
				}
                if(courseTeachingTeacherIdsMap.containsKey(dataList.get(i).getId())){
                	dataList.get(i).setTeachingTeacherIds(StringUtils.join(courseTeachingTeacherIdsMap.get(dataList.get(i).getId()), ","));
				}
            }
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }

        // 直播课判断时间，提前30分钟将课程改为进行中

        Integer studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());
        for (CourseScheduleDto record : dataList) {
            if (CourseScheduleType.LIVE.equals(record.getType()) && CourseStatusEnum.NOT_START.equals(record.getStatus())) {
                Date date = DateUtil.addMinutes(new Date(), studentRemindTime);
                if (date.after(record.getStartClassTime())) {
                    record.setStatus(CourseStatusEnum.UNDERWAY);
                }
            }
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public List<CourseScheduleDto> getTeacherCourseSchedulesWithDate(Integer userId, Date classDate, String type,Integer tenantId) {
        if (Objects.isNull(classDate)) {
            throw new BizException("请选择上课日期!");
        }
        List<CourseScheduleDto> teacherCourseSchedulesWithDate = courseScheduleDao.getTeacherCourseSchedulesWithDate(classDate, userId.longValue(), type);
        List<Long> allCourseScheduleIds = teacherCourseSchedulesWithDate.stream().map(CourseScheduleDto::getId).collect(Collectors.toList());
        List<Long> courseScheduleIds = teacherCourseSchedulesWithDate.stream()
                .filter(cs -> cs.getType().equals(CourseSchedule.CourseScheduleType.VIP) ||
						cs.getType().equals(CourseSchedule.CourseScheduleType.LIVE) ||
                        cs.getType().equals(CourseSchedule.CourseScheduleType.DEMO) ||
                        cs.getType().equals(CourseSchedule.CourseScheduleType.PRACTICE) ||
						cs.getType().equals(CourseScheduleType.COMM))
                .map(CourseScheduleDto::getId).collect(Collectors.toList());

        return getTeacherCourseFormat(userId, classDate, tenantId,
                teacherCourseSchedulesWithDate, allCourseScheduleIds, courseScheduleIds);
    }

	@Override
	public CourseScheduleDto getRecentCourseSchedule(Integer userId,Long courseId) {
		CourseScheduleDto courseSchedule = courseScheduleDao.getRecentCourseSchedule(userId,courseId);
		if(Objects.nonNull(courseSchedule)){
			if (Objects.nonNull(courseSchedule.getClassGroupId())) {
				List<StudentNameAndPhoneDto> courseStudentNameAndPhone = classGroupStudentMapperDao.findCourseStudentNameAndPhone(courseSchedule.getId().intValue());
				if(!CollectionUtils.isEmpty(courseStudentNameAndPhone)){
					List<String> studentNames = courseStudentNameAndPhone.stream().filter(s->Objects.nonNull(s)&&Objects.nonNull(s.getUserName())).map(StudentNameAndPhoneDto::getUserName).collect(Collectors.toList());
					courseSchedule.setStudentNames(StringUtils.join(studentNames, ","));
//					courseSchedule.setStudents(courseStudentNameAndPhone);
				}
			}
			Map<Integer, Long> studentNumCourseMap = MapUtil.convertIntegerMap(studentAttendanceDao.countStudentAttendancesByCourses(Collections.singletonList(courseSchedule.getId())));
			Long studentNum = studentNumCourseMap.get(courseSchedule.getId());
			if (Objects.nonNull(studentNum)) {
				courseSchedule.setIsCallNames(studentNum > 0 ? YesOrNoEnum.YES : YesOrNoEnum.NO);
			} else {
				courseSchedule.setIsCallNames(YesOrNoEnum.NO);
			}
		}
		return courseSchedule;
	}

	// 数据整理
    @Nullable
    private List<CourseScheduleDto> getTeacherCourseFormat(Integer userId, Date classDate, Integer tenantId,
                                                           List<CourseScheduleDto> teacherCourseSchedulesWithDate, List<Long> allCourseScheduleIds,
                                                           List<Long> courseScheduleIds) {
        // 直播课设置直播间信息
        Map<String, ImLiveBroadcastRoomDto> liveRoomMap = getLiveRoomMap(teacherCourseSchedulesWithDate);

        Map<Integer, String> subjectNameCourseMap = new HashMap<>();
        Map<Integer, String> subjectIdCourseMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(courseScheduleIds)) {
            List<Map<Integer, String>> subjectNameCourseMaps = subjectDao.findSubjectNameCourseMaps(courseScheduleIds);
            List<Map<Integer, String>> subjectIdCourseMaps = subjectDao.findClassGroupSubjectId(courseScheduleIds);
            subjectNameCourseMap = MapUtil.convertMybatisMap(subjectNameCourseMaps);
            subjectIdCourseMap = MapUtil.convertMybatisMap(subjectIdCourseMaps);
        }
        List<Map<Integer, Integer>> studentNumCourseMaps = null;
        if (!CollectionUtils.isEmpty(allCourseScheduleIds)) {
            studentNumCourseMaps = studentAttendanceDao.countStudentAttendancesByCourses(allCourseScheduleIds);
        }
        Map<Integer, Long> studentNumCourseMap = new HashMap<>();
        if (Objects.nonNull(studentNumCourseMaps)) {
            studentNumCourseMap = MapUtil.convertIntegerMap(studentNumCourseMaps);
        }

        if(CollectionUtils.isEmpty(teacherCourseSchedulesWithDate)){
            return teacherCourseSchedulesWithDate;
        }

        List<Integer> classGroupIds = teacherCourseSchedulesWithDate.stream().map(CourseScheduleDto::getClassGroupId).collect(Collectors.toList());
        List<Map<Integer, Integer>> classGroupStudentNumMaps = classGroupStudentMapperDao.countClassGroupsStudentNum(classGroupIds, ClassGroupStudentStatusEnum.NORMAL);
        Map<Integer, Long> classGroupStudentNumMap = MapUtil.convertIntegerMap(classGroupStudentNumMaps);

        String tenantConfigValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.ADVANCE_LEAVE_HOURS, tenantId);
        Integer advanceLeaveHours = 0;
        if(StringUtils.isNotEmpty(tenantConfigValue)){
            advanceLeaveHours = Integer.parseInt(tenantConfigValue);
        }
        List<Map<Long, Integer>> courseLeaveStudentNumMaps = studentAttendanceDao.countCourseLeaveStudentNumWithFourHoursAgo(allCourseScheduleIds, advanceLeaveHours);
        Map<Long,Long> courseLeaveStudentNumMap = MapUtil.convertIntegerMap((courseLeaveStudentNumMaps));

        List<CourseScheduleReview> AllCourseScheduleReviews = courseScheduleReviewDao.findByCourseSchedules(allCourseScheduleIds);
        Map<Integer, List<CourseScheduleReview>> idCourseReview=new HashMap<>();
        if(!CollectionUtils.isEmpty(AllCourseScheduleReviews)){
            idCourseReview=AllCourseScheduleReviews.stream().collect(Collectors.groupingBy(CourseScheduleReview::getCourseScheduleId));
        }

        Set<Long> homeworkCourseIds = studentServeService.getTeacherHomeworkCourseIdsWithMonday(userId, classDate);

		for (CourseScheduleDto courseScheduleDto : teacherCourseSchedulesWithDate) {
            Long studentNum = studentNumCourseMap.get(courseScheduleDto.getId());
            if (Objects.nonNull(studentNum)) {
                courseScheduleDto.setStudentAttendanceIsFirstTime(studentNum > 0 ? 0 : 1);
            } else {
                courseScheduleDto.setStudentAttendanceIsFirstTime(1);
            }
            if (Objects.nonNull(courseScheduleDto.getClassGroupId())) {
                //                String[] studentNames = classGroupStudentMapperDao.findCourseStudentName(courseScheduleDto.getId().intValue());
                List<StudentNameAndPhoneDto> courseStudentNameAndPhone = classGroupStudentMapperDao.findCourseStudentNameAndPhone(courseScheduleDto.getId().intValue());
                if(!CollectionUtils.isEmpty(courseStudentNameAndPhone)){
                    List<String> studentNames = courseStudentNameAndPhone.stream().filter(s->Objects.nonNull(s)&&Objects.nonNull(s.getUserName())).map(StudentNameAndPhoneDto::getUserName).collect(Collectors.toList());
                    courseScheduleDto.setStudentNames(StringUtils.join(studentNames, ","));
                    courseScheduleDto.setStudents(courseStudentNameAndPhone);
                }
            }
            courseScheduleDto.setSubjectName(subjectNameCourseMap.get(courseScheduleDto.getId()));
            courseScheduleDto.setSubjectId(subjectIdCourseMap.get(courseScheduleDto.getId()));
            Long leaveStudentNum = courseLeaveStudentNumMap.get(courseScheduleDto.getId());
            Long normalStudentNum = classGroupStudentNumMap.get(courseScheduleDto.getClassGroupId());
            if(Objects.nonNull(leaveStudentNum)&&Objects.nonNull(normalStudentNum)&&leaveStudentNum.intValue()==normalStudentNum.intValue()){
                courseScheduleDto.setEnableAdjustInToday(1);
            }
            List<CourseScheduleReview> courseScheduleReviews = idCourseReview.get(courseScheduleDto.getId().intValue());
            if(!CollectionUtils.isEmpty(courseScheduleReviews)){
                courseScheduleDto.setComplaintsStatus(1);
                CourseScheduleReview courseScheduleReview = courseScheduleReviews.get(0);
                courseScheduleDto.setReviewId(courseScheduleReview.getId());
            }else{
                courseScheduleDto.setComplaintsStatus(0);
            }

            if(homeworkCourseIds.contains(courseScheduleDto.getId())){
                courseScheduleDto.setEnableAssignHomework(1);
            }

            // 直播课直播间配置
            courseScheduleDto.setLiveBroadcastRoom(liveRoomMap.get(courseScheduleDto.getMusicGroupId()));
            if (Objects.nonNull(courseScheduleDto.getLiveBroadcastRoom())) {
                courseScheduleDto.setOs(courseScheduleDto.getLiveBroadcastRoom().getOs());
            }
        }

        // roomId
        List<String> roomIdList = teacherCourseSchedulesWithDate.stream().map(CourseScheduleDto::getLiveRoomId).collect(Collectors.toList());


        Map<String, List<ImLiveRoomVideoVo>> imLiveRoomVideoVoMap = getLiveRecord(roomIdList);

        for (CourseScheduleDto courseScheduleDto : teacherCourseSchedulesWithDate) {
            if (StringUtils.isNotEmpty(courseScheduleDto.getLiveRoomId())) {
                List<ImLiveRoomVideoVo> imLiveRoomVideoVoList = imLiveRoomVideoVoMap.get(courseScheduleDto.getLiveRoomId());
                if (CollectionUtils.isNotEmpty(imLiveRoomVideoVoList)) {
                    courseScheduleDto.setHasVideoRecord(true);
                    continue;
                }
                courseScheduleDto.setHasVideoRecord(false);
            }
        }

        // 查询直播间视频数
        Map<String, Integer> uidCountMap = imLiveRoomVideoService.queryRoomUidCountMap(roomIdList);
        for (CourseScheduleDto record : teacherCourseSchedulesWithDate) {
            if (Objects.isNull(record.getLiveRoomId())) {
                continue;
            }
            Integer count = uidCountMap.get(record.getLiveRoomId());
            if (count == null) {
                record.setVideoNum(0);
            }
            record.setVideoNum(count);
        }
        // 直播课判断时间，提前30分钟将课程改为进行中
        Integer studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());
        for (CourseScheduleDto record : teacherCourseSchedulesWithDate) {
            if (CourseScheduleType.LIVE.equals(record.getType()) && CourseStatusEnum.NOT_START.equals(record.getStatus())) {
                Date date = DateUtil.addMinutes(new Date(), studentRemindTime);
                if (date.after(record.getStartClassTime())) {
                    record.setStatus(CourseStatusEnum.UNDERWAY);
                }
            }
        }
        return teacherCourseSchedulesWithDate;
    }

    @Override
    public List<CourseScheduleDto> getTeacherHistoryCourseSchedulesWithDate(Integer userId, Date classDate) {
        if (Objects.isNull(classDate)) {
            throw new BizException("请选择上课日期!");
        }
        List<CourseScheduleDto> teacherCourseSchedulesWithDate = courseScheduleDao.getTeacherHistoryCourseSchedulesWithDate(classDate, userId.longValue());
        List<Long> courseScheduleIds = teacherCourseSchedulesWithDate.stream()
                .filter(cs -> cs.getType().equals(CourseSchedule.CourseScheduleType.VIP) || cs.getType().equals(CourseSchedule.CourseScheduleType.DEMO))
                .map(CourseScheduleDto::getId).collect(Collectors.toList());

        Map<Integer, String> subjectNameCourseMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(courseScheduleIds)) {
            List<Map<Integer, String>> subjectNameCourseMaps = subjectDao.findSubjectNameCourseMaps(courseScheduleIds);
            subjectNameCourseMap = MapUtil.convertMybatisMap(subjectNameCourseMaps);
        }
        for (CourseScheduleDto courseScheduleDto : teacherCourseSchedulesWithDate) {
            if (Objects.nonNull(courseScheduleDto.getClassGroupId())) {
                String[] studentNums = classGroupStudentMapperDao.findCourseStudentName(courseScheduleDto.getId().intValue());
                courseScheduleDto.setStudentNames(org.apache.commons.lang3.StringUtils.join(studentNums, ","));
            }
            courseScheduleDto.setSubjectName(subjectNameCourseMap.get(courseScheduleDto.getId()));
        }
        return teacherCourseSchedulesWithDate;
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void courseAdjustOnlyWithClassDate(List<CourseSchedule> courseSchedules, Integer operatorId) {
    	if(CollectionUtils.isEmpty(courseSchedules)){
    		throw new BizException("请指定课程");
		}
		businessLogger.info("courseAdjustOnlyWithClassDate: {}",JSON.toJSONString(courseSchedules));
		Map<Long, CourseSchedule> idCourseMap = courseSchedules.stream().collect(Collectors.toMap(CourseSchedule::getId, c -> c));
		List<Long> courseIds = courseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());
		List<CourseSchedule> existCourses = courseScheduleDao.findByCourseScheduleIds(courseIds);
		Date now = new Date();
		//获取操作日志
		List<CourseScheduleModifyLog> modifyLogs = getModifyLogs(courseIds,existCourses,operatorId,now);

		for (CourseSchedule existCours : existCourses) {
			if(now.compareTo(existCours.getStartClassTime())>0){
				throw new BizException("进行中或已结束课程不可调整");
			}
			CourseSchedule ac = idCourseMap.get(existCours.getId());

			if(now.compareTo(ac.getStartClassTime())>0){
				throw new BizException("课程调整时间不得小于当前时间");
			}

			int singleClassMinutes = DateUtil.minutesBetween(existCours.getStartClassTime(), existCours.getEndClassTime());

			existCours.setStatus(CourseStatusEnum.NOT_START);
			existCours.setClassDate(ac.getClassDate());
			existCours.setUpdateTime(now);
			existCours.setStartClassTime(ac.getStartClassTime());
			existCours.setEndClassTime(DateUtil.addMinutes(existCours.getStartClassTime(), singleClassMinutes));

            //课程结束时间必须在21：00之前
            if(DateUtil.getHour(existCours.getEndClassTime()) > 20 && DateUtil.getMinute(existCours.getEndClassTime()) > 0){
				throw new BizException("课程结束时间不能超过21:00，请重新调整上课时间");
            }

			if(!DateUtil.isSameDay(existCours.getStartClassTime(), existCours.getEndClassTime())){
				throw new BizException("课程暂不支持跨天");
			}
		}

		checkNewCourseSchedules(existCourses, false,false);
		courseScheduleDao.batchUpdate(existCourses);

		studentAttendanceDao.deleteByCourseSchedules(courseIds);
		//删除作业
		courseHomeworkService.delHomwworkByCourseScheduleId(courseIds);
		//删除评论
		courseScheduleReviewDao.delByCourseScheduleIds(courseIds);
		//删除申述
		courseScheduleComplaintsDao.delByCourseScheduleIds(courseIds);
		//清除考勤申述
		teacherAttendanceDao.clearAttendanceComplaints(courseIds);
		//推送学校端日志
		courseScheduleModifyLogService.pushSchoolCourseChange(existCourses.get(0));
		//保存日志调整信息
		saveModifyLog(modifyLogs,courseIds);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public Map<String, List<CourseSchedule>> courseAdjustOnlyWithClassDateCheck(VipGroupCourseAdjustInfoDto courseAdjustInfo) {
		if(StringUtils.isBlank(courseAdjustInfo.getCourseScheduleIds())){
			throw new BizException("请指定课程");
		}
		if(Objects.isNull(courseAdjustInfo.getCourseCreateStartTime())){
			throw new BizException("请指定课程调整起始时间");
		}
		if(Objects.isNull(courseAdjustInfo.getCourseTimes())){
			throw new BizException("请指定课程调整周期");
		}
		Date now=new Date();
		//所有课程编号
		List<Long> adjustCourseScheduleIds = Arrays.asList(courseAdjustInfo.getCourseScheduleIds().split(","))
				.stream().map(Long::parseLong).collect(Collectors.toList());
		//所有的课程
		List<CourseSchedule> courseSchedules = courseScheduleDao.findByCourseScheduleIds(adjustCourseScheduleIds);
		courseSchedules.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
		if(adjustCourseScheduleIds.size()!=courseSchedules.size()){
			throw new BizException("部分课程不存在");
		}

		Map<Long, CourseSchedule> idCourseMap = courseSchedules.stream().collect(Collectors.toMap(CourseSchedule::getId, c -> {
			try {
				return (CourseSchedule)org.apache.commons.beanutils.BeanUtils.cloneBean(c);
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
			return null;
		}));

		Calendar calendar = Calendar.getInstance();
		calendar.setTime(courseAdjustInfo.getCourseCreateStartTime());
		List<Date> courseStartDates = new ArrayList<>();

		Set<String> holidayDays = new HashSet<>();
		Integer tenantId = TenantContextHolder.getTenantId();
		if (courseAdjustInfo.isHoliday()) {
			String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING,tenantId);
			if(StringUtils.isNotBlank(holidaySetting)){
				holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
			}
		}

		Map<String, List<CourseSchedule>> result = new HashMap<String, List<CourseSchedule>>(){{
			put("normal", new ArrayList<>());
			put("revise", new ArrayList<>());
			put("failed", new ArrayList<>());
		}};

		while (true) {
			if (courseAdjustInfo.isHoliday() && holidayDays.contains(DateUtil.format(calendar.getTime(), "yyyy-MM-dd"))) {
				calendar.add(Calendar.DATE, 1);
				continue;
			}
			int i = calendar.get(Calendar.DAY_OF_WEEK);
			if (i == 1) {
				i = 7;
			} else {
				i = i - 1;
			}
			for (CourseTimeDto courseTime : courseAdjustInfo.getCourseTimes()) {
				if (Objects.isNull(courseTime.getDayOfWeek())||StringUtils.isBlank(courseTime.getStartClassTime())) {
					throw new BizException("排课循环周期错误");
				}
				if (courseTime.getDayOfWeek() < 1 || courseTime.getDayOfWeek() > 7) {
					throw new BizException("排课循环周期错误");
				}
				if (courseTime.getDayOfWeek() == i) {
					String dateYmdStr = DateUtil.dateToString(calendar.getTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
					dateYmdStr = dateYmdStr + " " + courseTime.getStartClassTime();
					Date courseStartTime = DateUtil.stringToDate(dateYmdStr, "yyyy-MM-dd HH:mm");

					if(courseStartTime.compareTo(now)<=0){
						throw new BizException("课程调整时间不得小于当前时间");
					}

					courseStartDates.add(DateUtil.stringToDate(dateYmdStr, DateUtil.EXPANDED_DATE_TIME_FORMAT));
					CourseSchedule currentCourseSchedule = courseSchedules.get(courseStartDates.size() - 1);
					int singleClassMinutes = DateUtil.minutesBetween(currentCourseSchedule.getStartClassTime(), currentCourseSchedule.getEndClassTime());
					Date courseEndTime = DateUtil.addMinutes(courseStartTime, singleClassMinutes);
					//校验课程是否结算
					this.checkCourseIsSettlement(currentCourseSchedule);
					int num = studentAttendanceDao.countStudentAttendenceNum(currentCourseSchedule.getId().intValue());
					if (num > 0) {
						throw new BizException("{}[{}]{}-{}课程已点名",
								currentCourseSchedule.getName(),
								currentCourseSchedule.getId(),
								DateUtil.dateToString(currentCourseSchedule.getStartClassTime(),
										DateUtil.EXPANDED_DATE_TIME_FORMAT),
								DateUtil.dateToString(currentCourseSchedule.getEndClassTime(),
										DateUtil.EXPANDED_DATE_TIME_FORMAT));
					}
					currentCourseSchedule.setStatus(CourseStatusEnum.NOT_START);
					currentCourseSchedule.setClassDate(courseStartTime);
					currentCourseSchedule.setStartClassTime(courseStartTime);
					currentCourseSchedule.setEndClassTime(courseEndTime);
				}
				if (courseStartDates.size() == adjustCourseScheduleIds.size()) {
					break;
				}
			}
			if (courseStartDates.size() == adjustCourseScheduleIds.size()) {
				break;
			}
			calendar.add(Calendar.DATE, 1);
		}
		for (CourseSchedule courseSchedule : courseSchedules) {
			boolean b = checkSingleCourseConflict(courseSchedule, courseSchedules, result);
			if(!b){
				result.get("failed").add(idCourseMap.get(courseSchedule.getId()));
			}
		}
		return result;
	}

	@Override
	public void checkCourseIsSettlement(CourseSchedule courseSchedule) {
		int settlementNum = courseScheduleTeacherSalaryDao.checkCourseIsSettlement(courseSchedule.getId().intValue());
		if(settlementNum <= 0){
			settlementNum = courseScheduleDao.checkCourseIsSettlementReport(courseSchedule.getId());
		}
		if (settlementNum > 0) {
			throw new BizException("{}[{}]{}-{}课程已结算",
					courseSchedule.getName(),
					courseSchedule.getId(),
					DateUtil.dateToString(courseSchedule.getStartClassTime(),
							DateUtil.ISO_EXPANDED_DATE_TIME_FORMAT),
					DateUtil.dateToString(courseSchedule.getEndClassTime(),
							DateUtil.ISO_EXPANDED_DATE_TIME_FORMAT));
		}
	}

	@Override
	public boolean courseTimeRevise(CourseSchedule courseSchedule, List<CourseSchedule> sameDayCourses) {
		int singleClassMinutes = DateUtil.minutesBetween(courseSchedule.getStartClassTime(), courseSchedule.getEndClassTime());
		sameDayCourses.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
		Date lastEndClassTime = courseSchedule.getStartClassTime();

		for (int i=0;i<sameDayCourses.size();i++){
			CourseSchedule cs1 = sameDayCourses.get(i);
			if(cs1.getEndClassTime().compareTo(lastEndClassTime)>=0){
				lastEndClassTime = cs1.getEndClassTime();
			}

			if(i!=sameDayCourses.size()-1&&DateUtil.minutesBetween(lastEndClassTime, sameDayCourses.get(i + 1).getStartClassTime())>=singleClassMinutes+10){
				break;
			}
		}

		lastEndClassTime = DateUtil.addMinutes(lastEndClassTime, 10);;

		Date endClassTime = DateUtil.addMinutes(lastEndClassTime, singleClassMinutes);
		if(!DateUtil.isSameDay(lastEndClassTime, endClassTime)){
			return false;
		}
		courseSchedule.setStartClassTime(lastEndClassTime);
		courseSchedule.setEndClassTime(endClassTime);
		return true;
	}

	@Override
	public boolean checkSingleCourseConflict(CourseSchedule courseSchedule, List<CourseSchedule> adjustCourses, Map<String, List<CourseSchedule>> result) {
		if(!DateUtil.isSameDay(courseSchedule.getStartClassTime(), courseSchedule.getEndClassTime())){
			throw new BizException("课程时间不允许跨天");
		}
		List<CourseSchedule> allCourseSchedules = courseScheduleDao.findByClassDate(Arrays.asList(DateUtil.dateToString(courseSchedule.getClassDate(), "yyyy-MM-dd")),TenantContextHolder.getTenantId());

		if(null == allCourseSchedules){
			allCourseSchedules = new ArrayList<>();
		}
		List<CourseSchedule> adjustCourseTemp = adjustCourses.stream().filter(c -> DateUtil.isSameDay(c.getClassDate(), courseSchedule.getClassDate())).collect(Collectors.toList());
		Set<Long> adjustCourseIds = adjustCourseTemp.stream().map(CourseSchedule::getId).collect(Collectors.toSet());
		allCourseSchedules= (List<CourseSchedule>) org.apache.commons.collections.CollectionUtils.union(allCourseSchedules.stream()
				.filter(c->!adjustCourseIds.contains(c.getId())).collect(Collectors.toList()),
				adjustCourseTemp);

		Map<Long, CourseSchedule> idCourseMap = allCourseSchedules.stream().collect(Collectors.toMap(CourseSchedule::getId, c -> c));
		Set<Long> courseScheduleIds = allCourseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toSet());
		courseScheduleIds.add(courseSchedule.getId());
		List<CourseScheduleStudentPayment> courseScheduleStudentPayments = courseScheduleStudentPaymentDao.findByCourseScheduleIds(new ArrayList<>(courseScheduleIds));
		Map<Long, Set<Integer>> courseStudentIdsMap = courseScheduleStudentPayments.stream()
				.collect(Collectors.groupingBy(CourseScheduleStudentPayment::getCourseScheduleId,
						Collectors.mapping(CourseScheduleStudentPayment::getUserId, Collectors.toSet())));

		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedules(new ArrayList<>(courseScheduleIds));
		Map<Long, Set<Integer>> courseTeacherIdsMap = courseScheduleTeacherSalaries.stream()
				.collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId,
						Collectors.mapping(CourseScheduleTeacherSalary::getUserId, Collectors.toSet())));
		Set<Integer> currentCourseTeacherIds = courseTeacherIdsMap.get(courseSchedule.getId());

		List<StudentAttendance> studentAttendances = studentAttendanceDao.findByCourseIds(new ArrayList<>(courseScheduleIds));
		Map<Long, Set<Integer>> courseLeaveStudentIdsMap = new HashMap<>();
		for (StudentAttendance sa : studentAttendances) {
			if(sa.getCourseScheduleId().equals(courseSchedule.getId())&&!StudentAttendanceStatusEnum.LEAVE.equals(sa.getStatus())){
				return false;
			}
			if(!courseLeaveStudentIdsMap.containsKey(sa.getCourseScheduleId())){
				courseLeaveStudentIdsMap.put(sa.getCourseScheduleId(), new HashSet<>());
			}
			CourseSchedule cs = idCourseMap.get(sa.getCourseScheduleId());
			if(DateUtil.addHours(sa.getCreateTime(),4).compareTo(cs.getStartClassTime())<0){
				courseLeaveStudentIdsMap.get(sa.getCourseScheduleId()).add(sa.getUserId());
			}
		}
		Set<Integer> currentCourseLeaveStudentIds = courseLeaveStudentIdsMap.get(courseSchedule.getId());
		Set<Integer> currentCourseStudentIds;
		if(null!=currentCourseLeaveStudentIds){
			currentCourseStudentIds = courseStudentIdsMap.get(courseSchedule.getId())
					.stream().filter(id->Objects.nonNull(currentCourseLeaveStudentIds)&&!currentCourseLeaveStudentIds.contains(id)).collect(Collectors.toSet());
		}else{
			currentCourseStudentIds = courseStudentIdsMap.get(courseSchedule.getId());
		}

		boolean isRepeat = false;
		List<CourseSchedule> repeatCourses = new ArrayList<>();

		allCourseSchedules.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
		for (CourseSchedule cs : allCourseSchedules) {
			if(cs.getId().equals(courseSchedule.getId())){
				continue;
			}

			if(cs.getStartClassTime().compareTo(courseSchedule.getEndClassTime())>0){
				break;
			}

			if(cs.getEndClassTime().compareTo(courseSchedule.getStartClassTime())<0){
				continue;
			}

			//判断两节课程时间是否重叠
			if (!cs.getStartClassTime().before(courseSchedule.getEndClassTime())
					||!cs.getEndClassTime().after(courseSchedule.getStartClassTime())) {
				continue;
			}

			Set<Integer> csLeaveStudentIds = courseLeaveStudentIdsMap.get(cs.getId());
			Set<Integer> repeatStudentIds = new HashSet<>();
			Set<Integer> csStudentIds;
			if(null!=csLeaveStudentIds){
				csStudentIds = courseStudentIdsMap.get(cs.getId()).stream()
						.filter(id->Objects.nonNull(csLeaveStudentIds)&&!csLeaveStudentIds.contains(id)).collect(Collectors.toSet());
			}else{
				csStudentIds = courseStudentIdsMap.get(cs.getId());
			}
			if(null!=csStudentIds){
				repeatStudentIds = csStudentIds.stream().filter(id -> currentCourseStudentIds.contains(id)).collect(Collectors.toSet());
			}

			Set<Integer> csTeacherIds = courseTeacherIdsMap.get(cs.getId());
			Set<Integer> repeatTeacherIds = new HashSet<>();
			if(null!=csTeacherIds){
				repeatTeacherIds = csTeacherIds.stream().filter(id -> Objects.nonNull(currentCourseTeacherIds)&&currentCourseTeacherIds.contains(id)).collect(Collectors.toSet());
			}

			if(!CollectionUtils.isEmpty(repeatStudentIds)||!CollectionUtils.isEmpty(repeatTeacherIds)){
				isRepeat = true;
				break;
			}
		}
		if(isRepeat){
			for (CourseSchedule cs : allCourseSchedules) {
				if(cs.getId().equals(courseSchedule.getId())){
					continue;
				}

				Set<Integer> csLeaveStudentIds = courseLeaveStudentIdsMap.get(cs.getId());
				Set<Integer> repeatStudentIds = new HashSet<>();
				Set<Integer> csStudentIds;
				if(null!=csLeaveStudentIds){
					csStudentIds = courseStudentIdsMap.get(cs.getId()).stream()
							.filter(id->Objects.nonNull(csLeaveStudentIds)&&!csLeaveStudentIds.contains(id)).collect(Collectors.toSet());
				}else{
					csStudentIds = courseStudentIdsMap.get(cs.getId());
				}
				if(null!=csStudentIds){
					repeatStudentIds = csStudentIds.stream().filter(id -> currentCourseStudentIds.contains(id)).collect(Collectors.toSet());
				}

				Set<Integer> csTeacherIds = courseTeacherIdsMap.get(cs.getId());
				Set<Integer> repeatTeacherIds = new HashSet<>();
				if(null!=csTeacherIds){
					repeatTeacherIds = csTeacherIds.stream().filter(id -> Objects.nonNull(currentCourseTeacherIds)&&currentCourseTeacherIds.contains(id)).collect(Collectors.toSet());
				}
				if(!CollectionUtils.isEmpty(repeatStudentIds)||!CollectionUtils.isEmpty(repeatTeacherIds)){
					repeatCourses.add(cs);
				}
			}
			boolean b = courseTimeRevise(courseSchedule, repeatCourses);
			if(b){
				result.get("revise").add(courseSchedule);
			}
			return b;
		}
		result.get("normal").add(courseSchedule);
		return true;
	}

	@Transactional(rollbackFor = Exception.class)
    public HttpResponseResult checkCourseSchedules(List<CourseSchedule> courseSchedules, boolean checkExistCourseSchedule,Boolean postponeFlag,ClassGroupStudentMapper classGroupStudentMapper) {

		if (CollectionUtils.isEmpty(courseSchedules)) {
			return BaseController.failed();
		}
		List<String> classDates = courseSchedules.stream().map(courseSchedule -> DateUtil.dateToString(courseSchedule.getClassDate(), "yyyy-MM-dd"))
				.collect(Collectors.toList());
		List<CourseSchedule> existCourseSchedules;
		if(Objects.nonNull(classGroupStudentMapper)){
			existCourseSchedules = courseScheduleDao.findByClassDateAndStu(classDates, TenantContextHolder.getTenantId(),classGroupStudentMapper.getUserId());
		}else {
			existCourseSchedules = courseScheduleDao.findByClassDate(classDates, TenantContextHolder.getTenantId());
		}

		//只需要调整课程信息的课程编号列表
		List<Long> updateCourseScheduleIds = courseSchedules
				.stream()
				.filter(courseSchedule -> Objects.nonNull(courseSchedule.getId()))
				.map(CourseSchedule::getId)
				.collect(Collectors.toList());

		//排除只需调整的课程
		existCourseSchedules = existCourseSchedules.stream()
				.filter(courseSchedule -> !updateCourseScheduleIds.contains(courseSchedule.getId()))
				.collect(Collectors.toList());
		//新课程对应的班级编号列表
		List<Integer> newCourseScheduleClassGroupIds = courseSchedules
				.stream()
				.map(CourseSchedule::getClassGroupId)
				.distinct()
				.collect(Collectors.toList());

		List<Long> existCourseScheduleIds = existCourseSchedules.stream()
				.map(CourseSchedule::getId)
				.collect(Collectors.toList());

		//合并新课程和已存在的课程
		List<CourseSchedule> allCourseSchedules;
		if (!CollectionUtils.isEmpty(existCourseSchedules)) {
			//如果是直播课，那么只校验同一个课程组的课程
			CourseSchedule courseSchedule = courseSchedules.get(0);
			if(courseSchedule.getType() == CourseScheduleType.LIVE){
				existCourseSchedules = existCourseSchedules.stream().
						filter(e -> e.getMusicGroupId().equals(courseSchedule.getMusicGroupId())).collect(Collectors.toList());
			}
			allCourseSchedules = ListUtils.sum(courseSchedules, existCourseSchedules);
		} else {
			allCourseSchedules = courseSchedules;
		}

		//所有课程编号
		Set<Long> allCourseScheduleIds = allCourseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toSet());
		//所有课程学员签到记录
		List<StudentAttendance> studentAttendances = studentAttendanceDao.findByCourseIds(new ArrayList<>(allCourseScheduleIds));
		//课程请假学员字典
		Map<Long, Set<Integer>> courseLeaveStudentMap = studentAttendances.stream()
				.filter(e -> StudentAttendanceStatusEnum.LEAVE.equals(e.getStatus()))
				.collect(Collectors.groupingBy(StudentAttendance::getCourseScheduleId, Collectors.mapping(StudentAttendance::getUserId, Collectors.toSet())));

		//所有课程的班级编号
		List<Integer> classGroupIds = allCourseSchedules
				.stream()
				.map(CourseSchedule::getClassGroupId)
				.distinct()
				.collect(Collectors.toList());
		//班级与学生的关联记录
		List<ClassGroupStudentMapper> classGroupStudentMappers = classGroupStudentMapperDao.findByClassGroups(classGroupIds);
		Boolean liveFlag = false;
		if(Objects.nonNull(classGroupStudentMapper)){
			classGroupStudentMappers.add(classGroupStudentMapper);
			if(classGroupStudentMapper.getGroupType() == LIVE){
				liveFlag = true;
			}
		}
		Map<Integer, List<ClassGroupStudentMapper>> classGroupStudentsMap = classGroupStudentMappers
				.stream()
				.collect(Collectors.groupingBy(ClassGroupStudentMapper::getClassGroupId));

		//根据课程获取助教id关联集合
		List<IntegerAndIntegerListDto> courseScheduleTeachingTeacherIdList = new ArrayList<>();
		if (!CollectionUtils.isEmpty(allCourseScheduleIds)) {
			courseScheduleTeachingTeacherIdList = courseScheduleDao.findCourseScheduleIdAndUserIdsMap(new ArrayList<>(allCourseScheduleIds), TeachTypeEnum.TEACHING.getCode());
		}
		Map<Long, IntegerAndIntegerListDto> courseScheduleTeachingTeacherMap = courseScheduleTeachingTeacherIdList.stream()
				.collect(Collectors.toMap(IntegerAndIntegerListDto::getId, integerAndIntegerListDto -> integerAndIntegerListDto));

		//班级助教关联ID集合
		List<IntegerAndIntegerListDto> classGroupAndUserIdsMap = courseScheduleDao.findClassGroupAndUserIdsMap(newCourseScheduleClassGroupIds, TeachTypeEnum.TEACHING.getCode());
		Map<Long, IntegerAndIntegerListDto> classGroupTeachingTeacherMap = classGroupAndUserIdsMap.stream()
				.collect(Collectors.toMap(IntegerAndIntegerListDto::getId, integerAndIntegerListDto -> integerAndIntegerListDto));

//        Set<Long> existCourseScheduleIdsSet=new HashSet<>(existCourseScheduleIds);

		Map<String, List<CourseSchedule>> existClassDateCoursesMap = allCourseSchedules.stream().collect(Collectors.groupingBy(c -> DateUtil.dateToString(c.getClassDate(), "yyyy-MM-dd")));
		Map<String, List<CourseSchedule>> newClassDateCoursesMap = courseSchedules.stream().collect(Collectors.groupingBy(c -> DateUtil.dateToString(c.getClassDate(), "yyyy-MM-dd")));
		if (allCourseSchedules.size() > 1) {
			for (Map.Entry<String, List<CourseSchedule>> classDateCourseEntry : newClassDateCoursesMap.entrySet()) {
				List<CourseSchedule> existClassDateCourses = existClassDateCoursesMap.get(classDateCourseEntry.getKey());
				if(CollectionUtils.isEmpty(existClassDateCourses)){
					continue;
				}
				List<CourseSchedule> newClassDateCourses=classDateCourseEntry.getValue();
				existClassDateCourses.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
				newClassDateCourses.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
				for (CourseSchedule preCourseSchedule : newClassDateCourses) {
					//调整后的课程截止日期不能超过学期时间
					if(preCourseSchedule.getValidEndTime() != null){
						if(preCourseSchedule.getValidEndTime().compareTo(preCourseSchedule.getClassDate()) < 0){
							throw new BizException("操作失败：不允许跨学期调整");
						}
					}
					if(preCourseSchedule.getValidStartTime() != null){
						if(preCourseSchedule.getValidStartTime().compareTo(preCourseSchedule.getClassDate()) > 0){
							throw new BizException("操作失败：不允许跨学期调整");
						}
					}
					for (CourseSchedule backCourseSchedule : existClassDateCourses) {

						//判断前后两节课是否存在冲突
						if (!preCourseSchedule.getStartClassTime().before(backCourseSchedule.getEndClassTime())
								||!preCourseSchedule.getEndClassTime().after(backCourseSchedule.getStartClassTime())) {
							continue;
						}
						if (preCourseSchedule.equals(backCourseSchedule)) {
							continue;
						}

						//如果存在时间重叠，则需要判断前后两节课的教师和学生是否存在冲突
						//主教冲突检测
						if (Objects.nonNull(preCourseSchedule.getActualTeacherId())
								&& preCourseSchedule.getActualTeacherId().equals(backCourseSchedule.getActualTeacherId())) {
							if(postponeFlag){
								return BaseController.failed();
							}else {
								if(liveFlag){
									return BaseController.failed(HttpStatus.CHECKPOINT,
											courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 1) + ",是否继续购买");
								}
								throw new BizException(courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 1));
							}
						}
						//助教冲突检测
						List<Integer> preTeachingTeacherIdList = new ArrayList<>();
						List<Integer> backTeachingTeacherIdList = new ArrayList<>();
						if (allCourseScheduleIds.contains(preCourseSchedule.getId())) {
							IntegerAndIntegerListDto integerAndIntegerListDto = courseScheduleTeachingTeacherMap.get(preCourseSchedule.getId());
							if (Objects.nonNull(integerAndIntegerListDto)) {
								preTeachingTeacherIdList.addAll(integerAndIntegerListDto.getIds());
							}
						}
						if (CollectionUtils.isEmpty(preTeachingTeacherIdList)&&Objects.isNull(preCourseSchedule.getId())) {
							IntegerAndIntegerListDto integerAndIntegerListDto = classGroupTeachingTeacherMap.get(preCourseSchedule.getClassGroupId().longValue());
							if (Objects.nonNull(integerAndIntegerListDto)) {
								preTeachingTeacherIdList.addAll(integerAndIntegerListDto.getIds());
							}
						}

						if (allCourseScheduleIds.contains(backCourseSchedule.getId())) {
							IntegerAndIntegerListDto integerAndIntegerListDto = courseScheduleTeachingTeacherMap.get(backCourseSchedule.getId());
							if (Objects.nonNull(integerAndIntegerListDto)) {
								backTeachingTeacherIdList.addAll(integerAndIntegerListDto.getIds());
							}
						}
//                        if (CollectionUtils.isEmpty(backTeachingTeacherIdList)) {
//                            IntegerAndIntegerListDto integerAndIntegerListDto = classGroupTeachingTeacherMap.get(backCourseSchedule.getClassGroupId().longValue());
//                            if (Objects.nonNull(integerAndIntegerListDto)) {
////								backTeachingTeacherIdList.addAll(integerAndIntegerListDto.getIds());
//                            }
//                        }

						preTeachingTeacherIdList.add(preCourseSchedule.getActualTeacherId());
						backTeachingTeacherIdList.add(backCourseSchedule.getActualTeacherId());

						if (!CollectionUtils.isEmpty(preTeachingTeacherIdList)
								&& !CollectionUtils.isEmpty(backTeachingTeacherIdList)) {
							List<Integer> repeatIds = preTeachingTeacherIdList
									.stream().filter(backTeachingTeacherIdList::contains)
									.collect(Collectors.toList());
							if (!CollectionUtils.isEmpty(repeatIds)) {
								if(postponeFlag){
									return BaseController.failed();
								}else {
									if(liveFlag){
										return BaseController.failed(HttpStatus.CHECKPOINT,
												courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 2) + ",是否继续购买");
									}
									throw new BizException(courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 2));
								}
							}
						}
						//学生冲突检测
						if (preCourseSchedule.getClassGroupId().equals(backCourseSchedule.getClassGroupId())) {
							//如果班级相同，则学生肯定存在冲突
							if(postponeFlag){
								return BaseController.failed();
							}else {
								if(liveFlag){
									return BaseController.failed(HttpStatus.CHECKPOINT,
											courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 3) + ",是否继续购买");
								}
								throw new BizException(courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 3));
							}
						}
						//如果班级不同，则需要检测两个班级是否存在重复的学生
						List<ClassGroupStudentMapper> preClassGroupStudents = classGroupStudentsMap.get(preCourseSchedule.getClassGroupId());
						List<ClassGroupStudentMapper> backClassGroupStudents = classGroupStudentsMap.get(backCourseSchedule.getClassGroupId());
						//如果有一个存在没有学生的班级则不存在冲突
						if (CollectionUtils.isEmpty(preClassGroupStudents) || CollectionUtils.isEmpty(backClassGroupStudents)) {
							continue;
						}
						//课程对应请假学员编号集合
						Set<Integer> preLeaveStudentIds = courseLeaveStudentMap.get(preCourseSchedule.getId());
						if(null == preLeaveStudentIds){
							preLeaveStudentIds = Collections.EMPTY_SET;
						}
						Set<Integer> backLeaveStudentIds = courseLeaveStudentMap.get(backCourseSchedule.getId());
						if(null == backLeaveStudentIds){
							backLeaveStudentIds = Collections.EMPTY_SET;
						}

						//当前课程所在班级的学生编号列表
						Set<Integer> finalPreLeaveStudentIds = preLeaveStudentIds;
						List<Integer> preClassGroupStudentIds = preClassGroupStudents.stream()
								.filter(e->!finalPreLeaveStudentIds.contains(e.getUserId()))
								.map(ClassGroupStudentMapper::getUserId)
								.collect(Collectors.toList());
						//后面一节课程所在班级的学生编号列表
						Set<Integer> finalBackLeaveStudentIds = backLeaveStudentIds;
						Set<Integer> backClassGroupStudentIds = backClassGroupStudents.stream()
								.filter(e->!finalBackLeaveStudentIds.contains(e.getUserId()))
								.map(ClassGroupStudentMapper::getUserId)
								.collect(Collectors.toSet());
						List<Integer> repeatStudentIds = preClassGroupStudentIds.stream()
								.filter(backClassGroupStudentIds::contains)
								.collect(Collectors.toList());
						if (!CollectionUtils.isEmpty(repeatStudentIds)) {
							if(postponeFlag){
								return BaseController.failed();
							}else {
								if(liveFlag){
									return BaseController.failed(HttpStatus.CHECKPOINT,
											courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 3) + ",是否继续购买");
								}
								throw new BizException(courseCheckInfo(preCourseSchedule, backCourseSchedule, existCourseScheduleIds, 3));
							}
						}
					}
				}
			}
		}
		return BaseController.succeed();
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
    public HttpResponseResult checkNewCourseSchedules(List<CourseSchedule> courseSchedules, boolean checkExistCourseSchedule,Boolean postponeFlag,ClassGroupStudentMapper classGroupStudentMapper) {

		return this.checkCourseSchedules(courseSchedules,checkExistCourseSchedule,postponeFlag,classGroupStudentMapper);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
    public HttpResponseResult checkNewCourseSchedules(List<CourseSchedule> courseSchedules, boolean checkExistCourseSchedule,Boolean postponeFlag) {

		return this.checkCourseSchedules(courseSchedules,checkExistCourseSchedule,postponeFlag,null);
    }

    private String courseCheckInfo(CourseSchedule preCourseSchedule, CourseSchedule backCourseSchedule, List<Long> existCourseScheduleIds, Integer type) {
        List<CourseSchedule> courseSchedules=new ArrayList<>();
        courseSchedules.add(preCourseSchedule);
        courseSchedules.add(backCourseSchedule);
        courseSchedules.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
        //提示信息
        StringBuffer errInfo = new StringBuffer("在");
        errInfo.append(DateUtil.dateToString(courseSchedules.get(1).getStartClassTime(), "yyyy-MM-dd HH:mm"));
        errInfo.append("至");
        errInfo.append(DateUtil.dateToString(courseSchedules.get(0).getEndClassTime().before(courseSchedules.get(1).getEndClassTime())?courseSchedules.get(0).getEndClassTime():courseSchedules.get(1).getEndClassTime(), "HH:mm"));
        errInfo.append("的时间段内");
        errInfo.append("安排的课程存在冲突,");
        errInfo.append("冲突课程为：");

        CourseSchedule courseSchedule = null;
        if (Objects.nonNull(preCourseSchedule.getId()) && existCourseScheduleIds.contains(preCourseSchedule.getId())) {
            courseSchedule = preCourseSchedule;
        }
        if (Objects.nonNull(backCourseSchedule.getId()) && existCourseScheduleIds.contains(backCourseSchedule.getId())) {
            courseSchedule = backCourseSchedule;
        }

		if (Objects.isNull(courseSchedule)) {
			errInfo.setLength(0);
			errInfo.append("您设置的循环周期存在时间冲突");
			return errInfo.toString();
		}

        String groupName = "";

        if (Objects.nonNull(courseSchedule.getId())) {
            if (courseSchedule.getType().equals(CourseSchedule.CourseScheduleType.VIP)) {
                VipGroup vipGroup = vipGroupDao.get(Long.valueOf(courseSchedule.getMusicGroupId()));
                if (Objects.nonNull(vipGroup)) {
                    groupName = vipGroup.getName();
                }
            }else if(courseSchedule.getType().equals(CourseSchedule.CourseScheduleType.PRACTICE)){
                PracticeGroup practiceGroup = practiceGroupDao.get(Long.valueOf(courseSchedule.getMusicGroupId()));
                if (Objects.nonNull(practiceGroup)) {
                    groupName = practiceGroup.getName();
                }
            }else{
                MusicGroup musicGroup = musicGroupDao.get(courseSchedule.getMusicGroupId());
                if (Objects.nonNull(musicGroup)) {
                    groupName = musicGroup.getName();
                }
            }
        }

        errInfo.append(groupName);
        errInfo.append("(");
        errInfo.append(courseSchedule.getMusicGroupId());
        errInfo.append(")-");
        errInfo.append(courseSchedule.getName());
        errInfo.append("(");
        errInfo.append(courseSchedule.getId());
        errInfo.append(")，");

        errInfo.append("类型为：");
        switch (type) {
            case 1:
                errInfo.append("主教冲突");
                break;
            case 2:
                errInfo.append("助教冲突");
                break;
            case 3:
                errInfo.append("学生冲突");
                break;
            default:
                errInfo.append("未知冲突");
                break;
        }

        return errInfo.toString();
    }

    /**
     * @Author: Joburgess
     * @Date: 2019/9/29
     * 生成课程名
     */
    @Override
    public void createCourseScheduleName(List<CourseSchedule> courseSchedules) {
        List<Integer> classGroupIds = courseSchedules.stream().map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
        HashSet<Integer> temp = new HashSet<>(classGroupIds);
        classGroupIds = new ArrayList<>(temp);
        Map<Integer, String> classIdSubjectNameMap = convertToMap(courseScheduleDao.findClassSubjects(classGroupIds));
        courseSchedules.forEach(courseSchedule -> {
        	if(StringUtils.isNotBlank(courseSchedule.getName())){
        		return;
			}
            if (courseSchedule.getType().equals(CourseSchedule.CourseScheduleType.MIX)) {
                courseSchedule.setName(courseSchedule.getType().getMsg());
            } else {
                StringBuilder className = new StringBuilder(classIdSubjectNameMap.get(courseSchedule.getClassGroupId()));
                className.append("-");
                className.append(courseSchedule.getType().getMsg());
                courseSchedule.setName(className.toString());
            }
            courseSchedule.setStatus(CourseStatusEnum.NOT_START);
        });
    }

    private Map<Integer, String> convertToMap(String[] result) {
        Map<Integer, String> map = new HashMap<>();
        for (String s : result) {
        	if(s.split("-").length>1){
				map.put(Integer.valueOf(s.split("-")[0]), s.split("-")[1].replace(",", "/"));
			}
        }
        return map;
    }

	public List<CourseScheduleModifyLog> getModifyLogs(List<Long> courseScheduleIds,List<CourseSchedule> courseSchedules,Integer userId,Date now){
		List<CourseScheduleModifyLog> insertCourseScheduleModifyLogList = new ArrayList<>();
		//获取调整前助教信息
		Map<Long, String> teachingTeacherMap = MapUtil.convertMybatisMap(courseScheduleTeacherSalaryDao.queryTeacherIdMap(courseScheduleIds,"TEACHING"));
		for (CourseSchedule e : courseSchedules) {
			// 调整日志
			e.setTeachingTeacherIds(teachingTeacherMap.get(e.getId()));
			CourseScheduleModifyLog courseScheduleModifyLog = new CourseScheduleModifyLog();
			courseScheduleModifyLog.setCourseScheduleId(e.getId());
			courseScheduleModifyLog.setCreateTime(now);
			courseScheduleModifyLog.setPreviousCourseSchedule(JsonUtil.toJSONString(e));
			courseScheduleModifyLog.setOperatorId(userId);
			insertCourseScheduleModifyLogList.add(courseScheduleModifyLog);
		}
		return insertCourseScheduleModifyLogList;
	}

    public void saveModifyLog(List<CourseScheduleModifyLog> modifyLogs,List<Long> courseIds){
		//保存日志调整信息
		//获取调整后助教信息
		Map<Long, String> newTeachingTeacherMap = MapUtil.convertMybatisMap(courseScheduleTeacherSalaryDao.queryTeacherIdMap(courseIds,"TEACHING"));
		List<CourseSchedule> newCourseScheduleList = courseScheduleDao.findByCourseScheduleIds(courseIds);
		Map<Long, List<CourseSchedule>> collect = newCourseScheduleList.stream().collect(Collectors.groupingBy(CourseSchedule::getId));
		modifyLogs.forEach(e->{
			CourseSchedule schedule = collect.get(e.getCourseScheduleId()).get(0);
			schedule.setTeachingTeacherIds(newTeachingTeacherMap.get(schedule.getId()));
			e.setCurrentCourseSchedule(JsonUtil.toJSONString(schedule));
		});
		if (modifyLogs.size() > 0) {
			courseScheduleModifyLogService.getDao().batchInsert(modifyLogs);
		}
	}

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void vipCourseAdjust(VipGroupCourseAdjustInfoDto vipGroupCourseAdjustInfo) {
		SysUser user = sysUserService.getUser();
		if (Objects.isNull(vipGroupCourseAdjustInfo.getVipGroupId())) {
			throw new BizException("请指定课程");
		}
		if (Objects.isNull(vipGroupCourseAdjustInfo.getCourseCreateStartTime())) {
			throw new BizException("请指定排课起始时间;");
		}
		if (CollectionUtils.isEmpty(vipGroupCourseAdjustInfo.getCourseTimes())) {
			throw new BizException("请指定排课周期");
		}
		if (StringUtils.isBlank(vipGroupCourseAdjustInfo.getCourseScheduleIds())) {
			throw new BizException("请指定需要调整的课程");
		}
		Date courseCreateStartTime = vipGroupCourseAdjustInfo.getCourseCreateStartTime();
		Date now = new Date();
		Date today = DateUtil.stringToDate(DateUtil.dateToString(now, DateUtil.ISO_EXPANDED_DATE_FORMAT),
				DateUtil.ISO_EXPANDED_DATE_FORMAT);
		if (courseCreateStartTime.before(today)) {
			throw new BizException("排课起始时间不能小于当前时间");
		}

		boolean needSchoolId = Objects.nonNull(vipGroupCourseAdjustInfo.getTeachMode()) && vipGroupCourseAdjustInfo.getTeachMode().equals(TeachModeEnum.OFFLINE) && Objects.isNull(vipGroupCourseAdjustInfo.getSchoolId());
		if (needSchoolId) {
			throw new BizException("请设置教学点");
		}

		int singleClassMinutes = 0;
		int courseNum = 0;
		Date coursesExpireDate = null;
		Date coursesStartDate = null;
		GroupType groupType = vipGroupCourseAdjustInfo.getGroupType();

		VipGroup vipGroup=null;
		VipGroupActivity vipGroupActivity = null;

		if(groupType == VIP || groupType == LIVE) {
			vipGroup = vipGroupDao.get(vipGroupCourseAdjustInfo.getVipGroupId().longValue());
			if(Objects.isNull(vipGroup)){
				throw new BizException("课程组信息不存在");
			}
			vipGroupActivity = vipGroupActivityDao.get(vipGroup.getVipGroupActivityId());
		}

		switch (vipGroupCourseAdjustInfo.getGroupType()){
			case PRACTICE:
				PracticeGroup practiceGroup = practiceGroupDao.get(vipGroupCourseAdjustInfo.getVipGroupId().longValue());
				if (Objects.isNull(practiceGroup)) {
					throw new BizException("指定课程不存在");
				}
				singleClassMinutes = practiceGroup.getSingleClassMinutes();
				courseNum = courseScheduleDao.countVipGroupCourses(practiceGroup.getId().intValue(),groupType.getCode());
				coursesExpireDate = practiceGroup.getCoursesExpireDate();
				coursesStartDate = practiceGroup.getCoursesStartDate();
				break;
			default:
				if (Objects.isNull(vipGroup)) {
					throw new BizException("指定课程不存在");
				}
				if(Objects.nonNull(vipGroup)
						&&!VipGroupStatusEnum.PROGRESS.equals(vipGroup.getStatus())
						&&!VipGroupStatusEnum.FINISHED.equals(vipGroup.getStatus())){
					throw new BizException("非进行中课程组不允许进行此操作");
				}
				singleClassMinutes = vipGroup.getSingleClassMinutes();
				courseNum = courseScheduleDao.countVipGroupCourses(vipGroup.getId().intValue(),groupType.getCode());
		}
		if (courseNum == 0) {
			throw new BizException("未排课的课程不能进行调整");
		}

		ClassGroup classGroup = classGroupDao.findByGroupAndType(vipGroupCourseAdjustInfo.getVipGroupId().toString(),groupType.getCode());
		if (Objects.isNull(classGroup)) {
			throw new BizException("对应班级不存在");
		}

		//所有课程编号
		List<Long> courseScheduleIds = Arrays.asList(vipGroupCourseAdjustInfo.getCourseScheduleIds().split(","))
				.stream().map(Long::parseLong).collect(Collectors.toList());
		//所有的课程
		List<CourseSchedule> courseSchedules = courseScheduleDao.findByCourseScheduleIds(courseScheduleIds);
		//校验可排课时间
		this.checkLiveAdjust(courseSchedules);
		//校验机构线上课人数上限
		this.checkOnlineCategory(courseSchedules,vipGroupCourseAdjustInfo.getTeachMode());
		courseSchedules.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
		//拷贝历史直播课
		List<CourseSchedule> liveCourseSchedules = JSON.parseArray(JSON.toJSONString(courseSchedules),CourseSchedule.class);
		//获取操作日志
		List<CourseScheduleModifyLog> modifyLogs = getModifyLogs(courseScheduleIds, courseSchedules, user.getId(), now);

		if (courseScheduleIds.size() != courseScheduleIds.size()) {
			throw new BizException("部分课程不存在");
		}

		Calendar calendar = Calendar.getInstance();
		calendar.setTime(courseCreateStartTime);
		List<Date> courseStartDates = new ArrayList<>();

		Set<String> holidayDays = new HashSet<>();

		Integer tenantId = TenantContextHolder.getTenantId();
		if (vipGroupCourseAdjustInfo.isHoliday()) {
			String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING,tenantId);
			if(StringUtils.isNotBlank(holidaySetting)){
				holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
			}
		}

		while (true) {
			if (vipGroupCourseAdjustInfo.isHoliday() && holidayDays.contains(DateUtil.format(calendar.getTime(), "yyyy-MM-dd"))) {
				calendar.add(Calendar.DATE, 1);
				continue;
			}
			int i = calendar.get(Calendar.DAY_OF_WEEK);
			if (i == 1) {
				i = 7;
			} else {
				i = i - 1;
			}
			for (CourseTimeDto courseTime : vipGroupCourseAdjustInfo.getCourseTimes()) {
				if (Objects.isNull(courseTime.getDayOfWeek())) {
					throw new BizException("排课循环周期错误");
				}
				if (courseTime.getDayOfWeek() < 1 || courseTime.getDayOfWeek() > 7) {
					throw new BizException("排课循环周期错误");
				}
				if (courseTime.getDayOfWeek() == i) {
					String dateYmdStr = DateUtil.dateToString(calendar.getTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
					dateYmdStr = dateYmdStr + " " + courseTime.getStartClassTime();
					Date courseStartTime = DateUtil.stringToDate(dateYmdStr, "yyyy-MM-dd HH:mm");
					Date courseEndTime = DateUtil.addMinutes(courseStartTime, singleClassMinutes);
					if(DateUtil.daysBetween(DateUtil.stringToDate(DateUtil.format(courseStartTime,DateUtil.DEFAULT_PATTERN),DateUtil.DEFAULT_PATTERN),
							DateUtil.stringToDate(DateUtil.format(courseEndTime,DateUtil.DEFAULT_PATTERN),DateUtil.DEFAULT_PATTERN)) > 0){
						throw new BizException("单节课时不允许跨天");
					}
					courseStartDates.add(DateUtil.stringToDate(dateYmdStr, DateUtil.EXPANDED_DATE_TIME_FORMAT));
					CourseSchedule courseSchedule = courseSchedules.get(courseStartDates.size() - 1);
					//校验课程是否结算
					this.checkCourseIsSettlement(courseSchedule);
					int num = studentAttendanceDao.countStudentAttendenceNum(courseSchedule.getId().intValue());
					if (num > 0) {
						throw new BizException("{}[{}]{}-{}课程已点名",
								courseSchedule.getName(),
								courseSchedule.getId(),
								DateUtil.dateToString(courseSchedule.getStartClassTime(),
										DateUtil.EXPANDED_DATE_TIME_FORMAT),
								DateUtil.dateToString(courseSchedule.getEndClassTime(),
										DateUtil.EXPANDED_DATE_TIME_FORMAT));
					}
					courseSchedule.setStatus(CourseStatusEnum.NOT_START);
					courseSchedule.setClassDate(courseStartTime);
					courseSchedule.setStartClassTime(courseStartTime);
					courseSchedule.setEndClassTime(courseEndTime);

					Integer allowOnlineToOffline = 0;
					if(Objects.nonNull(vipGroupActivity)){
						allowOnlineToOffline = vipGroupActivity.getAllowOnlineToOffline();
						ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(vipGroup.getId(), "VIP",null);
						if(activityUserMapper != null){
							if(vipGroup.getId().equals(activityUserMapper.getGiveVipGroupId())){
								allowOnlineToOffline = vipGroupActivity.getGiveAllowOnlineToOffline();
							}
						}
					}

					if (Objects.nonNull(vipGroupCourseAdjustInfo.getTeachMode())) {
						boolean onlineToOffline = courseSchedule.getTeachMode().equals(TeachModeEnum.ONLINE)
								&&vipGroupCourseAdjustInfo.getTeachMode().equals(TeachModeEnum.OFFLINE);
						boolean offlineToOnline = courseSchedule.getTeachMode().equals(TeachModeEnum.OFFLINE)
								&&vipGroupCourseAdjustInfo.getTeachMode().equals(TeachModeEnum.ONLINE);
						if(onlineToOffline && Objects.nonNull(vipGroupActivity) && (allowOnlineToOffline == 0 || allowOnlineToOffline == 3)){
							throw new BizException("此VIP课活动不支持线上课调整为线下课");
						}
						if(offlineToOnline&&Objects.nonNull(vipGroupActivity) && (allowOnlineToOffline == 2 || allowOnlineToOffline == 3)){
							throw new BizException("此VIP课活动不支持线下课调整为线上课");
						}
						if(onlineToOffline&&Objects.isNull(vipGroupCourseAdjustInfo.getSchoolId())){
							throw new BizException("请选择教学点");
						}
						courseSchedule.setTeachMode(vipGroupCourseAdjustInfo.getTeachMode());
						if (vipGroupCourseAdjustInfo.getTeachMode().equals(TeachModeEnum.OFFLINE)) {
							if(Objects.nonNull(vipGroupCourseAdjustInfo.getSchoolId())){
								courseSchedule.setSchoolId(vipGroupCourseAdjustInfo.getSchoolId());
							}
						} else {
							courseSchedule.setSchoolId(null);
						}
					}
				}
				if (courseStartDates.size() == courseScheduleIds.size()) {
					break;
				}
			}
			if (courseStartDates.size() == courseScheduleIds.size()) {
				break;
			}
			calendar.add(Calendar.DATE, 1);
		}
		checkNewCourseSchedules(courseSchedules, false,false);
		//如果是陪练课，调整时间不允许超过有效期
		if(groupType == PRACTICE){
			for (CourseSchedule e:courseSchedules) {
				//已点名的不允许调整
				List<StudentAttendance> studentAttendances = studentAttendanceDao.findByCourseId(e.getId());
				if(studentAttendances != null && studentAttendances.size() > 0){
					throw new BizException("调整失败: 课程已点名");
				}
				String classDate = DateUtil.format(e.getClassDate(), DateUtil.DEFAULT_PATTERN);
				String startClassTime = DateUtil.format(e.getStartClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
				Date startDateTime = DateUtil.stringToDate(classDate + " " + startClassTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
				Date endDateTime = DateUtil.addMinutes(startDateTime,singleClassMinutes);
				if(DateUtil.minutesBetween(startDateTime,coursesStartDate) > 0){
					throw new BizException("调整失败: 课程{}调整时间早于有效期",e.getId());
				}
				if(DateUtil.minutesBetween(coursesExpireDate,endDateTime) > 0){
					throw new BizException("调整失败: 课程{}截止时间超过课程有效期",e.getId());
				}
			}
		}
		if(groupType == VIP || groupType == LIVE){
			CourseSchedule courseSchedule = courseSchedules.stream().max(Comparator.comparing(CourseSchedule::getEndClassTime)).get();
			if(courseSchedule.getEndClassTime().compareTo(vipGroup.getCoursesExpireDate())>0){
				vipGroup.setCoursesExpireDate(courseSchedule.getEndClassTime());
				vipGroupDao.update(vipGroup);
			}

			// 直播课数据清除
			for (CourseSchedule item : courseSchedules) {
				if (LIVE == item.getGroupType()) {
					// 消息提醒
					item.setLiveRemind(0);
					// 直播间关联
					item.setLiveRoomId("");
					// 连堂课标识
					item.setContinuousCourse(false);
				}
			}
		}

		courseScheduleDao.batchUpdate(courseSchedules);
		classGroupService.updateClassGroupInfo(classGroup.getId());
		if(vipGroupCourseAdjustInfo.getCourseCreateStartTime().after(now)){
			teacherAttendanceDao.batchCleanCourseTeacherSignInfo(courseScheduleIds);
		}
		studentAttendanceDao.deleteByCourseSchedules(courseScheduleIds);
		//删除作业
		courseHomeworkService.delHomwworkByCourseScheduleId(courseScheduleIds);
		//删除评论
		courseScheduleReviewDao.delByCourseScheduleIds(courseScheduleIds);
		//删除申述
		courseScheduleComplaintsDao.delByCourseScheduleIds(courseScheduleIds);
		//清除考勤申述
		teacherAttendanceDao.clearAttendanceComplaints(courseScheduleIds);
//        vipGroupService.checkVipCourseIsInScore(vipGroupCourseAdjustInfo.getVipGroupId().longValue());
//        courseScheduleStudentPaymentService.updateVipGiveCourse(vipGroup.getId());
		//保存操作日志
		saveModifyLog(modifyLogs,courseScheduleIds);
		//发送直播课调整通知
		this.sendChatRoomMessage(liveCourseSchedules);
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public HttpResponseResult batchCourseAdjust(BatchCourseAdjustDto courseAdjustInfo) {
		if(CollectionUtils.isEmpty(courseAdjustInfo.getCourseScheduleIds())){
			return BaseController.failed("请指定需要调整的课程");
		}

		//未更新课程列表
		List<CourseSchedule> oldCourses = courseScheduleDao.findByCourseScheduleIds(courseAdjustInfo.getCourseScheduleIds());

		if(CollectionUtils.isEmpty(oldCourses)){
			return BaseController.succeed();
		}

		Set<String> musicGroupIds = oldCourses.stream().filter(c -> MUSIC.equals(c.getGroupType())).map(CourseSchedule::getMusicGroupId).collect(Collectors.toSet());
		if(!CollectionUtils.isEmpty(musicGroupIds)){
			List<MusicGroup> musicGroups = musicGroupDao.getMusicGroupByIds(new ArrayList<>(musicGroupIds));
			if(CollectionUtils.isEmpty(musicGroups)){
				throw new BizException("调整失败：乐团信息异常");
			}
			long errMusicGroupNum = musicGroups.stream().filter(m -> !MusicGroupStatusEnum.PROGRESS.equals(m.getStatus())).count();
			if(errMusicGroupNum>0){
				throw new BizException("调整失败：调整的课程所在乐团处于非进行中");
			}
		}

		Date now = new Date();

		//获取修改日志
		List<CourseScheduleModifyLog> modifyLogs = getModifyLogs(courseAdjustInfo.getCourseScheduleIds(),oldCourses,courseAdjustInfo.getOperatorId(),now);

		List<CourseScheduleTeacherSalary> oldCourseScheduleTeacherSalaryList = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseAdjustInfo.getCourseScheduleIds());
		Map<Long, List<CourseScheduleTeacherSalary>> courseSalaryListMap = new HashMap<>();
		if(!CollectionUtils.isEmpty(oldCourseScheduleTeacherSalaryList)){
			courseSalaryListMap = oldCourseScheduleTeacherSalaryList.stream().collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId));
		}

		//需要变更课酬的教师类型
		Set<TeachTypeEnum> changeSalaryTeachTypes = new HashSet<>();

		if(courseAdjustInfo.getChangeMainTeacher()){
			changeSalaryTeachTypes.add(TeachTypeEnum.BISHOP);
			courseScheduleTeacherSalaryDao.deleteWithCourseAndTeachRole(courseAdjustInfo.getCourseScheduleIds(), TeachTypeEnum.BISHOP);
		}
		if(courseAdjustInfo.getChangeTeachingTeacher()){
			changeSalaryTeachTypes.add(TeachTypeEnum.TEACHING);
			courseScheduleTeacherSalaryDao.deleteWithCourseAndTeachRole(courseAdjustInfo.getCourseScheduleIds(), TeachTypeEnum.TEACHING);
		}

		if(CollectionUtils.isEmpty(courseAdjustInfo.getClassGroupTeacherMapperList())){
			courseAdjustInfo.setClassGroupTeacherMapperList(new ArrayList<>());
		}

		//获取主教编号
		Integer mainTeacherId = null;
		if(!CollectionUtils.isEmpty(courseAdjustInfo.getClassGroupTeacherMapperList())){
			Map<Integer, Long> userNumMap = courseAdjustInfo.getClassGroupTeacherMapperList().stream().collect(Collectors.groupingBy(ClassGroupTeacherMapper::getUserId, Collectors.counting()));
			if(userNumMap.containsValue(Long.valueOf("2"))){
				throw new BizException("主教与助教不可重复");
			}
			ClassGroupTeacherMapper classGroupTeacherMapper = courseAdjustInfo.getClassGroupTeacherMapperList().stream().filter(c -> TeachTypeEnum.BISHOP.equals(c.getTeacherRole())).findAny().get();
			if(Objects.nonNull(classGroupTeacherMapper)){
				mainTeacherId = classGroupTeacherMapper.getUserId();
			}
		}

		//上课日期变更
		//间隔天数
		int betweenDays = 0;
		//节假日
		Set<String> holidayDays = new HashSet<>();
        Integer tenantId = TenantContextHolder.getTenantId();
        if(Objects.nonNull(courseAdjustInfo.getPauseDate())&&Objects.nonNull(courseAdjustInfo.getRecoveryDate())){
			if (courseAdjustInfo.getHoliday()) {
                String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING,tenantId);
                if(StringUtils.isNotBlank(holidaySetting)){
                    holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
                }
			}

			LocalDate pauseDate = LocalDate.parse(courseAdjustInfo.getPauseDate(), DateUtil.dateFormatter);
			LocalDate recoveryDate = LocalDate.parse(courseAdjustInfo.getRecoveryDate(), DateUtil.dateFormatter);

			betweenDays = (int) (recoveryDate.toEpochDay()-pauseDate.toEpochDay());
		}

		//上课时间变更
		LocalTime startTime = null;
		if(Objects.nonNull(courseAdjustInfo.getStartTime())){
			startTime = LocalTime.parse(courseAdjustInfo.getStartTime(), DateUtil.timeFormatter);
		}

		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = new ArrayList<>();

		oldCourses.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
		for (int i = 0; i < oldCourses.size(); i++) {
			if(Objects.nonNull(oldCourses.get(i).getNewCourseId())){
				throw new BizException("选择的课程中存在已合并课程，无法进行调整");
			}
			if(!CourseStatusEnum.NOT_START.equals(oldCourses.get(i).getStatus())){
				throw new BizException("选择的课程中存在进行中或已结束的课程，无法进行调整");
			}

			if(VIP.equals(oldCourses.get(i).getGroupType())){
				VipGroup vipGroup = vipGroupDao.get(Long.valueOf(oldCourses.get(i).getMusicGroupId()));
				if(Objects.isNull(vipGroup)){
					throw new BizException("课程组信息不存在");
				}
				VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroup.getVipGroupActivityId());
				Integer allowOnlineToOffline = 0;
				if(Objects.nonNull(vipGroupActivity)){
					allowOnlineToOffline = vipGroupActivity.getAllowOnlineToOffline();
					ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(vipGroup.getId(), "VIP",null);
					if(activityUserMapper != null){
						if(vipGroup.getId().equals(activityUserMapper.getGiveVipGroupId())){
							allowOnlineToOffline = vipGroupActivity.getGiveAllowOnlineToOffline();
						}
					}
				}
				boolean onlineToOffline = TeachModeEnum.ONLINE.equals(oldCourses.get(i).getTeachMode())&&TeachModeEnum.OFFLINE.equals(courseAdjustInfo.getTeachMode());
				boolean offlineToOnline = TeachModeEnum.OFFLINE.equals(oldCourses.get(i).getTeachMode())&&TeachModeEnum.ONLINE.equals(courseAdjustInfo.getTeachMode());
				if(onlineToOffline&&Objects.nonNull(vipGroupActivity)&&(allowOnlineToOffline == 0 || allowOnlineToOffline == 3)){
					throw new BizException("此VIP课活动不支持线上课调整为线下课");
				}
				if(offlineToOnline&&Objects.nonNull(vipGroupActivity)&&(allowOnlineToOffline == 2 || allowOnlineToOffline == 3)){
					throw new BizException("此VIP课活动不支持线下课调整为线上课");
				}
			}

			//上课日期变更
			if(betweenDays!=0){
				LocalDate classDate = LocalDateTime.ofInstant(oldCourses.get(i).getClassDate().toInstant(), DateUtil.zoneId).toLocalDate();
				classDate = classDate.plusDays(betweenDays);
				if (courseAdjustInfo.getHoliday() && holidayDays.contains(classDate.toString())) {
					betweenDays=betweenDays+7;
					i=i-1;
					continue;
				}
				oldCourses.get(i).setClassDate(Date.from(classDate.atStartOfDay().atZone(DateUtil.zoneId).toInstant()));
			}

			//上课时间变更
			if(Objects.nonNull(startTime)){
				//课程时长
				int minutes = DateUtil.minutesBetween(oldCourses.get(i).getStartClassTime(), oldCourses.get(i).getEndClassTime());
				LocalTime endTime = startTime.plusMinutes(minutes);
				String courseStartTimeStr = DateUtil.dateToString(oldCourses.get(i).getClassDate(), "yyyy-MM-dd") + " " + startTime.toString();
				String courseEndTimeStr = DateUtil.dateToString(oldCourses.get(i).getClassDate(), "yyyy-MM-dd") + " " + endTime.toString();
				oldCourses.get(i).setStartClassTime(DateUtil.stringToDate(courseStartTimeStr, "yyyy-MM-dd HH:mm"));
				try {
					oldCourses.get(i).setStartClassTimeStr(startTime.toString());
				} catch (ParseException e) {
					e.printStackTrace();
				}
				oldCourses.get(i).setEndClassTime(DateUtil.stringToDate(courseEndTimeStr, "yyyy-MM-dd HH:mm"));
				try {
					oldCourses.get(i).setEndClassTimeStr(endTime.toString());
				} catch (ParseException e) {
					e.printStackTrace();
				}
			}

			if(Objects.nonNull(courseAdjustInfo.getSchoolId())){
				oldCourses.get(i).setSchoolId(courseAdjustInfo.getSchoolId());
			}

			if(Objects.nonNull(courseAdjustInfo.getTeachMode())&&!courseAdjustInfo.getTeachMode().equals(oldCourses.get(i).getTeachMode())){
				oldCourses.get(i).setTeachMode(courseAdjustInfo.getTeachMode());
				if(VIP.equals(oldCourses.get(i).getGroupType())){
					changeSalaryTeachTypes.add(TeachTypeEnum.BISHOP);
					if(courseSalaryListMap.containsKey(oldCourses.get(i).getId())){
						List<CourseScheduleTeacherSalary> ctss = courseSalaryListMap.get(oldCourses.get(i).getId());
						for (CourseScheduleTeacherSalary teacherSalary : ctss) {
							//课酬调整后不能更新课酬
							if(!teacherSalary.getEnableChangeSalary()){
								continue;
							}
							courseScheduleTeacherSalaryService.updateSingleCourseTeacherSalary(oldCourses.get(i), teacherSalary);
						}
						courseScheduleTeacherSalaryDao.batchUpdate(ctss);
					}
				}
			}

			if(TeachModeEnum.ONLINE.equals(oldCourses.get(i).getTeachMode())){
				oldCourses.get(i).setSchoolId(null);
			}

			//主教变更
			if(Objects.nonNull(mainTeacherId)){
				oldCourses.get(i).setActualTeacherId(mainTeacherId);
			}

			if(oldCourses.get(i).getStartClassTime().compareTo(now)<0){
				throw new BizException("上课时间不得早于当前时间");
			}

			List<CourseScheduleTeacherSalary> oldCourseSalarys = courseSalaryListMap.get(oldCourses.get(i).getId());
			CourseScheduleTeacherSalary oldMainTeacherSalary = null;
			if(!CollectionUtils.isEmpty(oldCourseSalarys)){
				List<CourseScheduleTeacherSalary> mainTeachSalarys = oldCourseSalarys.stream().filter(c -> TeachTypeEnum.BISHOP.equals(c.getTeacherRole())).collect(Collectors.toList());
				oldMainTeacherSalary = CollectionUtils.isEmpty(mainTeachSalarys)?null:mainTeachSalarys.get(0);
			}

			for (ClassGroupTeacherMapper classGroupTeacherMapper : courseAdjustInfo.getClassGroupTeacherMapperList()) {
				if(!changeSalaryTeachTypes.contains(classGroupTeacherMapper.getTeacherRole())){
					continue;
				}
				CourseScheduleTeacherSalary courseScheduleTeacherSalary = new CourseScheduleTeacherSalary();
				courseScheduleTeacherSalary.setCourseScheduleId(oldCourses.get(i).getId());
				courseScheduleTeacherSalary.setGroupType(oldCourses.get(i).getGroupType());
				courseScheduleTeacherSalary.setMusicGroupId(oldCourses.get(i).getMusicGroupId());
				courseScheduleTeacherSalary.setTeacherRole(classGroupTeacherMapper.getTeacherRole());
				courseScheduleTeacherSalary.setUserId(classGroupTeacherMapper.getUserId());
				courseScheduleTeacherSalary.setClassGroupId(oldCourses.get(i).getClassGroupId());
				courseScheduleTeacherSalary.setCreateTime(now);
				courseScheduleTeacherSalary.setUpdateTime(now);

				if(TeachTypeEnum.BISHOP.equals(classGroupTeacherMapper.getTeacherRole())&&!oldMainTeacherSalary.getEnableChangeSalary()){
					courseScheduleTeacherSalary.setExpectSalary(oldMainTeacherSalary.getExpectSalary());
					courseScheduleTeacherSalary.setSubsidy(oldMainTeacherSalary.getSubsidy());
					courseScheduleTeacherSalary.setEnableChangeSalary(oldMainTeacherSalary.getEnableChangeSalary());
				}else{
					if(CourseScheduleType.PRACTICE.equals(oldCourses.get(i).getType())&&courseSalaryListMap.containsKey(oldCourses.get(i).getId())){
						courseScheduleTeacherSalary.setExpectSalary(courseSalaryListMap.get(oldCourses.get(i).getId()).get(0).getExpectSalary());
					}
					courseScheduleTeacherSalaryService.updateSingleCourseTeacherSalary(oldCourses.get(i), courseScheduleTeacherSalary);
				}
				courseScheduleTeacherSalaries.add(courseScheduleTeacherSalary);

				if (courseAdjustInfo.getConfirmGenerate() && !courseAdjustInfo.getAllowZeroSalary() && BigDecimal.ZERO.compareTo(courseScheduleTeacherSalary.getExpectSalary()) == 0) {
					TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
					return BaseController.failed(HttpStatus.MULTI_STATUS, "当前课程课酬预计为0,是否继续");
				}
			}
		}

		//老师结算表
		if (courseScheduleTeacherSalaries.size() > 0) {
			teacherAttendanceDao.batchDeleteByCourseSchedules(courseAdjustInfo.getCourseScheduleIds());
			courseScheduleTeacherSalaryService.batchInsert(courseScheduleTeacherSalaries);
		}

		try {
			checkNewCourseSchedules(oldCourses, false, false);
		} catch (Exception e) {
			TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
			String errMessage = e.getMessage();
			if (errMessage.equals("您设置的循环周期存在时间冲突")) {
				errMessage = "日期相同课程不可批量调整上课时间";
			}
			return BaseController.failed(HttpStatus.INTERNAL_SERVER_ERROR, errMessage);
		}

		if (!courseAdjustInfo.getConfirmGenerate()) {
			List<CourseScheduleTeacherSalary> cstss = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseAdjustInfo.getCourseScheduleIds());
			Map<Long, Set<Integer>> courseTeachTypeTeacherIdsMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(cstss)){
				courseTeachTypeTeacherIdsMap = cstss.stream().filter(c->TeachTypeEnum.TEACHING.equals(c.getTeacherRole())).collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId, Collectors.mapping(CourseScheduleTeacherSalary::getUserId, Collectors.toSet())));
			}
			Set<Integer> allTeacherIds = cstss.stream().map(CourseScheduleTeacherSalary::getUserId).collect(Collectors.toSet());
			List<SimpleUserDto> teachers = teacherDao.getUsersSimpleInfo(new ArrayList<>(allTeacherIds));
			Map<Integer, String> idNameMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(teachers)){
				idNameMap = teachers.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, s->s.getRealName(), (s1, s2)->s1));
			}
			//获取班级声部名称
			List<Map<Integer, String>> classGroupSubjectNameMaps = classGroupDao.findClassGroupSubjectNameMaps(courseAdjustInfo.getCourseScheduleIds());
			Map<Integer, String> classGroupSubjectNameMap = MapUtil.convertIntegerMap(classGroupSubjectNameMaps);

			List<Integer> schoolIds = oldCourses.stream().filter(o -> Objects.nonNull(o.getSchoolId())).map(CourseSchedule::getSchoolId).collect(Collectors.toList());
			Map<Integer, String> idSchoolNameMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(schoolIds)){
				List<School> schools = schoolDao.getSchools(schoolIds);
				idSchoolNameMap = schools.stream().collect(Collectors.toMap(School::getId, s->s.getName(), (s1, s2)->s1));
			}

			List<CourseScheduleEndDto> results = new ArrayList<>();
			for (CourseSchedule oldCours : oldCourses) {
				CourseScheduleEndDto courseEnd = new CourseScheduleEndDto();
				BeanUtils.copyProperties(oldCours, courseEnd);
				courseEnd.setTeacherName(idNameMap.get(oldCours.getActualTeacherId()));
				if(courseTeachTypeTeacherIdsMap.containsKey(oldCours.getId())){
					Set<Integer> teacherIds = courseTeachTypeTeacherIdsMap.get(oldCours.getId());
					Set<String> teacherNames = new HashSet<>();
					for (Integer teacherId : teacherIds) {
						teacherNames.add(idNameMap.get(teacherId));
					}
					courseEnd.setTeachingTeacherNames(StringUtils.join(teacherNames, ","));
				}
				courseEnd.setSubjectName(classGroupSubjectNameMap.get(oldCours.getId()));
				courseEnd.setSchoolName(idSchoolNameMap.get(oldCours.getSchoolId()));

				results.add(courseEnd);
			}
			TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
			oldCourses.sort(Comparator.comparing(CourseSchedule::getStartClassTime));
			return BaseController.failed(HttpStatus.PARTIAL_CONTENT, results, "");
		}

		courseScheduleDao.batchUpdate(oldCourses);

		courseScheduleModifyLogService.pushSchoolCourseChange(oldCourses.get(0));
		saveModifyLog(modifyLogs, courseAdjustInfo.getCourseScheduleIds());

		return BaseController.succeed();
	}

	@Override
    @Transactional(rollbackFor = Exception.class)
    public void coursePostpone(CoursePostponeDto coursePostPoneInfo) {
        SysUser user = sysUserService.getUser();
        if(StringUtils.isBlank(coursePostPoneInfo.getClassGroupIds())){
            throw new BizException("请指定班级");
        }
        if(Objects.isNull(coursePostPoneInfo.getPauseDate())){
            throw new BizException("请指定课程暂停时间");
        }
        if(Objects.isNull(coursePostPoneInfo.getRecoveryDate())){
            throw new BizException("请指定课程恢复时间");
        }
        //间隔天数
        int betweenDays = DateUtil.daysBetween(coursePostPoneInfo.getPauseDate(), coursePostPoneInfo.getRecoveryDate());
        List<CourseSchedule> classGroupCourseSchedulesWithDate = courseScheduleDao.findClassGroupCourseSchedulesWithDate(coursePostPoneInfo.getClassGroupIds(), coursePostPoneInfo.getPauseDate());

        if(CollectionUtils.isEmpty(classGroupCourseSchedulesWithDate)){
            throw new BizException("选择的班级上在指定日期之后不存在需要调整的课程");
        }
		Date date = new Date();
		List<Long> courseIds = classGroupCourseSchedulesWithDate.stream().map(e -> e.getId()).collect(Collectors.toList());
		//获取修改日志
		List<CourseScheduleModifyLog> modifyLogs = getModifyLogs(courseIds,classGroupCourseSchedulesWithDate,user.getId(),date);

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(coursePostPoneInfo.getRecoveryDate());

		Set<String> holidayDays = new HashSet<>();

        Integer tenantId = TenantContextHolder.getTenantId();
        if (coursePostPoneInfo.isSkipHoliday()) {
            String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING,tenantId);
            if(StringUtils.isNotBlank(holidaySetting)){
                holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
            }
		}

        classGroupCourseSchedulesWithDate.sort(Comparator.comparing(CourseSchedule::getStartClassTime));

        //保存修改记录
        for (int i=0;i<classGroupCourseSchedulesWithDate.size();i++) {
            CourseSchedule courseSchedule=classGroupCourseSchedulesWithDate.get(i);
            if(Objects.nonNull(courseSchedule.getNewCourseId())){
            	throw new BizException("选择的班级中存在已合并课程，无法进行顺延操作");
			}
            if(Integer.valueOf(1).equals(courseSchedule.getIsLock())){
				throw new BizException("选择的班级中部分课程已被锁定，无法进行顺延操作");
			}
            calendar.setTime(courseSchedule.getClassDate());
            calendar.add(Calendar.DATE, betweenDays);

            if (coursePostPoneInfo.isSkipHoliday() && holidayDays.contains(DateUtil.format(calendar.getTime(), "yyyy-MM-dd"))) {
                betweenDays=betweenDays+7;
                calendar.add(Calendar.DATE, betweenDays);
                i=i-1;
                continue;
            }
            classGroupCourseSchedulesWithDate.get(i).setClassDate(calendar.getTime());
        }
        checkNewCourseSchedules(classGroupCourseSchedulesWithDate,false,false);
        courseScheduleDao.batchUpdate(classGroupCourseSchedulesWithDate);
		courseScheduleModifyLogService.pushSchoolCourseChange(classGroupCourseSchedulesWithDate.get(0));
        //保存操作日志
		saveModifyLog(modifyLogs,courseIds);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchAppendVipGroupCourses(VipGroupCourseAdjustInfoDto vipGroupCourseAdjustInfo) {
        if (Objects.isNull(vipGroupCourseAdjustInfo.getVipGroupId())) {
            throw new BizException("请指定小课");
        }
        if (Objects.isNull(vipGroupCourseAdjustInfo.getCourseCreateStartTime())) {
            throw new BizException("请指定排课起始时间;");
        }
        if (CollectionUtils.isEmpty(vipGroupCourseAdjustInfo.getCourseTimes())) {
            throw new BizException("请指定排课周期");
        }
        if (Objects.isNull(vipGroupCourseAdjustInfo.getSingleClassPrice())) {
            throw new BizException("请指定课程单价");
        }
        if (Objects.isNull(vipGroupCourseAdjustInfo.getCourseCount())) {
            throw new BizException("请指定排课次数");
        }
        Date courseCreateStartTime = vipGroupCourseAdjustInfo.getCourseCreateStartTime();
        Date now = new Date();
        Date today = DateUtil.stringToDate(DateUtil.dateToString(now, DateUtil.ISO_EXPANDED_DATE_FORMAT),
                DateUtil.ISO_EXPANDED_DATE_FORMAT);
        if (courseCreateStartTime.before(today)) {
            throw new BizException("排课起始时间不能小于当前时间");
        }

        VipGroup vipGroup = vipGroupDao.get(vipGroupCourseAdjustInfo.getVipGroupId().longValue());
        if (Objects.isNull(vipGroup)) {
            throw new BizException("指定课程不存在");
        }

        if (vipGroup.getStatus().equals(VipGroupStatusEnum.APPLYING)) {
            throw new BizException("报名中的课程不能加课");
        }

        ClassGroup classGroup = classGroupDao.findByGroupAndType(vipGroupCourseAdjustInfo.getVipGroupId().toString(), GroupType.VIP.getCode());
        if (Objects.isNull(classGroup)) {
            throw new BizException("对应班级不存在");
        }

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(courseCreateStartTime);

		Set<String> holidayDays = new HashSet<>();

		if (vipGroupCourseAdjustInfo.isHoliday()) {
            Integer tenantId = TenantContextHolder.getTenantId();
            String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING,tenantId);
            if(StringUtils.isNotBlank(holidaySetting)){
                holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
            }
		}

        List<CourseSchedule> courseSchedules = new ArrayList<>();

        while (true) {
            if (vipGroupCourseAdjustInfo.isHoliday() && holidayDays.contains(DateUtil.format(calendar.getTime(), "yyyy-MM-dd"))) {
                calendar.add(Calendar.DATE, 1);
                continue;
            }
            int i = calendar.get(Calendar.DAY_OF_WEEK);
            if (i == 1) {
                i = 7;
            } else {
                i = --i;
            }
            for (CourseTimeDto courseTime : vipGroupCourseAdjustInfo.getCourseTimes()) {
                if (Objects.isNull(courseTime.getDayOfWeek())) {
                    throw new BizException("排课循环周期错误");
                }
                if (courseTime.getDayOfWeek() < 1 || courseTime.getDayOfWeek() > 7) {
                    throw new BizException("排课循环周期错误");
                }
                if (courseTime.getDayOfWeek() == i) {
                    String dateYmdStr = DateUtil.dateToString(calendar.getTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
                    dateYmdStr = dateYmdStr + " " + courseTime.getStartClassTime();
                    Date courseStartTime = DateUtil.stringToDate(dateYmdStr, "yyyy-MM-dd HH:mm");
                    Date courseEndTime = DateUtil.addMinutes(courseStartTime, vipGroup.getSingleClassMinutes());
                    CourseSchedule courseSchedule = new CourseSchedule();
                    courseSchedule.setGroupType(GroupType.VIP);
                    courseSchedule.setMusicGroupId(vipGroup.getId().toString());
                    courseSchedule.setClassGroupId(classGroup.getId());
                    courseSchedule.setClassDate(courseStartTime);
                    courseSchedule.setStartClassTime(courseStartTime);
                    courseSchedule.setEndClassTime(courseEndTime);
                    courseSchedule.setName(vipGroup.getName());

					if (!DateUtil.isSameDay(courseSchedule.getStartClassTime(), courseSchedule.getEndClassTime())) {
						throw new BizException("上课时间不允许跨天，请重新调整上课时间");
					}

                    if(vipGroupCourseAdjustInfo.getTeachMode().equals(TeachModeEnum.OFFLINE)){
						courseSchedule.setSchoolId(vipGroup.getTeacherSchoolId());
					}
                    courseSchedule.setStatus(CourseStatusEnum.NOT_START);
                    courseSchedule.setTeachMode(vipGroupCourseAdjustInfo.getTeachMode());
                    courseSchedules.add(courseSchedule);
                }
                if (courseSchedules.size() == vipGroupCourseAdjustInfo.getCourseCount()) {
                    break;
                }
            }
            if (courseSchedules.size() == vipGroupCourseAdjustInfo.getCourseCount()) {
                break;
            }
            calendar.add(Calendar.DATE, 1);
        }
//		checkNewCourseSchedules(courseSchedules,false);
//		courseScheduleDao.batchUpdate(courseSchedules);
        VipGroupApplyDto vipGroupApplyDto = new VipGroupApplyDto();
        VipGroupApplyBaseInfoDto vipGroupApplyBaseInfoDto = new VipGroupApplyBaseInfoDto();
        vipGroupApplyBaseInfoDto.setId(vipGroup.getId());
        vipGroupApplyDto.setVipGroupApplyBaseInfo(vipGroupApplyBaseInfoDto);
        vipGroupApplyDto.setCourseSchedules(courseSchedules);
        vipGroupApplyDto.setSingleClassPrice(vipGroupCourseAdjustInfo.getSingleClassPrice());
        vipGroupService.appendVipGroupCourseSchedules(vipGroupApplyDto);
        classGroupService.updateClassGroupInfo(classGroup.getId());
//        vipGroupService.checkVipCourseIsInScore(vipGroupCourseAdjustInfo.getVipGroupId().longValue());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public HttpResponseResult courseAdjust(List<CourseSchedule> newCourseSchedules, Boolean allowZeroSalary) {
    	//校验分部线上课排课人数上线
		this.checkOnlineCategory(newCourseSchedules);

        Date date = new Date();
		// 课程信息处理
		List<Long> courseScheduleIds = newCourseSchedules.stream().map(CourseSchedule::getId).distinct().collect(Collectors.toList());

        //已结算的课不能调整
        /*List<Map<Long, Integer>> courseSettlementMaps = courseScheduleTeacherSalaryDao.checkCoursesIsSettlement(courseScheduleIds);
        if(courseSettlementMaps!=null && courseSettlementMaps.size()>0){
        	throw new BizException("调整的课程中存在已结算的课程");
        }*/

        //查询数据库中的课程信息
        List<CourseSchedule> oldCourseScheduleList = courseScheduleDao.findByCourseScheduleIds(courseScheduleIds);
        //记录操作日志
		SysUser sysUser = sysUserService.getUser();
		//获取修改日志
		List<CourseScheduleModifyLog> modifyLogs = getModifyLogs(courseScheduleIds,oldCourseScheduleList,sysUser.getId(),date);

		Map<Long, CourseSchedule> oldCourseScheduleMap = oldCourseScheduleList.stream().collect(Collectors.toMap(CourseSchedule::getId, c -> c));

		List<CourseSchedule> liveCourseSchedules = JSON.parseArray(JSON.toJSONString(oldCourseScheduleList),CourseSchedule.class);
        for(CourseSchedule newCourseSchedule : newCourseSchedules){

        	//获取数据库中的记录
            CourseSchedule oldCourseSchedule = oldCourseScheduleMap.get(newCourseSchedule.getId());

            if (Objects.isNull(newCourseSchedule.getGroupType())) {
                newCourseSchedule.setGroupType(oldCourseSchedule.getGroupType());
            }
			newCourseSchedule.setValidEndTime(oldCourseSchedule.getValidEndTime());
			newCourseSchedule.setValidStartTime(oldCourseSchedule.getValidStartTime());

            //课程是否已结算
			this.checkCourseIsSettlement(oldCourseSchedule);

			if (newCourseSchedule.getGroupType() != GroupType.COMM) {
				// 学生已点名不能调整（请假可以调整）
				List<StudentAttendance> studentAttendanceList = studentAttendanceDao.findByCourseId(oldCourseSchedule.getId());

				for (StudentAttendance studentAttendance : studentAttendanceList) {
					if (studentAttendance.getStatus() != StudentAttendanceStatusEnum.LEAVE
						|| (studentAttendance.getStatus() == StudentAttendanceStatusEnum.LEAVE
							&&StringUtils.isBlank(studentAttendance.getRemark()))) {
						throw new BizException("{}[{}] {}-{}课程已点名", oldCourseSchedule.getName(), oldCourseSchedule.getId(), DateUtil.dateToString(
								oldCourseSchedule.getStartClassTime(), DateUtil.EXPANDED_DATE_TIME_FORMAT), DateUtil.dateToString(
								oldCourseSchedule.getEndClassTime(), DateUtil.EXPANDED_DATE_TIME_FORMAT));
					}
				}
			}

			if (VIP.equals(newCourseSchedule.getGroupType())){
				VipGroup vipGroup = vipGroupDao.get(Long.valueOf(oldCourseSchedule.getMusicGroupId()));
				if(Objects.isNull(vipGroup)){
					throw new BizException("课程组信息不存在");
				}
				VipGroupActivity vipGroupActivity = vipGroupActivityDao.get(vipGroup.getVipGroupActivityId());
				Integer allowOnlineToOffline = 0;
				if(Objects.nonNull(vipGroupActivity)){
					allowOnlineToOffline = vipGroupActivity.getAllowOnlineToOffline();
					ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(vipGroup.getId(), "VIP",null);
					if(activityUserMapper != null){
						if(vipGroup.getId().equals(activityUserMapper.getGiveVipGroupId())){
							allowOnlineToOffline = vipGroupActivity.getGiveAllowOnlineToOffline();
						}
					}
				}
				boolean onlineToOffline = TeachModeEnum.OFFLINE.equals(newCourseSchedule.getTeachMode())&&TeachModeEnum.ONLINE.equals(oldCourseSchedule.getTeachMode());
				if(onlineToOffline&&Objects.nonNull(vipGroupActivity)&&(allowOnlineToOffline==0||allowOnlineToOffline==3)){
					throw new BizException("此VIP课活动不支持线上课调整为线下课");
				}
				boolean offlineToOnline = TeachModeEnum.ONLINE.equals(newCourseSchedule.getTeachMode())&&TeachModeEnum.OFFLINE.equals(oldCourseSchedule.getTeachMode());
				if(offlineToOnline&&Objects.nonNull(vipGroupActivity)&&(allowOnlineToOffline==2||allowOnlineToOffline==3)){
					throw new BizException("此VIP课活动不支持线下课调整为线上课");
				}
			}

            if(!CollectionUtils.isEmpty(newCourseSchedule.getTeachingTeacherIdList())){
                if (newCourseSchedule.getTeachingTeacherIdList().contains(newCourseSchedule.getActualTeacherId())){
                    throw new BizException("主教和助教不可重复");
                }
            }

            if(newCourseSchedule.getStatus() == oldCourseSchedule.getStatus() && newCourseSchedule.getStatus() == CourseStatusEnum.OVER){
            	throw new BizException("已结束的课程需要调整课程状态");
            }

            /*if (oldCourseSchedule.getType().equals(CourseSchedule.CourseScheduleType.VIP)) {
                VipGroup vipGroup = vipGroupDao.get(Long.valueOf(oldCourseSchedule.getMusicGroupId()));
                if(Objects.isNull(vipGroup)){
                    throw new BizException("课程不存在");
                }
                Date endClassTime = DateUtil.addMinutes(newCourseSchedule.getStartClassTime(), vipGroup.getSingleClassMinutes());
                newCourseSchedule.setEndClassTime(endClassTime);
            }*/

            if(Objects.isNull(newCourseSchedule.getMusicGroupId())){
                newCourseSchedule.setMusicGroupId(oldCourseSchedule.getMusicGroupId());
            }

            if (Objects.isNull(newCourseSchedule.getTeachMode())) {
                newCourseSchedule.setTeachMode(oldCourseSchedule.getTeachMode());
            }

			if (newCourseSchedule.getTeachMode() != oldCourseSchedule.getTeachMode()
					&& newCourseSchedule.getTeachMode() == TeachModeEnum.OFFLINE
					&& Objects.isNull(newCourseSchedule.getSchoolId())) {
				throw new BizException("请设置教学点");
			}

            if (Objects.isNull(newCourseSchedule.getSchoolId())) {
                newCourseSchedule.setSchoolId(oldCourseSchedule.getSchoolId());
            }
			if (newCourseSchedule.getTeachMode() != oldCourseSchedule.getTeachMode() && newCourseSchedule.getTeachMode() == TeachModeEnum.ONLINE) {
				newCourseSchedule.setSchoolId(null);
			}

            if (Objects.isNull(newCourseSchedule.getActualTeacherId())) {
                newCourseSchedule.setActualTeacherId(oldCourseSchedule.getActualTeacherId());
            }
            if (Objects.isNull(newCourseSchedule.getType())) {
                newCourseSchedule.setType(oldCourseSchedule.getType());
            }
            if(Objects.isNull(newCourseSchedule.getStartClassTime())){
                newCourseSchedule.setStartClassTime(oldCourseSchedule.getStartClassTime());
            }
            if(newCourseSchedule.getEndClassTime() == null){
            	int minutes = DateUtil.minutesBetween(oldCourseSchedule.getStartClassTime(), oldCourseSchedule.getEndClassTime());
            	newCourseSchedule.setEndClassTime(DateUtil.addMinutes(newCourseSchedule.getStartClassTime(), minutes));
            }

            //课程开始时间必须在06：00之后
            if(DateUtil.getHour(newCourseSchedule.getStartClassTime()) < 6){
            	throw new BizException("课程开始时间必须在06:00之后，请重新调整上课时间");
            }

            //课程结束时间必须在21：00之前
            if(DateUtil.getHour(newCourseSchedule.getEndClassTime()) > 20 && DateUtil.getMinute(newCourseSchedule.getEndClassTime()) > 0){
				throw new BizException("课程结束时间不能超过21:00，请重新调整上课时间");
            }

            //判断上课时间是否跨天
			if (!DateUtil.isSameDay(newCourseSchedule.getStartClassTime(), newCourseSchedule.getEndClassTime())) {
				throw new BizException("上课时间不允许跨天，请重新调整上课时间");
			}
            if(Objects.isNull(newCourseSchedule.getClassGroupId())){
                newCourseSchedule.setClassGroupId(oldCourseSchedule.getClassGroupId());
            }
            if(Objects.isNull(newCourseSchedule.getClassDate())){
                newCourseSchedule.setClassDate(oldCourseSchedule.getClassDate());
            }
            if (newCourseSchedule.getStartClassTime().after(date)) {
                newCourseSchedule.setStatus(CourseStatusEnum.NOT_START);
				newCourseSchedule.setServiceProvider(null);
            }
            if (newCourseSchedule.getStartClassTime().before(date) && newCourseSchedule.getEndClassTime().after(date)) {
                newCourseSchedule.setStatus(CourseStatusEnum.UNDERWAY);
            }
            if (newCourseSchedule.getEndClassTime().before(date)) {
                newCourseSchedule.setStatus(CourseStatusEnum.OVER);
            }
			//调整后的课程截止日期不能超过学期时间
			if(newCourseSchedule.getValidEndTime() != null){
				if(newCourseSchedule.getValidEndTime().compareTo(newCourseSchedule.getClassDate()) < 0){
					throw new BizException("操作失败：不允许跨学期调整");
				}
			}
			if(newCourseSchedule.getValidStartTime() != null){
				if(newCourseSchedule.getValidStartTime().compareTo(newCourseSchedule.getClassDate()) > 0){
					throw new BizException("操作失败：不允许跨学期调整");
				}
			}

			if(newCourseSchedule.getStartClassTime().compareTo(date)<0){
				throw new BizException("调整无效,不能调整至已结束");
			}
			/*if (newCourseSchedule.getType() == CourseScheduleType.PRACTICE) {
				PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(newCourseSchedule.getMusicGroupId()));
				if (practiceGroup.getType() != TRIAL) {
					if (DateUtil.minutesBetween(newCourseSchedule.getStartClassTime(), practiceGroup.getCoursesStartDate()) > 0) {
						throw new BizException("调整失败: 调整时间不得早于开课时间");
					}
					if (!DateUtil.isSameDay(practiceGroup.getCoursesExpireDate(), newCourseSchedule.getEndClassTime())&&DateUtil.minutesBetween(practiceGroup.getCoursesExpireDate(), newCourseSchedule.getEndClassTime()) > 0) {
						throw new BizException("调整失败: 截止时间超过课程有效期");
					}
				}
			}*/
        }

		Set<String> musicGroupIds = oldCourseScheduleList.stream().filter(c -> MUSIC.equals(c.getGroupType())).map(CourseSchedule::getMusicGroupId).collect(Collectors.toSet());
        if(!CollectionUtils.isEmpty(musicGroupIds)){
			List<MusicGroup> musicGroups = musicGroupDao.getMusicGroupByIds(new ArrayList<>(musicGroupIds));
			if(CollectionUtils.isEmpty(musicGroups)){
				throw new BizException("调整失败：乐团信息异常");
			}
			long errMusicGroupNum = musicGroups.stream().filter(m -> !MusicGroupStatusEnum.PROGRESS.equals(m.getStatus())).count();
			if(errMusicGroupNum>0){
				throw new BizException("调整失败：调整的课程所在乐团处于非进行中");
			}
		}

		//课程冲突检测
        checkNewCourseSchedules(newCourseSchedules, false,false);

		Map<Long, List<TeacherAttendance>> teacherAttendanceMap = new HashMap<Long, List<TeacherAttendance>>();
		// 查询老师的考勤
		List<TeacherAttendance> teacherAttendances = teacherAttendanceDao.findTeacherIdByCourseSchedule(courseScheduleIds);

		for (TeacherAttendance ta : teacherAttendances) {
			List<TeacherAttendance> tas = teacherAttendanceMap.get(ta.getCourseScheduleId());
			if (tas == null) {
				tas = new ArrayList<TeacherAttendance>();
			}
			tas.add(ta);
			teacherAttendanceMap.put(ta.getCourseScheduleId(), tas);
		}

		Long courseScheduleId = null;
		List<TeacherAttendance> insertTeacherAttendanceList = new ArrayList<TeacherAttendance>();
		List<CourseScheduleTeacherSalary> insertCourseScheduleTeacherSalaryList = new ArrayList<CourseScheduleTeacherSalary>();

		// 获取课程对应教学点补贴
//		List<Map<Long, BigDecimal>> courseSubsidyByCourses = courseScheduleTeacherSalaryDao.findCourseSubsidyByCourses(courseScheduleIds);
//		Map<Integer, BigDecimal> courseSubsidyMap = MapUtil.convertIntegerMap(courseSubsidyByCourses);

		// 课程对应乐团结算方式集合
//		List<Map<String, String>> musicGroupSettlementTypeByCourse = courseScheduleTeacherSalaryDao.findMusicGroupSettlementTypeByCourse(courseScheduleIds);
//		Map<String, String> musicGroupSettlementsMap = MapUtil.convertMybatisMap(musicGroupSettlementTypeByCourse);

		Map<Long,Map<Integer,CourseScheduleTeacherSalary>> courseScheduleTeacherSalaryMap = new HashMap<Long, Map<Integer,CourseScheduleTeacherSalary>>();
		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseScheduleIds);

		for (CourseScheduleTeacherSalary ts : courseScheduleTeacherSalaryList) {
			Map<Integer, CourseScheduleTeacherSalary> map = courseScheduleTeacherSalaryMap.get(ts.getCourseScheduleId());
			if (map == null) {
				map = new HashMap<Integer, CourseScheduleTeacherSalary>();
			}
			map.put(ts.getUserId(), ts);
			courseScheduleTeacherSalaryMap.put(ts.getCourseScheduleId(), map);
		}
		List<CourseScheduleStudentPayment> insertCourseScheduleStudentPaymentList = new ArrayList<CourseScheduleStudentPayment>();

		for (CourseSchedule newCourseSchedule : newCourseSchedules) {
			courseScheduleId = newCourseSchedule.getId();
			CourseSchedule oldCourseSchedule = oldCourseScheduleMap.get(courseScheduleId);

			// 修改了主教/助教/课程状态，需修改考勤记录
			List<TeacherAttendance> teacherAttendanceList = teacherAttendanceMap.get(courseScheduleId);

			List<Integer> oldTeacherIdList = new ArrayList<Integer>();
			if(teacherAttendanceList != null){
				oldTeacherIdList = teacherAttendanceList.stream().map(TeacherAttendance::getTeacherId).distinct().collect(Collectors.toList());
			}
			List<Integer> newTeacherIdList = new ArrayList<>();
            newTeacherIdList.addAll(newCourseSchedule.getTeachingTeacherIdList());
            newTeacherIdList.add(newCourseSchedule.getActualTeacherId());

			// 清理原来考勤
			teacherAttendanceDao.deletebyCourseScheduleId(courseScheduleId);

			// 新增考勤
			for (Integer teacherId : newTeacherIdList) {
				TeacherAttendance ta = new TeacherAttendance();
				ta.setClassGroupId(newCourseSchedule.getClassGroupId());
				ta.setCourseScheduleId(courseScheduleId);
				ta.setCreateTime(date);
				ta.setGroupType(newCourseSchedule.getGroupType());
				ta.setMusicGroupId(newCourseSchedule.getMusicGroupId());
				ta.setTeacherId(teacherId);
				insertTeacherAttendanceList.add(ta);
			}

			if(newCourseSchedule.getStatus() == CourseStatusEnum.NOT_START && newCourseSchedule.getGroupType() == GroupType.MUSIC){
				//查询新生(之前排课没有这个人，后来新进来的学生)
				List<ClassGroupStudentMapper> newStudentList = classGroupStudentMapperDao.queryNewStudentListByCourseScheduleId(courseScheduleId);
				if(newStudentList.size() > 0){
					//生成courseScheduleStudentPayment记录
					for(ClassGroupStudentMapper cgsm : newStudentList){
						CourseScheduleStudentPayment sp = new CourseScheduleStudentPayment();
						sp.setClassGroupId(newCourseSchedule.getClassGroupId());
						sp.setCourseScheduleId(courseScheduleId);
						sp.setCreateTime(date);
						sp.setExpectPrice(new BigDecimal(0));
						sp.setGroupType(GroupType.MUSIC);
						sp.setMusicGroupId(newCourseSchedule.getMusicGroupId());
						sp.setUpdateTime(date);
						sp.setUserId(cgsm.getUserId());

						insertCourseScheduleStudentPaymentList.add(sp);
					}
				}
			}

			VipGroup vipGroup = vipGroupDao.findByCourseSchedule(courseScheduleId);

			if(Objects.nonNull(vipGroup)
					&&!VipGroupStatusEnum.PROGRESS.equals(vipGroup.getStatus())
					&&!VipGroupStatusEnum.FINISHED.equals(vipGroup.getStatus())){
				throw new BizException("非进行中或完成的课程组不允许进行此操作");
			}

			if(Objects.nonNull(vipGroup) && newCourseSchedule.getStartClassTime().after(date)){
				if(newCourseSchedule.getEndClassTime().compareTo(vipGroup.getCoursesExpireDate())>0){
					vipGroup.setCoursesExpireDate(newCourseSchedule.getEndClassTime());
				}
				vipGroup.setStatus(VipGroupStatusEnum.PROGRESS);
				vipGroup.setUpdateTime(date);
				vipGroupDao.update(vipGroup);
			}

			if (newCourseSchedule.getStatus() == CourseStatusEnum.NOT_START) {
				//如果已结束修改成了未开始，需要更新学生考勤记录
				studentAttendanceDao.deleteStudentAttendancesByCourse(courseScheduleId);
			}

			if ((newCourseSchedule.getStatus() != oldCourseSchedule.getStatus() && newCourseSchedule.getStatus() == CourseStatusEnum.NOT_START)) {

				// 清理课程作业
				courseHomeworkService.delHomwworkByCourseScheduleId(courseScheduleId);

				//删除申述
				courseScheduleComplaintsDao.delByCourseScheduleId(courseScheduleId);

				//删除评论
				courseScheduleReviewDao.delByCourseScheduleId(courseScheduleId);

				List<CourseScheduleStudentPayment> courseScheduleStudentPayments = courseScheduleStudentPaymentDao.findByCourseSchedule(courseScheduleId);
				if(!CollectionUtils.isEmpty(courseScheduleStudentPayments)){
					List<Integer> studentIds = courseScheduleStudentPayments.stream().map(CourseScheduleStudentPayment::getUserId).collect(Collectors.toList());
					studentServeService.updateExercisesSituation(oldCourseSchedule.getMusicGroupId(), oldCourseSchedule.getClassDate(), studentIds, oldCourseSchedule.getActualTeacherId());
				}
			}

			// 计算课程时长
			int oldMinutes = DateUtil.minutesBetween(oldCourseSchedule.getStartClassTime(), oldCourseSchedule.getEndClassTime());
			int newMinutes = DateUtil.minutesBetween(newCourseSchedule.getStartClassTime(), newCourseSchedule.getEndClassTime());

			// 如果修改了老师、课程类型、课程时长、教学模式，需要修改课酬
			if (!ListUtil.isEquals(oldTeacherIdList, newTeacherIdList)
					|| !newCourseSchedule.getActualTeacherId().equals(oldCourseSchedule.getActualTeacherId())
					|| newCourseSchedule.getType() != oldCourseSchedule.getType()
					|| newCourseSchedule.getTeachMode() != oldCourseSchedule.getTeachMode() || oldMinutes != newMinutes) {

//				Map<Integer, Teacher> teacherMap = null;
				Map<Integer, Map<CourseScheduleType, TeacherDefaultMusicGroupSalary>> musicGroupSalaryMap = new HashMap<Integer, Map<CourseScheduleType, TeacherDefaultMusicGroupSalary>>();
				Map<Integer, Map<Integer, TeacherDefaultVipGroupSalary>> vipGroupSalaryMap = new HashMap<Integer, Map<Integer, TeacherDefaultVipGroupSalary>>();

				if (newCourseSchedule.getGroupType() == GroupType.MUSIC) {
					// 所有教师列表
//					List<Teacher> teachers = teacherDao.findByTeacherIds(newTeacherIdList);
//					teacherMap = teachers.stream().collect(Collectors.toMap(Teacher::getId, teacher -> teacher));

					// 所有老师默认乐团课酬
					List<TeacherDefaultMusicGroupSalary> teacherDefaultMusicGroupSalaries = teacherDefaultMusicGroupSalaryDao.findByTeacher(newTeacherIdList);
					for (TeacherDefaultMusicGroupSalary tdms : teacherDefaultMusicGroupSalaries) {
						Map<CourseScheduleType, TeacherDefaultMusicGroupSalary> map = musicGroupSalaryMap.get(tdms.getUserId());
						if (map == null) {
							map = new HashMap<CourseSchedule.CourseScheduleType, TeacherDefaultMusicGroupSalary>();
						}
						map.put(tdms.getCourseScheduleType(), tdms);
						musicGroupSalaryMap.put(tdms.getUserId(), map);
					}
				} else if (newCourseSchedule.getGroupType() == GroupType.VIP || newCourseSchedule.getGroupType() == GroupType.LIVE) {
					// 所有老师默认vip课酬
					List<TeacherDefaultVipGroupSalary> teacherDefaultVipGroupSalaries = teacherDefaultVipGroupSalaryDao.queryByUserIdList(newTeacherIdList);
					for (TeacherDefaultVipGroupSalary tdvs : teacherDefaultVipGroupSalaries) {
						Map<Integer, TeacherDefaultVipGroupSalary> map = vipGroupSalaryMap.get(tdvs.getUserId());
						if (map == null) {
							map = new HashMap<Integer, TeacherDefaultVipGroupSalary>();
						}
						map.put(tdvs.getVipGroupCategoryId(), tdvs);
						vipGroupSalaryMap.put(tdvs.getUserId(), map);
					}
				}

				// 删除课酬
				List<Long> courseScheduleIdList = new ArrayList<Long>();
				courseScheduleIdList.add(courseScheduleId);

				List<CourseScheduleTeacherSalary> oldCourseScheduleTeacherSalaryList = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseScheduleIds);
				Map<Long, List<CourseScheduleTeacherSalary>> courseSalaryListMap = new HashMap<>();
				CourseScheduleTeacherSalary mainTeacherSalary = null;
				if(!CollectionUtils.isEmpty(oldCourseScheduleTeacherSalaryList)){
					courseSalaryListMap = oldCourseScheduleTeacherSalaryList.stream().collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId));
					List<CourseScheduleTeacherSalary> mainTeacherSalarys = oldCourseScheduleTeacherSalaryList.stream().filter(t -> TeachTypeEnum.BISHOP.equals(t.getTeacherRole())).collect(Collectors.toList());
					if(!CollectionUtils.isEmpty(mainTeacherSalarys)){
						mainTeacherSalary = mainTeacherSalarys.get(0);
					}
				}

				courseScheduleTeacherSalaryDao.batchDeleteByCourseScheduleIds(courseScheduleIdList);

				for (Integer teacherId : newTeacherIdList) {
					CourseScheduleTeacherSalary ts = new CourseScheduleTeacherSalary();
					ts.setClassGroupId(newCourseSchedule.getClassGroupId());
					ts.setCourseScheduleId(courseScheduleId);
					ts.setCreateTime(date);
					ts.setGroupType(newCourseSchedule.getGroupType());
					ts.setMusicGroupId(newCourseSchedule.getMusicGroupId());
					if (teacherId == newCourseSchedule.getActualTeacherId().intValue()) {
						ts.setTeacherRole(TeachTypeEnum.BISHOP);
					} else {
						ts.setTeacherRole(TeachTypeEnum.TEACHING);
					}
					ts.setUpdateTime(date);
					ts.setUserId(teacherId);

					if(TeachTypeEnum.BISHOP.equals(ts.getTeacherRole())&&Objects.nonNull(mainTeacherSalary)&&!mainTeacherSalary.getEnableChangeSalary()){
						ts.setExpectSalary(mainTeacherSalary.getExpectSalary());
						ts.setSubsidy(mainTeacherSalary.getSubsidy());
						ts.setEnableChangeSalary(mainTeacherSalary.getEnableChangeSalary());
					}else{
						// 重新生成课酬
						if (newCourseSchedule.getGroupType() == GroupType.MUSIC) {
							courseScheduleTeacherSalaryService.createMusicGroupCourseTeacherSalary(null, newCourseSchedule, ts);
						} else if (newCourseSchedule.getGroupType() == GroupType.VIP || newCourseSchedule.getGroupType() == LIVE) {
							Map<String, BigDecimal> salaryMap = vipGroupService.countVipGroupPredictFee1(vipGroup, teacherId);
							BigDecimal teacherSalary = salaryMap.get("offlineTeacherSalary");
							//是否是赠送课程
							ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(vipGroup.getId(),vipGroup.getGroupType(),null);
							if(activityUserMapper != null){
								if(vipGroup.getId().equals(activityUserMapper.getGiveVipGroupId())){
									teacherSalary = salaryMap.get("giveOfflineTeacherSalary");
								}
							}
							ts.setExpectSalary(teacherSalary);
						} else if (newCourseSchedule.getGroupType() == PRACTICE) {
							SysConfig practiceCourseSalaryConfig = sysConfigService.findByParamName(SysConfigService.PRACTICE_COURSE_SALARY);
							BigDecimal freeTeacherSalary = new BigDecimal(practiceCourseSalaryConfig.getParanValue());

							List<TeacherDefaultPracticeGroupSalary> teacherDefaultPracticeGroupSalaryList = teacherDefaultPracticeGroupSalaryDao
									.queryByUserId(teacherId);
							List<CourseScheduleTeacherSalary> teacherSalaries = courseSalaryListMap.get(ts.getCourseScheduleId());
							if(!CollectionUtils.isEmpty(teacherSalaries)&&freeTeacherSalary.compareTo(teacherSalaries.get(0).getExpectSalary())==0){
								ts.setExpectSalary(freeTeacherSalary);
							}else{
								if (teacherDefaultPracticeGroupSalaryList != null && teacherDefaultPracticeGroupSalaryList.size() > 0) {
									ts.setExpectSalary(teacherDefaultPracticeGroupSalaryList.get(0).getMainTeacherSalary());
								} else {
									throw new BizException("请设置老师 的网管课课酬");
								}
							}
							PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(newCourseSchedule.getMusicGroupId()));
							if(practiceGroup != null){
								Integer vipGroupActivityId = practiceGroup.getVipGroupActivityId();
								if(vipGroupActivityId != null){
									ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(practiceGroup.getId(),"PRACTICE",null);
									if(activityUserMapper != null){
										Map<String, BigDecimal> map = practiceGroupService.countPracticeGroupPredictFee1(practiceGroup, ts.getUserId());
										if(practiceGroup.getId().equals(activityUserMapper.getPracticeGroupId())){
											ts.setExpectSalary(map.get("onlineTeacherSalary"));
										}else {
											ts.setExpectSalary(map.get("giveTeacherDefaultSalary"));
										}
									}
								}
							}
						}
					}

					if(ts.getExpectSalary() == null){
						throw new BizException("请设置老师的课酬");
					}
					if(!allowZeroSalary&&BigDecimal.ZERO.compareTo(ts.getExpectSalary())==0){
						TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
						return BaseController.failed(HttpStatus.PARTIAL_CONTENT,"当前课程课酬预计为0,是否继续");
					}
					insertCourseScheduleTeacherSalaryList.add(ts);
				}
			}
		}

		if (insertTeacherAttendanceList.size() > 0) {
			teacherAttendanceDao.batchInsert(insertTeacherAttendanceList);
		}

		if (insertCourseScheduleTeacherSalaryList.size() > 0) {
			courseScheduleTeacherSalaryDao.batchInsert(insertCourseScheduleTeacherSalaryList);
		}

		if(newCourseSchedules.size() > 0){
            for (CourseSchedule course : newCourseSchedules) {
                course.setLiveRemind(0);
            }
            courseScheduleDao.batchUpdate(newCourseSchedules);
            List<Integer> classGroupIds = newCourseSchedules.stream().filter(courseSchedule -> Objects.nonNull(courseSchedule.getClassGroupId())).map(CourseSchedule::getClassGroupId).collect(Collectors.toList());
            if(!CollectionUtils.isEmpty(classGroupIds)){
                classGroupService.updateClassGroupsCourseInfo(classGroupIds);
            }
			courseScheduleModifyLogService.pushSchoolCourseChange(newCourseSchedules.get(0));
        }

		if(insertCourseScheduleStudentPaymentList.size() > 0){
			courseScheduleStudentPaymentDao.batchInsert(insertCourseScheduleStudentPaymentList);
		}

		//保存日志调整信息
		saveModifyLog(modifyLogs,courseScheduleIds);

		courseEventSource.courseStatusChange(courseScheduleIds);

		imUserFriendService.refreshCourseImUserFriend(courseScheduleId);
		// 推送
		try {
			Set<Integer> teacherIds = newCourseSchedules.stream().map(e -> e.getActualTeacherId()).collect(Collectors.toSet());
			if (teacherIds == null) {
				teacherIds = new HashSet<>();
			}
			teacherIds.addAll(oldCourseScheduleList.stream().map(e -> e.getActualTeacherId()).collect(Collectors.toSet()));
			Map<Integer, String> map = new HashMap<>(teacherIds.size());
			teacherIds.removeAll(Collections.singleton(null));
			teacherIds.forEach(e -> {
				map.put(e, e.toString());
			});
			if (map != null && map.size() > 0) {
				sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_COURSE_SCHEDULE_CHANGE_RESULT, map, null, 0, "7",
						"TEACHER");
			}

            // 推送消息到学校端
		} catch (Exception e) {
			e.printStackTrace();
		}
		// 直播课消息推送
		this.sendChatRoomMessage(liveCourseSchedules);
		return BaseController.succeed();
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void courseSwap(Long courseScheduleId1, Long courseScheduleId2) {
        if (courseScheduleId1.equals(courseScheduleId2)) {
            throw new BizException("请选择不同的课程");
        }
        CourseSchedule courseSchedule1 = courseScheduleDao.get(courseScheduleId1);
        CourseSchedule courseSchedule2 = courseScheduleDao.get(courseScheduleId2);

        if (Objects.isNull(courseSchedule1) || Objects.isNull(courseSchedule2)) {
            throw new BizException("未找到对应课程");
        }

        if(!courseSchedule1.getGroupType().equals(courseSchedule2.getGroupType())
            ||!courseSchedule1.getMusicGroupId().equals(courseSchedule2.getMusicGroupId())){
            throw new BizException("不能跨乐团进行课时交换");
        }

        int course1CourseMinutes = DateUtil.minutesBetween(courseSchedule1.getStartClassTime(), courseSchedule1.getEndClassTime());
        int course2CourseMinutes = DateUtil.minutesBetween(courseSchedule2.getStartClassTime(), courseSchedule2.getEndClassTime());

        CourseSchedule temp=new CourseSchedule();
        BeanUtils.copyProperties(courseSchedule1, temp);

        courseSchedule1.setClassDate(courseSchedule2.getClassDate());
        courseSchedule1.setStartClassTime(courseSchedule2.getStartClassTime());
        courseSchedule1.setEndClassTime(DateUtil.addMinutes(courseSchedule1.getStartClassTime(),course1CourseMinutes));

        courseSchedule2.setClassDate(temp.getClassDate());
        courseSchedule2.setStartClassTime(temp.getStartClassTime());
        courseSchedule2.setEndClassTime(DateUtil.addMinutes(courseSchedule2.getStartClassTime(),course2CourseMinutes));

        List<CourseSchedule> courseSchedules = new ArrayList<>();
        courseSchedules.add(courseSchedule1);
        courseSchedules.add(courseSchedule2);
        courseAdjust(courseSchedules, true);
    }


    @Override
    public List<CourseSchedule> findClassGroupNoStartCourseSchedules(List<Integer> classGroupIds, GroupType groupType) {
        return courseScheduleDao.findClassGroupNoStartCourseSchedules(classGroupIds, groupType);
    }


    @Override
    public Object findStudentCourseScheduleRecords(StudentCourseScheduleRecordQueryInfo queryInfo) {
        SysUser user = sysUserService.getUser();
        queryInfo.setUserId(user.getId().longValue());
        PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

        if (queryInfo.getIsAttend() == YesOrNoEnum.YES) {
            List<StudentCourseScheduleRecordDto> notStartRecords = courseScheduleDao.findStudentCourseScheduleNotStartRecords(user.getId().longValue(),
                    queryInfo.getDate());
            if (notStartRecords != null && notStartRecords.size() > 0) {
				notStartRecords.forEach(e -> {
                    e.setRestOfClass(notStartRecords.size());
                });
            }

            // 直播房间号ID
            List<String> liveRoomIds = notStartRecords.stream().map(e -> e.getLiveRoomId()).filter(Objects::nonNull).collect(Collectors.toList());

            Map<String, List<ImLiveRoomVideoVo>> imLiveRoomVideoVoMap = getLiveRecord(liveRoomIds);
            notStartRecords.forEach(e -> {
                if (imLiveRoomVideoVoMap != null && imLiveRoomVideoVoMap.containsKey(e.getLiveRoomId())) {
                    e.setHasVideoRecord(true);
                } else {
                    e.setHasVideoRecord(false);
                }
            });


            pageInfo.setRows(notStartRecords);
            return pageInfo;
        }

        Map<String, Object> params = new HashMap<String, Object>();
        if (queryInfo.getType() != null && queryInfo.getType().equals("MUSICGROUP")) {
            queryInfo.setType("MUSIC");
        }
        MapUtil.populateMap(params, queryInfo);

        List<StudentCourseScheduleRecordDto> dataList = null;
        int count = courseScheduleDao.countStudentCourseScheduleRecords(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = courseScheduleDao.findStudentCourseScheduleRecords(params);

            // 直播房间号ID
            List<String> liveRoomIds = dataList.stream().map(e -> e.getLiveRoomId()).filter(Objects::nonNull).collect(Collectors.toList());

            Map<String, List<ImLiveRoomVideoVo>> imLiveRoomVideoVoMap = getLiveRecord(liveRoomIds);
            dataList.forEach(e -> {
                if (imLiveRoomVideoVoMap != null && imLiveRoomVideoVoMap.containsKey(e.getLiveRoomId())) {
                    e.setHasVideoRecord(true);
                } else {
                    e.setHasVideoRecord(false);
                }
            });

        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void courseScheduleCommplaint(CourseScheduleComplaints courseScheduleComplaints) {
        SysUser user = sysUserService.getUser();
        if (Objects.isNull(courseScheduleComplaints.getCourseScheduleId())) {
            throw new BizException("请指定需要评价的课程");
        }

        if (StringUtils.isBlank(courseScheduleComplaints.getReason())) {
            throw new BizException("请说明写下您的评价");
        }

        if (StringUtils.length(courseScheduleComplaints.getReason()) > 150) {
            throw new BizException("字符长度超限");
        }

        int settlementTimes = courseScheduleTeacherSalaryDao.checkCourseIsSettlement(courseScheduleComplaints.getCourseScheduleId().intValue());
		if(settlementTimes <= 0){
			settlementTimes = courseScheduleDao.checkCourseIsSettlementReport(courseScheduleComplaints.getCourseScheduleId());
		}
        if (settlementTimes > 0) {
            throw new BizException("该课程已结算");
        }

        courseScheduleComplaints.setUserId(user.getId());

        CourseScheduleComplaints byUserAndCourse = courseScheduleComplaintsDao.findByUserAndCourse(courseScheduleComplaints.getUserId().longValue(), courseScheduleComplaints.getCourseScheduleId());
        if (byUserAndCourse != null) {
            throw new BizException("您已经对该课程做出了评价!");
        }

        CourseScheduleDto courseSchedules = courseScheduleDao.getCourseSchedules(courseScheduleComplaints.getCourseScheduleId());
        Date now = new Date();
        if (courseSchedules.getEndClassTime().after(now)) {
            throw new BizException("该课程还未结束");
        }
        int i = DateUtil.daysBetween(courseSchedules.getEndClassTime(), now);
        if (i > Integer.parseInt(sysConfigDao.findConfigValue(SysConfigService.VIP_APPEAL_DAYS_RANGE))) {
            throw new BizException("该课程已超过可申诉时间范围!");
        }
        courseScheduleComplaints.setUserId(user.getId());
        courseScheduleComplaints.setStatus(AuditStatusEnum.ING);
        if (courseScheduleComplaints.getScore() > 2) {
            courseScheduleComplaints.setStatus(AuditStatusEnum.PASS);
        }
        courseScheduleComplaintsDao.insert(courseScheduleComplaints);
        if (courseScheduleComplaints.getScore() > 2) {
            return;
        }
        //学员提起申诉通知
        CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleComplaints.getCourseScheduleId());
		GroupType groupType = courseSchedule.getGroupType();
		String musicGroupId = courseSchedule.getMusicGroupId();
		Integer targetUserId = null;
		switch (groupType) {
			case VIP:
				targetUserId = vipGroupDao.get(Long.parseLong(musicGroupId)).getEducationalTeacherId();
				break;
			case MUSIC:
				targetUserId = musicGroupDao.get(musicGroupId).getEducationalTeacherId();
				break;
			case PRACTICE:
				targetUserId = practiceGroupDao.get(Long.parseLong(musicGroupId)).getEducationalTeacherId();
				break;
			case COMM:
				targetUserId = coursesGroupDao.get(Long.parseLong(musicGroupId)).getEducationalTeacherId();
				break;
		}
		Map<String, Long> memo = new HashMap<>(1);
		memo.put("courseScheduleComplaintsId", courseScheduleComplaints.getId());
		sendSeoMessageSource.sendSeoMessage(user.getOrganId(),new SysUserRoleEnum[]{ORGAN_MANAGER},
				JSONObject.toJSONString(memo),targetUserId,MessageTypeEnum.BACKSTAGE_STUDENT_APPEAL,courseSchedule.getName(), user.getUsername());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void courseScheduleCommplaintAudit(Long id, Double teacherLiabilityRatio, Double studentLiabilityRatio, AuditStatusEnum status) {

        if (status == AuditStatusEnum.ING) {
            throw new BizException("审批状态不正确");
        }

        CourseScheduleComplaints courseScheduleComplaints = courseScheduleComplaintsDao.get(id);
        if (Objects.isNull(courseScheduleComplaints)) {
            throw new BizException("投诉记录不存在");
        }
        courseScheduleComplaints.setTeacherLiabilityRatio(teacherLiabilityRatio);
        courseScheduleComplaints.setStudentLiabilityRatio(studentLiabilityRatio);
        courseScheduleComplaints.setStatus(status);
        courseScheduleComplaintsDao.update(courseScheduleComplaints);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<Long> updateCourseScheduleToOverStatus() {

        Date date = new Date();
        //查询所有已上课乐团
        List<String> musicGroupIdList = courseScheduleDao.queryStartedCourseMusicGroupIdList(GroupType.MUSIC.getCode());

        //自动更新课程状态至“已结束”
        List<CourseSchedule> list = courseScheduleDao.queryFinishedWithNoUpdateStatus();

        List<CourseSchedule> updateList = new ArrayList<CourseSchedule>();
        List<Integer> classGroupIds = new ArrayList<Integer>();
        List<Long> courseScheduleIdList = new ArrayList<Long>();
        List<Long> courseIds = new ArrayList<Long>();

        for (CourseSchedule courseSchedule : list) {
            courseSchedule.setStatus(CourseStatusEnum.OVER);
            courseSchedule.setUpdateTime(date);
            updateList.add(courseSchedule);

            if (!classGroupIds.contains(courseSchedule.getClassGroupId())) {
                classGroupIds.add(courseSchedule.getClassGroupId());
            }

            if (courseSchedule.getGroupType() == GroupType.VIP) {
                courseScheduleIdList.add(courseSchedule.getId());
            }

            if(courseSchedule.getGroupType() == GroupType.MUSIC){
            	if(!musicGroupIdList.contains(courseSchedule.getMusicGroupId())){
            		//乐团第一次上课
            		MusicGroup musicGroup = musicGroupDao.get(courseSchedule.getMusicGroupId());
            		if(musicGroup != null){
            			musicGroup.setFirstCourseStartTime(courseSchedule.getClassDate());
            			musicGroup.setGroupMemberNum(courseScheduleStudentPaymentDao.countStudentNum(courseSchedule.getId().intValue()));

            			musicGroupDao.update(musicGroup);
            		}

            		musicGroupIdList.add(courseSchedule.getMusicGroupId());
            	}
            }
        }

		if (updateList.size() > 0) {
            courseScheduleDao.batchUpdate(updateList);
			//更新用户网管、vip课老师
			studentTeacherMapperService.updateStudentTeacherMapper(updateList);
			try {
				tenantAssetsInfoService.deductAmount(updateList);
			}catch (Exception e){
				businessLogger.info("deductAmount  >>>>>>>>>",e.getCause());
			}
			courseIds = updateList.stream().map(CourseSchedule::getId).collect(Collectors.toList());
			List<CourseSchedule> beMergeCourses = courseScheduleDao.getBeMergeCourseWithMainCourseIds(courseIds);
			if(!CollectionUtils.isEmpty(beMergeCourses)){
				beMergeCourses.forEach(c->c.setStatus(CourseStatusEnum.OVER));
				courseScheduleDao.batchUpdate(beMergeCourses);
			}
		}
        // 更新班级已上课次
        if (classGroupIds.size() > 0) {
            classGroupService.batchIncreaseClassTimes(classGroupIds);
        }
		try {
			practiceGroupService.practiceGroupRenewRemind();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return courseIds;
    }

    @Override
	@Transactional(rollbackFor = Exception.class)
	public List<Long> updateCourseScheduleToUnderway() {
    	Date date = new Date();
    	List<CourseSchedule> list = courseScheduleDao.queryUnderwayWithNoUpdateStatus();

        List<CourseSchedule> updateList = new ArrayList<CourseSchedule>();

        for (CourseSchedule courseSchedule : list) {
            courseSchedule.setStatus(CourseStatusEnum.UNDERWAY);
            courseSchedule.setUpdateTime(date);
            updateList.add(courseSchedule);
        }

        if (updateList.size() > 0) {
            courseScheduleDao.batchUpdate(updateList);
		}
		return updateList.stream().map(e->e.getId()).collect(Collectors.toList());
	}

	@Override
    public PageInfo queryTeacherClassCourseSchedule(CourseScheduleQueryInfo queryInfo) {
        SysUser user = sysUserService.getUser();
        if(!queryInfo.getFindTotal()){
            queryInfo.setUserId(user.getId().longValue());
        }
        PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

        if (Objects.isNull(queryInfo.getDate())) {
            queryInfo.setDate(new Date());
        }

        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<TeacherClassCourseSchudeleDto> dataList = null;
        int count = 0;
        if (Objects.isNull(queryInfo.getOnlyHistory()) || queryInfo.getOnlyHistory() == 0) {
            count = courseScheduleDao.countTeacherClassCourseSchedule(params);
        } else {
            count = courseScheduleDao.countTeacherHistoryClassCourseSchedule(params);
        }
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            if (Objects.isNull(queryInfo.getOnlyHistory()) || queryInfo.getOnlyHistory() == 0) {
                dataList = courseScheduleDao.queryTeacherClassCourseSchedule(params);
            } else {
                dataList = courseScheduleDao.queryTeacherHistoryClassCourseSchedule(params);
            }
			//获取老师姓名
			List<Integer> teacherIds = dataList.stream().map(e -> e.getActualTeacherId()).distinct().collect(Collectors.toList());
			List<SimpleUserDto> teacherDtos = teacherDao.getUsersSimpleInfo(teacherIds);
			Map<Integer, String> teacherDtoMap = teacherDtos.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, SimpleUserDto::getRealName));
			//获取学校名称
			List<Integer> schoolIds = dataList.stream().map(e -> e.getSchoolId()).distinct().collect(Collectors.toList());
			List<School> schoolList = schoolDao.getSchools(schoolIds);
			Map<Integer, String> schoolMap = schoolList.stream().collect(Collectors.toMap(School::getId, School::getName));
			//获取班级信息
			List<Integer> classGroupIds = dataList.stream().map(e -> e.getClassGroupId()).distinct().collect(Collectors.toList());
			List<ClassGroup> classGroups = classGroupDao.findByClassGroupIds(classGroupIds, null);
			Map<Integer, ClassGroup> classGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, Function.identity()));
			dataList.forEach(e -> {
				ClassGroup classGroup = classGroupMap.get(e.getClassGroupId());
				if (classGroup != null) {
					e.setTotalClassTimes(classGroup.getTotalClassTimes());
				}
				e.setSchoolName(schoolMap.get(e.getSchoolId()));
				e.setActualTeacherName(teacherDtoMap.get(e.getActualTeacherId()));
                e.setCurrentClassTimes(courseScheduleDao.countCurrentTimes(e.getClassGroupId().intValue(),new Date()));
                e.setTotalClassTimes(courseScheduleDao.countCurrentCourseTime(e.getCourseScheduleId()));
            });
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

	@Override
	public PageInfo<TeacherClassCourseSchudeleDto> queryTeacherVipClassCourseSchedule(CourseScheduleQueryInfo queryInfo) {
		SysUser user = sysUserService.getUser();
		if (null == user.getTenantId()) {
			throw new BizException("用户机构信息异常");
		}
		if(!queryInfo.getFindTotal()){
			queryInfo.setUserId(user.getId().longValue());
		}
		PageInfo<TeacherClassCourseSchudeleDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

		if (Objects.isNull(queryInfo.getDate())) {
			queryInfo.setDate(new Date());
		}

        // 直播课判断时间，提前30分钟将课程改为进行中
        Integer studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());

		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
        params.put("notStartCourseTime", studentRemindTime);
        if (queryInfo.getClassGroupId() != null) {
            ClassGroup classGroup = classGroupDao.get(queryInfo.getClassGroupId().intValue());
            params.put("groupType", classGroup.getGroupType().name());
        }
		//获取老师可以查看的班级课程
		if(queryInfo.getClassGroupId() != null && queryInfo.getUserId() != null){
			List<Long> courseIds = courseScheduleDao.queryTeacherCanViewClassCourseSchedule(queryInfo.getClassGroupId(),queryInfo.getUserId());
			if(CollectionUtils.isEmpty(courseIds)){
				return pageInfo;
			}
			params.put("courseIds", courseIds);
		}
		List<TeacherClassCourseSchudeleDto> dataList = null;
		int count;
		if (Objects.isNull(queryInfo.getOnlyHistory()) || queryInfo.getOnlyHistory() == 0) {
			count = courseScheduleDao.countTeacherClassCourseSchedule(params);
		} else {
			count = courseScheduleDao.countTeacherHistoryClassCourseSchedule(params);
		}
		if (count > 0) {
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			if (Objects.isNull(queryInfo.getOnlyHistory()) || queryInfo.getOnlyHistory() == 0) {
				dataList = courseScheduleDao.queryTeacherClassCourseSchedule(params);
			} else {
				dataList = courseScheduleDao.queryTeacherHistoryClassCourseSchedule(params);
			}
			//获取老师姓名
			List<Integer> teacherIds = dataList.stream().map(e -> e.getActualTeacherId()).distinct().collect(Collectors.toList());
			List<SimpleUserDto> teacherDtos = teacherDao.getUsersSimpleInfo(teacherIds);
			Map<Integer, String> teacherDtoMap = teacherDtos.stream().collect(Collectors.toMap(SimpleUserDto::getUserId, SimpleUserDto::getRealName));
			//获取学校名称
			List<Integer> schoolIds = dataList.stream().map(e -> e.getSchoolId()).distinct().collect(Collectors.toList());
			List<School> schoolList = schoolDao.getSchools(schoolIds);
			Map<Integer, String> schoolMap = schoolList.stream().collect(Collectors.toMap(School::getId, School::getName));
			//获取班级信息
			List<Integer> classGroupIds = dataList.stream().map(e -> e.getClassGroupId()).distinct().collect(Collectors.toList());
			List<ClassGroup> classGroups = classGroupDao.findByClassGroupIds(classGroupIds, null);
			Map<Integer, ClassGroup> classGroupMap = classGroups.stream().collect(Collectors.toMap(ClassGroup::getId, Function.identity()));

			List<Map<Integer, Integer>> classGroupStudentNumMaps = classGroupStudentMapperDao.countClassGroupsStudentNum(classGroupIds, ClassGroupStudentStatusEnum.NORMAL);
			Map<Integer, Long> classGroupStudentNumMap = MapUtil.convertIntegerMap(classGroupStudentNumMaps);

			List<Long> allCourseScheduleIds = dataList.stream().map(TeacherClassCourseSchudeleDto::getCourseScheduleId).collect(Collectors.toList());
			String tenantConfigValue = sysTenantConfigService.getTenantConfigValue(SysConfigService.ADVANCE_LEAVE_HOURS,user.getTenantId());
			Integer advanceLeaveHours = 0;
			if(StringUtils.isNotEmpty(tenantConfigValue)){
				advanceLeaveHours = Integer.parseInt(tenantConfigValue);
			}
			List<Map<Long, Integer>> courseLeaveStudentNumMaps = studentAttendanceDao.countCourseLeaveStudentNumWithFourHoursAgo(allCourseScheduleIds, advanceLeaveHours);
			Map<Long,Long> courseLeaveStudentNumMap = MapUtil.convertIntegerMap((courseLeaveStudentNumMaps));

			List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseIdsAndTeacherRole(allCourseScheduleIds, null);
			Map<Long, Map<TeachTypeEnum, Set<String>>> cttnMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(courseScheduleTeacherSalaries)){
				cttnMap = courseScheduleTeacherSalaries.stream().collect(Collectors.groupingBy(CourseScheduleTeacherSalary::getCourseScheduleId, Collectors.groupingBy(CourseScheduleTeacherSalary::getTeacherRole, Collectors.mapping(CourseScheduleTeacherSalary::getUserName, Collectors.toSet()))));
			}

			// 课程组信息
			Map<String, ImLiveBroadcastRoomDto> liveRoomMap = getCourseLiveRoomMap(dataList);

			ImLiveBroadcastRoomDto liveRoom;
			for (TeacherClassCourseSchudeleDto e : dataList) {
				ClassGroup classGroup = classGroupMap.get(e.getClassGroupId());
				if (classGroup != null) {
					e.setTotalClassTimes(classGroup.getTotalClassTimes());
				}
				e.setSchoolName(schoolMap.get(e.getSchoolId()));
				e.setActualTeacherName(teacherDtoMap.get(e.getActualTeacherId()));
				e.setCurrentClassTimes(courseScheduleDao.countCurrentTimes(e.getClassGroupId(),new Date()));
				e.setTotalClassTimes(courseScheduleDao.countCurrentCourseTime(e.getCourseScheduleId()));

				Long leaveStudentNum = courseLeaveStudentNumMap.get(e.getCourseScheduleId());
				Long normalStudentNum = classGroupStudentNumMap.get(e.getClassGroupId());
				if(Objects.nonNull(leaveStudentNum)&&Objects.nonNull(normalStudentNum)&&leaveStudentNum.intValue()==normalStudentNum.intValue()){
					e.setEnableAdjustInToday(1);
				}

				if(cttnMap.containsKey(e.getCourseScheduleId())&&cttnMap.get(e.getCourseScheduleId()).containsKey(TeachTypeEnum.TEACHING)){
					e.setTeachingNames(StringUtils.join(cttnMap.get(e.getCourseScheduleId()).get(TeachTypeEnum.TEACHING), ","));
				}

				// 直播课直播间配置
				if (liveRoomMap.containsKey(e.getMusicGroupId())) {
					liveRoom = liveRoomMap.get(e.getMusicGroupId());
					e.subjectId(liveRoom.getSubjectId()).setOs(liveRoom.getOs());
				}
			}
		}
		if (count == 0) {
			dataList = new ArrayList<>();
		}


        // 直播房间号ID
        List<String> liveRoomIds = dataList.stream().map(e -> e.getLiveRoomId()).filter(Objects::nonNull).collect(Collectors.toList());

        Map<String, List<ImLiveRoomVideoVo>> imLiveRoomVideoVoMap = getLiveRecord(liveRoomIds);
        dataList.forEach(e -> {
            if (imLiveRoomVideoVoMap != null && imLiveRoomVideoVoMap.containsKey(e.getLiveRoomId())) {
                e.setHasVideoRecord(true);
            } else {
                e.setHasVideoRecord(false);
            }
        });

        for (TeacherClassCourseSchudeleDto record : dataList) {
            if (CourseScheduleType.LIVE.equals(record.getType()) && CourseStatusEnum.NOT_START.equals(record.getStatus())) {
                Date date = DateUtil.addMinutes(new Date(), studentRemindTime);
                if (date.after(record.getStartClassTime())) {
                    record.setStatus(CourseStatusEnum.UNDERWAY);
                }
            }
        }

        pageInfo.setRows(dataList);
		return pageInfo;
	}

	/**
	 * 课程直播配置
	 * @param dataList List<TeacherClassCourseSchudeleDto>
	 * @return Map<String, ImLiveBroadcastRoomDto>
	 */
	public Map<String, ImLiveBroadcastRoomDto> getCourseLiveRoomMap(List<TeacherClassCourseSchudeleDto> dataList) {

		Map<String, ImLiveBroadcastRoomDto> liveRoomMap = Maps.newHashMap();
		{
			Map<Long, String> collect = dataList.stream()
					.filter(x -> StringUtils.isNotEmpty(x.getMusicGroupId()))
					.filter(x -> x.getType().equals(CourseScheduleType.LIVE))
					.collect(Collectors.toMap(TeacherClassCourseSchudeleDto::getId, TeacherClassCourseSchudeleDto::getMusicGroupId, (o, n) -> n));

			if (MapUtils.isNotEmpty(collect)) {

				List<Long> liveGroupIds = collect.values().stream().map(Long::parseLong).distinct().collect(Collectors.toList());

				liveRoomMap = vipGroupDao.getVipGroupByIds(liveGroupIds).stream()
						.filter(x -> Objects.nonNull(x.getLiveConfigJson()))
						.collect(Collectors.toMap(x -> String.valueOf(x.getId()),
								x -> JSON.parseObject(x.getLiveConfigJson(), ImLiveBroadcastRoomDto.class), (o, n) -> n));

			}

		}
		return liveRoomMap;
	}

	@Override
    public CourseAttendanceDetailHeadInfoDto findCourseAttendanceDetailHeadInfo(Long courseScheduleId) {
        if (Objects.isNull(courseScheduleId)) {
            throw new BizException("请指定课程");
        }
        CourseAttendanceDetailHeadInfoDto courseAttendanceDetailHeadInfoDto = courseScheduleDao.findByCourse(courseScheduleId);
        courseAttendanceDetailHeadInfoDto.setLatestAttendanceTime(studentAttendanceDao.findLatestAttendanceDate(courseScheduleId));
        courseAttendanceDetailHeadInfoDto.setStudentNum(studentAttendanceDao.getStudentNumByStatus(courseScheduleId, StudentAttendanceStatusEnum.NORMAL));
        courseAttendanceDetailHeadInfoDto.setLeaveStudentNum(studentAttendanceDao.getStudentNumByStatus(courseScheduleId, StudentAttendanceStatusEnum.LEAVE));
        courseAttendanceDetailHeadInfoDto.setTruantStudentNum(studentAttendanceDao.getStudentNumByStatus(courseScheduleId, StudentAttendanceStatusEnum.TRUANT));
        courseAttendanceDetailHeadInfoDto.setLateStudentNum(studentAttendanceDao.getStudentNumByStatus(courseScheduleId, StudentAttendanceStatusEnum.LATE));
        return courseAttendanceDetailHeadInfoDto;
    }

    @Override
    public void checkMusicGroupClassCourse(String musicGroupId) {
        if (Objects.isNull(musicGroupId)) {
            throw new BizException("请指定乐团");
        }
        List<Map<Integer, Long>> maps = courseScheduleDao.countClassCourseNumByMusicGroup(musicGroupId);
        Map<Integer, Long> classCourseNumMap = MapUtil.convertIntegerMap(maps);
        List<Integer> collect = classCourseNumMap
                .entrySet()
                .stream()
                .filter(integerLongEntry -> integerLongEntry.getValue() <= 0)
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(collect)) {
            List<String> classGroupNamesByClassGroups = classGroupDao.findClassGroupNamesByClassGroups(collect);
            String classNamesStr = StringUtils.join(classGroupNamesByClassGroups, ",");
            throw new BizException(classNamesStr + "班级未排课");
        }
    }

    @Override
    public Map<Integer, Integer> findCourseScheduleCurrentTimes(List<Integer> courseSchedules) {
        List<Map<Integer, Integer>> courseScheduleCurrentTimes = courseScheduleDao.findCourseScheduleCurrentTimes(courseSchedules);
        return MapUtil.convertIntegerMap(courseScheduleCurrentTimes);
    }

    @Override
    public List<CourseSchedule> findNoStartCoursesByClassGroupId(Integer classGroupId) {
        return courseScheduleDao.findCoursesByClassGroupId(classGroupId, CourseStatusEnum.NOT_START);
    }


    @Override
    public void studentSignInRemind() {
        List<Mapper> mapperList = courseScheduleDao.queryStudentNotStartCourseScheduleIn90Mins();
        if (mapperList != null && mapperList.size() > 0) {
            for (Mapper mapper : mapperList) {
                Map<Integer, String> receivers = new HashMap<Integer, String>();
                receivers.put((Integer) mapper.getKey(), mapper.getKey() + "");

                sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_ATTEND_COURSE_MESSAGE, receivers, null, 0, "2","STUDENT", mapper.getValue());
            }
        }
    }

    @Override
    public void teacherCoursesScheduleRemind() {
        List<Mapper> mapperList = courseScheduleDao.queryTeacherCoursesTimesOfTomorrow();
        for (Mapper mapper : mapperList) {
            Integer userId = (Integer) mapper.getKey();
            String teacherName = (String) mapper.getValue();

            Map<Integer, String> receivers = new HashMap<Integer, String>();
            receivers.put(userId, userId + "");

            sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_HAVE_COURSE_MESSAGE,
                    receivers, null, 0, "2","TEACHER", teacherName);
        }
    }

    @Override
    public List<CourseSchedule> findCourseScheduleByClassGroup(Integer classGroupId) {
        if (Objects.isNull(classGroupId)) {
            throw new BizException("请指定班级");
        }
        ClassGroup classGroup = classGroupDao.get(classGroupId);
        if (Objects.isNull(classGroup)) {
            throw new BizException("指定的班级不存在");
        }
        List<Integer> classGroupIds = new ArrayList<>();
        classGroupIds.add(classGroupId);
        if (classGroup.getType().equals(ClassGroupTypeEnum.MIX)) {
            List<ClassGroup> mixClassChildClassGroup = classGroupDao.findMixClassChildClassGroup(classGroupId);
            List<Integer> childClassGroupIds = mixClassChildClassGroup.stream()
                    .map(ClassGroup::getId)
                    .collect(Collectors.toList());
            classGroupIds.addAll(childClassGroupIds);
        }
        return courseScheduleDao.findByClassGroups(classGroupIds);
    }

    @Override
    public Object findVipGroupCourseSchedules(VipGroupQueryInfo queryInfo) {
        PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());

        if (Objects.isNull(queryInfo.getVipGroupId())) {
            throw new BizException("请指定课程");
        }

        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        VipGroup vipGroup = vipGroupDao.get(queryInfo.getVipGroupId());
        if (Objects.isNull(vipGroup)) {
            throw new BizException("指定的课程不存在");
        }

        List<CourseSchedule> dataList;
        if (vipGroup.getStatus().equals(VipGroupStatusEnum.APPLYING)) {
            dataList = JSON.parseArray(vipGroup.getCourseSchedulesJson(), CourseSchedule.class);
        } else {
            dataList = courseScheduleDao.findGroupCourseSchedules(vipGroup.getId().toString(), queryInfo.getGroupType());
        }
		if(!CollectionUtils.isEmpty(dataList)){
			if(StringUtils.isNotEmpty(queryInfo.getSearch())){
				dataList = dataList.stream().filter(e -> (e.getId() != null && e.getId().toString().equals(queryInfo.getSearch())) || e.getName().contains(queryInfo.getSearch())).collect(Collectors.toList());
			}
			if(StringUtils.isNotEmpty(queryInfo.getClassDate())){
				dataList = dataList.stream().filter(e->DateUtil.dateToString(e.getClassDate(),DateUtil.ISO_EXPANDED_DATE_FORMAT).equals(queryInfo.getClassDate())).collect(Collectors.toList());
			}
		}

        int count = dataList.size();
        if (count > 0) {
            pageInfo.setTotal(count);
            if (vipGroup.getStatus().equals(VipGroupStatusEnum.APPLYING)) {
                dataList = dataList.stream().skip(pageInfo.getOffset()).limit(pageInfo.getLimit()).collect(Collectors.toList());
                dataList.forEach(courseSchedule -> {
                    courseSchedule.setIsSettlement(0);
                });
            } else {
                List<Long> courseScheduleIds = dataList.stream()
                        .map(courseSchedule -> courseSchedule.getId())
                        .distinct().collect(Collectors.toList());
                List<Map<Long, Integer>> courseSettlementMaps = courseScheduleTeacherSalaryDao.checkCoursesIsSettlement(courseScheduleIds);
                Map<Long, Long> courseSettlementMap = MapUtil.convertIntegerMap(courseSettlementMaps);
                Map<Integer, Long> studentNumCourseMap = MapUtil.convertIntegerMap(studentAttendanceDao.countStudentAttendancesByCourses(courseScheduleIds));
                dataList.forEach(courseSchedule -> {
                    Long isSettlement = courseSettlementMap.get(courseSchedule.getId().longValue());
                    if (Objects.isNull(isSettlement) || isSettlement <= 0) {
                        courseSchedule.setIsSettlement(0);
                    } else {
                        courseSchedule.setIsSettlement(1);
                    }
                    Long studentNum = studentNumCourseMap.get(courseSchedule.getId());
                    if (Objects.nonNull(studentNum)) {
                        courseSchedule.setIsCallNames(studentNum > 0 ? YesOrNoEnum.YES : YesOrNoEnum.NO);
                    } else {
                        courseSchedule.setIsCallNames(YesOrNoEnum.NO);
                    }
                });
            }
        }
        pageInfo.setRows(dataList);
        Map<String, Object> result = new HashMap<>();
        result.put("pageInfo", pageInfo);
        result.put("singleClassMinutes", vipGroup.getSingleClassMinutes());
        return result;
    }

    @Override
    public List<CourseSchedule> createCourses(CourseGenerateDto courseGenerateInfo, boolean excludePastCourse) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(courseGenerateInfo.getCourseCreateStartTime());

        if(Objects.isNull(courseGenerateInfo.getCourseCount())||courseGenerateInfo.getCourseCount()<=0){
        	throw new BizException("可排课次数不足");
		}

        Set<String> holidayDays = new HashSet<>();

		if (courseGenerateInfo.isSkipHoliday()) {
            Integer tenantId = TenantContextHolder.getTenantId();
            String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING,tenantId);
            if(StringUtils.isNotBlank(holidaySetting)){
                holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
            }
		}

        List<CourseSchedule> courseSchedules = new ArrayList<>();
        Date now=new Date();

        while (true) {
			if (courseGenerateInfo.isSkipHoliday() && holidayDays.contains(DateUtil.format(calendar.getTime(), "yyyy-MM-dd"))) {
				calendar.add(Calendar.DATE, 1);
				continue;
			}
            int i = calendar.get(Calendar.DAY_OF_WEEK);
            if (i == 1) {
                i = 7;
            } else {
                i = --i;
            }
            for (CourseTimeDto courseTime : courseGenerateInfo.getCourseTimes()) {
                if (Objects.isNull(courseTime.getDayOfWeek())) {
                    throw new BizException("排课循环周期错误");
                }
                if (courseTime.getDayOfWeek() < 1 || courseTime.getDayOfWeek() > 7) {
                    throw new BizException("排课循环周期错误");
                }
                if (courseTime.getDayOfWeek() == i) {
                    String dateYmdStr = DateUtil.dateToString(calendar.getTime(), DateUtil.ISO_EXPANDED_DATE_FORMAT);
                    if(StringUtils.isBlank(courseTime.getStartClassTime())){
                    	throw new BizException("排课循环周期错误");
					}
                    dateYmdStr = dateYmdStr + " " + courseTime.getStartClassTime();
                    Date courseStartTime = DateUtil.stringToDate(dateYmdStr, "yyyy-MM-dd HH:mm");
                    if(excludePastCourse&&courseStartTime.before(now)){
                    	continue;
					}
                    Date courseEndTime = DateUtil.addMinutes(courseStartTime, courseGenerateInfo.getSingleClassMinutes());
                    if(!DateUtil.isSameDay(courseStartTime, courseEndTime)){
                    	throw new BizException("暂不支持单节课跨天");
					}
                    CourseSchedule courseSchedule = new CourseSchedule();
                    courseSchedule.setClassDate(courseStartTime);
                    courseSchedule.setStartClassTime(courseStartTime);
                    courseSchedule.setEndClassTime(courseEndTime);
                    courseSchedules.add(courseSchedule);
                }
                if (courseSchedules.size() == courseGenerateInfo.getCourseCount()) {
                    break;
                }
            }
            if (courseSchedules.size() == courseGenerateInfo.getCourseCount()) {
                break;
            }
            calendar.add(Calendar.DATE, 1);
        }
        return courseSchedules;
    }

    @Override
    public PageInfo<CourseScheduleEndDto> endFindCourseSchedules(EndCourseScheduleQueryInfo queryInfo) {
        PageInfo<CourseScheduleEndDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>(16);
        MapUtil.populateMap(params, queryInfo);
		Integer tenantId = TenantContextHolder.getTenantId();
		if("COURSE_TIME_ERROR".equals(queryInfo.getSearchType())){
			String courseStartTimeError = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_START_TIME_ERROR, tenantId);
			String courseEndTimeError = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_END_TIME_ERROR, tenantId);
			if(StringUtils.isEmpty(courseStartTimeError) || StringUtils.isEmpty(courseEndTimeError)){
				return pageInfo;
			}
			params.put("courseStartTimeError",courseStartTimeError);
			params.put("courseEndTimeError",courseEndTimeError);
		}

        int count = courseScheduleDao.endCountCourseSchedules(params);
        if(queryInfo.isExport() && count > 50000){
            throw new BizException("数据集太大，不能导出.最大数据集不能超过50000");
        }
		pageInfo.setTotal(count);
		params.put("offset", pageInfo.getOffset());
		List<CourseScheduleEndDto> results = courseScheduleDao.endFindCourseSchedules(params);
        if (!CollectionUtils.isEmpty(results)) {
            List<Long> courseScheduleIds = results.stream().map(CourseScheduleEndDto::getId).collect(Collectors.toList());
            List<Integer> teacherIds = results.stream().filter(t->Objects.nonNull(t.getActualTeacherId())).map(CourseSchedule::getActualTeacherId).collect(Collectors.toList());
			List<Integer> teachingUserIdsList = results.stream()
					.filter(u -> StringUtils.isNotEmpty(u.getTeachingTeacherIds()))
					.flatMap(u -> Arrays.stream(u.getTeachingTeacherIds().split(",")))
					.filter(StringUtils::isNotEmpty)
					.map(Integer::parseInt)
					.distinct()
					.collect(Collectors.toList());
			if(CollectionUtils.isNotEmpty(teachingUserIdsList)){
				teacherIds.addAll(teachingUserIdsList);
			}
			List<Map<Integer, String>> nameIdMaps = teacherDao.queryNameByIdList(teacherIds);
            Map<Integer, String> nameIdMap = MapUtil.convertIntegerMap(nameIdMaps);

			List<Map<Long, Long>> courseMergeCourseNum = courseScheduleDao.countCourseMergeCourseNum(courseScheduleIds);
			Map<Long, Long> courseMergeCourseNumMap = new HashMap<>();
			if(!CollectionUtils.isEmpty(courseMergeCourseNum)){
				courseMergeCourseNumMap = MapUtil.convertIntegerMap(courseMergeCourseNum);
			}
			//获取课程的预计人数
			List<CourseExpectPriceDto> courseScheduleStudentNum = courseScheduleStudentPaymentDao.getCourseScheduleStudentNum(courseScheduleIds);
			Map<Long, CourseExpectPriceDto> courseScheduleStudentNumMap = courseScheduleStudentNum.stream().collect(Collectors.toMap(CourseExpectPriceDto::getCourseId, Function.identity()));
			//获取班级声部名称
            List<Map<Integer, String>> classGroupSubjectNameMaps = classGroupDao.findClassGroupSubjectNameMaps(courseScheduleIds);
            Map<Integer, String> classGroupSubjectNameMap = MapUtil.convertIntegerMap(classGroupSubjectNameMaps);
            Map<Long, String> eduName = null;
            Map<String, MusicGroupBasicDto> cooperationMap = new HashMap<>();
			Map<Long, String> vipGroupTypeMap = new HashMap<>();
            if (queryInfo.isExport()) {
                List<Long> musicCourseIds = results.stream().filter(e -> e.getGroupType() == MUSIC).map(CourseScheduleEndDto::getId).collect(Collectors.toList());
                eduName = new HashMap<>(courseScheduleIds.size());
                if (musicCourseIds != null && musicCourseIds.size() > 0) {
                    eduName.putAll(MapUtil.convertMybatisMap(musicGroupDao.queryEduNameMapByCourseId(musicCourseIds)));
					//获取合作单位，只有groupType是 MUSIC 的才有合作单位
					List<MusicGroupBasicDto> maps = musicGroupDao.findMusicByCourseIds(musicCourseIds);
					for (MusicGroupBasicDto map : maps) {
						cooperationMap.put(map.getId(), map);
					}
                }
                List<Long> vipGroupIds = results.stream().filter(e -> e.getType() == CourseScheduleType.VIP)
						.map(e->Long.parseLong(e.getMusicGroupId())).distinct().collect(Collectors.toList());
				//获取vip课程具体课程类型
				if (CollectionUtils.isNotEmpty(vipGroupIds)){
					vipGroupTypeMap = MapUtil.convertMybatisMap(vipGroupDao.findVipGroupTypeMaps(vipGroupIds));
				}
				List<Long> vipCourseIds = results.stream().filter(e -> e.getGroupType() == VIP).map(CourseScheduleEndDto::getId).collect(Collectors.toList());
                if (vipCourseIds != null && vipCourseIds.size() > 0) {
                    eduName.putAll(MapUtil.convertMybatisMap(vipGroupDao.queryEduNameMapByCourseId(vipCourseIds)));
                }
                List<Long> practiceCourseIds = results.stream().filter(e -> e.getGroupType() == PRACTICE).map(CourseScheduleEndDto::getId).collect(Collectors.toList());
                if (practiceCourseIds != null && practiceCourseIds.size() > 0) {
                    eduName.putAll(MapUtil.convertMybatisMap(practiceGroupDao.queryEduNameMapByCourseId(practiceCourseIds)));
                }
            }

            for (CourseScheduleEndDto result : results) {
				if (queryInfo.isExport()) {
					if(result.getType() == CourseScheduleType.VIP){
						result.setCourseScheduleType(vipGroupTypeMap.get(Long.parseLong(result.getMusicGroupId())));
					}else {
						result.setCourseScheduleType(result.getType().getMsg());
					}
				}
				if(StringUtils.isNotEmpty(result.getTeachingTeacherIds())){
					List<String> ids = Arrays.stream(result.getTeachingTeacherIds().split(",")).collect(Collectors.toList());
					StringBuffer sb = new StringBuffer();
					for (String teachingId : ids) {
						String teachingName = nameIdMap.get(Integer.parseInt(teachingId));
						if(StringUtils.isNotEmpty(teachingName)){
							if(sb.length() > 0){
								sb.append(",");
							}
							sb.append(teachingName);
						}
					}
					result.setTeachingUserNames(sb.toString());
				}
                if (eduName != null) {
                    result.setEducationTeacherName(eduName.get(result.getId()));
                }
                result.setTeacher(null);
                result.setTeacherName(nameIdMap.get(result.getActualTeacherId()));
                result.setSubjectName(classGroupSubjectNameMap.get(result.getId()));
                result.setIsSettlement(Objects.isNull(result.getSettlementTime()) ? 0 : 1);
                if (courseMergeCourseNumMap.containsKey(result.getId())) {
                    result.setBeMerged(courseMergeCourseNumMap.get(result.getId()) > 0 ? true : false);
                }
				CourseExpectPriceDto priceDto = courseScheduleStudentNumMap.get(result.getId());
				if(Objects.nonNull(priceDto)){
					result.setStudentNum(priceDto.getStudentNum());
					result.setExpectPrice(priceDto.getExpectPrice());
				}
                MusicGroupBasicDto musicGroupBasicDto = cooperationMap.get(result.getId().toString());
                if(null != musicGroupBasicDto){
                    result.setCooperationOrgan(musicGroupBasicDto.getCooperationOrganName());
                }
            }
        }
        pageInfo.setRows(results);
        return pageInfo;
    }

    @Override
    public List<CourseScheduleEndDto> exportSuperCourseSchedules(ExportCourseScheduleQueryInfo queryInfo) {
        Map<String, Object> params = new HashMap<>(16);
		if (StringUtils.isNotEmpty(queryInfo.getMonth())) {
			Date month = DateUtil.stringToDate(queryInfo.getMonth(), DateUtil.ISO_YEAR_MONTH_FORMAT);
			params.put("startTime", DateUtil.dateToString(DateUtil.getFirstDayOfMonth(month),DateUtil.ISO_EXPANDED_DATE_FORMAT));
			params.put("endTime", DateUtil.dateToString(DateUtil.getLastDayOfMonth(month),DateUtil.ISO_EXPANDED_DATE_FORMAT));
		}
		if(StringUtils.isNotEmpty(queryInfo.getOrganIds())){
			List<Integer> collect = Arrays.stream(queryInfo.getOrganIds().split(",")).map(Integer::parseInt).collect(Collectors.toList());
			params.put("organIds", collect);
		}
		List<CourseScheduleEndDto> results = courseScheduleDao.exportSuperCourseSchedules(params);
		if(CollectionUtils.isEmpty(results)){
			return new ArrayList<>();
		}
		List<Integer> organIdList = results.stream().map(e -> e.getOrganId()).distinct().collect(Collectors.toList());
		Map<Integer, String> organNameMap = new HashMap<>();
		if(CollectionUtils.isNotEmpty(organIdList)){
			organNameMap = this.getMap("organization","id_","name_",organIdList,queryInfo.getTenantId(),Integer.class,String.class);
		}

		List<Integer> schoolIdList = results.stream().map(e -> e.getSchoolId()).filter(e->e != null).distinct().collect(Collectors.toList());

		Map<Integer, String> schoolameMap = new HashMap<>();
		if(CollectionUtils.isNotEmpty(schoolIdList)){
			schoolameMap = this.getMap("school","id_","name_",schoolIdList,queryInfo.getTenantId(),Integer.class,String.class);
		}

		List<Long> courseScheduleIds = results.stream().map(CourseScheduleEndDto::getId).collect(Collectors.toList());
		List<Integer> teacherIds = results.stream().filter(t->Objects.nonNull(t.getActualTeacherId())).map(CourseSchedule::getActualTeacherId).collect(Collectors.toList());
		List<Integer> teachingUserIdsList = results.stream()
				.filter(u -> StringUtils.isNotEmpty(u.getTeachingTeacherIds()))
				.flatMap(u -> Arrays.stream(u.getTeachingTeacherIds().split(",")))
				.filter(StringUtils::isNotEmpty)
				.map(Integer::parseInt)
				.distinct()
				.collect(Collectors.toList());
		if(CollectionUtils.isNotEmpty(teachingUserIdsList)){
			teacherIds.addAll(teachingUserIdsList);
		}
		List<Map<Integer, String>> nameIdMaps = teacherDao.queryNameByIdList(teacherIds);
		Map<Integer, String> nameIdMap = MapUtil.convertIntegerMap(nameIdMaps);

		List<Map<Long, Long>> courseMergeCourseNum = courseScheduleDao.countCourseMergeCourseNum(courseScheduleIds);
		Map<Long, Long> courseMergeCourseNumMap = new HashMap<>();
		if(!CollectionUtils.isEmpty(courseMergeCourseNum)){
			courseMergeCourseNumMap = MapUtil.convertIntegerMap(courseMergeCourseNum);
		}
		//获取课程的预计人数
		List<CourseExpectPriceDto> courseScheduleStudentNum = courseScheduleStudentPaymentDao.getCourseScheduleStudentNum(courseScheduleIds);
		Map<Long, CourseExpectPriceDto> courseScheduleStudentNumMap = courseScheduleStudentNum.stream().collect(Collectors.toMap(CourseExpectPriceDto::getCourseId, Function.identity()));
		//获取班级声部名称
		List<Map<Integer, String>> classGroupSubjectNameMaps = classGroupDao.findClassGroupSubjectNameMaps(courseScheduleIds);
		Map<Integer, String> classGroupSubjectNameMap = MapUtil.convertIntegerMap(classGroupSubjectNameMaps);
		Map<String, MusicGroupBasicDto> cooperationMap = new HashMap<>();
		Map<Long, String> vipGroupTypeMap = new HashMap<>();
		List<Long> musicCourseIds = results.stream().filter(e -> e.getGroupType() == MUSIC).map(CourseScheduleEndDto::getId).collect(Collectors.toList());
		Map<Long, String> eduName = new HashMap<>(courseScheduleIds.size());
		if (musicCourseIds != null && musicCourseIds.size() > 0) {
			eduName.putAll(MapUtil.convertMybatisMap(musicGroupDao.queryEduNameMapByCourseId(musicCourseIds)));
			//获取合作单位，只有groupType是 MUSIC 的才有合作单位
			List<MusicGroupBasicDto> maps = musicGroupDao.findMusicByCourseIds(musicCourseIds);
			for (MusicGroupBasicDto map : maps) {
				cooperationMap.put(map.getId(), map);
			}
		}
		List<Long> vipGroupIds = results.stream().filter(e -> e.getType() == CourseScheduleType.VIP)
				.map(e->Long.parseLong(e.getMusicGroupId())).distinct().collect(Collectors.toList());
		//获取vip课程具体课程类型
		if (CollectionUtils.isNotEmpty(vipGroupIds)){
			vipGroupTypeMap = MapUtil.convertMybatisMap(vipGroupDao.findVipGroupTypeMaps(vipGroupIds));
		}
		List<Long> vipCourseIds = results.stream().filter(e -> e.getGroupType() == VIP).map(CourseScheduleEndDto::getId).collect(Collectors.toList());
		if (vipCourseIds != null && vipCourseIds.size() > 0) {
			eduName.putAll(MapUtil.convertMybatisMap(vipGroupDao.queryEduNameMapByCourseId(vipCourseIds)));
		}
		List<Long> practiceCourseIds = results.stream().filter(e -> e.getGroupType() == PRACTICE).map(CourseScheduleEndDto::getId).collect(Collectors.toList());
		if (practiceCourseIds != null && practiceCourseIds.size() > 0) {
			eduName.putAll(MapUtil.convertMybatisMap(practiceGroupDao.queryEduNameMapByCourseId(practiceCourseIds)));
		}

		for (CourseScheduleEndDto result : results) {
			if(result.getType() == CourseScheduleType.VIP){
				result.setCourseScheduleType(vipGroupTypeMap.get(Long.parseLong(result.getMusicGroupId())));
			}else {
				result.setCourseScheduleType(result.getType().getMsg());
			}
			result.setOrganName(organNameMap.get(result.getOrganId()));
			result.setSchoolName(schoolameMap.get(result.getSchoolId()));
			if(StringUtils.isNotEmpty(result.getTeachingTeacherIds())){
				List<String> ids = Arrays.stream(result.getTeachingTeacherIds().split(",")).collect(Collectors.toList());
				StringBuffer sb = new StringBuffer();
				for (String teachingId : ids) {
					String teachingName = nameIdMap.get(Integer.parseInt(teachingId));
					if(StringUtils.isNotEmpty(teachingName)){
						if(sb.length() > 0){
							sb.append(",");
						}
						sb.append(teachingName);
					}
				}
				result.setTeachingUserNames(sb.toString());
			}
			if (eduName != null) {
				result.setEducationTeacherName(eduName.get(result.getId()));
			}
			result.setTeacher(null);
			result.setTeacherName(nameIdMap.get(result.getActualTeacherId()));
			result.setSubjectName(classGroupSubjectNameMap.get(result.getId()));
			result.setIsSettlement(Objects.isNull(result.getSettlementTime()) ? 0 : 1);
			if (courseMergeCourseNumMap.containsKey(result.getId())) {
				result.setBeMerged(courseMergeCourseNumMap.get(result.getId()) > 0 ? true : false);
			}
			CourseExpectPriceDto priceDto = courseScheduleStudentNumMap.get(result.getId());
			if(Objects.nonNull(priceDto)){
				result.setStudentNum(priceDto.getStudentNum());
				result.setExpectPrice(priceDto.getExpectPrice());
			}

			MusicGroupBasicDto musicGroupBasicDto = cooperationMap.get(result.getId().toString());
			if(null != musicGroupBasicDto){
				result.setCooperationOrgan(musicGroupBasicDto.getCooperationOrganName());
			}
		}
		return results;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void practiceCourseAdjust(CourseSchedule courseSchedule) {
        SysUser user = sysUserService.getUser();
        Long courseScheduleId = courseSchedule.getId();
        if(Objects.isNull(courseScheduleId)){
            throw new BizException("请指定课程");
        }
        if(Objects.isNull(courseSchedule.getStartClassTime())){
            throw new BizException("请指定上课时间");
        }
        CourseSchedule oldCourseSchedule = courseScheduleDao.get(courseScheduleId);
        if(Objects.isNull(oldCourseSchedule)){
            throw new BizException("课程不存在");
        }
        if(!oldCourseSchedule.getGroupType().equals(PRACTICE)){
            throw new BizException("请选择网管课");
        }
        //已点名的不允许调整
        List<StudentAttendance> studentAttendances = studentAttendanceDao.findByCourseId(courseScheduleId);
        if(studentAttendances != null && studentAttendances.size() > 0){
            throw new BizException("调整失败: 课程已点名");
        }
        //已结算的课酬不能调整
        CourseScheduleTeacherSalary teacherSalary = courseScheduleTeacherSalaryDao.queryByCourseScheduleIdAndUserId(oldCourseSchedule.getId(), oldCourseSchedule.getActualTeacherId());
        if(teacherSalary != null && teacherSalary.getSettlementTime() != null){
            throw new BizException("调整失败: 课程已结算");
        }
		Date date = new Date();
        //获取调整日志
		List<Long> courseIds = new ArrayList<>();
		courseIds.add(courseScheduleId);
		List<CourseSchedule> courseSchedules1 = new ArrayList<>();
		courseSchedules1.add(oldCourseSchedule);
		List<CourseScheduleModifyLog> modifyLogs = getModifyLogs(courseIds, courseSchedules1, user.getId(), date);

		PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(oldCourseSchedule.getMusicGroupId()));
        String classDate = DateUtil.format(courseSchedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
        String startClassTime = DateUtil.format(courseSchedule.getStartClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
        Date startDateTime = DateUtil.stringToDate(classDate + " " + startClassTime, DateUtil.EXPANDED_DATE_TIME_FORMAT);
        Date endDateTime = DateUtil.addMinutes(startDateTime,practiceGroup.getSingleClassMinutes());
		if(practiceGroup.getType() != TRIAL){
			if(DateUtil.minutesBetween(startDateTime,practiceGroup.getCoursesStartDate()) > 0){
				throw new BizException("调整失败: 调整时间不得早于开课时间");
			}
			if(DateUtil.minutesBetween(practiceGroup.getCoursesExpireDate(),endDateTime) > 0){
				throw new BizException("调整失败: 截止时间超过课程有效期");
			}
		}
        if(DateUtil.daysBetween(DateUtil.stringToDate(DateUtil.format(startDateTime,DateUtil.DEFAULT_PATTERN),DateUtil.DEFAULT_PATTERN),
                DateUtil.stringToDate(DateUtil.format(endDateTime,DateUtil.DEFAULT_PATTERN),DateUtil.DEFAULT_PATTERN)) > 0){
            throw new BizException("单节课时不允许跨天");
        }

        oldCourseSchedule.setUpdateTime(date);

        if(DateUtil.minutesBetween(endDateTime,date) > 0){
        	if(oldCourseSchedule.getStatus() == CourseStatusEnum.OVER){
        		throw new BizException("调整失败：已结束的课程无法调整至已结束");
			}
            oldCourseSchedule.setStatus(CourseStatusEnum.OVER);
        }else if(DateUtil.minutesBetween(date,startDateTime) > 0){
            oldCourseSchedule.setStatus(CourseStatusEnum.NOT_START);
        }else {
            oldCourseSchedule.setStatus(CourseStatusEnum.UNDERWAY);
        }
        Integer practiceCourseMinutes=25;
        SysConfig practiceCourseMinutesConfig = sysConfigService.findByParamName(SysConfigService.PRACTICE_COURSE_MINUTES);
        if(Objects.nonNull(practiceCourseMinutesConfig)){
            practiceCourseMinutes=practiceCourseMinutesConfig.getParanValue(Integer.class);
        }
        oldCourseSchedule.setClassDate(courseSchedule.getStartClassTime());
        oldCourseSchedule.setStartClassTime(courseSchedule.getStartClassTime());
        oldCourseSchedule.setEndClassTime(DateUtil.addMinutes(courseSchedule.getStartClassTime(),practiceCourseMinutes));

        List<CourseSchedule> courseSchedules=new ArrayList<>();
        courseSchedules.add(oldCourseSchedule);
        checkNewCourseSchedules(courseSchedules,false,false);
        if(oldCourseSchedule.getStatus() == CourseStatusEnum.NOT_START){
            teacherAttendanceDao.batchUpdateTeacher(courseSchedules.stream().map(e->e.getId()).collect(Collectors.toSet()),oldCourseSchedule.getActualTeacherId());
            //清空学生签到记录
            studentAttendanceDao.deleteStudentAttendancesByCourse(courseScheduleId);
        }
		// 删除作业
		courseHomeworkService.delHomwworkByCourseScheduleId(courseScheduleId);
        //删除评论
		courseScheduleReviewDao.delByCourseScheduleId(courseScheduleId);
		//删除申述
		courseScheduleComplaintsDao.delByCourseScheduleId(courseScheduleId);
		//清除考勤申述
		teacherAttendanceDao.clearAttendanceComplaint(courseScheduleId);
        courseScheduleDao.update(oldCourseSchedule);
		if(practiceGroup.getType() == TRIAL){
			practiceGroup.setCoursesStartDate(courseScheduleDao.getFirstTime(practiceGroup.getId(),PRACTICE.getCode()));
			practiceGroup.setCoursesExpireDate(courseScheduleDao.getLastTime(practiceGroup.getId(),PRACTICE.getCode()));
			practiceGroup.setUpdateTime(date);
			practiceGroupDao.update(practiceGroup);
		}
		//保存修改记录
		saveModifyLog(modifyLogs,courseIds);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void practiceCourseTeacherAdjust(Long courseScheduleId,Integer teacherId) {
        SysUser user = sysUserService.getUser();
        if(Objects.isNull(courseScheduleId)){
            throw new BizException("请指定课程");
        }
        if(Objects.isNull(teacherId)){
            throw new BizException("请指定上课老师");
        }
        CourseSchedule oldCourseSchedule = courseScheduleDao.getSurplusCourseWithGroup(courseScheduleId);
        if(Objects.isNull(oldCourseSchedule)){
            throw new BizException("调整失败: 课程已结束");
        }
        if(!oldCourseSchedule.getGroupType().equals(PRACTICE)){
            throw new BizException("请选择网管课");
        }
        if(teacherId.equals(oldCourseSchedule.getActualTeacherId())){
            throw new BizException("请选择不同的老师");
        }
        Date date = new Date();
		//获取调整日志
		List<Long> courseIds = new ArrayList<>();
		courseIds.add(courseScheduleId);
		List<CourseSchedule> courseSchedules1 = new ArrayList<>();
		courseSchedules1.add(oldCourseSchedule);
		List<CourseScheduleModifyLog> modifyLogs = getModifyLogs(courseIds, courseSchedules1, user.getId(), date);

        oldCourseSchedule.setUpdateTime(date);
        oldCourseSchedule.setActualTeacherId(teacherId);

        List<CourseSchedule> courseSchedules = new ArrayList<>();
        courseSchedules.add(oldCourseSchedule);
        Set<Long> courseScheduleIds = courseSchedules.stream().map(e -> e.getId()).collect(Collectors.toSet());
        checkNewCourseSchedules(courseSchedules,false,false);

        courseScheduleDao.update(oldCourseSchedule);

        PracticeGroup practiceGroup = practiceGroupDao.get(Long.valueOf(oldCourseSchedule.getMusicGroupId()));
		List<TeacherDefaultPracticeGroupSalary> teacherDefaultPracticeGroupSalaries = teacherDefaultPracticeGroupSalaryDao.queryByUserId(teacherId);
		if(teacherDefaultPracticeGroupSalaries == null || teacherDefaultPracticeGroupSalaries.size() == 0){
			throw new BizException("请设置老师默认课酬");
		}
		BigDecimal salary = teacherDefaultPracticeGroupSalaries.get(0).getMainTeacherSalary();
		if(practiceGroup != null){
			Integer vipGroupActivityId = practiceGroup.getVipGroupActivityId();
			if(vipGroupActivityId != null){
				ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(practiceGroup.getId(),"PRACTICE",null);
				if(activityUserMapper != null){
					Map<String, BigDecimal> map = practiceGroupService.countPracticeGroupPredictFee1(practiceGroup, teacherId);
					if(practiceGroup.getId().equals(activityUserMapper.getPracticeGroupId())){
						salary = map.get("onlineTeacherSalary");
					}else {
						salary = map.get("giveTeacherDefaultSalary");
					}
				}
			}
		}
		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedule(courseScheduleId);
		if(!CollectionUtils.isEmpty(courseScheduleTeacherSalaries)){
			SysConfig practiceCourseSalaryConfig = sysConfigService.findByParamName(SysConfigService.PRACTICE_COURSE_SALARY);
			BigDecimal teacherSalary = new BigDecimal(practiceCourseSalaryConfig.getParanValue());
			for (CourseScheduleTeacherSalary courseScheduleTeacherSalary : courseScheduleTeacherSalaries) {
				courseScheduleTeacherSalary.setUserId(teacherId);
				if(teacherSalary.compareTo(courseScheduleTeacherSalary.getExpectSalary())==0){
					continue;
				}
				courseScheduleTeacherSalary.setExpectSalary(salary);
			}
			courseScheduleTeacherSalaryDao.batchUpdate(courseScheduleTeacherSalaries);
		}


        teacherAttendanceDao.batchUpdateTeacher(courseScheduleIds,teacherId);
		//删除申述
		courseScheduleComplaintsDao.delByCourseScheduleId(courseScheduleId);
		//删除申述
		courseScheduleComplaintsDao.delByCourseScheduleIds(courseScheduleIds);
		//清除考勤申述
		teacherAttendanceDao.clearAttendanceComplaints(courseScheduleIds);
		//添加通讯录
		imUserFriendService.refreshCourseImUserFriend(courseScheduleId);
		//保存修改记录
		saveModifyLog(modifyLogs,courseIds);

        Map<Integer, String> map = new HashMap<>(2);
        map.put(teacherId,teacherId.toString());
        map.put(oldCourseSchedule.getActualTeacherId(),oldCourseSchedule.getActualTeacherId().toString());
        if (map != null && map.size() > 0) {
            sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_COURSE_SCHEDULE_CHANGE_RESULT,
                    map, null, 0, "7","TEACHER");
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void practiceGroupTeacherAdjust(String practiceGroupId, Integer teacherId, Integer subjectId, Integer educationalTeacherId) {
        SysUser user = sysUserService.getUser();
        if(StringUtils.isEmpty(practiceGroupId)){
            throw new BizException("请指定课程组");
        }
        if(Objects.isNull(teacherId)){
            throw new BizException("请指定上课老师");
        }
        List<CourseSchedule> courseSchedules = courseScheduleDao.findSurplusCourseWithGroup(PRACTICE,practiceGroupId);
        if(courseSchedules != null && courseSchedules.size() > 0){
            courseSchedules = courseSchedules.stream().filter(courseSchedule -> !courseSchedule.getActualTeacherId().equals(teacherId)).collect(Collectors.toList());
        }
		Date date = new Date();
		List<Long> courseIds1 = courseSchedules.stream().map(e -> e.getId()).collect(Collectors.toList());
		//获取修改日志
		List<CourseScheduleModifyLog> modifyLogs = new ArrayList<>();
		if(!CollectionUtils.isEmpty(courseIds1)){
			modifyLogs = getModifyLogs(courseIds1, courseSchedules, user.getId(), date);
		}
		PracticeGroup practiceGroup = practiceGroupDao.get(Long.valueOf(practiceGroupId));

		ClassGroup classGroup = classGroupDao.findByMusicGroupAndType(practiceGroupId, PRACTICE.getCode());
		//教务老师变更
        if(Objects.nonNull(educationalTeacherId)&&!educationalTeacherId.equals(practiceGroup.getEducationalTeacherId())){
			practiceGroup.setEducationalTeacherId(educationalTeacherId);
		}
		if(Objects.nonNull(subjectId)&&!subjectId.equals(practiceGroup.getSubjectId())){
			Subject subject = subjectDao.get(subjectId);
			String groupName = practiceGroup.getName();
			groupName=groupName.substring(groupName.indexOf("•"));
			groupName=subject.getName()+groupName;
			practiceGroup.setName(groupName);
			practiceGroup.setSubjectId(subjectId);
			classGroup.setSubjectIdList(subjectId.toString());
			classGroup.setName(groupName);
			classGroup.setSubjectIdList(subjectId.toString());
			classGroupDao.update(classGroup);
			courseScheduleDao.updateCourseNameByGroup(practiceGroupId, PRACTICE,groupName);
		}
		if(!teacherId.equals(practiceGroup.getUserId())){
			practiceGroup.setUserId(teacherId);
			classGroupTeacherMapperService.updateTeacher(practiceGroupId,teacherId, PRACTICE.getCode());
			classGroupTeacherSalaryDao.updateTeacher(practiceGroupId,teacherId, PRACTICE.getCode());
			if(courseSchedules == null || courseSchedules.size() == 0){
				practiceGroupDao.update(practiceGroup);
				return;
			}
			Map<Integer, String> teacherIdMap = new HashMap<>(10);
			teacherIdMap.put(teacherId,teacherId.toString());

			courseSchedules.forEach(e->{
				teacherIdMap.put(e.getActualTeacherId(),e.getActualTeacherId().toString());
				e.setUpdateTime(date);
				e.setActualTeacherId(teacherId);
			});
			checkNewCourseSchedules(courseSchedules,false,false);
			courseScheduleDao.batchUpdate(courseSchedules);
			Set<Long> courseScheduleIds = courseSchedules.stream().map(e -> e.getId()).collect(Collectors.toSet());
			if(practiceGroup.getType() != TRIAL){
				List<CourseSchedule> groupNotStartCourses = courseScheduleDao.findGroupNotStartCourses(practiceGroupId, PRACTICE);
				if(!CollectionUtils.isEmpty(groupNotStartCourses)){
					SysConfig practiceCourseSalaryConfig = sysConfigService.findByParamName(SysConfigService.PRACTICE_COURSE_SALARY);
					BigDecimal teacherSalary = new BigDecimal(practiceCourseSalaryConfig.getParanValue());
					List<Long> courseIds = groupNotStartCourses.stream().map(CourseSchedule::getId).collect(Collectors.toList());
					List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedules(courseIds);
					List<TeacherDefaultPracticeGroupSalary> teacherDefaultPracticeGroupSalaries = teacherDefaultPracticeGroupSalaryDao.queryByUserId(teacherId);
					if(teacherDefaultPracticeGroupSalaries == null || teacherDefaultPracticeGroupSalaries.size() == 0){
						throw new BizException("请设置老师默认课酬");
					}
					BigDecimal salary = teacherDefaultPracticeGroupSalaries.get(0).getMainTeacherSalary();
					if(practiceGroup != null){
						Integer vipGroupActivityId = practiceGroup.getVipGroupActivityId();
						if(vipGroupActivityId != null){
							ActivityUserMapper activityUserMapper = activityUserMapperDao.findVipUserMapper(practiceGroup.getId(),"PRACTICE",null);
							if(activityUserMapper != null){
								Map<String, BigDecimal> map = practiceGroupService.countPracticeGroupPredictFee1(practiceGroup, teacherId);
								if(practiceGroup.getId().equals(activityUserMapper.getPracticeGroupId())){
									salary = map.get("onlineTeacherSalary");
								}else {
									salary = map.get("giveTeacherDefaultSalary");
								}
							}
						}
					}
					for (CourseScheduleTeacherSalary courseScheduleTeacherSalary : courseScheduleTeacherSalaries) {
						courseScheduleTeacherSalary.setUserId(teacherId);
						if(teacherSalary.compareTo(courseScheduleTeacherSalary.getExpectSalary())==0){
							continue;
						}
						courseScheduleTeacherSalary.setExpectSalary(salary);
					}
					courseScheduleTeacherSalaryDao.batchUpdate(courseScheduleTeacherSalaries);
				}
			}
			teacherAttendanceDao.batchUpdateTeacher(courseScheduleIds,teacherId);
			//删除申述
			courseScheduleComplaintsDao.delByCourseScheduleIds(courseScheduleIds);
			//清除考勤申述
			teacherAttendanceDao.clearAttendanceComplaints(courseScheduleIds);
			//保存修改记录
			if(!CollectionUtils.isEmpty(modifyLogs))
				saveModifyLog(modifyLogs,courseIds1);

			if (teacherIdMap != null && teacherIdMap.size() > 0) {
				sysMessageService.batchSendMessage(MessageSender.JIGUANG, MessageTypeEnum.TEACHER_PUSH_COURSE_SCHEDULE_CHANGE_RESULT,
						teacherIdMap, null, 0, "7","TEACHER");
			}
		}
		practiceGroupDao.update(practiceGroup);
		imUserFriendService.refreshGroupImUserFriend(practiceGroup.getId().toString(), PRACTICE);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void cleanAttendance(String courseScheduleIds) {
        SysUser user = sysUserService.getUser();
        if(StringUtils.isEmpty(courseScheduleIds)){
            throw new BizException("参数校验失败");
        }
		businessLogger.info("清空老师和学生考勤,user:{},参数:{}",user.getId(),courseScheduleIds);
        List<Long> courseScheduleId = courseScheduleTeacherSalaryService.querySettlementScheduleId(courseScheduleIds);
        if(courseScheduleId == null || courseScheduleId.size() == 0){
            throw new BizException("课程已结算，无法清除考勤");
        }
        if(courseScheduleId.size()==1){
			VipGroup vipGroup = vipGroupDao.findByCourseSchedule(courseScheduleId.get(0));
			if(Objects.nonNull(vipGroup)
					&&!VipGroupStatusEnum.PROGRESS.equals(vipGroup.getStatus())
					&&!VipGroupStatusEnum.FINISHED.equals(vipGroup.getStatus())){
				throw new BizException("非进行中课程组不允许进行此操作");
			}
		}
        teacherAttendanceDao.batchCleanCourseTeacherSignInfo(courseScheduleId);
        studentAttendanceDao.deleteByCourseSchedules(courseScheduleId);
		//删除申述
		courseScheduleComplaintsDao.delByCourseScheduleIds(courseScheduleId);
		//清除考勤申述
		teacherAttendanceDao.clearAttendanceComplaints(courseScheduleId);
        //重置学生人数
		courseScheduleDao.resetStudentNum(courseScheduleId);
    }

	@Override
	public Object teacherCourseRemarkInfo(Long courseScheduleId) {
		SysUser user = sysUserService.getUser();
		if(courseScheduleId == null){
			throw new BizException("参数校验失败");
		}
		Map<String,Object> resultMap = new HashMap<>(7);
		//获取课程详情
		CourseSchedule schedule = courseScheduleDao.get(courseScheduleId);
		resultMap.put("courseScheduleName",schedule.getName());
		resultMap.put("courseScheduleClassDate",schedule.getClassDate());
		resultMap.put("courseScheduleStartClassTime",schedule.getStartClassTime());
		resultMap.put("courseScheduleEndClassTime",schedule.getEndClassTime());
		resultMap.put("courseScheduleContent",schedule.getTeachingContent());
		//获取当前课程的声部
		resultMap.put("subjectName",courseScheduleDao.getSubjectNameById(courseScheduleId));
		//获取学员评论详情
		resultMap.put("studentRemark",courseScheduleComplaintsDao.findByCourseScheduleId(courseScheduleId));
		//获取当前课程的学员列表
		resultMap.put("students",courseScheduleStudentPaymentDao.findStudents(courseScheduleId));
		//获取当前课程老师的评价列表
		resultMap.put("teacherRemark",courseScheduleEvaluateDao.findByCourseAndTeacher(user.getId(),courseScheduleId));
		return resultMap;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void teacherCourseRemarkCommit(TeacherRemarkCommitDto teacherRemarkCommitDt) {
		SysUser user = sysUserService.getUser();
		Long courseScheduleId = teacherRemarkCommitDt.getCourseScheduleId();
		CourseSchedule schedule = courseScheduleDao.get(courseScheduleId);
		if(schedule == null){
			throw new BizException("课程不存在");
		}
		if(StringUtils.isNotEmpty(schedule.getTeachingContent())){
			throw new BizException("您已提交过课程评价,请勿重复提交");
		}
		//是否提交过评论
		List<CourseScheduleEvaluate> byCourseAndTeacher = courseScheduleEvaluateDao.findByCourseAndTeacher(user.getId(), courseScheduleId);
		if(byCourseAndTeacher != null && byCourseAndTeacher.size() > 0){
			throw new BizException("您已提交过课程评价,请勿重复提交");
		}
		//修改教学内容
		if(StringUtils.isNotEmpty(teacherRemarkCommitDt.getTeachingContent())){
			schedule.setTeachingContent(teacherRemarkCommitDt.getTeachingContent());
			schedule.setUpdateTime(new Date());
			courseScheduleDao.update(schedule);
		}
		List<CourseScheduleEvaluate> courseScheduleEvaluates = teacherRemarkCommitDt.getCourseScheduleEvaluates();
		if(courseScheduleEvaluates != null || courseScheduleEvaluates.size() > 0){
			courseScheduleEvaluates.forEach(e->{
				e.setMusicGroupId(schedule.getMusicGroupId());
				e.setClassGroupId(schedule.getClassGroupId());
				e.setTeacherId(user.getId());
				e.setCourseScheduleId(courseScheduleId);
				e.setIsPushed(0);
				courseScheduleEvaluateDao.insert(e);
			});
		}
	}

	@Override
	public Map<String,Object> getTeacherCourseHeadInfo(GroupType groupType,String groupId,Long courseScheduleId){
		Map<String,Object> resultMap = new HashMap<>(8);
		Teacher teacher;
		switch (groupType){
			case PRACTICE:
				PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(groupId));
				resultMap.put("groupName",practiceGroup.getName());
				resultMap.put("groupStatus",practiceGroup.getGroupStatus().getCode());
				teacher = teacherDao.get(practiceGroup.getUserId());
				resultMap.put("groupTeacher",teacher.getRealName());
				resultMap.put("teacherIntroduction",teacher.getIntroduction());
				resultMap.put("teacherAvatar",teacher.getAvatar());
				//统计授课次数
				resultMap.put("teacherGiveLesson",courseScheduleTeacherSalaryDao.countTeacherGiveLesson(practiceGroup.getUserId()));
				break;
			case VIP:
				VipGroup vipGroup = vipGroupDao.get(Long.parseLong(groupId));
				resultMap.put("groupName",vipGroup.getName());
				resultMap.put("groupStatus",vipGroup.getStatus().getCode());
				teacher = teacherDao.get(vipGroup.getUserId());
				resultMap.put("groupTeacher",teacher.getRealName());
				resultMap.put("teacherIntroduction",teacher.getIntroduction());
				resultMap.put("teacherAvatar",teacher.getAvatar());
				//统计授课次数
				resultMap.put("teacherGiveLesson",courseScheduleTeacherSalaryDao.countTeacherGiveLesson(vipGroup.getUserId()));
				break;
			case COMM:
				CoursesGroup coursesGroup = coursesGroupDao.get(Long.parseLong(groupId));
				resultMap.put("groupName",coursesGroup.getName());
				resultMap.put("groupStatus",coursesGroup.getStatus().getCode());
				teacher = teacherDao.get(coursesGroup.getTeacherId());
				resultMap.put("groupTeacher",teacher.getRealName());
				resultMap.put("teacherIntroduction",teacher.getIntroduction());
				resultMap.put("teacherAvatar",teacher.getAvatar());
				//统计授课次数
				resultMap.put("teacherGiveLesson",courseScheduleTeacherSalaryDao.countTeacherGiveLesson(coursesGroup.getTeacherId()));
				break;
			case MUSIC:
				if(courseScheduleId != null){
					ClassGroup byCourseSchedule = classGroupDao.findByCourseSchedule(courseScheduleId.intValue(),null);
					resultMap.put("groupName",byCourseSchedule.getName());
					ClassGroupTeacherMapper groupTeachersRole = classGroupTeacherMapperService.getClassGroupTeachersRole(byCourseSchedule.getId(), "BISHOP");
					if(groupTeachersRole != null){
						teacher = teacherDao.get(groupTeachersRole.getUserId());
						resultMap.put("groupTeacher",teacher.getRealName());
						resultMap.put("teacherIntroduction",teacher.getIntroduction());
						resultMap.put("teacherAvatar",teacher.getAvatar());
						resultMap.put("teacherGiveLesson",courseScheduleTeacherSalaryDao.countTeacherGiveLesson(groupTeachersRole.getUserId()));
					}
				}
				break;
			default:
				throw new BizException("课程类型错误");
		}
		resultMap.put("startTime",courseScheduleDao.getMinCourseTime(groupType, groupId));
		resultMap.put("endTime",courseScheduleDao.getMaxCourseTime(groupType, groupId));
		return resultMap;
	}

	@Override
	public Object queryCourseInfoByGroupId(GroupType groupType, String groupId) {
    	List<CourseSchedule> courseSchedules = courseScheduleDao.findCourseByGroupId(groupType.getCode(),groupId);
    	Map<String,Object> resultMap = new HashMap<>(4);
    	resultMap.put("courseInfo",courseSchedules);
    	resultMap.put("count",courseSchedules.size());
		int online = courseSchedules.stream().filter(e -> e.getTeachMode() == TeachModeEnum.ONLINE).collect(Collectors.toList()).size();
		int offline = courseSchedules.stream().filter(e -> e.getTeachMode() == TeachModeEnum.OFFLINE).collect(Collectors.toList()).size();
		resultMap.put("online",online);
		resultMap.put("offline",offline);
		return resultMap;
	}

	@Override
	public Object teacherCourseMinutes(QueryInfo queryInfo) {
		SysUser user = sysUserService.getUser();
		PageInfo pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
		Map<String, Object> params = new HashMap<>(16);
		MapUtil.populateMap(params, queryInfo);
		params.put("userId",user.getId());
		int count = coursesGroupDao.countTeacherCourseGroups(params);
		if(count > 0){
			pageInfo.setTotal(count);
			params.put("offset", pageInfo.getOffset());
			List<CoursesGroup> courseGroups = coursesGroupDao.queryTeacherCourseGroups(params);
			List<Long> collect = courseGroups.stream().map(e -> e.getId()).collect(Collectors.toList());
			//获取冻结课时
			Map<Long,Long> freezeMinutesMap = MapUtil.convertIntegerMap(coursesGroupDao.findFreezeMinutes(collect));
			//获取已消耗课时
			Map<Long,Long> consumeMinutesMap = MapUtil.convertIntegerMap(coursesGroupDao.findConsumeMinutes(collect));
			courseGroups.forEach(e->{
				Long aLong = freezeMinutesMap.get(e.getId());
				e.setFreezeMinutes(aLong == null?0:aLong.intValue());
				Long aLong1 = consumeMinutesMap.get(e.getId());
				e.setConsumeMinutes(aLong1 == null?0:aLong1.intValue());
			});
			pageInfo.setRows(courseGroups);
		}
		return pageInfo;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updatePracticeGroupInDate(String practiceGroupId, Date courseStartDate, Date coursesExpireDate, Boolean onlyNormal) {
		SysUser user = sysUserService.getUser();
    	if(practiceGroupId == null || courseStartDate == null || coursesExpireDate == null){
    		throw new BizException("参数校验失败");
		}
		PracticeGroup practiceGroup = practiceGroupDao.get(Long.parseLong(practiceGroupId));
    	if(practiceGroup == null){
			throw new BizException("课程组不存在");
		}
		if(onlyNormal&&!GroupStatusEnum.NORMAL.equals(practiceGroup.getGroupStatus())){
			throw new BizException("非正常状态课程组不可编辑");
		}
		if(!GroupStatusEnum.NORMAL.equals(practiceGroup.getGroupStatus())&&!GroupStatusEnum.FINISH.equals(practiceGroup.getGroupStatus())){
			throw new BizException("非正常状态课程组不可编辑");
		}
		if(courseStartDate.after(coursesExpireDate)){
			throw new BizException("课程组有效结束时间不能早于课程组有效开始时间");
		}
		Date minCourseTime = courseScheduleDao.getMinCourseTime(PRACTICE, practiceGroupId);
		if(Objects.nonNull(minCourseTime)&&courseStartDate.compareTo(minCourseTime)>0){
			throw new BizException("课程组有效开始时间不得晚于第一节课开始时间");
		}
		Date maxCourseTime = courseScheduleDao.getMaxCourseTime(PRACTICE, practiceGroupId);
		if(Objects.nonNull(maxCourseTime)&&!DateUtil.isSameDay(maxCourseTime, coursesExpireDate)&&coursesExpireDate.compareTo(maxCourseTime)<0){
			throw new BizException("课程组有效结束时间不得早于最后一节课结束时间");
		}
//		if(Objects.nonNull(practiceGroup.getBuyMonths())){
//			LocalDate courseStartDay = LocalDateTime.ofInstant(practiceGroup.getCoursesStartDate().toInstant(), DateUtil.zoneId).toLocalDate();
//			LocalDate currentExpiredDay = courseStartDay.plusMonths(practiceGroup.getBuyMonths()).plusDays(-1);
//			Date courseExpiredDateTemp=Date.from(currentExpiredDay.atStartOfDay(DateUtil.zoneId).toInstant());
//			if(coursesExpireDate.before(courseExpiredDateTemp)){
//				throw new BizException("课程结束时间不得早于，{}", DateUtil.dateToString(courseExpiredDateTemp, "yyyy年MM月dd日"));
//			}
//		}
		Date now = new Date();
		CoursesGroupModifyLog coursesGroupModifyLog = new CoursesGroupModifyLog();
		coursesGroupModifyLog.setGroupId(practiceGroupId);
		coursesGroupModifyLog.setGroupType(PRACTICE.getCode());
		coursesGroupModifyLog.setOperatorId(user.getId());
		coursesGroupModifyLog.setPreviousGroup(JSONObject.toJSONString(practiceGroup));
		practiceGroup.setCoursesStartDate(courseStartDate);
		practiceGroup.setCoursesExpireDate(DateUtil.addSeconds(DateUtil.addDays(coursesExpireDate, 1),-1));
		if(practiceGroup.getCoursesExpireDate().compareTo(now)>=0||DateUtil.isSameDay(practiceGroup.getCoursesExpireDate(), now)){
			practiceGroup.setGroupStatus(GroupStatusEnum.NORMAL);
		}
		practiceGroup.setUpdateTime(now);
		practiceGroupDao.update(practiceGroup);
		coursesGroupModifyLog.setCurrentGroup(JSONObject.toJSONString(practiceGroup));
		coursesGroupModifyLogDao.insert(coursesGroupModifyLog);
	}

	@Override
	public List<CourseSchedule> queryNoStartCourseByTeacherId(String classGroupIds, Integer teacherId, String teacherRole) {
		return courseScheduleDao.queryNoStartCourseByTeacherId(classGroupIds,teacherId,teacherRole);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public HttpResponseResult classGroupTeacherAdjust(String classGroupIds, Integer teacherId, Integer targetTeacherId, Boolean allowZeroSalary) {
		List<Integer> teachingTeacherIdList = new ArrayList<Integer>();
		teachingTeacherIdList.add(targetTeacherId);
		List<CourseSchedule> courseSchedules = this.queryNoStartCourseByTeacherId(classGroupIds,teacherId,"BISHOP");
		//获取助教老师列表
		if(courseSchedules.size() > 0){
			courseSchedules.forEach(e->{
				e.setTeachingTeacherIdList(courseScheduleTeacherSalaryDao.queryTeacherByTeaching(e.getClassGroupId(),"TEACHING"));
				e.setActualTeacherId(targetTeacherId);
			});
			HttpResponseResult httpResponseResult = this.courseAdjust(courseSchedules, allowZeroSalary);
			if(httpResponseResult.getCode()==HttpStatus.PARTIAL_CONTENT.value()){
				return httpResponseResult;
			}
			//修改老师考勤
			Set<Integer> collect1 = courseSchedules.stream().map(e -> e.getClassGroupId()).collect(Collectors.toSet());
//			teacherAttendanceDao.deleteByCourseAndTeacherId(collect1,teacherId);
			batchUpdateClassTeacher(new ArrayList<>(collect1),teacherId,targetTeacherId);
		}
		List<CourseSchedule> courseSchedules1 = this.queryNoStartCourseByTeacherId(classGroupIds, teacherId, "TEACHING");
		if(courseSchedules1.size() > 0){
			courseSchedules1.forEach(e->{
				e.setActualTeacherId(courseScheduleTeacherSalaryDao.queryTeacherByBishop(e.getClassGroupId()));
				e.setTeachingTeacherIdList(teachingTeacherIdList);
			});
			HttpResponseResult httpResponseResult = this.courseAdjust(courseSchedules1, allowZeroSalary);
			if(httpResponseResult.getCode()==HttpStatus.PARTIAL_CONTENT.value()){
				return httpResponseResult;
			}
			Set<Integer> collect1 = courseSchedules1.stream().map(e -> e.getClassGroupId()).collect(Collectors.toSet());
////			teacherAttendanceDao.deleteByCourseAndTeacherId(collect1,teacherId);
			batchUpdateClassTeacher(new ArrayList<>(collect1),teacherId,targetTeacherId);
		}
		return BaseController.succeed();
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void batchPracticeGroupTeacherAdjust(String practiceGroupId, Integer teacherId, Integer subjectId, Integer educationalTeacherId,Integer oldTeacherId) {
		String[] split = practiceGroupId.split(",");
		for (int i = 0; i < split.length; i++) {
			this.practiceGroupTeacherAdjust(split[i],teacherId,subjectId,educationalTeacherId);
		}
		List<ClassGroup> classGroups = classGroupDao.queryClassGroups(practiceGroupId,"PRACTICE");
		if(classGroups.size() > 0){
			List<Integer> collect = classGroups.stream().map(e -> e.getId()).collect(Collectors.toList());
			batchUpdateClassTeacher(collect,oldTeacherId,teacherId);
		}
	}

	@Transactional(rollbackFor = Exception.class)
	public void batchUpdateClassTeacher(List<Integer> classGroupIds,Integer oldTeacherId,Integer newTeacherId){
		//修改班级老师
		List<ClassGroup> classGroupList = classGroupDao.findByClassGroupIds(classGroupIds,null);
		classGroupList.forEach(e->{
			e.setUserIds(newTeacherId.toString());
		});
		classGroupTeacherMapperService.batchUpdateTeacherByClassId(classGroupIds,oldTeacherId,newTeacherId);
		//旧老师退群,新老师加群
		if(classGroupList != null && classGroupIds.size() > 0){
			for (ClassGroup classGroup:classGroupList) {
				imGroupMemberService.quit(classGroup.getId().toString(), oldTeacherId);
				imGroupMemberService.join(classGroup.getId().toString(), newTeacherId, "指导老师", false);
			}
		}
	}

	@Override
	public Object teacherCourseHeadInfo(Long courseScheduleId) {
		SysUser user = sysUserService.getUser();
		CourseSchedule schedule = courseScheduleDao.get(courseScheduleId);
		if(schedule == null){
			throw new BizException("课程不存在");
		}
		Map<String,Object> teacherCourseHeadInfo = getTeacherCourseHeadInfo(schedule.getGroupType(), schedule.getMusicGroupId(), courseScheduleId);
		//获取课程规划
		MusicGroupTrainPlanSaveDto schoolTerm = musicGroupTrainPlanService.getSchoolTerm(schedule);
		int courseNumNo = courseScheduleDao.getCourseNumNo(schoolTerm);
		List<MusicGroupTrainPlan> planList = musicGroupTrainPlanService.queryPlan(schoolTerm);
		if(planList.size() >= courseNumNo){
			teacherCourseHeadInfo.put("coursePlan",planList.get(courseNumNo - 1).getPlan());
		}
		return teacherCourseHeadInfo;
	}

	@Override
	public Object teacherCourseRates(Long courseScheduleId) {
		SysUser user = sysUserService.getUser();
		CourseSchedule schedule = courseScheduleDao.get(courseScheduleId);
		if(schedule == null){
			throw new BizException("课程不存在");
		}
		businessLogger.info("teacherCourseRates courseScheduleId:{},userId{}",courseScheduleId,user.getId());
		//如果是乐团课
		List<CourseScheduleRateDto> courseScheduleRateDtos;
		CourseSchedule courseSchedule;
		if(schedule.getGroupType() == MUSIC){
			//获取当前班级
			courseScheduleRateDtos = courseScheduleDao.findCourseRate(null,schedule.getGroupType().getCode(),schedule.getClassGroupId(), user.getTenantId());
			//获取最近一次已结束的课程
			courseSchedule = courseScheduleDao.getLastEndCourse(null,schedule.getGroupType().getCode(),schedule.getClassGroupId());
		}else {
			//获取课程列表
			courseScheduleRateDtos = courseScheduleDao.findCourseRate(schedule.getMusicGroupId(),schedule.getGroupType().getCode(),null, user.getTenantId());
			//获取最近一次已结束的课程
			courseSchedule = courseScheduleDao.getLastEndCourse(schedule.getMusicGroupId(),schedule.getGroupType().getCode(),null);
		}
		if(courseScheduleRateDtos == null || courseScheduleRateDtos.size() == 0){
			return courseScheduleRateDtos;
		}
		List<Long> courseScheduleIds = courseScheduleRateDtos.stream().map(CourseScheduleRateDto::getCourseScheduleId).collect(Collectors.toList());
		List<CourseScheduleReview> courseReviews = courseScheduleReviewDao.findByCourseSchedules(courseScheduleIds);
		Map<Integer, List<CourseScheduleReview>> reviews = courseReviews.stream().collect(Collectors.groupingBy(CourseScheduleReview::getCourseScheduleId));
		List<CourseSchedule> courseScheduleList = courseScheduleDao.findByCourseScheduleIds(courseScheduleIds);
		Map<Long, CourseSchedule> courseMap = courseScheduleList.stream().collect(Collectors.groupingBy(e -> e.getId(),Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
		MusicGroupTrainPlanSaveDto schoolTerm = null;
		List<MusicGroupTrainPlan> planList = null;
		for (CourseScheduleRateDto scheduleRateDto : courseScheduleRateDtos) {
			CourseSchedule course = courseMap.get(scheduleRateDto.getCourseScheduleId());
			if(schoolTerm == null){
				schoolTerm = musicGroupTrainPlanService.getSchoolTerm(course);
				planList = musicGroupTrainPlanService.queryPlan(schoolTerm);
			}else {
				MusicGroupTrainPlanSaveDto schoolTerm1 = musicGroupTrainPlanService.getSchoolTerm(course);
				if(!DateUtil.format(schoolTerm1.getStartSchoolTerm(),DateUtil.ISO_EXPANDED_DATE_FORMAT).
						equals(DateUtil.format(schoolTerm.getStartSchoolTerm(),DateUtil.ISO_EXPANDED_DATE_FORMAT))){
					planList = musicGroupTrainPlanService.queryPlan(schoolTerm);
				}
				schoolTerm = schoolTerm1;
			}
			int courseNumNo = courseScheduleDao.getCourseNumNo(schoolTerm);
			if(planList.size() > courseNumNo){
				scheduleRateDto.setCoursePlan(planList.get(courseNumNo - 1).getPlan());
			}
			//获取课程规划
			if(courseSchedule != null){
				if(reviews.containsKey(scheduleRateDto.getCourseScheduleId().intValue())){
					List<CourseScheduleReview> review = reviews.get(scheduleRateDto.getCourseScheduleId().intValue());
					scheduleRateDto.setTeachingContent(review.get(0).getTeachingMaterial());
				}
				if(scheduleRateDto.getCourseScheduleId().equals(courseSchedule.getId())){
					scheduleRateDto.setIsDefault(true);
				}
			}
		}
		return courseScheduleRateDtos;
	}

	@Override
	public List<CourseScheduleModifyLog> queryCourseAdjustDetail(Long courseScheduleId) {
		//获取所有的更改记录
		Map<String,Object> param = new HashMap(1);
		param.put("courseScheduleId",courseScheduleId);
		Integer tenantId = TenantContextHolder.getTenantId();
		param.put("tenantId", tenantId);
		List<CourseScheduleModifyLog> scheduleModifyLogs = courseScheduleModifyLogService.getDao().findAll(param);
		List<CourseScheduleAuditDetailDto> courseSchedules = new ArrayList<>();
		for (CourseScheduleModifyLog modifyLog : scheduleModifyLogs) {
			CourseScheduleAuditDetailDto auditDetailDto = JSON.parseObject(modifyLog.getPreviousCourseSchedule(), CourseScheduleAuditDetailDto.class);
			auditDetailDto.setOperatorId(modifyLog.getOperatorId());
			courseSchedules.add(auditDetailDto);
			CourseScheduleAuditDetailDto auditDetailDto1 = JSON.parseObject(modifyLog.getCurrentCourseSchedule(), CourseScheduleAuditDetailDto.class);
			auditDetailDto1.setOperatorId(modifyLog.getOperatorId());
			courseSchedules.add(auditDetailDto1);
		}
		if(courseSchedules.size() > 0){
			Set<Integer> actualTeacherIds = courseSchedules.stream().map(e -> e.getActualTeacherId()).collect(Collectors.toSet());
			actualTeacherIds.addAll(courseSchedules.stream().map(e -> e.getTeacherId()).collect(Collectors.toSet()));
			actualTeacherIds.addAll(courseSchedules.stream().map(e -> e.getOperatorId()).collect(Collectors.toSet()));
			Set<Integer> schoolIds = courseSchedules.stream().map(e -> e.getSchoolId()).collect(Collectors.toSet());
			Map<Integer, String> actualTeacherMap = getMap("sys_user", "id_", "real_name_", actualTeacherIds,tenantId, Integer.class, String.class);
			Map<Integer, String> schoolMap = new HashMap<>();
			if(schoolIds != null && schoolIds.size() > 0){
				schoolMap = getMap("school", "id_", "name_", schoolIds,tenantId, Integer.class, String.class);
			}
			for (CourseScheduleModifyLog modifyLog : scheduleModifyLogs) {
				CourseScheduleAuditDetailDto preAuditDetailDto = JSON.parseObject(modifyLog.getPreviousCourseSchedule(), CourseScheduleAuditDetailDto.class);
				preAuditDetailDto.setTeacherName(actualTeacherMap.get(preAuditDetailDto.getTeacherId()));
				preAuditDetailDto.setActualTeacherName(actualTeacherMap.get(preAuditDetailDto.getActualTeacherId()));
				preAuditDetailDto.setSchoolName(schoolMap.get(preAuditDetailDto.getSchoolId()));
				preAuditDetailDto.setOperatorName(actualTeacherMap.get(modifyLog.getOperatorId()));
				if(StringUtils.isNotEmpty(preAuditDetailDto.getTeachingTeacherIds())){
					preAuditDetailDto.setTeachingTeacherNames(teacherDao.queryTeacherNameByTeacherIds(preAuditDetailDto.getTeachingTeacherIds()));
				}
				modifyLog.setPreviousCourseSchedule(JSONObject.toJSONString(preAuditDetailDto));

				CourseScheduleAuditDetailDto auditDetailDto = JSON.parseObject(modifyLog.getCurrentCourseSchedule(), CourseScheduleAuditDetailDto.class);
				auditDetailDto.setTeacherName(actualTeacherMap.get(auditDetailDto.getTeacherId()));
				auditDetailDto.setActualTeacherName(actualTeacherMap.get(auditDetailDto.getActualTeacherId()));
				auditDetailDto.setSchoolName(schoolMap.get(auditDetailDto.getSchoolId()));
				auditDetailDto.setOperatorName(actualTeacherMap.get(modifyLog.getOperatorId()));
				if(StringUtils.isNotEmpty(auditDetailDto.getTeachingTeacherIds())){
					auditDetailDto.setTeachingTeacherNames(teacherDao.queryTeacherNameByTeacherIds(auditDetailDto.getTeachingTeacherIds()));
				}
				modifyLog.setCurrentCourseSchedule(JSONObject.toJSONString(auditDetailDto));
			}
			return scheduleModifyLogs;
		}
		return null;
	}

	@Override
	public void courseRepeatCheck() {
		List<CourseRepeatCheckDto> courseRepeatCheckInfos = courseScheduleDao.getCourseRepeatCheckInfos();
		courseRepeatCheckInfos.sort(Comparator.comparing(CourseRepeatCheckDto::getStartClassTime));
		Map<Date, List<CourseRepeatCheckDto>> dayCourseMap = courseRepeatCheckInfos.stream().collect(Collectors.groupingBy(CourseRepeatCheckDto::getClassDate));
		Map<Long, Map<String, Object>> courseRepeatIdMap = new HashMap<>();
		int needAdjustCourseNum = 0;
		for (CourseRepeatCheckDto cs1 : courseRepeatCheckInfos) {
			if(!courseRepeatIdMap.containsKey(cs1.getCourseId())){
				Map<String, Object> result = new HashMap<>();
				result.put("repeatCourseId", new ArrayList<>());
				courseRepeatIdMap.put(cs1.getCourseId(), result);
//				courseRepeatIdMap.get(cs1.getCourseId()).add(cs1.getCourseId());
			}else{
				continue;
			}
			List<CourseRepeatCheckDto> dayCourses = dayCourseMap.get(cs1.getClassDate());
			for (CourseRepeatCheckDto cs2 : dayCourses) {
				if(cs1.getCourseId().equals(cs2.getCourseId())){
					continue;
				}
				if(cs2.getEndClassTime().compareTo(cs1.getStartClassTime())<=0){
					continue;
				}

				if(courseRepeatIdMap.containsKey(cs2.getCourseId())
						&&((List<Long>)courseRepeatIdMap.get(cs2.getCourseId()).get("repeatCourseId")).contains(cs1.getCourseId())){
					continue;
				}
				if(cs2.getStartClassTime().compareTo(cs1.getEndClassTime())>=0){
					List<Long> repeatCourseId = (List<Long>) courseRepeatIdMap.get(cs1.getCourseId()).get("repeatCourseId");
					if(CollectionUtils.isEmpty(((List<Long>)courseRepeatIdMap.get(cs1.getCourseId()).get("repeatCourseId")))){
						courseRepeatIdMap.remove(cs1.getCourseId());
					}
					break;
				}
				if(cs1.getStartClassTime().compareTo(cs2.getEndClassTime())>=0||cs1.getEndClassTime().compareTo(cs2.getStartClassTime())<=0){
					continue;
				}
				Set<Integer> repeatTeacherIds = cs1.getTeacherIds().stream().filter(teacherId -> cs2.getTeacherIds().contains(teacherId)).collect(Collectors.toSet());
				if(repeatTeacherIds.size()>0){
					courseRepeatIdMap.get(cs1.getCourseId()).put("cause", "冲突教师编号:"+StringUtils.join(repeatTeacherIds));
					((List<Long>)courseRepeatIdMap.get(cs1.getCourseId()).get("repeatCourseId")).add(cs2.getCourseId());
					needAdjustCourseNum++;
					continue;
				}

				Set<Integer> repeatStudentIds = cs1.getStudentIds().stream().filter(studentId -> cs2.getStudentIds().contains(studentId)).collect(Collectors.toSet());
				if(repeatStudentIds.size()>0){
					courseRepeatIdMap.get(cs1.getCourseId()).put("cause", "学生冲突:"+StringUtils.join(repeatStudentIds));
					((List<Long>)courseRepeatIdMap.get(cs1.getCourseId()).get("repeatCourseId")).add(cs2.getCourseId());
					needAdjustCourseNum++;
					continue;
				}
			}
			if(courseRepeatIdMap.containsKey(cs1.getCourseId())&&CollectionUtils.isEmpty(((List<Long>)courseRepeatIdMap.get(cs1.getCourseId()).get("repeatCourseId")))){
				courseRepeatIdMap.remove(cs1.getCourseId());
			}
		}
//		System.out.println("重复课程：" + JSON.toJSONString(courseRepeatIdMap));
//		System.out.println("需调整课程数量：" + needAdjustCourseNum);
//		System.out.println("重复课程数量：" + courseRepeatIdMap.size());
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public HttpResponseResult courseMerge(CourseMergeDto courseMergeInfo) {
		Date now = new Date();

		if(CollectionUtils.isEmpty(courseMergeInfo.getMergeCourseIds())){
			throw new BizException("请选择需要合并的课程");
		}
		int courseMergeTime = courseScheduleDao.countCourseMergeTime(courseMergeInfo.getId());
		if(courseMergeTime>0){
			throw new BizException("存在已合并课程");
		}
		List<Long> allCourseIds = new ArrayList<>();
		allCourseIds.add(courseMergeInfo.getId());
		allCourseIds.addAll(courseMergeInfo.getMergeCourseIds());
		List<CourseSchedule> allCourses = courseScheduleDao.findByCourseScheduleIds(allCourseIds);
//		int courseTypeNum = allCourses.stream().map(CourseSchedule::getType).collect(Collectors.toSet()).size();
//		if(courseTypeNum>1){
//			throw new BizException("课程合并仅支持同课程类型");
//		}
		for (CourseSchedule courseSchedule : allCourses) {
			if(Objects.nonNull(courseSchedule.getIsLock())&&courseSchedule.getIsLock()==1){
				throw new BizException("存在审核中的课程");
			}
			if(Objects.nonNull(courseSchedule.getNewCourseId())){
				throw new BizException("存在已合并课程");
			}
			if(courseSchedule.getStartClassTime().compareTo(now)<=0){
				throw new BizException("课程合并仅支持未开始的课程");
			}
			if(VIP.equals(courseSchedule.getGroupType())||PRACTICE.equals(courseSchedule.getGroupType())){
				throw new BizException("VIP课、网管课暂不支持课程合并");
			}
			if(CourseScheduleType.MUSIC_NETWORK.equals(courseSchedule.getType())){
				throw new BizException("乐团网管课暂不支持课程合并");
			}
			if(Objects.isNull(courseMergeInfo.getClassGroupId())&&courseMergeInfo.getId().equals(courseSchedule.getId())){
				courseMergeInfo.setClassGroupId(courseSchedule.getClassGroupId());
				courseMergeInfo.setGroupType(courseSchedule.getGroupType());
				courseMergeInfo.setMusicGroupId(courseSchedule.getMusicGroupId());
			}
			courseSchedule.setNote("课程合并");
			courseSchedule.setMergeFlag(2);
			courseSchedule.setMergeTime(now);
			courseSchedule.setNewCourseId(courseMergeInfo.getId());
		}
		List<CourseScheduleStudentPayment> courseScheduleStudentPayments = courseScheduleStudentPaymentDao.findByCourseScheduleIds(allCourseIds);

		Map<Integer, CourseScheduleStudentPayment> mainStudentCourseMap = courseScheduleStudentPayments.stream().filter(c -> courseMergeInfo.getId().equals(c.getCourseScheduleId())).collect(Collectors.toMap(CourseScheduleStudentPayment::getUserId, c -> c, (c1, c2) -> c1));

		Map<Integer, List<CourseScheduleStudentPayment>> otherStudentCourseMap = courseScheduleStudentPayments.stream().filter(c -> !courseMergeInfo.getId().equals(c.getCourseScheduleId())).collect(Collectors.groupingBy(CourseScheduleStudentPayment::getUserId));

		List<CourseScheduleStudentPayment> newPayments = new ArrayList<>();

		for (Map.Entry<Integer, List<CourseScheduleStudentPayment>> otherStudentCourseMapEntry : otherStudentCourseMap.entrySet()) {
			List<CourseScheduleStudentPayment> studentCoursePayments = otherStudentCourseMapEntry.getValue();
			if(mainStudentCourseMap.containsKey(otherStudentCourseMapEntry.getKey())){
				continue;
			}
			CourseScheduleStudentPayment newPayment = new CourseScheduleStudentPayment();
			BeanUtils.copyProperties(studentCoursePayments.get(0), newPayment);
			newPayment.setCourseScheduleId(courseMergeInfo.getId());
			newPayment.setClassGroupId(courseMergeInfo.getClassGroupId());
			newPayment.setMusicGroupId(courseMergeInfo.getMusicGroupId());
			newPayment.setGroupType(courseMergeInfo.getGroupType());
			newPayment.setExpectPrice(BigDecimal.ZERO);
			newPayment.setId(null);
			newPayment.setBeMerged(true);
			newPayments.add(newPayment);
		}

		List<CourseSchedule> courseSchedules=new ArrayList<>();
		courseMergeInfo.setNewCourseId(courseMergeInfo.getId());
		courseMergeInfo.setMergeFlag(1);
		courseMergeInfo.setMergeTime(now);
		courseSchedules.add(courseMergeInfo);
		List<CourseScheduleModifyLog> insertCourseScheduleModifyLogList = new ArrayList<>();

		List<CourseSchedule> mergeCourses = allCourses.stream()
				.filter(c->courseMergeInfo.getMergeCourseIds().contains(c.getId())&&!c.getId().equals(courseMergeInfo.getId())).collect(Collectors.toList());
		for (CourseSchedule courseSchedule : mergeCourses) {
			CourseScheduleModifyLog courseScheduleModifyLog = new CourseScheduleModifyLog();
			courseScheduleModifyLog.setCourseScheduleId(courseSchedule.getId());
			courseScheduleModifyLog.setCreateTime(now);
			courseScheduleModifyLog.setPreviousCourseSchedule(JsonUtil.toJSONString(courseSchedule));
			courseScheduleModifyLog.setCurrentCourseSchedule(JsonUtil.toJSONString(courseSchedule));
			courseScheduleModifyLog.setOperatorId(courseMergeInfo.getOperatorId());
			insertCourseScheduleModifyLogList.add(courseScheduleModifyLog);
		}
		courseScheduleDao.batchUpdate(mergeCourses);

		if(!CollectionUtils.isEmpty(courseScheduleStudentPayments)){
			courseScheduleStudentPayments.forEach(c->c.setBeMerged(false));
			courseScheduleStudentPaymentDao.batchUpdate(courseScheduleStudentPayments);
		}
		if(!CollectionUtils.isEmpty(newPayments))
			courseScheduleStudentPaymentDao.batchInsert(newPayments);

		HttpResponseResult httpResponseResult = courseAdjust(courseSchedules, courseMergeInfo.getAllowZeroSalary());
		if(httpResponseResult.getCode()==HttpStatus.PARTIAL_CONTENT.value()){
			return httpResponseResult;
		}

		//主课类型为基础技能课需要更新课酬
		if (CourseSchedule.CourseScheduleType.HIGH_ONLINE.equals(courseMergeInfo.getType()) || CourseSchedule.CourseScheduleType.HIGH.equals(courseMergeInfo.getType())) {
			List<CourseScheduleTeacherSalary> oldCourseSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedule(courseMergeInfo.getId());
			for (CourseScheduleTeacherSalary oldCourseSalary : oldCourseSalaries) {
				//课酬调整后不能更新课酬
				if(!oldCourseSalary.getEnableChangeSalary()){
					continue;
				}
				courseScheduleTeacherSalaryService.createMusicGroupCourseTeacherSalary(null, courseMergeInfo, oldCourseSalary);
				courseScheduleTeacherSalaryService.update(oldCourseSalary);
			}
		}

		if (insertCourseScheduleModifyLogList.size() > 0) {
			courseScheduleModifyLogService.getDao().batchInsert(insertCourseScheduleModifyLogList);
			courseScheduleModifyLogService.pushSchoolCourseChange(mergeCourses.get(0));
		}
		return BaseController.succeed();
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void mergeCourseSplit(Long mainCourseId, Integer operatorId) {
		List<CourseSchedule> courseSchedules = courseScheduleDao.findByMainMergeCourse(mainCourseId);
		if(CollectionUtils.isEmpty(courseSchedules)){
			throw new BizException("未找到相关合并课程信息");
		}
		Date now = new Date();
		List<Long> courseIds = courseSchedules.stream().map(CourseSchedule::getId).collect(Collectors.toList());
		CourseSchedule mainCourse = courseSchedules.stream().filter(c -> mainCourseId.equals(c.getId())).findFirst().get();
		if(now.compareTo(mainCourse.getStartClassTime())>=0){
			throw new BizException("已结束的主课暂不支持此操作");
		}

		List<CourseScheduleStudentPayment> courseScheduleStudentPayments = courseScheduleStudentPaymentDao.findByMainMergedCourse(mainCourseId);
		Map<Integer, CourseScheduleStudentPayment> studentMainCoursePaymentMap = courseScheduleStudentPayments.stream().filter(c -> c.getCourseScheduleId().equals(mainCourseId)).collect(Collectors.toMap(CourseScheduleStudentPayment::getUserId, c -> c, (c1, c2) -> c1));

		List<CourseScheduleStudentPayment> otherCoursePayments = courseScheduleStudentPayments.stream().filter(c -> c.getCourseScheduleId().equals(mainCourseId)).collect(Collectors.toList());

		List<Integer> deletePaymentIds = new ArrayList<>();
//		List<CourseScheduleStudentPayment> updatePayments = new ArrayList<>();

		otherCoursePayments.sort(Comparator.comparing(CourseScheduleStudentPayment::getCreateTime).reversed());
		for (CourseScheduleStudentPayment otherCoursePayment : otherCoursePayments) {
			CourseScheduleStudentPayment mainCoursePayment = studentMainCoursePaymentMap.get(otherCoursePayment.getUserId());
			if(Objects.isNull(mainCoursePayment.getBeMerged())){
				throw new BizException("当前课程暂不支持此操作");
			}
			if(mainCoursePayment.getBeMerged()){
				deletePaymentIds.add(mainCoursePayment.getId().intValue());
				continue;
			}
//			mainCoursePayment.setExpectPrice(mainCoursePayment.getExpectPrice().subtract(otherCoursePayment.getExpectPrice()));
//			updatePayments.add(mainCoursePayment);
		}

		if(!CollectionUtils.isEmpty(deletePaymentIds)){
			courseScheduleStudentPaymentDao.batchDeleteWithID(deletePaymentIds);
		}
//		courseScheduleStudentPaymentDao.batchUpdate(updatePayments);

		List<CourseScheduleModifyLog> insertCourseScheduleModifyLogList = new ArrayList<>();
		for (CourseSchedule courseSchedule : courseSchedules) {
			CourseScheduleModifyLog courseScheduleModifyLog = new CourseScheduleModifyLog();
			courseScheduleModifyLog.setCourseScheduleId(courseSchedule.getId());
			courseScheduleModifyLog.setCreateTime(now);
			courseScheduleModifyLog.setPreviousCourseSchedule(JsonUtil.toJSONString(courseSchedule));

			//课程信息变更
			courseSchedule.setNote("合并课程还原");
			courseSchedule.setMergeFlag(0);

			courseScheduleModifyLog.setCurrentCourseSchedule(JsonUtil.toJSONString(courseSchedule));
			courseScheduleModifyLog.setOperatorId(operatorId);
			insertCourseScheduleModifyLogList.add(courseScheduleModifyLog);
		}
		courseScheduleDao.batchUpdate(courseSchedules);
		courseScheduleDao.cleanCoursMergeId(courseIds);



		//主课类型为基础技能课需要更新课酬
		if (CourseSchedule.CourseScheduleType.HIGH_ONLINE.equals(mainCourse.getType()) || CourseSchedule.CourseScheduleType.HIGH.equals(mainCourse.getType())) {
			List<CourseScheduleTeacherSalary> oldCourseSalaries = courseScheduleTeacherSalaryDao.findByCourseSchedule(mainCourse.getId());
			for (CourseScheduleTeacherSalary oldCourseSalary : oldCourseSalaries) {
				//课酬调整后不能更新课酬
				if(!oldCourseSalary.getEnableChangeSalary()){
					continue;
				}
				courseScheduleTeacherSalaryService.createMusicGroupCourseTeacherSalary(null, mainCourse, oldCourseSalary);
				courseScheduleTeacherSalaryService.update(oldCourseSalary);
			}
		}

		if (insertCourseScheduleModifyLogList.size() > 0) {
			courseScheduleModifyLogService.pushSchoolCourseChange(courseSchedules.get(0));
			courseScheduleModifyLogService.getDao().batchInsert(insertCourseScheduleModifyLogList);
		}
    }

	@Override
	public int endCountCourseSchedules(Map<String, Object> params) {
		return courseScheduleDao.endCountCourseSchedules(params);
	}

    @Override
    public void checkOnlineCategory(Integer studentNum) {
		Integer tenantId = TenantContextHolder.getTenantId();
		//查询云教室扣费标准
		TenantConfig tenantConfig = tenantConfigService.getOne(new WrapperUtil<TenantConfig>()
				.hasEq("tenant_id_", tenantId).queryWrapper());
		if (Objects.isNull(tenantConfig)) {
			return;
		}
		//云教室人数上限（含老师）
		Integer cloudRoomUpLimit = Optional.ofNullable(tenantConfig.getConfig())
				.map(c -> JSON.parseObject(c, TenantConfigDetail.class))
				.map(TenantConfigDetail::getCloud_room_rule)
				.map(TenantConfigDetail.CloudRoomRule::getCloud_room_up_limit).orElse(null);
		if(studentNum >= cloudRoomUpLimit){
			throw new BizException("当前机构只允许{}人以内的线上课",cloudRoomUpLimit);
		}
	}

	@Override
	public void checkOnlineCategory(List<CourseSchedule> courseSchedules) {
		List<CourseSchedule> collect = courseSchedules.stream().filter(e -> e.getTeachMode() == TeachModeEnum.ONLINE).collect(Collectors.toList());
		if(collect != null && collect.size() > 0){
			checkMaxStudentNum(collect);
		}
	}

	private void checkMaxStudentNum(List<CourseSchedule> courseSchedules){
		Set<Long> vipGroupIds = courseSchedules.stream().filter(e->e.getGroupType() == VIP).map(e -> Long.parseLong(e.getMusicGroupId())).collect(Collectors.toSet());
		if(vipGroupIds != null && vipGroupIds.size() > 0){
			List<VipGroup> vipGroupList = vipGroupDao.getVipGroupByIds(vipGroupIds);
			if(vipGroupList != null && vipGroupList.size() > 0){
				Set<Integer> categoryIds = vipGroupList.stream().map(e -> e.getVipGroupCategoryId()).collect(Collectors.toSet());
				List<VipGroupCategory> categories = vipGroupCategoryDao.findByIds(categoryIds);
				if(categories != null && categories.size() > 0){
					Integer maxStudentNum = categories.stream().max(Comparator.comparing(VipGroupCategory::getStudentNum)).map(e->e.getStudentNum()).get();
					this.checkOnlineCategory(maxStudentNum);
				}
			}
		}
	}

	@Override
	public void checkOnlineCategory(List<CourseSchedule> courseSchedules, TeachModeEnum teachMode) {
		if(teachMode == TeachModeEnum.ONLINE){
			checkMaxStudentNum(courseSchedules);
		}
	}

	@Override
	public Object queryMusicCoursePlan(Integer userId, Date startDate, Date endDate) {

		List<CourseSchedule> courseScheduleList = courseScheduleDao.findStudentCoursesWithIncludeDateRange(userId, GroupType.MUSIC, startDate, endDate);

		List<CourseSchedulePlanDto> result = new ArrayList<CourseSchedulePlanDto>();

		for(CourseSchedule courseSchedule : courseScheduleList){
			CourseSchedulePlanDto dto = new CourseSchedulePlanDto();
			dto.setCourseSchedule(courseSchedule);

			//获取课程规划
			MusicGroupTrainPlanSaveDto schoolTerm = musicGroupTrainPlanService.getSchoolTerm(courseSchedule);
			int courseNumNo = courseScheduleDao.getCourseNumNo(schoolTerm);
			List<MusicGroupTrainPlan> planList = musicGroupTrainPlanService.queryPlan(schoolTerm);
			if(planList.size() >= courseNumNo){
				dto.setMusicGroupTrainPlan(planList.get(courseNumNo - 1));
			}

			result.add(dto);
		}

		return result;
	}

    @Override
    public PageInfo<EvaluateCourseDto> courseEvaluateList(EvaluateCourseQueryInfo queryInfo) {
        PageInfo<EvaluateCourseDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        List<EvaluateCourseDto> dataList = null;
        int count = courseScheduleDao.countCourseEvaluateList(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = courseScheduleDao.queryCourseEvaluateList(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<StudentStandardDto> studyStandardList(EvaluateCourseQueryInfo queryInfo) {
        PageInfo<StudentStandardDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        List<StudentStandardDto> dataList = null;
        int count = courseScheduleDao.countStudyStandard(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = courseScheduleDao.queryStudyStandard(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public PageInfo<StudentStandardDto> queryStudyStandardWaitVisit(EvaluateCourseQueryInfo queryInfo) {
        PageInfo<StudentStandardDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);
        List<StudentStandardDto> dataList = null;
        int count = courseScheduleDao.countStudyStandardWaitVisit(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = courseScheduleDao.queryStudyStandardWaitVisit(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

	@Override
	@Transactional(rollbackFor = Exception.class)
	public HttpResponseResult<List<CourseSchedule>> createCourse(ClassGroupDto groupDto) {
		Date date = new Date();
		Integer classGroupId = groupDto.getClassGroupId();
		String musicGroupId = groupDto.getMusicGroupId();
		CourseScheduleType type = groupDto.getCourseScheduleType();
		List<CourseTimeDto> courseTimeDtoList = groupDto.getCourseTimeDtoList();
		MusicGroup musicGroup = musicGroupDao.get(musicGroupId);

		TeachModeEnum teachMode = TeachModeEnum.OFFLINE;
		if (type.equals(CourseScheduleType.HIGH_ONLINE) || type.equals(MUSIC_NETWORK)) {
			teachMode = TeachModeEnum.ONLINE;
		}
		if (CollectionUtils.isEmpty(courseTimeDtoList)) {
			throw new BizException("排课次数必须大于0");
		}

		Integer schoolId = teachMode.equals(TeachModeEnum.ONLINE) ? null : musicGroup.getSchoolId();

		List<Subject> subjectList = subjectDao.findBySubjectByIdList(groupDto.getSubjectIdList());

		String subjectNames = subjectList.stream().map(Subject::getName).collect(Collectors.joining("/"));

		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

		//2、获取班级学生
		List<Integer> studentIds = Arrays.stream(groupDto.getUserIds().split(",")).map(id -> Integer.valueOf(id)).distinct().collect(Collectors.toList());
		List<Long> oldCourseIdList = Arrays.stream(groupDto.getCourseIds().split(",")).map(id -> Long.parseLong(id)).collect(Collectors.toList());

		//获取可转换课程时长，学员数量，课程数量
		CourseConvertSumDto courseConvertSumDto = courseScheduleConvertDao.sumCourseConvert(oldCourseIdList);

		//计算每节课的课酬
		List<CourseScheduleTeacherSalary> courseScheduleTeacherSalaryList = new ArrayList<>();
		List<CourseSchedule> courseScheduleList = new ArrayList<>();
		//校验老师是否冲突
		courseTimeDtoList.stream().forEach(e->classGroupTeacherMapperService.checkTeachingBishop(e.getClassGroupTeacherMapperList()));
		LocalDateTime now = LocalDateTime.now().withHour(0).withMinute(0).withSecond(0).withNano(0);
		//获取排课总时长
		int totalCourseTimes = 0;
		long totalCourseDuration = 0;
		for (CourseTimeDto courseTimeDto : courseTimeDtoList) {
			//获取排课总次数
			totalCourseTimes += courseTimeDto.getExpectCourseNum();
			//获取排课总时长
			totalCourseDuration += Duration.between(LocalDateTime.parse(DateUtil.dateToString(courseTimeDto.getStartDate(), "yyyy-MM-dd") + " " + courseTimeDto.getStartClassTime() + ":00", formatter),
							LocalDateTime.parse(DateUtil.dateToString(courseTimeDto.getStartDate(), "yyyy-MM-dd") + " " + courseTimeDto.getEndClassTime() + ":00", formatter))
					.toMinutes();
		}
		//校验排课时长
		if(groupDto.getCheckType() == 0 && totalCourseDuration > courseConvertSumDto.getCourseMinute()){
			throw new BizException("操作失败：课排课时长不允许超过{}分钟",courseConvertSumDto.getCourseMinute());
		}
		//检验排课次数
		if(groupDto.getCheckType() == 1){
			int length = groupDto.getCourseIds().split(",").length;
			if(totalCourseTimes > length){
				throw new BizException("操作失败：排课次数不允许超过{}次",length);
			}
		}
		//获取节假日
		Set<String> holidayDays = new HashSet<>();
		Integer tenantId = TenantContextHolder.getTenantId();
		String holidaySetting = sysTenantConfigService.getTenantConfigValue(SysConfigService.HOLIDAY_SETTING, tenantId);
		if (StringUtils.isNotBlank(holidaySetting)) {
			holidayDays = new HashSet<>(JSON.parseArray(holidaySetting, String.class));
		}
		int generateCourseTimes = 0;
		WhileNode:
		while (true) {
			int dayOfWeek = now.getDayOfWeek().getValue();
			for (CourseTimeDto courseTimeDto : courseTimeDtoList) {
				List<ClassGroupTeacherMapper> newClassGroupTeacherMapperList = courseTimeDto.getClassGroupTeacherMapperList();
				Integer teacherId = newClassGroupTeacherMapperList.stream().filter(ct -> TeachTypeEnum.BISHOP.equals(ct.getTeacherRole())).findAny().get().getUserId();
				if (Objects.isNull(teacherId)) {
					throw new BizException("请设置主教");
				}
				if (courseTimeDto.getDayOfWeek() < 1 || courseTimeDto.getDayOfWeek() > 7) {
					throw new BizException("排课循环周期错误，请核查");
				}
				if (Objects.isNull(courseTimeDto.getStartDate()) || Objects.isNull(courseTimeDto.getEndDate())) {
					throw new BizException("排课循环周期错误，请核查");
				}

				//跳过节假日
				if (courseTimeDto.getHoliday() && holidayDays.contains(now.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")))) {
					continue;
				}

				if (!courseTimeDto.getDayOfWeek().equals(dayOfWeek)) continue;

				Date classDate = DateConvertor.toDate(now);

				if (courseTimeDto.getStartDate().compareTo(classDate) > 0
						|| courseTimeDto.getEndDate().compareTo(classDate) < 0
						|| courseTimeDto.getExpectCourseNum() <= courseTimeDto.getCourseNum()) {
					if (courseTimeDto.getEndDate().compareTo(classDate) < 0 && courseTimeDto.getExpectCourseNum() > courseTimeDto.getCourseNum()) {
						throw new BizException("在指定的排课时间段内({}-{})无法完成预计课时数的排课", DateUtil.dateToString(courseTimeDto.getStartDate(), "yyyy.MM.dd"), DateUtil.dateToString(courseTimeDto.getEndDate(), "yyyy.MM.dd"));
					}
					if (totalCourseTimes <= generateCourseTimes) {
						break WhileNode;
					}
					continue;
				}

				//课时长度
				long classCourseDuration = Duration.between(LocalDateTime.parse(DateUtil.dateToString(courseTimeDto.getStartDate(), "yyyy-MM-dd") + " " + courseTimeDto.getStartClassTime() + ":00", formatter),
								LocalDateTime.parse(DateUtil.dateToString(courseTimeDto.getStartDate(), "yyyy-MM-dd") + " " + courseTimeDto.getEndClassTime() + ":00", formatter))
						.toMinutes();

				totalCourseDuration += classCourseDuration;

				courseTimeDto.setCourseNum(courseTimeDto.getCourseNum() + 1);
				generateCourseTimes += 1;

				if (totalCourseTimes < generateCourseTimes) {
					break WhileNode;
				}

				String startClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getStartClassTime() + ":00";
				String endClassTime = DateUtil.getDate(classDate) + " " + courseTimeDto.getEndClassTime() + ":00";

				CourseSchedule courseSchedule = new CourseSchedule();
				courseSchedule.setSchoolId(schoolId);
				courseSchedule.setClassGroupId(classGroupId);
				courseSchedule.setStatus(CourseStatusEnum.NOT_START);
				courseSchedule.setClassDate(classDate);
				courseSchedule.setStartClassTime(DateUtil.stringToDate(startClassTime));
				if (date.compareTo(courseSchedule.getStartClassTime()) > 0) {
					throw new BizException("课程开始时间不得早于当前时间");
				}
				courseSchedule.setEndClassTime(DateUtil.stringToDate(endClassTime));
				courseSchedule.setCreateTime(date);
				courseSchedule.setUpdateTime(date);
				courseSchedule.setTeachMode(teachMode);
				courseSchedule.setMusicGroupId(musicGroupId);
				courseSchedule.setType(groupDto.getCourseScheduleType());
				courseSchedule.setName(subjectNames + "-" + groupDto.getCourseScheduleType().getMsg());
				courseSchedule.setGroupType(groupDto.getGroupType());
				courseSchedule.setTeacherId(teacherId);
				courseSchedule.setActualTeacherId(teacherId);
				courseSchedule.setOrganId(musicGroup.getOrganId());
				courseSchedule.setConvertFlag(groupDto.getConvertFlag());
				courseScheduleDao.insert(courseSchedule);
				courseScheduleList.add(courseSchedule);
				courseScheduleTeacherSalaryService.initMusicGroupCourseTeacherSalary(newClassGroupTeacherMapperList,courseSchedule,
						musicGroup,courseScheduleTeacherSalaryList);
			}
			now = now.plusDays(1);
		}
		this.checkNewCourseSchedules(courseScheduleList, false, false);
		if(!groupDto.getConfirmGenerate()){
			return BaseController.succeed(courseScheduleList);
		}

		classGroupService.updateTotalClassTimes(classGroupId, courseScheduleList.size());

		//老师结算表
		if (!groupDto.getAllowZeroSalary() && courseScheduleTeacherSalaryList.stream().anyMatch(e->e.getExpectSalary().compareTo(BigDecimal.ZERO) == 0)) {
			return BaseController.failed(HttpStatus.MULTI_STATUS, "当前课程课酬预计为0,是否继续");
		}
		groupDto.getCourseScheduleTeacherSalaryList().addAll(courseScheduleTeacherSalaryList);
		//生成学生结算表
		groupDto.getCourseScheduleStudentPayments().addAll(courseScheduleStudentPaymentService.init(courseScheduleList,studentIds));
		courseScheduleModifyLogService.pushSchoolCourseChange(courseScheduleList.get(0),musicGroup);
		return BaseController.succeed(courseScheduleList);
	}

    @Override
    public List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto> getSummaryOfCurrentSemesterCourses(Integer coopId) {
		//获取合作单位关联的所有乐团列表
		List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
		List<CourseSchedule.CourseScheduleType> courseTypeList = Arrays.asList(
				CourseScheduleType.SINGLE,
				CourseScheduleType.MIX,
				CourseScheduleType.HIGH,
				CourseScheduleType.HIGH_ONLINE,
				CourseScheduleType.TRAINING_SINGLE,
				CourseScheduleType.TRAINING_MIX,
				CourseScheduleType.COMPREHENSIVE,
				CourseScheduleType.CLASSROOM
		);
		if(CollectionUtils.isEmpty(musicGroupIds)){
			return this.initClassOfSummary(courseTypeList,null);
		}
		//获取当前学期
		Date date = new Date();
		String startTerm = DateUtil.getStartTerm(date);
		String endTerm = DateUtil.getEndTerm(date);
		List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto> summary =
				courseScheduleDao.getSummaryOfCurrentSemesterCourses(musicGroupIds, startTerm, endTerm);
		return this.initClassOfSummary(courseTypeList,summary);
    }

	private List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto>
	initClassOfSummary(List<CourseSchedule.CourseScheduleType> courseTypeList,List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto> summary){
		Map<CourseScheduleType, DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto>
				map = new HashMap<>();
		if(CollectionUtils.isNotEmpty(summary)){
			map = summary.stream().collect(Collectors.groupingBy(e -> e.getScheduleType(), Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
		}
		List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto> result = new ArrayList<>();
		for (CourseScheduleType scheduleType : courseTypeList) {
			DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto dto = map.get(scheduleType);
			if(Objects.isNull(dto)){
				dto = new DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDetailDto();
				dto.setScheduleType(scheduleType);
			}
			result.add(dto);
		}
		return result;
	}

	@Override
	public List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterMapDto> getDailySummaryOfClassesForTheCurrentSemester(Integer coopId) {
		//获取合作单位关联的所有乐团列表
		List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
		//获取当前学期
		Date date = new Date();
		String startTerm = DateUtil.getStartTerm(date);
		String endTerm = DateUtil.getEndTerm(date);
		List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDto> summary;
		if(CollectionUtils.isEmpty(musicGroupIds)){
			summary = new ArrayList<>();
		}else {
			summary = courseScheduleDao.getDailySummaryOfClassesForTheCurrentSemester(musicGroupIds, startTerm, endTerm);
		}
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
		LocalDate startDate = LocalDate.parse(startTerm, formatter);
		LocalDate endDate = LocalDate.parse(endTerm, formatter);
		LocalDate now = LocalDate.now();
		List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterMapDto> result = new ArrayList<>();
		LocalDate currentWeekStart = startDate.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
		DateTimeFormatter format = DateTimeFormatter.ofPattern("MM月dd日");
		StringBuffer sb;
		StringBuffer sb1;
		int week = 0;
		List<CourseSchedule.CourseScheduleType> courseTypeList = Arrays.asList(
				CourseScheduleType.SINGLE,
				CourseScheduleType.MIX,
				CourseScheduleType.HIGH,
				CourseScheduleType.HIGH_ONLINE,
				CourseScheduleType.TRAINING_SINGLE,
				CourseScheduleType.TRAINING_MIX,
				CourseScheduleType.COMPREHENSIVE,
				CourseScheduleType.CLASSROOM
		);
		while (!currentWeekStart.isAfter(endDate)) {
			DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterMapDto vo =
					new DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterMapDto();
			String weekStart = currentWeekStart.format(format);
			sb = new StringBuffer("第").append(toChineseNumber(week)).append("周");
			LocalDate currentWeekEnd = currentWeekStart.with(TemporalAdjusters.nextOrSame(DayOfWeek.SUNDAY));
			String endStart = currentWeekEnd.format(format);
			sb1 = new StringBuffer("(").append(weekStart).append("-").append(endStart).append(")");
			// 判断课程状态
			String courseStatus;
			if (currentWeekEnd.isBefore(now)) {
				courseStatus = "END";
			} else if (currentWeekStart.isAfter(now)) {
				courseStatus = "NOT_START";
			} else {
				courseStatus = "ING";
			}
			vo.setCourseStatus(courseStatus);
			List<DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDto> currentWeekDates = new ArrayList<>();
			for (DailySummaryOfClassesForTheCurrentSemesterWrapper.DailySummaryOfClassesForTheCurrentSemesterDto dto : summary) {
				if (!dto.getCourseDate().isBefore(currentWeekStart) && !dto.getCourseDate().isAfter(currentWeekEnd)) {
					currentWeekDates.add(dto);
				}
			}
			week++;
			currentWeekStart = currentWeekEnd.plusDays(1);
			if(CollectionUtils.isNotEmpty(currentWeekDates)){
				currentWeekDates.forEach(dto -> dto.setDailySummaryOfClassesForTheCurrentSemesterDetailDtos(initClassOfSummary(courseTypeList, dto.getDailySummaryOfClassesForTheCurrentSemesterDetailDtos())));
				vo.setWeek(sb.toString());
				vo.setWeekTime(sb1.toString());
				vo.setDailySummaryOfClassesForTheCurrentSemesterDtos(currentWeekDates);
				result.add(vo);
			}
		}
		return result;
	}

	private static String toChineseNumber(int n) {
		String[] chineseNumbers = {"一", "二", "三", "四", "五", "六", "七", "八", "九", "十", "十一", "十二", "十三", "十四", "十五", "十六", "十七", "十八",
				"十九", "二十", "二十一", "二十二", "二十三", "二十四", "二十五", "二十六", "二十七", "二十八", "二十九", "三十", "三十一", "三十二", "三十三", "三十四",
				"三十五", "三十六", "三十七", "三十八", "三十九", "四十", "四十一", "四十二", "四十三", "四十四", "四十五", "四十六", "四十七", "四十八", "四十九", "五十"};
		if (n <= 50) {
			return chineseNumbers[n];
		} else {
			return Integer.toString(n);
		}
	}

	@Override
	public List<ClassesForDayDto> getListClassesForDay(Integer coopId,String classDate) {
		//获取合作单位关联的所有乐团列表
		List<String> musicGroupIds = musicGroupDao.findNormalByCooperationId(coopId);
		if(CollectionUtils.isEmpty(musicGroupIds)){
			return Collections.EMPTY_LIST;
		}
		List<ClassesForDayDto> listClassesForDay = courseScheduleDao.getListClassesForDay(musicGroupIds, classDate,null);
		this.initClasses(listClassesForDay);
		return listClassesForDay;
	}

	@Override
	public ClassesForDayDto getCourseDetail(Long courseId) {
		List<ClassesForDayDto> listClassesForDay = courseScheduleDao.getListClassesForDay(null, null,courseId);
		this.initClasses(listClassesForDay);
		return listClassesForDay.get(0);
	}


    /**
     * 查询课程信息
     *
     * @param queryInfo 查询条件
     */
    @Override
    public IPage<TeachingPointWrapper.CourseTeachingPoint> teachingPointCourse(TeachingPointWrapper.TeachingPointQuery queryInfo) {
        IPage<TeachingPointWrapper.CourseTeachingPoint> page = com.microsvc.toolkit.common.response.paging.QueryInfo.getPage(queryInfo);
        List<TeachingPointWrapper.CourseTeachingPoint> courseTeachingPoints = courseSchedulePlusMapper
                .teachingPointCourse(page, queryInfo);
        if (CollectionUtils.isEmpty(courseTeachingPoints)) {
            return page;
        }
        // 设置老师信息

        // 老师ID集合
        List<Integer> teacherIds = courseTeachingPoints.stream()
                .map(TeachingPointWrapper.CourseTeachingPoint::getTeacherId)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        Map<Integer, Teacher> teacherMap = teacherService.getMapByIds(teacherIds);

        // 设置签到/签退信息

        // 课程ID集合
        List<Long> courseIds = courseTeachingPoints.stream()
                .map(TeachingPointWrapper.CourseTeachingPoint::getCourseId)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
		//获取乐团编号
		List<String> musicGroupIds = courseTeachingPoints.stream().map(TeachingPointWrapper.CourseTeachingPoint::getMusicGroupId).collect(Collectors.toList());
		Map<String,String> coopNameMap = new HashMap<>();
		if (CollectionUtils.isNotEmpty(musicGroupIds)) {
			coopNameMap = cooperationOrganService.getDao().
					queryByGroupIds(musicGroupIds).stream().collect(Collectors.toMap(e -> e.getMusicGroupId(), e -> e.getName()));
		}
		//获取分部编号
		List<Integer> organIds = courseTeachingPoints.stream().map(TeachingPointWrapper.CourseTeachingPoint::getOrganId).collect(Collectors.toList());
		Map<Integer,Organization> organNameMap = new HashMap<>();
		if (CollectionUtils.isNotEmpty(musicGroupIds)) {
			organNameMap = organizationService.getMapByIds(organIds);
		}

        Map<Long, TeacherAttendance> attendanceMap = teacherAttendanceService.getMapByCourseIds(courseIds);

        for (TeachingPointWrapper.CourseTeachingPoint courseTeachingPoint : courseTeachingPoints) {
			courseTeachingPoint.setCoopName(coopNameMap.get(courseTeachingPoint.getMusicGroupId()));
			Organization organization = organNameMap.get(courseTeachingPoint.getOrganId());
			if (Objects.nonNull(organization)){
				courseTeachingPoint.setOrganName(organization.getName());
			}
			// 设置老师信息
            if (courseTeachingPoint.getTeacherId() != null) {
                Teacher teacher = teacherMap.get(courseTeachingPoint.getTeacherId());
                if (teacher != null) {
                    courseTeachingPoint.setTeacherName(teacher.getRealName());
                    courseTeachingPoint.setTeacherAvatar(teacher.getAvatar());
                }
            }

            // 设置签到/签退信息
            TeacherAttendance attendance = attendanceMap.get(courseTeachingPoint.getCourseId());
            if (attendance != null) {
                courseTeachingPoint.setSignPhoto(attendance.getSignInAttachments());
                courseTeachingPoint.setSignOutPhoto(attendance.getSignOutAttachments());
            }
            // 设置上课时间
            courseTeachingPoint.setStartClassTime(DateUtil.startDateAndEndTime(courseTeachingPoint.getCourseDate(), courseTeachingPoint.getStartClassTime()));
            courseTeachingPoint.setEndClassTime(DateUtil.startDateAndEndTime(courseTeachingPoint.getCourseDate(), courseTeachingPoint.getEndClassTime()));

        }

        return page.setRecords(courseTeachingPoints);
    }

    @Override
    public IPage<TeachingPointWrapper.ClassTeachingPoint> teachingPointClass(TeachingPointWrapper.TeachingPointQuery queryInfo) {
        IPage<TeachingPointWrapper.ClassTeachingPoint> page = com.microsvc.toolkit.common.response.paging.QueryInfo.getPage(queryInfo);
        List<TeachingPointWrapper.ClassTeachingPoint> courseTeachingPoints = courseSchedulePlusMapper
                .teachingPointClass(page, queryInfo);
        if (CollectionUtils.isEmpty(courseTeachingPoints)) {
            return page;
        }

        // 班级主教信息

        // 班级ID集合
        List<Integer> classIds = courseTeachingPoints.stream()
                .map(TeachingPointWrapper.ClassTeachingPoint::getClassGroupId)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        Map<Integer, Teacher> teacherMap = classGroupTeacherMapperService.getTeacherByClassIds(classIds);

        for (TeachingPointWrapper.ClassTeachingPoint courseTeachingPoint : courseTeachingPoints) {

            // 设置老师信息
            Teacher teacher = teacherMap.get(courseTeachingPoint.getClassGroupId());
            if (teacher != null) {
                courseTeachingPoint.setTeacherId(teacher.getId());
                courseTeachingPoint.setTeacherName(teacher.getRealName());
                courseTeachingPoint.setTeacherAvatar(teacher.getAvatar());
            }
        }
        return page.setRecords(courseTeachingPoints);

    }


    /**
     * 设置教学点
     *
     * @param teachingPoint 教学点信息
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean teachingPoint(TeachingPointWrapper.TeachingPoint teachingPoint) {

        ClassGroup classGroup = new ClassGroup();
        classGroup.setId(teachingPoint.getClassGroupId());
        classGroup.setTeachingPoint(teachingPoint.getTeachingPoint());
        classGroupDao.updateTeachingPointById(classGroup);

        // 未开始的课程修改教学点
        courseScheduleDao.updateTeachingPointByClassGroupId(teachingPoint.getClassGroupId(), teachingPoint.getTeachingPoint());
        return true;
    }

    @Override
    public List<CourseSchedule> getByTeacherAndTime(Integer teacherId, Date startTime, Date endTime) {
        return courseScheduleDao.getByTeacherAndTime(teacherId,startTime,endTime);
    }


    /**
     * 设置课程的场地
     *
     * @param teachingPoint
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean teachingPointCourse(TeachingPointWrapper.TeachingPoint teachingPoint) {
        CourseSchedule courseSchedule = courseScheduleDao.get(Long.valueOf(teachingPoint.getCourseScheduleId()));
        if (courseSchedule == null) {
            throw new BizException("课程不存在");
        }
        courseSchedule.setTeachingPoint(teachingPoint.getTeachingPoint());

        courseScheduleDao.update(courseSchedule);
        return true;
    }

	/**
	 * 更新课程共享方式
	 *
	 * @param updateCourseShareModeDto 模型入参
	 */
	@Transactional(rollbackFor = Exception.class)
	@Override
	public void updateCourseScheduleShareMode(UpdateCourseShareModeDto updateCourseShareModeDto) {
		Long courseId = updateCourseShareModeDto.getCourseId();
		CourseSchedule courseSchedule = courseScheduleDao.get(courseId);
		if (courseSchedule == null) {
			throw new BizException("课程不存在");
		}
		ShareModeEnum shareMode = updateCourseShareModeDto.getShareMode();
		if (ShareModeEnum.NO_SHARE.equals(shareMode)) {
			throw new BizException("不支持修改为不共享模式");
		}
		if (!shareMode.equals(courseSchedule.getShareMode())) {
			courseSchedule.setShareMode(shareMode);
			courseScheduleDao.update(courseSchedule);
		}
	}

    @Override
    public CourseScheduleDto detailLiveCourse(Long courseScheduleId) {
        CourseScheduleDto courseSchedule = liveGroupPlusMapper.detailLiveCourse(courseScheduleId);



        Integer studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());

        Date startDate = DateUtil.startDateAndEndTime(courseSchedule.getClassDate(), courseSchedule.getStartClassTime());
        courseSchedule.setStartClassTime(startDate);
        Date endDate = DateUtil.startDateAndEndTime(courseSchedule.getClassDate(), courseSchedule.getEndClassTime());
        courseSchedule.setEndClassTime(endDate);
        startDate = DateUtil.addMinutes(startDate, -studentRemindTime);
        // 课程是否开始
        if(CourseStatusEnum.NOT_START.equals(courseSchedule.getStatus()) && startDate.before(new Date())) {
            courseSchedule.setStatus(CourseStatusEnum.UNDERWAY);
        }
        // 设置老师信息
        Teacher teacher = teacherService.get(courseSchedule.getActualTeacherId());
        if (teacher != null) {
            courseSchedule.setTeacherId(teacher.getId());
            courseSchedule.setTeacherName(teacher.getRealName());
            courseSchedule.setHeadUrl(teacher.getAvatar());
        }

        // 查询直播课程组
        VipGroup vipGroup = vipGroupDao.findByCourseSchedule(courseScheduleId);
        if (vipGroup != null) {
            ImLiveBroadcastRoomDto imLiveBroadcastRoomDto = JSON.parseObject(vipGroup.getLiveConfigJson(), ImLiveBroadcastRoomDto.class);
            if (imLiveBroadcastRoomDto != null) {
                courseSchedule.setPreTemplate(imLiveBroadcastRoomDto.getPreTemplate());
            }
        }

        return courseSchedule;

    }

	@Override
	public void checkLiveAdjust(CourseSchedule oldCourseSchedule,String startRemindTime) {
		// 直播课
		if (GroupType.LIVE.equals(oldCourseSchedule.getGroupType())){
			if(org.springframework.util.StringUtils.isEmpty(startRemindTime)){
				startRemindTime = sysConfigDao.findConfigValue("live_class_start_remind_time");
			}
			if(org.springframework.util.StringUtils.isEmpty(startRemindTime)){
				startRemindTime = "30";
			}
			if(DateUtil.minutesBetween(new Date(),oldCourseSchedule.getStartClassTime()) < Integer.parseInt(startRemindTime)
					&& DateUtil.minutesBetween(new Date(),oldCourseSchedule.getEndClassTime()) >= 0){
				throw new BizException("直播课课程开始前{}分钟不允许调整",startRemindTime);
			}
		}
	}

	@Override
	public void checkLiveAdjust(List<CourseSchedule> courseSchedules) {
		String startRemindTime = sysConfigDao.findConfigValue("live_class_start_remind_time");
		courseSchedules.forEach(e->checkLiveAdjust(e,startRemindTime));
	}

	@Override
	public void resetLiveRemind(CourseSchedule courseSchedule) {
		if (GroupType.LIVE.equals(courseSchedule.getGroupType())) {
			// 重置连堂课，关联直播间，消息推送状态
			courseSchedule.setLiveRemind(0);
			courseSchedule.setContinuousCourse(false);
			courseSchedule.setLiveRoomId("");
		}
	}

	@Override
	public void resetLiveRemind(List<CourseSchedule> courseSchedules) {
		courseSchedules.forEach(e->resetLiveRemind(e));
	}

	@Override
	public void sendChatRoomMessage(List<CourseSchedule> courseSchedules) {

		List<CourseSchedule> collect = courseSchedules.stream()
				.filter(x -> x.getGroupType() == LIVE).collect(Collectors.toList());

		for (CourseSchedule item : collect) {

			// 是否是连堂课
			String continueCourseTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.ONLINE_CONTINUE_COURSE_TIME, item.getTenantId());
			if (StringUtils.isEmpty(continueCourseTime)) {
				continueCourseTime = "5";
			}

			// 查询最新课程信息
			CourseSchedule newCourseSchedule = courseScheduleDao.get(item.getId());
			// 连堂课更新标识
			boolean updateFlag = true;
			// 新课时间
			Date newCourseScheduleDate = getDateTime(newCourseSchedule.getClassDate(), newCourseSchedule.getStartClassTime()).toDate();
			Date oldCourseScheduleDate = getDateTime(item.getClassDate(), item.getStartClassTime()).toDate();
			if (newCourseScheduleDate.getTime() > oldCourseScheduleDate.getTime()) {
				newCourseSchedule = item;
				// 更新标识
				updateFlag = false;
			}

			List<CourseSchedule> newCourseSchedules = Lists.newArrayList();
			// 直播课关联直播间编号
			String liveRoomId = getCourseScheduleLiveRoomId(newCourseSchedule, continueCourseTime, newCourseSchedules, Lists.newArrayList(), updateFlag);

			if (StringUtils.isBlank(liveRoomId)) {
				liveRoomId = item.getLiveRoomId();
			}

			ImLiveBroadcastRoom liveRoom = imLiveBroadcastRoomService.getByRoomUid(liveRoomId);
			if (Objects.isNull(liveRoom)) {
				log.warn("liveRoom sendChatRoomMessage is null, liveRoomId:{}, updateFlag={}, item.liveId={}", liveRoomId,
						updateFlag, item.getLiveRoomId());

				/*boolean anyMatch = newCourseSchedules.stream().anyMatch(x -> x.getId().longValue() == x.getId())
						&& newCourseSchedules.size() > 1;
				if (!updateFlag && (Optional.ofNullable(item.getContinuousCourse()).orElse(false) || anyMatch)) {
					throw new BizException("课程信息已变更，请重新操作");
				}*/
				return;
			}

			try {
				LivePluginService pluginService = livePluginContext.getPluginService(liveRoom.getServiceProvider());

				LiveRoomMessage message = new LiveRoomMessage();
				message.setIsIncludeSender(1);
				message.setFromUserId(newCourseSchedule.getActualTeacherId().toString());
				message.setToChatRoomId(liveRoom.getRoomUid());
				message.setObjectName(LiveRoomMessage.LIVE_STATUS_CHANGE);

				// 发送用户信息
				LiveRoomMessage.MessageUser messageUser = LiveRoomMessage.MessageUser.builder()
						.sendUserId("")
						.sendUserName("")
						.avatarUrl("")
						.build();

				message.setContent(LiveRoomMessage.MessageContent.builder()
						.sendUserInfo(messageUser).build());

				pluginService.sendChatRoomMessage(message);
				log.info("liveRoom sendChatRoomMessage success, liveRoomId:{}", liveRoomId);
			} catch (Exception e) {
				log.error("liveRoom sendChatRoomMessage error", e);
			}

		}

		// 直播课消息推送
		/*ExecutorService executor = Executors.newCachedThreadPool();
		CompletableFuture.runAsync(() ->
			courseSchedules.parallelStream().filter(e->e.getGroupType() == LIVE).forEach(item -> {

			}),executor
		);
		executor.shutdown();*/
	}

    @Override
    public List<ExportCourseIncomeDto> exportCourseIncome(String month) {
		String cutFirstDayOfMonth = DateUtil.dateToString(DateUtil.getFirstDayOfMonth(DateUtil.stringToDate(month,DateUtil.ISO_YEAR_MONTH_FORMAT)),DateUtil.ISO_EXPANDED_DATE_FORMAT);
		String cutLastDayOfMonth = DateUtil.dateToString(DateUtil.getLastDayOfMonth(DateUtil.stringToDate(month,DateUtil.ISO_YEAR_MONTH_FORMAT)),DateUtil.ISO_EXPANDED_DATE_FORMAT);
		return courseScheduleDao.exportCourseIncome(cutFirstDayOfMonth,cutLastDayOfMonth);
    }

    @Override
    public void rtcRoomCreate() {

        List<TenantInfo> list = tenantInfoService.list();
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        for (TenantInfo tenantInfo : list) {
            //是否提前进入教室
            String courseBeforeBufferTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_BEFORE_BUFFER_TIME, tenantInfo.getId());
            if (StringUtils.isEmpty(courseBeforeBufferTime)) {
                courseBeforeBufferTime = "5";
            }
            Integer beforeTime = Integer.parseInt(courseBeforeBufferTime) + 5;

            List<CourseSchedule> scheduleList = courseScheduleDao.getNotStartOnlineNoLive(beforeTime, tenantInfo.getId());
            if (CollectionUtils.isEmpty(scheduleList)) {
                continue;
            }
            for (CourseSchedule courseSchedule : scheduleList) {
                try {
                    RTCRoomPluginService pluginService;
                    if (StringUtils.isBlank(courseSchedule.getServiceProvider())) {
                        pluginService = rtcRoomPluginContext.getPluginService();
                    } else {
                        pluginService = rtcRoomPluginContext.getPluginService(courseSchedule.getServiceProvider());
                    }
                    // 群组帐号注册
                    Teacher teacher = teacherDao.get(courseSchedule.getTeacherId());
                    if (Objects.nonNull(teacher)) {
                        try {
                            pluginService.register(courseSchedule.getTeacherId().toString(), teacher.getRealName(), teacher.getAvatar());
                        } catch (Exception e) {
                            log.error("直播房间群主注册失败: userId={}", courseSchedule.getTeacherId(), e);
                        }
                    }

                    //记录用户实际选择的房间
                    String roomId = courseSchedule.getId().toString();
                    if (courseSchedule.getGroupType() == GroupType.COMM) {
                        roomId = "I" + roomId;
                    } else {
                        roomId = "S" + roomId;
                    }

                    // 生成群组
                    pluginService.chatRoomCreate(roomId, courseSchedule.getName(), courseSchedule.getTeacherId().toString());
                    courseScheduleDao.updateLiveRemind(courseSchedule.getId(),1);
                } catch (Exception e) {
                    log.error("创建rtc房间失败", e);
                }
            }
        }
        

    }

    @Override
    public void destroyRtcRoom() {

        // 查询结束时间在昨天到现在的课程
        List<CourseSchedule> scheduleList = courseScheduleDao.getEndTimeBetweenYesterdayAndNow();
        if (CollectionUtils.isEmpty(scheduleList)) {
            return;
        }
        for (CourseSchedule courseSchedule : scheduleList) {
            try {
                RTCRoomPluginService pluginService;
                if (StringUtils.isBlank(courseSchedule.getServiceProvider())) {
                    pluginService = rtcRoomPluginContext.getPluginService();
                } else {
                    pluginService = rtcRoomPluginContext.getPluginService(courseSchedule.getServiceProvider());
                }
                if (TencentCloudRTCPlugin.PLUGIN_NAME.equals(pluginService.pluginName())) {
                    // 腾讯云群销毁
                    String roomId = courseSchedule.getId().toString();
                    if (courseSchedule.getGroupType() == GroupType.COMM) {
                        roomId = "I" + roomId;
                    } else {
                        roomId = "S" + roomId;
                    }
                    pluginService.chatRoomDestroy(roomId);
                }
            } catch (Exception e) {
                log.error("销毁rtc房间失败", e);
            }
        }
    }

    @Override
    public List<CourseScheduleStudentDto> queryDetailList(CourseDetailQueryInfo courseDetailQueryInfo) {


        // 直播课判断时间，提前30分钟将课程改为进行中
        Integer studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());

        if (StringUtils.isNotBlank(courseDetailQueryInfo.getGroupType())) {
            courseDetailQueryInfo.setGroupType(courseDetailQueryInfo.getGroupType().toUpperCase());
        }
        courseDetailQueryInfo.setNotStartCourseTime(studentRemindTime);
        List<CourseScheduleStudentDto> courseScheduleStudentDtos = courseScheduleDao.queryDetailList(courseDetailQueryInfo);


        // 直播房间号ID
        List<String> liveRoomIds = courseScheduleStudentDtos.stream().map(e -> e.getLiveRoomId()).filter(Objects::nonNull).collect(Collectors.toList());

        Map<String, List<ImLiveRoomVideoVo>> imLiveRoomVideoVoMap = getLiveRecord(liveRoomIds);
        courseScheduleStudentDtos.forEach(e -> {
            if (imLiveRoomVideoVoMap != null && imLiveRoomVideoVoMap.containsKey(e.getLiveRoomId())) {
                e.setHasVideoRecord(true);
            } else {
                e.setHasVideoRecord(false);
            }
        });


        // 直播课判断时间，提前30分钟将课程改为进行中
        for (CourseScheduleStudentDto record : courseScheduleStudentDtos) {
            if (CourseStatusEnum.NOT_START.equals(record.getCourseStatus())
                    && courseDetailQueryInfo.getGroupType().toUpperCase(Locale.ROOT).equals(GroupType.LIVE.getCode())) {
                Date date = DateUtil.addMinutes(new Date(), studentRemindTime);
                if (date.after(DateUtil.toDateTime(record.getStartClassTime()))) {
                    record.setCourseStatus(CourseStatusEnum.UNDERWAY);
                }
            }
        }

        return courseScheduleStudentDtos;
    }

    @Override
    public IPage<CourseScheduleDto> liveCoursePage(LiveGroupWrapper.LiveCoursePageQuery queryInfo) {
        IPage<CourseScheduleDto> page = com.microsvc.toolkit.common.response.paging.QueryInfo.getPage(queryInfo);

        // 直播课判断时间，提前30分钟将课程改为进行中
        Integer studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());

        queryInfo.setNotStartCourseTime(studentRemindTime);
        List<CourseScheduleDto> courseScheduleDtos = liveGroupPlusMapper.liveCoursePage(page, queryInfo);
        if (CollectionUtils.isEmpty(courseScheduleDtos)) {
            return page;
        }
        // id 集合
        List<Long> courseIds = courseScheduleDtos.stream().map(CourseScheduleDto::getId).collect(Collectors.toList());
        List<CourseScheduleDto> teacherCourseFormat = getTeacherCourseFormat(queryInfo.getTeacherId(), new Date(),
                queryInfo.getTenantId(), courseScheduleDtos, courseIds, courseIds);


        for (CourseScheduleDto record : teacherCourseFormat) {
            if (CourseScheduleType.LIVE.equals(record.getType()) && CourseStatusEnum.NOT_START.equals(record.getStatus())) {
                Date date = DateUtil.addMinutes(new Date(), studentRemindTime);
                if (date.after(record.getStartClassTime())) {
                    record.setStatus(CourseStatusEnum.UNDERWAY);
                }
            }
        }
        return page.setRecords(teacherCourseFormat);
    }

    @Override
	public List<CourseStudentDto> queryCourseStudentList(Long courseId) {
		List<CourseStudentDto> courseStudentDtos = studentAttendanceDao.queryCourseStudentList(courseId);
		if(CollectionUtils.isNotEmpty(courseStudentDtos)){
			List<Integer> studentIds = courseStudentDtos.stream().map(e -> e.getStudentId()).collect(Collectors.toList());
			List<SimpleUserDto> usersSimpleInfo = teacherDao.getUsersSimpleInfo(studentIds);
			Map<Integer, SimpleUserDto> simpleUserDtoMap = usersSimpleInfo.stream().collect(Collectors.groupingBy(e -> e.getUserId(), Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
			//获取
			CourseSchedule courseSchedule = courseScheduleDao.get(courseId);
			for (CourseStudentDto dto : courseStudentDtos) {
				SimpleUserDto userDto = simpleUserDtoMap.get(dto.getStudentId());
				if(Objects.nonNull(userDto)){
					dto.setStudentName(userDto.getNickName());
					dto.setStudentAvatar(userDto.getAvatar());
				}
				dto.setCourseStatus(courseSchedule.getStatus());
			}
		}
		return courseStudentDtos;
	}

	private void initClasses(List<ClassesForDayDto> listClassesForDay){
		if(CollectionUtils.isNotEmpty(listClassesForDay)){
			Set<String> musicGroupIds = listClassesForDay.stream().map(e -> e.getMusicGroupId()).collect(Collectors.toSet());
			//获取乐团名称
			Map<String,String> musicNameMap = MapUtil.convertIntegerMap(musicGroupDao.queryMusicGroupNameMap(musicGroupIds));
			//获取老师基本信息
			List<Integer> teacherIds = listClassesForDay.stream().map(e -> e.getTeacherId()).collect(Collectors.toList());
			List<SimpleUserDto> usersSimpleInfo = teacherDao.getUsersSimpleInfo(teacherIds);
			Map<Integer, SimpleUserDto> simpleUserDtoMap = usersSimpleInfo.stream().collect(Collectors.groupingBy(e -> e.getUserId(), Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
			//统计考勤信息
			List<Long> courseIds = listClassesForDay.stream().map(e -> e.getCourseId()).collect(Collectors.toList());
			List<ClassesForDayDto> dtoList = studentAttendanceDao.statisticalAttendanceInformation(courseIds);
			Map<Long, ClassesForDayDto> dtoMap = dtoList.stream().collect(Collectors.groupingBy(e -> e.getCourseId(), Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0))));
			//获取课程关联的作业
			List<CourseHomework> homeworks = courseHomeworkService.findHomeworkByCourse(courseIds);
			Map<Long, Long> homeworkMap = new HashMap<>();
			if(CollectionUtils.isNotEmpty(homeworks)){
				homeworkMap = homeworks.stream().collect(Collectors.groupingBy(e -> e.getCourseScheduleId(), Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0).getId())));
			}
			//获取课程关联的测评
			List<LessonExamination> lessonExaminations = lessonExaminationService.lambdaQuery().in(LessonExamination::getCourseScheduleId, courseIds).list();
			Map<Long, Long> examinationMap = new HashMap<>();
			if(CollectionUtils.isNotEmpty(lessonExaminations)){
				examinationMap = lessonExaminations.stream().collect(Collectors.groupingBy(e -> e.getCourseScheduleId(), Collectors.collectingAndThen(Collectors.toList(), v -> v.get(0).getId())));
			}
			for (ClassesForDayDto dto : listClassesForDay) {
				SimpleUserDto userDto = simpleUserDtoMap.get(dto.getTeacherId());
				if(Objects.nonNull(userDto)){
					dto.setTeacherName(userDto.getRealName());
					dto.setTeacherAvatar(userDto.getAvatar());
				}
				dto.setMusicGroupName(musicNameMap.get(dto.getMusicGroupId()));
				ClassesForDayDto dayDto = dtoMap.get(dto.getCourseId());
				if(Objects.nonNull(dayDto)){
					dto.setStudentNum(dayDto.getStudentNum());
					if(dto.getCourseStatus() != CourseStatusEnum.NOT_START){
						dto.setAttendanceNormalNum(dayDto.getAttendanceNormalNum());
						dto.setAttendanceLateNum(dayDto.getAttendanceLateNum());
						dto.setAttendanceLeaveNum(dayDto.getAttendanceLeaveNum());
						dto.setAttendanceTruantNum(dayDto.getAttendanceTruantNum());
					}
				}
				Long homeworkId = homeworkMap.get(dayDto.getCourseId());
				if(homeworkId != null){
					dto.setHomeworkType("HOMEWORK");
					dto.setHomeworkId(homeworkId);
				}
				Long lessonId = examinationMap.get(dayDto.getCourseId());
				if(lessonId != null){
					dto.setHomeworkType("LESSON");
					dto.setHomeworkId(lessonId);
				}
			}
		}
	}


	/**
	 * 获取老师直播课程时间
	 *
	 * @param courseScheduleId 课程编号
	 * @return CourseScheduleWrapper.CourseSchedule
	 */
	@Override
	public CourseScheduleWrapper.LiveCourseSchedule getLiveCourseScheduleTime(Long courseScheduleId) {

		CourseSchedule courseSchedule = courseScheduleDao.get(courseScheduleId);
		if (Objects.isNull(courseSchedule)) {
			throw new BizException("课程不存在");
		}

		if (courseSchedule.getTeachMode() == TeachModeEnum.OFFLINE) {
			throw new BizException("请前往线下教室");
		}

		// 是否是连堂课
		String continueCourseTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.ONLINE_CONTINUE_COURSE_TIME, courseSchedule.getTenantId());
		if (StringUtils.isEmpty(continueCourseTime)) {
			continueCourseTime = "5";
		}

		// 直播课关联直播间
		List<CourseSchedule> newCourseSchedules = Lists.newArrayList();
		// 课程时间长计算
		List<CourseScheduleWrapper.CourseScheduleTime> courseScheduleTimes = Lists.newArrayList();
		// 直播课关联直播间编号
		String liveRoomId = getCourseScheduleLiveRoomId(courseSchedule, continueCourseTime, newCourseSchedules, courseScheduleTimes, true);

		// 定时任务更新课程状态为进行中，判断当前课程状态是否匹配
		/*int studentRemindTime = Integer.parseInt(sysConfigService.findByParamName(SysConfigService.LIVE_CLASS_START_REMIND_TIME).getParanValue());

		Date date = DateUtil.startDateAndEndTime(courseSchedule.getClassDate(), courseSchedule.getStartClassTime());
		date = DateUtil.addMinutes(date, -studentRemindTime);
		// 课程是否开始
		if(CourseStatusEnum.NOT_START.equals(courseSchedule.getStatus()) && date.after(new Date())) {
			throw new BizException("直播课暂未开启，请稍后重试");
		}
		if (CourseStatusEnum.OVER == courseSchedule.getStatus()) {
			throw new BizException("直播课已结束");
		}*/

		// 直播间配置
		ImLiveBroadcastRoom liveBroadcastRoom = imLiveBroadcastRoomService.lambdaQuery()
				.eq(ImLiveBroadcastRoom::getRoomUid, liveRoomId)
				.last("LIMIT 1")
				.one();
		if (Objects.isNull(liveBroadcastRoom)) {
			throw new BizException("请先进入直播课教室");
		}

		String autoCloseNetworkRoomTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_AFTER_BUFFER_TIME, courseSchedule.getTenantId());
		if (StringUtils.isEmpty(autoCloseNetworkRoomTime)) {
			autoCloseNetworkRoomTime = "15";
		}

		return CourseScheduleWrapper.LiveCourseSchedule.builder()
				.liveRoomId(liveRoomId)
				.subjectId(liveBroadcastRoom.getSubjectId())
				.autoCloseFlag(true)
				//.autoCloseNetworkRoomTime(getCloseNetworkRoomTime(courseSchedule, continueCourseTime))
				.autoCloseNetworkRoomTime(autoCloseNetworkRoomTime)
				.surplusTime(DateUtil.secondsBetween(new Date(), courseSchedule.getEndClassTime()))
				.timestamp(DateTime.now().getMillis())
				.courseScheduleTimes(courseScheduleTimes)
				.build();
	}

	/**
	 * 同步连堂课状态，获取直播间编号
	 * @param courseSchedule CourseSchedule
	 * @param continueCourseTime 连堂课时长
	 * @param newCourseSchedules List<CourseSchedule>
	 * @param courseScheduleTimes List<CourseScheduleWrapper.CourseScheduleTime>
	 * @return String
	 */
	private String getCourseScheduleLiveRoomId(CourseSchedule courseSchedule,
											   String continueCourseTime,
											   List<CourseSchedule> newCourseSchedules,
											   List<CourseScheduleWrapper.CourseScheduleTime> courseScheduleTimes,
											   Boolean updateFlag) {
		String liveRoomId;
		// 课程开始时间
		courseSchedule.setStartClassTime(getDateTime(courseSchedule.getClassDate(), courseSchedule.getStartClassTime()).toDate());
		// 课程结束时间
		courseSchedule.setEndClassTime(getDateTime(courseSchedule.getClassDate(), courseSchedule.getEndClassTime()).toDate());

		// 是否是连堂课
		/*String continueCourseTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.ONLINE_CONTINUE_COURSE_TIME, courseSchedule.getTenantId());
		if (StringUtils.isEmpty(continueCourseTime)) {
			continueCourseTime = "5";
		}*/

		// 连堂课标准：同一个老师，同一天，同一课程组，连续上课
		List<CourseSchedule> continuousCourse = courseScheduleDao.getTeacherContinuousCourse(courseSchedule.getClassGroupId(),
				courseSchedule.getClassDate(), courseSchedule.getActualTeacherId());
		for (CourseSchedule item : continuousCourse) {
			// 设置开始时间
			item.setStartClassTime(getDateTime(item.getClassDate(), item.getStartClassTime()).toDate());
			// 设置结束时间
			item.setEndClassTime(getDateTime(item.getClassDate(), item.getEndClassTime()).toDate());
		}

		// 课程按开始时间排序
		continuousCourse = continuousCourse.stream()
				.sorted(Comparator.comparing(CourseSchedule::getStartClassTime))
				.collect(Collectors.toList());

		// 以当前课程为分界线，分为两个集合
		List<CourseSchedule> beforeCourse = Lists.newArrayList();
		List<CourseSchedule> afterCourse = Lists.newArrayList();

		CourseSchedule schedule;
		for (CourseSchedule item : continuousCourse) {
			if (item.getId().longValue() == courseSchedule.getId()) {
				continue;
			}
			// 重置连堂状态
			item.setContinuousCourse(false);

			schedule = JSON.parseObject(JSON.toJSONString(item), CourseSchedule.class);
			if (item.getEndClassTime().getTime() <= courseSchedule.getStartClassTime().getTime()) {
				beforeCourse.add(schedule);
			}

			if (courseSchedule.getEndClassTime().getTime() <= item.getStartClassTime().getTime()) {
				afterCourse.add(schedule);
			}
		}

		courseSchedule.setContinuousCourse(false);
		// 当前课程之前连堂课
		beforeCourse.add(JSON.parseObject(JSON.toJSONString(courseSchedule), CourseSchedule.class));
		// 当前课程之后连堂课
		afterCourse.add(0, JSON.parseObject(JSON.toJSONString(courseSchedule), CourseSchedule.class));

		// 课程前连堂状态
		updateContinuousCourseStatus(continueCourseTime, beforeCourse);

		// 课程连堂状态
		if (beforeCourse.get(beforeCourse.size() - 1).getContinuousCourse()) {

			// 倒序排序
			beforeCourse = beforeCourse.stream()
					.sorted(Comparator.comparing(CourseSchedule::getStartClassTime).reversed())
					.collect(Collectors.toList());

			for (int i = 0; i < beforeCourse.size(); i++) {
				CourseSchedule current = beforeCourse.get(i);
				if (current.getContinuousCourse()) {
					newCourseSchedules.add(current);
				}

				if (i + 1 < beforeCourse.size()) {
					CourseSchedule next = beforeCourse.get(i + 1);

					if (!next.getContinuousCourse()) {
						//newCourseSchedules.add(next);
						break;
					}
				}
			}

			// 正序排序
			beforeCourse = beforeCourse.stream()
					.sorted(Comparator.comparing(CourseSchedule::getStartClassTime))
					.collect(Collectors.toList());
		}

		// 课程后连堂状态
		updateContinuousCourseStatus(continueCourseTime, afterCourse);
		if (afterCourse.get(0).getContinuousCourse()) {

			for (int i = 0; i < afterCourse.size(); i++) {
				CourseSchedule current = afterCourse.get(i);
				if (current.getContinuousCourse()) {
					newCourseSchedules.add(current);
				}

				if (i + 1 < afterCourse.size()) {
					CourseSchedule next = afterCourse.get(i + 1);

					if (!next.getContinuousCourse()) {
						newCourseSchedules.add(next);
						break;
					}
				}
			}
		}

		if (CollectionUtils.isNotEmpty(newCourseSchedules)) {

            // newCourseSchedules集合去重
            newCourseSchedules = newCourseSchedules.stream()
                    .collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(CourseSchedule::getId))), ArrayList::new));

			// 课程按开始时间排序
			newCourseSchedules = newCourseSchedules.stream()
					.sorted(Comparator.comparing(CourseSchedule::getStartClassTime))
					.collect(Collectors.toList());

			// 连党课程信息
			for (CourseSchedule item : newCourseSchedules) {
				courseScheduleTimes.add(CourseScheduleWrapper.CourseScheduleTime.builder()
						.startTime(item.getStartClassTime().getTime())
						.endTime(item.getEndClassTime().getTime())
						.build());
			}

			// 获取连堂课第一节信息
			CourseSchedule finalCourseSchedule = newCourseSchedules.get(0);

			List<Long> collect = newCourseSchedules.stream()
					.map(CourseSchedule::getId)
					.distinct().collect(Collectors.toList());
			try {

				if (Optional.ofNullable(updateFlag).orElse(true)) {
					// 关联直播间编号
					courseScheduleDao.batchUpdateLiveRoomId(collect, finalCourseSchedule.getLiveRoomId());
				}

				if (CollectionUtils.size(collect) > 1) {
					collect = newCourseSchedules.subList(0, newCourseSchedules.size() - 1).stream()
							.map(CourseSchedule::getId)
							.distinct().collect(Collectors.toList());

					// 更新连堂课状态
					courseScheduleDao.batchUpdateContinuousCourse(collect, true);
				}

				if (!Optional.ofNullable(updateFlag).orElse(true)) {

					// 课程连堂状态
					if (beforeCourse.get(beforeCourse.size() - 1).getContinuousCourse()) {

						int size = beforeCourse.size();
						collect = beforeCourse.subList(size - 2, size).stream()
                                //.filter(o -> CourseStatusEnum.NOT_START.equals(o.getStatus()))
								.map(CourseSchedule::getId)
								.distinct().collect(Collectors.toList());
						if (CollectionUtils.isNotEmpty(collect)) {
							// 更新连堂课状态
							courseScheduleDao.batchUpdateContinuousCourse(collect, false);
						}
					}

					// 课程后连堂状态
					if (afterCourse.get(0).getContinuousCourse()) {
						collect = afterCourse.stream()
                                //.filter(o -> CourseStatusEnum.NOT_START.equals(o.getStatus()))
                                .map(CourseSchedule::getId).distinct().collect(Collectors.toList());

						if (CollectionUtils.isNotEmpty(collect)) {
							// 更新连堂课状态
							courseScheduleDao.batchUpdateContinuousCourse(collect, false);
						}

						collect = afterCourse.stream()
								.filter(o -> CourseStatusEnum.NOT_START.equals(o.getStatus()))
								.map(CourseSchedule::getId).distinct().collect(Collectors.toList());

						if (CollectionUtils.isNotEmpty(collect)) {
							// 关联直播间编号
							courseScheduleDao.batchUpdateLiveRoomId(collect, "");
						}
					}
				}

				// 更新连堂课状态,最后一节课重置
				CourseSchedule lastCourseSchedule = newCourseSchedules.get(newCourseSchedules.size() - 1);
				if (Objects.nonNull(lastCourseSchedule)) {
					courseScheduleDao.batchUpdateContinuousCourse(Lists.newArrayList(lastCourseSchedule.getId()), false);
				}

			} catch (Exception e) {
				log.error("getLiveCourseScheduleTime courseScheduleIds={}", collect, e);
			}

			// 连堂课直播间编号
			liveRoomId = finalCourseSchedule.getLiveRoomId();
		} else {

			// 当前课程信息
			courseScheduleTimes.add(CourseScheduleWrapper.CourseScheduleTime.builder()
					.startTime(courseSchedule.getStartClassTime().getTime())
					.endTime(courseSchedule.getEndClassTime().getTime())
					.build());

			// 更新连堂课状态
			courseScheduleDao.batchUpdateContinuousCourse(Lists.newArrayList(courseSchedule.getId()), false);

			liveRoomId = courseSchedule.getLiveRoomId();
		}
		return liveRoomId;
	}

	/**
	 * 更新当前课程连堂状态
	 * @param continueCourseTime 连堂时间间隔
	 * @param continuousCourse 连堂课程
	 */
	private void updateContinuousCourseStatus(String continueCourseTime, List<CourseSchedule> continuousCourse) {

		// continuousCourse对象相邻两个时间比较，如果时间间隔小于5分钟，那么合并为连堂课
		DateTime startTime;
		DateTime endTime;
		for (int i = 0; i < continuousCourse.size(); i++) {
			CourseSchedule current = continuousCourse.get(i);
			CourseSchedule next = null;
			if (i + 1 < continuousCourse.size()) {
				next = continuousCourse.get(i + 1);
			}
			if (next != null) {

				startTime = getDateTime(current.getClassDate(), current.getEndClassTime());
				endTime = getDateTime(next.getClassDate(), next.getStartClassTime());
				// 如果当前课程结束时间和下一节课开始时间间隔小于5分钟，那么合并时间（修复代码合并）
				if ((startTime.isBefore(endTime) || startTime.isEqual(endTime))
						&& startTime.plusSeconds(Integer.parseInt(continueCourseTime) * 60 + 10).isAfter(endTime)) {
					// 上节连堂状态
					current.setContinuousCourse(true);
					// 下节连堂状态
					next.setContinuousCourse(true);
				} else {
					current.setContinuousCourse(false);
				}
			}
		}

	}

	/**
	 * 课程开始时间
	 * @param day 开始天
	 * @param time 开始时间
	 * @return DateTime
	 */
	private DateTime getDateTime(Date day, Date time) {
		String liveStartTime = MessageFormat.format("{0} {1}", DateUtil.format(day, DateUtil.DEFAULT_PATTERN),
				DateUtil.format(time, DateUtil.EXPANDED_TIME_FORMAT));

		return DateTime.parse(liveStartTime, DateTimeFormat.forPattern(DateUtil.DEFAULT_PATTERN + " " + DateUtil.EXPANDED_TIME_FORMAT));
	}

	/**
	 * 课程结束后多少分钟关闭网络教室
	 * @param courseSchedule CourseSchedule
	 * @param continueCourseTime 连堂课延迟时间
	 * @return String
	 */
	public String getCloseNetworkRoomTime(CourseSchedule courseSchedule, String continueCourseTime) {

		String autoCloseNetworkRoomTime = sysTenantConfigService.getTenantConfigValue(SysConfigService.COURSE_AFTER_BUFFER_TIME, courseSchedule.getTenantId());
		if (StringUtils.isEmpty(autoCloseNetworkRoomTime)) {
			autoCloseNetworkRoomTime = "15";
		}
		CourseSchedule schedule = courseSchedule;
		//如果当前课程是连堂课，那么获取第一节课的课程编号
		while (true) {
			String classDate = DateUtil.format(schedule.getClassDate(), DateUtil.DEFAULT_PATTERN);
			String endClassTime = DateUtil.format(schedule.getEndClassTime(), DateUtil.EXPANDED_TIME_FORMAT);
			schedule = courseScheduleDao.getLastCourse(schedule.getClassGroupId(), classDate + " " + endClassTime, schedule.getActualTeacherId(), continueCourseTime);
			if (schedule != null) {
				autoCloseNetworkRoomTime = String.valueOf(DateUtil.minutesBetween(new Date(), schedule.getEndClassTime()));
			} else {
				break;
			}
		}
		return autoCloseNetworkRoomTime;
	}

}
